/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
//
// XarOutputDev.cc
//
#include "stdafx.h"
#include "XarOutputDev.h"
#include "splash/splashbitmap.h"
#include "PDFBitmap.h"
#include "PathHelpers.h"

extern "C" 
{
#include "..\png\pngstruct.h"
}

// Uncomment to enable small bitmaps merging
#define MERGE_SMALL_BITMAPS

int s_iBitmapCount = 0;
int g_iBitmapCount = 0;

void GetDecodeInfo(GfxImageColorMap* colorMap, bool &bDefaultDecode, bool &bInvertDecode);

void PNGAPI PngWriteData(png_structp png_ptr, png_bytep data, png_size_t length)
{
	if (!length)
		return;

	std::vector<BYTE> *pData = (std::vector<BYTE> *)(png_ptr->io_ptr);

	size_t n = pData->size() + length;
	if (pData->capacity() < n)
		pData->reserve(n + 1024 * 1024);

	pData->insert(pData->end(), (const unsigned char *)data, (const unsigned char *)data + length);
}

/******************************************************************************
   Function:	XarOutputDev::GrowBitmapObject
   Author:		Paul Gruzdev
   Purpose:     Grow greyscale bitmap mask propagating 255 (opaque) area
				one-pixel outside of the mask
   Returns:		void
   Parameters:	SplashBitmap * bitmap  
   Notes:       
******************************************************************************/
void XarOutputDev::GrowBitmapObject(SplashBitmap* bitmap)
{
	START_STOPWATCH_FN
		SplashColorPtr ptr = bitmap->getDataPtr();
	int w = bitmap->getWidth();
	int h = bitmap->getHeight();
	int rowsize = bitmap->getRowSize();
	SplashColorPtr pRow = ptr + rowsize;

	for(int i = 1; i < h-1; i++)
	{
		ptr = pRow;
		for(int j = 1; j < w-1; j++)
		{
			if(*(ptr + 3) == 255)
			{
				if(*(ptr - 1) == 0)
				{
					memcpy(ptr-4, ptr, 3);
					*(ptr - 1) = 254;
				}
				if(*(ptr + 7) == 0)
				{
					memcpy(ptr+4, ptr, 3);
					*(ptr + 7) = 254;
				}

				if(*(ptr - rowsize + 3) == 0)
				{
					memcpy(ptr-rowsize, ptr, 3);
					*(ptr - rowsize + 3) = 254;
				}
				if(*(ptr + rowsize + 3) == 0)
				{
					memcpy(ptr+rowsize, ptr, 3);
					*(ptr + rowsize + 3) = 254;
				}
			}
			ptr += 4;
		}
		pRow += rowsize;
	}
	STOP_STOPWATCH(__FUNCTION__)
}

/******************************************************************************
   Function:    AppendBitmap
   Author:      Paul Gruzdev
   Purpose:		Append a define bitmap node, check the /Decode for inversion
				and add a corresponding flag
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void AppendBitmap(GfxImageColorMap* colorMap, VXfTreeNode* pNode, CXfDefineBitmap* pBitmap)
{
	if (pBitmap->GetBitmapType() == CXfDefineBitmap::JPEG && pBitmap->GetColourSpace() == CXfDefineBitmap::CS_CMYK)
	{
		bool bDefaultDecode = true, bInvertDecode = false;
		GetDecodeInfo(colorMap, bDefaultDecode, bInvertDecode);
		pNode->AppendChild(new CXfZeroSizedNode(bInvertDecode ? TAG_CMYKJPEG_NORMAL : TAG_CMYKJPEG_INVERT));
	}
	pNode->AppendChild(pBitmap);
}

/******************************************************************************
   Function:	XarOutputDev::drawImageMask
   Author:		Paul Gruzdev
   Purpose:     Receive 1-bit image and make a transparent fill out of it using
				current fill colour as an object.
   Returns:		void
   Parameters:	GfxState * state  
   Parameters:	Object * ref  
   Parameters:	Stream * str  
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	bool invert  
   Parameters:	bool interpolate  
   Parameters:	bool inlineImg  
   Notes:       
******************************************************************************/
void XarOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
								 int width, int height, bool invert,
								 bool interpolate, bool inlineImg) 
{
START_STOPWATCH_FN
	if (g_settings.bIgnoreGraphics || m_clip.Empty()) return;

	GfxPath rect;
	AddRect(&rect, 0, 0, 1, 1);

	double x, y;
	state->transform(0, 1, &x, &y);
	DocCoord tl((LONG)x, (LONG)y);
	state->transform(1, 1, &x, &y);
	DocCoord tr((LONG)x, (LONG)y);
	state->transform(1, 0, &x, &y);
	DocCoord br((LONG)x, (LONG)y);
	state->transform(0, 0, &x, &y);
	DocCoord bl((LONG)x, (LONG)y);

	// Paul: we always, ALWAYS have to create path with filled and stroked attributes - this is because of the
	// 'outline' mode in Camelot - path ALWAYS should be at least stroked

	CXfPath* pPath = CreateXaraPath(&rect, state, TAG_PATH_FILLED_STROKED);
	if (AddGraphics(pPath, pPath, CXF_WINDING_RULE_NONZERO))
	{	
		CXfDefineBitmap* pMaskBitmap = NULL;
		Goffset strPos = str->getPos();
		BitmapMap::iterator itBitmap = strPos ? m_bitmaps.find(strPos) : m_bitmaps.end();
		if (m_bitmaps.end() == itBitmap)
		{
			GfxDeviceGrayColorSpace* primary = new GfxDeviceGrayColorSpace();
			Object decode(objNull);

			GfxImageColorMap colorMap(1, &decode, primary);

			pMaskBitmap = CreateDefineBitmap(str, width, height, &colorMap);
			m_bitmaps[strPos] = pMaskBitmap;

			AppendBitmap(&colorMap, pPath, pMaskBitmap);
		}
		else
			pMaskBitmap = itBitmap->second;

		if(pMaskBitmap!=NULL)
			RD_ARG(pMaskBitmap->Name.getstr());
		
		// Fill the rect
		AppendAttributes(pPath, state, false, true);

		BYTE byAlpha, byBlendMode;
		ConvertOpacity(0, state->getBlendMode(), byAlpha, byBlendMode);

		// PDF masks use 0 = draw current color, 1 = leave unchanged.
		pPath->AppendChild(new CXfBitmapTransparentFill(bl, br, tl, invert ? 255 : 0, invert ? 0 : 255, byBlendMode, pMaskBitmap, 0, 0));
		//pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_REPEATINGINVERTED));
		//pPath->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_REPEATINGINVERTED));
		pPath->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_NONREPEATING));
	}
	else
		delete pPath;

STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::setSoftMaskFromImageMask(GfxState *state,
											Object *ref, Stream *str,
											int width, int height,
											bool invert,
											bool inlineImg, double *baseMatrix) 
{ 
	RD;
/*
	DWORD dwWidth  = (DWORD)(state->getPageWidth());
	DWORD dwHeight = (DWORD)(state->getPageHeight());

	BYTE byTrans, byMode;
	ConvertOpacity(0, state->getBlendMode(), byTrans, byMode);
	CXfSoftMaskGroupTransparentFill* pFill = new CXfSoftMaskGroupTransparentFill(DocCoord(0, 0), DocCoord(dwWidth, 0), DocCoord(0, dwHeight), 0, 255, byMode, 0, 0, 0);
	pFill->SetRepeatType(RT_SIMPLE);
	pFill->SetClip(true); // This flag in Xara means that 'area outside mask objects is 100% transparent'
	pFill->SetLinked(true);

	m_TransparencyGroup.push_back(m_pInsertTo);
	pFill->SetAsTransparencyGroup(true);
	SetParentNode(pFill);

	drawImageMask(state, ref, str, width, height, invert, */
}

void XarOutputDev::unsetSoftMaskFromImageMask(GfxState *state, double *baseMatrix) 
{ 
	RD;
	//assert(0); // We need to track and handle this also
}

/******************************************************************************
   Function:	CanPassthroughBitmap
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		bool
   Parameters:	Stream * str  
   Parameters:	GfxImageColorMap * colorMap  
   Notes:       
******************************************************************************/
bool CanPassthroughBitmap(Stream* str, GfxImageColorMap* colorMap)
{
	GfxColorSpace* pSpace = colorMap->getColorSpace();
	GfxColorSpace* pAltSpace = pSpace->getMode() == csICCBased ? ((GfxICCBasedColorSpace*)pSpace)->getAlt() : NULL;
	GfxColorSpaceMode cs = pAltSpace ? pAltSpace->getMode() : pSpace->getMode();

	bool bDefaultDecode = true, bInvertDecode = false;
	GetDecodeInfo(colorMap, bDefaultDecode, bInvertDecode);

	// DDES-1223 fix: we can't pass-through the JPEG if it's inverted and is not CMYK, as we can only pass inverted CMYKs: we
	// have a special CMYK invert flag
	bool bCanPassThrough = (bDefaultDecode || csDeviceCMYK == cs && bInvertDecode);

	// First try to pass a JPEG (DCTStream) as a JPEG file to Xara - also check if it's an ICC Based colour space with a
	// ICC profile but actually a compatible JPEG file like CMYK, RGB or grey
	// DDES-1223 fix: we can't pass-through the JPEG if it's 
	return str->getKind() == strDCT && bCanPassThrough && (cs == csDeviceRGB || cs == csCalRGB || cs == csDeviceCMYK || cs == csDeviceGray || cs == csCalGray);
}

CXfDefineICC_CMYK *XarOutputDev::GetCMYKProfile()
{
	if (!m_cmykProfile)
	{
		Object catDict = m_pDoc->getXRef()->getCatalog();
		if (catDict.isDict())
		{
			Object outputIntents = catDict.dictLookup("OutputIntents");
			if (outputIntents.isArray() && outputIntents.arrayGetLength() == 1)
			{
				Object firstElement = outputIntents.arrayGet(0);
				if (firstElement.isDict())
				{
					Object profile = firstElement.dictLookup("DestOutputProfile");
					if (profile.isStream())
					{
						Object n = profile.streamGetDict()->lookup("N");
						if (n.isNum() && 4 == n.getNum())
						{
							Stream *iccStream = profile.getStream();
							int length = 0;
							if (unsigned char *profBuf = iccStream->toUnsignedChars(&length, 65536, 65536))
							{
								m_cmykProfile = new CXfDefineICC_CMYK(profBuf, length);
								m_pChapter->InsertFirstChild(m_cmykProfile);
								gfree(profBuf);
							}
						}
					}
				}
			}
		}
	}
	return m_cmykProfile;
}

/******************************************************************************
   Function:    XarOutputDev::CreateDefineBitmap
   Author:      Paul Gruzdev
   Purpose:		Create a TAG_DEFINEBITMAP_*** tag based on PDF stream and image options
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfDefineBitmap* XarOutputDev::CreateDefineBitmap(Stream* str, int width, int height, GfxImageColorMap* colorMap, int* maskColors, CXfDefineBitmap** pMask, BitmapData* bitmapData, Rect *pBounds)
{
START_STOPWATCH_FN

	if (bitmapData) bitmapData->buf = NULL;

	if (pBounds)
	{
		pBounds->x = pBounds->y = 0;
		pBounds->width = width;
		pBounds->height = height;
	}

	// First try to pass a JPEG (DCTStream) as a JPEG file to Xara - also check if it's an ICC Based colour space with a
	// ICC profile but actually a compatible JPEG file like CMYK, RGB or grey
	// DDES-1223 fix: we can't pass-through the JPEG if it's 
	if (CanPassthroughBitmap(str, colorMap))
	{
		// Get the next stream in chain that holds the JPEG - it can be any other non-plain (DDES-957)
		Stream* pstr = str->getNextStream();//str->getUndecodedStream();
		//Dict* pDict = str->getDict();
		int length = 0;
		// The next stream can be not plain, i.e. like Flate (ZIP) stream. Take whichever bytes there
		unsigned char* buf = pstr->toUnsignedChars(&length);
		//Object obj1 = pDict->lookup("Length");
		//if (obj1.isInt())
		//	length = obj1.getInt();
		// the next stream can't be Crypt
		//if (pstr->getKind() != strCrypt && pstr->getKind() != strWeird && length > 0)
		{
			/*
			if (g_settings.bDumpData)
			{
			FILE* out = fopen(FormatStringA("d:/temp/PDF_Bitmap%.3d.jpg", s_iBitmapCount-1).c_str(), "wb");
			if(out)
			{
			fwrite(buf, length, 1, out);
			fclose(out);
			}
			}
			*/

			CXfDefineBitmap* pDefine = new CXfDefineBitmap(TAG_DEFINEBITMAP_JPEG, nullptr, 0, FormatStringW(L"JPEG Bitmap %d", ++s_iBitmapCount).c_str());
			pDefine->Data.resize(length);
			memcpy(&pDefine->Data[0], buf, length);
			gfree(buf);
			//pstr->reset();
			//length = pstr->doGetChars(length, &pDefine->Data[0]);
			//pDefine->Data.resize(length);

			switch (colorMap->getNumPixelComps())
			{
			case 3:
				pDefine->SetColourSpace(CXfDefineBitmap::CS_RGB);
				break;
			case 4:
				pDefine->SetColourSpace(CXfDefineBitmap::CS_CMYK);
				break;
			default:
				pDefine->SetColourSpace(CXfDefineBitmap::CS_GREY);
				break;
			}

STOP_STOPWATCH(__FUNCTION__)
			return pDefine;
		}
	}

	PDFBitmap bitmap;
	bitmap.Create(str, width, height, colorMap, maskColors);

	if (bitmap.m_mask)
	{
		*pMask = new CXfDefineBitmap(TAG_DEFINEBITMAP_PNG_REAL, nullptr, 0, FormatStringW(L"PDF Mask %d", ++g_iBitmapCount).c_str());
		(*pMask)->Data.reserve(((width + 7) >> 3) * height);
		EncodePNGFromPDFData(&((*pMask)->Data), bitmap.m_mask, width, height, NULL, 0, 1, 1, true, true, PngWriteData);
		(*pMask)->SetColourSpace(CXfDefineBitmap::CS_RGB);
/*
		if (g_settings.bDumpData)
			(*pMask)->Save(FormatStringW(L"d:/temp/PDF_Mask%d.png", g_iBitmapCount-1).c_str());
*/
	}

	CXfDefineBitmap* pDefine = NULL;

	bool isCMYK = (4 == bitmap.m_nComps);

	if (bitmapData && (1 == bitmap.m_nComps || 3 == bitmap.m_nComps || 4 == bitmap.m_nComps) && (bitmapMergeList.empty() || bitmapMergeList.back().isCMYK == isCMYK) && 8 == bitmap.m_bpc)
	{
		bitmapData->buf       = bitmap.m_data;
		bitmapData->width     = width;
		bitmapData->height    = height;
		bitmapData->pbyPal    = bitmap.m_palette;
		bitmapData->iPalCount = bitmap.m_palCount;
		bitmapData->iBPC      = bitmap.m_bpc;
		bitmapData->iNComps   = bitmap.m_nComps;
		bitmapData->isCMYK    = isCMYK;

		bitmap.m_data = bitmap.m_palette = nullptr;
	}
	else
	{
		if (pBounds && 1 == bitmap.m_bpc && 1 == bitmap.m_nComps && bitmap.m_palette && 2 == bitmap.m_palCount)
		{
			pBounds->x = pBounds->y = pBounds->width = pBounds->height = 0;
			int stride = (width * bitmap.m_bpc * bitmap.m_nComps + 7) / 8;
			unsigned char *row = bitmap.m_data;
			for (int y = 0; y < height; ++y, row += stride)
			{
				unsigned char *p = row;
				int x = 0;
				for (int i = 0; i < stride; ++i, ++p)
				{
					for (int k = 0; k < 8 && x < width; ++k, ++x)
					{
						int index = ((*p) >> (7 - k)) & 1;
						index *= 3;

						if (bitmap.m_palette[index] || bitmap.m_palette[index + 1] || bitmap.m_palette[index + 2])
						{
							if (0 == pBounds->width)
							{
								pBounds->x = x;
								pBounds->y = y;
								pBounds->width = pBounds->height = 1;
							}
							else
							{
								if (x < pBounds->x)
								{
									pBounds->width += (pBounds->x - x);
									pBounds->x = x;
								}
								else if (x >= (pBounds->x + pBounds->width))
									pBounds->width = (x + 1) - pBounds->x;

								if (y < pBounds->y)
								{
									pBounds->height += (pBounds->y - y);
									pBounds->y = y;
								}
								else if (y >= (pBounds->y + pBounds->height))
									pBounds->height = (y + 1) - pBounds->y;
							}
						}
					}
				}
			}

			pBounds->y = height - (pBounds->y + pBounds->height);
		}

		pDefine = new CXfDefineBitmap(bitmap.m_nComps == 4 ? TAG_DEFINEBITMAP_PNG_CMYK : TAG_DEFINEBITMAP_PNG_REAL, nullptr, 0, FormatStringW(L"PDF Bitmap %d", ++s_iBitmapCount).c_str());

		if (bitmap.m_nComps == 4 && csDeviceCMYK == colorMap->getColorSpace()->getMode())
			pDefine->pProfileRefPtr = GetCMYKProfile();

		pDefine->Data.reserve(((width * bitmap.m_nComps * bitmap.m_bpc + 7) >> 3) * height);

		EncodePNGFromPDFData(&pDefine->Data, bitmap.m_data, width, height, bitmap.m_palette, bitmap.m_palCount, bitmap.m_bpc, bitmap.m_nComps, true, true, PngWriteData);

		pDefine->SetColourSpace(bitmap.m_nComps == 4 ? CXfDefineBitmap::CS_CMYK : (bitmap.m_nComps == 3 ? CXfDefineBitmap::CS_RGB : CXfDefineBitmap::CS_GREY));
/*
		if (g_settings.bDumpData)
			pDefine->Save(FormatStringW(L"d:/temp/PDF_Bitmap%.3d.png", s_iBitmapCount-1).c_str());*/
	}

STOP_STOPWATCH(__FUNCTION__)

	return pDefine;
}

/******************************************************************************
   Function:	XarOutputDev::drawImage
   Author:		Paul Gruzdev
   Purpose:     Draw a regular image. It may possible have a 1-bit mask defined
				by maskColors attribute
   Returns:		void
   Parameters:	GfxState * state  
   Parameters:	Object * ref  
   Parameters:	Stream * str  
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	GfxImageColorMap * colorMap  
   Parameters:	bool interpolate  
   Parameters:	int * maskColors  
   Parameters:	bool inlineImg  
   Notes:       
******************************************************************************/
void XarOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
							 int width, int height,
							 GfxImageColorMap *colorMap,
							 bool interpolate,
							 int *maskColors, bool inlineImg)
{
	DoDrawImage(state, ref, str, width, height, colorMap, interpolate, maskColors, inlineImg, true);
}

CXfPath* XarOutputDev::DoDrawImage(GfxState *state, Object *ref, Stream *str,
	int width, int height, GfxImageColorMap *colorMap,
	bool interpolate, int *maskColors, bool inlineImg, bool allowMerging, PDFRectangle *bbox)
{
START_STOPWATCH_FN
	if (g_settings.bIgnoreGraphics || m_clip.Empty() || width <= 0 || height <= 0) return NULL;

	GfxPath rect;
	if (bbox)
		AddRect(&rect, bbox->x1, bbox->y1, bbox->x2 - bbox->x1, bbox->y2 - bbox->y1);
	else
		AddRect(&rect, 0, 0, 1, 1);

	double blX = 0, blY = 0,
	       brX = 1, brY = 0,
	       trX = 1, trY = 1,
	       tlX = 0, tlY = 1;

	state->transform(blX, blY, &blX, &blY);
	state->transform(brX, brY, &brX, &brY);
	state->transform(trX, trY, &trX, &trY);
	state->transform(tlX, tlY, &tlX, &tlY);

	DocCoord bl(roundNumber(blX), roundNumber(blY));
	DocCoord br(roundNumber(brX), roundNumber(brY));
	DocCoord tr(roundNumber(trX), roundNumber(trY));
	DocCoord tl(roundNumber(tlX), roundNumber(tlY));

#ifdef MERGE_SMALL_BITMAPS
	if (allowMerging)
	{
		const double eps = 0.0001;
		if (abs(blX - tlX) > eps || abs(brX - trX) > eps || abs(blY - brY) > eps || abs(tlY - trY) > eps || (trX - blX) < 1.0 || (trY - blY) < 1.0)
			allowMerging = false;
		else
		{
			double pixWidth  = (trX - blX) / width;
			double pixHeight = (trY - blY) / height;

			if (bitmapMergeList.empty())
			{
				bitmapMergePixWidth  = pixWidth;
				bitmapMergePixHeight = pixHeight;
			}
			else if (abs(bitmapMergePixWidth - pixWidth) > eps || abs(bitmapMergePixHeight - pixHeight) > eps)
			//else if (abs(bitmapMergePixWidth - pixWidth) / bitmapMergePixWidth > 0.1 || abs(bitmapMergePixHeight - pixHeight) / bitmapMergePixHeight > 0.1)
			{
				allowMerging = false;
			}
		}
	}
#else
	allowMerging = false;
#endif

#ifdef FILTER_DEBUG
	printf("image:[%d %d w:%d h:%d (%d %d)]\n", bl.x, bl.y, br.x - bl.x, tl.y - bl.y, width, height);
#endif
	// What about soft masked bitmap that can exist in PDF? So instead of node bitmap we create a path filled with bitmap
	// and (probably) with bitmap transparent fill
/*
	CXfBitmap* pBitmap = new CXfBitmap(tl, tr, br, bl, pDefine);
	m_pInsertTo->AppendChild(pBitmap);
*/

	CXfDefineBitmap* pBitMask = NULL;
	BitmapMergeData mergeData;
	CXfDefineBitmap* pDefine = CreateDefineBitmap(str, width, height, colorMap, maskColors, &pBitMask, (!allowMerging || maskColors) ? NULL : &mergeData);
		
	if (pDefine)
	{
		RD_ARG(pDefine->Name.getstr());

		CXfPath* pPath = CreateXaraPath(&rect, state, TAG_PATH_FILLED_STROKED);

		if (AddGraphics(pPath, pPath, CXF_WINDING_RULE_NONZERO))
		{	
			// Add define bitmap here so we can easily transfer this path to another parent
			AppendBitmap(colorMap, pPath, pDefine);
			pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_NONREPEATING));
			pPath->AppendChild(new CXfBitmapFill(bl, br, tl, pDefine, 0, 0));
			pPath->AppendChild(new CXfZeroSizedNode(TAG_PHOTOATTR_FLAG));

			if(pBitMask)
			{
				pPath->AppendChild(pBitMask);
				BYTE byAlpha, byBlendMode;
				ConvertOpacity(0, state->getBlendMode(), byAlpha, byBlendMode);
				pPath->AppendChild(new CXfBitmapTransparentFill(bl, br, tl, 0, 255, byBlendMode, pBitMask, 0, 0));
				//pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_REPEATINGINVERTED));
				//pPath->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_REPEATINGINVERTED));
				pPath->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_NONREPEATING));
			}

			pPath->AddChild(new CXfLineColor(CXfLineColor::None));
			pPath->AddChild(new CXfLineWidth(250));
			pPath->AddChild(new CXfJoinStyle(CXF_JOIN_STYLE_MITRE)); // Like if you drag a photo from PG
		}
		else
		{
			delete pDefine;
			delete pPath;
			pPath = NULL;
		}

		return pPath;
	}
	else if (mergeData.buf)
	{		
		mergeData.left   = blX;
		mergeData.bottom = blY;
		mergeData.right  = trX;
		mergeData.top    = trY;
		bitmapMergeList.push_back(mergeData);
	}

STOP_STOPWATCH(__FUNCTION__)
	return NULL;
}

/******************************************************************************
   Function:	XarOutputDev::drawMaskedImage
   Author:		Paul Gruzdev
   Purpose:     Draw an image that has a /Mask attribute applied which is a
				stream that contains mask data.
				An image mask differs from an ordinary image in the following significant ways:
				 The image dictionary does not contain a ColorSpaceentry because sample
				  values represent masking properties (1 bit per sample) rather than colors.
				 The value of the BitsPerComponententry must be 1.
				 The Decode entry determines how the source samples are to be interpreted. If
				the Decode array is [ 0  1 ] (the default for an image mask), a sample value of 0
				marks the page with the current color, and a 1 leaves the previous contents unchanged. 
				If the Decode array is [ 1  0 ], these meanings are reversed.
   Returns:		void
   Parameters:	GfxState * state  
   Parameters:	Object * ref  
   Parameters:	Stream * str  
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	GfxImageColorMap * colorMap  
   Parameters:	bool interpolate  
   Parameters:	Stream * maskStr  
   Parameters:	int maskWidth  
   Parameters:	int maskHeight  
   Parameters:	bool maskInvert  
   Parameters:	bool maskInterpolate  
   Notes:       
******************************************************************************/
void XarOutputDev::drawMaskedImage(GfxState *state, Object *ref,
								   Stream *str, int width, int height,
								   GfxImageColorMap *colorMap,
								   bool interpolate,
								   Stream *maskStr, int maskWidth,
								   int maskHeight, bool maskInvert,
								   bool maskInterpolate) 
{ 
START_STOPWATCH_FN

	Goffset strPos = str->getPos();

	VXfTreeNode* pImage = DoDrawImage(state, ref, str, width, height, colorMap, interpolate, NULL, false, false);
	if (!pImage) return;

	CXfDefineBitmap* pMaskBitmap = NULL;
	Goffset maskStrPos = maskStr->getPos();
	BitmapMap::iterator itBitmap = maskStrPos ? m_bitmaps.find(maskStrPos) : m_bitmaps.end();
	if (m_bitmaps.end() == itBitmap)
	{
		GfxDeviceGrayColorSpace* primary = new GfxDeviceGrayColorSpace();
		Object decode(objNull);

		GfxImageColorMap colorMap(1, &decode, primary);

		pMaskBitmap = CreateDefineBitmap(maskStr, maskWidth, maskHeight, &colorMap);
		m_bitmaps[maskStrPos] = pMaskBitmap;

		AppendBitmap(&colorMap, pImage, pMaskBitmap);
	}
	else
		pMaskBitmap = itBitmap->second;

RD_ARG(pMaskBitmap->Name.getstr());

	double x, y;
	state->transform(0, 1, &x, &y);
	DocCoord tl(roundNumber(x), roundNumber(y));
	state->transform(1, 1, &x, &y);
	DocCoord tr(roundNumber(x), roundNumber(y));
	state->transform(1, 0, &x, &y);
	DocCoord br(roundNumber(x), roundNumber(y));
	state->transform(0, 0, &x, &y);
	DocCoord bl(roundNumber(x), roundNumber(y));

	pImage->AddChild(new CXfLineColor(CXfLineColor::None));
	pImage->AddChild(new CXfLineWidth(250));

	BYTE byAlpha, byBlendMode;
	ConvertOpacity(0, state->getBlendMode(), byAlpha, byBlendMode);

	// PDF masks use 0 = draw current color, 1 = leave unchanged.
	pImage->AppendChild(new CXfBitmapTransparentFill(bl, br, tl, maskInvert ? 255 : 0, maskInvert ? 0 : 255, byBlendMode, pMaskBitmap, 0, 0));
	//pImage->AppendChild(new CXfZeroSizedNode(TAG_FILL_REPEATINGINVERTED));
	//pImage->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_REPEATINGINVERTED));
	pImage->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_NONREPEATING));

STOP_STOPWATCH(__FUNCTION__)
}

/******************************************************************************
   Function:	XarOutputDev::drawSoftMaskedImage
   Author:		Paul Gruzdev
   Purpose:     Draw an image that has a /SMask attribute applied which represents
				a soft mask - an arbitrary image with arbitrary colour space and
				arbitrary bits per component/num components that acts as a 
				transparency channel for the main image.
   Returns:		void
   Parameters:	GfxState * state  
   Parameters:	Object * ref  
   Parameters:	Stream * str  
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	GfxImageColorMap * colorMap  
   Parameters:	bool interpolate  
   Parameters:	Stream * maskStr  
   Parameters:	int maskWidth  
   Parameters:	int maskHeight  
   Parameters:	GfxImageColorMap * maskColorMap  
   Parameters:	bool maskInterpolate  
   Notes:       
******************************************************************************/
void XarOutputDev::drawSoftMaskedImage(GfxState *state, Object *ref,
									   Stream *str, int width, int height,
									   GfxImageColorMap *colorMap,
									   bool interpolate,
									   Stream *maskStr,
									   int maskWidth, int maskHeight,
									   GfxImageColorMap *maskColorMap,
									   bool maskInterpolate) 
{ 
	START_STOPWATCH_FN;

	if (g_settings.bIgnoreGraphics || m_clip.Empty()) return;

	Goffset strPos = str->getPos();

	GfxColorSpace* pSpace = colorMap->getColorSpace();
	GfxColorSpace* pAltSpace = pSpace->getMode() == csICCBased ? ((GfxICCBasedColorSpace*)pSpace)->getAlt() : NULL;
	GfxColorSpaceMode cs = pAltSpace ? pAltSpace->getMode() : pSpace->getMode();
	bool bSameDimensions = width == maskWidth && height == maskHeight;

	bool bRGB       = cs == csDeviceRGB  || cs == csCalRGB;
	bool bGrayscale = cs == csDeviceGray || cs == csCalGray;

	const GfxColor *matteColor = maskColorMap->getMatteColor();

	bool bCanPassthrough = CanPassthroughBitmap(str, colorMap) && !matteColor;

	// Combine our mask with the image - if we can to do so - PNG support only RGBA or GrayA, so test whether our
	// streams confirm to these restrictions and combine if they do
	if (bSameDimensions && !bCanPassthrough && (bRGB || bGrayscale) && maskColorMap->getBits() == 8 && maskColorMap->getNumPixelComps() == 1)
	{
		GfxPath rect;
		AddRect(&rect, 0, 0, 1, 1);

		double x, y;
		state->transform(0, 1, &x, &y);
		DocCoord tl(roundNumber(x), roundNumber(y));
		state->transform(1, 1, &x, &y);
		DocCoord tr(roundNumber(x), roundNumber(y));
		state->transform(1, 0, &x, &y);
		DocCoord br(roundNumber(x), roundNumber(y));
		state->transform(0, 0, &x, &y);
		DocCoord bl(roundNumber(x), roundNumber(y));

		CXfPath* pPath = CreateXaraPath(&rect, state, TAG_PATH_FILLED_STROKED);
		if (AddGraphics(pPath, pPath, CXF_WINDING_RULE_NONZERO))
		{
			PDFBitmap bmImage, bmAlpha;
			bool bRes = bmImage.Create(str, width, height, colorMap);
			bRes = bRes && bmAlpha.Create(maskStr, width, height, maskColorMap);

			/*
			If the image data is preblended, the matte colour shall be specified
			by a Matte entry in the soft-mask image dictionary. The preblending computation,
			performed independently for each component, shall be

				c' = m + alpha * (c - m)

			where

			c' is the value to be provided in the image source data
			c is the original image component value
			m is the matte colour component value
			alpha is the corresponding mask sample
			*/

			if (matteColor)
			{
				// invert preblending computations
				BYTE *pComp = bmImage.m_data;
				const BYTE *pAlpha = bmAlpha.m_data;
				if (bGrayscale)
				{
					GfxGray gray;
					maskColorMap->getColorSpace()->getGray(matteColor, &gray);
					BYTE g = colToByte(gray);
					for (int j = 0; j < height; ++j)
					{
						for (int i = 0; i < width; ++i)
						{
							BYTE alpha = *(pAlpha++);

							BYTE c = alpha ? min(255, (*pComp - g) * 255 / alpha + g) : 255;
							(*pComp++) = c;
						}
					}
				}
				else
				{
					GfxRGB rgb;
					maskColorMap->getColorSpace()->getRGB(matteColor, &rgb);
					BYTE r = colToByte(rgb.r);
					BYTE g = colToByte(rgb.g);
					BYTE b = colToByte(rgb.b);
					for (int j = 0; j < height; ++j)
					{
						for (int i = 0; i < width; ++i)
						{
							BYTE alpha = *(pAlpha++), c;

							c = alpha ? min(255, (*pComp - r) * 255 / alpha + r) : 255;
							(*pComp++) = c;

							c = alpha ? min(255, (*pComp - g) * 255 / alpha + g) : 255;
							(*pComp++) = c;

							c = alpha ? min(255, (*pComp - b) * 255 / alpha + b) : 255;
							(*pComp++) = c;
						}
					}
				}
			}

			CXfDefineBitmap* pDefine = new CXfDefineBitmap(TAG_DEFINEBITMAP_PNG_REAL, nullptr, 0, FormatStringW(L"PDF Bitmap %d", ++g_iBitmapCount).c_str());

			pDefine->Data.reserve(bGrayscale ? (width * height) : (width * 3 * height));

			bRes = bRes && EncodePNGFromPDFDataAlpha(&pDefine->Data, bmImage.m_data, bmAlpha.m_data, width, height, bGrayscale, false, true, PngWriteData);

			pDefine->SetColourSpace(!bGrayscale ? CXfDefineBitmap::CS_RGB : CXfDefineBitmap::CS_GREY);

			RD_ARG(pDefine->Name.getstr());

			AppendBitmap(nullptr, pPath, pDefine); // Add define bitmap here so we can easily transfer this path to another parent
			// This is a single bitmap fill - make it a photo
			pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_NONREPEATING));
			pPath->AppendChild(new CXfBitmapFill(bl, br, tl, pDefine, 0, 0));
			pPath->AppendChild(new CXfZeroSizedNode(TAG_PHOTOATTR_FLAG));

			pPath->AddChild(new CXfLineColor(CXfLineColor::None));
			pPath->AddChild(new CXfLineWidth(250));
			pPath->AddChild(new CXfJoinStyle(CXF_JOIN_STYLE_MITRE)); // Like if you drag a photo from PG
		}
		else delete pPath;
	}
	else
	{
		Rect maskBounds(0, 0, maskWidth, maskHeight);
		CXfDefineBitmap* pMaskBitmap = NULL;
		Goffset maskStrPos = maskStr->getPos();
		BitmapMap::iterator itBitmap = maskStrPos ? m_bitmaps.find(maskStrPos) : m_bitmaps.end();
		if (m_bitmaps.end() == itBitmap)
		{
			pMaskBitmap = CreateDefineBitmap(maskStr, maskWidth, maskHeight, maskColorMap, NULL, NULL, NULL, &maskBounds);
			m_bitmaps[maskStrPos] = pMaskBitmap;
		}
		else
			pMaskBitmap = itBitmap->second;

		if (VXfTreeNode* pImage = DoDrawImage(state, ref, str, width, height, colorMap, interpolate, NULL, false, false,
			&PDFRectangle(
			(double)maskBounds.x / (double)maskWidth,
			(double)maskBounds.y / (double)maskHeight,
			(double)(maskBounds.x + maskBounds.width) / (double)maskWidth,
			(double)(maskBounds.y + maskBounds.height) / (double)maskHeight)))
		{
			if (NULL == pMaskBitmap->GetParent())
				AppendBitmap(maskColorMap, pImage, pMaskBitmap);

			RD_ARG(pMaskBitmap->Name.getstr());
			double x, y;
			state->transform(0, 1, &x, &y);
			DocCoord tl(roundNumber(x), roundNumber(y));
			state->transform(1, 1, &x, &y);
			DocCoord tr(roundNumber(x), roundNumber(y));
			state->transform(1, 0, &x, &y);
			DocCoord br(roundNumber(x), roundNumber(y));
			state->transform(0, 0, &x, &y);
			DocCoord bl(roundNumber(x), roundNumber(y));

			pImage->AddChild(new CXfLineColor(CXfLineColor::None));
			pImage->AddChild(new CXfLineWidth(250));
			double dfa = state->getFillOpacity();
			double dsa = state->getStrokeOpacity();
			BYTE byAlpha, byBlendMode;
			ConvertOpacity(0, state->getBlendMode(), byAlpha, byBlendMode);
			BYTE byEndTrans = BYTE((1.0 - dfa) * 255 + 0.5);
			BYTE byStartTrans = 255;

			pImage->AppendChild(new CXfBitmapTransparentFill(bl, br, tl, byStartTrans, byEndTrans, byBlendMode, pMaskBitmap, 0, 0));
			//pImage->AppendChild(new CXfZeroSizedNode(TAG_FILL_REPEATINGINVERTED));
			//pImage->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_REPEATINGINVERTED));
			pImage->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_NONREPEATING));
		}
	}
STOP_STOPWATCH(__FUNCTION__)
}

/******************************************************************************
   Function:    MergeBitmapData
   Author:      Denis Onischenko / Alex Shumakov
   Purpose:		Actual bitmap merge function
   Returns:     merged bitmap data
   Exceptions:  -
   Parameters:  list of bitmaps to merge
   Notes:       -
******************************************************************************/
XarOutputDev::BitmapMergeData XarOutputDev::MergeBitmapData(BitmapMergeList &bitmaps)
{
	BitmapMergeData res;

	if (bitmaps.empty()) return res;
	if (1 == bitmaps.size()) return bitmaps.front();

	res.iBPC = 8;
	res.iNComps = 4;
	res.isCMYK = bitmaps.front().isCMYK;

	BitmapMergeList::const_iterator it = bitmaps.begin();

	res.left   = it->left;
	res.right  = it->right;
	res.top    = it->top;
	res.bottom = it->bottom;

	for (++it; it != bitmaps.end(); ++it)
	{		
		// Determine bounding box for merged paths
		if (it->left < res.left)
			res.left = it->left;

		if (it->right > res.right)
			res.right = it->right;

		if (it->top > res.top)
			res.top = it->top;

		if (it->bottom < res.bottom)
			res.bottom = it->bottom;
	}
	
	res.width  = roundNumber((res.right - res.left) / bitmapMergePixWidth);
	res.height = roundNumber((res.top - res.bottom) / bitmapMergePixHeight);

	int size = 4 * res.width * res.height;
	if ( !size ) return res;
	res.buf = (unsigned char *)gmalloc(size);
	if ( !res.buf ) return res;

	if (res.isCMYK)
	{
		int cmykAlphaSize = res.width * res.height;
		res.cmykAlphaBuf = (unsigned char *)gmalloc(cmykAlphaSize);
		if ( !res.cmykAlphaBuf )
		{
			gfree(res.buf);
			return res;
		}

		memset(res.cmykAlphaBuf, 0xFF, cmykAlphaSize);
	}

	memset(res.buf, 0xFF, size);
	
	for( BitmapMergeList::const_iterator it = bitmaps.begin(); it != bitmaps.end(); ++it)
	{
		int srcX = 0;
		int srcY = 0;
		int srcWidth  = it->width;
		int srcHeight = it->height;		

		int dstX = roundNumber((it->left - res.left) / bitmapMergePixWidth);
		int dstY = roundNumber((res.top - it->top) / bitmapMergePixHeight);

		if (dstX >= res.width || dstY >= res.height || dstX + srcWidth <= 0 || dstY + srcHeight <= 0) continue;

		if (dstX < 0)
		{
			srcX = -dstX;
			srcWidth += dstX;
			dstX = 0;
		}

		if (dstY < 0)
		{
			srcY = -dstY;
			srcHeight += dstY;
			dstY = 0;
		}

		if (dstX + srcWidth > res.width)
			srcWidth = res.width - dstX;

		if (dstY + srcHeight > res.height)
			srcHeight = res.height - dstY;

		int srcStride = (it->pbyPal && it->iPalCount) ? it->width : it->iNComps * it->width;
		const BYTE *srcRow = (const BYTE *)(it->buf) + srcY * srcStride + srcX * it->iNComps;

		int dstStride = 4 * res.width;
		unsigned char *dstRow = res.buf + dstY * dstStride + dstX * 4;

		int dstAlphaStride = res.width;
		unsigned char *dstAlphaRow = res.cmykAlphaBuf ? res.cmykAlphaBuf + dstY * dstAlphaStride + dstX : NULL;

		for ( int j = 0; j < srcHeight; ++j, srcRow += srcStride, dstRow += dstStride )
		{
			const BYTE *srcPixel = srcRow;
			BYTE *dstPixel = dstRow;

			switch (it->iNComps)
			{
			case 1: // For greyscale images
				if (it->pbyPal && it->iPalCount)
				{
					for ( int i = 0; i < srcWidth; ++i )
					{
						BYTE index = *srcPixel++;
						if (index >= it->iPalCount) index = it->iPalCount - 1;
						BYTE g = it->pbyPal[index];
						*dstPixel++ = g;
						*dstPixel++ = g;
 						*dstPixel++ = g;
						*dstPixel++ = 0;
					}
				}
				else
				{
					for ( int i = 0; i < srcWidth; ++i )
					{
						BYTE g = *srcPixel++;
						*dstPixel++ = g;
						*dstPixel++ = g;
 						*dstPixel++ = g;
						*dstPixel++ = 0;
					}
				}
				break;
			case 3: // For RGB images
				if (it->pbyPal && it->iPalCount)
				{
					for ( int i = 0; i < srcWidth; ++i )
					{						
						BYTE index = *srcPixel++;
						if (index >= it->iPalCount) index = it->iPalCount - 1;
						BYTE *p = it->pbyPal + 3 * index;
						*dstPixel++ = *p++;
						*dstPixel++ = *p++;
						*dstPixel++ = *p;
						*dstPixel++ = 0;
					}
				}
				else
				{
					for ( int i = 0; i < srcWidth; ++i )
					{
						*dstPixel++ = *srcPixel++;
						*dstPixel++ = *srcPixel++;
						*dstPixel++ = *srcPixel++;
						*dstPixel++ = 0;
					}
				}
				break;
			case 4: // For CMYK images
				{
					BYTE *dstAlpha = dstAlphaRow;
					if (it->pbyPal && it->iPalCount)
					{
						for ( int i = 0; i < srcWidth; ++i )
						{						
							BYTE index = *srcPixel++;
							if (index >= it->iPalCount) index = it->iPalCount - 1;
							BYTE *p = it->pbyPal + 4 * index;
							*dstPixel++ = *p++;
							*dstPixel++ = *p++;
							*dstPixel++ = *p++;
							*dstPixel++ = 255 - *p;
							*dstAlpha++ = 0;
						}
					}
					else
					{
						for ( int i = 0; i < srcWidth; ++i )
						{
							*dstPixel++ = *srcPixel++;
							*dstPixel++ = *srcPixel++;
							*dstPixel++ = *srcPixel++;
							*dstPixel++ = *srcPixel++;

							*dstAlpha++ = 0;
						}
					}
				}
				dstAlphaRow += dstAlphaStride;
				break;
			}
		}

		if (it->buf)
			gfree(it->buf);
		if (it->pbyPal)
			delete [] it->pbyPal;
	}

	return res;
}

/******************************************************************************
   Function:	XarOutputDev::MergeSmallBitmaps
   Author:		Denis Onischenko / Alex Shumakov
   Purpose:     Determines if there are some paths on current page which can be merged into bigger paths
				( usually numerous small bitmaps ) and do this merge.
   Returns:		void
   Parameters:	None 
   Notes:       
******************************************************************************/
void XarOutputDev::MergeSmallBitmaps()
{
	if (bitmapMergeList.empty()) return;
	BitmapMergeData bitmapData = MergeBitmapData(bitmapMergeList);
	// Make sure path vector is clear before AddGraphics for avoid recursion
	bitmapMergeList.clear();

	if (!bitmapData.buf) return;

	DocCoord bl, br, tl, tr;

	bl.x = tl.x = roundNumber(bitmapData.left);
	br.x = tr.x = roundNumber(bitmapData.right);
	bl.y = br.y = roundNumber(bitmapData.bottom);
	tl.y = tr.y = roundNumber(bitmapData.top);

	GfxPath path;
	AddRect(&path, bl.x, bl.y, tr.x - bl.x, tr.y - bl.y);
			
	CXfPath* pPath = CreateXaraPath(&path, NULL, TAG_PATH_FILLED_STROKED);
	if (AddGraphics(pPath, pPath, CXF_WINDING_RULE_NONZERO))
	{
		CXfDefineBitmap *pDefineBitmap = new CXfDefineBitmap(bitmapData.isCMYK ? TAG_DEFINEBITMAP_PNG_CMYK : TAG_DEFINEBITMAP_PNG_REAL, nullptr, 0, FormatStringW(L"PDF Bitmap %d", ++s_iBitmapCount).c_str());

		if (bitmapData.isCMYK)
			pDefineBitmap->pProfileRefPtr = GetCMYKProfile();

		pDefineBitmap->Data.reserve(((bitmapData.width * bitmapData.iNComps * bitmapData.iBPC + 7) >> 3) * bitmapData.height);

		EncodePNGFromPDFData(&pDefineBitmap->Data, bitmapData.buf, bitmapData.width, bitmapData.height,
			bitmapData.pbyPal, bitmapData.iPalCount, bitmapData.iBPC, bitmapData.iNComps, true, true, PngWriteData);

		pDefineBitmap->SetColourSpace(bitmapData.isCMYK ? CXfDefineBitmap::CS_CMYK : (bitmapData.iNComps > 1 ? CXfDefineBitmap::CS_RGB : CXfDefineBitmap::CS_GREY));

/*
		if (g_settings.bDumpData)
			pDefineBitmap->Save(FormatStringW(L"d:/temp/PDF_Bitmap%.3d.png", s_iBitmapCount-1).c_str());
*/
		pPath->AppendChild(pDefineBitmap);
			
		// This is a single bitmap fill - make it a photo
		pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_NONREPEATING));
		pPath->AppendChild(new CXfBitmapFill(bl, br, tl, pDefineBitmap, 0, 0));
		pPath->AppendChild(new CXfZeroSizedNode(TAG_PHOTOATTR_FLAG));

		if (bitmapData.cmykAlphaBuf)
		{
			CXfDefineBitmap *pAlphaBitmap = new CXfDefineBitmap(TAG_DEFINEBITMAP_PNG_REAL, nullptr, 0, FormatStringW(L"PDF Bitmap %d", ++s_iBitmapCount).c_str());

			pAlphaBitmap->Data.reserve(bitmapData.width * bitmapData.height);

			EncodePNGFromPDFData(&pAlphaBitmap->Data, bitmapData.cmykAlphaBuf, bitmapData.width, bitmapData.height, NULL, 0, 8, 1, true, true, PngWriteData);

			pAlphaBitmap->SetColourSpace(CXfDefineBitmap::CS_GREY);

			pPath->AppendChild(pAlphaBitmap);
			pPath->AppendChild(new CXfBitmapTransparentFill(bl, br, tl, 0, 255, CXF_TRANSPARANCY_TYPE_MIX, pAlphaBitmap, 0, 0));
			pPath->AppendChild(new CXfZeroSizedNode(TAG_TRANSPARENTFILL_NONREPEATING));
		}

		pPath->AddChild(new CXfLineColor(CXfLineColor::None));
		pPath->AddChild(new CXfLineWidth(250));
		pPath->AddChild(new CXfJoinStyle(CXF_JOIN_STYLE_MITRE)); // Like if you drag a photo from PG
	}
	else
	{
		delete pPath;
	}

	if (bitmapData.buf)
		gfree(bitmapData.buf);
	if (bitmapData.cmykAlphaBuf)
		gfree(bitmapData.cmykAlphaBuf);
	if (bitmapData.pbyPal)
		delete [] bitmapData.pbyPal;
}
