/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

#include <list>

#include "../../poppler/poppler/GlobalParams.h"
#include "../../poppler/poppler/FontInfo.h"
#include "../../poppler/poppler/GfxFont.h"
#include "../../poppler/poppler/GfxState.h"
#include "../../poppler/poppler/PDFDoc.h"
#include "../../poppler/poppler/CharCodeToUnicode.h"
#include "../../poppler/splash/SplashFont.h"
#include "../../poppler/splash/SplashFontFile.h"
#include "../../poppler/splash/SplashFontEngine.h"
#include "../../poppler/splash/SplashFontFileID.h"

#include "common/psd/FontMapper.h"
#include "common/TextManager.h"

#include "../../common/XFTree/XFTree.h"
#include "../../common/AffineTransform.h"

#include "XarOutputSettings.h"

#include "FontDC.h"
#include "OTFDataLoaderCallback.h"

#define MAX_KERN_HIST_IDX 155
#define MAX_KERN_HIST_SIZE (MAX_KERN_HIST_IDX * 2 + 1)

CXfEmbeddedGlyphSizeHint* EnumerateDocumentGlyphs(VXfTreeNode* pChapter);

typedef std::map<int, CXfFontDef*> ExtraNodeMap;

class FontManager;

struct FontMetrics
{
	MILLIPOINT ascent;
	MILLIPOINT descent;
	FontMetrics(): ascent(0), descent(0) {}
};

struct SystemFontInfo
{
	std::wstring fontName;
	int          weight;
	bool         isItalic;

	SystemFontInfo() : weight(400), isItalic(false) {}

	bool operator<(const SystemFontInfo &info) const
	{
		int c =  fontName.compare(info.fontName);
		if( c < 0 ) return true;
		else if( c > 0 ) return false;

		if( weight < info.weight ) return true;
		else if( weight > info.weight ) return false;

		if( !isItalic && info.isItalic ) return true;
		else if( isItalic && !info.isItalic ) return false;

		return false;
	}

};

struct SystemFontMetrics
{
	std::map<wchar_t, MILLIPOINT> widths;
	std::map<DWORD32, MILLIPOINT> kerns;
};

int parseCharName(GfxFont *font, int charCode, Unicode *uBuf, int uLen);

Unicode MapLigature(const Unicode* u, int uLen);
int MapLigature(Unicode u, Unicode *buf);

void AdjustUnicode(Unicode *u, int uLen);

FontMetrics GetFontMetricsOS(const wchar_t *fontName, int weight, bool isItalic);

struct MappedFontDesc: public SystemFontInfo
{
	GfxFont*     m_pFont;
	Object 	     m_fontObj;
	bool         m_bEmbedded;
	SplashFont*  m_pSplashFont;

	typedef std::set<CharCode> CharSet;
	CharSet  m_charSet;



	UINT16 m_fsType;




	CXfFontDef*  node; // the corresponding Xara CXfFontDef record
	CXfFontDef*  pFontDefOrg; // this is the original font def, as 'node' attribute can be altered to point to '-emb' variant
	ExtraNodeMap extraNode; // this glyph [by index] is moved to another font def: either because previous glyph 
	// from different mfd (or even the same) was already created and had the same Unicode value as this one, or the glyph 
	// differs from the system font file
	//std::map<int, MILLIPOINT> mGlyphWidthCache; // This is a cache for this particular CHARCODE of this mfd and a glyph width that is
	// assigned to it. Even if the font is not embedded, the width is assigned and taken from the system from our font substitute.

	//std::wstring  fontName;
	std::wstring  fontNamePS;
	std::wstring  fontNameRaw;
	//int           weight;
	bool          isBold; // this flag has the following meaning: if current font is installed in the system, this particular
	                      // font weight described in this instance is not the lowest weight of the same typeface present in the system
	                      // If font in not installed, this should be false
	//bool          isItalic;
	bool          bPresentInTheSystem; // according to our font map
	MILLIPOINT    ascent;
	MILLIPOINT    descent;
	MILLIPOINT    emWidth;

	double        scale;

	SystemFontInfo substitute;

	std::set<wchar_t> unicodeSubset;
	std::map<int, int> charToUni; // given current charcode, stores Unicode value that is passed to Xara
	std::map<int, int> badmap; // this maps 'bad' unicode glyphs to correct ones: 0-31 invalid Unicode range is mapped to a sane range
	std::map<wchar_t, Rect> bboxMap;

	std::map<std::wstring, Unicode> nsLigatureMap;
		
	// parsing used fonts to use them later at font embedding stage
	FontManager* pFontManager;

	MappedFontDesc(FontManager* _pFontManager = NULL) 
		: pFontManager(_pFontManager)


		, m_pFont(nullptr)
		, m_fontObj(objNull)
		, m_bEmbedded(false)
		, m_pSplashFont(nullptr)

		, m_fsType(0)

		//, weight(400)
		, isBold(false)
		//, isItalic(false)
		, node(NULL)
		, emWidth(0)
		, ascent(0)
		, descent(0)
		, scale(1.0)
		, bPresentInTheSystem(false)
		, pFontDefOrg(NULL)
	{}

	virtual ~MappedFontDesc()
	{
		//if (m_pSplashFont)
		//	delete m_pSplashFont;

		if (m_pFont)
			m_pFont->decRefCnt();
	}

	void AddCharacter(wchar_t cUnicode) { unicodeSubset.insert(cUnicode); }
	bool CharacterExists(wchar_t wUnicode) { return unicodeSubset.find(wUnicode) != unicodeSubset.end(); }
	//bool LigatureMappingExists(int iCharcode) { return lctum.find(iCharcode) != lctum.end(); }

	MILLIPOINT GetCharWidthOS(wchar_t u); //Platform specific! returns character width of unicode
	MILLIPOINT GetKernSizeOS(wchar_t c1, wchar_t c2); //Platform specific!
	bool GetCharBoundingBoxOS(wchar_t u, Rect& rcBounds); // Platform specific: get the bbox of a char of this font from the system, if exists
	bool GlyphEmbeddedExtra(int charCode) { return extraNode.find(charCode) != extraNode.end(); }
	CXfFontDef* GetFontDef(int charCode, bool& bGlyphSubstituted);

	const SystemFontInfo &GetSubstitute(CXfFontDef* pFontDef, bool bBold, bool bItalic);

	FontMetrics GetFontMetricsOS();

	//bool AdjustPreciseCharRect(wchar_t c, Rect& rc);
};

// Single weight, a collection of MappedFontDesc's, both regular and italic
class MfdWeightData : public std::list<MappedFontDesc*>
{
public:
	MfdWeightData() : m_uiNextPrivateUnicodeRegular(0xE000), m_uiNextPrivateUnicodeItalic(0xE000) {}
	UINT m_uiNextPrivateUnicodeRegular;
	UINT m_uiNextPrivateUnicodeItalic;
};
// Map: weight ---> list of mfd's
class MfdWeightCollection : public std::map<int, MfdWeightData>
{
public:
	//CXfFontDef* GetFontDef(int w) { return find(w) != end() ? (*this)[w] : NULL; }
	void AddMfd(int w, MappedFontDesc* mfd)
	{
		(*this)[w].push_back(mfd);

		/*MfdWeightCollection::iterator it = find(w);
		if(it != end())
			it->second.push_back(mfd);
		else
		{
			(*this)[w] = MfdWeightData();
			(*this)[w].push_back(mfd);
		}*/
	}
};

class MfdMap : public std::map<std::wstring, MfdWeightCollection>
{
public:
	bool CheckWeight(std::wstring strName, int w)
	{
		if(find(strName) != end())
		{
			return (*this)[strName].find(w) != (*this)[strName].end();
		}

		return false;
	}

	UINT& GetNextPrivateUnicode(std::wstring strName, int w, bool bItalic) { return bItalic ? (*this)[strName][w].m_uiNextPrivateUnicodeItalic : (*this)[strName][w].m_uiNextPrivateUnicodeRegular; }
	// We get the font def from the FIRST mfd of the list, bearing in mind that it is shared between all elements of the list
	CXfFontDef* GetFontDef(std::wstring strName, int w)
	{
		if(find(strName) != end())
		{
			MfdWeightData& vec = (*this)[strName][w];
			if(vec.size())
				return vec.front()->node;
		}
		return NULL;
	}

	void AddFont(MappedFontDesc* mfd)
	{
		/*MfdWeightCollection newCollection;

		if(find(mfd->fontName) == end())
			(*this)[mfd->fontName] = newCollection;

		MfdWeightCollection& fwc = (*this)[mfd->fontName];

		fwc.AddMfd(mfd->weight, mfd);*/

		(*this)[mfd->fontName].AddMfd(mfd->weight, mfd);
	}
};

class GlyphDiffDocument
{
public:
	GlyphDiffDocument(std::wstring filename);
	void AddDiffGlyph(MappedFontDesc& mfd, PathVerb* pVerb, DocCoord* pCoord, ULONG ulNumCoords, PathVerb* pSysVerb, DocCoord* pSysCoord, ULONG ulNumSysCoords, std::wstring strMessage);
	void ScanAddFonts(VXfTreeNode* pRoot);
	void AddFont(CXfFontDef* pFont) { m_fonts.push_back(pFont); }

	virtual ~GlyphDiffDocument();

public:
	CXfXaraTree m_tree;
	std::wstring m_strDumpFilename;
	std::wstring m_strSource;
	bool m_bActive = false;
	bool m_bFirstPage = true;
	bool m_bDumpEmbeddedFonts = false;
	VXfTreeNode* m_pChapter = NULL;
	std::vector<CXfFontDef*> m_fonts;
};

class FontManager
{
	MfdMap      mfdMap; // typeface --> map:[weight --> mfd list]

	SplashFontEngine *fontEngine;

	GlobalParams* m_pGlobalParams;

	const wchar_t*           m_filename;
	VXfTreeNode*	         m_pChapter;
	int                      m_type3FontNameCount;

	CFontMapper fontMapper;
	GlyphDiffDocument m_dumpGlyphs; // only for debug purposes glyph diff document
	typedef std::list<CXfFontDef*> FontDefList;
	typedef std::map<CXfFontDef*, FontDefList> ExtraFontDefMap;
	ExtraFontDefMap extraDefs;
	//FontDefMap m_FontGlyphStoreMap;

public:
	PDFDoc*			 m_pDoc;

	std::map<std::string, std::wstring> mBuiltInFonts; // that's the set of font names that are distributed together with pdf2xar
	// AND it's name to be assigned and passed to Camelot

	typedef std::map<Ref, MappedFontDesc> FontMap;
	FontMap m_fontMap; // ref -> MappedFontDef

	FontManager(const wchar_t* filename, VXfTreeNode* pChapter, PDFDoc* pDoc, std::string strFilterDataDir, GlobalParams* pGlobalParams);
	~FontManager();

	std::wstring FindFontSubstitute(CXfFontDef* pFontDef, bool isBold, bool isItalic, int &substituteWeight, bool &substituteIsItalic);

	UINT MapCharacter(MappedFontDesc& mfd, UINT uiUnicode, UINT uiCharcode, bool bEmptyGlyph);
	Unicode MapNonStandardLigature(MappedFontDesc& mfd, Unicode* u, int uLen);

	bool CompareGlyphs(CXfFontGlyph* pGlyph1, CXfFontGlyph* pGlyph2);

	CXfFontGlyph* FindGlyph(wchar_t wUnicode, CXfFontDef* pFontDef, bool bBold, bool bItalic);
	//MILLIPOINT GetGlyphWidth(int charCode, wchar_t wUnicode, MappedFontDesc* pFontDesc);
	bool EmbedGlyph(MappedFontDesc& mfd, CharCode charCode, int iUnicode, bool bSubset, bool bNeedGlyph, bool bCanChangeNode, bool& bFontDiffersFromSystem);
	void AssignSpaceCharacter(MappedFontDesc* pmfd, wchar_t ch);
	void AssignSpaceCharacters(MappedFontDesc* pmfd);

	//bool EmbedDocumentFontGlyphs(bool bSubset, VXfTreeNode* pChapter);

	SplashFont* CreateSplashFont(GfxFont *gfxFont, SplashCoord mat[4], bool bUseFontCache = true);

	void CreateSplashFont(MappedFontDesc &fontDesc);
	MappedFontDesc &GetFontDesc(GfxFont* pFont);
	MappedFontDesc &GetFontDesc(const Ref &ref);

	MappedFontDesc *GetFont(GfxState *state);
	void InitialiseBuiltInFonts(std::string strFilterDataDir);
	CXfFontDef* FindOrCreateExtraFontDef(const MappedFontDesc &mfd, CXfFontGlyph* pGlyph = NULL, bool *pGlyphExists = NULL);

	void InitMappedFontDesc(MappedFontDesc &mappedFontDesc);
	void CreateFontDefs(bool bEmbedGlyphs, bool bSubset);

	GlyphDiffDocument& GetGlyphDiffDocument() { return m_dumpGlyphs; }

private:
	bool ProcessFontGlyphs(CXfFontDef* pFontDef, MappedFontDesc& mfd, bool embed, bool bSubset, bool bAlterFontDef = true);

	void CreateType3MappedFontDesc(MappedFontDesc &mappedFontDesc);

	//CXfFontDef* CreateNewFontEx(MappedFontDesc& mappedFontDesc, bool bEmbedGlyphs);

	double CompareOutlines(PBYTE Verbs1, DocCoord* Coords1, DWORD dwCount1, PBYTE Verbs2, DocCoord* Coords2, DWORD dwCount2, CGlyphDiff& diff);

	bool CompareGlyphWithSystem(MappedFontDesc& mfd, CXfFontGlyph* pGlyph, CGlyphDiff& diff, const std::wstring& ligChars);

	bool FontExists(const MappedFontDesc& mfd);
};
