/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "OTFDataLoaderCallback.h"

CXfFontOTLigatures *GetLigatures(CXfFontDef* pFontDef, bool bold, bool italic, bool bCreate)
{
	CXfFontOTLigatures *pFontOTLigatures = pFontDef->GetFontLigatures(bold, italic);
	if (bCreate && !pFontOTLigatures)
	{
		pFontOTLigatures = new CXfFontOTLigatures(BI_CODE(bold, italic));
		pFontDef->AddLigatures(pFontOTLigatures);

		VXfTreeNode *pInsertAfter = pFontDef;
		while (pInsertAfter->NextSibling() && TAG_FONT_GLYPH == pInsertAfter->NextSibling()->GetTag())
			pInsertAfter = pInsertAfter->NextSibling();

		pInsertAfter->GetParent()->InsertAfterChild(pInsertAfter, pFontOTLigatures);
	}

	return pFontOTLigatures;
}

std::vector<XarLigatureEntry>::iterator GetLigatureEntry(CXfFontOTLigatures *pFontOTLigatures, const std::wstring& ligChars, bool bCreate)
{
	std::vector<XarLigatureEntry>::iterator b = pFontOTLigatures->ligs.begin();	
	std::vector<XarLigatureEntry>::iterator e = pFontOTLigatures->ligs.end();

	std::vector<XarLigatureEntry>::iterator itLigature = e;	

	while( b != e )
	{
		std::vector<XarLigatureEntry>::iterator p = b + (e - b) / 2;

		size_t maxSize = max(p->Key.size(), ligChars.size());

		int cmp = 0;
		for (size_t i = 0; i < maxSize; ++i)
		{
			WCHAR c1 = i < p->Key.size() ? p->Key[i] : 0;
			WCHAR c2 = i < ligChars.size() ? ligChars[i] : 0;
			if (c1 == c2)
				continue;
			cmp = c1 < c2 ? -1 : 1;
			break;
		}

		/*if (p->KeyLth < ligLen)
			cmp = 1;
		else if (p->KeyLth > ligLen)
			cmp = -1;
		else
			cmp = wcsncmp(p->Key, ligChars, ligLen);*/

		if( 0 == cmp )
		{
			itLigature = p;
			break;
		}
		else if( cmp > 0 ) e = p;
		else b = p + 1;
	}		

	if (bCreate && pFontOTLigatures->ligs.end() == itLigature)
	{
		XarLigatureEntry lig;
		lig.KeyLth = (BYTE)ligChars.size();
		lig.Key = ligChars;
		itLigature = pFontOTLigatures->ligs.insert(e, lig);
	}

	return itLigature;
}

COTFDataLoaderCallback::COTFDataLoaderCallback(CXfFontOTLigatures *pFontOTLigatures): m_pFontOTLigatures(pFontOTLigatures)
{
}

bool COTFDataLoaderCallback::UseOpenTypeSubstitutionData()
{
	return true;
}

void COTFDataLoaderCallback::SetUnicodeMap(const std::unordered_map<WCHAR, DWORD>& Map)
{
}

void COTFDataLoaderCallback::SetLigatureData(std::unique_ptr<LigatureEntry[]> pLigEntries, const UINT& LigCnt)
{
	const LigatureEntry *pEnd = pLigEntries.get() + LigCnt;
	for (const LigatureEntry *pEntry = pLigEntries.get(); pEntry != pEnd; ++pEntry)
	{
		std::vector<XarLigatureEntry>::iterator itLigature = GetLigatureEntry(m_pFontOTLigatures, pEntry->Key, true);
		itLigature->Ligature = pEntry->Ligature;
		itLigature->Priority = pEntry->Priority;
		itLigature->GlyphID  = pEntry->GlyphID;
		itLigature->LigType  = pEntry->LigType;
	} 
}

void COTFDataLoaderCallback::SetStylisticSetData(std::unique_ptr<StylisticSetDataSet> pSSetDataSet, std::unique_ptr<StylisticSetSizeSet> pSSetSizeSet)
{
}

void COTFDataLoaderCallback::SetStylisticSetNames(std::unique_ptr<StylisticSetNameSet> pStylisticSetNames)
{
}

void COTFDataLoaderCallback::SetMaxLigatureSize(INT MaxSize)
{
}

void COTFDataLoaderCallback::SetTypoMetricsData(const TypoMetrics& i_metrics)
{
}
