/* ================================XARAHEADERSTART===========================
PDF2XAR, a PDF file to XAR file converter
Copyright (C) 1993-2014 Xara Group Ltd.
Copyright on certain contributions may be held in joint with their
respective authors.


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page &
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks,
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XarOutputDev.h"
#include "splash/splashbitmap.h"
#include "PDFBitmap.h"

void Convert4BPP2BYTE(unsigned char* src, unsigned char* dst, int width)
{
	for (int i = 0; i < width / 2; i++)
	{
		*dst++ = (*src) >> 4;
		*dst++ = (*src++) & 0x0F;
	}
	if (width % 2)
		*dst = (*src) >> 4;
}

/******************************************************************************
   Function:	ConvertBits2BYTE
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		void
   Parameters:	PBYTE pbySrc  
   Parameters:	PBYTE pbyDst  
   Parameters:	int iWidth  
   Parameters:	int iNumBits  
   Parameters:	int iNComp  
   Notes:       
******************************************************************************/
void ConvertBits2BYTE(PBYTE pbySrc, PBYTE pbyDst, int iWidth, int iNumBits, int iNComp)
{
	iNComp *= iWidth;
	
	if (16 == iNumBits)
	{
		for (int i = 0; i < iNComp; i++)
		{
			*pbyDst++ = *pbySrc;
			pbySrc += 2;
		}
	}
	else
	{
		int iNumPixInByte = 8 / iNumBits;
		int iMask = (1 << iNumBits) - 1;
		for (int i = 0; i < iNComp; i++)
		{
			int iShift = ((iNumPixInByte - 1) - (i % iNumPixInByte)) * iNumBits;
			int iIndex = (pbySrc[i / iNumPixInByte] >> iShift) & iMask;
			*pbyDst++ = iIndex;
		}
	}
}

/******************************************************************************
   Function:	ConvertTo8BPC
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		unsigned char*
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	unsigned char * buf  
   Parameters:	GfxImageColorMap * colorMap  
   Notes:       
******************************************************************************/
unsigned char* ConvertTo8BPC(int width, int height, unsigned char* buf, GfxImageColorMap* colorMap, bool bCMYK)
{
	GfxColorSpace* colorSpace = colorMap->getColorSpace();

	int iBPS = colorMap->getBits();
	int iNComps = colorMap->getNumPixelComps();
	bool bToByte;

	if (bCMYK)
		bToByte = iBPS != 8 && colorMap->useCMYKLine();// && (colorSpace->getMode() != csIndexed && colorSpace->getMode() != csSeparation);
	else
		bToByte = iBPS != 8 && colorMap->useRGBLine();// && (colorSpace->getMode() != csIndexed && colorSpace->getMode() != csSeparation);

	int iSrcRowLen = (width * iBPS * iNComps + 7) >> 3;
	int iDstRowLen = (bCMYK ? 4 : 3) * width;
	unsigned char* dst = (unsigned char*)gmalloc(iDstRowLen * height);
	unsigned char* src_temp = bToByte ? (unsigned char*)gmalloc(width * iNComps) : NULL;

	unsigned char* srcRow = buf;
	unsigned char* dstRow = dst;

	for (int i = 0; i < height; ++i, srcRow += iSrcRowLen, dstRow += iDstRowLen)
	{
		if (bToByte) ConvertBits2BYTE(srcRow, src_temp, width, iBPS, iNComps);
		unsigned char* ptr = bToByte ? src_temp : srcRow;
		// If useRGBLine returns false, then getRGBLine can't be used if iBPS < 8 because of the following line:
		// inp += nComps; - the input buffer increments by byte per each component 
		if (bCMYK)
			colorMap->getCMYKLine(ptr, dstRow, width);
		else
			colorMap->getRGBLine(ptr, dstRow, width);
	}

	if (src_temp) gfree(src_temp);
	return dst;
}

/******************************************************************************
   Function:    CreateMask
   Author:      Paul Gruzdev
   Purpose:		Create a 1-bpp image mask, based on an
				image stream buffer and colour key range of indexes(values) that need 
				to be excluded (transparent)
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
PBYTE CreateMask(unsigned char* pbySrc, int width, int height, GfxImageColorMap* colorMap, int* maskColors)
{
	int iNumBits = colorMap->getBits();
	int iNComp = colorMap->getNumPixelComps();
	int srcRowStride = (width * iNumBits * iNComp + 7) >> 3;
	int dstRowStride = (width + 7) >> 3; // number of bytes for monochrome 1-bit image mask one row
	int dstBufSize = dstRowStride * height;
	PBYTE pbyDst = new BYTE[dstBufSize];
	memset(pbyDst, 0, dstBufSize); // Fill every mask pixel with transparency = 0 (all opaque)
	PBYTE pbyDstPtr = pbyDst;

	int iNumPixInByte = 8 / iNumBits;
	int iMask = (1 << iNumBits) - 1;
	for (int i = 0; i < height; i++)
	{
		for (int j = 0; j < width; j++)
		{
			bool bTransparent = true;
			for (int k = 0; k < iNComp; k++)
			{
				int iIdx = j * iNComp + k;
				int iShift = ((iNumPixInByte - 1) - (iIdx % iNumPixInByte)) * iNumBits;
				int iIndex = (pbySrc[iIdx/iNumPixInByte] >> iShift) & iMask;
				bTransparent &= (iIndex >= maskColors[k * 2] && iIndex <= maskColors[k * 2 + 1]);
			}
			int iByteIdx = j / 8;
			int iShift = 7 - (j % 8);
			BYTE byVal = (bTransparent ? 1 : 0) << iShift;
			pbyDstPtr[iByteIdx] |= byVal;
		}
		pbySrc += srcRowStride;
		pbyDstPtr += dstRowStride;
	}

	return pbyDst;
}

void GetDecodeInfo(GfxImageColorMap* colorMap, bool &bDefaultDecode, bool &bInvertDecode)
{
	GfxColorSpace* pSpace = colorMap->getColorSpace();
	int bpc = colorMap->getBits();
	int nComps = colorMap->getNumPixelComps();

	std::vector<double> decodeLow;
	std::vector<double> decodeHigh;

	if (nComps)
	{	
		decodeLow.resize(nComps, 0.0);
		decodeHigh.resize(nComps, 0.0);

		pSpace->getDefaultRanges(&decodeLow[0], &decodeHigh[0], (1 << bpc) - 1);

		for (int i = 0; i < nComps; ++i)
			decodeHigh[i] += decodeLow[i];
	}

	bDefaultDecode = true;
	bInvertDecode = true;

	for (int i = 0; i < nComps; ++i)
	{
		if (decodeLow[i] != colorMap->getDecodeLow(i) || decodeHigh[i] != colorMap->getDecodeHigh(i))
			bDefaultDecode = false;
		else if (decodeLow[i] != colorMap->getDecodeHigh(i) || decodeHigh[i] != colorMap->getDecodeLow(i))
			bInvertDecode = false;
	}
}

/******************************************************************************
   Function:	PDFBitmap::Create
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		bool
   Parameters:	Stream * str  
   Parameters:	int width  
   Parameters:	int height  
   Parameters:	GfxImageColorMap * colorMap  
   Notes:       
******************************************************************************/
bool PDFBitmap::Create(Stream* str, int width, int height, GfxImageColorMap* colorMap, int* maskColors)
{
	GfxColorSpace* pSpace = colorMap->getColorSpace();
	m_bpc = colorMap->getBits();
	m_nComps = colorMap->getNumPixelComps();

	m_width = width;
	m_height = height;

	bool isCMYK = (csDeviceCMYK == GetBaseColorSpace(pSpace)->getMode());

	GfxColorSpace* pAltSpace = pSpace->getMode() == csICCBased ? ((GfxICCBasedColorSpace*)pSpace)->getAlt() : pSpace;

	bool bDefaultDecode = true, bInvertDecode = false;
	GetDecodeInfo(colorMap, bDefaultDecode, bInvertDecode);

	int row_stride = (width * m_bpc * m_nComps + 7) >> 3;
	int size = height * row_stride;
	m_data = new unsigned char[size];	
	str->reset();
	int actuallyGot = str->doGetChars(size, m_data);

	// We should create a mask before the conversion - do it here
	if (maskColors)
		m_mask = CreateMask(m_data, width, height, colorMap, maskColors);

	m_palette = nullptr;
	m_palCount = 0;

	if (!isCMYK && 1 == m_nComps && m_bpc <= 8 && (csDeviceGray != pAltSpace->getMode() || !bDefaultDecode && !bInvertDecode))
	{
		if (csIndexed == pSpace->getMode() && bDefaultDecode)
			m_palCount = ((GfxIndexedColorSpace*)pSpace)->getIndexHigh() + 1; // The high or max index plus one is the number of colours in the palette
		else
			m_palCount = 1 << m_bpc;

		if (isCMYK)
		{
			BYTE *p = m_palette = new BYTE[m_palCount * 4];
			for (int i = 0; i < m_palCount; ++i)
			{
				unsigned char index = i;
				GfxCMYK cmyk;
				colorMap->getCMYK(&index, &cmyk);
				*p++ = colToByte(cmyk.c);
				*p++ = colToByte(cmyk.m);
				*p++ = colToByte(cmyk.y);
				*p++ = 255 - colToByte(cmyk.k); // inverse the K
			}
			m_nComps = 4;
		}
		else
		{
			BYTE *p = m_palette = new BYTE[m_palCount * 3];
			for (int i = 0; i < m_palCount; ++i)
			{
				unsigned char index = i;
				GfxRGB rgb;
				colorMap->getRGB(&index, &rgb);
				*p++ = colToByte(rgb.r);
				*p++ = colToByte(rgb.g);
				*p++ = colToByte(rgb.b);
			}
			m_nComps = 3;
		}	
	}
	else
	{
		bool bConvert = false;

		switch (pAltSpace->getMode())
		{
		case csDeviceGray:
			bConvert = !bDefaultDecode && !bInvertDecode;
			break;
		case csDeviceRGB:
		case csDeviceCMYK:
			bConvert = !bDefaultDecode && !bInvertDecode || m_bpc < 8;
			break;
		default:
			bConvert = true;
			break;
		}

		if (bConvert)
		{
			unsigned char* new_data = ConvertTo8BPC(width, height, m_data, colorMap, isCMYK);
			gfree(m_data);
			m_data = new_data;
			m_bpc = 8;
			m_nComps = isCMYK ? 4 : 3;
		}
		else if (bInvertDecode)
		{
			unsigned char *p = m_data;
			for (int i = 0; i < size; ++i, ++p)
				*p = *p ^ 0xFF;
		}
	}

	return true;
}

PDFBitmap::~PDFBitmap()
{
	if (m_data) 
		gfree(m_data);

	if (m_mask) 
		delete [] m_mask;

	if (m_palette)
		delete [] m_palette;
}
