/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "Panose.h"
#include "FontDC.h"

#define FIXEDWIDTH (1 << 0)  /* Fixed-width font */
#define SERIF      (1 << 1)  /* Serif font */
#define SYMBOLIC   (1 << 2)  /* Symbolic font */
#define SCRIPT     (1 << 3)  /* Script font */
#define STANDARD   (1 << 5)  /* Uses the Adobe Standard Character Set */
#define ITALIC     (1 << 6)  /* Italic */
#define ALLCAP     (1 << 16) /* All-cap font */
#define SMALLCAP   (1 << 17) /* Small-cap font */
#define FORCEBOLD  (1 << 18) /* Force bold at small text sizes */

/******************************************************************************
   Function:	GetSystemPANOSE
   Author:		Paul Gruzdev
   Purpose:     Get PANOSE from the system
   Returns:		bool
   Parameters:	std::wstring strWinName  
   Parameters:	int weight  
   Parameters:	bool isItalic  
   Parameters:	PBYTE pbyPanose  
   Notes:       
******************************************************************************/
bool GetSystemPANOSE(const wchar_t *strWinName, int weight, bool isItalic, PBYTE pbyPanose)
{
	_ASSERT(weight>0);

	CFontDC hdc(strWinName, weight, isItalic);

	static const size_t LENGTH_FONT_FACE = 512;
	wchar_t fontFace[LENGTH_FONT_FACE];
	GetTextFaceW(hdc, LENGTH_FONT_FACE, fontFace);
	if (wcsncmp(fontFace, strWinName, LENGTH_FONT_FACE))
		return false;

	UINT uiCount = GetOutlineTextMetrics(hdc, 0, NULL);
	PBYTE pbyData = new BYTE[uiCount];
	GetOutlineTextMetrics(hdc, uiCount, (LPOUTLINETEXTMETRIC)pbyData);
	memcpy(pbyPanose, (void*)&((LPOUTLINETEXTMETRIC)pbyData)->otmPanoseNumber, sizeof(PANOSE));
	delete pbyData;

	return true;
}

/******************************************************************************
   Function:	ConstructPANOSE
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		bool
   Parameters:	BYTE * panose  
   Parameters:	GfxFont * font  
   Parameters:	Dict * pFontDict  
   Notes:       
******************************************************************************/
bool ConstructPANOSE(BYTE* panose, GfxFont* font, Dict* pFontDict)
{
	double dCapHeight = 0, dXHeight = 0;
	double dStemH = 0, dStemV = 0;
	int flags = font->getFlags();
	Object objDesc = pFontDict->lookup("FontDescriptor");

	if (objDesc.isDict())
	{
		Dict* pDescDict = objDesc.getDict();
		Object objTemp = pDescDict->lookup("CapHeight");
		dCapHeight = objTemp.isNum() ? objTemp.getNum() : 0;
		objTemp = pDescDict->lookup("XHeight");
		dXHeight = objTemp.isNum() ? objTemp.getNum() : 0;
		objTemp = pDescDict->lookup("StemH");
		dStemH = objTemp.isNum() ? objTemp.getNum() : 0;
		objTemp = pDescDict->lookup("StemV");
		dStemV = objTemp.isNum() ? objTemp.getNum() : 0;
	}

	memset(panose, 0, 10);
	// Family kind
	panose[0] = 2;
	if (flags & SYMBOLIC)
		panose[0] = 5;

	if (flags & SCRIPT)
		panose[0] = 3;
	// Serif style
	if (flags & SERIF)
		panose[1] = 2;
	else
		panose[1] = 0xB;
	// Weight
	if (font->getWeight() != GfxFont::WeightNotDefined)
		panose[2] = (int)font->getWeight() + 1;
	else
		panose[2] = 6;

	if (flags & FORCEBOLD)
		panose[2] = 8;
	// Proportion
	if (flags & FIXEDWIDTH)
		panose[3] = 9;
	else if (font->getStretch() != GfxFont::StretchNotDefined)
	{
		switch (font->getStretch())
		{
		case GfxFont::UltraCondensed:
		case GfxFont::ExtraCondensed:
			panose[3] = 8;
			break;
		case GfxFont::Condensed:
		case GfxFont::SemiCondensed:
			panose[3] = 6;
			break;
		case GfxFont::Normal:
			(flags & SERIF) ? 3 : 4;
			break;
		case GfxFont::SemiExpanded:
		case GfxFont::Expanded:
			panose[3] = 5;
			break;
		case GfxFont::ExtraExpanded:
		case GfxFont::UltraExpanded:
			panose[3] = 7;
			break;
		};
	}
	else
		panose[3] = (flags & SERIF) ? 3 : 4;
	// Contrast
	if (dStemH > 0 && dStemV > 0)
	{
		double dContrast = dStemH / dStemV;
		if (dContrast > 0.8)
			panose[4] = 2; // None
		else if (dContrast > 0.65)
			panose[4] = 3; // Very low
		else if (dContrast > 0.48)
			panose[4] = 4; // low
		else if (dContrast > 0.3)
			panose[4] = 5; // Medium low
		else if (dContrast > 0.2) // Medium
			panose[4] = 6;
		else if (dContrast > 0.15) // Medium high
			panose[4] = 7;
		else if (dContrast > 0.08) // High
			panose[4] = 8;
		else if (dContrast <= 0.08) // Very high
			panose[4] = 9;
	}
	if (flags & ITALIC)
		panose[7] = 9;
	else
		panose[7] = 2;

	if (dCapHeight > 0.0 && dXHeight > 0.0)
	{
		double dRat = dXHeight / dCapHeight;
		if (dRat < 0.5)
			panose[9] = 2;
		else if (dRat < 0.66)
			panose[9] = 3;
		else if (dRat >= 0.66)
			panose[9] = 4;
	}

	return true;
}

/******************************************************************************
   Function:	ExtractPANOSEandIBMInfo
   Author:		Paul Gruzdev
   Purpose:     Extract additional info about the font: IBM info and PANOSE array
   Returns:		bool - true if extracted
   Parameters:	Dict * pFontDict  
   Parameters:	BYTE & byIBMClassID - The font class and font subclass are registered 
				values assigned by IBM to each font family. This parameter is 
				intended for use in selecting an alternate font when the requested 
				font is not available. The font class is the most general and the 
				font subclass is the most specific.
					Class ID = 0 No Classification
					Class ID = 1 Oldstyle Serifs
					Class ID = 2 Transitional Serifs
					Class ID = 3 Modern Serifs
					Class ID = 4 Clarendon Serifs
					Class ID = 5 Slab Serifs
					Class ID = 7 Freeform Serifs
					Class ID = 8 Sans Serif
					Class ID = 9 Ornamentals
					Class ID = 10 Scripts
					Class ID = 12 Symbolic
   Parameters:	BYTE & byIBMSubclassID  
   Parameters:	BYTE * panose  
   Notes:       
******************************************************************************/
bool ExtractPANOSEandIBMInfo(Dict* pFontDict, BYTE& byIBMClassID, BYTE& byIBMSubclassID, BYTE* panose)
{
	Object dscf = pFontDict->lookup("DescendantFonts");	
	if (dscf.isArray())
	{
		Object font = dscf.getArray()->get(0);
		if (font.isDict())
		{
			Object desc = font.getDict()->lookup("FontDescriptor");
			if (desc.isDict())
			{
				Object style = desc.getDict()->lookup("Style");
				if (style.isDict())
				{
					Object str = style.getDict()->lookup("Panose");
					if (str.isString())
					{
						BYTE* pns = (BYTE*)(str.getString()->c_str());
						memcpy(panose, pns + 2, 10);
						byIBMClassID = pns[0];
						byIBMSubclassID = pns[1];
// 						wprintf(L"%s: IBM: %d %d PANOSE: %d %d %d %d %d %d %d %d %d %d\n", filename.c_str(), pns[0], pns[1],
// 							pns[2], pns[3], pns[4], pns[5], pns[6], pns[7], pns[8], pns[9], pns[10], pns[11]);
						return true;
					}
				}
			}
		}
	}
	return false;
}

/******************************************************************************
   Function:	MergePANOSE
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		void
   Parameters:	PBYTE panose1  
   Parameters:	PBYTE panose2  
   Notes:       
******************************************************************************/
void MergePANOSE(PBYTE panose1, const PBYTE panose2)
{
	for (int i = 0; i < 10; i++)
	{
		if(i == 2 || i == 7) // weight or letterform
		{
			if(panose1[i] && panose2[i]) // Both are valid - pick the lowest weight to be a 'normal' one
				panose1[i] = min(panose1[i], panose2[i]);
			else if(!panose1[i])
				panose1[i] = panose2[i];
		}
		else if (!panose1[i] && panose2[i])
			panose1[i] = panose2[i];
	}
}
