/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "GfxState.h"
#include "poppler/Page.h"
#include "common/XFTree/XFTree.h"
#include "splash/splash.h"
#include "splash/splashbitmap.h"
#include "splash/SplashPath.h"
#include "PathHelpers.h"

inline void ADJUST_RECT_XY(PDFRectangle* rect, double x,double y) { if(x<rect->x1) rect->x1=x; if(x>rect->x2) rect->x2=x; if(y<rect->y1) rect->y1=y; if(y>rect->y2) rect->y2=y; }
inline void ADJUST_RECT_XY(RECT* rect, LONG x, LONG y) { if(x<rect->left) rect->left=x; if(x>rect->right) rect->right=x; if(y<rect->top) rect->top=y; if(y>rect->bottom) rect->bottom=y; }

#define MaxCurveSplits (1 << 10)

inline double avg(double x, double y) 
{
	return 0.5 * (x + y);
}

inline void CheckCapacity(ClipperLib::Path& p)
{
	if (p.size() == p.capacity())
		p.reserve(p.size() + 1000);
}

/******************************************************************************
   Function:    FlattenCurve
   Author:      Paul Gruzdev
   Purpose:     Convert a bezier segment of the path to a set of lines
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void FlattenCurve(double x0, double y0,
				  double x1, double y1,
				  double x2, double y2,
				  double x3, double y3,
				  double flatness2,
				  ClipperLib::Path& out, RECT& rcBounds)
{
	double cx[MaxCurveSplits + 1][3];
	double cy[MaxCurveSplits + 1][3];
	int cNext[MaxCurveSplits + 1];
	double xl0, xl1, xl2, xr0, xr1, xr2, xr3, xx1, xx2, xh;
	double yl0, yl1, yl2, yr0, yr1, yr2, yr3, yy1, yy2, yh;
	double dx, dy, mx, my, tx, ty, d1, d2;
	int p1, p2, p3;

	// initial segment
	p1 = 0;
	p2 = MaxCurveSplits;
	cx[p1][0] = x0;  cy[p1][0] = y0;
	cx[p1][1] = x1;  cy[p1][1] = y1;
	cx[p1][2] = x2;  cy[p1][2] = y2;
	cx[p2][0] = x3;  cy[p2][0] = y3;
	cNext[p1] = p2;

	while (p1 < MaxCurveSplits) 
	{
		// get the next segment
		xl0 = cx[p1][0];  yl0 = cy[p1][0];
		xx1 = cx[p1][1];  yy1 = cy[p1][1];
		xx2 = cx[p1][2];  yy2 = cy[p1][2];
		p2 = cNext[p1];
		xr3 = cx[p2][0];  yr3 = cy[p2][0];

		// compute the distances (in device space) from the control points
		// to the midpoint of the straight line (this is a bit of a hack,
		// but it's much faster than computing the actual distances to the
		// line)
		mx = (xl0 + xr3) * 0.5;
		my = (yl0 + yr3) * 0.5;
		tx = xx1;
		ty = yy1;

		dx = tx - mx;
		dy = ty - my;
		d1 = dx*dx + dy*dy;

		tx = xx2; ty = yy2;
		dx = tx - mx;
		dy = ty - my;
		d2 = dx*dx + dy*dy;

		// if the curve is flat enough, or no more subdivisions are
		// allowed, add the straight line segment
		if (p2 - p1 == 1 || (d1 <= flatness2 && d2 <= flatness2)) 
		{
			ADJUST_RECT_XY(&rcBounds, (LONG)xr3, (LONG)yr3);
			CheckCapacity(out);
			out.push_back(ClipperLib::IntPoint((ClipperLib::cInt)xr3, (ClipperLib::cInt)yr3));
			//fPath->lineTo(xr3, yr3);
			p1 = p2;

			// otherwise, subdivide the curve
		} 
		else
		{
			xl1 = avg(xl0, xx1);
			yl1 = avg(yl0, yy1);
			xh = avg(xx1, xx2);
			yh = avg(yy1, yy2);
			xl2 = avg(xl1, xh);
			yl2 = avg(yl1, yh);
			xr2 = avg(xx2, xr3);
			yr2 = avg(yy2, yr3);
			xr1 = avg(xh, xr2);
			yr1 = avg(yh, yr2);
			xr0 = avg(xl2, xr1);
			yr0 = avg(yl2, yr1);
			// add the new subdivision points
			p3 = (p1 + p2) / 2;
			cx[p1][1] = xl1;  cy[p1][1] = yl1;
			cx[p1][2] = xl2;  cy[p1][2] = yl2;
			cNext[p1] = p3;
			cx[p3][0] = xr0;  cy[p3][0] = yr0;
			cx[p3][1] = xr1;  cy[p3][1] = yr1;
			cx[p3][2] = xr2;  cy[p3][2] = yr2;
			cNext[p3] = p2;
		}
	}
}

/******************************************************************************
   Function:	FlattenConvertPath
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		bool
   Parameters:	PBYTE Verbs  
   Parameters:	DocCoord * Coords  
   Parameters:	DWORD dwCount  
   Parameters:	ClipperLib::Paths & out  
   Parameters:	double flatness  
   Parameters:	RECT & rcBounds  
   Notes:       
******************************************************************************/
size_t FlattenConvertPathEx(PBYTE Verbs, DocCoord* Coords, DWORD dwCount, ClipperLib::Paths& out, double flatness, RECT& rcBounds)
{
	rcBounds.left = rcBounds.top = INT_MAX;
	rcBounds.right = rcBounds.bottom = INT_MIN;
	size_t iNumSubpaths = 0;

	//out.clear();
	int iIdx = (int)out.size()-1;
	double flatness2;
	BYTE flag;
	DWORD i=0;

	flatness2 = flatness * flatness;

	while (i < dwCount)
	{
		flag = Verbs[i];
		if (flag == PT_MOVETO) 
		{
			iNumSubpaths++;
			iIdx++;
			if (out.size() == out.capacity())
				out.reserve(out.size() + 100);
			out.push_back(ClipperLib::Path());

			CheckCapacity(out[iIdx]);
			out[iIdx].push_back(ClipperLib::IntPoint(Coords[i].x, Coords[i].y));
			ADJUST_RECT_XY(&rcBounds, Coords[i].x, Coords[i].y);
			++i;
		} 
		else 
		{
			if (flag & PT_BEZIERTO) 
			{
				FlattenCurve(Coords[i-1].x, Coords[i-1].y,
					Coords[i  ].x, Coords[i  ].y,
					Coords[i+1].x, Coords[i+1].y,
					Coords[i+2].x, Coords[i+2].y,
					flatness2, out[iIdx], rcBounds);
				i += 3;
			} 
			else 
			{
				CheckCapacity(out[iIdx]);
				out[iIdx].push_back(ClipperLib::IntPoint(Coords[i].x, Coords[i].y));
				ADJUST_RECT_XY(&rcBounds, Coords[i].x, Coords[i].y);
				++i;
			}

			if (Verbs[i-1] & PT_CLOSEFIGURE)
			{
				//fPath->close();
			}
		}
	}
	return iNumSubpaths;
}

/******************************************************************************
   Function:    FlattenConvertPath
   Author:      Paul Gruzdev
   Purpose:     Convert Xara path to exclude bezier curves and replace them with 
				sets of lines
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
size_t FlattenConvertPath(CXfPath* path, ClipperLib::Paths& out, double flatness, RECT& rcBounds)
{
	return path->Coords.size() ? FlattenConvertPathEx(&path->Verbs[0], &path->Coords[0], (DWORD)path->Verbs.size(), out, flatness, rcBounds) : 0;
}

/******************************************************************************
   Function:    PolygonsToPath
   Author:      Paul Gruzdev
   Purpose:     Convert polygons back to Xara path - currently for debugging
				purposes only, later we might use this to merge several 
				consecutive nested clipview controllers into one and 
				merging their paths via polygon merge.
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/

void PolygonsToPath(CXfPath* pPath, const ClipperLib::Paths& p)
{
	size_t c = pPath->Verbs.size();

	for (auto it = p.begin(); it != p.end(); ++it)
		c += it->size() + 1;

	pPath->Verbs.reserve(c);
	pPath->Coords.reserve(c);

	for (size_t i = 0; i < p.size(); i++)
	{
		pPath->Add(PT_MOVETO, DocCoord((LONG)p[i][0].X, (LONG)p[i][0].Y));
		for (size_t j = 1; j < p[i].size(); j++)
		{
			//BYTE byClose = j == p[i].size()-1 ? PT_CLOSEFIGURE : 0;
			pPath->Add(PT_LINETO/* | byClose*/, DocCoord((LONG)p[i][j].X, (LONG)p[i][j].Y));
		}

		// Camelot works better with 5-point rectangles:
		// for example see this Word import issue: DDES-15069

		pPath->Add(PT_LINETO | PT_CLOSEFIGURE, DocCoord((LONG)p[i][0].X, (LONG)p[i][0].Y));
	}
}

CXfPath* PolygonsToPath(ClipperLib::Paths& p, COLORREF crFill, BYTE byTrans, BYTE byLineTrans, BYTE byWinding)
{
	CXfPath* pPath = new CXfPath(TAG_PATH_FILLED_STROKED);
	PolygonsToPath(pPath, p);
	pPath->AppendChild(new CXfSimpleColor(GetRValue(crFill), GetGValue(crFill), GetBValue(crFill)));
	pPath->AppendChild(new CXfFlatFill(pPath->GetLastChild()));
	pPath->AppendChild(new CXfFlatTransparentFill(byTrans, CXF_TRANSPARANCY_TYPE_MIX));
	pPath->AppendChild(new CXfLineTransparency(byLineTrans, CXF_TRANSPARANCY_TYPE_MIX));
	pPath->AppendChild(new CXfWindingRule(byWinding));
	return pPath;
}

ClipperLib::PolyFillType XaraFillRuleToPoly(BYTE byWindingRule)
{
	switch (byWindingRule)
	{
	case CXF_WINDING_RULE_NONZERO:
		return ClipperLib::pftNonZero;
	case CXF_WINDING_RULE_NEGATIVE:
		return ClipperLib::pftNegative;
	case CXF_WINDING_RULE_EVENODD:
		return ClipperLib::pftEvenOdd;
	case CXF_WINDING_RULE_POSITIVE:
		return ClipperLib::pftPositive;
	}
	return ClipperLib::pftNonZero;
}

Rect FindPolygonsBounds(const ClipperLib::Paths &polygons)
{
	Rect bounds;

	for (ClipperLib::Paths::const_iterator itPath = polygons.begin(); itPath != polygons.end(); ++itPath)
	{
		for (ClipperLib::Path::const_iterator itPoint = itPath->begin(); itPoint != itPath->end(); ++itPoint)
		{
			Rect::InflateRectToContainPoint(bounds, itPoint->X, itPoint->X);
		}
	}

	return bounds;
}

PDFRectangle GetPathBounds(SplashPath* pPath)
{
	PDFRectangle bounds(0, 0, 0, 0);
	if (pPath && pPath->getLength())
	{
		double x, y;
		unsigned char flags;
		pPath->getPoint(0, &x, &y, &flags);

		bounds.x1 = bounds.x2 = x;
		bounds.y1 = bounds.y2 = y;

		for (int i = 1; i < pPath->getLength(); ++i)
		{
			pPath->getPoint(i, &x, &y, &flags);

			if (x < bounds.x1)
				bounds.x1 = x;
			else if (x > bounds.x2)
				bounds.x2 = x;

			if (y < bounds.y1)
				bounds.y1 = y;
			else if (y > bounds.y2)
				bounds.y2 = y;
		}
		
	}
	return bounds;
}

bool IsPathEmpty(SplashPath* pPath)
{
	PDFRectangle bounds = GetPathBounds(pPath);

	return bounds.x1 == bounds.x2 && bounds.y1 == bounds.y2;
}

/******************************************************************************
   Function:    SplashPathToGlyph
   Author:      Paul Gruzdev
   Purpose:		Convert a splash path to a Xara's Glyph def record
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool SplashPathToGlyph(SplashPath* pPath, CXfFontGlyph* pGlyph, double dMultiplier, const AffineTransform *pTransform)
{
	Rect bounds;
	DocCoord dcLast;
	std::vector<DocCoord> dcBezier;

	if (!IsPathEmpty(pPath))
	{
		for (int i = 0; i < pPath->getLength(); i++)
		{
			double x, y;
			unsigned char flags;
			BYTE byVerb = 0;
			pPath->getPoint(i, &x, &y, &flags);

			if (pTransform)
				pTransform->map(x, y, x, y);

			DocCoord dc((LONG)(x*dMultiplier), (LONG)(y*dMultiplier));
			bool bClose = !(flags & splashPathFirst) && (flags & splashPathClosed);

			if (flags & splashPathFirst)
			{
				if(bounds.isNull())
				{
					bounds.x = dc.x;
					bounds.y = dc.y;
				}
				else
					Rect::InflateRectToContainPoint(bounds, dc.x, dc.y);

				byVerb = PT_MOVETO;
				dcLast = dc;
			}
			else if (flags & splashPathCurve || dcBezier.size() == 2)
			{
				byVerb = PT_BEZIERTO;
				if (dcBezier.size() == 2) // we already stored two control points, adjust our bb
				{
					Rect::inflateRectToContainBezier(bounds, dcLast, dcBezier[0], dcBezier[1], dc);
					dcBezier.clear();
					dcLast = dc;
				}
				else
					dcBezier.push_back(dc);
			}
			else
			{
				dcLast = dc;
				byVerb = PT_LINETO;
				Rect::InflateRectToContainPoint(bounds, dc.x, dc.y);
			}

			byVerb |= bClose ? PT_CLOSEFIGURE : 0;
			pGlyph->AddPoint(byVerb, dc);
		}
	}

	pGlyph->SetBounds(bounds);
	return !pGlyph->Points.empty();
}

/******************************************************************************
   Function:    InitEmptyGlyph
   Author:      Pavel Ostapenko
   Purpose:		Create Xara's Glyph def record for space (empty) glyph
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool InitEmptyGlyph(CXfFontGlyph* pGlyph)
{
	//pGlyph->AddPoint(PT_CLOSEFIGURE, dc);
	pGlyph->SetBounds(Rect(0,0,0,0));
	return true;
}

/******************************************************************************
   Function:    convertPath
   Author:      Paul Gruzdev
   Purpose:		Convert a GFX path to a Splash path
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
SplashPath* convertPath(GfxState *state, GfxPath *path, bool dropEmptySubpaths)
{
	SplashPath *sPath;
	GfxSubpath *subpath;
	int n, i, j;

	n = dropEmptySubpaths ? 1 : 0;
	sPath = new SplashPath();
	for (i = 0; i < path->getNumSubpaths(); ++i) 
	{
		subpath = path->getSubpath(i);
		if (subpath->getNumPoints() > n) 
		{
			sPath->moveTo((SplashCoord)subpath->getX(0),
				(SplashCoord)subpath->getY(0));
			j = 1;
			while (j < subpath->getNumPoints()) 
			{
				if (subpath->getCurve(j)) 
				{
					sPath->curveTo((SplashCoord)subpath->getX(j),
						(SplashCoord)subpath->getY(j),
						(SplashCoord)subpath->getX(j+1),
						(SplashCoord)subpath->getY(j+1),
						(SplashCoord)subpath->getX(j+2),
						(SplashCoord)subpath->getY(j+2));
					j += 3;
				} else 
				{
					sPath->lineTo((SplashCoord)subpath->getX(j),
						(SplashCoord)subpath->getY(j));
					++j;
				}
			}
			if (subpath->isClosed()) {
				sPath->close();
			}
		}
	}
	return sPath;
}

/******************************************************************************
   Function:    CreatePathOutline
   Author:      Paul Gruzdev
   Purpose:		Create a path that is an outline of the input stroked path
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
SplashPath* CreatePathOutline(GfxPath* p, GfxState* state, bool bFlatten)
{
	// Create a fake bitmap to create a splash canvas
	SplashBitmap bm(1, 1, 4, splashModeRGB8, false);
	SplashPath* pIn = convertPath(NULL, p, true);
	Splash splash(&bm, true);
	splash.setLineCap(state->getLineCap());
	splash.setLineJoin(state->getLineJoin());
	splash.setLineWidth(state->getLineWidth());
	splash.setMiterLimit(state->getMiterLimit());
	//splash.setMatrix(state->getCTM());

	double* dash, start;
	int length;
	state->getLineDash(&dash, &length, &start);
	splash.setLineDash((SplashCoord*)dash, length, start);

	SplashPath* pOut = splash.makeStrokePath(pIn, state->getLineWidth(), bFlatten ? true : false); // we flatten it later
	delete pIn;
	return pOut;
}

/******************************************************************************
   Function:    DumpPath
   Author:      Paul Gruzdev
   Purpose:		Dump a GFX path to string
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void DumpPath(GfxPath* path, char* strDump)
{
	char psz[512];
	sprintf(strDump, "<start path>\n");

	for (int j = 0; j < path->getNumSubpaths(); ++j) 
	{
		GfxSubpath* subpath = path->getSubpath(j);

		for(int i = 0; i < subpath->getNumPoints(); i++)
		{
			*psz = 0;
			if(i == 0)
				sprintf(psz, "\tmoveto: %.2f %.2f\n", subpath->getX(i), subpath->getY(i));
			else if(!subpath->getCurve(i))
				sprintf(psz, "\tlineto: %.2f %.2f\n", subpath->getX(i), subpath->getY(i));
			else
				sprintf(psz, "\tbezierto: %.2f %.2f\n", subpath->getX(i), subpath->getY(i));
			strcat(strDump, psz);
		}
	}

	sprintf(psz, "<end path>\n");
	strcat(strDump, psz);
}

/******************************************************************************
   Function:    PathBoundingRect
   Author:      Paul Gruzdev
   Purpose:		Calc a bounding box of a GfxPath
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void PathBoundingRect(GfxPath* path, PDFRectangle* rect)
{
	rect->x1 = rect->y1 = INT_MAX;
	rect->x2 = rect->y2 = INT_MIN;
	for (int i = 0; i < path->getNumSubpaths(); ++i)
	{
		GfxSubpath* subpath = path->getSubpath(i);
		for (int j = 0; j < subpath->getNumPoints(); ++j) 
		{
			ADJUST_RECT_XY(rect, subpath->getX(j), subpath->getY(j));
		}
	}
}

/******************************************************************************
   Function:    PolygonsBoundingRect
   Author:      Denis Onishchenko
   Purpose:		Calc a bounding box of a ClipperLib::Paths
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void PolygonsBoundingRect(const ClipperLib::Paths* polygons, PDFRectangle* rect)
{
	rect->x1 = rect->y1 = INT_MAX;
	rect->x2 = rect->y2 = INT_MIN;
	for (ClipperLib::Paths::const_iterator itPath = polygons->begin(); itPath != polygons->end(); ++itPath)
	{
		for (ClipperLib::Path::const_iterator itPoint = itPath->begin(); itPoint != itPath->end(); ++itPoint)
		{
			ADJUST_RECT_XY(rect, itPoint->X, itPoint->Y);
		}
	}
}

/******************************************************************************
   Function:    SplashToXaraPath
   Author:      Paul Gruzdev
   Purpose:		Convert SplashPath ---> CXfPath
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfPath* SplashToXaraPath(SplashPath* path, GfxState* state, UINT uiTag)
{
	CXfPath* pPath = new CXfPath(uiTag);
	PDFRectangle bbox;
	bbox.x1 = INT_MAX; bbox.y1 = INT_MAX;
	bbox.x2 = INT_MIN; bbox.y2 = INT_MIN;

	double x,y;
	unsigned char flag;
	unsigned char gPrevPrevFlag = splashPathFirst;
	unsigned char gPrevFlag = splashPathFirst;
	for (int i=0; i<path->getLength(); ++i)
	{
		path->getPoint(i, &x, &y, &flag);
		bool bClose = !(flag & splashPathFirst) && (flag & splashPathClosed);
		BYTE byClose = bClose ? PT_CLOSEFIGURE : 0;
		if (state) state->transform(x, y, &x, &y);
		DocCoord dc((LONG)x, (LONG)y);
		ADJUST_RECT_XY(&bbox, x, y);
		pPath->Coords.push_back(dc);

		if (flag & splashPathFirst)
			pPath->Verbs.push_back(PT_MOVETO);
		// DDES-1182 fix. Poppler sub-curve for some reason is coded as curve, curve, moveto... we need to fix it
		else if (flag & splashPathCurve || ((gPrevPrevFlag & splashPathCurve) && (gPrevFlag & splashPathCurve)))
			pPath->Verbs.push_back(PT_BEZIERTO | byClose);
		else
			pPath->Verbs.push_back(PT_LINETO | byClose);

		gPrevPrevFlag = gPrevFlag;
		gPrevFlag = flag;
	}

	Rect& rcBounds = pPath->Bounds();
	rcBounds.x = (LONG)bbox.x1;
	rcBounds.y = (LONG)bbox.y1;
	rcBounds.width = (LONG)(bbox.x2 - bbox.x1 + 0.5);
	rcBounds.height = (LONG)(bbox.y2 - bbox.y1 + 0.5);

	return pPath;
}

/******************************************************************************
   Function:    CreateXaraPath
   Author:      Paul Gruzdev
   Purpose:		Convert a path from PDF into a Xara's CXfPath node object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfPath* CreateXaraPath(GfxPath* path, GfxState* state, UINT uiTag)
{
	bool bNeedClosing = (uiTag == TAG_PATH_FILLED) || (uiTag == TAG_PATH_FILLED_STROKED) ||
		(uiTag == TAG_PATH_RELATIVE_FILLED) || (uiTag == TAG_PATH_RELATIVE_FILLED_STROKED);

	if(path && path->getNumSubpaths())
	{
		CXfPath* pPath = new CXfPath(uiTag);
		CreateXaraPath(path, state, pPath->Coords, pPath->Verbs, pPath->Bounds(), bNeedClosing);
		return pPath;
	}

	return NULL;
}

void CreateXaraPath(GfxPath* path, GfxState* state, std::vector<DocCoord> &coords, std::vector<BYTE> &verbs, Rect& bounds, bool bNeedClosing)
{
	if (path && path->getNumSubpaths())
	{
		PDFRectangle bbox;
		bbox.x1 = INT_MAX; bbox.y1 = INT_MAX;
		bbox.x2 = INT_MIN; bbox.y2 = INT_MIN;

		for (int i = 0; i < path->getNumSubpaths(); i++)
		{
			GfxSubpath* pSub = path->getSubpath(i);
			// Drop empty sub-path, DDES-309 fix
			if (pSub->getNumPoints() <= 1)
				continue;

			DocCoord dcMoveTo;
			DWORD dwClose = (pSub->isClosed() || bNeedClosing) ? PT_CLOSEFIGURE : 0;

			for (int j = 0; j < pSub->getNumPoints(); j++)
			{
				bool bPrevCurve = j > 0 ? pSub->getCurve(j-1) : false;
				bool bPrevPrevCurve = j > 1 ? pSub->getCurve(j-2) : false;
				// DDES-1182 fix. Poppler sub-curve for some reason is coded as curve, curve, moveto... we need to fix it
				BYTE verb = j == 0 ? PT_MOVETO : (pSub->getCurve(j) | (bPrevPrevCurve && bPrevCurve) ? PT_BEZIERTO : PT_LINETO);
				bool bLastPoint = j == pSub->getNumPoints()-1;

				double dX = pSub->getX(j);
				double dY = pSub->getY(j);
				if(state)
					state->transform(dX, dY, &dX, &dY);

				DocCoord dc(roundNumber(dX), roundNumber(dY));
				ADJUST_RECT_XY(&bbox, dc.x, dc.y);

				if (verb == PT_MOVETO) dcMoveTo = dc;

				if (bLastPoint)
				{
					if(dwClose && dc != dcMoveTo)
					{
						verbs.push_back(verb);
						coords.push_back(dc);
						verb = PT_LINETO; dc = dcMoveTo; // close the last segment
					}

					verb |= dwClose;
				}

				verbs.push_back(verb);
				coords.push_back(dc);
			}
		}

		bounds.x = (LONG)bbox.x1;
		bounds.y = (LONG)bbox.y1;
		bounds.width = (LONG)(bbox.x2 - bbox.x1);// + 0.5);
		bounds.height = (LONG)(bbox.y2 - bbox.y1);// + 0.5);
	}
}

GfxPath* SplashPathToGfxPath(SplashPath* path, GfxState* state)
{
	GfxPath* pNew = new GfxPath();

	double x,y;
	unsigned char flag;
	for (int i = 0; i<path->getLength(); ++i)
	{
		path->getPoint(i, &x, &y, &flag);
		if (state)
		{
			state->transform(x, y, &x, &y);
		}
		bool bClose = !(flag & splashPathFirst) && (flag & splashPathClosed);

		if(flag & splashPathFirst)
			pNew->moveTo(x, y);
		else if(flag & splashPathCurve)
		{
			double x2, y2, x3, y3;
			unsigned char flagTemp;
			path->getPoint(i + 1, &x2, &y2, &flagTemp);
			path->getPoint(i + 2, &x3, &y3, &flagTemp);

			if (state)
			{
				state->transform(x2, y2, &x2, &y2);
				state->transform(x3, y3, &x3, &y3);
			}

			pNew->curveTo(x, y, x2, y2, x3, y3);
			i += 2;
		}
		else
		{
			pNew->lineTo(x, y);
		}
		if(bClose) pNew->close();
	}

	return pNew;
}

bool Intersects(const ClipperLib::Paths &polygon1, const ClipperLib::Paths &polygon2, BYTE byWindingRule1, BYTE byWindingRule2)
{
	if (polygon1.empty() || polygon2.empty()) return false;
	try
	{
		ClipperLib::Clipper c;
		c.AddPaths(polygon1, ClipperLib::ptSubject, true);
		c.AddPaths(polygon2, ClipperLib::ptClip, true);

		ClipperLib::Paths result;
		return !c.Execute(ClipperLib::ctIntersection, result, XaraFillRuleToPoly(byWindingRule1), XaraFillRuleToPoly(byWindingRule2)) || !result.empty();
	}
	catch (ClipperLib::clipperException& e)
	{
		return true;
	}
}

bool Intersects(const Rect &rect, const ClipperLib::Paths &polygon, BYTE byWindingRule)
{
	if (polygon.empty() || rect.empty()) return false;

	ClipperLib::Paths rectPolygon;
	AddRect(rectPolygon, rect);

	return Intersects(rectPolygon, polygon, CXF_WINDING_RULE_NONZERO, byWindingRule);
}

bool Intersects(const Rect &rect1, const Rect &rect2)
{
	return rect1.IntersectsWith(rect2);
}

void AddRect(GfxPath* path, double x, double y, double width, double height)
{
	path->moveTo(x, y);
	path->lineTo(x + width, y);
	path->lineTo(x + width, y + height);
	path->lineTo(x, y + height);
	path->close();
}

void AddRect(GfxPath* path, const Rect &rect)
{
	AddRect(path, rect.x, rect.y, rect.width, rect.height);
}

void AddRect(ClipperLib::Paths &polygons, const Rect &rect)
{
	if (!rect.empty() )
	{
		polygons.push_back(ClipperLib::Path());
		ClipperLib::Path &polygon = polygons.back();
		polygon.reserve(4);
		polygon.push_back(ClipperLib::IntPoint(rect.left(),  rect.top()));
		polygon.push_back(ClipperLib::IntPoint(rect.right(), rect.top()));
		polygon.push_back(ClipperLib::IntPoint(rect.right(), rect.bottom()));
		polygon.push_back(ClipperLib::IntPoint(rect.left(),  rect.bottom()));
	}
}

void AddRect(ClipperLib::Paths &polygons, double x, double y, double width, double height, GfxState* state)
{
		polygons.push_back(ClipperLib::Path());
		ClipperLib::Path &polygon = polygons.back();
		polygon.reserve(4);
		if (state)
		{
			double tx, ty;
			state->transform(x, y, &tx, &ty);
			polygon.push_back(ClipperLib::IntPoint(_round(tx), _round(ty)));
			state->transform(x + width, y, &tx, &ty);
			polygon.push_back(ClipperLib::IntPoint(_round(tx), _round(ty)));
			state->transform(x + width, y + height, &tx, &ty);
			polygon.push_back(ClipperLib::IntPoint(_round(tx), _round(ty)));
			state->transform(x, y + height, &tx, &ty);
			polygon.push_back(ClipperLib::IntPoint(_round(tx), _round(ty)));
		}
		else
		{
			polygon.push_back(ClipperLib::IntPoint(_round(x), _round(y)));
			polygon.push_back(ClipperLib::IntPoint(_round(x + width), _round(y)));
			polygon.push_back(ClipperLib::IntPoint(_round(x + width), _round(y + height)));
			polygon.push_back(ClipperLib::IntPoint(_round(x), _round(y + height)));
		}
}

class GfxVertexSource
{
	GfxPath *path;
	GfxState* state;
	GfxSubpath *subpath;
	int subpath_index, point_index, curve_remain_count;
public:
	GfxVertexSource(GfxPath *_path, GfxState* _state = NULL): path(_path), state(_state)
	{
		rewind(0);
	}

	void rewind(unsigned path_id)
	{
		subpath_index = point_index = curve_remain_count = 0;
		subpath = path->getNumSubpaths() ? path->getSubpath(0) : NULL;		
	}

	unsigned vertex(double* x, double* y)
	{				
		unsigned cmd = agg::path_cmd_stop;

		if (0 == path->getNumSubpaths() && path->isCurPt() && 0 == subpath_index)
		{
			x = y = 0;
			subpath_index++;
			return agg::path_cmd_move_to;
		}

		while (subpath)
		{
			if (point_index < subpath->getNumPoints())
			{
				*x = subpath->getX(point_index);
				*y = subpath->getY(point_index);
				if (state)
				{
					state->transform(*x, *y, x, y);
				}
				if (0 == point_index ) cmd = agg::path_cmd_move_to;
				else if ( curve_remain_count )
				{
					curve_remain_count--;
					cmd = agg::path_cmd_curve4;
				}
				else if (subpath->getCurve(point_index) && point_index + 2 < subpath->getNumPoints())
				{
					curve_remain_count = 2;
					cmd = agg::path_cmd_curve4;
				}
				else cmd = agg::path_cmd_line_to;
				point_index++;
				return cmd;
			}
			else
			{
				if (point_index)
				{
					cmd = agg::path_cmd_end_poly;
					if (subpath->isClosed()) cmd |= agg::path_flags_close;
				}
				subpath_index++;
				subpath = subpath_index < path->getNumSubpaths() ? path->getSubpath(subpath_index) : NULL;
				point_index = curve_remain_count = 0;
				if( agg::path_cmd_stop != cmd) return cmd;
			}
		}
		
		*x = *y = 0;
		return cmd;
	}
};

void AddPath(agg::path_storage &path_storage, GfxPath* path, GfxState* state)
{
	int numSubpaths = path->getNumSubpaths();
	for (int i = 0; i < numSubpaths; ++i)
	{
		GfxSubpath *subpath = path->getSubpath(i);
		if (int numPoints = subpath->getNumPoints())
		{
			double x = subpath->getX(0);
			double y = subpath->getY(0);
			if (state) state->transform(x, y, &x, &y);
			path_storage.move_to(x, y);
			for (int j = 1; j < numPoints;)
			{
				if (subpath->getCurve(j) && j + 2 < numPoints)
				{
					x = subpath->getX(j);
					y = subpath->getY(j);
					double x1 = subpath->getX(j + 1);
					double y1 = subpath->getY(j + 1);
					double x2 = subpath->getX(j + 2);
					double y2 = subpath->getY(j + 2);

					if (state)
					{
						state->transform(x, y, &x, &y);
						state->transform(x1, y1, &x1, &y1);
						state->transform(x2, y2, &x2, &y2);
					}

					path_storage.curve4(x, y, x1, y1, x2, y2);
					j += 3;
				}
				else
				{
					x = subpath->getX(j);
					y = subpath->getY(j);
					if (state) state->transform(x, y, &x, &y);
					path_storage.line_to(x, y);
					++j;
				}
			}
		}
		if (subpath->isClosed()) path_storage.close_polygon();
	}
}

template<class T> void SetAGGStroke(T &stroke, GfxState* state)
{
	stroke.width(state->getLineWidth());
	stroke.miter_limit(state->getMiterLimit());
	stroke.inner_miter_limit(state->getMiterLimit());

	switch (state->getLineCap())
	{
	default: case 0:
		stroke.line_cap(agg::butt_cap);
		break;
	case 1:
		stroke.line_cap(agg::round_cap);
		break;
	case 2:
		stroke.line_cap(agg::square_cap);
		break;
	}

	switch (state->getLineJoin())
	{
	default: case 0:
		stroke.line_join(agg::miter_join);
		stroke.inner_join(agg::inner_miter);
		break;
	case 1:
		stroke.line_join(agg::round_join);
		stroke.inner_join(agg::inner_round);
		break;
	case 2:
		stroke.line_join(agg::bevel_join);
		stroke.inner_join(agg::inner_bevel);
		break;
	}
}

template<class T> GfxPath* CreatePathFromAGGPipeline(T &pipeline)
{
	GfxPath* path = new GfxPath();

	unsigned cmd = 0;
	do
	{
		double x = 0, y = 0;
		cmd = pipeline.vertex(&x, &y);

		switch (cmd & agg::path_cmd_mask)
		{
		case agg::path_cmd_move_to:
			path->moveTo(x, y);
			break;
		case agg::path_cmd_line_to:
			path->lineTo(x, y);
			break;
		}

		switch (cmd & agg::path_flags_mask)
		{
		case agg::path_flags_close:
			path->close();
			break;
		}
	}
	while (agg::path_cmd_stop != (cmd & agg::path_cmd_mask));

	return path;
}

/******************************************************************************
   Function:    CreatePathOutline
   Author:      Denis Onishchenko
   Purpose:	Create a path that is an outline of the input stroked path
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
GfxPath* CreatePathOutline(GfxPath* path, GfxState* state)
{
	double* dash_array = NULL, dash_start = 0;
	int dash_length = 0;
	state->getLineDash(&dash_array, &dash_length, &dash_start);

	bool bUseSplash = false;

	if (dash_array)
	{
		double s = 0;
		for (int i = 0; i < dash_length; ++i)
		{
			if (dash_array[i] < 0) return NULL;
			s += dash_array[i];
		}
		if (s <= 0.001) return NULL;
		if (s < 1 ) bUseSplash = true;
	}

	if (bUseSplash)
	{
		if (SplashPath* splashPath = CreatePathOutline(path, state, true))
		{
			GfxPath *outlinePath = SplashPathToGfxPath(splashPath);
			delete splashPath;
			if (outlinePath) return outlinePath;
		}
	}

	//agg::path_storage path_storage;
	//AddPath(path_storage, path);

	//agg::conv_curve<agg::path_storage> curve(path_storage);

	GfxVertexSource source(path);
	agg::conv_curve<GfxVertexSource> curve(source);

	if (dash_array && dash_length)
	{
		//agg::conv_dash<agg::conv_curve<agg::path_storage> > dash(curve);
		agg::conv_dash<agg::conv_curve<GfxVertexSource> > dash(curve);

		int len = 1 == (dash_length & 1) ? 2 * dash_length : dash_length;
		for (int i = 0; i < len; i += 2)
		{
			dash.add_dash(dash_array[i % dash_length], dash_array[(i + 1) % dash_length]);
		}

		dash.dash_start(dash_start);

		//agg::conv_stroke<agg::conv_dash<agg::conv_curve<agg::path_storage> > > stroke(dash);
		agg::conv_stroke<agg::conv_dash<agg::conv_curve<GfxVertexSource> > > stroke(dash);
		SetAGGStroke(stroke, state);
		return CreatePathFromAGGPipeline(stroke);
	}
	else
	{
		//agg::conv_stroke<agg::conv_curve<agg::path_storage> > stroke(curve);
		agg::conv_stroke<agg::conv_curve<GfxVertexSource> > stroke(curve);

		SetAGGStroke(stroke, state);
		return CreatePathFromAGGPipeline(stroke);
	}
}

void AddPath(ClipperLib::Paths &polygons, GfxPath* path, GfxState* state, double approximation_scale)
{
	polygons.reserve(polygons.size() + path->getNumSubpaths());

	//agg::path_storage path_storage;
	//AddPath(path_storage, path, state);
	//agg::conv_curve<agg::path_storage> curve(path_storage);

	GfxVertexSource source(path, state);
	agg::conv_curve<GfxVertexSource> curve(source);

	curve.approximation_scale(approximation_scale);

	unsigned cmd = 0;
	do
	{
		double x = 0, y = 0;
		cmd = curve.vertex(&x, &y);

		switch (cmd & agg::path_cmd_mask)
		{
		case agg::path_cmd_move_to:
			polygons.push_back(ClipperLib::Path());
		case agg::path_cmd_line_to:
			assert(polygons.size() > 0);
			polygons.back().push_back(ClipperLib::IntPoint(roundNumber(x), roundNumber(y)));
			break;
		}
	}
	while (agg::path_cmd_stop != (cmd & agg::path_cmd_mask));
}
