/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once
#include "../../clipper/agg/include/agg_path_storage.h"
#include "../../clipper/agg/include/agg_conv_curve.h"
#include "../../clipper/agg/include/agg_conv_stroke.h"
#include "../../clipper/agg/include/agg_conv_dash.h"
#include "../../clipper/agg/include/agg_conv_transform.h"
#include "clipper\clipper.hpp"
#include "../../poppler/poppler/GfxState.h"
#include "../../poppler/splash/SplashPath.h"
#include "../../common/AffineTransform.h"
#include "common\xftree\xftree.h"

const double g_dFlatnessThreshold = 1000.0;

inline int roundNumber(double value)
{
	return (value > 0.0) ? int(value + 0.5) : int(value - 0.5);
}

size_t FlattenConvertPathEx(PBYTE Verbs, DocCoord* Coords, DWORD dwCount, ClipperLib::Paths& out, double flatness, RECT& rcBounds);
size_t FlattenConvertPath(CXfPath* path, ClipperLib::Paths& out, double flatness, RECT& rcBounds);
void PolygonsToPath(CXfPath* pPath, const ClipperLib::Paths& p);
CXfPath* PolygonsToPath(ClipperLib::Paths& p, COLORREF crFill, BYTE byTrans, BYTE byLineTrans, BYTE byWinding);

bool SplashPathToGlyph(SplashPath* sPath, CXfFontGlyph* pGlyph, double dMultiplier = 1.0, const AffineTransform *pTransform = NULL);
bool InitEmptyGlyph(CXfFontGlyph* pGlyph);
SplashPath* convertPath(GfxState *state, GfxPath *path, bool dropEmptySubpaths);
SplashPath* CreatePathOutline(GfxPath* p, GfxState* state, bool bFlatten);
void DumpPath(GfxPath* path, char* strDump);
void PathBoundingRect(GfxPath* path, PDFRectangle* rect);
void PolygonsBoundingRect(const ClipperLib::Paths* polygons, PDFRectangle* rect);
Rect FindPolygonsBounds(const ClipperLib::Paths &polygons);

PDFRectangle GetPathBounds(SplashPath* pPath);
bool IsPathEmpty(SplashPath* pPath);

CXfPath* SplashToXaraPath(SplashPath* path, GfxState* state, UINT uiTag);


CXfPath* CreateXaraPath(GfxPath* path, GfxState* state, UINT uiTag);
void CreateXaraPath(GfxPath* path, GfxState* state, std::vector<DocCoord> &coords, std::vector<BYTE> &verbs, Rect& bounds, bool bNeedClosing);


GfxPath* SplashPathToGfxPath(SplashPath* path, GfxState* state = NULL);

ClipperLib::PolyFillType XaraFillRuleToPoly(BYTE byWindingRule);
bool Intersects(const ClipperLib::Paths &polygon1, const ClipperLib::Paths &polygon2, BYTE byWindingRule1 = CXF_WINDING_RULE_NONZERO, BYTE byWindingRule2 = CXF_WINDING_RULE_NONZERO);
bool Intersects(const Rect &rect, const ClipperLib::Paths &polygon, BYTE byWindingRule = CXF_WINDING_RULE_NONZERO);
bool Intersects(const Rect &rect1, const Rect &rect2);

void AddRect(GfxPath* path, double x, double y, double width, double height);
void AddRect(GfxPath* path, const Rect &rect);
void AddRect(ClipperLib::Paths &polygons, const Rect &rect);
void AddRect(ClipperLib::Paths &polygons, double x, double y, double width, double height, GfxState* state = NULL);
void AddPath(agg::path_storage &path_storage, GfxPath* path, GfxState* state = NULL);
void AddPath(ClipperLib::Paths &polygons, GfxPath* path, GfxState* state = NULL, double approximation_scale = .005);
GfxPath* CreatePathOutline(GfxPath* path, GfxState* state);
