/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
//
// XarOutputDev.cc
//
#include "stdafx.h"
#include "XarOutputDev.h"
#include "splash/splash.h"
#include "splash/splashbitmap.h"
#include "splash/SplashState.h"
#include "splash/SplashPath.h"
#include "PathHelpers.h"

#define ADJUST_XY(x,y) { if(x<dMinX) dMinX=x; else if(x>dMaxX) dMaxX=x; if(y<dMinY) dMinY=y; else if(y>dMaxY) dMaxY=y; }
#undef Matrix // Undef a XarLib's XaraMatrix ---> Matrix define, we need a GfxState Matrix here!

/******************************************************************************
   Function:    XarOutputDev::CreateSplashRenderer
   Author:      Paul Gruzdev
   Purpose:		Create a Splash object to render all subsequent strokes and fills
				to. The size of the splash bitmap is set to the bounding box of the
				current clip path.
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool XarOutputDev::CreateSplashRenderer(GfxState* state)
{START_STOPWATCH_FN		

	if (m_clip.Empty() ) return false;
	PDFRectangle rect;
	PolygonsBoundingRect(&m_clip.GetPolygons(), &rect);

	rect.x1 = floor(rect.x1/1000.0);
	rect.y1 = floor(rect.y1/1000.0);
	rect.x2 = ceil(rect.x2/1000.0);
	rect.y2 = ceil(rect.y2/1000.0);

	int w = (int)(rect.x2 - rect.x1);//(state->getPageWidth()/1000);
	int h = (int)(rect.y2 - rect.y1);//(state->getPageHeight()/1000);
	SplashColor color;
	SplashColor bgColor;
	bgColor[0] = bgColor[1] = bgColor[2] = 0;
	bgColor[3] = 0;

	SplashThinLineMode thinLineMode = splashThinLineDefault;

	bitmap = new SplashBitmap(w, h, 4, splashModeXBGR8, true, true);

	if (!bitmap->getDataPtr()) 
	{
		delete bitmap;
		w = h = 1;
		bitmap = new SplashBitmap(w, h, 4, splashModeXBGR8, true, true);
	}

	splash = new Splash(bitmap, true);
	splash->setThinLineMode(thinLineMode);
	splash->setMinLineWidth(0.0);
	splash->setVectorAntialias(true);

	if (state) 
	{
		const double *ctm = state->getCTM();

		//PDFRectangle rect(0, 0, w, h);
		GfxState s(72, 72, &rect, 0, true); // This is the default matrix to map to the PNG bitmap
		s.concatCTM(ctm[0]/1000.0, ctm[1]/1000.0, ctm[2]/1000.0, ctm[3]/1000.0, ctm[4]/1000.0, ctm[5]/1000.0);
		splash->setMatrix((double *)s.getCTM());
	}

	color[3] = 255;

	splash->setStrokePattern(new SplashSolidColor(color));
	splash->setFillPattern(new SplashSolidColor(color));
	splash->setLineCap(splashLineCapButt);
	splash->setLineJoin(splashLineJoinMiter);
	splash->setLineDash(NULL, 0, 0);
	splash->setMiterLimit(10);
	splash->setFlatness(1);
	// the SA parameter supposedly defaults to false, but Acrobat
	// apparently hardwires it to true
	splash->setStrokeAdjust(true);
	splash->clear(bgColor, 0);
STOP_STOPWATCH(__FUNCTION__)
	return true;
}

/******************************************************************************
   Function:    XarOutputDev::stroke
   Author:      Paul Gruzdev
   Purpose:		Append the stroked path tag that corresponds to the current path
				stored in GfxState
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::stroke(GfxState* state)
{
	RD;
	GfxPath* path = state->getPath();

	if (path && path->getNumSubpaths() && !g_settings.bIgnoreGraphics && !m_clip.Empty())
	{
		static const double EPS = 1e-4;
		const double *ctm = state->getCTM();
		//bool bFillOutline = fabs(ctm[1]) > EPS || fabs(ctm[2]) > EPS || fabs(ctm[0] - ctm[3]) > EPS;
		bool bFillOutline = fabs(ctm[0] * ctm[0] + ctm[1] * ctm[1] - ctm[2] * ctm[2] - ctm[3] * ctm[3]) > EPS || 
				fabs(ctm[0] * ctm[2] + ctm[1] * ctm[3]) > EPS;

		CXfPath* pPathOutline = NULL;

		if (GfxPath* outline = CreatePathOutline(path, state))
		{
			pPathOutline = CreateXaraPath(outline, state, TAG_PATH_FILLED_STROKED);
			delete outline;
		}

		/*if( SplashPath* pOutline = CreatePathOutline(path, state, true) )
		{
			if(pOutline->getLength()) // 3704.pdf produces an empty outline from m(x,y) - l(x,y) empty dot path
				pPathOutline = SplashToXaraPath(pOutline, state, TAG_PATH_FILLED_STROKED);

			delete pOutline;
		}*/

		if (bFillOutline && !pPathOutline) return;

		// DDES-1082 (missing lines). From PDF reference:
		// A line width of 0 denotes the thinnest line that can be rendered at device resolution: 1 device pixel wide. 
		// However, some devices cannot reproduce 1-pixel lines, and on high-resolution devices, they are nearly invisible. 
		// Since the results of rendering such zero-width lines are device-dependent, their use is not recommended. 
		CXfPath* pPath = bFillOutline ? pPathOutline : CreateXaraPath(path, state, TAG_PATH_STROKED /*state->getLineWidth() > 0 ? TAG_PATH_STROKED : TAG_PATH*/);
		if (AddGraphics(pPath, pPathOutline, CXF_WINDING_RULE_NONZERO, true, state, true))
		{
			AppendAttributes(pPath, state, !bFillOutline, bFillOutline, false, bFillOutline);
		}
		else delete pPath;

		if (pPathOutline && pPath != pPathOutline) delete pPathOutline;
	}
STOP_STOPWATCH(__FUNCTION__)
}

/******************************************************************************
   Function:    XarOutputDev::DoFill
   Author:      Paul Gruzdev
   Purpose:		The actual filling op - take the state and the winding rule as
				parameters
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool XarOutputDev::DoFill(GfxState* state, BYTE byWindingRule)
{
START_STOPWATCH_FN
	bool bRet = false;
	// If we in the splash rendere mode (rendering a mesh for example), then render our fill into the 
	// splash renderer
	if (splash)
	{
		if (!g_settings.bIgnoreGraphics)
		{
			SplashPath* path = convertPath(state, state->getPath(), true);
			splash->fill(path, false);
			delete path;
		}
		bRet = true;
	}
	// otherwise (no Splash renderer) - pass this fill as a Xara's filled shape
	else
	{
		GfxPath* path = state->getPath();
		if (path && path->getNumSubpaths() && !g_settings.bIgnoreGraphics && !m_clip.Empty())
		{
			// Paul: we always, ALWAYS have to create path with filled and stroked attributes - this is because of the
			// 'outline' mode in Camelot - path ALWAYS should be at least stroked
			CXfPath* pPath = CreateXaraPath(path, state, TAG_PATH_FILLED_STROKED);
			if (AddGraphics(pPath, pPath, byWindingRule, true, state, false))
			{
				AppendAttributes(pPath, state, false, true);
				pPath->AppendChild(new CXfWindingRule(byWindingRule));
			}
			else delete pPath;

			bRet = true;
		}
	}
STOP_STOPWATCH(__FUNCTION__)
	return bRet;
}

bool XarOutputDev::tilingPatternFill(GfxState *state, Gfx *gfxA, Catalog *catalog, Object *str,
	const double *ptm, int paintType, int tilingType, Dict *resDict,
	const double *mat, const double *bbox,
	int x0, int y0, int x1, int y1,
	double xStep, double yStep)
{ RD;
	return OutputDev::tilingPatternFill(state, gfxA, catalog, str, ptm, paintType, tilingType, resDict,
		mat, bbox, x0, y0, x1, y1, xStep, yStep);
STOP_STOPWATCH(__FUNCTION__)
}

bool XarOutputDev::gouraudTriangleShadedFill(GfxState *state, GfxGouraudTriangleShading *shading)
{ 
	RD;
	// If we DON'T have a renderer in process, add a new one and render this type of fill to it
	if(!splash && !bitmap)
		CreateSplashRenderer(state);
STOP_STOPWATCH(__FUNCTION__)
	return false;
}

/******************************************************************************
   Function:    XarOutputDev::axialShadedFill
   Author:      Paul Gruzdev
   Purpose:		Render a linear fill of a current clip path as a linear filled
				shape
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool XarOutputDev::axialShadedFill(GfxState *state, GfxAxialShading *shading, double tMin, double tMax) 
{
	RD;

	if (g_settings.bIgnoreGraphics || m_clip.Empty()) return true;

	// Paul: we always, ALWAYS have to create path with filled and stroked attributes - this is because of the
	// 'outline' mode in Camelot - path ALWAYS should be at least stroked
	if (CXfPath* pPath = m_clip.CreateXaraPath(TAG_PATH_FILLED_STROKED))
	{
		if (AddGraphics(pPath, pPath, m_clip.GetWindingRule(), false))
		{
			AppendAxialFill(pPath, state, shading);
			AppendAttributes(pPath, state, false, false, true); // Append a transparency node if needed
			pPath->AppendChild(new CXfWindingRule(m_clip.GetWindingRule()));
		}
		else delete pPath;
	}

STOP_STOPWATCH(__FUNCTION__)
	return true;//OutputDev::axialShadedFill(state, shading, tMin, tMax);
}

/******************************************************************************
   Function:	XarOutputDev::radialShadedFill
   Author:		Paul Gruzdev
   Purpose:     Render elliptic fill to Xara
   Returns:		bool
   Parameters:	GfxState * state  
   Parameters:	GfxRadialShading * shading  
   Parameters:	double tMin  
   Parameters:	double tMax  
   Notes:       
******************************************************************************/
bool XarOutputDev::radialShadedFill(GfxState *state, GfxRadialShading *shading, double tMin, double tMax) 
{
	RD;
	// Sometimes we can't convert radial fill to Xara's elliptical fill, for example if the inner circle has a valid radius
	if(CanConvertRadialFill(shading) == false)
	{
		// If we DON'T have a renderer in process, add a new one and render this type of fill to it
		if(!splash && !bitmap)
			CreateSplashRenderer(state);
		return false; // Let poppler rasterise this fill
	}

	if (g_settings.bIgnoreGraphics || m_clip.Empty()) return true;

	// Paul: we always, ALWAYS have to create path with filled and stroked attributes - this is because of the
	// 'outline' mode in Camelot - path ALWAYS should be at least stroked
	if (CXfPath* pPath = m_clip.CreateXaraPath(TAG_PATH_FILLED_STROKED))
	{
		if (AddGraphics(pPath, pPath, m_clip.GetWindingRule(), false))
		{
			AppendRadialFill(pPath, state, shading);
			AppendAttributes(pPath, state, false, false, true); // Append a transparency node if needed
			pPath->AppendChild(new CXfWindingRule(m_clip.GetWindingRule()));
		}
		else delete pPath;
	}

STOP_STOPWATCH(__FUNCTION__)
	return true;
}

bool XarOutputDev::functionShadedFill(GfxState* state, GfxFunctionShading* shading)
{
	// If we DON'T have a renderer in process, add a new one and render this type of fill to it
	if(!splash && !bitmap)
		CreateSplashRenderer(state);
	return false;
}

bool XarOutputDev::patchMeshShadedFill(GfxState *state, GfxPatchMeshShading *shading)
{
	RD;
	if(splash)
	{
		delete splash;
		splash = NULL;
	}
	if(bitmap)
	{
		delete bitmap;
		bitmap = NULL;
	}
	// Mesh bounding box calculation is done via calculating a bb of the clip path that is filled
/*
	double dMinX = INT_MAX;
	double dMaxX = INT_MIN;
	double dMinY = INT_MAX;
	double dMaxY = INT_MIN;

	for(int i = 0; i < shading->getNPatches(); i++)
	{
		GfxPatch* patch = shading->getPatch(i);

		for(int j = 0; j < 4; j++)
			for(int k = 0; k < 4; k++)
			ADJUST_XY(patch->x[j][k], patch->y[j][k]);
	}
*/
	// We create a Splash renderer here and pass all the stroke/fill operations to the renderer
	CreateSplashRenderer(state);
STOP_STOPWATCH(__FUNCTION__)
	return false;
}

double CalcColourDiff(GfxColorSpace* pSpace, GfxColor* c1, GfxColor* c2)
{
	double dDiff = 0;
	for(int i = 0; i < pSpace->getNComps(); i++)
		dDiff += fabs(colToDbl(c1->c[i])-colToDbl(c2->c[i]));

	return dDiff;
}

bool XarOutputDev::InterpolateExponential(GfxUnivariateShading* shading, ExponentialFunction* pExp, std::vector<double>& bounds, double dx1, double dx2)
{
	for(double dx = dx1 + 0.01; dx < dx2; dx += 0.01)
	{
		GfxColor c1, c2;
		shading->getColor(dx1, &c1);
		shading->getColor(dx, &c2);
		double dDiff = CalcColourDiff(shading->getColorSpace(), &c1, &c2);
		if(dDiff >= 0.15)
		{
			bounds.push_back(dx);
			dx1 = dx;
		}
	}

	return true;
}

bool XarOutputDev::InterpolateStitchingFunction(GfxUnivariateShading* shading, std::vector<double>& bounds)
{
	if(!shading->getNFuncs())
		return false;
	StitchingFunction* func = (StitchingFunction*)shading->getFunc(0);
	
	int iIdx = 0;
	const double* pBounds = func->getBounds();
	bounds.push_back(pBounds[iIdx++]);
	for (int i = 0; i < func->getNumFuncs(); i++)
	{
		if (func->getFunc(i)->getType() == Function_exponential)
		{
			ExponentialFunction* pExp = (ExponentialFunction*)func->getFunc(i);
			bool isLinear = fabs(pExp->getE()-1.) < 1e-10;

			if(!isLinear)
			{
				double dx1 = bounds.back();
				double dx2 = pBounds[iIdx];
				InterpolateExponential(shading, pExp, bounds, dx1, dx2);
			}
		}

		bounds.push_back(pBounds[iIdx++]);
	}

	return bounds.size() > 0;
}
/******************************************************************************
   Function:    XarOutputDev::AppendAxialFill
   Author:      Paul Gruzdev
   Purpose:		Append the linear fill to Xara, also defining a set of 
				gradient colours first.
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
VXfTreeNode* XarOutputDev::AppendAxialFill(VXfTreeNode* pNode, GfxState* state, GfxAxialShading* shading)
{START_STOPWATCH_FN
	if(shading->getType() != Shading_Axial || !shading->getNFuncs())
		return NULL;

	const Function* pFunc = shading->getFunc(0);
	double dBound0 = 0, dBound1 = 1;
	std::vector<double> bounds;

	switch (pFunc->getType())
	{
	case Function_stitching:
		{
			StitchingFunction* func = (StitchingFunction*)pFunc;
			dBound0 = func->getBounds()[0];
			dBound1 = func->getBounds()[func->getNumFuncs()];
			InterpolateStitchingFunction(shading, bounds);
		}
		break;
	case Function_exponential:
		{
			ExponentialFunction* pExp = (ExponentialFunction*)pFunc;
			dBound0 = pExp->getDomainMin(0);
			dBound1 = pExp->getDomainMax(0);
			bounds.push_back(dBound0);
			InterpolateExponential(shading, pExp, bounds, dBound0, dBound1);
			bounds.push_back(dBound1);
		}
		break;
	case Function_sampled:
		{
			SampledFunction* func = (SampledFunction*)pFunc;
			int iSteps = func->getSampleNumber() > 10 ? 10 : func->getSampleNumber();

			for(int i = 1; i < iSteps; i++)
				bounds.push_back((double)i/(double)iSteps);
		}
	}
	bool bMultistage = bounds.size() > 2;//func->getNumFuncs() > 1;
	double x0, y0, x1, y1;
	shading->getCoords(&x0, &y0, &x1, &y1);
	// Here is the trick: we actually have to use a 3-point linear gradient, as a fill could have a transform that contains
	// a shear so the third point would needed
	double x3 = -y1+y0+x0;
	double y3 =  x1-x0+y0;

	double x2, y2;
	state->transform(x0, y0, &x2, &y2);
	DocCoord dcStart((LONG)x2, (LONG)y2);
	state->transform(x1, y1, &x2, &y2);
	DocCoord dcEnd((LONG)x2, (LONG)y2);
	state->transform(x3, y3, &x2, &y2);
	DocCoord dcEnd2((LONG)x2, (LONG)y2);

	GfxColor colStart;
	shading->getColor(dBound0, &colStart);

	CXfComplexColor* pStart = CreateComplexColour(shading->getColorSpace(), &colStart);
	GfxColor colEnd;
	shading->getColor(dBound1, &colEnd);

	CXfComplexColor* pEnd = CreateComplexColour(shading->getColorSpace(), &colEnd);

	//pNode->AppendChild(pStart);
	//pNode->AppendChild(pEnd);

	VXfTreeNode* pFill = NULL;
	if(!bMultistage)
	{
		pFill = (VXfTreeNode*)new CXfLinearFill3Point(TAG_LINEARFILL3POINT, dcStart, dcEnd, dcEnd2, pStart, pEnd, 0, 0);
	}
	else
	{
		CXfLinearFill3PointMultiStage* pLinear = new CXfLinearFill3PointMultiStage(TAG_LINEARFILLMULTISTAGE3POINT, 
			dcStart, dcEnd, dcEnd2, pStart, pEnd);

		for(size_t i = 1; i < bounds.size()-1; i++)
		{
			GfxColor col;
			double dPos = bounds[i];//func->getBounds()[i];
			shading->getColor(dPos, &col);

			CXfComplexColor* pCol = CreateComplexColour(shading->getColorSpace(), &col);
			//pNode->AppendChild(pCol);

			// DDES-1223: dPos should be in range [0..1], while real bounds can be in arbitrary range, need to do a range matching
			dPos = (dPos - dBound0)/(dBound1-dBound0);
			pLinear->AddMiddleColor(dPos, pCol);
		}
		pFill = (VXfTreeNode*)pLinear;
	}

	pNode->AppendChild(pFill);
STOP_STOPWATCH(__FUNCTION__)
	return pFill;
}

bool XarOutputDev::CanConvertRadialFill(GfxRadialShading* shading)
{
	double x0, y0, r0, x1, y1, r1;
	shading->getCoords(&x0, &y0, &r0, &x1, &y1, &r1);
	// Either one of radiuses should be 0
	bool bRadiusCondition = fabs(r0) < 0.000001 || fabs(r1) < 0.000001;
	// Centers should match exactly as Xara does not support radial highlight effect
	bool bCentersMatch = fabs(x1-x0) < 0.000001 && fabs(y1-y0) < 0.000001;
	return bCentersMatch && bRadiusCondition;
}

/******************************************************************************
   Function:    XarOutputDev::AppendAxialFill
   Author:      Paul Gruzdev
   Purpose:		Append the linear fill to Xara, also defining a set of 
				gradient colours first.
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
VXfTreeNode* XarOutputDev::AppendRadialFill(VXfTreeNode* pNode, GfxState* state, GfxRadialShading* shading)
{START_STOPWATCH_FN
	if(shading->getType() != Shading_Radial || !shading->getNFuncs())
		return NULL;

	const Function* pFunc = shading->getFunc(0);
	double dBound0 = 0, dBound1 = 1;
	std::vector<double> bounds;

	if (pFunc->getType() == Function_stitching)
	{
		StitchingFunction* func = (StitchingFunction*)pFunc;
		dBound0 = func->getBounds()[0];
		dBound1 = func->getBounds()[func->getNumFuncs()];
		InterpolateStitchingFunction(shading, bounds);
	}
	else if(pFunc->getType() == Function_exponential)
	{
		ExponentialFunction* pExp = (ExponentialFunction*)pFunc;
		dBound0 = pExp->getDomainMin(0);
		dBound1 = pExp->getDomainMax(0);
		bounds.push_back(dBound0);
		InterpolateExponential(shading, pExp, bounds, dBound0, dBound1);
		bounds.push_back(dBound1);
	}

	bool bMultistage = bounds.size() > 2;//func->getNumFuncs() > 1;

	double x0, y0, r0, x1, y1, r1;
	shading->getCoords(&x0, &y0, &r0, &x1, &y1, &r1);
	double x2, y2;
	double dRadius = r1 > r0 ? r1 : r0;
	bool bSwap = r0 > r1;
	double dcx = bSwap ? x1 : x0;
	double dcy = bSwap ? y1 : y0;
	state->transform(dcx, dcy, &x2, &y2);
	DocCoord dcCenter((LONG)x2, (LONG)y2);
	state->transform(dcx+dRadius, dcy, &x2, &y2);
	DocCoord dcMajor((LONG)x2, (LONG)y2);
	state->transform(dcx, dcy+dRadius, &x2, &y2);
	DocCoord dcMinor((LONG)x2, (LONG)y2);

	GfxColor colStart;
	shading->getColor(bSwap ? dBound1 : dBound0, &colStart);

	CXfComplexColor* pStart = CreateComplexColour(shading->getColorSpace(), &colStart);
	GfxColor colEnd;
	shading->getColor(bSwap ? dBound0 : dBound1, &colEnd);

	CXfComplexColor* pEnd = CreateComplexColour(shading->getColorSpace(), &colEnd);

	//pNode->AppendChild(pStart);
	//pNode->AppendChild(pEnd);

	VXfTreeNode* pFill = NULL;
	if(!bMultistage)
	{
		pFill = (VXfTreeNode*)new CXfEllipticalFill(TAG_ELLIPTICALFILL, dcCenter, dcMajor, dcMinor, pStart, pEnd, 0, 0);
	}
	else
	{
		CXfEllipticalFillMultiStage* pElliptical = new CXfEllipticalFillMultiStage(TAG_ELLIPTICALFILLMULTISTAGE, 
			dcCenter, dcMajor, dcMinor, pStart, pEnd);

		for(int i = 1; i < bounds.size()-1; i++)
		{
			GfxColor col;
			double dPos = bounds[i];
			double dPosTakeColour = bSwap ? bounds[bounds.size()-i-1] : bounds[i];
			shading->getColor(dPosTakeColour, &col);

			CXfComplexColor* pCol = CreateComplexColour(shading->getColorSpace(), &col);
			//pNode->AppendChild(pCol);
			pElliptical->AddMiddleColor(dPos, pCol);
		}
		pFill = (VXfTreeNode*)pElliptical;
	}

	pNode->AppendChild(pFill);
STOP_STOPWATCH(__FUNCTION__)
	return pFill;
}

/******************************************************************************
   Function:	XarOutputDev::FindRemoveBottomWhiteRects
   Author:		Paul Gruzdev
   Purpose:     Often PDFs have the most bottom white rectangles drawn prior 
				any graphics - they are completely useless for editing, 
				need to remove them.
   Returns:		bool
   Parameters:	VXfTreeNode * pPage  
   Notes:       
******************************************************************************/
bool XarOutputDev::FindRemoveBottomWhiteRects(VXfTreeNode* pPage)
{
	bool bRemoved = false;
	bool bSearchAgain = true;

	while(bSearchAgain)
	{
		bSearchAgain = false;
		VXfTreeNode* pFirstObject = pPage->FindFirstRenderableNode(true, true, true, true);
		if (pFirstObject &&
			(pFirstObject->IsRectangle() ||
				(pFirstObject->IsPath() && ((CXfPath*)pFirstObject)->IsSimpleRectangle())))
		{			
			if (pFirstObject->FindFirstNode(TAG_FLATFILL, false))
			{
				VXfTreeNodeVector v;
				pFirstObject->FindNodes(CTagSet(TAG_DEFINERGBCOLOUR, TAG_DEFINECOMPLEXCOLOUR), v, true);

				if(v.size())
				{
					CXfSimpleColor* pColor = (CXfSimpleColor*)v[0];
					if (pColor->Color == RGB(255, 255, 255))
					{
						//pFirstObject->GetParent()->RemoveChild(pFirstObject);
						pFirstObject->SetRemoved(true);
						bSearchAgain = true;
						bRemoved = true;
					}
				}
			}
		}
	}
	return bRemoved;
}
