#ifndef _TABLE_UNICODE_MASK_H_
#define _TABLE_UNICODE_MASK_H_

enum
{
	UNICODE_MASK_UNKNOWN        = 0x00, // 0000 0000 0000

	UNICODE_MASK_LETTER_CAPITAL      = 0x01,
	UNICODE_MASK_BULLET              = 0x02,
	UNICODE_MASK_QUOTES_CLOSE        = 0x04,
	UNICODE_MASK_QUOTES_OPEN         = 0x08,
	UNICODE_MASK_NUMBER              = 0x10,
	UNICODE_MASK_DASH                = 0x20,
	UNICODE_MASK_STATEMENT_END       = 0x40,
	UNICODE_MASK_INVISIBLE           = 0x80,
	UNICODE_MASK_XARA_WORD_DELIMITER = 0x100
};

// based on unicode character table definitions, i.e.
// http://www.fileformat.info/info/unicode/category/index.htm

int GetUnicodeMask(wchar_t u)
{
	static const size_t TABLE_UNICODE_MASK_LENGTH = 0x1FFFF;

	static bool initTable = false;
	static WORD tableUnicodeMask[TABLE_UNICODE_MASK_LENGTH];

	if (initTable)
		return tableUnicodeMask[static_cast<size_t>(u)];

	initTable = true;

	for (size_t i=0; i<TABLE_UNICODE_MASK_LENGTH; ++i)
	{
		tableUnicodeMask[i] = UNICODE_MASK_UNKNOWN;

		if (
			   i == L' ' 
			|| i == L'-' 
			)
			tableUnicodeMask[i] |= UNICODE_MASK_XARA_WORD_DELIMITER;

		if (
			/*   i == L'*' 
			|| i == L'+' 
			|| i == L'.' 
			|| i == L'~' 
			|| i == L'#' 
			||*/

			   i == 0x00B0 // some chars are from http://en.wikipedia.org/wiki/Bullet_(typography)

			
			|| i == 0xF09E // wingdings
			|| i == 0x00B7 // MIDDLE DOT

			|| i == 0x00BA

			|| i == 0xF0B7 // symbol
			|| i == 0xF09F // wingdings
			|| i == 0x2022 // BULLET

			|| i == 0x2023 // TRIANGULAR BULLET
			|| i == 0x2043 // HYPHEN BULLET
			|| i == 0x204C // BLACK LEFTWARDS BULLET
			|| i == 0x204D // BLACK RIGHTWARDS BULLET

			|| i == 0x2192 // RIGHTWARDS ARROW

			|| i == 0x2219 // BULLET OPERATOR

			|| i == 0xF06E // wingdings
			|| i == 0x25A0 // BLACK SQUARE

			|| i == 0x25A1

			|| i == 0xF0A0 // wingdings
			|| i == 0xF0A7 // wingdings
			|| i == 0x25AA // BLACK SMALL SQUARE

			|| i == 0x25AB
			|| i == 0x25C6
			|| i == 0x25CB
			|| i == 0x25CC

			|| i == 0xF06C // wingdings
			|| i == 0x25CF // BLACK CIRCLE

			|| i == 0x25CA
			|| i == 0x25D8 // INVERSE BULLET
			|| i == 0x25D9
			|| i == 0x25E6 // WHITE BULLET

			//|| i == 0x2619 // REVERSED ROTATED FLORAL HEART BULLET

			|| i == 0x263A
			|| i == 0x263B
			|| i == 0x2666

			|| i == 0x2705 // WHITE HEAVY CHECK MARK
			|| i == 0x2713 // CHECK MARK
			|| i == 0x2714 // HEAVY CHECK MARK

			|| i == 0x2756 // BLACK DIAMOND MINUS WHITE X
			|| i == 0xF076 // BLACK DIAMOND MINUS WHITE X (Wingdings)

			|| i == 0x2794 // HEAVY WIDE-HEADED RIGHTWARDS ARROW

			|| i == 0x2799 // HEAVY RIGHTWARDS ARROW

			|| i == 0x279B // DRAFTING POINT RIGHTWARDS ARROW
			|| i == 0x279C // HEAVY ROUND-TIPPED RIGHTWARDS ARROW
			|| i == 0x279D // TRIANGLE-HEADED RIGHTWARDS ARROW
			|| i == 0x279E // HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW
			|| i == 0x279F // DASHED TRIANGLE-HEADED RIGHTWARDS ARROW
			|| i == 0x27A0 // HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW
			|| i == 0x27A1 // BLACK RIGHTWARDS ARROW
			|| i == 0x27A2 // THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD
			|| i == 0x27A3 // THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD
			|| i == 0x27A4 // BLACK RIGHTWARDS ARROWHEAD
			|| i == 0x27A5 // HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW
			|| i == 0x27A6 // HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW
			|| i == 0x27A7 // SQUAT BLACK RIGHTWARDS ARROW
			|| i == 0x27A8 // HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW
			|| i == 0x27A9 // RIGHT-SHADED WHITE RIGHTWARDS ARROW
			|| i == 0x27AA // LEFT-SHADED WHITE RIGHTWARDS ARROW
			|| i == 0x27AB // BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW
			|| i == 0x27AC // FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW
			|| i == 0x27AD // HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
			|| i == 0x27AE // HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
			|| i == 0x27AF // NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW

			|| i == 0x27B1 // NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
			|| i == 0x27B2 // CIRCLED HEAVY WHITE RIGHTWARDS ARROW
			|| i == 0x27B3 // WHITE-FEATHERED RIGHTWARDS ARROW

			|| i == 0x27B5 // BLACK-FEATHERED RIGHTWARDS ARROW

			|| i == 0x27B8 // HEAVY BLACK-FEATHERED RIGHTWARDS ARROW

			|| i == 0x27BA // TEARDROP-BARBED RIGHTWARDS ARROW
			|| i == 0x27BB // HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW
			|| i == 0x27BC // WEDGE-TAILED RIGHTWARDS ARROW
			|| i == 0x27BD // HEAVY WEDGE-TAILED RIGHTWARDS ARROW
			|| i == 0x27BE // OPEN-OUTLINED RIGHTWARDS ARROW

			//|| i == 0x2765 // ROTATED HEAVY BLACK HEART BULLET
			//|| i == 0x2767 // ROTATED FLORAL HEART BULLET
			//|| i == 0x29BE // CIRCLED WHITE BULLET
			//|| i == 0x29BF // CIRCLED BULLET

			|| i == 0x2B95 // RIGHTWARDS BLACK ARROW
			|| i == 0x2B9A // THREE-D TOP-LIGHTED RIGHTWARDS EQUILATERAL ARROWHEAD
			|| i == 0x2B9E // BLACK RIGHTWARDS EQUILATERAL ARROWHEAD
			|| i == 0x2BA9 // BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW
			|| i == 0x2BAB // BLACK CURVED UPWARDS AND RIGHTWARDS ARROW

			|| i == 0xFE5F
			|| i == 0xFE61
			|| i == 0xFF03
			|| i == 0xFF0A
		 )
		 tableUnicodeMask[i] |= UNICODE_MASK_BULLET;
		
		if (
			   i == L'\''
			|| i == L'"'
			|| i == 0x00BB
			|| i == 0x2019
			|| i == 0x201D
			|| i == 0x203A
			|| i == 0x2E03
			|| i == 0x2E05
			|| i == 0x2E0A
			|| i == 0x2E0D
			|| i == 0x2E1D
			|| i == 0x2E21
		 )
		 tableUnicodeMask[i] |= UNICODE_MASK_QUOTES_CLOSE;
		
		if (
			   i == L'\''
			|| i == L'"'
			|| i == 0x00AB
			|| i == 0x2018
			|| i == 0x201B
			|| i == 0x201C
			|| i == 0x201F
			|| i == 0x2039
			|| i == 0x2E02
			|| i == 0x2E04
			|| i == 0x2E09
			|| i == 0x2E0C
			|| i == 0x2E1C
			|| i == 0x2E20
		 )
		 tableUnicodeMask[i] |= UNICODE_MASK_QUOTES_OPEN;
		
		if (
			// capital letters
			   i == 0x01C5
			|| i == 0x01C8
			|| i == 0x01CB
			|| i == 0x01F2
			||(i >= 0x1F88 && i <= 0x1F8F)
			||(i >= 0x1F98 && i <= 0x1F9F)
			||(i >= 0x1FA8 && i <= 0x1FAF)
			|| i == 0x1FBC
			|| i == 0x1FCC
			|| i == 0x1FFC
			//upper letters
			||(i >= 0x0041 && i <= 0x005A)
			||(i >= 0x00C0 && i <= 0x00DE)
			||(i >= 0x0100 && i <= 0x010E && !(i & 0x01))
			||(i >= 0x0110 && i <= 0x0136 && !(i & 0x01))
			||(i >= 0x0139 && i <= 0x0147 &&  (i & 0x01))
			||(i >= 0x014A && i <= 0x0178 && !(i & 0x01))
			||(i >= 0x0179 && i <= 0x0181 &&  (i & 0x01))
			||(i >= 0x0182 && i <= 0x0186 && !(i & 0x01))
			|| i == 0x0187
			|| i == 0x0189
			|| i == 0x018A
			|| i == 0x018B
			|| i == 0x018E
			|| i == 0x018F
			|| i == 0x0190
			|| i == 0x0191
			|| i == 0x0193
			|| i == 0x0194
			|| i == 0x0196
			|| i == 0x0197
			|| i == 0x0198
			|| i == 0x019C
			|| i == 0x019D
			|| i == 0x019F
			|| i == 0x01A0
			|| i == 0x01A2
			|| i == 0x01A4
			|| i == 0x01A6
			|| i == 0x01A7
			|| i == 0x01A9
			|| i == 0x01AC
			|| i == 0x01AE
			|| i == 0x01AF
			|| i == 0x01B1
			|| i == 0x01B2
			|| i == 0x01B3
			|| i == 0x01B5
			|| i == 0x01B7
			|| i == 0x01B8
			|| i == 0x01BC
			|| i == 0x01C4
			|| i == 0x01C7
			|| i == 0x01CA
			|| i == 0x01CD
			|| i == 0x01CF
			|| i == 0x01D1
			|| i == 0x01D3
			|| i == 0x01D5
			|| i == 0x01D7
			|| i == 0x01D9
			|| i == 0x01DB
			|| i == 0x01DE
			||(i >= 0x01E0 && i <= 0x01EE && !(i & 0x01))
			|| i == 0x01F1
			|| i == 0x01F4
			|| i == 0x01F6
			|| i == 0x01F7
			|| i == 0x01F8
			||(i >= 0x0200 && i <= 0x0232 && !(i & 0x01))
			|| i == 0x023A
			|| i == 0x023B
			|| i == 0x023E
			|| i == 0x0241
			|| i == 0x0243
			|| i == 0x0244
			|| i == 0x0245
			|| i == 0x0245
			|| i == 0x0246
			|| i == 0x0248
			|| i == 0x024A
			|| i == 0x024C
			|| i == 0x024E
			|| i == 0x0370
			|| i == 0x0372
			|| i == 0x0376
			|| i == 0x037F
			|| i == 0x0386
			|| i == 0x0388
			|| i == 0x0389
			|| i == 0x038A
			|| i == 0x038C
			|| i == 0x038E
			|| i == 0x038F
			||(i >= 0x0391 && i <= 0x03A1)
			||(i >= 0x03A3 && i <= 0x03AB)
			|| i == 0x03CF
			|| i == 0x03D2
			|| i == 0x03D3
			|| i == 0x03D4
			||(i >= 0x03D8 && i <= 0x03EE && !(i & 0x01))
			|| i == 0x03F4
			|| i == 0x03F7
			|| i == 0x03F9
			|| i == 0x03FA
			||(i >= 0x03F9 && i <= 0x042F)
			||(i >= 0x0460 && i <= 0x0480 && !(i & 0x01))
			||(i >= 0x048A && i <= 0x04C0 && !(i & 0x01))
			||(i >= 0x04C1 && i <= 0x04CD &&  (i & 0x01))
			||(i >= 0x04D0 && i <= 0x052E && !(i & 0x01))
			||(i >= 0x0531 && i <= 0x0556)
			||(i >= 0x10A0 && i <= 0x10C5)
			|| i == 0x10C7
			|| i == 0x10CD
			||(i >= 0x1E00 && i <= 0x1E94 && !(i & 0x01))
			||(i >= 0x1E9E && i <= 0x1EFE && !(i & 0x01))
			||(i >= 0x1F08 && i <= 0x1F0F)
			||(i >= 0x1F18 && i <= 0x1F1D)
			||(i >= 0x1F28 && i <= 0x1F2F)
			||(i >= 0x1F38 && i <= 0x1F3F)
			||(i >= 0x1F48 && i <= 0x1F4D)
			||(i >= 0x1F59 && i <= 0x1F5F &&  (i & 0x01))
			||(i >= 0x1F68 && i <= 0x1F6F)
			||(i >= 0x1FB8 && i <= 0x1FBB)
			||(i >= 0x1FC8 && i <= 0x1FCB)
			||(i >= 0x1FD8 && i <= 0x1FDB)
			||(i >= 0x1FE8 && i <= 0x1FEB)
			||(i >= 0x1FF8 && i <= 0x1FFB)
			// skipped double-struck capitals
			// skipped black-letter capitals
			// skipped glagolitic capitals
			|| i == 0x2C60
			|| i == 0x2C62
			|| i == 0x2C63
			|| i == 0x2C64
			|| i == 0x2C67
			|| i == 0x2C69
			|| i == 0x2C6B
			||(i >= 0x2C6D && i <= 0x2C70)
			|| i == 0x2C72
			|| i == 0x2C75
			|| i == 0x2C7E
			|| i == 0x2C7F
			// skipped coptic capital letters
			// skipped cyrillic extended capital letters
			||(i >= 0xA724 && i <= 0xA76E && !(i & 0x01))
			|| i == 0xA779
			|| i == 0xA77B
			|| i == 0xA77D
			|| i == 0xA77E
			|| i == 0xA780
			|| i == 0xA782
			|| i == 0xA784
			|| i == 0xA786
			|| i == 0xA78B
			|| i == 0xA78D
			|| i == 0xA790
			|| i == 0xA792
			|| i == 0xA796
			|| i == 0xA798
			|| i == 0xA79A
			|| i == 0xA79C
			|| i == 0xA79E
			|| i == 0xA7A0
			|| i == 0xA7A2
			|| i == 0xA7A4
			|| i == 0xA7A6
			|| i == 0xA7A8
			|| i == 0xA7AA
			|| i == 0xA7AB
			|| i == 0xA7AC
			|| i == 0xA7AD
			|| i == 0xA7B0
			|| i == 0xA7B1
			||(i >= 0xFF21 && i <= 0xFF3A)
			// skipped deseret capital letters
			// skipped warang citi capital letters
			// skipped mathematical (bold, script, fractur, etc.) capital letters
		 )
		 tableUnicodeMask[i] |= UNICODE_MASK_LETTER_CAPITAL;
		
		if (
		   (i >= L'0' && i <= L'9')
		 || i == 0x16EE
		 || i == 0x16EF
		 || i == 0x16F0
		 ||(i >= 0x2160 && i <= 0x2182)
		 ||(i >= 0x2185 && i <= 0x2188)
		 ||(i >= 0x2160 && i <= 0x2182)
		 || i == 0x3007
		 ||(i >= 0x3021 && i <= 0x3029)
		 ||(i >= 0x3038 && i <= 0x303A)
		 || i == 0x00B2
		 || i == 0x00B3
		 || i == 0x00B9
		 || i == 0x00BC
		 || i == 0x00BD
		 || i == 0x00BE
		 ||(i >= 0x09F4 && i <= 0x09F9)
		 ||(i >= 0x0B72 && i <= 0x0B77)
		 ||(i >= 0x0BF0 && i <= 0x0BF2)
		 ||(i >= 0x0C78 && i <= 0x0C7E)
		 ||(i >= 0x0D70 && i <= 0x0D75)
		 ||(i >= 0x0F2A && i <= 0x0F33)
		 ||(i >= 0x1369 && i <= 0x137C)
		 ||(i >= 0x17F0 && i <= 0x17F9)
		 || i == 0x19DA
		 || i == 0x2070
		 ||(i >= 0x2074 && i <= 0x2089)
		 ||(i >= 0x2150 && i <= 0x215F)
		 ||(i >= 0x2074 && i <= 0x2089)
		 || i == 0x2189
		 ||(i >= 0x2460 && i <= 0x249B)
		 ||(i >= 0x24EA && i <= 0x24FF)
		 ||(i >= 0x2776 && i <= 0x2793)
		 ||(i >= 0x3192 && i <= 0x3195)
		 ||(i >= 0x3220 && i <= 0x3229)
		 ||(i >= 0x3248 && i <= 0x325F)
		 ||(i >= 0x3280 && i <= 0x3289)
		 ||(i >= 0x32B1 && i <= 0x32BF)
		 ||(i >= 0x2150 && i <= 0x215F)
		 // skipped aegean, greek extended, coptic, aramaic, palmyrene, nabatean, etc. numbers
		 ||(i >= 0x1F100 && i <= 0x1F10C)
		 )
		 tableUnicodeMask[i] |= UNICODE_MASK_NUMBER;
		
		if (
			i == 0x002D
		 || i == 0x058A
		 || i == 0x05BE
		 || i == 0x1400
		 || i == 0x1806
		 ||(i >= 0x2010 && i <= 0x2015)
		 || i == 0x2E17
		 || i == 0x2E1A
		 || i == 0x2E3A
		 || i == 0x2E3B
		 || i == 0x2E40
		 || i == 0x301C
		 || i == 0x3030
		 || i == 0x30A0
		 || i == 0xFE31
		 || i == 0xFE32
		 || i == 0xFE58
		 || i == 0xFE63
		 || i == 0xFF0D
		)
			tableUnicodeMask[i] |= UNICODE_MASK_DASH;
		
		if (
		 // control symbols & space
		   (i <= 0x0020)
		 ||(i >= 0x007F && i <= 0x009F)
		 // surrogates
		 || i == 0x00AD
		 || i == 0xD800
		 || i == 0xDB7F
		 || i == 0xDB80
		 || i == 0xDBFF
		 || i == 0xDC00
		 || i == 0xDFFF
		 // separators
		 || i == 0x2028
		 || i == 0x2029
		 // spaces
		 || i == 0x00A0
		 || i == 0x1680
		 ||(i >= 0x2000 && i <= 0x200A)
		 || i == 0x202F
		 || i == 0x205F
		 || i == 0x3000
		)
			tableUnicodeMask[i] |= UNICODE_MASK_INVISIBLE;

		if (
		   i == L'!'
		|| i == L'.'
		|| i == L';'
		|| i == L'?'
		|| i == 0x00A1
		|| i == 0x00BF
		|| i == 0x037E
		|| i == 0x055C
		|| i == 0x05C3
		|| i == 0x055E
		|| i == 0x0589
		|| i == 0x061B
		|| i == 0x061E
		|| i == 0x061F
		|| i == 0x06D4
		||(i >= 0x0700 && i <= 0x0702)
		|| i == 0x07F9
		|| i == 0x1362
		|| i == 0x1364
		|| i == 0x1367
		|| i == 0x166E
		|| i == 0x1801
		|| i == 0x1803
		|| i == 0x1809
		|| i == 0x1944
		|| i == 0x1945
		|| i == 0x2026
		|| i == 0x203C
		|| i == 0x203D
		||(i >= 0x2047 && i <= 0x2049)
		|| i == 0x204F
		|| i == 0x2E18
		|| i == 0x2E2E
		|| i == 0x2E35
		|| i == 0x2E3C
		|| i == 0x3002
		|| i == 0xA60E
		|| i == 0xA60F
		|| i == 0xA673
		|| i == 0xFE52
		|| i == 0xFE54
		|| i == 0xFE56
		|| i == 0xFE57
		|| i == 0xFF01
		|| i == 0xFF0E
		|| i == 0xFF1B
		|| i == 0xFF1F
		|| i == 0xFF61
		)
			tableUnicodeMask[i] |= UNICODE_MASK_STATEMENT_END;
	}

	return GetUnicodeMask(u);
}

#endif