/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/

#include "stdafx.h"
#include "Tables.h"
#include "rapidjson/document.h"
#include "rapidjson/writer.h"
#include "rapidjson/stringbuffer.h"

using namespace rapidjson;
typedef GenericValue<UTF16<>> ValueU;

const UINT cuiSearchRegion = 250;
const UINT cuiMaxLineWidth = 1200;

// Word tables are constructed as that: 
// a) all lines forming table cells are filled rectangles or bars (in the cell rect connections)

Rect CalcRect(CXfPath* pPath)
{
	Rect rc;
	for (auto i = 0; i < pPath->Coords.size(); i++)
	{
		if (i == 0)
		{
			rc.x = pPath->Coords[i].x;
			rc.y = pPath->Coords[i].y;
		}
		else
			Rect::InflateRectToContainPoint(rc, pPath->Coords[i].x, pPath->Coords[i].y);
	}
	return rc;
}

TableSide::TableSide(Rect& rc, COLORREF cr, LONG lineWidth) : m_rc(rc), m_cr(cr), m_lw(lineWidth)
{
	m_type = rc.width > rc.height ? keHoriz : (rc.height > rc.width ? keVert : keJoin);
	if (TableObject::Ratio(rc) < 3 && rc.width <= cuiMaxLineWidth && rc.height <= cuiMaxLineWidth)
		m_type = keJoin;
}

TableSide* TableSide::FindTopLeft()
{
	TableSide* pNode = this;
	while (pNode->m_pLeft && pNode->m_pLeft->TableID() == this->TableID()) pNode = pNode->m_pLeft;
	while (pNode->m_pTop && pNode->m_pTop->TableID() == this->TableID()) pNode = pNode->m_pTop;
	return pNode;
}

TableSide* TableSide::FindBottomRight()
{
	TableSide* pNode = this;
	while (pNode->m_pRight && pNode->m_pRight->TableID() == this->TableID()) pNode = pNode->m_pRight;
	while (pNode->m_pBottom && pNode->m_pBottom->TableID() == this->TableID()) pNode = pNode->m_pBottom;
	return pNode;
}

size_t TableSide::NumColumns(TableSide* pStart)
{
	int id = pStart->TableID();
	size_t width = pStart->m_type == keHoriz ? 1 : 0;
	while (pStart->m_pRight && pStart->m_pRight->TableID() == id)
	{
		pStart = pStart->m_pRight;

		if (pStart->m_type == keHoriz)
			width++;
	}
	return width;
}

size_t TableSide::NumRows(TableSide* pStart)
{
	int id = pStart->TableID();
	size_t height = pStart->m_type == keVert ? 1 : 0;
	while (pStart->m_pBottom && pStart->m_pBottom->TableID() == id)
	{
		pStart = pStart->m_pBottom;
		if (pStart->m_type == keVert)
			height++;
	}
	return height;
}

TableSide* TableSide::NextJoin(Direction dir)
{
	TableSide* pNode = this;
	int id = TableID();

	switch (dir)
	{
	case up:
		while (pNode->m_pTop && pNode->m_pTop->TableID() == id)
		{
			pNode = pNode->m_pTop;
			if (pNode->m_type == keJoin)
				break;
		}
		break;
	case down:
		while (pNode->m_pBottom && pNode->m_pBottom->TableID() == id)
		{
			pNode = pNode->m_pBottom;
			if (pNode->m_type == keJoin)
				break;
		}
		break;
	case left:
		while (pNode->m_pLeft && pNode->m_pLeft->TableID() == id)
		{
			pNode = pNode->m_pLeft;
			if (pNode->m_type == keJoin)
				break;
		}
		break;
	case right:
		while (pNode->m_pRight && pNode->m_pRight->TableID() == id)
		{
			pNode = pNode->m_pRight;
			if (pNode->m_type == keJoin)
				break;
		}
		break;
	}
	return pNode;
}

bool TableSide::CheckDirection(TableSide* pOther, Direction dir)
{
	if ((dir == left || dir == right) && m_rc.height != pOther->rect().height)
		return false;
	if ((dir == up || dir == down) && m_rc.width != pOther->rect().width)
		return false;

	Point c2 = m_rc.center(), c1 = pOther->rect().center();

	if (dir == left && c1.x < c2.x && c1.y == c2.y)
		return true;
	else if (dir == right && c1.x > c2.x && c1.y == c2.y)
		return true;
	else if (dir == up && c1.x == c2.x && c1.y > c2.y)
		return true;
	else if (dir == down && c1.x == c2.x && c1.y < c2.y)
		return true;
	return false;
}

TableReconstruct::TableReconstruct(LONG lPageWidth, LONG lPageHeight, RefNodeMap& map, VXfTreeNode* pPage) : m_refMap(map), m_pPage(pPage)
{
	m_tree.Create(lPageWidth, lPageHeight, 8);
}

bool checkIntersection(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	// perform the test, return 1 if the boxes intersect 
	if ((MAX(x1, x2) >= MIN(x3, x4)) &&
		(MAX(x3, x4) >= MIN(x1, x2)) &&
		(MAX(y1, y2) >= MIN(y3, y4)) &&
		(MAX(y3, y4) >= MIN(y1, y2))) return true;

	return false;
}

void TableReconstruct::CheckLeft(std::queue<TableSide*>& checkQueue, TableSide* pNode)
{
	std::vector<VElementRect*> elems;

	m_tree.FindIntersectElements(elems, Rect(pNode->m_rc.x - cuiSearchRegion, pNode->m_rc.y, cuiSearchRegion, pNode->m_rc.height));

	for (auto i = 0; i < elems.size(); i++)
	{
		TableSide* pLeftSide = (TableSide*)elems[i];

// 		int x1, x2, x3, x4, y1, y2, y3, y4;
// 		pNode->GetRect(x1, y1, x2, y2);
// 		pLeftSide->GetRect(x3, y3, x4, y4);
// 
// 		if (!checkIntersection(x1, y1, x2, y2, x3, y3, x4, y4))
// 		{
// 			int vvv = 5;
// 		}

		if (pLeftSide != pNode && pNode->CheckDirection(pLeftSide, TableSide::left))
		{
			pLeftSide->m_pRight = pNode;
			pNode->m_pLeft = pLeftSide;
			pNode->SetAdded();
			pNode->m_iTableID = (int)m_uiTableCnt;

			if (!pLeftSide->IsAdded())
				checkQueue.push(pLeftSide);
			break;
		}
	}
}

void TableReconstruct::CheckRight(std::queue<TableSide*>& checkQueue, TableSide* pNode)
{
	std::vector<VElementRect*> elems;
	m_tree.FindIntersectElements(elems, Rect(pNode->m_rc.right(), pNode->m_rc.y, cuiSearchRegion, pNode->m_rc.height));

	for (auto i = 0; i < elems.size(); i++)
	{
		TableSide* pRightSide = (TableSide*)elems[i];
		if (pRightSide != pNode && pNode->CheckDirection(pRightSide, TableSide::right))
		{
			pRightSide->m_pLeft = pNode;
			pNode->m_pRight = pRightSide;
			pNode->SetAdded();
			pNode->m_iTableID = (int)m_uiTableCnt;

			if (!pRightSide->IsAdded())
				checkQueue.push(pRightSide);
			break;
		}
	}
}

void TableReconstruct::CheckTop(std::queue<TableSide*>& checkQueue, TableSide* pNode)
{
	std::vector<VElementRect*> elems;

	m_tree.FindIntersectElements(elems, Rect(pNode->m_rc.x, pNode->m_rc.y + pNode->m_rc.height, pNode->m_rc.width, cuiSearchRegion));

	for (auto i = 0; i < elems.size(); i++)
	{
		TableSide* pTopSide = (TableSide*)elems[i];
		if (pTopSide != pNode && pNode->CheckDirection(pTopSide, TableSide::up))
		{
			pTopSide->m_pBottom = pNode;
			pNode->m_pTop = pTopSide;
			pNode->SetAdded();
			pNode->m_iTableID = (int)m_uiTableCnt;

			if (!pTopSide->IsAdded())
				checkQueue.push(pTopSide);
			break;
		}
	}
}

void TableReconstruct::CheckBottom(std::queue<TableSide*>& checkQueue, TableSide* pNode)
{
	std::vector<VElementRect*> elems;
	m_tree.FindIntersectElements(elems, Rect(pNode->m_rc.x, pNode->m_rc.y - cuiSearchRegion, pNode->m_rc.width, cuiSearchRegion));

	for (auto i = 0; i < elems.size(); i++)
	{
		TableSide* pBottomSide = (TableSide*)elems[i];
		if (pBottomSide != pNode && pNode->CheckDirection(pBottomSide, TableSide::down))
		{
			pBottomSide->m_pTop = pNode;
			pNode->m_pBottom = pBottomSide;
			pNode->SetAdded();
			pNode->m_iTableID = (int)m_uiTableCnt;

			if (!pBottomSide->IsAdded())
				checkQueue.push(pBottomSide);
			break;
		}
	}
}

bool TableReconstruct::FindSiblings()
{
	std::queue<TableSide*> checkQueue;
	bool bFound = false;
	INT left = INT_MAX;
	INT top = INT_MIN;
	size_t idxLeftTop = 0;

	// Find the most left-top non added rectangle to start from
	for (size_t i = 0; i < m_sides.size(); i++)
	{
		if (m_sides[i].IsAdded() == false)
		{
			INT x = m_sides[i].m_rc.left();
			INT y = m_sides[i].m_rc._top();
			if (x < left && y >= top || x <= left && y > top)
			{
				left = x; top = y; idxLeftTop = i;
				bFound = true;
			}
		}
	}

	if (bFound)
	{
		checkQueue.push(&m_sides[idxLeftTop]);
	}

	int vvv = 5;
	while (checkQueue.size())
	{
		TableSide* pNode = checkQueue.front();

		if (pNode->m_type == TableSide::keHoriz)
		{
			CheckLeft(checkQueue, pNode);
			CheckRight(checkQueue, pNode);
		}
		else if (pNode->m_type == TableSide::keVert)
		{
			CheckTop(checkQueue, pNode);
			CheckBottom(checkQueue, pNode);
		}
		else
		{
			CheckLeft(checkQueue, pNode);
			CheckRight(checkQueue, pNode);
			CheckTop(checkQueue, pNode);
			CheckBottom(checkQueue, pNode);
		}

		if (!pNode->IsAdded())
		{
			pNode->SetAdded();
			pNode->m_iTableID = -1;
		}

		checkQueue.pop();
	}

	if (bFound) m_uiTableCnt++;
	return bFound;
}

void TableReconstruct::AddRects(VXfTreeNode* pPage)
{
	CXfPath* pPath = (CXfPath*)m_pPage->FindFirstNode(TAG_PATH_FILLED_STROKED);
	if (pPath && pPath->GetParent()->GetTag() == TAG_LAYER)
	{
		VXfTreeNode* pLayer = pPath->GetParent();
		for (size_t i = 0; i < pLayer->ChildrenCount(); i++)
		{
			VXfTreeNode* pNode = pLayer->GetChild(i);
			if (pNode->GetTag() == TAG_PATH_FILLED_STROKED)
			{
				CXfPath* pPath = (CXfPath*)pNode;
				if (pPath->IsSimpleRectangle() && pPath->FindFirstNode(TAG_FLATFILL))
				{
					COLORREF cr = ((CXfFlatFill*)pPath->FindFirstNode(TAG_FLATFILL))->GetRGBColor(m_refMap);
					LONG lw = (LONG)pPath->Context().m_dwLineWidth;
					m_sides.push_back(TableSide(CalcRect(pPath), cr, lw));
				}
			}
		}
	}

	for (size_t i = 0; i < m_sides.size(); i++)
	{
		std::vector<VElementRect*> elems;
		m_tree.FindIntersectElements(elems, m_sides[i].rect());
		bool bDuplicate = false;

		for (size_t j = 0; j < elems.size(); j++)
		{
			TableSide* pSide = (TableSide*)elems[j];
			// Guess what? PDF tables have totally duplicate join rectangles sometimes! So add only if no dup, correct the colour otherwise
			if (pSide->rect() == m_sides[i].rect())
			{
				pSide->color() = m_sides[i].color(); // we are higher in z-order - so change the colour to us
				m_sides[i].SetAdded(); // exclude this node as being duplicate
				bDuplicate = true;
			}
		}

		if (!bDuplicate)
			m_tree.AddElement(&m_sides[i]);
	}
}


double TableObject::Ratio(Rect& rc)
{
	INT iMaxDim = MAX(rc.width, rc.height);
	INT iMinDim = MIN(rc.width, rc.height);
	if (!iMinDim)
		return (double)INT_MAX;
	return (double)iMaxDim / (double)iMinDim;
}

void TableObject::FixJoins()
{
	TableSide* pNode = m_ptl;
	std::vector<TableSide*> row;
	
	while (pNode)
	{
		if (Ratio(pNode->rect()) < 3 && pNode->rect().width <= cuiMaxLineWidth && pNode->rect().height <= cuiMaxLineWidth)
		{
			pNode->m_type = TableSide::keJoin;
			row.push_back(pNode);
		}

		pNode = pNode->m_pBottom;
		assert(Ratio(pNode->rect()) > 3 || pNode->rect().width > cuiMaxLineWidth || pNode->rect().height <= cuiMaxLineWidth);
		if (pNode)
			pNode = pNode->m_pBottom;
	}
}

TableObject::TableObject(TableSide* pTopLeft) : m_ptl(pTopLeft)
{
	numCols = TableSide::NumColumns(pTopLeft);
	numRows = TableSide::NumRows(pTopLeft);
	TableSide* pNode = pTopLeft;

	for (size_t i = 0; i < numRows; i++)
	{
		m_rows.push_back(pNode);

		if (i == numRows - 1) break;

		pNode = pNode->m_pBottom;
		while (pNode->m_type != TableSide::keJoin)
			pNode = pNode->m_pBottom;
		if (!pNode) assert(false);
	}
}

LONG TableObject::getRowHeight(size_t idx)
{
	TableSide* pNextJoin = m_rows[idx]->NextJoin(TableSide::down);

	if (pNextJoin == nullptr)
	{
		if (m_rows[idx]->m_pBottom)
			return m_rows[idx]->point().y - m_rows[idx]->m_pBottom->rect().y;
		return 0;
	}

	return m_rows[idx]->point().y - pNextJoin->point().y;
}

TableSide* TableObject::column(TableSide* prow, size_t idx)
{
	TableSide* pNode = prow;
	for (size_t i = 0; i < idx; i++)
		pNode = pNode->NextJoin(TableSide::right);
	return pNode;
}

LONG TableObject::getColumnWidth(size_t idx)
{
	TableSide* pNode = column(m_ptl, idx);
	return pNode->NextJoin(TableSide::right)->point().x - pNode->point().x;
}

COLORREF TableObject::getOutlineColor(size_t row, size_t col)
{
	return column(m_rows[row], col)->color();
}

// The code snippet below is taken from Word import filter and modified a bit
std::wstring TableReconstruct::GetJSONTableString(TableSide* pTopLeft, TableObject& obj)
{
	Point ptTopLeft = pTopLeft->point();
	TableSide* pBottomRight = pTopLeft->FindBottomRight();
	Point ptBottomRight = pBottomRight->point();
	INT width = ptBottomRight.x - ptTopLeft.x;
	INT height = ptTopLeft.y - ptBottomRight.y;

	size_t numCols = obj.numCols;
	size_t numRows = obj.numRows;

	GenericStringBuffer<UTF16<>> s;
	Writer<GenericStringBuffer<UTF16<>>> writer(s);

	writer.StartObject();
	writer.String("sizeX"); // Each smart shape has it's own coordinate space with (0,0) in the centre of the shape, sizeX and sizeY are the distances from the centre to the shape boundaries
	writer.Int(width/2);
	writer.String("sizeY");
	writer.Int(height/2);
	writer.String("constrained");
	writer.Bool(false);
	writer.String("centred");
	writer.Bool(false);
	writer.String("forceConstrain");
	writer.Bool(false);
	writer.String("lineWidth");
	writer.Int(1000);
	writer.String("lineColour");
	writer.String("#000000");
	writer.String("hasStroke");
	writer.Bool(false);

	writer.String("numRows");
	writer.Int((INT)numRows);
	writer.String("numCols");
	writer.Int((INT)numCols);

	writer.String("outerPadding");
	writer.Int(0);
	writer.String("innerPadding");
	writer.Int(0);
	writer.String("outerPadding");
	writer.Int(0);
	writer.String("cellPadding");
	writer.Int(0);
	writer.String("cellPaddingL");
	writer.Int(0);
	writer.String("cellPaddingR");
	writer.Int(0);
	writer.String("cellPaddingT");
	writer.Int(0);
	writer.String("cellPaddingB");
	writer.Int(0);
	writer.String("handleOffset");
	writer.Int(10500);
	writer.String("showCaption");
	writer.Bool(false);
	writer.String("textColumn");
	writer.Bool(false);
	writer.String("realSelection");
	writer.Bool(true);
	writer.String("defaultCellText");
	writer.String("");
	writer.String("captionText");
	writer.String("");
	writer.String("captionHeight");
	writer.Int(0);
	writer.String("captionPadding");
	writer.Int(0);
	writer.String("left");
	writer.Int(-width/2);
	writer.String("top");
	writer.Int(height/2);
	writer.String("rowProps");
	writer.StartArray();
	for (size_t _i(0); _i < numRows; ++_i)
	{
		LONG lHeight = obj.getRowHeight(_i);
		writer.StartObject();
		writer.String("height");
		writer.Int(lHeight);
		writer.String("desiredHeight");
		writer.Int(lHeight);
		writer.String("exactHeight");
		writer.Bool(false);
		writer.EndObject();
	}
	writer.EndArray();

	writer.String("colProps");
	writer.StartArray();
	for (size_t _i(0); _i < numCols; ++_i)
	{
		writer.StartObject();
		writer.String("width");
		writer.Int(obj.getColumnWidth(_i));
		writer.EndObject();
	}
	writer.EndArray();

	writer.String("cellProps");
	writer.StartArray();
	for (size_t _i(0); _i < numRows; ++_i){
		writer.StartArray();
		for (size_t _j(0); _j < numCols; ++_j){
			{
				writer.StartObject();
				writer.String("id");
				writer.Int(100 + 10 * (int)numCols*(int)_i + 10 * (int)_j);
				writer.String("isHorizontallyMerged");
				writer.Bool(false);
				writer.String("isVerticallyMerged");
				writer.Bool(false);
// 				if (curCell.HorzSpan() > 1){ //1 - default, is not merged; > 1 - a number of cells in the merged group
// 					writer.String("mergedHorzSpan");
// 					writer.Int(curCell.HorzSpan());
// 				}
// 				if (curCell.VertSpan() != 1){ // 1 - default, is not merged; -1 - need to be counted
// 					writer.String("mergedVertSpan");
// 					writer.Int(curCell.VertSpan());
// 				}
// 				if (curCell.TableCellProperties().TextDirection().has()){
// 					writer.String("textDirection");
// 					if (curCell.TableCellProperties().TextDirection().get(eTAD_lrTb) == eTAD_btLr){
// 						writer.Int(90);
// 					}
// 					else if (curCell.TableCellProperties().TextDirection().get(eTAD_lrTb) == eTAD_tbRl){
// 						writer.Int(270);
// 					}
// 					else
// 						writer.Int(0);
// 				}
				writer.String("selected");
				writer.Bool(false);
				writer.EndObject();
			}
		}
		writer.EndArray();
	}
	writer.EndArray();

	writer.String("nextCellID");
	writer.Int(100 + (int)numCols*(int)numRows * 10);

	writer.EndObject();

	return s.GetString();
};

bool RectContains(Rect rc1, Rect rc2)
{
	if (rc1.x <= rc2.x && rc1.y <= rc2.y && rc1.right() >= rc2.right() && rc1._top() >= rc2._top())
		return true;
	return false;
}

CXfFlatFill* TableReconstruct::CreateFill(CXfPath* pPath, Rect rc, INT iTableID)
{
	CXfFlatFill* pFill = nullptr;
	std::vector<VElementRect*> elems;
	m_tree.FindIntersectElements(elems, rc);

	if (elems.size())
	{
		std::map<COLORREF, double> mapSquare;

		for (size_t i = 0; i < elems.size(); i++)
		{
			TableSide* pNode = (TableSide*)elems[i];
			if (iTableID != pNode->TableID() && RectContains(rc, pNode->rect()))
			{
				if (mapSquare.find(pNode->color()) == mapSquare.end())
					mapSquare[pNode->color()] = 0;

				mapSquare[pNode->color()] += (double)pNode->rect().width * (double)pNode->rect().height;
			}
		}
		
		double dMaxSquare = 0;
		COLORREF color = 0;
		for (std::map<COLORREF, double>::iterator it = mapSquare.begin(); it != mapSquare.end(); ++it)
			if (it->second > dMaxSquare) 
			{ 
				dMaxSquare = it->second; 
				color = it->first; 
			}

		double dCellSquare = (double)rc.width*(double)rc.height;
		double dRatio = dMaxSquare / dCellSquare;
		if (dRatio > 0.85)
		{
			CXfComplexColor* pColor = nullptr;

			if (m_color.find(color) == m_color.end())
			{
				pColor = new CXfComplexColor(color, CXfComplexColor::cmRGB, GetRValue(color) / 255.0, GetGValue(color) / 255.0, GetBValue(color) / 255.0, 0);
				pPath->AppendChild(pColor);
				m_color[color] = pColor;
			}
			else
				pColor = m_color[color];

			pFill = new CXfFlatFill(pColor);
		}
	}
	else
		pFill = new CXfFlatFill(CXfFlatFill::None);

	return pFill;
}

void TableReconstruct::DumpTableGrid(int iTableID)
{
	if (!m_pLayer)
	{
		m_pLayer = new CXfLayer();
		m_pLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE, L"Tables"));
		m_pPage->AppendChild(m_pLayer);
	}

	CXfGroup* pGroup = new CXfGroup();
	m_pLayer->AppendChild(pGroup);

	for (size_t i = 0; i < m_sides.size(); i++)
	{
		TableSide* pNode = &m_sides[i];
		if (pNode->TableID() == iTableID)
		{
			CXfPath* pPath = new CXfPath(TAG_PATH_FILLED_STROKED);

			Rect rc = pNode->rect();
			COLORREF color = pNode->color();
			CXfComplexColor* pColor = nullptr;

			if (m_color.find(color) == m_color.end())
			{
				pColor = new CXfComplexColor(color, CXfComplexColor::cmRGB, GetRValue(color) / 255.0, GetGValue(color) / 255.0, GetBValue(color) / 255.0, 0);
				pPath->AppendChild(pColor);
				m_color[color] = pColor;
			}
			else
				pColor = m_color[color];

			pPath->Add(PT_MOVETO, DocCoord(rc.x, rc.y));
			pPath->Add(PT_LINETO, DocCoord(rc.right(), rc.y));
			pPath->Add(PT_LINETO, DocCoord(rc.right(), rc.bottom()));
			pPath->Add(PT_LINETO | PT_CLOSEFIGURE, DocCoord(rc.x, rc.bottom()));

			pPath->AppendChild(new CXfLineWidth(0));
			pPath->AppendChild(new CXfLineColor(CXfLineColor::None));
			if (pColor)
				pPath->AppendChild(new CXfFlatFill(pColor));
			else
				pPath->AppendChild(new CXfFlatFill(CXfFlatFill::None));

			pGroup->AppendChild(pPath);
		}
	}
}

void TableReconstruct::BuildTable(TableSide* pTopLeft)
{
	if (!m_pLayer)
	{
		m_pLayer = new CXfLayer();
		m_pLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE, L"Tables"));
		m_pPage->AppendChild(m_pLayer);
	}

	Point tl = pTopLeft->point();
	Point br = pTopLeft->FindBottomRight()->point();
	TableObject obj(pTopLeft);

	CXfGroup* pGroup = new CXfGroup();
	pGroup->AppendChild(new CXfSmartShapeGroup(0, DocCoord(tl.x, br.y), DocCoord(br.x, tl.y), GetJSONTableString(pTopLeft, obj), XaraMatrix((tl.x + br.x)/2, (tl.y + br.y)/2), L"SmartTable"));
	m_pLayer->AppendChild(pGroup);
	CXfPath* pBg = new CXfPath(TAG_PATH_FILLED_STROKED);
	pBg->Add(PT_MOVETO, DocCoord(tl.x, tl.y));
	pBg->Add(PT_LINETO, DocCoord(br.x, tl.y));
	pBg->Add(PT_LINETO, DocCoord(br.x, br.y));
	pBg->Add(PT_LINETO | PT_CLOSEFIGURE, DocCoord(tl.x, br.y));

	pBg->AppendChild(new CXfLineWidth(501));
	pBg->AppendChild(new CXfLineColor(CXfLineColor::None));
	pBg->AppendChild(new CXfUserValue(L"SmartGroup\\Id", FormatStringW(L"%d", 0).c_str())); // Bg rect ID == 0
	pBg->AppendChild(new CXfFlatFill(CXfFlatFill::None));
	pBg->AppendChild(new CXfJoinStyle(0));
	pGroup->AppendChild(pBg);
	
	CXfTextStorySimple* pCaption = new CXfTextStorySimple(tl.x, tl.y, FALSE);
	pCaption->AppendChild(new CXfNodeDetails(FALSE, FALSE)); // Make the caption initially invisible
	pCaption->AppendChild(new CXfTextStoryWordWrapInfo(br.x - tl.x));
	pCaption->AppendChild(new CXfTextFontSize(10000));
	pCaption->AppendChild(new CXfUserValue(L"SmartGroup\\Id", FormatStringW(L"%d", 1).c_str())); // Caption ID == 1
	pCaption->AppendChild(new CXfTextLine());
	pCaption->GetLastChild()->AppendChild(new CXfZeroSizedNode(TAG_TEXT_EOL));
	pGroup->AppendChild(pCaption);

	for (auto i = 0; i < obj.numRows; i++)
	{
		for (auto j = 0; j < obj.numCols; j++)
		{
			CXfPath* pPath = new CXfPath(TAG_PATH_FILLED_STROKED);
			TableSide* pCell = obj.column(obj.m_rows[i], j);
			COLORREF color = obj.getOutlineColor(i, j);
			CXfComplexColor* pColor = nullptr;
			size_t uiId = 100 + i*obj.numCols * 10 + j * 10;

			if (m_color.find(color) == m_color.end())
			{
				pColor = new CXfComplexColor(color, CXfComplexColor::cmRGB, GetRValue(color) / 255.0, GetGValue(color) / 255.0, GetBValue(color) / 255.0, 0);
				pPath->AppendChild(pColor);
				m_color[color] = pColor;
			}
			else
				pColor = m_color[color];

			LONG lineWidth = pCell->rect().width;// +pCell->m_lw;
			Rect rc, rc_;
			pPath->Add(PT_MOVETO, pCell->coord());
			rc.x = pCell->point().x;
			rc.y = pCell->point().y;

			TableSide* pNode = pCell->NextJoin(TableSide::right);
			pPath->Add(PT_LINETO, pNode->coord());
			rc.width = pNode->point().x - rc.x;

			pNode = pNode->NextJoin(TableSide::down);
			pPath->Add(PT_LINETO, pNode->coord());
			rc.height = rc.y - pNode->point().y;
			rc_ = rc; rc_.y -= rc.height; // make this rect an up-right for tree matching

			pNode = pNode->NextJoin(TableSide::left);
			pPath->Add(PT_LINETO | PT_CLOSEFIGURE, pNode->coord());

			pPath->AppendChild(new CXfLineWidth(lineWidth));
			pPath->AppendChild(new CXfLineColor(pColor));
			pPath->AppendChild(new CXfUserValue(L"SmartGroup\\Id", FormatStringW(L"%d", uiId).c_str()));
			pPath->AppendChild(CreateFill(pPath, rc_, pCell->TableID()));
			pPath->AppendChild(new CXfJoinStyle(0));
			pGroup->AppendChild(pPath);

			CXfTextStorySimple* pStory = new CXfTextStorySimple(rc.x, rc.y, FALSE);
			pStory->AppendChild(new CXfTextStoryWordWrapInfo(rc.width));
			pStory->AppendChild(new CXfHeightInfo(rc.height));
			pStory->AppendChild(new CXfTextFontSize(10000));
			pStory->AppendChild(new CXfUserValue(L"SmartGroup\\Id", FormatStringW(L"%d", uiId + 1).c_str()));
			pStory->AppendChild(new CXfLineWidth(250));
			pStory->AppendChild(new CXfLineColor(CXfLineColor::None));
			pStory->AppendChild(new CXfTextLine());
			pStory->GetLastChild()->AppendChild(new CXfZeroSizedNode(TAG_TEXT_EOL));

			pGroup->AppendChild(pStory);
		}
	}
}

void TableReconstruct::FixJoins(TableSide* ptl)
{

// 	if (ptl->m_type != TableSide::keJoin && ptl->right && ptl->m_pBottom)
// 	{
// 		double dRatio = ptl->rect().
// 	}
}

bool TableReconstruct::Do()
{
	AddRects(m_pPage);

	while (FindSiblings());

	for (size_t idx = 0; idx < m_uiTableCnt; idx++)
	{
		TableSide* pNode = nullptr;
		for (size_t i = 0; i < m_sides.size(); i++)
		{
			if (m_sides[i].m_iTableID == idx)
			{
				pNode = &m_sides[i];
				break;
			}
		}
		if (pNode)
		{
			TableSide* ptl = pNode->FindTopLeft();
			size_t w = TableSide::NumColumns(ptl);
			size_t h = TableSide::NumRows(ptl);

			if (w > 1 || h > 1)
			{
//				DumpTableGrid(idx);
				BuildTable(ptl);
			}
		}
	}

	return true;
}
