#pragma once

#include "common/quadtree.h"
#include "common/XFTree/XFTree.h"

class TableSide : public VElementRect
{
public:
	enum Type {
		keUndefined,
		keHoriz,	// horizontal side of the table grid             |
		keVert,		// vertical side of the table grid		--- joining point ---
		keJoin		// joining point								 |
	};

	enum Direction { up, down, left, right };

	TableSide() {}
	TableSide(Rect& rc, COLORREF cr, LONG lineWidth);

	virtual void GetRect(int& x1, int& y1, int& x2, int& y2) { x1 = m_rc.x; y1 = m_rc.y; x2 = m_rc.right(); y2 = m_rc.bottom(); }
	bool IsAdded() { return m_bAdded; }
	int TableID() { return m_iTableID; }
	void SetAdded(bool b = true) { m_bAdded = b; }

	TableSide* FindTopLeft();
	TableSide* FindBottomRight();
	static size_t NumColumns(TableSide* pTopLeft);
	static size_t NumRows(TableSide* pTopLeft);
	TableSide* NextJoin(Direction dir);
	bool CheckDirection(TableSide* pOther, Direction dir);
	Rect& rect() { return m_rc; }
	Point point() { return m_rc.center(); }
	DocCoord coord() { Point c = m_rc.center();  return DocCoord(c.x, c.y); }
	COLORREF& color() { return m_cr; }

	TableSide* m_pLeft = nullptr;
	TableSide* m_pRight = nullptr;
	TableSide* m_pTop = nullptr;
	TableSide* m_pBottom = nullptr;

	Type m_type = keUndefined;

	Rect m_rc; // this is UP-RIGHT rectangle (in PDF or Xara coordinates, where 0, 0 is bottom left corner)
	COLORREF m_cr;
	LONG m_lw; // Guess what? Word draws table lines using rectangles, that also have their line widths!! Cope with that
	bool m_bAdded = false;
	int m_iTableID = 0;
};

// this class works on the TableSide that forms the table. It runs it and reconstructs all table properties
class TableObject
{
public:
	TableObject(TableSide* pTopLeft);

	TableSide* column(TableSide* prow, size_t idx);

	LONG getRowHeight(size_t idx);
	LONG getColumnWidth(size_t idx);
	COLORREF getOutlineColor(size_t row, size_t col);
	void FixJoins();
	static double Ratio(Rect& rc);

	size_t numRows = 0;
	size_t numCols = 0;
	std::vector<TableSide*> m_rows;
	TableSide* m_ptl;
};

class TableReconstruct
{
public:
	TableReconstruct(LONG lPageWidth, LONG lPageHeight, RefNodeMap& map, VXfTreeNode* pPage);
	void AddRects(VXfTreeNode* pPage);
	bool Do();
	void CheckLeft(std::queue<TableSide*>& checkQueue, TableSide* pNode);
	void CheckRight(std::queue<TableSide*>& checkQueue, TableSide* pNode);
	void CheckTop(std::queue<TableSide*>& checkQueue, TableSide* pNode);
	void CheckBottom(std::queue<TableSide*>& checkQueue, TableSide* pNode);

	bool FindSiblings();
	void FixJoins(TableSide* pTopLeft);
	void BuildTable(TableSide* pTopLeft);
	void DumpTableGrid(int iTableID);
	CXfFlatFill* CreateFill(CXfPath* pPath, Rect rc, INT iTableID);
	std::wstring GetJSONTableString(TableSide* pTopLeft, TableObject& obj);

public:
	std::vector<TableSide> m_sides;
//	std::vector<TableSide*> m_table; // The set of reconstructed tables, each one is the start of the table node graph
	std::map<COLORREF, CXfComplexColor*> m_color; // our table colours cached in a map for reuse
	CQuadtree m_tree; // this is a tree of TableSide* elements, that are actually stored in m_sides, tree is for the fast search of siblings
	VXfTreeNode* m_pPage;
	CXfLayer* m_pLayer = nullptr;
	RefNodeMap& m_refMap;
	UINT m_uiTableCnt = 0;
};
