/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
//
// XarOutputDev.cc
//
#include "stdafx.h"
#include "Type3GlyphVectorizer.h"
#include "PathHelpers.h"
#include "../../potrace/src/potracelib.h"

bool ConvertImageMaskToPathByPotrace(GfxState *state, Stream *str, int width, int height, bool invert, std::vector<DocCoord> &coords, std::vector<BYTE> &verbs)
{
	if (width <= 0 || height <= 0)
		return false;

	bool bRes = false;

	if (potrace_param_t *param = potrace_param_default())
	{
		//param->turdsize = 0;
		//param->opttolerance = 0;

		potrace_bitmap_s bitmap;
		memset(&bitmap, 0, sizeof(bitmap));
		bitmap.w = width;
		bitmap.h = height;
		int n = sizeof(potrace_word) * CHAR_BIT;
		bitmap.dy = (width + n - 1) / n;
		bitmap.map = new potrace_word[height * bitmap.dy];
	
		str->reset();
		int srcRowSize = (width + 7) >> 3;
		unsigned char* srcRow = new unsigned char[srcRowSize];
		unsigned char *srcRowEnd = srcRow + srcRowSize;

		for (int j = 0; j < height; ++j)
		{
			memset(srcRow, invert ? 0xFF : 0, srcRowSize);
			str->doGetChars(srcRowSize, srcRow);

			potrace_word *dstWord = bitmap.map + bitmap.dy * (height - 1 - j);

			size_t c = 0;
			*dstWord = 0;

			for (unsigned char *p = srcRow; p != srcRowEnd; ++p)
			{
				unsigned char b = invert ? *p : 0xFF - *p;

				*dstWord <<= CHAR_BIT;
				*dstWord |= b;
				if (sizeof(potrace_word) == (++c))
				{
					c = 0;
					++dstWord;
				}
			}

			if (c)
			{
				for (int i = 0; i < sizeof(potrace_word) - c; ++i)
				{
					*dstWord <<= CHAR_BIT;
					*dstWord |= 0xFF;
				}
			}
		}

		delete [] srcRow;

		if (potrace_state_t *st = potrace_trace(param, &bitmap))
		{
			if (POTRACE_STATUS_OK == st->status)
			{
				potrace_path_t *curve = st->plist;
				while (curve)
				{
					if (curve->curve.n >= 1)
					{
						verbs.push_back(PT_MOVETO);
						potrace_dpoint_t p = curve->curve.c[curve->curve.n - 1][2];
						double x, y;
						state->transform(p.x / (double)width, p.y / (double)height, &x, &y);
						coords.push_back(DocCoord((LONG)_round(x), (LONG)_round(y)));

						for (int i = 0; i < curve->curve.n; ++i)
						{
							switch (curve->curve.tag[i])
							{
							case POTRACE_CURVETO:
								for (int j = 0; j < 3; ++j)
								{
									verbs.push_back(curve->curve.n - 1 == i ? PT_BEZIERTO | PT_CLOSEFIGURE : PT_BEZIERTO);
									p = curve->curve.c[i][j];
									state->transform(p.x / (double)width, p.y / (double)height, &x, &y);
									coords.push_back(DocCoord((LONG)_round(x), (LONG)_round(y)));
								}
								break;
							default: case POTRACE_CORNER:
								verbs.push_back(PT_LINETO);
								p = curve->curve.c[i][1];
								state->transform(p.x / (double)width, p.y / (double)height, &x, &y);
								coords.push_back(DocCoord((LONG)_round(x), (LONG)_round(y)));

								verbs.push_back(curve->curve.n - 1 == i ? PT_LINETO | PT_CLOSEFIGURE : PT_LINETO);
								p = curve->curve.c[i][2];
								state->transform(p.x / (double)width, p.y / (double)height, &x, &y);
								coords.push_back(DocCoord((LONG)_round(x), (LONG)_round(y)));
								break;
							}
						}
					}
					curve = curve->next;
				}
				bRes = true;
			}
			potrace_state_free(st);
		}

		delete [] bitmap.map;

		potrace_param_free(param);
	}

	return bRes;
}

void ConvertImageMaskToPath(GfxState *state, Stream *str, int width, int height, bool invert, std::vector<DocCoord> &coords, std::vector<BYTE> &verbs)
{
	if (width <= 0 || height <= 0)
		return;

	if (ConvertImageMaskToPathByPotrace(state, str, width, height, invert, coords, verbs))
		return;

	str->reset();

	ClipperLib::Paths rects;

	int stride = (width + 7) >> 3;

	unsigned char* buf = new unsigned char[stride];

	for (int j = 0; j < height; ++j)
	{
		memset(buf, invert ? 0 : 0xFF, stride);
		str->doGetChars(stride, buf);

		unsigned char *p = buf;
		int begin = 0, end = 0;
		for (int i = 0; i < width; ++p)
		{
			unsigned char b = invert ? 0xFF - *p : *p;
			for (int k = 0; k < 8 && i < width; ++k, ++i, b <<= 1)
			{
				if (0 == (b & 0x80))
				{
					if (begin == end)
						begin = i;
					end = i + 1;
				}
				else if (begin != end)
				{
					AddRect(rects, Rect(begin, height - 1 - j, end - begin, 1));
					begin = end;
				}
			}
		}

		if (begin != end)
			AddRect(rects, Rect(begin, height - 1 - j, end - begin, 1));
	}

	delete [] buf;

	ClipperLib::Paths polygons;

	try
	{
		ClipperLib::Clipper c;
		c.AddPaths(rects, ClipperLib::ptSubject, true);
		c.Execute(ClipperLib::ctUnion, polygons, ClipperLib::pftNonZero, ClipperLib::pftNonZero);
	}
	catch( ClipperLib::clipperException&) {}

	size_t pointCount = 0;
	for(ClipperLib::Paths::const_iterator itPolygon = polygons.begin(); itPolygon != polygons.end(); ++itPolygon)
		pointCount += itPolygon->size();
	coords.reserve(coords.size() + pointCount);	
	verbs.reserve(verbs.size() + pointCount);	

	for(ClipperLib::Paths::const_iterator itPolygon = polygons.begin(); itPolygon != polygons.end(); ++itPolygon)
	{
		for (ClipperLib::Path::const_iterator itPoint = itPolygon->begin(); itPoint != itPolygon->end();)
		{
			ClipperLib::Path::const_iterator itNextPoint = itPoint;
			++itNextPoint;

			verbs.push_back(itPolygon->begin() == itPoint ? PT_MOVETO : PT_LINETO | (itPolygon->end() == itNextPoint ? PT_CLOSEFIGURE : 0));		

			double x, y;
			state->transform(itPoint->X / (double)width, itPoint->Y / (double)height, &x, &y);				

			coords.push_back(DocCoord((LONG)_round(x), (LONG)_round(y)));

			itPoint = itNextPoint;
		}
	}
}

void Type3GlyphVectorizer::drawImageMask(GfxState *state, Object *ref, Stream *str,
		     int width, int height, bool invert, bool interpolate,
		     bool inlineImg)
{
	if (!inlineImg || !m_coords.empty())
		m_bStatus = false;

	if (m_bStatus)
		ConvertImageMaskToPath(state, str, width, height, invert, m_coords, m_verbs);
	else
		OutputDev::drawImageMask(state, ref, str, width, height, invert, interpolate, inlineImg);
}

void Type3GlyphVectorizer::drawImage(GfxState *state, Object *ref, Stream *str,
		 int width, int height, GfxImageColorMap *colorMap,
		 bool interpolate, int *maskColors, bool inlineImg)
{
	m_bStatus = false;
	OutputDev::drawImage(state, ref, str, width, height, colorMap, interpolate, maskColors, inlineImg);
}

void Type3GlyphVectorizer::type3D0(GfxState * /*state*/, double wx, double /*wy*/)
{
	m_advanceWidth = wx;
}

void Type3GlyphVectorizer::type3D1(GfxState * /*state*/, double wx, double /*wy*/,
		       double /*llx*/, double /*lly*/, double /*urx*/, double /*ury*/)
{
	m_advanceWidth = wx;
	m_bIgnoreColour = true;
}

void Type3GlyphVectorizer::updateFillColorSpace(GfxState * /*state*/)
{
	if (!m_bIgnoreColour) m_bStatus = false;
}

void Type3GlyphVectorizer::updateFillColor(GfxState * /*state*/)
{
	if (!m_bIgnoreColour) m_bStatus = false;
}

void Type3GlyphVectorizer::updateBlendMode(GfxState * /*state*/)
{
	m_bStatus = false;
}	

void Type3GlyphVectorizer::updateFillOpacity(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::stroke(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::fill(GfxState * state)
{
	if (m_bStatus)
	{
		GfxPath* path = state->getPath();
		if (path && path->getNumSubpaths())
		{
			Rect bounds;
			CreateXaraPath(path, state, m_coords, m_verbs, bounds, true);

			//m_bStatus = false;
		}
	}
}

void Type3GlyphVectorizer::eoFill(GfxState * /*state*/)
{
	m_bStatus = false;
}

bool Type3GlyphVectorizer::tilingPatternFill(GfxState * /*state*/, Gfx * /*gfx*/, Catalog * /*cat*/, Object * /*str*/,
	const double * /*pmat*/, int /*paintType*/, int /*tilingType*/, Dict * /*resDict*/,
	const double * /*mat*/, const double * /*bbox*/,
	int /*x0*/, int /*y0*/, int /*x1*/, int /*y1*/,
	double /*xStep*/, double /*yStep*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::functionShadedFill(GfxState * /*state*/,
		   GfxFunctionShading * /*shading*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::axialShadedFill(GfxState * /*state*/, GfxAxialShading * /*shading*/, double /*tMin*/, double /*tMax*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::axialShadedSupportExtend(GfxState * /*state*/, GfxAxialShading * /*shading*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::radialShadedFill(GfxState * /*state*/, GfxRadialShading * /*shading*/, double /*sMin*/, double /*sMax*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::radialShadedSupportExtend(GfxState * /*state*/, GfxRadialShading * /*shading*/)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::gouraudTriangleShadedFill(GfxState *state, GfxGouraudTriangleShading *shading)
{
	m_bStatus = false;
	return true;
}

bool Type3GlyphVectorizer::patchMeshShadedFill(GfxState *state, GfxPatchMeshShading *shading)
{
	m_bStatus = false;
	return true;
}

void Type3GlyphVectorizer::clip(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::eoClip(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::clipToStrokePath(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::drawString(GfxState * state, const GooString * s)
{
	if (s->getLength() && 3 != state->getRender()) // text rendeing mode 3 == invisible text
		m_bStatus = false;
}

void Type3GlyphVectorizer::drawForm(Ref /*id*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::psXObject(Stream * /*psStream*/, Stream * /*level1Stream*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::beginTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/,
      GfxColorSpace * /*blendingColorSpace*/,
      bool /*isolated*/, bool /*knockout*/,
      bool /*forSoftMask*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::endTransparencyGroup(GfxState * /*state*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::paintTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::setSoftMask(GfxState * /*state*/, const double * /*bbox*/, bool /*alpha*/,
	   Function * /*transferFunc*/, GfxColor * /*backdropColor*/)
{
	m_bStatus = false;
}

void Type3GlyphVectorizer::clearSoftMask(GfxState * /*state*/)
{
	m_bStatus = false;
}

bool Type3GlyphVectorizer::abortCheckCbk(void *data)
{
	return ((Type3GlyphVectorizer *)data)->m_bStatus ? false : true;
}
