/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/

#pragma once

#include "../../poppler/poppler/OutputDev.h"
#include "../../../XarLib_vc7/doccoord.h"

class Type3GlyphVectorizer: public OutputDev 
{
public:

	Type3GlyphVectorizer(std::vector<DocCoord> &coords, std::vector<BYTE> &verbs):
		m_coords(coords), m_verbs(verbs), m_advanceWidth(0), m_bIgnoreColour(false), m_bStatus(true) {}

	virtual ~Type3GlyphVectorizer() {}

	//----- get info about output device

	// Does this device use upside-down coordinates?
	// (Upside-down means (0,0) is the top left corner of the page.)
	virtual bool upsideDown() { return false; }

	// Does this device use drawChar() or drawString()?
	virtual bool useDrawChar() { return false; }

	// Does this device use tilingPatternFill()?  If this returns false,
	// tiling pattern fills will be reduced to a series of other drawing
	// operations.
	virtual bool useTilingPatternFill() { return true; }

	// Does this device support specific shading types?
	// see gouraudTriangleShadedFill() and patchMeshShadedFill()
	virtual bool useShadedFills(int type) { return true; }

	// Does this device use FillColorStop()?
	virtual bool useFillColorStop() { return true; }

	// Does this device use drawForm()?  If this returns false,
	// form-type XObjects will be interpreted (i.e., unrolled).
	virtual bool useDrawForm() { return true; }

	// Does this device use beginType3Char/endType3Char?  Otherwise,
	// text in Type 3 fonts will be drawn with drawChar/drawString.
	virtual bool interpretType3Chars() { return false; }

	//----- update graphics state

	virtual void updateFillColorSpace(GfxState * /*state*/);
	virtual void updateFillColor(GfxState * /*state*/);
	virtual void updateBlendMode(GfxState * /*state*/);
	virtual void updateFillOpacity(GfxState * /*state*/);

	//----- path painting
	virtual void stroke(GfxState * /*state*/);
	virtual void fill(GfxState * /*state*/);
	virtual void eoFill(GfxState * /*state*/);
	virtual bool tilingPatternFill(GfxState * /*state*/, Gfx * /*gfx*/, Catalog * /*cat*/, Object * /*str*/,
				  const double * /*pmat*/, int /*paintType*/, int /*tilingType*/, Dict * /*resDict*/,
				  const double * /*mat*/, const double * /*bbox*/,
				  int /*x0*/, int /*y0*/, int /*x1*/, int /*y1*/,
				  double /*xStep*/, double /*yStep*/);
	virtual bool functionShadedFill(GfxState * /*state*/,
				   GfxFunctionShading * /*shading*/);
	virtual bool axialShadedFill(GfxState * /*state*/, GfxAxialShading * /*shading*/, double /*tMin*/, double /*tMax*/);
	virtual bool axialShadedSupportExtend(GfxState * /*state*/, GfxAxialShading * /*shading*/);
	virtual bool radialShadedFill(GfxState * /*state*/, GfxRadialShading * /*shading*/, double /*sMin*/, double /*sMax*/);
	virtual bool radialShadedSupportExtend(GfxState * /*state*/, GfxRadialShading * /*shading*/);
	virtual bool gouraudTriangleShadedFill(GfxState *state, GfxGouraudTriangleShading *shading);
	virtual bool patchMeshShadedFill(GfxState *state, GfxPatchMeshShading *shading);

	//----- path clipping
	virtual void clip(GfxState * /*state*/);
	virtual void eoClip(GfxState * /*state*/);
	virtual void clipToStrokePath(GfxState * /*state*/);

	//----- text drawing
	virtual void drawString(GfxState * /*state*/, const GooString * /*s*/);

	//----- image drawing
	virtual void drawImageMask(GfxState *state, Object *ref, Stream *str,
			     int width, int height, bool invert, bool interpolate,
			     bool inlineImg);

	virtual void drawImage(GfxState *state, Object *ref, Stream *str,
			 int width, int height, GfxImageColorMap *colorMap,
			 bool interpolate, int *maskColors, bool inlineImg);

	//----- Type 3 font operators
	virtual void type3D0(GfxState * /*state*/, double /*wx*/, double /*wy*/);

	virtual void type3D1(GfxState * /*state*/, double /*wx*/, double /*wy*/,
		       double /*llx*/, double /*lly*/, double /*urx*/, double /*ury*/);


	//----- form XObjects
	virtual void drawForm(Ref /*id*/);

	//----- PostScript XObjects
	virtual void psXObject(Stream * /*psStream*/, Stream * /*level1Stream*/);

	//----- transparency groups and soft masks
	virtual void beginTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/,
				      GfxColorSpace * /*blendingColorSpace*/,
				      bool /*isolated*/, bool /*knockout*/,
				      bool /*forSoftMask*/);
	virtual void endTransparencyGroup(GfxState * /*state*/);
	virtual void paintTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/);
	virtual void setSoftMask(GfxState * /*state*/, const double * /*bbox*/, bool /*alpha*/,
			   Function * /*transferFunc*/, GfxColor * /*backdropColor*/);
	virtual void clearSoftMask(GfxState * /*state*/);

	static bool abortCheckCbk(void *data);

	std::vector<DocCoord> &m_coords;
	std::vector<BYTE> &m_verbs;
	double m_advanceWidth;
	bool m_bIgnoreColour;
	bool m_bStatus;
};
