/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
//
// XarOutputDev.cc
//
#include "stdafx.h"

#ifdef USE_GCC_PRAGMAS
#pragma implementation
#endif

#include <string.h>
#include "XarOutputDev.h"
#include "splash/splash.h"
#include "splash/splashbitmap.h"
#include "splash/SplashState.h"
#include "splash/SplashPath.h"
#include "PNGWriterEx.h"

std::wstring TextStringToUnicode(GooString* textStr);

extern PBYTE RenderToSplash(PDFDoc* pDoc, int iPage, const char* pszOutName, int iResolution, DWORD &dwSize);
//------------------------------------------------------------------------
// XarOutputDev
//------------------------------------------------------------------------

XarOutputDev::XarOutputDev(const wchar_t* _filename, VXfTreeNode* pChapter, PDFDoc* pDoc, FontManager *pFontManager, AnnotationContext *pAnnots)
	: filename(_filename), m_pGfx(NULL),
	m_pTextCommentLayer(NULL), m_XTextNew(0), m_YTextNew(0),
	m_xBaseOld(DBL_MAX), m_yBaseOld(DBL_MAX),
	m_layerCount(0), m_textAreaID(0),
	m_pFontManager(pFontManager),
	m_pAnnots(pAnnots),
	m_pChapter(pChapter), 
	m_pInsertTo(NULL), bitmap(0), splash(0), m_pPage(0),
	m_pDoc(pDoc),  
	m_pTextClipController(NULL),
	m_bType3Char(false), m_type3FillColorSpace(NULL), m_type3StrokeColorSpace(NULL),
	m_anchodedGroupTag(0), m_bFirstPage(true), m_cmykProfile(nullptr)
{ RD;
STOP_STOPWATCH(__FUNCTION__)
}

XarOutputDev::~XarOutputDev() 
{ 
	RD;
	for (SoftMaskStack::const_iterator it = m_SoftMaskStack.begin(); it != m_SoftMaskStack.end(); ++it)
	{
		if (*it) delete *it;
	}
	m_SoftMaskStack.clear();

	if (splash) {
		delete splash;
	}
	if (bitmap) {
		delete bitmap;
	}

	STOP_STOPWATCH(__FUNCTION__)
}

void AddBackgroundBitmap(const wchar_t* filename, PDFDoc *pdfDoc, int pageNum, VXfTreeNode *pPage, DWORD dwWidth, DWORD dwHeight)
{
	std::string str = strW2A(filename);
	str += "_temp.png";

	DWORD dwSize = 0;
	if (PBYTE pbyPNG = RenderToSplash(pdfDoc, pageNum, str.c_str(), 72 * 4, dwSize))
	{
		CXfLayer* pLayer = new CXfLayer();
		pPage->AppendChild(pLayer);
		pLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE, L"Splash Background"));

		GfxPath rect;
		AddRect(&rect, 0, 0, dwWidth, dwHeight);

		CXfPath* pPath = CreateXaraPath(&rect, NULL, TAG_PATH_FILLED_STROKED);

		pLayer->AddChild(pPath);

		CXfDefineBitmap* pDefine = new CXfDefineBitmap(TAG_DEFINEBITMAP_PNG_REAL, pbyPNG, dwSize, L"Splash background");

		pPath->AppendChild(pDefine); // Add define bitmap here so we can easily transfer this path to another parent
		// This is a single bitmap fill - make it a photo
		pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_NONREPEATING));
		pPath->AppendChild(new CXfBitmapFill(DocCoord(0, 0), DocCoord(dwWidth, 0), DocCoord(0, dwHeight), pDefine, 0, 0));
		pPath->AppendChild(new CXfZeroSizedNode(TAG_PHOTOATTR_FLAG));
		pPath->AddChild(new CXfLineColor(CXfLineColor::None));
		pPath->AddChild(new CXfLineWidth(250));
		pPath->AddChild(new CXfJoinStyle(CXF_JOIN_STYLE_MITRE)); // Like if you drag a photo from PG


		delete [] pbyPNG;
	}
}

VXfTreeNode *XarOutputDev::CreatePage(int pageNum, GfxState *state)
{
	using namespace rapidjson;

	// For the first page we create TAG_SPREAD, for each consecutive page it should be PHASE2
	VXfTreeNode *pPage = new CXfZeroSizedNode(m_bFirstPage ? TAG_SPREAD : TAG_SPREAD_PHASE2);
	m_bFirstPage = false;
	m_pChapter->AddChild(pPage);

	DWORD dwWidth  = (DWORD)(state->getPageWidth());
	DWORD dwHeight = (DWORD)(state->getPageHeight());

	// flags=2: Show drop shadow = true, double page spread = false
	pPage->AddChild(new CXfSpreadInformation(dwWidth, dwHeight, 72000 * 12, 0, 2));

	std::wstring name = m_pAnnots->GetPageName(pageNum);

	GenericDocument<UTF16<>> d;
	d.SetObject();

	d.AddMember(L"SpreadName", GenericValue<UTF16<>>(name.c_str(), (SizeType)name.length(), d.GetAllocator()), d.GetAllocator());

	GenericStringBuffer<UTF16<>> buffer;
	Writer<GenericStringBuffer<UTF16<>>, UTF16<>, UTF16<>> writer(buffer);
	d.Accept(writer);

	pPage->AppendChild(new CXfGenericStringNode(TAG_SPREADINFORMATION_EXTRA, (const wchar_t *)buffer.GetString()));

	if (g_settings.bRasterize)
		AddBackgroundBitmap(filename, m_pDoc, pageNum, pPage, dwWidth, dwHeight);

	m_pDefLayer = new CXfLayer();
	pPage->AppendChild(m_pDefLayer);
	m_pDefLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE | TAG_LAYER_FLAG_ACTIVE,
		L"Layer 1"));

// Also create a 'bounding box' special layer to put all bboxes there
#if defined(_BBOX_LAYER)
	m_pBBoxLayer = new CXfLayer();
	pPage->AppendChild(m_pBBoxLayer);
	m_pBBoxLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE, 
		FormatStringW(L"Debug bboxes of page %d", pageNum).c_str()));
#else
	m_pBBoxLayer = NULL;
#endif

#ifdef _TEXT_COMMENT_LAYER
	m_pTextCommentLayer = new CXfLayer();
	pPage->AppendChild(m_pTextCommentLayer);
	m_pTextCommentLayer->AppendChild(new CXfLayerDetails(TAG_LAYER_FLAG_VISIBLE | TAG_LAYER_FLAG_PRINTABLE, 
		FormatStringW(L"Debug comments of page %d", pageNum).c_str()));
#else
	m_pTextCommentLayer = NULL;
#endif
	// Write the default line properties to make outline look happy
	//pPage->AddChild(new CXfLineColor(CXfLineColor::Black));
	//pPage->AddChild(new CXfLineWidth(250));

	//US letter/legal page detection and choice between inches/cm as ruler units
	static const double PAPER_SIZE_DETECTION_DELTA = 0.05;
	enum {GRID_TYPE_RECTANGULAR = 1, GRID_TYPE_ORTHOGONAL = 2};

	double pageWidthInches = m_pDoc->getPageMediaWidth(pageNum) / 72.;
	double pageHeightInches = m_pDoc->getPageMediaHeight(pageNum) / 72.;
	bool useInches = false;
	if (fabs(pageWidthInches-8.5) < PAPER_SIZE_DETECTION_DELTA && (fabs(pageHeightInches-14.) < PAPER_SIZE_DETECTION_DELTA || fabs(pageHeightInches-11.) < PAPER_SIZE_DETECTION_DELTA))
		useInches = true;
	else if (fabs(pageHeightInches-8.5) < PAPER_SIZE_DETECTION_DELTA && (fabs(pageWidthInches-14.) < PAPER_SIZE_DETECTION_DELTA || fabs(pageWidthInches-11.) < PAPER_SIZE_DETECTION_DELTA))
		useInches = true;

	if (useInches)
		pPage->AppendChild(new CXfGridRulerSettings(REF_DEFAULTUNIT_INCHES, 1., 8, GRID_TYPE_RECTANGULAR), false);
	else
		pPage->AppendChild(new CXfGridRulerSettings(REF_DEFAULTUNIT_CENTIMETRES, 1., 10, GRID_TYPE_RECTANGULAR), false);
	pPage->AppendChild(new CXfGridRulerOrigin(DocCoord(0,0)));

	return pPage;
}

/******************************************************************************
   Function:    XarOutputDev::startPage
   Author:      Paul Gruzdev
   Purpose:		append page tags and page properties to XfTree
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::startPage(int pageNum, GfxState *state, XRef *xref)
{
	RD;	

	//m_Layer.clear(); // clear all saved layers from the previous page
	m_layerCount = 0;
	m_OCStack.clear();
	std::wstring name = GetLayerName(NULL);
	m_OCStack.push_back(std::make_pair(name, true));

	DWORD dwWidth  = (DWORD)(state->getPageWidth());
	DWORD dwHeight = (DWORD)(state->getPageHeight());
	m_clip.Reset(0, 0, dwWidth, dwHeight); // add the initial clipping rect: page

	m_xBaseOld = DBL_MAX;
	m_yBaseOld = DBL_MAX;

	m_pPage = CreatePage(pageNum, state);
	m_pInsertTo = m_pCurLayer = m_pDefLayer;

	STOP_STOPWATCH(__FUNCTION__)
}

bool MergePaths(CXfPath *pPath1, CXfPath *pPath2)
{
	if (TAG_PATH_FILLED_STROKED != pPath1->GetTag())
		return false;

	if (TAG_PATH_FILLED_STROKED != pPath2->GetTag())
		return false;

	for (size_t i = 0; i < pPath1->Verbs.size(); ++i)
	{
		if (PT_BEZIERTO == (pPath1->Verbs[i] & ~PT_CLOSEFIGURE))
			return false;
	}

	for (size_t i = 0; i < pPath2->Verbs.size(); ++i)
	{
		if (PT_BEZIERTO == (pPath2->Verbs[i] & ~PT_CLOSEFIGURE))
			return false;
	}
	
	VXfTreeNode* pFillColour1 = nullptr;
	VXfTreeNode* pFillColour2 = nullptr;

	BYTE windingRule1 = CXF_WINDING_RULE_EVENODD;
	BYTE windingRule2 = CXF_WINDING_RULE_EVENODD;

	size_t childrenCount = pPath1->ChildrenCount();
	for (size_t i = 0; i < childrenCount; ++i)
	{
		VXfTreeNode *pChild = pPath1->GetChild(i);

		if (pChild->IsRemoved())
			continue;

		switch (pChild->GetTag())
		{
		case TAG_JOINSTYLE:
		case TAG_STARTCAP:
		case TAG_MITRELIMIT:
		case TAG_LINEWIDTH:
			break;
		case TAG_LINECOLOUR:
			if (CXfLineColor::None != ((CXfLineColor *)pChild)->type)
				return false;
			break;
		case TAG_FLATFILL:
			if (CXfFlatFill::Reference == ((CXfFlatFill *)pChild)->type)
				pFillColour1 = ((CXfFlatFill *)pChild)->pReference;
			else
				return false;
			break;
		case TAG_WINDINGRULE:
			windingRule1 = ((CXfWindingRule *)pChild)->Winding;
			break;
		default:
			return false;
		}
	}

	childrenCount = pPath2->ChildrenCount();
	for (size_t i = 0; i < childrenCount; ++i)
	{
		VXfTreeNode *pChild = pPath2->GetChild(i);

		if (pChild->IsRemoved())
			continue;

		switch (pChild->GetTag())
		{
		case TAG_JOINSTYLE:
		case TAG_STARTCAP:
		case TAG_MITRELIMIT:
		case TAG_LINEWIDTH:
			break;
		case TAG_LINECOLOUR:
			if (CXfLineColor::None != ((CXfLineColor *)pChild)->type)
				return false;
			break;
		case TAG_FLATFILL:
			if (CXfFlatFill::Reference == ((CXfFlatFill *)pChild)->type)
				pFillColour2 = ((CXfFlatFill *)pChild)->pReference;
			else
				return false;
			break;
		case TAG_WINDINGRULE:
			windingRule2 = ((CXfWindingRule *)pChild)->Winding;
			break;
		default:
			return false;
		}
	}

	if (pFillColour1 != pFillColour2)
		return false;	

	try
	{
		ClipperLib::Clipper c;

		ClipperLib::Paths polygon1, polygon2;		
		RECT bounds1, bounds2;
		FlattenConvertPath(pPath1, polygon1, 0, bounds1);
		FlattenConvertPath(pPath2, polygon2, 0, bounds2);

		c.AddPaths(polygon1, ClipperLib::ptSubject, true);
		c.AddPaths(polygon2, ClipperLib::ptClip, true);

		ClipperLib::Paths result;
		if (!c.Execute(ClipperLib::ctUnion, result, XaraFillRuleToPoly(windingRule1), XaraFillRuleToPoly(windingRule2)))
			return false;

		pPath1->clear();
		PolygonsToPath(pPath1, result);

	}
	catch (ClipperLib::clipperException&)
	{
		return false;
	}

	return true;
}

void MergePaths(VXfTreeNode *pNode)
{
	CXfPath *pPrevPath = nullptr;

	size_t childrenCount = pNode->ChildrenCount();
	for (size_t i = 0; i < childrenCount;)
	{
		VXfTreeNode *pChild = pNode->GetChild(i);

		if (pChild->IsRemoved())
		{
			++i;
			continue;
		}

		MergePaths(pChild);

		if (!pChild->IsPath())
		{
			pPrevPath = nullptr;
			++i;
			continue;
		}

		CXfPath *pPath = (CXfPath *)pChild;

		if (pPrevPath && MergePaths(pPrevPath, pPath))
		{
			pNode->RemoveChild(i, true);
		}
		else
		{
			pPrevPath = pPath;
			++i;
		}
	}
}

void XarOutputDev::endPage() 
{
	RD;

	OnEndPage();
	FindRemoveBottomWhiteRects(m_pPage);

	if (m_pCurLayer != m_pDefLayer && m_pCurLayer->ChildrenCount() < 2) m_pCurLayer->SetRemoved(true);

	//MergePaths(m_pDefLayer);

	STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::drawForm(Ref id)
{
}

void XarOutputDev::beginForm(Stream *str, GfxState * state)
{
	/*std::wstring name;

	if (Dict *dict = str->getDict())
	{
		Object obj = dict->lookup("XaraObjectName");
		if (obj.isString())
		{
			if (GooString *s = obj.getString())
				name = TextStringToUnicode(s);
		}
	}

	bool bLiveCopy = (0 == name.find(L"LiveCopy:"));

	if (bLiveCopy)
	{
		VXfTreeNode* pNode = m_pInsertTo;
		while (pNode)
		{
			if (TAG_SOFTMASKGROUP == pNode->GetTag())
			{
				bLiveCopy = false;
				break;
			}
			pNode = pNode->GetParent();
		}
	}

	if (!bLiveCopy)
	{
		m_formGroups.push_back(nullptr);
		return;
	}

	if (TAG_GROUP == m_pInsertTo->GetTag() && !m_pInsertTo->ChildrenCount())
		m_formGroups.push_back(nullptr);
	else
	{
		CXfGroup* pGroup = new CXfGroup();
		m_formGroups.push_back(pGroup);
		m_pInsertTo->AppendChild(pGroup);
		SetParentNode(pGroup);
	}

	double *ctm = state->getCTM();
	m_pInsertTo->AppendChild(new CXfSymbolGroup(Matrix(ctm[0], ctm[1], ctm[2], ctm[3], _round(ctm[4]), _round(ctm[5])), FALSE));
	m_pInsertTo->AppendChild(new CXfWizop(L"ObjectName", L"", name.c_str(), L""));*/
}

void XarOutputDev::endForm()
{
	/*if (!m_formGroups.empty())
	{
		if (m_formGroups.back())
			SetParentNode(m_formGroups.back()->GetParent());
		m_formGroups.pop_back();
	}*/
}

void XarOutputDev::saveState(GfxState *state) 
{ 
	RD_DI(1);
	TextReconstruct();

	m_clip.Save();
	m_clipControllers.push_back(NULL);
	m_SoftMaskStack.push_back(m_SoftMaskStack.size() ? m_SoftMaskStack.back() : NULL);
	STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::restoreState(GfxState *state) 
{ 
	RD_DI(-1);
	TextReconstruct();

	// Ok, we have a rendered to Splash content here - we need to pass it now as a shape with a bitmap fill
	if(splash && bitmap)
	{
		if (!m_clip.Empty())
		{
			if (CXfPath* pPath = m_clip.CreateXaraPath(TAG_PATH_FILLED_STROKED))
			{
				if( AddGraphics(pPath, pPath, m_clip.GetWindingRule(), false))
				{
					PDFRectangle rect;
					PolygonsBoundingRect(&m_clip.GetPolygons(), &rect);

					GrowBitmapObject(bitmap);
					CFileMemory mem;
					mem.Open(vcFile::orOpenForOverwrite);
					PNGWriterEx writer(PNGWriterEx::RGBA);
					writer.writeImgFile(bitmap, (FILE*)&mem, 72, 72);

					static int s_iBitmapNum = 0;
					CXfDefineBitmap* pBitmap = new CXfDefineBitmap(TAG_DEFINEBITMAP_PNG_REAL, mem.GetMemoryBuffer(), 
						mem.GetFileSize(), FormatStringW(L"PDF Rendered Content %d", ++s_iBitmapNum).c_str());
					pBitmap->SetColourSpace(CXfDefineBitmap::CS_RGB);
/*
					if (g_settings.bDumpData)
					{
						static int inum = 0;
						pBitmap->Save(FormatStringW(L"d:\\temp\\pdf_rendered%d.png", s_iBitmapNum-1).c_str());
					}
*/
					mem.Close();

					pPath->AppendChild(pBitmap);
					CXfBitmapFill* pFill = new CXfBitmapFill(DocCoord((LONG)rect.x1, (LONG)rect.y1), DocCoord((LONG)rect.x2, (LONG)rect.y1),
						DocCoord((LONG)rect.x1, (LONG)rect.y2), pBitmap, 0, 0);
					// This is a single bitmap fill - make it a photo
					pPath->AppendChild(new CXfZeroSizedNode(TAG_FILL_NONREPEATING));
					pPath->AppendChild(pFill);
					pPath->AppendChild(new CXfZeroSizedNode(TAG_PHOTOATTR_FLAG));
					AppendAttributes(pPath, state, false, false, true); // We need to clear the stroke attributes
					pPath->AppendChild(new CXfWindingRule(m_clip.GetWindingRule()));
				}
				else delete pPath;
			}
		}

		delete splash;
		delete bitmap;
		bitmap = NULL;
		splash = NULL;
	}

	while (m_clipControllers.size() && m_clipControllers.back())
	{
		SetParentNode(m_clipControllers.back()->GetParent());
		m_clipControllers.pop_back();
	}

	if (m_clipControllers.size()) m_clipControllers.pop_back();

	if (m_clip.GetTopPath())
	{
		MergeSmallBitmaps();
	}

	m_clip.Restore();

	if (m_SoftMaskStack.size())
	{
                CXfSoftMaskGroupTransparentFill *pCurSoftMask = m_SoftMaskStack.back();
                CXfSoftMaskGroupTransparentFill *pNewSoftMask = m_SoftMaskStack.size() > 1 ? *(++m_SoftMaskStack.rbegin()) : NULL;
		if (pCurSoftMask != pNewSoftMask)
		{
			MergeSmallBitmaps();
			TextBreakStory();
		}

		//if (m_SoftMaskStack.back()) delete m_SoftMaskStack.back();
		m_SoftMaskStack.pop_back();
	}
STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::updateAll(GfxState *state) { RD;
	updateLineDash(state);
	updateLineJoin(state);
	updateLineCap(state);
	updateLineWidth(state);
	updateFlatness(state);
	updateMiterLimit(state);
	updateStrokeAdjust(state);
	updateFillColorSpace(state);
	updateFillColor(state);
	updateStrokeColorSpace(state);
	updateStrokeColor(state);
}

void XarOutputDev::updateCTM(GfxState *state, double m11, double m12, double m21, double m22, double m31, double m32)
{
	TextReconstruct();
}

void XarOutputDev::updateLineDash(GfxState *state) 
{ RD;
}

void XarOutputDev::updateFlatness(GfxState *state) 
{ 
	RD_ARG(state->getFlatness());
}

void XarOutputDev::updateLineJoin(GfxState *state) 
{ RD;
}

void XarOutputDev::updateLineCap(GfxState *state) 
{ RD;
}

void XarOutputDev::updateMiterLimit(GfxState *state) 
{ RD;
}

void XarOutputDev::updateLineWidth(GfxState *state) 
{ 
	RD_ARG(state->getLineWidth());
}

void XarOutputDev::updateStrokeAdjust(GfxState * /*state*/) 
{ RD;
}

void XarOutputDev::updateFillColorSpace(GfxState *state) 
{ RD;
}

void XarOutputDev::updateStrokeColorSpace(GfxState *state) 
{ RD;
}

void XarOutputDev::updateFillColor(GfxState *state) 
{
	RD;
	if(splash)
	{
		GfxRGB rgb;
		state->getFillRGB(&rgb);
		SplashColor col;
		col[0] = colToByte(rgb.r);
		col[1] = colToByte(rgb.g);
		col[2] = colToByte(rgb.b);
		col[3] = 255;
		splash->setFillPattern(new SplashSolidColor(col));
	}
}

void XarOutputDev::updateStrokeColor(GfxState *state) 
{ RD;
}

void XarOutputDev::updateBlendMode(GfxState *state) 
{ 
	RD_ARG(state->getBlendMode());
}

void XarOutputDev::updateFillOpacity(GfxState *state) 
{
	RD_ARG(state->getFillOpacity());
}

void XarOutputDev::updateStrokeOpacity(GfxState *state) 
{
	RD_ARG(state->getStrokeOpacity());
}

void XarOutputDev::updateFillOverprint(GfxState *state) 
{ 
	RD_ARG(state->getFillOverprint());
}

void XarOutputDev::updateStrokeOverprint(GfxState *state) 
{ 
	RD_ARG(state->getStrokeOverprint());
}

void XarOutputDev::updateOverprintMode(GfxState *state) 
{ RD;
}

void XarOutputDev::updateTransfer(GfxState *state) 
{ RD;
}

VXfBounded* XarOutputDev::EncloseInSoftMaskGroup(VXfBounded* pBounded)
{
	if(m_SoftMaskStack.size() && m_SoftMaskStack.back())
	{
		CXfSoftMaskGroupTransparentFill* pSoftMaskGroup = m_SoftMaskStack.back();

		if(pSoftMaskGroup->GetRefCount() > 0) // we already inserted this group in a tree so we need to copy the whole group
			pSoftMaskGroup = (CXfSoftMaskGroupTransparentFill*)pSoftMaskGroup->DeepCopy();

		// We need a group to set a soft mask to it!
		CXfGroup* pNodeGroup = new CXfGroup();
		pNodeGroup->AppendChild(pBounded);
		pNodeGroup->Bounds() = pBounded->Bounds();
		if(pSoftMaskGroup)
		{
			pNodeGroup->AppendChild(pSoftMaskGroup);
			pSoftMaskGroup->AddRefCount();
		}
		//m_SoftMaskStack.back() = NULL;
		//m_pInsertTo->AppendChild(pNodeGroup);
		return pNodeGroup;
	}

	//m_pInsertTo->AddChild(pBounded);
	return pBounded;
}

/******************************************************************************
   Function:    XarOutputDev::fill
   Author:      Paul Gruzdev
   Purpose:		Append a filled path tag that corresponds to the current path
				stored in GfxState
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::fill(GfxState* state)
{
	RD;
	bool bRes = DoFill(state, CXF_WINDING_RULE_NONZERO); // Non-zero winding rule fill
}

void XarOutputDev::eoFill(GfxState *state) 
{ 
	RD;
	bool bRes = DoFill(state, CXF_WINDING_RULE_EVENODD); // Evenodd winding rule fill
}

/******************************************************************************
   Function:    XarOutputDev::clip
   Author:      Paul Gruzdev
   Purpose:		store a clip path in it's stack, transforming through the CTM
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::clip(GfxState *state) 
{
	RD;
	DoClip(state, false, CXF_WINDING_RULE_NONZERO); // non-zero winding rule
}

/******************************************************************************
   Function:    XarOutputDev::eoClip
   Author:      Paul Gruzdev
   Purpose:		Even-odd clip operation, store a clip path in it's stack, 
				transforming through the CTM. Note: currently no winding
				rule is passed to Xara
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::eoClip(GfxState *state) 
{
	RD;
	DoClip(state, false, CXF_WINDING_RULE_EVENODD); // evenodd winding rule
}

void XarOutputDev::clipToStrokePath(GfxState *state) 
{ 
	RD;
	DoClip(state, true, CXF_WINDING_RULE_NONZERO); // clip to stroke path - no fill, but take the path's line stroke as a clipping path
}

bool XarOutputDev::checkTransparencyGroup(GfxState *state, bool knockout) 
{ RD;
	if (state->getFillOpacity() != 1 || 
		state->getStrokeOpacity() != 1 ||
		state->getAlphaIsShape() ||
		state->getBlendMode() != gfxBlendNormal ||
		knockout ||
		m_SoftMaskStack.size() && m_SoftMaskStack.back())
		return true;

	return false;
}

/******************************************************************************
   Function:    XarOutputDev::beginTransparencyGroup
   Author:      Paul Gruzdev
   Purpose:		Start a new transparency group
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::beginTransparencyGroup(GfxState *state, const double *bbox,
	GfxColorSpace *blendingColorSpace, bool isolated, bool knockout, bool forSoftMask) 
{
	RD;

	CXfGroup* pGroup = NULL;

	if(forSoftMask)
	{
		double x, y;
		state->transform(bbox[0], bbox[1], &x, &y);
		DocCoord bl((LONG)x, (LONG)y);
		state->transform(bbox[0]+bbox[2], bbox[1], &x, &y);
		DocCoord br((LONG)x, (LONG)y);
		state->transform(bbox[0], bbox[1]+bbox[3], &x, &y);
		DocCoord tl((LONG)x, (LONG)y);
		BYTE byTrans, byMode;
		ConvertOpacity(0, state->getBlendMode(), byTrans, byMode);
		CXfSoftMaskGroupTransparentFill* pFill = new CXfSoftMaskGroupTransparentFill(bl, br, tl, 0, 255, byMode, 0, 0, 0);
		pFill->SetRepeatType(RT_SIMPLE);
		pFill->SetClip(true); // This flag in Xara means that 'area outside mask objects is 100% transparent'
		pFill->SetLinked(true);
		pGroup = pFill;
	}
	else
		// This group could contain just one element in case the transparency is set to 1 blend mode = normal,
		// so we should clean this kind of group later
		pGroup = new CXfGroup();

	m_transparencyGroupStack.push_back(std::make_pair(m_pInsertTo, blendingColorSpace));
	pGroup->SetAsTransparencyGroup(true);
	SetParentNode(pGroup);
}

/******************************************************************************
   Function:    XarOutputDev::endTransparencyGroup
   Author:      Paul Gruzdev
   Purpose:		close the transparency group
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::endTransparencyGroup(GfxState *state) 
{
	RD;
	if (!m_transparencyGroupStack.empty())
	{
		VXfTreeNode* pTransGroup = m_pInsertTo;

		while(pTransGroup->GetParent()) // We need to find our created transp group. It has no parent, it is a group with transparent group flag
			pTransGroup = pTransGroup->GetParent();

		if(!pTransGroup->IsGroup() || !((CXfGroup*)pTransGroup)->IsTransparencyGroup())
			printf("Warning: attaching non-native transparency group\n");

		SetParentNode(m_transparencyGroupStack.back().first);
		m_transparencyGroupStack.back().first = pTransGroup;
	}
}

/******************************************************************************
   Function:    XarOutputDev::paintTransparencyGroup
   Author:      Paul Gruzdev
   Purpose:		Take the current transparency group from it's stack and add a
				transparent fill attribute as the last child of the group
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::paintTransparencyGroup(GfxState *state, const double *bbox)
{
	RD0;
	//fprintf(stdout, " stroke trans: %f, fill trans: %f\n", __FUNCTION__, state->getStrokeOpacity(), state->getFillOpacity());
	bool bSoftMask = m_SoftMaskStack.size() && m_SoftMaskStack.back();
	BYTE byTrans, byMode;
	ConvertOpacity(state->getFillOpacity(), state->getBlendMode(), byTrans, byMode);

	if (!m_transparencyGroupStack.empty())
	{
		bool bTransApplied = false, bTransPromotedToObject = false, bLineTrans = false;
		VXfTreeNode* pTransGroup = m_transparencyGroupStack.back().first;
		m_transparencyGroupStack.pop_back();

		if (!bSoftMask && (state->getFillOpacity() < 1 || state->getBlendMode() != gfxBlendNormal))
		{
			VXfTreeNode* pAddTo = pTransGroup;
			bTransApplied = true;
			// Now, test whether we have just one renderable object inside the group - add flat trans to the child in this case
			// and remove the group
			if(pTransGroup->ChildrenCount() == 1 && pTransGroup->GetChild(0)->IsRenderable())
			{
				// DDES-1076 fix.
				// Now we check whether we already have some transparency applied in that object? If yes, then we can't promote
				VXfRenderable* pRenderable = (VXfRenderable*)pTransGroup->GetChild(0);
				if(pRenderable->Context().m_pTransparentFill == NULL)
				{
					bTransPromotedToObject = true;
					pAddTo = pTransGroup->GetChild(0);
					// DDES-1078. The object in which we promote could be a line!! in which case we need to set-up a line transparency
					if(pAddTo->GetTag() == TAG_PATH_STROKED || pAddTo->GetTag() == TAG_PATH_RELATIVE_STROKED)
						bLineTrans = true;
				}
			}

			pAddTo->AppendChild(bLineTrans ? (VXfTreeNode*)new CXfLineTransparency(byTrans, byMode) : 
				(VXfTreeNode*)new CXfFlatTransparentFill(byTrans, byMode));
		}

		if (bSoftMask)
		{
			bTransApplied = true;
			CXfSoftMaskGroupTransparentFill* pSoftMaskGroup = m_SoftMaskStack.back();

			if(pSoftMaskGroup->GetRefCount() > 0)
				pSoftMaskGroup = (CXfSoftMaskGroupTransparentFill*)pSoftMaskGroup->DeepCopy();

			if(pSoftMaskGroup)
			{
				pSoftMaskGroup->TransparencyType = byMode;
				pTransGroup->AppendChild(pSoftMaskGroup);
				pSoftMaskGroup->AddRefCount();
			}
			//m_SoftMaskStack.back() = NULL;
		}

		MergeSmallBitmaps();

		// Clean-up an empty transparency group
		if (bTransPromotedToObject && pTransGroup->GetTag() == TAG_GROUP && pTransGroup->ChildrenCount() == 1)
		{
			m_pInsertTo->AppendChild(pTransGroup->GetLastChild());
			pTransGroup->RemoveChild((UINT)0, false);
			delete pTransGroup;
		}
		else
			m_pInsertTo->AppendChild(pTransGroup);
	}
STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::setSoftMask(GfxState *state, const double *bbox, bool alpha, Function *transferFunc, GfxColor *backdropColor) 
{
	RD;
	if (!m_transparencyGroupStack.empty())
	{
		CXfSoftMaskGroupTransparentFill* pTransGroup = (CXfSoftMaskGroupTransparentFill*)m_transparencyGroupStack.back().first;
		GfxColorSpace *blendingColorSpace = m_transparencyGroupStack.back().second;
		m_transparencyGroupStack.pop_back();
		// Now set the soft mask's properties like backdrop colour, transfer function, etc
		// if the backdrop colour is greater than 0.5, then set the 'clip' flag to false, otherwise set it to true
		bool bInvert = false;
		
		GfxGray gray = 0;
		blendingColorSpace->getGray(backdropColor, &gray);

		bool bClip = gray/*backdropColor->c[0]*/ > 32767 ? false : true;
		if (transferFunc)
		{
			double t1=0, t2=1;
			transferFunc->transform(&t1, &t1);
			transferFunc->transform(&t2, &t2);
			bInvert = t1 > t2 ? true : false;
			pTransGroup->StartTransparency = (BYTE)(t1*255.9);
			pTransGroup->EndTransparency = (BYTE)(t2*255.9);
			// The invert won't be needed when the start and end trans values would actually work in Xara, but they don't currently
			pTransGroup->SetInvert(bInvert);
		}

		BYTE byXaraTranspType, byTemp;
		ConvertOpacity(0, state->getBlendMode(), byTemp, byXaraTranspType);
//		pTransGroup->TransparencyType = byXaraTranspType;

		if (bInvert) bClip = !bClip;
		pTransGroup->SetClip(bClip);
		pTransGroup->SetUseAlpha(alpha ? true : false);

		if (m_SoftMaskStack.size() && m_SoftMaskStack.back() != pTransGroup)
		{
			MergeSmallBitmaps();
			TextBreakStory();
			m_SoftMaskStack.back() = pTransGroup;
		}
	}
STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::clearSoftMask(GfxState *state) 
{ 
	RD;
	if (!m_SoftMaskStack.empty() && m_SoftMaskStack.back())
	{
		MergeSmallBitmaps();
		TextBreakStory();
		m_SoftMaskStack.back() = NULL;
	}
}

XarOutputDev::Color XarOutputDev::Color::MakeFromState(GfxColorSpace *pSpace, const GfxColor *pColor)
{
	Color color;
	pSpace->getRGB(pColor, &color.rgb);
	color.crRGB = RGB(colToByte(color.rgb.r), colToByte(color.rgb.g), colToByte(color.rgb.b));

	switch (color.colorSpaceMode = GetBaseColorSpace(pSpace)->getMode())
	{
	case csDeviceGray:
		pSpace->getGray(pColor, &color.gray);
		break;
	case csDeviceCMYK:
		pSpace->getCMYK(pColor, &color.cmyk);
		break;
	}

	return color;
}

CXfComplexColor *XarOutputDev::Color::CreateComplexColour() const
{
	switch (colorSpaceMode)
	{
	case csDeviceGray:
		return new CXfComplexColor(crRGB, colToDbl(gray));
	case csDeviceCMYK:
		return new CXfComplexColor(crRGB, CXfComplexColor::cmCMYK, colToDbl(cmyk.c), colToDbl(cmyk.m), colToDbl(cmyk.y), colToDbl(cmyk.k));
	case csDeviceRGB:
	default:
		return new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, colToDbl(rgb.r), colToDbl(rgb.g), colToDbl(rgb.b), 0);
	}

	return NULL;
}

bool XarOutputDev::Color::operator < (const Color &op) const
{
	if (colorSpaceMode != op.colorSpaceMode)
		return colorSpaceMode < op.colorSpaceMode;
	switch (colorSpaceMode)
	{
	case csDeviceGray:
		return gray < op.gray;
	case csDeviceN:
	case csDeviceCMYK:
		return (std::tr1::tie(cmyk.c, cmyk.m, cmyk.y, cmyk.k) < std::tr1::tie(op.cmyk.c, op.cmyk.m, op.cmyk.y, op.cmyk.k));
	case csDeviceRGB:
	default:
		return (std::tr1::tie(rgb.r, rgb.g, rgb.b) < std::tr1::tie(op.rgb.r, op.rgb.g, op.rgb.b));
	}
}

bool XarOutputDev::Color::operator != (const Color &op) const
{
	if (colorSpaceMode != op.colorSpaceMode)
		return true;
	switch (colorSpaceMode)
	{
	case csDeviceGray:
		return gray != op.gray;
	case csDeviceN:
	case csDeviceCMYK:
		return (std::tr1::tie(cmyk.c, cmyk.m, cmyk.y, cmyk.k) != std::tr1::tie(op.cmyk.c, op.cmyk.m, op.cmyk.y, op.cmyk.k));
	case csDeviceRGB:
	default:
		return (std::tr1::tie(rgb.r, rgb.g, rgb.b) != std::tr1::tie(op.rgb.r, op.rgb.g, op.rgb.b));
	}
}

bool XarOutputDev::Opacity::operator < (const Opacity &op) const
{
	if (mode != op.mode)
		return mode < op.mode;
	return value < op.value;
}

bool XarOutputDev::Opacity::operator != (const Opacity &op) const
{
	if (mode != op.mode)
		return true;
	return (value != op.value);
}

XarOutputDev::Opacity::Opacity()
	:value(0), mode(CXF_TRANSPARANCY_TYPE_MIX)
{
}

XarOutputDev::Opacity XarOutputDev::Opacity::MakeFromState(double dOpacity, GfxBlendMode modeGfx)
{
	Opacity op;
	ConvertOpacity(dOpacity, modeGfx, op.value, op.mode);
	return op;
}

CXfComplexColor *XarOutputDev::CreateComplexColour(const Color &color)
{
	auto it = m_colorMap.find(color);
	if (m_colorMap.end() != it)
		return it->second;

	CXfComplexColor *pComplexColor = color.CreateComplexColour();
	m_pChapter->InsertFirstChild(pComplexColor);
	m_colorMap[color] = pComplexColor;
	return pComplexColor;
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// Xara renderers
/******************************************************************************
   Function:    XarOutputDev::CreateComplexColour
   Author:      Paul Gruzdev
   Purpose:		Create XfTree node out of PDF colour and colour space definition
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfComplexColor* XarOutputDev::CreateComplexColour(GfxColorSpace* pSpace, const GfxColor* pColor)
{
	return CreateComplexColour(Color::MakeFromState(pSpace, pColor));
}

/******************************************************************************
   Function:    XarOutputDev::ConvertOpacity
   Author:      Paul Gruzdev
   Purpose:		Convert PDF's opacity value and a blend mode to Xara's transparency
				value and trans type
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::ConvertOpacity(double dOpacity, GfxBlendMode mode, BYTE& byTrans, BYTE& byType)
{
	byTrans = (BYTE)((1.0-dOpacity)*255.99);

	switch(mode)
	{
	case gfxBlendMultiply:
		byType = CXF_TRANSPARANCY_TYPE_STAINEDGLASS;
		break;
	case gfxBlendScreen:
		byType = CXF_TRANSPARANCY_TYPE_BLEACH;
		break;
	case gfxBlendOverlay:
		byType = CXF_TRANSPARENCY_TYPE_OVERLAY;
		break;
	case gfxBlendHardLight:
		byType = CXF_TRANSPARENCY_TYPE_HARDLIGHT;
		break;
	case gfxBlendSoftLight:
		byType = CXF_TRANSPARENCY_TYPE_SOFTLIGHT;
		break;
	case gfxBlendLuminosity:
		byType = CXF_TRANSPARENCY_TYPE_LUMINOSITY2;
		break;
	case gfxBlendColor:
		byType = CXF_TRANSPARENCY_TYPE_COLOUR;
		break;

	case gfxBlendNormal:
	default:
		byType = CXF_TRANSPARANCY_TYPE_MIX;
		break;
	}
}

/******************************************************************************
   Function:    XarOutputDev::AppendAttributes
   Author:      Paul Gruzdev
   Purpose:		Append the colour stroke/fill and line attributes to the
				provided pNode
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void XarOutputDev::AppendAttributes(VXfTreeNode* pNode, GfxState* state, bool bStroke, bool bFill, bool bFillOpacity, bool bStrokeColor)
{
START_STOPWATCH_FN
	// Xara's join style is in range [1..3] while PDF's is [0..2], the same for cap style
	// Update: apparently this is not true
	pNode->AppendChild(new CXfJoinStyle(state->getLineJoin() + 0));
	pNode->AppendChild(new CXfStartCap(TAG_STARTCAP, state->getLineCap() + 0));
	pNode->AppendChild(new CXfMitreLimit((LONG)state->transformWidth(state->getMiterLimit())));

	// DDES-1082 (missing lines). From PDF reference:
	// A line width of 0 denotes the thinnest line that can be rendered at device resolution: 1 device pixel wide. 
	// However, some devices cannot reproduce 1-pixel lines, and on high-resolution devices, they are nearly invisible. 
	// Since the results of rendering such zero-width lines are device-dependent, their use is not recommended. 
	// So for 1082 - there are 0 linewidth lines, do render them also
	if (bStroke /*&& state->getLineWidth() > 0.0*/)
	{
		LONG lLineWidth = (LONG)state->transformWidth(state->getLineWidth());
		pNode->AddChild(new CXfLineWidth(lLineWidth));

		double* pDash = NULL;
		double dStartDash;
		int iDashLen = 0;
		state->getLineDash(&pDash, &iDashLen, &dStartDash);

		if (pDash && iDashLen > 0)
		{
			LONG lStart = (LONG)state->transformWidth(dStartDash);
			CXfDefineDash* pDefine = new CXfDefineDash(TAG_DEFINEDASH, lStart, lLineWidth);
			for(int i = 0; i < iDashLen; i++)
			{
				LONG lDef = (LONG)state->transformWidth(pDash[i]);
				if (i == 0 && lDef == 0) lDef = 1; // Bodge DDES-1522 fix as Camelot misses the first dash if the width of the stroke is zero
				pDefine->AppendDashDef(lDef);
			}
			pNode->AppendChild(pDefine);
		}

		GfxColorSpace* pSpace = GetStrokeColorSpace(state);
		const GfxColor* pColor = GetStrokeColor(state);
		if (CXfComplexColor* pComplex = CreateComplexColour(pSpace, pColor))
		{
			//pNode->AppendChild(pComplex);
			pNode->AppendChild(new CXfLineColor(pComplex));
		}

		double dOpacity = state->getStrokeOpacity();
		GfxBlendMode mode = state->getBlendMode();
		if (dOpacity < 1.0 || gfxBlendNormal != mode)
		{
			BYTE byTrans, byType;
			ConvertOpacity(dOpacity, mode, byTrans, byType);
			pNode->AppendChild(new CXfLineTransparency(byTrans, byType));
		}
	}
	else
	{
		// Write 'none' line width
		pNode->AddChild(new CXfLineColor(CXfLineColor::None));
		pNode->AddChild(new CXfLineWidth(250));
	}

	if (bFill)
	{
		GfxColorSpace* pSpace = bStrokeColor ? GetStrokeColorSpace(state) : GetFillColorSpace(state);
		const GfxColor* pColor = bStrokeColor ? GetStrokeColor(state) : GetFillColor(state);
		if (CXfComplexColor* pComplex = CreateComplexColour(pSpace, pColor))
		{
			//pNode->AppendChild(pComplex);
			pNode->AppendChild(new CXfFlatFill(pComplex));
		}
	}

	if (bFill || bFillOpacity)
	{
		double dOpacity = bStrokeColor ? state->getStrokeOpacity() : state->getFillOpacity();
		GfxBlendMode mode = state->getBlendMode();
		if (dOpacity < 1.0 || gfxBlendNormal != mode)
		{
			BYTE byTrans, byType;
			ConvertOpacity(dOpacity, mode, byTrans, byType);
			pNode->AppendChild(new CXfFlatTransparentFill(byTrans, byType));
		}
	}

STOP_STOPWATCH(__FUNCTION__)
}

void XarOutputDev::SetParentNode(VXfTreeNode *pParent)
{
	if (pParent != m_pInsertTo)
	{
		m_clip.m_pClipController = nullptr;
		MergeSmallBitmaps();
		TextBreakStory();
		m_pInsertTo = pParent;
	}
}

CXfClipviewController *XarOutputDev::CreateClipviewController(CXfPath* pPath, BYTE windingRule)
{
	CXfClipviewController* pController = new CXfClipviewController();

#if defined(_BBOX_DRAW) // Set the controller box visible to outline the objects for the debug build
	pPath->AppendChild(new CXfLineWidth(501));
	pPath->AppendChild(new CXfSimpleColor(255, 0, 0));
	pPath->AppendChild(new CXfLineColor(pPath->GetLastChild()));
	pPath->AppendChild(new CXfLineTransparency(128, CXF_TRANSPARANCY_TYPE_MIX));
#else
	// Clipview path could be non-stroked like others that need to be stroked
	pPath->AppendChild(new CXfLineColor(CXfLineColor::None));
	pPath->AppendChild(new CXfLineWidth(0));
#endif
	pPath->AppendChild(new CXfWindingRule(windingRule));
	//pController->AddClipviewPath(pPath, windingRule);
	pController->AppendChild(pPath, false); // Add our clip path but do not update the bounding box of the controller!
	// We want to store there the actual bounding box of the elements to be clipped
	pController->AppendChild(new CXfZeroSizedNode(TAG_CLIPVIEW));
	return pController;
}

bool ToRect(const ClipperLib::Paths &paths, Rect &rect)
{
	if (1 == paths.size())
	{
		const ClipperLib::Path &path = paths.back();
		if ((4 == path.size() || 5 == path.size() && path[0].X == path[4].X && path[0].Y == path[4].Y) &&
		   ((path[0].X == path[3].X && path[1].X == path[2].X && path[0].Y == path[1].Y && path[2].Y == path[3].Y) ||
		    (path[0].X == path[1].X && path[2].X == path[3].X && path[0].Y == path[3].Y && path[1].Y == path[2].Y)))
		{
			rect.x      = min(path[0].X, path[2].X);
			rect.width  = max(path[0].X, path[2].X) - rect.x;
			rect.y      = min(path[0].Y, path[2].Y);
			rect.height = max(path[0].Y, path[2].Y) - rect.y;

			return true;
		}
	}

	return false;
}

bool IsRect(const CXfPath* pPath)
{
	switch (pPath->Verbs.size())
	{
	case 5:
		if (pPath->Coords[0] != pPath->Coords[4]) return false;
	case 4:
		if (PT_MOVETO != pPath->Verbs[0] || PT_LINETO != pPath->Verbs[1] || PT_LINETO != pPath->Verbs[2]) return false;
		for (size_t i = 3; i < pPath->Verbs.size(); ++i)
		{
			if (PT_LINETO != (pPath->Verbs[i] & ~PT_CLOSEFIGURE)) return false;
		}
		break;
	default:
		return false;
	}

	if (pPath->Coords[0].x == pPath->Coords[3].x && pPath->Coords[1].x == pPath->Coords[2].x &&
	    pPath->Coords[0].y == pPath->Coords[1].y && pPath->Coords[2].y == pPath->Coords[3].y)
		return true;

	if (pPath->Coords[0].x == pPath->Coords[1].x && pPath->Coords[2].x == pPath->Coords[3].x &&
	    pPath->Coords[0].y == pPath->Coords[3].y && pPath->Coords[1].y == pPath->Coords[2].y)
		return true;

	return false;
}

/******************************************************************************
   Function:	XarOutputDev::AddGraphics
   Author:	Denis Onishchenko
   Purpose:     Adds new graphics, applying clipping, soft mask and rearranging as necessary
   Returns:
   Parameters:	pPath - graphics to add
   Parameters:	pOutline - graphics outline.
   Parameters:	windingRule - even-odd or non-zero winding rule for outline
   Notes:       
******************************************************************************/

bool XarOutputDev::AddGraphics(CXfPath* pPath, CXfPath *pOutline, BYTE windingRule, bool bClip, GfxState* state, bool bStroke)
{
	//ClipperLib::Paths subjPolygons;

	if (pOutline)
	{
		pPath->SetBounds(pOutline->Bounds());
		//RECT bounds = {0};
		//FlattenConvertPath(pOutline, subjPolygons, g_dFlatnessThreshold, bounds);
	}
	/*else
	{
		AddRect(subjPolygons, pPath->Bounds());
		windingRule = CXF_WINDING_RULE_NONZERO;
	}*/

	if (bClip)
	{
		//try
		//{
			Rect clipRect;
			if (ToRect(m_clip.GetPolygons(), clipRect))
			{
				const Rect &bounds = pPath->Bounds();

				if (clipRect.x <= bounds.x && clipRect.x + clipRect.width >= bounds.x + bounds.width &&
					clipRect.y <= bounds.y && clipRect.y + clipRect.height >= bounds.y + bounds.height)
				{
					bClip = false;
				}
				else
				{
					if (!clipRect.Intersect(bounds))
						return false;

					if (pPath == pOutline && IsRect(pPath))
					{
						pPath->clear();
						pPath->Verbs.reserve(5);
						pPath->Coords.reserve(5);

						pPath->Add(PT_MOVETO, DocCoord(clipRect.x, clipRect.y));
						pPath->Add(PT_LINETO, DocCoord(clipRect.x + clipRect.width, clipRect.y));
						pPath->Add(PT_LINETO, DocCoord(clipRect.x + clipRect.width, clipRect.y + clipRect.height));
						pPath->Add(PT_LINETO, DocCoord(clipRect.x, clipRect.y + clipRect.height));
						pPath->Add(PT_LINETO | PT_CLOSEFIGURE, DocCoord(clipRect.x, clipRect.y));

						bClip = false;
					}

					/*ClipperLib::Clipper c;
					c.AddPaths(subjPolygons, ClipperLib::ptSubject, true);
					c.AddPaths(m_clip.GetPolygons(), ClipperLib::ptClip, true);

					if (c.Execute(ClipperLib::ctIntersection, subjPolygons, XaraFillRuleToPoly(windingRule), XaraFillRuleToPoly(m_clip.GetWindingRule())))
					{
						if (subjPolygons.empty())
							return false;

						windingRule = CXF_WINDING_RULE_NONZERO;
						if (pPath == pOutline)
						{
							pPath->clear();
							PolygonsToPath(pPath, subjPolygons);
							bClip = false;
						}
					}*/
				}
			}
			/*else
			{
				ClipperLib::Clipper c;
				c.AddPaths(subjPolygons, ClipperLib::ptSubject, true);
				c.AddPaths(m_clip.GetPolygons(), ClipperLib::ptClip, true);

				ClipperLib::Paths diffPolygons;
				if (c.Execute(ClipperLib::ctDifference, diffPolygons, XaraFillRuleToPoly(windingRule), XaraFillRuleToPoly(m_clip.GetWindingRule())) && diffPolygons.empty())
				{
					bClip = false;
				}
				else if (c.Execute(ClipperLib::ctIntersection, subjPolygons, XaraFillRuleToPoly(windingRule), XaraFillRuleToPoly(m_clip.GetWindingRule())))
				{
					if (subjPolygons.empty())
						return false;

					windingRule = CXF_WINDING_RULE_NONZERO;
					if (pPath == pOutline)
					{
						pPath->clear();
						PolygonsToPath(pPath, subjPolygons);
						bClip = false;
					}
				}
			}*/
		//}
		//catch (ClipperLib::clipperException&) {}
	}

	if (pOutline && state)
	{
		GraphicsLine line;
		if (CreateGraphicsLine(pOutline, state, bStroke, line))
		{
			line.nodePath = pPath;
			AddToUnderlineStrikethroughCheck(line);
			for (TextStories::iterator itTextStory = textStories.begin(); line.nodePath && (itTextStory != textStories.end()); ++itTextStory)
			{
				Rect rect;
				if (line.TransformToRect(itTextStory->matrixInv, rect))
				{
					for (TextLines::iterator itLine = itTextStory->lines.begin(); itLine != itTextStory->lines.end();)
					{
						TextLines::iterator itNext = itLine;
						++itNext;				
						if (itTextStory->lines.end() != itNext && itLine->CheckUnderlineStrikethrough(rect, line.color, line.opacity))
						{
							RemoveTextMarkupPath(line);
							//break;
						}
						itLine = itNext;
					}
				}
			}
		}
	}

	MergeSmallBitmaps();

	size_t uInsertIndex = m_pInsertTo->ChildrenCount();

	TextStories::const_reverse_iterator itPrevTextStory = textStories.rend();
	for (TextStories::const_reverse_iterator itTextStory = textStories.rbegin(); itTextStory != textStories.rend(); ++itTextStory)
	{
		bool bIntersection = false;

		VXfTreeNode *pNext = itTextStory->nodeStory->NextSibling();
		while (pNext && !pNext->IsTextStory())
		{
			if (!pNext->IsRemoved() && pNext->IsBounded() && !((VXfBounded *)pNext)->Bounds().empty() &&
				Intersects(((VXfBounded *)pNext)->Bounds(), pPath->Bounds()/*subjPolygons, windingRule*/))
			{
				bIntersection = true;
				break;
			}
			pNext = pNext->NextSibling();
		}

		if (bIntersection) break;

		if (Intersects(itTextStory->nodeStory->Bounds(), pPath->Bounds()/*subjPolygons, windingRule*/))
		{
			for (TextLines::const_iterator itLine = itTextStory->lines.begin(); itLine != itTextStory->lines.end(); ++itLine)
			{
				if (itLine->bbox.empty()) continue;

				if (Intersects(itLine->bbox, pPath->Bounds()/*subjPolygons, windingRule*/))
				{
					for (TextChars::const_iterator itChar = itLine->begin(); itChar != itLine->end(); ++itChar)
					{
						if (itChar->bbox.empty()) continue;
						if (Intersects(itChar->bbox, pPath->Bounds()/*subjPolygons, windingRule*/))
						{
							bIntersection = true;
							break;
						}
					}
					if (bIntersection) break;
				}
			}
			if (bIntersection) break;
		}

		itPrevTextStory = itTextStory;
	}

	if (textStories.rend() != itPrevTextStory)
	{
		for (size_t i = 0; i < m_pInsertTo->ChildrenCount(); ++i)
		{
			if (m_pInsertTo->GetChild(i) == itPrevTextStory->nodeStory)
			{
				uInsertIndex = i;
				break;
			}
		}
	}

	VXfBounded *pNode = EncloseInSoftMaskGroup(pPath);

	if (bClip)
	{
		if (m_clip.m_pClipController && uInsertIndex && m_pInsertTo->GetChild(uInsertIndex - 1) == m_clip.m_pClipController)
			m_clip.m_pClipController->AddChild(pNode);
		else
		{
			CXfClipviewController* pController = m_clip.GetClipviewController(true);
			pController->AddChild(pNode);
			m_pInsertTo->InsertBefore(uInsertIndex, pController);
		}
	}
	else
		m_pInsertTo->InsertBefore(uInsertIndex, pNode);

	AddToAnnotationContext(pPath);
	return true;
}

BOOL CXfTextWriteTextString(VXfTreeNode* pInsert, std::wstring str, std::wstring strTypeface, ULONG ulFontSize, bool bBold, 
							bool bItalic, bool bUnderline, COLORREF crTextColor)
{
	CXfTextString* pStr = new CXfTextString(str);
	pInsert->AppendChild(pStr);

	CXfFontDef* pFontDef = new CXfFontDef(strTypeface, strTypeface);
	pStr->AppendChild(pFontDef);
	pStr->AppendChild(new CXfFontTypeface(pFontDef));
	pStr->AppendChild(new CXfTextFontSize(ulFontSize));

	if(bBold)
		pStr->AppendChild(new CXfZeroSizedNode(TAG_TEXT_BOLD_ON));
	if(bItalic)
		pStr->AppendChild(new CXfZeroSizedNode(TAG_TEXT_ITALIC_ON));
	if(bUnderline)
		pStr->AppendChild(new CXfZeroSizedNode(TAG_TEXT_UNDERLINE_ON));

	pStr->AppendChild(new CXfSimpleColor(crTextColor));
	pStr->AppendChild(new CXfFlatFill(pStr->GetLastChild()));

	pStr->AppendChild(new CXfZeroSizedNode(TAG_LINECOLOUR_NONE));

	return TRUE;
}

void sAddTextCommentToEx(VXfTreeNode* pInsertTo, int iColorIdx, LONG x, LONG y, const wchar_t* psz)
{
	static COLORREF s_BackColor[5] = { RGB(0, 202, 0), RGB(155, 35, 0), RGB(0, 35, 165), RGB(192, 202, 0), RGB(0, 175, 192) };
	std::wstring strFontName = L"Arial";
	int weight = 700;
	bool bItalic = false;
	LONG lFontSize = 8000;

	SIZE sz;
	GetTextExtentPoint32W(CFontDC(strFontName.c_str(), weight, bItalic), psz, (int)wcslen(psz), &sz);

	static const double SQRT2 = sqrt(2.);
	sz.cx = MulDiv(sz.cx, lFontSize, 2048);
	sz.cy = MulDiv(sz.cy, lFontSize, 2048);

	CXfGroup* pGroup = new CXfGroup();
	pInsertTo->AppendChild(pGroup);
	CXfRect* pRect = new CXfRect(x+sz.cx/2, y+sz.cy/2, (LONG)(SQRT2*sz.cx), (LONG)(SQRT2*sz.cy));
	pGroup->AppendChild(pRect);

	pRect->AppendChild(new CXfSimpleColor(s_BackColor[iColorIdx % 5]));
	pRect->AppendChild(new CXfFlatFill(pRect->GetLastChild()));

	CXfTextStorySimple* pStory = new CXfTextStorySimple(x, y+lFontSize/5, 1);
	pGroup->AppendChild(pStory);
	pStory->AppendChild(new CXfTextLine());

	CXfTextWriteTextString(pStory->GetLastChild(), psz, strFontName, lFontSize, weight >= 700 ? true : false, 
		bItalic, false, RGB(255, 255, 255));
	pStory->GetLastChild()->AppendChild(new CXfZeroSizedNode(TAG_TEXT_EOL));
	pGroup->AppendChild(new CXfFlatTransparentFill(100, CXF_TRANSPARANCY_TYPE_MIX));
}

void XarOutputDev::sAddTextCommentEx(int iColorIdx, LONG x, LONG y, const wchar_t* fmt, wchar_t* psz)
{
	if (m_pTextCommentLayer)
		sAddTextCommentToEx(m_pTextCommentLayer, iColorIdx, x, y, psz);
}

void XarOutputDev::vaAddTextCommentEx(int iColorIdx, LONG x, LONG y, const wchar_t* fmt, va_list argList)
{
	if (m_pTextCommentLayer)
	{
		wchar_t* psz = new wchar_t[wcslen(fmt)+256];
		//va_list argList;
		//va_start(argList, fmt);
		int res = vswprintf(psz, fmt, argList);
		//va_end(argList);
		sAddTextCommentEx(iColorIdx, x, y, fmt, psz);
		delete [] psz;
	}
}

void XarOutputDev::AddTextCommentEx(int iColorIdx, LONG x, LONG y, const wchar_t* fmt, ...)
{
	if (m_pTextCommentLayer)
	{
		wchar_t* psz = new wchar_t[wcslen(fmt)+256];
		va_list argList;
		va_start(argList, fmt);
		int res = vswprintf(psz, fmt, argList);
		va_end(argList);
		sAddTextCommentEx(iColorIdx, x, y, fmt, psz);
		delete [] psz;
	}
}

void XarOutputDev::AddTextComment(LONG x, LONG y, const wchar_t* fmt, ...) {}
/*{
	if (m_pTextCommentLayer)
	{
		va_list argList;
		va_start(argList, fmt);
		vaAddTextCommentEx(0, x, y, fmt, argList);
		va_end(argList);
	}
}*/

void XarOutputDev::AddTextCommentLine(LONG x, LONG y, const wchar_t* fmt, ...)
{
	if (m_pTextCommentLayer)
	{
		va_list argList;
		va_start(argList, fmt);
		vaAddTextCommentEx(1, x, y, fmt, argList);
		va_end(argList);
	}
}
