/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "StdAfx.h"
#include "PathHelpers.h"
#include "XfClipperController.h"

#ifdef FILTER_DEBUG
	#include "common/HiResolutionTimer.h"
	double g_dPolyTimeAccumUpdatePoly = 0;
	double g_dPolyTimeAccumUnion = 0;
	double g_dPolyTimeAccumClip = 0;
	double g_dPolyTimeAccumDiff = 0;
#endif

inline size_t VertexCount(ClipperLib::Paths& p)
{
	size_t cnt=0;
	for(size_t i = 0; i < p.size(); i++)
		cnt += p[i].size();
	return cnt;
}

CXfClipperController::CXfClipperController(void) : m_byWindingRule(CXF_WINDING_RULE_NONZERO)
{
}

CXfClipperController::~CXfClipperController(void)
{
}

size_t CXfClipperController::GetPolygonVertexCount()
{
	return VertexCount(m_Paths);
}
size_t CXfClipperController::GetPolygonClipVertexCount()
{
	return VertexCount(m_ClippedPaths);
}

/******************************************************************************
   Function:    CXfClipperController::AddClipviewPath
   Author:      Paul Gruzdev
   Purpose:     Add a clipview path to our controller
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void CXfClipperController::AddClipviewPath(CXfPath* pClip, BYTE byWindingRule)
{
	m_byWindingRule = byWindingRule;
	m_Clip.clear();
	RECT rcBounds;
	FlattenConvertPath(pClip, m_Clip, g_dFlatnessThreshold, rcBounds);
	// Here is a little hack: we 'update' the clip path bbox to be more precise taken from the flattened path
	pClip->SetBounds(Rect(rcBounds.left, rcBounds.top, rcBounds.right-rcBounds.left, rcBounds.bottom-rcBounds.top));
	pClip->SetIsClippingPath(true);

	CXfClipviewController::AddChild(pClip, false);
}

/******************************************************************************
   Function:    CXfClipperController::UpdateObjectsPolygon
   Author:      Paul Gruzdev
   Purpose:		Update the poly that is a union of all child objects
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfClipperController::UpdateObjectsPolygon(ClipperLib::Paths& p)
{
	try
	{
		m_Paths.insert(m_Paths.end(), p.begin(), p.end());
/*
		ClipperLib::Paths solution;
		ClipperLib::Clipper c;
		c.AddPaths(m_Paths, ClipperLib::ptSubject, true);
		c.AddPaths(p, ClipperLib::ptSubject, true);
		c.Execute(ClipperLib::ctUnion, solution, ClipperLib::pftNonZero, type);
		m_Paths.clear();
		m_Paths = solution;*/
	}
	catch(ClipperLib::clipperException&)
	{
		return false;
	}
	return m_Paths.size() > 0;
}

/******************************************************************************
   Function:    CXfClipperController::CreateObjectPolygon
   Author:      Paul Gruzdev
   Purpose:		Create a polygon of the pChild object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfClipperController::CreateObjectPolygon(VXfTreeNode* pChild, ClipperLib::Paths& p)
{
	bool bRet = false;
	if(pChild->IsPath())
	{
		// TODO: deal with the stroked paths specially - we also have to flatten the stroke.
		// Also, if the path is stroked only, we add only a flattened stroke to our objects polygon
		// Update: this is already done in XarOutputDev::stroke
		RECT rcBounds;
		FlattenConvertPath((CXfPath*)pChild, p, g_dFlatnessThreshold, rcBounds);
		// Here is a little hack: we 'update' the clip path bbox to be more precise taken from the flattened path
		((CXfPath*)pChild)->SetBounds(Rect(rcBounds.left, rcBounds.top, rcBounds.right-rcBounds.left, rcBounds.bottom-rcBounds.top));
		bRet = true;
	}
	else if(pChild->IsBounded())
	{
		VXfBounded* pBounded = (VXfBounded*)pChild;
		Rect bounds = pBounded->Bounds();
		if(bounds.empty() == false)
		{
			p.clear();
			p.push_back(ClipperLib::Path());
			p.back().reserve(4);
			p.back().push_back(ClipperLib::IntPoint(bounds.left(), bounds.top()));
			p.back().push_back(ClipperLib::IntPoint(bounds.right(), bounds.top()));
			p.back().push_back(ClipperLib::IntPoint(bounds.right(), bounds.bottom()));
			p.back().push_back(ClipperLib::IntPoint(bounds.left(), bounds.bottom()));
			bRet = true;
		}
	}
	return bRet;
}

/******************************************************************************
   Function:    CXfClipperController::UpdateObjectsPolygon
   Author:      Paul Gruzdev
   Purpose:     Update objects poly via Path object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void CXfClipperController::UpdateObjectsPolygon(VXfTreeNode* pChild)
{
	if(!m_bUpdatePolygons)
		return;

#ifdef FILTER_DEBUG
	CHiResolutionTimer tim;
#endif

	ClipperLib::Paths p;
	CXfClipperController::CreateObjectPolygon(pChild, p);
	UpdateObjectsPolygon(p);

#ifdef FILTER_DEBUG
	g_dPolyTimeAccumUpdatePoly += tim.Current();
#endif

}

/******************************************************************************
   Function:    CXfClipperController::AddChild
   Author:      Paul Gruzdev
   Purpose:     Add a regular child to our controller [to be clipped]
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void CXfClipperController::AddChild(VXfTreeNode* pChild, bool bUpdateBBox)
{
	UpdateObjectsPolygon(pChild);
	CXfClipviewController::AddChild(pChild, bUpdateBBox);
}

/******************************************************************************
   Function:    CXfClipperController::ClipObjects
   Author:      Paul Gruzdev
   Purpose:     Clip the object's polygon with the clipping polygon
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfClipperController::ClipObjects(ClipperLib::Paths& result)
{
	bool bRes = false;
	try
	{
		ClipperLib::PolyFillType type = XaraFillRuleToPoly(m_byWindingRule);

		ClipperLib::Clipper c;
		c.AddPaths(m_Paths, ClipperLib::ptSubject, true);
		c.AddPaths(m_Clip, ClipperLib::ptClip, true);
		bRes = c.Execute(ClipperLib::ctIntersection, result, ClipperLib::pftNonZero, type);
	}
	catch(ClipperLib::clipperException&)
	{
	}
	return bRes;
}

/******************************************************************************
   Function:    CXfClipperController::GatherTextStoryPoly
   Author:      Paul Gruzdev
   Purpose:     Iterate through all elements of a text story and create an
				union polygon of all it's sub-strings
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfClipperController::GatherTextStoryPoly(VXfTreeNode* pStory, ClipperLib::Paths& pPoly)
{
	pPoly.clear();
	VXfTreeNodeVector v;
	pStory->FindTextRenderableNodes(v);
	for(VXfTreeNodeVector::iterator it = v.begin(); it != v.end(); ++it)
	{
		VXfBounded* pBounded = (VXfBounded*)*it;
		Rect bounds = pBounded->Bounds();
		if(bounds.empty() == false)
		{
			ClipperLib::Path p;
			p.push_back(ClipperLib::IntPoint(bounds.left(), bounds.top()));
			p.push_back(ClipperLib::IntPoint(bounds.right(), bounds.top()));
			p.push_back(ClipperLib::IntPoint(bounds.right(), bounds.bottom()));
			p.push_back(ClipperLib::IntPoint(bounds.left(), bounds.bottom()));
			pPoly.push_back(p);
		}
	}

	// Now create a Union of all child elements
	ClipperLib::Clipper clp;
	clp.AddPaths(pPoly, ClipperLib::ptSubject, true);
	pPoly.clear();
	clp.Execute(ClipperLib::ctUnion, pPoly, ClipperLib::pftNonZero, ClipperLib::pftNonZero);
	return true;
}
