/*================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "cxfunits.h"
#include "../../poppler/poppler/PDFDoc.h"
#include "../../poppler/poppler/ErrorCodes.h"
#include "../../poppler/poppler/Gfx.h"
#include "XarOutputDev.h"
#include "PrePassOutputDev.h"
#include "Tables.h"
#include "splash/splash.h"
#include "splash/splashbitmap.h"
#include "SplashOutputDev.h"
#ifdef FILTER_DEBUG
std::map<std::string, double> g_mStopWatch;
#endif

void RenderToSplash(PDFDoc* pDoc, int iPage, const char* pszOutName, int iResolution)
{
	SplashColor bgColor;
	bgColor[0] = bgColor[1] = bgColor[2] = 255;
	SplashOutputDev splash(splashModeXBGR8, 4, false, bgColor, true, splashThinLineDefault, false);

	splash.setVectorAntialias(true);
	splash.setFreeTypeHinting(true, true);

	splash.startDoc(pDoc);      

	pDoc->displayPageSlice(&splash, iPage, iResolution, iResolution, 0, false, true, false, -1, -1, -1, -1);

	SplashBitmap* bitmap = splash.getBitmap();
	bitmap->writeImgFile(splashFormatPng, (char*)pszOutName, 72, 72);
}

PBYTE RenderToSplash(PDFDoc* pDoc, int iPage, const char* pszOutName, int iResolution, DWORD &dwSize)
{
	dwSize = 0;
	RenderToSplash(pDoc, iPage, pszOutName, iResolution);

	if (FILE* in = fopen(pszOutName, "rb"))
	{
		dwSize = (DWORD)_filelength(_fileno(in));
		PBYTE pby = new BYTE[dwSize];
		fread(pby, dwSize, 1, in);

		fclose(in);
		_unlink(pszOutName);

		return pby;
	}

	return NULL;
}

#ifdef FILTER_DEBUG
extern double g_dLutTrans;
extern double g_dCMYK;
extern double g_dmemcpy;
extern double g_dpng_write_row;

extern double g_dPolyTimeAccumUpdatePoly;
extern double g_dPolyTimeAccumUnion;
extern double g_dPolyTimeAccumClip;
extern double g_dPolyTimeAccumDiff;
extern double g_dExecInt01;
extern double g_dCompareGlyphTime;
/*
extern 	double g_dExecIntExec;
extern 	double g_dExecIntBuild;
extern 	double g_dExecIntDispose;

extern 	double g_dExecInt02;
extern 	double g_dExecInt03;
extern 	double g_dExecInt04;
extern 	double g_dExecInt05;
extern 	double g_dExecInt06;
*/
extern double g_dCheckGlyphs;
#endif

const std::wstring CMD_KEY_OUTLINE = L"outline";
ErrorCategory popplerError = errSyntaxWarning;
wchar_t* g_pszInputName = NULL;
int g_iCurrentPage = 0;

//#define BUILD_FONT_HELPER_FUNCTIONS
#ifdef BUILD_FONT_HELPER_FUNCTIONS

/********************************************************************************************

>	void InstallFontsPrivate(const wchar_t* pszDir)

Author:		Paul
Created:	02/10/15
Inputs:		const std::wstring - path to the directory that contains font files

Returns:	void
Purpose:	This function scans the provided directory and installs each font into the system
			privately so that it can be used and enumerated during the current process session only.
			This is a proxy function to test Camelot's font subsystem on a large amount of fonts
			of different types without the need to install them all manually.

********************************************************************************************/

BOOL InstallFontsPrivate(const std::wstring strDir, bool bRecursive)
{
	WIN32_FIND_DATAW dta;
	std::wstring str = strDir + L"\\";
	str += std::wstring(L"*.*");

	HANDLE hFind = FindFirstFileW(str.c_str(), &dta);

	if (hFind == INVALID_HANDLE_VALUE)
		return FALSE;

	do
	{
		if (!wcscmp(dta.cFileName, L".") || !wcscmp(dta.cFileName, L".."))
			continue;

		size_t iLen = wcslen(dta.cFileName);

		if (dta.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (bRecursive)
				InstallFontsPrivate(strDir + L"\\" + std::wstring(dta.cFileName), true);
			continue;
		}
		// Just skip several unwanted types
		else if (iLen > 4 && (!wcscmp(&dta.cFileName[iLen - 3], L"xml") || !wcscmp(&dta.cFileName[iLen - 3], L"dat")))
			continue;

		std::wstring strFont = strDir + std::wstring(L"\\") + std::wstring(dta.cFileName);
		AddFontResourceExW(strFont.c_str(), FR_PRIVATE, 0);
	} 
	while (FindNextFileW(hFind, &dta));

	FindClose(hFind);
	return TRUE;
}

#endif // BUILD_FONT_HELPER_FUNCTIONS

std::string Utf16ToUtf8(const std::wstring &s)
{
	std::string ret;
	int len = WideCharToMultiByte(CP_UTF8, 0, s.c_str(), (int)s.length(), NULL, 0, NULL, NULL);
	if (len > 0)
	{
		ret.resize(len);
		WideCharToMultiByte(CP_UTF8, 0, s.c_str(), (int)s.length(), const_cast<char*>(ret.c_str()), len, NULL, NULL);
	}
	return ret;
}

LONG ConvertToInteger(std::wstring text, int& position)
{
	const wchar_t pos = TEXT('+');
	const wchar_t neg = TEXT('-');

	LONG value   = 0;
	BOOL isneg   = FALSE;
	LONG nchars  = (LONG)text.length();

	//position = SkipSpace(position);
	for(; text[position] == ' ' && position < text.length(); ++position);

	if (position>-1 && position<nchars)
	{
		LONG maxdigs = 10;

		isneg = (text[position]==neg);
		if (isneg || text[position]==pos)
			position = position + 1;//NextIndex(position);

		while (position<nchars && isdigit(text[position]))
		{
			value = value*10 + text[position] - '0';
			position = position + 1;
			maxdigs--;
		}

		if (maxdigs<0)
		{
			return ((isneg) ? 0x80000000 : 0x7FFFFFFF);
		}

		value = (isneg) ? (-value) : (value);
	}

	return value;
}

/****************************************************************************

>	BOOL IsPageInRange(INT32 PageNumber) const

	Author:		Gerry, modified by Paul
	Created:	18/10/2012, modified 12/09/2013

	Inputs:		PageNumber	- 
	Returns:	TRUE if ok, FALSE if bother
	Purpose:	

****************************************************************************/
BOOL IsPageInRange(std::wstring str, int PageNumber)
{
	if (str.empty())
		return(TRUE);

	int pos = 0;
	while (pos < str.length())
	{
		for(; !isdigit(str[pos]) && pos < str.length(); ++pos);
		//pos = m_PageRange.SkipToDigit(pos);

		//m_PageRange.ConvertToInteger(pos);
		INT32 Start = ConvertToInteger(str, pos); //_wtoi(&str[pos]);
		INT32 End = Start;

		for(; str[pos] == ' ' && pos < str.length(); ++pos);
		//pos = m_PageRange.SkipSpace(pos);
		if (pos < str.length() && str[pos] == '-')
		{
			pos++;
			if (pos < str.length())
				End = _wtoi(&str[pos]);//m_PageRange.ConvertToInteger(pos);
		}

		if (Start > End)
		{
			INT32 t = Start;
			Start = End;
			End = t;
		}

		if (PageNumber >= Start && PageNumber <= End)
			return(TRUE);
	}

	return(FALSE);
}

/****************************************************************************

>	BOOL UpdateProgress(int progress)

	Author:		Pavel Ostapenko
	Created:	19/11/2013

	Inputs:		progress = 0..100 
	Returns:	
	Purpose:	update progress.txt file in temporary directory only if new
	            progress value differs from previous one

****************************************************************************/
void UpdateProgress(int curProgress)
{
	if (curProgress < 0)
		curProgress = 0;
	else if (curProgress > 100)
		curProgress = 100;
	//_ASSERT(curProgress >=0 && curProgress <= 100);

	static int oldProgress = -1;
	if (oldProgress == curProgress)
		return;

	static std::wstring progressFilePath;
	if (progressFilePath.empty())
	{
		DWORD dwRetVal;
		DWORD dwBufSize=MAX_PATH;
		wchar_t lpPathBuffer[MAX_PATH];
		dwRetVal = GetTempPathW(MAX_PATH, lpPathBuffer);
		progressFilePath = &lpPathBuffer[0];
		progressFilePath += L"progress.txt";
	}

	DWORD t0 = GetTickCount();

	//std::fstream progressFile;
	//while (progressFile.open(progressFilePath.c_str(), std::ofstream::out | std::ofstream::binary), !progressFile.good())
	//	if (GetTickCount() - t0 > 20000)
	//		return;
	//progressFile << std::setfill('0') << std::setw(3) << curProgress;
	//progressFile.close();
//FILE* out1 = fopen("d:/temp/pdfi_prgs.txt", "at");

	FILE* out = NULL;
	while((out = _wfopen(progressFilePath.c_str(), L"wt")) == NULL)
	{
		if (GetTickCount() - t0 > 20)
		{
//			fprintf(out1, "Failed: %03d\n", curProgress);
//			fclose(out1);
			return;
		}
	}

	fprintf(out, "%03d", curProgress);
	fclose(out);

//fprintf(out1, "written %03d\n", curProgress);
//fclose(out1);

	oldProgress = curProgress;
}

bool CheckFileIsPDF(wchar_t* psz)
{
	long startpos = 0, score = 0;
	long* pScore = &score;

	FILE* f = _wfopen(psz, L"rb");
	if(f)
	{
		while (*pScore == 0 && startpos < 512)
		{
			//op << "Position " << startpos << "\n";
			//Look at the file. If it starts "%PDF-", we score 10, else 0.
			*pScore = 10;

			// Seek to start position
			fseek(f, startpos, SEEK_SET);

			// Check the first 5 bytes of header
			if (fgetc(f) != '%') *pScore = 0;
			if (fgetc(f) != 'P') *pScore = 0;
			if (fgetc(f) != 'D') *pScore = 0;
			if (fgetc(f) != 'F') *pScore = 0;
			if (fgetc(f) != '-') *pScore = 0;

			//Increment starting position
			startpos++;	
		}
		fclose(f);
		return *pScore != 0;
	}
	return false;
}

void PopplerErrorCallback(void *data, ErrorCategory category, Goffset pos, const char *msg)
{
	if (category == errUnimplemented)
		popplerError = category;
}

class ProgressData
{
public:
	ProgressData(int iPageNumA) : iCurrentCnt(0), iLastPageCnt(0), iCurPage(0), iPageNum(iPageNumA), bNewPage(false) {}
	void IncCurPage() { iCurPage++; bNewPage = true; }
	int iCurrentCnt;
	int iLastPageCnt;
	int iCurPage;
	int iPageNum;
	bool bNewPage;
};

/******************************************************************************
   Function:	AbortCheckCb
   Author:		Paul Gruzdev
   Purpose:     This is the abort callback from poppler we use to update
				progress. First we assume that on the first page we have 100 of
				objects (we expect 100 calls to abort). On the next page we update the number
				of objects to be the average of all calls for each previous page.
   Returns:		bool
   Parameters:	void * data  
   Notes:       
******************************************************************************/
bool AbortCheckCb(void* data)
{
	ProgressData* pData = (ProgressData*)data;

	if(pData->bNewPage)
	{
		pData->bNewPage = false;
		if(pData->iLastPageCnt == 0)
			pData->iLastPageCnt = pData->iCurrentCnt;
		else
			pData->iLastPageCnt = (pData->iCurrentCnt + pData->iLastPageCnt)/2;

		pData->iCurrentCnt = 0;
	}

	pData->iCurrentCnt++;
	//printf("prgs: %d\n", g_iCurrentCnt);

	int iSegment = 100/pData->iPageNum;

	if(pData->iLastPageCnt == 0)
	{
		int iProgress = pData->iCurrentCnt*iSegment/100;
		if(iProgress > iSegment)
			iProgress = iSegment;

		UpdateProgress(iProgress);
	}
	else
	{
		int iProgress = iSegment*pData->iCurPage + pData->iCurrentCnt*iSegment/pData->iLastPageCnt;
		UpdateProgress(iProgress);
	}

	return false;
}

std::string GetFilterDataDir()
{
	BYTE szModulePath[8192];
	unsigned char *p;

	if (!GetModuleFileNameA(GetModuleHandle(NULL), (LPSTR)szModulePath, 8192))
		return ".";

	p = _mbsrchr(szModulePath, '\\');
	*p = '\0';

	strcat((char*)szModulePath, "\\data");
	return (char*)szModulePath;
}

XarOutputSettings g_settings;

int _tmain(int argc, wchar_t* argv[])
{
#ifdef FILTER_DEBUG
	DWORD t0 = GetTickCount();
	CHiResolutionTimer tm;
#endif
	if (argc < 3)
	{
		std::cout << "Usage: pdf2xar input.pdf output.xar [options]" << std::endl;
		std::cout << "\tOther options are:" << std::endl;
		std::cout << "\toutline\t\t- draw vector shapes instead of text" << std::endl;
		std::cout << "\t[page=1,3-5]\t- convert/render only pages specified" << std::endl;
		std::cout << "\tnoglyphs\t- don't load embedded glyphs from PDF" << std::endl;
		std::cout << "\tpassword=\t- provide PDF encryption password" << std::endl;
		std::cout << "\tsplash\t\t- instead of XAR, render each page to PNG via Splash renderer" << std::endl;
		std::cout << "\tresolution=N\t- set the Splash rendering resolution. The default is 72" << std::endl;
		std::cout << "\tnoremoveclip\t- don't remove unnecessary clip operations - leave clipviews intact" << std::endl;
		std::cout << "\tnobase14subst\t- don't do Mac->Win base14 font name substitution" << std::endl;
		std::cout << "\tdump\t- dump Xar tree to tree.txt" << std::endl;
		std::cout << "\trasterize\t- render each page as bitmap (using Splash) and place it as the first element of the page" << std::endl;
		std::cout << "\tignoregraphics\t- don't import all graphics, import text only." << std::endl;
		std::cout << "\tlinks\t- display annotations as rectangles on a special layer." << std::endl;
		std::cout << "\tglyph-diffs\t- Dump glyphs different from the system that prevented to use system font into OutName-glyph-diffs.xar." << std::endl;
		std::cout << "\temb-fonts\t- Dump glyphs of all embedded fonts and store into OutName-EmbeddedFonts.xar." << std::endl;
		std::cout << "\tuse-embfonts\t- always use embedded fonts in the document" << std::endl;
		std::cout << "\tnolayers\t- import all page content into single layer" << std::endl;
		return -1;
	}

	std::wstring strPages;
	std::string password;
	bool bRenderToSplash = false;
#ifdef _DEBUG
	bool bDumpTree = true;
#else
	bool bDumpTree = false;
#endif
	int iResolution = 72;

	for(int i = 3; i < argc; i++)
	{
		std::wstring arg = argv[i];
		if (arg.empty())
			continue;
		if (!_wcsnicmp(argv[i], CMD_KEY_OUTLINE.c_str(), CMD_KEY_OUTLINE.length()))
			g_settings.bTextAsOutline = true;
		else if (!_wcsnicmp(arg.c_str(), L"makekernpairs", 14))
			g_settings.bReconstructKerningPairs = true;
		else if (!_wcsnicmp(arg.c_str(), L"use-embfonts", 8))
			g_settings.bAlwaysUseEmbeddedFonts = true;
		else if (!_wcsnicmp(arg.c_str(), L"removesmallkerns", 16))
			g_settings.bRemoveSmallKerns = true;
		else if (!_wcsnicmp(arg.c_str(), L"page=", 5))
			strPages = &(arg[5]);
		else if (!_wcsnicmp(arg.c_str(), L"rasterize", 9))
			g_settings.bRasterize = true;
		else if(!_wcsnicmp(arg.c_str(), L"password=", 9))
			password = Utf16ToUtf8(std::wstring(&(arg[9])));
		else if(!_wcsnicmp(arg.c_str(), L"noglyphs", 8))
			g_settings.bLoadEmbeddedGlyphs = false;
		else if(!_wcsnicmp(arg.c_str(), L"splash", 6))
			bRenderToSplash = true;
		else if(!_wcsnicmp(arg.c_str(), L"noremoveclip", 12))
			g_settings.bNoRemoveClipview = true;
		else if(!_wcsnicmp(arg.c_str(), L"nobase14subst", 13))
			g_settings.bNoBase14Substitution = true;
		else if(!_wcsnicmp(arg.c_str(), L"dump", 4))
		{
			bDumpTree = true;
			g_settings.bDumpData = true;
		}
		else if(!_wcsnicmp(arg.c_str(), L"ignoregraphics", 14))
			g_settings.bIgnoreGraphics = true;
		else if(!_wcsnicmp(arg.c_str(), L"links", 5))
			g_settings.bLinks = true;
		else if(!_wcsnicmp(arg.c_str(), L"rawfonts", 8))
			g_settings.bRawFontNames = true;
		else if(!_wcsnicmp(arg.c_str(), L"resolution=", 11))
		{
			iResolution = _wtoi(&(arg.c_str()[11]));
		}
		else if (!_wcsnicmp(arg.c_str(), L"maxstory=", 9))
		{
			int iVal = _wtoi(&(arg.c_str()[9]));
			if (iVal > 0) g_settings.iMaxStoryCount = iVal;
		}
		else if (!_wcsnicmp(arg.c_str(), L"glyph-diffs", 11))
			g_settings.bDumpGlyphDiffs = true;
		else if (!_wcsnicmp(arg.c_str(), L"emb-fonts", 9))
			g_settings.bDumpEmbeddedFonts = true;
		else if (!_wcsnicmp(arg.c_str(), L"nolayers", 8))
			g_settings.bNoLayers = true;
	}

	// Paul: now test whether we actually have a PDF, as Poppler lib seems to do pretty bad (hanging for a long time)
	// if it tries to open a regular text file
	if(!CheckFileIsPDF(argv[1]))
		return -1;

#ifdef BUILD_FONT_HELPER_FUNCTIONS
	InstallFontsPrivate(L"d:\\ExtraFonts", true);
#endif

	setErrorCallback(PopplerErrorCallback, nullptr);
	std::string strFilterData = GetFilterDataDir(); // We need to retain it up until the end because GlobalParams does not copy
	// and stores only a pointer to the string.
	globalParams = new GlobalParams(strFilterData.c_str());


	/*FILE *file = _wfopen(argv[1], L"rb");
	if (!file)
		return -1;

	_fseeki64(file, 0, SEEK_END);
	size_t fileLen = _ftelli64(file);
	_fseeki64(file, 0, SEEK_SET);

	char *memBuf = (char *)gmalloc(fileLen);
	fread(memBuf, 1, fileLen, file);

	fclose(file);

	AutoFreeMemStream pdfStream(memBuf, 0, fileLen, Object(objNull));
	PDFDoc doc(&pdfStream, password.empty() ? NULL : &GooString(password.c_str()));*/


	PDFDoc doc(static_cast<wchar_t *>(argv[1]), (int)wcslen(argv[1]), password.empty() ? NULL : &GooString(password.c_str()));

/*
	{
		PrePassOutputDev prepassRender(&doc);
		CHiResolutionTimer tm;

		for(int i = 0; i < doc.getNumPages(); i++)
		{
			if(strPages.empty() || IsPageInRange(strPages, i+1))
			{
					// Why 72K? The DPI sets dots per inch, the default value is 72. Given that we transform into millipoints, our end
					// units are millipoints, so there are 72K millipoints in one inch, that's why the resolution is set like this
					doc.displayPageSlice(&prepassRender, i+1, 72000, 72000, 0, false, true, false, -1, -1, -1, -1);
			}
		}
		wprintf(L"%s\t%.3f\tmsec", argv[1], tm.Current()*1000.0);
// 		std::map<int, CSubsetInfo>& info = prepassRender.m_SubsetInfo;
// 		for(std::map<int, CSubsetInfo>::iterator it = info.begin(); it != info.end(); ++it)
// 		{
// 			printf("ref: %d count: %d\n", it->first, it->second.subset.size());
// 		}
	}
*/
	g_pszInputName = argv[1];
/*#ifdef FILTER_DEBUG	
	std::cout
		<< "accessibility" << (doc.okToAccessibility() ? '+' : '-') << ' '
		<< "add_notes" << (doc.okToAddNotes() ? '+' : '-') << ' '
		<< "assemble" << (doc.okToAssemble() ? '+' : '-') << ' '
		<< "change" << (doc.okToChange() ? '+' : '-') << ' '
		<< "copy" << (doc.okToCopy() ? '+' : '-') << ' '
		<< "fill_form" << (doc.okToFillForm() ? '+' : '-') << ' '
		<< "print" << (doc.okToPrint() ? '+' : '-') << ' '
		<< "print_hi_res" << (doc.okToPrintHighRes() ? '+' : '-') << ' '
		<< std::endl
	;
#endif*/

	if (!doc.isOk())
	{
		if (doc.getErrorCode() == errEncrypted)
		{
			if (popplerError == errUnimplemented)
			{
				std::cout << "Unimplemented encryption!" << std::endl;
				return -5;
			}
			std::cout << "Password required!" << std::endl;
			return -3;
		}
		else if (doc.getErrorCode() == errPermission)
		{
			std::cout << "No permission!" << std::endl;
			return -4;
		}
		else if (popplerError == errUnimplemented)
		{
			std::cout << "Unimplemented feature!" << std::endl;
			return -6;
		}
		std::cout << "Error opening pdf!" << std::endl;
		return -2;
	}
	// DDES-12142:
	// Since Xara is an editing application, we need to justify that user supplied enough information (such as owner password),
	// to be able to change the document
	else if(!doc.okToChange(false))
	{
		std::cout << "Password required!" << std::endl;
		return -4;
	}

	int pagesCount = doc.getNumPages();
	// Paul: I need this for batch conversion using PowerShell
	wprintf(L"-------------- Generating: %s\n", argv[1]);
	/*std::cout << std::endl << "pages: " << pagesCount;
	for (int i=0; i<pagesCount; ++i)
		std::cout << std::endl << "\t" << doc.getPageMediaWidth(i+1) << " x " << doc.getPageMediaHeight(i+1) << std::endl << std::endl;
	*/

	CXarExport* pExp = XarLibrary::CreateExporter();

#ifdef _DEBUG												
//	RenderToSplash(&doc, 1, "d:\\temp\\pdf_output.png", 72);
#endif
	CXarExport* pExporter = pExp;						

	IStream* strOut = NULL;
	BOOL ok = FALSE;
	if(argc > 2)
	{
		HRESULT hRes = SHCreateStreamOnFileEx(argv[2], STGM_CREATE | STGM_WRITE, FILE_ATTRIBUTE_NORMAL, TRUE, NULL, &strOut);
		ok = hRes == S_OK && strOut != NULL;
		ok = ok && pExporter->StartExport(strOut);
	}

	if (!ok)
	{
		printf("Failed to StartExport");
		return -1;
	}

#ifndef _DEBUG
	try
	{
#endif

	char pszVersion[32]; pszVersion[0] = 0;
	wchar_t szModulePath[8192];
	GetModuleFileNameW(GetModuleHandle(NULL), szModulePath, 8192);

	DWORD handle = 0;
	UINT size = GetFileVersionInfoSizeW(szModulePath, &handle);
	BYTE* versionInfo = new BYTE[size];
	if (GetFileVersionInfoW(szModulePath, handle, size, versionInfo))
	{
		// we have version information
		UINT len = 0;
		VS_FIXEDFILEINFO*   vsfi = NULL;
		VerQueryValueW(versionInfo, L"\\", (void**)&vsfi, &len);
		sprintf(pszVersion, "%d,%d,%d,%d", HIWORD(vsfi->dwProductVersionMS), LOWORD(vsfi->dwProductVersionMS), 
		HIWORD(vsfi->dwProductVersionLS), LOWORD(vsfi->dwProductVersionLS));
	}
	delete[] versionInfo;

#ifdef _DEBUG
	bDumpTree = true; // always dump Xar tree in debug mode
#endif

	int         reportStages = 100;
	int         reportCount  = 0;
	int         reportI      = 0;
	std::string lastReportString;

	UpdateProgress(0);

	for(int i = 0; i < pagesCount; i++)
	{
		if(strPages.empty() || IsPageInRange(strPages, i+1))
			++reportCount;
	}

	CXfXaraTree tree;
	tree.GetRoot()->AppendChild(new CXfFileHeader("PDFImportCPP", pszVersion, pszVersion));
	VXfTreeNode* pDoc = new CXfZeroSizedNode(TAG_DOCUMENT);
	tree.GetRoot()->AppendChild(pDoc);

	CXfViewport* pViewport = new CXfViewport(DocCoord(0, 0), DocCoord((LONG)(doc.getPageMediaWidth(1)*1000), (LONG)(doc.getPageMediaHeight(1)*1000)));
	pDoc->AppendChild(pViewport);
	// Create and add a new document info tag that is now *required* for all multi-page imported documents
	CXfDocumentInformation* pDocInfo = new CXfDocumentInformation(1); // we have only one chapter
	pDocInfo->AddChapterInfo(reportCount);
	pDoc->AppendChild(pDocInfo);

	pDoc->AppendChild(new CXfDocumentFlags(DOCFLAG_EMPTYLINESSIGNIFICANT | DOCFLAG_WORD_LINE_SPACING/* | DOCFLAG_PROMOTEURLS*/));
	//DOCFLAG_LEGACYAPTS | DOCFLAG_ALLPAGESTHESAME | DOCFLAG_SAMELAYERSONALLPAGES | DOCFLAG_EMPTYLINESSIGNIFICANT));
	//DOCFLAG_EMPTYLINESSIGNIFICANT | DOCFLAG_SIMULATEWEBLENGTHS | DOCFLAG_PROMOTEURLS));
		/*| DOCFLAG_MULTILAYER | DOCFLAG_ALLLAYERSVISIBLE*/

#ifdef _DEBUG
	pDoc->AppendChild(new CXfDefineDefaultUnits(REF_DEFAULTUNIT_MILLIPOINTS, REF_DEFAULTUNIT_COMP_POINTS)); // better to work in MP when testing documents
#endif
	VXfTreeNode* pChapter = new CXfZeroSizedNode(TAG_CHAPTER);
	pDoc->AppendChild(pChapter);

	//pExporter->WriteZeroSizedRecord(TAG_DOWN);

#ifdef FILTER_DEBUG
	CHiResolutionTimer tmInit;
#endif 

	FontManager fontManager(argv[2], pChapter, &doc, strFilterData.c_str(), globalParams);

#ifdef _ENABLE_PREPASS
	if (!bRenderToSplash && !g_settings.bTextAsOutline)
	{
		PrePassOutputDev prepassRender(&fontManager);
		for (int i = 1; i <= pagesCount; i++)
		{
			if (strPages.empty() || IsPageInRange(strPages, i))
			{
				if (Page* pdfPage = doc.getPage(i))
				{
					Gfx *gfx = pdfPage->createGfx(&prepassRender, 72000, 72000, 0, false, false, -1, -1, -1, -1, false, nullptr, nullptr);

					pdfPage->display(gfx);

					// draw annotations

					Annots *annotList = pdfPage->getAnnots();
					int annotCount = annotList->getNumAnnots();
					for (int j = 0; j < annotCount; ++j)
						annotList->getAnnot(j)->draw(gfx, false);

					delete gfx;
				}
			}
		}

		fontManager.CreateFontDefs(g_settings.bLoadEmbeddedGlyphs, false);
	}
#endif

	AnnotationContext annots(&doc);
	XarOutputDev xarRender(argv[2], pChapter, &doc, &fontManager, &annots);
#ifdef FILTER_DEBUG
	double dInit = tmInit.Current();
#endif

	ProgressData progressData(reportCount);

	for (int i = 1; i <= pagesCount; i++)
	{
		if (strPages.empty() || IsPageInRange(strPages, i))
		{
			if (bRenderToSplash)
			{
				std::string str = strW2A(argv[2]);
				str += FormatStringA("_%.3d.png", i);
				RenderToSplash(&doc, i, str.c_str(), iResolution);
			}
			else if (Page* pdfPage = doc.getPage(g_iCurrentPage = i))
			{
				// Why 72K? The DPI sets dots per inch, the default value is 72. Given that we transform into millipoints, our end
				// units are millipoints, so there are 72K millipoints in one inch, that's why the resolution is set like this
				Gfx *gfx = pdfPage->createGfx(&xarRender, 72000, 72000, 0, false, true, -1, -1, -1, -1, false, AbortCheckCb, (void*)&progressData);
				xarRender.SetGfx(gfx);
				annots.Create(pdfPage, (DWORD)(gfx->getState()->getPageWidth()), (DWORD)(gfx->getState()->getPageHeight()));

				pdfPage->display(gfx);

				// draw annotations

				Annots *annotList = pdfPage->getAnnots();
				if (annotList->getNumAnnots() > 0)
				{
					xarRender.BeginAnnotations(); // Paul Gruzdev: notify output dev that we are starting to draw annotations

					for (int j = 0; j < annotList->getNumAnnots(); ++j)
						annotList->getAnnot(j)->draw(gfx, false);
				}

				delete gfx;

				// temporary disable for stable fix of DDES-1003 prior the release
				annots.ProcessAnnotationsQuadTree(i, &doc, pDoc);
				progressData.IncCurPage();
#ifdef _ENABLE_TABLES_RECONSTRUCTION
				TableReconstruct table((LONG)gfx->getState()->getPageWidth(), (LONG)gfx->getState()->getPageHeight(), tree.GetRefs(), xarRender.GetCurrentPage());
				table.Do();
#endif
				if (g_settings.bLinks) annots.ProcessAnnotations(&doc, &xarRender);
#ifdef _BBOX_DRAW
				// Already drawn above
				if (!g_settings.bLinks) annots.ProcessAnnotations(&doc, &xarRender);
#endif
			}

// 			VXfTreeNode* pPage = tree.GetRoot()->FindFirstNode(CTagSet(TAG_SPREAD, TAG_SPREAD_PHASE2));
// 			if(pPage)
// 			{
// //				tree.Write(pPage, pExporter);
// 				if (out)
// 					(*out) << "-----------------------  " << i << "  -----------------------" << std::endl << pPage->Dump(1);
// //				pPage->GetParent()->RemoveChild(pPage);
// 			}

			UpdateProgress(++reportI * reportStages / reportCount);
		}
	}

 	annots.ProcessTOC(&doc, pChapter);


	// DDES-1470: we can't convert the picked most frequent URL colour to HSV when creating it - we need to match every other URL colour found
	// with the most frequent one in the same colour space etc. So only convert it after the whole document is processed
	annots.ConvertURLColourHSV();

#ifdef FILTER_DEBUG
	CHiResolutionTimer tmEmbed;
#endif

	// Now write all our used glyphs
 	if (g_settings.bLoadEmbeddedGlyphs && !bRenderToSplash)
	{
		CXfEmbeddedGlyphSizeHint* pHint = EnumerateDocumentGlyphs(pChapter);
		if (pHint) pChapter->InsertFirstChild(pHint);
	}

	if (g_settings.bDumpEmbeddedFonts)
		fontManager.GetGlyphDiffDocument().ScanAddFonts(pDoc);
#ifdef FILTER_DEBUG
	double dEmbed = tmEmbed.Current();
#endif

	tree.GetRoot()->AppendChild(new CXfZeroSizedNode(TAG_ENDOFFILE));

#ifdef FILTER_DEBUG
CHiResolutionTimer tmWrite;
#endif

	if(!bRenderToSplash)
		tree.Write(tree.GetRoot(), pExporter);

#ifdef FILTER_DEBUG
	double dWrite = tmWrite.Current();
#endif

	std::ofstream *out = NULL;
	TCHAR pszDumpTreeName[8192]; pszDumpTreeName[0] = 0;

	if (bDumpTree)
	{
		GetModuleFileName(NULL, pszDumpTreeName, 8192);
		TCHAR* ptr = pszDumpTreeName + wcslen(pszDumpTreeName) - 1;
		while (*ptr != '\\')
			ptr--;
		*(ptr + 1) = 0;
		_tcscat(pszDumpTreeName, _T("tree.txt"));
		out = new std::ofstream(pszDumpTreeName);
	}

	if(out)
	{
		wprintf(L"Dumping tree to %s\n", pszDumpTreeName);
		(*out) << tree.GetRoot()->Dump(0);
	}

#ifdef FILTER_DEBUG
	/*
	std::cout << "Polygon operation update poly time: " << g_dPolyTimeAccumUpdatePoly << std::endl;
	std::cout << "Polygon operation union time: " << g_dPolyTimeAccumUnion << std::endl;
	std::cout << "Polygon operation clip time: " << g_dPolyTimeAccumClip << std::endl;
	std::cout << "Polygon operation diff time: " << g_dPolyTimeAccumDiff << std::endl;
	std::cout << "--------------------------------------" << std::endl;*/
// 	std::cout << "Polygon operation AddPath time: " << g_dAddPathPre << std::endl;
// 	std::cout << "Polygon operation AddPath find local minima time: " << g_dAddPathLocal << std::endl;
// 	std::cout << "Polygon operation AddPath find local minima iteration cound: " << g_dFindLocalCountThousand << std::endl;
/*
	std::cout << "g_dExecIntExec time: " << g_dExecIntExec << std::endl;
	std::cout << "g_dExecIntExec build time: " << g_dExecIntBuild << std::endl;
	std::cout << "g_dExecInt dispose time: " << g_dExecIntDispose << std::endl;
	printf("01:%f 02:%f 03:%f 04:%f 05:%f 06:%f\n", g_dExecInt01, g_dExecInt02, g_dExecInt03, g_dExecInt04, g_dExecInt05, g_dExecInt06);
*/
//printf("Font update:%f\n", g_dExecInt01);
/*
wprintf(L"%s***GDI check glyphs: %f\n", argv[2], g_dCheckGlyphs);
FILE* fout = fopen("GDIFontLog.txt", "at");
fwprintf(fout, L"%s***GDI check glyphs: %f\n", argv[2], g_dCheckGlyphs);
fclose(fout);*/

#endif

	if (out)
	{
		out->close();
		delete out;
	}
/*
	pExporter->WriteZeroSizedRecord(TAG_UP); // chapter
	pExporter->WriteZeroSizedRecord(TAG_UP); // document
	pExporter->WriteZeroSizedRecord(TAG_ENDOFFILE);*/

	delete globalParams;
	delete pExp;
	strOut->Release();

#ifdef FILTER_DEBUG
	double dTotal = tm.Current();

#define PCT(x) ", " << (x*100.0/dTotal) << "%"

	std::ofstream("pdf2xar.timing.txt") << (GetTickCount()-t0);

	std::cout << "Init font map, enum fonts: " << dInit << PCT(dInit) << std::endl;
	std::cout << "Embed document glyphs: " << dEmbed<< PCT(dEmbed) << std::endl;
	std::cout << std::endl;
	std::cout << "EncodePNG LUT transform: " << g_dLutTrans << PCT(g_dLutTrans) << std::endl;
	std::cout << "EncodePNG CMYK: " << g_dCMYK << PCT(g_dCMYK) << std::endl;
	std::cout << "EncodePNG png_write_row: " << g_dpng_write_row << PCT(g_dpng_write_row) << std::endl;
	std::cout << std::endl;
	std::cout << "Write XAR Tree: " << dWrite << PCT(dWrite) << std::endl;
	std::cout << "Total: " << dTotal << std::endl;
	std::cout << "------------------------------------------------------------------" << std::endl;
	double dStopWatchTotal = 0;
	for(std::map<std::string, double>::iterator it = g_mStopWatch.begin(); it != g_mStopWatch.end(); ++it)
	{
		printf("%s: %.3f secs\n", it->first.c_str(), it->second);
		dStopWatchTotal += it->second;
	}
	std::cout << "------------------------------------------------------------------" << std::endl;
	printf("Stop watch counted total: %.3f secs\n", dStopWatchTotal);

	printf("Compare glyph time: %f\n", g_dCompareGlyphTime);
#endif

#ifndef _DEBUG
	}
	catch(...)
	{
		return -1;
	}
#endif

	return 0;
}
