/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// stdafx.h : include file for standard system include files,
// or project specific include files that are used frequently, but
// are changed infrequently
//

#pragma once

#define _CRT_NON_CONFORMING_SWPRINTFS
#define _CRT_SECURE_NO_WARNINGS

#include "targetver.h"
#include <atlbase.h>
#include <stdio.h>
#include <tchar.h>
#include <wchar.h>
#include <time.h>
#include <windows.h>
#include <assert.h>
#include <math.h>
#include <float.h>
#include <io.h>
// TODO: reference additional headers your program requires here
#include <string>
#include <vector>
#include <array>
#include <map>
#include <unordered_map>
#include <set>
#include <algorithm>
#include <cctype>
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>

#include <config.h>

// Features
#define _ENABLE_WHITESPACE_RECONSTRUCTION
#define _ENABLE_SMALL_KERNING_REPLACEMENT
#define _ENABLE_BULLETED_AND_NUMBERED_LISTS
//#define _ENABLE_TABLES_RECONSTRUCTION
//#define _ENABLE_TEXT_AREA_MODE
//#define _ENABLE_TAB_STOP_DETECTION
//#define _ENABLE_TEXT_FLOW_RECONSTRUCTION

#ifndef _ENABLE_TABLES_RECONSTRUCTION // embedded graphics "eats" parts of the table graphics into the story, so currently no way to enable both (later step table reconstruction)
// or we need better way to scan all graphics inside the anchor groups also
#define _ENABLE_EMBEDDED_GRAPHICS
#endif

#define _ENABLE_PREPASS

#define _ENABLE_RELATIVE_LINE_SPACING

// debugging ------------------------------
//#define FILTER_DEBUG       // timing and additional stats
//#define _BBOX_DRAW         // draw text story bounding boxes
//#define _FLOW_DRAW         // draw lines between sequential text elements
//#define _ELEMENT_DRAW      // draw individual text elements bounding boxes
//#define _LOGICAL_BOUNDS    // draw logical bounds instead of physical ones
//#define _KERNING_DRAW      // draw kernings bounding boxes
//#define _EOL_DRAW          // draw eol detection results

//#define _TEXT_COMMENT_LAYER
//#define EXTRA_LOGGING

//#define _DRAW_PARAGRAPH_ALIGNMENT_DETECTION

//#define _EXPLICIT_RD_LOGGING // logging in Release mode
// ----------------------------------------

#if (defined(_BBOX_DRAW) || defined(_ELEMENT_DRAW) || defined(_KERNING_DRAW) || defined(_EOL_DRAW) || defined(_FLOW_DRAW))
  #define _BBOX_LAYER
#endif

#ifdef FILTER_DEBUG
  #ifdef _DEBUG
    #define START_STOPWATCH_FN
    #define STOP_STOPWATCH(fnName)
  #else
    #define START_STOPWATCH_FN CHiResolutionTimer __stopWatchFunc__;
    extern std::map<std::string, double> g_mStopWatch;
    #define STOP_STOPWATCH(fnName) { g_mStopWatch.find(fnName) == g_mStopWatch.end() ? g_mStopWatch[fnName] = __stopWatchFunc__.Current() : g_mStopWatch[fnName] += __stopWatchFunc__.Current(); }
  #endif
#else
  #define START_STOPWATCH_FN
  #define STOP_STOPWATCH(fnName)
#endif

#if (defined(_DEBUG) || defined(_EXPLICIT_RD_LOGGING))
	#define RD          renderDump(__FUNCTION__)
	#define RD0         renderDump(__FUNCTION__, 0, false)
	#define RD_DI(di)   renderDump(__FUNCTION__, di)
	#define RD_ARG(num) renderDumpArg<>(__FUNCTION__, 0, true, num)
	#define RD_SS       ,std::stringstream &ss
	#define RD_SS0      std::stringstream &ss,
	#define RD_SS1      std::stringstream &ss
	#define RD_SS_ARG   ,ss
	#define RD_SS_ARG0  ss,
	#define RD_SS_ARG1  ss
#else
	#ifdef FILTER_DEBUG
		#define RD START_STOPWATCH_FN
		#define RD0 START_STOPWATCH_FN
		#define RD_DI(di) START_STOPWATCH_FN
		#define RD_ARG(num)
		#define RD_SS
		#define RD_SS0
		#define RD_SS1
		#define RD_SS_ARG
		#define RD_SS_ARG0
		#define RD_SS_ARG1
	#else
		#define RD
		#define RD0
		#define RD_DI(di)
		#define RD_ARG(num)
//		#define RD_ARG(T,num)
		#define RD_SS
		#define RD_SS0
		#define RD_SS1
		#define RD_SS_ARG
		#define RD_SS_ARG0
		#define RD_SS_ARG1
	#endif
#endif

#include "common/decodepng.h"
#include "common/vcFile.h"
#include "common/StringUtils.h"
#include "common/HiResolutionTimer.h"

#if (_MSC_VER >= 1800)
inline LONG _round(double r) { return std::round(r); }
#else
inline LONG _round(double r) { return static_cast<LONG>( (r > 0.0) ? floor(r + 0.5) : ceil(r - 0.5) ); }
#endif
