#include "stdafx.h"

/******************************************************************************
   Function:	RGB2HSV
   Author:		Paul Gruzdev
   Purpose:     Convert from RGB to Hue Saturaton Value colour space.
   Returns:		void
   Parameters:	double R  
   Parameters:	double G  
   Parameters:	double B  
   Parameters:	double & H  
   Parameters:	double & S  
   Parameters:	double & V  
   Notes:       This function is taken exactly as is from Camelot, 
				ColourContextHSVT::ConvertFromCIET, colcntx.cpp code
				because we need to match exactly our rgb in the filter
				with the one shown in Camelot
******************************************************************************/
void RGB2HSV(double R, double G, double B, double& H, double& S, double& V)
{
	// Now, convert the RGB value into HSV
	double Min, Max, Delta;
	Min = min(R, G);
	Min = min(Min, B);
	Max = max(R, G);
	Max = max(Max, B);

	V = Max;
	Delta = Max - Min;

	if (fabs(Delta) > 0.000001)
	{
		if (Max > 0.0)
			S = Delta / Max;
		else
			S = 0.0;

		if (R == Max)
			H = (G - B) / Delta;
		else if (G == Max)
			H = 2.0 + (B - R) / Delta;
		else
			H = 4 + (R - G) / Delta;

		H = H / 6.0;
		if (H < 0.0) H += 1.0;
	}
	else
	{
		H = S = 0.0;
	}
}

void Lab2RGB(double dL, double da, double db, double& r, double& g, double& b) {
    double var_Y = (dL + 16) / 116.0;
    double var_X = (da / 500.0) + var_Y;
    double var_Z = var_Y - (db / 200);

    if (pow(var_Y, 3) > 0.008856) {
        var_Y = pow(var_Y, 3);
    } else {
        var_Y = (var_Y - 16.0 / 116) / 7.787;
    }

    if (pow(var_X, 3) > 0.008856) {
        var_X = pow(var_X, 3);
    } else {
        var_X = (var_X - 16.0 / 116) / 7.787;
    }

    if (pow(var_Z, 3) > 0.008856) {
        var_Z = pow(var_Z, 3);
    } else {
        var_Z = (var_Z - 16.0 / 116) / 7.787;
    }

    double X = var_X * 95.047;
    double Y = var_Y * 100.000;
    double Z = var_Z * 108.883;

    var_X = X / 100.0;
    var_Y = Y / 100.0;
    var_Z = Z / 100.0;

    double var_R = var_X * 3.2406 + var_Y * -1.5372 + var_Z * -0.4986;
    double var_G = var_X * -0.9689 + var_Y * 1.8758 + var_Z * 0.0415;
    double var_B = var_X * 0.0557 + var_Y * -0.2040 + var_Z * 1.0570;

    if (var_R > 0.0031308) {
        var_R = 1.055 * pow(var_R, (1 / 2.4)) - 0.055;
    } else {
        var_R = 12.92 * var_R;
    }

    if (var_G > 0.0031308) {
        var_G = 1.055 * pow(var_G, (1 / 2.4)) - 0.055;
    } else {
        var_G = 12.92 * var_G;
    }

    if (var_B > 0.0031308) {
        var_B = 1.055 * pow(var_B, (1 / 2.4)) - 0.055;
    } else {
        var_B = 12.92 * var_B;
    }

    // clamping
    r = max(0.0, min(1.0, var_R));
    g = max(0.0, min(1.0, var_G));
    b = max(0.0, min(1.0, var_B));
}
