/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#ifndef HIRESOLUTIONTIMER_H
#define HIRESOLUTIONTIMER_H			// Guard against multiple occurrences of this header file

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


/*************************************************************************
Class:				CHiResolutionTimer
Base Class:			none
Author:				Paul Gruzdev
Description:		string and character set independent base class for 
			        CElapsedTimer (which was the original class that was
					split in two), CAtlElapsedTimer and CWAtlElapsedtimer.
Pure Virtual:		No
Known Issues:		None
Usage Notes:		This class is not intended to be used directly.  In MFC
					apps use CElapsedTimer, in ATL apps use either CAtlElapsedTimer
					or CWAtlElapsedTimer.
Overridable Methods:	
Override Notes:		
**************************************************************************/
class CHiResolutionTimer  
{
public:
	CHiResolutionTimer();
	virtual ~CHiResolutionTimer();
	void Start();
	double Current();
	double Tolerance()	{ return m_dTolerance; }

private:
	double	m_dTolerance;
	BOOL	m_bHighPerformance;
	double	m_dStart;
	double	m_dTicksPerSec;

};


/******************************************************************************
   Function:    CHiResolutionTimer::CHiResolutionTimer
   Author:      Paul Gruzdev
   Purpose:     Constructor
   Returns:     Void
   Parameters:  
   Notes:       
******************************************************************************/
inline CHiResolutionTimer::CHiResolutionTimer()
{
	//	Start the timer right away
	Start();
}

/******************************************************************************
   Function:    CHiResolutionTimer::~CHiResolutionTimer
   Author:      Paul Gruzdev
   Purpose:     Destructor
   Returns:     Void
   Notes:       
******************************************************************************/
inline CHiResolutionTimer::~CHiResolutionTimer()
{
}

/******************************************************************************
   Function:    CHiResolutionTimer::Start
   Author:      Paul Gruzdev
   Purpose:     Start or reset the timer
   Returns:     Void
   Notes:       A high-performance timer will be used if one is available.
				This function cannot handle a high performance timer that
				returns negative numbers.
******************************************************************************/
inline void CHiResolutionTimer::Start()
{
	LARGE_INTEGER	liTicks;

	if (QueryPerformanceCounter(&liTicks))
	{
		m_dStart = (double)liTicks.HighPart * (65536.0*65536.0) + (double)liTicks.LowPart;
		m_bHighPerformance = TRUE;
		QueryPerformanceFrequency(&liTicks);
		m_dTicksPerSec = (double)liTicks.HighPart * (65536.0*65536.0) + (double)liTicks.LowPart;
		m_dTolerance = 1.0 / m_dTicksPerSec;
	}
	else
	{
		m_dStart = (double)GetTickCount();
		m_bHighPerformance = FALSE;
		m_dTicksPerSec = 1000.0;
		m_dTolerance = 0.025;
	}
}

/******************************************************************************
   Function:    CHiResolutionTimer::Current
   Author:      Paul Gruzdev
   Purpose:     Return current elapsed time
   Returns:     double - the elapsed time since construction or Start(), in
					seconds.
   Notes:       
******************************************************************************/
inline double CHiResolutionTimer::Current()
{
	double			dNow;
	LARGE_INTEGER	liTicks;

	if (m_bHighPerformance)
	{
		QueryPerformanceCounter(&liTicks);
		dNow = (double)liTicks.HighPart * (65536.0*65536.0) + (double)liTicks.LowPart;
	}
	else
	{
		dNow = (double)GetTickCount();
	}
	return (dNow - m_dStart) / m_dTicksPerSec;
}

#endif