/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"

std::wstring LoadStringFromResourceW(HINSTANCE hInst, UINT uid)
{
	//  Get the id of the string table block containing the target string
	const DWORD blockID = (uid >> 4) + 1;
	//  Get the offset of the target string in the block
	const DWORD itemID = uid % 0x10;
	HRSRC hResInfo = FindResource(hInst, MAKEINTRESOURCE(blockID), RT_STRING);
	HGLOBAL hData = LoadResource(hInst, hResInfo);
	const WCHAR* tableDataBlock = (LPCWSTR)LockResource(hData);
	DWORD dwSize = SizeofResource(hInst, hResInfo);

	const WCHAR* stringPtr = NULL;
	WCHAR stringLen;
	DWORD searchOffset = 0;
	DWORD stringIndex = 0;

	//  Search through the section for the appropriate entry.
	//  The first two bytes of each entry is the length of the string
	//  followed by the Unicode string itself. All strings entries 
	//  are stored one after another with no padding.
	while(searchOffset < dwSize)
	{
		if (stringIndex == itemID)
		{
			//  If the string has size. use it!
			if (tableDataBlock[searchOffset] != 0x0000)
			{
				stringPtr = &tableDataBlock[searchOffset + 1];
				stringLen = tableDataBlock[searchOffset];
			}
			//  Nothing there -
			else
			{
				stringPtr = NULL;
				stringLen = 0;
			}

			//  Done
			break;
		}

		//  Go to the next string in the table
		searchOffset += tableDataBlock[searchOffset] + 1;
		//  Bump the index
		stringIndex++;
	}

	FreeResource(hData);
	return std::wstring(stringPtr, stringLen);
}

std::string LoadStringFromResourceA(HINSTANCE hInst, UINT uid)
{
	USES_CONVERSION;
	std::wstring str = LoadStringFromResourceW(hInst, uid);
	return W2A(str.c_str());
}

std::wstring ColorToHexString(COLORREF cr)
{
	wchar_t psz[12];
	swprintf_s(psz, 12, L"%2.2X%2.2X%2.2X", GetRValue(cr), GetGValue(cr), GetBValue(cr));
	return psz;
}

std::string FormatStringA(const char* psz, ...)
{
	std::string strRes;

	va_list args;
	va_start(args, psz);

	int iResLen = _vscprintf(psz, args) + 1;
	strRes.resize(iResLen);

	vsprintf_s(&strRes[0], iResLen, psz, args);
	va_end(args);

	// Apparently you can't just resize the string to say 5000 and then apply a '+' or a '+=' operator if your string is 
	// smaller, they just won't work!! Seems strange but true. 
	strRes.resize(strlen(strRes.c_str()));
	return strRes;
}

std::wstring FormatStringW(const wchar_t* psz, ...)
{
	std::wstring strRes;

	va_list args;
	va_start(args, psz);
	int iResLen = _vscwprintf(psz, args) + 1;
	strRes.resize(iResLen);

	vswprintf_s(&strRes[0], iResLen, psz, args);
	va_end(args);
	// Apparently you can't just resize the string to say 5000 and then apply a '+' or a '+=' operator if your string is 
	// smaller, they just won't work!! Seems strange but true. 
	strRes.resize(wcslen(strRes.c_str()));
	return strRes;
}

std::string strW2A(std::wstring str)
{
	USES_CONVERSION;
	return W2A(str.c_str());
}

std::wstring strA2W(std::string str)
{
	USES_CONVERSION;
	return A2W(str.c_str());
}

void StripChars(std::wstring& str, std::wstring chars) {
	for (size_t i = 0; i < str.length(); i++)
		if (chars.find(str[i]) != std::wstring::npos) {
			str.erase(i, 1);
			i--;
		}
}
