/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/

#include "stdafx.h"
#include "xarlib.h"
#include "oiltypes.h"
#include "ensure.h"
#include "errors.h"
#include "TextManager.h"

/********************************************************************************************
>	CharDescription::CharDescription(WCHAR CharCode, WORD TypefaceHandle, BOOL Bold, BOOL Italic)

	Author:		Simon
	Created:	20/3/95
	Inputs:		CharCode	  : Unicode character, LO-BYTE identical to ASCII
				TypefaceHandle: This must be in the range 0..16384(2 power 14)
				Bold		  : Bold or not
				Italic		  : Italic or not		 
	Purpose:	CharDescription constructor
********************************************************************************************/

CharDescription::CharDescription(WCHAR CharCode, std::wstring strTypefaceA, int weight, BOOL Italic, BOOL isGlyphID)
{
	 charHandle = CharCode   <<16;		// CharCode occupies HI-WORD
	 //charHandle+=(Bold   ? 1<<15 : 0);	// Top bit of LOW-WORD
	 charHandle+=(Italic ? 1<<14 : 0);	// Next bit 
	 charHandle+=(isGlyphID ? 1<<15 : 0);	// Next bit 
	 charHandle+= weight & 0x00003FFF;	// Lowest 14 bits of LOW-WORD
	 strTypeface = strTypefaceA;
}



/********************************************************************************************
>	CharDescription::CharDescription(const CharDescription& that)

	Author:		Peter
	Created:	24/4/95
	Inputs:		Reference to a CharDescriptor
	Purpose:	CharDescription constructor	from another CharDescriptor
********************************************************************************************/

CharDescription::CharDescription(const CharDescription& that)
{
	charHandle = that.charHandle;
	strTypeface = that.strTypeface;
}

/**********************************************************************************************
>	static BOOL TextManager::GetLogFontFromCharDescriptor(CDC* pDC, CharDescription& ChDesc,
						LPLOGFONT pLogFont, LONG LogicalHeight=-1)

	Author:		Ed Cornes / Paul Gruzdev
	Created:	12/1/96 / 25/04/2015
	Inputs:		pDC           - ptr to DC
				ChDesc        - CharDescription containing font handle, bold, italic
				LogicalHeight - desired height in DC pixels (or -1 for dpi - ie 1" high)
	Outputs:	pLogFont      - Logfont corresponding to given ChDesc at specified height
	Returns:	FALSE if fails
	Purpose:	Given a char descriptor, get its corresponding Logfont, and Font metrics if required
	Note:		Assumes MM_TextMode	
*********************************************************************************************/

BOOL TextManager::GetLogFontFromCharDescriptor(HDC hDC, CharDescription& ChDesc, LOGFONTW* pLogFont, LONG LogicalHeight)
{
	ERROR2IF(pLogFont==NULL,FALSE,"TextManager::GetLogFontFromCharDescriptor() pLogFont==NULL");
	ERROR2IF(     hDC==NULL,FALSE,"TextManager::GetLogFontFromCharDescriptor() pDC==NULL");

	memset(pLogFont, 0, sizeof(LOGFONTW));

	if(ChDesc.GetTypeface().length() > 31)
	{
		wcsncpy_s(pLogFont->lfFaceName, LF_FACESIZE, ChDesc.GetTypeface().c_str(), 31);
		pLogFont->lfFaceName[31] = 0;
	}
	else 
		wcscpy_s(pLogFont->lfFaceName, LF_FACESIZE, ChDesc.GetTypeface().c_str());

	pLogFont->lfWeight = ChDesc.GetWeight();
	pLogFont->lfItalic = ChDesc.GetItalic() ? 255 : 0;
	pLogFont->lfCharSet = DEFAULT_CHARSET;
	pLogFont->lfOutPrecision = OUT_STROKE_PRECIS;
	pLogFont->lfClipPrecision = CLIP_STROKE_PRECIS;
	pLogFont->lfQuality = DRAFT_QUALITY;
	pLogFont->lfPitchAndFamily = 34;
  	//*pLogFont = pEnumLogFont->elfLogFont;

	// if not specified, set LogicalHeight to dpi (ie 1")
	if (LogicalHeight==-1)
		LogicalHeight = GetDeviceCaps(hDC, LOGPIXELSY);

	// set font height, aspect (100%), bold, italic
 	pLogFont->lfHeight = -LogicalHeight;
 	pLogFont->lfWidth  = 0;
	// MW 13-07-07 We pass the Weight and Italic fields as stored in the font cache, i.e., we
	// only ever pass the values we got during font enumeration rather than trying to ask for
	// bold/italic variants of the family typeface, which results in synthesized fonts.
	return TRUE;
}

/***************************************************************************************
>	static BOOL TextManager::GetTTCharPath(CharDescription& ChDesc, DocCoord** ppCoords,
										 PathVerb** ppVerbs, ULONG* pNumCoords, CDC* pDC=NULL)

	Author:		MarkG / Paul Gruzdev
	Created:	21/3/95
	Inputs:		ChDesc= description of char
				pDC= (possible) pointer to a DC (for optimisation) defaults to NULL
	Outputs:	ppCoords=>pointer to path coords buffer
				ppVerbs=>pointer to path verbs buffer
				pNumCoords=number of elements in path
	Returns:	FALSE if fails
	Purpose:	Get the path associated with a given char: returns a glyph outline path of
				a font that is present in the system.
	Global:		Assumes MM_TextMode	
********************************************************************************************/

BOOL TextManager::GetTTCharPath(CharDescription& ChDesc, DocCoord** ppCoords, PathVerb** ppVerbs, ULONG* pNumCoords, HDC hSrcDC)
{
	ERROR2IF(ppCoords==NULL,FALSE,"TextManager::GetTTCharPath pCoords==NULL");
	ERROR2IF(ppVerbs==NULL,FALSE,"TextManager::GetTTCharPath ppVerbs==NULL");
	ERROR2IF(pNumCoords==NULL,FALSE,"TextManager::GetTTCharPath pNumCoords==NULL");

	const size_t CharPathBufferSize=2048;
	static std::vector<DocCoord>	CharPathCoordArray( CharPathBufferSize );
	static std::vector<PathVerb>	CharPathVerbArray( CharPathBufferSize );

	// Never try to get the outlines of a control character
	if (ChDesc.GetCharCode() < 32)
	{
		*ppCoords   = &CharPathCoordArray[0];
		*ppVerbs    = &CharPathVerbArray[0];
		*pNumCoords = 0;
		return TRUE;
	}

	// ensure we have a DC
	BOOL LocalDC=(hSrcDC==NULL);
	HDC hActualDC = NULL;
	if (LocalDC)
	{
		HDC hScreenDC = ::GetDC(NULL);
		if (hScreenDC==NULL)
		{
			ERROR3("TextManager::GetCharPath() - failed to create DC");
			return FALSE;
		}
		if (!(hActualDC = CreateCompatibleDC(hScreenDC)))
		{
			ERROR3("TextManager::GetCharPath() - CreateCompatibleDC() failed");
			ReleaseDC(NULL, hScreenDC);
			return FALSE;
		}
		ReleaseDC(NULL, hScreenDC);
		//hActualDC = hScreenDC;
	}
	else
		hActualDC = hSrcDC;
	
	// get a log font from the char description, then the path data from the logfont
	LOGFONTW	CharLogFont;
	BOOL ok=GetLogFontFromCharDescriptor(hActualDC, ChDesc, &CharLogFont);
	DWORD Coords=static_cast<DWORD>(CharPathCoordArray.size());
	if (ok)
	{
		LONG TextHeight = -4096;	// This ought to be -2048 but this doesn't return correct outlines on Times Italic on o's etc.
		
		// Workaround for "Rokkitt" Google font, which does not like this huge size. See BZ10864.
		if (wcscmp(CharLogFont.lfFaceName, L"Rokkitt") == 0)
			TextHeight = -1024;

		while( true )
		{
			ok = GetBezierFromChar(hActualDC, ChDesc.GetCharCode(), &CharLogFont, &Coords, (POINT*)&CharPathCoordArray[0], 
				&CharPathVerbArray[0], TextHeight, ChDesc.IsGlyphID());
			if( ok || CharPathCoordArray.size() >= Coords )
				break;

			// That seems to a size related error, increase buffer size
			Coords = ( Coords + 1023 ) & ~0x3FF; 
			TRACE( _T("Resize 2 %d (char %04x)\n"), Coords, ChDesc.GetCharCode() );	
			CharPathCoordArray.resize( Coords );
			CharPathVerbArray.resize( Coords );
		} 
// Paul: we can't get the default char's outline - we need the exact character glyph
/*
		// If that failed try again with the default character
		if (!ok)
		{
			// Get the default character to use if a charater is not present in the font.
			WCHAR DefChar = (unsigned char)'?';
			TEXTMETRIC FontTextData;
#ifdef _UNICODE
			if (GetTextMetrics(hActualDC, &FontTextData))
				DefChar = FontTextData.tmDefaultChar;
#else
			if (GetTextMetrics(hActualDC, &FontTextData))
				DefChar = (unsigned char)FontTextData.tmDefaultChar;
#endif

			while( true )
			{
				ok = GetBezierFromChar(hActualDC, DefChar, &CharLogFont, &Coords, (POINT*)&CharPathCoordArray[0], 
					&CharPathVerbArray[0], TextHeight);
				if( ok || CharPathCoordArray.size() <= Coords )
					break;

				// That seems to a size related error, increase buffer size
				Coords = ( Coords + 1023 ) & ~0x3FF; 
				TRACE( _T("Resize 2 %d\n"), Coords );	
				CharPathCoordArray.resize( Coords );
				CharPathVerbArray.resize( Coords );
			} 
		}
*/
		ERROR3IF(!ok, "TextManager::GetCharPath error from GetBezierFromChar");
	}

	// if ok, set outputs, tidy up and return
	if (ok)
	{
		*ppCoords   = &CharPathCoordArray[0];
		*ppVerbs    = &CharPathVerbArray[0];
		*pNumCoords = static_cast<ULONG>(Coords);
	}

	if (LocalDC)	
		DeleteDC(hActualDC);
		//ReleaseDC(NULL, hActualDC);
	return ok;
}

/***************************************************************************************
>	static BOOL TextManager::GetBezierFromChar(CDC* pDC,const WCHAR CharNumber,
											   LPLOGFONT pLogFont, DWORD* NoPolyElements,
											   POINT* pPolyCordBuffer, BYTE* pPolyVerbBuffer,
											   LONG TextHeight)

	Author:		MarkG
	Created:	25/1/95
	Purpose:	To generate the bezier paths which represent a character from a TrueType and, eventually, an ATM, font character
	Inputs:		pDC=>device context
				CharNumber=character number of char	this is a wide Unicode value
				pLogFontStruct=>LOGFONTW struct defining char, the size field is in logcords (=device cords in MM_TextMode) 
				NoPolyElements=Number of elements in PolyDraw arrays, 
					If NoPolyElements=0	then NoPolyElements is set to no. of elements required
					If NoPolyElements<>0 then generate paths in PolyDraw arrays
					On return a hint about how large buffer is needed
				pPolyCordBuffer=>Cord array in PolyDraw system
				pPolyVerbBuffer=>Verb array in PolyDraw system
				TextHeight=the height at which we retrieve the glyph from Windows (must have
						   the same sign as pLogFont->lfHeight, i.e., negative)
	Global:		Assumes MM_TextMode	
	Outputs:	NoPolyElements=No. required if entry value=0
				PolyDrawArrays (Verb,Cord) filled if NoPolyElements<>0, CordArrays hold cords in 1/72000's
	Returns:	If FALSE returned then error has occured
********************************************************************************************/
		
#define FIXEDTOLONG(FIXED) (*((long *)&FIXED))
#define SCALEX(jobby) (MulDiv(MulDiv(jobby,72000,DivConstX),pLogFont->lfHeight,TextHeight))
#define SCALEY(jobby) (MulDiv(MulDiv(jobby,72000,DivConstY),pLogFont->lfHeight,TextHeight))  
   
BOOL TextManager::GetBezierFromChar(HDC hDC,WCHAR CharNumber, LPLOGFONTW pLogFont, DWORD* NoPolyElements, POINT* pPolyCordBuffer, BYTE* pPolyVerbBuffer, LONG TextHeight, BOOL bGlyphIndex)
{
	// Prevent the array allocation below from throwing an exception in low memory conditions
	//ScopedDisableNewException sdne;

	if (*NoPolyElements==0)
	{ 
		*NoPolyElements=2000; // Assume that 500 element array is big enough, although we still check on this later!!
		return TRUE;
	}		 

	LONG DivConstX=GetDeviceCaps(hDC, LOGPIXELSX)<<16;
	LONG DivConstY=GetDeviceCaps(hDC, LOGPIXELSY)<<16;
	
	LOGFONTW tmp=*pLogFont;
	tmp.lfHeight=TextHeight;
	tmp.lfWidth=0;
	HFONT UnHintedCFont = CreateFontIndirectW(&tmp);

	if(UnHintedCFont == NULL)
	{
		ERROR3("CreateFontIndirectW failed");
		return FALSE;
	}

	//HFONT UnHintedCFont = CreateFontW(-tmp.lfHeight,tmp.lfWidth,0,0,tmp.lfWeight,tmp.lfItalic,0,0,DEFAULT_CHARSET,OUT_OUTLINE_PRECIS,CLIP_STROKE_PRECIS,PROOF_QUALITY,DEFAULT_PITCH,tmp.lfFaceName);
	HFONT pOldCFont=(HFONT)SelectObject(hDC, UnHintedCFont);
	
	MAT2 DefaultMatrix={ {0,1} ,{0,0}, {0,0}, {0,1} };
	TEXTMETRIC ATextMetricStruct;
	GetTextMetrics(hDC, &ATextMetricStruct);
	GLYPHMETRICS GlyphMetricsBuffer;

	UINT format = GGO_NATIVE;
	if (bGlyphIndex)
		format |= GGO_GLYPH_INDEX;

	// Get the size of the glyph data
	DWORD SizeOfGlyphData = GetGlyphOutlineW(hDC, CharNumber, format, &GlyphMetricsBuffer, 0, NULL, &DefaultMatrix);

	if (SizeOfGlyphData==-1)
	{
		//Lets restore the DC font status
		SelectObject(hDC, pOldCFont);
		DeleteObject(UnHintedCFont);
		return FALSE;
	}

//if(CharNumber == 'o')
//	SizeOfGlyphData = sizeof(TTPOLYGONHEADER) + sizeof(TTPOLYCURVE) + sizeof(POINTFX) * 3;

 	// claim memory for glyph data
	BYTE* pGlyphBuffer = new BYTE[SizeOfGlyphData];
	if (pGlyphBuffer == NULL)
	{
		//Lets restore the DC font status
		SelectObject(hDC, pOldCFont);
		DeleteObject(UnHintedCFont);
		return FALSE;
	}

	DWORD Error = 0;
/*
if(CharNumber == 'o')
{
	memset(pGlyphBuffer, 0, SizeOfGlyphData);
	TTPOLYGONHEADER* pHead = (TTPOLYGONHEADER*)pGlyphBuffer;
	pHead->cb = SizeOfGlyphData;
	pHead->dwType = 0;
	pHead->pfxStart.x.value = 248; pHead->pfxStart.y.value = 0;
	TTPOLYCURVE* pCurve = (TTPOLYCURVE*)(pGlyphBuffer + sizeof(TTPOLYGONHEADER));
	pCurve->wType = TT_PRIM_QSPLINE;
	pCurve->cpfx = 4;
	pCurve->apfx[0].x.value = 0; pCurve->apfx[0].y.value = 1000;
	pCurve->apfx[1].x.value = 1000; pCurve->apfx[1].y.value = 2000;
	pCurve->apfx[2].x.value = 2000; pCurve->apfx[2].y.value = 1000;
	pCurve->apfx[3].x.value = 2000-248; pCurve->apfx[3].y.value = 0;
}
else*/
	// Get the glyph outline data
	Error = GetGlyphOutlineW(hDC, CharNumber, format, &GlyphMetricsBuffer, SizeOfGlyphData, pGlyphBuffer, &DefaultMatrix);

	//Lets restore the DC font status
	SelectObject(hDC, pOldCFont);
	DeleteObject(UnHintedCFont);

	if (Error == -1)
 	{
		delete[] pGlyphBuffer;
		ERROR2(FALSE, "GetGlyphOutline failed");
	}

	// We now have the GlyphOutline structure in the GlyphBuffer, this consists of TTPolygon etc headers 
	// and records
	
	BYTE* pPosInGlyphBuffer=(BYTE*)pGlyphBuffer;// pointer to our current position in GlyphBuffer
	DWORD ContourSize;		   					// size of data controlled by a PolyHeader structure
	TTPOLYGONHEADER* pPolyHeader;				// needed to allow access to structure data
	TTPOLYCURVE* pPolyCurve;					// ditto
	DWORD	CurrentPolyIndex=0;					// Current index in the output PolyBuffer holding beziers etc
	WORD 	NoOfArrayElements;					// No of elements in a Qspline or prim_line record
	LONG	CurrentX;							// Last point of current path (line or quad b-spline) or previous one
	LONG	CurrentY;
	DWORD	FirstContourIndex;					// The index in PolyCordBuffer of the first member of the current contour

	while (pPosInGlyphBuffer<(BYTE*)pGlyphBuffer+SizeOfGlyphData) 
	{
		// make sure we have a sensible amount of data left
		ptrdiff_t remaining_space = (BYTE*)pGlyphBuffer + SizeOfGlyphData - pPosInGlyphBuffer;
		if (remaining_space < sizeof(TTPOLYGONHEADER))
		{
			// we should either have no data left or at least a complete new polygon header,
			// so this is probably a slightly broken font - simply ignore the extra data, see
			// DDES-9484
			TRACE(_T("Ignoring %d byte(s) of stray data at end of polygons"), remaining_space);
			break;
		}

		// Right lets process a TTPOLYGONHEADER and its associated spline and line records	
		pPolyHeader=(TTPOLYGONHEADER*)pPosInGlyphBuffer;
		ContourSize=pPolyHeader->cb;
		CurrentX=FIXEDTOLONG(pPolyHeader->pfxStart.x);
		CurrentY=FIXEDTOLONG(pPolyHeader->pfxStart.y);
		pPolyCordBuffer[CurrentPolyIndex].x=SCALEX(CurrentX); // First entry is always MoveTo to set 1st point of bezier etc
		pPolyCordBuffer[CurrentPolyIndex].y=SCALEY(CurrentY);
		pPolyVerbBuffer[CurrentPolyIndex]  =PT_MOVETO;
		FirstContourIndex=CurrentPolyIndex;
		CurrentPolyIndex++;							 
						   		
		pPosInGlyphBuffer=pPosInGlyphBuffer+sizeof(TTPOLYGONHEADER);  // pPosInGlyphBuffer=>1st TTPOLYCURVE
		
		// Lets process all the TTPOLYGONCURVE structs that exist within the current TTPOLYGONHEADER domain (contour)
		do
		{
			pPolyCurve=(TTPOLYCURVE*)pPosInGlyphBuffer;
			NoOfArrayElements=pPolyCurve->cpfx;
			
			DWORD i;
			switch (pPolyCurve->wType)
			{	
			case TT_PRIM_LINE:
				if( CurrentPolyIndex + NoOfArrayElements >= *NoPolyElements )	// are we overflowing poly draw buffer
				{
					delete[] pGlyphBuffer;
					TRACE(_T("overflowing polydraw buffer\n"));
					*NoPolyElements = CurrentPolyIndex + NoOfArrayElements + 1; // NB + 1 because >=
					return FALSE;
				}

				for	(i=0;i<NoOfArrayElements;i++)
				{
					CurrentX=FIXEDTOLONG(pPolyCurve->apfx[i].x);
					pPolyCordBuffer[CurrentPolyIndex].x=SCALEX(CurrentX);		// First entry is always MoveTo to set 1st point of bezier etc
					CurrentY=FIXEDTOLONG(pPolyCurve->apfx[i].y);
					pPolyCordBuffer[CurrentPolyIndex].y=SCALEY(CurrentY);
					pPolyVerbBuffer[CurrentPolyIndex]=PT_LINETO;
					CurrentPolyIndex++;
				}
				break;

				
			case TT_PRIM_QSPLINE:
				if( CurrentPolyIndex + ( NoOfArrayElements * 3 ) >= *NoPolyElements )	// are we overflowing poly draw buffer
				{
					delete[] pGlyphBuffer;
					TRACE(_T("overflowing polydraw buffer\n"));
					*NoPolyElements = CurrentPolyIndex + ( NoOfArrayElements * 3 ) + 1; // NB + 1 because >=
					return FALSE;
				}
				
				for ( i=0;i+1<NoOfArrayElements;i++)
				{
					pPolyVerbBuffer[CurrentPolyIndex]=PT_BEZIERTO;
					pPolyVerbBuffer[CurrentPolyIndex+1]=PT_BEZIERTO;
					pPolyVerbBuffer[CurrentPolyIndex+2]=PT_BEZIERTO;

					//TRACE("i=%d\n",i);
					// Lets set CurrentPolyIndex [+0] & [+1]
					pPolyCordBuffer[CurrentPolyIndex].x=
// This can overflow - best to scale the components separately and do the divide before the shift
//					SCALEX((CurrentX+((FIXEDTOLONG(pPolyCurve->apfx[i].x))<<1))/3);
					SCALEX(CurrentX/3) + SCALEX(((FIXEDTOLONG(pPolyCurve->apfx[i].x))/3)<<1);
					
					pPolyCordBuffer[CurrentPolyIndex].y=
// This can overflow - best to scale the components separately and do the divide before the shift
//					SCALEY((CurrentY+((FIXEDTOLONG(pPolyCurve->apfx[i].y))<<1))/3);
					SCALEY(CurrentY/3) + SCALEY(((FIXEDTOLONG(pPolyCurve->apfx[i].y))/3)<<1);
/*					
	double dSrcX = FIXEDTOLONG(pPolyCurve->apfx[i].x)/65536.0;
	double dSrcY = FIXEDTOLONG(pPolyCurve->apfx[i].y)/65536.0;
	double dC1X = (pPolyCordBuffer[CurrentPolyIndex].x - SCALEX(CurrentX/3))*3.0/2.0 * 4096 / 72000.0;
	double dC1Y = (pPolyCordBuffer[CurrentPolyIndex].y - SCALEY(CurrentY/3))*3.0/2.0 * 4096 / 72000.0;
	if(dC1X != dSrcX || dC1Y != dSrcY)
	{
		int vvv=65;
	}
*/
					if (i+2==NoOfArrayElements)	 // are we on last processable point in record?
					{ // yes we are, so just get last point
						CurrentX=FIXEDTOLONG(pPolyCurve->apfx[i+1].x);
						pPolyCordBuffer[CurrentPolyIndex+2].x=SCALEX(CurrentX);
						CurrentY=FIXEDTOLONG(pPolyCurve->apfx[i+1].y);	
						pPolyCordBuffer[CurrentPolyIndex+2].y=SCALEY(CurrentY);	
					}
					else 
					{ // no we're not, so we have to calculate midpoint between the two following points
// This can overflow - best to do the divides before the adds
//						CurrentX=(FIXEDTOLONG(pPolyCurve->apfx[i].x)+FIXEDTOLONG(pPolyCurve->apfx[i+1].x))>>1;
						CurrentX=(FIXEDTOLONG(pPolyCurve->apfx[i].x)>>1)+(FIXEDTOLONG(pPolyCurve->apfx[i+1].x)>>1);
						pPolyCordBuffer[CurrentPolyIndex+2].x=SCALEX(CurrentX);
// This can overflow - best to do the divides before the adds
//						CurrentY=(FIXEDTOLONG(pPolyCurve->apfx[i].y)+FIXEDTOLONG(pPolyCurve->apfx[i+1].y))>>1;
						CurrentY=(FIXEDTOLONG(pPolyCurve->apfx[i].y)>>1)+(FIXEDTOLONG(pPolyCurve->apfx[i+1].y)>>1);
						pPolyCordBuffer[CurrentPolyIndex+2].y=SCALEY(CurrentY);
					}

					pPolyCordBuffer[CurrentPolyIndex+1].x=
// This can overflow - best to scale the components separately and do the divide before the shift
//					SCALEX((CurrentX+((FIXEDTOLONG(pPolyCurve->apfx[i].x))<<1))/3);
					SCALEX(CurrentX/3) + SCALEX(((FIXEDTOLONG(pPolyCurve->apfx[i].x))/3)<<1);
					
					pPolyCordBuffer[CurrentPolyIndex+1].y=
// This can overflow - best to scale the components separately and do the divide before the shift
//					SCALEY((CurrentY+((FIXEDTOLONG(pPolyCurve->apfx[i].y))<<1))/3);
					SCALEY(CurrentY/3) + SCALEY(((FIXEDTOLONG(pPolyCurve->apfx[i].y))/3)<<1);
					
					CurrentPolyIndex=CurrentPolyIndex+3;

				}
				break;
				
				default:
				delete[] pGlyphBuffer;
				TRACE(_T("oh my god\n"));
				return FALSE;
				//AfxThrowMemoryException();
			}

			pPosInGlyphBuffer=pPosInGlyphBuffer+sizeof(TTPOLYCURVE)+(NoOfArrayElements-1)*sizeof(POINTFX);

		} while (pPosInGlyphBuffer<(BYTE*)pPolyHeader+ContourSize);
		
		if ( (pPolyCordBuffer[FirstContourIndex].x != pPolyCordBuffer[CurrentPolyIndex-1].x) ||
				 (pPolyCordBuffer[FirstContourIndex].y != pPolyCordBuffer[CurrentPolyIndex-1].y))
		{
			if( CurrentPolyIndex + 1 >= *NoPolyElements )	// are we overflowing poly draw buffer
			{
				delete[] pGlyphBuffer;
				TRACE(_T("overflowing polydraw buffer\n"));
				*NoPolyElements = CurrentPolyIndex + 2; // NB + 1 because >=
				return FALSE;
			}
				
			pPolyVerbBuffer[CurrentPolyIndex]=PT_LINETO | PT_CLOSEFIGURE;
			pPolyCordBuffer[CurrentPolyIndex]=pPolyCordBuffer[FirstContourIndex];
			CurrentPolyIndex++;
		}
		else 
		{
			pPolyVerbBuffer[CurrentPolyIndex-1]=pPolyVerbBuffer[CurrentPolyIndex-1] | PT_CLOSEFIGURE;
 		}

	}		
	delete[] pGlyphBuffer;

	*NoPolyElements=CurrentPolyIndex;
	return TRUE; 
}

/******************************************************************************
   Function:	CGlyphDiff::GlyphsSimilar
   Author:		Paul Gruzdev
   Purpose:     The number of subpaths is important if the letters are relatively
				small and comparable, for example 'i' and small-cap 'I'. In this
				case the difference may be onlya a 1.3% in EM square. So we have to take 
				into account the number of subpaths, that differ in these letters.
				2925.pdf is an example
   Returns:		bool
   Notes:       
******************************************************************************/
bool CGlyphDiff::GlyphsSimilar()
{
	double dDiff = DiffIntegral();

	if(iNumSubpaths1 == iNumSubpaths2)
		return dDiff <= GLYPH_SQUARE_DIFF_THRESHOLD;

	wprintf(L"glyph %c [%X]: different sub-path count %d vs %d\n", wUnicode, wUnicode, iNumSubpaths1, iNumSubpaths2);

	// This '&' char often has either 3 or 4 sub-paths that visually are the same, too many examples from our big set.
	if(wUnicode == '&' && (iNumSubpaths1 == 3 || iNumSubpaths1 == 4) && (iNumSubpaths2 == 3 || iNumSubpaths2 == 4))
		return dDiff <= GLYPH_SQUARE_DIFF_THRESHOLD;

	return dDiff <= 0.01; // 1% for the rest	
}

void FindBezierBounds(RECT& rBounds, const DocCoord P0, const DocCoord P1, const DocCoord P2, const DocCoord P3)
{
	float A = (float)P3.x - 3 * (float)P2.x + 3 * (float)P1.x - (float)P0.x;
	float B = 3 * (float)P2.x - 6 * (float)P1.x + 3 * (float)P0.x;
	float C = 3 * (float)P1.x - 3 * (float)P0.x;
	float D = (float)P0.x;

	float E = (float)P3.y - 3 * (float)P2.y + 3 * (float)P1.y - (float)P0.y;
	float F = 3 * (float)P2.y - 6 * (float)P1.y + 3 * (float)P0.y;
	float G = 3 * (float)P1.y - 3 * (float)P0.y;
	float H = (float)P0.y;

	float x, y;
	float xMin = (float)INT_MAX;
	float yMin = (float)INT_MAX;
	float xMax = 0;
	float yMax = 0;

	for (float t = 0.0f; t <= 1.0f; t += 0.01f)
	{
		x = A * t * t * t + B * t * t + C * t + D;
		if (x < xMin)
			xMin = x;
		if (x > xMax)
			xMax = x;
		y = E * t * t * t + F * t * t + G * t + H;
		if (y < yMin)
			yMin = y;
		if (y > yMax)
			yMax = y; 
	}

	rBounds.left = (LONG)xMin;
	rBounds.top = (LONG)yMin;
	rBounds.right = (LONG)xMax;
	rBounds.bottom = (LONG)yMax;
}

/******************************************************************************
   Function:	TextManager::GetMaxHeight
   Author:		Paul Gruzdev
   Purpose:     Find the maximum extrema of the current glyph. Used to find
				capital height or x-height constants for a particular font.
   Returns:		LONG
   Parameters:	PBYTE pVerbs  
   Parameters:	DocCoord * pCoords  
   Parameters:	DWORD dwCount  
   Parameters:	Flatness & type  
   Notes:       
******************************************************************************/
LONG TextManager::GetMaxHeight(PBYTE pVerbs, DocCoord* pCoords, DWORD dwCount, TextManager::Flatness& type)
{
	TextManager::Flatness f = mt_unknown;
	LONG max = LONG_MIN;

	for(DWORD i = 0; i < dwCount; i++)
	{
		LONG prev = i > 0 ? pCoords[i-1].y : LONG_MIN;
		LONG cur = pCoords[i].y;

		switch(pVerbs[i])
		{
		case PT_LINETO:
			{
				if(prev > max)
				{
					max = prev;
					f = prev == cur ? mt_flat : mt_pointy;
				}
				if(cur >= max)
				{
					max = cur;
					f = prev == cur ? mt_flat : mt_pointy;
				}
			}
			break;
		case PT_BEZIERTO:
			{
				LONG to = pCoords[i+2].y;

				if (prev > max)
				{
					f = mt_round;
					max = prev;
				}

				if (to > max)
				{
					f = mt_round;
					max = to;
				}

				RECT rc;
				FindBezierBounds(rc, pCoords[i-1], pCoords[i], pCoords[i+1], pCoords[i+2]);
				if(rc.bottom > max)
				{
					f = mt_round;
					max = rc.bottom;
				}
				i += 2;
			}
			break;
		}
	}

	type = f;
	return max;
}

LONG TextManager::FindCapHeight(std::wstring strTypeface, int weight, bool bItalic)
{
	static std::map<CharDescription, LONG> s_cache;
	CharDescription default('A', strTypeface, weight, bItalic);
	std::map<CharDescription, LONG>::iterator it = s_cache.find(default);
	if(it != s_cache.end())
		return it->second;

	std::vector<std::pair<LONG, Flatness>> vec;

	LONG max = LONG_MIN;
	LONG sum = 0;
	LONG height_count = 0;
	for(WCHAR i = 'A'; i <= 'Z'; i++)
	{
		PathVerb* verb;
		DocCoord* coord;
		ULONG count;
		CharDescription ChDesc(i, strTypeface, weight, bItalic);
		if(GetTTCharPath(ChDesc, &coord, &verb, &count, NULL))
		{
			TextManager::Flatness type;
			LONG lHeight = GetMaxHeight(verb, coord, count, type);
			vec.push_back(std::make_pair(lHeight, type));
			if(type != mt_unknown)
			{
				sum += lHeight;
				height_count++;
			}
			//if(lHeight > max && type != mt_unknown)
			//	max = lHeight;
		}
	}

	max = height_count ? sum/height_count : 0;
	s_cache[default] = max;
	return max;
}

int CALLBACK EnumFontFamExProcPrecise(ENUMLOGFONTEXW *lpelfe,    // logical-font data
									  NEWTEXTMETRICEXW *lpntme,  // physical-font data
									  DWORD FontType,           // type of font
									  LPARAM lParam             // application-defined data
									  )
{
	CEnumTextFontData* pData = (CEnumTextFontData*)lParam;
	bool bItalic = lpelfe->elfLogFont.lfItalic != 0 ? true : false;

	if(!_wcsicmp(lpelfe->elfLogFont.lfFaceName, pData->strName.c_str()) && lpelfe->elfLogFont.lfWeight == pData->iWeight &&
		bItalic == pData->bItalic)
		return 0;

	return 1;
}

bool TextManager::FontExistInTheSystem(std::wstring unicodeFontName, int weight, bool bItalic)
{
	HDC hdc = GetDC(NULL);
	LOGFONTW lf;
	memset(&lf, 0, sizeof(LOGFONTW));
	wcsncpy_s(lf.lfFaceName, LF_FACESIZE, unicodeFontName.c_str(), LF_FACESIZE-1);
	lf.lfCharSet = DEFAULT_CHARSET;
	lf.lfPitchAndFamily = 0;

	CEnumTextFontData dta;
	dta.strName = unicodeFontName;
	dta.iWeight = weight;
	dta.bItalic = bItalic;

	bool bRes = EnumFontFamiliesExW(hdc, &lf, (FONTENUMPROCW)EnumFontFamExProcPrecise, (LPARAM)&dta, 0) == 0;
	ReleaseDC(0,hdc);
	return bRes;
}

int CALLBACK EnumFontFamExProcForDoes(ENUMLOGFONTEXW *lpelfe,    // logical-font data
									  NEWTEXTMETRICEXW *lpntme,  // physical-font data
									  DWORD FontType,           // type of font
									  LPARAM lParam             // application-defined data
									  )
{

	if (!_wcsicmp(lpelfe->elfLogFont.lfFaceName, (wchar_t*)lParam))
		return 0;

	return 1;
}

bool TextManager::DoesFontExistInTheSystem(HDC hdc, std::wstring unicodeFontName)
{
	LOGFONTW lf;
	memset(&lf, 0, sizeof(LOGFONTW));
	wcsncpy_s(lf.lfFaceName, LF_FACESIZE, unicodeFontName.c_str(), LF_FACESIZE-1);
	lf.lfCharSet = DEFAULT_CHARSET;
	lf.lfPitchAndFamily = 0;

	return EnumFontFamiliesExW(hdc, &lf, (FONTENUMPROCW)EnumFontFamExProcForDoes, (LPARAM)unicodeFontName.c_str(), 0) == 0;
}

/******************************************************************************
   Function:	GetFontMetrics
   Author:		Paul Gruzdev
   Purpose:		Retrieve font metrics given font name weight italic and font size in pixels
   Returns:		BOOL
   Parameters:	const std::wstring & strFontName
   Parameters:	DWORD weight
   Parameters:	bool bItalic
   Parameters:	LONG lFontSize
   Parameters:	TEXTMETRIC & tm
   Notes:       
******************************************************************************/
BOOL TextManager::GetFontMetrics(const std::wstring &strFontName, DWORD weight, bool bItalic, LONG lFontSize, TEXTMETRICW& tm)
{
	BOOL bRes = FALSE;
	if (HDC hdc = GetDC(NULL)) // take the screen dc
	{
		if (HFONT fnt = CreateFontW(-lFontSize, 0, 0, 0, weight, bItalic ? 255 : 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, DEFAULT_PITCH, strFontName.c_str()))
		{
			HFONT oldFont = (HFONT)SelectObject(hdc, fnt);

			static const size_t LENGTH_FONT_FACE = 512;
			wchar_t fontFace[LENGTH_FONT_FACE];
			GetTextFaceW(hdc, LENGTH_FONT_FACE, fontFace);
			if (!wcsncmp(fontFace, strFontName.c_str(), LENGTH_FONT_FACE))
			{
				if (GetTextMetricsW(hdc, &tm))
					bRes = TRUE;
			}
			else
			{
				_ASSERT(0); // We are trying to use the font typeface name that is NOT present in the system.
			}
			SelectObject(hdc, oldFont);
			DeleteObject(fnt);
		}
		ReleaseDC(0, hdc);
	}

	return bRes;
}

/******************************************************************************
   Function:	GetCharWidth
   Author:		Paul Gruzdev
   Purpose:		Return the width of a char if the font size was set to 72000
   Returns:		MILLIPOINT - the width of a char of the font size of 72000
   Parameters:	const std::wstring & strFontName
   Parameters:	DWORD weight
   Parameters:	bool bItalic
   Parameters:	wchar_t chr
   Notes:       
******************************************************************************/
MILLIPOINT TextManager::GetCharWidth(const std::wstring &strFontName, DWORD weight, bool bItalic, wchar_t chr/*= 'M'*/)
{
	static const INT EM_ERROR = -1;
	INT width = EM_ERROR;

	if (HDC hdc = GetDC(NULL)) // take the screen dc
	{
		if (HFONT fnt = CreateFontW(-2048, 0, 0, 0, weight, bItalic ? 255 : 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY, DEFAULT_PITCH, strFontName.c_str()))
		{
			HFONT oldFont = (HFONT)SelectObject(hdc, fnt);

			static const size_t LENGTH_FONT_FACE = 512;
			wchar_t fontFace[LENGTH_FONT_FACE];
			GetTextFaceW(hdc, LENGTH_FONT_FACE, fontFace);
			if (!wcsncmp(fontFace, strFontName.c_str(), LENGTH_FONT_FACE))
			{
				ABC abc;
				if (GetCharABCWidths(hdc, chr, chr, &abc))
					width = abc.abcA + abc.abcB + abc.abcC;
				else if (!GetCharWidth32(hdc, chr, chr, &width))
					width = EM_ERROR;
			}
			else
			{
				_ASSERT(0); // We are trying to use the font typeface name that is NOT present in the system.
			}
			SelectObject(hdc, oldFont);
			DeleteObject(fnt);
		}
		ReleaseDC(0, hdc);
	}

	// That's exactly the width that Camelot calculates, for comparison with us
	//int iw = MulDiv(iWidth, 72000, 2048); 
	return EM_ERROR != width ? MulDiv(width, GetDefaultHeight(), 2048) : width;
}

/******************************************************************************
   Function:	TextManager::GlyphExists
   Author:		Paul Gruzdev
   Purpose:     Check the glyph of a specified font exist in the system, so later on
				we could take it outline for example
   Returns:		bool
   Parameters:	std::wstring strWinName  
   Parameters:	int weight  
   Parameters:	bool isItalic  
   Parameters:	wchar_t wUnicode - Unicode to test
   Notes:       
******************************************************************************/
BOOL TextManager::GlyphExists(std::wstring strWinName, int weight, bool isItalic, wchar_t wUnicode)
{
	BOOL bRes = TRUE;
	HDC hdc = GetDC(NULL); // take the screen dc
	HFONT fnt = CreateFontW(-2048,0,0,0,weight,isItalic ? 255 : 0,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,DEFAULT_PITCH,strWinName.c_str());
	HFONT oldFont = (HFONT) SelectObject(hdc, fnt);

	MAT2 mat;
	memset((void*)&mat, 0, sizeof(MAT2));
	mat.eM11.value = 1; mat.eM22.value = 1;

	WCHAR w = wUnicode;
	WORD wOut;
	GetGlyphIndicesW(hdc, &w, 1, &wOut, GGI_MARK_NONEXISTING_GLYPHS);

	if(wOut == 0xFFFF) // the Unicode glyph is absent - bail
		bRes = FALSE;

	SelectObject(hdc, oldFont);
	ReleaseDC(0, hdc);
	DeleteObject(fnt);
	return bRes;
}

BOOL TextManager::GetGIDs(std::wstring strWinName, int weight, bool isItalic, std::wstring& strIn, std::wstring& strOut)
{
	if(strOut.length() < strIn.length())
		strOut.resize(strIn.length());

	HDC hdc = GetDC(NULL); // take the screen dc
	HFONT fnt = CreateFontW(-2048,0,0,0,weight,isItalic ? 255 : 0,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,DEFAULT_PITCH,strWinName.c_str());
	HFONT oldFont = (HFONT) SelectObject(hdc, fnt);

	MAT2 mat;
	memset((void*)&mat, 0, sizeof(MAT2));
	mat.eM11.value = 1; mat.eM22.value = 1;

	DWORD dwRet = GetGlyphIndicesW(hdc, &strIn[0], (int)strIn.length(), (LPWORD)&strOut[0], GGI_MARK_NONEXISTING_GLYPHS);

	SelectObject(hdc, oldFont);
	ReleaseDC(0, hdc);
	DeleteObject(fnt);
	return dwRet != GDI_ERROR;
}

/******************************************************************************
   Function:	RetrieveFontData
   Author:		Paul Gruzdev
   Purpose:		
   Returns:		BYTE*
   Parameters:	std::string unicodeFontName
   Parameters:	int weight
   Parameters:	int italic
   Parameters:	int & iSize
   Notes:       
******************************************************************************/
BYTE* TextManager::RetrieveFontData(std::wstring strFontName, int weight, int italic, int& iSize)
{
	// font data that will be stored
	unsigned char* fontData = 0;
	// get the device context of the desktop
	HDC hdc = GetDC(0);
	// create th font to be selected into the dc
	HFONT fontToTest = CreateFontW(-1000,0,0,0,weight,italic,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,DEFAULT_PITCH,strFontName.c_str());
	// the old font that will be used
	HFONT oldFont = 0;
	// select the new font into the device context
	if (fontToTest)
	{
		try
		{
			oldFont = (HFONT) SelectObject(hdc,fontToTest);

			static const size_t LENGTH_FONT_FACE = 512;
			wchar_t fontFace[LENGTH_FONT_FACE];
			GetTextFaceW(hdc, LENGTH_FONT_FACE, fontFace);
			// Test whether we created the exact font we wanted to, not the substitution
			if (wcsncmp(fontFace, strFontName.c_str(), LENGTH_FONT_FACE))
				throw false;

			// Paul: #9064 MS Gothic.ttc issue fix.
			/* From MSDN: The name of a font metric table from which the font data is to be retrieved. 
			This parameter can identify one of the metric tables documented in the TrueType Font Files 
			specification published by Microsoft Corporation. If this parameter is zero, the information 
			is retrieved starting at the beginning of the file for TrueType font files or from the 
			beginning of the data for the currently selected font for TrueType Collection files. 
			To retrieve the data from the beginning of the file for TrueType Collection files specify 
			'ttcf' (0x66637474). */
			DWORD dwTable = 0x66637474; // Set this initially to a 'ttcf' to get the whole ttc data

			// passing this tells the API we want the size of the font data
			DWORD sizeOfFontData = GetFontData(hdc,dwTable,0,0,0);

			if(sizeOfFontData == -1) // This is not TTC, take the regular font file
			{
				dwTable = 0; // reset this if ttc signature fails
				sizeOfFontData = GetFontData(hdc,dwTable,0,0,0);
			}

			if(sizeOfFontData==-1)
				throw false;
			// check to see if size of font data is zero if it is get out
			if(sizeOfFontData==0)
				throw(false);

			fontData = new unsigned char[sizeOfFontData];

			// we need to get the font data so we can ask FreeType to determine if
			// this is a unicode font.
			if(GetFontData(hdc,dwTable,0,fontData,sizeOfFontData)== GDI_ERROR)
				throw (false);

			iSize = sizeOfFontData;
			throw(true);
		}
		// an error occurred get out
		catch(bool& success)
		{
			SelectObject(hdc,oldFont);
			ReleaseDC(0,hdc);
			DeleteObject(fontToTest);

			if(success && fontData)
				return fontData;

			// No success - we have to try a different method
			if(fontData)
			{
				delete [] fontData;
				fontData = NULL;
			}
		}
	}

	return fontData;
}
