/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/

#pragma once

#include "xarlib.h"
#include "doccoord.h"

typedef BYTE PathVerb;

const double GLYPH_SQUARE_DIFF_THRESHOLD = 0.025;
const double GLYPH_SQUARE_DIFF_THRESHOLD_HIGH = 0.05;
const LONG GLYPH_WIDTH_DIFF_THRESGHOLD = 720;


class CEnumTextFontData 
{
public:
	std::wstring strName;
	int iWeight;
	bool bItalic;
};

class CharDescription
{
public:
	// Needs sorting out
	CharDescription() { charHandle = 0; };
	CharDescription(const CharDescription& that);
	CharDescription(WCHAR CharCode, std::wstring strTypefaceA, int weightA, BOOL Italic, BOOL isGlyphID = FALSE);

	WCHAR GetCharCode() const		{ return (WCHAR)(charHandle >> 16); }
	std::wstring GetTypeface() const	{ return strTypeface; }
	BOOL GetWeight() const			{ return (WORD)(charHandle & 0x00003FFF); }
	BOOL GetItalic() const			{ return ((charHandle & 0x00004000) != 0); }
	BOOL IsGlyphID() const			{ return ((charHandle & 0x00008000) != 0); }

	bool operator<(const CharDescription& f) const
	{
		int iFaceDiff = wcscmp(strTypeface.c_str(), f.strTypeface.c_str());
		int iWeightDiff = GetWeight() - f.GetWeight();
		int iItalicDiff = GetItalic() < f.GetItalic() ? -1 : (GetItalic() > f.GetItalic() ? 1 : 0);

		if(iFaceDiff != 0) return iFaceDiff < 0;
		if(iWeightDiff != 0) return iWeightDiff < 0;
		if(iItalicDiff != 0) return iItalicDiff < 0;

		return false;
	}

public:
	ULONG charHandle;	  // The char handle used in GCache 
	std::wstring strTypeface;

	CharDescription& operator=(const CharDescription& that) { this->charHandle =that.charHandle; this->strTypeface = that.strTypeface; return *this; }
	BOOL operator==(const CharDescription& that) const { return this->charHandle==that.charHandle && this->strTypeface == that.strTypeface; }
	BOOL operator!=(const CharDescription& that) const { return this->charHandle!=that.charHandle || this->strTypeface != that.strTypeface; }
};

class CGlyphDiff
{
public:
	CGlyphDiff(wchar_t wUni) : wUnicode(wUni), xor(DBL_MAX), intersect(DBL_MAX), square(0.0), bGlyphNotPresent(false), bWidthMismatch(false),
	iNumSubpaths1(0), iNumSubpaths2(0) {}
	double DiffIntegral() { return xor/(72000.0*72000.0); }
	double DiffSquare() { return square > 0 ? xor/square : 0.0; }
	double DiffGlyph() { return xor/intersect; }
	bool OutlineDiffer() { return !bGlyphNotPresent && !bWidthMismatch; }
	bool GlyphNotPresent() { return bGlyphNotPresent; }
	bool GlyphsSimilar();
	wchar_t wUnicode;
	double xor;
	double intersect;
	double square;
	bool bGlyphNotPresent;
	bool bWidthMismatch;
	size_t iNumSubpaths1;
	size_t iNumSubpaths2;
};

/********************************************************************************************
>	class TextManager:

	Author:		Mark/Paul Gruzdev
	Created:	21/1/95 - 25/04/2015
	Purpose:	Class that holds operations with glyph outlines stored in (LONG, LONG)
				(-72K..72K, -72K..72K) format, bezier curves. The exactly format Camelot uses
				to store the glyphs.
********************************************************************************************/

class TextManager
{
public:
	enum Flatness { mt_flat, mt_round, mt_pointy, mt_unknown };

	static BOOL GetLogFontFromCharDescriptor(HDC hDC, CharDescription& ChDesc, LPLOGFONTW pLogFont,
												LONG LogicalHeight=-1);

	static BOOL GetTTCharPath(CharDescription& ChDesc, DocCoord** ppCoords, PathVerb** ppVerbs, ULONG* pNumCoords, HDC hDC);
  
  	static BOOL GetBezierFromChar(HDC hDC, WCHAR CharNumber, LPLOGFONTW pLogFont,
  								  DWORD* NoElements, POINT* pPolyCordBuffer, BYTE* pPolyVerbBuffer,
								  LONG TextHeight, BOOL bGlyphIndex = FALSE);

	static LONG GetMaxHeight(PBYTE pVerbs, DocCoord* pCoords, DWORD dwCount, TextManager::Flatness& type);
	static LONG FindCapHeight(std::wstring strTypeface, int weight, bool bItalic);

	static MILLIPOINT GetCharWidth(const std::wstring &strFontName, DWORD weight, bool bItalic, wchar_t chr = 'M');
	static BOOL GetFontMetrics(const std::wstring &strFontName, DWORD weight, bool bItalic, LONG lFontSize, TEXTMETRICW& tm);
	static bool FontExistInTheSystem(std::wstring unicodeFontName, int weight, bool bItalic);
	static bool DoesFontExistInTheSystem(HDC hdc, std::wstring unicodeFontName);
	static BOOL GlyphExists(std::wstring strWinName, int weight, bool isItalic, wchar_t wUnicode);
	static BOOL GetGIDs(std::wstring strWinName, int weight, bool isItalic, std::wstring& strIn, std::wstring& strOut);
	static BYTE* RetrieveFontData(std::wstring strFontName, int weight, int italic, int& iSize);

	static LONG GetDefaultHeight() { return 72000; }	// size of char-paths/advance-widths in caches etc
};
