/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

template<class InIt, class OutIt> inline
void utf8_encode(InIt in, const InIt end, OutIt out)
{
    while( !(in == end) ) {
        unsigned long wc = static_cast<wchar_t>(*in); ++in;
    over:
        if( wc < 0x80 ) {
            *out = static_cast<char>(wc); ++out;
            continue;
        }

        if( sizeof(wchar_t) == 2 && 
            wc >= 0xD800 && wc < 0xE000 ) 
        {//handle surrogates for UTF-16
            if( wc >= 0xDC00 ) { wc = '?'; goto over; }
            if( in == end ) return;
            unsigned long lo = static_cast<wchar_t>(*in); ++in;
            if( lo >= 0xDC00 && wc < 0xE000 ) {
                wc  = 0x10000 + ((wc & 0x3FF) << 10 | lo & 0x3FF);
            } else { 
                *out = '?'; ++out; wc = lo;
                goto over;
            }
        }

        char c; int shift; 
        if( wc < 0x800 )     { shift = 6;  c = ('\xFF' << 6); } else
        if( wc < 0x10000 )   { shift = 12; c = ('\xFF' << 5); } else
        if( wc < 0x200000 )  { shift = 18; c = ('\xFF' << 4); } else
        if( wc < 0x4000000 ) { shift = 24; c = ('\xFF' << 3); } else 
                             { shift = 30; c = ('\xFF' << 2); }
        do {
            c |= (wc >> shift) & 0x3f;
            *out = c; ++out;
            c = char(0x80); shift -= 6;
        } while( shift >= 0 );
    }
}

template<class InIt, class OutIt> inline
void utf8_decode(InIt in, const InIt end, OutIt out)
{
    int cnt;
    for(; !(in == end); ++in) {
        unsigned long wc = static_cast<unsigned char>(*in);
    over:
        if( wc & 0x80 ) {
            if( 0xC0 == (0xE0 & wc) ) { cnt = 1; wc &= ~0xE0; } else
            if( 0xE0 == (0xF0 & wc) ) { cnt = 2; wc &= ~0xF0; } else
            if( 0xF0 == (0xF8 & wc) ) { cnt = 3; wc &= ~0xF8; } else
            if( 0xF8 == (0xFC & wc) ) { cnt = 4; wc &= ~0xFC; } else
            if( 0xFC == (0xFE & wc) ) { cnt = 5; wc &= ~0xFE; } else
                { *out = wchar_t('?'); ++out; continue; };//invalid start code
            if( 0 == wc ) wc = ~0UL;//codepoint encoded with overlong sequence
            do {
                if( ++in == end ) return;
                unsigned char c = static_cast<unsigned char>(*in);
                if( 0x80 != (0xC0 & c) )
                    { *out = static_cast<wchar_t>(wc); ++out; wc = c; goto over; }
                wc <<= 6; wc |= c & ~0xC0;
            } while( --cnt );
            if( 0x80000000 & wc ) wc = '?';//codepoint exceeds unicode range
            if( sizeof(wchar_t) == 2 && wc > 0xFFFF )
            {//handle surrogates for UTF-16
                wc -= 0x10000;
                *out = static_cast<wchar_t>(0xD800 | ((wc >> 10) & 0x3FF)); ++out;
                *out = static_cast<wchar_t>(0xDC00 | (wc & 0x3FF)); ++out;
                continue;
            }
        }
        *out = static_cast<wchar_t>(wc); ++out;
    }
}
