/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XFTree.h"

/******************************************************************************
   Function:    CXfAttributeContext::Update
   Author:      Paul Gruzdev
   Purpose:		Update the current context with the attribute node
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfAttributeContext::Update(VXfTreeNode* pNode)
{
	if(pNode->IsWebAddress())
		m_pWebAddress = (CXfWebAddress*)pNode;
	else if(pNode->IsStrokeStyle())
		m_pLineColor = (CXfLineColor*)pNode;
	else if(pNode->IsFillStyle())
	{
		if(pNode->IsTransparentFill())
			m_pTransparentFill = (VXFFillStyleNode*)pNode;
		else
			m_pFill = (VXFFillStyleNode*)pNode;
	}
	else
	{
		switch(pNode->GetTag())
		{
		case TAG_LINEWIDTH:
			m_dwLineWidth = ((CXfLineWidth*)pNode)->Width.get();
			break;
		case TAG_MITRELIMIT:
			m_dwMitreLimit = ((CXfMitreLimit*)pNode)->Limit.get();
			break;
		case TAG_DASHSTYLE:
			m_lDashStyle = ((CXfDashStyle*)pNode)->Style.get();
			break;
		case TAG_WINDINGRULE:
			m_byWindingRule = ((CXfWindingRule*)pNode)->Winding.get();
			break;
		case TAG_LINETRANSPARENCY:
			m_pLineTrans = (CXfLineTransparency*)pNode;
			break;
		case TAG_JOINSTYLE:
			m_uJoinStyle = ((CXfJoinStyle*)pNode)->Style;
			break;
		case TAG_STARTCAP:
			m_uStartCap = ((CXfStartCap*)pNode)->Style;
			break;
		case TAG_REPEL_DISTANCE:
			m_lRepelDistance = ((CXfNodeValue*)pNode)->GetValue();
			break;
		case TAG_REPEL_DISTANCE_V:
			m_lRepelDistanceV = ((CXfNodeValue*)pNode)->GetValue();
			break;
		case TAG_WIZOP:
			m_pWizop = (CXfWizop*)pNode;
			break;
		case TAG_SHADOWCONTROLLER:
			m_pShadowController = (CXfDropShadowController*)pNode;
			break;
		case TAG_URL_COLOUR_OVERRIDE:
			m_bURLColourOverride = true;
			break;

		default:
			return false;
		}
	}

	return true;
}
