/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include <atlconv.h>
#include "XFTree.h"

std::string XaraTagToString(ULONG Tag);

/******************************************************************************
   Function:    CXfDefineBitmap::CXfDefineBitmap
   Author:      Paul Gruzdev
   Purpose:     Construct a bitmap object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfDefineBitmap::CXfDefineBitmap(CXaraFileRecord & rec) : Name(rec), pProfileRefPtr(0), ProfileRef(0), m_cs(CS_Undefined)
{
	int iCMYKDiff = 0;
	if(GetTag() == TAG_DEFINEBITMAP_PNG_CMYK)
	{
		m_cs = CS_CMYK;
		ProfileRef.Read(&rec);
		iCMYKDiff = 4;
	}
	PBYTE pbyBuffer = rec.GetBuffer();
	ULONG ulSize = rec.GetSize();
	ULONG ulDiff = (ULONG)Name.getstr().length()*2+2 + iCMYKDiff;
	ulSize -= ulDiff;
	pbyBuffer += ulDiff;
	Data.resize(ulSize);
	memcpy(&Data[0], pbyBuffer, ulSize);
}


CXfDefineBitmap* CXfBitmap::GetReferenceBitmap(RefNodeMap& map)
{
	ULONG id = BitmapRef.get();
	if(map.find(id) != map.end())
		return (CXfDefineBitmap*)map[id];
	return NULL;
}

/******************************************************************************
   Function:    CXfDefineBitmap::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfDefineBitmap::CreateDumpString()
{
	USES_CONVERSION;
	char psz[256];
	sprintf(psz, "%s", W2A(GetBitmapName().c_str()));
	return psz;
}

/******************************************************************************
   Function:    CXfBitmap::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfBitmap::CreateDumpString()
{
	char psz[128];
	sprintf(psz, "[(%.3f %.3f) (%.3f %.3f) (%.3f %.3f) (%.3f %.3f)]", 
		(float)BottomLeft.get().x/1000.0, (float)BottomLeft.get().y/1000.0, 
		(float)BottomRight.get().x/1000.0, (float)BottomRight.get().y/1000.0, 
		(float)TopRight.get().x/1000.0, (float)TopRight.get().y/1000.0, 
		(float)TopLeft.get().x/1000.0, (float)TopLeft.get().y/1000.0);
	return psz;
}

/******************************************************************************
   Function:    CXfBitmapProperties::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfBitmapProperties::CreateDumpString()
{
	char psz[256];
	sprintf(psz, "Ref: %d, p1: %d p2: %d", BitmapRef, Prop1, Prop2);
	return psz;
}

/******************************************************************************
   Function:    CXfDefineBitmap::GetBitmapExtension
   Author:      Paul Gruzdev
   Purpose:		Conversion bitmap type ---> file extention
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::wstring CXfDefineBitmap::GetBitmapExtension()
{
	switch(GetTag())
	{
	case TAG_DEFINEBITMAP_PNG:
	case TAG_DEFINEBITMAP_PNG_REAL:
	case TAG_DEFINEBITMAP_PNG_CMYK:
		return std::wstring(L"png");
		break;
	case TAG_DEFINEBITMAP_JPEG:
	case TAG_DEFINEBITMAP_JPEG8BPP:
		return std::wstring(L"jpg");
		break;
	case TAG_DEFINEBITMAP_BMP:
	case TAG_DEFINEBITMAP_BMPZIP:
		return std::wstring(L"bmp");
		break;
	case TAG_DEFINEBITMAP_GIF:
		return std::wstring(L"gif");
	}

	return L"";
}

/******************************************************************************
   Function:    CXfDefineBitmap::GetBitmapType
   Author:      Paul Gruzdev
   Purpose:		Return type of a bitmap: jpeg, png, bmp or gif
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfDefineBitmap::eType CXfDefineBitmap::GetBitmapType()
{
	switch(GetTag())
	{
	case TAG_DEFINEBITMAP_PNG:
	case TAG_DEFINEBITMAP_PNG_REAL:
	case TAG_DEFINEBITMAP_PNG_CMYK:
		return PNG;
		break;
	case TAG_DEFINEBITMAP_JPEG:
	case TAG_DEFINEBITMAP_JPEG8BPP:
		return JPEG;
		break;
	case TAG_DEFINEBITMAP_BMP:
	case TAG_DEFINEBITMAP_BMPZIP:
		return BMP;
		break;
	case TAG_DEFINEBITMAP_GIF:
		return GIF;
	}

	return Undefined;
}
