/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

/********************************************************************************************
>	class CXfDefineICC_CMYK

Author:		Denis Onishchenko
Created:	07/01/19
Purpose:	Represents TAG_DEFINEICC_CMYK
********************************************************************************************/
class CXfDefineICC_CMYK : public VXfTreeNode
{
public:

	CXfDefineICC_CMYK(CXaraFileRecord & rec)
	{
		PBYTE pbyBuffer = rec.GetBuffer();
		ULONG ulSize = rec.GetSize();
		Data.resize(ulSize);
		memcpy(&Data[0], pbyBuffer, ulSize);
	}

	CXfDefineICC_CMYK(LPVOID pBuffer, size_t size) : VXfTreeNode(TAG_DEFINEICC_CMYK)
	{
		if (pBuffer && size)
		{
			Data.resize(size);
			memcpy(&Data[0], pBuffer, size);
		}
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();

		BOOL ok = TRUE;

		if (Data.size()) ok = rec.WriteBuffer(&Data[0], (ULONG)Data.size());
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

public:
	std::vector<BYTE> Data;
};

/********************************************************************************************
>	class CXfDefineBitmap

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_DEFINEBITMAP_*
********************************************************************************************/
class CXfDefineBitmap : public VXfTreeNode
{
public:
	enum eType { Undefined, PNG, JPEG, BMP, GIF };
	enum eColourSpace { CS_Undefined, CS_RGB, CS_CMYK, CS_GREY };

	CXfDefineBitmap(CXaraFileRecord & rec);

	CXfDefineBitmap(ULONG uTag, LPVOID pBuffer, size_t size, const wchar_t* name) : VXfTreeNode(uTag), ProfileRef(0), 
		pProfileRefPtr(0), m_cs(CS_Undefined)
	{
		if (pBuffer && size)
		{
			Data.resize(size);
			memcpy(&Data[0], pBuffer, size);
		}
		Name = name;
	}

	void SetColourSpace( eColourSpace cs) { m_cs = cs; }
	eColourSpace GetColourSpace() { return m_cs; }
	void SetProfileRef(VXfTreeNode* pProfile) { pProfileRefPtr = pProfile; }
	void SetProfileRef(ULONG ulProfileRef) { ProfileRef = ulProfileRef; }
	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		BOOL ok = Name.Write(&rec);
		if(GetTag() == TAG_DEFINEBITMAP_PNG_CMYK)
		{
			if(pProfileRefPtr) ProfileRef = pProfileRefPtr->GetRef();
			if(ok) ok = ProfileRef.Write(&rec);
		}

		if(ok && GetTag() == TAG_DEFINEBITMAP_JPEG8BPP)
		{
			BYTE bPalSize = Palette.size() ? (BYTE)(Palette.size()/3 - 1) : 0;
			ok = ok && rec.WriteBYTE(bPalSize); // number of palette entries
			if(Palette.size())
				ok = ok && rec.WriteBuffer(&Palette[0], (ULONG)Palette.size());
		}

		if(ok && Data.size()) ok = rec.WriteBuffer(&Data[0], (ULONG)Data.size());
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	BOOL Save(const wchar_t* pszName)
	{
		FILE* out = NULL; _wfopen_s(&out, pszName, L"wb");
		if(out)
		{
			fwrite(&Data[0], Data.size(), 1, out);
			fclose(out);
			return TRUE;
		}
		return FALSE;
	}
	virtual std::string CreateDumpString();

	virtual bool IsRefNode() { return true; }
	std::wstring GetBitmapExtension();
	eType GetBitmapType();
	std::wstring GetBitmapName() { return Name.getstr(); }

public:
	CXfString Name;
	std::vector<BYTE> Data;
	std::vector<BYTE> Palette;
	eColourSpace m_cs;
	// CMYK bitmap must contain a reference to the profile definition record, it can be 0 though
	CXfULONG ProfileRef;
	VXfTreeNode* pProfileRefPtr;
};

/********************************************************************************************
>	class CXfBitmap

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_NODE_BITMAP
********************************************************************************************/
class CXfBitmap : public VXfRenderable
{
public:
	CXfBitmap( CXaraFileRecord & rec ) : VXfRenderable(TAG_NODE_BITMAP),
		TopLeft(rec), TopRight(rec), BottomRight(rec), BottomLeft(rec), BitmapRef(rec), pBitmapRefPtr(0)
	{
	}

	CXfBitmap(DocCoord& tl, DocCoord& tr, DocCoord& br, DocCoord& bl, VXfTreeNode* pBitmapRef) : VXfRenderable(TAG_NODE_BITMAP),
		TopLeft(tl), TopRight(tr), BottomRight(br), BottomLeft(bl), BitmapRef(0), pBitmapRefPtr(pBitmapRef)
	{
	}

	virtual bool IsNodeBitmap() {return true;}
	CXfDefineBitmap* GetReferenceBitmap(RefNodeMap& map);

	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		if(pBitmapRefPtr) BitmapRef = pBitmapRefPtr->GetRef();
		BOOL ok = TopLeft.Write(&rec);
		if(ok) ok = TopRight.Write(&rec);
		if(ok) ok = BottomRight.Write(&rec);
		if(ok) ok = BottomLeft.Write(&rec);
		if(ok) ok = BitmapRef.Write(&rec);
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	CXfDocCoord TopLeft;
	CXfDocCoord TopRight;
	CXfDocCoord BottomRight;
	CXfDocCoord BottomLeft;
	CXfLONG BitmapRef;
	VXfTreeNode* pBitmapRefPtr;
};

/********************************************************************************************
>	class CXfBitmapProperties

Author:		Paul Gruzdev
Created:	17/01/13
Purpose:	Represents TAG_BITMAP_PROPERTIES
********************************************************************************************/
class CXfBitmapProperties : public VXfTreeNode
{
public:
	CXfBitmapProperties(CXaraFileRecord & rec)
		: BitmapRef(rec), Prop1(rec), Prop2(rec)
	{
	}

	CXfBitmapProperties(VXfTreeNode* pBitmapRef, ULONG prop1 = 0, ULONG prop2 = 0) : VXfTreeNode(TAG_BITMAP_PROPERTIES),
		BitmapRef(0), pBitmapRefPtr(pBitmapRef), Prop1(prop1), Prop2(prop2)
	{
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		if(pBitmapRefPtr) BitmapRef = pBitmapRefPtr->GetRef();
		BOOL ok = BitmapRef.Write(&rec);
		if(ok) ok = Prop1.Write(&rec);
		if(ok) ok = Prop2.Write(&rec);
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfULONG BitmapRef;
	CXfULONG Prop1;
	CXfULONG Prop2;
	VXfTreeNode* pBitmapRefPtr;
};
