/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XFTree.h"

/******************************************************************************
   Function:    CXfLineColor::GetRGBColor
   Author:      Paul Gruzdev
   Purpose:     Find an RGB color by reference
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
COLORREF CXfLineColor::GetRGBColor(RefNodeMap& map)
{
	if(type == Black)
		return 0;
	else if(type == White)
		return 0x00FFFFFF;
	else if( type == None)
		return 0xFF000000;

	if(map.find(ColorRef) != map.end())
	{
		VXfTreeNode* pColor = map[ColorRef];
		if(pColor->GetTag() == TAG_DEFINERGBCOLOUR)
		{
			CXfSimpleColor* pSimple = (CXfSimpleColor*)pColor;
			return pSimple->Color;
		}
		else if(pColor->GetTag() == TAG_DEFINECOMPLEXCOLOUR)
		{
			CXfComplexColor* pComplex = (CXfComplexColor*)pColor;
			// TODO: here we have to convert or get somehow an RGB from a complex possibly CMYK definition
			return pComplex->Color;
		}
	}

	return 0xFF000000;
}

/******************************************************************************
   Function:    CXfFlatFill::GetRGBColor
   Author:      Paul Gruzdev
   Purpose:     Find an RGB color by reference
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
COLORREF CXfFlatFill::GetRGBColor(RefNodeMap& map)
{
	if(type == Black)
		return 0;
	else if(type == White)
		return 0x00FFFFFF;
	else if( type == None)
		return 0xFF000000;

	if(pReference || map.find(ColorRef) != map.end())
	{
		VXfTreeNode* pColor = pReference ? pReference : map[ColorRef];
		if(pColor->GetTag() == TAG_DEFINERGBCOLOUR)
		{
			CXfSimpleColor* pSimple = (CXfSimpleColor*)pColor;
			return pSimple->Color;
		}
		else if(pColor->GetTag() == TAG_DEFINECOMPLEXCOLOUR)
		{
			CXfComplexColor* pComplex = (CXfComplexColor*)pColor;
			// TODO: here we have to convert or get somehow an RGB from a complex possibly CMYK definition
			return pComplex->Color;
		}
	}

	return 0xFF000000;
}

/******************************************************************************
   Function:	CXfSimpleColor::Compare
   Author:		Paul Gruzdev
   Purpose:     Compare two colour objects *strict*
   Returns:		bool
   Parameters:	CXfSimpleColor * pOther  
   Notes:       
******************************************************************************/
bool CXfSimpleColor::Compare(CXfSimpleColor* pOther)
{
	return Color == pOther->Color;
}

/******************************************************************************
   Function:	CXfComplexColor::Compare
   Author:		Paul Gruzdev
   Purpose:     Compare two colour objects *strict*
   Returns:		bool
   Parameters:	CXfComplexColor * pOther  
   Notes:       
******************************************************************************/
bool CXfComplexColor::Compare(CXfComplexColor* pOther)
{
	if (ColourModel != pOther->ColourModel) return false;
	if (ColourType != pOther->ColourType) return false;

	return Desc1 == pOther->Desc1 && Desc2 == pOther->Desc2 && Desc3 == pOther->Desc3 && Desc4 == pOther->Desc4;
}
