/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

/********************************************************************************************
>	class CXfSimpleColor

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_DEFINERGBCOLOUR
********************************************************************************************/
class CXfSimpleColor : public VXfTreeNode
{
public:
	CXfSimpleColor( CXaraFileRecord & rec ) 
	{
		BYTE r, g, b;
		rec.ReadBYTE( &r );
		rec.ReadBYTE( &g );
		rec.ReadBYTE( &b );
		Color = RGB( r, g, b );
	}

	CXfSimpleColor(COLORREF cr) : Color(cr) { SetTag(TAG_DEFINERGBCOLOUR); }
	CXfSimpleColor(BYTE r, BYTE g, BYTE b) : Color(RGB(r, g, b)) { SetTag(TAG_DEFINERGBCOLOUR); }
	CXfSimpleColor(const CXfSimpleColor& s) : VXfTreeNode(TAG_DEFINERGBCOLOUR), Color(s.Color) {}
	virtual bool IsRefNode() { return true; }
	virtual bool IsComplex() { return false; }
	virtual VXfTreeNode* Copy() { return new CXfSimpleColor(*this); }
	bool Compare(CXfSimpleColor* pOther);

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_DEFINERGBCOLOUR);
		Rec.Init();
		Rec.WriteBYTE(GetRValue(Color));
		Rec.WriteBYTE(GetGValue(Color));
		Rec.WriteBYTE(GetBValue(Color));
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	COLORREF Color;
};

/********************************************************************************************
>	class CXfComplexColor

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_DEFINECOMPLEXCOLOR
********************************************************************************************/
class CXfComplexColor : public CXfSimpleColor
{
public:
	CXfComplexColor(CXaraFileRecord & rec) 
		: CXfSimpleColor( rec ),
		  ColourModel( rec ),
		  ColourType( rec ),
		  EntryIndex( rec ),
		  ParentColour( rec ),
		  Desc1( rec ),
		  Desc2( rec ),
		  Desc3( rec ),
		  Desc4( rec ),
		  ColourName( rec )
	{
	}
	// Colour model
	enum { cmRGB = 2, cmCMYK = 3, cmHSV = 4, cmGreyscale = 5 };
	// Colour type
	enum { ctNormal = 0, ctSpot = 1, ctTint = 2, ctLinked = 3, ctShade = 4 };

	CXfComplexColor(COLORREF crSimple, BYTE colType, double c1, double c2, double c3, double c4) 
		: CXfSimpleColor(crSimple), ColourModel(colType), ColourType(ctNormal), EntryIndex(0), ParentColour(0),
		Desc1(c1), Desc2(c2), Desc3(c3), Desc4(c4)
	{
		SetTag(TAG_DEFINECOMPLEXCOLOUR);
	}
/*
	CXfComplexColor(COLORREF crSimple, ULONG C, ULONG M, ULONG Y, ULONG K) 
		: CXfSimpleColor(crSimple), ColourModel(cmCMYK), ColourType(ctNormal), EntryIndex(0), ParentColour(0),
		Desc1(C), Desc2(M), Desc3(Y), Desc4(K)
	{

	}
*/
	CXfComplexColor(COLORREF crSimple, double H, double S, double V) 
		: CXfSimpleColor(crSimple), ColourModel(cmHSV), ColourType(ctNormal), EntryIndex(0), ParentColour(0),
		Desc1(H), Desc2(S), Desc3(V), Desc4(0)
	{
		SetTag(TAG_DEFINECOMPLEXCOLOUR);
	}

	CXfComplexColor(COLORREF crSimple, double dGrey) 
		: CXfSimpleColor(crSimple), ColourModel(cmGreyscale), ColourType(ctNormal), EntryIndex(0), ParentColour(0),
		Desc1(dGrey), Desc2(0), Desc3(0), Desc4(0)
	{
		SetTag(TAG_DEFINECOMPLEXCOLOUR);
	}

	CXfComplexColor(const CXfComplexColor& s) : CXfSimpleColor(s.Color), ColourModel(s.ColourModel), ColourType(s.ColourType),
			EntryIndex(s.EntryIndex), ParentColour(s.ParentColour), Desc1(s.Desc1), Desc2(s.Desc2), Desc3(s.Desc3), 
			Desc4(s.Desc4), ColourName(s.ColourName) { SetTag(TAG_DEFINECOMPLEXCOLOUR); }


	void SetName(wchar_t* psz) { ColourName.getstr() = psz; }
	void SetParent(LONG lParent) { ParentColour = lParent; }
	virtual VXfTreeNode* Copy() { return new CXfComplexColor(*this); }
	virtual bool IsRefNode() { return true; }
	virtual bool IsComplex() { return true; }
	std::string CreateDumpString();
	bool Compare(CXfComplexColor* pOther);

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		// First of all write a simple colour
		BOOL ok = Rec.WriteBYTE(GetRValue(Color));
		ok = ok && Rec.WriteBYTE(GetGValue(Color));
		ok = ok && Rec.WriteBYTE(GetBValue(Color));
		// Now do all the complex stuff
		ColourModel.Write(&Rec);
		ok = ok && ColourType.Write(&Rec);
		ok = ok && EntryIndex.Write(&Rec);
		ok = ok && ParentColour.Write(&Rec);
		ok = ok && Desc1.Write(&Rec);
		ok = ok && Desc2.Write(&Rec);
		ok = ok && Desc3.Write(&Rec);
		ok = ok && Desc4.Write(&Rec);
		ok = ok && ColourName.Write(&Rec);
		return ok && pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfBYTE ColourModel;
	CXfBYTE ColourType;
	CXfULONG EntryIndex;
	CXfLONG ParentColour;
	CXfFIXED24 Desc1, Desc2, Desc3, Desc4;
	CXfString ColourName;
};
