/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "common/StringUtils.h"
#include "XFTree.h"

#ifdef _DEBUG
void ValidateTransparencyType(BYTE t)
{
	switch(t)
	{
	case CXF_TRANSPARANCY_TYPE_NONE:
	case CXF_TRANSPARANCY_TYPE_MIX:
	case CXF_TRANSPARANCY_TYPE_STAINEDGLASS:
	case CXF_TRANSPARANCY_TYPE_BLEACH:
	case CXF_TRANSPARENCY_TYPE_CONTRAST:
	case CXF_TRANSPARENCY_TYPE_SATURATION:
	case CXF_TRANSPARENCY_TYPE_DARKEN:
	case CXF_TRANSPARENCY_TYPE_LIGHTEN:
	case CXF_TRANSPARENCY_TYPE_BRIGHTNESS:
	case CXF_TRANSPARENCY_TYPE_LUMINOSITY:
	case CXF_TRANSPARENCY_TYPE_OVERLAY:
	case CXF_TRANSPARENCY_TYPE_HARDLIGHT:
	case CXF_TRANSPARENCY_TYPE_SOFTLIGHT:
	case CXF_TRANSPARENCY_TYPE_LUMINOSITY2:
	case CXF_TRANSPARENCY_TYPE_COLOUR:
	case CXF_TRANSPARENCY_TYPE_ENHANCE:
	case CXF_TRANSPARENCY_TYPE_BEVEL:
		break;
	default:
		assert(false);
	}
}
#endif

/******************************************************************************
   Function:    CXfString::CXfString
   Author:      Paul Gruzdev
   Purpose:     CXfString constructor
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfString::CXfString(CXaraFileRecord & rec)
{
	Read(rec);
}			

void CXfString::Read(CXaraFileRecord& rec)
{
	Value.resize(rec.GetSize()/2 + 1);
	rec.ReadWideString(&Value[0], (ULONG)Value.length());
	Value.resize(wcslen(&Value[0]));
}

CXfStringA::CXfStringA(CXaraFileRecord & rec)
{
	Read(rec);
}			

void CXfStringA::Read(CXaraFileRecord& rec)
{
	Value.resize(rec.GetSize() + 1);
	rec.ReadBuffer((PBYTE)&Value[0], (ULONG)Value.length());
	Value.resize(strlen(&Value[0]));
}

/******************************************************************************
   Function:    CXfPath::CXfPath
   Author:      Paul Gruzdev
   Purpose:     Construct a path object out of Xara path tag
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfPath::CXfPath(CXaraFileRecord& rec, bool f, bool s) : bFilled(f), bStroked(s)
{
	unsigned PtNum;
	switch( rec.GetTag() )
	{
	case TAG_PATH:
	case TAG_PATH_FILLED:
	case TAG_PATH_STROKED:
	case TAG_PATH_FILLED_STROKED:
	case TAG_CLIPVIEW_PATH:
		{
		CXfULONG num( rec );
		PtNum = num;
		Verbs.resize( PtNum );
		Coords.resize( PtNum );
		{
		for( ULONG i = 0; i < PtNum; ++i )
			rec.ReadBYTE( &Verbs[i] );
		}
		for( ULONG i = 0; i < PtNum; ++i )
			rec.ReadCoord( &Coords[i] );
		break;
		}
	case TAG_PATH_RELATIVE:
	case TAG_PATH_RELATIVE_FILLED:
	case TAG_PATH_RELATIVE_STROKED:
	case TAG_PATH_RELATIVE_FILLED_STROKED:
		{
		assert( rec.GetSize() % 9 == 0 );
		PtNum = rec.GetSize() / 9;
		Verbs.resize( PtNum );
		Coords.resize( PtNum );
		for( ULONG i = 0; i < PtNum; ++i )
		{
			rec.ReadBYTE( &Verbs[i] );
			rec.ReadCoordInterleaved( &Coords[i] );
			if ( i != 0 )
			{
				Coords[i].x = Coords[i-1].x - Coords[i].x;
				Coords[i].y = Coords[i-1].y - Coords[i].y;
			}
		}
		break;
		}
	default:
		assert( false );
	}
}

CXfPath::CXfPath(const CXfPath& other) :  VXfRenderable(other.GetTag()), bFilled(other.bFilled), bStroked(other.bStroked)
{
	CopyPoints(&other);
}

/******************************************************************************
   Function:    CXfPath::CXfPath
   Author:      Paul Gruzdev
   Purpose:     Construct a path object for the import
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfPath::CXfPath(UINT uTag, PBYTE pVerbs, DocCoord* pCoords, DWORD dwCount) : bFilled(false), bStroked(false)
{
	SetTag(uTag);

	switch(uTag)
	{
	case TAG_PATH_FILLED:
	case TAG_PATH_RELATIVE_FILLED:
		bFilled = true;
		break;
	case TAG_PATH_STROKED:
	case TAG_PATH_RELATIVE_STROKED:
		bStroked = true;
		break;
	case TAG_PATH_FILLED_STROKED:
	case TAG_PATH_RELATIVE_FILLED_STROKED:
		bFilled = true;
		bStroked = true;
		break;
	}

	if(pVerbs && pCoords && dwCount)
	{
		Verbs.resize(dwCount);
		Coords.resize(dwCount);
		memcpy(&Verbs[0], pVerbs, dwCount);
		memcpy(&Coords[0], pCoords, dwCount*sizeof(DocCoord));
	}
}

/******************************************************************************
   Function:    CXfPath::TestIsStroked
   Author:      Paul Gruzdev
   Purpose:     test the path object based on a context whether it is actually stroked
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfPath::TestIsStroked()
{
	return Context().m_pLineColor && Context().m_pLineColor->type != CXfLineColor::None && Context().m_dwLineWidth > 0;
}

/******************************************************************************
   Function:    CXfPath::TestIsFilled
   Author:      Paul Gruzdev
   Purpose:     test the path object based on a context whether it is actually filled
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfPath::TestIsFilled()
{
	bool bFlatFillNone = Context().m_pFill && Context().m_pFill->IsFlatFill() && 
		((CXfFlatFill*)Context().m_pFill)->type == CXfFlatFill::None;

	return Context().m_pFill && !bFlatFillNone;
}

/******************************************************************************
   Function:    CXfPath::Write
   Author:      Paul Gruzdev
   Purpose:     Write path to xar stream
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
BOOL CXfPath::Write(CXarExport* pExporter, ULONG* pRecordNum)
{
	// Now pass the path to the exporter
	CXaraFileRecord Rec(GetTag());
	Rec.Init();
	Rec.Reinit(GetTag(), (LONG)(sizeof(UINT32)+Verbs.size()+Coords.size()*sizeof(DocCoord)));
	Rec.WriteULONG((ULONG)Verbs.size());
	for(size_t i = 0; i < Verbs.size(); i++)
		Rec.WriteBYTE(Verbs[i]);
	for(size_t i = 0; i < Coords.size(); i++)
		Rec.WriteCoord(Coords[i]);
	ULONG ulPathRef;
	return pExporter->WriteRecord(&Rec, &ulPathRef);
}

/******************************************************************************
   Function:    CXfPath::IsSimpleRectangle
   Author:      Paul Gruzdev
   Purpose:     Check whether this path is actually a simple rectangle
				by inspecting it's lines
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfPath::IsSimpleRectangle()
{
	if(Verbs.empty() || Coords.empty())
		return false;

	if(Verbs.size() != 4 && Verbs.size() != 5) // m l l l(c) or m l l l l(with probable close)
		return false;
	if(Verbs[0] != PT_MOVETO)
		return false;
	DocCoord dcStart = Coords[0];
	DocCoord dcPrev = dcStart;
	int iCloseCount = 0;
	for(size_t i = 1; i < Verbs.size(); i++)
	{
		if(!(Verbs[i] & PT_LINETO)) // check for bezier or a compound path
			return false;
		if(Verbs[i] & PT_CLOSEFIGURE) // check for bezier or a compound path
			iCloseCount++;

		DocCoord dc = Coords[i];

		if(dc.x - dcPrev.x != 0 && dc.y - dcPrev.y != 0)
			return false; // We have the rule that each point at least lies on a same X or Y plate
		dcPrev = dc;
	}

	if(iCloseCount > 1)
		return false;
	return dcPrev.x == dcStart.x && dcPrev.y == dcStart.y;
}

/******************************************************************************
Function:    CXfPath::IsSimpleOrComplexRectangle
Author:      Ekaterina Dolguicheva
Purpose:     Check whether this path is actually a simple or complex rectangle
by inspecting it's lines
Returns:
Exceptions:
Parameters:
Notes:
******************************************************************************/
bool CXfPath::IsSimpleOrComplexRectangle()
{
	if (Verbs.empty() || Coords.empty())
		return false;

	if (Verbs.size() != 4 && Verbs.size() != 5) // m l l l(c) or m l l l l(with probable close)
		return false;
	if (Verbs[0] != PT_MOVETO)
		return false;
	DocCoord dcStart = Coords[0];
	DocCoord dcPrev = dcStart;
	int iCloseCount = 0;

	int x_diff1 = Coords[0].x - Coords[1].x;
	int x_diff2 = Coords[1].x - Coords[2].x;
	int y_diff1 = Coords[0].y - Coords[1].y;
	int y_diff2 = Coords[1].y - Coords[2].y;

	for (size_t i = 1; i < Verbs.size(); i++)
	{
		if (!(Verbs[i] & PT_LINETO)) // check for bezier or a compound path
			return false;
		if (Verbs[i] & PT_CLOSEFIGURE) // check for bezier or a compound path
			iCloseCount++;

		DocCoord dc = Coords[i];
		if (i == 3){ // check and compare with the opposite side
			if (!(dc.x - dcPrev.x == x_diff1 && dc.y - dcPrev.y == y_diff1)){
				return false;
			} 
		}
		else if (i == 4){
			if (!(dc.x - dcPrev.x == x_diff2 && dc.y - dcPrev.y == y_diff2)){
				return false;
			}
		}
		
		dcPrev = dc;
	}

	if (iCloseCount > 1)
		return false;
	return dcPrev.x == dcStart.x && dcPrev.y == dcStart.y;
}

/******************************************************************************
   Function:    CXfPath::CopyPoints
   Author:      Paul Gruzdev
   Purpose:     Copy points from the source path
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void CXfPath::CopyPoints(PBYTE pVerbs, DocCoord* pCoords, ULONG ulNumCoords)
{
	Verbs.resize(ulNumCoords);
	Coords.resize(ulNumCoords);
	memcpy(&Verbs[0], pVerbs, ulNumCoords);
	memcpy(&Coords[0], pCoords, ulNumCoords*sizeof(DocCoord));
}

void CXfPath::CopyPoints(const CXfPath* pSrc)
{
	Verbs.clear();
	Coords.clear();
	for(size_t i = 0; i < pSrc->Verbs.size(); i++)
	{
		Verbs.push_back(pSrc->Verbs[i]);
		Coords.push_back(pSrc->Coords[i]);
	}
}

/******************************************************************************
   Function:    CXfPath::IsEqualTo
   Author:      Paul Gruzdev
   Purpose:     Compare two paths
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfPath::IsEqualTo(CXfPath* pOther)
{
	if(Verbs.size() != pOther->Verbs.size() || Coords.size() != pOther->Coords.size())
		return false;

	for(size_t i = 0; i < Verbs.size(); i++)
	{
		if(Verbs[i] != pOther->Verbs[i])
			return false;
		if(Coords[i].x != pOther->Coords[i].x || Coords[i].y != pOther->Coords[i].y)
			return false;
	}
	return true;
}

/******************************************************************************
   Function:    CXfTextRuler::CXfTextRuler
   Author:      Paul Gruzdev
   Purpose:     Construct a text ruler object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfTextRuler::CXfTextRuler(CXaraFileRecord& rec)
{
	UINT16 count;
	rec.ReadUINT16(&count);
	Stops.resize(count);
	for(size_t i = 0; i < count; ++i)
	{
		rec.ReadBYTE(&Stops[i].TypeAndFlags);
		rec.ReadLONG(&Stops[i].Position);
		
		if((Stops[i].TypeAndFlags & 3) == DecimalTab)
			rec.ReadUINT16(&Stops[i].DecimalChar);

		if(Stops[i].TypeAndFlags & 4)
			rec.ReadUINT16(&Stops[i].FillerChar);
	}
}

/******************************************************************************
   Function:    CXfFillMultiStageColors::CXfFillMultiStageColors
   Author:      Paul Gruzdev
   Purpose:     Constructor
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfFillMultiStageColors::CXfFillMultiStageColors( CXaraFileRecord & rec )
	: StartColorRef( rec ), EndColorRef( rec ), pStartColor(0), pEndColor(0)
{
	CXfULONG NumCols( rec );
	Positions.resize( NumCols );
	ColorRefs.resize( NumCols );

	for( unsigned i = 0; i < NumCols; ++i )
	{
		rec.ReadDOUBLE( &Positions[ i ] );
		rec.ReadLONG( &ColorRefs[ i ] );
	}
}

/******************************************************************************
   Function:    CXfTextAnchorPosition::CXfTextAnchorPosition
   Author:      Paul Gruzdev
   Purpose:		CXfTextAnchorPosition construct
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfTextAnchorPosition::CXfTextAnchorPosition(CXaraFileRecord& rec)
{
	size_t iCount = rec.GetSize()/sizeof(ULONG);

	for(size_t i = 0; i < iCount; i++)
	{
		ULONG ulRef;
		rec.ReadULONG(&ulRef);
		if(ulRef != TEXTSTORY_IMPORT_TAG_NOT_IN_DOC)
			m_vAnchoredGroupID.push_back(ulRef);
	}
}

CXfTextEmbeddedAnchorPosInfo::CXfTextEmbeddedAnchorPosInfo(CXaraFileRecord& rec) :
			VerticalOffset(rec),
			LeftMargin(rec),
			RightMargin(rec),
			TopMargin(rec),
			BottomMargin(rec),
			CharWidth(rec),
			FontAscent(rec),
			FontDescent(rec)
{

	// cater for records written before the NoDescender flag was introduced (development versions only):
	// only read the bNoDescender field if the record is big enough
	if (rec.GetSize() >= TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO_SIZE)
		bNoDescender.Read(&rec);
}

#define ADJUST_RECT_XY(x,y) { if(x<x1) x1=x; if(x>x2) x2=x; if(y<y1) y1=y; if(y>y2) y2=y; }
/******************************************************************************
   Function:    CXfFontGlyph::CXfFontGlyph
   Author:      Paul Gruzdev
   Purpose:     Construct a font glyph object out of Xara tag
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfFontGlyph::CXfFontGlyph(CXaraFileRecord& rec) : 	Flags(rec),
			CharCode(rec),
			Width(rec),
			FontAscent(rec),
			FontDescent(rec),
			FontEMWidth(rec)
{
	ULONG PtNum;
	BOOL ok = rec.ReadULONG(&PtNum);
	Verbs.resize(PtNum);
	Points.resize(PtNum);
	int x1 = INT_MAX, y1 = INT_MAX, x2 = INT_MIN, y2 = INT_MIN;

	for(ULONG i = 0; i < PtNum; ++i)
	{
		rec.ReadCoord(&Points[i]);
		ADJUST_RECT_XY(Points[i].x, Points[i].y);
	}

	if(PtNum)
		SetBounds(Rect(x1, y1, x2-x1, y2-y1));

	for(ULONG i = 0; i < PtNum; ++i)
		rec.ReadBYTE(&Verbs[i]);

	if(Flags & FG_HAS_INTERNAL_LEADING)
		FontInternalLeading.Read(&rec);

	if(Flags & FG_HAS_EXTERNAL_LEADING)
		FontExternalLeading.Read(&rec);
}

/******************************************************************************
   Function:    CXfFontGlyph::AddPoint
   Author:      Paul Gruzdev
   Purpose:     Add a glyph path point
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
void CXfFontGlyph::AddPoint(BYTE verb, DocCoord coord)
{
	Verbs.push_back(verb);
	Points.push_back(coord);
}

std::string CXfFontGlyph::Dump()
{
	char psz[128];
	sprintf(psz, "pts:%d\n", Verbs.size());
	std::string s = psz;

	char psz1[128];
	for(size_t i = 0; i < Verbs.size(); i++)
	{
		BYTE byVerb = Verbs[i] & 0xFE;
		char cv = byVerb == PT_MOVETO ? 'm' : (byVerb == PT_LINETO ? 'l' : 'c');
		sprintf(psz1, "(%d,%d)%c", Points[i].x, Points[i].y, cv);
		s += psz1;
		if(Verbs[i] & PT_CLOSEFIGURE) s += "/cp";
		s += "\n";
	}
	return s;
}
/******************************************************************************
   Function:    CXfFontGlyph::Write
   Author:      Paul Gruzdev
   Purpose:     Write the glyph to Xar stream
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
BOOL CXfFontGlyph::Write(CXarExport* pExporter, ULONG* pulTagId)
{
	CXaraFileRecord rec(GetTag());
	rec.Init();
	BOOL ok = Flags.Write(&rec);
	if(ok) ok = CharCode.Write(&rec);
	if(ok) ok = Width.Write(&rec);
	if(ok) ok = FontAscent.Write(&rec);
	if(ok) ok = FontDescent.Write(&rec);
	if(ok) ok = FontEMWidth.Write(&rec);

	assert(Verbs.size() == Points.size());
	if(ok) ok = rec.WriteULONG((ULONG)Verbs.size());

	if(Verbs.size())
	{
		//for(size_t i = 0; i < Points.size(); ++i)
		//if(ok) ok = rec.WriteCoord(Points[i]);
		if(ok) ok = rec.WriteBuffer((BYTE*)&Points[0], (ULONG)Points.size()*2*sizeof(LONG));
		//for(size_t i = 0; i < Verbs.size(); ++i)
		//if(ok) ok = rec.WriteBYTE(Verbs[i]);
		if(ok) ok = rec.WriteBuffer(&Verbs[0], (ULONG)Verbs.size());
	}

	if(Flags & FG_HAS_INTERNAL_LEADING)
		if(ok) ok = FontInternalLeading.Write(&rec);

	if(Flags & FG_HAS_EXTERNAL_LEADING)
		if(ok) ok = FontExternalLeading.Write(&rec);

	return ok && pExporter->WriteRecord(&rec, pulTagId);
}

/******************************************************************************
   Function:    CXfFontKerningPairs::CXfFontKerningPairs
   Author:      Paul Gruzdev
   Purpose:     CXfFontKerningPairs construct
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfFontKerningPairs::CXfFontKerningPairs(CXaraFileRecord& rec) : Flags(rec)
{
	ULONG ulNum;
	BOOL ok = rec.ReadULONG(&ulNum);
	//Pairs.resize(ulNum);
	for(size_t i = 0; i < ulNum; ++i)
		Pairs.push_back(KerningPair(rec));
}

XarLigatureEntry::XarLigatureEntry(CXaraFileRecord& rec)
{
	if (rec.GetTag() == TAG_FONT_OTLIGATURES_EXT)
		rec.ReadBYTE(&KeyLth);
	else
	{
		UINT16 var;
		rec.ReadUINT16(&var);
		KeyLth = (BYTE)var;
	}

//	assert(KeyLth <= LIGATURE_MAX_CHARS);

	Key.resize(KeyLth);

	for (UINT16 i = 0; i < KeyLth; i++)
		rec.ReadWCHAR(&Key[i]);

	rec.ReadWCHAR(&Ligature);

	if (rec.GetTag() == TAG_FONT_OTLIGATURES_EXT)
	{
		BYTE bt;
		rec.ReadBYTE(&bt);
		LigType = (LigatureSubsetFlag)bt;

		rec.ReadUINT16(&Priority);
		rec.ReadULONG(&GlyphID);
	}
	else
	{
		UINT16 var;
		rec.ReadUINT16(&var);
		LigType = (LigatureSubsetFlag)var;
	}
}

CXfFontOTLigatures::CXfFontOTLigatures(CXaraFileRecord& rec) : VXfTreeNode(rec.GetTag())
{
	KerningFlags.Read(&rec);
	ULONG cnt;
	rec.ReadULONG(&cnt);
	ligs.reserve(cnt);
	for (size_t i = 0; i < cnt; i++)
		ligs.push_back(XarLigatureEntry(rec));
}

CXfFontOTStylisticSets::CXfFontOTStylisticSets(CXaraFileRecord& rec) : VXfTreeNode(rec.GetTag())
{
	BYTE uNonEmptyStylisticSets = 0;

	BOOL ok = FontIdentificationFlags.Read(&rec);
	if (ok) ok = rec.ReadBYTE(&uNonEmptyStylisticSets);

	for (BYTE i = 0; i < uNonEmptyStylisticSets; ++i)
	{
		BYTE uStyleSetIndex = 0;
		if (ok) ok = rec.ReadBYTE(&uStyleSetIndex);

		UINT16 uStyleSetSize = 0;
		if (ok) ok = rec.ReadUINT16(&uStyleSetSize);

		for (UINT16 j = 0; j < uStyleSetSize; ++j)
		{
			WCHAR wKey = 0;
			WCHAR wOut = 0;
			DWORD wOutGlyph = 0;

			if (ok) ok = rec.ReadWCHAR(&wKey);
			if (ok) ok = rec.ReadWCHAR(&wOut);
			if (ok) ok = rec.ReadULONG(&wOutGlyph);

			StylisticSetEntry SSetEntry;
			SSetEntry.Key = wKey;
			SSetEntry.StylisticChar = wOut;
			SSetEntry.SubGlyphID = wOutGlyph;

			arrStylisticSets[uStyleSetIndex].push_back(SSetEntry);
		}
	}
}

CXfFontOTStylisticSetNames::CXfFontOTStylisticSetNames(CXaraFileRecord& rec) : VXfTreeNode(rec.GetTag())
{
	BYTE uNonEmptyStylisticSets = 0;

	BOOL ok = Flags.Read(&rec);
	if (ok) ok = rec.ReadBYTE(&uNonEmptyStylisticSets);

	for (BYTE i = 0; i < uNonEmptyStylisticSets; ++i)
	{
		BYTE uStyleSetIndex = 0;
		if (ok) ok = rec.ReadBYTE(&uStyleSetIndex);

		UINT16 uStyleSetSize = 0;
		if (ok) ok = rec.ReadUINT16(&uStyleSetSize);

		for (UINT16 j = 0; j < uStyleSetSize; ++j)
		{
			LANGID LCID = 0;
			if (ok) ok = rec.ReadUINT16(&LCID);

			BYTE StrSize = 0;
			if (ok) ok = rec.ReadBYTE(&StrSize);

			std::string Str;

			for (BYTE c = 0; c < StrSize; ++c)
			{
				WCHAR chr = 0;
				if (ok) ok = rec.ReadWCHAR(&chr);
				Str.push_back((CHAR)chr);
			}

			arrStylisticSetNames[uStyleSetIndex][LCID] = Str;
		}
	}
}

CXfFontOTSubstitutionRules::CXfFontOTSubstitutionRules(CXaraFileRecord& rec) : VXfTreeNode(rec.GetTag())
{
	BOOL ok = Flags.Read(&rec);

	UINT8 SetSize = 0;
	if (ok) ok = rec.ReadBYTE(&SetSize);

	for (UINT8 SetInd = 0; SetInd < SetSize; ++SetInd)
	{
		UINT8 GlyphSetEnumVal = 0;
		if (ok) ok = rec.ReadBYTE(&GlyphSetEnumVal);
		GlyphSetEnum CurGlyphSet = (GlyphSetEnum)GlyphSetEnumVal;

		UINT16 RuleContainerSize = 0;
		if (ok) ok = rec.ReadUINT16(&RuleContainerSize);

		std::vector<SubstitutionRule> RulesToRead;

		for (UINT16 ContainerInd = 0; ContainerInd < RuleContainerSize; ++ContainerInd)
		{
			SubstitutionRule Rule;

			// prefix reading
			UINT16 PrefixContainerSize = 0;
			if (ok) ok = rec.ReadUINT16(&PrefixContainerSize);
			Rule.Pre.resize(PrefixContainerSize);

			for (UINT16 PrefixContainerInd = 0; PrefixContainerInd < PrefixContainerSize; ++PrefixContainerInd)
			{
				UINT16 PrefixSize = 0;
				if (ok) ok = rec.ReadUINT16(&PrefixSize);
				Rule.Pre[PrefixContainerInd].resize(PrefixSize);

				for (UINT16 PrefixInd = 0; PrefixInd < PrefixSize; ++PrefixInd)
				if (ok) ok = rec.ReadWCHAR(&Rule.Pre[PrefixContainerInd][PrefixInd]);
			}

			// key reading
			UINT16 KeyContainerSize = 0;
			if (ok) ok = rec.ReadUINT16(&KeyContainerSize);
			Rule.Keys.resize(KeyContainerSize);

			for (UINT16 KeyContainerInd = 0; KeyContainerInd < KeyContainerSize; ++KeyContainerInd)
			{
				UINT16 KeySize = 0;
				if (ok) ok = rec.ReadUINT16(&KeySize);
				Rule.Keys[KeyContainerInd].resize(KeySize);

				for (UINT16 KeyInd = 0; KeyInd < KeySize; ++KeyInd)
				if (ok) ok = rec.ReadWCHAR(&Rule.Keys[KeyContainerInd][KeyInd]);
			}

			// suffix reading
			UINT16 PostContainerSize = 0;
			if (ok) ok = rec.ReadUINT16(&PostContainerSize);
			Rule.Post.resize(PostContainerSize);

			for (UINT16 PostContainerInd = 0; PostContainerInd < PostContainerSize; ++PostContainerInd)
			{
				UINT16 PostSize = 0;
				if (ok) ok = rec.ReadUINT16(&PostSize);
				Rule.Post[PostContainerInd].resize(PostSize);

				for (UINT16 PostInd = 0; PostInd < PostSize; ++PostInd)
				if (ok) ok = rec.ReadWCHAR(&Rule.Post[PostContainerInd][PostInd]);
			}

			// table ID reading
			if (ok) ok = rec.ReadUINT16(&Rule.SubstitutionTableID);

			// priority reading
			if (ok) ok = rec.ReadUINT16(&Rule.Priority);

			// lang data reading
			UINT8 LangDataSize = 0;
			if (ok) ok = rec.ReadBYTE(&LangDataSize);
			Rule.Langs.resize(LangDataSize);

			for (UINT8 LangInd = 0; LangInd < LangDataSize; ++LangInd)
			{
				ULONG LangCode = 0;
				if (ok) ok = rec.ReadULONG(&LangCode);
				Rule.Langs[LangInd] = LangCode;
			}

			RulesToRead.push_back(Rule);
		}

		RuleMaxSizes RuleSetMaxSizesToRead;
		if (ok) ok = rec.ReadBYTE(&RuleSetMaxSizesToRead.Pre);
		if (ok) ok = rec.ReadBYTE(&RuleSetMaxSizesToRead.Key);
		if (ok) ok = rec.ReadBYTE(&RuleSetMaxSizesToRead.Post);

		RuleSet.insert(std::make_pair(CurGlyphSet, std::make_pair(RulesToRead, RuleSetMaxSizesToRead)));
	}
}

CXfFontOTSubstitutionTables::CXfFontOTSubstitutionTables(CXaraFileRecord& rec) : VXfTreeNode(rec.GetTag())
{
	BOOL ok = Flags.Read(&rec);

	UINT16 SetSize = 0;
	if (ok) ok = rec.ReadUINT16(&SetSize);

	for (UINT16 SetInd = 0; SetInd < SetSize; ++SetInd)
	{
		SubstitutionTable SubTableToRead;

		// UID reading
		UINT16 SubTableUID = 0;
		if (ok) ok = rec.ReadUINT16(&SubTableUID);
		SubTableToRead.UID = SubTableUID;

		// Substitution Type reading
		BYTE SubTypeVal = 0;
		if (ok) ok = rec.ReadBYTE(&SubTypeVal);
		SubTableToRead.SubstitutionType = (GlyphSubstitutionType)SubTypeVal;

		// Substitution Entries reading
		UINT16 EntriesSize = 0;
		if (ok) ok = rec.ReadUINT16(&EntriesSize);

		for (UINT16 EntryInd = 0; EntryInd < EntriesSize; ++EntryInd)
		{
			std::wstring vecKeys;
			BYTE KeySize = 0;
			if (ok) ok = rec.ReadBYTE(&KeySize);
			vecKeys.resize(KeySize);

			for (BYTE KeyInd = 0; KeyInd < KeySize; ++KeyInd)
			if (ok) ok = rec.ReadWCHAR(&vecKeys[KeyInd]);

			std::vector<std::pair<WCHAR, DWORD>> vecVal;
			BYTE ValSize = 0;
			if (ok) ok = rec.ReadBYTE(&ValSize);
			vecVal.resize(ValSize);

			for (BYTE ValInd = 0; ValInd < ValSize; ++ValInd)
			{
				if (ok) ok = rec.ReadWCHAR(&vecVal[ValInd].first);
				if (ok) ok = rec.ReadULONG(&vecVal[ValInd].second);
			}

			SubTableToRead.SubstitutionMap.insert(std::make_pair(vecKeys, vecVal));
		}

		TableSet.insert(std::make_pair(SubTableUID, SubTableToRead));
	}

}

CXfFontOTRuleMaxSizes::CXfFontOTRuleMaxSizes(CXaraFileRecord& rec)
{
	BOOL ok = Flags.Read(&rec);

	if (ok) ok = rec.ReadBYTE(&MaxSizes.Pre);
	if (ok) ok = rec.ReadBYTE(&MaxSizes.Key);
	if (ok) ok = rec.ReadBYTE(&MaxSizes.Post);

}

/******************************************************************************
   Function:    CXfFontKerningPairs::Write
   Author:      Paul Gruzdev
   Purpose:     Write object data to Xar stream
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
BOOL CXfFontKerningPairs::Write(CXarExport* pExporter, ULONG* pulTagId)
{
	CXaraFileRecord rec(GetTag());
	rec.Init();
	BOOL ok = Flags.Write(&rec);
	ok = rec.WriteULONG((ULONG)Pairs.size());
	for(size_t i = 0; i < Pairs.size(); ++i)
	{
		if (ok) ok = Pairs[i].Write(&rec);
	}

	pExporter->WriteRecord(&rec, pulTagId);

	return ok;
}

/******************************************************************************
   Function:    VXfTreeNode::DeepCopy
   Author:      Paul Gruzdev
   Purpose:     Recursively copy current node and all of it's child nodes
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
VXfTreeNode* VXfTreeNode::DeepCopy()
{
	VXfTreeNode* pCopy = Copy();

	if(pCopy)
	{
		for(size_t i = 0; i < m_Childern.size(); i++)
		{
			VXfTreeNode* pChildCopy = m_Childern[i]->DeepCopy();
			if(pChildCopy) pCopy->AppendChild(pChildCopy);
		}
	}
	return pCopy;
}
