/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

#include "cxfunits.h"

//
// Application records
//

class CXfFileHeader : public VXfTreeNode
{
public:
	CXfFileHeader(const char* pszProducer, const char* pszProducerVersion, const char* pszBuild) : Producer(pszProducer), 
		ProducerVersion(pszProducerVersion), ProducerBuild(pszBuild), VXfTreeNode(TAG_FILEHEADER) {}

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum)
	{
		CXaraFileRecord Rec(TAG_FILEHEADER);
		BOOL ok = Rec.Init();
		if (ok) ok = Rec.WriteBuffer((BYTE*)"CXN",3);
		if (ok) ok = Rec.WriteULONG(123);				// File size
		if (ok) ok = Rec.WriteULONG(0);					// Native/Web link ID
		if (ok) ok = Rec.WriteULONG(0);					// Precompression flags
		
		if (ok) ok = Producer.Write(&Rec);
		if (ok) ok = ProducerVersion.Write(&Rec);
		if (ok) ok = ProducerBuild.Write(&Rec);				// Producer build
		if (ok) ok = pExporter->WriteRecord(&Rec);
		return ok;
	}

	CXfStringA Producer;
	CXfStringA ProducerVersion;
	CXfStringA ProducerBuild;
};

class CXfDocumentFlags : public CXfNodeValue
{
public:
	CXfDocumentFlags(CXaraFileRecord& rec) : CXfNodeValue(rec) {}
	CXfDocumentFlags(LONG flags) : CXfNodeValue(TAG_DOCUMENTFLAGS, flags) {}
};

class CXfViewport : public VXfTreeNode
{
public:
	CXfViewport( CXaraFileRecord & rec )
		: BottomLeft( rec ), TopRight( rec )
	{
	}
	CXfViewport(DocCoord bl, DocCoord tr) : VXfTreeNode(TAG_VIEWPORT), BottomLeft(bl), TopRight(tr) {}

	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum)
	{
		CXaraFileRecord Rec(TAG_VIEWPORT);
		Rec.Init();
		BOOL ok = Rec.WriteCoord(BottomLeft);
		ok = Rec.WriteCoord(TopRight);
		return pExporter->WriteRecord(&Rec);
	}

	const CXfDocCoord BottomLeft;
	const CXfDocCoord TopRight;
};

class CXfSpreadInformation : public VXfTreeNode
{
public:
	CXfSpreadInformation( CXaraFileRecord & rec )
		: Width( rec ), Height( rec ), Margin( rec ), 
		Bleed( rec ), SpreadFlags( rec )
	{
	}

	virtual std::string CreateDumpString();

	bool IsDoublePageSpread() { return (BYTE)SpreadFlags & 1 ? true : false; }
	bool IsDropShadow() { return (BYTE)SpreadFlags & 2 ? true : false; }		// Bit 0 = DPS, Bit 1 = DropShadow
	bool IsSelectedSpread() { return (BYTE)SpreadFlags & 4 ? true : false; }	// Bit 2 = Selected Spread
	bool IsPrintWholeSpread() { return (BYTE)SpreadFlags & 8 ? true : false; }	// Bit 3 = Print Whole Spread
	bool IsNegateX() { return (BYTE)SpreadFlags & 16 ? true : false; }			// Bit 4 = Negate X
	bool IsNegateY() { return (BYTE)SpreadFlags & 32 ? true : false; }			// Bit 5 = Negate Y
	bool IsAllowPageEdgeDrag() { return (BYTE)SpreadFlags & 64 ? false : true; }	// Bit 6 = ! Allow Page Edge Drag

	CXfSpreadInformation(LONG width, LONG height, LONG margin, LONG bleed, BYTE flags) : Width(width), Height(height), 
		Margin(margin), Bleed(bleed), SpreadFlags(flags), VXfTreeNode(TAG_SPREADINFORMATION) {}

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_SPREADINFORMATION);
		Rec.Init();
		BOOL ok = Width.Write(&Rec);
		ok = Height.Write(&Rec);
		ok = Margin.Write(&Rec);
		ok = Bleed.Write(&Rec);
		ok = SpreadFlags.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfLONG Width;
	CXfLONG Height;
	CXfLONG Margin;
	CXfLONG Bleed;
	CXfBYTE SpreadFlags;
};

class CXfWebsitePropertiesExport : public VXfTreeNode
{
public:
	CXfWebsitePropertiesExport(CXaraFileRecord & rec)
		: IsSinglePageSite(rec), PageSiteMode(rec), IsSiteScaleToFit(rec), IsSiteWithThumbnails(rec), IsSitePDFAlongside(rec)
	{
	}

	CXfBOOL IsSinglePageSite;
	CXfLONG PageSiteMode;
	CXfBOOL IsSiteScaleToFit;
	CXfBOOL IsSiteWithThumbnails;
	CXfBOOL IsSitePDFAlongside;
};

class CXfWebsiteTransition : public VXfTreeNode
{
public:
	CXfWebsiteTransition(CXaraFileRecord & rec)
		: TransitionEffect(rec), TransitionSpeed(rec){}

	CXfLONG TransitionEffect;
	CXfLONG TransitionSpeed;
};

class CXfPresentationPageProperties : public VXfTreeNode
{
public:
	CXfPresentationPageProperties(CXaraFileRecord & rec)
		: TransitionEffect(rec), TransitionSpeed(rec){}

	CXfLONG TransitionEffect;
	CXfLONG TransitionSpeed;
};

class CXfPresentationLayerProperties : public VXfTreeNode
{
public:
	CXfPresentationLayerProperties(CXaraFileRecord & rec)
		: AnimationEffect(rec), AnimationSpeed(rec){}

	CXfLONG AnimationEffect;
	CXfLONG AnimationSpeed;
};

class CXfImplicitTabDistance : public VXfTreeNode
{
public:
	CXfImplicitTabDistance( CXaraFileRecord & rec )
		: Distance( rec )
	{
	}

	CXfLONG Distance;
};


// We need layer to be renderable to store the most global attribute context here, for the import filters where the tree 
// is built just adding child nodes, not using the tree object itself.
class CXfLayer : public VXfRenderable
{
public:
	CXfLayer() : VXfRenderable(TAG_LAYER) {}
	CXfLayer(CXaraFileRecord& rec) {}
};

class CXfLayerDetails : public VXfTreeNode
{
public:
	enum { eIsVisible = 1, eIsLocked = 2, eIsPrintable = 4, eIsActive = 8 };
	CXfLayerDetails(CXaraFileRecord& rec) : Flags(rec), Name(rec) {}
	CXfLayerDetails(BYTE flags, const wchar_t* pszName) : Flags(flags), Name(pszName), VXfTreeNode(TAG_LAYERDETAILS) {}

	void SetVisible(bool b) { b ? Flags.get() |= eIsVisible : Flags.get() &= ~eIsVisible; }
	void SetLocked(bool b) { b ? Flags.get() |= eIsLocked : Flags.get() &= ~eIsLocked; }
	void SetPrintable(bool b) { b ? Flags.get() |= eIsPrintable : Flags.get() &= ~eIsPrintable; }
	void SetActive(bool b) { b ? Flags.get() |= eIsActive : Flags.get() &= ~eIsActive; }

	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_LAYERDETAILS);
		Rec.Init();
		BOOL ok = Flags.Write(&Rec);
		ok = Name.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfBYTE Flags;
	CXfString Name;
};


class CXfPresentationLayerFlag : public VXfTreeNode
{
public:
	CXfPresentationLayerFlag(CXaraFileRecord& rec) : Flag(rec) {}
	
	CXfBOOL Flag;
};

class CXfDocumentInformation : public VXfTreeNode
{
public:
	class ChapterInfo
	{
	public:
		ChapterInfo(ULONG uNumSpreads) : ChapterStructureFlags(0), NumSpreads(uNumSpreads) {}
		CXfUINT16 ChapterStructureFlags;		// All flags reserved, set to 0.
		CXfULONG NumSpreads;					// The number of spreads in this document
		BOOL Write(CXaraFileRecord* pRec)
		{
			BOOL ok = ChapterStructureFlags.Write(pRec);
			return ok && NumSpreads.Write(pRec);
		}
	};

	CXfDocumentInformation(ULONG uNumChapters) : VXfTreeNode(TAG_DOCUMENTINFORMATION), DocumentStructureFlags(0), NumChapters(uNumChapters) {}
	void AddChapterInfo(ULONG uNumSpreads) { m_Chapters.push_back(ChapterInfo(uNumSpreads)); NumChapters = (ULONG)m_Chapters.size(); }
	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_DOCUMENTINFORMATION);
		Rec.Init();
		BOOL ok = DocumentStructureFlags.Write(&Rec);
		ok = ok && NumChapters.Write(&Rec);

		for(size_t i = 0; i < m_Chapters.size(); i++)
			ok = ok && m_Chapters[i].Write(&Rec);

		return ok && pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfUINT16 DocumentStructureFlags;		// All flags reserved, set to 0.
	CXfULONG NumChapters;					// The number of chapters in this document
	std::vector<ChapterInfo> m_Chapters;
};

class CXfGridRulerSettings : public VXfTreeNode
{
public:
	CXfGridRulerSettings( CXaraFileRecord & rec )
		: gridUnits( rec ), gridDivisions( rec ), gridSubdivisions( rec ), gridType( rec )
	{
	}

	CXfGridRulerSettings(long _gridUnits, double _gridDivisions, ULONG _gridSubdivisions, BYTE _gridType)
		: VXfTreeNode(TAG_GRIDRULERSETTINGS)
		, gridUnits(_gridUnits), gridDivisions(_gridDivisions), gridSubdivisions(_gridSubdivisions), gridType(_gridType)
	{
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum)
	{
		CXaraFileRecord Rec(TAG_GRIDRULERSETTINGS, TAG_GRIDRULERSETTINGS_SIZE);
		Rec.Init();
		gridUnits       .Write(&Rec);
		gridDivisions   .Write(&Rec);
		gridSubdivisions.Write(&Rec);
		gridType        .Write(&Rec);
		return pExporter->WriteRecord(&Rec);
	}

	CXfLONG   gridUnits;
	CXfdouble gridDivisions;
	CXfULONG  gridSubdivisions;
	CXfBYTE   gridType;
};

class CXfGridRulerOrigin : public VXfTreeNode
{
public:
	CXfGridRulerOrigin( CXaraFileRecord & rec )
		: gridOrigin( rec )
	{
	}

	CXfGridRulerOrigin(DocCoord _gridOrigin)
		: VXfTreeNode(TAG_GRIDRULERORIGIN)
		, gridOrigin(_gridOrigin)
	{
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum)
	{
		CXaraFileRecord Rec(TAG_GRIDRULERORIGIN, TAG_GRIDRULERORIGIN_SIZE);
		Rec.Init();
		gridOrigin.Write(&Rec);
		return pExporter->WriteRecord(&Rec);
	}

	CXfDocCoord gridOrigin;
};

