/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include <atlconv.h>
#include <sstream>
#include "XFTree.h"
#include "../StringUtils.h"

#include "../XaraTagToString.h"

/******************************************************************************
   Function:    VXfTreeNode::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string VXfTreeNode::Dump(int iIndent)
{
	//char psz[4096];
	std::string str;
	for(int i = 0; i < iIndent*4; i++)
		str += " ";

	if(IsRemoved())
		str += "[REMOVED] ";

	std::string strDump = this->CreateDumpString();
	if(strDump.empty())
	{
		if(this->IsRefNode())
			strDump = FormatStringA("<%s %d>", XaraTagToString(GetTag()).c_str(), GetRef());
			//return FormatStringA("<%s %d>", XaraTagToString(GetTag()).c_str(), GetRef());
		else
			//return FormatStringA("<%s>", XaraTagToString(GetTag()).c_str());
			strDump = FormatStringA("<%s>", XaraTagToString(GetTag()).c_str());
		//strDump = psz;
	}
	else
	{
		std::string sTemp;
		if(this->IsRefNode())
		{
			//return FormatStringA("<%s %d, %s>", XaraTagToString(GetTag()).c_str(), GetRef(), strDump.c_str());
			//strDump = FormatStringA("<%s %d, %s>", XaraTagToString(GetTag()).c_str(), GetRef(), strDump.c_str());
			sTemp = FormatStringA("<%s %d, ", XaraTagToString(GetTag()).c_str(), GetRef());
			sTemp += strDump;
			sTemp += ">"; strDump = sTemp;
		}
		else
		{
			//return FormatStringA("<%s, %s>", XaraTagToString(GetTag()).c_str(), strDump.c_str());
			sTemp = FormatStringA("<%s, ", XaraTagToString(GetTag()).c_str());
			sTemp += strDump;
			sTemp += ">"; strDump = sTemp;
		}
		//strDump = psz;
	}

	str += strDump;

	if(this->IsBounded())
	{
		Rect b = ((VXfBounded*)this)->Bounds();
		//return FormatStringA(" bounds:[%d %d %d %d]", b.left(), b.top(), b.right(), b.bottom());
		strDump = FormatStringA(" bounds:[%d %d %d %d]", b.left(), b.top(), b.right(), b.bottom());
		str += strDump;
	}

	str += "\n";
/*
FILE* out = fopen("d:\\temp\\XaraTreeInteractive.txt", "at");
fprintf(out, "%s", str.c_str());
fclose(out);
*/
	for(size_t i = 0; i < m_Childern.size(); i++)
		str += m_Childern[i]->Dump(iIndent+1);

	return str;
}

/******************************************************************************
   Function:    CXfNodeValue::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfNodeValue::CreateDumpString()
{
	return FormatStringA("Value: %d", GetValue());
}

/******************************************************************************
   Function:    CXfFloatNode::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFloatNode::CreateDumpString()
{
	return FormatStringA("Value: %.3f", GetValue());
}

/******************************************************************************
   Function:    CXfTextStoryComplex::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextStoryComplex::CreateDumpString()
{
	FIXED16 abcd[4];
	LONG ef[2];
	Matrix m = Transform;
	m.GetComponents(abcd, ef);
	return FormatStringA("[%.2f %.2f %.2f %.2f] [%d %d]", abcd[0].MakeDouble(), abcd[1].MakeDouble(), abcd[2].MakeDouble(), 
		abcd[3].MakeDouble(), ef[0], ef[1]);
}

/******************************************************************************
   Function:    CXfExtrusionControllerComplex::Dump
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfExtrusionControllerComplex::CreateDumpString() 
{
	FIXED16 abcd[4];
	LONG ef[2];
	Matrix m = Transform;
	m.GetComponents(abcd, ef);
	return FormatStringA("[%.2f %.2f %.2f %.2f] [%d %d]", abcd[0].MakeDouble(), abcd[1].MakeDouble(), abcd[2].MakeDouble(), 
		abcd[3].MakeDouble(), ef[0], ef[1]);
};

/******************************************************************************
   Function:    CXfExtrusionControllerTrans::Dump
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfExtrusionControllerTrans::CreateDumpString() 
{
	return FormatStringA("angle:[%d]", Angle);
};

/******************************************************************************
   Function:    CXfTextLanguage::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextLanguage::CreateDumpString()
{
	ULONG ulLangCode = LangCode;
	ULONG ulCountryCode = CountryCode;
	return FormatStringA("lang: %s, country: %s", (char*)&ulLangCode, (char*)&ulCountryCode);
}

/******************************************************************************
   Function:    CXfTextString::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextString::CreateDumpString()
{
	USES_CONVERSION;
	std::string str;
	str += "str: ";
	str += W2A(m_str.c_str());
	
	std::ostringstream ss;
	ss << " // ";
	for (size_t i=0; i<m_str.length(); ++i)
		ss << std::hex << m_str[i] << ' ';
	str += ss.str();
	//strcat(psz, ss.str().c_str());
	
	return str;
}

std::string CXfTextStringPos::CreateDumpString()
{
	return FormatStringA("[%d %d] %s", Pos.get().x, Pos.get().y, CXfTextString::CreateDumpString().c_str());
}

std::string CXfTextStringPos2::CreateDumpString()
{
	return FormatStringA("[%d %d %d %d] %s", StartOffset->x, StartOffset->y, EndOffset->x, EndOffset->y,
		CXfTextString::CreateDumpString().c_str());
}

std::string CXfTextCharPos::CreateDumpString()
{
	return FormatStringA("[%d %d] %s", Pos.get().x, Pos.get().y, CXfTextString::CreateDumpString().c_str());
}

std::string CXfTextCharPos2::CreateDumpString()
{
	return FormatStringA("[%d %d %d %d] %s", StartOffset->x, StartOffset->y, EndOffset->x, EndOffset->y,
		CXfTextString::CreateDumpString().c_str());
}

/******************************************************************************
Function:    CXfTxtParaBorder::Dump
Author:      Ekaterina Dolgunicheva
Purpose:		Dump!!
******************************************************************************/
std::string CXfTxtParaBorder::CreateDumpString()
{
	return FormatStringA("colorRef:%d, width:%d, offset:%d, dashID:%d", ColorRef, Width, Offset, DashStyle);
}

/******************************************************************************
   Function:    CXfTextSymbolChar::Dump
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextSymbolChar::CreateDumpString()
{
	return FormatStringA("fontRef:%d,char:%d", FontRef, Char);
}

/******************************************************************************
   Function:    CXfSpecialHyphen::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfSpecialHyphen::CreateDumpString()
{
	return FormatStringA("IsSoft: %d, KillPrev: %d, IsAuto: %d", (int)IsSoft(), (int)KillPrev(), (int)IsAuto());
}

/******************************************************************************
   Function:    CXfFontEmbeddingPolicy::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFontEmbeddingPolicy::CreateDumpString()
{
	std::string strPrefix;
	if (GlyphFlags & FG_BOLD) strPrefix = "B";
	if (GlyphFlags & FG_ITALIC) strPrefix += "I";

	return strPrefix + FormatStringA(" Flags: %u, GlyphFlags: %u", (unsigned int)Flags, (unsigned int)GlyphFlags);
}

/******************************************************************************
   Function:    CXfTextAnchorPosition::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextAnchorPosition::CreateDumpString()
{
	std::string str;

	if(m_vAnchoredGroupID.empty())
		str = "[EMPTY]";
	else
	{
		for(size_t i = 0; i < m_vAnchoredGroupID.size(); i++)
		{
			char psz[16];
			_itoa(m_vAnchoredGroupID[i], psz, 10);
			str += psz;
			str += " ";
		}
	}

	return str;
}

/******************************************************************************
   Function:    CXfTextEmbeddedAnchorPosInfo::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextEmbeddedAnchorPosInfo::CreateDumpString()
{
	return FormatStringA("voffset: %d, l: %d, r: %d, t: %d, b: %d, w: %d, a: %d, d: %d, no descender: %d",
		VerticalOffset.get(), LeftMargin.get(), RightMargin.get(), TopMargin.get(), BottomMargin.get(),
		CharWidth.get(), FontAscent.get(),  FontDescent.get(),  bNoDescender.get());
}

/******************************************************************************
   Function:    CXfAnchoringMode::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/

std::string CXfAnchoringMode::CreateDumpString()
{
	std::string s;
	switch (HorizontalMode.get())
	{
	case AnchorH_Character:
		s = "AnchorH_Character, ";
		break;
	case AnchorH_Column:
		s = "AnchorH_Column, ";
		break;
	case AnchorH_Page:
		s = "AnchorH_Page, ";
		break;
	default:
		s = "AnchorH_Unknown, ";
		break;
	}

	switch (VerticalMode.get())
	{
	case AnchorV_BaseLine:
		s += "AnchorV_BaseLine";
		break;
	case AnchorV_AscentLine:
		s += "AnchorV_AscentLine";
		break;
	case AnchorV_ParagraphStart:
		s += "AnchorV_ParagraphStart";
		break;
	case AnchorV_ParagraphStartAscent:
		s += "AnchorV_ParagraphStartAscent";
		break;
	case AnchorV_Page:
		s += "AnchorV_Page";
		break;
	default:
		s += "AnchorV_Unknown";
		break;
	}


	return s;
}

/******************************************************************************
   Function:    CXfTextStorySimple::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextStorySimple::CreateDumpString()
{
	return FormatStringA("anchor: %d, %d, autokern: %d", Anchor->x, Anchor->y, AutoKern);
}

/******************************************************************************
Function:    CXfTextStoryInsideSimple::Dump
Author:      Ekaterina Dolgunicheva
Purpose:		Dump!!
Returns:
Exceptions:
Parameters:
Notes:
******************************************************************************/
std::string CXfTextStoryInsideSimple::CreateDumpString()
{
	return FormatStringA("anchor: %d, %d, autokern: %d, paddings: %d", Anchor->x, Anchor->y, AutoKern, PaddingForText);
}

/******************************************************************************
   Function:    CXfFontDef::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFontDef::CreateDumpString()
{
	USES_CONVERSION;
	std::string str = FormatStringA("full:%s,face:%s w:%d,altw:%d,flags:%d,rw:%d,rl:%d %X%X%X%X%X%X%X%X%X%X ", W2A((const wchar_t*)FullFontName), W2A((const wchar_t*)TypeFaceName), 
		weight, altweight, Flags, RealPanoseWeight, RealPanoseLetterform, (unsigned int)PANOSE[0], (unsigned int)PANOSE[1], (unsigned int)PANOSE[2], 
		(unsigned int)PANOSE[3], (unsigned int)PANOSE[4], (unsigned int)PANOSE[5], (unsigned int)PANOSE[6], (unsigned int)PANOSE[7], 
		(unsigned int)PANOSE[8], (unsigned int)PANOSE[9]);

	for(size_t i = 0; i < m_WeightList.size(); i++)
		str += FormatStringA("(%d %d)", m_WeightList[i].Weight, m_WeightList[i].VariantFlags);

	return str;
}

/******************************************************************************
Function:    CXfFontKerningPairs::Dump
Author:      Paul Gruzdev
Purpose:		Dump!!
Returns:
Exceptions:
Parameters:
Notes:
******************************************************************************/
std::string CXfFontKerningPairs::CreateDumpString()
{
	USES_CONVERSION;
	std::string str = FormatStringA("Amount: %d ", Pairs.size());

	for (size_t i = 0; i < Pairs.size(); i++)
	{
		str += FormatStringA("[%c %c]: %d(%d). ", Pairs[i].First, Pairs[i].Second, Pairs[i].Amount, Pairs[i].Amount/72);
	}

	return str;
}

/******************************************************************************
   Function:	CXfExtraFontDef::CreateDumpString
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		std::string
   Notes:       
******************************************************************************/
std::string CXfExtraFontDef::CreateDumpString()
{
	USES_CONVERSION;
	std::string s1 = FormatStringA("full: %s face:%s fullface:%s w:%d ", W2A(FullFontName.getstr().c_str()), W2A(TypefaceName.getstr().c_str()),
		W2A(FullTypefaceName.getstr().c_str()), (int)Weight);

	if((BYTE)Flags & EXTRA_FONT_DEF_BOLD)
		s1 += "bd ";
	if((BYTE)Flags & EXTRA_FONT_DEF_ITALIC)
		s1 += "it ";
	if((BYTE)Flags & EXTRA_FONT_DEF_REPLACED)
		s1 += FormatStringA("replaced, org:%s ", W2A(OriginalFontName.get()));
	if((BYTE)Flags & EXTRA_FONT_DEF_MULTI_WEIGHT)
		s1 += FormatStringA("mw, org-w:%d ", OriginalBaseWeight);

	if ((BYTE)Flags & EXTRA_FONT_DEF_REPLACED_BOLD)
		s1 += "rep-bd ";
	if ((BYTE)Flags & EXTRA_FONT_DEF_REPLACED_ITALIC)
		s1 += "rep-it ";
	if ((BYTE)Flags & EXTRA_FONT_DEF_MISSING_VARIANT)
		s1 += "missing-var ";

	return s1;
}
/******************************************************************************
   Function:    CXfWizop::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfWizop::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("n: %s s: %s p: %s", W2A((const wchar_t*)Name), W2A((const wchar_t*)String), W2A((const wchar_t*)Parameter));
}

/******************************************************************************
   Function:    CXfWebAddress::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfWebAddress::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("%s %s", W2A((const wchar_t*)URL), W2A((const wchar_t*)Frame));
}

/******************************************************************************
   Function:    CXfFontTypeface::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFontTypeface::CreateDumpString()
{
	return FormatStringA("font ref: %d", FontRef);
}

/******************************************************************************
   Function:    CXfExtraFontInfo::Dump
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfExtraFontInfo::CreateDumpString()
{
	return FormatStringA("font ref: %d", FontRef);
}

std::string CXfAnchoredGroupInfo::CreateDumpString()
{
	return FormatStringA("Tag: %d, ref point[%d %d]", ImportTag.get(), ReferencePoint.get().x, ReferencePoint.get().y);
}

std::string CXfAnchoredGroupInfo2::CreateDumpString()
{
	return FormatStringA("Vertically only: %d, Anchor at drop point: %d", VerticallyOnly.get(), AnchorAtDropPoint.get());
}

/******************************************************************************
   Function:    CXfEmbeddedAnchoredGroupInfo::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/

std::string CXfEmbeddedAnchoredGroupInfo::CreateDumpString()
{
	return FormatStringA("Primary: %d", bPrimary.get());
}

/******************************************************************************
   Function:    CXfEmbeddedTransformedAnchoredGroupInfo::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/

std::string CXfEmbeddedTransformedAnchoredGroupInfo::CreateDumpString()
{
	return FormatStringA("Primary: %d [%.2f %.2f %.2f %.2f]", bPrimary.get(), MatrixA.get().MakeDouble(), MatrixB.get().MakeDouble(), MatrixC.get().MakeDouble(), MatrixD.get().MakeDouble());
}

/******************************************************************************
Function:    CXfTextChangeCaps::Dump
Author:      Ekaterina Dolgunicheva
Purpose:		Dump!!
Returns:
Exceptions:
Parameters:
Notes:
******************************************************************************/
std::string CXfTextChangeCaps::CreateDumpString()
{
	return FormatStringA("type: %d", Type.get());
}

/******************************************************************************
   Function:    CXfStyleDef::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfStyleDef::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("type: %d, name: %s, base: %d", Type.get(), W2A(Name.getstr().c_str()), BaseStyle.get());
}

/******************************************************************************
   Function:    CXfTextNumberedItemStart::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextNumberedItemStart::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("level: %d, num: %d, start num: %d, format: %s, font ref: %d, bold: %d, italic: %d", (UINT16)Level, (ULONG)Number, (ULONG)StartNumber, W2A((const wchar_t*)Format), FontRef, Bold, Italic);
}

/******************************************************************************
   Function:    CXfTextBulletedItemStart::Dump
   Author:      Denis Onischenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextBulletedItemStart::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("lvl: %d; char: %x (font ref: %d; bold %d; italic %d)", (UINT16)Level, (UINT16)BulletChar, FontRecordRef, Bold, Italic);
}

/******************************************************************************
   Function:    CXfTextRuler::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextRuler::CreateDumpString()
{
	std::stringstream ss;
	for(size_t i = 0; i < Stops.size(); i++)
	{
		ss << "[pos" << (i+1) << ": " << Stops[i].Position << ", align :";

		switch (Stops[i].GetType())
		{
		case LeftTab:
			ss << "left";
			break;
		case RightTab:
			ss << "right";
			break;
		case CentreTab:
			ss << "center";
			break;
		}

		ss << "] ";
	}
	return ss.str();
}

/******************************************************************************
   Function:    CXfTextLineInfo::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextLineInfo::CreateDumpString()
{
	return FormatStringA("w: %d h: %d off: %d", Width, Height, Offset);
}

/******************************************************************************
   Function:    CXfTextLineInfo2::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextLineInfo2::CreateDumpString()
{
	return FormatStringA("l: %d r: %d asc: %d desc: %d", Left, Right, LineAscent, LineDescent);
}

/******************************************************************************
   Function:    CXfObjectBounds::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfObjectBounds::CreateDumpString()
{
	return FormatStringA("%d %d, w: %d h: %d", BottomLeft->x, TopRight->y, TopRight->x - BottomLeft->x, 
		TopRight->y - BottomLeft->y);
}

/******************************************************************************
   Function:	CXfSymbolGroup::CreateDumpString
   Author:		Denis Onishchenko
   Purpose:     
   Returns:		std::string
   Notes:       
******************************************************************************/
std::string CXfSymbolGroup::CreateDumpString()
{
	FIXED16 abcd[4];
	LONG ef[2];
	Matrix m = InstanceTransform;
	m.GetComponents(abcd, ef);
	return FormatStringA("%.2f %.2f %.2f %.2f %d %d", abcd[0].MakeDouble(), abcd[1].MakeDouble(), abcd[2].MakeDouble(), 
		abcd[3].MakeDouble(), ef[0], ef[1]);
}

/******************************************************************************
   Function:	CXfUserValue::CreateDumpString
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		std::string
   Notes:       
******************************************************************************/
std::string CXfUserValue::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("key: %s val: %s", W2A(Key.getstr().c_str()), W2A(Value.getstr().c_str()));
}

/******************************************************************************
   Function:    CXfTextStoryWordWrapInfo::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextStoryWordWrapInfo::CreateDumpString()
{
	return FormatStringA("w: %d", BlockWidth);
}

/******************************************************************************
   Function:    CXfTextStoryLinkInfo::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextStoryLinkInfo::CreateDumpString()
{
	return FormatStringA("flags: %X, next: %d, this: %d", Flags, NextInFlowID, ThisID);
}

/******************************************************************************
   Function:    CXfKerning::CXfTextStoryTranslationInfo
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextStoryTranslationInfo::CreateDumpString()
{
	return FormatStringA("%d %d", Position->x, Position->y);
}

/******************************************************************************
   Function:    CXfMitreLimit::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfMitreLimit::CreateDumpString()
{
	return FormatStringA("%d", (LONG)Limit);
}

/******************************************************************************
   Function:    CXfDashStyle::Dump
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfDashStyle::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("ID: %d", (LONG)Style);
}

/******************************************************************************
   Function:    CXfDefineDash::Dump
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfDefineDash::CreateDumpString()
{
	std::stringstream ss;
	ss << "start: " << DashStart << ", width: " << DesignLineWidth << ", [";
	for(size_t i = 0; i < DashDef.size(); i++)
		ss << DashDef[i] << " ";
	ss << "]";
	return ss.str();
}

/******************************************************************************
   Function:    CXfFlatFill::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFlatFill::CreateDumpString()
{
	return FormatStringA("%d", (LONG)ColorRef);
}

std::string CXfLinearFill::CreateDumpString()
{
	return FormatStringA("(%d, %d), %d -> (%d, %d), %d", StartPoint->x, StartPoint->y, (LONG)StartColorRef, EndPoint->x, EndPoint->y, (LONG)EndColorRef);
}

std::string CXfCircularFill::CreateDumpString()
{
	return FormatStringA("(%d, %d), %d -> (%d, %d), %d", StartPoint->x, StartPoint->y, (LONG)StartColorRef, EndPoint->x, EndPoint->y, (LONG)EndColorRef);
}

std::string CXfBitmapFill::CreateDumpString()
{
	return FormatStringA("bl(%d, %d) br(%d, %d) tl(%d, %d)", BottomLeft->x, BottomLeft->y, BottomRight->x, BottomRight->y, TopLeft->x, TopLeft->y);
}

std::string CXfFlatTransparentFill::CreateDumpString()
{
	return FormatStringA("trans: %d, mode: %d", Transparency, TransparencyType);
}

/******************************************************************************
   Function:    CXfSpreadInformation::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfSpreadInformation::CreateDumpString()
{
	return FormatStringA("Size: %.1fcm to %.1fcm Flags: 0x%X", (double)((LONG)Width)/72000.0*2.54, (double)((LONG)Height)/72000.0*2.54, (int)(BYTE)SpreadFlags);
}

/******************************************************************************
   Function:    CXfViewport::CreateDumpString
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfViewport::CreateDumpString()
{
	return FormatStringA("bl: (%d, %d) tr: (%d, %d)", BottomLeft->x, BottomLeft->y, TopRight->x, TopRight->y);
}

/******************************************************************************
   Function:    CXfKerning::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfKerning::CreateDumpString()
{
	return FormatStringA("kern: %d, font size: %d", Kern, TextContext().m_dwFontSize);
}

/******************************************************************************
   Function:    CXfLineWidth::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfLineWidth::CreateDumpString()
{
	return FormatStringA("%d", Width);
}

/******************************************************************************
   Function:	CXfJoinStyle::CreateDumpString
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		std::string
   Notes:       
******************************************************************************/
std::string CXfJoinStyle::CreateDumpString()
{
	return FormatStringA("%d", (int)Style.get());
}

/******************************************************************************
   Function:    CXfLineColor::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfLineColor::CreateDumpString()
{
	return FormatStringA("%d", ColorRef);
}

/******************************************************************************
   Function:    CXfClipviewController::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfClipviewController::CreateDumpString()
{
	return FormatStringA("objects:[%d %d %d %d]", m_rcObjects.x, m_rcObjects.y, m_rcObjects.right(), m_rcObjects.bottom());
}

/******************************************************************************
   Function:    CXfRect::CreateDumpString
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfRect::CreateDumpString()
{
	return FormatStringA("dim:[%d %d] center:[%d %d]", RX, RY, Center->x, Center->y);
}

/******************************************************************************
   Function:    CXfRectComplex::CreateDumpString
   Author:      Ekaterina Dolgunicheva
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfRectComplex::CreateDumpString()
{
	return FormatStringA("major:[%d %d] minor:[%d %d] center:[%d %d]", RX->x, RX->y, RY->x, RY->y, Center->x, Center->y);
}
/******************************************************************************
   Function:    CXfPath::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfPath::CreateDumpString()
{
	std::string s = FormatStringA("pts:%d ", Verbs.size());

	if(Verbs.size() < 10)
	{
		s += "[";
		char psz1[128];
		for(size_t i = 0; i < Verbs.size(); i++)
		{
			BYTE byVerb = Verbs[i] & 0xFE;
			char cv = byVerb == PT_MOVETO ? 'm' : (byVerb == PT_LINETO ? 'l' : 'c');
			sprintf(psz1, "%c(%d,%d)", cv, Coords[i].x, Coords[i].y);
			s += psz1;
			if(Verbs[i] & PT_CLOSEFIGURE) s += "cp";
			if(i < Verbs.size() - 1) s += ",";
		}
		s += "]";
	}
	return s;
}

/******************************************************************************
   Function:    CXfFontGlyph::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:     Dump!! the font glyph object
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfFontGlyph::CreateDumpString()
{
	std::string strCharCode;
	//strCharCode = (CharCode <= 32) ? FormatStringA("%X", CharCode) : FormatStringA("%c", CharCode);
	strCharCode = FormatStringA("%X '%c'", CharCode, CharCode);
	std::string strPrefix;
	if(Flags & FG_BOLD) strPrefix = "B";
	if(Flags & FG_ITALIC) strPrefix += "I";

	return strPrefix + FormatStringA(" Code: %s%s a:%d d:%d w:%d em:%d", strCharCode.c_str(), Verbs.size() ? "" : " [Empty Path]",
		FontAscent.get(), FontDescent.get(), Width.get(), FontEMWidth.get());
}

/******************************************************************************
   Function:    CXfEmbeddedGlyphSizeHint::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:     Dump!! glyph size hint
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfEmbeddedGlyphSizeHint::CreateDumpString()
{
	return FormatStringA("points: %d, glyphs: %d, font variants: %d", TotalNumPoints, NumGlyphs, NumFontVariants);
}

/******************************************************************************
   Function:    CXfLayerDetails::CreateDumpString
   Author:      Denis Onishchenko
   Purpose:     Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfLayerDetails::CreateDumpString()
{
	USES_CONVERSION;
	std::stringstream ss;
	ss << "Name: " << W2A(Name.get()) <<
		", Visible: "   << ((eIsVisible   == (eIsVisible   & Flags.get())) ? "1" : "0") <<
		", Locked: "    << ((eIsLocked    == (eIsLocked    & Flags.get())) ? "1" : "0") <<
		". Printable: " << ((eIsPrintable == (eIsPrintable & Flags.get())) ? "1" : "0") <<
		", Active: "    << ((eIsActive    == (eIsActive    & Flags.get())) ? "1" : "0");
	return ss.str();
}

/******************************************************************************
   Function:    CXfDocumentInformation::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:     Dump!! glyph size hint
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfDocumentInformation::CreateDumpString()
{
	return FormatStringA("chapters: %d, spreads1: %d", NumChapters, m_Chapters.size() ? m_Chapters[0].NumSpreads : 0);
}

/******************************************************************************
   Function:    CXfLineTransparency::CreateDumpString
   Author:      Pavel Ostapenko
   Purpose:     Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfLineTransparency::CreateDumpString()
{
	std::stringstream ss;
	ss << "type:" << static_cast<int>(TransparencyType) << " value:" << static_cast<int>(Transparency);
	return ss.str();
}

/******************************************************************************
   Function:    CXfWindingRule::CreateDumpString
   Author:      Paul Gruzdev
   Purpose:     Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfWindingRule::CreateDumpString()
{
	std::string rule;
	switch(Winding)
	{
	case CXF_WINDING_RULE_NONZERO:
		rule = "nz";
		break;
	case CXF_WINDING_RULE_NEGATIVE:
		rule = "neg";
		break;
	case CXF_WINDING_RULE_EVENODD:
		rule = "eo";
		break;
	case CXF_WINDING_RULE_POSITIVE:
		rule = "pos";
		break;
	default:
		rule = "wrong";
	}
	return rule.c_str();
}

/******************************************************************************
   Function:    CXfComplexColor::CreateDumpString
   Author:      Pavel Ostapenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfComplexColor::CreateDumpString()
{
	std::stringstream ss;
	switch (ColourModel)
	{
	case CXfComplexColor::cmRGB:
		ss << "rgb(" << Desc1->MakeDouble() << ',' << Desc2->MakeDouble() << ',' << Desc3->MakeDouble() << ')';
		break;
	case CXfComplexColor::cmCMYK:
		ss << "cmyk(" << Desc1->MakeDouble() << ',' << Desc2->MakeDouble() << ',' << Desc3->MakeDouble() << ',' << Desc4->MakeDouble() << ')';
		break;
	case CXfComplexColor::cmHSV:
		ss << "hsv(" << Desc1->MakeDouble() << ',' << Desc2->MakeDouble() << ',' << Desc3->MakeDouble() << ',' << Desc4->MakeDouble() << ')';
		break;
	case CXfComplexColor::cmGreyscale:
		ss << "greyscale(" << Desc1->MakeDouble() << ')';
		break;
	}

	switch(ColourType)
	{
	case ctSpot:
		ss << " spot";
		break;
	case ctTint:
		ss << " tint(" << ParentColour << ">";
		break;
	case ctLinked:
		ss << " linked(" << ParentColour << ">";
		break;
	case ctShade:
		ss << " shade(" << ParentColour << ">";
		break;
	}

	if(EntryIndex > 0)
		ss << " idx:" << EntryIndex;

	if(ColourName.getstr().length())
		ss << " name: " << ColourName.getstr().c_str();

	return ss.str();
}


/******************************************************************************
   Function:    CXfComplexColor::CreateDumpString
   Author:      Pavel Ostapenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfTextScriptOn::CreateDumpString()
{
	std::stringstream ss;
	ss << "offset:" << ScriptOffset->MakeDouble() << " size:" << ScriptFontSize->MakeDouble();
	return ss.str();
}

/******************************************************************************
   Function:	CXfSoftMaskGroupTransparentFill::CreateDumpString
   Author:		Paul Gruzdev
   Purpose:     Dump!!
   Returns:		std::string
   Notes:       
******************************************************************************/
std::string CXfSoftMaskGroupTransparentFill::CreateDumpString()
{
	return FormatStringA("start: %d end: %d type: %d", (int)StartTransparency, (int)EndTransparency, (int)TransparencyType);
}

/******************************************************************************
   Function:    CXfNodeDetails::CreateDumpString
   Author:      Denis Onishchenko
   Purpose:		Dump!!
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::string CXfNodeDetails::CreateDumpString()
{
	std::stringstream ss;
	ss << "visible: " << (IsVisible() ? "1" : "0") << ", locked: " << (IsLocked() ? "1" : "0");
	return ss.str();
}

std::string CXfFontOTLigatures::CreateDumpString()
{
	USES_CONVERSION;
	std::string ret;
	for (size_t i = 0; i < ligs.size(); i++)
	{
		std::wstring str(ligs[i].Key.begin(), ligs[i].Key.end());
		if (GetTag() == TAG_FONT_OTLIGATURES)
			ret += FormatStringA("'%s'->%X(%d) ", W2A(str.c_str()), ligs[i].Ligature, ligs[i].LigType);
		else if (GetTag() == TAG_FONT_OTLIGATURES_EXT)
			ret += FormatStringA("'%s'->%X(%d)[GID:%d;p:%d] ", W2A(str.c_str()), ligs[i].Ligature, ligs[i].LigType, ligs[i].GlyphID, ligs[i].Priority);
	}
	return ret;
}

std::string CXfTextOpenTypeLigatures::CreateDumpString()
{
	if (LigatureSubset == (LIG_SUB_STANDARD + LIG_SUB_HISTORIC + LIG_SUB_DISCRETIONARY))
		return "all";

	std::string ret;
	if (LigatureSubset & LIG_SUB_NONE)
		ret = "none";
	if (LigatureSubset & LIG_SUB_STANDARD)
		ret += " standard";
	if (LigatureSubset & LIG_SUB_HISTORIC)
		ret += " historic";
	if (LigatureSubset & LIG_SUB_DISCRETIONARY)
		ret += " discretionary";
	if (LigatureSubset & LIG_SUB_CONTEXTUAL)
		ret += " contextual";

	return ret;
}

std::string CXfTextLigatureStart::CreateDumpString()
{
	return FormatStringA("Uni: %X ", (WCHAR)LigatureUnicode);
}

std::string CXfFontOTStylisticSets::CreateDumpString()
{
	std::string str;
	for (size_t i = 0; i < STYLISTIC_SETS_MAX_SIZE; i++)
	{
		if (arrStylisticSets[i].size())
		{
			str += FormatStringA("%d:[", i);
			std::vector<StylisticSetEntry>& vec = arrStylisticSets[i];
			for (size_t j = 0; j < vec.size(); j++)
			{
				StylisticSetEntry& entry = vec[j];
				str += FormatStringA("'%c'(%X)->%X,GID:%d, ", entry.Key, entry.Key, entry.StylisticChar, entry.SubGlyphID);
			}
			str += FormatStringA("] ");
		}
	}
	return str;
}

std::string DumpSubstTable(SubstitutionTable& table)
{
	USES_CONVERSION;
	static char* types[] = { "SINGLE",
		"MULTIPLE",
		"LIGATURE",
		"SINGLE_STANDARD",
		"MULTIPLE_STANDARD",
		"LIGATURE_STANDARD" };

	std::string str = FormatStringA("UID: %d type: %s, ", table.UID, types[table.SubstitutionType]);
	for (std::unordered_map<std::wstring, std::vector<std::pair<WCHAR, DWORD>>>::iterator it = table.SubstitutionMap.begin(); it != table.SubstitutionMap.end(); ++it)
	{
		std::wstring key(it->first.begin(), it->first.end());
		std::wstring value;
		for (size_t i = 0; i < it->second.size(); i++)
			value += FormatStringW(L"%X ", it->second[i].first);
		str += FormatStringA("'%s'->%s ", W2A(key.c_str()), W2A(value.c_str()));
	}
	return str;
}

std::string CXfFontOTSubstitutionRules::DumpSubstRule(SubstitutionRule& rule)
{
	USES_CONVERSION;
	std::wstring pre, post, keys;
	std::string langs;

	if (rule.Pre.empty()) pre = L"<empty>";
	for (size_t i = 0; i < rule.Pre.size(); i++)
	{
		std::wstring s(rule.Pre[i].begin(), rule.Pre[i].end());
		if (!pre.empty()) pre += L",";
		pre += L"'" + s + L"'";
	}

	if (rule.Post.empty()) post = L"<empty>";
	for (size_t i = 0; i < rule.Post.size(); i++)
	{
		std::wstring s(rule.Post[i].begin(), rule.Post[i].end());
		if (!post.empty()) post += L",";
		post += L"'" + s + L"'";
	}

	if (rule.Keys.empty()) keys = L"<empty>";
	for (size_t i = 0; i < rule.Keys.size(); i++)
	{
		std::wstring s(rule.Keys[i].begin(), rule.Keys[i].end());
		if (!keys.empty()) keys += L",";
		keys += L"'" + s + L"'";
	}

	for (size_t i = 0; i < rule.Langs.size(); i++)
		langs += FormatStringA("%d ", rule.Langs[i]);

	return FormatStringA("ID: %d pre: %s post: %s keys: %s langs: %s", rule.SubstitutionTableID, W2A(pre.c_str()), W2A(post.c_str()), W2A(keys.c_str()), langs.c_str());
}

std::string CXfFontOTSubstitutionTables::CreateDumpString()
{
	std::string str = FormatStringA("\nFlags: %d, ", Flags.get());

	for (SubstitutionTableSet::iterator it = TableSet.begin(); it != TableSet.end(); ++it)
	{
		str += DumpSubstTable(it->second);
		str += "\n";
	}
	return str;
}

// The below should only be defined for DumpXarVC12 solution
#ifdef FULL_STRINGS_DUMP
const char* g_pszGlyphSet[] = 
{
	"GSET_NONE",
	"GSET_ACCESS_ALL_ALTERNATES",
	"GSET_ABOVE_BASE_FORMS",
	"GSET_ABOVE_BASE_MARK",
	"GSET_ABOVE_BASE_SUBSTITUTIONS",
	"GSET_VERTICAL_FRACTIONS",
	"GSET_AKHAND",
	"GSET_ANCIENT_LIGATURES",
	"GSET_BELOW_BASE_FORMS",
	"GSET_BELOW_BASE_MARK",
	"GSET_BELOW_BASE_SUBSTITUTIONS",
	"GSET_CAPITALS_TO_PETITE_CAPITALS",
	"GSET_CAPITALS_TO_SMALL_CAPITALS",
	"GSET_CONTEXTUAL_ALTERNATES",
	"GSET_CASE_SENSITIVE_FORMS",
	"GSET_GLYPH_COMPOSITION_DECOMPOSITION",
	"GSET_CONJUNCT_FORM_AFTER_RO",
	"GSET_CONJUNCT_FORMS",
	"GSET_CONTEXTUAL_LIGATURES",
	"GSET_CENTERED_CJK_PUNCTUATION",
	"GSET_CAPITAL_SPACING",
	"GSET_CONTEXTUAL_SWASH",
	"GSET_CURSIVE_ATTACHMENT",
	"GSET_CHARACTER_VARIANTS_00",
	"GSET_CHARACTER_VARIANTS_01",
	"GSET_CHARACTER_VARIANTS_02",
	"GSET_CHARACTER_VARIANTS_03",
	"GSET_CHARACTER_VARIANTS_04",
	"GSET_CHARACTER_VARIANTS_05",
	"GSET_CHARACTER_VARIANTS_06",
	"GSET_CHARACTER_VARIANTS_07",
	"GSET_CHARACTER_VARIANTS_08",
	"GSET_CHARACTER_VARIANTS_09",
	"GSET_CHARACTER_VARIANTS_10",
	"GSET_CHARACTER_VARIANTS_99",
	"GSET_DROP_CAPS",
	"GSET_DISTANCE",
	"GSET_DISCRETIONARY_LIGATURES",
	"GSET_DENOMINATORS",
	"GSET_DIPTHONGS",
	"GSET_DOTLESS_FORMS",
	"GSET_EXPERT_FORMS",
	"GSET_FINAL_GLYPH_ON_LINE",
	"GSET_TERMINAL_FORMS_2",
	"GSET_TERMINAL_FORMS_3",
	"GSET_TERMINAL_FORMS",
	"GSET_FLATTENED_ACCENTS_OVER_CAPITALS",
	"GSET_DIAGONAL_FRACTIONS",
	"GSET_FULL_WIDTHS",
	"GSET_HALF_FORMS",
	"GSET_HALANT_FORMS",
	"GSET_ALTERNATIVE_HALF_WIDTHS",
	"GSET_HISTORICAL_FORMS",
	"GSET_HORIZONTAL_KANA_ALTERNATIVES",
	"GSET_HISTORIC_LIGATURES",
	"GSET_HANJA_TO_HANGUL",
	"GSET_HOJO_KANJI_FORMS",
	"GSET_HALF_WIDTHS",
	"GSET_INITIAL_FORMS",
	"GSET_ISOLATED_FORMS",
	"GSET_ITALICS",
	"GSET_JUSTIFICATION_ALTERNATIVES",
	"GSET_JAPANESE_FORMS",
	"GSET_JIS2004_FORMS",
	"GSET_JIS78_FORMS",
	"GSET_JIS83_FORMS",
	"GSET_JIS90_FORMS",
	"GSET_HORIZONTAL_KERNING",
	"GSET_LEFT_BOUNDS",
	"GSET_STANDARD_LIGATURES",
	"GSET_LEADING_JAMO_FORMS",
	"GSET_LINING_FIGURES",
	"GSET_LOCALIZED_FORMS",
	"GSET_MARK_POSITIONING",
	"GSET_MEDIAL_FORMS_2",
	"GSET_MEDIAL_FORMS",
	"GSET_MATHEMATICAL_GREEK",
	"GSET_MARK_TO_MARK",
	"GSET_MARK_POSITIONING_VIA_SUBSTITUTION",
	"GSET_ALTERNATE_ANNOTATION_FORMS",
	"GSET_NLC_KANJI_FORMS",
	"GSET_NUKTA_FORMS",
	"GSET_NUMERATORS",
	"GSET_OLDSTYLE_FIGURES",
	"GSET_OPTICAL_BOUNDS",
	"GSET_ORDINALS",
	"GSET_ORNAMENTS",
	"GSET_PROPORTIONAL_ALTERNATE_METRICS",
	"GSET_LOWERCASE_TO_PETITE_CAPITALS",
	"GSET_PROPORTIONAL_KANA",
	"GSET_PROPORTIONAL_NUMBERS",
	"GSET_PRE_BASE_FORMS",
	"GSET_PRE_BASE_SUBSTITUTIONS",
	"GSET_POST_BASE_FORMS",
	"GSET_POST_BASE_SUBSTITUTIONS",
	"GSET_PROPORTIONAL_WIDTH",
	"GSET_QUARTER_WIDTHS",
	"GSET_RANDOMIZE",
	"GSET_RAKAR_FORMS",
	"GSET_REQUIRED_LIGATURES",
	"GSET_REPH_FORM",
	"GSET_RIGHT_BOUNDS",
	"GSET_RIGHT_TO_LEFT_ALTERNATES",
	"GSET_RIGHT_TO_LEFT_MIRRORED_FORMS",
	"GSET_RUBY_NOTATIONAL_FORMS",
	"GSET_STYLISTIC_ALTERNATIVES",
	"GSET_SCIENTIFIC_INFERIORS",
	"GSET_LOWERCASE_TO_SMALL_CAPITALS",
	"GSET_SIMPLIFIED_FORMS",
	"GSET_STYLE_SET_1",
	"GSET_STYLE_SET_2",
	"GSET_STYLE_SET_3",
	"GSET_STYLE_SET_4",
	"GSET_STYLE_SET_5",
	"GSET_STYLE_SET_6",
	"GSET_STYLE_SET_7",
	"GSET_STYLE_SET_8",
	"GSET_STYLE_SET_9",
	"GSET_STYLE_SET_10",
	"GSET_STYLE_SET_11",
	"GSET_STYLE_SET_12",
	"GSET_STYLE_SET_13",
	"GSET_STYLE_SET_14",
	"GSET_STYLE_SET_15",
	"GSET_STYLE_SET_16",
	"GSET_STYLE_SET_17",
	"GSET_STYLE_SET_18",
	"GSET_STYLE_SET_19",
	"GSET_STYLE_SET_20",
	"GSET_SCRIPT_STYLE",
	"GSET_SUBSCRIPT",
	"GSET_SUPERSCRIPT",
	"GSET_SWASH",
	"GSET_TITLING",
	"GSET_TRAILING_JAMO_FORMS",
	"GSET_TRADITIONAL_NAME_FORMS",
	"GSET_TABULAR_NUMBERS",
	"GSET_TRADITIONAL_FORMS",
	"GSET_THIRD_WIDTHS",
	"GSET_UNICASE",
	"GSET_ALTERNATE_VERTICAL_METRICS",
	"GSET_VATTU_VARIANTS",
	"GSET_VERTICAL_ALTERNATES",
	"GSET_ALTERNATE_VERTICAL_HALF_METRICS",
	"GSET_VOWEL_JAMO_FORMS",
	"GSET_VERTICAL_KANA_ALTERNATES",
	"GSET_VERTICAL_KERNING",
	"GSET_PROPORTIONAL_ALTERNATE_VERTICAL_METRICS",
	"GSET_VERTICAL_ROTATION_ALTERNATES",
	"GSET_SLASHED_ZERO",
	"GSET_REQUIRED_FEATURE"
};
#else
const char** g_pszGlyphSet = 0;
#endif

std::string CXfFontOTSubstitutionRules::CreateDumpString()
{
	std::string str;
	for (SubstitutionRuleSet::iterator it = RuleSet.begin(); it != RuleSet.end(); ++it)
	{
		str += g_pszGlyphSet ? FormatStringA("%s: ", g_pszGlyphSet[it->first]) : FormatStringA("%d: ", it->first);
		for (size_t i = 0; i < it->second.first.size(); i++)
			str += CXfFontOTSubstitutionRules::DumpSubstRule(it->second.first[i]) + "\n";
	}

	return str;
}

std::string CXfGenericStringNode::CreateDumpString()
{
	USES_CONVERSION;
	return FormatStringA("%s", W2A(strData.getstr().c_str()));
}

std::string CXfGenericNode::CreateDumpString() {
	return "";

// 	std::string str;
// 	for (size_t i = 0; i < Data.size(); i++) {
// 		str += FormatStringA("%.2X ", Data[i]);
// 		if (i && (i % 50 == 0))
// 			str += "\n";
// 	}
// 	if (Data.size() <= 20) {
// //		str += "\n";
// 		for (size_t i = 0; i < Data.size(); i++)
// 			if (Data[i] >= 32)
// 				str += FormatStringA("'%c' ", (char)Data[i]);
// 			else
// 				str += FormatStringA("0x%.2X ", (int)Data[i]);
// 	}
// 	return str;
}

std::string CXfShadow::CreateDumpString()
{
	return FormatStringA("Bias: %f, Gain: %f, Darkness: %f", Profile.Bias, Profile.Gain, Darkness);
}

std::string CXfDropShadowController::CreateDumpString()
{
	return FormatStringA("Type: %u, Blur: %d, OffsetX: %d, OffsetY: %d, FloorAngle: %d, FloorHeight: %d, WallScale: %d, GlowWidth: %d",
		Type, Blur, Offset->x, Offset->y, FloorAngle, FloorHeight, WallScale, GlowWidth);
}
