/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

typedef enum RepeatType
{
	RT_NOREPEATTYPE = 0,
	RT_SIMPLE,
	RT_REPEATING,
	RT_REPEATINVERTED,
	RT_REPEATEXTRA,

	RT_BLURENHANCE = 7,
} RepeatType;

/********************************************************************************************
>	class CXfProfile

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Read bias/gain data
********************************************************************************************/
struct CXfProfile
{
	CXfProfile( CXaraFileRecord & rec )
		: Bias( rec ), Gain( rec )
	{
	}

	CXfProfile(double b, double g) : Bias(b), Gain(g){}

	BOOL Write(CXaraFileRecord* Rec)
	{
		BOOL ok = Bias.Write(Rec);
		ok = Gain.Write(Rec);
		return ok;
	}

	CXfdouble Bias;
	CXfdouble Gain;
};


////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////
// Fills

/********************************************************************************************
>	class CXfFlatFill

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_FLATFILL
********************************************************************************************/
class CXfFlatFill : public VXFFillStyleNode
{
public:
	enum Type { Reference, None, White, Black };

	CXfFlatFill(CXaraFileRecord& rec)
		: ColorRef( rec ), type(Reference), pReference(0), VXFFillStyleNode(TAG_FLATFILL) {}

	CXfFlatFill(ULONG Ref) : ColorRef(Ref), type(Reference), VXFFillStyleNode(TAG_FLATFILL), pReference(0) {}
	CXfFlatFill(Type tp) : ColorRef(-1), type(tp), pReference(0), VXFFillStyleNode(TAG_FLATFILL) {}
	CXfFlatFill(VXfTreeNode* pRef) : ColorRef(-1), type(Reference), pReference(pRef), VXFFillStyleNode(TAG_FLATFILL) {}
	CXfFlatFill(const CXfFlatFill& s) : ColorRef(s.ColorRef), type(s.type), pReference(s.pReference), VXFFillStyleNode(TAG_FLATFILL) {}
	virtual bool IsFlatFill() { return true; }
	virtual bool IsNone() { return type == None; }
	virtual COLORREF GetRGBColor(RefNodeMap& map);
	virtual VXfTreeNode* Copy() { return new CXfFlatFill(*this); }

	std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		if (type == Black)
			return pExporter->WriteZeroSizedRecord(TAG_FLATFILL_BLACK, pRecordNum);
		else if (type == White)
			return pExporter->WriteZeroSizedRecord(TAG_FLATFILL_WHITE, pRecordNum);
		else if (type == None)
			return pExporter->WriteZeroSizedRecord(TAG_FLATFILL_NONE, pRecordNum);

		CXaraFileRecord Rec(TAG_FLATFILL);
		Rec.Init();
		if(pReference) ColorRef = pReference->GetRef();
		BOOL ok = ColorRef.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfLONG ColorRef;
	Type type;
	VXfTreeNode* pReference; // This is the pointer to an actual colour definition node. Used when constructing XfTree on import
};

class CXfLinearFillBase : public VXFFillStyleNode
{
public:
	CXfLinearFillBase(CXaraFileRecord& rec) : StartPoint(rec), EndPoint(rec) {}
	CXfLinearFillBase(UINT uiTag, const DocCoord& dcStart, const DocCoord& dcEnd) : VXFFillStyleNode(uiTag), StartPoint(dcStart), EndPoint(dcEnd) {}
	CXfLinearFillBase(const CXfLinearFillBase& s) : VXFFillStyleNode(s.GetTag()), StartPoint(s.StartPoint), EndPoint(s.EndPoint) {}
	BOOL Write(CXaraFileRecord* Rec)
	{
		BOOL ok = StartPoint.Write(Rec);
		if(ok) ok = EndPoint.Write(Rec);
		return ok;
	}

	CXfDocCoord StartPoint;
	CXfDocCoord EndPoint;
};

struct CXfFillProfileColors
{
	CXfFillProfileColors( CXaraFileRecord & rec )
		: StartColorRef( rec ), EndColorRef( rec ), Profile( rec ), pStartColor(0), pEndColor(0)
	{
	}
	CXfFillProfileColors(LONG ls, LONG le, double db, double dg) : StartColorRef(ls), EndColorRef(le), Profile(db, dg), 
		pStartColor(0), pEndColor(0){}

	CXfFillProfileColors(VXfTreeNode* pStartC, VXfTreeNode* pEndC, double db, double dg) : 
		pStartColor(pStartC), pEndColor(pEndC), Profile(db, dg) {}

	BOOL Write(CXaraFileRecord* Rec)
	{
		if(pStartColor) StartColorRef = pStartColor->GetRef();
		if(pEndColor) EndColorRef = pEndColor->GetRef();
		BOOL ok = StartColorRef.Write(Rec);
		if(ok) ok = EndColorRef.Write(Rec);
		if(ok) ok = Profile.Write(Rec);
		return ok;
	}

	CXfLONG StartColorRef;
	CXfLONG EndColorRef;
	VXfTreeNode* pStartColor;
	VXfTreeNode* pEndColor;
	CXfProfile Profile;
};

template< class Base >
struct CXfFillProfile: Base, CXfFillProfileColors
{
	CXfFillProfile( CXaraFileRecord & rec )
		: Base( rec ), CXfFillProfileColors( rec )
	{
	}

	CXfFillProfile(UINT uiTag, DocCoord& dcCenter, DocCoord& dcRX, DocCoord& dcRY, 
		VXfTreeNode* pStartC, VXfTreeNode* pEndC, double db, double dg) 
		: Base(uiTag, dcCenter, dcRX, dcRY), CXfFillProfileColors(pStartC, pEndC, db, dg) {}

	CXfFillProfile(const CXfFillProfile& s) : Base(s), CXfFillProfileColors(s) {}
	virtual VXfTreeNode* Copy() { return new CXfFillProfile(*this); }
	virtual BOOL Write(CXarExport* pExport, ULONG* ulRecordRef = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		BOOL ok = Base::Write(&Rec);
		if(ok) ok = CXfFillProfileColors::Write(&Rec);
		if(ok) ok = pExport->WriteRecord(&Rec, ulRecordRef);

		return ok;
	}
};

class CXfLinearFill : public CXfLinearFillBase, public CXfFillProfileColors
{
public:
	CXfLinearFill( CXaraFileRecord & rec )
		: CXfLinearFillBase( rec ), CXfFillProfileColors( rec ) {}
	CXfLinearFill(DocCoord& dcStart, DocCoord& dcEnd, ULONG ulStartColRef, 
		ULONG ulEndColRef, double db, double dg) : CXfLinearFillBase(TAG_LINEARFILL, dcStart, dcEnd), CXfFillProfileColors(ulStartColRef, ulEndColRef, db, dg)
	{
	}

	CXfLinearFill(DocCoord& dcStart, DocCoord& dcEnd, VXfTreeNode* pStartC, VXfTreeNode* pEndC, 
		double db, double dg) : CXfLinearFillBase(TAG_LINEARFILL, dcStart, dcEnd), 
		CXfFillProfileColors(pStartC, pEndC, db, dg)
	{
	}
	CXfLinearFill(const CXfLinearFill& s) : CXfLinearFillBase(TAG_LINEARFILL, s.StartPoint, s.EndPoint), 
		CXfFillProfileColors(s.StartColorRef, s.EndColorRef, s.Profile.Bias, s.Profile.Gain)
	{
		pStartColor = s.pStartColor;
		pEndColor = s.pEndColor;
	}

	std::string CreateDumpString();


	virtual VXfTreeNode* Copy() { return new CXfLinearFill(*this); }
	virtual bool IsLinearFill() { return true; }
	virtual BOOL Write(CXarExport* pExport, ULONG* ulRecordRef = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		BOOL ok = CXfLinearFillBase::Write(&Rec);
		if(ok) ok = CXfFillProfileColors::Write(&Rec);
		if(ok) ok = pExport->WriteRecord(&Rec, ulRecordRef);

		return ok;
	}
};

class CXfCircularFill : public CXfLinearFillBase, public CXfFillProfileColors
{
public:
	CXfCircularFill( CXaraFileRecord & rec )
		: CXfLinearFillBase( rec ), CXfFillProfileColors( rec ) {}
	CXfCircularFill(DocCoord& dcStart, DocCoord& dcEnd, ULONG ulStartColRef, 
		ULONG ulEndColRef, double db, double dg) : CXfLinearFillBase(TAG_CIRCULARFILL, dcStart, dcEnd), CXfFillProfileColors(ulStartColRef, ulEndColRef, db, dg)
	{
	}

	CXfCircularFill(DocCoord& dcStart, DocCoord& dcEnd, VXfTreeNode* pStartC, VXfTreeNode* pEndC, 
		double db, double dg) : CXfLinearFillBase(TAG_CIRCULARFILL, dcStart, dcEnd), 
		CXfFillProfileColors(pStartC, pEndC, db, dg)
	{
	}
	CXfCircularFill(const CXfCircularFill& s) : CXfLinearFillBase(TAG_CIRCULARFILL, s.StartPoint, s.EndPoint), 
		CXfFillProfileColors(s.StartColorRef, s.EndColorRef, s.Profile.Bias, s.Profile.Gain)
	{
		pStartColor = s.pStartColor;
		pEndColor = s.pEndColor;
	}
	virtual VXfTreeNode* Copy() { return new CXfCircularFill(*this); }
	virtual bool IsCircularFill() { return true; }

	std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExport, ULONG* ulRecordRef = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		BOOL ok = CXfLinearFillBase::Write(&Rec);
		if(ok) ok = CXfFillProfileColors::Write(&Rec);
		if(ok) ok = pExport->WriteRecord(&Rec, ulRecordRef);

		return ok;
	}
};

struct CXfFillMultiStageColors
{
	CXfFillMultiStageColors( CXaraFileRecord & rec );
	CXfFillMultiStageColors(VXfTreeNode* pStartC, VXfTreeNode* pEndC) : pStartColor(pStartC), pEndColor(pEndC) {}
	CXfFillMultiStageColors(const CXfFillMultiStageColors& s) : StartColorRef(s.StartColorRef),
				EndColorRef(s.EndColorRef),
				Positions(s.Positions),
				ColorRefs(s.ColorRefs),
				pStartColor(s.pStartColor),
				pEndColor(s.pEndColor),
				ColorRefPtr(s.ColorRefPtr)
	{
	}

	void AddMiddleColor(double dPos, VXfTreeNode* pColor)
	{
		Positions.push_back(dPos);
		ColorRefPtr.push_back(pColor);
	}

	BOOL Write(CXaraFileRecord* Rec)
	{
		if(pStartColor) StartColorRef = pStartColor->GetRef();
		if(pEndColor) EndColorRef = pEndColor->GetRef();
		BOOL ok = StartColorRef.Write(Rec);
		if(ok) ok = EndColorRef.Write(Rec);
		if(ok) ok = Rec->WriteULONG((ULONG)Positions.size());
		bool bRefPtr = ColorRefPtr.size() > 0;

		for(size_t i = 0; i < Positions.size(); i++)
		{
			if(ok) ok = Rec->WriteDOUBLE(Positions[i]);
			LONG lRef = bRefPtr ? (ColorRefPtr[i])->GetRef() : ColorRefs[i];
			if(ok) ok = Rec->WriteLONG(lRef);
		}

		return ok;
	}

	CXfLONG StartColorRef;
	CXfLONG EndColorRef;
	std::vector< double > Positions;
	std::vector< LONG > ColorRefs;
	// Interactive tree creation attributes:
	VXfTreeNode* pStartColor;
	VXfTreeNode* pEndColor;
	std::vector<VXfTreeNode*> ColorRefPtr;
};

template< class Base >
class CXfFillMultiStage : public Base, public CXfFillMultiStageColors
{
public:
	CXfFillMultiStage( CXaraFileRecord & rec )
		: Base( rec ), CXfFillMultiStageColors( rec )
	{
	}

	CXfFillMultiStage(UINT uiTag, DocCoord& dcStart, DocCoord& dcEnd, VXfTreeNode* pStartC, VXfTreeNode* pEndC) 
		: Base(uiTag, dcStart, dcEnd), CXfFillMultiStageColors(pStartC, pEndC) {}

	CXfFillMultiStage(UINT uiTag, DocCoord& dcCenter, DocCoord& dcMajor, DocCoord& dcMinor, VXfTreeNode* pStartC, VXfTreeNode* pEndC)
		: Base(uiTag, dcCenter, dcMajor, dcMinor), CXfFillMultiStageColors(pStartC, pEndC) {}

	CXfFillMultiStage(const CXfFillMultiStage& s) : Base(s), CXfFillMultiStageColors(s) {}
	virtual VXfTreeNode* Copy() { return new CXfFillMultiStage(*this); }
	virtual BOOL Write(CXarExport* pExport, ULONG* ulRecordRef = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		BOOL ok = Base::Write(&Rec);
		if(ok) ok = CXfFillMultiStageColors::Write(&Rec);

		return pExport->WriteRecord(&Rec, ulRecordRef);
	}

};

//typedef CXfFillProfile< CXfLinearFillBase > CXfLinearFill;
typedef CXfFillMultiStage< CXfLinearFillBase > CXfLinearFillMultiStage;

typedef CXfLinearFillBase CXfCircularFillBase;
typedef CXfLinearFillMultiStage CXfCircularFillMultiStage;

class CXfEllipticalFillBase : public VXFFillStyleNode
{
public:
	CXfEllipticalFillBase( CXaraFileRecord & rec )
		: Center ( rec ), RX( rec ), RY( rec )

	{
	}

	CXfEllipticalFillBase(UINT uiTag, DocCoord& dcCenter, DocCoord& dcRX, DocCoord& dcRY) 
		: VXFFillStyleNode(uiTag), Center(dcCenter), RX(dcRX), RY(dcRY) {}

	BOOL Write(CXaraFileRecord* Rec)
	{
		BOOL ok = Center.Write(Rec);
		if(ok) ok = RX.Write(Rec);
		if(ok) ok = RY.Write(Rec);
		return ok;
	}
	virtual bool IsEllipticalFill() { return true; }
	CXfDocCoord Center;
	CXfDocCoord RX, RY;
};

typedef CXfFillProfile< CXfEllipticalFillBase > CXfEllipticalFill;
typedef CXfFillMultiStage< CXfEllipticalFillBase > CXfEllipticalFillMultiStage;

class CXfLinearFill3PointBase : public VXFFillStyleNode
{
public:
	CXfLinearFill3PointBase( CXaraFileRecord & rec )
		: StartPoint( rec ), EndPoint( rec ), EndPoint2( rec )
	{
	}

	CXfLinearFill3PointBase(UINT uiTag, DocCoord& dcStart, DocCoord& dcEnd, DocCoord& dcEnd2) 
		: VXFFillStyleNode(uiTag), StartPoint(dcStart), EndPoint(dcEnd), EndPoint2(dcEnd2) {}

	virtual bool IsLinearFill() { return true; }

	BOOL Write(CXaraFileRecord* Rec)
	{
		BOOL ok = StartPoint.Write(Rec);
		if(ok) ok = EndPoint.Write(Rec);
		if(ok) ok = EndPoint2.Write(Rec);
		return ok;
	}

	CXfDocCoord StartPoint;
	CXfDocCoord EndPoint;
	CXfDocCoord EndPoint2;
};

typedef CXfFillProfile< CXfLinearFill3PointBase > CXfLinearFill3Point;
typedef CXfFillMultiStage< CXfLinearFill3PointBase > CXfLinearFill3PointMultiStage;

class CXfFlatTransparentFill : public VXFFillStyleNode
{
public:
	CXfFlatTransparentFill( CXaraFileRecord & rec )
		: Transparency( rec ), TransparencyType( rec )
	{
	}

	CXfFlatTransparentFill(BYTE bTrans, BYTE bType) : Transparency(bTrans), TransparencyType(bType), 
		VXFFillStyleNode(TAG_FLATTRANSPARENTFILL) 
	{
		ValidateTransparencyType(bType);
	}
	virtual bool IsTransparentFill() { return true; }
	virtual bool IsFlatFill() { return true; }
	virtual std::string CreateDumpString();
	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		Transparency.Write(&rec);
		TransparencyType.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	CXfBYTE Transparency;
	CXfBYTE TransparencyType;
};

struct CXfFillProfileTransparency
{
	CXfFillProfileTransparency( CXaraFileRecord & rec )
		: StartTransparency( rec ), EndTransparency( rec ), 
		TransparencyType( rec ), Profile( rec )
	{
	}

	const CXfBYTE StartTransparency;
	const CXfBYTE EndTransparency;
	const CXfBYTE TransparencyType;
	const CXfProfile Profile;
};

template< class Base >
class CXfTransparentFill : public Base, public CXfFillProfileTransparency
{
public:
	CXfTransparentFill( CXaraFileRecord & rec )
		: Base( rec ), CXfFillProfileTransparency( rec )
	{
	}

	virtual bool IsTransparentFill() { return true; }
};

typedef CXfTransparentFill< CXfLinearFillBase > CXfLinearTransparentFill;
typedef CXfTransparentFill< CXfLinearFill3PointBase > CXfLinearTransparentFill3Point;
typedef CXfTransparentFill< CXfCircularFillBase > CXfCircularTransparentFill;
typedef CXfTransparentFill< CXfEllipticalFillBase > CXfEllipticalTransparentFill;

class CXfBitmapFill : public VXFFillStyleNode
{
public:
	CXfBitmapFill( CXaraFileRecord & rec )
		: BottomLeft( rec ), BottomRight( rec ), 
		TopLeft( rec ), BitmapRef( rec ), Profile( rec ), pBitmapRefPtr(NULL), VXFFillStyleNode(TAG_BITMAPFILL)
	{
	}

	CXfBitmapFill(DocCoord bl, DocCoord br, DocCoord tl, VXfTreeNode* pDefine, double dBias, double dGain)
		: VXFFillStyleNode(TAG_BITMAPFILL), BottomLeft(bl), BottomRight(br), TopLeft(tl), pBitmapRefPtr(pDefine), Profile(dBias, dGain)
	{
	}

	virtual bool IsBitmapFill() { return true; }
	virtual std::string CreateDumpString();

	CXfDefineBitmap* GetReferenceBitmap(RefNodeMap& map)
	{
		ULONG id = BitmapRef.get();
		if(map.find(id) != map.end())
			return (CXfDefineBitmap*)map[id];
		return NULL;
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		if(pBitmapRefPtr) BitmapRef = pBitmapRefPtr->GetRef();
		BOOL ok = BottomLeft.Write(&rec);
		if (ok) ok = BottomRight.Write(&rec);
		if (ok) ok = TopLeft.Write(&rec);
		if (ok) ok = BitmapRef.Write(&rec);
		if (ok) ok = Profile.Write(&rec);
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	CXfDocCoord BottomLeft;
	CXfDocCoord BottomRight;
	CXfDocCoord TopLeft;
	CXfLONG BitmapRef;
	VXfTreeNode* pBitmapRefPtr;
	CXfProfile Profile;
};

class CXfBitmapTransparentFill : public VXFFillStyleNode
{
public:
	CXfBitmapTransparentFill( CXaraFileRecord & rec )
		: BottomLeft( rec ), BottomRight( rec ), TopLeft( rec ), 
		StartTransparency( rec ), EndTransparency( rec ), TransparencyType( rec ),
		BitmapRef( rec ), Profile( rec ), pBitmapRefPtr(0)
	{
	}

	CXfBitmapTransparentFill(DocCoord bl, DocCoord br, DocCoord tl, BYTE byStart, BYTE byEnd, BYTE byTransType, VXfTreeNode* pDefine, double dBias, double dGain)
		: VXFFillStyleNode(TAG_BITMAPTRANSPARENTFILL), BottomLeft(bl), BottomRight(br), TopLeft(tl), StartTransparency(byStart),
		EndTransparency(byEnd), TransparencyType(byTransType), pBitmapRefPtr(pDefine), Profile(dBias, dGain)
	{
		ValidateTransparencyType(byTransType);
	}
	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		if(pBitmapRefPtr) BitmapRef = pBitmapRefPtr->GetRef();
		BOOL ok = BottomLeft.Write(&rec);
		if (ok) ok = BottomRight.Write(&rec);
		if (ok) ok = TopLeft.Write(&rec);
		if (ok) ok = StartTransparency.Write(&rec);
		if (ok) ok = EndTransparency.Write(&rec);
		if (ok) ok = TransparencyType.Write(&rec);
		if (ok) ok = BitmapRef.Write(&rec);
		if (ok) ok = Profile.Write(&rec);
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	virtual bool IsTransparentFill() { return true; }
	CXfDocCoord BottomLeft;
	CXfDocCoord BottomRight;
	CXfDocCoord TopLeft;
	CXfBYTE StartTransparency;
	CXfBYTE EndTransparency;
	CXfBYTE TransparencyType;
	CXfLONG BitmapRef;
	VXfTreeNode* pBitmapRefPtr;
	CXfProfile Profile;
};

class CXfSoftMaskGroupTransparentFill : public CXfGroup
{
public:
	CXfSoftMaskGroupTransparentFill( CXaraFileRecord & rec )
		: BottomLeft( rec ), BottomRight( rec ), TopLeft( rec ), 
		StartTransparency( rec ), EndTransparency( rec ), TransparencyType( rec ),
		Flags( rec ), Profile( rec ), m_iRefCount(0)
	{
		m_Repeat = (RepeatType)(Flags & 0x3);
		m_bUseAlpha = Flags & 0x80 ? true : false;
		m_bInvert = Flags & 0x40 ? true : false;
		m_bClip = Flags & 0x20 ? true : false;
		m_bLinked = Flags & 0x10 ? true : false;
	}

	CXfSoftMaskGroupTransparentFill(DocCoord bl, DocCoord br, DocCoord tl, BYTE byStartTrans, BYTE byEndTrans, 
		BYTE byTransType, BYTE byFlags, double dBias, double dGain) : 
		BottomLeft(bl), BottomRight(br), TopLeft(tl), 
		StartTransparency(byStartTrans), EndTransparency(byEndTrans), TransparencyType(byTransType),
		Flags(byFlags), Profile(dBias, dGain), CXfGroup(TAG_SOFTMASKGROUP), m_iRefCount(0)
	{
		ValidateTransparencyType(byTransType);
		m_Repeat = (RepeatType)(Flags & 0x3);
		m_bUseAlpha = Flags & 0x80 ? true : false;
		m_bInvert = Flags & 0x40 ? true : false;
		m_bClip = Flags & 0x20 ? true : false;
		m_bLinked = Flags & 0x10 ? true : false;
	}

	CXfSoftMaskGroupTransparentFill(const CXfSoftMaskGroupTransparentFill& src) : 
			BottomLeft(src.BottomLeft), BottomRight(src.BottomRight), TopLeft(src.TopLeft), 
			StartTransparency(src.StartTransparency),
			EndTransparency(src.EndTransparency),
			TransparencyType(src.TransparencyType),
			Flags(src.Flags), Profile(src.Profile.Bias, src.Profile.Gain), CXfGroup(TAG_SOFTMASKGROUP), m_iRefCount(0)
	{
		m_Repeat = (RepeatType)(Flags & 0x3);
		m_bUseAlpha = Flags & 0x80 ? true : false;
		m_bInvert = Flags & 0x40 ? true : false;
		m_bClip = Flags & 0x20 ? true : false;
		m_bLinked = Flags & 0x10 ? true : false;
	}

	int GetRefCount() { return m_iRefCount; }
	int AddRefCount() { return ++m_iRefCount; }
	virtual bool IsSoftMaskGroup() { return true; }
	void SetRepeatType(RepeatType t) { m_Repeat = t; }
	RepeatType GetRepeatType() { return m_Repeat; }

	void SetUseAlpha(bool bUseAlpha) { m_bUseAlpha = bUseAlpha; }
	bool GetUseAlpha() const { return m_bUseAlpha; }

	void SetInvert(bool bInvert) { m_bInvert = bInvert; }
	bool GetInvert() const { return m_bInvert; }

	void SetClip(bool bClip) { m_bClip = bClip; }
	bool GetClip() const { return m_bClip; }

	void SetLinked(bool bLinked) { m_bLinked = bLinked; }
	bool GetLinked() const { return m_bLinked; }

	virtual VXfTreeNode* Copy() { return new CXfSoftMaskGroupTransparentFill(*this); }
	virtual bool IsTransparentFill() { return true; }
	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		BYTE byFlags = (BYTE)(m_Repeat & 0x3);
		if (m_bUseAlpha)
			byFlags |= 0x80;
		if (m_bInvert)
			byFlags |= 0x40;
		if (m_bClip)
			byFlags |= 0x20;
		if (m_bLinked)
			byFlags |= 0x10;

		Flags = byFlags;
		CXaraFileRecord rec(GetTag());
		rec.Init();
		BOOL ok = BottomLeft.Write(&rec);
		if (ok) ok = BottomRight.Write(&rec);
		if (ok) ok = TopLeft.Write(&rec);
		if (ok) ok = StartTransparency.Write(&rec);
		if (ok) ok = EndTransparency.Write(&rec);
		if (ok) ok = TransparencyType.Write(&rec);
		if (ok) ok = Flags.Write(&rec);
		if (ok) ok = Profile.Write(&rec);
		return ok && pExporter->WriteRecord(&rec, plRecordNum);
	}

	bool m_bUseAlpha;
	RepeatType m_Repeat;
	bool m_bInvert;
	bool m_bClip;
	bool m_bLinked;
	int m_iRefCount; // This is a 'usage' of a soft mask group count. Same soft mask could be used more than once, in this case
	// all consequent 'usage' must be done via copying a soft mask sub-tree

	CXfDocCoord BottomLeft;
	CXfDocCoord BottomRight;
	CXfDocCoord TopLeft;
	CXfBYTE StartTransparency;
	CXfBYTE EndTransparency;
	CXfBYTE TransparencyType;
	CXfBYTE Flags;
	CXfProfile Profile;
};
