/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Attributes
//

/********************************************************************************************
>	class CXfLineColor

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_LINECOLOR TAG_LINECOLOR_NONE TAG_LINECOLOR_BLACK TAG_LINECOLOR_WHITE
********************************************************************************************/
class CXfLineColor : public VXfTreeNode
{
public:

	enum Type { Reference=0, None, White, Black };
	CXfLineColor(CXaraFileRecord & rec)
		: ColorRef( rec ), type(Reference), pReference(0)
	{
	}

	CXfLineColor(ULONG uRef) : ColorRef(uRef), type(Reference), VXfTreeNode(TAG_LINECOLOUR), pReference(0) {}
	CXfLineColor(VXfTreeNode* pRef) : ColorRef(-1), type(Reference), VXfTreeNode(TAG_LINECOLOUR), pReference(pRef) {}
	CXfLineColor(Type t) : ColorRef(-1), pReference(0), type(t), VXfTreeNode(TAG_LINECOLOUR) {}
	CXfLineColor(const CXfLineColor& s) : ColorRef(s.ColorRef), pReference(s.pReference), type(s.type), VXfTreeNode(TAG_LINECOLOUR) {}
	virtual VXfTreeNode* Copy() { return new CXfLineColor(*this); }
	virtual std::string CreateDumpString();

	COLORREF GetRGBColor(RefNodeMap& map);
	virtual bool IsStrokeStyle() { return true; }
	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		static ULONG s_Tag[] = { TAG_LINECOLOUR, TAG_LINECOLOUR_NONE, TAG_LINECOLOUR_WHITE, TAG_LINECOLOUR_BLACK };
		if(type == Reference)
		{
			CXaraFileRecord Rec(TAG_LINECOLOUR);
			Rec.Init();
			// If we have a reference - get it from the actual node
			if(pReference) ColorRef = pReference->GetRef();
			BOOL ok = ColorRef.Write(&Rec);
			return pExporter->WriteRecord(&Rec, pRecordNum);
		}
		return (type >= None && type <= Black) ? pExporter->WriteZeroSizedRecord(s_Tag[type], pRecordNum) : FALSE;
	}

	Type	type;
	CXfLONG ColorRef;
	VXfTreeNode* pReference; // this one if for direct reference to a colour node, on writing step
};

/********************************************************************************************
>	class CXfLineTransparency

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_LINECOLOR TAG_LINECOLOR_NONE TAG_LINECOLOR_BLACK TAG_LINECOLOR_WHITE
********************************************************************************************/
class CXfLineTransparency : public VXfTreeNode
{
public:
	CXfLineTransparency( CXaraFileRecord & rec )
		: Transparency( rec ), TransparencyType( rec )
	{
	}

	CXfLineTransparency(BYTE bTrans, BYTE bType) : Transparency(bTrans), TransparencyType(bType), 
		VXfTreeNode(TAG_LINETRANSPARENCY)
	{
		ValidateTransparencyType(bType);
	}

	BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(TAG_LINETRANSPARENCY);
		rec.Init();
		Transparency.Write(&rec);
		TransparencyType.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfBYTE Transparency;
	CXfBYTE TransparencyType;
};


class CXfWindingRule : public VXfTreeNode
{
public:
	CXfWindingRule(CXaraFileRecord& rec) : Winding( rec ) {}
	CXfWindingRule(BYTE rule) : Winding(rule), VXfTreeNode(TAG_WINDINGRULE) {}
	virtual VXfTreeNode* Copy() { return new CXfWindingRule(Winding); }
	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		Winding.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfBYTE Winding;
};

class CXfLineWidth : public VXfTreeNode
{
public:
	CXfLineWidth (LONG lWidth) : Width(lWidth), VXfTreeNode(TAG_LINEWIDTH)
	{
	}

	CXfLineWidth( CXaraFileRecord & rec )
		: Width( rec )
	{
	}
	virtual VXfTreeNode* Copy() { return new CXfLineWidth(Width); }
	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(TAG_LINEWIDTH);
		rec.Init();
		Width.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	CXfLONG Width;
};

class CXfJoinStyle : public VXfTreeNode
{
public:
	CXfJoinStyle( CXaraFileRecord & rec ) : Style( rec ) {}
	CXfJoinStyle(BYTE byStyle) : Style(byStyle), VXfTreeNode(TAG_JOINSTYLE) 
	{
		assert(byStyle < 3); // Should be in range [0..2]: mitre=0, round=1, bevelled=2
	}
	CXfJoinStyle(ULONG uTag, BYTE byStyle) : Style(byStyle), VXfTreeNode(uTag) 
	{
		assert(byStyle < 3); // Should be in range [0..2]: butt=0, round=1, square=2
	}
	virtual VXfTreeNode* Copy() { return new CXfJoinStyle(GetTag(), Style); }
	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		Style.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfBYTE Style;
};

typedef CXfJoinStyle CXfStartCap;

class CXfMitreLimit : public VXfTreeNode
{
public:
	CXfMitreLimit( CXaraFileRecord & rec ) : Limit( rec ){}
	CXfMitreLimit(LONG lim) : Limit(lim), VXfTreeNode(TAG_MITRELIMIT) {}
	virtual VXfTreeNode* Copy() { return new CXfMitreLimit(Limit); }

	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* plRecordNum = NULL)
	{
		CXaraFileRecord rec(GetTag());
		rec.Init();
		Limit.Write(&rec);
		return pExporter->WriteRecord(&rec, plRecordNum);
	}

	CXfLONG Limit;
};

class CXfArrowHead : public VXfTreeNode
{
public:
	CXfArrowHead( CXaraFileRecord & rec )
		: ArrowId( rec ), ScaleWidth( rec ), ScaleHeight( rec )
	{
	}

	const CXfLONG ArrowId;
	const CXfFIXED16 ScaleWidth;
	const CXfFIXED16 ScaleHeight;
};
