/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XFTree.h"

/******************************************************************************
   Function:    CXfTreeNodeFactory::CreateTreeNode
   Author:      Paul Gruzdev
   Purpose:		Create a tree node if given type out of a Xara record stream
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
VXfTreeNode* CXfTreeNodeFactory::CreateTreeNode(CXaraFileRecord& rec)
{
	ULONG uTag = rec.GetTag();
	VXfTreeNode* pNode = NULL;

	switch(uTag)
	{
#define CREATE_NODE(__tag, __class)\
		case __tag:\
		pNode = new __class(rec);\
		break;
#define CREATE_NODE_PARAM(__tag, __class, __param)\
		case __tag:\
		pNode = new __class(__param);\
		break;
////////////////////////////////////////////////////////////////
		CREATE_NODE(TAG_SHADOW, CXfShadow);
		CREATE_NODE(TAG_SHADOWCONTROLLER, CXfDropShadowController);
		CREATE_NODE(TAG_VIEWPORT, CXfViewport);
		CREATE_NODE(TAG_SPREADINFORMATION, CXfSpreadInformation);
		CREATE_NODE(TAG_WEBSITE_PROPERTIES_EXPORT, CXfWebsitePropertiesExport);
		CREATE_NODE(TAG_WEBSITE_TRANSITION, CXfWebsiteTransition);
		CREATE_NODE(TAG_PRESENTATION_PAGE_PROPERTIES, CXfPresentationPageProperties);
		CREATE_NODE(TAG_PRESENTATION_LAYER_PROPERTIES, CXfPresentationLayerProperties);
		CREATE_NODE(TAG_IMPLICIT_TAB_DISTANCE, CXfImplicitTabDistance);
		CREATE_NODE(TAG_LAYER, CXfLayer);
		CREATE_NODE(TAG_LAYERDETAILS, CXfLayerDetails);
		CREATE_NODE(TAG_PRESENTATION_LAYER_FLAG, CXfPresentationLayerFlag);
		CREATE_NODE(TAG_GRIDRULERSETTINGS, CXfGridRulerSettings);
		CREATE_NODE(TAG_GRIDRULERORIGIN, CXfGridRulerOrigin);

		CREATE_NODE(TAG_WEBADDRESS, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_BOUNDINGBOX, CXfWebAddressBoundingBox);
		CREATE_NODE(TAG_WEBADDRESS_PAGE, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_PAGE_BOUNDINGBOX, CXfWebAddressBoundingBox);
		CREATE_NODE(TAG_WEBADDRESS_OBJECT, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_OBJECT_BOUNDINGBOX, CXfWebAddressBoundingBox);
		CREATE_NODE(TAG_WEBADDRESS_LAYER, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_LAYER_BOUNDINGBOX, CXfWebAddressBoundingBox);
		CREATE_NODE(TAG_WEBADDRESS_SCALEDPOPUP, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_SCALEDPOPUP_BOUNDINGBOX, CXfWebAddressBoundingBox);
		CREATE_NODE(TAG_WEBADDRESS_FILE, CXfWebAddress);
		CREATE_NODE(TAG_WEBADDRESS_FILE_BOUNDINGBOX, CXfWebAddressBoundingBox);

		CREATE_NODE(TAG_WIZOP, CXfWizop);
		CREATE_NODE(TAG_RECTANGLE_SIMPLE, CXfRect);
		CREATE_NODE(TAG_ELLIPSE_SIMPLE, CXfRect);
		CREATE_NODE(TAG_RECTANGLE_SIMPLE_ROUNDED, CXfRectRounded);
		CREATE_NODE(TAG_RECTANGLE_COMPLEX, CXfRectComplex);
		CREATE_NODE(TAG_ELLIPSE_COMPLEX, CXfEllipseComplex);
		CREATE_NODE(TAG_RECTANGLE_COMPLEX_ROUNDED, CXfRectComplexRounded);

		case TAG_PATH:
		case TAG_PATH_RELATIVE:
			pNode = new CXfPath(rec);
			break;
		case TAG_PATH_FILLED:
		case TAG_PATH_RELATIVE_FILLED:
			pNode = new CXfPath(rec, true, false);
			break;
		case TAG_PATH_STROKED:
		case TAG_PATH_RELATIVE_STROKED:
			pNode = new CXfPath(rec, false, true);
			break;
		case TAG_PATH_FILLED_STROKED:
		case TAG_PATH_RELATIVE_FILLED_STROKED:
			pNode = new CXfPath(rec, true, true);
			break;

		CREATE_NODE(TAG_CLIPVIEW_PATH, CXfClipViewPath);
		CREATE_NODE(TAG_NODE_BITMAP, CXfBitmap);
		CREATE_NODE(TAG_BITMAP_PROPERTIES, CXfBitmapProperties);

		CREATE_NODE(TAG_CLIPVIEWCONTROLLER, CXfClipviewController);
		CREATE_NODE(TAG_GROUP, CXfGroup);
		CREATE_NODE(TAG_OBJECTBOUNDS, CXfObjectBounds);
		////////////////////////////////////////////////////
		// Bitmap records
		case TAG_DEFINEBITMAP_JPEG:
		case TAG_DEFINEBITMAP_JPEG8BPP:
		case TAG_DEFINEBITMAP_PNG:
		case TAG_DEFINEBITMAP_PNG_REAL:
		case TAG_DEFINEBITMAP_BMP:
			pNode = new CXfDefineBitmap(rec);
			break;
		////////////////////////////////////////////////////
		// Attribute nodes
		CREATE_NODE(TAG_DEFINERGBCOLOUR, CXfSimpleColor);
		CREATE_NODE(TAG_DEFINECOMPLEXCOLOUR, CXfComplexColor);
		CREATE_NODE(TAG_DEFINEDASH, CXfDefineDash);
		CREATE_NODE(TAG_DEFINEDASH_SCALED, CXfDefineDash);
		CREATE_NODE(TAG_WINDINGRULE, CXfWindingRule);
		CREATE_NODE(TAG_LINECOLOUR, CXfLineColor);
		CREATE_NODE(TAG_LINEWIDTH, CXfLineWidth);
		CREATE_NODE(TAG_JOINSTYLE, CXfJoinStyle);
		CREATE_NODE(TAG_STARTCAP, CXfStartCap);
		CREATE_NODE(TAG_MITRELIMIT, CXfMitreLimit);
		CREATE_NODE(TAG_DASHSTYLE, CXfDashStyle);
		CREATE_NODE(TAG_ARROWHEAD, CXfArrowHead);
		CREATE_NODE(TAG_ARROWTAIL, CXfArrowHead);

		CREATE_NODE_PARAM(TAG_LINECOLOUR_NONE, CXfLineColor, CXfLineColor::None);
		CREATE_NODE_PARAM(TAG_LINECOLOUR_WHITE, CXfLineColor, CXfLineColor::White);
		CREATE_NODE_PARAM(TAG_LINECOLOUR_BLACK, CXfLineColor, CXfLineColor::Black);
		CREATE_NODE(TAG_SYMBOLGROUP, CXfSymbolGroup);
		CREATE_NODE(TAG_SYMBOLGROUP_LIVECOPY, CXfSymbolGroup);
		CREATE_NODE(TAG_USERVALUE, CXfUserValue);
		CREATE_NODE(TAG_NODEDETAILS, CXfNodeDetails);
		/////////////////////////////////////////////////////
		// Text nodes
		CREATE_NODE(TAG_FONT_DEF_TRUETYPE, CXfFontDef);
		CREATE_NODE(TAG_FONT_DEF_ATM, CXfFontDef);
		CREATE_NODE(TAG_TEXT_EXTRA_TT_FONT_DEF, CXfExtraFontDef);
		CREATE_NODE(TAG_TEXT_EXTRA_ATM_FONT_DEF, CXfExtraFontDef);
		CREATE_NODE(TAG_TEXT_EXTRA_FONT_INFO, CXfExtraFontInfo);

		CREATE_NODE(TAG_TEXT_LANGUAGE, CXfTextLanguage);
		CREATE_NODE(TAG_TEXT_STORY_WORD_WRAP_INFO, CXfTextStoryWordWrapInfo);
		CREATE_NODE(TAG_TEXT_STORY_LINK_INFO, CXfTextStoryLinkInfo);
		CREATE_NODE(TAG_TEXT_FONT_TYPEFACE, CXfFontTypeface);
		CREATE_NODE(TAG_TEXT_FONT_TYPEFACE_SHADOWED, CXfFontTypeface);
		CREATE_NODE(TAG_TEXT_STORY_SIMPLE, CXfTextStorySimple);
		CREATE_NODE(TAG_TEXT_STORY_COMPLEX, CXfTextStoryComplex);
		CREATE_NODE(TAG_TEXT_STORY_INSIDE_SIMPLE, CXfTextStoryInsideSimple);
		CREATE_NODE(TAG_TEXT_STORY_INSIDE_COMPLEX, CXfTextStoryInsideComplex);
		CREATE_NODE(TAG_TEXT_LINE, CXfTextLine); // Store text context for the text line as a whole
		CREATE_NODE(TAG_TEXT_LINE_INFO, CXfTextLineInfo);
		CREATE_NODE(TAG_TEXT_LINE_INFO2, CXfTextLineInfo2);
		CREATE_NODE(TAG_TEXT_CHAR, CXfTextString);
		CREATE_NODE(TAG_TEXT_STRING, CXfTextString);
		CREATE_NODE(TAG_TEXT_STRING_POS, CXfTextStringPos);
		CREATE_NODE(TAG_TEXT_STRING_POS2, CXfTextStringPos2);
		CREATE_NODE(TAG_TEXT_SYMBOL_CHAR_POS, CXfTextCharPos);
		CREATE_NODE(TAG_TEXT_SYMBOL_CHAR_POS2, CXfTextCharPos2);
		CREATE_NODE(TAG_TEXT_CHANGE_CAPS, CXfTextChangeCaps);
		CREATE_NODE(TAG_TEXT_EOL, CXfTextEOL);
		CREATE_NODE(TAG_TEXT_BREAK_TYPE, CXfTextBreakType);
		CREATE_NODE(TAG_TEXT_TAB, CXfTextTab);
		CREATE_NODE(TAG_TEXT_SOFT_RETURN, CXfTextSoftReturn);
		CREATE_NODE(TAG_TEXT_ASPECT_RATIO, CXfAspectRatio);
		CREATE_NODE(TAG_TEXT_STORY_HEIGHT_INFO, CXfHeightInfo);
		CREATE_NODE(TAG_TEXT_STORY_TRANSLATION_INFO, CXfTextStoryTranslationInfo);
		CREATE_NODE(TAG_TEXT_FONT_SIZE, CXfTextFontSize);
		CREATE_NODE(TAG_TEXT_KERN, CXfKerning);
		CREATE_NODE(TAG_TEXT_LINESPACE_ABSOLUTE, CXfLinespaceAbsolute);
		CREATE_NODE(TAG_TEXT_LINESPACE_LEADING, CXfLinespaceLeading);
		CREATE_NODE(TAG_TEXT_LINESPACE_RATIO, CXfLinespaceRatio);
		CREATE_NODE(TAG_TEXT_BASELINE, CXfTextBaseline);
		CREATE_NODE(TAG_TEXT_TRACKING, CXfTextTracking);
		CREATE_NODE(TAG_TEXT_SCRIPT_ON, CXfTextScriptOn);
		CREATE_NODE(TAG_TEXT_RULER, CXfTextRuler);
		CREATE_NODE(TAG_TEXT_SYMBOL_CHAR, CXfTextSymbolChar);
		CREATE_NODE(TAG_FONT_EMBEDDING_POLICY, CXfFontEmbeddingPolicy);

		CREATE_NODE(TAG_TEXT_SPACE_BEFORE, CXfTextSpaceBefore)
		CREATE_NODE(TAG_TEXT_SPACE_AFTER, CXfTextSpaceAfter)
		CREATE_NODE(TAG_TEXT_FIRST_INDENT, CXfTextFirstIndent)
		CREATE_NODE(TAG_TEXT_LEFT_INDENT, CXfTextLeftIndent)
		CREATE_NODE(TAG_TEXT_RIGHT_INDENT, CXfTextRightIndent)

		CREATE_NODE(TAG_TEXT_BULLETED_ITEM_START, CXfTextBulletedItemStart)
		CREATE_NODE(TAG_TEXT_NUMBERED_ITEM_START, CXfTextNumberedItemStart)

		CREATE_NODE(TAG_REPEL_DISTANCE, CXfRepelDistance)
		CREATE_NODE(TAG_REPEL_DISTANCE_V, CXfRepelDistanceV)
		CREATE_NODE(TAG_STYLE_DEF, CXfStyleDef)
		CREATE_NODE(TAG_STYLE, CXfStyle)
		CREATE_NODE(TAG_TEXT_PARAGRAPH_STYLE, CXfTextParagraphStyle)
		CREATE_NODE(TAG_DOCUMENT_URL_COLOUR, CXfDocumentURLColour)
		CREATE_NODE(TAG_DOCUMENT_VISITED_URL_COLOUR, CXfDocumentVisitedURLColour)
		CREATE_NODE(TAG_DOCUMENT_MOUSE_OVER_URL_COLOUR, CXfDocumentMouseOverURLColour)
		CREATE_NODE(TAG_TXTBACKGROUNDCOLOUR, CXfTxtBackgroundColour)
		CREATE_NODE(TAG_TXT_PARA_BACKGROUNDCOLOUR, CXfTxtParaBackgroundColour)
		CREATE_NODE(TAG_TXT_PARA_TOP_BORDER, CXfTxtParaBorder)
		CREATE_NODE(TAG_TXT_PARA_LEFT_BORDER, CXfTxtParaBorder)
		CREATE_NODE(TAG_TXT_PARA_RIGHT_BORDER, CXfTxtParaBorder)
		CREATE_NODE(TAG_TXT_PARA_BOTTOM_BORDER, CXfTxtParaBorder)
		CREATE_NODE(TAG_TXT_PARA_INTERNAL_BORDER, CXfTxtParaBorder)
		CREATE_NODE(TAG_DOCUMENTFLAGS, CXfDocumentFlags)
		CREATE_NODE(TAG_TEXT_PRESENT_BY_PARAGRAPH, CXfTextPresentByParagraph)

		CREATE_NODE(TAG_TEXT_ANCHOR_POSITION, CXfTextAnchorPosition)
		CREATE_NODE(TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO, CXfTextEmbeddedAnchorPosInfo)
		CREATE_NODE(TAG_ANCHORED_GROUP_INFO, CXfAnchoredGroupInfo)
		CREATE_NODE(TAG_ANCHORED_GROUP_INFO2, CXfAnchoredGroupInfo2)
		CREATE_NODE(TAG_ANCHORING_MODE, CXfAnchoringMode)
		CREATE_NODE(TAG_EMBEDDED_ANCHORED_GROUP_INFO, CXfEmbeddedAnchoredGroupInfo)

		CREATE_NODE(TAG_EXTRUSIONCONTROLLER_COMPLEX, CXfExtrusionControllerComplex);
		CREATE_NODE(TAG_EXTRUSIONCONTROLLER_TRANS, CXfExtrusionControllerTrans);
		////////////////////////////////////////////////////
		// Fills
		CREATE_NODE(TAG_FLATFILL, CXfFlatFill);
		CREATE_NODE_PARAM(TAG_FLATFILL_NONE, CXfFlatFill, CXfFlatFill::None);
		CREATE_NODE_PARAM(TAG_FLATFILL_BLACK, CXfFlatFill, CXfFlatFill::Black);
		CREATE_NODE_PARAM(TAG_FLATFILL_WHITE, CXfFlatFill, CXfFlatFill::White);
		CREATE_NODE(TAG_LINEARFILL, CXfLinearFill);
		CREATE_NODE(TAG_LINEARFILLMULTISTAGE, CXfLinearFillMultiStage);
		CREATE_NODE(TAG_LINEARFILL3POINT, CXfLinearFill3Point);
		CREATE_NODE(TAG_LINEARFILLMULTISTAGE3POINT, CXfLinearFill3PointMultiStage);
		CREATE_NODE(TAG_CIRCULARFILL, CXfCircularFill);
		CREATE_NODE(TAG_CIRCULARFILLMULTISTAGE, CXfCircularFillMultiStage);
		CREATE_NODE(TAG_ELLIPTICALFILL, CXfEllipticalFill);
		CREATE_NODE(TAG_ELLIPTICALFILLMULTISTAGE, CXfEllipticalFillMultiStage);
		CREATE_NODE(TAG_BITMAPFILL, CXfBitmapFill);
		CREATE_NODE(TAG_BITMAPBOUNDSFILL, CXfBitmapFill);
		CREATE_NODE(TAG_LINETRANSPARENCY, CXfLineTransparency);
		CREATE_NODE(TAG_FLATTRANSPARENTFILL, CXfFlatTransparentFill);
		CREATE_NODE(TAG_LINEARTRANSPARENTFILL, CXfLinearTransparentFill);
		CREATE_NODE(TAG_LINEARTRANSPARENTFILL3POINT, CXfLinearTransparentFill3Point);
		CREATE_NODE(TAG_CIRCULARTRANSPARENTFILL, CXfCircularTransparentFill);
		CREATE_NODE(TAG_ELLIPTICALTRANSPARENTFILL, CXfEllipticalTransparentFill);
		CREATE_NODE(TAG_BITMAPTRANSPARENTFILL, CXfBitmapTransparentFill);
		CREATE_NODE(TAG_SOFTMASKGROUP, CXfSoftMaskGroupTransparentFill);
		CREATE_NODE(TAG_SMARTSHAPEGROUP, CXfSmartShapeGroup);
		CREATE_NODE(TAG_FONT_GLYPH, CXfFontGlyph);
		CREATE_NODE(TAG_FONT_GLYPH_EXTRA_WEIGHT, CXfFontGlyph);
		CREATE_NODE(TAG_FONT_KERNINGPAIRS, CXfFontKerningPairs);
		CREATE_NODE(TAG_FONT_KERNINGPAIRS_EXTRA_WEIGHT, CXfFontKerningPairs);
		CREATE_NODE(TAG_TEXT_OPEN_TYPE_LIGATURES, CXfTextOpenTypeLigatures);
		CREATE_NODE(TAG_FONT_OTLIGATURES, CXfFontOTLigatures);
		CREATE_NODE(TAG_TEXT_OPEN_TYPE_STYLISTICSETS, CXfTextOpenTypeStylisticSets);
		CREATE_NODE(TAG_FONT_OTSTYLISTICSETS, CXfFontOTStylisticSets);
		CREATE_NODE(TAG_FONT_OTLIGATURES_EXT, CXfFontOTLigatures);
		CREATE_NODE(TAG_TEXT_LIGATURE_START, CXfTextLigatureStart);
		CREATE_NODE(TAG_SPREADINFORMATION_EXTRA, CXfGenericStringNode); // This is a generic class. More concrete class should be placed above this line
		CREATE_NODE(TAG_FONT_OTSTYLISTICSETNAMES, CXfFontOTStylisticSetNames);
		CREATE_NODE(TAG_FONT_OTSUBSTITUTIONRULES, CXfFontOTSubstitutionRules);
		CREATE_NODE(TAG_FONT_OTSUBSTITUTIONTABLES, CXfFontOTSubstitutionTables);
		CREATE_NODE(TAG_FONT_OTRULEMAXSIZES, CXfFontOTRuleMaxSizes);
		///////////////////////////////////////////////////////////////////////

		default:
			pNode = new CXfGenericNode(rec);
	}

	if(pNode)
	{
		pNode->SetTag(uTag);
		pNode->SetRef(rec.GetRecordNumber());
	}

	return pNode;
}
