/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

/********************************************************************************************
>	class CXfNodeValue

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Proxy class that represents a LONG value
********************************************************************************************/
class CXfNodeValue : public VXfTreeNode
{
public:
	CXfNodeValue(ULONG ulTag, LONG val) : VXfTreeNode(ulTag), Value(val), pReference(0) {}
	CXfNodeValue(ULONG ulTag, VXfTreeNode* pRef) : VXfTreeNode(ulTag), Value(0), pReference(pRef) {}
	CXfNodeValue(CXaraFileRecord& rec) : Value(rec), pReference(0) {}
	LONG GetValue() { return Value.get(); }

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum)
	{
		CXaraFileRecord Rec(GetTag());
		BOOL ok = Rec.Init();
		if (pReference) Value = pReference->GetRef();
		if (ok) ok = Rec.WriteLONG(Value);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}
	CXfLONG Value;
	VXfTreeNode* pReference;
	virtual std::string CreateDumpString();
};

#define DEFINE_VALUE_CLASS(wrapper_name, tag)	\
class wrapper_name : public CXfNodeValue	\
{											\
public:										\
	wrapper_name() : CXfNodeValue(tag,(LONG)0){}		\
	wrapper_name(const LONG& def) : CXfNodeValue(tag, def) {} \
	wrapper_name(VXfTreeNode* pRef) : CXfNodeValue(tag, pRef) {} \
	wrapper_name(CXaraFileRecord& rec) : CXfNodeValue(rec) {}	\
	virtual VXfTreeNode* Copy() { return new wrapper_name(Value); }   \
};

DEFINE_VALUE_CLASS(CXfHeightInfo, TAG_TEXT_STORY_HEIGHT_INFO)
DEFINE_VALUE_CLASS(CXfTextFontSize, TAG_TEXT_FONT_SIZE)
DEFINE_VALUE_CLASS(CXfTextBaseline, TAG_TEXT_BASELINE)
DEFINE_VALUE_CLASS(CXfTextTracking, TAG_TEXT_TRACKING)
DEFINE_VALUE_CLASS(CXfLinespaceAbsolute, TAG_TEXT_LINESPACE_ABSOLUTE)
DEFINE_VALUE_CLASS(CXfLinespaceLeading, TAG_TEXT_LINESPACE_LEADING)

DEFINE_VALUE_CLASS(CXfTextSpaceBefore, TAG_TEXT_SPACE_BEFORE)
DEFINE_VALUE_CLASS(CXfTextSpaceAfter, TAG_TEXT_SPACE_AFTER)
DEFINE_VALUE_CLASS(CXfTextFirstIndent, TAG_TEXT_FIRST_INDENT)
DEFINE_VALUE_CLASS(CXfTextLeftIndent, TAG_TEXT_LEFT_INDENT)
DEFINE_VALUE_CLASS(CXfTextRightIndent, TAG_TEXT_RIGHT_INDENT)

DEFINE_VALUE_CLASS(CXfStyle, TAG_STYLE)
DEFINE_VALUE_CLASS(CXfTextParagraphStyle, TAG_TEXT_PARAGRAPH_STYLE)

DEFINE_VALUE_CLASS(CXfDocumentURLColour, TAG_DOCUMENT_URL_COLOUR)
DEFINE_VALUE_CLASS(CXfDocumentVisitedURLColour, TAG_DOCUMENT_VISITED_URL_COLOUR)
DEFINE_VALUE_CLASS(CXfDocumentMouseOverURLColour, TAG_DOCUMENT_MOUSE_OVER_URL_COLOUR)
DEFINE_VALUE_CLASS(CXfTxtBackgroundColour, TAG_TXTBACKGROUNDCOLOUR)
DEFINE_VALUE_CLASS(CXfTxtParaBackgroundColour, TAG_TXT_PARA_BACKGROUNDCOLOUR)

DEFINE_VALUE_CLASS(CXfRepelDistance, TAG_REPEL_DISTANCE)
DEFINE_VALUE_CLASS(CXfRepelDistanceV, TAG_REPEL_DISTANCE_V)
DEFINE_VALUE_CLASS(CXfTextOpenTypeStylisticSets, TAG_TEXT_OPEN_TYPE_STYLISTICSETS)
