/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

const double dSQRT2 = 1.414213562373;

/********************************************************************************************
>	class CXfRect

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_RECTANGLE_SIMPLE tag
********************************************************************************************/
class CXfRect : public VXfRenderable
{
public:
	CXfRect( CXaraFileRecord & rec )
		: Center( rec ), RX( rec ), RY( rec )
	{
	}
	CXfRect(LONG cx, LONG cy, LONG wx, LONG wy)
		:Center(DocCoord(cx,cy)),RX(wx),RY(wy)
	{
		SetTag(TAG_RECTANGLE_SIMPLE);
		LONG w = (LONG)((double)wx/dSQRT2);
		LONG h = (LONG)((double)wy/dSQRT2);
		SetBounds(Rect(cx-w/2, cy-h/2, w, h));
	}

	virtual std::string CreateDumpString();

	virtual bool IsSimple() { return true; }
	virtual bool IsRounded() { return false; }
	virtual bool IsRectangle() { return GetTag() == TAG_RECTANGLE_SIMPLE || GetTag() == TAG_RECTANGLE_SIMPLE_ROUNDED; }
	virtual bool IsEllipse() { return GetTag() == TAG_ELLIPSE_SIMPLE; }
	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId /* = NULL */)
	{
		CXaraFileRecord Rec(TAG_RECTANGLE_SIMPLE);
		Rec.Reinit(TAG_RECTANGLE_SIMPLE, 4*4);
		Center.Write(&Rec);
		RX.Write(&Rec);
		RY.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfDocCoord Center;
	CXfLONG RX, RY;
};

/********************************************************************************************
>	class CXfRectRounded

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	represents TAG_RECTANGLE_ROUNDED
********************************************************************************************/
class CXfRectRounded: public CXfRect
{
public:
	CXfRectRounded( CXaraFileRecord & rec )
		: CXfRect( rec ), Roundness( rec )
	{
	}
	virtual bool IsSimple() { return true; }
	virtual bool IsRounded() { return true; }

	const CXfdouble Roundness;
};

/********************************************************************************************
>	class CXfRectComplex

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_RECTANGLE_COMPLEX
********************************************************************************************/
class CXfRectComplex : public VXfRenderable
{
public:
	CXfRectComplex( CXaraFileRecord & rec )
		: Center( rec ), RX( rec ), RY( rec )
	{
	}
	virtual bool IsSimple() { return false; }
	virtual bool IsRounded() { return false; }

	virtual std::string CreateDumpString();

	virtual bool IsRectangle() { return GetTag() == TAG_RECTANGLE_COMPLEX || GetTag() == TAG_RECTANGLE_COMPLEX_ROUNDED; }
	virtual bool IsEllipse() { return GetTag() == TAG_ELLIPSE_COMPLEX; }

	const CXfDocCoord Center;
	const CXfDocCoord RX, RY;
};

typedef CXfRectComplex CXfEllipseComplex;

/********************************************************************************************
>	class CXfRectComplexRounded

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_RECTANGLE_COMPLEX_ROUNDED
********************************************************************************************/
class CXfRectComplexRounded : public CXfRectComplex
{
public:
	CXfRectComplexRounded( CXaraFileRecord & rec )
		: CXfRectComplex( rec ), Roundness( rec )
	{
	}
	virtual bool IsSimple() { return false; }
	virtual bool IsRounded() { return true; }

	const CXfdouble Roundness;
};

/********************************************************************************************
>	class CXfPath

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents all path creating tags
********************************************************************************************/
class CXfPath : public VXfRenderable
{
public:
	CXfPath(CXaraFileRecord& rec, bool f=false, bool s=false);
	CXfPath(UINT uTag, PBYTE pVerbs=0, DocCoord* pCoords=0, DWORD dwCount=0);
	CXfPath(const CXfPath& other);

	virtual VXfTreeNode* Copy() { return new CXfPath(*this); }
	virtual bool IsPath() { return true; }
	bool IsStroked() { return bStroked; }
	bool IsFilled() { return bFilled; }
	bool TestIsStroked();
	bool TestIsFilled();
	bool IsSimpleRectangle();
	bool IsSimpleOrComplexRectangle();
	void CopyPoints(const CXfPath* pSrc);
	void CopyPoints(PBYTE pVerbs, DocCoord* pCoords, ULONG ulNumCoords);
	bool IsEqualTo(CXfPath* pOther);
	void clear() { Verbs.clear(); Coords.clear(); Bounds().clear(); }
	size_t size() { return Verbs.size(); }
	void Add(BYTE byVerb, DocCoord dc)
	{
		Verbs.push_back(byVerb);
		Coords.push_back(dc);
		Rect::InflateRectToContainPoint(Bounds(), dc.x, dc.y);
	}

	virtual std::string CreateDumpString();
	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum /* = NULL */);

	typedef std::vector<BYTE> CVerbs;
	CVerbs Verbs;
	typedef std::vector<DocCoord> CCoords;
	CCoords Coords;
	bool bFilled;
	bool bStroked;
};

typedef CXfPath CXfClipViewPath;
