/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

/********************************************************************************************
>	class CXfShadow

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Read/write TAG_SHADOW shadow profile from Xara stream
********************************************************************************************/
class CXfShadow : public VXfRenderable
{
public:
	CXfShadow(double b, double g, double d) : Profile(b, g), Darkness(d) { SetTag(TAG_SHADOW); }

	CXfShadow( CXaraFileRecord & rec )
		: Profile( rec ), Darkness( rec )
	{
	}

	std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_SHADOW);
		Rec.Init();
		BOOL ok = Profile.Write(&Rec);
		ok = Darkness.Write(&Rec);
		return pExporter->WriteRecord(&Rec);
	}

	CXfProfile Profile;
	CXfdouble Darkness;
};

/********************************************************************************************
>	class CXfDropShadowController

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Read/write a grouped shadow controller
********************************************************************************************/
class CXfDropShadowController : public VXfRenderable
{
public:
	enum { WallShadow=1, FloorShadow=2, GlowShadow=3 };
	CXfDropShadowController(BYTE bType, LONG lBlur, DocCoord dcOff, LONG lAngle, LONG lHeight, LONG lScale, LONG lWidth) :
	  Type(bType), Blur(lBlur), Offset(dcOff), FloorAngle(lAngle), FloorHeight(lHeight), WallScale(lScale), GlowWidth(lWidth)
	  {
		  SetTag(TAG_SHADOWCONTROLLER);
	  }

	  CXfDropShadowController( CXaraFileRecord & rec )
		  : Type(rec), Blur(rec), Offset(rec), FloorAngle(rec), FloorHeight(rec), WallScale(rec), GlowWidth(rec)
	  {
	  }

	  std::string CreateDumpString();

	  CXfShadow* GetShadowNode()
	  {
		  return (CXfShadow*)FindFirstNode(TAG_SHADOW, false);
	  }
	  BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	  {
		  CXaraFileRecord Rec(TAG_SHADOWCONTROLLER);
		  Rec.Init();
		  BOOL ok = Type.Write(&Rec);
		  ok = Blur.Write(&Rec);
		  ok = Offset.Write(&Rec);
		  ok = FloorAngle.Write(&Rec);
		  ok = FloorHeight.Write(&Rec);
		  ok = WallScale.Write(&Rec);
		  ok = GlowWidth.Write(&Rec);

		  return pExporter->WriteRecord(&Rec, pRecordNum);
	  }

	  CXfBYTE Type;
	  CXfLONG Blur;
	  CXfDocCoord Offset;
	  CXfLONG FloorAngle;
	  CXfLONG FloorHeight;
	  CXfLONG WallScale;
	  CXfLONG GlowWidth;
};
