/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XFTree.h"

/******************************************************************************
   Function:	CXfFontDef::CXfFontDef
   Author:		Paul Gruzdev
   Purpose:     construct on export - out of Xara file record
   Returns:		
   Parameters:	CXaraFileRecord & rec  
   Notes:       
******************************************************************************/
CXfFontDef::CXfFontDef(CXaraFileRecord& rec) : FullFontName(rec), TypeFaceName(rec), m_bExternal(false), m_bEmbedded(false)
{
	memset(m_pPairs, 0, 4 * sizeof(CXfFontKerningPairs*));
	memset(m_pLigatures, 0, 4 * sizeof(CXfFontOTLigatures*));
	memset(m_pStylisticSets, 0, 4 * sizeof(CXfFontOTStylisticSets*));
	memset(m_pSubstRules, 0, 4 * sizeof(CXfFontOTSubstitutionRules*));
	memset(m_pSubstTables, 0, 4 * sizeof(CXfFontOTSubstitutionTables*));
	memset(m_pRuleMaxSizes, 0, 4 * sizeof(CXfFontOTRuleMaxSizes*));

	BOOL ok = rec.ReadBuffer( PANOSE, 10 );
	if (rec.BytesLeft() == 0) // Old-format records end here
		return;

	ok = ok && rec.ReadBYTE(&RealPanoseWeight);
	ok = ok && rec.ReadBYTE(&RealPanoseLetterform);
	ok = ok && rec.ReadINT16(&weight);

	if (rec.BytesLeft() == 0)
		ok = FALSE;

	ok = ok && rec.ReadUINT16(&Flags);

	if (ok && (Flags & MultiWeight && Flags & WeightList))
	{
		for(INT16 w = 0; w != -1;)
		{
			ok = ok && rec.ReadINT16(&w);
			if (ok && w != -1)
			{
				UINT16 VariantFlags;
				ok = ok && rec.ReadUINT16(&VariantFlags);
				AddWeightRecord(w, VariantFlags & 1 ? true : false);
			}
			else w = -1;
		}
	}
	else if(ok && (Flags & AlternativeWeight) && ~(Flags & MultiWeight))
		rec.ReadINT16(&altweight);
}

/******************************************************************************
   Function:	CXfFontDef::Write
   Author:		Paul Gruzdev
   Purpose:     Write to Xara tree (import)
   Returns:		BOOL
   Parameters:	CXarExport * pExporter  
   Parameters:	ULONG * pulTagId  
   Notes:       
******************************************************************************/
BOOL CXfFontDef::Write(CXarExport* pExporter, ULONG* pulTagId/* = NULL*/)
{
	CXaraFileRecord Rec(TAG_FILEHEADER);
	Rec.Reinit(TAG_FONT_DEF_TRUETYPE, TAG_FONT_DEF_TRUETYPE_SIZE);
	FullFontName.Write(&Rec);
	TypeFaceName.Write(&Rec);

	BYTE panose[10];
	memcpy(panose, PANOSE, 10);
	panose[2] = 6; // Xar format doc: For compatibility with program versions prior to 9.0 this should always have Weight=Medium
	panose[7] = 2; // For compatibility with program versions prior to 9.0 this should always have Letterform=Normal.
	Rec.WriteBuffer(panose, 10);

	Rec.WriteBYTE(PANOSE[2]); // RealPanoseWeight
	Rec.WriteBYTE(PANOSE[7]); // RealPanoseLetterform

	if (weight > 0)
	{
		Rec.WriteINT16(weight);
		//UINT16 flags = 0x0002;
		Rec.WriteINT16(Flags);

		// Temp auto - hack - we add a weight list automatically here, if we actually have italic glyphs
		// Update: Camelot does not read these
/*
		if(!m_ItalicGlyph.empty())
			AddWeightRecord(weight, true);
		else
			AddWeightRecord(weight, false);
*/
		if(m_WeightList.size())
		{
			for(size_t i = 0; i < m_WeightList.size(); i++)
			{
				Rec.WriteINT16(m_WeightList[i].Weight);
				Rec.WriteUINT16(m_WeightList[i].VariantFlags);
			}
			Rec.WriteUINT16(-1); // write weight list terminator
		}
		// Alternative weight is the second [high] weight of a non-multiweight font
		if(altweight > 0 && (Flags & AlternativeWeight) && ~(Flags & MultiWeight))
			Rec.WriteINT16(altweight);
	}

	//std::wcout << " +font:" << FullFontName.getstr() <<" ";
	return pExporter->WriteRecord(&Rec, pulTagId);
}

/******************************************************************************
   Function:	CXfFontDef::GetGlyph
   Author:		Paul Gruzdev
   Purpose:     Get glyph from font def given its Unicode and bold/italic modifiers
   Returns:		CXfFontGlyph*
   Parameters:	ULONG uCode  
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
CXfFontGlyph* CXfFontDef::GetGlyph(ULONG uCode, bool bBold, bool bItalic)
{ 
	GlyphMap& map = GetGlyphMap(bBold, bItalic);
	return map.find(uCode) != map.end() ? map[uCode] : NULL;
}

/******************************************************************************
   Function:	CXfFontDef::AddGlyph
   Author:		Paul Gruzdev
   Purpose:     Add glyph to one of four maps
   Returns:		void
   Parameters:	ULONG unicode  
   Parameters:	CXfFontGlyph * pGlyph  
   Parameters:	bool bAddToTree  
   Notes:       
******************************************************************************/
void CXfFontDef::AddGlyph(ULONG unicode, CXfFontGlyph* pGlyph, bool bAddToTree)
{
	// Now, find the proper place either next to the font def record, or next to the last glyph
	if(bAddToTree)
	{
		GlyphMap& glMap = GetGlyphMap(pGlyph->isBold(), pGlyph->isItalic());

		if(glMap.empty())
			GetParent()->InsertAfterChild(this, pGlyph);
		else
		{
			CXfFontGlyph* pInsertAfter = glMap.rbegin()->second;
			pInsertAfter->GetParent()->InsertAfterChild(pInsertAfter, pGlyph);
		}
	}

	GlyphMap& map = GetGlyphMap(pGlyph->isBold(), pGlyph->isItalic());
	map[unicode] = pGlyph;
}

/******************************************************************************
   Function:	CXfFontDef::UpdateEmbeddingPolicy
   Author:		Denis Onishchenko
   Purpose:     
   Returns:		void
   Parameters:	bool bBold
   Parameters:	bool bItalic
   Parameters:	UINT16 flags
   Notes:       
******************************************************************************/
void CXfFontDef::UpdateEmbeddingPolicy(bool bBold, bool bItalic, UINT16 flags)
{
	UpdateEmbeddingPolicy(BI_CODE(bBold, bItalic), flags);
}

/******************************************************************************
   Function:	CXfFontDef::UpdateEmbeddingPolicy
   Author:		Denis Onishchenko
   Purpose:     
   Returns:		void
   Parameters:	UINT16 glyphFlags
   Parameters:	UINT16 flags
   Notes:       
******************************************************************************/
void CXfFontDef::UpdateEmbeddingPolicy(UINT16 glyphFlags, UINT16 flags)
{
	auto it = m_EmbeddingPolicy.find(glyphFlags);
	if (m_EmbeddingPolicy.end() != it)
		it->second->Flags = ((UINT16)it->second->Flags) | flags;
	else
	{
		CXfFontEmbeddingPolicy *pEmbeddingPolicy = new CXfFontEmbeddingPolicy(flags, glyphFlags);
		m_EmbeddingPolicy[glyphFlags] = pEmbeddingPolicy;
		if (VXfTreeNode *pParent = GetParent())
			pParent->InsertAfterChild(this, pEmbeddingPolicy);
	}
}

/******************************************************************************
   Function:	CXfFontDef::GetEmbeddingPolicy
   Author:		Denis Onishchenko
   Purpose:     
   Returns:		UINT16
   Parameters:	bool bBold
   Parameters:	bool bItalic
   Notes:       
******************************************************************************/
UINT16 CXfFontDef::GetEmbeddingPolicy(bool bBold, bool bItalic)
{
	auto it = m_EmbeddingPolicy.find(BI_CODE(bBold, bItalic));

	if (m_EmbeddingPolicy.end() != it)
		return it->second->Flags;

	return 0;
}

/******************************************************************************
   Function:	CXfFontDef::RemoveGlyph
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		void
   Parameters:	ULONG unicode  
   Parameters:	CXfFontGlyph * pGlyph  
   Parameters:	bool bRemoveFromTree  
   Notes:       
******************************************************************************/
void CXfFontDef::RemoveGlyph(ULONG unicode, CXfFontGlyph* pGlyph, bool bRemoveFromTree)
{
	GlyphMap& glMap = GetGlyphMap(pGlyph->isBold(), pGlyph->isItalic());
	if (glMap.find(unicode) != glMap.end())
	{
		glMap.erase(unicode);
		if (bRemoveFromTree && pGlyph->GetParent())
			pGlyph->GetParent()->RemoveChild(pGlyph, true);
	}
}

void CXfFontDef::MarkGlyphsRemoved(bool bBold, bool bItalic)
{
	GlyphMap& glMap = GetGlyphMap(bBold, bItalic);
	for(GlyphMap::iterator it = glMap.begin(); it != glMap.end(); ++it)
		it->second->SetRemoved(true);
	// clear the map also
	glMap.clear();
}
/******************************************************************************
   Function:	CXfFontDef::CalcGlyphVerbCount
   Author:		Paul Gruzdev
   Purpose:     For Camelot glyph hint - calc all verbs count
   Returns:		size_t
   Notes:       
******************************************************************************/
size_t CXfFontDef::CalcGlyphVerbCount()
{
	size_t iCount = 0;
	for (size_t i = 0; i < 4; ++i)
	{
		for (GlyphMap::iterator it = m_Glyphs[i].begin(); it != m_Glyphs[i].end(); ++it)
			iCount += it->second->Verbs.size();
	}

	return iCount;
}

/******************************************************************************
   Function:	CXfFontDef::DeleteGlyphs
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		void
   Notes:       
******************************************************************************/
void CXfFontDef::DeleteGlyphs()
{
	for (size_t i = 0; i < 4; ++i)
	{
		for (GlyphMap::iterator it = m_Glyphs[i].begin(); it != m_Glyphs[i].end(); ++it)
			delete it->second;
	}
}

void CXfFontDef::DeleteKerningPairs()
{
	for (size_t i = 0; i < 4; ++i)
	{
		if (m_pPairs[i])
		{
			delete m_pPairs[i];
			m_pPairs[i] = 0;
		}
	}
}

void CXfFontDef::DeleteLigatures()
{
	for (size_t i = 0; i < 4; ++i)
	{
		if (m_pLigatures[i])
		{
			delete m_pLigatures[i];
			m_pLigatures[i] = 0;
		}
	}
}

void CXfFontDef::DeleteStylisticSets()
{
	for (size_t i = 0; i < 4; ++i)
	{
		if (m_pStylisticSets[i])
		{
			delete m_pStylisticSets[i];
			m_pStylisticSets[i] = 0;
		}
	}
}

void CXfFontDef::DeleteEmbeddingPolicies()
{
	for (auto it = m_EmbeddingPolicy.begin(); it != m_EmbeddingPolicy.end(); ++it)
		delete it->second;
}

/******************************************************************************
   Function:	CXfFontDef::GetGlyphMap
   Author:		Paul Gruzdev
   Purpose:     Get variant glyph map
   Returns:		GlyphMap&
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
GlyphMap& CXfFontDef::GetGlyphMap(bool bBold, bool bItalic)
{
	return m_Glyphs[BI_CODE(bBold, bItalic)];
}

/******************************************************************************
   Function:	CalcSubsetChecksum
   Author:		Paul Gruzdev
   Purpose:		Calc a simple check-sum of all unicodes in the current subset:
				u1*1 + u2*2 + u3*3 + u4*4...
   Returns:		DWORD
   Parameters:	bool bBold
   Parameters:	bool bItalic
   Notes:       
******************************************************************************/
DWORD CXfFontDef::CalcSubsetChecksum(bool bBold, bool bItalic)
{
	DWORD dwRes = 0, dwCnt = 1;
	GlyphMap& map = GetGlyphMap(bBold, bItalic);
	for (GlyphMap::iterator it = map.begin(); it != map.end(); ++it)
		dwRes += (DWORD)it->second->CharCode * dwCnt++;
	return dwRes;
}

/******************************************************************************
   Function:	CXfFontDef::GetKerningPairs
   Author:		Paul Gruzdev
   Purpose:     Get variant kerning pairs associtated record
   Returns:		CXfFontKerningPairs*
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
CXfFontKerningPairs*& CXfFontDef::GetFontKerningPairs(bool bBold, bool bItalic)
{
	return m_pPairs[BI_CODE(bBold, bItalic)];
}

CXfFontOTLigatures*& CXfFontDef::GetFontLigatures(bool bBold, bool bItalic)
{
	return m_pLigatures[BI_CODE(bBold, bItalic)];
}

/******************************************************************************
   Function:	CXfFontDef::AddKerningPairs
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		void
   Parameters:	CXfFontKerningPairs * pPairs  
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
void CXfFontDef::AddKerningPairs(CXfFontKerningPairs* pPairs)
{
	GetFontKerningPairs(pPairs->IsBold(), pPairs->IsItalic()) = pPairs;
}

/******************************************************************************
   Function:	CXfFontDef::GetGlyphs
   Author:		Paul Gruzdev
   Purpose:     Get all glyphs of specified variants into an array of glyphs
   Returns:		bool
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
bool CXfFontDef::GetGlyphs(std::vector<CXfFontGlyph*>& glyphs, bool bBold, bool bItalic)
{
	GlyphMap& map = GetGlyphMap(bBold, bItalic);
	for(GlyphMap::iterator it = map.begin(); it != map.end(); ++it)
		glyphs.push_back(it->second);
	return glyphs.size() > 0;
}

/******************************************************************************
   Function:	CXfExtraFontDef::CXfExtraFontDef
   Author:		Paul Gruzdev
   Purpose:     constructor
   Returns:		
   Parameters:	CXaraFileRecord & rec  
   Notes:       
******************************************************************************/
CXfExtraFontDef::CXfExtraFontDef(CXaraFileRecord& rec) :
			Flags(rec),
			FullFontName(rec),
			FullTypefaceName(rec),
			TypefaceName(rec),
			Weight(rec),
			Flags2(0)
{
	if(Flags & EXTRA_FONT_DEF_REPLACED)
		OriginalFontName.Read(rec);

	BOOL ok = TRUE;
	if(Flags & EXTRA_FONT_DEF_MULTI_WEIGHT)
		ok = OriginalBaseWeight.Read(&rec);

	if(Flags & EXTRA_FONT_DEF_REPLACED && Flags & EXTRA_FONT_DEF_MULTI_WEIGHT)
		ok = ok && ReplacedWeight.Read(&rec);

	if(Flags & EXTRA_FONT_DEF_FLAGS2)
		ok = ok && Flags2.Read(&rec);

	if(Flags2 & EXTRA_FONT_DEF_GOOGLE_FONT)
		GoogleFontFamily.Read(rec);

	if(Flags2 & EXTRA_FONT_DEF_REPLACED_GOOGLE_FONT && Flags & EXTRA_FONT_DEF_REPLACED)
		ReplacedGoogleFontFamily.Read(rec);
}

/******************************************************************************
   Function:	CXfTextRuler::formRulerWithTabsGreaterThanValue
   Author:		Ekaterina Dolgunicheva
   Purpose:     Fill the text ruler only with tabs which values are greater than the first line indent of the line
   Returns:		
   Parameters:	CXfTextRuler* newRuler   
   Parameters:	LONG value   
   Notes:       
******************************************************************************/
void CXfTextRuler::formRulerWithTabsGreaterThanValue(CXfTextRuler* newRuler, LONG indentValue) 
{
	if (!Stops.size()) return;
	for(size_t i = 0; i < Stops.size(); i++)
	{
		if (Stops[i].Position > indentValue) newRuler->Stops.push_back(Stops[i]);
	}
}

/******************************************************************************
   Function:	CXfTextLine::AddChild
   Author:		Paul Gruzdev
   Purpose:     This virtual also increments the number of child sub-strings
				to utilise later
   Returns:		void
   Parameters:	VXfTreeNode * pChild  
   Parameters:	bool bUpdateBBox  
   Notes:       
******************************************************************************/
void CXfTextLine::AddChild(VXfTreeNode* pChild, bool bUpdateBBox)
{
	if (pChild->IsTextString() || pChild->GetTag() == TAG_TEXT_CHAR)
	{
		m_uiStringCount++;
		// Increment the story counter too
		if (GetParent() && (GetParent()->IsTextStorySimple() || GetParent()->IsTextStoryComplex()))
			((CXfTextStorySimple*)GetParent())->m_uiStringCount++;
	}

	VXfTextRenderable::AddChild(pChild, bUpdateBBox);
}

/******************************************************************************
Function:	CXfTextString::AddChild
Author:		Paul Gruzdev
Purpose:     Virtual subst to count the number of URL sub-strings
Returns:		void
Parameters:	VXfTreeNode * pChild
Parameters:	bool bUpdateBBox
Notes:
******************************************************************************/
void CXfTextString::AddChild(VXfTreeNode* pChild, bool bUpdateBBox)
{
	if (pChild->IsWebAddress() && GetParent())
	{
		CXfTextLine* pLine = (CXfTextLine*)GetParent();
		CXfTextStorySimple* pStory = (CXfTextStorySimple*)pLine->GetParent();
		pLine->m_uiURLCount++;
		if (pStory) pStory->m_uiURLCount++;
	}

	VXfTextRenderable::AddChild(pChild, bUpdateBBox);
}
