/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

#include <iostream>
#include <windows.h>
#include "../GSUBEntry.h"
#include "../../../fontlib/otfTools.h"

#define BI_CODE(bBold, bItalic) (bBold ? 1 : 0) + (bItalic ? 2 : 0)

// we could use TAG_NOT_IN_DOC for the NextInFlowTag field that is written out, but
// that could, in theory, change, so use this value instead to indicate "no successor"
// also used to terminate the AnchorPosition record tag list
#define TEXTSTORY_IMPORT_TAG_NOT_IN_DOC (0xFFFFFFFFUL)

//
// Text
//

class CXfTextStoryWordWrapInfo : public VXfTreeNode
{
public:
	CXfTextStoryWordWrapInfo(LONG width, BYTE flags = 1) : BlockWidth(width), Flags(flags) 
	{
		SetTag(TAG_TEXT_STORY_WORD_WRAP_INFO);
	}

	CXfTextStoryWordWrapInfo(CXaraFileRecord& rec) : BlockWidth(rec), Flags(rec) 
	{
	}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_WORD_WRAP_INFO, TAG_TEXT_STORY_WORD_WRAP_INFO_SIZE);
		Rec.Init();
		BlockWidth.Write(&Rec);
		Flags.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfLONG BlockWidth;
	CXfBYTE Flags;
};

// TAG_TEXT_STORY_LINK_INFO
class CXfTextStoryLinkInfo : public VXfTreeNode
{
public:
	CXfTextStoryLinkInfo(CXaraFileRecord& rec) : Flags(rec), NextInFlowID(rec), ThisID(rec) {}

	CXfTextStoryLinkInfo(BYTE flags, LONG nextInFlowID, LONG thisID) : Flags(flags), NextInFlowID(nextInFlowID), ThisID(thisID)
	{
		SetTag(TAG_TEXT_STORY_LINK_INFO);
	}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_LINK_INFO, TAG_TEXT_STORY_LINK_INFO_SIZE);
		Rec.Init();
		Flags.Write(&Rec);
		NextInFlowID.Write(&Rec);
		ThisID.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBYTE Flags;
	CXfLONG NextInFlowID;
	CXfLONG ThisID;
};

class CXfTextLanguage : public VXfTreeNode
{
public:
	CXfTextLanguage(CXaraFileRecord& rec) : LangCode(rec), CountryCode(rec) {}
	virtual std::string CreateDumpString();

	CXfULONG LangCode;
	CXfULONG CountryCode;
};

class CXfTextStoryTranslationInfo : public VXfTreeNode
{
public:
	CXfTextStoryTranslationInfo(CXaraFileRecord& rec) : Position(rec) {}
	virtual std::string CreateDumpString();

	CXfDocCoord Position;
};

class CXfKerning : public VXfTextRenderable
{
public:
	CXfKerning(DocCoord k)
		:Kern(k)
	{
		SetTag(TAG_TEXT_KERN);
	}

	CXfKerning(CXaraFileRecord& rec) : Kern(rec)
	{
	}

	virtual std::string CreateDumpString();
	virtual bool IsTextKern() { return true; }

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_KERN, 2*4);
		Rec.Init();
		Kern.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfDocCoord Kern;
};

class CXfAspectRatio : public CXfFloatNode
{
public:
	CXfAspectRatio(const FIXED16 &aspect) :CXfFloatNode(aspect)
	{
		SetTag(TAG_TEXT_ASPECT_RATIO);
	}
	CXfAspectRatio(CXaraFileRecord& rec) : CXfFloatNode(rec){}
	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_ASPECT_RATIO);
		Rec.Reinit(TAG_TEXT_ASPECT_RATIO, 4);
		Value.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}
};

class CXfLinespaceRatio : public CXfFloatNode
{
public:
	CXfLinespaceRatio(const FIXED16 &ratio) :CXfFloatNode(ratio)
	{
		SetTag(TAG_TEXT_LINESPACE_RATIO);
	}

	CXfLinespaceRatio(CXaraFileRecord& rec) : CXfFloatNode(rec){}

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_LINESPACE_RATIO);
		Rec.Reinit(TAG_TEXT_LINESPACE_RATIO, 4);
		Value.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}
};

/********************************************************************************************
>	class CXfTextStorySimple

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	TAG_TEXT_STORY_SIMPLE node
********************************************************************************************/
class CXfTextStorySimple : public VXfTextRenderable
{
public:
	CXfTextStorySimple() : m_pPrevLink(0), m_pNextLink(0), m_dwMainTextBoxID(0), m_uiStringCount(0), m_uiURLCount(0)
	{ SetTag(TAG_TEXT_STORY_SIMPLE); }
	CXfTextStorySimple(CXaraFileRecord & rec)
		: Anchor(rec), AutoKern(rec), m_pPrevLink(0), m_pNextLink(0), m_dwMainTextBoxID(0), m_uiStringCount(0), m_uiURLCount(0)
	{
	}
	CXfTextStorySimple(LONG x, LONG y, LONG lAutoKern) : VXfTextRenderable(TAG_TEXT_STORY_SIMPLE), Anchor(DocCoord(x, y)), AutoKern(lAutoKern) {}
	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_SIMPLE);
		Rec.Reinit(TAG_TEXT_STORY_SIMPLE, TAG_TEXT_STORY_SIMPLE_SIZE);
		Anchor.Write(&Rec);
		AutoKern.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	virtual bool IsTextStory() { return true; }
	virtual bool IsTextStorySimple() { return true; }
	virtual bool IsRefNode() { return true; }
	UINT GetStringCount() { return m_uiStringCount; }
	UINT GetURLCount() { return m_uiURLCount; }

	CXfDocCoord Anchor;
	CXfLONG AutoKern;
	CXfTextStorySimple* m_pPrevLink;
	CXfTextStorySimple* m_pNextLink;
	DWORD m_dwMainTextBoxID;
	UINT m_uiStringCount;
	UINT m_uiURLCount;
};

/********************************************************************************************
>	class CXfTextStoryInsideSimple

Author:		Ekaterina Dolgunicheva
Created:	11/01/18
Purpose:	TAG_TEXT_STORY_INSIDE_SIMPLE node
********************************************************************************************/
class CXfTextStoryInsideSimple : public VXfTextRenderable
{
public:
	CXfTextStoryInsideSimple() : m_pPrevLink(0), m_pNextLink(0), m_dwMainTextBoxID(0), m_uiStringCount(0), m_uiURLCount(0)
	{
		SetTag(TAG_TEXT_STORY_INSIDE_SIMPLE);
	}
	CXfTextStoryInsideSimple(CXaraFileRecord & rec)
		: Anchor(rec), AutoKern(rec), PaddingForText(rec), m_pPrevLink(0), m_pNextLink(0), m_dwMainTextBoxID(0), m_uiStringCount(0), m_uiURLCount(0)
	{
	}
	CXfTextStoryInsideSimple(LONG x, LONG y, LONG lAutoKern, LONG padding) : VXfTextRenderable(TAG_TEXT_STORY_INSIDE_SIMPLE), Anchor(DocCoord(x, y)), AutoKern(lAutoKern), PaddingForText(padding) {}
	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_INSIDE_SIMPLE);
		Rec.Reinit(TAG_TEXT_STORY_INSIDE_SIMPLE, TAG_TEXT_STORY_INSIDE_SIMPLE_SIZE);
		Anchor.Write(&Rec);
		AutoKern.Write(&Rec);
		PaddingForText.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	virtual bool IsTextStory() { return true; }
	virtual bool IsTextStorySimple() { return true; }
	virtual bool IsRefNode() { return true; }
	UINT GetStringCount() { return m_uiStringCount; }
	UINT GetURLCount() { return m_uiURLCount; }

	CXfDocCoord Anchor;
	CXfLONG AutoKern;
	CXfLONG PaddingForText;
	CXfTextStorySimple* m_pPrevLink;
	CXfTextStorySimple* m_pNextLink;
	DWORD m_dwMainTextBoxID;
	UINT m_uiStringCount;
	UINT m_uiURLCount;
};

/********************************************************************************************
>	class CXfTextStoryComplex

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	TAG_TEXT_STORY_COMPLEX node
********************************************************************************************/
class CXfTextStoryComplex : public CXfTextStorySimple
{
public:
	CXfTextStoryComplex(const Matrix &m = Matrix(), LONG autoKern = 0)
		:Transform(m)
	{
		SetTag(TAG_TEXT_STORY_COMPLEX);
		AutoKern = autoKern;
	}

	CXfTextStoryComplex(CXaraFileRecord & rec) : Transform(rec)
	{
		AutoKern.Read(&rec);
		Matrix m = Transform;
		DocCoord dc;
		m.GetTranslation(dc);
		Anchor = dc;
	}
	virtual std::string CreateDumpString();

	virtual bool IsTextStory() { return true; }
	virtual bool IsTextStoryComplex() { return true; }
	bool IsTextStoryActuallySimple();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_COMPLEX);
		Rec.Reinit(TAG_TEXT_STORY_COMPLEX, 24+4);
		Transform.Write(&Rec);
		AutoKern.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfMatrix Transform;
};

/********************************************************************************************
>	class CXfTextStoryInsideComplex

Author:		Ekaterina Dolgunicheva
Created:	11/01/18
Purpose:	TAG_TEXT_STORY_INSIDE_COMPLEX node
********************************************************************************************/
class CXfTextStoryInsideComplex : public CXfTextStoryInsideSimple
{
public:
	CXfTextStoryInsideComplex(const Matrix &m = Matrix(), LONG autoKern = 0, LONG padding = 0)
		:Transform(m)
	{
		SetTag(TAG_TEXT_STORY_INSIDE_COMPLEX);
		AutoKern = autoKern;
		PaddingForText = padding;
	}

	CXfTextStoryInsideComplex(CXaraFileRecord & rec) : Transform(rec)
	{
		AutoKern.Read(&rec);
		PaddingForText.Read(&rec);
		Matrix m = Transform;
		DocCoord dc;
		m.GetTranslation(dc);
		Anchor = dc;
	}

	virtual bool IsTextStory() { return true; }
	virtual bool IsTextStoryComplex() { return true; }
	bool IsTextStoryActuallySimple();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STORY_INSIDE_COMPLEX, TAG_TEXT_STORY_INSIDE_COMPLEX_SIZE);
		Rec.Init();
		Transform.Write(&Rec);
		AutoKern.Write(&Rec);
		PaddingForText.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfMatrix Transform;
};

/********************************************************************************************
>	class CXfTextChangeCaps

Author:		Ekaterina Dolgunicheva
Created:	29/06/18
Purpose:	TAG_TEXT_CHANGE_CAPS node
********************************************************************************************/
class CXfTextChangeCaps : public VXfTextRenderable
{
public:
	enum CapsType {
		AS_TYPED = 0,
		ALL_CAPS = 1,
		ALL_LOWER = 2,
		INITIAL_CAPS = 3,
		SENTENCE_CASE = 4,
		SMALL_CAPS = 5
	};

	CXfTextChangeCaps(CXaraFileRecord& rec) : Type(rec)
	{
	}
	virtual std::string CreateDumpString();

	CXfBYTE Type;
};

/********************************************************************************************
>	class CXfStyleDef

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	TAG_STYLE_DEF node. It should be based on text renderable in order to accumulate
			all the text styles right inside
********************************************************************************************/
class CXfStyleDef : public VXfTextRenderable
{
public:
	enum StyleType {
		NO_STYLE,				// used for uninitialised style objects
		PARAGRAPH_STYLE,		// a style that is meant to be applied to complete text paragraphs
		CHARACTER_STYLE,		// a style that is meant to be applied to arbitrary text runs
		OBJECT_STYLE			// a style that can be applied to arbitrary objects
	};

	CXfStyleDef() {}
	CXfStyleDef(CXaraFileRecord& rec) :
		Type(rec),
		Flags(rec),
		Name(rec),
		HTMLTag(rec),
		BaseStyle(rec),
		NextParaStyle(rec),
		OutlineLevel(rec)
	{
	}

	CXfStyleDef(BYTE type, ULONG flags, std::wstring name, std::wstring htmltag, LONG basestyle, LONG nextparastyle, 
		LONG outlinelevel) : 
		Type(type),
		Flags(flags),
		Name(name),
		HTMLTag(htmltag),
		BaseStyle(basestyle),
		NextParaStyle(nextparastyle),
		OutlineLevel(outlinelevel)
		{
		}

	CXfStyleDef* FindParentStyle(RefNodeMap& map);
	virtual bool IsRefNode() { return true; }
	virtual bool IsStyleDef() { return true; }

	bool IsLanguageDefined() { return FindFirstNode(TAG_TEXT_LANGUAGE, false) != NULL; }
	bool IsTypefaceDefined() { return FindFirstNode(TAG_TEXT_FONT_TYPEFACE, false) != NULL; }
	bool IsFontSizeDefined() { return FindFirstNode(TAG_TEXT_FONT_SIZE, false) != NULL; }
	bool IsFlatFillDefined() { return FindFirstNode(TAG_FLATFILL, false) != NULL; }
	bool IsTrackingDefined() { return FindFirstNode(TAG_TEXT_TRACKING, false) != NULL; }
	bool IsSuperscriptDefined() { return FindFirstNode(TAG_TEXT_SUPERSCRIPT_ON, false) != NULL; }
	bool IsSubcriptDefined() { return FindFirstNode(TAG_TEXT_SUBSCRIPT_ON, false) != NULL; }
	bool IsScriptDefined() { return FindFirstNode(TAG_TEXT_SCRIPT_ON, false) != NULL; }
	bool IsAspectRatioDefined() { return FindFirstNode(TAG_TEXT_ASPECT_RATIO, false) != NULL; }
	bool IsBaselineShiftDefined() { return FindFirstNode(TAG_TEXT_BASELINE, false) != NULL; }
	bool IsLigatureDefined() { return FindFirstNode(TAG_TEXT_OPEN_TYPE_LIGATURES, false) != NULL; }
	bool IsStylisticSetDefined() { return FindFirstNode(TAG_TEXT_OPEN_TYPE_STYLISTICSETS, false) != NULL; }

	bool IsBoldOnDefined() { return FindFirstNode(TAG_TEXT_BOLD_ON, false) != NULL; }
	bool IsBoldOffDefined() { return FindFirstNode(TAG_TEXT_BOLD_OFF, false) != NULL; }

	bool IsItalicOnDefined() { return FindFirstNode(TAG_TEXT_ITALIC_ON, false) != NULL; }
	bool IsItalicOffDefined() { return FindFirstNode(TAG_TEXT_ITALIC_OFF, false) != NULL; }

	bool IsUnderlineOnDefined() { return FindFirstNode(TAG_TEXT_UNDERLINE_ON, false) != NULL; }
	bool IsUnderlineOffDefined() { return FindFirstNode(TAG_TEXT_UNDERLINE_OFF, false) != NULL; }

	bool IsStrikeThroughOnDefined() { return FindFirstNode(TAG_TEXT_STRIKETHROUGH_ON, false) != NULL; }
	bool IsStrikeThroughOffDefined() { return FindFirstNode(TAG_TEXT_STRIKETHROUGH_OFF, false) != NULL; }

	//bool IsBackgroundColourDefined() { return FindFirstNode(TAG_TXTBACKGROUNDCOLOUR, false) != NULL; }
	bool IsBackgroundColourDefined() { return TextContext().m_ulBackgroundColourRef != 0; }


	virtual std::string CreateDumpString();

	CXfBYTE Type;
	CXfULONG Flags;
	CXfString Name;
	CXfString HTMLTag;
	CXfLONG BaseStyle;
	CXfLONG NextParaStyle;
	CXfLONG OutlineLevel;
};

/********************************************************************************************
>	class CXfExtrusionControllerComplex

Author:		Ekaterina Dolgunicheva
Created:	02/06/2015
Purpose:	TAG_EXTRUSIONCONTROLLER_COMPLEX node
********************************************************************************************/
class CXfExtrusionControllerComplex : public CXfFloatNode
{
public:
	CXfExtrusionControllerComplex(CXaraFileRecord & rec) : Transform(rec)
	{
	}

	virtual std::string CreateDumpString();

	CXfMatrix Transform;
};

/********************************************************************************************
>	class CXfExtrusionControllerTrans

Author:		Ekaterina Dolgunicheva
Created:	02/06/2015
Purpose:	TAG_EXTRUSIONCONTROLLER_TRANS node
********************************************************************************************/
class CXfExtrusionControllerTrans : public CXfFloatNode
{
public:
	CXfExtrusionControllerTrans( CXaraFileRecord & rec )
		: Angle( rec )
	{
	}

	virtual std::string CreateDumpString();

	CXfLONG Angle;
};

class CXfTextLine : public VXfTextRenderable
{
public:
	CXfTextLine(CXaraFileRecord& rec) : m_uiStringCount(0), m_uiURLCount(0) {}
	CXfTextLine() : m_uiStringCount(0), m_uiURLCount(0) { SetTag(TAG_TEXT_LINE); }

	virtual void AddChild(VXfTreeNode* pChild, bool bUpdateBBox = true);
	UINT GetStringCount() { return m_uiStringCount; }
	UINT GetURLCount() { return m_uiURLCount; }
	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_LINE);
		Rec.Reinit(TAG_TEXT_LINE, 0);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}
	UINT m_uiStringCount;
	UINT m_uiURLCount;
};

class CXfTextLineInfo : public VXfTreeNode
{
public:
	CXfTextLineInfo( CXaraFileRecord & rec )
		: Width( rec ), Height( rec ), Offset( rec )
	{
	}

	std::string CreateDumpString();

	CXfULONG Width;
	CXfULONG Height;
	CXfLONG Offset;
};

class CXfTextLineInfo2 : public VXfTreeNode
{
public:
	CXfTextLineInfo2(CXaraFileRecord & rec) : Left(rec), Right(rec), LineAscent(rec), LineDescent(rec)
	{
	}

	std::string CreateDumpString();

	CXfLONG Left;	// left hand edge of line (untransformed)
	CXfLONG Right;	// right hand edge of line (untransformed)
	CXfLONG LineAscent;	// line ascent (space reserved above baseline)
	CXfLONG LineDescent;	// line descent (space reserved below baseline)};
};

class CXfTextScriptOn : public VXfTreeNode
{
public:
	CXfTextScriptOn(const FIXED16 &fontSize, const FIXED16 &baselineOffset)
		: VXfTreeNode(TAG_TEXT_SCRIPT_ON), ScriptFontSize(fontSize), ScriptOffset(baselineOffset)
	{}

	CXfTextScriptOn( CXaraFileRecord & rec ) : ScriptOffset(rec), ScriptFontSize(rec)
	{
	}

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_SCRIPT_ON, 2*4);
		Rec.Init();
		ScriptOffset.Write(&Rec);
		ScriptFontSize.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	std::string CreateDumpString();

	CXfFIXED16 ScriptOffset;
	CXfFIXED16 ScriptFontSize;
};

class CXfTextRuler : public VXfRenderable
{
public:
	enum
	{
		LeftTab=0x0,	// Left align tab stop.
		RightTab=0x1,	// Right align tab stop.
		CentreTab=0x2,	// Centre align tab stop.
		DecimalTab=0x3	// Decimal tab stop.
	};

	CXfTextRuler(CXaraFileRecord& rec);
	CXfTextRuler(): VXfRenderable(TAG_TEXT_RULER) {}

	bool HasStops() { return Stops.size() > 0; }

	std::string CreateDumpString();

	class CTabStop
	{
	public:
		CTabStop(BYTE typeAndFlags = 0, LONG position = 0, UINT16 decimalChar = 0, UINT16 fillerChar = 0):
			TypeAndFlags(typeAndFlags), Position(position), DecimalChar(decimalChar), FillerChar(fillerChar) {}

		BYTE TypeAndFlags;
		LONG Position;
		UINT16 DecimalChar;
		UINT16 FillerChar;

		UINT16 GetType() { return TypeAndFlags & 3; }
		bool IsDecimalTab() { return GetType() == DecimalTab; }
		bool HasFillerChar() { return (TypeAndFlags & 4) != 0; }
	};

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_RULER);
		Rec.Init();

		UINT16 count = (UINT16)Stops.size();
		Rec.WriteUINT16(count);
		for (UINT16 i = 0; i < count; ++i)
		{
			Rec.WriteBYTE(Stops[i].TypeAndFlags);
			Rec.WriteLONG(Stops[i].Position);

			if (Stops[i].IsDecimalTab())
				Rec.WriteUINT16(Stops[i].DecimalChar);
			if (Stops[i].HasFillerChar())
				Rec.WriteUINT16(Stops[i].FillerChar);
		}

		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	void formRulerWithTabsGreaterThanValue(CXfTextRuler* newRuler, LONG indentValue); 

	typedef std::vector< CTabStop > CStops;
	CStops Stops;
};

#define FG_BOLD		0x01				// BIT(0)
#define FG_ITALIC	0x02				// BIT(1)
#define FG_HAS_INTERNAL_LEADING	0x04	//BIT(2)
#define FG_HAS_EXTERNAL_LEADING	0x08	//BIT(3)
/********************************************************************************************
>	class CXfFontGlyph

Author:		Paul Gruzdev
Created:	19/09/13
Purpose:	Represents TAG_FONT_GLYPH tag
********************************************************************************************/
class CXfFontGlyph : public VXfBounded
{
public:
	CXfFontGlyph(CXaraFileRecord& rec);
	CXfFontGlyph(ULONG uFlags, UINT16 uCharCode, LONG lWidth, LONG lFontAscent, LONG lFontDescent, LONG lFontEMWidth) 
		: VXfBounded(TAG_FONT_GLYPH), Flags(uFlags), CharCode(uCharCode), Width(lWidth), FontAscent(lFontAscent), FontDescent(lFontDescent),
		FontEMWidth(lFontEMWidth) {}

	void AddPoint(BYTE verb, DocCoord coord);
	void AddPoint(BYTE verb, LONG x, LONG y) { AddPoint(verb, DocCoord(x, y)); }
	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL);
	virtual std::string CreateDumpString();
	bool isBold() { return Flags & FG_BOLD ? true : false; }
	bool isItalic() { return Flags & FG_ITALIC ? true : false; }
	void setBold(bool b) { Flags =  b ? (Flags.get() | FG_BOLD) : (Flags.get() & ~FG_BOLD); }
	PBYTE GetVerbs() { return Verbs.empty() ? 0 : &Verbs[0]; }
	DocCoord* GetCoords() { return Points.empty() ? 0 : &Points[0]; }
	ULONG NumCoords() { return (ULONG)Verbs.size(); }

	std::string Dump();
	virtual VXfTreeNode* Copy() {
		CXfFontGlyph* pGlyph = new CXfFontGlyph(Flags, CharCode, Width, FontAscent, FontDescent, FontEMWidth);
		pGlyph->Points = Points;
		pGlyph->Verbs = Verbs;
		pGlyph->FontInternalLeading = FontInternalLeading;
		pGlyph->FontExternalLeading = FontExternalLeading;
		return pGlyph;
	}

	CXfULONG Flags;
	CXfUINT16 CharCode;
	CXfLONG Width;
	CXfLONG FontAscent;
	CXfLONG FontDescent;
	CXfLONG FontEMWidth;
	std::vector<DocCoord> Points;
	std::vector<BYTE> Verbs;
	CXfLONG FontInternalLeading;
	CXfLONG FontExternalLeading;
};

/********************************************************************************************
>	class CXfFontKerningPairs

Author:		Paul Gruzdev
Created:	19/09/13
Purpose:	Represents TAG_FONT_KERNINGPAIRS tag
********************************************************************************************/
class CXfFontKerningPairs : public VXfTreeNode
{
public:
	class KerningPair
	{
	public:
		KerningPair() : First(0), Second(0), Amount(0) {}
		KerningPair(CXaraFileRecord& rec) : First(rec), Second(rec), Amount(rec) {}
		KerningPair(UINT16 f, UINT16 s, LONG a) : First(f), Second(s), Amount(a) {}

		BOOL Write(CXaraFileRecord* rec)
		{
			BOOL ok = First.Write(rec);
			if(ok) ok = Second.Write(rec);
			if(ok) ok = Amount.Write(rec);
			return ok;
		}

		CXfUINT16 First;
		CXfUINT16 Second;
		CXfLONG Amount;
	};

	CXfFontKerningPairs(CXaraFileRecord& rec);
	CXfFontKerningPairs(ULONG uFlags) : VXfTreeNode(TAG_FONT_KERNINGPAIRS), Flags(uFlags) {}

	void AddPair(KerningPair& p ) { Pairs.push_back(p); }
	void AddPair(UINT16 first, UINT16 second, UINT16 amount) { Pairs.push_back(KerningPair(first, second, amount)); }
	virtual BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL);
	virtual std::string CreateDumpString();

	bool IsBold() { return Flags & 1 ? true : false; }
	bool IsItalic() { return Flags & 2 ? true : false; }

	CXfULONG Flags;
	std::vector<KerningPair> Pairs;
};

typedef std::vector<CXfFontKerningPairs::KerningPair> KerningPairVector;

class XarLigatureEntry : public LigatureEntry
{
public:
	XarLigatureEntry() {}
	XarLigatureEntry(CXaraFileRecord& rec);
};

class CXfFontOTLigatures : public VXfTreeNode
{
public:
	CXfFontOTLigatures(CXaraFileRecord& rec);
	CXfFontOTLigatures(CXfULONG kerningFlags) : VXfTreeNode(TAG_FONT_OTLIGATURES), KerningFlags(kerningFlags) {}

	virtual std::string CreateDumpString();
	bool IsBold() { return KerningFlags & 1 ? true : false; }
	bool IsItalic() { return KerningFlags & 2 ? true : false; }

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		KerningFlags.Write(&Rec);

		ULONG count = (ULONG)ligs.size();
		Rec.WriteULONG(count);
		for (ULONG i = 0; i < count; ++i)
		{
			UINT16 len = ligs[i].KeyLth;
			Rec.WriteUINT16(len);
			for (UINT16 c = 0; c < len; ++c)
				Rec.WriteWCHAR(ligs[i].Key[c]);
			Rec.WriteWCHAR(ligs[i].Ligature);
			Rec.WriteUINT16(ligs[i].LigType);
		}

		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfULONG KerningFlags;
	std::vector<XarLigatureEntry> ligs;
};

class CXfTextOpenTypeLigatures : public VXfTreeNode
{
public:
	CXfTextOpenTypeLigatures(CXaraFileRecord& rec) : VXfTreeNode(TAG_TEXT_OPEN_TYPE_LIGATURES), LigatureSubset(rec) {}
	CXfTextOpenTypeLigatures(UINT16 ligatureSubset) : VXfTreeNode(TAG_TEXT_OPEN_TYPE_LIGATURES), LigatureSubset(ligatureSubset) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_OPEN_TYPE_LIGATURES, 2);
		Rec.Init();
		LigatureSubset.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfUINT16 LigatureSubset;
};

class CXfTextLigatureStart : public VXfTreeNode
{
public:
	CXfTextLigatureStart(CXaraFileRecord& rec) : VXfTreeNode(TAG_TEXT_LIGATURE_START), LigatureUnicode(rec) {}
	CXfTextLigatureStart(WCHAR wLigatureUnicode) : VXfTreeNode(TAG_TEXT_LIGATURE_START), LigatureUnicode(wLigatureUnicode) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_LIGATURE_START, 2);
		Rec.Init();
		LigatureUnicode.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfUINT16 LigatureUnicode;
};

class CXfFontOTStylisticSets : public VXfTreeNode
{
public:
	CXfFontOTStylisticSets(CXaraFileRecord& rec);
	bool IsBold() { return FontIdentificationFlags & 1 ? true : false; }
	bool IsItalic() { return FontIdentificationFlags & 2 ? true : false; }

	virtual std::string CreateDumpString();
public:
	CXfULONG FontIdentificationFlags;	// Bold/Italic set
	std::array<std::vector<StylisticSetEntry>, 20> arrStylisticSets;
};

class CXfFontOTStylisticSetNames : public VXfTreeNode
{
public:
	CXfFontOTStylisticSetNames(CXaraFileRecord& rec);
public:
	CXfULONG Flags;
	CXfBYTE NonEmptyStylisticSets;

	std::array<std::unordered_map<LANGID, std::string>, 20> arrStylisticSetNames;
};

// TAG_FONT_OTSUBSTITUTIONRULES
class CXfFontOTSubstitutionRules : public VXfTreeNode
{
public:
	CXfFontOTSubstitutionRules(CXaraFileRecord& rec);
	bool IsBold() { return Flags & 1 ? true : false; }
	bool IsItalic() { return Flags & 2 ? true : false; }
	virtual std::string CreateDumpString();
	static std::string DumpSubstRule(SubstitutionRule& rule);
public:
	CXfULONG Flags;
	SubstitutionRuleSet RuleSet;
};

// TAG_FONT_OTSUBSTITUTIONTABLES
class CXfFontOTSubstitutionTables : public VXfTreeNode
{
public:
	CXfFontOTSubstitutionTables(CXaraFileRecord& rec);
	bool IsBold() { return Flags & 1 ? true : false; }
	bool IsItalic() { return Flags & 2 ? true : false; }

	virtual std::string CreateDumpString();
public:
	CXfULONG Flags;
	SubstitutionTableSet TableSet;
};

// TAG_FONT_OTRULEMAXSIZES
class CXfFontOTRuleMaxSizes : public VXfTreeNode
{
public:
	CXfFontOTRuleMaxSizes(CXaraFileRecord& rec);
	bool IsBold() { return Flags & 1 ? true : false; }
	bool IsItalic() { return Flags & 2 ? true : false; }
public:
	CXfULONG Flags;
	RuleMaxSizes MaxSizes;
};

class CWeightRecord
{
public:
	CWeightRecord() : Weight(0), VariantFlags(0) {}
	CWeightRecord(INT16 w, UINT16 vf) : Weight(w), VariantFlags(vf) {}
	INT16 Weight;
	UINT16 VariantFlags;
};

typedef std::map<ULONG, CXfFontGlyph*> GlyphMap;
/********************************************************************************************
>	class CXfFontDef

Author:		Paul Gruzdev
Created:	Sep 2013
Purpose:	Represents TAG_FONT_DEF_TRUETYPE tag
********************************************************************************************/
class CXfFontDef : public VXfTreeNode
{
public:
	enum FontFlags 
	{
		Replaced = 0x1, 
		MultiWeight = 0x02,
		WeightList = 0x04,
		AlternativeWeight = 0x08 
	};

	CXfFontDef(std::wstring strFullName, std::wstring strFaceName, int w = -1,
		BYTE byRealPanoseWeight = 0, BYTE byRealPanoseLetterform = 0, int iFlags = 0) : FullFontName(strFullName), TypeFaceName(strFaceName), 
		weight(w), RealPanoseWeight(byRealPanoseWeight), RealPanoseLetterform(byRealPanoseLetterform), Flags(iFlags), 
		m_bExternal(false), m_bEmbedded(false),	altweight(0)
	{
		memset(m_pPairs, 0, 4 * sizeof(CXfFontKerningPairs*));
		memset(m_pLigatures, 0, 4 * sizeof(CXfFontOTLigatures*));
		memset(m_pStylisticSets, 0, 4 * sizeof(CXfFontOTStylisticSets*));
		memset(m_pSubstRules, 0, 4 * sizeof(CXfFontOTSubstitutionRules*));
		memset(m_pSubstTables, 0, 4 * sizeof(CXfFontOTSubstitutionTables*));
		memset(m_pRuleMaxSizes, 0, 4 * sizeof(CXfFontOTRuleMaxSizes*));

		SetTag(TAG_FONT_DEF_TRUETYPE);
		_ASSERT(!strFaceName.empty());
		memset(&PANOSE[0], 0, sizeof(PANOSE));
	}

	CXfFontDef(const CXfFontDef& other) : VXfTreeNode(TAG_FONT_DEF_TRUETYPE), FullFontName(other.FullFontName), TypeFaceName(other.TypeFaceName), 
		weight(other.weight), RealPanoseWeight(other.RealPanoseWeight), RealPanoseLetterform(other.RealPanoseLetterform), 
		Flags(other.Flags), m_bExternal(other.m_bExternal), m_bEmbedded(other.m_bEmbedded), altweight(other.altweight)
	{
		memset(m_pPairs, 0, 4 * sizeof(CXfFontKerningPairs*));
		memset(m_pLigatures, 0, 4 * sizeof(CXfFontOTLigatures*));
		memset(m_pStylisticSets, 0, 4 * sizeof(CXfFontOTStylisticSets*));
		memset(m_pSubstRules, 0, 4 * sizeof(CXfFontOTSubstitutionRules*));
		memset(m_pSubstTables, 0, 4 * sizeof(CXfFontOTSubstitutionTables*));
		memset(m_pRuleMaxSizes, 0, 4 * sizeof(CXfFontOTRuleMaxSizes*));

		memcpy((void*)PANOSE, (void*)other.PANOSE, 10);
	}
	CXfFontDef(CXaraFileRecord & rec);
	void DeleteGlyphs();
	void DeleteKerningPairs();
	void DeleteLigatures();
	void DeleteStylisticSets();
	void DeleteEmbeddingPolicies();
	virtual std::string CreateDumpString();

	virtual bool IsRefNode() { return true; }

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL);

	void AddGlyph(ULONG unicode, CXfFontGlyph* pGlyph, bool bAddToTree = true);
	void RemoveGlyph(ULONG unicode, CXfFontGlyph* pGlyph, bool bRemoveFromTree = true);
	void MarkGlyphsRemoved(bool bBold, bool bItalic);
	void AddKerningPairs(CXfFontKerningPairs* pPairs);
	void AddLigatures(CXfFontOTLigatures* pLigs) { GetFontLigatures(pLigs->IsBold(), pLigs->IsItalic()) = pLigs; }
	void AddStylisticSets(CXfFontOTStylisticSets* pSets) { GetFontStylisticSets(pSets->IsBold(), pSets->IsItalic()) = pSets; }
	void AddSubstitutionRules(CXfFontOTSubstitutionRules* pRules) { GetFontSubstitutionRules(pRules->IsBold(), pRules->IsItalic()) = pRules; }
	void AddSubstitutionTables(CXfFontOTSubstitutionTables* ptr) { GetFontSubstitutionTables(ptr->IsBold(), ptr->IsItalic()) = ptr; }
	void AddRuleMaxSizes(CXfFontOTRuleMaxSizes* ptr) { GetFontRuleMaxSizes(ptr->IsBold(), ptr->IsItalic()) = ptr; }
	CXfFontKerningPairs*& GetFontKerningPairs(bool bBold, bool bItalic);
	CXfFontOTLigatures*& GetFontLigatures(bool bBold, bool bItalic);
	CXfFontOTStylisticSets*& GetFontStylisticSets(bool bBold, bool bItalic) { return m_pStylisticSets[BI_CODE(bBold, bItalic)]; }
	CXfFontOTSubstitutionRules*& GetFontSubstitutionRules(bool bBold, bool bItalic) { return m_pSubstRules[BI_CODE(bBold, bItalic)]; }
	CXfFontOTSubstitutionTables*& GetFontSubstitutionTables(bool bBold, bool bItalic) { return m_pSubstTables[BI_CODE(bBold, bItalic)]; }
	CXfFontOTRuleMaxSizes*& GetFontRuleMaxSizes(bool bBold, bool bItalic) { return m_pRuleMaxSizes[BI_CODE(bBold, bItalic)]; }
	bool HasOpenTypeFeatures(bool b, bool i) {
		return (GetFontLigatures(b, i) != nullptr || GetFontStylisticSets(b, i) != nullptr || GetFontSubstitutionRules(b, i) != nullptr ||
			GetFontSubstitutionTables(b, i) != nullptr || GetFontRuleMaxSizes(b, i) != nullptr) ? true : false;
	}

	GlyphMap& GetGlyphMap(bool bBold, bool bItalic);
	CXfFontGlyph* GetGlyph(ULONG uCode, bool bBold, bool bItalic);
	bool GetGlyphs(std::vector<CXfFontGlyph*>& glyphs, bool bBold, bool bItalic);
	bool HasGlyphs() { return !m_Glyphs[0].empty() || !m_Glyphs[1].empty() || !m_Glyphs[2].empty() || !m_Glyphs[3].empty(); }
	bool HasGlyphs(bool bBold, bool bItalic) { return GetGlyphMap(bBold, bItalic).size() > 0; }
	int NumVariants() { return m_Glyphs[0].size() ? 1 : 0 + m_Glyphs[1].size() ? 1 : 0 +
		m_Glyphs[2].size() ? 1 : 0 + m_Glyphs[3].size() ? 1 : 0; }
	size_t NumGlyphs() { return m_Glyphs[0].size() + m_Glyphs[1].size() + m_Glyphs[2].size() + m_Glyphs[3].size(); }
	size_t CalcGlyphVerbCount();
	bool IsExternal() { return m_bExternal; }
	void SetExternal(bool e) { m_bExternal = e; }
	bool IsEmbedded() { return m_bEmbedded; }
	void SetEmbedded(bool e) { m_bEmbedded = e; }
	bool IsMultiweight() { return Flags & MultiWeight ? true : false; }
	DWORD CalcSubsetChecksum(bool bBold, bool bItalic);

	void AddWeightRecord(INT16 iWeight, bool bHasItalic)
	{
		m_WeightList.push_back(CWeightRecord(iWeight, bHasItalic ? 1 : 0)); 
		Flags |= WeightList; 
	}

	void UpdateEmbeddingPolicy(bool bBold, bool bItalic, UINT16 flags);
	void UpdateEmbeddingPolicy(UINT16 glyphFlags, UINT16 flags);
	UINT16 GetEmbeddingPolicy(bool bBold, bool bItalic);

	virtual VXfTreeNode* Copy() { return new CXfFontDef(*this); }

	CXfString FullFontName;
	CXfString TypeFaceName;
	BYTE PANOSE[10];
	INT16  weight;
	INT16 altweight; // Alternative weight is the second [high] weight of a non-multiweight font
	UINT16 Flags;
	BYTE RealPanoseWeight;
	BYTE RealPanoseLetterform;
	GlyphMap m_Glyphs[4];

	std::map<UINT16, class CXfFontEmbeddingPolicy *> m_EmbeddingPolicy;

	CXfFontKerningPairs* m_pPairs[4];
	CXfFontOTLigatures* m_pLigatures[4];
	CXfFontOTStylisticSets* m_pStylisticSets[4];
	CXfFontOTSubstitutionRules* m_pSubstRules[4];
	CXfFontOTSubstitutionTables* m_pSubstTables[4];
	CXfFontOTRuleMaxSizes* m_pRuleMaxSizes[4];
	std::vector<CWeightRecord> m_WeightList;
	bool m_bExternal; // this font ref is a valid font ref but it has no glyphs because the font present in the system 
	// or it has not been embedded. Note - font def record can aggregate several - up to 4 different font variants, some of them
	// may be present in the system, some of them - not. External means that every variant is absent
	bool m_bEmbedded; // This flag means that if any variant (normal, italic, bold, bolditalic) of this font def WILL have glyphs
	// attached to this font def - i.e. some glyphs need to be embedded along with this def.
};

/********************************************************************************************
>	class CXfFontTypeface

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_FONT_TYPEFACE and TAG_TEXT_FONT_TYPEFACE_SHADOWED
********************************************************************************************/
class CXfFontTypeface : public VXfTreeNode
{
public:
	CXfFontTypeface( CXaraFileRecord & rec )
		: FontRef( rec )
		, fontRefNode(NULL)
	{
	}

	CXfFontTypeface(VXfTreeNode *_fontRefNode, bool bShadowed = false) 
		: VXfTreeNode(bShadowed ? TAG_TEXT_FONT_TYPEFACE_SHADOWED : TAG_TEXT_FONT_TYPEFACE), fontRefNode(_fontRefNode) 
	{
	}
	
	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		if (fontRefNode)
		{
			FontRef = fontRefNode->GetRef();
			if(fontRefNode->IsRemoved() || FontRef == (ULONG)-1)
			{
				assert(false);
				wprintf(L"Trying to reference removed font %s\n", ((CXfFontDef*)fontRefNode)->FullFontName.getstr().c_str());
			}
		}
		//std::cout << " +fontRef:" << fontRefNode->GetRef() << " ";
		BOOL ok = FontRef.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfULONG FontRef;
	VXfTreeNode *fontRefNode;
};

/********************************************************************************************
>	class CXfTextString

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_STRING, TAG_TEXT_CHAR
********************************************************************************************/
class CXfTextString : public VXfTextRenderable
{
public:
	CXfTextString()
	{
		SetTag(TAG_TEXT_STRING);
	}

	CXfTextString(const std::wstring &str) : m_str(str)
	{ 
		SetTag(TAG_TEXT_STRING);
	}

	CXfTextString(CXaraFileRecord& rec)
	{
		int iCount = rec.GetSize()/2;
		m_str.resize(iCount);
		rec.ReadBuffer((BYTE*)&m_str[0], iCount*2);
	}

	virtual bool IsTextString() { return true; }
	virtual std::string CreateDumpString();
	virtual void AddChild(VXfTreeNode* pChild, bool bUpdateBBox = true);

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_STRING);
		Rec.Init();
		Rec.WriteWCHARs(const_cast<wchar_t *>(m_str.c_str()), (ULONG)m_str.length());
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	std::wstring m_str;
};

/********************************************************************************************
>	class CXfTextEOL

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_EOL tag
********************************************************************************************/
class CXfTextEOL : public VXfTextRenderable
{
public:
	CXfTextEOL(CXaraFileRecord& rec) {}
};

/********************************************************************************************
>	class CXfTextBreakType

Author:		Ekaterina Dolgunicheva
Created:	10/11/2015
Purpose:	Represents TAG_TEXT_BREAK_TYPE tag
********************************************************************************************/
class CXfTextBreakType : public VXfTextRenderable
{
public:
	CXfTextBreakType(CXaraFileRecord& rec) : BreakType(rec) {}

	CXfBYTE BreakType;
};

/********************************************************************************************
>	class CXfTextFontSize

Author:		Paul Ostapenko
Created:	15/08/13
Purpose:	Represents TAG_TEXT_FONT_SIZE tag
********************************************************************************************/
/*
class CXfTextFontSize : public VXfTreeNode
{
public:
	CXfTextFontSize(LONG _sizeMillipoints) 
		:sizeMillipoints(_sizeMillipoints) 
	{
		SetTag(TAG_TEXT_FONT_SIZE);
	}
	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_FONT_SIZE);
		Rec.Reinit(TAG_TEXT_FONT_SIZE, 4);
		Rec.WriteLONG(sizeMillipoints);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}
	virtual std::string CreateDumpString();

private:
	LONG sizeMillipoints;
};
*/
/********************************************************************************************
>	class CXfTextSoftReturn

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_SOFT_RETURN tag
********************************************************************************************/
class CXfTextSoftReturn : public VXfTextRenderable
{
public:
	CXfTextSoftReturn(CXaraFileRecord& rec) {}
};

/********************************************************************************************
>	class CXfTextStringPos

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_STRING_POS
********************************************************************************************/
class CXfTextStringPos : public CXfTextString
{
public:
	CXfTextStringPos(CXaraFileRecord& rec)
	{
		BOOL ok = Pos.Read(&rec);

		int iCount = (rec.GetSize()-sizeof(DocCoord))/2;
		if(ok && iCount > 0)
		{
			m_str.resize(iCount);
			rec.ReadBuffer((BYTE*)&m_str[0], iCount*2);
		}
	}
	virtual std::string CreateDumpString();
	CXfDocCoord Pos;
};

class CXfTextStringPos2 : public CXfTextString
{
public:
	CXfTextStringPos2(CXaraFileRecord& rec)// : CXfTextString(rec), StartOffset(rec), EndOffset(rec), TopOffset(rec) {}
	{
		BOOL ok = StartOffset.Read(&rec);
		ok = ok && EndOffset.Read(&rec);

		int iCount = (rec.GetSize()-16)/2;
		if(ok && iCount > 0)
		{
			m_str.resize(iCount);
			rec.ReadBuffer((BYTE*)&m_str[0], iCount*2);
		}
	}

	virtual std::string CreateDumpString();
	CXfDocCoord StartOffset;
	CXfDocCoord EndOffset;
};

/********************************************************************************************
>	class CXfTextCharPos

Author:		Alex Shumakov
Created:	6/7/2016
Purpose:	Represents TAG_TEXT_SYMBOL_CHAR_POS
********************************************************************************************/
class CXfTextCharPos : public CXfTextString
{
public:
	CXfTextCharPos(CXaraFileRecord& rec)
	{
		BOOL ok = Pos.Read(&rec);

		int iCount = (rec.GetSize()-sizeof(DocCoord))/2;
		if(ok && iCount > 0)
		{
			m_str.resize(iCount);
			rec.ReadBuffer((BYTE*)&m_str[0], iCount*2);
		}
	}
	virtual std::string CreateDumpString();
	CXfDocCoord Pos;
};

/********************************************************************************************
>	class CXfTextCharPos2

Author:		Alex Shumakov
Created:	6/7/2016
Purpose:	Represents TAG_TEXT_SYMBOL_CHAR_POS2
********************************************************************************************/
class CXfTextCharPos2 : public CXfTextString
{
public:
	CXfTextCharPos2(CXaraFileRecord& rec)// : CXfTextString(rec), StartOffset(rec), EndOffset(rec), TopOffset(rec) {}
	{
		BOOL ok = StartOffset.Read(&rec);
		ok = ok && EndOffset.Read(&rec);

		int iCount = (rec.GetSize()-16)/2;
		if(ok && iCount > 0)
		{
			m_str.resize(iCount);
			rec.ReadBuffer((BYTE*)&m_str[0], iCount*2);
		}
	}

	virtual std::string CreateDumpString();
	CXfDocCoord StartOffset;
	CXfDocCoord EndOffset;
};

/********************************************************************************************
>	class CXfTxtParaBorder

Author:		Ekaterina Dolgunicheva
Created:	14/01/20
Purpose:	Represents TAG_TXT_PARA_TOP_BORDER/TAG_TXT_PARA_LEFT_BORDER/TAG_TXT_PARA_RIGHT_BORDER/TAG_TXT_PARA_BOTTOM_BORDER/TAG_TXT_PARA_INTERNAL_BORDER node
********************************************************************************************/
class CXfTxtParaBorder : public VXfTreeNode
{
public:
	//enum BorderType{ top_bdr, left_bdr, right_bdr, bottom_bdr, internal_bdr };

	CXfTxtParaBorder(CXaraFileRecord& rec) : ColorRef(rec), Width(rec), Offset(rec), DashStyle(rec) {};

	CXfTxtParaBorder(LONG tagID, LONG colorRef, ULONG width, ULONG offset, LONG dashStyle) : 
		ColorRef(colorRef), Width(width), Offset(offset), DashStyle(dashStyle)
	{
		SetTag(tagID);
	};

	CXfTxtParaBorder(LONG tagID, VXfTreeNode* pRef, ULONG width, ULONG offset, LONG dashStyle) : 
		ColorRef(-1), Width(width), Offset(offset), DashStyle(dashStyle), pReference(pRef)
	{
		SetTag(tagID);
	};

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(GetTag(), sizeof(ULONG)* 2 + sizeof(LONG)* 2);
		Rec.Init();
		if (pReference) ColorRef = pReference->GetRef();
		Rec.WriteLONG(ColorRef);
		Rec.WriteULONG(Width);
		Rec.WriteULONG(Offset);
		Rec.WriteLONG(DashStyle);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfLONG ColorRef;
	CXfULONG Width;
	CXfULONG Offset;
	CXfLONG DashStyle;
	VXfTreeNode* pReference; // This is the pointer to an actual colour definition node. Used when constructing XfTree on import
};

/********************************************************************************************
>	class CXfTextAnchorPosition

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_TEXT_ANCHOR_POSITION node
********************************************************************************************/
class CXfTextAnchorPosition : public VXfTextRenderable
{
public:
	CXfTextAnchorPosition(ULONG tag)
	{ 
		SetTag(TAG_TEXT_ANCHOR_POSITION);

		m_vAnchoredGroupID.reserve(2);
		m_vAnchoredGroupID.push_back(tag);
		m_vAnchoredGroupID.push_back(-1);
	}

	CXfTextAnchorPosition(CXaraFileRecord& rec);

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_ANCHOR_POSITION, (LONG)(sizeof(UINT32) * m_vAnchoredGroupID.size()));
		Rec.Init();
		for( size_t i = 0; i < m_vAnchoredGroupID.size(); ++i)
			Rec.WriteULONG(m_vAnchoredGroupID[i]);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	std::vector<ULONG> m_vAnchoredGroupID;
};

/********************************************************************************************
>	class CXfTextEmbeddedAnchorPosInfo

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO node
********************************************************************************************/
class CXfTextEmbeddedAnchorPosInfo : public VXfTreeNode
{
public:
	CXfTextEmbeddedAnchorPosInfo(LONG verticalOffset,
		LONG leftMargin, LONG rightMargin, LONG topMargin, LONG bottomMargin,
		LONG charWidth, LONG fontAscent, LONG fontDescent, BYTE hasNoDescender) :
		VerticalOffset(verticalOffset),
		LeftMargin(leftMargin),
		RightMargin(rightMargin),
		TopMargin(topMargin),
		BottomMargin(bottomMargin),
		CharWidth(charWidth),
		FontAscent(fontAscent),
		FontDescent(fontDescent),
		bNoDescender(hasNoDescender)
	{ 
		SetTag(TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO);
	}

	CXfTextEmbeddedAnchorPosInfo(CXaraFileRecord& rec);

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO, TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO_SIZE);
		Rec.Init();
		VerticalOffset.Write(&Rec);
		LeftMargin.Write(&Rec);
		RightMargin.Write(&Rec);
		TopMargin.Write(&Rec);
		BottomMargin.Write(&Rec);
		CharWidth.Write(&Rec);
		FontAscent.Write(&Rec);
		FontDescent.Write(&Rec);
		bNoDescender.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfLONG VerticalOffset;
	CXfLONG LeftMargin;
	CXfLONG RightMargin;
	CXfLONG TopMargin;
	CXfLONG BottomMargin;
	CXfLONG CharWidth;
	CXfLONG FontAscent;
	CXfLONG FontDescent;
	CXfBYTE bNoDescender;
};

/********************************************************************************************
>	class CXfAnchoredGroupInfo

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_ANCHORED_GROUP_INFO node
********************************************************************************************/
class CXfAnchoredGroupInfo : public VXfTreeNode
{
public:
	CXfAnchoredGroupInfo(ULONG tag, const DocCoord &refPoint) :
		ImportTag(tag), ReferencePoint(refPoint)
	{ 
		SetTag(TAG_ANCHORED_GROUP_INFO);
	}

	CXfAnchoredGroupInfo(CXaraFileRecord& rec) : ImportTag(rec), ReferencePoint(rec) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_ANCHORED_GROUP_INFO, TAG_ANCHORED_GROUP_INFO_SIZE);
		Rec.Init();
		ImportTag.Write(&Rec);
		ReferencePoint.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfULONG ImportTag;
	CXfDocCoord ReferencePoint;
};

/********************************************************************************************
>	class CXfAnchoredGroupInfo2

Author:		Ekaterina Dolgunicheva
Created:	09/01/14
Purpose:	Represents TAG_ANCHORED_GROUP_INFO2 node
********************************************************************************************/
class CXfAnchoredGroupInfo2 : public VXfTreeNode
{
public:
	CXfAnchoredGroupInfo2(BOOL verticallyOnly, BOOL anchorAtDropPoint) :
		VerticallyOnly(verticallyOnly), AnchorAtDropPoint(anchorAtDropPoint)
	{
		SetTag(TAG_ANCHORED_GROUP_INFO2);
	}

	CXfAnchoredGroupInfo2(CXaraFileRecord& rec) : VerticallyOnly(rec), AnchorAtDropPoint(rec) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_ANCHORED_GROUP_INFO2, TAG_ANCHORED_GROUP_INFO2_SIZE);
		Rec.Init();
		VerticallyOnly.Write(&Rec);
		AnchorAtDropPoint.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBOOL VerticallyOnly;
	CXfBOOL AnchorAtDropPoint;
};

/********************************************************************************************
>	class CXfAnchoringMode

Author:		Ekaterina Dolgunicheva
Created:	14/06/14
Purpose:	Represents TAG_ANCHORING_MODE node
********************************************************************************************/

enum
{
	AnchorH_Character = 0x00,
	AnchorH_Column    = 0x01,
	AnchorH_Page      = 0x02,		
};

enum
{
	AnchorV_BaseLine             = 0x00,
	AnchorV_AscentLine           = 0x01,
	AnchorV_ParagraphStart       = 0x02,
	AnchorV_ParagraphStartAscent = 0x03,
	AnchorV_Page                 = 0x04
};

class CXfAnchoringMode : public VXfTreeNode
{
public:

	CXfAnchoringMode(BYTE horizontalMode, BYTE verticalMode) :
		HorizontalMode(horizontalMode), VerticalMode(verticalMode)
	{
		SetTag(TAG_ANCHORING_MODE);
	}

	CXfAnchoringMode(CXaraFileRecord& rec) : HorizontalMode(rec), VerticalMode(rec) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_ANCHORING_MODE, TAG_ANCHORING_MODE_SIZE);
		Rec.Init();
		HorizontalMode.Write(&Rec);
		VerticalMode.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBYTE HorizontalMode;
	CXfBYTE VerticalMode;
};

/********************************************************************************************
>	class CXfTextPresentByParagraph

Author:		Ekaterina Dolgunicheva
Created:	04/06/18
Purpose:	Represents TAG_TEXT_PRESENT_BY_PARAGRAPH
********************************************************************************************/
class CXfTextPresentByParagraph : public VXfTreeNode
{
public:
	CXfTextPresentByParagraph(BYTE present) : bPresent(present)
	{
		SetTag(TAG_TEXT_PRESENT_BY_PARAGRAPH);
	}
	CXfTextPresentByParagraph(CXaraFileRecord& rec) : bPresent(rec) {}

	CXfBYTE bPresent;
};

/********************************************************************************************
>	class CXfEmbeddedAnchoredGroupInfo

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_EMBEDDED_ANCHORED_GROUP_INFO node
********************************************************************************************/
class CXfEmbeddedAnchoredGroupInfo : public VXfTreeNode
{
public:
	CXfEmbeddedAnchoredGroupInfo(BYTE primary) : bPrimary(primary)
	{
		SetTag(TAG_EMBEDDED_ANCHORED_GROUP_INFO);
	}

	CXfEmbeddedAnchoredGroupInfo(CXaraFileRecord& rec) : bPrimary(rec) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_EMBEDDED_ANCHORED_GROUP_INFO, TAG_EMBEDDED_ANCHORED_GROUP_INFO_SIZE);
		Rec.Init();
		bPrimary.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBYTE bPrimary;
};

/********************************************************************************************
>	class CXfEmbeddedTransformedAnchoredGroupInfo

Author:		Denis Onishchenko
Created:	18/10/16
Purpose:	Represents TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO node
********************************************************************************************/
class CXfEmbeddedTransformedAnchoredGroupInfo : public VXfTreeNode
{
public:
	CXfEmbeddedTransformedAnchoredGroupInfo(BYTE primary, FIXED16 a, FIXED16 b, FIXED16 c, FIXED16 d) :
		bPrimary(primary), MatrixA(a), MatrixB(b), MatrixC(c), MatrixD(d)
	{
		SetTag(TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO);
	}

	CXfEmbeddedTransformedAnchoredGroupInfo(CXaraFileRecord& rec) : bPrimary(rec) {}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO, TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO_SIZE);
		Rec.Init();
		bPrimary.Write(&Rec);
		MatrixA.Write(&Rec);
		MatrixB.Write(&Rec);
		MatrixC.Write(&Rec);
		MatrixD.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBYTE bPrimary;
	CXfFIXED16 MatrixA;
	CXfFIXED16 MatrixB;
	CXfFIXED16 MatrixC;
	CXfFIXED16 MatrixD;

};

// flag (1<<0) was used as a transient flag in V7 and below to indicate "JustIndented", so
// better not to use it until all other flags have been used

// If the format string ends in a literal character (rather than a conversion specification),
// e.g., a full stop, optionally followed by a tab, then suppress that character if this flag is set
#define NUMBERED_ITEM_FLAG_SUPPRESS_TERMINATOR (1<<1)

// A special number that indicates that this item should be updated when it is renumbered
// the next time.
#define NUMBER_UNDEFINED (MAXLONG)
#define NUMBERED_ITEM_MAX (MAXLONG / 2)

/********************************************************************************************
>	class CXfTextNumberedItemStart

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_TEXT_NUMBERED_ITEM_START node
********************************************************************************************/
class CXfTextNumberedItemStart : public VXfTextRenderable
{
public:
	CXfTextNumberedItemStart(CXaraFileRecord& rec) : 
		Level(rec), 
		Bold(rec), 
		Italic(rec), 
		FontRef(rec),
		fontRefNode(NULL), 
		StartNumber(rec), 
		Number(rec), 
		NumberingFlags(rec), 
		Format(rec) {}

	CXfTextNumberedItemStart(const std::wstring &format, ULONG number = 1, ULONG startNumber = 1, UINT16 level = 0, VXfTreeNode *_fontRefNode = NULL, BOOL bold = FALSE, BOOL italic = FALSE, UINT16 numberingFlags = 0) :
		VXfTextRenderable(TAG_TEXT_NUMBERED_ITEM_START),
		Level(level),
		Bold(bold),
		Italic(italic),
		fontRefNode(_fontRefNode),
		FontRef(-1),
		StartNumber(startNumber),
		Number(number),
		NumberingFlags(numberingFlags),
		Format(format) {}

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_NUMBERED_ITEM_START);
		Rec.Init();
		Level.Write(&Rec);
		Bold.Write(&Rec);
		Italic.Write(&Rec);
		FontRef = fontRefNode && !fontRefNode->IsRemoved() ? fontRefNode->GetRef() : -1;
		FontRef.Write(&Rec);
		StartNumber.Write(&Rec);
		Number.Write(&Rec);
		NumberingFlags.Write(&Rec);
		Format.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	std::string CreateDumpString();

	CXfUINT16 Level;
	CXfBOOL Bold;
	CXfBOOL Italic;
	CXfULONG FontRef;		// record number of -1 for "no font override"
	VXfTreeNode *fontRefNode;
	// end of AutoListMarker superclass information
	CXfULONG StartNumber;
	CXfULONG Number;
	CXfUINT16 NumberingFlags;
	CXfString Format;
};

/********************************************************************************************
>	class CXfTextBulletedItemStart

Author:		Paul Gruzdev
Created:	11/11/12
Purpose:	Represents TAG_TEXT_BULLETED_ITEM_START node
********************************************************************************************/
class CXfTextBulletedItemStart : public VXfTextRenderable
{
public:
	CXfTextBulletedItemStart(UINT16 bulletChar, UINT16 level = 0, VXfTreeNode *_fontRefNode = NULL, BOOL bold = FALSE, BOOL italic = FALSE) :
		VXfTextRenderable(TAG_TEXT_BULLETED_ITEM_START),
		BulletChar(bulletChar),
		Level(level),
		Bold(bold),
		Italic(italic),
		fontRefNode(_fontRefNode),
		FontRecordRef(-1) {}

	CXfTextBulletedItemStart(CXaraFileRecord& rec) : 
		Level(rec), 
		Bold(rec), 
		Italic(rec), 
		FontRecordRef(rec), 
		fontRefNode(NULL), 
		BulletChar(rec) {}

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_BULLETED_ITEM_START, TAG_TEXT_BULLETED_ITEM_START_SIZE);
		Rec.Init();
		Level.Write(&Rec);
		Bold.Write(&Rec);
		Italic.Write(&Rec);
		FontRecordRef = fontRefNode && !fontRefNode->IsRemoved() ? fontRefNode->GetRef() : -1;
		FontRecordRef.Write(&Rec);
		BulletChar.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	std::string CreateDumpString();

	CXfUINT16 Level;
	CXfBOOL Bold;
	CXfBOOL Italic;
	CXfULONG FontRecordRef;		// record number of -1 for "no font override"
	VXfTreeNode *fontRefNode;
	  // end of AutoListMarker superclass information
	CXfUINT16 BulletChar;
};

/********************************************************************************************
>	class CXfTextSymbolChar

Author:		Ekaterina Dolgunicheva
Created:	01/12/2015
Purpose:	Represents TAG_TEXT_SYMBOL_CHAR node
********************************************************************************************/
class CXfTextSymbolChar : public VXfTextRenderable
{
public:
	CXfTextSymbolChar(CXaraFileRecord& rec) : 
	  Char(rec),
	  FontRef(rec) {}

	  std::string CreateDumpString();

	  CXfUINT16 Char;
	  CXfULONG FontRef;		// record number of -1 for "no font override"
};

/********************************************************************************************
>	class CXfFontEmbeddingPolicy

Author:		Ekaterina Dolgunicheva
Created:	05/06/2018
Purpose:	Represents TAG_FONT_EMBEDDING_POLICY node
********************************************************************************************/
class CXfFontEmbeddingPolicy : public VXfTextRenderable
{
public:
	enum FontFlags{
		RESTRICTED_LICENSE	= 0x02,
		PREVIEW_AND_PRINT	= 0x04,
		EDITABLE		= 0x08,
		NO_SUBSETTING		= 0x100,
		BITMAP_EMBEDDING	= 0x200
	};
	CXfFontEmbeddingPolicy(CXaraFileRecord& rec) :
		Flags(rec),
		GlyphFlags(rec) {}

	CXfFontEmbeddingPolicy(UINT16 flags, UINT16 glyphFlags) :
		Flags(flags),
		GlyphFlags(glyphFlags)
	{
		SetTag(TAG_FONT_EMBEDDING_POLICY);
	}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_FONT_EMBEDDING_POLICY, TAG_FONT_EMBEDDING_POLICY_SIZE);
		Rec.Init();
		Flags.Write(&Rec);
		GlyphFlags.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfUINT16 Flags;
	CXfUINT16 GlyphFlags;
};

/********************************************************************************************
>	class CXfTextTab

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Represents TAG_TEXT_TAB tag
********************************************************************************************/
class CXfTextTab : public VXfTextRenderable
{
public:
	CXfTextTab(CXaraFileRecord& rec) {}
};

/********************************************************************************************
>	class CXfSpecialHyphen

Author:		Denis Onishchenko
Created:	14/06/16
Purpose:	Represents TAG_TEXT_SPECIAL_HYPHEN tag
********************************************************************************************/
class CXfSpecialHyphen : public VXfTextRenderable
{
public:
	CXfSpecialHyphen(BYTE flags) : Flags(flags) 
	{
		SetTag(TAG_TEXT_SPECIAL_HYPHEN);
	}

	CXfSpecialHyphen(CXaraFileRecord& rec) : Flags(rec) 
	{
	}

	bool IsSoft() const
	{
		return 0 != (1 & Flags);
	}

	bool KillPrev() const
	{
		return 0 != (2 & Flags);
	}

	bool IsAuto() const
	{
		return 0 != (4 & Flags);
	}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_SPECIAL_HYPHEN, TAG_TEXT_SPECIAL_HYPHEN_SIZE);
		Rec.Init();
		Flags.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfBYTE Flags;
};

/********************************************************************************************
>	class CXfEmbeddedGlyphSizeHint

Author:		Paul Gruzdev
Created:	14/03/14
Purpose:	Represents TAG_EMBEDDED_GLYPHS_SIZE_HINT tag
********************************************************************************************/
class CXfEmbeddedGlyphSizeHint : public VXfTreeNode
{
public:
	CXfEmbeddedGlyphSizeHint() : VXfTreeNode(TAG_EMBEDDED_GLYPHS_SIZE_HINT), TotalNumPoints(0), NumGlyphs(0), NumFontVariants(0), 
		TotalNumKernPairs(0), NumKerningTables(0) {}

	CXfEmbeddedGlyphSizeHint(CXaraFileRecord& rec) : 
		TotalNumPoints(rec),
		NumGlyphs(rec),
		NumFontVariants(rec),
		TotalNumKernPairs(rec),
		NumKerningTables(rec) {}

	CXfEmbeddedGlyphSizeHint(ULONG dwTotalNumPoints, ULONG dwNumGlyphs, ULONG dwNumFontVariants, ULONG dwTotalNumKernPairs,
		ULONG dwNumKerningTables) : VXfTreeNode(TAG_EMBEDDED_GLYPHS_SIZE_HINT), TotalNumPoints(dwTotalNumPoints),
		NumGlyphs(dwNumGlyphs),
		NumFontVariants(dwNumFontVariants),
		TotalNumKernPairs(dwTotalNumKernPairs),
		NumKerningTables(dwNumKerningTables) {}

	std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pulTagId = NULL)
	{
		CXaraFileRecord Rec(GetTag(), TAG_EMBEDDED_GLYPHS_SIZE_HINT_SIZE);
		Rec.Init();
		TotalNumPoints.Write(&Rec);
		NumGlyphs.Write(&Rec);
		NumFontVariants.Write(&Rec);
		TotalNumKernPairs.Write(&Rec);
		NumKerningTables.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pulTagId);
	}

	CXfULONG TotalNumPoints;		// - the total number of points across all glyphs
	CXfULONG NumGlyphs;				// - the number of glyphs in all fonts and variants
	CXfULONG NumFontVariants;		// - the number of font variants (where each font definition record can introduce up to 4 embedded glyph variants: regular/italic/bold/bold italic)
	CXfULONG TotalNumKernPairs;		// - the total number of kerning pairs across all fonts
	CXfULONG NumKerningTables;		// - the number of kerning tables
};

/********************************************************************************************
>	class CXfExtraFontDef

Author:		Paul Gruzdev
Created:	5 September 2014
Purpose:	Represents TAG_TEXT_EXTRA_TT_FONT_DEF/TAG_TEXT_EXTRA_ATM_FONT_DEF tags
********************************************************************************************/
class CXfExtraFontDef : public VXfTreeNode
{
public:
	enum {
		EXTRA_FONT_DEF_BOLD				= 0x01,
		EXTRA_FONT_DEF_ITALIC			= 0x02,
		EXTRA_FONT_DEF_REPLACED			= 0x04,
		EXTRA_FONT_DEF_REPLACED_BOLD	= 0x08,
		EXTRA_FONT_DEF_REPLACED_ITALIC	= 0x10,
		EXTRA_FONT_DEF_MISSING_VARIANT	= 0x20,
		EXTRA_FONT_DEF_MULTI_WEIGHT		= 0x40,
		EXTRA_FONT_DEF_FLAGS2			= 0x80
	};
	enum {
		// values for the Flags2 field
		EXTRA_FONT_DEF_GOOGLE_FONT			= 0x01,
		EXTRA_FONT_DEF_REPLACED_GOOGLE_FONT	= 0x02
	};
	CXfExtraFontDef() : VXfTreeNode(TAG_TEXT_EXTRA_TT_FONT_DEF) {}
	CXfExtraFontDef(CXaraFileRecord& rec);
	//TAG_TEXT_EXTRA_TT_FONT_DEF
	//TAG_TEXT_EXTRA_ATM_FONT_DEF
	virtual std::string CreateDumpString();

	CXfBYTE Flags;
	CXfString FullFontName;
	CXfString FullTypefaceName;
	CXfString TypefaceName;
	CXfINT16 Weight;
	CXfString OriginalFontName;
	CXfINT16 OriginalBaseWeight;
	CXfINT16 ReplacedWeight;
	CXfBYTE Flags2;
	CXfString GoogleFontFamily;
	CXfString ReplacedGoogleFontFamily;
};


/********************************************************************************************
>	class CXfExtraFontInfo

Author:		Ekaterina Dolgunicheva
Created:	31 July 2015
Purpose:	Represents TAG_TEXT_EXTRA_FONT_INFO tags
********************************************************************************************/
class CXfExtraFontInfo : public VXfTreeNode
{
public:
	CXfExtraFontInfo( CXaraFileRecord & rec )
		: FontRef( rec )
		, fontRefNode(NULL)
	{
	}

	CXfExtraFontInfo(VXfTreeNode *_fontRefNode) 
		: fontRefNode(_fontRefNode) 
	{
		SetTag(TAG_TEXT_EXTRA_FONT_INFO);
	}
	
	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_TEXT_EXTRA_FONT_INFO);
		Rec.Init();
		if (fontRefNode)
		{
			FontRef = fontRefNode->GetRef();
			if(fontRefNode->IsRemoved() || FontRef == (ULONG)-1)
			{
				assert(false);
				wprintf(L"Trying to reference removed font %s\n", ((CXfFontDef*)fontRefNode)->FullFontName.getstr().c_str());
			}
		}
		//std::cout << " +fontRef:" << fontRefNode->GetRef() << " ";
		BOOL ok = FontRef.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfULONG FontRef;
	VXfTreeNode *fontRefNode;
};