/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "XFTree.h"

/******************************************************************************
   Function:    GetMCharWidthOfAFontW
   Author:      Paul Gruzdev
   Purpose:     Find a width of an 'M' char of a given font having it size 
				set to 2048. 
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       Better we export this code to some common code as it is used
				every where it each filter that needs conversion between
				EMS and points/millipoints
******************************************************************************/
int GetCharWidthOfAFontW(int iChar, std::wstring strFontName, DWORD weight, bool bItalic)
{
	HDC hdc = GetDC(NULL); // take the screen dc
	HFONT fnt = CreateFontW(-2048,0,0,0,weight,bItalic ? 255 : 0,0,0,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,PROOF_QUALITY,DEFAULT_PITCH,strFontName.c_str());
	HFONT oldFont = (HFONT) SelectObject(hdc, fnt);

	int iWidth;
	ABC abc;

	if(GetCharABCWidths(hdc, iChar, iChar, &abc))
		iWidth = abc.abcA+abc.abcB+abc.abcC;
	else if(!GetCharWidth32(hdc, iChar, iChar, &iWidth))
		iWidth = 0;

	SelectObject(hdc, oldFont);
	ReleaseDC(0, hdc);
	DeleteObject(fnt);

	return iWidth;
}

/******************************************************************************
   Function:    CXfTextContext::FindFontTypeface
   Author:      Paul Gruzdev
   Purpose:     Find a font name that corresponds to this context
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
std::wstring CXfTextContext::FindFontTypeface(RefNodeMap& map)
{
	std::wstring str = L"Times New Roman";
	if(m_ulFontRef > 0 && map.find(m_ulFontRef) != map.end())
	{
		CXfFontDef* pFont = (CXfFontDef*)map[m_ulFontRef];
		str = pFont->TypeFaceName.getstr();
	}

	return str;
}

bool CXfTextContext::IsFontTypefaceSpecified(RefNodeMap& map)
{
	return m_ulFontRef > 0 && map.find(m_ulFontRef) != map.end() ? true : false;
}

/******************************************************************************
   Function:    CXfTextContext::CalculateCharWidthInMillipoints
   Author:      Paul Gruzdev
   Purpose:     Calculate the width of a given char in millipoints
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
MILLIPOINT CXfTextContext::CalculateCharWidthInMillipoints(int iChar, RefNodeMap& map)
{
	std::wstring strTypeface = FindFontTypeface(map);
	if(strTypeface.empty())
		return 0;

	int iWidth = GetCharWidthOfAFontW(iChar, strTypeface, m_bBoldOn ? 700 : 400, m_bItalicOn);
	return iWidth*m_dwFontSize/2048;
}

/******************************************************************************
   Function:    CXfTextContext::CalculateCharWidthInTwips
   Author:      Paul Gruzdev
   Purpose:     Calculate the width of a given char in twips
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
double CXfTextContext::CalculateCharWidthInTwips(int iChar, RefNodeMap& map)
{
	return CalculateCharWidthInMillipoints(iChar, map)/50.0;
}

/******************************************************************************
   Function:    CXfTextContext::CalculateCharWidthInTwips
   Author:      Paul Gruzdev
   Purpose:     Calculate the width of a given char in twips
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
LONG CXfTextContext::CalculateEMSInTwips(LONG lTracking, RefNodeMap& map)
{
	std::wstring strTypeface = FindFontTypeface(map);
	if(strTypeface.empty())
		return 0;

	double dVal = CalculateCharWidthInTwips('M', map);
	return (LONG)(dVal*lTracking/1000.0);
}

/******************************************************************************
   Function:    CXfTextContext::Update
   Author:      Paul Gruzdev
   Purpose:		Update the current text context with the text attribute node
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool CXfTextContext::Update(VXfTreeNode* pNode)
{
	switch(pNode->GetTag())
	{
	case TAG_TEXT_JUSTIFICATION_LEFT:
	case TAG_TEXT_JUSTIFICATION_CENTRE:
	case TAG_TEXT_JUSTIFICATION_RIGHT:
	case TAG_TEXT_JUSTIFICATION_FULL:
		m_ulJustification = pNode->GetTag();
		break;
	case TAG_TEXT_ASPECT_RATIO:
		m_dAspect = ((CXfAspectRatio*)pNode)->GetValue();
		break;
	case TAG_TEXT_FONT_SIZE:
		m_dwFontSize = ((CXfNodeValue*)pNode)->GetValue();
		break;
	case TAG_TEXT_FONT_TYPEFACE:
		m_ulFontRef = ((CXfFontTypeface*)pNode)->FontRef.get();
		break;
	case TAG_TEXT_LINESPACE_ABSOLUTE:
		m_lLinespace = ((CXfLinespaceAbsolute*)pNode)->GetValue();
		break;
	case TAG_TEXT_LINESPACE_RATIO:
		m_dLinespaceRatio = ((CXfLinespaceRatio*)pNode)->GetValue();
		break;
	case TAG_TEXT_BOLD_OFF:
		m_bBoldOff = true;
		m_bBoldOn = false;
		break;
	case TAG_TEXT_BOLD_ON:
		m_bBoldOn = true;
		break;
	case TAG_TEXT_ITALIC_OFF:
		m_bItalicOff = true;
		m_bItalicOn = false;
		break;
	case TAG_TEXT_ITALIC_ON:
		m_bItalicOn = true;
		break;
	case TAG_TEXT_SCRIPT_OFF:
		m_bScriptOff = true;
		m_bScriptOn = false;
		break;
	case TAG_TEXT_SCRIPT_ON:
		m_bScriptOn = true;
		m_dScriptOffset = ((CXfTextScriptOn*)pNode)->ScriptOffset->MakeDouble();
		m_dScriptFontSize = ((CXfTextScriptOn*)pNode)->ScriptFontSize->MakeDouble();
		break;
	case TAG_TEXT_OPEN_TYPE_LIGATURES:
		m_uiLigatureSubset = ((CXfTextOpenTypeLigatures*)pNode)->LigatureSubset;
		break;
	case TAG_TEXT_OPEN_TYPE_STYLISTICSETS:
		m_ulStylisticSet = ((CXfTextOpenTypeStylisticSets*)pNode)->Value;
		break;
	case TAG_TEXT_CHANGE_CAPS:
		m_bCapsType = ((CXfTextChangeCaps*)pNode)->Type;
		break;
	case TAG_TEXT_OT_CONTEXT_ALT_ON:
		m_bContextualAlternates = true;
		break;
	case TAG_TEXT_OT_CONTEXT_ALT_OFF:
		m_bContextualAlternates = false;
		break;
	case TAG_TEXT_WIDOWS_AND_ORPHANS_ON:
		m_bWidowOrphanControl = true;
		break;
	case TAG_TEXT_WIDOWS_AND_ORPHANS_OFF:
		m_bWidowOrphanControl = false;
		break;
	case TAG_TEXT_UNDERLINE_OFF:
		m_bUnderlineOff = true;
		m_bUnderlineOn = false;
		break;
	case TAG_TEXT_UNDERLINE_ON:
		m_bUnderlineOn = true;
		break;
	case TAG_TEXT_STRIKETHROUGH_ON:
		m_bStrikeThroughOn = true;
		break;
	case TAG_TEXT_STRIKETHROUGH_OFF:
		m_bStrikeThroughOff = true;
		m_bStrikeThroughOn = false;
		break;
	case TAG_TEXT_SUBSCRIPT_ON:
		m_bSubscript = true;
		break;
	case TAG_TEXT_SUPERSCRIPT_ON:
		m_bSuperscript = true;
		break;
	case TAG_TEXT_BASELINE:
		m_lBaselineShift = ((CXfTextBaseline*)pNode)->GetValue();
		break;
	case TAG_TEXT_TRACKING:
		m_lTracking = ((CXfTextTracking*)pNode)->GetValue();
		break;
	case TAG_TEXT_FONT_SIZE_RELATIVE_KERN:
	case TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING:
		m_bFontSizeRelative = true;
		break;
	case TAG_TEXT_STORY_HEIGHT_INFO:
		m_dwHeightInfo = ((CXfHeightInfo*)pNode)->GetValue();
		break;
	case TAG_TEXT_SPACE_BEFORE:
		m_lSpaceBefore = ((CXfTextSpaceBefore*)pNode)->GetValue();
		break;
	case TAG_TEXT_SPACE_AFTER:
		m_lSpaceAfter = ((CXfTextSpaceAfter*)pNode)->GetValue();
		break;
	case TAG_TEXT_FIRST_INDENT:
		m_lFirstIndent = ((CXfTextFirstIndent*)pNode)->GetValue();
		break;
	case TAG_TEXT_LEFT_INDENT:
		m_lLeftIndent = ((CXfTextLeftIndent*)pNode)->GetValue();
		break;
	case TAG_TEXT_RIGHT_INDENT:
		m_lRightIndent = ((CXfTextRightIndent*)pNode)->GetValue();
		break;
	case TAG_TEXT_PARAGRAPH_STYLE:
		m_ulParaStyle = ((CXfTextParagraphStyle*)pNode)->GetValue();
		break;
	case TAG_STYLE:
		m_ulCharStyle = ((CXfStyle*)pNode)->GetValue();
		break;
	case TAG_TEXT_NUMBERED_ITEM_START:
	case TAG_TEXT_BULLETED_ITEM_START:
		m_bNumberingList = true;
		break;
	case TAG_TEXT_RULER:
		{
			m_pRuler = (CXfTextRuler*)pNode;
			m_lRulerPos1 = m_pRuler->Stops.size() ? m_pRuler->Stops[0].Position : 0;
			m_lRulerPos2 = m_pRuler->Stops.size() > 1 ? m_pRuler->Stops[1].Position : 0;
		}
		break;
	case TAG_TEXT_LANGUAGE:
		{
			CXfTextLanguage* pLang = (CXfTextLanguage*)pNode;
			m_bLanguageSpecified = true;
			m_dwLangCode = pLang->LangCode;
			m_dwCountryCode = pLang->CountryCode;
		}
		break;
	case TAG_TEXT_KEEP_LINES_TOGETHER_ON:
		m_bKeepLinesTogether = true;
		break;
	case TAG_TEXT_KEEP_LINES_TOGETHER_OFF:
		m_bKeepLinesTogether = false;
		break;
	case TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_ON:
		m_bKeepWithNextParagraph = true;
		break;
	case TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_OFF:
		m_bKeepWithNextParagraph = false;
		break;
	case TAG_TXTBACKGROUNDCOLOUR:
		{
			m_ulBackgroundColourRef = ((CXfTxtBackgroundColour*)pNode)->GetValue();
		}
		break;
	case TAG_TXT_PARA_BACKGROUNDCOLOUR:
		m_ulParaBackgroundColourRef = ((CXfTxtBackgroundColour*)pNode)->GetValue();
		break;
	case TAG_TXT_PARA_TOP_BORDER:
		m_pTopBorder = (CXfTxtParaBorder*)pNode;
		break;
	case TAG_TXT_PARA_LEFT_BORDER:
		m_pLeftBorder = (CXfTxtParaBorder*)pNode;
		break;
	case TAG_TXT_PARA_RIGHT_BORDER:
		m_pRightBorder = (CXfTxtParaBorder*)pNode;
		break;
	case TAG_TXT_PARA_BOTTOM_BORDER:
		m_pBottomBorder = (CXfTxtParaBorder*)pNode;
		break;
	case TAG_TXT_PARA_INTERNAL_BORDER:
		m_pInternalBorder = (CXfTxtParaBorder*)pNode;
		break;
	default:
		return false;
	}

	return true;
}

/******************************************************************************
   Function:    CXfStyleDef::FindParentStyle
   Author:      Paul Gruzdev
   Purpose:		Find a style def this style is based on
   Returns:     
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
CXfStyleDef* CXfStyleDef::FindParentStyle(RefNodeMap& map)
{
	if(BaseStyle > 0 && map.find((LONG)BaseStyle) != map.end())
	{
		return (CXfStyleDef*)map[(LONG)BaseStyle];
	}

	return NULL;
}
