/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

class CXfTextRuler;
class CXfTxtParaBorder;
/********************************************************************************************
>	class CXfTextContext

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	A context of attributes that holds all text attributes
********************************************************************************************/
class CXfTextContext
{
public:
	enum
	{
		keJustifyLeft	= TAG_TEXT_JUSTIFICATION_LEFT,
		keJustifyCenter	= TAG_TEXT_JUSTIFICATION_CENTRE,
		keJustifyRight	= TAG_TEXT_JUSTIFICATION_RIGHT,
		keJustifyFull	= TAG_TEXT_JUSTIFICATION_FULL
	};

	CXfTextContext() : m_dAspect(1.0f), m_dwFontSize(16000), m_ulFontRef(0), m_lLinespace(0), 
					m_bBoldOn(false), 
					m_bBoldOff(false), 
					m_bScriptOn(false),
					m_dScriptFontSize(1.0f),
					m_dScriptOffset(0),
					m_bScriptOff(false),
					m_bItalicOn(false), 
					m_bItalicOff(false), 
					m_bUnderlineOn(false), 
					m_bUnderlineOff(false), 
					m_bStrikeThroughOn(false),
					m_bStrikeThroughOff(false),
					m_lBaselineShift(0), 
					m_lTracking(0),
					m_bFontSizeRelative(false),
					m_ulJustification(keJustifyLeft),
					m_dwHeightInfo(0),
					m_lSpaceBefore(0),
					m_lSpaceAfter(0),
					m_lFirstIndent(0),
					m_lLeftIndent(0),
					m_dLinespaceRatio(1.0),
					m_ulParaStyle(0),
					m_ulCharStyle(0),
					m_bSubscript(false),
					m_bSuperscript(false),
					m_lRightIndent(0),
					m_bNumberingList(false),
					m_lRulerPos1(0),
					m_lRulerPos2(0),
					m_bLanguageSpecified(false),
					m_dwLangCode(0),
					m_dwCountryCode(0),
					m_bKeepLinesTogether(false),
					m_bKeepWithNextParagraph(false),
					m_ulParaBackgroundColourRef(0),
					m_pRuler(0),
					m_ulBackgroundColourRef(0),
					m_pTopBorder(0),
					m_pLeftBorder(0),
					m_pRightBorder(0),
					m_pBottomBorder(0),
					m_pInternalBorder(0),
					m_uiLigatureSubset(1),
					m_ulStylisticSet(0),
					m_bCapsType(0),
					m_bContextualAlternates(false),
					m_bWidowOrphanControl(false)
	{}

	bool Update(VXfTreeNode* pNode);
	void Reset() { *this = CXfTextContext(); }
	std::wstring FindFontTypeface(RefNodeMap& map);
	bool IsFontTypefaceSpecified(RefNodeMap& map);
	MILLIPOINT CalculateCharWidthInMillipoints(int iChar, RefNodeMap& map);
	double CalculateCharWidthInTwips(int iChar, RefNodeMap& map);
	LONG CalculateEMSInTwips(LONG lTracking, RefNodeMap& map);
	bool IsLanguageSpecified() { return m_bLanguageSpecified; }

	bool ContainsIndents() { return m_lFirstIndent || m_lLeftIndent || m_lRightIndent; }
	bool IndentsDiffer(CXfTextContext& c) { return m_lFirstIndent != c.m_lFirstIndent || 
		m_lLeftIndent != c.m_lLeftIndent || m_lRightIndent != c.m_lRightIndent; }
	// Per-character attributes
	double	m_dAspect;
	DWORD	m_dwFontSize;
	ULONG	m_ulFontRef;
	ULONG	m_ulParaStyle;
	ULONG	m_ulCharStyle;
	ULONG	m_ulBackgroundColourRef;
	bool	m_bBoldOn;
	bool	m_bBoldOff;
	bool	m_bScriptOn;
	double	m_dScriptFontSize;
	double	m_dScriptOffset;
	bool	m_bScriptOff;
	bool	m_bItalicOn;
	bool	m_bItalicOff;
	bool	m_bSubscript;
	bool	m_bSuperscript;
	bool	m_bUnderlineOn;
	bool	m_bUnderlineOff;
	bool	m_bStrikeThroughOn;
	bool	m_bStrikeThroughOff;
	LONG	m_lBaselineShift;
	LONG	m_lTracking;
	bool	m_bFontSizeRelative; // font size relative kerning and tracking: value of 1000 means not EM size but font size
	// Per-line attributes
	LONG	m_lLinespace;
	double	m_dLinespaceRatio;
	ULONG	m_ulJustification;
	DWORD	m_dwHeightInfo;
	LONG	m_lSpaceBefore;
	LONG	m_lSpaceAfter;
	LONG	m_lFirstIndent;
	LONG	m_lLeftIndent;
	LONG	m_lRightIndent;
	bool	m_bNumberingList;
	LONG	m_lRulerPos1;
	LONG	m_lRulerPos2;
	CXfTextRuler* m_pRuler;
	bool	m_bLanguageSpecified;
	DWORD	m_dwLangCode;
	DWORD	m_dwCountryCode;
	bool	m_bKeepWithNextParagraph;
	bool	m_bKeepLinesTogether;
	ULONG	m_ulParaBackgroundColourRef;
	CXfTxtParaBorder* m_pLeftBorder;
	CXfTxtParaBorder* m_pRightBorder;
	CXfTxtParaBorder* m_pTopBorder;
	CXfTxtParaBorder* m_pBottomBorder;
	CXfTxtParaBorder* m_pInternalBorder;
	UINT16	m_uiLigatureSubset;
	DWORD	m_ulStylisticSet;
	BYTE	m_bCapsType;
	bool	m_bContextualAlternates;
	bool	m_bWidowOrphanControl;
};
