/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

#define CXF_SHADOWTYPE_WALL		1
#define CXF_SHADOWTYPE_FLOOR	2
#define CXF_SHADOWTYPE_GLOW		3

#include <vector>
#include <map>
#include <set>
#include <string>
#include "../SimpleTypes.h" // integer Point, Size and Rect classes
#include <algorithm>

#include "xarlib.h"
#include "oiltypes.h"
#include "fixed16.h"
#include "fixed24.h"
#include "matrix.h"

#ifndef Matrix
//This is for the compatibility between different version of XarLib
#define Matrix XaraMatrix
#endif


#ifndef TRACEUSER
	#define TRACEUSER(a, b) 
#endif

#ifndef ERROR3IF
	#define ERROR3IF(a, b)
#endif

//#include "tstring.h"

#define SIMPLE_XF_READ_RECORD_EX( wrapper_name, xf_type, xf_read_func, xf_write_func )	\
class wrapper_name							\
{											\
public:										\
	wrapper_name( ) {}						\
	wrapper_name( const xf_type& def ) : Value(def) {} \
	wrapper_name( CXaraFileRecord & rec )	\
	{ rec.xf_read_func( &Value ); }			\
											\
	operator xf_type () const				\
	{ return Value; }						\
	BOOL Write( CXaraFileRecord* rec )		\
	{ return rec->xf_write_func ( Value ); }	\
	BOOL Read(CXaraFileRecord* rec)		\
	{ return rec->xf_read_func(&Value); }	\
											\
	xf_type& get()						    \
	{ return Value; }						\
											\
	xf_type const* operator->() const	    \
	{ return &Value; }						\
	xf_type& operator=(const xf_type dat)   \
	{ return Value = dat; }					\
											\
private:									\
	xf_type Value;							\
};	

#define SIMPLE_XF_READ_RECORD( xf_type, xf_read_func, xf_write_func )	\
	SIMPLE_XF_READ_RECORD_EX( CXf##xf_type, xf_type, xf_read_func, xf_write_func )

SIMPLE_XF_READ_RECORD( BYTE, ReadBYTE, WriteBYTE )
SIMPLE_XF_READ_RECORD( BOOL, ReadBOOL, WriteBOOL )
SIMPLE_XF_READ_RECORD( LONG, ReadLONG, WriteLONG )
SIMPLE_XF_READ_RECORD( ULONG, ReadULONG, WriteULONG )
SIMPLE_XF_READ_RECORD( UINT16, ReadUINT16, WriteUINT16 )
SIMPLE_XF_READ_RECORD( INT16, ReadINT16, WriteINT16 )
SIMPLE_XF_READ_RECORD( double, ReadDOUBLE, WriteDOUBLE )
SIMPLE_XF_READ_RECORD( DocCoord, ReadCoord, WriteCoord )
SIMPLE_XF_READ_RECORD_EX( CXfDocCoordInterleaved, DocCoord, ReadCoordInterleaved, WriteCoordInterleaved )
SIMPLE_XF_READ_RECORD( FIXED16, ReadFIXED16, WriteFIXED16 )
SIMPLE_XF_READ_RECORD( FIXED24, ReadFIXED24, WriteFIXED24 )
SIMPLE_XF_READ_RECORD( Matrix, ReadMatrix, WriteMatrix )

class VXfTreeNode;
class CXfGroup;
class CXfClipviewController;
class VXfBounded;
typedef std::vector<VXfTreeNode*> VXfTreeNodeVector;
typedef std::map<UINT, VXfTreeNode*> RefNodeMap;
typedef std::map<UINT, VXfTreeNode*> PrevLinkMap;
typedef std::map<std::wstring, std::vector<VXfTreeNode*>> ObjectNameMap;

class CTagSet : public std::set<UINT>
{
public:
	CTagSet(UINT t1) {insert(t1);}
	CTagSet(UINT t1, UINT t2) {insert(t1); insert(t2);}
	CTagSet(UINT t1, UINT t2, UINT t3) {insert(t1); insert(t2); insert(t3);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4) {insert(t1); insert(t2); insert(t3); insert(t4);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5, UINT t6) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5); insert(t6);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5, UINT t6, UINT t7) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5); insert(t6); insert(t7);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5, UINT t6, UINT t7, UINT t8) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5); insert(t6); insert(t7); insert(t8);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5, UINT t6, UINT t7, UINT t8, UINT t9) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5); insert(t6); insert(t7); insert(t8); insert(t9);}
	CTagSet(UINT t1, UINT t2, UINT t3, UINT t4, UINT t5, UINT t6, UINT t7, UINT t8, UINT t9, UINT t10) {insert(t1); insert(t2); insert(t3); insert(t4); insert(t5); insert(t6); insert(t7); insert(t8); insert(t9); insert(t10);}
};
/********************************************************************************************
>	class VXfTreeNode

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The base virtual class for the Xara tree node that incapsulates each tag
********************************************************************************************/
class VXfTreeNode
{
public:
	VXfTreeNode(UINT uiTag = -1) : m_uiTag(uiTag), m_uiRef(-1), m_pParent(0), m_pNext(0), m_pPrev(0), m_bHTMLText(false), m_bRemoved(false) {}
	virtual ~VXfTreeNode()
	{
		for(size_t i = 0; i < m_Childern.size(); i++)
			delete m_Childern[i];
	}

	void SetTag(UINT ui) { m_uiTag = ui; }
	void SetRef(UINT ui) { m_uiRef = ui; }
	void SetObjectName(std::wstring str) { m_strObjectName = str; }
	void SetHTMLText() { m_bHTMLText = true; }
	bool IsHTMLText() { return m_bHTMLText; }
	std::wstring GetObjectName() { return m_strObjectName; }
	UINT GetTag() const { return m_uiTag; }
	UINT GetRef() { return m_uiRef; }
	void SetParent(VXfTreeNode* p) { m_pParent = p; }
	void SetPrev(VXfTreeNode* p) { m_pPrev = p; }
	void SetNext(VXfTreeNode* p) { m_pNext = p; }
	VXfTreeNode* GetParent() { return m_pParent; }
	std::vector<VXfTreeNode*>& GetChildren() { return m_Childern; }
	size_t ChildrenCount() { return m_Childern.size(); }
	VXfTreeNode* GetChild(size_t iIdx) { return m_Childern[iIdx]; }
	VXfTreeNode* GetLastChild() { return m_Childern.size() ? m_Childern.back() : NULL; }
	bool HasChildren() { return m_Childern.size() > 0; }
	bool MoveChildrenTo(VXfTreeNode* pTarget);

	VXfTreeNode* NextSibling() { return m_pNext; }
	VXfTreeNode* PrevSibling() { return m_pPrev; }
	VXfTreeNode* GetRoot();
	void UpdateContexts(VXfTreeNode* pChild);
	void RebuildContexts();

	void SetRemoved(bool b) { m_bRemoved = b; }
	bool IsRemoved() { return m_bRemoved; }
	VXfBounded* FindParentBounded();
	VXfTreeNode* FindParentNode(UINT uiTag);

	virtual void AddChild(VXfTreeNode* pChild, bool bUpdateBBox = true);
	void InsertFirstChild(VXfTreeNode* pChild, bool bUpdateBBox = true);
	VXfTreeNode* InsertBefore(size_t iIdx, VXfTreeNode* pChild, bool bUpdateBBox = true);
	VXfTreeNode* InsertAfterChild(VXfTreeNode* pAfter, VXfTreeNode* pChild);
	VXfTreeNode* AppendChild(VXfTreeNode* pChild, bool bUpdateBBox = true) { AddChild(pChild, bUpdateBBox); return pChild; }
	bool RemoveChild(UINT iIdx, bool bDelete = true);
	bool RemoveChild(VXfTreeNode* pChild, bool bDelete = true);
	void ClearChildren() { m_Childern.clear(); }
	bool IsSmartTableNode(VXfTreeNode* pNode);
	virtual bool IsRefNode() { return false; }
	virtual bool IsRenderable() { return false; }
	virtual bool IsBounded() { return false; }
	virtual bool IsTextRenderable() { return false; }
	virtual bool IsFillStyle() { return false; }
	virtual bool IsStrokeStyle() { return false; }
	virtual bool IsTransparentFill() { return false; }
	virtual bool IsNodeBitmap() {return false;}
	virtual bool IsWebAddress() { return false; }
	virtual bool IsRectangle() { return false; }
	virtual bool IsEllipse() { return false; }
	virtual bool IsPath() { return false; }
	virtual bool IsTextStory() { return false; }
	virtual bool IsTextStorySimple() { return false; }
	virtual bool IsTextStoryComplex() { return false; }
	virtual bool IsTextString() { return false;  }
	virtual bool IsTextKern() { return false;  }
	virtual bool IsStyleDef() { return false; }
	virtual bool IsSoftMaskGroup() { return false; }
	virtual bool IsClipperController() { return false; }
	virtual bool IsGroup() { return false; }
	virtual bool UpdateBBox(Rect& rcAdd) { return false; }
	/////////////////////////////////////////////////////////////////////////////////////////////
	// Find methods
	VXfTreeNode* FindFirstNode(UINT uiTag, bool bRecursive=true, UINT* puiIdx = NULL);
	VXfTreeNode* FindFirstNode(CTagSet Tags, bool bRecursive=true, UINT* puiIdx = NULL);
	VXfTreeNode* FindLastNode(CTagSet Tags, bool bRecursive=true, UINT* puiIdx = NULL);
	bool FindNodes(UINT uiTag, VXfTreeNodeVector& result, bool bRecursive=true);
	bool FindNodes(CTagSet Tags, VXfTreeNodeVector& result, bool bRecursive=true);
	VXfTreeNode* NextNode(bool bRecursive=true);
	bool FindNodesThatContainNodes(CTagSet Tags, VXfTreeNodeVector& result, bool bRecursive, CTagSet tContain, bool bContainRecursive);
	bool FindNodesThatNotContainNodes(CTagSet Tags, VXfTreeNodeVector& result, bool bRecursive, CTagSet tContain, bool bContainRecursive);
	bool FindPageNodes(VXfTreeNodeVector& result);
	VXfTreeNode* FindFirstRenderableNode(bool bRecursive, bool bExcludeClipPath, bool bExcludeLayer, bool bIgnoreRemoved);
	VXfTreeNode* FindFirstWebaddressNode(bool bRecursive, bool bIgnoreRemoved);
	bool FindRenderableNodes(VXfTreeNodeVector& result);
	bool FindTextRenderableNodes(VXfTreeNodeVector& result);
	CXfGroup* FindAnchoredGroup(UINT id);
	bool IsThisAnchoredBitmap();
	bool IsThisAnchoredBitmap(VXfTreeNodeVector& vWizOps);
	CXfClipviewController* FindParentClipperController();

	virtual std::string CreateDumpString() { return ""; }
	std::string Dump(int iIndent);
	//////////////////////////////////////////////////////////////
	// writing methods
	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		return pExporter->WriteZeroSizedRecord(GetTag(), pRecordNum);
	}

	VXfTreeNode* DeepCopy(); // Copy the current node, and recursively all it's child nodes
	virtual VXfTreeNode* Copy()
	{
		// If you are using a copy method but get here, probably you should implement a copying method via
		// return new CXf**** the actual node object
		assert(false);
		return NULL;
	}
private:

	void Reserve()
	{
		if (m_Childern.capacity() == m_Childern.size())
			m_Childern.reserve(m_Childern.size() + 1000);
	}

	UINT m_uiTag;
	UINT m_uiRef;
	VXfTreeNodeVector m_Childern;
	VXfTreeNode* m_pParent;
	VXfTreeNode* m_pNext;
	VXfTreeNode* m_pPrev;
	std::wstring m_strObjectName;
	bool m_bHTMLText;
	bool m_bRemoved;
};

/********************************************************************************************
>	class VXFFillStyleNode

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The base virtual class that represents a Xara's fill style, possible a transparent
			fill
********************************************************************************************/
class VXFFillStyleNode : public VXfTreeNode
{
public:
	VXFFillStyleNode() : VXfTreeNode((UINT)-1) {}
	VXFFillStyleNode(UINT uiTag) : VXfTreeNode(uiTag) {}
	virtual bool IsFillStyle() { return true; }
	virtual bool IsFlatFill() { return false; }
	virtual bool IsLinearFill() { return false; }
	virtual bool IsCircularFill() { return false; }
	virtual bool IsEllipticalFill() { return false; }
	virtual bool IsBitmapFill() { return false; }
	virtual bool IsNone() { return false; }
	virtual COLORREF GetRGBColor(RefNodeMap& map) { return 0; }
};

/********************************************************************************************
>	class VXfBounded

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The base virtual class that represents a Xara's node that can have a bounding box
			but possibly not explicitly rendered such as text story
********************************************************************************************/
class VXfBounded : public VXfTreeNode, public VElementRect
{
public:
	VXfBounded() {}
	VXfBounded(ULONG uTag) : VXfTreeNode(uTag) {}
	virtual bool IsBounded() { return true; }

	Rect& Bounds() { return m_rcBounds; }
	void SetBounds(Rect rc) { m_rcBounds = rc; }

	virtual bool UpdateBBox(Rect& rcAdd);
	virtual void GetRect(int& x1, int& y1, int& x2, int& y2);
protected:
	Rect m_rcBounds;
};

#include "XFAttributeContext.h"

/********************************************************************************************
>	class VXfRenderable

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The base virtual class that represents a Xara's node that can be rendered, such
			as a shape, regular shape, or a text char
********************************************************************************************/
class VXfRenderable : public VXfBounded
{
public:
	VXfRenderable() : m_bClipPath(false) {}
	VXfRenderable(ULONG uTag) : VXfBounded(uTag), m_bClipPath(false) {}
	virtual bool IsRenderable() { return true; }
	virtual bool IsSimple() { return true; }
	virtual bool IsRounded() { return false; }

	CXfAttributeContext& Context() { return m_context; }
	void UpdateContext(CXfAttributeContext& c) { m_context = c; }
	bool IsClippingPath() { return m_bClipPath; }
	void SetIsClippingPath(bool b) { m_bClipPath = b; }

private:
	CXfAttributeContext m_context;
	bool m_bClipPath;
};

#include "XFTextContext.h"
/********************************************************************************************
>	class VXfTextRenderable

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The base virtual class that represents a Xara's node that can be rendered, such
			as a shape, regular shape, or a text char
********************************************************************************************/
class VXfTextRenderable : public VXfRenderable
{
public:
	VXfTextRenderable () {}
	VXfTextRenderable(UINT uiTag) : VXfRenderable(uiTag) {}
	CXfTextContext& TextContext() { return m_TextContext; }
	void UpdateTextContext(CXfTextContext& c) { m_TextContext = c; }
	virtual bool IsTextRenderable() { return true; }
	bool ShouldApplyURLColor();
	bool ShouldApplyURLUnderline();

private:
	CXfTextContext m_TextContext;
};

/********************************************************************************************
>	class CXfXaraTree

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	This is a Xara tree container. Is used to build and access Xara document tree
********************************************************************************************/
class CXfXaraTree
{
public:
	CXfXaraTree();
	BOOL HandleRecord(CXaraFileRecord* pRecord);
	VXfTreeNode* GetRoot() { return &m_pRoot; }
	RefNodeMap& GetRefs() { return m_Refs; }
	ObjectNameMap& GetObjectNameMap() { return m_ObjectName; }
	bool RefExists(UINT id) { return m_Refs.find(id) != m_Refs.end(); }
	static BOOL Write(VXfTreeNode* pNode, CXarExport* pExporter, ULONG* pRecordNum = NULL);
	void Delete();

private:
	VXfTreeNode m_pRoot;
	VXfTreeNode* m_pLast;
	VXfTreeNode* m_pActualFontDef;
	RefNodeMap m_Refs;
	PrevLinkMap m_PrevLink;
	ObjectNameMap m_ObjectName;
	//bool		m_bTextStory;
	std::vector<VXfTreeNode*> m_ParentStack;
	std::vector<CXfAttributeContext> m_AttrContext;
	std::vector<CXfTextContext> m_TextContext;
};

/********************************************************************************************
>	class CXfTreeNodeFactory

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	The static class that creates a Xara node based on a Xara file record
********************************************************************************************/
class CXfTreeNodeFactory
{
public:
	static VXfTreeNode* CreateTreeNode(CXaraFileRecord& rec);
};

/********************************************************************************************
>	class CXfZeroSizedNode

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Information Xara node
********************************************************************************************/
class CXfZeroSizedNode : public VXfTreeNode
{
public:
	CXfZeroSizedNode(UINT uTag) { SetTag(uTag); }
	CXfZeroSizedNode(CXaraFileRecord & rec) {}
	virtual VXfTreeNode* Copy() { return new CXfZeroSizedNode(GetTag()); }
};

class CXfGenericNode : public VXfTreeNode
{
public:
	CXfGenericNode(CXaraFileRecord& rec)
	{
		if(rec.GetSize())
		{
			Data.resize(rec.GetSize());
			memcpy(&Data[0], rec.GetBuffer(), rec.GetSize());
		}
	}
	virtual std::string CreateDumpString();
	std::vector<BYTE> Data;
};

class CXfFloatNode : public VXfTreeNode
{
public:
	CXfFloatNode()
	{
	}

	CXfFloatNode(const FIXED16 &dv)
		:Value(dv)
	{
	}

	CXfFloatNode(CXaraFileRecord& rec) : Value(rec)
	{
	}

	virtual std::string CreateDumpString();
	double GetValue() { return Value.get().MakeDouble(); }
	CXfFIXED16 Value;
};

/********************************************************************************************
>	class CXfString

Author:		Paul Gruzdev
Created:	21/10/12
Purpose:	Read/write string to/from Xara stream
********************************************************************************************/
class CXfString
{											
public:
	CXfString() {}
	CXfString(const std::wstring &psz) : Value(psz) {}
	CXfString(const wchar_t* psz) : Value(psz) {}
	CXfString( CXaraFileRecord & rec );
	void Read(CXaraFileRecord& rec);

	BOOL Write(CXaraFileRecord* rec)
	{
//#ifdef _UNICODE
		return rec->WriteWideString(Value.c_str());
//#else
//		return rec->WriteUnicode((TCHAR*)Value.c_str());
//#endif
	}
	operator wchar_t const * () const				
	{ return Value.c_str(); }						
											
	wchar_t const * get() const			    
	{ return Value.c_str(); }						

	std::wstring& getstr() { return Value; }						

private:									
	std::wstring Value;
};	

class CXfStringA
{											
public:
	CXfStringA() {}
	CXfStringA(const std::string &psz) : Value(psz) {}
	CXfStringA(const char* psz) : Value(psz) {}
	CXfStringA(CXaraFileRecord & rec);
	void Read(CXaraFileRecord& rec);
	BOOL Write(CXaraFileRecord* rec)
	{
		return rec->WriteBuffer((PBYTE)Value.c_str(), (ULONG)Value.length()+1);
	}
	operator char const * () const				
	{ return Value.c_str(); }						

	char const * get() const			    
	{ return Value.c_str(); }						

	std::string& getstr() { return Value; }						

private:									
	std::string Value;
};	

class CXfGenericStringNode : public VXfTreeNode
{
public:
	CXfGenericStringNode(const UINT tag, const std::wstring &str) : VXfTreeNode(tag), strData(str) {}
	CXfGenericStringNode(CXaraFileRecord & rec) : strData(rec) {}

	virtual std::string CreateDumpString();

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		strData.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

public:
	CXfString strData;
};

/********************************************************************************************
>	class CXfGroup

Author:		Paul Gruzdev
Created:	21/10/13
Purpose:	General group class.
********************************************************************************************/
class CXfGroup : public VXfBounded
{
public:
	CXfGroup() : VXfBounded(TAG_GROUP), m_bTransparencyGroup(false) {}
	CXfGroup(UINT uiTag) : VXfBounded(uiTag), m_bTransparencyGroup(false) {}
	CXfGroup(CXaraFileRecord & rec) : m_bTransparencyGroup(false) {}
	virtual bool IsRefNode() { return true; }
	virtual bool IsGroup() { return true; }
	void SetAsTransparencyGroup(bool bSet) { m_bTransparencyGroup = bSet; }
	bool IsTransparencyGroup() { return m_bTransparencyGroup; }
public:
	bool m_bTransparencyGroup;
};

/********************************************************************************************
>	class CXfClipviewController

Author:		Paul Gruzdev
Created:	12/01/14
Purpose:	Xara's clipview controller class
********************************************************************************************/
class CXfClipviewController : public CXfGroup
{
public:
	CXfClipviewController() : CXfGroup(TAG_CLIPVIEWCONTROLLER), m_bUpdatePolygons(true) {}
	CXfClipviewController(CXaraFileRecord & rec) {}
	std::string CreateDumpString();
	virtual bool IsRefNode() { return true; }
	virtual bool UpdateBBox(Rect& rcAdd);
	virtual void UpdateObjectsPolygon(VXfTreeNode* pChild) {};
	void SetUpdatePolygons(bool b) { m_bUpdatePolygons = b; }
	Rect& GetObjectsRect() { return m_rcObjects; }
	bool CalcObjectsRect(Rect& rcObjects);
protected:
	Rect m_rcObjects; // This is a bbox of all unclipped objects underneath (children)
	bool m_bUpdatePolygons;
};

/********************************************************************************************
>	class CXfObjectBounds

Author:		Paul Gruzdev
Created:	21/10/13
Purpose:	Child node of an object with it's bounds: TAG_OBJECTBOUNDS
********************************************************************************************/
class CXfObjectBounds : public VXfTreeNode
{
public:
	CXfObjectBounds(CXaraFileRecord & rec) : BottomLeft(rec), TopRight(rec) {}

	virtual std::string CreateDumpString();

	CXfDocCoord BottomLeft;
	CXfDocCoord TopRight;
};

class CXfDefineDefaultUnits : public VXfTreeNode
{
public:
	CXfDefineDefaultUnits(CXaraFileRecord& rec) : PageUnits(rec), FontUnits(rec) {}
	CXfDefineDefaultUnits(LONG lPageUnits, LONG lFontUnits) : PageUnits(lPageUnits), FontUnits(lFontUnits), VXfTreeNode(TAG_DEFINE_DEFAULTUNITS) {}

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		PageUnits.Write(&Rec);
		FontUnits.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfLONG PageUnits;
	CXfLONG FontUnits;
};

/********************************************************************************************
>	class CXfSymbolGroup

Author:		Denis Onishchenko
Created:	04/10/17
Purpose:	Compound objects whose contents may be automatically copied or rendered in place of each other: TAG_SYMBOLGROUP/TAG_SYMBOLGROUP_LIVECOPY
********************************************************************************************/

class CXfSymbolGroup : public VXfTreeNode
{
public:
	CXfSymbolGroup(const Matrix &instanceTransform, BOOL pageRepeat, BOOL liveCopy = TRUE) : VXfTreeNode(liveCopy ? TAG_SYMBOLGROUP_LIVECOPY : TAG_SYMBOLGROUP), InstanceTransform(instanceTransform), PageRepeat(pageRepeat)
	{
	}

	CXfSymbolGroup(CXaraFileRecord & rec) : InstanceTransform(rec), PageRepeat(rec) {}

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		InstanceTransform.Write(&Rec);
		PageRepeat.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfMatrix InstanceTransform;
	CXfBOOL PageRepeat;
};

/********************************************************************************************
>	class CXfUserValue

Author:		Paul Gruzdev
Created:	26/02/2016
Purpose:	Representation of TAG_USERVALUE record
********************************************************************************************/
class CXfUserValue : public VXfTreeNode
{
public:
	CXfUserValue(const wchar_t *key, const wchar_t *value) : VXfTreeNode(TAG_USERVALUE), Key(key), Value(value) {}

	CXfUserValue(CXaraFileRecord & rec) : Key(rec), Value(rec) {}

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_USERVALUE);
		Rec.Init();
		Key.Write(&Rec);
		Value.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	virtual std::string CreateDumpString();

	CXfString Key;
	CXfString Value;
};

/********************************************************************************************
>	class CXfNodeDetails

Author:		Denis Onishchenko
Created:	05/04/2016
Purpose:	Representation of TAG_NODEDETAILS record
********************************************************************************************/
class CXfNodeDetails : public VXfTreeNode
{
public:
	CXfNodeDetails(CXaraFileRecord & rec) : Flags(rec) {}

	CXfNodeDetails(BYTE flags) : VXfTreeNode(TAG_NODEDETAILS), Flags(flags) {}

	CXfNodeDetails(BOOL bVisible, BOOL bLocked) : VXfTreeNode(TAG_NODEDETAILS), Flags((bVisible ? 1 : 0) | (bLocked ? 2 : 0)) {}

	BOOL IsVisible() const
	{
		return 1 == (1 & Flags);
	}

	BOOL IsLocked() const
	{
		return 2 == (2 & Flags);
	}

	virtual std::string CreateDumpString();

	BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TAG_NODEDETAILS, TAG_NODEDETAILS_SIZE);
		Rec.Init();
		Flags.Write(&Rec);
		return pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfBYTE Flags;
};

#ifdef _DEBUG
	void ValidateTransparencyType(BYTE t);
#else
	#define ValidateTransparencyType
#endif

#include "XFBitmap.h"
#include "XFFill.h"
#include "XFColor.h"
#include "XFRenderable.h"
#include "XFLineProps.h"
#include "XFNodeValue.h"
#include "XFDocument.h"
#include "XFText.h"
#include "XFWeb.h"
#include "XFShadow.h"
#include "XFDash.h"
#include "XFSmartShapeGroup.h"
