/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#pragma once

/********************************************************************************************
>	class CXfWebAddress

Author:		Paul Gruzdev
Created:	17/01/13
Purpose:	TAG_WEBADDRESS
********************************************************************************************/
class CXfWebAddress : public VXfRenderable
{
public:
	enum { External, Page, Object, Layer, ScaledPopup, File };

	CXfWebAddress() : VXfRenderable(TAG_WEBADDRESS) {}
	CXfWebAddress(UINT uType, const wchar_t* pszURI, const wchar_t* pszFrame) : VXfRenderable(-1), Type(uType), URL(pszURI), Frame(pszFrame) 
	{
		SetTag(TagByType(uType));
	}

	ULONG TagByType(UINT uType)
	{
		switch(uType)
		{
		case External:
			return HasBoundingBox() ? TAG_WEBADDRESS_BOUNDINGBOX : TAG_WEBADDRESS;
		case Page:
			return HasBoundingBox() ? TAG_WEBADDRESS_PAGE_BOUNDINGBOX : TAG_WEBADDRESS_PAGE;
		case Object:
			return HasBoundingBox() ? TAG_WEBADDRESS_OBJECT_BOUNDINGBOX : TAG_WEBADDRESS_OBJECT;
		case Layer:
			return HasBoundingBox() ? TAG_WEBADDRESS_LAYER_BOUNDINGBOX : TAG_WEBADDRESS_LAYER;
		case ScaledPopup:
			return HasBoundingBox() ? TAG_WEBADDRESS_SCALEDPOPUP_BOUNDINGBOX : TAG_WEBADDRESS_SCALEDPOPUP;
		case File:
			return HasBoundingBox() ? TAG_WEBADDRESS_FILE_BOUNDINGBOX : TAG_WEBADDRESS_FILE;
		}
		return TAG_WEBADDRESS;
	}

	ULONG TagByType() { return TagByType(Type); }

	void SetType(CXaraFileRecord& rec)
	{
		switch(rec.GetTag())
		{
		case TAG_WEBADDRESS:
		case TAG_WEBADDRESS_BOUNDINGBOX:
			Type = External;
			break;
		case TAG_WEBADDRESS_PAGE:
		case TAG_WEBADDRESS_PAGE_BOUNDINGBOX:
			Type = Page;
			break;
		case TAG_WEBADDRESS_OBJECT:
		case TAG_WEBADDRESS_OBJECT_BOUNDINGBOX:
			Type = Object;
			break;
		case TAG_WEBADDRESS_LAYER:
		case TAG_WEBADDRESS_LAYER_BOUNDINGBOX:
			Type = Layer;
			break;
		case TAG_WEBADDRESS_SCALEDPOPUP:
		case TAG_WEBADDRESS_SCALEDPOPUP_BOUNDINGBOX:
			Type = ScaledPopup;
			break;
		case TAG_WEBADDRESS_FILE:
		case TAG_WEBADDRESS_FILE_BOUNDINGBOX:
			Type = File;
			break;
		}
	}

	CXfWebAddress(CXaraFileRecord& rec) : URL(rec), Frame(rec) { SetType(rec); }

	void Init(CXaraFileRecord& rec)
	{
		URL.Read(rec);
		Frame.Read(rec);
		SetType(rec);
	}

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TagByType());
		Rec.Init();
		BOOL ok = URL.Write(&Rec);
		ok = ok && Frame.Write(&Rec);
		return ok && pExporter->WriteRecord(&Rec, pRecordNum);
	}
	virtual std::string CreateDumpString();

	virtual bool IsWebAddress() { return true; }
	virtual bool HasBoundingBox() { return false; }

	UINT Type;
	CXfString URL;
	CXfString Frame;
};

/********************************************************************************************
>	class CXfWebAddressBoundingBox

Author:		Paul Gruzdev
Created:	17/01/13
Purpose:	TAG_WEBADDRESS_BOUNDINGBOX
********************************************************************************************/
class CXfWebAddressBoundingBox : public CXfWebAddress
{
public:
	CXfWebAddressBoundingBox(CXaraFileRecord & rec)
		: BottomLeft(rec), TopRight(rec)
	{
		Init(rec);
	}

	CXfWebAddressBoundingBox(DocCoord dcBottomLeft, DocCoord dcTopRight, UINT uType, const wchar_t* pszURI, const wchar_t* pszFrame)
		: BottomLeft(dcBottomLeft), TopRight(dcTopRight), CXfWebAddress(uType, pszURI, pszFrame) 
	{
		SetTag(TagByType(uType));
	}

	virtual bool IsWebAddress() { return true; }
	virtual bool HasBoundingBox() { return true; }

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(TagByType());
		Rec.Init();
		BOOL ok = BottomLeft.Write(&Rec);
		ok = ok && TopRight.Write(&Rec);
		ok = ok && URL.Write(&Rec);
		ok = ok && Frame.Write(&Rec);
		return ok && pExporter->WriteRecord(&Rec, pRecordNum);
	}

	CXfDocCoordInterleaved BottomLeft;
	CXfDocCoordInterleaved TopRight;
	//const CXfWebAddress Address;
};

/********************************************************************************************
Author:		Ekaterina Dolgunicheva	
Created:	10/07/15
Purpose:	Helpers for CXfWizop - Splitting the parameter that contains margin values (_nagmar:<top>:<right>:<bottom>:<left>) 
			into an integer vector
********************************************************************************************/
inline bool colon(wchar_t c){
	return c == ':';
}

inline bool notcolon(wchar_t c){
	return c != ':';
}

static std::vector<int> split(const std::wstring& s){
	typedef std::wstring::const_iterator iter;
	std::vector<int> ret;
	iter i = s.begin();
	while(i!=s.end()){
		i = std::find_if(i, s.end(), notcolon); // find the beginning of a word
		iter j = std::find_if(i, s.end(), colon); // find the end of the same word
		if(i!=s.end()){
			ret.push_back(_wtoi(std::wstring(i, j).c_str())); //insert the word into vector
			i = j;
		}
	}
	return ret;
}

/********************************************************************************************
>	class CXfWizop

Author:		Paul Gruzdev; Ekaterina Dolgunicheva (ParseObjectNameIncludeAlignedEmbedding and ParseObjectMargins)
Created:	17/01/13, 10/07/15
Purpose:	TAG_WIZOP
********************************************************************************************/
class CXfWizop: public VXfTreeNode
{
public:
	enum { AnchorPosition, PrimaryGroup, SecondaryGroup };
	enum { InlineEmbedded, LeftAligned, RightAligned };

	CXfWizop(CXaraFileRecord & rec)
		: Name(rec), String(rec), Parameter(rec), PatchSpace(rec)
	{
	}
	CXfWizop(const wchar_t* sName, const wchar_t* sString, const wchar_t* sParam, const wchar_t* sPatchSpace)
		: VXfTreeNode(TAG_WIZOP), Name(sName), String(sString), Parameter(sParam) 
	{
	}
	bool IsObjectName() { return Name.getstr() == L"ObjectName"; }
	bool IsHTMLText() { return Name.getstr() == L"ObjectName" && Parameter.getstr() == L"HTMLText"; }

	virtual BOOL Write(CXarExport* pExporter, ULONG* pRecordNum = NULL)
	{
		CXaraFileRecord Rec(GetTag());
		Rec.Init();
		BOOL ok = Name.Write(&Rec);
		ok = ok && String.Write(&Rec);
		ok = ok && Parameter.Write(&Rec);
		ok = ok && PatchSpace.Write(&Rec);
		return ok && pExporter->WriteRecord(&Rec, pRecordNum);
	}

	static bool ParseObjectName(std::wstring str, UINT& uiID, UINT& eType)
	{
		//std::wstring str = Parameter.getstr();
		if(str.substr(0, 4) == L"_nag")
		{
			std::wstring strType = str.substr(4, 3);
			//_naganc:id
			std::wstring strNum = str.substr(8, str.length()-8);
			uiID = _wtoi(strNum.c_str());
			if(strType == L"anc") eType = AnchorPosition;
			else if(strType == L"pri") eType = PrimaryGroup;
			else if(strType == L"sec") eType = SecondaryGroup;

			return true;
		}
		return false;
	}

	static bool ParseObjectNameIncludeAlignedEmbedding(std::wstring str, UINT& uiID, UINT& embType, UINT& eType)
	{
		//std::wstring str = Parameter.getstr();
		std::wstring embStr = str.substr(0, 4);
		if (embStr == L"_nag") embType = InlineEmbedded;
		else if (embStr == L"_nlg") embType = LeftAligned;
		else if (embStr == L"_nrg") embType = RightAligned;
		else 
			return false;

		std::wstring strType = str.substr(4, 3);
		std::wstring strNum = str.substr(8, str.length()-8);
		uiID = _wtoi(strNum.c_str());
		if(strType == L"anc") eType = AnchorPosition;
		else if(strType == L"pri") eType = PrimaryGroup;
		else if(strType == L"sec") eType = SecondaryGroup;

		return true;

	}

	static bool ParseObjectMargins(std::wstring str, std::vector<int>& vals)
	{
		//std::wstring str = Parameter.getstr(); // _nagmar:<top>:<right>:<bottom>:<left>
		if(str.substr(4, 3) == L"mar")
		{
			std::wstring strMargins = str.substr(8, strMargins.length()-8);
			vals = split(strMargins);
		}
		return false;
	}

	static std::wstring CreateObjectName(UINT uiID, UINT eType)
	{
		std::wstring s = L"_nag";
		if(eType == AnchorPosition) s += L"anc";
		else if(eType == PrimaryGroup) s += L"pri";
		else if(eType == SecondaryGroup) s += L"sec";
		wchar_t psz[16];
		swprintf_s(psz, 16, L":%d", uiID);
		s += psz;
		return s;
	}

	static std::wstring CreateObjectNameIncludeAlignedEmbedding(UINT uiID, UINT embType, UINT eType)
	{
		std::wstring s = L"";
		if (embType == InlineEmbedded) s += L"_nag";
		else if (embType == LeftAligned) s += L"_nlg";
		else if (embType == RightAligned) s += L"_nrg";

		if(eType == AnchorPosition) s += L"anc";
		else if(eType == PrimaryGroup) s += L"pri";
		else if(eType == SecondaryGroup) s += L"sec";
		wchar_t psz[16];
		swprintf_s(psz, 16, L":%d", uiID);
		s += psz;
		return s;
	}

	virtual std::string CreateDumpString();

	CXfString Name;
	CXfString String;
	CXfString Parameter;
	CXfString PatchSpace;
};
