
static std::string XaraTagToString(ULONG Tag)
{
	switch (Tag)
	{
#define TAG_OUT(__n)\
	case __n: return std::string(#__n);\
	break;
// The new auto-generated tags from a proxy utility DumpXaraTag. Keep them updated

// This file defines all the allocated tags in the v2 CorelXARA file format


	//////////////////////////////////////////////////////////////////////////
	// IMPORTANT															//
	// To keep things tidy & maintainable, tags from previous releases of	//
	// camelot are now FROZEN. You MUST NOT add tags with values in "old"	//
	// ranges: See the bottom of this file for the current tag range to use	//
	//																		//
	// Allocated ranges are:												//
	// Range		Comments												//
	// ---------    --------												//
	// 0-3999		Initial release (Camelot v1.5, 1st file format release) //
	// 4000-4999	Second release (Camelot v2.0)							//
	//////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// Camelot version 1.5 tags - range 0..3999 - FROZEN!
/////////////////////////////////////////////////////////////////////////////
//some harmless comments
//-----------------
// General tags

			TAG_OUT(TAG_UNDEFINED)

			TAG_OUT(TAG_UP)
			TAG_OUT(TAG_DOWN)
			TAG_OUT(TAG_FILEHEADER)
			TAG_OUT(TAG_ENDOFFILE)

//-----------------
// Tag management
			TAG_OUT(TAG_ATOMICTAGS)
			TAG_OUT(TAG_ESSENTIALTAGS)
			TAG_OUT(TAG_TAGDESCRIPTION)

//-----------------
// Render-related tags
			TAG_OUT(TAG_NONRENDERSECTION_START)
			TAG_OUT(TAG_NONRENDERSECTION_END)
			TAG_OUT(TAG_RENDERING_PAUSE)
			TAG_OUT(TAG_RENDERING_RESUME)

//-----------------
// Compression & precompression tags 

			TAG_OUT(TAG_STARTCOMPRESSION)
			TAG_OUT(TAG_ENDCOMPRESSION)

//-----------------
// Document tags

			TAG_OUT(TAG_DOCUMENT)
			TAG_OUT(TAG_CHAPTER)
			TAG_OUT(TAG_SPREAD)
			TAG_OUT(TAG_LAYER)
			TAG_OUT(TAG_PAGE)
			TAG_OUT(TAG_SPREADINFORMATION)

			TAG_OUT(TAG_GRIDRULERSETTINGS)
			TAG_OUT(TAG_GRIDRULERORIGIN)
			TAG_OUT(TAG_LAYERDETAILS)
			TAG_OUT(TAG_GUIDELAYERDETAILS)

			TAG_OUT(TAG_SPREADSCALING_ACTIVE)
			TAG_OUT(TAG_SPREADSCALING_INACTIVE)

//-----------------
// Colour reference tags

			TAG_OUT(TAG_DEFINERGBCOLOUR)
			TAG_OUT(TAG_DEFINECOMPLEXCOLOUR)

//-----------------
// Bitmap reference tags

			TAG_OUT(TAG_PREVIEWBITMAP_BMP)
			TAG_OUT(TAG_PREVIEWBITMAP_GIF)
			TAG_OUT(TAG_PREVIEWBITMAP_JPEG)
			TAG_OUT(TAG_PREVIEWBITMAP_PNG)
			TAG_OUT(TAG_PREVIEWBITMAP_TIFFLZW)

			TAG_OUT(TAG_DEFINEBITMAP_BMP)
			TAG_OUT(TAG_DEFINEBITMAP_GIF)
			TAG_OUT(TAG_DEFINEBITMAP_JPEG)
			TAG_OUT(TAG_DEFINEBITMAP_PNG)
			TAG_OUT(TAG_DEFINEBITMAP_BMPZIP)
			TAG_OUT(TAG_DEFINESOUND_WAV)
			TAG_OUT(TAG_DEFINEBITMAP_JPEG8BPP)

//-----------------
// View tags

			TAG_OUT(TAG_VIEWPORT)
			TAG_OUT(TAG_VIEWQUALITY)
			TAG_OUT(TAG_DOCUMENTVIEW)

//-----------------
// Document unit tags

			TAG_OUT(TAG_DEFINE_PREFIXUSERUNIT)
			TAG_OUT(TAG_DEFINE_SUFFIXUSERUNIT)
			TAG_OUT(TAG_DEFINE_DEFAULTUNITS)

//-----------------
// Document info tags

			TAG_OUT(TAG_DOCUMENTCOMMENT)
			TAG_OUT(TAG_DOCUMENTDATES)
			TAG_OUT(TAG_DOCUMENTUNDOSIZE)
			TAG_OUT(TAG_DOCUMENTFLAGS)

//-----------------
// Name Gallery tags

			TAG_OUT(TAG_NAMEGAL_DOCCOMP)

//-----------------
// Object tags

			TAG_OUT(TAG_PATH)
			TAG_OUT(TAG_PATH_FILLED)
			TAG_OUT(TAG_PATH_STROKED)
			TAG_OUT(TAG_PATH_FILLED_STROKED)
			TAG_OUT(TAG_GROUP)
			TAG_OUT(TAG_BLEND)
			TAG_OUT(TAG_BLENDER)
			TAG_OUT(TAG_MOULD_ENVELOPE)
			TAG_OUT(TAG_MOULD_PERSPECTIVE)
			TAG_OUT(TAG_MOULD_GROUP)
			TAG_OUT(TAG_MOULD_PATH)
			TAG_OUT(TAG_PATH_FLAGS)
			TAG_OUT(TAG_GUIDELINE)
			TAG_OUT(TAG_PATH_RELATIVE)
			TAG_OUT(TAG_PATH_RELATIVE_FILLED)
			TAG_OUT(TAG_PATH_RELATIVE_STROKED)
			TAG_OUT(TAG_PATH_RELATIVE_FILLED_STROKED)
			TAG_OUT(TAG_PATHREF_TRANSFORM)

//-----------------
// Attribute tags

			TAG_OUT(TAG_FLATFILL)
			TAG_OUT(TAG_LINECOLOUR)
			TAG_OUT(TAG_LINEWIDTH)
			TAG_OUT(TAG_LINEARFILL)
			TAG_OUT(TAG_CIRCULARFILL)
			TAG_OUT(TAG_ELLIPTICALFILL)
			TAG_OUT(TAG_CONICALFILL)
			TAG_OUT(TAG_BITMAPFILL)
			TAG_OUT(TAG_CONTONEBITMAPFILL)
			TAG_OUT(TAG_FRACTALFILL)
			TAG_OUT(TAG_FILLEFFECT_FADE)
			TAG_OUT(TAG_FILLEFFECT_RAINBOW)
			TAG_OUT(TAG_FILLEFFECT_ALTRAINBOW)
			TAG_OUT(TAG_FILL_REPEATING)
			TAG_OUT(TAG_FILL_NONREPEATING)
			TAG_OUT(TAG_FILL_REPEATINGINVERTED)
			TAG_OUT(TAG_FLATTRANSPARENTFILL)
			TAG_OUT(TAG_LINEARTRANSPARENTFILL)
			TAG_OUT(TAG_CIRCULARTRANSPARENTFILL)
			TAG_OUT(TAG_ELLIPTICALTRANSPARENTFILL)
			TAG_OUT(TAG_CONICALTRANSPARENTFILL)
			TAG_OUT(TAG_BITMAPTRANSPARENTFILL)
			TAG_OUT(TAG_FRACTALTRANSPARENTFILL)
			TAG_OUT(TAG_LINETRANSPARENCY)
			TAG_OUT(TAG_STARTCAP)
			TAG_OUT(TAG_ENDCAP)
			TAG_OUT(TAG_JOINSTYLE)
			TAG_OUT(TAG_MITRELIMIT)
			TAG_OUT(TAG_WINDINGRULE)
			TAG_OUT(TAG_QUALITY)
			TAG_OUT(TAG_TRANSPARENTFILL_REPEATING)
			TAG_OUT(TAG_TRANSPARENTFILL_NONREPEATING)
			TAG_OUT(TAG_TRANSPARENTFILL_REPEATINGINVERTED)

// Arrows and dash patterns
			TAG_OUT(TAG_DASHSTYLE)
			TAG_OUT(TAG_DEFINEDASH)
			TAG_OUT(TAG_ARROWHEAD)
			TAG_OUT(TAG_ARROWTAIL)
			TAG_OUT(TAG_DEFINEARROW)
			TAG_OUT(TAG_DEFINEDASH_SCALED)

// User Attributes
			TAG_OUT(TAG_USERVALUE)

// special colour fills
			TAG_OUT(TAG_FLATFILL_NONE)
			TAG_OUT(TAG_FLATFILL_BLACK)
			TAG_OUT(TAG_FLATFILL_WHITE)
			TAG_OUT(TAG_LINECOLOUR_NONE)
			TAG_OUT(TAG_LINECOLOUR_BLACK)
			TAG_OUT(TAG_LINECOLOUR_WHITE)

// Bitmaps
// general node bitmap - don't know how long this will stay like this
// There are two names for tag 198 for backwards compatibility reasons
			TAG_OUT(TAG_NODE_BITMAP)
//			TAG_OUT(TAG_BITMAP_OBJECT)
			TAG_OUT(TAG_NODE_CONTONEDBITMAP)


// New fill type records
//
// Tags range 200 - 299

			TAG_OUT(TAG_SQUAREFILL)
			TAG_OUT(TAG_SQUARETRANSPARENTFILL)
			TAG_OUT(TAG_THREECOLFILL)
			TAG_OUT(TAG_THREECOLTRANSPARENTFILL)
			TAG_OUT(TAG_FOURCOLFILL)
			TAG_OUT(TAG_FOURCOLTRANSPARENTFILL)
//Mark Howitt, 8/10/97. Include new repeating grad fill tag
			TAG_OUT(TAG_FILL_REPEATING_EXTRA)
//Chris Snook, 8/12/99. We also need one for the transparencies!
			TAG_OUT(TAG_TRANSPARENTFILL_REPEATING_EXTRA)


//-----------------
// Regular shapes
// Similarly, tags in the range 1000 - 1999 are reserved for regular shapes.
// Not all are defined in the spec, those that are are marked. However all these
// get used somewhere (I think).

// Ellipses
			TAG_OUT(TAG_ELLIPSE_SIMPLE)
			TAG_OUT(TAG_ELLIPSE_COMPLEX)

// Rectangles
			TAG_OUT(TAG_RECTANGLE_SIMPLE)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_REFORMED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_STELLATED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_STELLATED_REFORMED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_ROUNDED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_ROUNDED_REFORMED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED)
			TAG_OUT(TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED_REFORMED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_REFORMED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_STELLATED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_STELLATED_REFORMED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_ROUNDED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_ROUNDED_REFORMED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED)
			TAG_OUT(TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED_REFORMED)

// Polygons
			TAG_OUT(TAG_POLYGON_COMPLEX)
			TAG_OUT(TAG_POLYGON_COMPLEX_REFORMED)
			TAG_OUT(TAG_POLYGON_COMPLEX_STELLATED)
			TAG_OUT(TAG_POLYGON_COMPLEX_STELLATED_REFORMED)
			TAG_OUT(TAG_POLYGON_COMPLEX_ROUNDED)
			TAG_OUT(TAG_POLYGON_COMPLEX_ROUNDED_REFORMED)
			TAG_OUT(TAG_POLYGON_COMPLEX_ROUNDED_STELLATED)
			TAG_OUT(TAG_POLYGON_COMPLEX_ROUNDED_STELLATED_REFORMED)

// General regular shapes
//
// (The first, TAG_REGULAR_SHAPE_PHASE was used briefly during development of version
// 2 of Camelot. It contains _all_ information about regular shapes, including the
// UTCentrePoint. The UTCentrePoint is not guaranteed to be DocCoord(0,0) for all
// regular shapes, but once this was not the case and a few files (on the CD!)
// are old enough to have regular shapes with UTCentrePoint != DocCoord(0,0).
// We still need to have this tag since some files may have been created which
// use it; Charles gets annoyed if files suddenly stop loading for no apparent reason.
// The exporting code has been removed, the importing code is in kernel/rechrshp.cpp
// The second tag below is now used for all regular shapes, any old regualar shapes
// get fixed to place UTCentrePoint at DocCoord(0,0), UTMajorAxis, UTMinorAxis and
// the TransformMatrix get altered accordingly, before saving. The code for fixing
// regular shapes is in kernel/cxfrgshp.cpp.
// Understand? - good, I'm not going to repeat myself.
//
// Ach

			TAG_OUT(TAG_REGULAR_SHAPE_PHASE_1)
			TAG_OUT(TAG_REGULAR_SHAPE_PHASE_2)

// Text related records
//
// Tags range 2000 - 2999 are reserved for text related records; this may change in the
// future but it will do for now.

// Text objects

// Text definitions
			TAG_OUT(TAG_FONT_DEF_TRUETYPE)
			TAG_OUT(TAG_FONT_DEF_ATM)

// vanilla text story objects
			TAG_OUT(TAG_TEXT_STORY_SIMPLE)
			TAG_OUT(TAG_TEXT_STORY_COMPLEX)

// text story objects on a path
			TAG_OUT(TAG_TEXT_STORY_SIMPLE_START_LEFT)
			TAG_OUT(TAG_TEXT_STORY_SIMPLE_START_RIGHT)
			TAG_OUT(TAG_TEXT_STORY_SIMPLE_END_LEFT)
			TAG_OUT(TAG_TEXT_STORY_SIMPLE_END_RIGHT)
			TAG_OUT(TAG_TEXT_STORY_COMPLEX_START_LEFT)
			TAG_OUT(TAG_TEXT_STORY_COMPLEX_START_RIGHT)
			TAG_OUT(TAG_TEXT_STORY_COMPLEX_END_LEFT)
			TAG_OUT(TAG_TEXT_STORY_COMPLEX_END_RIGHT)

// Text story information records
			TAG_OUT(TAG_TEXT_STORY_WORD_WRAP_INFO)
			TAG_OUT(TAG_TEXT_STORY_INDENT_INFO)

// other text story related objects
			TAG_OUT(TAG_TEXT_LINE)
			TAG_OUT(TAG_TEXT_STRING)
			TAG_OUT(TAG_TEXT_CHAR)
			TAG_OUT(TAG_TEXT_EOL)
			TAG_OUT(TAG_TEXT_KERN)
			TAG_OUT(TAG_TEXT_CARET)
			TAG_OUT(TAG_TEXT_LINE_INFO)

// Text attributes
			TAG_OUT(TAG_TEXT_LINESPACE_RATIO)
			TAG_OUT(TAG_TEXT_LINESPACE_ABSOLUTE)
			TAG_OUT(TAG_TEXT_JUSTIFICATION_LEFT)
			TAG_OUT(TAG_TEXT_JUSTIFICATION_CENTRE)
			TAG_OUT(TAG_TEXT_JUSTIFICATION_RIGHT)
			TAG_OUT(TAG_TEXT_JUSTIFICATION_FULL)
			TAG_OUT(TAG_TEXT_FONT_SIZE)
			TAG_OUT(TAG_TEXT_FONT_TYPEFACE)
			TAG_OUT(TAG_TEXT_BOLD_ON)
			TAG_OUT(TAG_TEXT_BOLD_OFF)
			TAG_OUT(TAG_TEXT_ITALIC_ON)
			TAG_OUT(TAG_TEXT_ITALIC_OFF)
			TAG_OUT(TAG_TEXT_UNDERLINE_ON)
			TAG_OUT(TAG_TEXT_UNDERLINE_OFF)
			TAG_OUT(TAG_TEXT_SCRIPT_ON)
			TAG_OUT(TAG_TEXT_SCRIPT_OFF)
			TAG_OUT(TAG_TEXT_SUPERSCRIPT_ON)
			TAG_OUT(TAG_TEXT_SUBSCRIPT_ON)
			TAG_OUT(TAG_TEXT_TRACKING)
			TAG_OUT(TAG_TEXT_ASPECT_RATIO)
			TAG_OUT(TAG_TEXT_BASELINE)
			TAG_OUT(TAG_TEXT_STRIKETHROUGH_ON)
			TAG_OUT(TAG_TEXT_STRIKETHROUGH_OFF)

											// Next	2923


//-----------------
// Imagesetting and other print-related tags - reserved range 3500 - 3599
// Imagesetting attributes
			TAG_OUT(TAG_OVERPRINTLINEON)
			TAG_OUT(TAG_OVERPRINTLINEOFF)
			TAG_OUT(TAG_OVERPRINTFILLON)
			TAG_OUT(TAG_OVERPRINTFILLOFF)
			TAG_OUT(TAG_PRINTONALLPLATESON)
			TAG_OUT(TAG_PRINTONALLPLATESOFF)
													// Full up
// Document Print/Imagesetting options
			TAG_OUT(TAG_PRINTERSETTINGS)
			TAG_OUT(TAG_IMAGESETTING)
			TAG_OUT(TAG_COLOURPLATE)
													// Full up
// Registration mark records
			TAG_OUT(TAG_PRINTMARKDEFAULT)
			TAG_OUT(TAG_PRINTMARKCUSTOM)
													// Next 3511
													// Last 3599


/********************************************
These have not been used in v1 of the file format, so therefore have been removed
//-----------------
// Transform attributes

// Misc transformations
			TAG_OUT(TAG_TRANSFORM_IDENTITY)
			TAG_OUT(TAG_TRANSFORM_MATRIX)
			TAG_OUT(TAG_TRANSFORM_TRANSLATION)
			TAG_OUT(TAG_TRANSFORM_GENERAL)

// Rotations
			TAG_OUT(TAG_TRANSFORM_ROTATION)
			TAG_OUT(TAG_TRANSFORM_ROTATION_CENTRE)
			TAG_OUT(TAG_TRANSFORM_ROTATION_POINT)

// Shears
			TAG_OUT(TAG_TRANSFORM_SHEAR_X)
			TAG_OUT(TAG_TRANSFORM_SHEAR_Y)

// Scalings
			TAG_OUT(TAG_TRANSFORM_SCALE)
			TAG_OUT(TAG_TRANSFORM_SCALE_X)
			TAG_OUT(TAG_TRANSFORM_SCALE_Y)

// Reflections
			TAG_OUT(TAG_TRANSFORM_REFLECTION_X)
			TAG_OUT(TAG_TRANSFORM_REFLECTION_Y)
**********************************************/



/////////////////////////////////////////////////////////////////////////////
// Camelot version 2.0 tags - range 4000-4999
/////////////////////////////////////////////////////////////////////////////

//-----------------
// Stroke Provider tags
			TAG_OUT(TAG_VARIABLEWIDTHFUNC)
			TAG_OUT(TAG_VARIABLEWIDTHTABLE)
			TAG_OUT(TAG_STROKETYPE)
			TAG_OUT(TAG_STROKEDEFINITION)
			TAG_OUT(TAG_STROKEAIRBRUSH)
											// Next 4005

// Next Free Tag = 4010 (please leave me some space for future expansion)

			TAG_OUT(TAG_NOISEFILL)
			TAG_OUT(TAG_NOISETRANSPARENTFILL)
			TAG_OUT(TAG_MOULD_BOUNDS)

// One moved and one new pathref tags
			TAG_OUT(TAG_PATHREF_IDENTICAL)
			TAG_OUT(TAG_PATHREF_TRANSLATE)
			TAG_OUT(TAG_EXPORT_HINT)
											// Next 4016

//-----------------
// New Web Address tags
			TAG_OUT(TAG_WEBADDRESS)
			TAG_OUT(TAG_WEBADDRESS_BOUNDINGBOX)
											// Next 4022

//-------------------
// Frmae Layer  tags
			TAG_OUT(TAG_LAYER_FRAMEPROPS)
			TAG_OUT(TAG_SPREAD_ANIMPROPS)

//-----------------
// Template Argument/Ops / WizOp / Webster Properties tags
			TAG_OUT(TAG_WIZOP)
			TAG_OUT(TAG_WIZOP_STYLE)
			TAG_OUT(TAG_WIZOP_STYLEREF)
											

/////////////////////////////////////////////////////////////////////////////
/// Camelot version 2.1 tags i.e. next version after 2.0
/////////////////////////////////////////////////////////////////////////////

//-----------------
// Shadows
			TAG_OUT(TAG_SHADOWCONTROLLER)
			TAG_OUT(TAG_SHADOW)

//-----------------
// Bevels
			TAG_OUT(TAG_BEVEL)

// variable tags
			TAG_OUT(TAG_BEVATTR_INDENT)
			TAG_OUT(TAG_BEVATTR_LIGHTANGLE)
			TAG_OUT(TAG_BEVATTR_CONTRAST)
			TAG_OUT(TAG_BEVATTR_TYPE)

// the bevel inking node tag
			TAG_OUT(TAG_BEVELINK)

//-----------------
// Blend on a curve tags
			TAG_OUT(TAG_BLENDER_CURVEPROP)
			TAG_OUT(TAG_BLEND_PATH)
			TAG_OUT(TAG_BLENDER_CURVEANGLES)

// Group Transparency Groups
			TAG_OUT(TAG_GROUPTRANSP)
			TAG_OUT(TAG_CACHEBMP)
			TAG_OUT(TAG_CACHEDNODESGROUP)

//-----------------
// Contouring tags
			TAG_OUT(TAG_CONTOURCONTROLLER)
			TAG_OUT(TAG_CONTOUR)

//-----------------
// Sets and their properties
			TAG_OUT(TAG_SETSENTINEL)
			TAG_OUT(TAG_SETPROPERTY)

// more Blended on a curve tags
			TAG_OUT(TAG_BLENDPROFILES)
			TAG_OUT(TAG_BLENDERADDITIONAL)
			TAG_OUT(TAG_NODEBLENDPATH_FILLED)

//-----------------
// Multi stage fill tags
			TAG_OUT(TAG_LINEARFILLMULTISTAGE)
			TAG_OUT(TAG_CIRCULARFILLMULTISTAGE)
			TAG_OUT(TAG_ELLIPTICALFILLMULTISTAGE)
			TAG_OUT(TAG_CONICALFILLMULTISTAGE)

//harmless test comment
//-----------------
// painting mode attribute tags

// brush attribute tags
			TAG_OUT(TAG_BRUSHATTR)
			TAG_OUT(TAG_BRUSHDEFINITION)
			TAG_OUT(TAG_BRUSHDATA)
			TAG_OUT(TAG_MOREBRUSHDATA)
			TAG_OUT(TAG_MOREBRUSHATTR)


//-----------------
// ClipView tags
			TAG_OUT(TAG_CLIPVIEWCONTROLLER)
			TAG_OUT(TAG_CLIPVIEW)

//-----------------
// Feathering tags
			TAG_OUT(TAG_FEATHER)

//-----------------
// Bar properties
			TAG_OUT(TAG_BARPROPERTY)

//-----------------
// Other multi stage fill tags
			TAG_OUT(TAG_SQUAREFILLMULTISTAGE)

//-----------------
// Tags for saving current attributes. Currently unused.

// more brush tags
			TAG_OUT(TAG_EVENMOREBRUSHDATA)
			TAG_OUT(TAG_EVENMOREBRUSHATTR)
			TAG_OUT(TAG_TIMESTAMPBRUSHDATA)
			TAG_OUT(TAG_BRUSHPRESSUREINFO)
			TAG_OUT(TAG_BRUSHPRESSUREDATA)
			TAG_OUT(TAG_BRUSHATTRPRESSUREINFO)
			TAG_OUT(TAG_BRUSHCOLOURDATA)
			TAG_OUT(TAG_BRUSHPRESSURESAMPLEDATA)
			TAG_OUT(TAG_BRUSHTIMESAMPLEDATA)
			TAG_OUT(TAG_BRUSHATTRFILLFLAGS)
			TAG_OUT(TAG_BRUSHTRANSPINFO)
			TAG_OUT(TAG_BRUSHATTRTRANSPINFO)

// the nudge size is now a part of the document
			TAG_OUT(TAG_DOCUMENTNUDGE)

// bitmap properties - stores the settings in the b.p. dialogue for each bitmap
			TAG_OUT(TAG_BITMAP_PROPERTIES)
			TAG_OUT(TAG_DOCUMENTBITMAPSMOOTHING)

// Extended bitmap processing data generated by XPE
			TAG_OUT(TAG_XPE_BITMAP_PROPERTIES)

// Bitmap file format placeholder when bitmap data is generated on the fly from master and XPE info
			TAG_OUT(TAG_DEFINEBITMAP_XPE)

// Current Attributes container and geometry bounds
			TAG_OUT(TAG_CURRENTATTRIBUTES)
			TAG_OUT(TAG_CURRENTATTRIBUTEBOUNDS)

// New 3-point linear fill records
			TAG_OUT(TAG_LINEARFILL3POINT)
			TAG_OUT(TAG_LINEARFILLMULTISTAGE3POINT)
			TAG_OUT(TAG_LINEARTRANSPARENTFILL3POINT)

// New Duplication distance records
			TAG_OUT(TAG_DUPLICATIONOFFSET)

// Live Effect node tags
			TAG_OUT(TAG_LIVE_EFFECT)
			TAG_OUT(TAG_LOCKED_EFFECT)
			TAG_OUT(TAG_FEATHER_EFFECT)

// Compound group rendering stream warning (e.g. transparent groups, live effects, etc.)
			TAG_OUT(TAG_COMPOUNDRENDER)

			TAG_OUT(TAG_OBJECTBOUNDS)

			TAG_OUT(TAG_DIMENSION)

			TAG_OUT(TAG_SPREAD_PHASE2)
			TAG_OUT(TAG_CURRENTATTRIBUTES_PHASE2)
			TAG_OUT(TAG_SPREAD_FLASHPROPS)
			TAG_OUT(TAG_PRINTERSETTINGS_PHASE2)

			TAG_OUT(TAG_DOCUMENTINFORMATION)

			TAG_OUT(TAG_CLIPVIEW_PATH)
			TAG_OUT(TAG_DEFINEBITMAP_PNG_REAL)

			TAG_OUT(TAG_TEXT_STRING_POS)

			TAG_OUT(TAG_SPREAD_FLASHPROPS2)

			TAG_OUT(TAG_TEXT_LINESPACE_LEADING)

//-----------------
// Extrusion attributes continue at 4300
			TAG_OUT(TAG_EXTRUSIONDEPTH)
			TAG_OUT(TAG_EXTRUSIONANGLE1)
			TAG_OUT(TAG_EXTRUSIONANGLE2)
			TAG_OUT(TAG_EXTRUSIONANGLE3)


			TAG_OUT(TAG_SPREAD_PHOTO)

			TAG_OUT(TAG_EXTRUSIONGROUP)
			TAG_OUT(TAG_EXTRUSIONCONTROLLER)

//-----------------
// New ICC profile and bitmap tags in reserved range 4150-4159
			TAG_OUT(TAG_DEFINEBITMAP_PNG_CMYK)
			TAG_OUT(TAG_DEFINEICC_CMYK)
			TAG_OUT(TAG_DEFINEBITMAP_EXTERNAL)


//-----------------
// New bitmap attributes in reserved range 4160-4189
			TAG_OUT(TAG_BITMAPSHARPEN)
			TAG_OUT(TAG_BITMAPGAMMA)
			TAG_OUT(TAG_BITMAPGAMMA_RED)
			TAG_OUT(TAG_BITMAPGAMMA_GREEN)
			TAG_OUT(TAG_BITMAPGAMMA_BLUE)
			TAG_OUT(TAG_BITMAPCONTRAST)
			TAG_OUT(TAG_BITMAPSATURATION)
			TAG_OUT(TAG_BITMAPINPUTRANGE)
			TAG_OUT(TAG_BITMAPINPUTRANGE_RED)
			TAG_OUT(TAG_BITMAPINPUTRANGE_GREEN)
			TAG_OUT(TAG_BITMAPINPUTRANGE_BLUE)
			TAG_OUT(TAG_BITMAPOUTPUTRANGE)
			TAG_OUT(TAG_BITMAPOUTPUTRANGE_RED)
			TAG_OUT(TAG_BITMAPOUTPUTRANGE_GREEN)
			TAG_OUT(TAG_BITMAPOUTPUTRANGE_BLUE)
			TAG_OUT(TAG_BITMAPSHARPEN_NEW)
			TAG_OUT(TAG_BITMAPCURVE_RED)
			TAG_OUT(TAG_BITMAPCURVE_GREEN)
			TAG_OUT(TAG_BITMAPCURVE_BLUE)
			TAG_OUT(TAG_BITMAPTEMPERATURE)
			TAG_OUT(TAG_BITMAPCHECKSUM)
			TAG_OUT(TAG_BITMAPCURVE_RGB)
			TAG_OUT(TAG_BITMAPHUEADJUST)
			TAG_OUT(TAG_BITMAPHUE)
			TAG_OUT(TAG_BITMAPHUESTRENGTH)
			TAG_OUT(TAG_BITMAPSHADOWS)
			TAG_OUT(TAG_BITMAPHIGHLIGHTS)
			TAG_OUT(TAG_BITMAPCROSSPROCESS)

//-----------------
// New node tags (in reserved range 4190-4199)

			TAG_OUT(TAG_NODEDETAILS)


/////////////////////////////////////////////////////////////////////////////
// XaraLX 0.6
/////////////////////////////////////////////////////////////////////////////

// New text tags for tabbing and paragraph-based indent support

// Text objects
			TAG_OUT(TAG_TEXT_TAB)

// Text attributes
			TAG_OUT(TAG_TEXT_LEFT_INDENT)
			TAG_OUT(TAG_TEXT_FIRST_INDENT)
			TAG_OUT(TAG_TEXT_RIGHT_INDENT)
			TAG_OUT(TAG_TEXT_RULER)

// New text tags for linked text areas and text flow

			TAG_OUT(TAG_TEXT_STORY_HEIGHT_INFO)
			TAG_OUT(TAG_TEXT_STORY_LINK_INFO)
			TAG_OUT(TAG_TEXT_STORY_TRANSLATION_INFO)

// New text tags for space before/after and for soft/hard hyphens

			TAG_OUT(TAG_TEXT_SPACE_BEFORE)
			TAG_OUT(TAG_TEXT_SPACE_AFTER)
			TAG_OUT(TAG_TEXT_SPECIAL_HYPHEN)

// Tags recognized by Xtreme Pro 3.2 end here

			TAG_OUT(TAG_TEXT_SOFT_RETURN)

// Tags for extra font information for external filters

			TAG_OUT(TAG_TEXT_EXTRA_FONT_INFO)
			TAG_OUT(TAG_TEXT_EXTRA_TT_FONT_DEF)
			TAG_OUT(TAG_TEXT_EXTRA_ATM_FONT_DEF)

// Text repelling

			TAG_OUT(TAG_REPEL)
			TAG_OUT(TAG_REPEL_DISTANCE)

// document option to render text with a URL applied as underlined
			TAG_OUT(TAG_UNDERLINE_URLS)
			TAG_OUT(TAG_TEXT_RUN_BREAK)

			TAG_OUT(TAG_EXTRUSIONCONTROLLER_TRANS)
			TAG_OUT(TAG_EXTRUSIONCONTROLLER_COMPLEX)
			TAG_OUT(TAG_EXTRUSIONBEVELDEPTH)
			TAG_OUT(TAG_EXTRUSIONGLOSS)
			TAG_OUT(TAG_EXTRUSIONBEVELTYPE)
			TAG_OUT(TAG_EXTRUSIONCORNERTYPE)
			TAG_OUT(TAG_EXTRUSIONLIGHT1ANGLE1)
			TAG_OUT(TAG_EXTRUSIONLIGHT1ANGLE2)
			TAG_OUT(TAG_EXTRUSIONLIGHT1COLOUR)
			TAG_OUT(TAG_EXTRUSIONLIGHT2ANGLE1)
			TAG_OUT(TAG_EXTRUSIONLIGHT2ANGLE2)
			TAG_OUT(TAG_EXTRUSIONLIGHT2COLOUR)
			TAG_OUT(TAG_EXTRUSIONLIGHT3ANGLE1)
			TAG_OUT(TAG_EXTRUSIONLIGHT3ANGLE2)
			TAG_OUT(TAG_EXTRUSIONLIGHT3COLOUR)
			TAG_OUT(TAG_EXTRUSIONPERSPECTIVE)

			TAG_OUT(TAG_EXTRUSIONGROUP_PHASE2)

			TAG_OUT(TAG_EFFECTATTR_FLAG)
			TAG_OUT(TAG_PHOTOATTR_FLAG)

//-----------------
// New symbol tags in reserved range 4340-4349
			TAG_OUT(TAG_SYMBOLDEFINITION)
			TAG_OUT(TAG_SYMBOLDEFINITION_LINKED)
			TAG_OUT(TAG_SYMBOLREF_TRANS)
			TAG_OUT(TAG_SYMBOLREF_COMPLEX)

// Tags recognized by Xtreme Pro 4.0 end here

			TAG_OUT(TAG_FONT_GLYPH)
			TAG_OUT(TAG_FONT_KERNINGPAIRS)
			TAG_OUT(TAG_TEXT_LANGUAGE)
			TAG_OUT(TAG_TEXT_SPELL_OPTIONS)
			TAG_OUT(TAG_TEXT_STRING_POS2)

			TAG_OUT(TAG_URL_COLOUR_OVERRIDE)
			TAG_OUT(TAG_DOCUMENT_URL_COLOUR)
			TAG_OUT(TAG_DOCUMENT_MOUSE_OVER_URL_COLOUR)
			TAG_OUT(TAG_DOCUMENT_VISITED_URL_COLOUR)
			TAG_OUT(TAG_WEBSITE_PROPERTIES)
			TAG_OUT(TAG_SUPPORTFOLDER_PATH)

			TAG_OUT(TAG_LINEARTRANSPARENTFILLMULTISTAGE)
			TAG_OUT(TAG_LINEARTRANSPARENTFILLMULTISTAGE3POINT)
			TAG_OUT(TAG_CIRCULARTRANSPARENTFILLMULTISTAGE)
			TAG_OUT(TAG_ELLIPTICALTRANSPARENTFILLMULTISTAGE)
			TAG_OUT(TAG_CONICALTRANSPARENTFILLMULTISTAGE)
			TAG_OUT(TAG_SQUARETRANSPARENTFILLMULTISTAGE)

			TAG_OUT(TAG_WEBSITE_PAGE_PROPERTIES)

			TAG_OUT(TAG_SPREAD_FLASHPROPS3)

			TAG_OUT(TAG_AFFILIATEID)

//-----------------
// Very New Web Address tags
			TAG_OUT(TAG_WEBADDRESS_PAGE)
			TAG_OUT(TAG_WEBADDRESS_PAGE_BOUNDINGBOX)
			TAG_OUT(TAG_WEBADDRESS_OBJECT)
			TAG_OUT(TAG_WEBADDRESS_OBJECT_BOUNDINGBOX)
			TAG_OUT(TAG_WEBADDRESS_LAYER)
			TAG_OUT(TAG_WEBADDRESS_LAYER_BOUNDINGBOX)
			TAG_OUT(TAG_WEBADDRESS_SCALEDPOPUP)
			TAG_OUT(TAG_WEBADDRESS_SCALEDPOPUP_BOUNDINGBOX)

			TAG_OUT(TAG_SPREAD_FLASHPROPS4)

// Tags recognized by Web Designer 5.0 end here

// Composite bitmap group rendering stream warning
			TAG_OUT(TAG_PHOTOGROUPRENDER)
			TAG_OUT(TAG_BITMAPCLONEFILL)

			TAG_OUT(TAG_BITMAPSHARPEN_PHASE3)

			TAG_OUT(TAG_BITMAPINTELLIGENTSCALE)

			TAG_OUT(TAG_REGIONLAYERDETAILS)
			TAG_OUT(TAG_REDEYEPATCH)

			TAG_OUT(TAG_ISCALEDEFINITION)
			TAG_OUT(TAG_ISCALEDEFINITION_MASK)
			TAG_OUT(TAG_ISCALEDEF)
			TAG_OUT(TAG_ISCALEDEF_MASK)

			TAG_OUT(TAG_TEXT_ANCHOR_POSITION)
			TAG_OUT(TAG_ANCHORED_GROUP_INFO)

// Tags recognized by Xtreme Pro 5.1 end here

			TAG_OUT(TAG_FTP_DETAILS)
			TAG_OUT(TAG_SPREAD_BLANKHTMLPAGE)

			TAG_OUT(TAG_MENU_DETAILS)
			TAG_OUT(TAG_DOCUMENT_MENU_TEXT_COLOUR)
			TAG_OUT(TAG_DOCUMENT_MENU_BACK_COLOUR)
			TAG_OUT(TAG_DOCUMENT_MENU_TEXT_OVER_COLOUR)
			TAG_OUT(TAG_DOCUMENT_MENU_BACK_OVER_COLOUR)
			TAG_OUT(TAG_DOCUMENT_MENU_BORDER_COLOUR)

			TAG_OUT(TAG_TEXT_BULLETED_ITEM_START)
			TAG_OUT(TAG_TEXT_AUTO_LIST_CHAR)
			TAG_OUT(TAG_BITMAPISCALE)
			TAG_OUT(TAG_TEXT_ATTRIBUTE_SHADOW)
			TAG_OUT(TAG_TEXT_FONT_TYPEFACE_SHADOWED)

			TAG_OUT(TAG_LAYER_PAPER_BACKGROUND)

			TAG_OUT(TAG_TEXT_NUMBERED_ITEM_START)

			TAG_OUT(TAG_EXTENDEDLAYERDETAILS)

			TAG_OUT(TAG_BITMAPBOUNDSFILL)

			TAG_OUT(TAG_SOFTMASKGROUP)

			TAG_OUT(TAG_NODEMASKDETAILS)

			TAG_OUT(TAG_BITMAPDEFAULTTRANS)

			TAG_OUT(TAG_BITMAPISCALE2)

			TAG_OUT(TAG_CLONEFLAG)

			TAG_OUT(TAG_NAVBAR_START)
			TAG_OUT(TAG_NAVBAR_END)
			TAG_OUT(TAG_NAVBAR_MENUSTYLE_INFO)
			TAG_OUT(TAG_NAVBAR_BUTTON_STRETCH_INFO)

			TAG_OUT(TAG_DOCUMENT_HIGHSLIDE_INFO)

			TAG_OUT(TAG_WEBADDRESS_FILE)
			TAG_OUT(TAG_WEBADDRESS_FILE_BOUNDINGBOX)

			TAG_OUT(TAG_NAVBAR_BUTTON_MENU)
			TAG_OUT(TAG_NAVBAR_BUTTON_MENU_ITEM)
			TAG_OUT(TAG_NAVBAR_BUTTON_MENU_SEPARATOR)

			TAG_OUT(TAG_SPREAD_ANIMPROPS2)
			TAG_OUT(TAG_COMPATIBILITYATTR_FLAG)
			TAG_OUT(TAG_NAVBAR_LOCAL_MENUSTYLE)

// Tags recognized by Web Designer 6.0 end here

// Tags recognized by PGD6.1/DesignerPro 6.1 end here

			TAG_OUT(TAG_TEXT_STORY_PHOTOSHOP_POSITIONING)

			TAG_OUT(TAG_RESOLUTION_HINT)

			TAG_OUT(TAG_SYMBOLGROUP)

			TAG_OUT(TAG_BITMAPFILL_AUTOCLONE)

			TAG_OUT(TAG_PRESENTATION_DOC_PROPERTIES)
			TAG_OUT(TAG_PRESENTATION_PAGE_PROPERTIES)
			TAG_OUT(TAG_USERVALUE_PRESENTATION)

			TAG_OUT(TAG_WEBSITE_PROPERTIES2)
			TAG_OUT(TAG_WEBSITE_PAGE_PROPERTIES2)

			TAG_OUT(TAG_PRESENTATION_LAYER_PROPERTIES)
			TAG_OUT(TAG_LAYER_REPEL_WITHIN)
			TAG_OUT(TAG_REPEL_DISTANCE_V)
			TAG_OUT(TAG_ANCHORED_GROUP_INFO2)

			TAG_OUT(TAG_FILEDEFINITION)
			TAG_OUT(TAG_FILEREFERENCE)

			TAG_OUT(TAG_FTP_WEBSITEURL)
			TAG_OUT(TAG_PRESENTATION_LAYER_FLAG)

			TAG_OUT(TAG_WIDGETDEFINITION)
			TAG_OUT(TAG_WIDGETREFERENCE)

			TAG_OUT(TAG_EXPORTFEATURES)

			TAG_OUT(TAG_AUTOGENERATEDNAME)

			TAG_OUT(TAG_WIDGETREVISION)

// Tags recognized by Web Designer 7.0 end here

			TAG_OUT(TAG_SYMBOLGROUP_LIVECOPY)

// Tags recognized by PGD7.1/DesignerPro 7.1 end here

			TAG_OUT(TAG_EMBEDDED_ANCHORED_GROUP_INFO)
			TAG_OUT(TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO)
			TAG_OUT(TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO)

			TAG_OUT(TAG_SHUFFLE)

			TAG_OUT(TAG_NODECLONEREFS)
			TAG_OUT(TAG_CLONEREF)
			TAG_OUT(TAG_CLONESOURCE)
			TAG_OUT(TAG_BITMAP_HIDDEN)

			TAG_OUT(TAG_STYLE_DEF)
			TAG_OUT(TAG_STYLE)
			TAG_OUT(TAG_TEXT_PARAGRAPH_STYLE)
			TAG_OUT(TAG_TEXT_PRESENT_BY_PARAGRAPH)

			TAG_OUT(TAG_COLSELGROUP)
			TAG_OUT(TAG_COLSELMARKUP_WEB)

			TAG_OUT(TAG_PRIVATE_FILTER_INFO)
			TAG_OUT(TAG_ATTR_FILTER_INFO)

			TAG_OUT(TAG_COLSELMARKUP)
			TAG_OUT(TAG_LOCK_POPUP_LAYER)

			TAG_OUT(TAG_WIDGETDESIGNSIZE)
			TAG_OUT(TAG_WIDGET_EXTRAINFO)

			TAG_OUT(TAG_CLONESOURCE2)

			TAG_OUT(TAG_TEXT_DO_NOT_SYNCH)

// Tags recognized by version 8.1.3 end here

			TAG_OUT(TAG_TEXT_SIMPLE_MACRO_MARKER)
			TAG_OUT(TAG_TEXT_MACRO_CHAR)

// Tags recognized by version 8.1 end here

			TAG_OUT(TAG_ONLINEINFO_NEXT_ID)
			TAG_OUT(TAG_ONLINEINFO_ID)
			TAG_OUT(TAG_ONLINEINFO_PAGEID)
			TAG_OUT(TAG_GOOGLE_WEB_FONTS_LIST_PATH)

			TAG_OUT(TAG_WEBSITE_PROPERTIES_ANALYTICS)

			TAG_OUT(TAG_WSM_NAVBAR)
			TAG_OUT(TAG_WSM_NAVBAR_BUTTON)
			TAG_OUT(TAG_TEXT_LINE_INFO2)

			TAG_OUT(TAG_ALTERNATEBITMAP_JPEG)
			TAG_OUT(TAG_ALTERNATEBITMAP_PNG)
			TAG_OUT(TAG_ALTERNATEBITMAP_PNG_REAL)

			TAG_OUT(TAG_FTP_DETAILS2)

			TAG_OUT(TAG_FONT_GLYPH_EXTRA_WEIGHT)
			TAG_OUT(TAG_FONT_KERNINGPAIRS_EXTRA_WEIGHT)
			TAG_OUT(TAG_TEXT_STORY_WORD_LINE_SPACING)

			TAG_OUT(TAG_LAST_HTML_PUBLISH_NAME)

// Tags recognized by version 9.0 end here
// Tags recognized by version 9.1 end here
// Tags recognized by version 9.2 end here

			TAG_OUT(TAG_TEXT_BREAK_TYPE)
			TAG_OUT(TAG_TEXT_STORY_AUTOFLOW)

			TAG_OUT(TAG_DOCUMENTNUDGE_PHASE2)

			TAG_OUT(TAG_SMARTSHAPEGROUP)

			TAG_OUT(TAG_LIVESHARE)
			TAG_OUT(TAG_PAGEVARIANTDEFINITION)
			TAG_OUT(TAG_PAGEVARIANTREFERENCE)

			TAG_OUT(TAG_IMPLICIT_TAB_DISTANCE)
			TAG_OUT(TAG_WEBSITE_PROPERTIES_EXPORT)
			TAG_OUT(TAG_TEXT_FONT_SIZE_RELATIVE_KERN)
			TAG_OUT(TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING)
			TAG_OUT(TAG_WEBSITE_TRANSITION)
			TAG_OUT(TAG_USERVALUE_V10)
			TAG_OUT(TAG_EMBEDDED_GLYPHS_SIZE_HINT)

			TAG_OUT(TAG_DOCUMENTVIEW_VARIANT)
			TAG_OUT(TAG_VARIANT_STYLE_DELTA)

			TAG_OUT(TAG_SPREAD_THUMB)
			TAG_OUT(TAG_SPREAD_THUMB_RETINA)

// Tags recognized by version 10.0 end here

			TAG_OUT(TAG_COLSELENHANCE_ATTR)
			TAG_OUT(TAG_ANCHORING_MODE)
			TAG_OUT(TAG_PAGEVARIANTSLIST)
			TAG_OUT(TAG_PAGEVARIANTDESCRIPTION)
			TAG_OUT(TAG_EFFECTATTR_FLAG_PHASE2)
			TAG_OUT(TAG_COLSELENHANCE_TRANSP)

// Tags recognized by version 10.1 end here

			TAG_OUT(TAG_CMYKJPEG_NORMAL)
			TAG_OUT(TAG_CMYKJPEG_INVERT)

			TAG_OUT(TAG_TXTBACKGROUNDCOLOUR)
			TAG_OUT(TAG_TXTBACKGROUNDCOLOUR_NONE)
			TAG_OUT(TAG_TXTBACKGROUNDCOLOUR_BLACK)
			TAG_OUT(TAG_TXTBACKGROUNDCOLOUR_WHITE)

			TAG_OUT(TAG_WEBSITE_PROPERTIES_EXPORT2)

			TAG_OUT(TAG_MOULDBRUSHATTR)

			TAG_OUT(TAG_TEXT_HYPHENATION_OFF)
			TAG_OUT(TAG_TEXT_HYPHENATION_RELATIVE)
			TAG_OUT(TAG_TEXT_HYPHENATION_ABSOLUTE)
			TAG_OUT(TAG_TEXT_HYPHENATION_DOCUMENT_DEFAULT)
			TAG_OUT(TAG_DOCUMENT_HYPHENATION_DEFAULTS_RELATIVE)
			TAG_OUT(TAG_DOCUMENT_HYPHENATION_DEFAULTS_ABSOLUTE)

			TAG_OUT(TAG_BRUSHDEFINITION_MOULD)
			TAG_OUT(TAG_MOULDBRUSHDATA)

			TAG_OUT(TAG_FTP_DETAILS3)

			TAG_OUT(TAG_DOCUMENT_HYPHENATION_CUSTOM_LIST)

			TAG_OUT(TAG_WEBSITE_PROPERTIES_EXPORT3)

			TAG_OUT(TAG_FONT_EMBEDDING_POLICY)

			TAG_OUT(TAG_TEXT_EOL_VIRTUAL)

			TAG_OUT(TAG_ANCHORED_GROUP_SCALING_INTENDED)
			TAG_OUT(TAG_ANCHORED_GROUP_SCALING_RELATIVE)
			TAG_OUT(TAG_ANCHORED_GROUP_SCALING_DISABLED)

			TAG_OUT(TAG_ANCHORED_GROUP_WRAP_LEFT)
			TAG_OUT(TAG_ANCHORED_GROUP_WRAP_RIGHT)

			TAG_OUT(TAG_ANCHORED_GROUP_MARGINS)

			TAG_OUT(TAG_TEXT_MACRO_PAGE_NUMBER_OFFSET)

// Tags recognized by version 11.0 end here

			TAG_OUT(TAG_TEXT_SYMBOL_CHAR)

			TAG_OUT(TAG_REPEL_RECT_REPEL)

			TAG_OUT(TAG_TEXT_SYMBOL_CHAR_POS)
			TAG_OUT(TAG_TEXT_SYMBOL_CHAR_POS2)

			TAG_OUT(TAG_TEXT_ALM_INDENT_LEVEL)

			TAG_OUT(TAG_ONLINE_PASTE_INFO)

			TAG_OUT(TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_ON)
			TAG_OUT(TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_OFF)
			TAG_OUT(TAG_TEXT_KEEP_LINES_TOGETHER_ON)
			TAG_OUT(TAG_TEXT_KEEP_LINES_TOGETHER_OFF)

			TAG_OUT(TAG_TEXT_ALM_LINKS)

			TAG_OUT(TAG_SOURCE_CLIPART_PATH)

			TAG_OUT(TAG_TEXT_TOC_MARKER_OUTLINE_LEVEL)

			TAG_OUT(TAG_TEXT_STORY_VERTICAL_ALIGN_CENTRE)
			TAG_OUT(TAG_TEXT_STORY_VERTICAL_ALIGN_BOTTOM)

			TAG_OUT(TAG_ANCHORED_GROUP_INFO3)

			TAG_OUT(TAG_TEXT_TOC_MARKER_FLAGS)

			TAG_OUT(TAG_FONT_LIGATURES)

			TAG_OUT(TAG_INDEXEDCOLOURIDS)
			TAG_OUT(TAG_TEXT_OPEN_TYPE_LIGATURES)
			TAG_OUT(TAG_FONT_OTLIGATURES)

			TAG_OUT(TAG_ONLINEINFO_DATALIST)
			TAG_OUT(TAG_ONLINEINFO_BITMAPNAME)
			TAG_OUT(TAG_ONLINEINFO_NEXTBITMAPID)

			TAG_OUT(TAG_TEXT_OPEN_TYPE_STYLISTICSETS)
			TAG_OUT(TAG_FONT_OTSTYLISTICSETS)
			TAG_OUT(TAG_FONT_OTLIGATURES_EXT)

			TAG_OUT(TAG_TEXT_LIGATURE_START)
			TAG_OUT(TAG_TEXT_LIGATURE_SLAVE)

			TAG_OUT(TAG_FONT_OTSTYLISTICSETNAMES)

			TAG_OUT(TAG_TEXT_STYLISTIC_SET_SUB)

			TAG_OUT(TAG_DOCUMENTHISTORY_ENTRY_CREATE)
			TAG_OUT(TAG_DOCUMENTHISTORY_ENTRY)

			TAG_OUT(TAG_ATTR_COMMENT_MARKER)
			TAG_OUT(TAG_ATTR_COMMENT_DATA)
			TAG_OUT(TAG_TEXT_COMMENT_ANCHOR_POS_INFO)
			TAG_OUT(TAG_FONT_DEF_VARIANTS_INFO)

			TAG_OUT(TAG_DOCUMENT_DEFAULT_LANGUAGE)

			TAG_OUT(TAG_ANCHORED_GROUP_PAGE_OFFSET)

			TAG_OUT(TAG_TXT_LIST_PROPERTIES)
			TAG_OUT(TAG_TXT_LIST_NUMBER_PROPERTIES)
			TAG_OUT(TAG_TXT_LIST_BULLET_PROPERTIES)
			TAG_OUT(TAG_TXT_LIST_FONT_SIZE)
			TAG_OUT(TAG_TXT_LIST_VERTICAL_OFFSET)

			TAG_OUT(TAG_TXT_LIST_FONT_COLOUR)
			TAG_OUT(TAG_TXT_LIST_FONT_COLOUR_NONE)
			TAG_OUT(TAG_TXT_LIST_FONT_COLOUR_BLACK)
			TAG_OUT(TAG_TXT_LIST_FONT_COLOUR_WHITE)

			TAG_OUT(TAG_ANCHORED_GROUP_AFFECT_LINE_FLOW)

			TAG_OUT(TAG_NEXT_COMMENT_ID)

			TAG_OUT(TAG_LIST_STYLE_SUPPLEMENT)

// Tags recognized by version 15.0 end here

			TAG_OUT(TAG_SPREAD_DESIGN_INFO)
			TAG_OUT(TAG_SPREADINFORMATION_EXTRA)

// two new types for text stories inside object (atomic records)
			TAG_OUT(TAG_TEXT_STORY_INSIDE_SIMPLE)
			TAG_OUT(TAG_TEXT_STORY_INSIDE_COMPLEX)

			TAG_OUT(TAG_COMPANY_WORKSPACE)

// Tags recognized by version 15.1 end here

			TAG_OUT(TAG_FONT_OTSUBSTITUTIONRULES)
			TAG_OUT(TAG_FONT_OTSUBSTITUTIONTABLES)
			TAG_OUT(TAG_FONT_OTRULEMAXSIZES)

			TAG_OUT(TAG_TEXT_CHANGE_CAPS)

			TAG_OUT(TAG_TEXT_WIDOWS_AND_ORPHANS_ON)
			TAG_OUT(TAG_TEXT_WIDOWS_AND_ORPHANS_OFF)

			TAG_OUT(TAG_TEXT_STORY_AUTO_SIZE_TO_FIT)

			TAG_OUT(TAG_TEXT_OT_CONTEXT_ALT_ON)
			TAG_OUT(TAG_TEXT_OT_CONTEXT_ALT_OFF)

// Tags recognized by version 16.0 end here

			TAG_OUT(TAG_TEXT_OT_STYLE_ALT_ON)
			TAG_OUT(TAG_TEXT_OT_STYLE_ALT_OFF)

			TAG_OUT(TAG_TEXT_OT_TITLE_ALT_ON)
			TAG_OUT(TAG_TEXT_OT_TITLE_ALT_OFF)

			TAG_OUT(TAG_TEXT_OT_SWASHES_ON)
			TAG_OUT(TAG_TEXT_OT_SWASHES_OFF)

			TAG_OUT(TAG_TEXT_TEMPLATE_CHAR_FLAG)

			TAG_OUT(TAG_TEXT_AUTO_FIELD_MARKER)

// Tags recognized by version 16.1 end here

			TAG_OUT(TAG_TEXT_STORY_SHRINK_ON_OVERFLOW)

			TAG_OUT(TAG_BRANDDOCUMENT)
			TAG_OUT(TAG_SMARTFIELD_DICT)

			TAG_OUT(TAG_TXT_PARA_BACKGROUNDCOLOUR)
			TAG_OUT(TAG_TXT_PARA_BACKGROUNDCOLOUR_NONE)
			TAG_OUT(TAG_TXT_PARA_BACKGROUNDCOLOUR_BLACK)
			TAG_OUT(TAG_TXT_PARA_BACKGROUNDCOLOUR_WHITE)

			TAG_OUT(TAG_DOCUMENT_METADATA)

// Tags recognized by version 16.2 end here

// new webaddress tag
			TAG_OUT(TAG_WEBADDRESS_SMARTFIELD)
			TAG_OUT(TAG_WEBADDRESS_SMARTFIELD_BOUNDINGBOX)

// paragraph background border
// Old border data - currently unused
			TAG_OUT(TAG_TXT_PARA_TOP_BORDER)
			TAG_OUT(TAG_TXT_PARA_BOTTOM_BORDER)
			TAG_OUT(TAG_TXT_PARA_LEFT_BORDER)
			TAG_OUT(TAG_TXT_PARA_RIGHT_BORDER)
			TAG_OUT(TAG_TXT_PARA_INTERNAL_BORDER)

// persistent angles
			TAG_OUT(TAG_NODEANGLE)
			TAG_OUT(TAG_NODEANGLE4638)

// Tags recognized by version 16.3 end here

			TAG_OUT(TAG_WEBADDRESS_NOFOLLOW)


/////////////////////////////////////////////////////////////////////////////
// TAG RANGES
// Add new tags just before this comment
// The current range for tags is 4000-4999. DO NOT define tags with values
// outside this range. Ask Gerry if you need to extend the range or
// do anything else unusual.
/////////////////////////////////////////////////////////////////////////////


		default:
		{
			char psz[20];
			_itoa_s(Tag, psz, 20, 10);
			return psz;
		}
	}

	return "";
}
