/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "vcFile.h"
#include "dibfunctions.h"
#include "decodepng.h"

#ifdef FILTER_DEBUG
#include "common/HiResolutionTimer.h"

double g_dLutTrans = 0;
double g_dCMYK = 0;
double g_dmemcpy = 0;
double g_dpng_write_row = 0;
#endif 

extern "C" 
{
#include "..\png\png.h"
#include "..\png\pngstruct.h"
#include "..\png\pngconf.h"
#include "..\png\pnglibconf.h"
#include "..\png\pnginfo.h"
}

/******************************************************************************
   Function:    CopyPNGScanlineToRGBRow
   Author:      Paul Gruzdev
   Purpose:     Translate the jpeg/png row to dib row
   Returns:     void
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                PBYTE pbyJPEG - 
                int iWidth - 
   Notes:       
******************************************************************************/
void CopyPNGScanlineToRGBRow(PBYTE pbyDib, PBYTE pbyJPEG, int iWidth)
{
	for(int i = 0; i < iWidth; i++)
	{
		pbyDib[2] = *pbyJPEG++;
		pbyDib[1] = *pbyJPEG++;
		pbyDib[0] = *pbyJPEG++;
		pbyDib+=3;
	}
}

/******************************************************************************
   Function:    CopyPNGScanlineToRGBARow
   Author:      Paul Gruzdev
   Purpose:     Translate PNG RGBA to dib BGRA
   Returns:     void
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                PBYTE pbyJPEG - 
                int iWidth - 
   Notes:       
******************************************************************************/
void CopyPNGScanlineToRGBARow(PBYTE pbyDib, PBYTE pbyJPEG, int iWidth)
{
	for(int i = 0; i < iWidth; i++)
	{
		pbyDib[2] = *pbyJPEG++;
		pbyDib[1] = *pbyJPEG++;
		pbyDib[0] = *pbyJPEG++;
		pbyDib[3] = *pbyJPEG++;
		pbyDib+=4;
	}
}

/******************************************************************************
   Function:    CopyPNGScanlineToRGBASplit
   Author:      Paul Gruzdev
   Purpose:     Split version of RGBA rearrange function
   Returns:     void
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                PBYTE pbyTrans - 
                PBYTE pbyJPEG - 
                int iWidth - 
   Notes:       
******************************************************************************/
void CopyPNGScanlineToRGBASplitInvertAlpha(PBYTE pbyDib, PBYTE pbyTrans, PBYTE pbyJPEG, int iWidth)
{
	for(int i = 0; i < iWidth; i++)
	{
		pbyDib[2] = *pbyJPEG++;
		pbyDib[1] = *pbyJPEG++;
		pbyDib[0] = *pbyJPEG++;
		*pbyTrans++ = (BYTE)(255 - *pbyJPEG++);
		pbyDib+=3;
	}
}

/******************************************************************************
   Function:    Copy4bitPNGScanlineToRGBASplitInvertAlpha
   Author:      Alex Kotlikov
   Purpose:     Transfer 4-bit png row to RGBA format, invert the alpha values.
   Returns:     void
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                PBYTE pbyJPEG - 
                RGBQUAD* prgbPalette - 
                int iWidth - 
   Notes:       
******************************************************************************/
void Copy4bitPNGScanlineToRGBASplitInvertAlpha(PBYTE pbyDib, PBYTE pbyJPEG, RGBQUAD* prgbPalette, int iWidth)
{
	for(int i = 0; i < iWidth; i++)
	{
		BYTE byPixel = pbyJPEG[ i / 2 ];
		if (i % 2)
			byPixel = byPixel >> 4;
		else
			byPixel = byPixel & 0x0F;

		pbyDib[2] = prgbPalette[ byPixel ].rgbBlue;
		pbyDib[1] = prgbPalette[ byPixel ].rgbGreen;
		pbyDib[0] = prgbPalette[ byPixel ].rgbRed;

		pbyDib += 3;
	}
}

/******************************************************************************
   Function:    SplitGreyA
   Author:      Paul Gruzdev
   Purpose:     Split one Grey-A row into two separate image and transparency buffers
   Returns:     void
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                PBYTE pbyTrans - 
                PBYTE pbyJPEG - 
                int iWidth - 
   Notes:       
******************************************************************************/
void SplitGreyAInvertAlpha(PBYTE pbyDib, PBYTE pbyTrans, PBYTE pbyJPEG, int iWidth)
{
	for(int i = 0; i < iWidth; i++)
	{
		*pbyDib++ = *pbyJPEG++;
		*pbyTrans++ = (BYTE)(255 - *pbyJPEG++);
	}
}

typedef struct
{
	BYTE red;
	BYTE green;
	BYTE blue;
} rgb_color_struct;

//void ima_png_error(const png_struct* png_ptr, const char* message)
PNG_FUNCTION(void, ima_png_error,(png_const_structrp png_ptr, png_const_charp error_message), PNG_NORETURN)
{
	throw error_message;
	//TRACE("PNG: error %s\n", message);
}

void PNGAPI PngReadDataFromMemFile(png_structp png_ptr, png_bytep data, png_size_t length)
{
	vcFile* pFile = (vcFile*)png_ptr->io_ptr;
	pFile->Read((PVOID)data, (DWORD)length);
}


void PNGAPI PngWriteDataToMemFile(png_structp png_ptr, png_bytep data, png_size_t length)
{
	vcFile* pFile = (vcFile*)png_ptr->io_ptr;
	pFile->Write((PVOID)data, (DWORD)length);
}

/******************************************************************************
   Function:    DecodePNG
   Author:      Paul Gruzdev
   Purpose:     Decode PNG image given memfile object
   Returns:     bool - true if success, false if not.
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool DecodePNG(vcFile* pFile, PBYTE& pbyDib, DWORD& dwDibSize, PBYTE& pbyTrans, DWORD& dwTransSize)
{
	int iChannels = 0;
	return DecodePNG(pFile, pbyDib, dwDibSize, pbyTrans, dwTransSize, iChannels);
}
bool DecodePNG(vcFile* pFile, PBYTE& pbyDib, DWORD& dwDibSize, PBYTE& pbyTrans, DWORD& dwTransSize, int& iChannels)
{
	int number_passes;
	
	png_struct *png_ptr;
	png_info *info_ptr;

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
		(void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		return false; // can't create read struct
	}
	
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		return false; //
	}

	if (setjmp(png_ptr->jmp_buf_local))
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return false;
	}

	//png_init_io(png_ptr, pFile);
	//png_ptr->io_ptr = (png_voidp)pFile;
	//png_set_read_fn()
	png_set_read_fn(png_ptr, (png_voidp)pFile, PngReadDataFromMemFile);
   //png_rw_ptr read_data_fn)

	png_read_info(png_ptr, info_ptr);

	if(info_ptr != NULL)
		iChannels = info_ptr->channels;

	int pixel_depth = info_ptr->pixel_depth;//(info_ptr->pixel_depth<24) ? info_ptr->pixel_depth: 24;
	int row_stride = info_ptr->width * ((pixel_depth+7)>>3);

	switch (pixel_depth)
	{
	case 1:
		row_stride = (info_ptr->width/8);
		if(info_ptr->width % 8)
			row_stride++;
		break;
	case 4:
		row_stride = (info_ptr->width/2);
		if(info_ptr->width % 2)
			row_stride++;
		break;
	}

	pbyDib = pbyTrans = 0;
	dwDibSize = dwTransSize = 0;
	bool bHasTrans = false;
	int iImageBPP = pixel_depth;

	if(pixel_depth == 16 || pixel_depth == 32)
	{
		iImageBPP -= 8;
		bHasTrans = true;
	}

	if(pixel_depth == 4)
		iImageBPP = 24;


	// Allocate image bits
	if(AllocateDIB(info_ptr->width, info_ptr->height, iImageBPP, pbyDib, dwDibSize) != 0)
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return false;
	}
	// Allocate transparency bits
	if(bHasTrans)
	{
		if(AllocateDIB(info_ptr->width, info_ptr->height, 8, pbyTrans, dwTransSize) != 0)
		{
			delete pbyDib;
			png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
			return false;
		}
	}

	RGBQUAD* prgbPalette = 0;

	if (info_ptr->num_palette>0)
	{
		prgbPalette = (RGBQUAD*)(pbyDib+sizeof(BITMAPINFOHEADER));
		rgb_color_struct* pPal = (rgb_color_struct*)info_ptr->palette;

		for(int i = 0; i < info_ptr->num_palette; i++)
		{
			prgbPalette[i].rgbRed   = pPal[i].red;
			prgbPalette[i].rgbGreen = pPal[i].green;
			prgbPalette[i].rgbBlue  = pPal[i].blue; 
		}
	}

	BITMAPINFOHEADER* pbi = (BITMAPINFOHEADER*)pbyDib;
	PBYTE pbyBits = pbyDib + sizeof(BITMAPINFOHEADER) + pbi->biClrUsed * 4;
	int iDibRowLen = BMPWIDTH(abs(pbi->biWidth) * pbi->biBitCount);
	byte *row_pointers = new byte[row_stride];

	BITMAPINFOHEADER* pbit = (BITMAPINFOHEADER*)pbyTrans;
	PBYTE pbyTransBits = 0;
	int iTransRowLen = 0;

	if(bHasTrans)
	{
		pbyTransBits = pbyTrans + sizeof(BITMAPINFOHEADER) + pbit->biClrUsed * 4;
		iTransRowLen = BMPWIDTH(abs(pbit->biWidth) * pbit->biBitCount);
	}

	if (info_ptr->interlace_type)
		number_passes = png_set_interlace_handling(png_ptr);
	else
		number_passes = 1;

	for (int pass=0; pass< number_passes; pass++) 
	{
		int y=0;
		for(int i = 0; i < (int)info_ptr->height; i++)
		{
// PGruzdev: status control handling disabled here
/*
			if(oStatus.SetProgress(i*pass))
			{
				delete row_pointers;
				png_read_end(png_ptr, info_ptr);				
				png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
				pFile->Close();
				return EID_USER_ABORTED;
			}
*/
			PBYTE pbyRow = pbyBits + (info_ptr->height-i-1)*iDibRowLen;
			PBYTE pbyTransRow = pbyTransBits + (info_ptr->height-i-1)*iTransRowLen;
/*
			if (info_ptr->interlace_type) // I do not know what is this ???
			{
				//if (pass>0)
					//iter.GetRow(row_pointers, row_stride);
			}
*/
			png_read_row(png_ptr, row_pointers, NULL);

			switch (pixel_depth)
			{
			case 16:
				SplitGreyAInvertAlpha(pbyRow, pbyTransRow, row_pointers, info_ptr->width);
			case 24:
				CopyPNGScanlineToRGBRow(pbyRow, row_pointers, info_ptr->width);
				break;
			case 32:
				CopyPNGScanlineToRGBASplitInvertAlpha(pbyRow, pbyTransRow, row_pointers, info_ptr->width);
				//CopyPNGScanlineToRGBARow(pbyRow, row_pointers, info_ptr->width);
				break;

			case 4:
				Copy4bitPNGScanlineToRGBASplitInvertAlpha(pbyRow, row_pointers, prgbPalette, info_ptr->width);
				break;

			default:
				memcpy(pbyRow, row_pointers, row_stride);
			}
		}
	}

	delete[] row_pointers;
	
	png_read_end(png_ptr, info_ptr);	
	png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
	
	return true;
}

static void decode_rle8(BYTE* dst, BYTE* src, DWORD inlen, int w, int h)
{
	// Decompress pixel data
	BYTE rl, rl1, i;			// runlength
	BYTE clridx, clridx1;		// colorindex
	int buffer_x = 0;
	BYTE *ps, *end, *buffer_y;
	ps = src;
	end = ps + inlen;
	int dststride = BMPWIDTH(w*8);//*4;
	bool blip = false;

	buffer_y = dst + dststride*(h-1);

	while (ps < end && buffer_y >= dst)
	{
		rl = rl1 = *ps++;
		clridx = clridx1 = *ps++;
		if (rl == 0)
			if (clridx == 0)
			{
				// new scanline
				if (!blip)
				{
					// if we didnt already jumped to the new line, do it now
					buffer_x  = 0;
					buffer_y -= dststride;
				}
				continue;
			}
			else if (clridx == 1)
				// end of bitmap
				break;
			else if (clridx == 2)
			{
				// next 2 bytes mean column- and scanline- offset
				buffer_x += (1* (*ps++));
				buffer_y -= (dststride * (*ps++));
				continue;
			}
			else if (clridx > 2)
				rl1 = clridx;

			for ( i = 0; i < rl1; i++ )
			{
				if (!rl) clridx1 = *ps++;
				//memcpy (buffer_y + buffer_x, &pMap[clridx1], 3);
				buffer_y[buffer_x++] = clridx1;
				//buffer_x += 4;
				if (buffer_x >= dststride)
				{
					buffer_x  = 0;
					buffer_y -= dststride;
					blip = true;
				}
				else
					blip = false;
			}
			// pad in case rl == 0 and clridx in [3..255]
			if (rl == 0 && (clridx & 0x01)) ps++;
	}
}
/******************************************************************************
   Function:    EncodePNG
   Author:      Paul Gruzdev
   Purpose:     Encode PNG image to the virtual file
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  CIOMemFile* pFile - 
                PBYTE pbyDib, PBYTE pbyTrans
                bool bInterlaced - 
   Notes:       
******************************************************************************/
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, PBYTE pbyTrans, bool bInterlaced, bool bInvertAlpha/*=true*/, bool bFastPNG)
{
	bInterlaced=false; // disable this mode - we do not need it, but we catch some coding errors sometimes
	png_struct *png_ptr;
	png_info *info_ptr;

	// Create and initialize the png_struct with the desired error handler
	//* functions.  If you want to use the default stderr and longjump method,
	//* you can supply NULL for the last three parameters.  We also check that
	//* the library version is compatible with the one used at compile time,
	//* in case we are using dynamically linked libraries.  REQUIRED.

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
	  (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		// If we get here, we had a problem reading the file  
		return false;
	}

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	if(bFastPNG)
	{
		png_set_filter(png_ptr, 0, PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_NO_COMPRESSION);
	}
	else
		png_set_compression_level(png_ptr, Z_BEST_SPEED);

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call.
 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	BITMAPINFOHEADER* pbi = (BITMAPINFOHEADER*)pbyDib;
	PBYTE pbyBits = pbyDib + sizeof(BITMAPINFOHEADER) + pbi->biClrUsed * 4;
	int iDibRowLen = BMPWIDTH(abs(pbi->biWidth) * pbi->biBitCount);

	BITMAPINFOHEADER* pbit = (BITMAPINFOHEADER*)pbyTrans;
	PBYTE pbyTransBits = 0;
	int iTransRowLen = 0;

	if(pbit)
	{
		pbyTransBits = pbyTrans + sizeof(BITMAPINFOHEADER) + pbit->biClrUsed * 4;
		iTransRowLen = BMPWIDTH(abs(pbit->biWidth) * pbit->biBitCount);
	}

	int iBPP = pbi->biBitCount;
	if(pbit)
		iBPP += 8;

	int iWidth = abs(pbi->biWidth); // DDES-10533: DIB that comes from EMF may contain negative w and h which meams mirroring X and Y
	int iHeight = abs(pbi->biHeight);
	//int row_stride = iWidth * ((pbi->biBitCount+7)>>3);
	int row_stride = iWidth * ((iBPP+7)>>3);

	// set up the output control  
	 //png_init_io(png_ptr, fp);
	//png_set_write_fn(png_ptr, (png_voidp)pFile, PngWriteDataToJFile);
	png_ptr->io_ptr = (png_voidp)pFile;
	png_ptr->write_data_fn = PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBPP;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth/8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth/info_ptr->channels;

	switch(iBPP)
	{
	case 1:
	case 4:
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
		break;
// Have to handle greyscale case also, later
/*
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
		break;
*/
	case 16:
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
		break;
	case 24:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
		break;
	case 32:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
		break;
	default:
		{
			png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
			return false;
		}
	}
	info_ptr->compression_type = 0;
	info_ptr->filter_type = 0;
	info_ptr->valid = 0;
	info_ptr->interlace_type = bInterlaced ? 1 : 0;
	info_ptr->rowbytes = row_stride;

	// set the palette if there is one  
	if (info_ptr->color_type & PNG_COLOR_MASK_PALETTE)
	{
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = new png_color[256];
		info_ptr->num_palette = 256;
		RGBQUAD* pPal = (RGBQUAD*)(pbyDib + sizeof(BITMAPINFOHEADER));

		for (int i=0; i<256; i++)
		{
			info_ptr->palette[i].red   = pPal[i].rgbRed;
			info_ptr->palette[i].green = pPal[i].rgbGreen;
			info_ptr->palette[i].blue  = pPal[i].rgbBlue;
		}
	}  

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];
/*
	CStatusControl oStatus(pStatus);
	oStatus.SetProgressRange(iHeight);
*/
	for(int i = 0; i < iHeight; i++)
	{
/*
		if(oStatus.SetProgress(i))
		{
			delete [] row_pointers;
			// It is REQUIRED to call this to finish writing the rest of the file  
			//png_write_end(png_ptr, info_ptr);
			// if you malloced the palette, free it here  
			if (info_ptr->palette)
				delete[] (info_ptr->palette);

			// clean up after the write, and free any memory allocated  
			png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

			// close the file  
			return false;

		}
*/
		switch(iBPP)
		{
		case 24:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					pbyRow+=3;
				}
			}
			break;
		case 32:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE pbyTransRow = pbyTransBits + (iHeight-i-1)*iTransRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					
					//Changed by V.Abrutski 06.11.2006
//					*ptr++ = 255 - pbyTransRow[0];

					if (bInvertAlpha)
						*ptr++ = 255 - pbyTransRow[0];
					else
						*ptr++ = pbyTransRow[0];
					//end of changes


					pbyRow += 3;
					pbyTransRow ++;
				}
			}
			break;
// 1788 bug fix
		case 16:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE pbyTransRow = pbyTransBits + (iHeight-i-1)*iTransRowLen;
				PBYTE ptr = row_pointers;
				// Get the value and alpha from the two buffers
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = *pbyRow++;

					//Changed by V.Abrutski 06.11.2006
//					*ptr++ = 255 - pbyTransRow[0];

					if (bInvertAlpha)
						*ptr++ = 255 - pbyTransRow[0];
					else
						*ptr++ = pbyTransRow[0];
					//end of changes


					pbyTransRow++;
				}
			}
			break;
		default:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				memcpy(row_pointers, pbyRow, row_stride);
			}

		}

		png_write_row(png_ptr, row_pointers);
	}

	delete [] row_pointers;

   // It is REQUIRED to call this to finish writing the rest of the file  
   png_write_end(png_ptr, info_ptr);

   // if you malloced the palette, free it here  
   if (info_ptr->palette)
		delete[] (info_ptr->palette);

   // clean up after the write, and free any memory allocated  
   png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

	// that's it  
	return true;
}

/******************************************************************************
   Function:    EncodePNG
   Author:      Paul Gruzdev
   Purpose:     Encode PNG image to the virtual file
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  CIOMemFile* pFile - 
                PBYTE pbyDib, PBYTE pbyTrans
                bool bInterlaced - 
   Notes:       
******************************************************************************/
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, bool bInterlaced, bool bIngoreAlpha, bool bColour16BitDIB, bool bFastPNG, bool bIsCMYK)
{
	bInterlaced=false; // disable this mode - we do not need it, but we catch some coding errors sometimes
	png_struct *png_ptr;
	png_info *info_ptr;

	// Create and initialize the png_struct with the desired error handler
	//* functions.  If you want to use the default stderr and longjump method,
	//* you can supply NULL for the last three parameters.  We also check that
	//* the library version is compatible with the one used at compile time,
	//* in case we are using dynamically linked libraries.  REQUIRED.

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
	  (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		// If we get here, we had a problem reading the file  
		return false;
	}

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	if(bFastPNG)
	{
		png_set_filter(png_ptr, 0, PNG_ALL_FILTERS);//PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_BEST_SPEED);//Z_NO_COMPRESSION);
	}
	else
		png_set_compression_level(png_ptr, Z_BEST_SPEED);

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call.
 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	BITMAPINFOHEADER* pbi = (BITMAPINFOHEADER*)pbyDib;
	int iClrUsed = pbi->biClrUsed;

	if(!iClrUsed)
	{
		if (pbi->biBitCount <= 8)
			iClrUsed = (short) 1 << pbi->biBitCount;
		else if(pbi->biCompression == BI_BITFIELDS) // additional 3 numbers for such a compression and 24, 32 bits
		{
			iClrUsed = 3;
		}
	}

	PBYTE pbyBits = pbyDib + pbi->biSize + iClrUsed * sizeof(RGBQUAD);
	RGBQUAD* pPal = (RGBQUAD*)(pbyDib + pbi->biSize);
	int iDibRowLen = BMPWIDTH(abs(pbi->biWidth) * pbi->biBitCount);

	int iBPP = pbi->biBitCount;
	if(pbi->biBitCount == 16 && bColour16BitDIB)
		iBPP = 24;

	int iWidth = abs(pbi->biWidth); // Paul: DDES-10533 fix. BITMAPINFO may contain negative w and h meaning mirroring by X and Y respectively
	int iHeight = abs(pbi->biHeight);
	int row_stride = iWidth * ((iBPP+7)>>3);

	PBYTE pbyUncompressed = NULL;
	if(pbi->biCompression == BI_RLE8 && pbi->biBitCount == 8)
	{
		pbyUncompressed = new BYTE[iDibRowLen*iHeight];
		decode_rle8(pbyUncompressed, pbyBits, pbi->biSizeImage, iWidth, iHeight);
		pbyBits = pbyUncompressed;
	}

	DWORD pdw16BitPalette[3];
	memset(pdw16BitPalette, 0, sizeof(DWORD)*3);
	DWORD pdwShift[3];
	memset(pdwShift, 0, sizeof(DWORD)*3);
	if(pbi->biBitCount == 16 && bColour16BitDIB)
	{
		if(pbi->biCompression == BI_BITFIELDS)
			memcpy(pdw16BitPalette, pPal, sizeof(DWORD)*3);
		else
		{
			pdw16BitPalette[0] = 0x7C00;
			pdw16BitPalette[1] = 0x3E0;
			pdw16BitPalette[2] = 0x1F;
		}

		while(!(pdw16BitPalette[0] & 1))
		{
			pdw16BitPalette[0] >>= 1;
			pdwShift[0]++;
		}
		while(!(pdw16BitPalette[1] & 1))
		{
			pdw16BitPalette[1] >>= 1;
			pdwShift[1]++;
		}
		while(!(pdw16BitPalette[2] & 1))
		{
			pdw16BitPalette[2] >>= 1;
			pdwShift[2]++;
		}
	}

	// set up the output control  
	 //png_init_io(png_ptr, fp);
	//png_set_write_fn(png_ptr, (png_voidp)pFile, PngWriteDataToJFile);
	png_ptr->io_ptr = (png_voidp)pFile;
	png_ptr->write_data_fn = PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBPP;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth/8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth/info_ptr->channels;

	switch(iBPP)
	{
	case 1:
	case 4:
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
		break;
// Have to handle greyscale case also, later
/*
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
		break;
*/
	case 16:
		// we are looking at destination BPP, not source. So 16 bits are always greyA
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
		break;
	case 24:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
		break;
	case 32:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
		break;
	default:
		{
			png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
			return false;
		}
	}
	info_ptr->compression_type = 
	info_ptr->filter_type = 
	info_ptr->valid = 0;
	info_ptr->interlace_type = bInterlaced ? 1 : 0;
	info_ptr->rowbytes = row_stride;

	// set the palette if there is one  
	if (info_ptr->color_type & PNG_COLOR_MASK_PALETTE)
	{
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = new png_color[iClrUsed];
		info_ptr->num_palette = iClrUsed;
		RGBQUAD* pPal = (RGBQUAD*)(pbyDib + pbi->biSize);

		for (int i=0; i<iClrUsed; i++)
		{
			info_ptr->palette[i].red   = pPal[i].rgbRed;
			info_ptr->palette[i].green = pPal[i].rgbGreen;
			info_ptr->palette[i].blue  = pPal[i].rgbBlue;
		}
	}

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];

	for(int i = 0; i < iHeight; i++)
	{
		switch(pbi->biBitCount)
		{
		case 24:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					pbyRow+=3;
				}
			}
			break;
		case 32:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;

				if(bIsCMYK)
				{
					// CMYK - just copy inverting values - Xara has them in format similar to Photoshop
					//memcpy(row_pointers, pbyRow, row_stride);

					for(int x = 0; x < iWidth;x++)
					{
						*ptr++ = pbyRow[0];
						*ptr++ = pbyRow[1];
						*ptr++ = pbyRow[2];
						*ptr++ = 255-pbyRow[3];
						pbyRow += 4;
					}
				}
				else
				{
					// swap blue and red
					for(int x = 0; x < iWidth;x++)
					{
						*ptr++ = pbyRow[2];
						*ptr++ = pbyRow[1];
						*ptr++ = pbyRow[0];
						if(!bIngoreAlpha)
							*ptr++ = 255 - pbyRow[3];
						else
							*ptr++ = 0;

						pbyRow += 4;
					}
				}
			}
			break;
// 1788 bug fix
		case 16:
			{
				if(bColour16BitDIB)
				{
					PWORD pwRow = (PWORD)(pbyBits + (iHeight-i-1)*iDibRowLen);
					PBYTE ptr = row_pointers;

					for(int x = 0; x < iWidth;x++)
					{
						if(pdwShift[0])
							*ptr++ = (BYTE)(((*pwRow >> pdwShift[0]) & pdw16BitPalette[0]) * 255 / pdw16BitPalette[0]);
						else
							*ptr++ = (BYTE)((*pwRow & pdw16BitPalette[0]) * 255 / pdw16BitPalette[0]);

						if(pdwShift[1])
							*ptr++ = (BYTE)(((*pwRow >> pdwShift[1]) & pdw16BitPalette[1]) * 255 / pdw16BitPalette[1]);
						else
							*ptr++ = (BYTE)((*pwRow & pdw16BitPalette[1]) * 255 / pdw16BitPalette[1]);

						if(pdwShift[2])
							*ptr++ = (BYTE)(((*pwRow >> pdwShift[2]) & pdw16BitPalette[2]) * 255 / pdw16BitPalette[2]);
						else
							*ptr++ = (BYTE)((*pwRow & pdw16BitPalette[2]) * 255 / pdw16BitPalette[2]);

						pwRow++;

					}
				}
				else
				{
					PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
					PBYTE ptr = row_pointers;

					// Get the value and alpha from the two buffers
					for(int x = 0; x < iWidth;x++)
					{
						*ptr++ = *pbyRow++;
						//*ptr++ = 255 - *pbyRow++;
					// PG: don't invert alpha for greyscale case
						*ptr++ = *pbyRow++;
					}
				}
			}
			break;
		default:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				memcpy(row_pointers, pbyRow, row_stride);
			}

		}

		png_write_row(png_ptr, row_pointers);
	}

	delete [] row_pointers;

   // It is REQUIRED to call this to finish writing the rest of the file  
   png_write_end(png_ptr, info_ptr);

   // if you malloced the palette, free it here  
   if (info_ptr->palette)
		delete[] (info_ptr->palette);

   // clean up after the write, and free any memory allocated  
   png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

   if(pbyUncompressed)
	   delete pbyUncompressed;

	// that's it  
	return true;
}

/******************************************************************************
   Function:    EncodePNG
   Author:      Ekaterina Dolgunicheva
   Purpose:     Encode PNG image to the virtual file + Replace the 'Thrown color' with the transparency
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  CIOMemFile* pFile - 
                PBYTE pbyDib, PBYTE pbyTrans
                bool bInterlaced - 
				bool pThrowColor - replace this color with the transparency
   Notes:       
******************************************************************************/
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, bool bInterlaced, png_color pThrownColor)
{
	bInterlaced=false; // disable this mode - we do not need it, but we catch some coding errors sometimes
	png_struct *png_ptr;
	png_info *info_ptr;

	// Create and initialize the png_struct with the desired error handler
	//* functions.  If you want to use the default stderr and longjump method,
	//* you can supply NULL for the last three parameters.  We also check that
	//* the library version is compatible with the one used at compile time,
	//* in case we are using dynamically linked libraries.  REQUIRED.

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
	  (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		// If we get here, we had a problem reading the file  
		return false;
	}

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	png_set_filter(png_ptr, 0, PNG_ALL_FILTERS);//PNG_NO_FILTERS);
	png_set_compression_level(png_ptr, Z_BEST_SPEED);//Z_NO_COMPRESSION);

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call.
 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	BITMAPINFOHEADER* pbi = (BITMAPINFOHEADER*)pbyDib;
	int iClrUsed = pbi->biClrUsed;

	if(!iClrUsed)
	{
		if (pbi->biBitCount <= 8)
			iClrUsed = (short) 1 << pbi->biBitCount;
		else if(pbi->biCompression == BI_BITFIELDS) // additional 3 numbers for such a compression and 24, 32 bits
		{
			iClrUsed = 3;
		}
	}

	PBYTE pbyBits = pbyDib + pbi->biSize + iClrUsed * sizeof(RGBQUAD);
	RGBQUAD* pPal = (RGBQUAD*)(pbyDib + pbi->biSize);
	int iDibRowLen = BMPWIDTH(abs(pbi->biWidth) * pbi->biBitCount);

	int iBPP = pbi->biBitCount;
	int iWidth = abs(pbi->biWidth); // Paul: DDES-10533 fix. BITMAPINFO may contain negative w and h meaning mirroring by X and Y respectively
	int iHeight = abs(pbi->biHeight);
	int row_stride = iWidth * ((iBPP+7)>>3);

	PBYTE pbyUncompressed = NULL;
	if(pbi->biCompression == BI_RLE8 && pbi->biBitCount == 8)
	{
		pbyUncompressed = new BYTE[iDibRowLen*iHeight];
		decode_rle8(pbyUncompressed, pbyBits, pbi->biSizeImage, iWidth, iHeight);
		pbyBits = pbyUncompressed;
	}

	DWORD pdw16BitPalette[3];
	memset(pdw16BitPalette, 0, sizeof(DWORD)*3);
	DWORD pdwShift[3];
	memset(pdwShift, 0, sizeof(DWORD)*3);

	// set up the output control  
	 //png_init_io(png_ptr, fp);
	//png_set_write_fn(png_ptr, (png_voidp)pFile, PngWriteDataToJFile);
	png_ptr->io_ptr = (png_voidp)pFile;
	png_ptr->write_data_fn = PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBPP;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth/8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth/info_ptr->channels;

	switch(iBPP)
	{
	case 1:
	case 4:
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
		break;
	case 16:
		// we are looking at destination BPP, not source. So 16 bits are always greyA
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
		break;
	case 24:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
		break;
	case 32:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
		break;
	default:
		{
			png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
			return false;
		}
	}
	info_ptr->compression_type = 
	info_ptr->filter_type = 
	info_ptr->valid = 0;
	info_ptr->interlace_type = bInterlaced ? 1 : 0;
	info_ptr->rowbytes = row_stride;

	// set the palette if there is one  
	if (info_ptr->color_type & PNG_COLOR_MASK_PALETTE)
	{
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = new png_color[iClrUsed];
		info_ptr->num_palette = iClrUsed;
		RGBQUAD* pPal = (RGBQUAD*)(pbyDib + pbi->biSize);

		for (int i=0; i<iClrUsed; i++)
		{
			info_ptr->palette[i].red   = pPal[i].rgbRed;
			info_ptr->palette[i].green = pPal[i].rgbGreen;
			info_ptr->palette[i].blue  = pPal[i].rgbBlue;
		}
	}

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];

	for(int i = 0; i < iHeight; i++)
	{
		switch(pbi->biBitCount)
		{
		case 24:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					pbyRow+=3;
				}
			}
			break;
		case 32:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					if(pbyRow[2] == pThrownColor.red && pbyRow[1] == pThrownColor.green && pbyRow[0] == pThrownColor.blue)
						*ptr++ = 0;
					else
						*ptr++ = 255 - pbyRow[3];
					pbyRow += 4;
				}
			}
			break;
// 1788 bug fix
		case 16:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				PBYTE ptr = row_pointers;

				// Get the value and alpha from the two buffers
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = *pbyRow++;
					//*ptr++ = 255 - *pbyRow++;
				// PG: don't invert alpha for greyscale case
					*ptr++ = *pbyRow++;
				}
			}
			break;
		default:
			{
				PBYTE pbyRow = pbyBits + (iHeight-i-1)*iDibRowLen;
				memcpy(row_pointers, pbyRow, row_stride);
			}

		}

		png_write_row(png_ptr, row_pointers);
	}

	delete [] row_pointers;

   // It is REQUIRED to call this to finish writing the rest of the file  
   png_write_end(png_ptr, info_ptr);

   // if you malloced the palette, free it here  
   if (info_ptr->palette)
		delete[] (info_ptr->palette);

   // clean up after the write, and free any memory allocated  
   png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

   if(pbyUncompressed)
	   delete pbyUncompressed;

	// that's it  
	return true;
}

bool ReadRawPipeBuffer(HANDLE hProcess, HANDLE hPipe, PBYTE pbyBuffer, DWORD dwSize, DWORD& dwActually)
{
	dwActually = 0;

	while(TRUE)
	{
/*		DWORD	dwAvail = 0;
		if(PeekNamedPipe(hPipe, NULL, 0, NULL, &dwAvail, NULL) && dwAvail >= dwSize) */
		{
			if (!ReadFile(hPipe, pbyBuffer, dwSize, &dwActually,NULL) || !dwActually)
			{
				if (GetLastError() == ERROR_BROKEN_PIPE)
					break; // pipe done - normal exit path.
				else
					return false;
			}

			dwSize	  -= dwActually;
			pbyBuffer += dwActually;
			
			if( 0 == dwSize )
				break;
		}
/*		else if(WaitForSingleObject(hProcess, 0) != WAIT_OBJECT_0)
		{
			Sleep(50);
		}
		else
			break; */
	}

	return true;
}

/******************************************************************************
   Function:    EncodeRawPNG
   Author:      Paul Gruzdev
   Purpose:     Encode PNG image given the RAW read pipe to the virtual file
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  vcFile* pFile - 
                HANDLE hReadPipe - the pipe to read RAW from
                bool bInterlaced - 
   Notes:       
******************************************************************************/
bool EncodeRawPNG(vcFile* pFile, HANDLE hRawProcess, HANDLE hReadPipe, int iWidth, int iHeight, bool bInterlaced, ProgressCallback progress, LPVOID lpParam)
{
	png_struct *png_ptr;
	png_info *info_ptr;

	// Create and initialize the png_struct with the desired error handler
	//* functions.  If you want to use the default stderr and longjump method,
	//* you can supply NULL for the last three parameters.  We also check that
	//* the library version is compatible with the one used at compile time,
	//* in case we are using dynamically linked libraries.  REQUIRED.

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
	  (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
	{
		// If we get here, we had a problem reading the file  
		return false;
	}

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	{
		png_set_filter(png_ptr, 0, PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_NO_COMPRESSION);
	}

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call.
 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	int iBPP = 24;
	int row_stride = iWidth * ((iBPP+7)>>3);

	// set up the output control  
	 //png_init_io(png_ptr, fp);
	//png_set_write_fn(png_ptr, (png_voidp)pFile, PngWriteDataToJFile);
	png_ptr->io_ptr = (png_voidp)pFile;
	png_ptr->write_data_fn = PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBPP;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth/8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth/info_ptr->channels;

	info_ptr->color_type = PNG_COLOR_TYPE_RGB;
	info_ptr->compression_type = 
	info_ptr->filter_type = 
	info_ptr->interlace_type = bInterlaced ? 1 : 0;
	info_ptr->valid = 0;
	info_ptr->rowbytes = row_stride;

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];
	int lastp = 0;

	for(int i = 0; i < iHeight; i++)
	{
		DWORD dwActually = 0;
		ReadRawPipeBuffer(hRawProcess, hReadPipe, row_pointers, row_stride, dwActually);

		if(dwActually != row_stride)
		{
			int vvv=5;
		}

		if(progress)
		{
			int p = i*100/iHeight;
			if (p > lastp)
			{
				progress(lpParam, p);
				lastp = p;
			}
		}
/*
		// swap blue and red
		for(int x = 0; x < iWidth;x++)
		{
			*ptr++ = pbyRow[2];
			*ptr++ = pbyRow[1];
			*ptr++ = pbyRow[0];
			pbyRow+=3;
		}
*/
		png_write_row(png_ptr, row_pointers);
	}

	delete [] row_pointers;

	// It is REQUIRED to call this to finish writing the rest of the file  
	png_write_end(png_ptr, info_ptr);

	// if you malloced the palette, free it here  
	if (info_ptr->palette)
		delete[] (info_ptr->palette);

	// clean up after the write, and free any memory allocated  
	png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

	// that's it  
	return true;
}


/******************************************************************************
   Function:    GetPNGInfo
   Author:      Paul Gruzdev
   Purpose:     Get PNG info
   Returns:     bool - true if success, false if not.
   Exceptions:  
   Parameters:  
   Notes:       
******************************************************************************/
bool GetPNGInfo(vcFile* pFile, long& lWidth, long& lHeight, double& dDPI, int* pSamplesPerPixel)
{
	png_struct *png_ptr;
	png_info *info_ptr;

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (void *)NULL, (png_error_ptr)ima_png_error, NULL);

	if (png_ptr == NULL) return false; // can't create read struct
	
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		return false; //
	}
/*
	if(setjmp(png_jmpbuf(png_ptr)))
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return false;
	}
*/
	png_set_read_fn(png_ptr, (png_voidp)pFile, PngReadDataFromMemFile);

	try
	{
		png_read_info(png_ptr, info_ptr);
	}
	catch (png_const_charp message)
	{
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return false;
		message; // unreferenced param warning removal
	}

	lWidth = info_ptr->width;
	lHeight = info_ptr->height;
	dDPI = info_ptr->x_pixels_per_unit;
	dDPI = dDPI*2.54035459116 / 100.0;
	if(pSamplesPerPixel)
		*pSamplesPerPixel = info_ptr->pixel_depth;

	png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
	
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/******************************************************************************
   Function:    EncodePNG
   Author:      Paul Gruzdev
   Purpose:     Encode PNG image to the virtual file
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  vcFile* pFile - 
                PBYTE pbyDib, PBYTE pbyTrans
                bool bInterlaced - 
   Notes:       
******************************************************************************/
bool EncodePNGFromRawData(vcFile* pFile, PBYTE pbyData, int iWidth, int iHeight, PBYTE pPalette, int iPaletteCount, 
						  EPixelFormat eFormat, bool bFastPNG, bool bRowPad32)
{
	png_struct *png_ptr;
	png_info *info_ptr;

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
		return false;

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	if(bFastPNG)
	{
		png_set_filter(png_ptr, 0, PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_NO_COMPRESSION);
	}
	else
		png_set_compression_level(png_ptr, Z_BEST_SPEED);

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call. 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	int iBPP;
	switch(eFormat)
	{
	case keFormatBGR:
	case keFormatRGB24:
	case keFormatRGB32:
	case keFormatRGB15:
		iBPP = 24;
		break;
	case keFormatBGRA:
	case keFormatARGB:
	case keFormatCMYK:
		iBPP = 32;
		break;
	case keFormatGreyA:
		iBPP = 16;
		break;
	case keFormatGrey:
	case keFormatPalette:
		iBPP = 8;
		break;
	default:
		png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
		return false;
	}

	int iClrUsed = pPalette ? iPaletteCount : 0;

	PBYTE pbyBits = pbyData;
	int iDibRowLen = bRowPad32 ? BMPWIDTH(iWidth * (eFormat == keFormatRGB15 ? 16 : (eFormat == keFormatRGB32 ? 32 : iBPP))) : iWidth * ((iBPP+7)>>3);;
	int row_stride = iWidth * ((iBPP+7)>>3);

	png_ptr->io_ptr = (png_voidp)pFile;
	png_ptr->write_data_fn = PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBPP;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth/8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth/info_ptr->channels;

	switch(iBPP)
	{
	case 1:
	case 4:
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
		break;
// Have to handle greyscale case also, later
/*
	case 8:
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
		break;
*/
	case 16:
		// we are looking at destination BPP, not source. So 16 bits are always greyA
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
		break;
	case 24:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
		break;
	case 32:
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA;
		break;
	default:
		{
			png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
			return false;
		}
	}
	info_ptr->compression_type = 
	info_ptr->filter_type = 
	info_ptr->valid = 0;
	info_ptr->interlace_type = 0;
	info_ptr->rowbytes = row_stride;

	// set the palette if there is one  
	if (info_ptr->color_type & PNG_COLOR_MASK_PALETTE)
	{
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = new png_color[iClrUsed];
		info_ptr->num_palette = iClrUsed;

		for (int i=0; i<iClrUsed; i++)
		{
			info_ptr->palette[i].red   = pPalette[i*4];
			info_ptr->palette[i].green = pPalette[i*4+1];
			info_ptr->palette[i].blue  = pPalette[i*4+2];
		}
	}

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];

	for(int i = 0; i < iHeight; i++)
	{
		switch(eFormat)
		{
		case keFormatBGR:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				PBYTE ptr = row_pointers;
				// swap blue and red
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = pbyRow[2];
					*ptr++ = pbyRow[1];
					*ptr++ = pbyRow[0];
					pbyRow+=3;
				}
			}
			break;
		case keFormatRGB24:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				PBYTE ptr = row_pointers;
				// don't swap blue and red - just copy
				for(int x = 0; x < iWidth;x++)
				{
					*ptr++ = *pbyRow++;
					*ptr++ = *pbyRow++;
					*ptr++ = *pbyRow++;
				}
			}
			break;
		case keFormatRGB32:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				PBYTE ptr = row_pointers;
				// don't swap blue and red - just copy
				for(int x = 0; x < iWidth;x++)
				{
					pbyRow++; // Skip the reserved byte
					*ptr++ = *pbyRow++;
					*ptr++ = *pbyRow++;
					*ptr++ = *pbyRow++;
				}
			}
			break;
		case keFormatRGB15:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				PBYTE ptr = row_pointers;
				// don't swap blue and red - just copy
				for(int x = 0; x < iWidth;x++)
				{
					WORD wPix = *((WORD*)pbyRow);
					*ptr++ = (BYTE)(wPix >> 10);
					*ptr++ = (BYTE)((wPix >> 5) & 31);
					*ptr++ = (BYTE)(wPix & 31);
					pbyRow += 2;
				}
			}
			break;
		case keFormatARGB:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				PBYTE ptr = row_pointers;

				{
					// swap blue and red
					for(int x = 0; x < iWidth;x++)
					{
						*ptr++ = pbyRow[1];
						*ptr++ = pbyRow[2];
						*ptr++ = pbyRow[3];
						*ptr++ = pbyRow[0];

						pbyRow += 4;
					}
				}
			}
			break;
// 1788 bug fix
		case keFormatGreyA:
			{
				{
					PBYTE pbyRow = pbyBits + i*iDibRowLen;
					PBYTE ptr = row_pointers;

					// Get the value and alpha from the two buffers
					for(int x = 0; x < iWidth;x++)
					{
						*ptr++ = *pbyRow++;
						*ptr++ = *pbyRow++;
					}
				}
			}
			break;
		default:
			{
				PBYTE pbyRow = pbyBits + i*iDibRowLen;
				memcpy(row_pointers, pbyRow, row_stride);
			}
		}

		png_write_row(png_ptr, row_pointers);
	}

	delete [] row_pointers;

   // It is REQUIRED to call this to finish writing the rest of the file  
   png_write_end(png_ptr, info_ptr);

   // if you malloced the palette, free it here  
   if (info_ptr->palette)
		delete[] (info_ptr->palette);

   // clean up after the write, and free any memory allocated  
   png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

	// that's it  
	return true;
}

/******************************************************************************
   Function:    EncodePNG
   Author:      Paul Gruzdev
   Purpose:     Encode PNG image to the virtual file
   Returns:     bool - true if ok, false if not
   Exceptions:  
   Parameters:  CIOMemFile* pFile - 
                PBYTE pbyDib, PBYTE pbyTrans
                bool bInterlaced - 
   Notes:       
******************************************************************************/
bool EncodePNGFromPDFData(png_voidp pFile, PBYTE pbyData, int iWidth, int iHeight, PBYTE pPalette, int iPaletteCount, 
						  int iBitsPerComp, int iNComps, bool bFastPNG, bool bInvertKChannel/* = true*/, void (PNGAPI *pfnWriteData)(png_structp, png_bytep, png_size_t))
{
	png_struct *png_ptr;
	png_info *info_ptr;

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
		return false;

	   // Allocate/initialize the image information data.  REQUIRED  
	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
		return false;
	}

	if(bFastPNG)
	{
		png_set_filter(png_ptr, 0, PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_NO_COMPRESSION);
	}
 	else
		// This setting boosts the PNG encoding speed 3.6 times
 		png_set_compression_level(png_ptr, Z_BEST_SPEED);

	// Set error handling.  REQUIRED if you aren't supplying your own
	//* error hadnling functions in the png_create_write_struct() call. 
	if (setjmp(png_ptr->jmp_buf_local))
	{
	  // If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr,  (png_infopp)&info_ptr);
		return false;
	}

	int iBPP = iBitsPerComp * iNComps;
	int iDibRowLen = pPalette ? (iWidth * iBitsPerComp + 7) >> 3 : ((iWidth * iBitsPerComp + 7) >> 3) * iNComps;
	// In the RGB case we need to pass indexed data as is, so the actual NComps should be reduced to 1 (index)
	if (pPalette/* && iNComps == 3*/) // Denis: DDES-16253
		iNComps = 1;

	int iClrUsed = pPalette ? iPaletteCount : 0;
	PBYTE pbyBits = pbyData;
	int row_stride = (iWidth * iBPP + 7) >> 3;

	png_ptr->io_ptr = pFile;
	png_ptr->write_data_fn = pfnWriteData ? pfnWriteData : PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = iBitsPerComp*iNComps;
	info_ptr->channels = iNComps;
	info_ptr->bit_depth = iBitsPerComp;

	if(iNComps == 1 && !pPalette)
		info_ptr->color_type = PNG_COLOR_TYPE_GRAY;
	else if(iNComps == 1 && pPalette && iPaletteCount)
		info_ptr->color_type = PNG_COLOR_TYPE_PALETTE;
	else if(iNComps == 3)
		info_ptr->color_type = PNG_COLOR_TYPE_RGB;
	else if(iNComps == 4)
		info_ptr->color_type = PNG_COLOR_TYPE_RGB_ALPHA; // actually CMYK but we should pass it as this

	info_ptr->compression_type = 
	info_ptr->filter_type = 
	info_ptr->valid = 0;
	info_ptr->interlace_type = 0;
	info_ptr->rowbytes = row_stride;

	// set the palette if there is one  
	if (info_ptr->color_type & PNG_COLOR_MASK_PALETTE && pPalette && iClrUsed)
	{
		info_ptr->valid |= PNG_INFO_PLTE;
		info_ptr->palette = new png_color[iClrUsed];
		info_ptr->num_palette = iClrUsed;

		memcpy(info_ptr->palette, pPalette, iClrUsed*3);
	}

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	if (iNComps != 1 && pPalette || iNComps == 4 && bInvertKChannel)
	{
		for (int i = 0; i < iHeight; i++)
		{
			byte* row_pointers = new byte[row_stride];

			PBYTE pbyRow = pbyBits + i*iDibRowLen;
			PBYTE ptr = row_pointers;
			PBYTE pbyRowPtr = pbyRow;

			if (iNComps != 1 && pPalette)
			{
#ifdef FILTER_DEBUG
CHiResolutionTimer tm;
#endif
				LUTTransformRow(pbyRow, ptr, pPalette, iWidth, iBitsPerComp, iNComps);

#ifdef FILTER_DEBUG
				g_dLutTrans += tm.Current();
#endif
			}
			else if (iNComps == 4 && bInvertKChannel) // CMYK case
			{
#ifdef FILTER_DEBUG
				CHiResolutionTimer tm;
#endif

				for(int j = 0; j < iWidth; j++)
				{
					memcpy(ptr, pbyRowPtr, 3); // copy CMY as is
					ptr += 3; pbyRowPtr += 3;
					// For some reason in order for Xara to display it correctly, we have to invert the K channel
					// So we need to invert it back on PDF export
					*ptr++ = 255-*pbyRowPtr++;
				}

#ifdef FILTER_DEBUG
				g_dCMYK += tm.Current();
#endif

			}
			else
				memcpy(row_pointers, pbyRow, row_stride);

#ifdef FILTER_DEBUG
			CHiResolutionTimer tm;
#endif
			png_write_row(png_ptr, row_pointers);
#ifdef FILTER_DEBUG
			g_dpng_write_row += tm.Current();
#endif

			delete [] row_pointers;
		}
	}
	else
	{
#ifdef FILTER_DEBUG
		CHiResolutionTimer tm;
#endif
		PBYTE pbyRow = pbyBits;
		for (int i = 0; i < iHeight; ++i, pbyRow += iDibRowLen)
			png_write_row(png_ptr, pbyRow);

#ifdef FILTER_DEBUG
		g_dpng_write_row += tm.Current();
#endif
	}

#ifdef FILTER_DEBUG
		CHiResolutionTimer tm;
#endif

	// It is REQUIRED to call this to finish writing the rest of the file  
	png_write_end(png_ptr, info_ptr);

#ifdef FILTER_DEBUG
		g_dpng_write_row += tm.Current();
#endif

	// if you malloced the palette, free it here  
	if (info_ptr->palette)
		delete[] (info_ptr->palette);

	// clean up after the write, and free any memory allocated  
	png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

	// that's it  
	return true;
}

/******************************************************************************
Function:    EncodePNGFromPDFDataAlpha
Author:      Paul Gruzdev
Purpose:     Encode PNG image to the virtual file
Returns:     bool - true if ok, false if not
Exceptions:
Parameters:  CIOMemFile* pFile -
PBYTE pbyDib, PBYTE pbyTrans
bool bInterlaced -
Notes:
******************************************************************************/
bool EncodePNGFromPDFDataAlpha(png_voidp pFile, PBYTE pbyImage, PBYTE pbyAlpha, int iWidth, int iHeight, bool bGreyscale, bool bInvertAlpha/*=true*/, bool bFastPNG, void (PNGAPI *pfnWriteData)(png_structp, png_bytep, png_size_t))
{
	bool bInterlaced = false; // disable this mode - we do not need it, but we catch some coding errors sometimes
	png_struct *png_ptr;
	png_info *info_ptr;

	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, (void *)NULL, NULL, NULL);

	if (png_ptr == NULL)
		return false;

	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL)
	{
		png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
		return false;
	}

	if (bFastPNG)
	{
		png_set_filter(png_ptr, 0, PNG_NO_FILTERS);
		png_set_compression_level(png_ptr, Z_NO_COMPRESSION);
	}
	else
		png_set_compression_level(png_ptr, Z_BEST_SPEED);

	if (setjmp(png_ptr->jmp_buf_local))
	{
		// If we get here, we had a problem reading the file  
		png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);
		return false;
	}
	int iChannels = bGreyscale ? 2 : 4;
	int row_stride = iWidth * iChannels;

	png_ptr->io_ptr = pFile;
	png_ptr->write_data_fn = pfnWriteData ? pfnWriteData : PngWriteDataToMemFile;

	// set the file information here  
	info_ptr->width = iWidth;
	info_ptr->height = iHeight;
	info_ptr->pixel_depth = bGreyscale ? 16 : 32;
	info_ptr->channels = (info_ptr->pixel_depth >= 8) ? info_ptr->pixel_depth / 8 : 1;
	info_ptr->bit_depth = info_ptr->pixel_depth / info_ptr->channels;

	info_ptr->color_type = bGreyscale ? PNG_COLOR_TYPE_GRAY_ALPHA : PNG_COLOR_TYPE_RGB_ALPHA;

	info_ptr->compression_type = 0;
	info_ptr->filter_type = 0;
	info_ptr->valid = 0;
	info_ptr->interlace_type = bInterlaced ? 1 : 0;
	info_ptr->rowbytes = row_stride;

	// write the file information  
	png_write_info(png_ptr, info_ptr);
	// If you are only writing one row at a time, this works  

	byte* row_pointers = new byte[row_stride];
	int iDibRowLen = bGreyscale ? iWidth : iWidth * 3;

	for (int i = 0; i < iHeight; i++)
	{
		PBYTE pbyRow = pbyImage + i*iDibRowLen;
		PBYTE pbyTransRow = pbyAlpha + i*iWidth;
		PBYTE ptr = row_pointers;
		for (int x = 0; x < iWidth; x++)
		{
			if(!bGreyscale)
			{
				*ptr++ = *pbyRow++;
				*ptr++ = *pbyRow++;
				*ptr++ = *pbyRow++;
			}
			else
			{
				*ptr++ = *pbyRow++;
			}

			if (bInvertAlpha)
				*ptr++ = 255 - *pbyTransRow;
			else
				*ptr++ = *pbyTransRow;

			pbyTransRow++;
		}

		png_write_row(png_ptr, row_pointers);
	}

	delete[] row_pointers;

	png_write_end(png_ptr, info_ptr);
	png_destroy_write_struct(&png_ptr, (png_infopp)&info_ptr);

	return true;
}
