/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#ifndef DECODE_PNG_H
#define DECODE_PNG_H

#include "png/png.h"

enum EPixelFormat 
{ 
	keFormatUndefined = -1, 
	keFormatBGR = 0, 
	keFormatBGRA, 
	keFormatARGB,  // Format is: 8:alpha, 8:red, 8:green, 8:blue
	keFormatRGB24, // 8:red, 8:green, 8:blue
	keFormatRGB32, // Format is: 8:reserved, 8:red, 8:green, 8:blue
	keFormatRGB15, // This is a 16-bit colour format as follows: 1:reserved, 5:red, 5:green, 5:blue
	keFormatGreyA, 
	keFormatGrey,
	keFormatPalette,
	keFormatCMYK,  // Format is: 8:cyan, 8:magenta, 8:yellow, 8: blacK
	keFormatRGB		// General RGB
};
class vcFile;
bool DecodePNG(vcFile* pFile, PBYTE& pbyDib, DWORD& dwDibSize, PBYTE& pbyTrans, DWORD& dwTransSize);
bool DecodePNG(vcFile* pFile, PBYTE& pbyDib, DWORD& dwDibSize, PBYTE& pbyTrans, DWORD& dwTransSize, int& iChannels);
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, PBYTE pbyTrans, bool bInterlaced, bool bInvertAlpha=true, bool bFastPNG=true);
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, bool bInterlaced, bool bIngoreAlpha=false, bool bColour16BitDIB=false, bool bFastPNG=true, bool bIsCMYK=false);
bool EncodePNG(vcFile* pFile, PBYTE pbyDib, bool bInterlaced, png_color pThrownColor);

bool EncodePNGFromRawData(vcFile* pFile, PBYTE pbyData, int iWidth, int iHeight, PBYTE pPalette, int iPaletteCount, 
						  EPixelFormat eFormat, bool bFastPNG, bool bRowPad32 = true);
bool EncodePNGFromPDFData(png_voidp pFile, PBYTE pbyData, int iWidth, int iHeight, PBYTE pPalette, int iPaletteCount, 
						  int iBitsPerComp, int iNComps, bool bFastPNG, bool bInvertKChannel = true, void (PNGAPI *pfnWriteData)(png_structp, png_bytep, png_size_t) = nullptr);
bool EncodePNGFromPDFDataAlpha(png_voidp pFile, PBYTE pbyImage, PBYTE pbyAlpha, int iWidth, int iHeight, bool bGreyscale, bool bInvertAlpha, bool bFastPNG, void (PNGAPI *pfnWriteData)(png_structp, png_bytep, png_size_t) = nullptr);

typedef void (*ProgressCallback)(LPVOID, int);

bool EncodeRawPNG(vcFile* pFile, HANDLE hRawProcess, HANDLE hReadPipe, int iWidth, int iHeight, bool bInterlaced, ProgressCallback progress, LPVOID lpParam);

bool GetPNGInfo(vcFile* pFile, long& lWidth, long& lHeight, double& dDPI, int* pSamplesPerPixel=NULL);
#endif //DECODE_PNG_H
