/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "dibfunctions.h"

/******************************************************************************
   Function:    AllocateDIB
   Author:      Paul Gruzdev
   Purpose:     Create the BITMAPINFOHEADER and image bytes buffer.
   Returns:     int -
   Exceptions:  
   Parameters:  DWORD dwWidth - 
                DWORD dwHeight - 
                WORD wBitCount - 
                PBYTE& rspDIB - 
                DWORD& dwDibSize - 
   Notes:       
******************************************************************************/
CErrorAutoThrow AllocateDIB(DWORD dwWidth, DWORD dwHeight, WORD wBitCount, PBYTE& rspDIB, DWORD& dwDibSize)
{
	BITMAPINFOHEADER BI;		// info on DIB
	PBITMAPINFOHEADER pBI;		// pointer to the DIB
	PBYTE spDIB;				// handle to the DIB

	rspDIB = 0;

	BI.biSize = sizeof(BITMAPINFOHEADER);
	BI.biWidth = dwWidth;
	BI.biHeight = dwHeight;
	BI.biPlanes = 1;
	BI.biBitCount = wBitCount;
	BI.biCompression = BI_RGB;
	BI.biXPelsPerMeter = 0;
	BI.biYPelsPerMeter = 0;
	if (BI.biBitCount == 24 || BI.biBitCount == 32)
	{
		BI.biClrUsed = 0;
		BI.biClrImportant = 0;
	}
	else
	{
		BI.biClrUsed = (short) 1 << BI.biBitCount;
		BI.biClrImportant = (short) 1 << BI.biBitCount;
	}

	BI.biSizeImage = (DWORD) BMPWIDTH(BI.biWidth * BI.biBitCount) * BI.biHeight;

	// time to allocate some space and lock it in
	if (BI.biBitCount == 24 || BI.biBitCount == 32)
		dwDibSize =  sizeof(BITMAPINFOHEADER) + BI.biSizeImage;
	else
		dwDibSize = sizeof(BITMAPINFOHEADER) + BI.biClrUsed * 4 + BI.biSizeImage;

	spDIB = new BYTE [dwDibSize];

	if (!spDIB)
		return EID_MEMORY_ERROR;

	// move the info into the bitmap
	pBI = (PBITMAPINFOHEADER) spDIB;
	*pBI = BI;


	if (BI.biBitCount == 8)
	{
		PBYTE pbyPalette = spDIB + sizeof(BITMAPINFOHEADER);
		for (DWORD c = 0; c < BI.biClrUsed; c ++)
		{
			pbyPalette[c*4]   = (BYTE)c;
			pbyPalette[c*4+1] = (BYTE)c;
			pbyPalette[c*4+2] = (BYTE)c;
			pbyPalette[c*4+3] = (BYTE)c;
		}
	}

	rspDIB = spDIB;
	return EID_NO_ERROR;
}

/******************************************************************************
   Function:    NewDIB
   Author:      Paul Gruzdev
   Purpose:     Creates a new DIB
   Returns:     CErrorAutoThrow -
   Parameters:  PBYTE &rspDest - destination image data
                int iWidth - DIB width in pixels
                int iHeight - DIB height in pixels
                int iColorDepth - bits per pixel
                bool bGreyScale - 
				const CColorRef * pkColor - color to fill the DIB with.  If NULL
						or uninitialized, it will be filled with 0.  The default
						is NULL.
				BYTE * pbyColorTable - optional color table paletted dibs or 
						NULL (default) for default.  The table must be the correct
						size for the bit depth, even if fewer colors are specified.
				int iColorTableCount - optional color table number of palette
						entries, or -1 for default (maximum based on pixel size).
   Notes:       If specified, the color table is assumed to be the "correct"
				size for the bit depth.  This color table is not used for 24 bpp 
				or 8 bpp greyscale images; in all other cases it is simply copied 
				into the new dib.
******************************************************************************/
CErrorAutoThrow NewDIB(PBYTE &rspDest, DWORD& dwDibSize, int iWidth, int iHeight, int iColorDepth, bool bGreyScale, 
					   BYTE* pbyColorTable, int iColorTableCount)
{
	PBYTE					byStorePic;
	CErrorAutoThrow			oAutoError;
	DWORD					dwBmpWd;
	DWORD					dwColorTableSize;		// color table size in bytes
//	DWORD					dwBytesFilled;
//	DWORD					dwBytesPerRow;
//	DWORD					dwCopy;
	BITMAPINFOHEADER		BI;		// structure of a DIB image header
	PBITMAPINFOHEADER		lpBI;		// pointer to the Bitmap info
	int						iCnt;
	int						iRed;
	int						iGreen;
	int						iBlue;
	RGBQUAD *				pPalette = NULL;
	static BYTE byFinalPal4[64] =
	{
		0, 0, 0, 0,
		0, 0, 128, 0,
		0, 128, 0, 0,
		0, 128, 128, 0,
		128, 0, 0, 0,
		128, 0, 128, 0,
		128, 128, 0, 0,

		192, 192, 192, 0,
		128, 128, 128, 0,

		0, 0, 255, 0,
		0, 255, 0, 0,
		0, 255, 255, 0,
		255, 0, 0, 0,
		255, 0, 255, 0,
		255, 255, 0, 0,
		255, 255, 255, 0};

	// did they send me anything
	if ((iWidth < 1) || (iHeight < 1))
		return EID_INVALID_PARAMETER_PASSED;
	if (bGreyScale && (iColorDepth != 8) && (iColorDepth != 16))
		return EID_INVALID_PARAMETER_PASSED;
	if ((iColorDepth != 24) && (iColorDepth != 8) && (iColorDepth != 4) && (iColorDepth != 1) &&
		(iColorDepth != 32) && (iColorDepth != 16))
		return EID_INVALID_PARAMETER_PASSED;

	dwBmpWd = (DWORD) BMPWIDTH((DWORD) iWidth * (DWORD) iColorDepth);

	BI.biSize = sizeof(BITMAPINFOHEADER);
	BI.biWidth = iWidth;
	BI.biHeight = iHeight;
	BI.biSizeImage = dwBmpWd * (DWORD) iHeight;
	BI.biPlanes = 1;
	BI.biBitCount = (WORD) iColorDepth;
	BI.biCompression = BI_RGB;
	BI.biXPelsPerMeter = 0;
	BI.biYPelsPerMeter = 0;
	if (iColorDepth <= 8)
		BI.biClrUsed = 1 << iColorDepth;
	//  Need a palette for GreyA also
	else if (iColorDepth == 16)
		BI.biClrUsed = 1 << 8;
	else
		BI.biClrUsed = 0;
	if ((iColorTableCount <= 0) || (iColorTableCount > (int)BI.biClrUsed))
		iColorTableCount = BI.biClrUsed;
	BI.biClrImportant = iColorTableCount;

	// allocate the space needed for DIB
	dwDibSize = sizeof(BI) + (DWORD) BI.biSizeImage + (DWORD) (4 * BI.biClrUsed);
	rspDest = new BYTE[dwDibSize];
	if (!rspDest)
	{
		return EID_MEMORY_ERROR;
	}

	lpBI = (PBITMAPINFOHEADER)rspDest;
	*lpBI = BI;
	
	byStorePic = (PBYTE) lpBI + sizeof(BI);

	// fill in the color table
	dwColorTableSize = (DWORD) BI.biClrUsed * 4;
	if (iColorTableCount != 0)
	{
		pPalette = (RGBQUAD *) byStorePic;
		if (bGreyScale)
		{
			for (iCnt = 0; iCnt < iColorTableCount; iCnt++)
			{
				*byStorePic++ = (BYTE) iCnt;
				*byStorePic++ = (BYTE) iCnt;
				*byStorePic++ = (BYTE) iCnt;
				*byStorePic++ = 0;
			}
		}
		else if (pbyColorTable != NULL)
		{
			memcpy(byStorePic, pbyColorTable, iColorTableCount * 4);
		}
		else if (iColorTableCount == 2)
		{
			*byStorePic++ = 0;
			*byStorePic++ = 0;
			*byStorePic++ = 0;
			*byStorePic++ = 0;
			*byStorePic++ = 255;
			*byStorePic++ = 255;
			*byStorePic++ = 255;
			*byStorePic++ = 255;
		}
		else if (iColorTableCount <= 16)
		{
			memcpy(byStorePic, byFinalPal4, iColorTableCount * 4);
		}
		else
		{
			_ASSERT(dwColorTableSize == 256*4);
			for (iRed = 0; iRed <= 255;)
			{
				for (iGreen = 0; iGreen <= 255;)
				{
					for (iBlue = 0; iBlue <= 255;)
					{
						*byStorePic++ = (BYTE) iBlue;
						*byStorePic++ = (BYTE) iRed;
						*byStorePic++ = (BYTE) iGreen;
						*byStorePic++ = 0;
						iBlue += 51;
					}
					iGreen += 51;
				}
				iRed += 51;
			}
		}
	}

	// Point to the beginning of the image data
	byStorePic = (PBYTE) lpBI + sizeof(BI) + dwColorTableSize;

	return EID_NO_ERROR;
}

/******************************************************************************
   Function:    SetPalette
   Author:      Paul Gruzdev
   Purpose:     Set the palette (color lookup table) for indexed pixels.
   Returns:     none
   Exceptions:  
   Parameters:  const RGBQUAD * - pointer to palette.
				int iPaletteCount - number of entries in palette.  The default
						is -1, which uses the maximum size based on the bits
						per pixel.
   Notes:       
******************************************************************************/
void SetPalette(PBYTE pbyDib, RGBQUAD * pbyPalette, int iPaletteCount)
{
	_ASSERT(pbyPalette != NULL);
	PBITMAPINFOHEADER pbi = (PBITMAPINFOHEADER)pbyDib;
	RGBQUAD* pPalette = (RGBQUAD*)(pbyDib + sizeof(BITMAPINFOHEADER));

	// Get the maximum number of colors in the palette
	int iMaxCount = 1 << pbi->biBitCount;
	if (iMaxCount > (1 << 16))
		iMaxCount = 0;
	else if (iMaxCount > 256)
		iMaxCount = 256;

	// set the palette count
	if (iPaletteCount < 0)
		iPaletteCount = iMaxCount;
	_ASSERT(iPaletteCount <= iMaxCount);
	if (iPaletteCount > iMaxCount)
		iPaletteCount = iMaxCount;
	pbi->biClrUsed = iPaletteCount;

	for (int i = 0;  i < iPaletteCount;  ++i)
		pPalette[i] = pbyPalette[i];
}

/******************************************************************************
   Function:    SaveBMP
   Author:      Paul Gruzdev
   Purpose:     This is a helper function to save a DIB image as a BMP file on disk. 
				Used for debug purposes only
   Returns:     bool -
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                DWORD dwDibSize - 
                const char* pszName - 
   Notes:       
******************************************************************************/
bool SaveBMP(PBYTE pbyDib, DWORD dwDibSize, LPCTSTR pszName)
{
	FILE* out = _tfopen(pszName, _T("wb"));

	if(out)
	{
		BITMAPINFOHEADER* pbi = (BITMAPINFOHEADER*)pbyDib;
		BITMAPFILEHEADER head;

		dwDibSize = dwDibSize == 0 ? pbi->biSizeImage : dwDibSize;
		head.bfType = 0x4D42;
		head.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + pbi->biClrUsed*4;
		head.bfSize = head.bfOffBits + pbi->biSizeImage;
		head.bfReserved1 = 0;
		head.bfReserved2 = 0;

		fwrite(&head, sizeof(head), 1, out);
		fwrite(pbyDib, dwDibSize, 1, out);
		fclose(out);
	}
	else
	{
		TCHAR tch[500];
		_stprintf(tch, _T("Can't create %s"), pszName);
		MessageBox(NULL, tch, _T("Error"), MB_OK);
		return false;
	}

	return true;
}

/******************************************************************************
   Function:    InvertDIB
   Author:      Paul Gruzdev
   Purpose:     Invert the colours of the image
   Returns:     CErrorAutoThrow - Standard JID return code in exception wrapper.
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
   Notes:       
******************************************************************************/
CErrorAutoThrow InvertDIB(PBYTE pbyDib)
{
	PBITMAPINFOHEADER pbi = (PBITMAPINFOHEADER)pbyDib;
	if(pbi->biBitCount != 8 && pbi->biBitCount != 16 && pbi->biBitCount != 24 && pbi->biBitCount != 32)
		return EID_INVALID_PARAMETER_PASSED;

	PBYTE pbyBits = pbyDib + sizeof(BITMAPINFOHEADER) + pbi->biClrUsed * 4;
	int iDibRowLen = BMPWIDTH(pbi->biWidth * pbi->biBitCount);

	for(int i = 0; i < pbi->biHeight; i++)
	{
		PBYTE pbyPtr = pbyBits;
		switch(pbi->biBitCount)
		{
		case 8:
			{
				for(int j = 0; j < pbi->biWidth; j++)
					*pbyPtr++ = 255-*pbyPtr;
			}
			break;
		case 16:
			{
				for(int j = 0; j < pbi->biWidth; j++)
				{
					*pbyPtr++ = 255-*pbyPtr;
					pbyPtr++;
				}
			}
			break;
		case 24:
			{
				for(int j = 0; j < pbi->biWidth*3; j++)
					*pbyPtr++ = 255-*pbyPtr;
			}
			break;
		case 32:
			{
				for(int j = 0; j < pbi->biWidth; j++)
				{
					*pbyPtr++ = 255-*pbyPtr;
					*pbyPtr++ = 255-*pbyPtr;
					*pbyPtr++ = 255-*pbyPtr;
					pbyPtr++;
				}
			}
			break;
		}
		pbyBits += iDibRowLen;
	}

	return EID_NO_ERROR;
}

/******************************************************************************
   Function:    LUTTransformRow
   Author:      Paul Gruzdev
   Purpose:     Translate a row of indexed data into a 8-bit per component
				output row. Indexed data is packed in src and could be 2 bpi,
				4 bpi and 8 bits per index. Number of components could be 1 for
				gray data, 3 for RGB data and 4 for CMYK data
   Returns:     true/false
   Exceptions:  
   Parameters:  PBYTE pbySrc - 
   Notes:       
******************************************************************************/
bool LUTTransformRow(PBYTE pbySrc, PBYTE pbyDst, PBYTE pbyLUT, int iWidth, int iNumBits, int iNComp)
{
	switch(iNumBits)
	{
	case 1:
	case 2:
	case 4:
		{
			int iNumPixInByte = 8/iNumBits;
			int iMask = (1 << iNumBits) - 1;
			for(int i = 0; i < iWidth; i++)
			{
				int iShift = ((iNumPixInByte-1) - (i % iNumPixInByte))*iNumBits;
				int iIndex = (pbySrc[i/iNumPixInByte] >> iShift) & iMask;
				memcpy(pbyDst, &pbyLUT[iIndex], iNComp);
				pbyDst += iNComp;
			}

		}
		break;
	case 8:
		{
			for(int i = 0; i < iWidth; i++)
			{
				//pbyDst[0] = 
				memcpy(pbyDst, &pbyLUT[(*pbySrc++)*iNComp], iNComp);
				pbyDst += iNComp;
			}
		}
		break;
	default:
		return false;
	}

	return true;
}

/******************************************************************************
   Function:    BitmapToDIB
   Author:      Paul Gruzdev
   Purpose:     This code will convert a Device Dependent Bitmap to a DIB.
   Returns:     BErrorCode -
   Exceptions:  
   Parameters:  HBITMAP hBitMap - bitmap
                PHANDLE phNewDIB - returned bitmap
   Parameters:  bool ignoreTransparency - reset the 4th channel to avoid getting errors due to the fact that
				the EMF transparency tools are not processed now (Ekaterina Dolgunicheva)
   Notes:       
******************************************************************************/
BErrorCode BitmapToDIB(HBITMAP hBitMap, PBYTE* phNewDIB, DWORD& dwDibSize, bool ignoreTransparency)
{
	BITMAPINFOHEADER	BI;
	PBITMAPINFOHEADER	lpBI;
	BITMAP				sBitMap;
	HPALETTE			hPrevPal;
	HPALETTE			hMyPal;
	HPALETTE			hPal = NULL;
	HDC					hDC;
	DWORD				dwMemNeeded;
	WORD				wNumColors;
	PLOGPALETTE			lpPal;

	if (!hBitMap)
		return EID_INVALID_PARAMETER_PASSED;
	hMyPal = 0;
	lpPal = 0;

	hDC = CreateDC(_T("DISPLAY"), NULL, NULL, NULL);

	wNumColors = (WORD) (GetDeviceCaps(hDC, SIZEPALETTE));
	if (wNumColors == 256)
	{
		PBYTE hPalette = new BYTE[sizeof(LOGPALETTE) + (WORD) 1024];
		// Allocate for the logical palette structure
		//hPalette = GlobalAlloc(GHND, sizeof(LOGPALETTE) + (WORD) 1024);
		if (!hPalette)
		{
			DeleteDC(hDC);
			return EID_MEMORY_ERROR;
		}

		lpPal = (PLOGPALETTE)hPalette;

		GetSystemPaletteEntries(hDC, (WORD) 0, (WORD) wNumColors, (PPALETTEENTRY) &lpPal->palPalEntry[0].peRed);
		lpPal->palVersion = 0x300;
		lpPal->palNumEntries = (int) 256;

		hMyPal = CreatePalette((PLOGPALETTE) lpPal);
		delete[] hPalette;
	}
	else
	{
		hPal = (HPALETTE) GetStockObject(DEFAULT_PALETTE);
	}

	DeleteDC(hDC);

	// tell me about the picture
	GetObject(hBitMap, sizeof(sBitMap), (PSTR) & sBitMap);

	// setup the info about the DIB
	BI.biSize = sizeof(BITMAPINFOHEADER);
	BI.biWidth = sBitMap.bmWidth;
	BI.biHeight = sBitMap.bmHeight;
	BI.biPlanes = 1;
	BI.biBitCount = (WORD) (sBitMap.bmPlanes * sBitMap.bmBitsPixel);
	if ((BI.biBitCount > 1) && (BI.biBitCount < 4))
		BI.biBitCount = 4;
	if ((BI.biBitCount > 4) && (BI.biBitCount < 8))
		BI.biBitCount = 8;
	if (BI.biBitCount > 8 && BI.biBitCount < 24)
		BI.biBitCount = 24;
	BI.biCompression = BI_RGB;
	BI.biSizeImage = (DWORD) BMPWIDTH(BI.biWidth * BI.biBitCount) * BI.biHeight;
	BI.biXPelsPerMeter = 0;
	BI.biYPelsPerMeter = 0;
	if (BI.biBitCount >= 24)
	{
		BI.biClrUsed = 0;
		BI.biClrImportant = 0;
	}
	else
	{
		BI.biClrUsed = 1 << BI.biBitCount;
		BI.biClrImportant = 1 << BI.biBitCount;
	}

	// give me some space to put this in
	dwMemNeeded = sizeof(BI) + DibPalSize(BI.biClrUsed, BI.biBitCount) + BI.biSizeImage;
	*phNewDIB = new BYTE[dwMemNeeded];
	if (!*phNewDIB)
	{
		if (hMyPal)
			DeleteObject(hMyPal);
		return EID_MEMORY_ERROR;
	}

	dwDibSize = dwMemNeeded;
	lpBI = (PBITMAPINFOHEADER)*phNewDIB;
	*lpBI = BI;

	hDC = CreateDC(_T("DISPLAY"), NULL, NULL, NULL);

	if (hMyPal)
		hPrevPal = SelectPalette(hDC, hMyPal, false);
	else
		hPrevPal = SelectPalette(hDC, hPal, false);
	RealizePalette(hDC);

	// now give me the bits, and rgb values
	if (GetDIBits(hDC, hBitMap, 0, (WORD) BI.biHeight, (PSTR) lpBI + (DWORD) lpBI->biSize + (DWORD) DibPalSize(BI.biClrUsed, BI.biBitCount),
				  (PBITMAPINFO) lpBI, DIB_RGB_COLORS) == 0)
	{

		delete *phNewDIB;

		SelectPalette(hDC, hPrevPal, false);
		DeleteDC(hDC);

		if (hMyPal)
			DeleteObject(hMyPal);

		return EID_FUNCTION_UNABLE_TO_COMPLETE;
	}

	if (ignoreTransparency && BI.biBitCount == 32){ //Fill an alpha channel with zeros to avoid the influence of EMF alpha blending tools
		PBYTE offset = (PBYTE)lpBI + (DWORD)lpBI->biSize + (DWORD)DibPalSize(BI.biClrUsed, BI.biBitCount);
		int iResAD = BMPWIDTH(BI.biWidth * 32) - BI.biWidth * 4;
		for (int i = 0; i < BI.biHeight; i++){
			for (int j = 0; j < BI.biWidth; j++){
				offset += 3;
				*offset++ = 0;
			}
			offset += iResAD;
		}
	}
	
	// GetDIBits is a real messed up function
	// reset everything in case it screwed with my info
	BI.biSize = sizeof(BITMAPINFOHEADER);
	BI.biWidth = sBitMap.bmWidth;
	BI.biHeight = sBitMap.bmHeight;
	BI.biPlanes = 1;
	BI.biBitCount = (WORD) (sBitMap.bmPlanes * sBitMap.bmBitsPixel);
	if ((BI.biBitCount > 1) && (BI.biBitCount < 4))
		BI.biBitCount = 4;
	if ((BI.biBitCount > 4) && (BI.biBitCount < 8))
		BI.biBitCount = 8;
	if (BI.biBitCount > 8 && BI.biBitCount < 24)
		BI.biBitCount = 24;
	BI.biCompression = BI_RGB;
	BI.biSizeImage = (DWORD) BMPWIDTH(BI.biWidth * BI.biBitCount) *
		BI.biHeight;
	BI.biXPelsPerMeter = 0;
	BI.biYPelsPerMeter = 0;
	if (BI.biBitCount >= 24)
	{
		BI.biClrUsed = 0;
		BI.biClrImportant = 0;
	}
	else
	{
		BI.biClrUsed = 1 << BI.biBitCount;
		BI.biClrImportant = 1 << BI.biBitCount;
	}
	*lpBI = BI;

	SelectPalette(hDC, hPrevPal, false);
	DeleteDC(hDC);

	if (hMyPal)
		DeleteObject(hMyPal);

	return EID_NO_ERROR;
}

/**
 *  Save bitmap (just apiece of code from MSDN).
 */
/******************************************************************************
   Function:    SaveBMP
   Author:      MSDN
   Purpose:     This is a helper function to save a DIB image as a BMP file on disk. 
				Used for converting WMF to BMP
   Returns:     bool -
   Exceptions:  
   Parameters:  PBYTE pbyDib - 
                DWORD dwDibSize - 
                const char* pszName - 
   Notes:       
******************************************************************************/
bool SaveBMPFile(LPCTSTR pszFile, HBITMAP hBMP, BITMAPINFO *pbi) 
{
    HANDLE hf;                  // file handle 
    BITMAPFILEHEADER hdr;       // bitmap file-header 
    PBITMAPINFOHEADER pbih;     // bitmap info-header 
    LPBYTE lpBits;              // memory pointer 
    DWORD dwTotal;              // total count of bytes 
    DWORD cb;                   // incremental count of bytes 
    BYTE *hp;                   // byte pointer 
    DWORD dwTmp; 

    pbih = (PBITMAPINFOHEADER) pbi; 
    lpBits = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbih->biSizeImage);

    if (!lpBits) 
         return false;

	HDC hDC = GetDC(0);
	DWORD biClrUsed = pbi->bmiHeader.biClrUsed;
    // Retrieve the color table (RGBQUAD array) and the bits 
    // (array of palette indices) from the DIB. 
    if (!GetDIBits(hDC, hBMP, 0, (WORD) pbih->biHeight, lpBits, pbi, 
        DIB_RGB_COLORS)) 
    {
		// Free memory. 
		GlobalFree((HGLOBAL)lpBits);
		ReleaseDC(0, hDC);
        return false;
    }
	pbi->bmiHeader.biClrUsed = biClrUsed;

    // Create the .BMP file. 
    hf = CreateFile(pszFile, 
                   GENERIC_READ | GENERIC_WRITE, 
                   (DWORD) 0, 
                    NULL, 
                   CREATE_ALWAYS, 
                   FILE_ATTRIBUTE_NORMAL, 
                   (HANDLE) NULL); 
    if (hf == INVALID_HANDLE_VALUE) 
	{
		// Free memory. 
		GlobalFree((HGLOBAL)lpBits);
		ReleaseDC(0, hDC);
		return false;
	}
    hdr.bfType = 0x4d42;        // 0x42 = "B" 0x4d = "M" 
    // Compute the size of the entire file. 
    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
                 pbih->biSize + pbih->biClrUsed 
                 * sizeof(RGBQUAD) + pbih->biSizeImage); 
    hdr.bfReserved1 = 0; 
    hdr.bfReserved2 = 0; 

    // Compute the offset to the array of color indices. 
    hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
                    pbih->biSize + pbih->biClrUsed 
                    * sizeof (RGBQUAD); 

    // Copy the BITMAPFILEHEADER into the .BMP file. 
    if (!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER), 
        (LPDWORD) &dwTmp,  NULL)) 
    {
		// Free memory. 
		GlobalFree((HGLOBAL)lpBits);
		ReleaseDC(0, hDC);
		return false;
    }

    // Copy the BITMAPINFOHEADER and RGBQUAD array into the file. 
    if (!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER) 
                  + pbih->biClrUsed * sizeof (RGBQUAD), 
                  (LPDWORD) &dwTmp, ( NULL))) 
	{
		// Free memory. 
		GlobalFree((HGLOBAL)lpBits);
		ReleaseDC(0, hDC);
		return false;
	}

    // Copy the array of color indices into the .BMP file. 
    dwTotal = cb = pbih->biSizeImage; 
    hp = lpBits; 
    if (!WriteFile(hf, (LPSTR) hp, (int) cb, (LPDWORD) &dwTmp,NULL)) 
	{
		// Free memory. 
		GlobalFree((HGLOBAL)lpBits);
		ReleaseDC(0, hDC);
		return false;
	}

    // Close the .BMP file. 
	CloseHandle(hf);

    // Free memory. 
    GlobalFree((HGLOBAL)lpBits);
	ReleaseDC(0, hDC);
	return true;
}
