/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "vcFile.h" // Class definition

/******************************************************************************
   Function:    CFileDisk::CFileDisk
   Purpose:     Default constructor
   Returns:     Void
   Notes:       Sets disk file name to empty string. To do anything
				interesting (open, read, write, etc), you must first
				call SetFilePathname().
******************************************************************************/
 CFileDisk::CFileDisk()
{
	/* Initialize member variables.	 */
	m_pDiskFile = 0;
	m_CurrentActivity = aNone;
	m_iErrorCode = EID_NO_ERROR;
}

/******************************************************************************
   Function:    CFileDisk::CFileDisk
   Purpose:     Constructor that sets the name of the file on disk to read/write.
   Returns:     Void
   Parameters:  LPCTSTR szFilePathname - Pathname to disk file to read/write.
******************************************************************************/
#ifndef _UNICODE
 CFileDisk::CFileDisk(LPCTSTR szFilePathname)
{
	m_iErrorCode = EID_NO_ERROR;

	/* Set the pathname and initialize remaining variables.	 */
	m_strFilename = szFilePathname;
	m_pDiskFile = 0;//INVALID_HANDLE_VALUE;
	m_CurrentActivity = aNone;
}
#endif

CFileDisk::CFileDisk(const wchar_t* szFilePathname)
{
	m_iErrorCode = EID_NO_ERROR;

	/* Set the pathname and initialize remaining variables.	 */
	m_strFilenameW = szFilePathname;
	m_pDiskFile = 0;//INVALID_HANDLE_VALUE;
	m_CurrentActivity = aNone;
}

/******************************************************************************
   Function:    CFileDisk::~CFileDisk
   Purpose:     Destructor.
   Returns:     Void
******************************************************************************/
 CFileDisk::~CFileDisk()
{
	/* Validate input:
	 *        - the file should not be open	 */

	/* If the file has not yet been closed, close it (this really should not happen).	 */
	if (m_pDiskFile != 0/*INVALID_HANDLE_VALUE*/)
	{
		fclose (m_pDiskFile);
		m_pDiskFile = 0;
		m_iErrorCode = EID_NO_ERROR;
	}

	m_CurrentActivity = aNone;
}

/******************************************************************************
   Function:    CFileDisk::SetFilePathname
   Purpose:     Set file pathname.
   Returns:     Void
   Parameters:  LPCTSTR szFilePathname - Pathname to set.
   Throws:		CBException object.
   Notes:       - SetFilePathname() should only be called when the pathname
				  has not yet been set. It allows the caller to use
				  the default constructor (which does not set a pathname),
				  and then call SetFilePathname() to set the pathname.
******************************************************************************/
 void CFileDisk::SetFilePathname(LPCTSTR szFilePathname)
{
	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we must not have a file pathname yet
	 *      - the input string must be small enough for us to hold it	 */
	if (!m_strFilename.empty())
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return;
	}

	/* Set the pathname. */
#ifndef _UNICODE
	m_strFilename = szFilePathname;
#else
	m_strFilenameW = szFilePathname;
#endif

	return;
}

/******************************************************************************
   Function:    CFileDisk::GetFilePathname
   Purpose:     Get the file pathname.
   Returns:     LPCTSTR - File pathname.
   Notes:       
******************************************************************************/
 LPCTSTR CFileDisk::GetFilePathname() const
{
	/* Return pathname.	 */
#ifndef _UNICODE
	return m_strFilename.c_str();
#else
	return m_strFilenameW.c_str();
#endif
}

/******************************************************************************
   Function:    CFileDisk::Open
   Purpose:     Open file.
   Returns:     Void
   Parameters:  OpenReason ReasonForOpening - Indicates why the file is to be opened.
   Throws:		CBException object.
   Notes:       
******************************************************************************/
 void CFileDisk::Open(const OpenReason ReasonForOpening)
{
	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we must have a pathname
	 *      - we must not already be open
	 *      - we must not have a valid handle
	 */

	if ((m_strFilename.empty() && m_strFilenameW.empty()) ||
		!(m_CurrentActivity == aNone) ||
		!(m_pDiskFile == 0/*INVALID_HANDLE_VALUE*/))
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return;
	}

	errno_t err = 0;
	/* Open the underlying disk file according to caller's wishes.	 */
	switch (ReasonForOpening)
	{
	case orOpenForRead:
		{
			err = m_strFilenameW.empty() ? fopen_s(&m_pDiskFile, m_strFilename.c_str(), "rb") : 
										 _wfopen_s(&m_pDiskFile, m_strFilenameW.c_str(), L"rb");
			m_CurrentActivity = aReading;
		}
		break;
	case orOpenForOverwrite:
		{
			err = m_strFilenameW.empty() ? fopen_s(&m_pDiskFile, m_strFilename.c_str(), "wb+") : 
										 _wfopen_s(&m_pDiskFile, m_strFilenameW.c_str(), L"wb+");
			m_CurrentActivity = aWriting;
		}
		break;
	case orOpenForReadWrite:
		{
			err = m_strFilenameW.empty() ? fopen_s(&m_pDiskFile, m_strFilename.c_str(), "rb+") : 
						  				 _wfopen_s(&m_pDiskFile, m_strFilenameW.c_str(), L"rb+");
			m_CurrentActivity = aReadingWriting;
		}
		break;
	default:
		m_pDiskFile = 0;
		break;
	}

	// Raise an exception if we were unable to open the file.
	if (err || m_pDiskFile == 0)
	{
		m_CurrentActivity = aNone;

		// Set error code depending on what we were attempting.
		if (ReasonForOpening == orOpenForRead)
			m_iErrorCode = EID_OPEN_FILE_ERROR;
		else
			m_iErrorCode = EID_CREATE_FILE_ERROR;
	}

	return;
}

/******************************************************************************
   Function:    CFileDisk::Close
   Purpose:     Close the file.
   Returns:     Void
******************************************************************************/
 void CFileDisk::Close()
{

	/* If the file handle is valid, close it. */
	if (m_pDiskFile != 0)
	{
		fclose( m_pDiskFile );
		m_pDiskFile = 0;
	}

	/* No current activity.	 */
	m_CurrentActivity = aNone;

	return;
}

/******************************************************************************
   Function:    CFileDisk::Delete
   Purpose:     Delete file.
   Returns:     Void
******************************************************************************/
 void CFileDisk::Delete()
{
	m_iErrorCode = EID_NO_ERROR;

	/* Validate input.	 */
	if (m_strFilename.empty() && m_strFilenameW.empty())
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return;
	}

	/* Close the file and delete it.	 */
	Close();
	BOOL bRes = m_strFilenameW.empty() ? DeleteFileA(m_strFilename.c_str()) : DeleteFileW(m_strFilenameW.c_str());
	if (!bRes)
	{
		m_iErrorCode = EID_DELETE_ERROR;
	}
}

/******************************************************************************
   Function:    CFileDisk::Read
   Purpose:     Read from the file.
   Returns:     UINT - Number of bytes actually read from file (may be less than uCount).
   Parameters:  PVOID pBuffer - Buffer in which to return data read from file.
                UINT uCount - Number of bytes to read from file.
				bool bReadExact - true if an error should be raised if less than uCount 
								  bytes are read.
   Throws:      CBException object.
   Notes:       
******************************************************************************/
 DWORD CFileDisk::Read(PVOID pBuffer, DWORD dwCount, bool bReadExact)
{
	DWORD dwBytesActuallyRead = 0;	// Count of bytes read from file
	bool bReadSuccessful = false;	// true if successfully read file, false otherwise
	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we should be open for reading
	 *      - we should have a valid handle to read
	 */

	bool bReadingOrWriting = ((m_CurrentActivity == aReadingWriting) || (m_CurrentActivity == aReading));

	if (!bReadingOrWriting || m_pDiskFile == 0 || pBuffer == 0)
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return 0;
	}

	/* Read from the file and check for errors.	 */
	bReadSuccessful = fread( pBuffer, 1, dwCount, m_pDiskFile) != 0;
	dwBytesActuallyRead = (DWORD)dwCount;

	if ((dwBytesActuallyRead == 0 && dwCount > 0) || (!bReadSuccessful && GetLastError() == ERROR_HANDLE_EOF))
	{
		/* End of file is not an error.		 */
		bReadSuccessful = true;
	}
	if (!bReadSuccessful || bReadExact && (dwCount != dwBytesActuallyRead))
	{
		m_iErrorCode = EID_READ_ERROR;
		return 0;
	}

	/* Return the number of bytes that were read from the file.	 */
	return dwBytesActuallyRead;
}

bool CFileDisk::Eof()
{
	if (m_pDiskFile == 0)
		return true;

	if(m_CurrentActivity != aReading)
		return false;

	return feof(m_pDiskFile) ? true : false;
}

/******************************************************************************
   Function:    CFileDisk::Write
   Purpose:     Write to file.
   Returns:     Void
   Parameters:  const PVOID pBuffer - Buffer containing data to write to file.
                DWORD dwCount - Size of buffer.
   Throws:		CBException object.
   Notes:       
******************************************************************************/
 void CFileDisk::Write(const PVOID pBuffer, const DWORD dwCount)
{ 
	bool  bWriteSuccessful = false;	// true if file successfully written, false otherwise
	DWORD dwBytesActuallyWritten = 0;	// Number of bytes actually written to file

	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we should be open for writing
	 *      - we should have a valid file handle to which to write
	 */
	bool bReadingOrWriting = ((m_CurrentActivity == aReadingWriting) || (m_CurrentActivity == aWriting));

	if (!bReadingOrWriting || (m_pDiskFile == 0) || pBuffer == NULL)
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return;
	}
	/* Write the file and check for errors.	 */
	bWriteSuccessful = fwrite(pBuffer, dwCount, 1, m_pDiskFile) != 0;
	dwBytesActuallyWritten = dwCount;

	if (!bWriteSuccessful || (dwBytesActuallyWritten != dwCount))
		m_iErrorCode = EID_WRITE_ERROR;

	return;
}

/******************************************************************************
   Function:    CFileDisk::FlushFile
   Purpose:     Flushes file to disk.
   Returns:     Void
   Throws:		CBException object.
   Notes:       
******************************************************************************/
 void CFileDisk::Flush()
{
	bool bFlushSuccessful = false;	// true if file successfully flushed, false otherwise

	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we should be open for writing
	 *      - we should have a valid file handle to flush	 */
	bool bReadingOrWriting = ((m_CurrentActivity == aReadingWriting) || (m_CurrentActivity == aWriting));

	if (!bReadingOrWriting || (m_pDiskFile == 0))
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return;
	}

	/* Flush the file to disk and check for errors.	 */
	bFlushSuccessful = fflush(m_pDiskFile) != 0;

	if (!bFlushSuccessful)
		m_iErrorCode = EID_WRITE_ERROR;

	return;
}

 /******************************************************************************
    Function:	CFileDisk::GetFileSize
    Author:		Paul Gruzdev
    Purpose:    get the file size, static function
    Returns:		LONG
    Parameters:	FILE * f  
    Notes:       
 ******************************************************************************/
 LONG CFileDisk::GetFileSize(FILE* f)
{
	LONG lInitial = ftell(f);
	fseek(f, 0, SEEK_END); // seek to end of file
	LONG size = ftell(f); // get current file pointer
	fseek(f, lInitial, SEEK_SET); // seek back to beginning of file
	return size;
}

/******************************************************************************
   Function:    CFileDisk::GetFileSize
   Purpose:     Get size of file.
   Returns:     DWORD - Size of file.
   Throws:		CBException object.
   Notes:       
******************************************************************************/
DWORD CFileDisk::GetFileSize()
{
	DWORD dwFileSize = 0;		// File size

	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we should be doing something (ie, reading or writing)
	 *      - we should have a valid file handle
	 */
	if ((m_CurrentActivity == aNone) || (m_pDiskFile == 0))
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return 0;
	}

	/* Get the size of the file and check for an error.	 */
	fseek( m_pDiskFile, 0, SEEK_END);
	dwFileSize = ftell(m_pDiskFile);
	fseek( m_pDiskFile, 0, SEEK_SET);

	if (dwFileSize == FILESIZE_ERROR)
	{
		m_iErrorCode = EID_FILESIZE_ERROR;
		return 0;
	}

	/* Return filesize to caller.	 */

	return dwFileSize;
}

/******************************************************************************
   Function:    CFileDisk::Seek
   Purpose:     Move to the specified position in the file.
   Returns:     DWORD - Current position after the Seek() is performed
   Parameters:  LONG lDistance - Distance to Seek() (can be positive or negative).
                OriginForSeek eFrom - Origin from whence to seek (beginning, current, end).
   Notes:       
******************************************************************************/
DWORD CFileDisk::Seek(LONG lDistance, OriginForSeek eFrom)
{
	DWORD dwNewPosition = 0L;	// Current position after Seek() performed

	m_iErrorCode = EID_NO_ERROR;

	/* Validate input:
	 *        - we should be doing something (ie, reading or writing)
	 *      - we should have a valid file handle
	 */
	if ((m_CurrentActivity == aNone) || (m_pDiskFile == 0))
	{
		m_iErrorCode = EID_INVALID_PARAMETER_PASSED;
		return 0;
	}

	/* Seek based on specified seek origin and check for error.	 */
	switch (eFrom)
	{
	case ofsBegin:
		fseek(m_pDiskFile, lDistance, SEEK_SET);
		dwNewPosition = ftell(m_pDiskFile);
		break;
	case ofsCurrent:
		fseek(m_pDiskFile, lDistance, SEEK_CUR);
		dwNewPosition = ftell(m_pDiskFile);
		break;
	case ofsEnd:
		fseek(m_pDiskFile, lDistance, SEEK_END);
		dwNewPosition = ftell(m_pDiskFile);
		break;
	default:
		dwNewPosition = FILESIZE_ERROR;
		break;
	}
	if (dwNewPosition == FILESIZE_ERROR)
	{
		m_iErrorCode = EID_FILESIZE_ERROR;
		return 0;
	}

	/* Return new file position. */
	return dwNewPosition;
}