/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "vcFile.h"
#include "errorlib\bexception.h"

#ifdef _DEBUG
#undef	THIS_FILE
static	char THIS_FILE[] = __FILE__;
#ifndef DEBUG_NEW
#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
#endif // DEBUG_NEW
#define new DEBUG_NEW
#endif // _DEBUG

/******************************************************************************
   Function:    CFileMemory::ReAllocate
   Author:      Paul Gruzdev
   Purpose:     Allocates or ReAllocates the appropriate memory block type as needed
   Returns:     BOOL - True for success
   Parameters:  DWORD dwNewSize - Number of bytes needed (actual allocation will vary
								  based on granularity)
   Notes:       
******************************************************************************/
BOOL CFileMemory::ReAllocate(DWORD dwNewSize) 
{
	DWORD		dwGranularSize;
	PBYTE		pNewBlock;

	switch(m_Mode)
	{
		case moNormal: // Normal internal heap managed memory block
			dwGranularSize = ((dwNewSize / m_dwGranularity) + 1) * m_dwGranularity;
			pNewBlock = (PBYTE)realloc(m_pMemoryBuffer, dwGranularSize);
			if (!pNewBlock)
				return FALSE;
			m_pMemoryBuffer = pNewBlock;
			m_dwMemoryAllocated = dwGranularSize;
			break;
		case moSharedPointerExternal: // Can't realloc an external pointer
			return FALSE;
			
		default:
			ATLASSERT("Other modes are not implemented");
	}

	return TRUE;
}

/******************************************************************************
   Function:    CFileMemory::CFileMemory
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  none
   Notes:       
******************************************************************************/
CFileMemory::CFileMemory()
{
	m_pMemoryBuffer = NULL;
	m_dwMemorySize = 0;
	m_dwMemoryPosition = 0;
	m_dwMemoryAllocated = 0;
	m_dwMemoryOrigin = 0;
	m_dwGranularity = DEFAULT_GRANULARITY;
	m_Reason = orOpenNone;
	m_Mode = moNormal;
}

/******************************************************************************
   Function:    CFileMemory::CFileMemory
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  none
   Notes:       
******************************************************************************/
CFileMemory::CFileMemory(const OpenReason ReasonForOpening)
{
	m_pMemoryBuffer = NULL;
	m_dwMemorySize = 0;
	m_dwMemoryPosition = 0;
	m_dwMemoryAllocated = 0;
	m_dwMemoryOrigin = 0;
	m_dwGranularity = DEFAULT_GRANULARITY;
	m_Reason = ReasonForOpening;
	m_Mode = moNormal;
}


/******************************************************************************
   Function:    CFileMemory::~CFileMemory
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  - 
   Notes:       
******************************************************************************/
CFileMemory::~CFileMemory()
{
	Delete();
}

/******************************************************************************
   Function:    CFileMemory::Open
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  const OpenReason ReasonForOpening - 
   Notes:       
******************************************************************************/
#pragma warning(disable:4100)
void CFileMemory::Open(const OpenReason ReasonForOpening)
{
	if (m_Reason != orOpenNone) {
		throw CBException(EID_INVALID_STATE);
	}
	m_dwMemoryPosition = m_dwMemoryOrigin;
	m_Reason = ReasonForOpening;
}
#pragma warning(default:4100)

/******************************************************************************
   Function:    CFileMemory::Write
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  const PVOID pBuffer - 
                const DWORD dwCount - 
   Notes:       
******************************************************************************/
void CFileMemory::Write(const PVOID pBuffer, const DWORD dwCount)
{
	DWORD	dwMinSizeRequired = m_dwMemoryPosition + dwCount;

	if (m_Reason != orOpenForReadWrite && m_Reason != orOpenForOverwrite) {
		throw CBException(EID_INVALID_STATE);
	}

	/* If we need more space to write to, reallocate our buffer
	 */
	if (dwMinSizeRequired > m_dwMemoryAllocated)
		if (!ReAllocate(dwMinSizeRequired))
			throw CBException(EID_MEMORY_ERROR);

	/* Copy the new data into place.
	 */
	memcpy((PCHAR) m_pMemoryBuffer + m_dwMemoryPosition, pBuffer, dwCount);

	/* Update the memory used counter
	 */
	if (dwMinSizeRequired > m_dwMemorySize)
	{
		m_dwMemorySize = dwMinSizeRequired;
	}

	/* Update the position.
	 */
	m_dwMemoryPosition += dwCount;

}

/******************************************************************************
   Function:    CFileMemory::Flush
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  None
   Notes:       
******************************************************************************/
void CFileMemory::Flush()
{
	//nothing to do
}

/******************************************************************************
   Function:    CFileMemory::Read
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     DWORD
   Parameters:  PVOID pBuffer - 
                DWORD dwCount - 
                bool bReadExact (= false) - 
   Notes:       
******************************************************************************/
DWORD CFileMemory::Read(PVOID pBuffer, DWORD dwCount, bool bReadExact /*= false*/ )
{

	if (m_Reason != orOpenForRead && m_Reason != orOpenForReadWrite) {
		throw CBException(EID_INVALID_STATE);
	}
	
	if (!m_pMemoryBuffer)
	{
		CBException BException(EID_MEMORY_ERROR);		// Exception class

		throw BException;
	}

	DWORD dwAmountToCopy;

	if (dwCount + m_dwMemoryPosition > m_dwMemorySize)
		dwAmountToCopy = m_dwMemorySize - m_dwMemoryPosition;
	else
		dwAmountToCopy = dwCount;

	memcpy(pBuffer, (PCHAR) m_pMemoryBuffer + m_dwMemoryPosition, dwAmountToCopy);
	m_dwMemoryPosition += dwAmountToCopy;

	if (bReadExact && (dwAmountToCopy != dwCount))
	{
		CBException BException(EID_READ_ERROR);		// Exception class

		throw BException;
	}

	return dwAmountToCopy;
}

bool CFileMemory::Eof()
{
	if (m_Reason != orOpenForRead)
	{
		throw CBException(EID_INVALID_STATE);
	}

	if (!m_pMemoryBuffer)
	{
		CBException BException(EID_MEMORY_ERROR);		// Exception class
		throw BException;
	}

	return m_dwMemoryPosition >= m_dwMemorySize;
}

/******************************************************************************
   Function:    CFileMemory::Close
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  None
   Notes:       
******************************************************************************/
void CFileMemory::Close()
{
	m_dwMemoryPosition = m_dwMemoryOrigin;
	m_Reason = orOpenNone;
}

/******************************************************************************
   Function:    CFileMemory::Delete
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     void
   Parameters:  None
   Notes:       
******************************************************************************/
void CFileMemory::Delete()
{
	switch(m_Mode)
	{
		case moNormal: // Normal internal heap managed memory block
			if (m_pMemoryBuffer)
			{
				m_dwMemorySize = 0;
				m_dwMemoryPosition = 0;
				m_dwMemoryAllocated = 0;
				m_dwMemoryOrigin = 0;
				if(m_pMemoryBuffer)
					free(m_pMemoryBuffer);
				m_pMemoryBuffer = NULL;
			}
			break;
		case moSharedPointerExternal: // External pointers to NOT get deleted
			m_dwMemorySize = 0;
			m_dwMemoryPosition = 0;
			m_dwMemoryOrigin = 0;
			m_dwMemoryAllocated = 0;
			m_pMemoryBuffer = NULL;
			break;

	}

	m_Reason = orOpenNone;
}

/******************************************************************************
   Function:    CFileMemory::GetFileSize
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     DWORD
   Parameters:  None
   Notes:       
******************************************************************************/
DWORD CFileMemory::GetFileSize()
{
	if (m_Reason == orOpenNone) { 
		throw CBException(EID_INVALID_STATE);
	}

	return m_dwMemorySize - m_dwMemoryOrigin;
}


/******************************************************************************
   Function:    CFileMemory::Seek
   Author:      Paul Gruzdev
   Purpose:     
   Returns:     DWORD
   Parameters:  LONG lDistance - 
                OriginForSeek eFrom - 
   Notes:       
******************************************************************************/
DWORD CFileMemory::Seek(LONG lDistance, OriginForSeek eFrom)
{

	CBException BException(EID_NO_ERROR);

	if (m_Reason == orOpenNone) { 
		throw CBException(EID_INVALID_STATE);
	}

	switch (eFrom)
	{
	case (ofsBegin):
		if (lDistance < 0)
		{
			m_dwMemoryPosition = FILESIZE_ERROR;
		}
		else
		{
			m_dwMemoryPosition = lDistance + m_dwMemoryOrigin;

			/* If we sought past the end of memory, grow the size of the memory.
			 */
			if (m_dwMemoryPosition > m_dwMemoryAllocated)
			{
				if (!ReAllocate(m_dwMemoryPosition))
					throw CBException(EID_MEMORY_ERROR);
			}
		}

		break;

	case (ofsCurrent):
		if (lDistance + (long) m_dwMemoryPosition < 0)
		{
			m_dwMemoryPosition = FILESIZE_ERROR;
		}
		else
		{
			m_dwMemoryPosition += lDistance;
			if (m_dwMemoryPosition > m_dwMemoryAllocated)
			{
				if (!ReAllocate(m_dwMemoryPosition))
					throw CBException(EID_MEMORY_ERROR);
			}
		}

		break;

	case (ofsEnd):
		if ((lDistance + (long) m_dwMemorySize < 0) || (lDistance > 0))
			m_dwMemoryPosition = FILESIZE_ERROR;
		else
			m_dwMemoryPosition = m_dwMemorySize + lDistance;

		break;

	default:
		m_dwMemoryPosition = FILESIZE_ERROR;
		break;
	}

	if (m_dwMemoryPosition == FILESIZE_ERROR)
	{
		BException.SetErrorCode(EID_FILESIZE_ERROR);
		throw BException;
	}

	return m_dwMemoryPosition - m_dwMemoryOrigin;
}

/******************************************************************************
   Function:    CFileMemory::GetTempMemPtr
   Author:      Paul Gruzdev
   Purpose:		Allow access to the memory to avoid unneccessary copying of memory.
   Returns:     BYTE* - The memory buffer.  
   Parameters:  None
   Notes:       THIS FUNCTION IS LIKELY TO BE REMOVED IN FAVOR OF A FUTURE HANDLE IMPLEMENTATION
				The pointer will change with each reallocation so do not hold on to it
				after making a write/fluch/etc. call on the file object.
******************************************************************************/
BYTE* CFileMemory::GetTempMemPtr()
{
    if (m_Reason == orOpenNone)
		throw CBException(EID_INVALID_STATE);

	if (!m_pMemoryBuffer)
		throw CBException(EID_INVALID_STATE);

	return (BYTE*)m_pMemoryBuffer + m_dwMemoryOrigin;
}

/******************************************************************************
   Function:    CFileMemory::SetSize
   Author:      Paul Gruzdev
   Purpose:     Sets the size of the memory buffer for subsequent operations.
   Returns:     void
   Parameters:  DWORD dwCount - the desired size of the memory buffer.  This must be larger than the current seek pointer .
   Notes:       The size passed in will be the size of the buffer MINUS the
				memory origin.   
				Throws EID_INVALID_STATE and EID_MEMORY_ERROR CBExceptions on error.
******************************************************************************/
void CFileMemory::SetSize(DWORD dwCount)
{
	DWORD	dwMinSizeRequired = dwCount + m_dwMemoryOrigin;

	if (m_Reason == orOpenNone) { 
		throw CBException(EID_INVALID_STATE);
	}

	// Is there any work to do?
	if (m_dwMemorySize == dwCount) 
		return;

	// Is the new size smaller than our current position?
	if (dwMinSizeRequired < m_dwMemoryPosition)
		throw CBException(EID_INVALID_STATE);

	// If we need more space to write to, allocate it and check for error.
	if (dwMinSizeRequired > m_dwMemoryAllocated)
	{
		// We can't grow an external block
		if (m_Mode == moSharedPointerExternal)
			throw CBException(EID_INVALID_STATE);

		if (!ReAllocate(dwMinSizeRequired))
			throw CBException(EID_MEMORY_ERROR);
	}

	m_dwMemorySize = dwMinSizeRequired;
}


/******************************************************************************
   Function:    CFileMemory::SetGranularity
   Author:      Paul Gruzdev
   Purpose:     Resets the granularity of the allocations
   Returns:     void
   Parameters:  DWORD dwSize - the desired size of the memory allocation blocks.
   Notes:       
******************************************************************************/
void CFileMemory::SetGranularity(DWORD dwSize)
{
	if (!dwSize)
		throw CBException(EID_INVALID_STATE);
	m_dwGranularity = dwSize;

}

/******************************************************************************
   Function:    CFileMemory::SetPointerMemoryMode
   Author:      Paul Gruzdev
   Purpose:     Attaches a pointer to this mem file object
   Returns:     BOOL - TRUE for success, FALSE otherwise
   Parameters:  BYTE* pbyMemory - the pointer to attach to
                DWORD dwMemorySize - the size of the memory block pointed to by pbyMemory
   Notes:       This function can throw exceptions of type BErrorCode
******************************************************************************/
BOOL CFileMemory::SetPointerMemoryMode(BYTE*	pbyMemory, DWORD dwMemorySize)
{
	if(m_Reason != orOpenNone) // can only be done on closed memory file
		throw EID_INVALID_STATE;

	if(!pbyMemory) // must have pointer to memory
		throw EID_INVALID_PARAMETER_PASSED;
	if(!dwMemorySize) // must have some actual size
		throw EID_INVALID_PARAMETER_PASSED;
	
	m_pMemoryBuffer = pbyMemory;
	m_dwMemoryAllocated = dwMemorySize;
	m_dwMemorySize = dwMemorySize;
	m_dwMemoryOrigin = 0;

	m_Mode = moSharedPointerExternal;

	return TRUE;
}

/******************************************************************************
   Function:    CFileMemory::Detach
   Author:      Paul Gruzdev
   Purpose:     Detaches and returns an internal allocated memory block
   Returns:     LPVOID
   Parameters:  void
   Notes:       
******************************************************************************/
LPVOID CFileMemory::Detach() 
{
	Close();
	LPVOID lpBuf = m_pMemoryBuffer;
	m_pMemoryBuffer = 0;
	Delete();

	return lpBuf;
}


/******************************************************************************
   Function:    CFileMemory::SetMemoryOrigin()
   Author:      Paul Gruzdev
   Purpose:     Sets the logical start (offset in bytes) of the memory block
   Returns:     BOOL - True for success
   Parameters:  
   Notes:       
******************************************************************************/
BOOL CFileMemory::SetMemoryOrigin(DWORD dwOrigin)
{
	if (m_Reason != orOpenNone)		// Can only be done on closed memory file
		throw CBException(EID_INVALID_STATE);

	if(m_Mode == moSharedPointerExternal)
		if(dwOrigin >= m_dwMemorySize)
			throw EID_INVALID_PARAMETER_PASSED;

	m_dwMemoryOrigin = dwOrigin;

	return TRUE;
}
