/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "StdAfx.h"
#include "common/vcFile.h"
#include "common/unzip.h"
#include "common/HiResolutionTimer.h"
#include "common/BigEndian.h"
#include "FontMapper.h"
#include <iterator>
#include <math.h>
#include <iostream>
#include <iomanip>
#include "Ngram.h"

//#define FONTMAPPER_MATCHING_DEBUG
std::set<std::wstring> g_sFonts;

#define UNI(x)      L ## x

wchar_t* g_pszFontTable [] [2] = 
{
	{UNI("Arial"), UNI("ArialMT")}, 
	{UNI("Arial Black"), UNI("Arial-Black")},
	{UNI("Arial Narrow"), UNI("ArialNarrow")},
	{UNI("Book Antiqua"), UNI("BookAntiqua")},
	{UNI("Bookman Old Style"), UNI("BookmanOldStyle")},
	{UNI("Bookshelf Symbol 7"), UNI("BookshelfSymbolSeven")},
	{UNI("Century Gothic"), UNI("CenturyGothic")},
	{UNI("Comic Sans MS"), UNI("ComicSansMS")},
	{UNI("Courier New"), UNI("CourierNewPSMT")},
	{UNI("Estrangelo Edessa"), UNI("EstrangeloEdessa")},
	{UNI("Franklin Gothic Medium"), UNI("FranklinGothic-Medium")},
	{UNI("Geotype TT"), UNI("GeotypeTT")},
	{UNI("ITC Zapf Dingbats"), UNI("ZapfDingbats")},
	{UNI("Lucida Console"), UNI("LucidaConsole")},
	{UNI("Lucida Sans Unicode"), UNI("LucidaSansUnicode")},
	{UNI("Mangal"), UNI("Mangal-Regular")},
	{UNI("Microsoft Sans Serif"), UNI("MicrosoftSansSerif")},
	{UNI("Monotype Corsiva"), UNI("MonotypeCorsiva")},
	{UNI("Myriad Pro"), UNI("MyriadPro")},
	{UNI("Myriad Pro"), UNI("MyriadPro-Regular")},
	{UNI("Myriad Pro Cond"), UNI("MyriadPro-Cond")},
	{UNI("MS Reference Sans Serif"), UNI("MSReferenceSansSerif")},
	{UNI("MS Reference Specialty"), UNI("MSReferenceSpecialty")},
	{UNI("MV Boli"), UNI("MVBoli")},
	{UNI("OCR-A BT"), UNI("OCRAbyBT-Regular")},
	{UNI("OCR-B 10 BT"), UNI("OCRB10PitchBT-Regular")},
	{UNI("Palatino Linotype"), UNI("PalatinoLinotype-Roman")},
	{UNI("Times New Roman"), UNI("TimesNewRomanPSMT")},
	{UNI("Trebuchet MS"), UNI("TrebuchetMS")},
	{UNI("Tunga"), UNI("Tunga-Regular")},
	{UNI("Virtual DJ"), UNI("VirtualDJ")},
	{UNI("Wingdings"), UNI("Wingdings-Regular")},
	{UNI("Wingdings 2"), UNI("Wingdings2")},
	{UNI("Wingdings 3"), UNI("Wingdings3")},
	{UNI("WP CyrillicA"), UNI("WP-CyrillicA")},
	{UNI("WP CyrillicB"), UNI("WP-CyrillicB")},
	{NULL, NULL}
};

#define SWAPWORD(x) MAKEWORD(HIBYTE(x), LOBYTE(x))
#define SWAPLONG(x) MAKELONG(SWAPWORD(HIWORD(x)), SWAPWORD(LOWORD(x)))

//Header of names table
typedef struct _tagTT_NAME_TABLE_HEADER
{
	USHORT uFSelector; //format selector. Always 0
	USHORT uNRCount; //Name Records count
	USHORT uStorageOffset; //Offset for strings storage, from start of the table
} TT_NAME_TABLE_HEADER;

//Record in names table
typedef struct _tagTT_NAME_RECORD
{
	USHORT uPlatformID;
	USHORT uEncodingID;
	USHORT uLanguageID;
	USHORT uNameID;
	USHORT uStringLength;
	USHORT uStringOffset; //from start of storage area
} TT_NAME_RECORD;

/******************************************************************************
   Function:	CFontMapper::TypefacePresentInTheSystem
   Author:		Paul Gruzdev
   Purpose:     Check whether specific font typeface name is present in the system.
				We need this because Camelot assigns a system substitute not for
				the exact variant (name+weight+it), but for the entire typeface.
   Returns:		bool
   Parameters:	std::wstring strTest  
   Notes:       
******************************************************************************/
bool CFontMapper::TypefacePresentInTheSystem(std::wstring fontKey)
{
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);
	return g_sFonts.find(fontKey) != g_sFonts.end();
}

FontMapData::FontMapData(const std::wstring &strTypeface, const std::wstring &strPostScriptName, int iWeight, bool bItalic)
{
	m_strKey = strTypeface;
	std::transform(m_strKey.begin(), m_strKey.end(), m_strKey.begin(), std::tolower);
	m_strPostScriptName = strPostScriptName;
	m_strTypeface = strTypeface;
	m_iWeight = iWeight;
	m_bItalic = bItalic;
}

void AddFont(CEnumFontData* pefd, const std::wstring &strTypeface, int iWeight, bool bItalic, const std::wstring &strPostScriptName, CCPanose &panose)
{
	// For the new PDF fuzzy font comparison we definitely need all font variants including the direct mapping
// 	if(strTypeface == strPostScriptName)
// 		return false;
	if(!pefd) return;
	FontMapData dta(strTypeface, strPostScriptName, iWeight, bItalic);
	dta.m_panose = panose;
	FontMapDataSet* pv = pefd->pfmds;
	EnumOrderList *peol = pefd->peol;
	CFontMap* pm = pefd->pfm;
	CFontWeightMap* pfw = pefd->pfw;

	if(pv && pv->find(dta) == pv->end())
	{
//wprintf(L"%s\t\t%d\t\t%s\n", strTypeface.c_str(), iWeight, bItalic ? L"true" : L"false");
//		std::wcout << L"FONT: " << std::setw(40) << strTypeface << L' ' << std::setw(3) << iWeight << L' ' << (bItalic ? L'I' : L'-') << std::endl;
		FontMapDataSet::const_iterator it = pv->insert(dta);
		if (peol) peol->push_back(it);
	}

	std::wstring fontKey = strPostScriptName;
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);

	if(pm && pm->find(fontKey) == pm->end())
	{
		pm->insert(std::pair<std::wstring, FontMapData>(fontKey, dta));
	}

	if (pfw)
		(*pfw)[strTypeface].insert(iWeight);
}

int CALLBACK EnumFamExCallBack(ENUMLOGFONTEXW* lpelfe, NEWTEXTMETRICEXW* lpntme, DWORD FontType, LPARAM lParam)
{
	USES_CONVERSION;
	CEnumFontData* pv = (CEnumFontData*)lParam;

	if((FontType & TRUETYPE_FONTTYPE) && (lpelfe->elfLogFont.lfFaceName[0] != TEXT('@')) || (FontType & DEVICE_FONTTYPE))
	{
		// Paul: another speed enhancement: since we pick a font file based on logfont, check whether we already added the font with the
		// same properties like typeface/weight/italic
		FontMapData dta(lpelfe->elfLogFont.lfFaceName, L"", lpelfe->elfLogFont.lfWeight, lpelfe->elfLogFont.lfItalic ? true : false);
		// Look up our font map data set
		if(pv->pfmds->find(dta) != pv->pfmds->end())
			return 1;

		HFONT hFont = CreateFontIndirectW(&lpelfe->elfLogFont);
		HFONT hOldFont = (HFONT)SelectObject(pv->hdc, hFont);

		CCPanose panose;
		UINT otmSize = GetOutlineTextMetrics(pv->hdc, 0, NULL);
		if (otmSize >= sizeof(OUTLINETEXTMETRIC))
		{
			OUTLINETEXTMETRIC *potm = (OUTLINETEXTMETRIC *)new BYTE[otmSize];

			ZeroMemory(potm, otmSize);
			if (GetOutlineTextMetrics(pv->hdc, otmSize, potm) >= sizeof(OUTLINETEXTMETRIC))
			{
				panose = potm->otmPanoseNumber;
			}

			delete [] potm;
		}

		try
		{
			if(lpntme->ntmTm.ntmFlags & NTM_TYPE1)
			{
				DWORD sizeOfFontData = GetFontData(pv->hdc, 0, 0, 0, 0);

				if(sizeOfFontData == -1 || !sizeOfFontData)
					throw(false);

				PBYTE pbyData = new BYTE[sizeOfFontData];
				const char* pbyPtr = (const char*)pbyData;

				if(GetFontData(pv->hdc, 0, 0, pbyData, sizeOfFontData)== GDI_ERROR)
					throw(false);

				char* pszMagic = "PostScript";
				bool bFound = false;
				
				for(;(PBYTE)pbyPtr < pbyData + sizeOfFontData; pbyPtr++)
				{
					if(!strncmp((const char*)pbyPtr, pszMagic, strlen(pszMagic)))
					{
						pbyPtr += strlen(pszMagic)+1;
						bFound=true;
						break;
					}
				}

				if(bFound)
				{
					char* pszPS = (char*)pbyPtr + strlen(pbyPtr) + 1;
					AddFont(pv, A2W(pbyPtr), lpelfe->elfLogFont.lfWeight, lpelfe->elfLogFont.lfItalic ? true : false, A2W(pszPS), panose);
				}
				else
				{
	/*
	#ifdef _DEBUG
					FILE* out = fopen("d:/temp/font.type1", "wb");
					if(out)
					{
						fwrite(pbyData, sizeOfFontData, 1, out);
						fclose(out);
					}
	#endif*/
					pszMagic = "!PS-AdobeFont-1.0:";
					pbyPtr = (const char*)pbyData;

					for(;(PBYTE)pbyPtr < pbyData + sizeOfFontData; pbyPtr++)
					{
						if(!strncmp((const char*)pbyPtr, pszMagic, strlen(pszMagic)))
						{
							pbyPtr += strlen(pszMagic)+1;
							bFound=true;
							break;
						}
					}

					if(bFound)
					{
						char* ptr = (char*)pbyPtr;
						for(;*ptr != ' ' && (PBYTE)pbyPtr < pbyData + sizeOfFontData; ptr++);
						if(*ptr == ' ') *ptr = 0;
						char* pszPS = (char*)pbyPtr;
						AddFont(pv, lpelfe->elfLogFont.lfFaceName, lpelfe->elfLogFont.lfWeight, lpelfe->elfLogFont.lfItalic ? true : false, A2W(pszPS), panose);
					}
					else
						std::wcerr << L"Type1 not added: " << lpelfe->elfLogFont.lfFaceName << std::endl;
				}

				delete[] pbyData;
				throw(true);
			}
			//////////////////////////////////////////////////////////////
			char* buf = "name";
			DWORD* pdw = (PDWORD)buf;
			DWORD sizeOfFontData = GetFontData(pv->hdc, *pdw, 0, 0, 0);
			bool bFound = false;

			if (sizeOfFontData == -1 || !sizeOfFontData)
				throw(false);

			PBYTE pbyData = new BYTE[sizeOfFontData];
			PBYTE pbyPtr = pbyData;

			if (GetFontData(pv->hdc, *pdw, 0, pbyData, sizeOfFontData) == GDI_ERROR)
				throw(false);

			TT_NAME_TABLE_HEADER* pHead = (TT_NAME_TABLE_HEADER*)pbyPtr;
			pbyPtr += sizeof(TT_NAME_TABLE_HEADER);
			pHead->uNRCount = SWAPWORD(pHead->uNRCount);
			pHead->uStorageOffset = SWAPWORD(pHead->uStorageOffset);

			for (int i = 0; i < pHead->uNRCount; i++)
			{
				TT_NAME_RECORD* pRecord = (TT_NAME_RECORD*)pbyPtr;
				pbyPtr += sizeof(TT_NAME_RECORD);
				pRecord->uNameID = SWAPWORD(pRecord->uNameID);

				if (pRecord->uNameID == 6) // 6 means the PostScript font name
				{
					pRecord->uStringLength = SWAPWORD(pRecord->uStringLength);
					pRecord->uStringOffset = SWAPWORD(pRecord->uStringOffset);
					PBYTE pbyStringData = pbyData + pHead->uStorageOffset + pRecord->uStringOffset;
					if(pbyStringData[0] == 0) // this is Highly possible a Unicode string
					{
						std::wstring str;
						str.resize(pRecord->uStringLength/2);
						memcpy(&str[0], pbyStringData, pRecord->uStringLength);
						intel_wstr((PSHORT)&str[0], pRecord->uStringLength/2);
						AddFont(pv, lpelfe->elfLogFont.lfFaceName, lpelfe->elfLogFont.lfWeight, lpelfe->elfLogFont.lfItalic ? true : false, str, panose);
						bFound = true;
					}
				}
			}

			#ifdef _DEBUG
			if(!bFound)
				std::wcerr << lpelfe->elfLogFont.lfFaceName << L" font not added because PostScript name not found" << std::endl;
			#endif

			delete[] pbyData;
			throw(true);
		}
		catch (bool& success)
		{
			SelectObject(pv->hdc, hOldFont);
			DeleteObject(hFont);
	/*
			#ifdef _DEBUG
			if(!success)
				wprintf(L"%s font not added, exception is thrown\n", lpelfe->elfLogFont.lfFaceName);
			#endif
	*/
			if(!success && pv->bExitOnException)
				return 0;
		}
	}
	return 1;
}

int CALLBACK GlobalEnumFamExCallBack(ENUMLOGFONTEXW* lpelfe, NEWTEXTMETRICEXW* lpntme, DWORD FontType, LPARAM lParam)
{
	if((FontType & TRUETYPE_FONTTYPE) && (lpelfe->elfLogFont.lfFaceName[0] != TEXT('@')) || (FontType & DEVICE_FONTTYPE))
	{
		_wcslwr(lpelfe->elfLogFont.lfFaceName);
		// Speed-up the enumeration process by not enumerating fonts we once processed
		//if (!g_sFonts.insert(lpelfe->elfLogFont.lfFaceName).second)
		//	return 1;
		if(g_sFonts.find(lpelfe->elfLogFont.lfFaceName) != g_sFonts.end())
			return 1;
		g_sFonts.insert(lpelfe->elfLogFont.lfFaceName);

		LOGFONTW logfont;
		// Setup the logfont structure to look for ANSI character sets in the enumeration
		memset(&logfont, 0, sizeof(LOGFONTW));
		wcsncpy_s(logfont.lfFaceName, LF_FACESIZE, lpelfe->elfLogFont.lfFaceName, LF_FACESIZE-1);

		logfont.lfPitchAndFamily = 0;
		logfont.lfCharSet = DEFAULT_CHARSET;
		// Enumerate looking for TRUE_TYPE fonts and filling fontInfo
		EnumFontFamiliesExW(((CEnumFontData *)lParam)->hdc, &logfont, (FONTENUMPROCW)EnumFamExCallBack, (LPARAM)lParam, DEVICE_FONTTYPE);
	}

	return 1;
}

void EnumAvailableFonts(const std::wstring &fName, CEnumFontData* pParam)
{
	LOGFONTW logfont;
	// Setup the logfont structure to look for default character sets in the enumeration
	memset(&logfont, 0, sizeof(LOGFONTW));
	if (!fName.empty())
		wcsncpy_s(logfont.lfFaceName, LF_FACESIZE, fName.c_str(), LF_FACESIZE - 1);

	g_sFonts.clear();
	logfont.lfCharSet = DEFAULT_CHARSET;
	// Enumerate looking for all type of fonts
	EnumFontFamiliesExW(pParam->hdc, &logfont, (FONTENUMPROCW)GlobalEnumFamExCallBack, (LPARAM)pParam, RASTER_FONTTYPE | DEVICE_FONTTYPE | TRUETYPE_FONTTYPE);
}

void EnumAvailableFontsOnePass(const std::wstring &fName, CEnumFontData* pParam)
{
	LOGFONTW logfont;
	// Setup the logfont structure to look for default character sets in the enumeration
	memset(&logfont, 0, sizeof(LOGFONTW));
	if (!fName.empty())
		wcsncpy_s(logfont.lfFaceName, LF_FACESIZE, fName.c_str(), LF_FACESIZE - 1);

	g_sFonts.clear();
	logfont.lfCharSet = DEFAULT_CHARSET;
	// Enumerate looking for all type of fonts
	EnumFontFamiliesExW(pParam->hdc, &logfont, (FONTENUMPROCW)EnumFamExCallBack, (LPARAM)pParam, RASTER_FONTTYPE | DEVICE_FONTTYPE | TRUETYPE_FONTTYPE);
}

bool SortPredicate(const FontMapData& d1, const FontMapData& d2)
{
	return wcscmp(d1.m_strTypeface.c_str(), d2.m_strTypeface.c_str()) < 0;
}

void SaveFontMap(FontMapDataVector& fdVector, const TCHAR* file_out)
{
	FILE* out = _tfopen(file_out, _T("wt"));
	//TCHAR pszbib[8192]; 
#ifdef _UNICODE
	CFileDisk dsk((std::wstring(file_out) + L".bin").c_str());
#else
	CFileDisk dsk((std::string(file_out) + ".bin").c_str());
#endif
	dsk.Open(vcFile::orOpenForOverwrite);
	dsk.WriteDWord((DWORD)fdVector.size());

	fprintf(out, "Typeface\tWeight\tItalic\tPostScriptName\n");
	fprintf(out, "------------------------------------------------------------------------------------\n");
	for(size_t i = 0;i < fdVector.size(); i++)
	{
		FontMapData& fmd = fdVector[i];
		std::string strFace, strPS;
		utf8_encode(fmd.m_strTypeface.begin(), fmd.m_strTypeface.end(), std::back_inserter(strFace));
		utf8_encode(fmd.m_strPostScriptName.begin(), fmd.m_strPostScriptName.end(), std::back_inserter(strPS));
		fprintf(out, "%s\t%d\t%s\t%s\n", strFace.c_str(), fmd.m_iWeight, fmd.m_bItalic ? "true" : "false", strPS.c_str());

		fmd.Serialize(&dsk);
	}

	//CFontMapper::SaveNgramData(fdVector, &dsk);
	dsk.Close();
	fclose(out);
}

int MergeFonts(const TCHAR* file1, const TCHAR* file2, const TCHAR* file_out)
{
	USES_CONVERSION;
	CFileDisk f1(file1);
	CFileDisk f2(file2);

	FontMapDataSet fm;

	f1.Open(vcFile::orOpenForRead);
	if(f1.GetError() != EID_NO_ERROR)
		return -1;

	CFontMapper::PopulateFontMapDataSet(fm, &f1);
	f1.Close();

	f2.Open(vcFile::orOpenForRead);
	if(f2.GetError() != EID_NO_ERROR)
		return -2;

	CFontMapper::PopulateFontMapDataSet(fm, &f2);
	f2.Close();

	std::vector<FontMapData> fdVector;
	FontMapDataSet::iterator it = fm.begin();

	for(;it != fm.end(); ++it)
		fdVector.push_back(*it);

	std::sort(fdVector.begin(), fdVector.end(), SortPredicate);
	SaveFontMap(fdVector, file_out);

	return 0;
}

void EnumFonts(const TCHAR* pszFileName)
{
	USES_CONVERSION;

	FontMapDataSet fm;
	CEnumFontData efd(&fm, 0, 0, true);

	EnumAvailableFonts(L"", &efd);
	FontMapDataSet::iterator it = fm.begin();
	std::vector<FontMapData> fdVector;

	for(;it != fm.end(); ++it)
		fdVector.push_back(*it);

	std::sort(fdVector.begin(), fdVector.end(), SortPredicate);
	SaveFontMap(fdVector, pszFileName);
}

void FontMapData::Serialize(vcFile* pFile)
{
	pFile->WriteDWord((DWORD)m_strTypeface.length());
	pFile->WriteDWord((DWORD)m_strPostScriptName.length());
	pFile->Write((PVOID)m_strTypeface.c_str(), (DWORD)m_strTypeface.length()*sizeof(wchar_t));
	pFile->Write((PVOID)m_strPostScriptName.c_str(), (DWORD)m_strPostScriptName.length()*sizeof(wchar_t));
	pFile->WriteDWord((DWORD)m_iWeight);
	pFile->WriteByte(m_bItalic ? 'i' : 'n');
}

void FontMapData::DeSerialize(vcFile* pFile)
{
	m_strTypeface.resize(pFile->ReadDWord());
	m_strPostScriptName.resize(pFile->ReadDWord());
	pFile->Read((PVOID)&m_strTypeface[0], (DWORD)m_strTypeface.size()*sizeof(wchar_t));
	pFile->Read((PVOID)&m_strPostScriptName[0], (DWORD)m_strPostScriptName.size()*sizeof(wchar_t));
	m_iWeight = pFile->ReadDWord();
	BYTE byIt;
	pFile->Read(&byIt, 1);
	m_bItalic = byIt == 'i' ? true : false;
}

CFontMapper::CFontMapper()
{
	m_bBackMap = false;
	Init();
}

CFontMapper::CFontMapper(bool bBack)
{
	m_bBackMap = bBack;
	Init();
}

void CFontMapper::Init()
{
}

CFontMapper::~CFontMapper()
{
}

bool TrimRightIf(std::wstring& str, const std::wstring &strPostfix)
{
	size_t iPostfixLen = strPostfix.length();
	if((int)str.length() >= iPostfixLen && str.substr((int)str.length()-iPostfixLen) == strPostfix)
	{
		str = str.substr(0, str.length()-iPostfixLen);
		return true;
	}

	return false;
}

std::string CFontMapper::MapFontNameBack(const std::string &strInitial, int& weight, bool& bBold, bool& bItalic)
{
	USES_CONVERSION;

	std::wstring str = A2W(strInitial.c_str());
	std::wstring strRes = MapFontNameBackW(str, weight, bBold, bItalic);
	return W2A(strRes.c_str());
}

std::string CFontMapper::MapFontName(const std::string &strInitial, int weight, bool bItalic)
{
	USES_CONVERSION;

	std::wstring str = A2W(strInitial.c_str());
	std::wstring strRes = MapFontNameW(str, weight, bItalic);
	return W2A(strRes.c_str());
}

/******************************************************************************
   Function:	FontMapData::DecomposePostscript
   Author:		Paul Gruzdev
   Purpose:     
   Returns:		std::wstring
   Parameters:	std::wstring str  
   Parameters:	bool & bBold  
   Parameters:	bool & bItalic  
   Notes:       
******************************************************************************/
std::wstring FontMapData::DecomposePostscript(std::wstring str, int& weight, bool& bItalic)
{
	weight = 400;
	TrimRightIf(str, UNI("MT")); // Get rid of the Monotype sign. Arial-BoldMT -> Arial-Bold
	TrimRightIf(str, UNI("-Regular"));
	TrimRightIf(str, UNI("-Roman"));
	TrimRightIf(str, UNI("-Mono"));

	if(TrimRightIf(str, UNI("-BoldItalic")))
	{
		weight = 700;
		bItalic = true;
	}
	else if(TrimRightIf(str, UNI("-BoldOblique")))
	{
		weight = 700;
		bItalic = true;
	}
	else if(TrimRightIf(str, UNI("-Oblique")))
		bItalic = true;
	else if(TrimRightIf(str, UNI("-Italic")))
		bItalic = true;
	else if(TrimRightIf(str, UNI("-Bold")))
		weight = 700;

	// Now restore spaces between capital letters
	for(size_t pos = 0; pos < str.length(); pos++)
	{
		if(str[pos] == '-') str[pos] = ' ';
		else if(pos > 0 && (str[pos] >= 'A' && str[pos] <= 'Z') &&
			(str[pos-1] < 'A' || str[pos-1] > 'Z') && str[pos-1] != '-' && str[pos-1] != ' ')
		{
			str.insert(str.begin()+pos, ' ');
			pos++;
		}
	}
	return str;
}
std::wstring CFontMapper::MapFontNameBackW(const std::wstring &strInitial, int& weight, bool& bBold, bool& bItalic)
{
	bool bFound = false;
	bBold = false;
	bItalic = false;
	std::wstring str = strInitial;
	if(!strInitial.length())
		return str;

	_ASSERT(m_FontMap.size() > 0); // make an assersion if we are not initialised. Better call PopulateFromResource first
	_ASSERT(m_bBackMap);

	std::wstring fontKey = strInitial;
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);

	CFontMap::const_iterator fontMapIt = m_FontMap.find(fontKey);
	if(fontMapIt != m_FontMap.end())
	{
		const FontMapData &fmd = fontMapIt->second;
		weight = fmd.m_iWeight;
		bBold = fmd.m_bBold;
		bItalic = fmd.m_bItalic;
		str = fmd.m_strTypeface;
		bFound = true;
	}
/*
	for(size_t i = 0; i < m_fmd.size();i++)
	{
		FontMapData& fmd = m_fmd[i];
		if(fmd.m_strPostScriptName == strInitial)
		{
			bBold = fmd.m_iWeight >= 700;
			bItalic = fmd.m_bItalic;
			str = fmd.m_strTypeface;
			bFound=true;
			break;
		}
	}
*/
	// Try to decompose the PostScriptName into styles
	if(!bFound)
	{
		std::wstring res = FontMapData::DecomposePostscript(strInitial, weight, bItalic);
		bBold = weight >= 700 ? true : false;
		return res;
	}

	return str;
}

/******************************************************************************
   Function:	FontMapData::GenerateWeightString
   Author:		Paul Gruzdev
   Purpose:     Convert weight value into PostScript-like variant name
   Returns:		std::string
   Parameters:	int iWeight  
   Notes:       
******************************************************************************/
std::wstring FontMapData::GenerateWeightString(int iWeight)
{
	switch(iWeight)
	{
	case 100: return L"Thin";
	case 200: return L"ExtraLight";
	case 300: return L"Light";
	case 400: return L"Regular";
	case 500: return L"Medium";
	case 600: return L"Semibold";
	case 700: return L"Bold";
	case 800: return L"ExtraBold";
	case 900: return L"Black";
	}
	// Convert non-standard weight into it's string representation
	wchar_t psz[8];
	return _itow(iWeight, psz, 10);
}

/******************************************************************************
   Function:	FontMapData::ToPostscript
   Author:		Paul Gruzdev
   Purpose:     General try to simulate PostScript name
   Returns:		std::wstring
   Parameters:	std::wstring str  
   Parameters:	bool bBold  
   Parameters:	bool bItalic  
   Notes:       
******************************************************************************/
std::wstring FontMapData::ToPostscript(std::wstring str, int weight, bool bItalic, bool bAddHyphen)
{
	// Try to simulate the PostScript name. It definitely does not have any spaces
	for(size_t pos = 0; pos != std::wstring::npos; )
	{
		pos = str.find(' ', pos);
		if(pos != std::wstring::npos)
			str.erase(pos, 1);
	}

	if(weight == 700 && (str.find(UNI("Bold")) != std::wstring::npos || str.find(UNI("Bd")) != std::wstring::npos))
		weight = 400; // drop bold if already in the name

	if(bItalic && (str.find(UNI("Italic")) != std::wstring::npos || str.find(UNI("It")) != std::wstring::npos))
		bItalic = false; // drop bold if already in the name

 	if(bAddHyphen && (weight != 400 || bItalic))
 		str += UNI("-");

	if(weight != 400)
		str += GenerateWeightString(weight);//UNI("Bold");
	if(bItalic)
		str += UNI("Italic");
	return str;
}

std::wstring CFontMapper::MapFontNameW(const std::wstring &strInitial, int weight, bool bItalic)
{
	//std::wstring strDecorated = FontMapData::DecorateFontName(strInitial, bBold ? 700 : 400, bItalic);
	std::wstring str;
	int iIdx = -1;

	if(!m_bBackMap)
	{
		FontMapData fmd(strInitial, L"", weight, bItalic);
		FontMapDataSet::iterator it = m_DataSet.find(fmd);
		if(it != m_DataSet.end())
			return it->m_strPostScriptName;

		/*
		if(m_FontMap.find(strDecorated) != m_FontMap.end())
		{
			CFontMap::const_iterator fontMapIt = m_FontMap.find(strDecorated);
			const FontMapData &fmd = (*fontMapIt).second;
			bool bBoldMatch = (bBold && fmd.m_iWeight >= 700) || (!bBold && fmd.m_iWeight < 700);
			if(fmd.m_strTypeface == strInitial && bBoldMatch && fmd.m_bItalic == bItalic)
				return fmd.m_strPostScriptName;
		}*/
/*
		for(size_t i = 0; i < m_fmd.size();i++)
		{
			FontMapData& fmd = m_fmd[i];
			bool bBoldMatch = (bBold && fmd.m_iWeight >= 700) || (!bBold && fmd.m_iWeight < 700);
			if(fmd.m_strTypeface == strInitial && bBoldMatch && fmd.m_bItalic == bItalic)
			{
				iIdx = i;
				break;
			}
		}*/
	}
	else _ASSERT(0);

// 	if(iIdx >= 0)
// 		str = m_fmd[iIdx].m_strPostScriptName;
// 	else

	return FontMapData::ToPostscript(strInitial, weight, bItalic, true);
}

/******************************************************************************
Function:    LoadFontMapDataFromResource
Author:      Paul Gruzdev
Purpose:     Load the custom resource to memory
Returns:     bool -
Exceptions:  
Parameters:  int iID - 
PSD_ICCProfile& profile - 
Notes:       
******************************************************************************/
PBYTE CFontMapper::LoadFontMapDataFromResource(HINSTANCE hInst, int iID, TCHAR* pszResType, DWORD& dwSize)
{
	PBYTE pbyRes = NULL;
	// Obtain current resource handle
	//HINSTANCE	hInst = _Module.GetResourceInstance();
	HRSRC		hRes;
	HGLOBAL		hData;

	hRes = FindResource(hInst, MAKEINTRESOURCE(iID), pszResType);

	// Find and load
	if (hRes && (hData = LoadResource(hInst, hRes)) != NULL)
	{
		// Get resource size
		dwSize = SizeofResource(hInst, hRes);

		// Lock loaded resource in memory
		LPVOID lpvRes = LockResource(hData);

		if (!lpvRes)
			return false;

		if(dwSize >= 4 && *((DWORD*)lpvRes) == 0x04034b50) // ZIP signature
		{
			HZIP hZip = OpenZip(lpvRes, dwSize, 0);
			bool bRes = false;

			if (hZip)
			{
				ZIPENTRY ZipEntry = {0};
				GetZipItem(hZip, -1, &ZipEntry); 
				const int iNumOfFiles = ZipEntry.index;

				//Unzip the first entry in the zip archive
				if(iNumOfFiles > 0)
				{
					GetZipItem(hZip, 0, &ZipEntry);
					dwSize = ZipEntry.unc_size;
					pbyRes = new BYTE [dwSize+1];
					UnzipItem(hZip, 0, pbyRes, dwSize);
					pbyRes[dwSize] = 0; // add a trailing 0 if we are loading a string resource
				}

				CloseZip(hZip);
				bRes = true;
			}
		}
		else
		{
			// Actual copy
			pbyRes = new BYTE[dwSize+1];
			memcpy (pbyRes, lpvRes, dwSize);
			pbyRes[dwSize] = 0; // add a trailing 0 if we are loading a string resource
		}

		UnlockResource(hData);
		FreeResource(hData);
	}

	return pbyRes;
}

bool CFontMapper::PopulateFromResource(HINSTANCE hInst, int iID, TCHAR* pszResType)
{
	bool bRes = false;
	DWORD dwSize;
	PBYTE pbyData = LoadFontMapDataFromResource(hInst, iID, pszResType, dwSize);

	if(pbyData)
	{
		CFileMemory file;
		file.SetPointerMemoryMode(pbyData, dwSize);
		file.Open(vcFile::orOpenForRead);
		bRes = PopulateFontMapBinary(&file, m_bBackMap);
		file.Close();
	}

	return bRes;
}

bool CFontMapper::PopulateFontMapDataSet(FontMapDataSet& fm, vcFile* pFile)
{
	bool bRes = false;
	std::string str;
	str.resize(512);

	// get the first two strings
	int iCnt = pFile->GetString(&str[0], 512);
	iCnt = pFile->GetString(&str[0], 512);

	for(;!pFile->Eof();)
	{
		int iCnt = pFile->GetString(&str[0], 512);
		if(iCnt)
		{
			FontMapData fmd;
			fmd.DeSerialize(str.c_str());
			if(fmd.m_strTypeface != L"" && fm.find(fmd) == fm.end())
			{
				fm.insert(fmd);
				bRes = true;
			}
		}
	}

	return bRes;
}

bool CFontMapper::PopulateFontMap(vcFile* pFile, bool bBack)
{
	bool bRes = false;
	std::string str;
	str.resize(512);

	// get the first two strings
	int iCnt = pFile->GetString(&str[0], 512);
	iCnt = pFile->GetString(&str[0], 512);

	for(;!pFile->Eof();)
	{
		int iCnt = pFile->GetString(&str[0], 512);

		if(iCnt)
		{
			FontMapData fmd;
			fmd.DeSerialize(str.c_str());
			std::wstring str = bBack ? fmd.m_strPostScriptName : 
				FontMapData::DecorateFontName(fmd.m_strTypeface, fmd.m_iWeight >= 700 ? 700 : 400, fmd.m_bItalic);

			if(!str.empty())
			{
				CFontMapper::fontMapIts.push_back(  m_FontMap.insert( std::pair<std::wstring,FontMapData>(str,fmd) )  );
				bRes = true;
			}
		}
	}

	return bRes;
}

bool CFontMapper::PopulateFontMapBinary(vcFile* pFile, bool bBack)
{
	bool bRes = false;
	DWORD dwSize = pFile->ReadDWord();
	CFontMapper::fontMapIts.reserve((size_t) dwSize);

	for(DWORD i = 0; i < dwSize; i++)
	{
		FontMapData fmd;
		fmd.DeSerialize(pFile);
		std::wstring str = bBack ? fmd.m_strPostScriptName : 
			FontMapData::DecorateFontName(fmd.m_strTypeface, fmd.m_iWeight >= 700 ? 700 : 400, fmd.m_bItalic);

		if (!str.empty())  // 2013.10.26 Pavel Ostapenko: keep records with same font names
		{
			std::wstring fontKey = str;
			std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);

			m_FontMap.insert( std::pair<std::wstring,FontMapData>(fontKey, fmd));
			bRes = true;
		}
	}

	return bRes;
}

std::wstring CFontMapper::MapBase14FontName(const std::wstring &macName)
{
	static std::map<std::wstring,std::wstring> macNameMap;
	if (macNameMap.empty())
	{
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"helvetica", L"Arial"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"arialmt", L"Arial"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"arial mt", L"Arial"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"arial-mt", L"Arial"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"courier", L"Courier New"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"timesroman", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times roman", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times-roman", L"Times New Roman"));
		// ZapfDingbats and Wingdings fonts have different character set and encoding, so we should not treat Wingdings as alias to ZapfDingbats
		//macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"zapfdingbats", L"Wingdings"));
		//macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"zapf dingbats", L"Wingdings"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"palatino", L"Palatino Linotype"));
	}

	std::wstring fontKey = macName;
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);
	std::map<std::wstring,std::wstring>::const_iterator macNameMapIt = macNameMap.find(fontKey);
	if (macNameMapIt == macNameMap.end())
		return macName;
	return macNameMapIt->second;
}

std::wstring CFontMapper::MapMacFontName(const std::wstring &macName)
{
	static std::map<std::wstring,std::wstring> macNameMap;
	if (macNameMap.empty())
	{
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"helvetica", L"Arial"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"gadget", L"Arial Black"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"palatino", L"Book Antiqua"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"courier", L"Courier New"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"charcoal", L"Impact"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"monaco", L"Lucida Console"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"geneva", L"Tahoma"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"new york", L"MS Serif"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"new-york", L"MS Serif"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times roman", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"times-roman", L"Times New Roman"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"zapf dingbats", L"Wingdings"));
		macNameMap.insert(std::make_pair<std::wstring,std::wstring>(L"zapfdingbats", L"Wingdings"));
	}
	std::wstring fontKey = macName;
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);
	std::map<std::wstring,std::wstring>::const_iterator macNameMapIt = macNameMap.find(fontKey);
	if (macNameMapIt == macNameMap.end())
		return macName;
	return macNameMapIt->second;
}

bool CFontMapper::ApproximateFontFaceStrict(const std::wstring &name, std::wstring &face, int &weight, bool& isBold, bool &isItalic)
{
	std::wstring fontKey = name;
	std::transform(fontKey.begin(), fontKey.end(), fontKey.begin(), std::tolower);
	CFontMap::const_iterator fontMapDataSetIt = m_FontMap.find(fontKey);
	if (fontMapDataSetIt != m_FontMap.end())
	{
		face     = fontMapDataSetIt->second.m_strTypeface;
		weight   = fontMapDataSetIt->second.m_iWeight;
		isBold   = fontMapDataSetIt->second.m_bBold;
		isItalic = fontMapDataSetIt->second.m_bItalic;
		return true;
	}
	FontNameDesc desc = NormalizeEmbeddedFontName(name);
	face      = desc.nameStrictCompare;
	weight    = desc.GetWeight();
	//isBold    = weight >= 700; // Paul: not sure, for current tests only
	isItalic  = desc.isItalic();

	// No need to find it here as it is not present in the system
	//FindFontBoldFlag(face, weight, isBold, isItalic);

	return false; // the font does not exist in the map(system), we needed to decompose it
}

/******************************************************************************
   Function:	CFontMapper::FixUpFontProperties
   Author:		Paul Gruzdev
   Purpose:     Given the font name, if present in the system, fix it up 
				for uppercase-lowercase characters in typeface name
				taken from the system, also assigning proper bold flag
   Returns:		std::wstring
   Parameters:	std::wstring face  
   Parameters:	int weight  
   Parameters:	bool isItalic  
   Parameters:	bool & bBold  
   Notes:       
******************************************************************************/
std::wstring CFontMapper::FixUpFontProperties(const std::wstring &face, int weight, bool isItalic, bool& bBold)
{
	// Now try to find the font in the system and take a 'bold' flag from there
	FontMapData fmd(face, L"", weight, isItalic);
	FontMapDataSet::iterator itFmdSys = m_DataSet.find(fmd);
	if(itFmdSys != m_DataSet.end())
	{
		bBold = itFmdSys->m_bBold;
		return itFmdSys->m_strTypeface;
	}
	return face;
}

bool CFontMapper::FindFontBoldFlag(const std::wstring &face, int weight, bool& isBold, bool isItalic)
{
	// Now try to find the font in the system and take a 'bold' flag from there
	FontMapData fmd(face, L"", weight, isItalic);
	FontMapDataSet::iterator itFmdSys = m_DataSet.find(fmd);
	if(itFmdSys != m_DataSet.end())
	{
		//FontMapData fmSys = m_DataSet[fmd];
		isBold = itFmdSys->m_bBold;
		return true;
	}
	else 
		isBold = false; // font is not present in the system, it is multiweight, bold should be off
	return false;

}
/******************************************************************************
Function:    Approximate font name to face matching
Author:      Pavel Ostapenko
Notes:       Ngram-based table lookup using heuristic ranking with distance 
             calculation using modifier parsing and normalization of original 
			 FontMapper table.
			 Version 5.1 (2013.10.29)
******************************************************************************/
bool CFontMapper::ApproximateFontFace(const std::wstring &name, std::wstring &face, int &weight, bool &isItalic)
{
	struct ComparedEntry
	{
		size_t fontNgramDescSetsI;
		float  d;
		bool   operator< (const ComparedEntry &op) const {return d < op.d;}
	};
	std::set<ComparedEntry> comparedEntries;

	FontNameDesc desc = NormalizeEmbeddedFontName(name);
/*
	// Paul: a temp hack - now just return a decomposed font name and it's modifiers
	{
		face     = desc.GetNameW();
		weight   = static_cast<int>(desc.weight * 1000.f + .5f);
		isItalic = desc.modifiers.find(Modifier::modifiersCache.find(L"italic")->second) != desc.modifiers.end();
		return false;
	}
*/
	std::map<size_t,float> ngrams;
	GetNgramsInd<FontNgrams,FontNgram>(desc.name, NGRAM, CFontMapper::ngrams, ngrams, 1.);
	std::set<size_t> entries;

	for (std::map<size_t,float>::const_iterator ngramIt = ngrams.begin(); ngramIt != ngrams.end(); ++ngramIt)
	{
		if (ngramIt->first == NGRAM_INVALID_INDEX)
			continue;
		const FontNgram &cacheEntry = CFontMapper::ngrams[ngramIt->first];
		for (std::vector<size_t>::const_iterator entryIt = cacheEntry.descs.begin(); entryIt != cacheEntry.descs.end(); ++entryIt)
			entries.insert(*entryIt);
	}

	#ifdef FONTMAPPER_MATCHING_DEBUG
	std::wcout << L"\n*** +++";
	#endif	
	for (std::set<size_t>::const_iterator entryIt = entries.begin(); entryIt != entries.end(); ++entryIt)
	{
		FontNgramDescSet &entry = CFontMapper::ngramDescSets[*entryIt];
		ComparedEntry compared;
		float dModFont = FontNameDesc::ModifiersDistance(desc.modifiers, desc.weight, entry.modsFont, entry.weight);
		float dModFace = FontNameDesc::ModifiersDistance(desc.modifiers, desc.weight, entry.modsFace, entry.weight);
		if (dModFont < 0.f || dModFace < 0.f)
			continue;

		float dMod = .01f*dModFont + .99f*dModFace;
		float subsDistance;
		float subsDistance1 = substring_distanceInd(ngrams, entry.ngrams, CFontMapper::ngrams, NGRAM);
		if (subsDistance1 > .6666666f)
			continue;
		float subsDistance2 = substring_distanceInd(entry.ngrams, ngrams, CFontMapper::ngrams, NGRAM);
		subsDistance = sqrtf(.5f*(subsDistance1*subsDistance1 + subsDistance2*subsDistance2));
		if (subsDistance > .33333333f)
			continue;

		compared.d = .05f*dMod + .95f*subsDistance;
		compared.fontNgramDescSetsI = *entryIt;
		comparedEntries.insert(compared);

		#ifdef FONTMAPPER_MATCHING_DEBUG
		if (!fontMapIts.empty())
			std::wcout << L"\n*** " << fontMapIts[*entryIt]->second.m_strTypeface << L" w=" << fontMapIts[*entryIt]->second.m_iWeight << L", i=" << fontMapIts[*entryIt]->second.m_bItalic << L" : "  << compared.d;
		#endif
	}
	#ifdef FONTMAPPER_MATCHING_DEBUG
	std::wcout << L"\n*** ^^^";
	#endif

	if (comparedEntries.empty())
	{
		face     = desc.GetNameW();
		weight   = static_cast<int>(desc.weight * 1000.f + .5f);
		isItalic = desc.modifiers.find(Modifier::modifiersCache.find(L"italic")->second) != desc.modifiers.end();
		return false;
	}

	if (fontMapIts.empty())
	{
		face = L"*font found*";
		return true;
	}
	size_t fontNgramDescSetsI = comparedEntries.begin()->fontNgramDescSetsI;
	CFontMap::const_iterator fontMapIt = fontMapIts[fontNgramDescSetsI];
	const FontMapData &fontMapEntry = fontMapIt->second;

	face     = fontMapEntry.m_strTypeface;
	weight   = static_cast<int>(CFontMapper::ngramDescSets[fontNgramDescSetsI].weight * 1000.f + .5f);
	isItalic = CFontMapper::ngramDescSets[fontNgramDescSetsI].italic;
	return true;
}

void CFontMapper::LoadNgramData()
{
	ModifiersInit();

	m_DataSet.clear();
	FontMapDataSet& fm = m_DataSet;
	CEnumFontData efd(&m_DataSet, &m_FontMap, &m_WeightMap, true, &m_EnumOrderList);
	// This calls the Windows system, which in it's turn loads fonts and PS names which are always stored in 
	// big endian, so we need to switch to it and then possibly switch back.
	EByteOrder order = GetByteOrder();
	SetByteOrder(keBigEndian);
	EnumAvailableFonts(L"", &efd);
	SetByteOrder(order); // restore the byte order. TIFF can have both, PSD has big endian (for example).

	std::map<std::wstring, int> mLowestWeight;
	std::map<std::wstring, std::set<int>> mWeight;
	// First of all gather all the weights of a particular typeface
	for (FontMapDataSet::const_iterator fmIt = fm.begin(); fmIt != fm.end(); ++fmIt)
	{
		if (fmIt->m_strPostScriptName.empty())
			continue;

		auto it = mLowestWeight.find(fmIt->m_strTypeface);

		if (it == mLowestWeight.end())
			mLowestWeight[fmIt->m_strTypeface] = fmIt->m_iWeight;
		else if (fmIt->m_iWeight < it->second)
			it->second = fmIt->m_iWeight;
		mWeight[fmIt->m_strTypeface].insert(fmIt->m_iWeight);
	}

	// Now set bold and multiweight flags for m_DataSet
	for (FontMapDataSet::iterator fmIt = fm.begin(); fmIt != fm.end(); ++fmIt)
	{
		std::wstring sTypeface = fmIt->m_strTypeface;
		int iWeight = fmIt->m_iWeight;
		fmIt->m_bMultiweight = mWeight[sTypeface].size() > 2;
		fmIt->m_bBold = !fmIt->m_bMultiweight && (mLowestWeight[sTypeface] != iWeight);
//wprintf(L"%s\t%d\t%s\t%s\n", sTypeface.c_str(), iWeight, fmIt->m_bBold ? L"bold" : L"regular", fmIt->m_bItalic ? L"true" : L"false");
	}
	// Now set the bold and multiweight flags to m_FontMap
	for (CFontMap::iterator fmIt = m_FontMap.begin(); fmIt != m_FontMap.end(); ++fmIt)
	{
		std::wstring sTypeface = fmIt->second.m_strTypeface;
		int iWeight = fmIt->second.m_iWeight;
		fmIt->second.m_bMultiweight = mWeight[sTypeface].size() > 2;
		fmIt->second.m_bBold = !fmIt->second.m_bMultiweight && mLowestWeight[sTypeface] != iWeight;
	}
	/*
	#define FONTMAP_INSERT(font,face,w,i) fm.insert(FontMapData(face,font##L" customsubst",w,i))
	#define FONTMAP_INSERT_SERIES(font,face) \
		 FONTMAP_INSERT(font,face,400,false) \
		;FONTMAP_INSERT(font##L" Bold",face,700,false) \
		;FONTMAP_INSERT(font##L" Italic",face,400,true) \
		;FONTMAP_INSERT(font##L" Bold Italic",face,700,true) \
		;FONTMAP_INSERT(font##L" Light",face,300,false) \
		;FONTMAP_INSERT(font##L" Light Italic",face,300,true) \
		;FONTMAP_INSERT(font##L" Condensed",face,400,false) \
		;FONTMAP_INSERT(font##L" Condensed Bold",face,700,false) \
		;FONTMAP_INSERT(font##L" Condensed Italic",face,400,true) \
		;FONTMAP_INSERT(font##L" Condensed Bold Italic",face,700,true)

	m_FontMap.clear();
	FONTMAP_INSERT_SERIES(L"Helvetica",L"Arial");
	FONTMAP_INSERT_SERIES(L"Gadget",L"Arial Black");
	FONTMAP_INSERT_SERIES(L"Palatino",L"Book Antiqua");
	FONTMAP_INSERT_SERIES(L"Charcoal",L"Impact");
	FONTMAP_INSERT_SERIES(L"Monaco",L"Lucida Console");
	FONTMAP_INSERT_SERIES(L"Geneva",L"Tahoma");
	FONTMAP_INSERT_SERIES(L"New York",L"MS Serif");
	FONTMAP_INSERT_SERIES(L"Palatino",L"Palatino Linotype");
	FONTMAP_INSERT_SERIES(L"Times",L"Times New Roman");
	FONTMAP_INSERT_SERIES(L"Zapf Dingbats",L"Wingdings");
	*/
}

/*static*/ std::wstring CFontMapper::LoadWString(vcFile *pFile)
{
	static wchar_t str[0xFFFF];
	BYTE l;
	pFile->Read(&l, 1);
	pFile->Read(&str[0],2*l);
	return std::wstring(&str[0],l);
}

/*static*/float CFontMapper::LoadFloat(vcFile *pFile)
{
	DWORD dw = pFile->ReadDWord();
	return *reinterpret_cast<float *>(&dw);
}

/*static*/ void CFontMapper::SaveNgramData(vcFile* pFile)
{
	// Generate ngrams, descriptors and cache
	//ModifiersInit();
	//CFontMapper::ConvertDataToNgrams(fdVector);

	//serialize
	//+ modifiers
	pFile->WriteDWord((DWORD)Modifier::modifiers.size());
	for (Modifier::Modifiers::const_iterator modIt = Modifier::modifiers.begin(); modIt != Modifier::modifiers.end(); ++modIt)
	{
		pFile->WriteDWord((DWORD)modIt->tokens.size());
		for (std::vector<std::wstring>::const_iterator tokenIt = modIt->tokens.begin(); tokenIt != modIt->tokens.end(); ++tokenIt)
		{
			pFile->WriteByte((BYTE)tokenIt->size());
			pFile->WriteString(tokenIt->c_str());
		}

		pFile->WriteDWord((DWORD)modIt->incompatible.size());
		for (std::set<size_t>::const_iterator it = modIt->incompatible.begin(); it != modIt->incompatible.end(); ++it)
			pFile->WriteDWord((DWORD)*it);

		pFile->WriteDWord(*reinterpret_cast<const DWORD *>( &(modIt->penalty)) );
		pFile->WriteDWord(*reinterpret_cast<const DWORD *>( &(modIt->weight )) );
	}

	//+ modifiers cache
	pFile->WriteDWord((DWORD)Modifier::modifiersCache.size());
	for (Modifier::ModifiersCache::const_iterator cacheEntryIt = Modifier::modifiersCache.begin(); cacheEntryIt != Modifier::modifiersCache.end(); ++cacheEntryIt)
	{
		pFile->WriteByte((BYTE)cacheEntryIt->first.size());
		pFile->WriteString(cacheEntryIt->first.c_str());
		pFile->WriteDWord((DWORD)cacheEntryIt->second);
	}

	//+ ngrams cache
	pFile->WriteDWord((DWORD)CFontMapper::ngramsSet.size());
	for (FontNgramsSet::const_iterator fontNgramsSetIt = CFontMapper::ngramsSet.begin(); fontNgramsSetIt != CFontMapper::ngramsSet.end(); ++fontNgramsSetIt)
	{
		pFile->WriteByte((BYTE)fontNgramsSetIt->first.length());
		pFile->WriteString(fontNgramsSetIt->first.c_str());
		pFile->WriteDWord((DWORD)fontNgramsSetIt->second.size());
		for each (size_t ngramDescSetsI in fontNgramsSetIt->second)
			pFile->WriteDWord((DWORD)ngramDescSetsI);
	}

	//+ descriptors
	pFile->WriteDWord((DWORD)CFontMapper::ngramDescSets.size());
	for (FontNgramDescSets::const_iterator descIt = CFontMapper::ngramDescSets.begin(); descIt != CFontMapper::ngramDescSets.end(); ++descIt)
	{
		pFile->WriteDWord((DWORD)descIt->ngrams.size());
		for (std::map<size_t,float>::const_iterator ngramIt = descIt->ngrams.begin(); ngramIt != descIt->ngrams.end(); ++ngramIt)
		{
			pFile->WriteDWord((DWORD)ngramIt->first);
			pFile->WriteDWord(*reinterpret_cast<const DWORD *>( &(ngramIt->second)) );
		}

		pFile->WriteDWord((DWORD)descIt->modsFace.size());
		for (std::set<size_t>::const_iterator it = descIt->modsFace.begin(); it != descIt->modsFace.end(); ++it)
			pFile->WriteDWord((DWORD)*it);

		pFile->WriteDWord((DWORD)descIt->modsFont.size());
		for (std::set<size_t>::const_iterator it = descIt->modsFont.begin(); it != descIt->modsFont.end(); ++it)
			pFile->WriteDWord((DWORD)*it);

		float w = descIt->weight;
		if (descIt->italic)
			w *= -1.f;
		pFile->WriteDWord(*reinterpret_cast<DWORD *>(&w));
	}
}

template<class DataVector>
void CFontMapper::ConvertDataToNgrams(const DataVector &fdVector)
{
	size_t italicModI = Modifier::modifiersCache.find(L"italic")->second;
	size_t customSubstModI = Modifier::modifiersCache.find(L"customsubst")->second;
	for (DataVector::const_iterator dataIt = fdVector.begin(); dataIt != fdVector.end(); ++dataIt)
	{
		FontNgramDescSet desc;
		FontNameDesc descFont = NormalizeEmbeddedFontName(dataIt->m_strPostScriptName);
		FontNameDesc descFace = NormalizeEmbeddedFontName(dataIt->m_strTypeface);
		desc.modsFace = descFace.modifiers;
		desc.modsFont = descFont.modifiers;
		desc.weight   = dataIt->m_iWeight / 1000.f;
		desc.italic   = descFont.modifiers.find(italicModI) != descFont.modifiers.end();

		if (descFont.modifiers.find(customSubstModI) != descFont.modifiers.end())
		{
			GetNgrams(descFont.name, NGRAM, desc.ngramsWString, .99f);
			GetNgrams(descFace.name, NGRAM, desc.ngramsWString, .01f);
		}
		else
		{
			GetNgrams(descFont.name, NGRAM, desc.ngramsWString, .01f);
			GetNgrams(descFace.name, NGRAM, desc.ngramsWString, .99f);
		}

		if (dataIt->m_bItalic)
		{
			desc.italic = true;
			desc.modsFace.insert(italicModI);
			desc.modsFont.insert(italicModI);
		}

		CFontMapper::ngramDescSets.push_back(desc);
		for (std::map<std::wstring,float>::const_iterator ngramIt = desc.ngramsWString.begin(); ngramIt != desc.ngramsWString.end(); ++ngramIt)
		{
			FontNgramsSet::iterator cacheIt = CFontMapper::ngramsSet.insert(std::make_pair<std::wstring,std::vector<size_t> >(ngramIt->first, std::vector<size_t>())).first;
			(*cacheIt).second.push_back(ngramDescSets.size()-1);
		}
	}

	size_t fontNgramsSetI = 0;
	for (FontNgramsSet::const_iterator fontNgramsSetIt=CFontMapper::ngramsSet.begin(); fontNgramsSetIt!=CFontMapper::ngramsSet.end(); ++fontNgramsSetIt, ++fontNgramsSetI)
	{
		const std::wstring &ngram = fontNgramsSetIt->first;

		for each (size_t ngramDescSetsI in fontNgramsSetIt->second)
		{
			FontNgramDescSet &ngramDescSet = CFontMapper::ngramDescSets[ngramDescSetsI];
			ngramDescSet.ngrams.insert(std::make_pair<size_t,float>(fontNgramsSetI,ngramDescSet.ngramsWString.find(ngram)->second));
		}
	}
}

FontMapDataSet::const_iterator CFontMapper::FindClosestFont(const CCPanose &panose) const
{
	EnumOrderList::const_iterator it = m_EnumOrderList.begin();
	if (m_EnumOrderList.end() == it) return m_DataSet.end();
	ULONG minDist = CCPanose::Distance(panose, (*it)->m_panose);
	FontMapDataSet::const_iterator res = *it++;

	for (; it != m_EnumOrderList.end(); ++it)
	{
		ULONG dist = CCPanose::Distance(panose, (*it)->m_panose);
		if (dist < minDist)
		{
			res = *it;
			minDist = dist;
		}
	}

	return res;
}

/*static*/ FontMapIts               CFontMapper::fontMapIts;
/*static*/ FontNgrams               CFontMapper::ngrams;
/*static*/ FontNgramsSet            CFontMapper::ngramsSet;
/*static*/ FontNgramDescSets        CFontMapper::ngramDescSets;
/*static*/ Modifier::Modifiers      Modifier::modifiers;
/*static*/ Modifier::ModifiersCache Modifier::modifiersCache;
/*static*/ //CFontMap                 CFontMapper::fontMapDataSet;
