#pragma once

#include <map>
#include <set>
#include <vector>
#include <list>
#include "common/Utf8.h"
#include "../../../XarLib_vc7/ccpanose.h"

class vcFile; // forward declaration
void EnumFonts(const TCHAR* pszFileName);
int MergeFonts(const TCHAR* file1, const TCHAR* file2, const TCHAR* file_out);

class FontMapData
{
public:
	std::wstring	m_strTypeface;
	std::wstring	m_strPostScriptName;
	std::wstring	m_strKey;
	int				m_iWeight;
	// this is only an informative flag, to indicate that this weight is *NOT* the lowest in the system; it also doesn't 
	// change order in multiset, so it may be mutable to be accessible from multiset::iterator
	mutable bool	m_bBold;
	// this is also an information flag, it means that typeface contains more than two different weights in the system
	mutable bool	m_bMultiweight;
	bool			m_bItalic;
	CCPanose        m_panose;

	FontMapData(): m_iWeight(400), m_bItalic(false), m_bBold(false), m_bMultiweight(false)
	{
	}

	FontMapData(const std::wstring &strTypeface, const std::wstring &strPostScriptName, int iWeight, bool bItalic);

	FontMapData(const FontMapData& f)
	{
		m_strKey = f.m_strKey;
		m_strPostScriptName = f.m_strPostScriptName;
		m_strTypeface = f.m_strTypeface;
		m_iWeight = f.m_iWeight;
		m_bBold = f.m_bBold;
		m_bMultiweight = f.m_bMultiweight;
		m_bItalic = f.m_bItalic;
		m_panose = f.m_panose;
	}

	bool operator==(const FontMapData& f) const
	{
		return m_strKey == f.m_strKey && m_bItalic == f.m_bItalic && m_iWeight == f.m_iWeight/* && !memcmp(&m_panose, &f.m_panose, sizeof(m_panose))*/;
	}

	static std::wstring DecorateFontName(std::wstring strTypeface, int iWeight, bool bItalic)
	{
		std::wstring str = strTypeface;
		str += iWeight;
		str += bItalic ? 'i' : 'n';
		return str;
	}

	static std::wstring ToPostscript(std::wstring str, int weight, bool bItalic, bool bAddHyphen);
	static std::wstring DecomposePostscript(std::wstring str, int& weight, bool& bItalic);
	static std::wstring GenerateWeightString(int iWeight);

	void Serialize(vcFile* pFile);
	void DeSerialize(vcFile* pFile);

	void DeSerialize(const char* psz)
	{
		//USES_CONVERSION;
		std::string str = psz;
		size_t iIdx = str.find('\t');
		if(iIdx != std::string::npos)
		{
			std::string sub = str.substr(0, iIdx);
			utf8_decode(sub.begin(), sub.end(), std::back_inserter(m_strTypeface));
			//m_strTypeface = A2W(str.substr(0, iIdx).c_str());
		}
		else
			return;

		size_t iStart = iIdx+1;
		iIdx = str.find('\t', iStart);
		if(iIdx != std::string::npos)
			m_iWeight = atoi(str.substr(iStart, iIdx-iStart).c_str());
		else
			return;

		iStart = iIdx+1;
		iIdx = str.find('\t', iStart);
		if(iIdx != std::string::npos)
		{
			if(str.substr(iStart, iIdx-iStart) == "true")
				m_bItalic = true;
			else
				m_bItalic = false;
		}
		else
			return;

		//m_strPostScriptName = A2W(str.substr(iIdx+1, str.length()-iIdx-1).c_str());
		std::string sub = str.substr(iIdx+1, str.length()-iIdx-1);
		utf8_decode(sub.begin(), sub.end(), std::back_inserter(m_strPostScriptName));
	}

	bool operator<(const FontMapData& f) const
	{
		//int iFaceDiff = wcscmp(m_strTypeface.c_str(), f.m_strTypeface.c_str());
		int iFaceDiff = wcscmp(m_strKey.c_str(), f.m_strKey.c_str());
		int iWeightDiff = m_iWeight - f.m_iWeight;
		int iItalicDiff = m_bItalic < f.m_bItalic ? -1 : (m_bItalic > f.m_bItalic ? 1 : 0);
		//int iPanoseDiff = memcmp(&m_panose, &f.m_panose, sizeof(m_panose));

		if(iFaceDiff != 0) return iFaceDiff < 0;
		if(iWeightDiff != 0) return iWeightDiff < 0;
		if(iItalicDiff != 0) return iItalicDiff < 0;
		//if(iPanoseDiff != 0) return iPanoseDiff < 0;

		return false;
		//bool bRes =  < 0 || m_iWeight < f.m_iWeight || bLess;
		//bRes = bRes || wcscmp(m_strPostScriptName.c_str(), f.m_strPostScriptName.c_str()) < 0;
// 		if(!wcscmp(m_strTypeface.c_str(), f.m_strTypeface.c_str()) && m_iWeight == f.m_iWeight && m_bItalic == f.m_bItalic)
// 			return false;
// 		return bRes;
	}
};

typedef std::multimap<std::wstring, FontMapData> CFontMap;
typedef std::multiset<FontMapData> FontMapDataSet;
typedef std::vector<FontMapData> FontMapDataVector;
typedef std::map<std::wstring, std::set<int>> CFontWeightMap;
typedef std::list<FontMapDataSet::const_iterator> EnumOrderList;

class CEnumFontData
{
public:
	//CEnumFontData() : pfmds(0), peol(0), pfm(0), pfw(0), bExitOnException(true) {}
	CEnumFontData(FontMapDataSet* ps, CFontMap* pfmA, CFontWeightMap* pfwA, bool bExitOnExceptionA, EnumOrderList *_peol = 0) : pfmds(ps), peol(_peol), pfm(pfmA), pfw(pfwA), bExitOnException(bExitOnExceptionA), hdc(NULL)
	{
		//hdc = CreateCompatibleDC(NULL);
		hdc = GetDC(NULL);//CreateIC(_T("DISPLAY"), 0, 0, 0);//GetDC(GetDesktopWindow());

	}

	~CEnumFontData()
	{
		ReleaseDC(0, hdc);
		//DeleteDC(hdc); //ReleaseDC(GetDesktopWindow(), hdc);
	}

	FontMapDataSet* pfmds;	// This is a unique set of fonts
	EnumOrderList *peol; // this list used to retain enum order needed in FindClosestFont
	CFontMap* pfm;			// this is a 'back' map from PostScript font name to Windows font data struct
	CFontWeightMap* pfw;	// collection of weights for each typeface name
	bool bExitOnException;  // this is a flag whether we should immediately exist enumeration if a font data exception is thrown,
							// say for system fonts and recursive enumeration, or whether we should continue enumerating (false)
							// for the single pass enumeration process
	HDC hdc;
};

void SaveFontMap(FontMapDataVector& fdVector, const TCHAR* file_out);
void EnumAvailableFonts(const std::wstring &fName, CEnumFontData* pParam);

enum { NGRAM = 4 };
struct FontNgram
{
	std::wstring        name;
	std::vector<size_t> descs;
	size_t              index;
	bool operator< (const FontNgram &op) const {return name.compare(op.name) < 0;}
	bool operator< (const std::wstring &op) const {return name.compare(op) < 0;}
};
typedef std::vector<FontNgram> FontNgrams;
typedef std::vector<std::wstring> FontMods;
typedef std::set<std::wstring> FontModsSet;
typedef std::map<std::wstring, std::vector<size_t> > FontNgramsSet;
typedef std::vector<CFontMap::const_iterator> FontMapIts;
struct FontNgramDescSet
{
	std::map<size_t,float>       ngrams;
	std::map<std::wstring,float> ngramsWString;
	std::set<size_t>             modsFont;
	std::set<size_t>             modsFace;
	float                        weight;
    bool                         italic;
};
typedef std::vector<FontNgramDescSet> FontNgramDescSets;

///////////////////////////////////////////
// Font mapper class - map Adobe font name to Windows name and back
class CFontMapper
{
public:
	FontMapDataSet	m_DataSet;
	EnumOrderList   m_EnumOrderList; // this list used to retain enum order needed in FindClosestFont

	CFontMap	m_FontMap;
	bool		m_bBackMap;
	//FontMapDataVector	m_fmd;
	CFontWeightMap m_WeightMap;
	
	CFontMapper();
	CFontMapper(bool bBack);
	void Init();
	virtual ~CFontMapper();

	std::string MapFontNameBack(const std::string &strInitial, int& weight, bool& bBold, bool& bItalic);
	std::string MapFontName(const std::string &strInitial, int weight, bool bItalic);

	std::wstring MapFontNameBackW(const std::wstring &strInitial, int& weight, bool& bBold, bool& bItalic);
	std::wstring MapFontNameW(const std::wstring &strInitial, int weight, bool bItalic);

	static std::wstring MapBase14FontName(const std::wstring &macName);
	static std::wstring MapMacFontName(const std::wstring &macName);
	////////////////////////////////////////////////
	// New methods
	PBYTE LoadFontMapDataFromResource(HINSTANCE hInst, int iID, TCHAR* pszResType, DWORD& dwSize);
	//bool PopulateFontMapDataVector(vcFile* pFile);
	bool PopulateFontMap(vcFile* pFile, bool bBack);
	bool PopulateFontMapBinary(vcFile* pFile, bool bBack);
	static bool CFontMapper::PopulateFontMapDataSet(FontMapDataSet& fm, vcFile* pFile);
	bool PopulateFromResource(HINSTANCE hInst, int iID, TCHAR* pszResType);

	bool FindFontBoldFlag(const std::wstring &face, int weight, bool& isBold, bool isItalic);
	std::wstring FixUpFontProperties(const std::wstring &face, int weight, bool isItalic, bool& bBold);
	// system font matching
	static bool TypefacePresentInTheSystem(std::wstring fontKey);

	// Approximate matching support
	bool ApproximateFontFace(const std::wstring &name, std::wstring &face, int &weight, bool &isItalic);
	bool ApproximateFontFaceStrict(const std::wstring &name, std::wstring &face, int &weight, bool& isBold, bool &isItalic);
	static void SaveNgramData(vcFile* pFile);
	template<class DataVector> static void ConvertDataToNgrams(const DataVector &fdVector);
	void LoadNgramData();
	static std::wstring LoadWString(vcFile *pFile);
	static float LoadFloat(vcFile *pFile);

	FontMapDataSet::const_iterator FindClosestFont(const CCPanose &panose) const;
private:
//	static CFontMap          fontMapDataSet;
	static FontMapIts        fontMapIts;
	static FontNgrams        ngrams;
	static FontNgramDescSets ngramDescSets;
	static FontNgramsSet     ngramsSet;
};
