/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "vcFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#ifndef DEBUG_NEW
#define DEBUG_NEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
#endif // DEBUG_NEW
#define new DEBUG_NEW
#endif // _DEBUG

/******************************************************************************
   Function:    vcFile::ReadDWord
   Purpose:     Reads a DWORD value.
   Returns:     DWORD - Value read from file.
   Parameters:  const eByteOrder ByteOrder - Byte order in which DWORD is stored
				in file (defaults to eIntelByteOrder).
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown by
				the call to Read() (and thus throws it).
******************************************************************************/
DWORD vcFile::ReadDWord(const eByteOrder ByteOrder)
{

	BYTE abyBuffer[sizeof(DWORD)];	// Buffer to hold data
	DWORD dwReturnValue = 0;	// Return value
	short nLoopCount = 0;		// Loop counter

	/* Read 4 bytes from the file.	 */
	Read(abyBuffer, sizeof(DWORD), TRUE);

	/* Convert (as appropriate) the 4 bytes to a DWORD value	 */
	if (ByteOrder == vcFile::boIntelByteOrder)
	{
		/* Convert from intel-stored to internal DWORD value. */
		for (nLoopCount = 3; nLoopCount > -1; nLoopCount--)
		{
			dwReturnValue = (dwReturnValue << 8) + abyBuffer[nLoopCount];
		}
	}
	else
	{
		/* Convert from motorola-stored to internal DWORD value. */
		for (nLoopCount = 0; nLoopCount < 4; nLoopCount++)
		{
			dwReturnValue = (dwReturnValue << 8) + abyBuffer[nLoopCount];
		}
	}

	return dwReturnValue;
}

/******************************************************************************
   Function:    vcFile::ReadWord
   Purpose:     Reads a WORD value.
   Returns:     WORD - Value read from file.
   Parameters:  const eByteOrder ByteOrder - Byte order in which DWORD is stored
				in file (defaults to eIntelByteOrder).
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown
				by the call to Read() (and thus throws it).
******************************************************************************/
WORD vcFile::ReadWord(const eByteOrder ByteOrder)
{
	BYTE abyBuffer[sizeof(WORD)];	// Buffer to hold data
	WORD wReturnValue = 0;		// Return value

	/* Read 2 bytes from the file.	 */
	Read(abyBuffer, sizeof(WORD));

	/* Convert (as appropriate) the 2 bytes to a DWORD value	 */
	if (ByteOrder == vcFile::boIntelByteOrder)
	{
		/* Convert from intel-stored to internal WORD value.	 */
		wReturnValue = abyBuffer[1];
		wReturnValue = (WORD) ((wReturnValue << 8) + abyBuffer[0]);
	}
	else
	{
		/* Convert from motorola-stored to internal WORD value.	 */
		wReturnValue = abyBuffer[0];
		wReturnValue = (WORD) ((wReturnValue << 8) + abyBuffer[1]);
	}

	return wReturnValue;
}

/******************************************************************************
   Function:    vcFile::WriteString
   Purpose:     Writes a Unicode string to the file .
   Returns:     Void
   Parameters:  const wchar_t* psz - Unicode string
                const eByteOrder ByteOrder - Byte order in which to store
				DWORD in file (defaults to eIntelByteOrder).
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown
				by the call to Write() (and thus throws it).
******************************************************************************/
void vcFile::WriteString(const wchar_t* psz, eByteOrder ByteOrder)
{
	size_t iLen = wcslen(psz);

	for(size_t i = 0; i < iLen; i++)
		WriteWord(psz[i], ByteOrder);
}

void vcFile::__fprintf(vcFile* out, LPCTSTR lpszFormat, ...)
{
	TCHAR szBuffer[512];
	va_list args;

	va_start(args, lpszFormat);
	_stprintf(szBuffer, lpszFormat, args);
	va_end(args);
	out->Write(szBuffer, (DWORD)_tcslen(szBuffer));
}

void vcFile::fprintf(LPCTSTR lpszFormat, ...)
{
	TCHAR szBuffer[512];
	va_list args;

	va_start(args, lpszFormat);
	_stprintf(szBuffer, lpszFormat, args);
	va_end(args);
	Write(szBuffer, (DWORD)_tcslen(szBuffer));
}

int vcFile::GetString(char* psz, size_t iMaxSize)
{
	size_t iIdx = 0;
	for(; iIdx < iMaxSize-1;)
	{
		if(Eof())
			break;

		char ch;
		Read(&ch, 1, true);

		if(ch == 0x0A)
			break;
		else if(ch != 0x0D)
			psz[iIdx++] = ch;
	}

	psz[iIdx] = 0;
	return (int)iIdx;
}

/******************************************************************************
   Function:    vcFile::WriteDWord
   Purpose:     Writes a DWORD value.
   Returns:     Void
   Parameters:  const DWORD dwData - DWORD to write to file.
                const eByteOrder ByteOrder - Byte order in which to store
				DWORD in file (defaults to eIntelByteOrder).
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown
				by the call to Write() (and thus throws it).
******************************************************************************/
void vcFile::WriteDWord(const DWORD dwData, const eByteOrder ByteOrder)
{
	BYTE abyBuffer[sizeof(DWORD)];	// Buffer to write to file

	if (ByteOrder == vcFile::boIntelByteOrder)
	{
		/* Convert DWORD to intel byte order. */
		abyBuffer[0] = (BYTE) dwData;
		abyBuffer[1] = (BYTE) (dwData >> 8);
		abyBuffer[2] = (BYTE) (dwData >> 16);
		abyBuffer[3] = (BYTE) (dwData >> 24);

	}
	else
	{
		/* Convert DWORD to motorola byte order. */
		abyBuffer[0] = (BYTE) (dwData >> 24);
		abyBuffer[1] = (BYTE) (dwData >> 16);
		abyBuffer[2] = (BYTE) (dwData >> 8);
		abyBuffer[3] = (BYTE) dwData;
	}

	/* Write buffer to output file.	 */
	Write(abyBuffer, sizeof(DWORD));
}

/******************************************************************************
   Function:    vcFile::WriteWord
   Purpose:     Write a WORD value.
   Returns:     Void
   Parameters:  const WORD dwData - WORD to write to file.
                const eByteOrder ByteOrder - Byte order in which to store
				DWORD in file (defaults to eIntelByteOrder).
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown
				by the call to Write() (and thus throws it).
******************************************************************************/
void vcFile::WriteWord(const WORD dwData, const eByteOrder ByteOrder)
{

	BYTE abyBuffer[sizeof(WORD)];	// Buffer to write to file

	if (ByteOrder == vcFile::boIntelByteOrder)
	{

		/* Convert WORD to intel byte order.
		 */
		abyBuffer[0] = (BYTE) dwData;
		abyBuffer[1] = (BYTE) (dwData >> 8);

	}
	else
	{
		/* Convert WORD to motorola byte order.
		 */
		abyBuffer[0] = (BYTE) (dwData >> 8);
		abyBuffer[1] = (BYTE) dwData;
	}

	/* Write buffer to output file.
	 */
	Write(abyBuffer, sizeof(WORD));

}

/******************************************************************************
   Function:    vcFile::WriteByte
   Purpose:     Write a BYTE value.
   Returns:     Void
   Parameters:  const BYTE byData - BYTE to write to file.
   Throws:		BException object.
   Notes:       This function does not catch the BException thrown
				by the call to Write() (and thus throws it).
******************************************************************************/
void vcFile::WriteByte(const BYTE byData)
{
	BYTE byBuffer;				// Buffer to write to file

	byBuffer = byData;
	Write(&byBuffer, 1);
}


/******************************************************************************
   Function:    vcFile::Copy
   Purpose:     Copy one file object to another.
   Returns:     Void
   Parameters:  vcFile* pSource - Pointer to file object to copy.
                CStatusControl * pParentStatusControl - Used to indicate status.
   Throws:		BException object.
   Notes:       
******************************************************************************/
void vcFile::Copy(vcFile * pSource)
{
	BYTE abyBuffer[DEFAULT_READSIZE]; // Byte buffer
	UINT uByteCount = 0;          	  // Number of bytes read from file
	UINT uByteCountTotal = 0;

	/* Open the source and destination files.	 */
	pSource->Open(orOpenForRead);
	Open(orOpenForOverwrite);

	/* Continue reading from the source and writing to the destination until
	 * end of source file hit, or an exception occurs.		 */
	do
	{
		/* Read a chunk of data from the source, and write it to the destination 
			* (this object is the source).
			*
			* Note that we are not catching any exceptions, but rather allowing them
			* to fall back to the caller.
			*/			
		uByteCount = pSource->Read(abyBuffer, sizeof(abyBuffer));			
		Write(abyBuffer, uByteCount);
		uByteCountTotal += uByteCount;
	}

	while (uByteCount == sizeof(abyBuffer));

	/* Close the source and destination files.	 */
	pSource->Close();
	Close();
}


/******************************************************************************
   Function:    vcFile::Append
   Purpose:     Append a vcFile object to the current one.
   Returns:     Void
   Parameters:  vcFile* pSource - Source object.
   Notes:       
******************************************************************************/
void vcFile::Append(vcFile * pSource)
{
	BYTE abyBuffer[DEFAULT_READSIZE]; // Byte buffer
	UINT uByteCount = 0;		      // Number of bytes read from file
	UINT uByteCountTotal = 0;

		/* Open the source file for reading.
		 */
	pSource->Open(vcFile::orOpenForRead);

	/* Continue reading from the source and writing to the destination until
	 * end of source file hit, or an exception occurs.	 */
	do
	{
		/* Read from the source file object and write to the current file object. */
		uByteCount = pSource->Read(abyBuffer, sizeof(abyBuffer));
		Write(abyBuffer, uByteCount);
		uByteCountTotal += uByteCount;
	}
	while (uByteCount == sizeof(abyBuffer));

	pSource->Close();
}
