/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#ifndef	BErrorCodeS_H
#define	BErrorCodeS_H

#if !(defined(_WINDOWS_))

#pragma warning(disable:4201;disable:4514)
#include "windows.h"
#pragma warning(default:4201)

#endif /*!(defined(_WINDOWS_)*/

#ifdef _DEBUG
#define _BOASSERT(expr) \
	if (!(expr) && (1 == _CrtDbgReport(_CRT_ASSERT, __FILE__, __LINE__, NULL, #expr))) \
		_CrtDbgBreak();
#else
#define _BOASSERT(expr) 0;
#endif

// this define is used in assertions marking invalid paths which we should never execute
#define ExecutingInvalidPath	0

// We want errors as enums.  Remove the following define for errors as defines.
#define ENUM_ERRORS

/////////////////////////////////////////////////////////////////////////
//
//                   Error Return Codes
//
////////////////////////////////////////////////////////////////////////

enum BErrorCode
{
	#define BE_NO_ERROR						 0	 // normal, non-error return
	EID_NO_ERROR = BE_NO_ERROR,

	#define BE_USER_ABORTED					 100 // The user aborted the operation
	EID_USER_ABORTED = BE_USER_ABORTED,				 

	#define BE_INVALID_PARAMETER_PASSED      101 // An input parameter was bad
	EID_INVALID_PARAMETER_PASSED = BE_INVALID_PARAMETER_PASSED,

	#define BE_MEMORY_ERROR                  102 // Enough memory could not be allocated
	EID_MEMORY_ERROR = BE_MEMORY_ERROR,

	#define BE_FILE_TYPE_UNKNOWN             103 // The file could not be identified
	EID_FILE_TYPE_UNKNOWN = BE_FILE_TYPE_UNKNOWN,

	#define BE_UNABLE_TO_LOAD_DLL            104 // The DLL could not be found / loaded
	EID_UNABLE_TO_LOAD_DLL = BE_UNABLE_TO_LOAD_DLL,

	#define BE_FUNCTION_IN_USE               105 // Function does not support reentrancy
	EID_FUNCTION_IN_USE = BE_FUNCTION_IN_USE,

	#define BE_FUNCTION_UNABLE_TO_COMPLETE	 106 // Some non-error condition caused the function to fail
	EID_FUNCTION_UNABLE_TO_COMPLETE = BE_FUNCTION_UNABLE_TO_COMPLETE,

	#define BE_SOURCE_BUSY					 107 // The source DIB / Document is busy / can't be accessed now
	EID_SOURCE_BUSY = BE_SOURCE_BUSY,

	#define BE_LAYERS_EXCEEDED				 108 // There are too many layers to complete
	EID_LAYERS_EXCEEDED = BE_LAYERS_EXCEEDED,

	#define BE_INVALID_EXECUTION_PATH		 109 // We are executing a path we should never hit.
	EID_INVALID_EXECUTION_PATH = BE_INVALID_EXECUTION_PATH,

	#define BE_INVALID_STATE				 110 // The requested operation is not valid in the object's
	EID_INVALID_STATE = BE_INVALID_STATE,

	#define BE_WINDOWS_API_ERROR 		     117 // An error due to a Windows API failing (for example,
												 // CreatePalette fails but everything up to its call was OK)
	EID_WINDOWS_API_ERROR = BE_WINDOWS_API_ERROR,

	#define BE_UNSPECIFIED_ERROR			 118 // Something happened to fail, but not important enough 
											     // for it's own error code 
	EID_UNSPECIFIED_ERROR = BE_UNSPECIFIED_ERROR,

	#define BE_PARAMETER_NOT_FOUND			 123 // parameter not found in repository
	EID_PARAMETER_NOT_FOUND = BE_PARAMETER_NOT_FOUND,

	#define BE_NOT_IMPLEMENTED				 124 // should only occur in debug, means function not written yet
	EID_NOT_IMPLEMENTED = BE_NOT_IMPLEMENTED,

	#define BE_BUILDTYPEMISMATCH			 125 // the build type of a DLL does not match the build type (debug/release) of the app
	EID_BUILDTYPEMISMATCH = BE_BUILDTYPEMISMATCH,

	#define BE_NO_WEB_BROWSER				 128 // No web browser could be found on the system.
	EID_NO_WEB_BROWSER = BE_NO_WEB_BROWSER,

	#define	BE_PARAMETER_OUT_OF_RANGE		 129 // parameter failed a range check
	EID_PARAMETER_OUT_OF_RANGE = BE_PARAMETER_OUT_OF_RANGE,

	#define BE_PARAMETER_TYPE_ERROR		     130 // parameter does not have the expected type (bool, real, int)
	EID_PARAMETER_TYPE_ERROR = BE_PARAMETER_TYPE_ERROR,

	#define BE_PARAMETER_INVALID			 131 // value of a parameter is invalid
	EID_PARAMETER_INVALID = BE_PARAMETER_INVALID,

	#define BE_PARAMETER_ALREADY_EXISTS		 133 // attempt to define a parameter that already exists
	EID_PARAMETER_ALREADY_EXISTS = BE_PARAMETER_ALREADY_EXISTS,

	#define BE_STRING_COPY_ERROR			 135 // c function string copy failed.
	EID_STRING_COPY_ERROR = BE_STRING_COPY_ERROR,

	#define BE_NO_VIEW						 136  // the active document does not have a view
	EID_NO_VIEW = BE_NO_VIEW,

	#define BE_CREATE_FILE_ERROR             200
	EID_CREATE_FILE_ERROR = BE_CREATE_FILE_ERROR,

	#define BE_OPEN_FILE_ERROR               201
	EID_OPEN_FILE_ERROR = BE_OPEN_FILE_ERROR,

	#define BE_READ_ERROR                    202
	EID_READ_ERROR = BE_READ_ERROR,

	#define BE_WRITE_ERROR                   203
	EID_WRITE_ERROR = BE_WRITE_ERROR,

	#define BE_DELETE_ERROR					 204 
	EID_DELETE_ERROR = BE_DELETE_ERROR,

	#define BE_COPYFILE_ERROR				 205
	EID_COPYFILE_ERROR = BE_COPYFILE_ERROR,

	#define BE_MOVEFILE_ERROR				 206
	EID_MOVEFILE_ERROR = BE_MOVEFILE_ERROR,

	#define BE_EMPTY_FILE_ERROR				 207
	EID_EMPTY_FILE_ERROR = BE_EMPTY_FILE_ERROR,

	#define BE_FILESIZE_ERROR				 208
	EID_FILESIZE_ERROR = BE_FILESIZE_ERROR,
	
	#define BE_SKIPPED_OVERWRITE             209 // The user chose not to overwrite an existing file
	EID_SKIPPED_OVERWRITE = BE_SKIPPED_OVERWRITE,

	#define BE_INVALID_DIR_NAME              210
	EID_INVALID_DIR_NAME = BE_INVALID_DIR_NAME,

	#define BE_INVALID_FILE_NAME             211
	EID_INVALID_FILE_NAME = BE_INVALID_FILE_NAME,

	#define BE_STRING_LOAD_ERROR             212
	EID_STRING_LOAD_ERROR = BE_STRING_LOAD_ERROR,

	#define BE_ENOENT                        214
	EID_ENOENT = BE_ENOENT,

	#define BE_CORRUPT_FILE					 216
	EID_CORRUPT_FILE = BE_CORRUPT_FILE,

    #define BE_SAVE_ERROR                    217
    EID_SAVE_ERROR = BE_SAVE_ERROR,

	#define BE_BMP_INVALID                   300
	EID_BMP_INVALID = BE_BMP_INVALID,

	#define BE_GIF_INVALID                   304
	EID_GIF_INVALID = BE_GIF_INVALID,

	#define BE_ICO_INVALID                   305
	EID_ICO_INVALID = BE_ICO_INVALID,

	#define BE_PCX_INVALID                   314
	EID_PCX_INVALID = BE_PCX_INVALID,

	#define BE_PIC_INVALID                   315
	EID_PIC_INVALID = BE_PIC_INVALID,

	#define BE_TGA_INVALID                   317
	EID_TGA_INVALID = BE_TGA_INVALID,

	#define BE_TIF_INVALID                   318
	EID_TIF_INVALID = BE_TIF_INVALID,

	#define BE_WMF_INVALID                   319
	EID_WMF_INVALID = BE_WMF_INVALID,

	#define BE_AVI_INVALID  								 322
	EID_AVI_INVALID = BE_AVI_INVALID,

	#define BE_WAV_INVALID                   324
	EID_WAV_INVALID = BE_WAV_INVALID,

	#define BE_PSD_INVALID									 325 
	EID_PSD_INVALID = BE_PSD_INVALID,

    #define BE_POSTSCRIPT_ERROR   343
    EID_POSTSCRIPT_ERROR = BE_POSTSCRIPT_ERROR,

    #define BE_JP2_INVALID   344
    EID_JP2_INVALID = BE_JP2_INVALID,

	#define BE_GIF_BAD_CODE                  503
	EID_GIF_BAD_CODE = BE_GIF_BAD_CODE,

	#define BE_IFF_NO_COLORMAP               504
	EID_IFF_NO_COLORMAP = BE_IFF_NO_COLORMAP,

	#define BE_IFF_NO_IMAGE                  505
	EID_IFF_NO_IMAGE = BE_IFF_NO_IMAGE,

	#define BE_IFF_NO_IMAGE_DEF              506
	EID_IFF_NO_IMAGE_DEF = BE_IFF_NO_IMAGE_DEF,

	#define BE_IFF_UNSUPPORTED               507
	EID_IFF_UNSUPPORTED = BE_IFF_UNSUPPORTED,

	#define BE_JPG_BAD_HUFF_INDEX            508
	EID_JPG_BAD_HUFF_INDEX = BE_JPG_BAD_HUFF_INDEX,

	#define BE_JPG_BAD_QUANT_INDEX           509
	EID_JPG_BAD_QUANT_INDEX = BE_JPG_BAD_QUANT_INDEX,

	#define BE_JPG_BAD_RESTART_VAL           510
	EID_JPG_BAD_RESTART_VAL = BE_JPG_BAD_RESTART_VAL,

	#define BE_JPG_CORRUPT_HUFF_TABLE        511
	EID_JPG_CORRUPT_HUFF_TABLE = BE_JPG_CORRUPT_HUFF_TABLE,

	#define BE_JPG_CORRUPTED                 512
	EID_JPG_CORRUPTED = BE_JPG_CORRUPTED,

	#define BE_JPG_FACT_TOO_LARGE            513
	EID_JPG_FACT_TOO_LARGE = BE_JPG_FACT_TOO_LARGE,

	#define BE_JPG_MISSING_HUFFCODE          514
	EID_JPG_MISSING_HUFFCODE = BE_JPG_MISSING_HUFFCODE,

	#define BE_JPG_NO_ARITH                  515
	EID_JPG_NO_ARITH = BE_JPG_NO_ARITH,

	#define BE_JPG_NO_FRACTIONS              516
	EID_JPG_NO_FRACTIONS = BE_JPG_NO_FRACTIONS,

	#define BE_JPG_NO_GEN_INFO               517
	EID_JPG_NO_GEN_INFO = BE_JPG_NO_GEN_INFO,

	#define BE_JPG_NO_HUFFTABLES             518
	EID_JPG_NO_HUFFTABLES = BE_JPG_NO_HUFFTABLES,

	#define BE_JPG_SOF_IS_BAD                519
	EID_JPG_SOF_IS_BAD = BE_JPG_SOF_IS_BAD,

	#define BE_JPG_TOO_MANY_SAMP             520
	EID_JPG_TOO_MANY_SAMP = BE_JPG_TOO_MANY_SAMP,

	#define BE_JPG_UNEXPECTED_MARKER         521
	EID_JPG_UNEXPECTED_MARKER = BE_JPG_UNEXPECTED_MARKER,

	#define BE_JPG_UNKNOWN_COLORSPACE        522
	EID_JPG_UNKNOWN_COLORSPACE = BE_JPG_UNKNOWN_COLORSPACE,

	#define BE_JPG_UNKNOWN_VER               523
	EID_JPG_UNKNOWN_VER = BE_JPG_UNKNOWN_VER,

	#define BE_JPG_UNKNOWN_WIDTH             524
	EID_JPG_UNKNOWN_WIDTH = BE_JPG_UNKNOWN_WIDTH,

	#define BE_JPG_UNSUPPORTED_DATAPREC      525
	EID_JPG_UNSUPPORTED_DATAPREC = BE_JPG_UNSUPPORTED_DATAPREC,

	#define BE_JPG_UNSUPPORTED_SAMP_FACT     526
	EID_JPG_UNSUPPORTED_SAMP_FACT = BE_JPG_UNSUPPORTED_SAMP_FACT,

	#define BE_JPG_UNSUPPORTED_SOF           527
	EID_JPG_UNSUPPORTED_SOF = BE_JPG_UNSUPPORTED_SOF,

	#define BE_TIF_COMP                      531
	EID_TIF_COMP = BE_TIF_COMP,

	#define BE_TIF_INVALID_HUFF_CODE         532
	EID_TIF_INVALID_HUFF_CODE = BE_TIF_INVALID_HUFF_CODE,

	#define BE_TIF_MULTIPLANE                533
	EID_TIF_MULTIPLANE = BE_TIF_MULTIPLANE,

	#define BE_TIF_PIXEL                     534
	EID_TIF_PIXEL = BE_TIF_PIXEL,

	#define BE_TIF_PREDICTOR                 535
	EID_TIF_PREDICTOR = BE_TIF_PREDICTOR,

	#define BE_TIF_UNKNOWN_PREDICTOR         536
	EID_TIF_UNKNOWN_PREDICTOR = BE_TIF_UNKNOWN_PREDICTOR,

	#define BE_TIF_UNSUPPORTED               537
	EID_TIF_UNSUPPORTED = BE_TIF_UNSUPPORTED,

	#define BE_WMF_UNABLE_TO_GET_METABITS    538
	EID_WMF_UNABLE_TO_GET_METABITS = BE_WMF_UNABLE_TO_GET_METABITS,

	#define BE_PSD_UNSUPPORTED_COLOR_MODEL   543
	EID_PSD_UNSUPPORTED_COLOR_MODEL = BE_PSD_UNSUPPORTED_COLOR_MODEL,
	
	#define BE_PSD_UNSUPPORTED_COMPRESSION	544
	EID_PSD_UNSUPPORTED_COMPRESSION = BE_PSD_UNSUPPORTED_COMPRESSION,

	#define BE_PSD_UNSUPPORTED_BIT_DEPTH	545
	EID_PSD_UNSUPPORTED_BIT_DEPTH = BE_PSD_UNSUPPORTED_BIT_DEPTH,

	#define BE_PSD_CMYK_UNSUPPORTED			546
	EID_PSD_CMYK_UNSUPPORTED = BE_PSD_CMYK_UNSUPPORTED,

	#define BE_TIF_PAGE_NOT_PRESENT			647
	EID_TIF_PAGE_NOT_PRESENT = BE_TIF_PAGE_NOT_PRESENT,

	#define BE_TIF_ICC_ERROR				648
	EID_TIF_ICC_ERROR = BE_TIF_ICC_ERROR,

	#define BE_PNG_INVALID					 551 // <- Something generally wrong with file            Fatal
	EID_PNG_INVALID = BE_PNG_INVALID,

	#define BE_PNG_CHUNK_OUTOFORDER		 552 // <- Chunks were too out of order to process image  Fatal
	EID_PNG_CHUNK_OUTOFORDER = BE_PNG_CHUNK_OUTOFORDER,

	#define BE_PNG_CHUNK_LENGTH_ERROR		 553 // <- Chunk of set length was wrong length           Fatal
	EID_PNG_CHUNK_LENGTH_ERROR = BE_PNG_CHUNK_LENGTH_ERROR,

	#define BE_PNG_INVALID_DATA			 554 // <- Contents of one or more chunks is invalid      Non-Fatal
	EID_PNG_INVALID_DATA = BE_PNG_INVALID_DATA,

	#define BE_PNG_NOIMAGE					 555 // <- There was no image in the PNG file             Fatal
	EID_PNG_NOIMAGE = BE_PNG_NOIMAGE,

	#define BE_PNG_UNSUPPORTED_FILTER		 556 // <- Reader does not support this filter type       Fatal
	EID_PNG_UNSUPPORTED_FILTER = BE_PNG_UNSUPPORTED_FILTER,

	#define BE_PNG_UNSUPPORTED_COLOR		 557 // <- Reader does not support this color type        Fatal
	EID_PNG_UNSUPPORTED_COLOR = BE_PNG_UNSUPPORTED_COLOR,

	#define BE_PSD_MULTILAYERED			 558
	EID_PSD_MULTILAYERED = BE_PSD_MULTILAYERED,

	#define BE_ICO_NOICON					 560
	EID_ICO_NOICON = BE_ICO_NOICON,

	#define BE_GIF_UNSUPPORTED              561 // gif format not supported
	EID_GIF_UNSUPPORTED = BE_GIF_UNSUPPORTED,

	#define BE_PSD_UNSUPPORTEDFEATURE       562 // Unsupported PSD feature
	EID_PSD_UNSUPPORTEDFEATURE = BE_PSD_UNSUPPORTEDFEATURE,

	#define BE_PSD_ADJUSTMENT_LAYERS        563 // Adjustment layers encountered in PSD file
	EID_PSD_ADJUSTMENT_LAYERS = BE_PSD_ADJUSTMENT_LAYERS,

	#define BE_TIF_UNNECESSARY_IFD          564 // We don't need to read this IFD (we've already got what we need)
	EID_TIF_UNNECESSARY_IFD = BE_TIF_UNNECESSARY_IFD,

	#define BE_PSD_INVALID_ADJUSTMENTKEY    565 // Invalid adjustment key found in PSD file
	EID_PSD_INVALID_ADJUSTMENTKEY = BE_PSD_INVALID_ADJUSTMENTKEY,

	#define BE_PSD_LAYER_EFFECTS			566 // The PSD file contains a layer(s) with layer effects applied
	EID_PSD_LAYER_EFFECTS = BE_PSD_LAYER_EFFECTS,

	#define BE_PSD_OBJECT_LAYER_EFFECTS			567 // The PSD file contains an object layer effects
	EID_PSD_OBJECT_LAYER_EFFECTS = BE_PSD_OBJECT_LAYER_EFFECTS,

	#define BE_PSD_TEXT_LAYER				568 // The PSD file contains an object layer effects
	EID_PSD_TEXT_LAYER = BE_PSD_TEXT_LAYER,

	#define BE_PSD_UNSUPPORTED_COLSPACE 569 // The PSD image is in unsupported colour space
	EID_PSD_UNSUPPORTED_COLSPACE = BE_PSD_UNSUPPORTED_COLSPACE,


	#define	BE_TGA_PALETTED_ALPHA		 570 // Targa file is paletted and contains an alpha channel
	EID_TGA_PALETTED_ALPHA = BE_TGA_PALETTED_ALPHA,

	#define BE_XML_BAD_ELEMENT              572
	EID_XML_BAD_ELEMENT	= BE_XML_BAD_ELEMENT,

	#define BE_XML_BAD_ATTRIBUTE            573
	EID_XML_BAD_ATTRIBUTE = BE_XML_BAD_ATTRIBUTE,

	#define BE_XML_UNKNOWN_ELEMENT          574
	EID_XML_UNKNOWN_ELEMENT	= BE_XML_UNKNOWN_ELEMENT,

	#define BE_XML_UNKNOWN_ATTRIBUTE        575
	EID_XML_UNKNOWN_ATTRIBUTE = BE_XML_UNKNOWN_ATTRIBUTE,

	#define BE_XML_BAD_TEXT                 576
	EID_XML_BAD_TEXT = BE_XML_BAD_TEXT,

	#define	BE_XML_NOREQUIRED_ELEMENT		578
	EID_XML_NOREQUIRED_ELEMENT = BE_XML_NOREQUIRED_ELEMENT,

	#define	BE_XML_NOREQUIRED_ATTRIBUTE		579
	EID_XML_NOREQUIRED_ATTRIBUTE = BE_XML_NOREQUIRED_ATTRIBUTE,

	#define	BE_XML_UNRESOLVED_REFERENCE	     580
	EID_XML_UNRESOLVED_REFERENCE = BE_XML_UNRESOLVED_REFERENCE,

	#define	BE_XML_ILLEGAL_ATTRIBUTE 		581
	EID_XML_ILLEGAL_ATTRIBUTE = BE_XML_ILLEGAL_ATTRIBUTE,

	#define	BE_XML_ILLEGAL_ELEMENT 			582
	EID_XML_ILLEGAL_ELEMENT = BE_XML_ILLEGAL_ELEMENT,

	#define BE_IMAGE_TOO_LARGE 				 602
	EID_IMAGE_TOO_LARGE = BE_IMAGE_TOO_LARGE,

	#define BE_NO_THUMBNAIL 				 603
	EID_NO_THUMBNAIL = BE_NO_THUMBNAIL,

	#define BE_CONDUIT_READS_ONLY			 650
	EID_CONDUIT_READS_ONLY = BE_CONDUIT_READS_ONLY,

	#define BE_CONDUIT_WRITES_ONLY			 651
	EID_CONDUIT_WRITES_ONLY = BE_CONDUIT_WRITES_ONLY,

	#define BE_TWAIN_ERROR_OPENING_DS        700
	EID_TWAIN_ERROR_OPENING_DS = BE_TWAIN_ERROR_OPENING_DS,

	#define BE_TWAIN_ERROR_OPENING_DSM       701
	EID_TWAIN_ERROR_OPENING_DSM = BE_TWAIN_ERROR_OPENING_DSM,

	#define BE_TWAIN_ERROR_LOADING_DRIVER    702
	EID_TWAIN_ERROR_LOADING_DRIVER = BE_TWAIN_ERROR_LOADING_DRIVER,

	#define BE_TWAIN_GENERL_SELECTION_ERROR  703
	EID_TWAIN_GENERL_SELECTION_ERROR = BE_TWAIN_GENERL_SELECTION_ERROR,

	#define BE_TWAIN_ERROR_CLOSING_DS        704
	EID_TWAIN_ERROR_CLOSING_DS = BE_TWAIN_ERROR_CLOSING_DS,

	#define BE_TWAIN_ERROR_CLOSING_DSM       705
	EID_TWAIN_ERROR_CLOSING_DSM = BE_TWAIN_ERROR_CLOSING_DSM,

	#define BE_TWAIN_ERROR_DISABLING_DS      706
	EID_TWAIN_ERROR_DISABLING_DS = BE_TWAIN_ERROR_DISABLING_DS,

	#define BE_TWAIN_ERROR_ENABLING_DS       707
	EID_TWAIN_ERROR_ENABLING_DS = BE_TWAIN_ERROR_ENABLING_DS,

	#define BE_TWAIN_ERROR					 708
	EID_TWAIN_ERROR = BE_TWAIN_ERROR,

	#define BE_TWAIN_CAPNOTSUPPORTED		 709
	EID_TWAIN_CAPNOTSUPPORTED = BE_TWAIN_CAPNOTSUPPORTED,

	#define BE_UNABLE_TO_OPEN_CLIPBOARD     803
	EID_UNABLE_TO_OPEN_CLIPBOARD = BE_UNABLE_TO_OPEN_CLIPBOARD,

	#define BE_UNABLE_TO_GET_CLIPBOARD_DATA 804
	EID_UNABLE_TO_GET_CLIPBOARD_DATA = BE_UNABLE_TO_GET_CLIPBOARD_DATA,

	#define BE_NO_REC_CLIPBOARD_FORMATS     805
	EID_NO_REC_CLIPBOARD_FORMATS = BE_NO_REC_CLIPBOARD_FORMATS,

	#define BE_EMPTY_CLIPBOARD_DATA			810
	EID_EMPTY_CLIPBOARD_DATA = BE_EMPTY_CLIPBOARD_DATA,

	#define BE_AVI_NO_AVI_HEADER            1000
	EID_AVI_NO_AVI_HEADER = BE_AVI_NO_AVI_HEADER,

	#define BE_AVI_NO_STREAM_INFO           1001
	EID_AVI_NO_STREAM_INFO = BE_AVI_NO_STREAM_INFO,

	#define BE_AVI_NO_STREAM_HEADER         1002
	EID_AVI_NO_STREAM_HEADER = BE_AVI_NO_STREAM_HEADER,

	#define BE_AVI_NO_STREAM_FORMAT         1003
	EID_AVI_NO_STREAM_FORMAT = BE_AVI_NO_STREAM_FORMAT,

	#define BE_AVI_NO_MOVI                  1004
	EID_AVI_NO_MOVI = BE_AVI_NO_MOVI,

	#define BE_AVI_CANT_FIND_FRAME          1005
	EID_AVI_CANT_FIND_FRAME = BE_AVI_CANT_FIND_FRAME,

	#define BE_AVI_NO_DECOMPRESSOR          1006
	EID_AVI_NO_DECOMPRESSOR = BE_AVI_NO_DECOMPRESSOR,

	#define BE_AVI_DECOMP_FORMAT_SIZE       1007
	EID_AVI_DECOMP_FORMAT_SIZE = BE_AVI_DECOMP_FORMAT_SIZE,

	#define BE_AVI_DECOMPRESS_BEGIN         1008
	EID_AVI_DECOMPRESS_BEGIN = BE_AVI_DECOMPRESS_BEGIN,

	#define BE_AVI_DECOMPRESS               1009
	EID_AVI_DECOMPRESS = BE_AVI_DECOMPRESS,

	#define BE_AVI_DECOMPRESS_END           1010
	EID_AVI_DECOMPRESS_END = BE_AVI_DECOMPRESS_END,

	#define BE_AVI_CANT_LOAD_LIB            1011
	EID_AVI_CANT_LOAD_LIB = BE_AVI_CANT_LOAD_LIB,

	#define     BE_AVI_CANT_SAVE_TO_RAM     1012
	EID_AVI_CANT_SAVE_TO_RAM  = BE_AVI_CANT_SAVE_TO_RAM,

// Compression errors
	#define BE_COMPRESSION_ERROR            1100
	EID_COMPRESSION_ERROR = BE_COMPRESSION_ERROR,

	#define BE_NEED_MORE_TO_DECOMPRESS      1101
	EID_NEED_MORE_TO_DECOMPRESS = BE_NEED_MORE_TO_DECOMPRESS,

	// New XML error codes
	#define BE_XML_PARSE_ERROR              1560
	EID_XML_PARSE_ERROR	= BE_XML_PARSE_ERROR,

//  PhotoShop plugin host range 20000 - 21000
	#define BE_PSPLUGIN_HOSTINSUFFICIENT					 20001
	EID_PSPLUGIN_HOSTINSUFFICIENT = BE_PSPLUGIN_HOSTINSUFFICIENT,

	#define BE_PSPLUGIN_PROPERTYUNDEFINED					 20002
	EID_PSPLUGIN_PROPERTYUNDEFINED = BE_PSPLUGIN_PROPERTYUNDEFINED,

	#define BE_PSPLUGIN_LOADFAILED					 20003
	EID_PSPLUGIN_LOADFAILED = BE_PSPLUGIN_LOADFAILED,

	#define BE_PSPLUGIN_FEATURENOTSUPPORT					 20004
	EID_PSPLUGIN_FEATURENOTSUPPORT = BE_PSPLUGIN_FEATURENOTSUPPORT,

	#define BE_PSPLUGIN_INVALIDPOINT					 20005
	EID_PSPLUGIN_INVALIDPOINT = BE_PSPLUGIN_INVALIDPOINT,

	#define BE_PSPLUGIN_BADPARAMS					 20006
	EID_PSPLUGIN_BADPARAMS = BE_PSPLUGIN_BADPARAMS,

	#define BE_PSPLUGIN_CANTREADFORMAT					 20007
	EID_PSPLUGIN_CANTREADFORMAT = BE_PSPLUGIN_CANTREADFORMAT,

	#define BE_PSPLUGIN_NOSCANNER					 20008
	EID_PSPLUGIN_NOSCANNER = BE_PSPLUGIN_NOSCANNER,

	#define BE_PSPLUGIN_SCANNERPROBLEM					 20009
	EID_PSPLUGIN_SCANNERPROBLEM = BE_PSPLUGIN_SCANNERPROBLEM,

	#define BE_PSPLUGIN_GPF					 20010
	EID_PSPLUGIN_GPF = BE_PSPLUGIN_GPF,

	#define BE_PSPLUGIN_UNKNOWNERROR					 20999
	EID_PSPLUGIN_UNKNOWNERROR = BE_PSPLUGIN_UNKNOWNERROR,
//  End of Photoshop Plugin range

	#define BE_UNDEFINED_ERROR             65535
	EID_UNDEFINED_ERROR = BE_UNDEFINED_ERROR,

// Added by OM;
	#define BE_NOTHING_TO_PROCESS					 22000
	EID_NOTHING_TO_PROCESS = BE_NOTHING_TO_PROCESS,
};

#endif	// BErrorCodeS_H
