/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include "xarlib.h"
#include "cxfile.h"
#include "ccfile.h"

class CImporter : public CXarImport
{
public:
	CImporter();
	virtual ~CImporter();

	virtual BOOL PrepareImport(TCHAR* pFileName);
	virtual BOOL PrepareImport(IStream* pStream);
	virtual BOOL PrepareImport(BYTE* pBuffer, ULONG Size);
	virtual BOOL SetHandler(void* pMagic, RecordHandler* pHandlerFunk);
	virtual BOOL DoImport(BOOL bDeleteRecordAfter = TRUE);
	virtual LONG GetProgress();
 
protected:
	CXaraFile* m_pCXImpFile;
	CCDiskFile* m_pCCImpDiskFile;
	CCMemFile* m_pCCImpMemFile;
	CCOleStream* m_pCCImpOleFile;

	LONG m_StreamLen;
	LONG m_StreamPos;
};

class CExporter : public CXarExport
{
public:
	CExporter();
	virtual ~CExporter();

	virtual BOOL StartExport(TCHAR* pFileName);
	virtual BOOL StartExport(IStream* pStream);
	virtual BOOL StartBufferedExport();
	virtual BOOL WriteRecord(CXaraFileRecord* pRecord, ULONG* pRecordNum = NULL);
	virtual BOOL WriteZeroSizedRecord(ULONG Tag, ULONG* pRecordNum = NULL);
	virtual BOOL GetExportBuffer(BYTE** ppBuffer, ULONG* pSize);
	virtual LONG GetRecordCount();

protected:
	CXaraFile* m_pCXExpFile;
	CCDiskFile* m_pCCExpDiskFile;
	CCMemFile* m_pCCExpMemFile;
	CCOleStream* m_pCCExpOleFile;
};


CXarImport* XarLibrary::CreateImporter()
{
	return(new CImporter);
}

CXarExport* XarLibrary::CreateExporter()
{
	return(new CExporter);
}

CImporter::CImporter()
{
	m_pCXImpFile = NULL;
	m_pCCImpDiskFile = NULL;
	m_pCCImpMemFile = NULL;
	m_pCCImpOleFile = NULL;

	m_StreamLen = 0;
	m_StreamPos = 0;
}

CImporter::~CImporter()
{
	if (m_pCXImpFile)
	{
		m_pCXImpFile->Close();
	
		delete m_pCXImpFile;
		m_pCXImpFile = NULL;
	}

	if (m_pCCImpDiskFile)
	{
		m_pCCImpDiskFile->close();
		delete m_pCCImpDiskFile;
		m_pCCImpDiskFile = NULL;
	}

	if (m_pCCImpMemFile)
	{
		m_pCCImpMemFile->close();
		delete m_pCCImpMemFile;
		m_pCCImpMemFile = NULL;
	}

	if (m_pCCImpOleFile)
	{
		m_pCCImpOleFile->close();
		delete m_pCCImpOleFile;
		m_pCCImpOleFile = NULL;
	}
}


BOOL CImporter::PrepareImport(TCHAR* pFileName)
{
	if (m_pCXImpFile || m_pCCImpDiskFile || m_pCCImpMemFile || m_pCCImpOleFile)
	{
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCImpDiskFile = new CCDiskFile(1024, FALSE, TRUE);
	if (!m_pCCImpDiskFile)
		return(E_OUTOFMEMORY);

	PathName FileName(pFileName);

	if (!m_pCCImpDiskFile->open(FileName, ios::in | ios::binary))
	{
		// Failed to open the file...
		TRACE("Failed to open file in PrepareImport\n");
		return(FALSE);
	}

	m_StreamLen = m_pCCImpDiskFile->Size();

	// Create our CXaraFile object
	m_pCXImpFile = new CXaraFile;
	if (!m_pCXImpFile)
		return(FALSE);

	// Initialise the standard handlers
	BOOL ok = m_pCXImpFile->SetUpHandlers();

	if (ok)
		ok = m_pCXImpFile->OpenToRead(m_pCCImpDiskFile);

	return(ok);
}

BOOL CImporter::PrepareImport(IStream* pStream)
{
	if (m_pCXImpFile || m_pCCImpDiskFile || m_pCCImpMemFile || m_pCCImpOleFile)
	{
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCImpOleFile = new CCOleStream(pStream, 1024, FALSE, FALSE);
	if (!m_pCCImpOleFile)
		return(E_OUTOFMEMORY);

	// If we get to here then the new CCOleStream has taken ownership of our IStream reference
	// so we must AddRef it to prevent the client's stream being deleted when the CCOleStream goes away
	pStream->AddRef();

//	LARGE_INTEGER Pos = {0};
//	HRESULT hRes = pStream->Seek(Pos, STREAM_SEEK_SET, NULL);

	m_StreamLen = m_pCCImpOleFile->Size();

	// Create our CXaraFile object
	m_pCXImpFile = new CXaraFile;
	if (!m_pCXImpFile)
		return(FALSE);

	// Initialise the standard handlers
	BOOL ok = m_pCXImpFile->SetUpHandlers();

	if (ok)
		ok = m_pCXImpFile->OpenToRead(m_pCCImpOleFile);

	return(ok);
}

BOOL CImporter::PrepareImport(BYTE* pBuffer, ULONG Size)
{
	if (m_pCXImpFile || m_pCCImpDiskFile || m_pCCImpMemFile || m_pCCImpOleFile)
	{
		return(FALSE);
	}

	TCHAR PathBuf[MAX_PATH];
	if (GetTempPath(MAX_PATH, PathBuf) == 0)
		return(FALSE);

	TCHAR Buffer[MAX_PATH];
	if (GetTempFileName(PathBuf, _T("FLI"), 0, Buffer) == 0)
		return(FALSE);

	CCDiskFile TempFile(1024,FALSE,TRUE);
	PathName FileName(Buffer);
	if (!TempFile.open(FileName, ios::out | ios::binary))
	{
		// Failed to open the file...
		TRACE("Failed to open temp file in PrepareImport\n");
		return(FALSE);
	}
	if (TempFile.write(pBuffer, Size).bad())
	{
		// Failed to write the file...
		TRACE("Failed to write temp file in PrepareImport\n");
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCImpDiskFile = new CCDiskFile(1024, FALSE, TRUE);
	if (!m_pCCImpDiskFile)
		return(E_OUTOFMEMORY);

	if (!m_pCCImpDiskFile->open(FileName, ios::in | ios::binary))
	{
		// Failed to open the file...
		TRACE("Failed to open temp file in PrepareImport\n");
		return(FALSE);
	}

	m_StreamLen = m_pCCImpDiskFile->Size();

//	// First off, we have to try and open the file
//	m_pCCImpMemFile = new CCMemFile(FALSE, TRUE);
//	if (!m_pCCImpMemFile)
//		return(FALSE);

//	if (!m_pCCImpMemFile->open(pBuffer, Size, CCMemRead))
//	{
//		// Failed to open the file...
//		TRACE("Failed to open file in PrepareImport\n");
//		return(FALSE);
//	}

	// Create our CXaraFile object
	m_pCXImpFile = new CXaraFile;
	if (!m_pCXImpFile)
		return(FALSE);

	// Initialise the standard handlers
	BOOL ok = m_pCXImpFile->SetUpHandlers();

	if (ok)
		ok = m_pCXImpFile->OpenToRead(m_pCCImpDiskFile);

	return(ok);
}

BOOL CImporter::SetHandler(void* pMagic, RecordHandler* pHandlerFunc)
{
	// Inform the standard record handler of the callback function
	if (!m_pCXImpFile)
		return(FALSE);

	m_pCXImpFile->SetExternalRecordHandler(pMagic, pHandlerFunc);

	return(TRUE);
}

BOOL CImporter::DoImport(BOOL bDeleteRecordAfter)
{
	BOOL ok = TRUE;

	while (!m_pCXImpFile->IsEndOfFile() && ok)
		ok = m_pCXImpFile->ReadNextRecord(bDeleteRecordAfter);

	if (m_pCCImpDiskFile)
	{
		m_pCCImpDiskFile->close();
		delete m_pCCImpDiskFile;
		m_pCCImpDiskFile = NULL;
	}

	if (m_pCCImpMemFile)
	{
		m_pCCImpMemFile->close();
		delete m_pCCImpMemFile;
		m_pCCImpMemFile = NULL;
	}

	return(ok);
}


LONG CImporter::GetProgress()
{
	if (m_StreamLen == 0)
		return(0);
	
	m_StreamPos = m_pCXImpFile->GetFilePos();

	return((LONG)(100.0 * ((double)m_StreamPos / (double)m_StreamLen)));
}


CExporter::CExporter()
{
	m_pCXExpFile = NULL;
	m_pCCExpDiskFile = NULL;
	m_pCCExpMemFile = NULL;
	m_pCCExpOleFile = NULL;
}

CExporter::~CExporter()
{
	if (m_pCXExpFile)
	{
		m_pCXExpFile->Close();
		delete m_pCXExpFile;
		m_pCXExpFile = NULL;
	}

	if (m_pCCExpDiskFile)
	{
		m_pCCExpDiskFile->close();
		delete m_pCCExpDiskFile;
		m_pCCExpDiskFile = NULL;
	}

	if (m_pCCExpMemFile)
	{
		m_pCCExpMemFile->close();
		delete m_pCCExpMemFile;
		m_pCCExpMemFile = NULL;
	}

	if (m_pCCExpOleFile)
	{
		m_pCCExpOleFile->close();
		delete m_pCCExpOleFile;
		m_pCCExpOleFile = NULL;
	}
}

BOOL CExporter::StartExport(TCHAR* pFileName)
{
	if (m_pCXExpFile || m_pCCExpDiskFile || m_pCCExpMemFile || m_pCCExpOleFile)
	{
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCExpDiskFile = new CCDiskFile(1024, FALSE, FALSE);
	if (!m_pCCExpDiskFile)
		return(FALSE);

	PathName FileName(pFileName);

	if (!m_pCCExpDiskFile->open(FileName, ios::out | ios::binary))
	{
		// Failed to open the file...
		TRACE("Failed to open file in PrepareExport\n");
		return(FALSE);
	}

	// Create our CXaraFile object
	m_pCXExpFile = new CXaraFile;
	if (!m_pCXExpFile)
		return(FALSE);

	BOOL ok = m_pCXExpFile->OpenToWrite(m_pCCExpDiskFile);

	return(ok);
}

BOOL CExporter::StartExport(IStream* pStream)
{
	if (m_pCXExpFile || m_pCCExpDiskFile || m_pCCExpMemFile || m_pCCExpOleFile)
	{
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCExpOleFile = new CCOleStream(pStream, 1024, FALSE, FALSE);
	if (!m_pCCExpOleFile)
		return(FALSE);

	// If we get to here then the new CCOleStream has taken ownership of our IStream reference
	// so we must AddRef it to prevent the client's stream being deleted when the CCOleStream goes away
	pStream->AddRef();

	// Create our CXaraFile object
	m_pCXExpFile = new CXaraFile;
	if (!m_pCXExpFile)
		return(FALSE);

	BOOL ok = m_pCXExpFile->OpenToWrite(m_pCCExpOleFile);

	return(ok);
}

BOOL CExporter::StartBufferedExport()
{
	if (m_pCXExpFile || m_pCCExpDiskFile || m_pCCExpMemFile || m_pCCExpOleFile)
	{
		return(FALSE);
	}

	// First off, we have to try and open the file
	m_pCCExpMemFile = new CCMemFile(FALSE, FALSE);
	if (!m_pCCExpMemFile)
		return(FALSE);

	if (!m_pCCExpMemFile->open(NULL, 0, CCMemWrite))
	{
		// Failed to open the file...
		TRACE("Failed to open file in PrepareExport\n");
		return(FALSE);
	}

	// Create our CXaraFile object
	m_pCXExpFile = new CXaraFile;
	if (!m_pCXExpFile)
		return(FALSE);

	BOOL ok = m_pCXExpFile->OpenToWrite(m_pCCExpMemFile);

	return(ok);
}

BOOL CExporter::GetExportBuffer(BYTE** ppBuffer, ULONG* pSize)
{
	if (!ppBuffer || !pSize)
		return(FALSE);
	
	if (!m_pCCExpMemFile)
		return(FALSE);

	if (!m_pCCExpMemFile->GetBuffer(ppBuffer, pSize))
		return(FALSE);

	return(TRUE);
}

LONG CExporter::GetRecordCount()
{
	return m_pCXExpFile->GetRecordNum();
}

BOOL CExporter::WriteRecord(CXaraFileRecord* pRecord, ULONG* pRecordNum)
{
	if (!pRecord)
		return(FALSE);

	if (!m_pCXExpFile)
		return(FALSE);

	ULONG RecordNum = m_pCXExpFile->Write(pRecord);
	if (pRecordNum)
		*pRecordNum = RecordNum;

	return(TRUE);
}

BOOL CExporter::WriteZeroSizedRecord(ULONG Tag, ULONG* pRecordNum)
{
	if (!m_pCXExpFile)
		return(FALSE);

	CXaraFileRecord Rec(Tag, 0);
	Rec.Init();
	ULONG RecordNum = m_pCXExpFile->Write(&Rec);
	if (pRecordNum)
		*pRecordNum = RecordNum;

	return(TRUE);
}
