/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/winoil/camelot.cpp 63    14/05/04 11:12 Marc $
// camelot.cpp : Defines the class behaviors for the application.


#include "stdafx.h"
#include "camelot.h"


/********************************************************************************************

>	BOOL IsWin32s()

	Author:		Tim Browse
	Created:	21/9/93
	Returns:	TRUE if running Win32s, or the debug preference ForceWinVer has been set 
				to pretend that this is the case; FALSE otherwise. TRUE for both Win16+win32s
				and Chicago. FALSE for NT only.
	Purpose:	For use by 32-bit code that needs to change its behaviour depending on 
				whether it is running under Win32s or Win32 (NT).
				This function only exists in 32-bit builds. It is in-line in camelot.h
	SeeAlso:	RUNNING_WIN32S();IsWin32c;IsWin32NT

********************************************************************************************/

/********************************************************************************************

>	BOOL IsWin32c()

	Author:		Andy Pennell
	Created:	15/2/94
	Returns:	TRUE if running Win32c (e.g. Chicago), FALSE otherwise (e.g. NT or Win32s).
	Purpose:	For use by 32-bit code that needs to change its behaviour depending on 
				whether it is running under Win32s or Win32 (NT).
				This function only exists in 32-bit builds. It is in-line in camelot.h
	SeeAlso:	RUNNING_WIN32S();IsWin32s;IsWin32NT

********************************************************************************************/

/********************************************************************************************

>	BOOL IsWin32NT()

	Author:		Andy Pennell
	Created:	15/2/94
	Returns:	TRUE if running Win32NT, FALSE otherwise (e.g. Chicago, Win32s).
	Purpose:	For use by 32-bit code that needs to change its behaviour depending on 
				whether it is running under Win32s or Win32 (NT).
				This function only exists in 32-bit builds. It is in-line in camelot.h
	SeeAlso:	RUNNING_WIN32S();IsWin32c;IsWin32s

********************************************************************************************/


BOOL sbWinver32c = FALSE;


/********************************************************************************************

>	static BOOL IsNewWindowsUI()

	Author:		Andy Pennell
	Created:	1/11/94
	Returns:	TRUE if running Win4 or later.
	Purpose:	Determine whether the 'new' Chicago-style UI is required.
	Scope:		public static

********************************************************************************************/

BOOL sbNewWindowsUI = FALSE;					// TRUE for Win >=4, FALSE if older

BOOL IsNewWindowsUI()
{
	return sbNewWindowsUI;
}

/********************************************************************************************

>	static BOOL IsPartialNewWindowsUI()

	Author:		Neville Humphrys
	Created:	28/2/96
	Returns:	TRUE if running Windows NT 3.51 or WIndows 95 or later.
	Purpose:	Determine whether we are running on NT 3.51 or Windows 95 or later and hence
				have at least partial support for the 'new' Chicago-style UI.
				Useful as things like the system tool tips colours are only defined on
				Windows	95 or NT 3.51 or later.
	Scope:		public static

********************************************************************************************/

BOOL sbPartialNewWindowsUI = FALSE;				// TRUE for Win >=351, FALSE if older

BOOL IsPartialNewWindowsUI()
{
	return sbPartialNewWindowsUI;
}


/********************************************************************************************

>	BOOL InitFlareLibrary()

	Author:		Gerry
	Created:	17/08/2004
	Inputs:		
	Returns:	TRUE if successfull, FALSE if failed.
	Purpose:	Initialises the library

********************************************************************************************/

BOOL InitFlareLibrary()
{
	// work out host OS. Would like to use Force variable, but cannot call prefs code yet
	const DWORD ver = GetVersion();

	BOOL fBadWin32s = FALSE;
	BOOL fNoMBCSSupport = FALSE;

	sbNewWindowsUI = FALSE;						// We don't have a new shell UI by default
	sbPartialNewWindowsUI = FALSE;				// We don't have a partial new shell UI by default
TRACEUSER("Neville","OS Major version %d Minor Version %d\n",LOBYTE(LOWORD(ver)),HIBYTE(LOWORD(ver)));

	if (ver & 0x80000000)
	{
		if (LOBYTE(LOWORD(ver)) > 3)
		{
			// Windows 95
			sbNewWindowsUI = TRUE;			// We have a new shell UI
			sbPartialNewWindowsUI = TRUE;	// We have a greater than partial new shell UI
			sbWinver32c = TRUE;				// Chicago is 4.0 - check win32s bit as Cairo
		}
		else
		{
			return(FALSE);					// Indicate failure as we can't run on Win32s
		}
	}
	else
	{
		// Some form of NT
		sbWinver32c = FALSE;

		if (LOBYTE(LOWORD(ver)) > 3)
		{
			sbNewWindowsUI = TRUE;				// ready for Cario or NT 4
			sbPartialNewWindowsUI = TRUE;			// We at least have partial New shell UI
		}
		else if ((LOBYTE(LOWORD(ver)) == 3) && (HIBYTE(LOWORD(ver)) > 50))
		{
			// We are on NT 3.51 and so support at least some of the new UI features
			sbPartialNewWindowsUI = TRUE;			// We at least have partial New shell UI
		}
	}

	return(TRUE);
}



/********************************************************************************************

>	BOOL ExitFlareLibrary()

	Author:		Gerry
	Created:	17/08/2004
	Inputs:		
	Returns:	TRUE if succesful, FALSE otherwise (not that you can do much about it)

********************************************************************************************/

BOOL ExitFlareLibrary()
{
	return(TRUE);
}



#ifdef _DEBUG

void CamTrace(const char *lpszFormat, ...)
{
	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	CHAR szBuffer[512];

	nBuf = vsnprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);
	OutputDebugStringA(szBuffer);
	va_end(args);
}

void CamTrace(const wchar_t *lpszFormat, ...)
{
	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	WCHAR szBuffer[512];

	nBuf = _vsnwprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);
	OutputDebugStringW(szBuffer);
	va_end(args);
}

BOOL IsUserName(const char *wanted)
{
	const char* current_user = NULL;

	if (current_user == NULL)
	{
		// for efficiency's sake the user is only read once then remembered

		// check for "<space>-u" in the command-line
		const char *Name = strstr( GetCommandLineA(), " -u" );
		if (Name)
			current_user = Name + 3;
		// if that fails try the environment
		else
			current_user = getenv("LOGNAME");
	
		if (current_user == NULL)
			current_user = "";
	}

	if (current_user[0] == 0)
		return FALSE;										// if no env var found
	else
	{
		if (lstrcmpiA(current_user, "ALL") == 0)			// internal
 			return TRUE;								// show all TRACEs
	}

	return lstrcmpiA(current_user, wanted) == 0;			// internal
}

#endif
