/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /camelot/intel/CCMATHS.CPP 15    28/11/94 12:45 Alex $


#include "stdafx.h"
#include "fixedasm.h"
#include "xlongasm.h"
#include "ccmaths.h"
#include "fixed16.h"


/********************************************************************************************

>	XLONG Mul(const LONG operand1, const LONG operand2) 

	Author: 	Mario Shamtani
	Created:	26/4/93
	Inputs:		Two LONG values
	SeeAlso:	Xlongasm.h
	Outputs:	None
	Returns:	A reference to an xlong with result of multiplication.
	Purpose:	Multiplication of two 32-bit signed numbers (xlongs). 
	Errors:		None.

********************************************************************************************/
/*
Technical notes:

********************************************************************************************/

XLONG Mul(const LONG operand1, const LONG operand2)
{
	XLONG result;

	XMul32(operand1, operand2, result);

	return result;
}

/********************************************************************************************

>	XLONG Asl (const LONG, UINT)

	Author: 	Mario Shamtani
	Created:	5/5/93
	Inputs:		LONG - value to be shifted
				UINT - shift count
	Outputs:	None
	Returns:	XLONG - shifted number. 
	Purpose:	Performs arithmetic shift left. 
	Errors:		None.

********************************************************************************************/
/*
Technical notes:

	Using the ASL provided with xlong for now.

********************************************************************************************/

XLONG Asl (const LONG value, UINT shift)
{   
	XLONG result;
	
	result = MakeXLong(value);
	
	return 	(result << shift);

}

/********************************************************************************************

>	XLONG Asr (const LONG, UINT)

	Author: 	Mario Shamtani
	Created:	5/5/93
	Inputs:		LONG - value to be shifted
				UINT - shift count
	Outputs:	None
	Returns:	XLONG - shifted number. 
	Purpose:	Performs arithmetic shift right. 
	Errors:		None.

********************************************************************************************/
/*
Technical notes:

	Using the ASR provided with xlong for now.

********************************************************************************************/

XLONG Asr (const LONG value, UINT shift)
{   
	XLONG result;

	result = MakeXLong(value);
	
	return 	(result >> shift);
}

/********************************************************************************************

>	LONG MulDiv32By32(const LONG operand1, const LONG operand2, const LONG operand3) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	SeeAlso:	Xlongasm.h
	Inputs:		operand1 - multiplicand 1
				operand2 - multiplicand 2
				operand3 - divisor
	Outputs:	None
	Returns:	LONG -  result of multiply-divide. 
	Purpose:	Multiplies two 32-bit values and divides the result by another 32-bit value. 
	Errors:		None.

********************************************************************************************/
/*
Technical notes:

********************************************************************************************/

LONG MulDiv32By32(const LONG operand1, const LONG operand2, const LONG operand3)  
{
	return Mul32Div32( operand1, operand2, operand3 );
}

/**********************************************************************************************

>	XLONG MakeXLong (const LONG operand)
 	
	Author: 	Mario Shamtani
	Created:	5/5/93
	SeeAlso:	Xlongasm.h
	Inputs:		Long value to be converted.
	Outputs:	None.
	Returns:	an xlong value.
	Purpose:	To convert a long value to an xlong. 
	Errors:		None.

**********************************************************************************************/
/*
Technical notes:

**********************************************************************************************/
  								
XLONG MakeXLong (const LONG operand)
{
	XLONG result;
	
	LongToXlong(operand, result);

	return result;
}

/**********************************************************************************************

>	XLONG MakeXLong (const double operand)
 	
	Author: 	Mario Shamtani
	Created:	5/5/93
	SeeAlso:	Xlongasm.h
	Inputs:		double value to be converted.
	Outputs:	None.
	Returns:	an xlong value.
	Purpose:	To convert a double value to an xlong. 
	Errors:		None.

**********************************************************************************************/
/*
Technical notes:

**********************************************************************************************/
  								
XLONG MakeXLong(const double operand)
{
	xlong result;

	DoubleToXlong(operand, result);

	return result;
}
/********************************************************************************************

>	FIXED16 Div32By32( LONG arg1, LONG arg2 )

	Author: 	Andy
	Created:	11/5/93
	Inputs:		two signed 32-bit numbers
	Outputs:	None
	Returns:	FIXED16 result of arg1/arg2
	Scope:		Private to the CCMATHS library.
	Purpose:	Obtain a fixed-point representation of the ratio between two signed
			numbers.
	Errors:		None. Div0 and overflow will create a div0 exception.
	See Also:	InternalDiv32by32

This fn is inline.

********************************************************************************************/

/********************************************************

< Maths Types
< Maths Conversion Table
< Conversion

comment:
\pard\plain
\trowd \trgaph108\trleft-108 
\cellx1593\cellx3294\cellx4995\cellx6696\cellx8397\cellx10098\pard\plain 
\qc \intbl \fs20\lang2057 {\b From}{\b  }{\b \\}{\b  }{\b To}{\b \cell }{\b int}{\b \cell }
{\b LONG}{\b \cell }{\b XLONG}{\b \cell }{\b FIXED16}{\b \cell }{\b double}{\b \cell }\row 
\qc\intbl {\b int}{\b \cell }-\cell auto
\cell constructor
\par operator=\cell constructor
\par operator=\cell auto\cell \pard \intbl \row \pard \qc\intbl {\b LONG}{\b \cell }
built-in cast\cell -\cell MakeXlong()
\par constructor
\par operator=
\par cast operator\cell operator=
\par constructor\cell auto\cell \pard \intbl \row \pard \qc\intbl {\b XLONG}{\b \cell }
MakeInt()\cell MakeLong()\cell -\cell MakeFixed16()\cell MakeDouble()\cell \pard 
\intbl \row \pard \qc\intbl {\b FIXED16}{\b \cell }MakeInt()\cell MakeLong()\cell 
MakeXlong()\cell -\cell MakeDouble()\cell \row \qc\intbl {\b double}
{\b \cell }explicit cast\cell explicit cast\cell MakeXlong()\cell operator=
\par constructor\cell -\cell \pard \intbl \row \pard 

******************************************************************/
