/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
/********************************************************************************************

>	CCPanose.cpp

	Author:		Andy Hayward
	Created:	24/05/06
	SeeAlso:	<???>

********************************************************************************************/

#include "stdafx.h"
#include "ccpanose.h"

/********************************************************************************************

>	CCPanose::CCPanose()

	Author:		Andy Hayward
	Created:	24/05/96
	Inputs:		-
	Returns:	-
	Purpose:	Default constructor for class CCPanose

********************************************************************************************/

CCPanose::CCPanose()
{
	SetAllToAny();
}

/********************************************************************************************

>	CCPanose::CCPanose(const PANOSE &PanoseNumber)

	Author:		Andy Hayward
	Created:	15/08/96
	Inputs:		PanoseNumber	- panose number to copy
	Returns:	-
	Purpose:	Constructor

********************************************************************************************/

CCPanose::CCPanose(const PANOSE &PanoseNumber)
{
	mFamilyType			= PanoseNumber.bFamilyType;
    mSerifStyle			= PanoseNumber.bSerifStyle; 
    mWeight				= PanoseNumber.bWeight; 
    mProportion			= PanoseNumber.bProportion; 
    mContrast			= PanoseNumber.bContrast;
    mStrokeVariation	= PanoseNumber.bStrokeVariation; 
    mArmStyle			= PanoseNumber.bArmStyle; 
    mLetterform			= PanoseNumber.bLetterform; 
    mMidline			= PanoseNumber.bMidline; 
    mXHeight			= PanoseNumber.bXHeight; 	
}

/********************************************************************************************

>	void CCPanose::SetAllToAny()

	Author:		Andy Hayward
	Created:	24/07/96
	Inputs:		-
	Returns:	-
	Purpose:	Sets all 10 components of the Panose number to PAN_ANY

********************************************************************************************/

void CCPanose::SetAllToAny()
{
	SetAll(PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY, PAN_ANY);
}

/********************************************************************************************

>	void CCPanose::SetAll(
			BYTE FamilyType,
			BYTE SerifStyle,
			BYTE Weight,
			BYTE Proportion,
			BYTE Contrast,
			BYTE StrokeVariation,
			BYTE ArmStyle,
			BYTE LetterForm,
			BYTE Midline,
			BYTE XHeight);

	Author:		Andy Hayward
	Created:	24/07/96
	Inputs:		Obvious really - the 10 components of the Panose number
	Returns:	-
	Purpose:	Sets the components of the Panose number to those given

********************************************************************************************/

void CCPanose::SetAll(
		BYTE FamilyType,
		BYTE SerifStyle,
		BYTE Weight,
		BYTE Proportion,
		BYTE Contrast,
		BYTE StrokeVariation,
		BYTE ArmStyle,
		BYTE LetterForm,
		BYTE Midline,
		BYTE XHeight)
{
	mFamilyType			= FamilyType;
    mSerifStyle			= SerifStyle; 
    mWeight				= Weight; 
    mProportion			= Proportion; 
    mContrast			= Contrast;
    mStrokeVariation	= StrokeVariation; 
    mArmStyle			= ArmStyle; 
    mLetterform			= LetterForm; 
    mMidline			= Midline; 
    mXHeight			= XHeight; 
}

/********************************************************************************************

>	void CCPanose::operator=(const CCPanose & x)

	Author:		Andy Hayward
	Created:	24/07/96
	Inputs:		x - another CCPanose object to copy
	Returns:	-
	Purpose:	Copies a CCPanose object

********************************************************************************************/

void CCPanose::operator=(const CCPanose & x)
{
	this->mFamilyType = x.mFamilyType;
	this->mSerifStyle = x.mSerifStyle; 
 	this->mWeight = x.mWeight; 
	this->mProportion = x.mProportion; 
	this->mContrast = x.mContrast; 
	this->mStrokeVariation = x.mStrokeVariation; 
	this->mArmStyle = x.mArmStyle; 
	this->mLetterform = x.mLetterform; 
	this->mMidline = x.mMidline; 
	this->mXHeight = x.mXHeight; 
}

/********************************************************************************************

>	BOOL CCPanose::BodgeCCPanose(CCPanose *pCCPanose, BOOL Bold, BOOL Italic)

	Author:		Andy Hayward
	Created:	26/07/96
	Inputs:		Bold	- Bold flag to bodge in
				Italic	- Italic flag to bodge in
	Returns:	TRUE if successful, FALSE otherwise
	Purpose:	Bodges the bold and italic flags into the Weight and Letterform components.

********************************************************************************************/

BOOL CCPanose::BodgePanoseNumber(BOOL IsBold, BOOL IsItalic)
{
	if (mFamilyType != PAN_FAMILY_PICTORIAL)
	{
		// only do this if we're not a symbol font

		if (IsBold==FALSE)
		{
			// regular font
			if (mWeight==PAN_ANY || mWeight==PAN_NO_FIT || mWeight>PAN_WEIGHT_MEDIUM)
			{
				mWeight=PAN_WEIGHT_MEDIUM;
			}
		}
		else
		{
			// bold font
			if (mWeight==PAN_ANY || mWeight==PAN_NO_FIT || mWeight<PAN_WEIGHT_BOLD)
			{
				mWeight=PAN_WEIGHT_BOLD;
			}
		}

		if (IsItalic==FALSE)
		{
			// upright font
			if (mLetterform==PAN_ANY || mLetterform==PAN_NO_FIT)
			{
				mLetterform=PAN_LETT_NORMAL_CONTACT;
			}
			else if (mLetterform>=PAN_LETT_OBLIQUE_CONTACT && mLetterform<=PAN_LETT_OBLIQUE_SQUARE)
			{
				mLetterform=mLetterform-7;		// !!HACK!!
			}
		}
		else
		{
			// italic font
			if (mLetterform==PAN_ANY || mLetterform==PAN_NO_FIT)
			{
				mLetterform=PAN_LETT_OBLIQUE_CONTACT;
			}
			else if (mLetterform>=PAN_LETT_NORMAL_CONTACT && mLetterform<=PAN_LETT_NORMAL_SQUARE)
			{
				mLetterform=mLetterform+7;		// !!HACK!!
			}
		}

	}
	
	return TRUE;	// we have succeeded, haven't we?
}

/********************************************************************************************

>	BOOL CCPanose::BodgeToNormal()

	Author:		Andy Hayward
	Created:	19/08/96
	Inputs:		-
	Returns:	TRUE if successful, FALSE otherwise
	Purpose:	Bodges the Panose number to normalise it, i.e. regular weight, non-italic.
	See Also:	CCPanose::BodgePanoseNumber

********************************************************************************************/

BOOL CCPanose::BodgeToNormal()
{
	return BodgePanoseNumber(FALSE, FALSE);
}

/********************************************************************************************

>	ULONG CCPanose::Distance(const CCPanose &x, const CCPanose &y)
	ULONG CCPanose::DistanceFamilyType(BYTE x, BYTE y);
	ULONG CCPanose::DistanceSerifStyle(BYTE x, BYTE y);
	ULONG CCPanose::DistanceWeight(BYTE x, BYTE y);
	ULONG CCPanose::DistanceProportion(BYTE x, BYTE y);
	ULONG CCPanose::DistanceContrast(BYTE x, BYTE y);
	ULONG CCPanose::DistanceStrokeVariation(BYTE x, BYTE y);
	ULONG CCPanose::DistanceArmStyle(BYTE x, BYTE y);
	ULONG CCPanose::DistanceLetterform(BYTE x, BYTE y);
	ULONG CCPanose::DistanceMidline(BYTE x, BYTE y);
	ULONG CCPanose::DistanceXHeighte(BYTE x, BYTE y);

	Author:		Andy Hayward
	Created:	19/08/96
	Inputs:		x, y	- CCPanose objects to determine the distance between
	Returns:	distance between x and y, as a CCPanoseDistance object
	Purpose:	Calculates the distance between x and y
	See Also:	class CCPanose

********************************************************************************************/

ULONG CCPanose::Distance(const CCPanose &x, const CCPanose &y)
{
	ULONG Distance = 0;

	Distance += DistanceFamilyType(x.mFamilyType, y.mFamilyType);
	Distance += DistanceSerifStyle(x.mSerifStyle, y.mSerifStyle);
	Distance += DistanceWeight(x.mWeight, y.mWeight);
	Distance += DistanceProportion(x.mProportion, y.mProportion);
	Distance += DistanceContrast(x.mContrast, y.mContrast);
	Distance += DistanceStrokeVariation(x.mStrokeVariation, y.mStrokeVariation);
	Distance += DistanceArmStyle(x.mArmStyle, y.mArmStyle);
	Distance += DistanceLetterform(x.mLetterform, y.mLetterform);
	Distance += DistanceMidline(x.mMidline, y.mMidline);
	Distance += DistanceXHeight(x.mXHeight, y.mXHeight);
	
	return Distance;
}

ULONG CCPanose::DistanceFamilyType(BYTE x, BYTE y)
{
	//	Specifies the font family.
	//	
	//	Value	Meaning
	//	
	//		0	Any
	//		1	No fit
	//		2	Text and display
	//		3	Script
	//		4	Decorative
	//		5	Pictorial
	//	
	//	High weightings, since we _really_ want a font from the same family.
	
	const static ULONG Metric[6][6] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x000000ff, 0x000000ff, 0x000000ff},
		{0x00001000, 0x01000000, 0x000000ff, 0x00000000, 0x000000ff, 0x000000ff},
		{0x00001000, 0x01000000, 0x000000ff, 0x000000ff, 0x00000000, 0x000000ff},
		{0x00001000, 0x01000000, 0x000000ff, 0x000000ff, 0x000000ff, 0x00000000}};

	if (x<0 || x>5) x = 1;
	if (y<0 || y>5) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceSerifStyle(BYTE x, BYTE y)
{
	const static ULONG Metric[16][16] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000004, 0x00000004, 0x00000004, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000010},
		{0x00001000, 0x01000000, 0x00000004, 0x00000000, 0x00000004, 0x00000004, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000010},
		{0x00001000, 0x01000000, 0x00000004, 0x00000004, 0x00000000, 0x00000004, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000010},
		{0x00001000, 0x01000000, 0x00000004, 0x00000004, 0x00000004, 0x00000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000010},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000000, 0x00000010, 0x00000010, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000000, 0x00000010, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000010, 0x00000000, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000000, 0x00000020},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000000}};

	if (x<0 || x>15) x = 1;
	if (y<0 || y>15) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceWeight(BYTE x, BYTE y)
{
	const static ULONG Metric[12][12] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024, 0x00000031, 0x00000040, 0x00000051},
		{0x00001000, 0x01000000, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024, 0x00000031, 0x00000040},
		{0x00001000, 0x01000000, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024, 0x00000031},
		{0x00001000, 0x01000000, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024},
		{0x00001000, 0x01000000, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019},
		{0x00001000, 0x01000000, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010},
		{0x00001000, 0x01000000, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009},
		{0x00001000, 0x01000000, 0x00000031, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004},
		{0x00001000, 0x01000000, 0x00000040, 0x00000031, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001},
		{0x00001000, 0x01000000, 0x00000051, 0x00000040, 0x00000031, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000}};

	if (x<0 || x>11) x = 1;
	if (y<0 || y>11) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceProportion(BYTE x, BYTE y)
{
	const static ULONG Metric[10][10] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000040, 0x00000004, 0x00000080, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000000, 0x00000080, 0x00000004, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000004, 0x00000080, 0x00000000, 0x000000c0, 0x00000080},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000080, 0x00000004, 0x000000c0, 0x00000000, 0x00000080},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000080, 0x00000080, 0x00000000}};

	if (x<0 || x>9) x = 1;
	if (y<0 || y>9) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceContrast(BYTE x, BYTE y)
{
	const static ULONG Metric[10][10] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024, 0x00000031},
		{0x00001000, 0x01000000, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019, 0x00000024},
		{0x00001000, 0x01000000, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010, 0x00000019},
		{0x00001000, 0x01000000, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009, 0x00000010},
		{0x00001000, 0x01000000, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004, 0x00000009},
		{0x00001000, 0x01000000, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001, 0x00000004},
		{0x00001000, 0x01000000, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000, 0x00000001},
		{0x00001000, 0x01000000, 0x00000031, 0x00000024, 0x00000019, 0x00000010, 0x00000009, 0x00000004, 0x00000001, 0x00000000}};

	if (x<0 || x>9) x = 1;
	if (y<0 || y>9) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceStrokeVariation(BYTE x, BYTE y)
{
	const static ULONG Metric[9][9] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000010, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000000, 0x00000010, 0x00000010, 0x00000020, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000000, 0x00000010, 0x00000020, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000010, 0x00000000, 0x00000020, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000010, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000010, 0x00000000, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020, 0x00000000}};

	if (x<0 || x>8) x = 1;
	if (y<0 || y>8) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceArmStyle(BYTE x, BYTE y)
{
	const static ULONG Metric[12][12] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010},
		{0x00001000, 0x01000000, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000}};

	if (x<0 || x>11) x = 1;
	if (y<0 || y>11) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceLetterform(BYTE x, BYTE y)
{
	const static ULONG Metric[16][16] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010},
		{0x00001000, 0x01000000, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000040, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000040, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000040, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000010, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000000}};

	if (x<0 || x>15) x = 1;
	if (y<0 || y>15) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceMidline(BYTE x, BYTE y)
{
	const static ULONG Metric[14][14] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000010, 0x00000010, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000000, 0x00000010, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000010, 0x00000010, 0x00000000, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000040, 0x00000040, 0x00000000, 0x00000010, 0x00000010, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000020, 0x00000040, 0x00000010, 0x00000000, 0x00000010, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000020, 0x00000010, 0x00000010, 0x00000000, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000000, 0x00000010, 0x00000010, 0x00000020, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000010, 0x00000000, 0x00000010, 0x00000040, 0x00000020, 0x00000040},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000010, 0x00000010, 0x00000000, 0x00000040, 0x00000040, 0x00000020},
		{0x00001000, 0x01000000, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000000, 0x00000010, 0x00000010},
		{0x00001000, 0x01000000, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000010, 0x00000000, 0x00000010},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000040, 0x00000040, 0x00000020, 0x00000010, 0x00000010, 0x00000000}};

	if (x<0 || x>13) x = 1;
	if (y<0 || y>13) y = 1;

	return Metric[x][y];
}

ULONG CCPanose::DistanceXHeight(BYTE x, BYTE y)
{
	const static ULONG Metric[8][8] = {
		{0x00010000, 0x00100000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000},
		{0x00100000, 0x10000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000},
		{0x00001000, 0x01000000, 0x00000000, 0x00000020, 0x00000020, 0x00000010, 0x00000040, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000000, 0x00000020, 0x00000040, 0x00000010, 0x00000040},
		{0x00001000, 0x01000000, 0x00000020, 0x00000020, 0x00000000, 0x00000040, 0x00000040, 0x00000010},
		{0x00001000, 0x01000000, 0x00000010, 0x00000040, 0x00000040, 0x00000000, 0x00000020, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000010, 0x00000040, 0x00000020, 0x00000000, 0x00000020},
		{0x00001000, 0x01000000, 0x00000040, 0x00000040, 0x00000010, 0x00000020, 0x00000020, 0x00000000}};

	if (x<0 || x>7) x = 1;
	if (y<0 || y>7) y = 1;

	return Metric[x][y];
}
