/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
/********************************************************************************************

>	CCPanose.h

	Author:		Andy Hayward
	Created:	24/05/96
	Purpose:	Kernel Panose related class and functions

********************************************************************************************/

#ifndef INC_CCPANOSE
#define INC_CCPANOSE

/********************************************************************************************

>	class CCPanose

	Author:		Andy Hayward
	Created:	24/05/96
	Base Class:	CCObject
	Purpose:	Class to hold and manipulate Panose numbers
	See also:	class FontMan, class ATMFontMan, class TTFontMan

********************************************************************************************/

class CCPanose
{
// private member variables
private:
	// identical to the member variables of the Win32 PANOSE structure. We duplicate them here
	// since this is kernel code - platform independent (alledgedly!)
	BYTE mFamilyType;
	BYTE mSerifStyle;
	BYTE mWeight;
	BYTE mProportion;
	BYTE mContrast;
	BYTE mStrokeVariation;
	BYTE mArmStyle;
	BYTE mLetterform;
	BYTE mMidline;
	BYTE mXHeight;

// constructor, copy constructor, destructor, assignment
public:
	CCPanose();
	CCPanose(const PANOSE &PanoseNumber);

// public member functions
public:
	void SetAllToAny();			// Sets all components of the panose number to PAN_ANY
	void SetAll(BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, BYTE, BYTE);
	void operator=(const CCPanose & x);

	BOOL BodgePanoseNumber(BOOL Bold, BOOL Italic);
	BOOL BodgeToNormal();

// static member functions
public:
	static ULONG Distance(const CCPanose &x, const CCPanose &y);
private:
	static ULONG DistanceFamilyType(BYTE x, BYTE y);
	static ULONG DistanceSerifStyle(BYTE x, BYTE y);
	static ULONG DistanceWeight(BYTE x, BYTE y);
	static ULONG DistanceProportion(BYTE x, BYTE y);
	static ULONG DistanceContrast(BYTE x, BYTE y);
	static ULONG DistanceStrokeVariation(BYTE x, BYTE y);
	static ULONG DistanceArmStyle(BYTE x, BYTE y);
	static ULONG DistanceLetterform(BYTE x, BYTE y);
	static ULONG DistanceMidline(BYTE x, BYTE y);
	static ULONG DistanceXHeight(BYTE x, BYTE y);
	
// separate set and get functions, to hide the internals of this class. the internals
// may change - we could have a member structure of the OS's panose class.
public:
	void SetFamilyType(BYTE value) {mFamilyType = value;};
	void SetSerifStyle(BYTE value) {mSerifStyle = value;};
	void SetWeight(BYTE value) {mWeight = value;};
	void SetProportion(BYTE value) {mProportion = value;};
	void SetContrast(BYTE value) {mContrast = value;};
	void SetStrokeVariation(BYTE value) {mStrokeVariation = value;};
	void SetArmStyle(BYTE value) {mArmStyle = value;};
	void SetLetterform(BYTE value) {mLetterform = value;};
	void SetMidline(BYTE value) {mMidline = value;};
	void SetXHeight(BYTE value) {mXHeight = value;};

    BYTE GetFamilyType() const {return mFamilyType;};
    BYTE GetSerifStyle() const {return mSerifStyle;};
    BYTE GetWeight() const {return mWeight;};
    BYTE GetProportion() const {return mProportion;};
    BYTE GetContrast() const {return mContrast;};
    BYTE GetStrokeVariation() const {return mStrokeVariation;};
    BYTE GetArmStyle() const {return mArmStyle;};
    BYTE GetLetterform() const {return mLetterform;};
    BYTE GetMidline() const {return mMidline;};
    BYTE GetXHeight() const {return mXHeight;};
};

#endif