/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
/*
	$Header: /Camelot/winoil/costream.cpp 5     11/09/96 3:39p Justinf $

	costream.cpp

	This class is a C++ stream derivative that can read and write COM structured storage.
	It is similar to the standard fstream disk-file class.
*/

#include "stdafx.h"
//#include "camtypes.h"
#include <limits.h>
#include <stddef.h>
#include <string.h>
#include <functional>

#include "costream.h"

typedef int filedesc;
#define filebuftext			1
#define filebufopenprot		1
#define iosnocreate			0
#define iosnoreplace		0

/********************************************************************************************
>	costreambuf::costreambuf(IStream* pIStream = 0)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStream	-	A pointer to a valid IStream interface that will be used
								for I/O by this costreambuf.  By default 0, ie. the
								costreambuf is not yet attached to any IStream interface.
	Purpose:	Constructs an costreambuf, optionally already attached to an open IStream
				interface.  If no IStream is passed then you should call attach or one of
				the open_* functions explicitly before trying to do I/O.  If you wish to
				prevent the costreambuf releasing the IStream when it is destroyed then
				make sure you've called AddRef on it one extra time.
	SeeAlso:	costream; costreambuf::open_file; costreambuf::open_stream;
				costreambuf::attach; costreambuf::is_open; costreambuf::~costreambuf;
				IStream; IStream::AddRef
********************************************************************************************/

costreambuf::costreambuf(IStream* pIStream, size_t cbBuffer /*= COM_BUFSIZ*/)
  :	m_pIStream(0), m_pIStorage(0), m_fAppend(0), m_vecBufferIn(cbBuffer), 
    m_vecBufferOut(cbBuffer), m_eDirection( eEither )
{
	InitBuffers();

	// If an IStream was given then make sure a buffer is allocated etc.
	if (pIStream != 0) attach(pIStream);
}



/********************************************************************************************
>	costreambuf::costreambuf(const TCHAR* pcszFilePath,
							 int nMode,
							 int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pcszFilePath	-	the full path to the non-COM file that is to be
									opened as a "pseudo-" structured storage.
				nMode			-	the access mode to be used to open the file; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									file; see the C++ stream documentation.
	Purpose:	Constructs an costreambuf attached to a a non-structured storage file,
				using the STGM_CONVERT access mode.  This is like calling the default
				constructor followed by open_file.
	SeeAlso:	costream; IStream; costreambuf::open_file; costreambuf::~costreambuf
********************************************************************************************/

costreambuf::costreambuf(const TCHAR* pcszFilePath, int nMode, int nProt, size_t cbBuffer /*= COM_BUFSIZ*/)
  :	m_pIStream(0), m_pIStorage(0), m_fAppend(0), m_vecBufferIn(cbBuffer), 
    m_vecBufferOut(cbBuffer), m_eDirection( eEither )
{
	InitBuffers();

	open_file(pcszFilePath, nMode, nProt);
}



/********************************************************************************************
>	costreambuf::costreambuf(IStorage* pIStorage, const TCHAR* pcszStreamName,
							 int nMode,
							 int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStorage		-	the open IStorage interface that contains the IStream
									this costreambuf will be constructed from.
				pcszStreamName	-	the name of the IStream within the IStorage.
				nMode			-	the access mode to be used to open the stream; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									stream; see the C++ stream documentation.
	Purpose:	Constructs this costreambuf from an IStream within the given IStorage.
	SeeAlso:	costreambuf::costreambuf; costreambuf::open_stream; costreambuf::close;
				costreambuf::attach; IStorage::OpenStream
********************************************************************************************/

costreambuf::costreambuf(IStorage* pIStorage, const TCHAR* pcszStreamName,
						 int nMode, int nProt, size_t cbBuffer /*= COM_BUFSIZ*/)
  :	m_pIStream(0), m_pIStorage(0), m_fAppend(0), m_vecBufferIn(cbBuffer), 
    m_vecBufferOut(cbBuffer), m_eDirection( eEither )
{
	InitBuffers();

	open_stream(pIStorage, pcszStreamName, nMode, nProt);
}


#if defined(NEED_TO_SPEC_BUFFER_IN_COSTRAMBUF)
/********************************************************************************************
>	costreambuf::costreambuf(IStream* pIStream, char* pchReserve, int nLength)

	Author:		JustinF
	Created:	7/8/96
	Inputs:		pIStream		-	A pointer to a valid IStream interface.  If 0 then
									the costreambuf is not attached to any IStream yet.
				pchReserve		-	the memory to use as a pending-data buffer.  If 0
									then the costreambuf is unbuffered.
				nLength			-	the size of the given memory, in characters.  If zero
									then the costreambuf is unbuffered.
	Purpose:	Constructs an costreambuf which will use the given reserve area as a
				buffer.
	SeeAlso:	costreambuf::costreambuf; costreambuf::open_stream; costreambuf::close;
				costreambuf::attach; streambuf::setbuf
********************************************************************************************/

costreambuf::costreambuf(IStream* pIStream, char* pchReserve, int nLength, size_t cbBuffer /*= COM_BUFSIZ*/)
  :	m_pIStream(0), m_pIStorage(0), m_fAppend(0), m_vecBuffer(cbBuffer)
{
	setbuf(pchReserve, nLength);
	if (pIStream != 0) attach(pIStream);
}
#endif


/********************************************************************************************
>	costreambuf::costreambuf(const costreambuf& other)

	Author:		JustinF
	Created:	7/8/96
	Inputs:		other		-		reference to a constant costreambuf that is to
									be copied by this constructor.
	Purpose:	Copy-constructor.  If the other costreambuf is open that the IStream it
				refers to is "cloned".
	SeeAlso:	costreambuf::costreambuf; costreambuf::do_clone; IStream::Clone
********************************************************************************************/

costreambuf::costreambuf(const costreambuf& other)
  :	m_pIStream(0), m_pIStorage(0), m_fAppend(0)
{
	do_clone(other);
}


/********************************************************************************************
>	void costreambuf::InitBuffers()

	Author:		LukeH
	Created:	30/06/08
	Purpose:	Centeralised initialisation of the buffers for input and output
	SeeAlso:	costreambuf::costreambuf
********************************************************************************************/

void costreambuf::InitBuffers()
{
	setg( &m_vecBufferIn[0], &m_vecBufferIn[0], &m_vecBufferIn[0] );
	setp( &m_vecBufferOut[0], &m_vecBufferOut[0] + m_vecBufferOut.size() - 1 ); // -1 makes overflow nicer, *pptr always safe
}


/********************************************************************************************
>	virtual costreambuf::~costreambuf()

	Author:		JustinF
	Created:	5/8/96
	Purpose:	Destroys an costreambuf.  If the buffer is attached to an open IStream
				interface then the buffer is flushed and the IStream is released.  Any
				IStorage pointers that were created by the costreambuf are also released.
				If you wish to prevent the costreambuf releasing the attached IStream when
				it is destroyed then make sure you've called AddRef on it.
	SeeAlso:	costreambuf::sync; IStream::Release; IStorage::Release
********************************************************************************************/

costreambuf::~costreambuf()
{
	// Make sure the stream is closed.
	if (is_open()) close();
}



/********************************************************************************************
>	virtual int costreambuf::is_open() const

	Author:		JustinF
	Created:	5/8/96
	Returns:	Non-zero (TRUE) if this costreambuf is currently attached to an open valid
				IStream interface, zero (FALSE) if it isn't.
	Purpose:	Tests if this buffer is attached to an open IStream.
	SeeAlso:	costreambuf::attach; costreambuf::get_IStream; costream::is_open
********************************************************************************************/

int costreambuf::is_open() const
{
	return m_pIStream != 0;
}



/********************************************************************************************
>	virtual costreambuf::operator IStream*() const

	Author:		JustinF
	Created:	5/8/96
	Returns:	The IStream interface that this costreambuf is currently attached to, or
				0 if it isn't attached.
	Purpose:	Allows direct access to the attached IStream.  Handle the return value
				with care, or you'll screw the stream!
	SeeAlso:	costreambuf::attach; costreambuf::is_open
********************************************************************************************/

costreambuf::operator IStream*() const
{
	return m_pIStream;
}



/********************************************************************************************
>	virtual costreambuf& costreambuf::operator=(const costreambuf& other)

	Author:		JustinF
	Created:	9/8/96
	Inputs:		other		-		the other costreambuf, which will be copied.
	Returns:	A reference to this costreambuf.
	Purpose:	Assignment operator for costreambufs.  This assigns an independent copy
				of other to this costreambuf.
	SeeAlso:	costreambuf::costreambuf; costreambuf::do_clone; IStream::Clone
********************************************************************************************/

costreambuf& costreambuf::operator=(const costreambuf& other)
{
	do_clone(other);
	return *this;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::get_access_mode(int* pnMode, int* pnProt) const

	Author:		JustinF
	Created:	17/8/96
	Outputs:	pnMode		-	if not null then the access-mode used to open this
								costreambuf's attached IStream.
				pnProt		-	if not null then the sharing-mode used to open this
								costreambuf's Istream.
	Returns:	NOERROR if successful, STG_E_INVALIDFUNCTION if the costreambuf is not
				attached, STG_E_INVALIDFLAG if it's IStream was opened in a mode that has
				no ios/costream equivalent etc.
	Purpose:	Returns the access and sharing modes used to open the IStream that this
				constreambuf is attached to, such as ios::in, costream::sh_none etc.
	Errors:		(See Returns).
	SeeAlso:	costream::get_access_mode
********************************************************************************************/

HRESULT costreambuf::get_access_mode(int* pnMode, int* pnProt) const
{
	// Check if we are attached.
	if (!is_open()) return STG_E_INVALIDFUNCTION;

	// Try to get the IStream's current status.
	STATSTG statstg;
	HRESULT hr = m_pIStream->Stat(&statstg, STATFLAG_NONAME);
	if (FAILED(hr)) return hr;

	// Convert the STGM flags into the ios equivalents and return a success code.
	return stgm_to_ios(&statstg.grfMode, pnMode, pnProt) ? NOERROR : STG_E_INVALIDFLAG;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::set_size(size_t cbNewSize)

	Author:		JustinF
	Created:	9/8/96
	Inputs:		cbNewsize		-	the new size of the pre-allocated space for the
									stream, in bytes.
	Returns:	NOERROR if successful.  STG_E_INVALIDFUNCTION is this costreambuf is not
				currently attach to an open IStream.  See IStream::SetSize for other error
				values.
	Purpose:	Sets the amount of structured storage to pre-allocate for this costreambuf.
				Streams, of course, will expand if written past their end - this call is a
				good way to reserve some disk space before committing anything important
				to it.
	Errors:		(See Returns).
	SeeAlso:	costreambuf; costreambuf::is_open; IStream::SetSize
********************************************************************************************/

HRESULT costreambuf::set_size(size_t cbNewSize)
{
	// Is there a stream to resize?
	if (!is_open()) return STG_E_INVALIDFUNCTION;
	
	// Convert the size_t to a ULARGE_INTEGER and return the result.
	ULARGE_INTEGER ul;
	ul.LowPart  = static_cast<DWORD>(cbNewSize & 0xFFFFFFFF);
	ul.HighPart = 0;
	return m_pIStream->SetSize(ul);
}



/********************************************************************************************
>	virtual HRESULT costreambuf::attach(IStream* pIStream)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStream	-	the IStream to attach to.  If you wish to retain control
								over when the IStream is released then call AddRef on it
								before the attached costreambuf is destroyed, as it will
								try to destroy it for you otherwise.
	Returns:	An COM error code: STG_E_INVALIDFUNCTION if this costreambuf is already
				attached to an IStream (you must close it first); NOERROR if the
				call is successful.
	Purpose:	Attaches an already-open IStream interface to this costreambuf, so that
				it can be treated as a C++ stream.  If the stream is buffer then it
				allocates a block of memory for pending data transfers.
	Errors:		(see Returns)
	SeeAlso:	costream::close; costreambuf::~costreambuf; streambuf::unbuffered
********************************************************************************************/

HRESULT costreambuf::attach(IStream* pIStream)
{
	// Make sure we are not already attached.
	if (is_open()) return STG_E_INVALIDFUNCTION;

	// Remember the COM stream interface.
	m_pIStream = pIStream;

	// Success.
	return NOERROR;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::close()

	Author:		JustinF
	Created:	5/8/96
	Returns:	NOERROR if successful, a structured storage STG_E_* code if it fails.
	Purpose:	Closes the attached IStream interface, flushing any pending data and
				releasing the IStream.  If you want to prevent the costreambuf releasing
				the IStream then make sure you've called AddRef on the IStream one extra
				time.  Note that if the stream was opened using the open_file function,
				in "convert" mode, then the costreambuf will also release the parent
				storage of the stream.
	SeeAlso:	costreambuf::~costreambuf; costreambuf::attach; costreambuf::sync;
				costreambuf::is_open; IStream::AddRef; IStream::Release
********************************************************************************************/

HRESULT costreambuf::close()
{
	// Make sure the stream is open.
	if (!is_open()) return STG_E_INVALIDFUNCTION;

	// Make sure it is flushed.
	HRESULT hr = hr_sync();

	// Release the attached stream etc.
	m_pIStream->Release();
	m_pIStream = 0;

	// Release the parent storage, if known.
	if (m_pIStorage != 0)
	{
		m_pIStorage->Release();
		m_pIStorage = 0;
	}

	// Make sure the append flag is restored to the default.
	m_fAppend = 0;
	
	// Success.
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::open_stream(IStorage* pIStorage,
											 const TCHAR* pcszStreamName,
											 int nMode,
											 int nProt = costream::sh_read | costream::write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStorage		-	the open IStorage interface that contains the IStream
									to be opened.
				pcszStreamName	-	the name of the IStream to be opened.
				nMode			-	the access mode to be used to open the stream; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									stream; see the C++ stream documentation.
	Returns:	STG_E_INVALIDPOINTER if pIStorage is 0; STG_E_INVALIDNAME if the
				stream name is 0 or invalid ANSI; STG_E_INVALIDFUNCTION if this
				costreambuf is currently open (ie. you must call close first) etc
	Purpose:	Opens an IStream for use by this costreambuf.
	Errors:		ERROR3 if 0 is passed for pcszStreamName, or if no IStorage is is
				available.
	SeeAlso:	costreambuf::open_file; costreambuf::close; costreambuf::ios_to_stgm;
				costreambuf::attach; IStorage::OpenStream
********************************************************************************************/

HRESULT costreambuf::open_stream(IStorage* pIStorage, const TCHAR* pcszStreamName,
								 int nMode, int nProt)
{
	// If necessary convert the stream's name to Unicode (this works for null pointers).
	const WCHAR* pcwcszStreamName;
#ifndef _UNICODE
	pcwcszStreamName = convert_string(pcszStreamName);
#else
	pcwcszStreamName = pcszStreamName;
#endif

	// Make sure we really have a storage and a valid stream-name to use.
	ERROR3IF(pIStorage == 0, "Null IStorage* in costreambuf::open_stream");
	ERROR3IF(pcszStreamName == 0,
			 "Null or invalid stream-name in costreambuf::open_stream");
	ERROR3IF(lstrlenW(pcwcszStreamName) > 31,
			 "Stream name too long in costreambuf::open_stream");

	// Fail if any of the above, or if this costreambuf is already attached and open.
	if (pIStorage == 0) return STG_E_INVALIDPOINTER;
	if (pcwcszStreamName == 0) return STG_E_INVALIDNAME;
	if (is_open()) return STG_E_INVALIDFUNCTION;

	// Clear any flags we may have been passed that are not supported by the current
	// release of structured storage.
	nMode &= ~(costream::transacted | costream::convert);
	nProt = (nProt | costream::sh_none) & ~(costream::sh_read | costream::sh_write);

	// Try to convert the access mode.
	DWORD grfMode;
	if (!ios_to_stgm(&nMode, &nProt, &grfMode)) return STG_E_INVALIDFLAG;

	// Try to open the given stream and attach it.
	IStream* pIStream;
	HRESULT hr = pIStorage->OpenStream(pcwcszStreamName, 0, grfMode, 0, &pIStream);
	if (SUCCEEDED(hr) && SUCCEEDED(hr = attach(pIStream)))
	{
		// We're in business.  Now apply the ios::ate, ios::app, and ios::trunc access
		// modes, which have no structured-storage equivalent.
		//   (i) ios::app means "append" and forces all writes to be at the end of
		//		 the stream, no matter where the write seek-pointer actually is.
		//  (ii) ios::ate means "append" the initial write to the end of the stream,
		//		 after which the seek pointer is free to point anywhere.  In other
		//		 words, move the write seek-pointer to the end of the stream after
		//		 opening it.
		// (iii) ios::trunc means "truncate" the stream, which commences writing at
		//		 the beginning of the stream.
		if (nMode & (ios::ate | ios::app))
		{
			// Remember the ios::app append mode, so that we can always force a seek to
			// the end of the stream after every write.
			m_fAppend = (nMode & ios::app);

			// Try to seek to the end of the stream.
			if (FAILED(hr = hr_seekoff(0, ios::end, 0, nMode))) close();

		}
		else if (nMode & ios::trunc)
		{
			// Try to seek to the beginning of the stream.
			if (FAILED(hr = hr_seekoff(0, ios::beg, 0, nMode))) close();
		}
	}

	// Return whatever happened.
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::open_file(const TCHAR* pcszFilePath,
										   int nMode,
										   int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pcszFilePath	-	the full path to the non-structured storage file to be
									opened.
				nMode			-	the access mode to be used to open the file; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									file; see the C++ stream documentation.
	Returns:	STG_E_INVALIDFLAG if the given access mode is invalid; STG_E_INVALIDNAME if
				the file path is 0 or invalid ANSI; STG_E_INVALIDFUNCTION if this
				costreambuf is currently open (ie. you must call close first) etc
	Purpose:	Opens a non-COM file as a "fake" structured storage, using the STGM_CONVERT
				access mode.  The bytes in the file are accessed by opening a root-storage
				containing a single stream called "CONTENTS".  See the OLE docs for an
				explanation of this special mode.
	Errors:		(See Returns), also ERROR3 if pcszFilePath is 0.
	SeeAlso:	costreambuf::costreambuf; costreambuf::open_stream; costreambuf::close;
				::StgCreateDocfile
********************************************************************************************/

HRESULT costreambuf::open_file(const TCHAR* pcszFilePath, int nMode, int nProt)
{
	// If necessary, convert the file-path to the Unicode character set.
	const WCHAR* pcwcszFilePath;
#ifndef _UNICODE
	pcwcszFilePath = convert_string(pcszFilePath);
#else
	pcwcszFilePath = pcszFilePath;
#endif

	// Make sure we really have a file to use.
	ERROR3IF(pcszFilePath == 0, "Null file-path in costreambuf::open_file");

	// Fail if a stream is already attached and open etc.
	if (pcwcszFilePath == 0) return STG_E_INVALIDNAME;
	if (is_open()) return STG_E_INVALIDFUNCTION;

	// Work out the equivalent structured storage access code, forcing STGM_CONVERT.
	DWORD grfMode;
	int nTempMode = nMode | costream::convert;
	if (!ios_to_stgm(&nTempMode, &nProt, &grfMode)) return STG_E_INVALIDFLAG;

	// Try to open the non-COM file as a root-storage.  NB. don't use StgOpenStorage here,
	// as it doesn't support STGM_CONVERT (even though the COM docs imply that it does!)
	HRESULT hr = StgCreateDocfile(pcwcszFilePath, grfMode, 0, &m_pIStorage);
	if (FAILED(hr)) return hr;

	// Now try to open the default CONTENTS stream, using the original (not STGM_CONVERT)
	// access mode.
	return open_stream(m_pIStorage, TEXT("CONTENTS"), nMode, nProt);
}



/********************************************************************************************
>	virtual HRESULT costreambuf::hr_underflow(int* pnChar)

	Author:		JustinF
	Created:	5/8/96	
	Outputs:	pnChar		-		pointer to where to store the next character read.		
	Returns:	STG_E_INVALIDFUNCTION if the costreambuf is not currently attached to an
				open IStream; STG_E_INSUFFICIENTMEMORY if a buffer cannot be allocated;
				STG_E_READFAULT if a character was not available to read.  Returns S_FALSE
				if the end of the stream has been reached.
	Purpose:	Called when the buffer underflows and more data must be ready from the
				attached IStream.  Note the liberal casts of characters to unsigned chars,
				and then to ints.  Make sure you understand why this is so before you
				change anything!
	Errors:		(See Returns).
	SeeAlso:	costreambuf::underflow; streambuf::underflow; streambuf::allocate;
				streambuf::sync; IStream::Read
********************************************************************************************/

HRESULT costreambuf::hr_underflow(int* pnChar)
{
	// Make sure this output parameter is supplied and has a default value.
	ERROR3IF(pnChar == 0, "Null output parameter in costreambuf::hr_underflow");

	// Check we're not also writing 
	ERROR3IF(m_eDirection == eWrite, "We don't support reading and writing from same costreambuf");
	m_eDirection = eRead;

	// Make sure the stream is open.
	if (!is_open())
		return STG_E_INVALIDFUNCTION;

	// Check for and initialise the character to be returned.
	if (pnChar == 0)
		return STG_E_INVALIDPARAMETER;
	*pnChar = EOF;

	char*	pBase = &m_vecBufferIn[0];
	char*	pStart = pBase;
	size_t	blen = m_vecBufferIn.size();

	// If there is data already in the buffer return that.
	if (gptr() != egptr())
	{
		*pnChar = (int) (unsigned char) *gptr();
		return NOERROR;
	}

	// Try to flush the buffer.
	HRESULT		hr = hr_sync();
	if (FAILED(hr))
		return hr;

	// The stream is buffered, so read as much as we can.  If we reach the end of the
	// file then we return S_FALSE, rather than an error code.
	ULONG	cbRead;
	hr = m_pIStream->Read(pStart, static_cast<ULONG>(blen), &cbRead);
	if (FAILED(hr))
		return hr;
	if (cbRead > ULONG(blen))
		return STG_E_READFAULT;

	// If we didn't read anything then we must be at the end of the stream.
	if (cbRead == 0)
		return S_FALSE;

	// Success, so reset the buffer pointers and return the first character read.
	setg(pBase, pStart, pStart + size_t(cbRead));
	*pnChar = traits_type::to_int_type( *gptr() );

	// There's some data, and they will be more afterwards.
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::hr_overflow(int nChar = EOF)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		nChar		-	the character that has overflowed the buffer, by default
								none.
	Returns:	STG_E_INVALIDFUNCTION if the costreambuf is not currently attached to
				an open IStream; STG_E_INSUFFICIENTMEMORY if a data buffer cannot be
				allocated; STG_E_WRITEFAULT if it was not possible to write the given
				character to the IStream, for some reason.
	Purpose:	Called when the buffer overflows with the given character and data must be
				written to the attached IStream.  Note the liberal casts of characters to
				unsigned chars, and then to ints.  Make sure you understand why this is so
				before you change anything!
	Errors:		(See Returns).
	SeeAlso:	costreambuf::overflow; streambuf::overflow; streambuf::allocate;
				streambuf::sync; IStream::Write
********************************************************************************************/

HRESULT costreambuf::hr_overflow(int ch)
{
	// Make sure the stream is open and that there is a reserve area allocated.
	if (!is_open()) return STG_E_INVALIDFUNCTION;

	// Check we're not also reading 
	ERROR3IF(m_eDirection == eRead, "We don't support reading and writing from same costreambuf");
	m_eDirection = eWrite;

	if( ch != traits_type::eof() )
	{
//		assert( std::less_equal<char *>()( pptr(), epptr() ) );
		*pptr() = ch;
		pbump(1);
			
		// Try to flush the overflowing buffer.
		HRESULT hr = hr_sync();
		if (FAILED(hr))
			return hr;
	}

	return NO_ERROR;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::hr_sync()

	Author:		JustinF
	Created:	7/8/96
	Returns:	An HRESULT describing what went wrong, or NOERROR if successful.
	Purpose:	Called when the pending-data buffer must be flushed.  This function is
				the same as the standard streambuf::sync overridable, but gives more
				useful error information to its caller.  It is used in preference to
				the sync function in costreambuf member functions.
	SeeAlso:	costreambuf::sync; streambuf::sync; costreambuf::hr_overflow;
				costreambuf:hr_underflow;
********************************************************************************************/

HRESULT costreambuf::hr_sync()
{
	// Signal an error if the stream isn't open already.
	if (!is_open()) return STG_E_INVALIDFUNCTION;

	// If there is some output data waiting to be written then flush it.
	HRESULT hr = NOERROR;
	int		nBufCount = static_cast<int>(pptr() - pbase());
	if (nBufCount > 0)
	{
		// Should we always append writes to the end of the stream?
		if (m_fAppend)
		{
			// Yes, seek to the end of the stream.
			LARGE_INTEGER li = { 0, 0 };
			hr = m_pIStream->Seek(li, STREAM_SEEK_END, 0);
			if (FAILED(hr)) return hr;
		}

		// Write out the contents of the buffer.
		ULONG cbWritten;
		hr = m_pIStream->Write(pbase(), nBufCount, &cbWritten);
		if (FAILED(hr) || cbWritten < UINT(nBufCount))
		{
			// We possibly didn't write all of the buffer, so move the data at the end
			// of the buffer that wasn't written to the beginning of the buffer.
			if (cbWritten > 0)
			{
				pbump(-((int) cbWritten));
				memmove(pbase(), pbase() + cbWritten, (int) (nBufCount - cbWritten));
			}

			// Signal an "error".
			return hr;
		}
	}

	// Empty the put area.
	pbump(-nBufCount);

	// If there is some input data then discard it.
	nBufCount = static_cast<int>(egptr() - gptr());
	if (nBufCount > 0)
	{
		// Convert the negation of the nBufCount ULONG into a (signed) LARGE_INTEGER.
		LARGE_INTEGER li;
		li.LowPart = ~nBufCount;
		li.HighPart = ~0;
		if (++li.LowPart == 0) ++li.HighPart;

		// Move the seek pointer back to the beginning of the data to be discarded, so
		// that we'll read it next time.  Note that we can't use seekoff() here.
		hr = m_pIStream->Seek(li, STREAM_SEEK_CUR, 0);
		if (FAILED(hr))
			return hr;
	}

	// Empty the get area.
	if( 0 != nBufCount )
		setg( &m_vecBufferIn[0], &m_vecBufferIn[0], &m_vecBufferIn[0] );

	return hr;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::hr_seekoff(streamoff nOffset,
											ios::seek_dir eDir,
											streampos* pnNewPos = 0,
											int nMode = ios::in | ios::out)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		nOffset		-	the new (relative) offset to seek to.
				eDir		-	the direction to seek to or the position to seek from.
								See the streambuf documentation for these.
				nMode		-	whether this call is for the input seek pointer or the
								output seek pointer.  This parameter is IGNORED as there
								is only one combined seek pointer (cf. the fstream class)
	Outputs:	pnNewPos	-	If not 0, contains the new stream position.
				Note that the streampos and streamoff types only have a 32-bit
				range, even though the structured storage specification allows for 64-bit
				seeks.  Trying to seek past the range allowed by streampos/streamoff is an
				E_NOTIMPL error.
	Returns:	NOERROR if successful; STG_E_INVALIDFUNCTION if there is no attached open
	`			IStream; STG_E_INVALIDFLAG if eDir is invalid etc.
	Purpose:	Moves the seek pointer of the this costreambuf's attached IStream
				interface, optionally returning the new absolute position.
	Errors:		ERROR3 if eDir has an invalid value or if the seek position to return is
				outside the range of streampos.
	SeeAlso:	ios::seek_dir; streambuf::seekoff; fstream::seekoff; IStream::Seek
********************************************************************************************/

HRESULT costreambuf::hr_seekoff(pos_type nOffset, ios_base::seekdir eDir, streampos* pnNewPos, ios_base::openmode nMode)
{
	// Make sure the stream is open and the buffer is correctly flushed first.
	if (!is_open()) return STG_E_INVALIDFUNCTION;

	// Either ios::in or ios::out, or both, must be specified.  Note that we ignore
	// its value anyway (it's always treated as ios::in | ios::out).
	if (!(nMode & (ios::in | ios::out))) return STG_E_INVALIDFLAG;

	// Try to flush the buffer.
	HRESULT hr = hr_sync();
	if (FAILED(hr)) return hr;

	// Convert the iostream seek direction into an COM stream seek direction.
	STREAM_SEEK skMode;
	switch (eDir)
	{
	case ios::beg:	skMode = STREAM_SEEK_SET; break;
	case ios::cur:	skMode = STREAM_SEEK_CUR; break;
	case ios::end:	skMode = STREAM_SEEK_END; break;
	default:		ERROR3("Bad seek direction in costreambuf::seekoff");
					return STG_E_INVALIDFLAG;
	}

	// Sign-extend the seek offset to 64 bits.
	LARGE_INTEGER li;
	li.LowPart = (DWORD) nOffset;
	li.HighPart = (nOffset < 0) ? -1 : 0;

	// Now move the seek pointer (note that as the input and output buffers are always tied,
	// we don't have a use for the nMode argument, which should always be ios::in | ios::out).
	ULARGE_INTEGER uhypNewPos;
	hr = m_pIStream->Seek(li, skMode, &uhypNewPos);
	if (FAILED(hr)) return hr;

	// Check if we have exceeded the range of streampos.
	if (uhypNewPos.HighPart != 0)
	{
		ERROR3("Seek offset too large for type 'streampos' in costreambuf::seekoff");
		return E_NOTIMPL;
	}

	// Return the new position.
	if (pnNewPos != 0) *pnNewPos = (streampos) uhypNewPos.LowPart;	
	return hr;
}



/********************************************************************************************
>	virtual int costreambuf::underflow()

	Author:		JustinF
	Created:	5/8/96	
	Returns:	EOF if the costreambuf is not currently attached to an open IStream; or if
				a buffer cannot be allocated; or if a character was not available to read.
				If successful returns the first character read.
	Purpose:	Called when the costreambuf underflows and more data must be read by it
				from an attached IStream.
	Errors:		(See Returns).
	SeeAlso:	costreambuf::hr_underflow; streambuf::underflow; streambuf::allocate;
				streambuf::sync; costreambuf::overflow; IStream::Read
********************************************************************************************/

int costreambuf::underflow()
{
	int nChar;
	return SUCCEEDED(hr_underflow(&nChar)) ? nChar : EOF;
}



/********************************************************************************************
>	virtual int costreambuf::overflow(int nChar = EOF)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		nChar		-	the character that has overflowed the buffer, by default
								none.
	Returns:	EOF if there isn't an attached open IStream interface, or if no buffer
				is allocated, or if the buffer cannot be flushed, or if the given character
				cannot be written.  Returns zero if successful.
	Purpose:	Called when the buffer overflows with the given character and data must be
				written to the attached IStream.
	Errors:		(See Returns).
	SeeAlso:	costreambuf::hr_overflow; streambuf::overflow; streambuf::allocate;
				streambuf::sync; costreambuf::underflow; IStream::Write
********************************************************************************************/

int costreambuf::overflow(int nChar)
{
	return SUCCEEDED(hr_overflow(nChar)) ? 0 : EOF;
}



/********************************************************************************************
>	virtual int costreambuf::sync()

	Author:		JustinF
	Created:	5/8/96
	Returns:	EOF if there is an error while trying to flush the buffer, or if there is
				no attached IStream interface, or if not all of the buffer was flushed.
				Returns zero if successful.
	Purpose:	Called when the pending-data buffer must be flushed.
	Errors:		(See Returns).
	SeeAlso:	streambuf::sync; IStream::Seek; costreambuf::overflow;
				costreambuf::underflow
********************************************************************************************/

int costreambuf::sync()
{
	return SUCCEEDED(hr_sync()) ? 0 : EOF;
}



/********************************************************************************************
>	virtual streampos costreambuf::seekoff(off_type nOffset,
										   ios::seek_dir eDir,
										   int nMode = ios::in | ios::out)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		nOffset		-	the new (relative) offset to seek to.
				eDir		-	the direction to seek to or the position to seek from.
								See the streambuf documentation for these.
				nMode		-	whether this call is for the input seek pointer or the
								output seek pointer.  This parameter is IGNORED as there
								is only one combined seek pointer (cf. the fstream class)
	Returns:	EOF if there is an error, or the new seek position of this costreambuf
				if successful.  Note that the streampos and streamoff types only have a
				32-bit range, even though the structured storage specification allows for
				64-bit seeks.  Trying to seek past the range allowed by streampos/streamoff
				is an error.
	Purpose:	Moves the seek pointer of the this costreambuf's attached IStream interface.
	Errors:		ERROR3 if eDir has an invalid value or if the seek position to return is
				outside the range of streampos.
	SeeAlso:	ios::seek_dir; streambuf::seekoff; fstream::seekoff; IStream::Seek
********************************************************************************************/

costreambuf::pos_type costreambuf::seekoff(off_type pos, ios_base::seekdir eDir, 
										   ios_base::openmode nMode /*= ios::in | ios::out*/)
{
	streampos sp;
	return SUCCEEDED(hr_seekoff(pos, eDir, &sp, nMode)) ? sp : EOF;
}

/********************************************************************************************
>	virtual streampos costreambuf::seekpos(pos_type pos,
										   ios::seek_dir eDir,
										   int nMode = ios::in | ios::out)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		nOffset		-	the new (relative) offset to seek to.
				eDir		-	the direction to seek to or the position to seek from.
								See the streambuf documentation for these.
				nMode		-	whether this call is for the input seek pointer or the
								output seek pointer.  This parameter is IGNORED as there
								is only one combined seek pointer (cf. the fstream class)
	Returns:	EOF if there is an error, or the new seek position of this costreambuf
				if successful.  Note that the streampos and streamoff types only have a
				32-bit range, even though the structured storage specification allows for
				64-bit seeks.  Trying to seek past the range allowed by streampos/streamoff
				is an error.
	Purpose:	Moves the seek pointer of the this costreambuf's attached IStream interface.
	Errors:		ERROR3 if eDir has an invalid value or if the seek position to return is
				outside the range of streampos.
	SeeAlso:	ios::seek_dir; streambuf::seekoff; fstream::seekoff; IStream::Seek
********************************************************************************************/

costreambuf::pos_type costreambuf::seekpos(pos_type pos, ios_base::openmode nMode /*= ios::in | ios::out*/)
{
	pos_type sp;
	return SUCCEEDED(hr_seekoff(pos, ios::beg, &sp, nMode)) ? sp : EOF;
}


/********************************************************************************************
>	virtual HRESULT costreambuf::do_clone(const costreambuf& other)

	Author:		JustinF
	Created:	9/8/96
	Inputs:		other		-		another costreambuf object
	Returns:	The error code for the IStream::Clone operation, NOERROR if OK.
	Purpose:	Changes this costreambuf to be an independent copy of the other.
	Errors:		(See Returns).
	SeeAlso:	costreambuf::costreambuf; costreambuf::operator=; IStream::Clone
********************************************************************************************/

HRESULT costreambuf::do_clone(const costreambuf& other)
{
	// Duplicate the size of the original streams buffers
	m_vecBufferIn.resize( other.m_vecBufferIn.size() );
	m_vecBufferOut.resize( other.m_vecBufferOut.size() );
	InitBuffers();

	// Make sure this stream releases it's interface pointers.  If the other stream is
	// not open then there's nothing else to do.
	if (is_open()) close();
	if (!other.is_open()) return NOERROR;

	// Get an independent "clone" of the other's IStream.  I assume here that the Clone
	// member AddRefs the IStream pointer it returns.
	IStream* pClone;
	HRESULT hr = ((IStream*) other)->Clone(&pClone);
	if (FAILED(hr)) return hr;

	// If the other costreambuf refers to a parent IStorage of its IStream, then add
	// a reference to it, so it won't disappear before this costreambuf has finished
	// with it.
	if ((m_pIStorage = other.m_pIStorage) != 0) m_pIStorage->AddRef();

	// Duplicate the other's append-mode flag.
	m_fAppend = other.m_fAppend;
	
	// Now properly attach the IStream to this costreambuf.
	return attach(pClone);
}



/********************************************************************************************
>	virtual streambuf* costreambuf::setbuf(char* pchReserve, int nLength)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pchReserve		-	pointer to the block of memory to use as a pending-data
									buffer.  If 0 then this stream becomes unbuffered.
				nLength			-	the number of characters in the block.
	Returns:	Zero if the costreambuf is attached to an open IStream interface and there
				is already a buffer allocated (cf. costream::setbuf), (this) if successful.
	Purpose:	Sets this costreambuf to use the given area as a pending-data buffer.
	Errors:		(See Returns).
	SeeAlso:	costreambuf::is_open; streambuf::ebuf; streambuf::unbuffered;
				streambuf::setb
********************************************************************************************/

streambuf* costreambuf::setbuf(char* pchReserve, int nLength)
{
//	assert(FALSE);
	//CAM_ENSURE( FALSE, "Please tell Luke that we need setbuf" );
	return this;

/*	// Make sure the stream *isn't* open with data in the old reserve.
	if (is_open() && ebuf() != 0) return 0;

	// If no valid reserve area is passed then "unbuffer" then stream, otherwise set the
	// new reserve area.
	if (pchReserve != 0 && nLength > 0)
		setb(pchReserve, pchReserve + nLength, 0);
	else
		unbuffered(1);

	// Success.
	return this; */
}



/********************************************************************************************
>	static const WCHAR* costreambuf::convert_string(const CHAR* pcsz)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pcsz	-	pointer to the ANSI string to convert to the Unicode character
							set.
	Returns:	A pointer to a temporary (NOT thread-safe) buffer containing the equivalent
				string in the Unicode character set.  Make sure you have finished with this
				return value before convert_string is called again!  Returns 0 if
				the ANSI string isn't valid ANSI, or is 0.
	Purpose:	Converts an ANSI string into a Unicode string, as required by the COM
				libraries.  This helper function is not required in Unicode builds.
	Errors:		ERROR2 if the given string is too long (more than _MAX_PATH characters).
	SeeAlso:	mbstowcs; _MAX_PATH
********************************************************************************************/

const WCHAR* costreambuf::convert_string(const CHAR* pcsz)
{
	// Make sure we really have something to convert.
	if (pcsz == 0) return 0;

	// Check if the string is too long.
	const size_t nchMax = _MAX_PATH + 1;
	ERROR3IF(strlen(pcsz) > nchMax, "ANSI string too long in costreambuf::convert_string" );
	
	// Convert in our private buffer (NB. this is *not* reentrant code!)
	static WCHAR wcszBuf[nchMax];
	size_t nResult = mbstowcs(wcszBuf, pcsz, lstrlenA(pcsz) + 1);
	ERROR3IF(nResult == -1, "Invalid ANSI string in costreambuf::convert_string");
	return (nResult != -1 && nResult < nchMax) ? wcszBuf : 0;
}



/********************************************************************************************
>	virtual int costreambuf::ios_to_stgm(int* pnMode, int* pnProt, DWORD* pgrfResult) const

	Author:		JustinF
	Created:	6/8/96
	Inputs:		pnMode		-		pointer to the ios I/O mode to convert.
				pnProt		-		pointer to the ios sharing mode to convert.
	Outputs:	pnMode		-		the fully-specified ios I/O mode, with all implied modes
									made explicit.
				pnProt		-		the fully-specified ios sharing mode, with all implied
									modes made explicit.
				pgrfResult	-		points to where to store the converted mode, expressed
									as STGM flags.  This can be passed to the structured
									storage functions.
	Returns:	Non-zero if the mode is valid, zero if it is invalid (eg. neither ios::in
				or ios::out were specified).
	Purpose:	Converts a file-access and sharing mode, as enumerated by the C++ stream
				library, into its fully-specified equivalent for the iostream library and
				COM structured storage.  This function allows the open_* functions to take
				ios-like access modes, which makes the class more integrated with the C++
				stream library and makes porting existing ios calls much easier.  BE VERY
				CAREFUL IF YOU MODIFY THIS FUNCTION, AS THE LOGIC IS QUITE INTRICATE (many
				modes imply other modes: if the caller omits to specify modes that are
				implied by other modes, this function will "fill in the gaps").
	Errors:		ERROR3 if the given ios mode is invalid or cannot be converted to an COM
				equivalent.
	SeeAlso:	costreambuf::stgm_to_ios; costreambuf::open_file; costreambuf::open_stream;
				::StgOpenStorage; ::StgCreateDocfile
********************************************************************************************/

int costreambuf::ios_to_stgm(int* pnMode, int* pnProt, DWORD* pgrfResult) const
{
	// Check that we have somewhere to store the results.
	ERROR3IF(pnMode == 0 || pnProt == 0 || pgrfResult == 0,
			 "Null input/output parameter in costreambuf::ios_to_stgm");

	// Convert the modes.  Some modes imply others (or the opposite of others).
	*pgrfResult = 0;

	// Structured storage has no notion of text files, so force ios::binary.
	if (!(*pnMode & ios::binary))
	{
		TRACE( _T("costreambuf::ios_to_stgm: attempt to open costream in ios::text mode\n") );
		*pnMode |= ios::binary;
	}

	// What about "simple" mode?
	if (*pnMode & costream::simple)
	{
		// Yes, we want the efficient "simple" mode.  "Simple" mode implies STGM_CREATE,
		// STGM_READWRITE, and STGM_SHARE_EXCLUSIVE.
		*pnMode = (*pnMode | ios::in | ios::out) & ~ios::_Noreplace;
		*pnProt = (*pnProt | costream::sh_none) & ~(costream::sh_read | costream::sh_write);
		*pgrfResult |= STGM_SIMPLE;
	}

	// What about "priority" mode?
	if (*pnMode & costream::priority)
	{
		// Yes, we want "priority" mode.  "Priority" mode implies STGM_DIRECT and STGM_READ.
		*pnMode = (*pnMode | ios::in) & ~costream::transacted;
		*pgrfResult |= STGM_PRIORITY;
	}

	// Delete the stream when it is finally closed?
	if (*pnMode & costream::temp)
	{
		// Yes, this is a scratch, temporary file.
		*pgrfResult |= STGM_DELETEONRELEASE;
	}

	// Is this stream etc being "converted"?
	if (*pnMode & costream::convert)
	{
		// Yes, this is a conversion.  This implies that the old stream must exist, in
		// order to be converted, and that access should be read-write.
		*pnMode = (*pnMode | ios::in | ios::out) & ~ios::_Noreplace;
		*pgrfResult |= STGM_CONVERT;
	}

	// Is this "no scratch file" mode (only supported under Windows 95)?
	if (*pnMode & costream::noscratch)
	{
		*pgrfResult |= STGM_NOSCRATCH;
	}

	// Work out if we are asking for transacted or direct I/O.
	if (*pnMode & costream::transacted)
	{
		// We want transactions.
		*pgrfResult |= STGM_TRANSACTED;
	}

	// If we are asked to truncate the file then we must be outputing.
	if (*pnMode & ios::trunc) *pnMode |= ios::out;
	
	// Work out if we are creating a new stream etc or only opening an existing one.
	// The COM default is that we want the stream to already exist.
	if (!(*pnMode & ios::_Nocreate))
	{
		// Do we insist that the stream must not already exist?
		if (*pnMode & ios::_Noreplace)
		{
			// Yes, the stream must not exist.
			*pgrfResult |= STGM_FAILIFTHERE;
		}
		else
		{
			// We want to create a new stream irrespective of whether it already exists
			// or not.
			*pgrfResult |= STGM_CREATE;
		}
	}

	// Work out if we are reading or writing (we must choose one or the other).
	if (*pnMode & ios::out)
	{
		// We're writing.  Are we reading as well?
		if (*pnMode & ios::in)
		{
			// Yes, mode is read and write.
			*pgrfResult |= STGM_READWRITE;
		}
		else
		{
			// No, mode is only write.
			*pgrfResult |= STGM_WRITE;
		}

		// If we are not reading and not appending and not creating then we must be
		// truncating.
		if (!(*pnMode & (ios::in | ios::app | ios::ate | ios::_Noreplace)))
		{
			*pnMode |= ios::trunc;
		}
	}
	else if (*pnMode & ios::in)
	{
		// We're only reading.
		*pgrfResult |= STGM_READ;
	}
	else
	{
		// The access mode hasn't specified either ios::in or ios::out.  It must as there
		// is no default mode.
		ERROR3("Invalid access mode in costreambuf::ios_to_stgm");
		return 0;
	}

	// Work out how to share the stream.
	switch (*pnProt & (costream::sh_read | costream::sh_write | costream::sh_none))
	{
	case costream::sh_read:
		*pgrfResult |= STGM_SHARE_DENY_WRITE;
		break;

	case costream::sh_write:
		*pgrfResult |= STGM_SHARE_DENY_READ;
		break;

	case costream::sh_none:
		*pgrfResult |= STGM_SHARE_EXCLUSIVE;
		break;

	default:
		ERROR3("No sharing-mode specified in costream::ios_to_stgm");
		// Falls through to the official STGM_SHARE_* default sharing mode ...

	case costream::sh_read | costream::sh_write:
		*pgrfResult |= STGM_SHARE_DENY_NONE;		
		break;
	}

	// Success.
	return 1;
}



/********************************************************************************************
>	virtual int costreambuf::stgm_to_ios(DWORD* pgrfMode, int* pnMode, int* pnProt) const

	Author:		JustinF
	Created:	17/8/96
	Inputs:		pgrfMode	-	the STGM mode to convert.
	Outputs:	pgrfMode	-	the fully-specified STGM access mode.  This can be passed
								to the structured storage functions.
				pnMode		-	if not null, the fully-specified ios I/O mode, with all
								implied modes made explicit, that is the equivalent of
								*pgrfMode.
				pnProt		-	if not null, the fully-specified ios sharing mode, with all
								implied modes made explicit, that is the equivalent of
								*pgrfMode.
	Returns:	Non-zero if the mode is valid, zero if it is invalid (ie. it cannot be
				represented by an ios/costream equivalent).
	Purpose:	Converts a file-access and sharing mode, as enumerated by the C++ stream
				library, into its fully-specified equivalent for COM structured storage.
				BE VERY CAREFUL IF YOU MODIFY THIS FUNCTION, AS THE LOGIC IS QUITE INTRICATE
				(many modes imply other modes: if the caller omits to specify modes that are
				implied by other modes, this function will "fill in the gaps").
	Notes:		What this function returns is dependent on the particular implementation
				of Structured Storage beneath it.  You can only reliably expect ios::binary,
				ios::in, ios::out, ios::app and the ios::sh_* sharing modes to be returned
				correctly.  Other modes are unlikely to be recorded with the stream, acting
				only when a stream is created or opened.
	Errors:		ERROR3 if the given ios mode is invalid or cannot be converted to an STGM
				equivalent.
	SeeAlso:	costreambuf::ios_to_stgm; costream::get_access_mode;
				costreambuf::get_access_mode; costreambuf::open_stream
********************************************************************************************/

int costreambuf::stgm_to_ios(DWORD* pgrfMode, int* pnMode, int* pnProt) const
{
	// Make sure we have something to do.
	ERROR3IF(pgrfMode == 0, "Null input parameter in costreambuf::stgm_to_ios");
	ERROR3IF(pnMode == 0 && pnProt == 0, "Redundant call to costreambuf::stgm_to_ios");

	// First, the open mode.
	if (pnMode != 0)
	{
		// Initialise the output, which is always 'binary'.
		*pnMode = ios::binary;

		// Check for read/write specified as read | write, which has a different value.
		const DWORD dwInvalid = STGM_READ | STGM_WRITE;
		if ((*pgrfMode & dwInvalid) == dwInvalid)
		{
			// Break in debug builds, correct in retail builds.
//			TRACEUSER("JustinF", "STGM_READ | WRITE specified in costreambuf::stgm_to_ios\n");
			*pgrfMode = (*pgrfMode & ~dwInvalid) | STGM_READWRITE;
		}

		// First, work out the update mode.
		switch (*pgrfMode & (STGM_DIRECT | STGM_TRANSACTED | STGM_SIMPLE))
		{
		case STGM_TRANSACTED:
			*pnMode |= costream::transacted;
			break;

		case STGM_SIMPLE:
			*pnMode |= costream::simple;
			break;

		case STGM_DIRECT:
			// This is the default if STGM_TRANSACTED is not specified.
			break;

		default:
			ERROR3("Unknown STGM update mode in costreambuf::stgm_to_ios");
			return 0;
		}

		// Next, the read/write mode.
		switch (*pgrfMode & (STGM_READ | STGM_WRITE | STGM_READWRITE))
		{
		case STGM_READ:
			*pnMode |= ios::in;
			break;

		case STGM_WRITE:
			*pnMode |= ios::out;
			break;

		case STGM_READWRITE:
			*pnMode |= ios::in | ios::out;
			break;

		default:
			ERROR3("Unknown STGM read/write mode in costreambuf::stgm_to_ios");
			return 0;
		}

		// Next, the creation mode.
		switch (*pgrfMode & (STGM_CREATE | STGM_CONVERT | STGM_FAILIFTHERE))
		{
		case STGM_CREATE:
			// This is the default for the iostream library, !nocreate && !noreplace
			break;

		case STGM_CONVERT:
			*pnMode |= costream::convert | ios::_Nocreate;
			break;

		case STGM_FAILIFTHERE:
			*pnMode |= ios::_Noreplace;
			break;

		default:
			ERROR3("Unknown STGM creation mode in costreambuf::stgm_to_ios");
			return 0;
		}
		
		// Next, some miscellaneous singleton modes.
		if (*pgrfMode & STGM_PRIORITY) *pnMode |= costream::priority;
		if (*pgrfMode & STGM_DELETEONRELEASE) *pnMode |= costream::temp;
		if (*pgrfMode & STGM_NOSCRATCH) *pnMode |= costream::noscratch;

		// Finally, if we know that this stream was opened with ios::app, return
		// that as well.  Note that we never return ios::ate or ios::trunc.
		if (m_fAppend) *pnMode |= ios::app;
	}

	// Next, the sharing mode.
	if (pnProt != 0)
	{
		// This is easy as the STGM flags are mutually exclusive, unlike the
		// costream equivalents.
		switch (*pgrfMode & (STGM_SHARE_DENY_READ | STGM_SHARE_DENY_WRITE |
							 STGM_SHARE_DENY_NONE | STGM_SHARE_EXCLUSIVE))
		{
		case STGM_SHARE_DENY_READ:
			*pnProt = costream::sh_write;
			break;

		case STGM_SHARE_DENY_WRITE:
			*pnProt = costream::sh_read;
			break;

		case STGM_SHARE_DENY_NONE:
			*pnProt = costream::sh_read | costream::sh_write;
			break;

		case STGM_SHARE_EXCLUSIVE:
			*pnProt = costream::sh_none;
			break;

		default:
			ERROR3("Unknown STGM_SHARE_* mode in costreambuf::stgm_to_ios");
			*pnProt = 0;
			return 0;
		}
	}
	
	// Success.
	return 1;
}



/////////////////////////////////////////////////////////////////////////////////////////////



/********************************************************************************************
>	costream::costream(IStream* pIStream = 0)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStream	-	A pointer to a valid IStream interface that will be used
								for I/O by this costream.  By default null, ie. the
								costream is not yet attached to any IStream interface.
	Purpose:	Constructs an costream, optionally already attached to an open IStream
				interface.  If no IStream* is passed then you should call attach or one of
				the open_* functions explicitly before trying to do I/O.  If you wish to
				prevent the costream releasing the IStream when it is destroyed then
				make sure you've called AddRef on it one extra time.
	SeeAlso:	costreambuf; costream::open_file; costream::open_stream;
				costream::attach; costream::is_open; costream::~costream;
				IStream; IStream::AddRef
********************************************************************************************/

costream::costream(IStream* pIStream)
  :	iostream(new costreambuf(pIStream))
{
	// Tell our inherited stream objects that they are responsible for deallocating buffers.
/*	istream::delbuf(1);
	ostream::delbuf(1); */
}



/********************************************************************************************
>	costream::costream(const TCHAR* pcszFilePath,
					   int nMode,
					   int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pcszFilePath	-	the full path to the non-COM file that is to be
									opened as a "pseudo-" structured storage.
				nMode			-	the access mode to be used to open the file; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									file; see the C++ stream documentation.
	Purpose:	Constructs an costream attached to a a non-structured storage file, using
				the STGM_CONVERT access mode.  This is like calling the default constructor
				followed by a call to open_file.
	SeeAlso:	costreambuf; IStream; costream::open_file; costream::~costream;
				costream::on_fail
********************************************************************************************/

costream::costream(const TCHAR* pcszFilePath, int nMode, int nProt)
  : iostream(new costreambuf)
{
	// Tell our inherited stream objects that they are responsible for deallocating buffers.
/*	istream::delbuf(1);
	ostream::delbuf(1); */

	// Try to open the non-COM file as a stream.
	HRESULT hr = rdbuf()->open_file(pcszFilePath, nMode, nProt);
	if (FAILED(hr))
	{
		// We failed so update the stream's state flags.
		on_fail(hr);
		istream::setstate( istream::failbit );
		ostream::setstate( ostream::failbit );
	}
}



/********************************************************************************************
>	costream::costream(IStorage* pIStorage, const TCHAR* pcszStreamName,
						 int nMode,
						 int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStorage		-	the open IStorage interface that contains the IStream
									this costream will be constructed from.
				pcszStreamName	-	the name of the IStream within the IStorage.
				nMode			-	the access mode to be used to open the stream; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									stream; see the C++ stream documentation.
	Purpose:	Constructs this costream from an IStream within the given IStorage.
	SeeAlso:	costream::costream; costream::open_stream; costream::close;
				costream::attach; IStorage::OpenStream
********************************************************************************************/

costream::costream(IStorage* pIStorage, const TCHAR* pcszStreamName, int nMode, int nProt)
  :	iostream(new costreambuf)
{
	// Tell our inherited stream objects that they are responsible for deallocating buffers.
/*	istream::delbuf(1);
	ostream::delbuf(1); */

	// Try to open the stream.
	HRESULT hr = rdbuf()->open_stream(pIStorage, pcszStreamName, nMode, nProt);
	if (FAILED(hr))
	{
		// We failed so update the stream's state flags.
		on_fail(hr);
		istream::setstate( istream::failbit );
		ostream::setstate( ostream::failbit );
	}
}



/********************************************************************************************
>	costream::costream(IStream* pIStream, char* pchReserve, int nLength)

	Author:		JustinF
	Created:	7/8/96
	Inputs:		pIStream		-	A pointer to a valid IStream interface.  If 0 then
									the costream is not attached to any IStream yet.
				pchReserve		-	the memory to use as a pending-data buffer.  If 0
									then the costream is unbuffered.
				nLength			-	the size of the given memory, in characters.  If zero
									then the costream is unbuffered.
	Purpose:	Constructs an costream which will use the given reserve area as a
				buffer.
	SeeAlso:	costream::costream; costreambuf::costreambuf
********************************************************************************************/

#if defined(NEED_TO_SPEC_BUFFER_IN_COSTRAMBUF)
costream::costream(IStream* pIStream, char* pchReserve, int nLength)
  :	iostream(new costreambuf(pIStream, pchReserve, nLength))
{
	// Tell our inherited stream objects that they are responsible for deallocating buffers.
/*	istream::delbuf(1);
	ostream::delbuf(1); */
}
#endif


/********************************************************************************************
>	costream::costream(const costream& other)

	Author:		JustinF
	Created:	7/8/96
	Inputs:		other		-		the other costream that this should be constructed
									as a copy of.
	Purpose:	Copy-constructor for an costream.  If the other costream is attached to
				an open IStream then its stream is "cloned".
	SeeAlso:	costream::costream; costreambuf::costreambuf; IStream::Clone
********************************************************************************************/

costream::costream(const costream& other)
  :	iostream(new costreambuf(*other.rdbuf()))
{
	// Tell our inherited stream objects that they are responsible for deallocating buffers.
/*	istream::delbuf(1);
	ostream::delbuf(1); */

	// These constructors should throw exceptions!  This is a bodgy test to see if a
	// properly heinous state has come about.
	if (is_open() != other.is_open())
	{
		// We failed so update the stream's state flags.  Note the faked HRESULT.
		on_fail(STG_E_INSUFFICIENTMEMORY);
		istream::setstate( istream::failbit );
		ostream::setstate( ostream::failbit );
	}
}



/********************************************************************************************
>	virtual costream::~costream()

	Author:		JustinF
	Created:	5/8/96
	Purpose:	Destroys an costream.  If the costream is attached to an open IStream
				interface then the buffer is flushed and the IStream is released.  Any
				IStorage pointers that were created by the costream are also released.
				If you wish to prevent the costream releasing the attached IStream when it
				is destroyed then make sure you've called AddRef on it.
	SeeAlso:	costream::costream; costreambuf::~costreambuf
********************************************************************************************/

costream::~costream()
{
	delete rdbuf();
}



/********************************************************************************************
>	virtual int costream::is_open() const

	Author:		JustinF
	Created:	5/8/96
	Returns:	Non-zero (TRUE) if this costream is currently attached to an open valid
				IStream interface, zero (FALSE) if it isn't.
	Purpose:	Tests if this stream is attached to an open IStream.
	SeeAlso:	costream::attach; costream::get_IStream; costreambuf::is_open
********************************************************************************************/

int costream::is_open() const
{
	return rdbuf()->is_open();
}



/********************************************************************************************
>	virtual costreambuf* costream::rdbuf() const

	Author:		JustinF
	Created:	5/8/96
	Returns:	A pointer to the costreambuf associated with this costream.
	Purpose:	Low-level hacker function.  Allows direct calls to the costreambuf object.
	SeeAlso:	ios::rdbuf; costreambuf
********************************************************************************************/

costreambuf* costream::rdbuf() const
{
	// Remember that there is only one buffer that is used for both input and output, so we
	// can call either istream::rdbuf or ostream::rdbuf, they return the same object.
	return (costreambuf*) ostream::rdbuf();
}



/********************************************************************************************
>	virtual costream::operator IStream*() const

	Author:		JustinF
	Created:	5/8/96
	Returns:	The IStream interface that this costream is currently attached to, or
				0 if it isn't attached.
	Purpose:	Allows direct access to the attached IStream.  Handle the return value
				with care, or you'll screw the stream!
	SeeAlso:	costream::attach; costream::is_open; costreambuf::get_IStream
********************************************************************************************/

costream::operator IStream*() const
{
	return *rdbuf();
}



/********************************************************************************************
>	virtual streambuf* costream::setbuf(char* pchReserve, int nLength)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pchReserve		-	pointer to the block of memory to use as a pending-data
									buffer.  If 0 then this stream becomes unbuffered.
				nLength			-	the number of characters in the block.
	Returns:	0 if the costream is attached to an open IStream interface or if
				there is already a buffer allocated.
	Purpose:	Sets this costream to use the given area as a pending-data buffer.
	Errors:		(See Returns).
	SeeAlso:	costream::is_open; costream::on_fail; costreambuf::setbuf;
				streambuf::ebuf; streambuf::unbuffered; streambuf::setb
********************************************************************************************/

streambuf* costream::setbuf(char* pchReserve, int nLength)
{
	// The stream must not be open already.
	if (is_open())
	{
		on_fail(STG_E_INVALIDFUNCTION);
		return 0;
	}
	
	// Try to allocate a buffer.
	if (!rdbuf()->setbuf(pchReserve, nLength))
	{
		// It failed.
		on_fail(STG_E_INSUFFICIENTMEMORY);
		return 0;
	}

	// Success.
	return rdbuf();
}



/********************************************************************************************
>	virtual int costream::get_access_mode(int* pnMode, int* pnProt) const

	Author:		JustinF
	Created:	17/8/96
	Outputs:	pnMode		-	if not null then the access-mode used to open this
								costreambuf's attached IStream.
				pnProt		-	if not null then the sharing-mode used to open this
								costreambuf's Istream.
	Returns:	Non-zero if successful, zero if the costream is not	attached, or if it's
				IStream was opened in a mode that has no ios/costream equivalent etc.
	Purpose:	Returns the access and sharing modes used to open the IStream that this
				constreambuf is attached to, such as ios::in, costream::sh_none etc.
	Errors:		(See Returns).
	SeeAlso:	costreambuf::get_access_mode
********************************************************************************************/

int costream::get_access_mode(int* pnMode, int* pnProt) const
{
	// Delegate this to the costreambuf.
	return SUCCEEDED(rdbuf()->get_access_mode(pnMode, pnProt));
}



/********************************************************************************************
>	virtual HRESULT costream::set_size(size_t cbNewSize)

	Author:		JustinF
	Created:	9/8/96
	Inputs:		cbNewsize		-	the new size of the pre-allocated space for the
									stream, in bytes.
	Returns:	NOERROR if successful.  STG_E_INVALIDFUNCTION is this costreambuf is not
				currently attach to an open IStream.  See IStream::SetSize for other error
				values.
	Purpose:	Sets the amount of structured storage to pre-allocate for this costream.
				Streams, of course, will expand if written past their end - this call is a
				good way to reserve some disk space before committing anything important
				to it.
	Errors:		(See Returns).
	SeeAlso:	costream; costreambuf::set_size; IStream::SetSize
********************************************************************************************/

HRESULT costream::set_size(size_t cbNewSize)
{
	// Delegate this to the buffer object.
	HRESULT hr = rdbuf()->set_size(cbNewSize);
	if (FAILED(hr)) on_fail(hr);
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costream::attach(IStream* pIStream)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStream	-	the IStream to attach to.  If you wish to retain control
								over when the IStream is released then call AddRef on it
								before the attached costream is destroyed, as it will
								try to destroy it for you otherwise.
	Returns:	An COM error code; or NOERROR if successful.
	Purpose:	Attaches an already-open IStream interface to this costream, so that
				it can be treated as a C++ stream.  If the stream is buffer then it
				allocates a block of memory for pending data transfers.
	Errors:		Sets the stream's 'fail' flag if the stream is already attached to an
				open IStream interface.
	SeeAlso:	costream::close; costreambuf::attach; costream::~costream;
				costream::on_fail; streambuf::unbuffered
********************************************************************************************/

HRESULT costream::attach(IStream* pIStream)
{
	// Delegate this to the attached costreambuf object.
	HRESULT hr = rdbuf()->attach(pIStream);
	if (FAILED(hr)) on_fail(hr);
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costreambuf::close()

	Author:		JustinF
	Created:	5/8/96
	Returns:	An COM error code; or NOERROR if successful.
	Purpose:	Closes the attached IStream interface, flushing any pending data and releasing
				the IStream.  If you want to prevent the costream releasing the IStream
				then make sure you've called AddRef on the IStream one extra time.
	Errors:		Sets the stream's 'fail' flag if the stream cannot be closed, or isn't
				already attached and open.
	SeeAlso:	costream::~costream; costream::attach; costream::is_open;
				costream::on_fail; IStream::AddRef; IStream::Release
********************************************************************************************/

HRESULT costream::close()
{
	// Delegate this to the attached costreambuf object.
	HRESULT hr = rdbuf()->close();
	if (FAILED(hr))
	{
		// It failed.
		on_fail(hr);
	}
	else
	{
		// Success, so clear the inherited streams' state.
		istream::clear();
		ostream::clear();
	}

	// Return the success code.
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costream::open_stream(IStorage* pIStorage,
										  const TCHAR* pcszStreamName,
										  int nMode,
										  int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pIStorage		-	the open IStorage interface that contains the IStream
									to be opened.
				pcszStreamName	-	the name of the IStream to be opened.
				nMode			-	the access mode to be used to open the stream; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									stream; see the C++ stream documentation.
	Returns:	STG_E_INVALIDPOINTER if pIStorage is 0; STG_E_INVALIDNAME if the
				stream name is 0 or invalid ANSI; STG_E_INVALIDFUNCTION if this
				costream is currently open (ie. you must call close first) etc
	Purpose:	Opens an IStream for use by this costream.
	Errors:		ERROR3 if 0 is passed for pcszStreamName, or if no IStorage is is
				available.
	SeeAlso:	costream::open_file; costream::close; costream::attach;
				costream::on_fail; costreambuf::open_stream; IStorage::OpenStream
********************************************************************************************/

HRESULT costream::open_stream(IStorage* pIStorage, const TCHAR* pcszStreamName,
							   int nMode, int nProt)
{
	HRESULT hr = rdbuf()->open_stream(pIStorage, pcszStreamName, nMode, nProt);
	if (FAILED(hr)) on_fail(hr);
	return hr;
}



/********************************************************************************************
>	virtual HRESULT costream::open_file(const TCHAR* pcszFilePath,
										int nMode,
										int nProt = costream::sh_read | costream::sh_write)
	Author:		JustinF
	Created:	5/8/96
	Inputs:		pcszFilePath	-	the full path to the non-structured storage file to be
									opened.
				nMode			-	the access mode to be used to open the file; see
									the C++ stream documentation for these flags.
				nProt			-	the sharing access mode to be used to open the
									file; see the C++ stream documentation.
	Returns:	An COM error code, or NOERROR if successful.
	Purpose:	Opens a non-COM file as a "fake" structured storage.  The bytes in the file
				are accessed by opening a root-storage containing a single stream called
				"CONTENTS".  See the OLE docs for explanation of this special mode.
	Errors:		Sets the streams 'fail' bit if the file cannot be opened, or if the
				costream is already attached to an open IStream.
	SeeAlso:	costream::costream; costream::open_stream; costream::close;
				costream::on_fail; ::StgOpenStorage
********************************************************************************************/

HRESULT costream::open_file(const TCHAR* pcszFilePath, int nMode, int nProt)
{
	// Delegate this to the costreambuf.
	HRESULT hr = rdbuf()->open_file(pcszFilePath, nMode, nProt);
	if (FAILED(hr)) on_fail(hr);
	return hr;
}



/********************************************************************************************
>	virtual void costream::on_fail(HRESULT hr)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		hr		-		the COM error code generated by the operation that failed.
	Purpose:	Called whenever an costream I/O operation fails.  This function sets the
				'fail' flag of the costream.  Derived classes can override this function
				for more fancy error-handling.  Make sure you call this base-class version
				of the function at the beginning of function that overrides it.
********************************************************************************************/

void costream::on_fail(HRESULT /* hr */)
{
	// Reset the appropriate state flags in the inhertited stream objects.
	istream::clear(istream::rdstate() | istream::failbit);
	ostream::clear(ostream::rdstate() | ostream::failbit);
}



/////////////////////////////////////////////////////////////////////////////////////////////



// This is the one and only instance of this class.
costreambuf::init costreambuf::init::the_instance;



/********************************************************************************************
>	costreambuf::init::costreambuf::init(LPVOID lpvReserved = 0)

	Author:		JustinF
	Created:	5/8/96
	Inputs:		lpvReserved		-	not used, must be 0 under 32-bit COM.
	Purpose:	Tries to initialise the COM libraries, remembering if this is done
				successfully or not.
	Errors:		Throws an exception if the COM libraries cannot be initialised.
	SeeAlso:	costreambuf::init::~costreambuf::init; ::CoInitialize
********************************************************************************************/

costreambuf::init::init(LPVOID lpvReserved)
  :	m_fInitialised(SUCCEEDED(CoInitialize(lpvReserved)))
{
	if (!m_fInitialised)
	{
		ERROR3("Couldn't initialise COM libraries!");
	#if (_OLE_VER >= 0x200)
		AfxThrowOleException(E_OUTOFMEMORY);
	#endif
	}
}



/********************************************************************************************
>	costreambuf::init::~costreambuf::init()

	Author:		JustinF
	Created:	5/8/96
	Purpose:	Deinitialises the COM libraries provided that they were correctly
				initialised by the costreambuf::init constructor.
	SeeAlso:	costreambuf::init::costreambuf::init; ::CoUninitialize
********************************************************************************************/

costreambuf::init::~init()
{
	if (m_fInitialised) CoUninitialize();
}
