/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/


// Constants and types for the v2 file format

#ifndef INC_CXFDEFS
#define INC_CXFDEFS


/////////////////////////////////////////////////////////////////////////////
// Camelot version 1.5 tags
/////////////////////////////////////////////////////////////////////////////


#define CXF_IDWORD1			0x41524158
#define CXF_IDWORD2			0x0a0da3a3
#define CXF_UNKNOWN_SIZE	(-1)

//-------------------------------------------------------------------------------------
// Data Types
//-------------------------------------------------------------------------------------
typedef unsigned short	UINT16;
typedef short			INT16;

//-------------------------------------------------------------------------------------
// Record Data Constants
//-------------------------------------------------------------------------------------
#define CXF_TRANSPARANCY_TYPE_NONE			0
#define CXF_TRANSPARANCY_TYPE_MIX			1
#define CXF_TRANSPARANCY_TYPE_STAINEDGLASS	2
#define CXF_TRANSPARANCY_TYPE_BLEACH		3
#define CXF_TRANSPARENCY_TYPE_CONTRAST		13
#define CXF_TRANSPARENCY_TYPE_SATURATION	16
#define CXF_TRANSPARENCY_TYPE_DARKEN		19
#define CXF_TRANSPARENCY_TYPE_LIGHTEN		22
#define CXF_TRANSPARENCY_TYPE_BRIGHTNESS	25
#define CXF_TRANSPARENCY_TYPE_LUMINOSITY	28
// New transparency types:
// Paul: the following defs are used in filters only, we have to correct them here as
// oftenly XarLib is updated taking cxftags.h/cxfdefs.h from Camelot
#define CXF_TRANSPARENCY_TYPE_OVERLAY				43
#define CXF_TRANSPARENCY_TYPE_HARDLIGHT				46
#define CXF_TRANSPARENCY_TYPE_SOFTLIGHT				49
#define CXF_TRANSPARENCY_TYPE_LUMINOSITY2			52
#define CXF_TRANSPARENCY_TYPE_COLOUR				55
#define CXF_TRANSPARENCY_TYPE_ENHANCE				34
#define CXF_TRANSPARENCY_TYPE_BEVEL					37
#define CXF_TRANSPARENCY_TYPE_COLOUR_DODGE			61
#define CXF_TRANSPARENCY_TYPE_COLOUR_BURN			58

// Paul: the following defs are used in filters only, we have to correct them here as
// oftenly XarLib is updated taking cxftags.h/cxfdefs.h from Camelot
/*
enum LineCapType
{
	LineCapButt,
	LineCapRound,
	LineCapSquare
};
*/
#define CXF_CAP_STYLE_BUTT		0
#define CXF_CAP_STYLE_ROUND		1
#define CXF_CAP_STYLE_SQUARE		2
/*
enum JointType
{
	MitreJoin,
	RoundJoin,
	BevelledJoin
};
*/
#define CXF_JOIN_STYLE_MITRE	0
#define CXF_JOIN_STYLE_ROUND	1
#define CXF_JOIN_STYLE_BEVELLED	2

/*
enum WindingType
{
	NonZeroWinding,
	NegativeWinding,
	EvenOddWinding,
	PositiveWinding
};
*/
#define CXF_WINDING_RULE_NONZERO	0
#define CXF_WINDING_RULE_NEGATIVE	1
#define CXF_WINDING_RULE_EVENODD	2
#define CXF_WINDING_RULE_POSITIVE	3

//-------------------------------------------------------------------------------------
// References
//-------------------------------------------------------------------------------------

// Default bitmap definitions for the v2 file format
// These are the built in bitmaps that can be referred to by just their reference number
// as they are assumed to be built in.
// As with all other default references, all their numbers go from -1 downwards i.e. more negative
// and hence occupy the negative range of a LONG record number.
// Unlike colours, where this is defined in a separate file, cfxcols.h, the bitmaps are
// liklely to be the one default reference which is the default bitmap.

#define REF_DEFAULTBITMAP_BUILTIN	(-1)

//-------------------------------------------------------------------------------------
// Record sizes
//-------------------------------------------------------------------------------------

//-----------------
// Size of Compression records

#define TAG_STARTCOMPRESSION_SIZE	4
#define TAG_ENDCOMPRESSION_SIZE		8

//-----------------
// Document records
#define TAG_CHAPTER_SIZE			0
#define TAG_SPREAD_SIZE				0
#define TAG_SPREAD_PHASE2_SIZE		0
#define TAG_PAGE_SIZE				16
#define TAG_SPREADINFORMATION_SIZE	17

#define TAG_GRIDRULERSETTINGS_SIZE	17
#define TAG_GRIDRULERORIGIN_SIZE	8

//----------------
// Tag management
#define TAG_ATOMICTAGS_SIZE			CXF_UNKNOWN_SIZE
#define TAG_ESSENTIALTAGS_SIZE		CXF_UNKNOWN_SIZE
#define TAG_TAGDESCRIPTION_SIZE		CXF_UNKNOWN_SIZE

//-----------------
// Render-related tags
#define TAG_NONRENDERSECTION_START_SIZE	0
#define TAG_NONRENDERSECTION_END_SIZE	0
#define TAG_RENDERING_PAUSE_SIZE		0
#define TAG_RENDERING_RESUME_SIZE		0

//----------------
// Layers
#define TAG_LAYER_SIZE						0
#define TAG_LAYERDETAILS_SIZE				CXF_UNKNOWN_SIZE
#define TAG_EXTENDEDLAYERDETAILS_SIZE		CXF_UNKNOWN_SIZE
#define TAG_GUIDELAYERDETAILS_SIZE			CXF_UNKNOWN_SIZE
#define TAG_REGIONLAYERDETAILS_SIZE			CXF_UNKNOWN_SIZE
#define TAG_LAYER_FLAG_VISIBLE				0x1
#define TAG_LAYER_FLAG_LOCKED				0x2
#define TAG_LAYER_FLAG_PRINTABLE			0x4
#define TAG_LAYER_FLAG_ACTIVE				0x8
#define TAG_LAYER_FLAG_PAGEBACKGROUND		0x10
#define TAG_LAYER_FLAG_BACKGROUND			0x20
#define TAG_LAYER_FLAG_REGIONINVERT			0x40
#define TAG_LAYER_FLAG_AUTOCREATED  		0x80
#define TAG_LAYER_FLAG_PASTEBOARDBACKGROUND	0x100
#define TAG_LAYER_FLAG_REGIONPINK			0x200
#define TAG_LAYER_FLAG_REGIONSELECTION		0x400

// New Frame Layer tags in new flags byte.
#define TAG_LAYER_FLAG_SOLID		0x1 //
#define TAG_LAYER_FLAG_OVERLAY		0x2 //
#define TAG_LAYER_FLAG_HIDDEN		0x4 //

#define TAG_LAYER_FRAMEPROPS_SIZE	5
#define TAG_SPREAD_ANIMPROPS_SIZE	28


#define TAG_SPREADSCALING_ACTIVE_SIZE	24
#define TAG_SPREADSCALING_INACTIVE_SIZE	24

//-----------------
// Size of Colour reference records

#define TAG_DEFINERGBCOLOUR_SIZE		3
#define TAG_DEFINECOMPLEXCOLOUR_SIZE	29

//-----------------
// View records
#define TAG_VIEWPORT_SIZE		16
#define TAG_VIEWQUALITY_SIZE	1

#define TAG_DOCUMENTVIEW_SIZE	24

//-----------------
// Document unit tags

#define TAG_DEFINE_PREFIXUSERUNIT_SIZE		28
#define TAG_DEFINE_SUFFIXUSERUNIT_SIZE		28
#define TAG_DEFINE_DEFAULTUNITS_SIZE		8

//-----------------
// Document info tags

#define TAG_DOCUMENTDATES_SIZE				8
#define TAG_DOCUMENTUNDOSIZE_SIZE			4
#define TAG_DOCUMENTFLAGS_SIZE				4
#define TAG_DOCUMENTINFORMATION_SIZE		CXF_UNKNOWN_SIZE

//-----------------
// document nudge size tag
#define TAG_DOCUMENTNUDGESIZE						4
const LONG TAG_DOCUMENTNUDGEPHASE2SIZE				= sizeof(double)*1;

//-----------------
// Attribute record sizes
#define TAG_LINECOLOUR_SIZE							4
#define TAG_FLATFILL_SIZE							4
#define TAG_LINEWIDTH_SIZE							4
#define TAG_LINEARFILL_SIZE							40		// was 24, BUT + 16 for profile == 40
#define TAG_LINEARFILL3POINT_SIZE					48
#define TAG_CIRCULARFILL_SIZE						40		// was 24, BUT + 16 for profile == 40
#define TAG_ELLIPTICALFILL_SIZE						48		// was 32, BUT + 16 for profile == 48
#define TAG_CONICALFILL_SIZE						40		// was 24, BUT + 16 for profile == 40
#define TAG_SQUAREFILL_SIZE							48		// was 32, BUT + 16 for profile == 48
#define TAG_THREECOLFILL_SIZE						36
#define TAG_FOURCOLFILL_SIZE						40
#define TAG_BITMAPFILL_SIZE							44		// was 28, BUT + 16 for profile == 44
#define TAG_BITMAPCLONEFILL_SIZE					40		// 24 for three coords + 16 for profile == 40
#define TAG_BITMAPBOUNDSFILL_SIZE					28		// 24 for three coords + 4 for bitmap == 28
#define TAG_CONTONEBITMAPFILL_SIZE					52		// was 36, BUT + 16 for profile == 52
#define TAG_FRACTALFILL_SIZE						69		// was 53, BUT + 16 for profile == 69
#define TAG_NOISEFILL_SIZE							61		// was 45, BUT + 16 for profile == 61
#define TAG_FILLEFFECT_FADE_SIZE					0
#define TAG_FILLEFFECT_RAINBOW_SIZE					0
#define TAG_FILLEFFECT_ALTRAINBOW_SIZE				0
#define TAG_FILL_REPEATING_SIZE						0
#define TAG_FILL_NONREPEATING_SIZE					0
#define TAG_FILL_REPEATINGINVERTED_SIZE				0
//Mark Howitt, 8/10/97. define new repeating grad fill tag size.
#define TAG_FILL_REPEATING_EXTRA_SIZE				0
//Chris Snook, 8/12/99. We also need one for the transparencies!
#define TAG_TRANSPARENTFILL_REPEATING_EXTRA_SIZE	0

#define TAG_FLATTRANSPARENTFILL_SIZE				2
#define TAG_LINEARTRANSPARENTFILL_SIZE				35		// was 19, BUT + 16 for profile == 35
#define TAG_LINEARTRANSPARENTFILL3POINT_SIZE		43
#define TAG_CIRCULARTRANSPARENTFILL_SIZE			35		// was 19, BUT + 16 for profile == 35
#define TAG_ELLIPTICALTRANSPARENTFILL_SIZE			43		// was 27, BUT + 16 for profile == 43
#define TAG_CONICALTRANSPARENTFILL_SIZE				35		// was 19, BUT + 16 for profile == 35
#define TAG_SQUARETRANSPARENTFILL_SIZE				43		// was 27, BUT + 16 for profile == 43
#define TAG_THREECOLTRANSPARENTFILL_SIZE			28
#define TAG_FOURCOLTRANSPARENTFILL_SIZE				29
#define TAG_BITMAPTRANSPARENTFILL_SIZE				47		// was 31, BUT + 16 for profile == 47
#define TAG_FRACTALTRANSPARENTFILL_SIZE				64		// was 48, BUT + 16 for profile == 64
#define TAG_NOISETRANSPARENTFILL_SIZE				56		// was 40, BUT + 16 for profile == 56
#define TAG_LINETRANSPARENCY_SIZE					2
#define TAG_STARTCAP_SIZE							1
#define TAG_ENDCAP_SIZE								1
#define TAG_JOINSTYLE_SIZE							1
#define TAG_MITRELIMIT_SIZE							4
#define TAG_WINDINGRULE_SIZE						1
#define TAG_QUALITY_SIZE							4
#define TAG_TRANSPARENTFILL_REPEATING_SIZE			0
#define TAG_TRANSPARENTFILL_NONREPEATING_SIZE		0
#define TAG_TRANSPARENTFILL_REPEATINGINVERTED_SIZE	0
#define TAG_USERVALUEATTRSIZE						CXF_UNKNOWN_SIZE
#define TAG_WIZOP_STYLEREF_SIZE						4

#define TAG_FLATFILL_NONE_SIZE						0
#define TAG_FLATFILL_BLACK_SIZE						0
#define TAG_FLATFILL_WHITE_SIZE						0
#define TAG_LINECOLOUR_NONE_SIZE					0
#define TAG_LINECOLOUR_BLACK_SIZE					0
#define TAG_LINECOLOUR_WHITE_SIZE					0

//-----------------
// Path records
#define TAG_PATH_SIZE				CXF_UNKNOWN_SIZE
#define TAG_PATH_FLAGS_SIZE			CXF_UNKNOWN_SIZE
#define TAG_PATH_FLAGS_SMOOTH		0x1
#define TAG_PATH_FLAGS_ROTATE		0x2
#define TAG_PATH_FLAGS_ENDPOINT		0x4
#define TAG_PATH_FLAGS_TRACEANCHOR	0x8
#define TAG_PATH_RELATIVE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_PATHREF_INDENTICAL_SIZE	4
#define TAG_PATHREF_TRANSFORM_SIZE	28

//-----------------
// Arrows and dash patterns
#define TAG_DASHSTYLE_SIZE							4
#define TAG_ARROWHEAD_SIZE							12
#define TAG_ARROWTAIL_SIZE							12

//-----------------
// Blends
#define TAG_BLEND_SIZE				3
#define TAG_BLENDER_SIZE			8
#define TAG_BLEND_FLAG_ONETOONE		(1 << 0)	// Bit 0 is the one to one flag
#define TAG_BLEND_FLAG_ANTIALIAS	(1 << 1)	// Bit 1 is the antialias flag
#define TAG_BLEND_FLAG_TANGENTIAL	(1 << 2)	// Bit 2 is the tangential flag
#define TAG_BLEND_COLEFFECT_MASK	0xf0		// Top 4 bits of the flags byte contains the col blend type
#define TAG_BLEND_COLEFFECT_SHIFT	4			//		The col blend type can be either, fade, rainbow or alt rainbow

//-----------------
// Moulds
#define TAG_MOULD_ENVELOPE_SIZE		4
#define TAG_MOULD_PERSPECTIVE_SIZE	4
#define TAG_MOULD_GROUP_SIZE		0
#define TAG_MOULD_PATH_SIZE			CXF_UNKNOWN_SIZE

// ----------------
// node bitmap
#define TAG_NODE_BITMAP_SIZE				36
#define TAG_NODE_CONTONEDBITMAP_SIZE		44

// bitmap properties - stores the settings in the b.p. dialogue for each bitmap
#define TAG_BITMAP_PROPERTIES_SIZE			(sizeof(LONG)+8)
#define TAG_DOCUMENTBITMAPSMOOTHINGSIZE		5
#define TAG_BITMAPCHECKSUM_DUMMYVALUE		0xDeadBeef

//-----------------
// Misc objects
#define TAG_GUIDELINE_SIZE			5

// ----------------
// Printing and Imagesetting data
// Imagesetting attributes - these are all boolean tags with no actual record data
const LONG TAG_OVERPRINTLINEON_SIZE			= 0;
const LONG TAG_OVERPRINTLINEOFF_SIZE		= 0;
const LONG TAG_OVERPRINTFILLON_SIZE			= 0;
const LONG TAG_OVERPRINTFILLOFF_SIZE		= 0;
const LONG TAG_PRINTONALLPLATESON_SIZE		= 0;
const LONG TAG_PRINTONALLPLATESOFF_SIZE		= 0;
// Main printing and Imagesetting data records
const LONG TAG_PRINTERSETTINGS_SIZE			= 45;
const LONG TAG_IMAGESETTING_SIZE			= 15;
const LONG TAG_COLOURPLATE_SIZE				= 22;
// Printers mark records
const LONG TAG_PRINTMARKDEFAULT_SIZE		= 1;
const LONG TAG_PRINTMARKCUSTOM_SIZE			= CXF_UNKNOWN_SIZE;
const LONG TAG_PRINTERSETTINGS_PHASE2_SIZE	= CXF_UNKNOWN_SIZE;

//-----------------
// Size of transform attribute records

#define TAG_TRANSFORM_IDENTITY_SIZE			0		
#define TAG_TRANSFORM_MATRIX_SIZE			16	
#define TAG_TRANSFORM_TRANSLATION_SIZE		8
#define TAG_TRANSFORM_GENERAL_SIZE			24
#define TAG_TRANSFORM_ROTATION_SIZE			4
#define TAG_TRANSFORM_ROTATION_CENTRE_SIZE	4
#define TAG_TRANSFORM_ROTATION_POINT_SIZE	12
#define TAG_TRANSFORM_SHEAR_X_SIZE			4
#define TAG_TRANSFORM_SHEAR_Y_SIZE			4
#define TAG_TRANSFORM_SCALE_SIZE			4
#define TAG_TRANSFORM_SCALE_X_SIZE			4
#define TAG_TRANSFORM_SCALE_Y_SIZE			4
#define TAG_TRANSFORM_REFLECTION_X_SIZE		0
#define TAG_TRANSFORM_REFLECTION_Y_SIZE		0

//-----------------
// Size of regular shape records
// Not all of these are defined in the spec, those that are are marked as such.
// Note - any object with reformed edges automatically has an unknown size.

// rectangle			: size += 0
// ellipse				: size += 0
// polygon				: size += 2  (number of sides)
// simple				: size += 16 (centre.x, centre.y, width, height)
// complex				: size += 24 (centre.x, centre.y, major.x, major.y, minor.x, minor.y)
// roundedness			: size += 8  (roundedness)
// stellation			: size += 16
// rounded + stellated	: size += 8
// reformed				: size = CXF_UNKNOWN_SIZE

// Ellipses
#define TAG_ELLIPSE_SIMPLE_SIZE									16					// defined
#define TAG_ELLIPSE_COMPLEX_SIZE								24					// defined

// Rectangles
#define TAG_RECTANGLE_SIMPLE_SIZE								16					// defined
#define TAG_RECTANGLE_SIMPLE_REFORMED_SIZE						CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_SIMPLE_STELLATED_SIZE						32
#define TAG_RECTANGLE_SIMPLE_STELLATED_REFORMED_SIZE			CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_SIMPLE_ROUNDED_SIZE						24					// defined
#define TAG_RECTANGLE_SIMPLE_ROUNDED_REFORMED_SIZE				CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED_SIZE				48
#define TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED_REFORMED_SIZE	CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_COMPLEX_SIZE								24					// defined
#define TAG_RECTANGLE_COMPLEX_REFORMED_SIZE						CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_COMPLEX_STELLATED_SIZE					40
#define TAG_RECTANGLE_COMPLEX_STELLATED_REFORMED_SIZE			CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_COMPLEX_ROUNDED_SIZE						32					// defined
#define TAG_RECTANGLE_COMPLEX_ROUNDED_REFORMED_SIZE				CXF_UNKNOWN_SIZE
#define TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED_SIZE			56
#define TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED_REFORMED_SIZE	CXF_UNKNOWN_SIZE

// Polygons
#define TAG_POLYGON_COMPLEX_SIZE								26					// defined
#define TAG_POLYGON_COMPLEX_REFORMED_SIZE						CXF_UNKNOWN_SIZE	// defined
#define TAG_POLYGON_COMPLEX_STELLATED_SIZE						42					// defined
#define TAG_POLYGON_COMPLEX_STELLATED_REFORMED_SIZE				CXF_UNKNOWN_SIZE	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_SIZE						34					// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_REFORMED_SIZE				CXF_UNKNOWN_SIZE	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_STELLATED_SIZE				58					// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_STELLATED_REFORMED_SIZE		CXF_UNKNOWN_SIZE

// General regular shapes
// (You need to read the prose above the corresponding definitions in kernel/cxftags.h
// if you wish to know the differences between these two - ach)
#define TAG_REGULAR_SHAPE_PHASE_1_SIZE				CXF_UNKNOWN_SIZE
#define TAG_REGULAR_SHAPE_PHASE_2_SIZE				CXF_UNKNOWN_SIZE

// Text definitions

#define TAG_FONT_DEF_TRUETYPE_SIZE					CXF_UNKNOWN_SIZE
#define TAG_FONT_DEF_ATM_SIZE						CXF_UNKNOWN_SIZE

// Text objects

#define TAG_TEXT_STORY_SIMPLE_SIZE					12		// was 8, but + 4 for new autokern flag == 12
#define TAG_TEXT_STORY_COMPLEX_SIZE					28		// was 24, but + 4 for new autokern flag == 28

const ULONG TAG_TEXT_STORY_INSIDE_SIMPLE_SIZE = TAG_TEXT_STORY_SIMPLE_SIZE + 4; // plus we need to save the padding value
const ULONG TAG_TEXT_STORY_INSIDE_COMPLEX_SIZE = TAG_TEXT_STORY_COMPLEX_SIZE + 4;

#define TAG_TEXT_STORY_SIMPLE_START_LEFT_SIZE		12		// was 8, but + 4 for new autokern flag == 12
#define TAG_TEXT_STORY_SIMPLE_START_RIGHT_SIZE		12		// was 8, but + 4 for new autokern flag == 12
#define TAG_TEXT_STORY_SIMPLE_END_LEFT_SIZE			12		// was 8, but + 4 for new autokern flag == 12
#define TAG_TEXT_STORY_SIMPLE_END_RIGHT_SIZE		12		// was 8, but + 4 for new autokern flag == 12
#define TAG_TEXT_STORY_COMPLEX_START_LEFT_SIZE		36		// was 32, but + 4 for new autokern flag == 36
#define TAG_TEXT_STORY_COMPLEX_START_RIGHT_SIZE		36		// was 32, but + 4 for new autokern flag == 36
#define TAG_TEXT_STORY_COMPLEX_END_LEFT_SIZE		36		// was 32, but + 4 for new autokern flag == 36
#define TAG_TEXT_STORY_COMPLEX_END_RIGHT_SIZE		36		// was 32, but + 4 for new autokern flag == 36

#define TAG_TEXT_STORY_WORD_WRAP_INFO_SIZE			5
#define TAG_TEXT_STORY_INDENT_INFO_SIZE				8

#define TAG_TEXT_LINE_SIZE							0
#define TAG_TEXT_STRING_SIZE						CXF_UNKNOWN_SIZE
#define TAG_TEXT_CHAR_SIZE							2
#define TAG_TEXT_EOL_SIZE							0
#define TAG_TEXT_KERN_SIZE							8
#define TAG_TEXT_CARET_SIZE							0
#define TAG_TEXT_LINE_INFO_SIZE						12

// Text attributes

#define TAG_TEXT_LINESPACE_RATIO_SIZE				4
#define TAG_TEXT_LINESPACE_ABSOLUTE_SIZE			4
#define TAG_TEXT_JUSTIFICATION_LEFT_SIZE			0
#define TAG_TEXT_JUSTIFICATION_CENTRE_SIZE			0
#define TAG_TEXT_JUSTIFICATION_RIGHT_SIZE			0
#define TAG_TEXT_JUSTIFICATION_FULL_SIZE			0
#define TAG_TEXT_FONT_TYPEFACE_SIZE					4
#define TAG_TEXT_FONT_SIZE_SIZE						4				
#define TAG_TEXT_BOLD_ON_SIZE						0
#define TAG_TEXT_BOLD_OFF_SIZE						0
#define TAG_TEXT_ITALIC_ON_SIZE						0
#define TAG_TEXT_ITALIC_OFF_SIZE					0
#define TAG_TEXT_UNDERLINE_ON_SIZE					0
#define TAG_TEXT_UNDERLINE_OFF_SIZE					0
#define TAG_TEXT_SCRIPT_ON_SIZE						8	// explicit offset, size
#define TAG_TEXT_SCRIPT_OFF_SIZE					0
#define TAG_TEXT_SUPERSCRIPT_ON_SIZE				0	// implicit offset, size
#define TAG_TEXT_SUBSCRIPT_ON_SIZE					0	// implicit offset, size
#define TAG_TEXT_TRACKING_SIZE						4
#define TAG_TEXT_ASPECT_RATIO_SIZE					4
#define TAG_TEXT_BASELINE_SIZE						4
#define TAG_TEXT_STRIKETHROUGH_ON_SIZE				0
#define TAG_TEXT_STRIKETHROUGH_OFF_SIZE				0


/////////////////////////////////////////////////////////////////////////////
// Camelot version 2.0 - additional tags
/////////////////////////////////////////////////////////////////////////////

// ----------------
// New-style strokes
const LONG TAG_VARIABLEWIDTHFUNC_SIZE		= 4;
const LONG TAG_VARIABLEWIDTHTABLE_SIZE		= CXF_UNKNOWN_SIZE;
const LONG TAG_STROKETYPE_SIZE				= 4;
const LONG TAG_STROKEDEFINITION_SIZE		= CXF_UNKNOWN_SIZE;
const LONG TAG_STROKEAIRBRUSH_SIZE			= CXF_UNKNOWN_SIZE;

// Mould source object bounds
const LONG TAG_MOULD_BOUNDS_SIZE			= 16;

// Web Address records
const LONG TAG_WEBADDRESS_SIZE				= CXF_UNKNOWN_SIZE;

// Template Argument/Ops / WizOp / Webster Properties records
const LONG TAG_WIZOP_SIZE					= CXF_UNKNOWN_SIZE;

/////////////////////////////////////////////////////////////////////////////
/// Camelot version 2.1 tags i.e. next version after 2.0
/////////////////////////////////////////////////////////////////////////////

//-----------------
// Shadows
const LONG TAG_SHADOWCONTROLLER_SIZE		= 29;
const LONG TAG_SHADOW_SIZE					= 24;	// was 16, BUT + 8 for Darkness = 24.

//-----------------
// bevel attributes
#define TAG_BEVEL_INDENT_SIZE				4	// long
#define TAG_BEVEL_DPI_SIZE					4
#define TAG_BEVEL_TYPE_SIZE					4
#define TAG_BEVEL_LIGHTANGLE_SIZE			8	// double

//-----------------
// Blend on a curve tags
const LONG TAG_BLENDER_CURVEPROP_SIZE		= (sizeof(double)*2);
const LONG TAG_BLEND_PATH_SIZE				= CXF_UNKNOWN_SIZE;
const LONG TAG_BLENDER_CURVEANGLES_SIZE		= (sizeof(double)*2);
const LONG TAG_BLENDPROFILES_SIZE			= (sizeof(double)*6);
const LONG TAG_BLENDERADDITIONAL_SIZE		= 17;	// was 16, BUT + 1 for 1-BYTE bitfield storage = 17.
const LONG TAG_NODEBLENDPATH_FILLED_SIZE	= 4;

//-----------------
// Contour tags
const LONG TAG_CONTOUR_CONTROLLER_SIZE		= 4 + 4 + 1 + 8 + 8 + 8 + 8;
const LONG TAG_CONTOUR_SIZE					= 0;

//-----------------
#define TAG_LINEARFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_LINEARFILLMULTISTAGE3POINT_SIZE	CXF_UNKNOWN_SIZE
#define TAG_CIRCULARFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_CONICALFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_ELLIPTICALFILLMULTISTAGE_SIZE	CXF_UNKNOWN_SIZE
#define TAG_SQUAREFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE

#define TAG_LINEARTRANSPARENTFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_LINEARTRANSPARENTFILLMULTISTAGE3POINT_SIZE	CXF_UNKNOWN_SIZE
#define TAG_CIRCULARTRANSPARENTFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_CONICALTRANSPARENTFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE
#define TAG_ELLIPTICALTRANSPARENTFILLMULTISTAGE_SIZE	CXF_UNKNOWN_SIZE
#define TAG_SQUARETRANSPARENTFILLMULTISTAGE_SIZE		CXF_UNKNOWN_SIZE

//-----------------
// Sets and their properties.
const LONG TAG_SETSENTINEL_SIZE				= 0;
const LONG TAG_SETPROPERTY_SIZE				= CXF_UNKNOWN_SIZE;
const LONG TAG_BARPROPERTY_SIZE				= CXF_UNKNOWN_SIZE;

//-----------------
// painting mode size tags
//const LONG TAG_PAINTINGMODE_SIZE			= 1;

// brush attribute tags
const ULONG TAG_BRUSHATTR_SIZE				= (sizeof(LONG) * 4) + (sizeof(double) *2) +1;
const ULONG TAG_BRUSHDEFINITION_SIZE		= (sizeof(LONG)); //CXF_UNKNOWN_SIZE;
#define		TAG_BRUSHTILE_FLAG				0x1
#define		TAG_BRUSHROTATE_FLAG			0x2
const ULONG TAG_BRUSHDATA_SIZE				= (ULONG)CXF_UNKNOWN_SIZE; //TAG_BRUSHATTR_SIZE;
const ULONG TAG_MOREBRUSHDATA_SIZE			= (sizeof(LONG) * 11) + (sizeof(double) *3);
const ULONG TAG_MOREBRUSHATTR_SIZE			= TAG_MOREBRUSHDATA_SIZE + (sizeof(LONG));
const ULONG TAG_EVENMOREBRUSHDATA_SIZE		= (sizeof(LONG) * 2);
const ULONG TAG_EVENMOREBRUSHATTR_SIZE		= TAG_EVENMOREBRUSHDATA_SIZE + 1;
const ULONG TAG_TIMESTAMPBRUSHDATA_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;
const ULONG TAG_BRUSHPRESSUREINFO_SIZE		= sizeof(LONG) * 7;
const ULONG TAG_BRUSHATTRPRESSUREINFO_SIZE	= sizeof(LONG) * 7;
const ULONG TAG_BRUSHPRESSUREDATA_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;
const ULONG TAG_BRUSHSAMPLEDATA_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;
const ULONG TAG_BRUSHATTRFILLFLAGS_SIZE		= 1;
const ULONG TAG_BRUSHTRANSPINFO_SIZE		= (sizeof(LONG)*6) + (sizeof(double)*2);
const ULONG TAG_BRUSHATTRTRANSPINFO_SIZE	= (sizeof(LONG)*6) + (sizeof(double)*2);

#define		BRUSHFILLFLAG_LOCALFILL			0x1
#define		BRUSHFILLFLAG_LOCALTRANSP		0x2
#define		BRUSHFILLFLAG_NAMEDCOL			0x4
#define TAG_SCALETOWIDTH_FLAG               0x1

//-----------------
// ClipView tags
const LONG TAG_CLIPVIEW_SIZE				= 0;
const LONG TAG_CLIPVIEW_CONTROLLER_SIZE		= 0;

//-----------------
// Feather attribute
const ULONG TAG_FEATHER_SIZE				= 20;	// Millipoint value. was 4, BUT + 16 for profile == 20

//-----------------
// Current Attribute Group
const ULONG TAG_CURRENTATTRIBUTES_SIZE		= 1;
	const ULONG ATTRIBUTEGROUP_INK			= 1;
	const ULONG ATTRIBUTEGROUP_TEXT			= 2;
	const ULONG ATTRIBUTEGROUP_DIMENSION	= 3;
	const ULONG ATTRIBUTEGROUP_OPEN_INK		= 4;
	const ULONG ATTRIBUTEGROUP_MASK			= 5;
	const ULONG ATTRIBUTEGROUP_BRUSH		= 6;
const ULONG TAG_CURRENTATTRIBUTEBOUNDS_SIZE = sizeof(LONG) * 4;

//-----------------
// Duplication Offset record
const ULONG TAG_DUPLICATIONOFFSET_SIZE		= sizeof(LONG) * 2;

//-----------------
// CompoundRender records size
const ULONG TAG_COMPOUNDRENDER_SIZE			= 20;	// Flags + 2 * Coord

//-----------------
// ObjectBounds records size
const ULONG TAG_OBJECTBOUNDS_SIZE			= 16;		// 2 * Coord


const ULONG TAG_SPREAD_ANIMPROPS2_SIZE		= sizeof(LONG)*1;
const ULONG TAG_SPREAD_FLASHPROPS_SIZE		= sizeof(LONG)*3;
const ULONG TAG_SPREAD_FLASHPROPS2_SIZE		= sizeof(LONG)*1;
const ULONG TAG_SPREAD_FLASHPROPS3_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;
const ULONG TAG_SPREAD_FLASHPROPS4_SIZE		= sizeof(LONG)*1;

const ULONG TAG_BITMAPCONTRAST_SIZE			= sizeof(double)*1;
const ULONG TAG_BITMAPCROSSPROCESS_SIZE		= sizeof(double)*1;
const ULONG TAG_BITMAPRANGE_SIZE			= sizeof(BYTE)*1;
const ULONG TAG_BITMAPSHARPEN_SIZE			= sizeof(LONG)*1;
const ULONG TAG_BITMAPSATURATION_SIZE		= sizeof(double)*1;
const ULONG TAG_BITMAPINTELLIGENTSCALE_SIZE	= sizeof(LONG)*13 + 1;
const ULONG TAG_BITMAPISCALE_SIZE			= sizeof(LONG) + sizeof(double)*2 + 1;
const ULONG TAG_BITMAPISCALE2_SIZE			= sizeof(LONG) + sizeof(double)*4 + 1;

const ULONG TAG_EXTRUSIONDEPTH_SIZE			= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONANGLE1_SIZE		= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONANGLE2_SIZE		= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONANGLE3_SIZE		= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONBEVELDEPTH_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONGLOSS_SIZE			= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONBEVELTYPE_SIZE		= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONCORNERTYPE_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT1ANGLE1_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT1ANGLE2_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT1COLOUR_SIZE	= 4; // Copied from TAG_FLATFILL_SIZE for one colour reference?
const ULONG TAG_EXTRUSIONLIGHT2ANGLE1_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT2ANGLE2_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT2COLOUR_SIZE	= 4; // Copied from TAG_FLATFILL_SIZE for one colour reference?
const ULONG TAG_EXTRUSIONLIGHT3ANGLE1_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT3ANGLE2_SIZE	= sizeof(LONG) * 1;
const ULONG TAG_EXTRUSIONLIGHT3COLOUR_SIZE	= 4; // Copied from TAG_FLATFILL_SIZE for one colour reference?
const ULONG TAG_EXTRUSIONPERSPECTIVE_SIZE	= sizeof(LONG) * 1;

const ULONG TAG_EXTRUSIONGROUP_SIZE					= 0;
const ULONG TAG_EXTRUSIONCONTROLLER_SIZE			= 0;
const ULONG TAG_EXTRUSIONCONTROLLER_TRANS_SIZE		= 8;
const ULONG TAG_EXTRUSIONCONTROLLER_COMPLEX_SIZE	= 24;

//-----------------
// Node record size and flags
const ULONG TAG_NODEDETAILS_SIZE	= sizeof(BYTE)*1;
const ULONG TAG_NODE_FLAG_VISIBLE	= 0x1;
const ULONG TAG_NODE_FLAG_LOCKED	= 0x2;
const ULONG TAG_NODEMASKDETAILS_SIZE= sizeof(BYTE)*1;
const ULONG TAG_NODE_FLAG_PREMASK	= 0x1;


/////////////////////////////////////////////////////////////////////////////
// XaraLX 0.6
/////////////////////////////////////////////////////////////////////////////

// ------------------------------------------------------------
// New text tags for tabbing and paragraph-based indent support

// Text objects
const UINT32 TAG_TEXT_TAB_SIZE				= 0;

// Text attributes
const UINT32 TAG_TEXT_LEFT_INDENT_SIZE		= 4;
const UINT32 TAG_TEXT_FIRST_INDENT_SIZE		= 4;
const UINT32 TAG_TEXT_RIGHT_INDENT_SIZE		= 4;
const UINT32 TAG_TEXT_RULER_SIZE			= UINT32(CXF_UNKNOWN_SIZE);

const UINT32 TAG_TEXT_STORY_HEIGHT_INFO_SIZE     = 4;
const UINT32 TAG_TEXT_STORY_LINK_INFO_SIZE       = 9;	// BYTE + 2 ULONGs
const UINT32 TAG_TEXT_STORY_TRANSLATION_INFO_SIZE = 8;

const UINT32 TAG_TEXT_STORY_INSIDE_SHAPE_INFO_SIZE = 4; // one LONG/MILLIPOINT
const UINT32 TAG_TEXT_SPACE_BEFORE_SIZE		= 4;
const UINT32 TAG_TEXT_SPACE_AFTER_SIZE		= 4;

const UINT32 TAG_TEXT_SPECIAL_HYPHEN_SIZE   = 1;  // BYTE
#define		 TEXT_SPECIAL_HYPHEN_IS_SOFT    0x1
#define		 TEXT_SPECIAL_HYPHEN_KILL_PREV  0x2
#define		 TEXT_SPECIAL_HYPHEN_IS_AUTO	0x4	  // overrides TEXT_SPECIAL_HYPHEN_IS_SOFT

const UINT32 TAG_TEXT_SOFT_RETURN_SIZE      = 0;

const UINT32 TAG_TEXT_EXTRA_FONT_INFO_SIZE  = 4;
const UINT32 TAG_TEXT_EXTRA_FONT_DEF_SIZE = UINT32(CXF_UNKNOWN_SIZE);
// values for the Flags field
#define		 TEXT_EXTRA_FONT_DEF_BOLD				0x01
#define		 TEXT_EXTRA_FONT_DEF_ITALIC				0x02
#define		 TEXT_EXTRA_FONT_DEF_REPLACED			0x04
#define		 TEXT_EXTRA_FONT_DEF_REPLACED_BOLD		0x08
#define		 TEXT_EXTRA_FONT_DEF_REPLACED_ITALIC	0x10
#define		 TEXT_EXTRA_FONT_DEF_MISSING_VARIANT    0x20
#define		 TEXT_EXTRA_FONT_DEF_MULTI_WEIGHT		0x40
#define		 TEXT_EXTRA_FONT_DEF_FLAGS2             0x80
// values for the Flags2 field
#define      TEXT_EXTRA_FONT_DEF_GOOGLE_FONT			0x01
#define      TEXT_EXTRA_FONT_DEF_REPLACED_GOOGLE_FONT	0x02

const UINT32	TAG_EFFECTATTR_FLAG_SIZE	= 0;
const UINT32	TAG_PHOTOATTR_FLAG_SIZE	= 0;

const ULONG TAG_SYMBOLDEFINITION_SIZE			= UINT32(CXF_UNKNOWN_SIZE);
const ULONG TAG_SYMBOLDEFINITION_LINKED_SIZE	= UINT32(CXF_UNKNOWN_SIZE);	// Not used at present
const ULONG TAG_SYMBOLREF_TRANS_SIZE			= 12;	// Ref + DocCoord
const ULONG TAG_SYMBOLREF_COMPLEX_SIZE			= 28;	// Ref + Matrix

const UINT32 TAG_FONT_GLYPH_SIZE			= UINT32(CXF_UNKNOWN_SIZE);
// values for the flags field (NB - the bold flag does not describe the boldness of the variant
// itself but the bold flag that is used to select this variant, which is a different thing for
// fonts with two bold or two non-bold weights)
#define      FONTGLYPH_BOLD                         0x01
#define      FONTGLYPH_ITALIC                       0x02
#define		 FONTGLYPH_HAS_INTERNAL_LEADING			0x04
#define		 FONTGLYPH_HAS_EXTERNAL_LEADING			0x08

const UINT32 TAG_FONT_KERNINGPAIRS_SIZE     = UINT32(CXF_UNKNOWN_SIZE);
// values for the flags field (see FONTGLYPH_... above)
#define      FONTKERNING_BOLD                       0x01
#define      FONTKERNING_ITALIC                     0x02

const UINT32 TAG_FONT_EMBEDDING_POLICY_SIZE = 4; // 2 bytes per security flags + 2 byte of glyph flag 

const UINT32 TAG_TEXT_LANGUAGE_SIZE					= 8;

const UINT32 TAG_TEXT_HYPHENATION_SIZE_OFF				= 0; // (separate tag used...)
const UINT32 TAG_TEXT_HYPHENATION_SIZE_RELATIVE = 6 * 1 + 4 + 4; // we do not store flag for isEnabled and isRelative (separate tag used...)
const UINT32 TAG_TEXT_HYPHENATION_SIZE_ABSOLUTE = 6 * 1 + 4 + 4; // we do not store flag for isEnabled and isRelative (separate tag used...)
const UINT32 TAG_TEXT_HYPHENATION_SIZE_DOCUMENT_DEFAULT = 0; // (separate tag used...)

const UINT32 TAG_DOCUMENT_URL_COLOUR_SIZE			= 4;
const UINT32 TAG_DOCUMENT_MOUSE_OVER_URL_COLOUR_SIZE= 4;
const UINT32 TAG_DOCUMENT_VISITED_URL_COLOUR_SIZE	= 4;

const UINT32 TAG_DOCUMENT_COLOUR_SIZE				= 4;

const UINT32 TAG_TEXT_ANCHOR_POSITION_SIZE			= UINT32(CXF_UNKNOWN_SIZE);
const UINT32 TAG_ANCHORED_GROUP_INFO_SIZE			= 12;	// Tag + DocCoord

const UINT32 TAG_TEXT_BULLETED_ITEM_START_SIZE		= 10;	// UINT16, BOOL, BOOL, LONG and WCHAR
const UINT32 TAG_TEXT_NUMBERED_ITEM_START_SIZE		= UINT32(CXF_UNKNOWN_SIZE);

const UINT32 TAG_SOFTMASKGROUP_SIZE					= 44;	// 3x DocCoord, 3x BYTE, 2x DOUBLE, 1x BYTE

const UINT32 TAG_BITMAPDEFAULTTRANS_SIZE			= 1;	// 1x BYTE

const UINT32 TAG_CLONEFLAG_SIZE						= 0;	// If present is clone, else not

const UINT32 TAG_NAVBAR_START_SIZE					= 36;	// ULONG, Matrix, LONG, LONG

const UINT32 TAG_PRESENTATION_PAGE_PROPERTIES_SIZE	= 8;	// LONG, LONG
const UINT32 TAG_PRESENTATION_LAYER_PROPERTIES_SIZE = 8;	// LONG, LONG
const UINT32 TAG_PRESENTATION_LAYER_FLAG_SIZE		= 4;	// BOOL

// values for the NavBar flags field (See class NavBarFlags)
#define NAVBAR_FLAGS_IS_VERTICAL					0x0001
#define NAVBAR_FLAGS_LEFTTOPJUSTIFIED				0x0002
#define NAVBAR_FLAGS_RIGHTBOTTOMJUSTIFIED			0x0004
#define NAVBAR_FLAGS_STRETCHY						0x0008
#define NAVBAR_FLAGS_KEEPSAMEBUTTONWIDTHS			0x0010
#define NAVBAR_FLAGS_FIRSTBUTTONDIFFERENT			0x0020
#define NAVBAR_FLAGS_LASTBUTTONDIFFERENT			0x0040
#define NAVBAR_FLAGS_REVERSEORDER					0x0080
#define NAVBAR_FLAGS_PRIMARY						0x0100
#define NAVBAR_FLAGS_AUTOMATIC						0x0200
#define NAVBAR_FLAGS_AUTOORDER						0x0400
#define NAVBAR_FLAGS_AUTOMENU						0x0800

const UINT32 TAG_NAVBAR_END_SIZE					= 0;

const UINT32 TAG_NAVBAR_BUTTON_STRETCH_INFO_SIZE	= 16;	// 4x LONG

const ULONG TAG_DOCUMENT_HIGHSLIDE_INFO_SIZE		= 6 * sizeof(BYTE) + sizeof(LONG);	// 6 BOOLs + 1 LONG

const UINT32 TAG_NAVBAR_BUTTON_MENU_SIZE			= 0;
const UINT32 TAG_NAVBAR_BUTTON_MENU_ITEM_SIZE		= UINT32(CXF_UNKNOWN_SIZE);
const UINT32 TAG_NAVBAR_BUTTON_MENU_SEPARATOR_SIZE	= 0;
const UINT32 TAG_COMPATIBILITYATTR_FLAG_SIZE		= 0;

const UINT32 TAG_RESOLUTION_HINT_SIZE				= sizeof(double);

const UINT32 TAG_SYMBOLGROUP_SIZE					= 24 + sizeof(BYTE);		// Matrix + 1 BOOL

const UINT32 TAG_REPEL_DISTANCE_H_SIZE				= 4;	// LONG
const UINT32 TAG_REPEL_DISTANCE_V_SIZE				= 4;	// LONG
const UINT32 TAG_ANCHORED_GROUP_INFO2_SIZE			= 2;	// 2x BOOL
const UINT32 TAG_ANCHORED_GROUP_INFO3_SIZE			= 1;	// 1x BOOL

const UINT32 TAG_WIDGETREFERENCE_SIZE				= 4;	// Ref

const UINT32 TAG_EMBEDDED_ANCHORED_GROUP_INFO_SIZE	= 1;	// 1x BOOL
const UINT32 TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO_SIZE = 33;	// 8x LONG, 1x BOOL
const UINT32 TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO_SIZE	= 17;	// 1x BOOL, 4x LONG
const UINT32 TAG_ANCHORED_GROUP_SCALING_INFO_SIZE = 5; // 1x MILLIPOINT or 1 1x Fixed16 + BYTE for flags
const UINT32 TAG_ANCHORED_GROUP_SCALING_INFO_DISABLED_SIZE = 0; // just the tag is saved...
const UINT32 TAG_ANCHORED_GROUP_MARGINS_SIZE = 4 * sizeof(LONG); // 4 millipoint values

const UINT32 TAG_SHUFFLE_SIZE						= 3 * sizeof(LONG);

const UINT32 TAG_CLONESOURCE_SIZE					= 8;	// 1x coord
const UINT32 TAG_BITMAP_HIDDEN_SIZE					= 0;	// Nothing!
const UINT32 TAG_STYLE_SIZE							= 4;	// 1x LONG - also for TAG_TEXT_PARAGRAPH_STYLE
const UINT32 TAG_TEXT_PRESENT_BY_PARAGRAPH_SIZE		= 1;	// 1x BOOL
const UINT32 TAG_LOCK_POPUP_LAYER_SIZE				= 4;	// BOOL
const UINT32 TAG_CLONESOURCE2_SIZE					= 24;	// Matrix

// values for the Style flags field
#define STYLE_FLAGS_IS_NORMAL_STYLE                 0x00000001
#define STYLE_FLAGS_KEEP_EVEN_IF_UNUSED             0x00000002
#define STYLE_FLAGS_IS_HIDDEN						0x00000004

const UINT32 TAG_WIDGETDESIGNSIZE_SIZE				= 8;	// 2x LONG
const UINT32 TAG_WIDGET_EXTRAINFO_SIZE				= 12;	// 3x LONG

const UINT32 TAG_TEXT_SIMPLE_MACRO_MARKER_SIZE		= 4;	// 2x UINT16

// values for the Simple Macro flags field
#define SIMPLE_MACRO_FLAGS_TYPE_UNKNOWN             0x0001

const UINT32 TAG_TEXT_MACRO_PAGE_NUMBER_OFFSET_SIZE = 4;		// 1x LONG

const UINT32 TAG_TEXT_MACRO_CHAR_SIZE				= 2;	// 1x UINT16

const UINT32 TAG_ONLINEINFO_NEXT_ID_SIZE			= 4;	// 1x LONG
const UINT32 TAG_ONLINEINFO_ID_SIZE					= 4;	// 1x LONG

// values for the WSM NavBar Flags field
const UINT32 WSM_NAVBAR_FLAGS_ADD_PAGES				= 0x00000001;	// add new pages automatically

// values for the WSM NavBar alignment field
const BYTE WSM_NAVBAR_ALIGN_NONE					= 0;	// No alignment set up in WSM
const BYTE WSM_NAVBAR_ALIGN_TL						= 1;
const BYTE WSM_NAVBAR_ALIGN_T						= 2;
const BYTE WSM_NAVBAR_ALIGN_TR						= 3;
const BYTE WSM_NAVBAR_ALIGN_L						= 4;
const BYTE WSM_NAVBAR_ALIGN_C						= 5;
const BYTE WSM_NAVBAR_ALIGN_R						= 6;
const BYTE WSM_NAVBAR_ALIGN_BL						= 7;
const BYTE WSM_NAVBAR_ALIGN_B						= 8;
const BYTE WSM_NAVBAR_ALIGN_BR						= 9;

const UINT32 TAG_WSM_NAVBAR_SIZE					= 29;	// 4x LONG, 1x BYTE, 3x FIXED16
const UINT32 TAG_WSM_NAVBAR_BUTTON_SIZE				= UINT32(CXF_UNKNOWN_SIZE);
const UINT32 TAG_TEXT_LINE_INFO2_SIZE				= 16;

const UINT16 FONT_RECORD_FLAG_REPLACED				= 1u;
const UINT16 FONT_RECORD_FLAG_MULTI_WEIGHT			= 2u;
const UINT16 FONT_RECORD_FLAG_WEIGHT_LIST			= 4u;
const UINT16 FONT_RECORD_FLAG_HAS_ALTERNATIVE_WEIGHT = 8u;	// alternative numeric weight follows
const UINT16 FONT_RECORD_FLAG_IS_ALTERNATIVE_WEIGHT = 16u;	// this record represents the alternative weight, not the base weight

const UINT16 FONT_VARIANT_FLAG_HAS_ITALIC			= 1u;

const UINT32 TAG_TEXT_BREAK_TYPE_SIZE				= 1u;	// 1x BYTE

const BYTE TEXT_BREAK_TYPE_COLUMN_BREAK				= 1u;
const BYTE TEXT_BREAK_TYPE_PAGE_BREAK_ANY			= 2u;
const BYTE TEXT_BREAK_TYPE_PAGE_BREAK_LEFT			= 3u;
const BYTE TEXT_BREAK_TYPE_PAGE_BREAK_RIGHT			= 4u;
const BYTE TEXT_BREAK_TYPE_PICTURE_BREAK_ANY		= 5u;
const BYTE TEXT_BREAK_TYPE_PICTURE_BREAK_LEFT		= 6u;
const BYTE TEXT_BREAK_TYPE_PICTURE_BREAK_RIGHT		= 7u;

const LONG TAG_LIVESHARE_SIZE						= CXF_UNKNOWN_SIZE;
const LONG TAG_PAGEVARIANTDEFINITION_SIZE			= CXF_UNKNOWN_SIZE;
const LONG TAG_PAGEVARIANTREFERENCE_SIZE			= sizeof(LONG);

const LONG TAG_IMPLICIT_TAB_DISTANCE_SIZE			= sizeof(LONG);

const LONG TAG_EMBEDDED_GLYPHS_SIZE_HINT_SIZE		= 20u;	// 5x ULONG

const LONG TAG_DOCUMENTVIEW_VARIANT_SIZE			= sizeof(LONG);
const LONG TAG_VARIANT_STYLE_DELTA_SIZE				= TAG_PAGEVARIANTREFERENCE_SIZE;

const LONG TAG_ANCHORING_MODE_SIZE					= 2;	// 2x BYTE
const LONG TAG_PAGEVARIANTDESCRIPTION_SIZE			= CXF_UNKNOWN_SIZE;

const LONG TAG_TXTBACKGROUNDCOLOUR_SIZE				= 4;	// 1x Colour reference
const LONG TAG_TXTBACKGROUNDCOLOUR_NONE_SIZE		= 0;
const LONG TAG_TXTBACKGROUNDCOLOUR_BLACK_SIZE		= 0;
const LONG TAG_TXTBACKGROUNDCOLOUR_WHITE_SIZE		= 0;

const LONG TAG_MOULDBRUSHATTR_SIZE					= sizeof(ULONG) + sizeof(ULONG);	// Brush handle + flags
const ULONG TAG_BRUSHDEFINITION_MOULD_SIZE			= sizeof(ULONG) + sizeof(ULONG) + sizeof(ULONG);
const ULONG TAG_MOULDBRUSHDATA_SIZE					= (ULONG)CXF_UNKNOWN_SIZE;

const ULONG TAG_TEXT_SYMBOL_CHAR_SIZE				= sizeof(WCHAR) + sizeof(LONG); // wchar + fontref (long)
const ULONG TAG_TEXT_SYMBOL_CHAR_POS_SIZE			= static_cast<ULONG>(CXF_UNKNOWN_SIZE);
const ULONG TAG_TEXT_ALM_INDENT_LEVEL_SIZE			= sizeof(UINT16);
const ULONG TAG_TEXT_ALM_LINKS_SIZE					= 2 * sizeof(BYTE); // 2 x BOOL flags

const ULONG TAG_TEXT_TOC_MARKER_OUTLINE_LEVEL_SIZE	= sizeof(BYTE);
const ULONG TAG_TEXT_TOC_MARKER_FLAGS_SIZE	= sizeof(UINT16);

const ULONG TAG_ONLINE_PASTE_INFO_SIZE				= sizeof(LONG) * 7;		// a DocCoord, a DocRect and an enum

const UINT32 TAG_FONT_OTLIGATURES_SIZE				= UINT32(CXF_UNKNOWN_SIZE);
const INT16 TAG_TEXT_OPEN_TYPE_LIGATURES_SIZE		= sizeof(INT16);

const UINT32 TAG_FONT_OTSTYLISTICSETS_SIZE			= UINT32(CXF_UNKNOWN_SIZE);
const INT16 TAG_TEXT_OPEN_TYPE_STYLISTIC_SET_SIZE	= sizeof(ULONG);
const INT16 TAG_TEXT_LIGATURE_START_SIZE			= sizeof(WCHAR);
const UINT32 TAG_FONT_OTSTYLISTICSETNAMES_SIZE		= UINT32(CXF_UNKNOWN_SIZE);
const INT16 TAG_TEXT_STYLISTIC_SET_SUB_SIZE			= sizeof(WCHAR);

const LONG TAG_DOCUMENTHISTORY_ENTRY_SIZE           = CXF_UNKNOWN_SIZE;

const UINT16 FONT_DEF_VARIANT_FLAGS_ITALIC			= 1;
const UINT16 FONT_DEF_VARIANT_FLAGS_REPLACED		= 2;

const ULONG TAG_ANCHORED_GROUP_PAGE_OFFSET_SIZE		= 6;		// A LONG, a byte and a BOOL

const ULONG TAG_TXT_LIST_PROPERTIES_SIZE			= 4 + 2 * sizeof(ULONG); // font ref + 2 level values
const ULONG TAG_TXT_LIST_NUMBER_PROPERTIES_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;
const ULONG TAG_TXT_LIST_BULLET_PROPERTIES_SIZE		= TAG_TXT_LIST_PROPERTIES_SIZE + sizeof(WCHAR); // additionally a bullet char
const ULONG TAG_TXT_LIST_FONT_SIZE_SIZE				= 4;
const ULONG TAG_TXT_LIST_VERTICAL_OFFSET_SIZE		= 4;
const ULONG TAG_TXT_LIST_FONT_COLOUR_SIZE			= 4;
const ULONG TAG_TXT_LIST_FONT_COLOUR_NONE_SIZE		= 0;
const ULONG TAG_TXT_LIST_FONT_COLOUR_BLACK_SIZE		= 0;
const ULONG TAG_TXT_LIST_FONT_COLOUR_WHITE_SIZE		= 0;

const ULONG TAG_LIST_STYLE_SUPPLEMENT_SIZE			= (ULONG)CXF_UNKNOWN_SIZE;

const ULONG TAG_SPREAD_DESIGN_INFO_SIZE				= sizeof(LONG) * 4;
const ULONG TAG_SPREADINFORMATION_EXTRA_SIZE		= (ULONG)CXF_UNKNOWN_SIZE;


const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_SIZE		= 4;	// 1x Colour reference
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_NONE_SIZE	= 0;
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_BLACK_SIZE	= 0;
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_WHITE_SIZE	= 0;

#endif // INC_CXFDEFS
