/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// Header for the controlling class of the v2 file format

#ifndef INC_CXFILE
#define INC_CXFILE

#include "doccoord.h"
#include "list.h"
#include "ccfile.h"
#include "taglists.h"
#include "xarlib.h"
#include "cxfrech.h"

class CCLexFile;
class CXaraFileRecord;
class StripSubTreeRecordHandler;
class CXaraFileMapTagToHandler;

//-----------------------------------------------

class StandardDefaultRecordHandler : public CamelotRecordHandler
{
public:
	StandardDefaultRecordHandler()
	{
		m_pfnRecordHandler = NULL;
		m_pMagic = NULL;
	};

	// Pure virtual functions that must be overridden.
	virtual ULONG*	GetTagList() { static ULONG TagList[]={CXFRH_TAG_LIST_END}; return (ULONG*)TagList; }
	virtual BOOL	HandleRecord(CXaraFileRecord* pCXaraFileRecord);

	virtual BOOL	IsStreamed(ULONG Tag) { return FALSE; }

	void SetExternalHandler(void* pMagic, RecordHandler pfnRecordHandler)
	{
		m_pfnRecordHandler = pfnRecordHandler;
		m_pMagic = pMagic;
	}

protected:
	RecordHandler* m_pfnRecordHandler;
	void* m_pMagic;
};


/********************************************************************************************

>	class CXaraFile : public CCObject

	Author:		Markn
	Created:	20/5/96
	Base Class:	-
	Purpose:	Encapsulates the v2 file format functionality.
				It provides a standard way for reading and writing v2 format files.

				It uses a general file handling class (in this case CCFile) to do all the reading
				and writing to the file.  This class provides a record-based interface,
				which also allows this class (and asscociated class) to provide extra error handling
				functionality.

	SeeAlso:	CCFile

********************************************************************************************/


class CXaraFile
{
public:
	CXaraFile();
	~CXaraFile();

	BOOL OpenToWrite(CCLexFile* pCCFile);
	BOOL OpenToRead(CCLexFile* pCCFile);
	BOOL Close();

	// Function to turn Compression on or off on the underlying CCFile
	// Reading
	BOOL SetCompression(BOOL NewState);

	// Writing
	BOOL StartCompression();
	BOOL StopCompression();
	BOOL IsCompressionOn();

	// This allows people access to the CCFile which is being used
	// Note: only people who have a genuine reason need to access this - e.g. bitmap savers
	CCLexFile* GetCCFile(void) const;

	// This allows the record size to be got at when reading
	ULONG GetCurrentRecordSize();

//	LONG StartRecord(ULONG Tag,LONG Size);
//	BOOL EndRecord(void);
	LONG StartStreamedRecord(ULONG Tag,LONG Size);
	BOOL EndStreamedRecord(ULONG *RecordSize);

	// These are used during writing???
//	ULONG GetRecordTag();
	LONG  GetRecordNum() { return RecordNumber; }

	void SetExternalRecordHandler(void* pMagic, RecordHandler* pfnRecordHandler);

#if FALSE
	// Write functions

	BOOL Write(LONG n);
	BOOL Write(const DocCoord& Coord);
	BOOL Write(FLOAT f);
	BOOL Write(double d);

	BOOL WriteWCHAR(WCHAR w);

	BOOL WriteCCPanose(const CCPanose& Panose);

//	BOOL WriteBitmapSource(const BitmapSource& Source, UINT Height, BaseCamelotFilter* pThisFilter);

	BOOL Write(TCHAR* pStr);				// Calls WriteUnicode() (Use WriteASCII() for ASCII writing)
	BOOL WriteASCII(TCHAR* pStr);
	BOOL WriteUnicode(TCHAR* pStr);
#endif
	// Writes a complete record out to file
	// DO NOT use this for records which will be referenced
	ULONG Write(CXaraFileRecord* pRecord);

	
protected:	
	BOOL Write(ULONG n);
	BOOL Write(BYTE b);
	BOOL Write(BYTE* pBuf,ULONG Size);

	BOOL Read(ULONG* pULONG);
	BOOL Read(BYTE* pBYTE);
	BOOL Read(BYTE* pBuf,ULONG Size);


public:
#if FALSE
	// Writes a complete record out to file
	// Use this for records which will be referenced
	ULONG WriteDefinitionRecord(CXaraFileRecord* pRecord);

	// Read functions
	BOOL Read(FLOAT* pf);
	BOOL Read(double* pd);

	BOOL ReadWCHAR(WCHAR *pw);

	BOOL ReadCCPanose(CCPanose *pPanose);

	BOOL Read(TCHAR* pStr,ULONG MaxChars);
	BOOL ReadUnicode(TCHAR* pStr,ULONG MaxChars);
#endif

	BOOL ReadNextRecord(BOOL bDeleteRecordAfter = TRUE);
	BOOL ReadNextRecordHeader();

	void SetEndOfFile() { EndOfFile = TRUE; }
	BOOL IsEndOfFile() { return(EndOfFile); }

	ULONG	GetNumBytesWritten()			{ return NumBytesWritten; }
	ULONG	GetNumBytesRead()				{ return NumBytesRead; }
	ULONG	GetTotalNumBytesToRead()		{ return TotalNumBytesToRead; }
	void	SetTotalNumBytesToRead(ULONG n)	{ TotalNumBytesToRead = n; }

	FilePos GetFilePos();

	BOOL SetUpHandlers(void);

	void SetDefaultRecordHandler(CXaraFileRecordHandler* pHandler);
	CXaraFileRecordHandler* GetDefaultRecordHandler();

	void SetStripSubTreeRecordHandler(StripSubTreeRecordHandler* pHandler);
	StripSubTreeRecordHandler* GetStripSubTreeRecordHandler();

	// This function will ensure that the next sub tree is thrown away.
	void StripNextSubTree();

	// Error function
	void GotError(UINT errorID);
	void GotError( UINT errorID , const TCHAR* errorString);

	// Record Handler registration
	BOOL RegisterRecordHandlers(List* pListOfHandlers);

	// Camelot-specific stuff
//	void				SetFilter(BaseCamelotFilter* pThisFilter)	{ pFilter = pThisFilter; }
//	BaseCamelotFilter*	GetFilter()									{ return pFilter; }
	ULONG				GetLastReadTag()							{ return ReadTag; }

	// Tag management functions
	void AddAtomicTag(AtomicTagListItem* pItem);
	void AddEssentialTag(EssentialTagListItem* pItem);

	BOOL IsTagInAtomicList(UINT Tag);
	BOOL IsTagInEssentialList(UINT Tag);

	BOOL WriteRemainingAtomicTagDefinitions();

protected:

	CXaraFileRecordHandler* FindHandler(ULONG Tag);
	CXaraFileRecordHandler* SearchForHandler(ULONG Tag);
	CXaraFileRecordHandler* LookUpHandler(ULONG Tag);

	void SetUpHandlerMap();
	void ResetHandlers();

protected:
	// Should only be used by people who know what they are doing e.g. compression
	// Hence why it is protected
	ULONG WriteRecordHeader(ULONG Tag, LONG Size);

	BOOL FixStreamedRecordHeader(ULONG *RecordSize);

	void IncNumBytesWritten(ULONG n){ NumBytesWritten += n; }
	void IncNumBytesRead(ULONG n)	{ NumBytesRead	  += n; }

protected:
	CXaraFileRecord*	pRecord;
	BOOL				WriteToRecord;

// This rampant private changed by Gerry on 14/7/97
//private:
protected:
	CCLexFile*			pCCFile;
	LONG				RecordNumber;
	ULONG				NumBytesWritten;
	ULONG				NumBytesRead;
	ULONG				TotalNumBytesToRead;
	ULONG				ProgressID;

	// Header data for the record currently being read in
	ULONG				ReadTag;
	ULONG				ReadSize;

	// Record Handler list
	List* pRecordHandlerList;

	// The default record handler
	CXaraFileRecordHandler* pDefaultRecordHandler;
	StandardDefaultRecordHandler* pStandardDefaultRecordHandler;

	// Handler used to strip sub trees when an unrecognised atomic record is read
	StripSubTreeRecordHandler* pStripSubTreeRecordHandler;
	StripSubTreeRecordHandler* pStandardStripSubTreeRecordHandler;

	// Streamed Record vars
	FilePos	StartOfStreamedRecord;
	BOOL	WritingStreamedRecord;
	BOOL	CompOffDueToStreamedRecord;
	BOOL	EndOfFile;

	// Camelot-specific stuff
//	BaseCamelotFilter*			pFilter;
	CXaraFileMapTagToHandler*	pMap;

	// Tag management variables
	AtomicTagList*		pAtomicTagList;
	EssentialTagList*	pEssentialTagList;
};


#if FALSE
/********************************************************************************************

>	class NULLXaraFile : public CXaraFile

	Author:		Markn
	Created:	3/6/96
	Base Class:	CXaraFile
	Purpose:	Dummy CXaraFile that swallows up all bytes written to it

	SeeAlso:	CXaraFile

********************************************************************************************/

class NULLXaraFile : public CXaraFile
{
public:
	NULLXaraFile() { /* empty */ }

	virtual BOOL OpenToWrite(CCLexFile* pThisCCFile);
	virtual BOOL Close();
	virtual BOOL Write(BYTE b);

	virtual FilePos GetFilePos();

	virtual BOOL IsCompressionOn();
	virtual BOOL StartCompression();
	virtual BOOL StopCompression();

	// This allows people access to the CCFile which is being used
	// Note: only people who have a genuine reason need to access this - e.g. bitmap savers
	virtual CCLexFile* GetCCFile() const;

protected:
	virtual BOOL FixStreamedRecordHeader(ULONG *RecordSize);
};
#endif

#endif	// INC_CXFILE
