/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// Defines a class that handles the fast look-up of tags when importing v2 files.

#include "stdafx.h"
#include "cxfrech.h"
#include "cxfmap.h"

// This will get Camelot to display the filename and linenumber of any memory allocations
// that are not released at program exit
#define new CAM_DEBUG_NEW

/********************************************************************************************

>	CXaraFileMapTagToHandler::CXaraFileMapTagToHandler()

	Author:		Markn
	Created:	28/8/96
	Purpose:	The default constructor
	SeeAlso:	-

********************************************************************************************/

CXaraFileMapTagToHandler::CXaraFileMapTagToHandler()
{
	pMap = NULL;
}

/********************************************************************************************

>	CXaraFileMapTagToHandler::~CXaraFileMapTagToHandler()

	Author:		Markn
	Created:	28/8/96
	Purpose:	The destructor
	SeeAlso:	-

********************************************************************************************/

CXaraFileMapTagToHandler::~CXaraFileMapTagToHandler()
{
	if (pMap != NULL)
		delete pMap;
}


/********************************************************************************************

>	void CXaraFileMapTagToHandler::Add(ULONG Tag,CXaraFileRecordHandler* pHandler)

	Author:		Markn
	Created:	28/8/96
	Imputs:		Tag = the tag that the given handler handles
				pHandler = ptr to a record handler
	Returns:	-
	Purpose:	Adds a map entry, associating the tag with the handler
	SeeAlso:	-

********************************************************************************************/

void CXaraFileMapTagToHandler::Add(ULONG Tag,CXaraFileRecordHandler* pHandler)
{
	if (pHandler == NULL)
		return;

	if (pMap == NULL)
		pMap = new CMapPtrToPtr;

	if (pMap != NULL)
		pMap->SetAt((void*)Tag,(void*)pHandler);
}

/********************************************************************************************

>	CXaraFileRecordHandler* CXaraFileMapTagToHandler::LookUp(ULONG Tag)

	Author:		Markn
	Created:	28/8/96
	Imputs:		Tag = the tag
	Returns:	ptr to the handler for the given tag, or NULL if one can't be found
	Purpose:	Returns the handler associated with the given tag
	SeeAlso:	-

********************************************************************************************/

CXaraFileRecordHandler* CXaraFileMapTagToHandler::LookUp(ULONG Tag)
{
	CXaraFileRecordHandler* pHandler = NULL;

	if (pMap != NULL)
	{
		if (pMap->Lookup((void*)Tag,(void*&)pHandler) == 0)
			pHandler = NULL;
	}

	return pHandler;
}
