/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// Header for the controlling class of the v2 file format

#ifndef INC_CXFREC
#define INC_CXFREC

#include "doccoord.h"
#include "cxfdefs.h"

#undef FIXED16
#undef FIXED24
#undef ANGLE


class DocCoord;
class XaraMatrix;
class Path;
class BaseCamelotFilter;
class CCPanose;
class CCLexFile;
class StringBase;
class _bstr_t;
class fixed16;
class fixed24;
typedef fixed16 FIXED16;
typedef fixed24 FIXED24;
typedef fixed16 ANGLE;

class FTTypeList;

//---------------------------------------------------------

#define CXF_RECORD_CHUNK 256

/********************************************************************************************

>	class CXaraFileRecord

	Author:		Markn
	Created:	22/5/96
	Purpose:	This is the class that encapsulates a record in the new v2 format.

********************************************************************************************/

class CXaraFileRecord
{
public:
	CXaraFileRecord(ULONG Tag,LONG Size = -1);
	CXaraFileRecord(CXaraFileRecord& other);
	virtual ~CXaraFileRecord();

	void DumpTypes(void);
	void WriteAsText(CCLexFile* pFile);

	virtual	CXaraFileRecord* GetCopy();		// Returns a ptr to an exact copy

	virtual	BOOL Init(BOOL ZeroMemBlock = FALSE);
	virtual	BOOL Reinit(ULONG ThisTag,LONG Size);

	virtual BOOL WriteBOOL(BOOL b);
	virtual	BOOL WriteBYTE(BYTE b);
	virtual	BOOL WriteULONG(ULONG n);
	virtual	BOOL WriteLONG(LONG n);
	virtual BOOL WriteReference(LONG n);
	virtual	BOOL WriteUINT16(UINT16 n);
	virtual	BOOL WriteINT16(INT16 n);
	virtual	BOOL WriteBuffer(BYTE* pBuf,ULONG BufSize);
	virtual	BOOL WriteFLOAT(FLOAT f);
	virtual	BOOL WriteDOUBLE(double d);
	virtual	BOOL WriteFIXED16(FIXED16 n);
	virtual	BOOL WriteFIXED24(FIXED24 n);
	virtual BOOL WriteANGLE(ANGLE a);
	virtual BOOL WriteWCHAR(WCHAR w);
	virtual BOOL WriteWCHARs(WCHAR* w, ULONG Count);

	virtual	BOOL WriteCoord(const DocCoord& Coord);
	virtual	BOOL WriteCoordTrans(const DocCoord& Coord,LONG dx,LONG dy);
	virtual	BOOL WriteCoordInterleaved(const DocCoord& Coord);
	virtual	BOOL WriteCoordTransInterleaved(const DocCoord& Coord,LONG dx,LONG dy);
//	virtual	BOOL WritePath(Path* pPath);
//	virtual	BOOL WritePathRelative(Path* pPath);
//	virtual	BOOL WritePathTrans(Path* pPath,LONG dx,LONG dy);
//	virtual	BOOL WritePathRelativeTrans(Path* pPath,LONG dx,LONG dy);
	virtual	BOOL WriteXOrd(LONG XOrdinate);
	virtual	BOOL WriteYOrd(LONG YOrdinate);
	virtual BOOL WriteCCPanose(const CCPanose &MyCCPanose);

	virtual	BOOL WriteASCII(TCHAR* pStr);
	virtual	BOOL WriteUnicode(TCHAR* pStr);
	virtual	BOOL WriteBSTR(_bstr_t bstr);
	virtual BOOL WriteWideString(const wchar_t* str);

	virtual	BOOL WriteMatrix(const XaraMatrix & m);
	virtual	BOOL WriteMatrixTrans(const XaraMatrix & m,LONG dx,LONG dy);

	virtual	ULONG GetTag()				{ return Tag; }
	virtual	ULONG GetSize();
	virtual	BYTE* GetBuffer()			{ return (BYTE*)pBuffer; }
	virtual ULONG BytesLeft() { return GetSize() - CurrentPos; }

	virtual FTTypeList* GetTypeList(void)	{ return(m_pTypeList); };

	virtual	LONG  GetRecordNumber();
	virtual	void  SetRecordNumber(ULONG n);

	virtual	BOOL IsDataSectionFull()	{ return (!(SizeIsKnown && CurrentPos < KnownSize)); }

	virtual	void ResetReadPos();	// Resets read pos to beginning of the record
	virtual BOOL Skip(ULONG Count); // Skip n bytes from the record

	virtual BOOL ReadBOOL(BOOL* pb);
	virtual	BOOL ReadBYTE(BYTE* pb);
	virtual	BOOL ReadBYTEnoError(BYTE* pb);
	virtual	BOOL ReadULONG(ULONG* pn);
	virtual	BOOL ReadLONG(LONG* pn);
	virtual	BOOL ReadLONGnoError(LONG* pn);
	virtual	BOOL ReadUINT16(UINT16* pn);
	virtual	BOOL ReadINT16(INT16* pn);
	virtual	BOOL ReadBuffer(BYTE* pBuf,ULONG BufSize);
	virtual	BOOL ReadBuffernoError(BYTE* pBuf,ULONG BufSize);
	virtual	BOOL ReadFLOAT(FLOAT* pf);
	virtual	BOOL ReadDOUBLE(double* pd);
	virtual	BOOL ReadDOUBLEnoError(double* pd);
	virtual	BOOL ReadFIXED16(FIXED16 *pn);
	virtual	BOOL ReadFIXED24(FIXED24 *pn);
	virtual BOOL ReadANGLE(ANGLE *pa);
	virtual BOOL ReadWCHAR(WCHAR *pw);

	virtual	BOOL ReadCoord(DocCoord* pCoord);
	virtual	BOOL ReadCoordTrans(DocCoord* pCoord,LONG dx,LONG dy);
	virtual	BOOL ReadCoordInterleaved(DocCoord* pCoord);
	virtual	BOOL ReadCoordTransInterleaved(DocCoord* pCoord,LONG dx,LONG dy);
//	virtual	BOOL ReadPath(Path* pPath);
//	virtual	BOOL ReadPathRelative(Path* pPath);
//	virtual	BOOL ReadPathTrans(Path* pPath,LONG dx,LONG dy);
//	virtual	BOOL ReadPathRelativeTrans(Path* pPath,LONG dx,LONG dy);
	virtual	BOOL ReadXOrd(LONG* pXOrdinate);
	virtual	BOOL ReadYOrd(LONG* pYOrdinate);
	virtual BOOL ReadCCPanose(CCPanose *pPanose);

	virtual	BOOL ReadUnicode(StringBase* pStr);
	virtual	BOOL ReadUnicode(TCHAR* pStr,ULONG MaxChars);
	virtual	BOOL ReadASCII(TCHAR* pStr,ULONG MaxChars);
	virtual	BOOL ReadBSTR(_bstr_t* bstr, ULONG MaxChars);
	virtual BOOL ReadWideString(wchar_t* pStr, ULONG MaxChars);

	virtual	BOOL ReadMatrix(XaraMatrix *pMatrix);
	virtual	BOOL ReadMatrixTrans(XaraMatrix *pMatrix,LONG dx,LONG dy);

private:
	void SetUpVars(ULONG ThisTag,LONG Size);

	void FreeBuffer();
	BOOL AllocBuffer(LONG Size);
	BOOL ExtendBuffer(LONG Size);

	BOOL	Initialised;

	ULONG	Tag;
	ULONG	BufferSize;
	ULONG	KnownSize;
	BOOL	SizeIsKnown;
	LONG	RecordNumber;

	void*	pBuffer;
	ULONG	CurrentPos;
	BOOL	Overflow;

	FTTypeList* m_pTypeList;
};

/********************************************************************************************

>	class CamelotFileRecord : public CXaraFileRecord

	Author:		Markn
	Created:	24/6/96
	Purpose:	This derived class does all the things that are Camelot-specific.

********************************************************************************************/

class CamelotFileRecord : public CXaraFileRecord
{
public:
	CamelotFileRecord(BaseCamelotFilter* pFilter,ULONG Tag,LONG Size = -1);
	~CamelotFileRecord();

	// Translates the coords and the paths automatically on writing
	virtual	BOOL WriteCoord(const DocCoord& Coord);
	virtual	BOOL WriteCoordInterleaved(const DocCoord& Coord);
//	virtual	BOOL WritePath(Path* pPath);
//	virtual	BOOL WritePathRelative(Path* pPath);
	virtual	BOOL WriteMatrix(const XaraMatrix & m);
	virtual	BOOL WriteXOrd(LONG XOrdinate);
	virtual	BOOL WriteYOrd(LONG YOrdinate);

	// Translates the coords and the paths automatically on reading
	virtual	BOOL ReadCoord(DocCoord* pCoord);
	virtual	BOOL ReadCoordInterleaved(DocCoord* pCoord);
//	virtual	BOOL ReadPath(Path* pPath);
//	virtual	BOOL ReadPathRelative(Path* pPath);
	virtual	BOOL ReadMatrix(XaraMatrix *pMatrix);
	virtual	BOOL ReadXOrd(LONG* pXOrdinate);
	virtual	BOOL ReadYOrd(LONG* pYOrdinate);

private:
	DocCoord CoordOrigin;
};

#endif	// INC_CXFREC
