/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/kernel/cxfrech.cpp 5     31/03/04 13:39 Phil $
// Implementation of the record handler class of the v2 file format

#include "stdafx.h"
#include "cxfrech.h"
//#include "doccolor.h"
//#include "colcomp.h"
//#include "bmpcomp.h"
//#include "unitcomp.h"
//#include "infocomp.h"
//#include "camfiltr.h"
#include "fixmem.h"

#include "cxfile.h"
#include "cxfrec.h"
#include "cxftags.h"


// This will get Camelot to display the filename and linenumber of any memory allocations
// that are not released at program exit
#define new CAM_DEBUG_NEW

//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------


/********************************************************************************************

>	CXaraFileRecordHandler::CXaraFileRecordHandler()

	Author:		Markn
	Created:	28/5/96
	Inputs:		-
	Purpose:	Creates a record handler object.
				These are used by CXaraFile for reading the file.
				Handlers handle records as they are read from a file
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

CXaraFileRecordHandler::CXaraFileRecordHandler()
{
}

/********************************************************************************************

>	CXaraFileRecordHandler::~CXaraFileRecordHandler()

	Author:		Markn
	Created:	28/5/96
	Purpose:	Default destructor
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

CXaraFileRecordHandler::~CXaraFileRecordHandler()
{
}


/********************************************************************************************

>	BOOL CXaraFileRecordHandler::IsTagInList(ULONG Tag)

	Author:		Markn
	Created:	28/5/96
	Inputs:		Tag	= a tag value
	Returns:	TRUE if tag is in handler's tag list
				FALSE otherwise
	Purpose:	Helper function that searchs the handler's tag list for a given tag
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::IsTagInList(ULONG Tag)
{
	const ULONG* pTagList = GetTagList();

	ERROR2IF(pTagList == NULL,FALSE,"NULL pTagList - is this a default handler?");
	
	while (*pTagList != CXFRH_TAG_LIST_END)
	{
		if (*pTagList == Tag)
			return TRUE;

		pTagList++;
	}

	return FALSE;
}

/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::HandleStreamedRecord(CXaraFile * pCXFile, ULONG Tag,ULONG Size,ULONG RecordNumber)

	Author:		Neville Humphrys
	Created:	7/6/96
	Inputs:		pCXFile = access to the CXaraFile class
				Tag	 = the tag value
				Size = size of record
				RecordNumber = the record number in the file
	Returns:	TRUE if handled ok
				FALSE otherwise
	Purpose:	This is the baseclass streamed record handler. It should always be overriden.
				We don't want to make it pure virtual and force all handlers to define it
				as most do not require it.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::HandleStreamedRecord(CXaraFile * pCXFile, ULONG Tag,ULONG Size,ULONG RecordNumber)
{
	ERROR2(FALSE,"CXaraFileRecordHandler::HandleStreamedRecord Baseclass handler called")
	return FALSE;
}


/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::Init(BaseCamelotFilter* pThisBaseCamelotFilter)

	Author:		Markn
	Created:	28/5/96
	Inputs:		pThisBaseCamelotFilter = ptr to the filter that will read the file.
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	Initialises the handler.
	
				Base class does nowt
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::Init(CXaraFile* pCXFile)
{
	return TRUE;
}

/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::BeginImport()

	Author:		Markn
	Created:	8/8/96
	Inputs:		pThisBaseCamelotFilter = ptr to the filter that will read the file.
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	Informs the record handler that importing has begun.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::BeginImport()
{
	return TRUE;
}

/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::EndImport()

	Author:		Markn
	Created:	31/7/96
	Inputs:		-
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	Informs the record handler that importing is over

				If you override this func, be sure to call the base class varient
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::EndImport()
{
	// Just return TRUE in the base class
	return TRUE;
}

/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::BeginSubtree(UINT Tag)

	Author:		Markn
	Created:	2/9/96
	Inputs:		Tag = tag value of the tag this handler last handled
	Returns:	TRUE if this func wants exclusive handling of the tag
				FALSE otherwise
	Purpose:	Informs the record handler that a subtree is following a tag of type 'Tag'

				If you override this func and you do not wish other parts of the system to be informed of the
				subtree start, you should return TRUE

				The base class always returns FALSE
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::BeginSubtree(UINT Tag)
{
	return FALSE;	// FALSE means pass onto other parts of the system
}

/********************************************************************************************

>	virtual BOOL CXaraFileRecordHandler::EndSubtree(UINT Tag)

	Author:		Markn
	Created:	2/9/96
	Inputs:		Tag = tag value of the tag this handler last handled
	Returns:	TRUE if this func wants exclusive handling of the tag
				FALSE otherwise
	Purpose:	Informs the record handler that a subtree that followed a tag of type 'Tag' has ended

				If you override this func and you do not wish other parts of the system to be informed of the
				subtree start, you should return TRUE

				The base class always returns FALSE
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CXaraFileRecordHandler::EndSubtree(UINT Tag)
{
	return FALSE;	// FALSE means pass onto other parts of the system
}




//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

/********************************************************************************************

>	CamelotRecordHandler::CamelotRecordHandler()

	Author:		Markn
	Created:	28/5/96
	Inputs:		-
	Purpose:	Creates a camelot record handler object.
				These are used by CXaraFile for reading the file.
				Handlers handle records as they are read from a file.

				This is a special varient that contains Camelot-specific functionality
		
	Errors:		-
	SeeAlso:	Init()

********************************************************************************************/

CamelotRecordHandler::CamelotRecordHandler()
{
//	pBaseCamelotFilter = NULL;
	m_pCXFile = NULL;
}

/********************************************************************************************

>	CamelotRecordHandler::~CamelotRecordHandler()

	Author:		Markn
	Created:	28/5/96
	Purpose:	Default destructor
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

CamelotRecordHandler::~CamelotRecordHandler()
{
}

/********************************************************************************************

>	virtual BOOL CamelotRecordHandler::Init(BaseCamelotFilter* pThisBaseCamelotFilter)

	Author:		Markn
	Created:	28/5/96
	Inputs:		pThisBaseCamelotFilter = ptr to the filter that will read the file.
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	Initialises the handler.
	
				The base class forms a link between the handler and the filter class

				If you override this func, be sure to call the base class varient
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CamelotRecordHandler::Init(CXaraFile* pCXFile)
{
	ERROR2IF(pCXFile == NULL,FALSE,"pCXFile is NULL");

	m_pCXFile = pCXFile;

	return TRUE;
}

/********************************************************************************************

>	virtual BOOL CamelotRecordHandler::IsStreamed(ULONG Tag)

	Author:		Neville Humphrys
	Created:	7/6/96
	Inputs:		The tag of the record
	Returns:	TRUE if this is a streamed record
				FALSE otherwise
	Purpose:	Function to find out if the record is streamed or not.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CamelotRecordHandler::IsStreamed(ULONG Tag)
{
	// Baseclass returns False, i.e. not streamed
	return FALSE;
}

/********************************************************************************************

>	BOOL CamelotRecordHandler::SetCompression(BOOL NewState)

	Author:		Neville Humphrys
	Created:	6/6/96
	Inputs:		The new compression state required, True for on, False for off
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	Function to turn Compression on or off on the underlying CCFile.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CamelotRecordHandler::SetCompression(BOOL NewState)
{
	return m_pCXFile->SetCompression(NewState);
}

#if FALSE
/********************************************************************************************

>	virtual BOOL CamelotRecordHandler::GetDocColour(LONG ColourRef,DocColour* pDocColour)

	Author:		Markn
	Created:	31/5/96
	Inputs:		ColourRef  = a reference to a colour defined earlier in the file
				pDocColour = ptr to a DocColour object
	Returns:	TRUE if ok
				FALSE otherwise
	Purpose:	This dereferences a colour ref value, and creates a DocColour from it.
				The resultant DocColour is put into pDocColour.

				The ColourRef value is typically found in records that define a colour or colours.
				It references a colour defined earlier in the file.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CamelotRecordHandler::GetDocColour(LONG ColourRef,DocColour* pDocColour)
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	ERROR2IF(pDocColour == NULL,FALSE,"pDocColour param is NULL");

	ColourListComponent* pColComp = pBaseCamelotFilter->GetColourDocComponent();

	ERROR2IF(pColComp == NULL,FALSE,"No colour component when getting a colour");

	return (pColComp->GetDocColour(ColourRef,pDocColour));
}

/********************************************************************************************

>	KernelBitmap* CamelotRecordHandler::GetReadBitmapReference(LONG BitmapRef)

	Author:		Neville Humphrys
	Created:	12/6/96
	Inputs:		BitmapRef	= a reference to a colour defined earlier in the file
	Outputs:	-
	Returns:	pBitmap		= ptr to a KernelBitmap object
	Purpose:	This dereferences a bitmap ref value, and returns a pointer to a KernelBitmap
				from it.
				The resultant KernelBitmap can then be used by the caller e.g. in fills.

				The BitmapRef value is typically found in records that define a bitmap.
				It references a bitmap defined earlier in the file.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

KernelBitmap* CamelotRecordHandler::GetReadBitmapReference(LONG BitmapRef)
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");

	BitmapListComponent* pBmpComp = pBaseCamelotFilter->GetBitmapDocComponent();

	ERROR2IF(pBmpComp == NULL,FALSE,"No bitmap component when getting a bitmap");

	return (pBmpComp->GetReadBitmapReference(BitmapRef));
}

//------------------------------------------------------------------------------------------
// The following functions map onto functions provided by BaseCamelotFilter
//
// For more details, see the headers of the BaseCamelotFilter func equivalents

ColourListComponent* CamelotRecordHandler::GetColourDocComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetColourDocComponent());
}

BitmapListComponent* CamelotRecordHandler::GetBitmapDocComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetBitmapDocComponent());
}

UnitListComponent* CamelotRecordHandler::GetUnitDocComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetUnitDocComponent());
}

DocInfoComponent* CamelotRecordHandler::GetDocInfoComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetDocInfoComponent());
}

PrintComponent* CamelotRecordHandler::GetPrintComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetPrintComponent());
}

FontComponent* CamelotRecordHandler::GetFontComponent()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->GetFontComponent());
}

BOOL CamelotRecordHandler::IsImportingAtPosition()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->IsImportingAtPosition());
}

BOOL CamelotRecordHandler::IsImporting()
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->IsImporting());
}

BOOL CamelotRecordHandler::InsertNode(Node* pNode)
{
	ERROR2IF(pBaseCamelotFilter == NULL,FALSE,"pBaseCamelotFilter is NULL");
	return (pBaseCamelotFilter->InsertNode(pNode));
}

void CamelotRecordHandler::EndOfFile()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->EndOfFile();
}

void CamelotRecordHandler::SetTotalNumBytesToRead(ULONG s)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetTotalNumBytesToRead(s);
}

void CamelotRecordHandler::IncProgressBarCount(ULONG n)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncProgressBarCount(n);
}

void CamelotRecordHandler::SetTotalProgressBarCount(ULONG n)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetTotalProgressBarCount(n);
}

BOOL CamelotRecordHandler::IncInsertLevel()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->IncInsertLevel();

	return FALSE;
}

BOOL CamelotRecordHandler::DecInsertLevel()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->DecInsertLevel();

	return FALSE;
}

Node* CamelotRecordHandler::GetInsertContextNode()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetInsertContextNode();

	return NULL;
}

void CamelotRecordHandler::SetInsertContextNode(Node* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetInsertContextNode(pNode);
}

void CamelotRecordHandler::InsertNextNodeAsChild()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->InsertNextNodeAsChild();
}

void CamelotRecordHandler::InsertNextNodeAsSibling()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->InsertNextNodeAsSibling();
}


InsertTreeContext *CamelotRecordHandler::GetInsertContext(void)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return(pBaseCamelotFilter->GetInsertContext());

	return(NULL);
}

void CamelotRecordHandler::RestoreInsertContext(InsertTreeContext *pOldState)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->RestoreInsertContext(pOldState);
}


Document* CamelotRecordHandler::GetDocument()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetDocument();

	return NULL;
}

Spread* CamelotRecordHandler::GetSpread()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetSpread();

	return NULL;
}

Document* CamelotRecordHandler::GetLastDocumentInserted()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLastDocumentInserted();

	return NULL;
}

Chapter* CamelotRecordHandler::GetLastChapterInserted()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLastChapterInserted();

	return NULL;
}

Spread* CamelotRecordHandler::GetLastSpreadInserted()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLastSpreadInserted();

	return NULL;
}

Layer* CamelotRecordHandler::GetLastLayerInserted()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLastLayerInserted();

	return NULL;
}

NodePath* CamelotRecordHandler::GetLastNodePathInserted()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLastNodePathInserted();

	return NULL;
}

void CamelotRecordHandler::SetLastDocumentInserted(Document* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetLastDocumentInserted(pNode);
}

void CamelotRecordHandler::SetLastChapterInserted(Chapter* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetLastChapterInserted(pNode);
}

void CamelotRecordHandler::SetLastSpreadInserted(Spread* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetLastSpreadInserted(pNode);
}

void CamelotRecordHandler::SetLastLayerInserted(Layer* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetLastLayerInserted(pNode);
}

void CamelotRecordHandler::SetLastNodePathInserted(NodePath* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetLastNodePathInserted(pNode);
}

Layer * CamelotRecordHandler::GetFirstImportedLayer()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetFirstImportedLayer();
	return NULL;
}

void CamelotRecordHandler::SetFirstImportedLayer(Layer* pNode)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetFirstImportedLayer(pNode);
}

UINT CamelotRecordHandler::GetDocumentInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetDocumentInsertedCount();

	return 0;
}

UINT CamelotRecordHandler::GetChapterInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetChapterInsertedCount();

	return 0;
}

UINT CamelotRecordHandler::GetSpreadInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetSpreadInsertedCount();

	return 0;
}

UINT CamelotRecordHandler::GetLayerInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetLayerInsertedCount();

	return 0;
}

UINT CamelotRecordHandler::GetSetSentinelInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetSetSentinelInsertedCount();

	return 0;
}


void CamelotRecordHandler::IncDocumentInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncDocumentInsertedCount();
}

void CamelotRecordHandler::IncChapterInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncChapterInsertedCount();
}

void CamelotRecordHandler::IncSpreadInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncSpreadInsertedCount();
}

void CamelotRecordHandler::IncLayerInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncLayerInsertedCount();
}

void CamelotRecordHandler::IncSetSentinelInsertedCount()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->IncSetSentinelInsertedCount();
}

void CamelotRecordHandler::SetCoordOrigin(const DocCoord& Origin)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetCoordOrigin(Origin);
}

DocCoord CamelotRecordHandler::GetCoordOrigin()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetCoordOrigin();

	return DocCoord(0,0);
}
#endif

void CamelotRecordHandler::AddAtomicTag(AtomicTagListItem* pItem)
{
	if (m_pCXFile != NULL)
		m_pCXFile->AddAtomicTag(pItem);
}

void CamelotRecordHandler::AddEssentialTag(EssentialTagListItem* pItem)
{
	if (m_pCXFile != NULL)
		m_pCXFile->AddEssentialTag(pItem);
}

#if FALSE
BOOL CamelotRecordHandler::AddTagDescription(TagDescriptionListItem* pItem)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->AddTagDescription(pItem);

	return FALSE;
}

BOOL CamelotRecordHandler::SetDocumentNudgeSize(UINT newVal)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->SetDocumentNudgeSize(newVal);

	return FALSE;
}
#endif

BOOL CamelotRecordHandler::IsTagInAtomicList(UINT Tag)
{
	if (m_pCXFile != NULL)
		return m_pCXFile->IsTagInAtomicList(Tag);

	return FALSE;
}

BOOL CamelotRecordHandler::IsTagInEssentialList(UINT Tag)
{
	if (m_pCXFile != NULL)
		return m_pCXFile->IsTagInEssentialList(Tag);

	return FALSE;
}


#if FALSE
TagDescriptionListItem* CamelotRecordHandler::GetTagDescription(UINT Tag)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->GetTagDescription(Tag);

	return NULL;
}

BOOL CamelotRecordHandler::SetPreCompression(ULONG Flags)
{
	if (m_pCXFile != NULL)
		return m_pCXFile->SetPreCompression(Flags);

	return FALSE;
}

ULONG CamelotRecordHandler::GetPreCompression()
{
	if (m_pCXFile != NULL)
		return m_pCXFile->GetPreCompression();

	return 0;
}

BOOL CamelotRecordHandler::UnrecognisedTag(UINT Tag)
{
	if (m_pCXFile != NULL)
		return m_pCXFile->UnrecognisedTag(Tag);

	return FALSE;
}

void CamelotRecordHandler::AddPathRecordRefToList(NodePath* pNodePath, UINT RecordNumber)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->AddPathRecordRefToList(pNodePath,RecordNumber);
}

UINT CamelotRecordHandler::FindPathRecordRefRecordNumber(NodePath* pNodePath)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->FindPathRecordRefRecordNumber(pNodePath);

	return 0;
}

NodePath* CamelotRecordHandler::FindPathRecordRefPath(UINT RecordNumber)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->FindPathRecordRefPath(RecordNumber);

	return NULL;
}

void CamelotRecordHandler::SetImportFileType(char* pFileType)
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		pBaseCamelotFilter->SetImportFileType(pFileType);
}

BOOL CamelotRecordHandler::IsOpeningMinimalWebFormat()
{
	ERROR3IF(pBaseCamelotFilter == NULL,"pBaseCamelotFilter is NULL");
	if (pBaseCamelotFilter != NULL)
		return pBaseCamelotFilter->IsOpeningMinimalWebFormat();

	return FALSE;
}
#endif


//-----------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------


/********************************************************************************************

>	StripSubTreeRecordHandler::StripSubTreeRecordHandler()

	Author:		Markn
	Created:	19/8/96
	Inputs:		-
	Returns:	-
	Purpose:	Default constructor
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

StripSubTreeRecordHandler::StripSubTreeRecordHandler()
{
	StripSubTreeOff();
	LevelCounter = 0;
}


/********************************************************************************************

>	virtual BOOL StripSubTreeRecordHandler::BeginImport()

	Author:		Markn
	Created:	19/8/96
	Inputs:		-
	Returns:	TRUE if OK, FALSE otherwise
	Purpose:	Informs the record handler that importing is about to begin
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL StripSubTreeRecordHandler::BeginImport()
{
	StripSubTreeOff();
	LevelCounter = 0;

	return TRUE;
}

/********************************************************************************************

>	virtual ULONG* StripSubTreeRecordHandler::GetTagList()

 	Author:		Markn
	Created:	19/8/96
	Inputs:		-
	Returns:	Ptr to a list of tag values, terminated by CXFRH_TAG_LIST_END
	Purpose:	Provides the record handler system with a list of records handled by this
				handler
	SeeAlso:	-

********************************************************************************************/

ULONG* StripSubTreeRecordHandler::GetTagList()
{
	static ULONG TagList[] = {CXFRH_TAG_LIST_END};

	return (ULONG*)&TagList;
}

/********************************************************************************************

>	virtual BOOL StripSubTreeRecordHandler::HandleRecord(CXaraFileRecord* pCXaraFileRecord)

 	Author:		Markn
	Created:	19/8/96
	Inputs:		pCXaraFileRecord = ptr to the record to handle
	Returns:	TRUE if handled ok, FALSE otherwise
	Purpose:	The main record handling function
	SeeAlso:	-

********************************************************************************************/

BOOL StripSubTreeRecordHandler::HandleRecord(CXaraFileRecord* pCXaraFileRecord)
{
	ERROR3IF(!IsStripSubTreeOn(),"This should never be called if it's not stripping a subtree");
	if (!IsStripSubTreeOn())
		return TRUE;

	ERROR2IF(pCXaraFileRecord == NULL,FALSE,"pCXaraFileRecord is NULL");

	switch (pCXaraFileRecord->GetTag())
	{
		case TAG_UP:
			if (LevelCounter > 0)
				LevelCounter--;

			if (LevelCounter == 0)
				StripSubTreeOff();
			break;

		case TAG_DOWN:
			LevelCounter++;
			break;
	}

	return TRUE;
}


/********************************************************************************************

>	virtual BOOL StripSubTreeRecordHandler::IsTagInList(ULONG Tag)

 	Author:		Markn
	Created:	19/8/96
	Inputs:		Tag = tag value of a record
	Returns:	TRUE if it's to be handled by this handler
				FALSE otherwise
	Purpose:	This decides whether the given Tag should be handled by the sub tree stripper or not.
	SeeAlso:	-

********************************************************************************************/

BOOL StripSubTreeRecordHandler::IsTagInList(ULONG Tag)
{
	// If the stripper is not turned on, there's no point in trying
	if (!IsStripSubTreeOn())
		return FALSE;

	// If the next tag is not a down node (i.e. it's not the start of a sub-tree)
	// and the level counter is 0 (i.e. we're not in the depths of a sub-tree either)
	// then we should return FALSE & turn the stripper off
	if (Tag != TAG_DOWN && LevelCounter == 0)
	{
		StripSubTreeOff();
		return FALSE;
	}

	return TRUE;
}

