/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /Camelot/kernel/cxfrech.h 3     23/03/04 18:08 Phil $
// Header for the record handler class of the v2 file format

#ifndef INC_CXFRECH
#define INC_CXFRECH

#include "listitem.h"
#include "doccoord.h"

#define CXFRH_TAG_LIST_END	(0xffffffff)

class CXaraFileRecord;
class BaseCamelotFilter;
class DocColour;
class KernelBitmap;
class Node;
class ColourListComponent;
class BitmapListComponent;
class UnitListComponent;
class DocInfoComponent;
class PrintComponent;
class FontComponent;
class CXaraFile;

class Document;
class Chapter;
class Spread;
class Layer;
class NodePath;

class AtomicTagListItem;
class EssentialTagListItem;
class TagDescriptionListItem;

/********************************************************************************************

>	class CXaraFileRecordHandler : public ListItem

	Author:		Markn
	Created:	29/5/96
	Purpose:	This is the class that encapsulates a record handler in the new v2 format.

********************************************************************************************/

class CXaraFileRecordHandler : public ListItem
{
public:
	CXaraFileRecordHandler();
	~CXaraFileRecordHandler();

	// Pure virtual functions that must be overridden.
	virtual ULONG*	GetTagList() = 0;
	virtual BOOL	HandleRecord(CXaraFileRecord* pCXaraFileRecord) = 0;
	virtual BOOL	HandleStreamedRecord(CXaraFile * pCXFile, ULONG Tag,ULONG Size,ULONG RecordNumber);

	// Find out if the record is streamed or not
	virtual BOOL IsStreamed(ULONG Tag) = 0;

//	virtual void IncProgressBarCount(ULONG n) = 0;

	virtual BOOL IsTagInList(ULONG Tag);

	virtual BOOL Init(CXaraFile* pCXFile);
	virtual BOOL BeginImport();
	virtual BOOL EndImport();
	virtual BOOL BeginSubtree(UINT Tag);
	virtual BOOL EndSubtree(UINT Tag);
};


/********************************************************************************************

>	class CamelotRecordHandler : public CXaraFileRecordHandler

	Author:		Markn
	Created:	29/5/96
	Purpose:	This is the class that encapsulates a camelot-specific record handler in the new v2 format.

				It's main purpose is to make it easy for record handlers to access BaseCamelotFilter
				functions by providing mirror functions into the filter.

				It also hides the fact that there is an underlying filter.

********************************************************************************************/

class CamelotRecordHandler : public CXaraFileRecordHandler
{
public:
	CamelotRecordHandler();
	~CamelotRecordHandler();

	virtual BOOL IsCamelotHandler() { return TRUE; }

//	virtual BOOL Init(BaseCamelotFilter* pBaseCamelotFilter);
	virtual BOOL Init(CXaraFile* pCXFile);
#if FALSE

	// Funcs for extracting reusable data items
	BOOL GetDocColour(LONG ColourRef,DocColour* pDocColour);
	KernelBitmap* GetReadBitmapReference(LONG BitmapRef);

	// functions that map onto BaseCamelotFilter functions
	ColourListComponent*	GetColourDocComponent();
	BitmapListComponent*	GetBitmapDocComponent();
	UnitListComponent*		GetUnitDocComponent();
	DocInfoComponent*		GetDocInfoComponent();
	PrintComponent*			GetPrintComponent();
	FontComponent*			GetFontComponent();

	BOOL		IsImporting();
	BOOL		IsImportingAtPosition();

	BOOL		InsertNode(Node* pNode);
	BOOL		IncInsertLevel();
	BOOL		DecInsertLevel();
	Node*		GetInsertContextNode();
	void		SetInsertContextNode(Node* pNode);
	void		InsertNextNodeAsChild();
	void		InsertNextNodeAsSibling();

	// These calls are used for building the document structure when load a file
	//-----------
	Document*	GetLastDocumentInserted();
	Chapter*	GetLastChapterInserted();
	Spread*		GetLastSpreadInserted();
	Layer*		GetLastLayerInserted();
	NodePath*	GetLastNodePathInserted();

	void	SetLastDocumentInserted(Document* pNode);
	void	SetLastChapterInserted(Chapter* pNode);
	void	SetLastSpreadInserted(Spread* pNode);
	void	SetLastLayerInserted(Layer* pNode);
	void	SetLastNodePathInserted(NodePath* pNodePath);
	
	Layer * GetFirstImportedLayer();
	void	SetFirstImportedLayer(Layer* pLayer);

	UINT	GetDocumentInsertedCount();
	UINT	GetChapterInsertedCount();
	UINT	GetSpreadInsertedCount();
	UINT	GetLayerInsertedCount();
	UINT	GetSetSentinelInsertedCount();

	void	IncDocumentInsertedCount();
	void	IncChapterInsertedCount();
	void	IncSpreadInsertedCount();
	void	IncLayerInsertedCount();
	void	IncSetSentinelInsertedCount();


	BOOL AddTagDescription(TagDescriptionListItem* pItem);
	BOOL SetDocumentNudgeSize(UINT newVal);


//	TagDescriptionListItem* GetTagDescription(UINT Tag);
#endif

	void AddAtomicTag(AtomicTagListItem* pItem);
	void AddEssentialTag(EssentialTagListItem* pItem);
	BOOL IsTagInAtomicList(UINT Tag);
	BOOL IsTagInEssentialList(UINT Tag);
	BOOL UnrecognisedTag(UINT Tag);

#if FALSE
	// Path record references
//	void		AddPathRecordRefToList(NodePath* pNodePath, UINT RecordNumber);
//	UINT		FindPathRecordRefRecordNumber(NodePath* pNodePath);
//	NodePath*	FindPathRecordRefPath(UINT RecordNumber);

//-----------

	Document*	GetDocument();
	Spread*		GetSpread();

	void		SetCoordOrigin(const DocCoord& Origin);
	DocCoord	GetCoordOrigin();

	void	EndOfFile();
	void	SetTotalNumBytesToRead(ULONG s);
	void	IncProgressBarCount(ULONG n);
	void	SetTotalProgressBarCount(ULONG n);

	BaseCamelotFilter* GetBaseCamelotFilter() { return pBaseCamelotFilter; }
	virtual BOOL  SetPreCompression(ULONG Flags);
	virtual ULONG GetPreCompression();

#endif

	// Function to turn Compression on or off on the underlying CCFile
	virtual BOOL  SetCompression(BOOL NewState);

	// Find out if the record is streamed or not
	virtual BOOL IsStreamed(ULONG Tag);

//	virtual void	SetImportFileType(char* pFileType);
//	virtual	BOOL	IsOpeningMinimalWebFormat();

protected:
//	BaseCamelotFilter* pBaseCamelotFilter;
	CXaraFile* m_pCXFile;
};


/********************************************************************************************

>	class StripSubTreeRecordHandler : public CamelotRecordHandler

	Author:		Markn
	Created:	19/8/96
	Purpose:	This is the class is used to strip a subtree of records from the file.

				It works counting the number of TAG_DOWN & TAG_UP records it gets.

				If the next record it gets is a TAG_DOWN, it increments the tree level counter.
				If the next record it gets is a TAG_UP, it decrements the tree level counter.
				If the next record is neither a TAG_UP or TAG_DOWN, and the level counter > 0, it is thrown away
				If the next record is neither a TAG_UP or TAG_DOWN, and the level counter == 0, it is passed on to it's proper handler

********************************************************************************************/

class StripSubTreeRecordHandler : public CamelotRecordHandler
{
public:
	StripSubTreeRecordHandler();

	virtual ULONG*	GetTagList();
	virtual BOOL	HandleRecord(CXaraFileRecord* pCXaraFileRecord);
	virtual BOOL	IsTagInList(ULONG Tag);
	virtual BOOL	BeginImport();

	virtual void StripSubTreeOn()	{ StripSubTree = TRUE; }
	virtual void StripSubTreeOff()	{ StripSubTree = FALSE; }
	virtual BOOL IsStripSubTreeOn()	{ return StripSubTree; }

private:
	BOOL StripSubTree;
	UINT LevelCounter;
};

#endif	// INC_CXFRECH
