
// This file defines all the allocated tags in the v2 CorelXARA file format

#ifndef INC_CXFTAGS
#define INC_CXFTAGS

	//////////////////////////////////////////////////////////////////////////
	// IMPORTANT															//
	// To keep things tidy & maintainable, tags from previous releases of	//
	// camelot are now FROZEN. You MUST NOT add tags with values in "old"	//
	// ranges: See the bottom of this file for the current tag range to use	//
	//																		//
	// Allocated ranges are:												//
	// Range		Comments												//
	// ---------    --------												//
	// 0-3999		Initial release (Camelot v1.5, 1st file format release) //
	// 4000-4999	Second release (Camelot v2.0)							//
	//////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// Camelot version 1.5 tags - range 0..3999 - FROZEN!
/////////////////////////////////////////////////////////////////////////////
//some harmless comments
//-----------------
// General tags

#define TAG_UNDEFINED (0xffffffff)		// Special tag that is never used.  Used to detect errors

#define TAG_UP					0
#define TAG_DOWN				1
#define TAG_FILEHEADER			2
#define TAG_ENDOFFILE			3

//-----------------
// Tag management
#define TAG_ATOMICTAGS				10
#define TAG_ESSENTIALTAGS			11
#define TAG_TAGDESCRIPTION			12

//-----------------
// Render-related tags
#define TAG_NONRENDERSECTION_START	20
#define TAG_NONRENDERSECTION_END	21
#define TAG_RENDERING_PAUSE			22
#define TAG_RENDERING_RESUME		23

//-----------------
// Compression & precompression tags 

#define TAG_STARTCOMPRESSION	30
#define TAG_ENDCOMPRESSION		31

//-----------------
// Document tags

#define TAG_DOCUMENT			40
#define TAG_CHAPTER				41
#define TAG_SPREAD				42
#define TAG_LAYER				43
#define TAG_PAGE				44
#define TAG_SPREADINFORMATION	45

#define TAG_GRIDRULERSETTINGS	46
#define TAG_GRIDRULERORIGIN		47
#define TAG_LAYERDETAILS		48
#define TAG_GUIDELAYERDETAILS	49

#define TAG_SPREADSCALING_ACTIVE	52
#define TAG_SPREADSCALING_INACTIVE	53

//-----------------
// Colour reference tags

#define TAG_DEFINERGBCOLOUR		50
#define TAG_DEFINECOMPLEXCOLOUR	51

//-----------------
// Bitmap reference tags

#define TAG_PREVIEWBITMAP_BMP		60
#define TAG_PREVIEWBITMAP_GIF		61
#define TAG_PREVIEWBITMAP_JPEG		62
#define TAG_PREVIEWBITMAP_PNG		63
#define TAG_PREVIEWBITMAP_TIFFLZW	64

#define TAG_DEFINEBITMAP_BMP		65
#define TAG_DEFINEBITMAP_GIF		66
#define TAG_DEFINEBITMAP_JPEG		67
#define TAG_DEFINEBITMAP_PNG		68
#define TAG_DEFINEBITMAP_BMPZIP		69
#define TAG_DEFINESOUND_WAV			70
#define TAG_DEFINEBITMAP_JPEG8BPP	71

//-----------------
// View tags

#define TAG_VIEWPORT				80
#define TAG_VIEWQUALITY				81
#define TAG_DOCUMENTVIEW			82

//-----------------
// Document unit tags

#define TAG_DEFINE_PREFIXUSERUNIT		85
#define TAG_DEFINE_SUFFIXUSERUNIT		86
#define TAG_DEFINE_DEFAULTUNITS			87

//-----------------
// Document info tags

#define TAG_DOCUMENTCOMMENT				90
#define TAG_DOCUMENTDATES				91
#define TAG_DOCUMENTUNDOSIZE			92
//#define TAG_DOCUMENTINDEXING			xx
#define TAG_DOCUMENTFLAGS				93

//-----------------
// Name Gallery tags

#define TAG_NAMEGAL_DOCCOMP				95

//-----------------
// Object tags

#define TAG_PATH							100
#define TAG_PATH_FILLED						101
#define TAG_PATH_STROKED					102
#define TAG_PATH_FILLED_STROKED				103
#define TAG_GROUP							104
#define TAG_BLEND							105
#define TAG_BLENDER							106
#define TAG_MOULD_ENVELOPE					107
#define TAG_MOULD_PERSPECTIVE				108
#define TAG_MOULD_GROUP						109
#define TAG_MOULD_PATH						110
#define TAG_PATH_FLAGS						111
#define TAG_GUIDELINE						112
#define TAG_PATH_RELATIVE					113
#define TAG_PATH_RELATIVE_FILLED			114
#define TAG_PATH_RELATIVE_STROKED			115
#define TAG_PATH_RELATIVE_FILLED_STROKED	116
//#define TAG_PATHREF_INDENTICAL				117		// Not used by version 1.5
#define TAG_PATHREF_TRANSFORM				118

//-----------------
// Attribute tags

#define TAG_FLATFILL								150
#define TAG_LINECOLOUR								151
#define TAG_LINEWIDTH								152
#define TAG_LINEARFILL								153
#define TAG_CIRCULARFILL							154
#define TAG_ELLIPTICALFILL							155
#define TAG_CONICALFILL								156
#define TAG_BITMAPFILL								157
#define TAG_CONTONEBITMAPFILL						158
#define TAG_FRACTALFILL								159
#define TAG_FILLEFFECT_FADE							160
#define TAG_FILLEFFECT_RAINBOW						161
#define TAG_FILLEFFECT_ALTRAINBOW					162
#define TAG_FILL_REPEATING							163
#define TAG_FILL_NONREPEATING						164
#define TAG_FILL_REPEATINGINVERTED					165
#define TAG_FLATTRANSPARENTFILL						166
#define TAG_LINEARTRANSPARENTFILL					167
#define TAG_CIRCULARTRANSPARENTFILL					168
#define TAG_ELLIPTICALTRANSPARENTFILL				169
#define TAG_CONICALTRANSPARENTFILL					170
#define TAG_BITMAPTRANSPARENTFILL					171
#define TAG_FRACTALTRANSPARENTFILL					172
#define TAG_LINETRANSPARENCY						173
#define TAG_STARTCAP								174
#define TAG_ENDCAP									175
#define TAG_JOINSTYLE								176
#define TAG_MITRELIMIT								177
#define TAG_WINDINGRULE								178
#define TAG_QUALITY									179
#define TAG_TRANSPARENTFILL_REPEATING				180
#define TAG_TRANSPARENTFILL_NONREPEATING			181
#define TAG_TRANSPARENTFILL_REPEATINGINVERTED		182

// Arrows and dash patterns
#define TAG_DASHSTYLE								183
#define TAG_DEFINEDASH								184
#define TAG_ARROWHEAD								185
#define TAG_ARROWTAIL								186
#define TAG_DEFINEARROW								187
#define TAG_DEFINEDASH_SCALED						188

// User Attributes
#define TAG_USERVALUE								189

// special colour fills
#define TAG_FLATFILL_NONE							190
#define TAG_FLATFILL_BLACK							191
#define TAG_FLATFILL_WHITE							192
#define TAG_LINECOLOUR_NONE							193
#define TAG_LINECOLOUR_BLACK						194
#define TAG_LINECOLOUR_WHITE						195

// Bitmaps
// general node bitmap - don't know how long this will stay like this
// There are two names for tag 198 for backwards compatibility reasons
#define TAG_NODE_BITMAP								198
#define TAG_BITMAP_OBJECT							198
#define TAG_NODE_CONTONEDBITMAP						199


// New fill type records
//
// Tags range 200 - 299

#define TAG_SQUAREFILL								200
#define TAG_SQUARETRANSPARENTFILL					201
#define TAG_THREECOLFILL							202
#define TAG_THREECOLTRANSPARENTFILL					203
#define TAG_FOURCOLFILL								204
#define TAG_FOURCOLTRANSPARENTFILL					205
//Mark Howitt, 8/10/97. Include new repeating grad fill tag
#define TAG_FILL_REPEATING_EXTRA					206
//Chris Snook, 8/12/99. We also need one for the transparencies!
#define TAG_TRANSPARENTFILL_REPEATING_EXTRA			207


//-----------------
// Regular shapes
// Similarly, tags in the range 1000 - 1999 are reserved for regular shapes.
// Not all are defined in the spec, those that are are marked. However all these
// get used somewhere (I think).

// Ellipses
#define TAG_ELLIPSE_SIMPLE									1000	// defined
#define TAG_ELLIPSE_COMPLEX									1001	// defined

// Rectangles
#define TAG_RECTANGLE_SIMPLE								1100	// defined
#define TAG_RECTANGLE_SIMPLE_REFORMED						1101
#define TAG_RECTANGLE_SIMPLE_STELLATED						1102
#define TAG_RECTANGLE_SIMPLE_STELLATED_REFORMED				1103
#define TAG_RECTANGLE_SIMPLE_ROUNDED						1104	// defined
#define TAG_RECTANGLE_SIMPLE_ROUNDED_REFORMED				1105
#define TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED				1106
#define TAG_RECTANGLE_SIMPLE_ROUNDED_STELLATED_REFORMED		1107
#define TAG_RECTANGLE_COMPLEX								1108	// defined
#define TAG_RECTANGLE_COMPLEX_REFORMED						1109
#define TAG_RECTANGLE_COMPLEX_STELLATED						1110
#define TAG_RECTANGLE_COMPLEX_STELLATED_REFORMED			1111
#define TAG_RECTANGLE_COMPLEX_ROUNDED						1112	// defined
#define TAG_RECTANGLE_COMPLEX_ROUNDED_REFORMED				1113
#define TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED				1114
#define TAG_RECTANGLE_COMPLEX_ROUNDED_STELLATED_REFORMED	1115

// Polygons
#define TAG_POLYGON_COMPLEX									1200	// defined
#define TAG_POLYGON_COMPLEX_REFORMED						1201	// defined
#define TAG_POLYGON_COMPLEX_STELLATED						1212	// defined
#define TAG_POLYGON_COMPLEX_STELLATED_REFORMED				1213	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED							1214	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_REFORMED				1215	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_STELLATED				1216	// defined
#define TAG_POLYGON_COMPLEX_ROUNDED_STELLATED_REFORMED		1217

// General regular shapes
//
// (The first, TAG_REGULAR_SHAPE_PHASE was used briefly during development of version
// 2 of Camelot. It contains _all_ information about regular shapes, including the
// UTCentrePoint. The UTCentrePoint is not guaranteed to be DocCoord(0,0) for all
// regular shapes, but once this was not the case and a few files (on the CD!)
// are old enough to have regular shapes with UTCentrePoint != DocCoord(0,0).
// We still need to have this tag since some files may have been created which
// use it; Charles gets annoyed if files suddenly stop loading for no apparent reason.
// The exporting code has been removed, the importing code is in kernel/rechrshp.cpp
// The second tag below is now used for all regular shapes, any old regualar shapes
// get fixed to place UTCentrePoint at DocCoord(0,0), UTMajorAxis, UTMinorAxis and
// the TransformMatrix get altered accordingly, before saving. The code for fixing
// regular shapes is in kernel/cxfrgshp.cpp.
// Understand? - good, I'm not going to repeat myself.
//
// Ach

#define TAG_REGULAR_SHAPE_PHASE_1					1900
#define TAG_REGULAR_SHAPE_PHASE_2					1901

// Text related records
//
// Tags range 2000 - 2999 are reserved for text related records; this may change in the
// future but it will do for now.

// Text objects

// Text definitions
#define TAG_FONT_DEF_TRUETYPE						2000
#define TAG_FONT_DEF_ATM							2001

// vanilla text story objects
#define TAG_TEXT_STORY_SIMPLE						2100
#define TAG_TEXT_STORY_COMPLEX						2101

// text story objects on a path
#define TAG_TEXT_STORY_SIMPLE_START_LEFT			2110
#define TAG_TEXT_STORY_SIMPLE_START_RIGHT			2111
#define TAG_TEXT_STORY_SIMPLE_END_LEFT				2112
#define TAG_TEXT_STORY_SIMPLE_END_RIGHT				2113
#define TAG_TEXT_STORY_COMPLEX_START_LEFT			2114
#define TAG_TEXT_STORY_COMPLEX_START_RIGHT			2115
#define TAG_TEXT_STORY_COMPLEX_END_LEFT				2116
#define TAG_TEXT_STORY_COMPLEX_END_RIGHT			2117

// Text story information records
#define TAG_TEXT_STORY_WORD_WRAP_INFO				2150
#define TAG_TEXT_STORY_INDENT_INFO					2151

// other text story related objects
#define TAG_TEXT_LINE								2200
#define TAG_TEXT_STRING								2201
#define TAG_TEXT_CHAR								2202
#define TAG_TEXT_EOL								2203
#define TAG_TEXT_KERN								2204
#define TAG_TEXT_CARET								2205
#define TAG_TEXT_LINE_INFO							2206

// Text attributes
#define TAG_TEXT_LINESPACE_RATIO					2900
#define TAG_TEXT_LINESPACE_ABSOLUTE					2901
#define TAG_TEXT_JUSTIFICATION_LEFT					2902
#define TAG_TEXT_JUSTIFICATION_CENTRE				2903
#define TAG_TEXT_JUSTIFICATION_RIGHT				2904
#define TAG_TEXT_JUSTIFICATION_FULL					2905
#define TAG_TEXT_FONT_SIZE							2906
#define TAG_TEXT_FONT_TYPEFACE						2907
#define TAG_TEXT_BOLD_ON							2908
#define TAG_TEXT_BOLD_OFF							2909
#define TAG_TEXT_ITALIC_ON							2910
#define TAG_TEXT_ITALIC_OFF							2911
#define TAG_TEXT_UNDERLINE_ON						2912
#define TAG_TEXT_UNDERLINE_OFF						2913
#define TAG_TEXT_SCRIPT_ON							2914	// explicit offset, size
#define TAG_TEXT_SCRIPT_OFF							2915
#define TAG_TEXT_SUPERSCRIPT_ON						2916	// implicit offset, size
#define TAG_TEXT_SUBSCRIPT_ON						2917	// implicit offset, size
#define TAG_TEXT_TRACKING							2918
#define TAG_TEXT_ASPECT_RATIO						2919
#define TAG_TEXT_BASELINE							2920
#define TAG_TEXT_STRIKETHROUGH_ON					2921
#define TAG_TEXT_STRIKETHROUGH_OFF					2922

											// Next	2923


//-----------------
// Imagesetting and other print-related tags - reserved range 3500 - 3599
// Imagesetting attributes
const ULONG TAG_OVERPRINTLINEON						= 3500;
const ULONG TAG_OVERPRINTLINEOFF					= 3501;
const ULONG TAG_OVERPRINTFILLON						= 3502;
const ULONG TAG_OVERPRINTFILLOFF					= 3503;
const ULONG TAG_PRINTONALLPLATESON					= 3504;
const ULONG TAG_PRINTONALLPLATESOFF					= 3505;
													// Full up
// Document Print/Imagesetting options
const ULONG TAG_PRINTERSETTINGS						= 3506;
const ULONG TAG_IMAGESETTING						= 3507;
const ULONG TAG_COLOURPLATE							= 3508;
													// Full up
// Registration mark records
const ULONG TAG_PRINTMARKDEFAULT					= 3509;
const ULONG TAG_PRINTMARKCUSTOM						= 3510;
													// Next 3511
													// Last 3599


/********************************************
These have not been used in v1 of the file format, so therefore have been removed
//-----------------
// Transform attributes

// Misc transformations
#define TAG_TRANSFORM_IDENTITY			
#define TAG_TRANSFORM_MATRIX			
#define TAG_TRANSFORM_TRANSLATION		
#define TAG_TRANSFORM_GENERAL			

// Rotations
#define TAG_TRANSFORM_ROTATION			
#define TAG_TRANSFORM_ROTATION_CENTRE	
#define TAG_TRANSFORM_ROTATION_POINT	

// Shears
#define TAG_TRANSFORM_SHEAR_X			
#define TAG_TRANSFORM_SHEAR_Y			

// Scalings
#define TAG_TRANSFORM_SCALE				
#define TAG_TRANSFORM_SCALE_X			
#define TAG_TRANSFORM_SCALE_Y			

// Reflections
#define TAG_TRANSFORM_REFLECTION_X		
#define TAG_TRANSFORM_REFLECTION_Y		
**********************************************/



/////////////////////////////////////////////////////////////////////////////
// Camelot version 2.0 tags - range 4000-4999
/////////////////////////////////////////////////////////////////////////////

//-----------------
// Stroke Provider tags
const ULONG TAG_VARIABLEWIDTHFUNC		= 4000;
const ULONG TAG_VARIABLEWIDTHTABLE		= 4001;
const ULONG TAG_STROKETYPE				= 4002;
const ULONG TAG_STROKEDEFINITION		= 4003;
const ULONG TAG_STROKEAIRBRUSH			= 4004;
											// Next 4005

// Next Free Tag = 4010 (please leave me some space for future expansion)

const ULONG TAG_NOISEFILL				= 4010;
const ULONG TAG_NOISETRANSPARENTFILL	= 4011;
const ULONG TAG_MOULD_BOUNDS			= 4012;

// One moved and one new pathref tags
const ULONG TAG_PATHREF_IDENTICAL		= 4013;
const ULONG TAG_PATHREF_TRANSLATE		= 4014;
const ULONG TAG_EXPORT_HINT				= 4015;
											// Next 4016

//-----------------
// New Web Address tags
const ULONG TAG_WEBADDRESS				= 4020;
const ULONG TAG_WEBADDRESS_BOUNDINGBOX	= 4021;
											// Next 4022

//-------------------
// Frmae Layer  tags
const ULONG TAG_LAYER_FRAMEPROPS		= 4030;
const ULONG TAG_SPREAD_ANIMPROPS		= 4031;

//-----------------
// Template Argument/Ops / WizOp / Webster Properties tags
const ULONG TAG_WIZOP					= 4040;
const ULONG TAG_WIZOP_STYLE				= 4041;
const ULONG TAG_WIZOP_STYLEREF			= 4042;
											

/////////////////////////////////////////////////////////////////////////////
/// Camelot version 2.1 tags i.e. next version after 2.0
/////////////////////////////////////////////////////////////////////////////

//-----------------
// Shadows
const ULONG TAG_SHADOWCONTROLLER		= 4050;
const ULONG TAG_SHADOW					= 4051;

//-----------------
// Bevels
const ULONG TAG_BEVEL					= 4052;

// variable tags
const ULONG TAG_BEVATTR_INDENT			= 4053;
const ULONG TAG_BEVATTR_LIGHTANGLE		= 4054;
const ULONG TAG_BEVATTR_CONTRAST		= 4055;
const ULONG TAG_BEVATTR_TYPE			= 4056;

// the bevel inking node tag
const ULONG TAG_BEVELINK				= 4057;

//-----------------
// Blend on a curve tags
const ULONG TAG_BLENDER_CURVEPROP		= 4060;
const ULONG TAG_BLEND_PATH				= 4061;
const ULONG TAG_BLENDER_CURVEANGLES		= 4062;

// Group Transparency Groups
const ULONG TAG_GROUPTRANSP				= 4063;
const ULONG TAG_CACHEBMP				= 4064;
const ULONG TAG_CACHEDNODESGROUP		= 4065;

//-----------------
// Contouring tags
const ULONG TAG_CONTOURCONTROLLER		= 4066;
const ULONG TAG_CONTOUR					= 4067;

//-----------------
// Sets and their properties
const ULONG TAG_SETSENTINEL				= 4070;
const ULONG TAG_SETPROPERTY				= 4071;

// more Blended on a curve tags
const ULONG TAG_BLENDPROFILES			= 4072;
const ULONG TAG_BLENDERADDITIONAL		= 4073;
const ULONG TAG_NODEBLENDPATH_FILLED	= 4074;

//-----------------
// Multi stage fill tags
const ULONG TAG_LINEARFILLMULTISTAGE		= 4075;
const ULONG TAG_CIRCULARFILLMULTISTAGE		= 4076;
const ULONG TAG_ELLIPTICALFILLMULTISTAGE	= 4077;
const ULONG TAG_CONICALFILLMULTISTAGE		= 4078;

//harmless test comment
//-----------------
// painting mode attribute tags
//const ULONG TAG_PAINTINGMODE			= 4078;

// brush attribute tags
const ULONG TAG_BRUSHATTR				= 4079;
const ULONG TAG_BRUSHDEFINITION			= 4080;
const ULONG TAG_BRUSHDATA				= 4081;
const ULONG TAG_MOREBRUSHDATA			= 4082;
const ULONG TAG_MOREBRUSHATTR			= 4083;


//-----------------
// ClipView tags
const ULONG TAG_CLIPVIEWCONTROLLER		= 4084;
const ULONG TAG_CLIPVIEW				= 4085;

//-----------------
// Feathering tags
const ULONG TAG_FEATHER					= 4086;

//-----------------
// Bar properties
const ULONG TAG_BARPROPERTY				= 4087;

//-----------------
// Other multi stage fill tags
const ULONG TAG_SQUAREFILLMULTISTAGE	= 4088;

//-----------------
// Tags for saving current attributes. Currently unused.
// const ULONG TAG_STARTCURRENTATTR		= 4100;
// const ULONG TAG_ENDCURRENTATTR		= 4101;

// more brush tags
const ULONG TAG_EVENMOREBRUSHDATA		= 4102;
const ULONG TAG_EVENMOREBRUSHATTR		= 4103;
const ULONG	TAG_TIMESTAMPBRUSHDATA		= 4104;
const ULONG TAG_BRUSHPRESSUREINFO		= 4105;
const ULONG TAG_BRUSHPRESSUREDATA		= 4106;
const ULONG TAG_BRUSHATTRPRESSUREINFO	= 4107;
const ULONG TAG_BRUSHCOLOURDATA			= 4108;
const ULONG TAG_BRUSHPRESSURESAMPLEDATA	= 4109;
const ULONG TAG_BRUSHTIMESAMPLEDATA		= 4110;
const ULONG TAG_BRUSHATTRFILLFLAGS		= 4111;
const ULONG TAG_BRUSHTRANSPINFO			= 4112;
const ULONG TAG_BRUSHATTRTRANSPINFO		= 4113;

// the nudge size is now a part of the document
const ULONG TAG_DOCUMENTNUDGE			= 4114;

// bitmap properties - stores the settings in the b.p. dialogue for each bitmap
const ULONG TAG_BITMAP_PROPERTIES		= 4115;
const ULONG TAG_DOCUMENTBITMAPSMOOTHING	= 4116;

// Extended bitmap processing data generated by XPE
const ULONG TAG_XPE_BITMAP_PROPERTIES	= 4117;

// Bitmap file format placeholder when bitmap data is generated on the fly from master and XPE info
const ULONG TAG_DEFINEBITMAP_XPE		= 4118;

// Current Attributes container and geometry bounds
const ULONG TAG_CURRENTATTRIBUTES		= 4119;
const ULONG TAG_CURRENTATTRIBUTEBOUNDS	= 4120;

// New 3-point linear fill records
const ULONG TAG_LINEARFILL3POINT			= 4121;
const ULONG TAG_LINEARFILLMULTISTAGE3POINT	= 4122;
const ULONG TAG_LINEARTRANSPARENTFILL3POINT	= 4123;

// New Duplication distance records
const ULONG TAG_DUPLICATIONOFFSET			= 4124;

// Live Effect node tags
const ULONG TAG_LIVE_EFFECT					= 4125;
const ULONG TAG_LOCKED_EFFECT				= 4126;
const ULONG TAG_FEATHER_EFFECT				= 4127;

// Compound group rendering stream warning (e.g. transparent groups, live effects, etc.)
const ULONG TAG_COMPOUNDRENDER				= 4128;

const ULONG TAG_OBJECTBOUNDS				= 4129;

const ULONG TAG_DIMENSION					= 4130;

const ULONG TAG_SPREAD_PHASE2				= 4131;
const ULONG TAG_CURRENTATTRIBUTES_PHASE2	= 4132;
const ULONG TAG_SPREAD_FLASHPROPS			= 4134;
const ULONG TAG_PRINTERSETTINGS_PHASE2		= 4135;

const ULONG TAG_DOCUMENTINFORMATION			= 4136;

const ULONG TAG_CLIPVIEW_PATH				= 4137;
const ULONG TAG_DEFINEBITMAP_PNG_REAL		= 4138;

const ULONG TAG_TEXT_STRING_POS				= 4139;

const ULONG TAG_SPREAD_FLASHPROPS2			= 4140;

const ULONG TAG_TEXT_LINESPACE_LEADING		= 4141;

//-----------------
// Extrusion attributes continue at 4300
const ULONG TAG_EXTRUSIONDEPTH				= 4142;
const ULONG TAG_EXTRUSIONANGLE1				= 4143;
const ULONG TAG_EXTRUSIONANGLE2				= 4144;
const ULONG TAG_EXTRUSIONANGLE3				= 4145;

//const ULONG TAG_SYMBOLREF					= 4146;	// deprecated

const ULONG TAG_SPREAD_PHOTO				= 4147;

const ULONG TAG_EXTRUSIONGROUP				= 4148;
const ULONG TAG_EXTRUSIONCONTROLLER			= 4149;

//-----------------
// New ICC profile and bitmap tags in reserved range 4150-4159
const ULONG TAG_DEFINEBITMAP_PNG_CMYK		= 4150;
const ULONG TAG_DEFINEICC_CMYK				= 4151;
const ULONG TAG_DEFINEBITMAP_EXTERNAL		= 4152;


//-----------------
// New bitmap attributes in reserved range 4160-4189
const ULONG TAG_BITMAPSHARPEN				= 4160; // this is obsolete, just for compatibility issues, use 4175 instead
const ULONG TAG_BITMAPGAMMA					= 4161;
const ULONG TAG_BITMAPGAMMA_RED 			= 4162;
const ULONG TAG_BITMAPGAMMA_GREEN			= 4163;
const ULONG TAG_BITMAPGAMMA_BLUE			= 4164;
const ULONG TAG_BITMAPCONTRAST				= 4165;
const ULONG TAG_BITMAPSATURATION			= 4166;
const ULONG TAG_BITMAPINPUTRANGE    		= 4167;
const ULONG TAG_BITMAPINPUTRANGE_RED   		= 4168;
const ULONG TAG_BITMAPINPUTRANGE_GREEN		= 4169;
const ULONG TAG_BITMAPINPUTRANGE_BLUE		= 4170;
const ULONG TAG_BITMAPOUTPUTRANGE    		= 4171;
const ULONG TAG_BITMAPOUTPUTRANGE_RED		= 4172;
const ULONG TAG_BITMAPOUTPUTRANGE_GREEN		= 4173;
const ULONG TAG_BITMAPOUTPUTRANGE_BLUE		= 4174;
const ULONG TAG_BITMAPSHARPEN_NEW			= 4175;
const ULONG TAG_BITMAPCURVE_RED				= 4176;
const ULONG TAG_BITMAPCURVE_GREEN			= 4177;
const ULONG TAG_BITMAPCURVE_BLUE			= 4178;
const ULONG TAG_BITMAPTEMPERATURE			= 4179;
const ULONG TAG_BITMAPCHECKSUM				= 4180;
const ULONG TAG_BITMAPCURVE_RGB				= 4181;
const ULONG TAG_BITMAPHUEADJUST				= 4182;
const ULONG TAG_BITMAPHUE					= 4183;
const ULONG TAG_BITMAPHUESTRENGTH			= 4184;
const ULONG TAG_BITMAPSHADOWS				= 4185;
const ULONG TAG_BITMAPHIGHLIGHTS			= 4186;
const ULONG TAG_BITMAPCROSSPROCESS			= 4187;

//-----------------
// New node tags (in reserved range 4190-4199)

const ULONG TAG_NODEDETAILS					= 4190;


/////////////////////////////////////////////////////////////////////////////
// XaraLX 0.6
/////////////////////////////////////////////////////////////////////////////

// New text tags for tabbing and paragraph-based indent support

// Text objects
const ULONG TAG_TEXT_TAB						= 4200;

// Text attributes
const ULONG TAG_TEXT_LEFT_INDENT				= 4201;
const ULONG TAG_TEXT_FIRST_INDENT				= 4202;
const ULONG TAG_TEXT_RIGHT_INDENT				= 4203;
const ULONG TAG_TEXT_RULER						= 4204;

// New text tags for linked text areas and text flow

const ULONG TAG_TEXT_STORY_HEIGHT_INFO			= 4205;
const ULONG TAG_TEXT_STORY_LINK_INFO			= 4206;
const ULONG TAG_TEXT_STORY_TRANSLATION_INFO		= 4207;

// New text tags for space before/after and for soft/hard hyphens

const ULONG TAG_TEXT_SPACE_BEFORE				= 4208;
const ULONG TAG_TEXT_SPACE_AFTER				= 4209;
const ULONG TAG_TEXT_SPECIAL_HYPHEN				= 4210;

// Tags recognized by Xtreme Pro 3.2 end here

const ULONG TAG_TEXT_SOFT_RETURN				= 4211;

// Tags for extra font information for external filters

const ULONG TAG_TEXT_EXTRA_FONT_INFO			= 4212;
const ULONG TAG_TEXT_EXTRA_TT_FONT_DEF			= 4213;
const ULONG TAG_TEXT_EXTRA_ATM_FONT_DEF			= 4214;

// Text repelling

const ULONG TAG_REPEL							= 4215; // obsolete
const ULONG TAG_REPEL_DISTANCE					= 4216;

// document option to render text with a URL applied as underlined
const ULONG TAG_UNDERLINE_URLS					= 4217;
const ULONG TAG_TEXT_RUN_BREAK					= 4218;	// a marker for export filters

const ULONG TAG_EXTRUSIONCONTROLLER_TRANS		= 4300;
const ULONG TAG_EXTRUSIONCONTROLLER_COMPLEX		= 4301;
const ULONG TAG_EXTRUSIONBEVELDEPTH				= 4302;
const ULONG TAG_EXTRUSIONGLOSS					= 4303;
const ULONG TAG_EXTRUSIONBEVELTYPE				= 4304;
const ULONG TAG_EXTRUSIONCORNERTYPE				= 4305;
const ULONG TAG_EXTRUSIONLIGHT1ANGLE1			= 4306;
const ULONG TAG_EXTRUSIONLIGHT1ANGLE2			= 4307;
const ULONG TAG_EXTRUSIONLIGHT1COLOUR			= 4308;
const ULONG TAG_EXTRUSIONLIGHT2ANGLE1			= 4309;
const ULONG TAG_EXTRUSIONLIGHT2ANGLE2			= 4310;
const ULONG TAG_EXTRUSIONLIGHT2COLOUR			= 4311;
const ULONG TAG_EXTRUSIONLIGHT3ANGLE1			= 4312;
const ULONG TAG_EXTRUSIONLIGHT3ANGLE2			= 4313;
const ULONG TAG_EXTRUSIONLIGHT3COLOUR			= 4314;
const ULONG TAG_EXTRUSIONPERSPECTIVE			= 4315;

const ULONG TAG_EXTRUSIONGROUP_PHASE2			= 4316;

const ULONG TAG_EFFECTATTR_FLAG					= 4330;
const ULONG TAG_PHOTOATTR_FLAG					= 4331;

//-----------------
// New symbol tags in reserved range 4340-4349
const ULONG TAG_SYMBOLDEFINITION				= 4340;
const ULONG TAG_SYMBOLDEFINITION_LINKED			= 4341;	// Not used at present
const ULONG TAG_SYMBOLREF_TRANS					= 4342;
const ULONG TAG_SYMBOLREF_COMPLEX				= 4343;

// Tags recognized by Xtreme Pro 4.0 end here

const ULONG TAG_FONT_GLYPH                      = 4350;
const ULONG TAG_FONT_KERNINGPAIRS               = 4351;
const ULONG TAG_TEXT_LANGUAGE					= 4352;
const ULONG TAG_TEXT_SPELL_OPTIONS				= 4353;
const ULONG TAG_TEXT_STRING_POS2				= 4354;

const ULONG TAG_URL_COLOUR_OVERRIDE				= 4360;
const ULONG TAG_DOCUMENT_URL_COLOUR				= 4361;
const ULONG TAG_DOCUMENT_MOUSE_OVER_URL_COLOUR	= 4362;
const ULONG TAG_DOCUMENT_VISITED_URL_COLOUR		= 4363;
const ULONG TAG_WEBSITE_PROPERTIES              = 4364;
const ULONG TAG_SUPPORTFOLDER_PATH				= 4365;

const ULONG TAG_LINEARTRANSPARENTFILLMULTISTAGE			= 4366;
const ULONG TAG_LINEARTRANSPARENTFILLMULTISTAGE3POINT	= 4367;
const ULONG TAG_CIRCULARTRANSPARENTFILLMULTISTAGE		= 4368;
const ULONG TAG_ELLIPTICALTRANSPARENTFILLMULTISTAGE		= 4369;
const ULONG TAG_CONICALTRANSPARENTFILLMULTISTAGE		= 4370;
const ULONG TAG_SQUARETRANSPARENTFILLMULTISTAGE			= 4371;

const ULONG TAG_WEBSITE_PAGE_PROPERTIES			= 4372;

const ULONG TAG_SPREAD_FLASHPROPS3				= 4373;

const ULONG TAG_AFFILIATEID						= 4374;

//-----------------
// Very New Web Address tags
const ULONG TAG_WEBADDRESS_PAGE						= 4375;
const ULONG TAG_WEBADDRESS_PAGE_BOUNDINGBOX			= 4376;
const ULONG TAG_WEBADDRESS_OBJECT					= 4377;
const ULONG TAG_WEBADDRESS_OBJECT_BOUNDINGBOX		= 4378;
const ULONG TAG_WEBADDRESS_LAYER					= 4379;
const ULONG TAG_WEBADDRESS_LAYER_BOUNDINGBOX		= 4380;
const ULONG TAG_WEBADDRESS_SCALEDPOPUP				= 4381;
const ULONG TAG_WEBADDRESS_SCALEDPOPUP_BOUNDINGBOX	= 4382;

const ULONG TAG_SPREAD_FLASHPROPS4					= 4383;

// Tags recognized by Web Designer 5.0 end here

// Composite bitmap group rendering stream warning
const ULONG TAG_PHOTOGROUPRENDER					= 4384;
const ULONG TAG_BITMAPCLONEFILL						= 4385;

const ULONG TAG_BITMAPSHARPEN_PHASE3				= 4386;

const ULONG TAG_BITMAPINTELLIGENTSCALE				= 4387;

const ULONG TAG_REGIONLAYERDETAILS					= 4388;		// Depracated - Use a flag in TAG_EXTENDEDLAYERDETAILS record
const ULONG TAG_REDEYEPATCH							= 4389;

const ULONG TAG_ISCALEDEFINITION					= 4390;
const ULONG TAG_ISCALEDEFINITION_MASK				= 4391;
const ULONG TAG_ISCALEDEF							= 4392;
const ULONG TAG_ISCALEDEF_MASK						= 4393;

const ULONG TAG_TEXT_ANCHOR_POSITION				= 4394;
const ULONG TAG_ANCHORED_GROUP_INFO					= 4395;

// Tags recognized by Xtreme Pro 5.1 end here

const ULONG TAG_FTP_DETAILS							= 4396;
const ULONG TAG_SPREAD_BLANKHTMLPAGE				= 4397;

const ULONG TAG_MENU_DETAILS						= 4398;
const ULONG TAG_DOCUMENT_MENU_TEXT_COLOUR			= 4399;
const ULONG TAG_DOCUMENT_MENU_BACK_COLOUR			= 4400;
const ULONG TAG_DOCUMENT_MENU_TEXT_OVER_COLOUR		= 4401;
const ULONG TAG_DOCUMENT_MENU_BACK_OVER_COLOUR		= 4402;
const ULONG TAG_DOCUMENT_MENU_BORDER_COLOUR			= 4403;

const ULONG TAG_TEXT_BULLETED_ITEM_START			= 4404;
const ULONG TAG_TEXT_AUTO_LIST_CHAR					= 4405;
const ULONG TAG_BITMAPISCALE						= 4406;
const ULONG TAG_TEXT_ATTRIBUTE_SHADOW				= 4407;
const ULONG TAG_TEXT_FONT_TYPEFACE_SHADOWED			= 4408;

const ULONG TAG_LAYER_PAPER_BACKGROUND				= 4409;

const ULONG TAG_TEXT_NUMBERED_ITEM_START			= 4410;

const ULONG TAG_EXTENDEDLAYERDETAILS				= 4411;

const ULONG TAG_BITMAPBOUNDSFILL					= 4412;

const ULONG TAG_SOFTMASKGROUP						= 4413;

const ULONG TAG_NODEMASKDETAILS						= 4414;

const ULONG TAG_BITMAPDEFAULTTRANS					= 4415;

const ULONG TAG_BITMAPISCALE2						= 4416;

const ULONG TAG_CLONEFLAG							= 4418;

const ULONG TAG_NAVBAR_START						= 4419;
const ULONG TAG_NAVBAR_END							= 4420;
const ULONG TAG_NAVBAR_MENUSTYLE_INFO				= 4421;
const ULONG TAG_NAVBAR_BUTTON_STRETCH_INFO			= 4422;

const ULONG TAG_DOCUMENT_HIGHSLIDE_INFO				= 4423;

const ULONG TAG_WEBADDRESS_FILE						= 4424;
const ULONG TAG_WEBADDRESS_FILE_BOUNDINGBOX			= 4425;

const ULONG TAG_NAVBAR_BUTTON_MENU					= 4426;
const ULONG TAG_NAVBAR_BUTTON_MENU_ITEM				= 4427;
const ULONG TAG_NAVBAR_BUTTON_MENU_SEPARATOR		= 4428;

const ULONG TAG_SPREAD_ANIMPROPS2					= 4429;
const ULONG TAG_COMPATIBILITYATTR_FLAG				= 4430;
const ULONG TAG_NAVBAR_LOCAL_MENUSTYLE				= 4431;

// Tags recognized by Web Designer 6.0 end here

// Tags recognized by PGD6.1/DesignerPro 6.1 end here

const ULONG TAG_TEXT_STORY_PHOTOSHOP_POSITIONING	= 4432;

const ULONG TAG_RESOLUTION_HINT						= 4433;

const ULONG TAG_SYMBOLGROUP							= 4434;

const ULONG TAG_BITMAPFILL_AUTOCLONE				= 4435;

const ULONG TAG_PRESENTATION_DOC_PROPERTIES			= 4436;
const ULONG TAG_PRESENTATION_PAGE_PROPERTIES		= 4437;
const ULONG TAG_USERVALUE_PRESENTATION				= 4438;

const ULONG TAG_WEBSITE_PROPERTIES2					= 4439;
const ULONG TAG_WEBSITE_PAGE_PROPERTIES2			= 4440;

const ULONG TAG_PRESENTATION_LAYER_PROPERTIES		= 4441;
const ULONG TAG_LAYER_REPEL_WITHIN                  = 4442;
const ULONG TAG_REPEL_DISTANCE_V					= 4443;
const ULONG TAG_ANCHORED_GROUP_INFO2				= 4444;

const ULONG TAG_FILEDEFINITION						= 4445;
const ULONG TAG_FILEREFERENCE						= 4446;

const ULONG TAG_FTP_WEBSITEURL						= 4447;
const ULONG TAG_PRESENTATION_LAYER_FLAG				= 4448;

const ULONG TAG_WIDGETDEFINITION					= 4449;
const ULONG TAG_WIDGETREFERENCE						= 4450;

const ULONG TAG_EXPORTFEATURES						= 4451;

const ULONG TAG_AUTOGENERATEDNAME					= 4452;

const ULONG TAG_WIDGETREVISION						= 4453;

// Tags recognized by Web Designer 7.0 end here

const ULONG TAG_SYMBOLGROUP_LIVECOPY				= 4454;

// Tags recognized by PGD7.1/DesignerPro 7.1 end here

const ULONG TAG_EMBEDDED_ANCHORED_GROUP_INFO		= 4455;
const ULONG TAG_TEXT_EMBEDDED_ANCHOR_POS_INFO		= 4456;
const ULONG TAG_EMBEDDED_TRANSFORMED_ANCHORED_GROUP_INFO = 4457;

const ULONG TAG_SHUFFLE								= 4458;

const ULONG TAG_NODECLONEREFS						= 4459;	// Not used, yet!
const ULONG TAG_CLONEREF							= 4460;	// Not used, yet!
const ULONG TAG_CLONESOURCE							= 4461;
const ULONG TAG_BITMAP_HIDDEN						= 4462;

const ULONG TAG_STYLE_DEF							= 4463;
const ULONG TAG_STYLE								= 4464;
const ULONG TAG_TEXT_PARAGRAPH_STYLE				= 4465;
const ULONG TAG_TEXT_PRESENT_BY_PARAGRAPH			= 4466;

const ULONG TAG_COLSELGROUP							= 4467;
const ULONG TAG_COLSELMARKUP_WEB					= 4468; // Variant of TAG_ELLIPSE_SIMPLE

const ULONG TAG_PRIVATE_FILTER_INFO					= 4469;
const ULONG TAG_ATTR_FILTER_INFO					= 4470;

const ULONG TAG_COLSELMARKUP						= 4471; // Variant of TAG_REGULAR_SHAPE_PHASE_2
const ULONG TAG_LOCK_POPUP_LAYER					= 4472;

const ULONG TAG_WIDGETDESIGNSIZE					= 4473;
const ULONG TAG_WIDGET_EXTRAINFO					= 4474;

const ULONG TAG_CLONESOURCE2						= 4475;

const ULONG TAG_TEXT_DO_NOT_SYNCH					= 4476;

// Tags recognized by version 8.1.3 end here

const ULONG TAG_TEXT_SIMPLE_MACRO_MARKER            = 4477;
const ULONG TAG_TEXT_MACRO_CHAR						= 4478;

// Tags recognized by version 8.1 end here

const ULONG TAG_ONLINEINFO_NEXT_ID					= 4479;
const ULONG TAG_ONLINEINFO_ID						= 4480;
const ULONG TAG_ONLINEINFO_PAGEID					= 4481;
const ULONG TAG_GOOGLE_WEB_FONTS_LIST_PATH			= 4482;

const ULONG TAG_WEBSITE_PROPERTIES_ANALYTICS		= 4483;

const ULONG TAG_WSM_NAVBAR							= 4484;
const ULONG TAG_WSM_NAVBAR_BUTTON					= 4485;
const ULONG TAG_TEXT_LINE_INFO2						= 4486;

const ULONG TAG_ALTERNATEBITMAP_JPEG				= 4487;
const ULONG TAG_ALTERNATEBITMAP_PNG					= 4488;
const ULONG TAG_ALTERNATEBITMAP_PNG_REAL			= 4489;

const ULONG TAG_FTP_DETAILS2						= 4490;

const ULONG TAG_FONT_GLYPH_EXTRA_WEIGHT				= 4491;
const ULONG TAG_FONT_KERNINGPAIRS_EXTRA_WEIGHT		= 4492;
const ULONG TAG_TEXT_STORY_WORD_LINE_SPACING		= 4493;

const ULONG TAG_LAST_HTML_PUBLISH_NAME				= 4494;

// Tags recognized by version 9.0 end here
// Tags recognized by version 9.1 end here
// Tags recognized by version 9.2 end here

const ULONG TAG_TEXT_BREAK_TYPE						= 4495;
const ULONG TAG_TEXT_STORY_AUTOFLOW					= 4496;

const ULONG TAG_DOCUMENTNUDGE_PHASE2				= 4497;

const ULONG TAG_SMARTSHAPEGROUP						= 4498;

const ULONG TAG_LIVESHARE							= 4499;
const ULONG TAG_PAGEVARIANTDEFINITION				= 4500;
const ULONG TAG_PAGEVARIANTREFERENCE				= 4501;

const ULONG TAG_IMPLICIT_TAB_DISTANCE				= 4502;
const ULONG TAG_WEBSITE_PROPERTIES_EXPORT			= 4503;
const ULONG TAG_TEXT_FONT_SIZE_RELATIVE_KERN		= 4504;
const ULONG TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING	= 4505;
const ULONG TAG_WEBSITE_TRANSITION					= 4506;
const ULONG TAG_USERVALUE_V10						= 4507;
const ULONG TAG_EMBEDDED_GLYPHS_SIZE_HINT			= 4508;

const ULONG TAG_DOCUMENTVIEW_VARIANT				= 4509;
const ULONG TAG_VARIANT_STYLE_DELTA					= 4510;

const ULONG TAG_SPREAD_THUMB						= 4511;
const ULONG TAG_SPREAD_THUMB_RETINA					= 4512;

// Tags recognized by version 10.0 end here

const ULONG TAG_COLSELENHANCE_ATTR					= 4513;
const ULONG TAG_ANCHORING_MODE						= 4514;
const ULONG TAG_PAGEVARIANTSLIST					= 4515;
const ULONG TAG_PAGEVARIANTDESCRIPTION				= 4516;
const ULONG TAG_EFFECTATTR_FLAG_PHASE2				= 4517;
const ULONG TAG_COLSELENHANCE_TRANSP				= 4518;

// Tags recognized by version 10.1 end here

const ULONG TAG_CMYKJPEG_NORMAL						= 4519;
const ULONG TAG_CMYKJPEG_INVERT						= 4520;

const LONG TAG_TXTBACKGROUNDCOLOUR					= 4521;
const LONG TAG_TXTBACKGROUNDCOLOUR_NONE				= 4522;
const LONG TAG_TXTBACKGROUNDCOLOUR_BLACK			= 4523;
const LONG TAG_TXTBACKGROUNDCOLOUR_WHITE			= 4524;

const ULONG TAG_WEBSITE_PROPERTIES_EXPORT2          = 4525;

const ULONG TAG_MOULDBRUSHATTR						= 4526;

const ULONG TAG_TEXT_HYPHENATION_OFF				= 4527;
const ULONG TAG_TEXT_HYPHENATION_RELATIVE			= 4528;
const ULONG TAG_TEXT_HYPHENATION_ABSOLUTE			= 4529;
const ULONG TAG_TEXT_HYPHENATION_DOCUMENT_DEFAULT	= 4530;
const ULONG TAG_DOCUMENT_HYPHENATION_DEFAULTS_RELATIVE = 4531;
const ULONG TAG_DOCUMENT_HYPHENATION_DEFAULTS_ABSOLUTE = 4532;

const ULONG TAG_BRUSHDEFINITION_MOULD				= 4533;
const ULONG TAG_MOULDBRUSHDATA						= 4534;

const ULONG TAG_FTP_DETAILS3						= 4535;

const ULONG TAG_DOCUMENT_HYPHENATION_CUSTOM_LIST	= 4536;

const ULONG TAG_WEBSITE_PROPERTIES_EXPORT3			= 4537;

const ULONG TAG_FONT_EMBEDDING_POLICY				= 4538;

const ULONG TAG_TEXT_EOL_VIRTUAL					= 4539;

const ULONG TAG_ANCHORED_GROUP_SCALING_INTENDED = 4540;
const ULONG TAG_ANCHORED_GROUP_SCALING_RELATIVE = 4541;
const ULONG TAG_ANCHORED_GROUP_SCALING_DISABLED = 4542;

const ULONG TAG_ANCHORED_GROUP_WRAP_LEFT = 4543;
const ULONG TAG_ANCHORED_GROUP_WRAP_RIGHT = 4544;

const ULONG TAG_ANCHORED_GROUP_MARGINS = 4545;

const ULONG TAG_TEXT_MACRO_PAGE_NUMBER_OFFSET = 4546;

// Tags recognized by version 11.0 end here

const ULONG TAG_TEXT_SYMBOL_CHAR = 4547;

const ULONG TAG_REPEL_RECT_REPEL = 4548;

const ULONG TAG_TEXT_SYMBOL_CHAR_POS = 4549;
const ULONG TAG_TEXT_SYMBOL_CHAR_POS2 = 4550;

const ULONG TAG_TEXT_ALM_INDENT_LEVEL = 4551;

const ULONG TAG_ONLINE_PASTE_INFO = 4552;

const ULONG TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_ON  = 4553;
const ULONG TAG_TEXT_KEEP_WITH_NEXT_PARAGRAPH_OFF = 4554;
const ULONG TAG_TEXT_KEEP_LINES_TOGETHER_ON       = 4555;
const ULONG TAG_TEXT_KEEP_LINES_TOGETHER_OFF      = 4556;

const ULONG TAG_TEXT_ALM_LINKS = 4557;

const ULONG TAG_SOURCE_CLIPART_PATH = 4558;

const ULONG TAG_TEXT_TOC_MARKER_OUTLINE_LEVEL = 4559;

const ULONG TAG_TEXT_STORY_VERTICAL_ALIGN_CENTRE = 4560;
const ULONG TAG_TEXT_STORY_VERTICAL_ALIGN_BOTTOM = 4561;

const ULONG TAG_ANCHORED_GROUP_INFO3 = 4562;

const ULONG TAG_TEXT_TOC_MARKER_FLAGS = 4563;

const ULONG TAG_FONT_LIGATURES = 4564; //unused

const ULONG TAG_INDEXEDCOLOURIDS = 4565;
const ULONG TAG_TEXT_OPEN_TYPE_LIGATURES = 4566;
const ULONG TAG_FONT_OTLIGATURES = 4567;

const ULONG TAG_ONLINEINFO_DATALIST = 4568;
const ULONG TAG_ONLINEINFO_BITMAPNAME = 4569;
const ULONG TAG_ONLINEINFO_NEXTBITMAPID = 4570;

const ULONG TAG_TEXT_OPEN_TYPE_STYLISTICSETS = 4571;
const ULONG TAG_FONT_OTSTYLISTICSETS = 4572;
const ULONG TAG_FONT_OTLIGATURES_EXT = 4573;

const ULONG TAG_TEXT_LIGATURE_START = 4574;
const ULONG TAG_TEXT_LIGATURE_SLAVE = 4575;

const ULONG TAG_FONT_OTSTYLISTICSETNAMES = 4576;

const ULONG TAG_TEXT_STYLISTIC_SET_SUB = 4577;

const ULONG TAG_DOCUMENTHISTORY_ENTRY_CREATE = 4578;
const ULONG TAG_DOCUMENTHISTORY_ENTRY = 4579;

const ULONG TAG_ATTR_COMMENT_MARKER = 4580;
const ULONG TAG_ATTR_COMMENT_DATA = 4581;
const ULONG TAG_TEXT_COMMENT_ANCHOR_POS_INFO = 4582;
const ULONG TAG_FONT_DEF_VARIANTS_INFO = 4583;

const ULONG TAG_DOCUMENT_DEFAULT_LANGUAGE = 4584;

const ULONG TAG_ANCHORED_GROUP_PAGE_OFFSET = 4585;

const ULONG TAG_TXT_LIST_PROPERTIES			= 4586;
const ULONG TAG_TXT_LIST_NUMBER_PROPERTIES	= 4587;
const ULONG TAG_TXT_LIST_BULLET_PROPERTIES	= 4588;
const ULONG TAG_TXT_LIST_FONT_SIZE			= 4589;
const ULONG TAG_TXT_LIST_VERTICAL_OFFSET	= 4590;

const ULONG TAG_TXT_LIST_FONT_COLOUR		= 4591;
const ULONG TAG_TXT_LIST_FONT_COLOUR_NONE	= 4592;
const ULONG TAG_TXT_LIST_FONT_COLOUR_BLACK	= 4593;
const ULONG TAG_TXT_LIST_FONT_COLOUR_WHITE	= 4594;

const ULONG TAG_ANCHORED_GROUP_AFFECT_LINE_FLOW = 4595;

const ULONG TAG_NEXT_COMMENT_ID 			= 4596;

const ULONG TAG_LIST_STYLE_SUPPLEMENT		= 4597;

// Tags recognized by version 15.0 end here

const ULONG TAG_SPREAD_DESIGN_INFO			= 4598;
const ULONG TAG_SPREADINFORMATION_EXTRA		= 4599;

// two new types for text stories inside object (atomic records)
const ULONG TAG_TEXT_STORY_INSIDE_SIMPLE	= 4600;
const ULONG TAG_TEXT_STORY_INSIDE_COMPLEX	= 4601;

const ULONG TAG_COMPANY_WORKSPACE			= 4602;

// Tags recognized by version 15.1 end here

const ULONG TAG_FONT_OTSUBSTITUTIONRULES	= 4603;
const ULONG TAG_FONT_OTSUBSTITUTIONTABLES	= 4604;
const ULONG TAG_FONT_OTRULEMAXSIZES			= 4605;

const ULONG TAG_TEXT_CHANGE_CAPS			= 4606;

const ULONG TAG_TEXT_WIDOWS_AND_ORPHANS_ON	= 4607;
const ULONG TAG_TEXT_WIDOWS_AND_ORPHANS_OFF = 4608;

const ULONG TAG_TEXT_STORY_AUTO_SIZE_TO_FIT = 4609;

const ULONG TAG_TEXT_OT_CONTEXT_ALT_ON		= 4610;
const ULONG TAG_TEXT_OT_CONTEXT_ALT_OFF		= 4611;

const ULONG TAG_TEXT_OT_STYLE_ALT_ON		= 4612;
const ULONG TAG_TEXT_OT_STYLE_ALT_OFF		= 4613;

const ULONG TAG_TEXT_OT_TITLE_ALT_ON		= 4614;
const ULONG TAG_TEXT_OT_TITLE_ALT_OFF		= 4615;

const ULONG TAG_TEXT_OT_SWASHES_ON			= 4616;
const ULONG TAG_TEXT_OT_SWASHES_OFF			= 4617;

const ULONG TAG_TEXT_TEMPLATE_CHAR_FLAG		= 4618;

const ULONG	TAG_TEXT_AUTO_FIELD_MARKER			= 4619;
const ULONG TAG_TEXT_STORY_SHRINK_ON_OVERFLOW	= 4620;

const ULONG TAG_BRANDDOCUMENT					= 4621;
const ULONG TAG_SMARTFIELD_DICT					= 4622;

const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR		= 4623;
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_NONE	= 4624;
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_BLACK	= 4625;
const LONG TAG_TXT_PARA_BACKGROUNDCOLOUR_WHITE	= 4626;

const LONG TAG_DOCUMENT_METADATA				= 4627;

// new webaddress tag
const ULONG TAG_WEBADDRESS_SMARTFIELD				= 4628;
const ULONG TAG_WEBADDRESS_SMARTFIELD_BOUNDINGBOX	= 4629;

// paragraph background border
const LONG TAG_TXT_PARA_BACKGROUND_BORDER			= 4630;
const LONG TAG_TXT_PARA_TOP_BORDER					= 4631;
const LONG TAG_TXT_PARA_BOTTOM_BORDER				= 4632;
const LONG TAG_TXT_PARA_LEFT_BORDER					= 4633;
const LONG TAG_TXT_PARA_RIGHT_BORDER				= 4634;
const LONG TAG_TXT_PARA_INTERNAL_BORDER				= 4635;

// persistent angles
const LONG TAG_NODEANGLE							= 4636;
const LONG TAG_NODEANGLE4638						= 4638;

const ULONG TAG_WEBADDRESS_NOFOLLOW					= 4639;


/////////////////////////////////////////////////////////////////////////////
// TAG RANGES
// Add new tags just before this comment
// The current range for tags is 4000-4999. DO NOT define tags with values
// outside this range. Ask Gerry if you need to extend the range or
// do anything else unusual.
/////////////////////////////////////////////////////////////////////////////

#endif
