/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /camelot/kernel/DocCoord.H 9     9/05/95 18:24 Peter $

/*********************************************************************************************

>	class DocCoord

	Author:		Mario Shamtani
	Created:	20/5/1993          
	Purpose:	Structure holding a 32-bit cartesian DocCoordinate.
	Errors:		None.

*********************************************************************************************/

#ifndef INC_DOCCOORD
#define INC_DOCCOORD

#include <math.h>

class DocCoord
{
public:
	LONG x;
	LONG y;
	
	inline void translate(LONG thatX, LONG thatY);

	friend DocCoord operator-(const DocCoord&);

	inline BOOL operator==(const DocCoord&) const;
	inline BOOL operator!=(const DocCoord&) const;

	double Distance(const DocCoord&) const;

	DocCoord(LONG x1, LONG y1) { x=x1; y=y1; }
	DocCoord() {} 
};   

/********************************************************************************************

>	inline void DocCoord::translate(const LONG thatX, const LONG thatY)

	Author: 	Mario Shamtani
	Created:	21/5/93
	Inputs:		translation DocCoordinates.
	Outputs:	None
	Returns:	None.
	Purpose:	tranlates a DocCoord by thatX and thatY.
	Errors:		None.

********************************************************************************************/
/*
Technical notes:

********************************************************************************************/

inline void DocCoord::translate(LONG thatX, LONG thatY) 
{
	this->x += thatX;
	this->y += thatY;
}

//-----------------------------------------------------------------------------------------

inline DocCoord operator-(const DocCoord& input)
{
	DocCoord result;
	result.x = -input.x;
	result.y = -input.y;
	return result;
}

//-----------------------------------------------------------------------------------------

inline BOOL DocCoord::operator==(const DocCoord& input) const
{
	return ((input.x == x) && (input.y == y));
}

//-----------------------------------------------------------------------------------------

inline BOOL DocCoord::operator!=(const DocCoord& input) const
{
	return ((input.x != x) || (input.y != y));
}


#endif // INC_DOCCOORD
