/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/winoil/ensure.cpp 24    7/08/96 21:30 Richard $

#include "stdafx.h"  

#include "ensure.h"
#include "errors.h"
#include "camelot.h"
//#include "andy.h"
//#include "simon.h"				// for IDS_CONTINUE
//#include "errorbox.h"			// for CInformErrorDialog::GetSafeParent

#ifdef AFX_DBG1_SEG                                               
#pragma code_seg(AFX_DBG1_SEG)
#endif

// NOTE: in separate module so it can replaced if needed.

#ifdef _ENSURES
int NEAR AssertBusy = FALSE;				  // for testing diagnostics			
#endif

#if 0
static void InternalAssert(LPCSTR AssertDescription, LPCSTR lpszFileName, int nLine, BOOL UseMFC)
{
	if (!UseMFC)
		MessageBeep(MB_ICONASTERISK);

#ifdef _ENSURES

	char sz[255];
	static char BASED_CODE szTitle[] = "Ensure Failed!";
	static char BASED_CODE szMessage[] = "%s, File %s, Line %d"; 
	static char BASED_CODE szMessageNoReason[] = "Error %d@%s";			// note number first

	// get app name or NULL if unknown (don't call assert)
#ifndef _AFXDLL
	const char* pszAppName = afxCurrentAppName;
#else
	#if WIN32
	const char* pszAppName = AfxGetAppName();
	#else
	const char* pszAppName = _AfxGetAppData()->appCurrentAppName;
	#endif
#endif   

	if (AssertDescription != NULL)
		wsprintf(sz, 
			(LPCSTR) szMessage,                       
			AssertDescription, 
			lpszFileName, 
			nLine);    
	else   
		wsprintf(sz, 
			(LPCSTR) szMessageNoReason,
			nLine,
			lpszFileName);
	
#ifdef _DEBUG
	if (afxTraceEnabled)
	{
		// assume the debugger or auxiliary port
		#ifndef _MAC
		::OutputDebugString(sz);
		::OutputDebugString(", ");
		::OutputDebugString(szTitle);
		::OutputDebugString("\n\r");
		#else
		// Mac version needs them all together
		_tcscat(sz, ", ");
		_tcscat(sz,szTitle);
		_tcscat(sz,"\n\r");
		::OutputDebugString(sz);
		#endif
	}
#endif


	if (AssertBusy > 0)
	{
		// getting assertion while inside an assertion
		//  - it has already been traced, but 
		// break into the debugger (or Dr Watson log)
		#ifdef _DEBUG
		DebugBreak();									// else Win32s might die (just beeps)
		#endif
		return;
	}

	AssertBusy++;

	if ( Error::IsInRenderThread() )
	{
		TRACE("In RenderThread so clearing up system");
		Error::RenderThreadReset();
		UseMFC = FALSE;								// mustn't use MFC for render errors
	}

	int result;
	TCHAR RealErrorMsg[256];

	if (UseMFC)
	{
		// use our error handler to report ensures as it is more robust instead of MessageBox
		// The arrangement of buttons is the same as the exception handler dialog, not the same
		// as the original Ensure dialog (which was Abort/Retry/Ignore)
		ErrorInfo Info;

		// try to preserve original error message
		lstrcpy( RealErrorMsg, Error::GetErrorString() );

		Error::SetErrorSerious( sz );			// set error msg (vaping any previous one)
		Info.ErrorMsg = 0;
		Info.Button[0] = IDS_CONTINUE;
		Info.Button[1] = IDS_QUIT;
		if (IsWin32NT())
		{
			// only NT can cope with an Abort button
			Info.Button[2] = IDS_ABORT;
			Info.Button[3] = IDS_DEBUG;
		}
		else
		{
			Info.Button[2] = IDS_DEBUG;
		}
													// no default button (or Help)
		Info.Cancel = 1;							// Esc = Continue

		result = InformGeneral( ERRORTYPE_ENSURE, &Info );
	}
	else
	{
		CCamApp::DisableSystem();

		// e.g. if this an MFC assert, don't use our lovely error handler becuase that itself
		// uses MFC. Instead we use good old MessageBox

#ifndef EXCLUDE_FROM_RALPH
		const HWND Parent = CInformErrorDialog::GetSafeParent()->GetSafeHwnd();
#else
		const HWND Parent = NULL;
#endif

		const int nCode = ::MessageBox(Parent, sz, szTitle,
			MB_SYSTEMMODAL | MB_ICONHAND | MB_ABORTRETRYIGNORE);

		CCamApp::EnableSystem();

		// convert MessageBox return value into a sane button ID
		switch (nCode)
		{
			case IDABORT:
				result = IDS_ABORT;
				break;
			case IDRETRY:
				result = IDS_DEBUG;
				break;
			case IDIGNORE:
			default:
				result = IDS_CONTINUE;
				break;
		}
	}

	AssertBusy--;

	switch (result)
	{
		case IDS_DEBUG:
			DebugBreak();
			break;
		case IDS_CONTINUE:
			break;

		case IDS_QUIT:
			// quit in a slightly nice way
			{
				CWnd *MainWnd = AfxGetApp()->m_pMainWnd;
				if (MainWnd && MainWnd->IsWindowVisible() )
				{
					MainWnd->DestroyWindow();
					break;
				}
			}
			nobreak;							// if no window, try nasty exit

		case IDS_ABORT:
			// Abort - exit nastily
			AfxAbort();
			return;
			break;								// unlikely to get here
	}

	// if we ever get here we just carry on as if nothing happened

	if (UseMFC)
	{
		if (RealErrorMsg[0])
			Error::SetErrorSerious( RealErrorMsg );		// restore previous error msg
		else
			Error::ClearError();
	}

#else
	// parameters not used if non-debug
	(void)lpszFileName;
	(void)nLine;
#endif // _ENSURES

}
#endif

/***********************************************************************************************

>	void EnsureFailedLine(LPCSTR AssertDescription, LPCSTR lpszFileName, int nLine)

	Author:		Simon M
	Created:	3/5/93
	Inputs:		AssertDescription: Reason for failure description  
				lpszFileName:      Filename of file that caused failure
				nLine:             Line number that caused failure
	Outputs:	-
	Returns:	-     
	Purpose:    This function should only ever be invoked from the ENSURE macro 
				which is similar to the ASSERT macro, the only difference being that 
				it takes a parameter which describes the reason for the failure. 
	Errors:		-
	SeeAlso:	-

***********************************************************************************************/
/* Technical notes: 

This function is an almost direct copy of the AfxAssertFailedLine function which is
called from the ASSERT macro.
*/ 

void EnsureFailedLine(LPCSTR AssertDescription, LPCSTR lpszFileName, int nLine)
{
	static TCHAR szTitle[] = _T("Ensure Failed!");
	static TCHAR szMessage[] = _T("%hs, File %hs, Line %d");
	TCHAR sz[4096];

	wsprintf(sz, szMessage, AssertDescription, lpszFileName, nLine);

	const int nCode = ::MessageBox(NULL, sz, szTitle, MB_SYSTEMMODAL | MB_ICONHAND | MB_ABORTRETRYIGNORE);

	switch (nCode)
	{
	case IDABORT:
		//DebugBreak();
		abort();
		break;
	case IDRETRY:
		DebugBreak();
		break;
	case IDIGNORE:
	default:
		break;
	}
}


#if FALSE
/***********************************************************************************************

>	void AfxAssertLineFailed(LPCSTR lpszFileName, int nLine)

	Author:		Alex
	Created:	28/2/94
	Inputs:		lpszFileName:      Filename of file that caused failure
				nLine:             Line number that caused failure
	Outputs:	-
	Returns:	-     
	Purpose:    This is a more robust version than MFC provides.
	Errors:		-
	SeeAlso:	-

***********************************************************************************************/

#if COMPILER_REVISION >= 3
// VC2 Beta
BOOL AFXAPI AfxAssertFailedLine(LPCSTR lpszFileName, int nLine)
#else
// VC1 or Chicago pre-Beta VC2
void AFXAPI AfxAssertFailedLine(LPCSTR lpszFileName, int nLine)
#endif
{
	#ifdef _MAC
	// ignore an incredibly annoying Mac assert which is something to do with rendering
	// the icons on our scroll bars
	if ( (nLine==119) && (lstrcmp(lpszFileName, "winbtn.cpp")==0) )
		return FALSE;
	#endif

	// must NOT use MFC for reporting this
	InternalAssert( "MFC Ensure", lpszFileName, nLine, FALSE );

	#if COMPILER_REVISION >= 3
	return FALSE;
	#endif
}

#endif