/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/winoil/errors.cpp 6     21/04/04 17:24 Gerry $
// errors.cpp : implementation file
//

/*
*/



#include "stdafx.h"

#if FALSE
#include "errors.h"
#include "camelot.h"
#include "strings.h"
#include "tool.h"
#include "oilmods.h"
#include "basestr.h"
#include "alex.h"			// For IDS_SERIOUSERROR
#include "timdlg.h"
#include "tim.h"
#include "andy.h"
#include "simon.h"			// for IDS_HELP, the text on help buttons
#include "ensure.h"
#include "resource.h"
#include "ctrlhelp.h"
#include "errorbox.h"
#include "helpuser.h"
#include "basebar.h"		// For gallery creation error box fix...
#include "justin2.h"
#include "ralphint.h"
#include "ralphdoc.h"
#include "document.h"
#include "ralpherr.h"
#endif


static BOOL ErrorHasBeenReported = TRUE;
static UINT	InSetError = 0;			// In either of the SetError routines
static UINT ErrorBoxRecurse = 0;	// Incremented as per how many recursive error boxes we have

// Used to indicate what kind of error message is currently defined.
typedef enum
{
	ERRORSTAT_NONE,
	ERRORSTAT_TEXT,
	ERRORSTAT_ID
} ErrorStatus;

static ErrorStatus ErrStatus = ERRORSTAT_NONE;


UINT Error::RenderThread = 0;	// We're not in the thread

#if FALSE
/********************************************************************************************

>	ErrorInfo::ErrorInfo()

	Author:		Tim
	Created:	23/05/94
	Purpose:	Initialises the fields of the error info structure to sensible defaults.

********************************************************************************************/

ErrorInfo::ErrorInfo()
{
	ErrorMsg  = 0;
	Title	  = 0;
	Button[0] = IDS_OK;
	Button[1] = 0;
	Button[2] = 0;
	Button[3] = 0;
	OK        = 1;
	Cancel    = 2;
	Help      = 0;				// by default there is no help button
}
#endif


/********************************************************************************************

>	HRESULT Error::ErrIDToHRESULT(UINT ErrID)

	Author:		Chris
	Created:	23/05/94
	Purpose:	Convert our ErrID's to HRESULTS if we're not interested return S_OK

********************************************************************************************/
HRESULT Error::ErrIDToHRESULT(UINT ErrID)
{
#ifdef RALPH
	return GetHRESULTFromID(ErrID);
#endif
	return S_OK;
}


#if FALSE
/********************************************************************************************

>	HRESULT Error::GetRalphError()

	Author:		Chris
	Created:	23/05/94
	Purpose:	ERROR1 - No local reporting -pass the mapped Error code to the harness
				ERROR2 - Report the Error locally - don't pass to handler 
				ERROR3 - Report as ERROR2's - won't occur in retails - 

********************************************************************************************/
HRESULT Error::GetRalphError()
{
	
	UINT Err = Error::GetRalphErrorNumber();
	CString ErrStr( Error::GetErrorString());
	HRESULT hr = S_OK;

	// ERROR2's 3's
	if(Err==0)
	{
		if(!ErrStr.IsEmpty())
		{
#ifdef _DEBUG
			MessageBox(NULL,ErrStr,"Error",MB_OK);
#endif
			// make sure we clear ERROR2's 'cause we report them now
			Error::ClearError();
		}	
	}

	//ERROR1
	if(Err!=0)
	{
		// Do Mapping
		hr = ErrIDToHRESULT(Err);
	
	}
	// ERROR 2's 3's
	if(Err==0)
	{
		if(!ErrStr.IsEmpty())
		   hr = E_FAIL;					// RALPH_E_INTERNAL;
	}
	
	return hr;
}
#endif

#if FALSE
/********************************************************************************************

>	int InformGeneral(UINT Error, UINT ModID, UINT ErrorMsg, 
					  UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4,
					  UINT OK, UINT Cancel)

	Author:		Rik (then Andy)
	Created:	15/6/93
	Inputs:		Error - either ERRORTYPE_NORMAL, 
							   ERRORTYPE_ERROR, 
							   ERRORTYPE_WARNING or 
							   ERRORTYPE_SERIOUS
				ModID - Module ID to obtain messages (0=Kernel).
				ErrorMsg - The Error Message to display. 0 means use static Error
				Butt1-4  - The Text for the buttons. Up to 4 buttons can be specified
						   If Butt1 is 0, it means use 'IDS_OK', i.e. "OK".
				OK - which button (1-4) should be the OK (default) button.
				Cancel - button should be the 'Cancel' button (and hence responds to Esc).
	Returns:	The number of the button used to close the dialog.
	Purpose:	The underlying function for [Tool-Module]Inform[Warning-Error].
	SeeAlso:	InformWarning; InformGeneral; ToolInformWarning; ToolInformError; 
				ModuleInformWarning; ModuleInformError

********************************************************************************************/
#ifdef RALPH
// **RALPH** error reporting

int InformGeneral(UINT Error, UINT modID, UINT ErrorMsg, 
				  UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4,
				  UINT OK, UINT Cancel)
{    
	
	// ** this is the RALPH version of  InformError **
	// it's job is to report the error back to ralph's current harness
	

	// get a pointer to the kernel document
	Document *pkDoc = Document::GetCurrent();
	RalphDocument * pRalphDoc = NULL;
	if(pkDoc==NULL)
		return IDS_OK;
	// get a pointer to it's attached ralphdoc
	pRalphDoc = pkDoc->GetRalphDoc();

	if(pRalphDoc== NULL)
		return IDS_OK;


	if (ErrorMsg != 0)
	{
		// It's a new error message
		Error::SetError(ErrorMsg, modID);
	}

	// Make sure we haven't already reported this error!
/*	ENSURE((!ErrorHasBeenReported), 
		   "Ralph InformGeneral: This error has already been reported!");
*/
	if ( Error::IsInRenderThread() )
		TRACE("Ralph InformGeneral called within RenderThread => serious rendering error");


	ErrStatus = ERRORSTAT_NONE;

	if ( Error::IsInRenderThread() )
	{
		TRACE("In RenderThread so clearing up system");
		Error::RenderThreadReset();
	}
	
	// if we're outside a ralph method
	// pass ERROR1's back to the harness
	// and report other errors immediately
	// and clear the error
	HRESULT hr = Error::GetRalphError();
	if(!pRalphDoc->IsInRalphMethod())
	{
		// get the error mapped to an HRESULT
		if(hr !=S_OK && hr!= RALPH_E_INTERNAL)
		{
			if(pRalphDoc)
			{
				pRalphDoc->PassErrorToHarness(hr);
			}
		}
		// and clear the error
		Error::ClearError();
	}

   	// We don't want to report this error again.
	ErrorHasBeenReported = TRUE;
	ErrorBoxRecurse--;
	return IDS_OK ; //result;
}

#else

// **CAMELOT Error reporting** 
int InformGeneral(UINT Error, UINT modID, UINT ErrorMsg, 
				  UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4,
				  UINT OK, UINT Cancel)
{    
	
	// Make sure there is at least one valid button.
	if (Butt1 == 0) Butt1 = IDS_OK;

	if (ErrorBoxRecurse)
	{
		// Oh dear oh dear. Someone wants to put up an error box in the error handler. This is
		// **BAD** news. The most likely cause is an exception within this routine (or the other
		// InformGeneral).
		TRACE("Recursive InformGeneral - Error in error handler! (1)\n");
		// Don't risk putting up another error box - just return OK & hope for the best
		::MessageBeep(MB_OK); // shouldn't cause too much trouble
		return(OK);
	}

	ErrorBoxRecurse++;
	if (ErrorMsg != 0)
	{
		// It's a new error message
		Error::SetError(ErrorMsg, modID);
	}

	// Make sure we haven't already reported this error!
	ENSURE((!ErrorHasBeenReported), 
		   "InformGeneral: This error has already been reported!");

	// In retail builds, we report the error anyway, just in case - it's better to have
	// two errors reported than none at all!
	CInformErrorDialog	MyErrorBox(NULL);

    // Set the params according to what was supplied
	MyErrorBox.m_StaticTextStr 	= ErrorMsg;
	MyErrorBox.m_ButtonStr[0] 	= Butt1;
	MyErrorBox.m_ButtonStr[1] 	= Butt2;
	MyErrorBox.m_ButtonStr[2] 	= Butt3;
	MyErrorBox.m_ButtonStr[3] 	= Butt4;
	MyErrorBox.m_ErrorBoxType 	= Error;
	MyErrorBox.m_OwnerModule  	= modID;
	MyErrorBox.m_OK 			= OK;
	MyErrorBox.m_Cancel 		= Cancel;

#ifndef EXCLUDE_FROM_RALPH
	// See if there is any on-line help associated with the given warning/error message.
	// If there is then we will add a "Help" button to the dialog.
	if (CanHelpUser(Error::GetErrorNumber()))
	{
		// There is some on-line help for this message.  First, find out how many buttons
		// were specified by the caller (we know they specified the first).
		int nButtons = 1 + (Butt2 != 0) + (Butt3 != 0) + (Butt4 != 0);

		// Do we have room for another button?
		if (nButtons <= 4)
		{
			// Make the next available button the help button.
			MyErrorBox.m_ButtonStr[nButtons] = IDS_HELP;
			MyErrorBox.m_Help = nButtons + 1;
		}
#ifdef _DEBUG
		else
		{
			TRACEUSER("JustinF", "Message 0x%lX - no room for help button\n",
									(ULONG) Error::GetErrorNumber());
		}
#endif
	}
#endif

 	// Disable the system's functionality for serious errors (i.e. stop rendering etc).
 	if (Error == ERRORTYPE_SERIOUS || Error == ERRORTYPE_ENSURE) CCamApp::DisableSystem();
	if ( Error::IsInRenderThread() )
		TRACE("InformGeneral called within RenderThread => serious rendering error");

	// Bodge because ReleaseCapture() sometimes doesn't send WM_CANCELMODE (e.g. to custom controls) which
	// appears to be a bug somewhere in the Windows API
	if (GetCapture()) SendMessage(GetCapture(), WM_CANCELMODE, 0, 0);
	ReleaseCapture();

#ifndef EXCLUDE_FROM_RALPH
	// Keep Control Helper system informed
	ControlHelper::InformModalDialogOpened();

	// Bodge so error boxes are given focus when bars/galleries are being created
	BaseBar::StartErrorBox();
#endif

	// 'Do' the dialog
	int result = MyErrorBox.DoModal();

	ErrStatus = ERRORSTAT_NONE;

#ifndef EXCLUDE_FROM_RALPH
	// Make sure we forget the old help content.
	SetNextMsgHelpContext(0);

	// Keep Control Helper system informed
	ControlHelper::InformModalDialogClosed();

	// Bodge so error boxes are given focus when bars/galleries are being created
	BaseBar::FinishErrorBox();
#endif

	if ( Error::IsInRenderThread() )
	{
		TRACE("In RenderThread so clearing up system");
		Error::RenderThreadReset();
	}

	// Enable system if necessary.
 	if (Error == ERRORTYPE_SERIOUS || Error == ERRORTYPE_ENSURE) CCamApp::EnableSystem();

	// We don't want to report this error again.
	ErrorHasBeenReported = TRUE;
	Error::ClearError();
		
	ErrorBoxRecurse--;
	return result;
}

#endif

/********************************************************************************************

>	int InformGeneral(UINT Error, ErrorInfo *pInfo, UINT ModID)

	Author:		Tim
	Created:	04/07/94
	Inputs:		Error - the type of error (e.g. ERRORTYPE_SERIOUS)
				pInfo - pointer to the ErrorInfo object which desvribes this 'error'.
				ModID - module ID, if required (may be 0).
	Returns:	ID of the button that was pressed (NB. NOT the index of the button - i.e.
				it returns a resource ID).
	Purpose:	The underlying function for [Tool-Module]Inform[Warning-Error].
	SeeAlso:	InformWarning; InformGeneral; ToolInformWarning; ToolInformError; 
				ModuleInformWarning; ModuleInformError

********************************************************************************************/
#ifdef RALPH
int InformGeneral(UINT Error, ErrorInfo* pInfo, UINT ModID)
{
	if (!pInfo) return IDS_OK; // Catch this early
	return InformGeneral(Error,ModID, pInfo->ErrorMsg, 
				  pInfo->Button[1],pInfo->Button[2],pInfo->Button[3],pInfo->Button[4],
				  pInfo->OK,pInfo->Cancel);

	
}


#else
int InformGeneral(UINT Error, ErrorInfo* pInfo, UINT ModID)
{
	if (!pInfo) return IDS_OK; // Catch this early

	// Make sure there is at least one valid button.
	if (pInfo->Button[0] == 0) pInfo->Button[0] = IDS_OK;

	if (ErrorBoxRecurse)
	{
		// Oh dear oh dear. Someone wants to put up an error box in the error handler. This is
		// **BAD** news. The most likely cause is an exception within this routine (or the other
		// InformGeneral).
		TRACE("Recursive InformGeneral - Error in error handler! (2)\n");

		// Don't risk putting up another error box - just return OK & hope for the best
		::MessageBeep(MB_OK); // shouldn't cause too much trouble
		return(pInfo->OK);
	}

	ErrorBoxRecurse++;
	if (pInfo->ErrorMsg != 0)
	{
		// It's a new error message
		Error::SetError(pInfo->ErrorMsg, ModID);
	}

	// Make sure we haven't already reported this error!
	ENSURE((!ErrorHasBeenReported), "InformGeneral: This error has already been reported!");

	// In retail builds, we report the error anyway, just in case - it's better to have
	// two errors reported than none at all!

	CInformErrorDialog	MyErrorBox(NULL);
    
    // Set the params according to what was supplied
	MyErrorBox.m_StaticTextStr 	= pInfo->ErrorMsg;
	MyErrorBox.m_TitleStr		= pInfo->Title;
	MyErrorBox.m_ButtonStr[0] 	= pInfo->Button[0];
	MyErrorBox.m_ButtonStr[1] 	= pInfo->Button[1];
	MyErrorBox.m_ButtonStr[2] 	= pInfo->Button[2];
	MyErrorBox.m_ButtonStr[3] 	= pInfo->Button[3];
	MyErrorBox.m_ErrorBoxType 	= Error;
	MyErrorBox.m_OwnerModule  	= ModID;
	MyErrorBox.m_OK 			= pInfo->OK;
	MyErrorBox.m_Cancel 		= pInfo->Cancel;

#ifndef EXCLUDE_FROM_RALPH
	// See if there is any on-line help associated with the given warning/error message.
	// If there is then we will add a "Help" button to the dialog.
	if (CanHelpUser(Error::GetErrorNumber()))
	{
		// There is some on-line help for this message.  First, find out how many buttons
		// were specified by the caller (we know they specified the first).
		int nButtons = 1 + (pInfo->Button[1] != 0) +
						   (pInfo->Button[2] != 0) +
						   (pInfo->Button[3] != 0);

		// Do we have room for another button?
		if (nButtons <= 4)
		{
			// Make the next available button the help button.
			MyErrorBox.m_ButtonStr[nButtons] = IDS_HELP;
			MyErrorBox.m_Help = nButtons + 1;
		}
#ifdef _DEBUG
		else
		{
			TRACEUSER("JustinF", "Message 0x%lX - no room for help button\n",
									(ULONG) Error::GetErrorNumber());
		}
#endif
	}
#endif

 	// Disable the system's functionality for serious errors (i.e. stop rendering etc).
 	if (Error == ERRORTYPE_SERIOUS || Error == ERRORTYPE_ENSURE) CCamApp::DisableSystem();
														// BODGE TO TEST WARNING BOXES ON LOADING 
	if ( Error::IsInRenderThread() )
		TRACE("InformGeneral called within RenderThread => serious rendering error");

	// Bodge because ReleaseCapture() sometimes doesn't send WM_CANCELMODE (e.g. to custom
	// controls) which appears to be a bug somewhere in the Windows API.
	if (GetCapture()) SendMessage(GetCapture(), WM_CANCELMODE, 0, 0);
	ReleaseCapture();	

#ifndef EXCLUDE_FROM_RALPH
	// Keep Control Helper system informed
	ControlHelper::InformModalDialogOpened();
#endif

	// 'Do' the dialog
	int result = MyErrorBox.DoModal();
	ErrStatus = ERRORSTAT_NONE;

#ifndef EXCLUDE_FROM_RALPH
	// Make sure we forget the old help content.
	SetNextMsgHelpContext(0);

	// Keep Control Helper system informed
	ControlHelper::InformModalDialogClosed();
#endif

	if ( Error::IsInRenderThread() )
	{
		TRACE("In RenderThread so clearing up system");
		Error::RenderThreadReset();
	}

	// Enable system if necessary.
 	if (Error == ERRORTYPE_SERIOUS || Error == ERRORTYPE_ENSURE) CCamApp::EnableSystem();

	// We don't want to report this error again.
	ErrorHasBeenReported = TRUE;
	Error::ClearError();
		
	// Convert the result into an ID which is a bit more understandable.  Note that even
	// though have changed the dialog code to allow up to FIVE buttons in a dialog (the
	// extra one means there is always room for a help button), the fifth button, which
	// could only be help, never closes the dialog, so clicking it should never get you here.
	ENSURE((result >= 1) && (result <= 4), "Bad return code from error dialog");
	ErrorBoxRecurse--;
	if ((result >= 1) && (result <= 4))
	{
		return pInfo->Button[result-1];
	}
	else
	{
		// Disaster - simulate cancel event
		return pInfo->Button[pInfo->Cancel];
	}
}

#endif

/********************************************************************************************

>	void InformLastError(UINT ErrorType = ERRORTYPE_ERROR)

	Author:		Andy
	Created:	24/8/93
	Inputs:		ErrorType is the error type (e.g. ERRORTYPE_WARNING) Defaults to ERRORTYPE_ERROR
				THIS PARAM IS NOW IGNORED.
	Outputs:	None
	Returns:	None
	Purpose:	Report the last error that was set with ERROR, ERRORIF, TOOLERROR et al.
				Always presents the user with an OK button. If there isn't an unreported
				error, does nothing.
	SeeAlso:	ERROR; ERRORIF; TOOLERROR

********************************************************************************************/

void InformLastError(UINT)
{
	if (ErrStatus != ERRORSTAT_NONE)
	{
		InformError();
		ErrStatus = ERRORSTAT_NONE;
	}
}



/********************************************************************************************

>	int InformError(UINT ErrorMsg = 0, 
					UINT Butt1 = 0, UINT Butt2 = 0, UINT Butt3 = 0, UINT Butt4 = 0, 
					UINT OK = 1, UINT Cancel = 2)

	int ToolInformError(UINT ToolID, <as above>)

	int	ModuleInformError(UINT ModuleID, <as above>)


	int InformSeriousError(UINT ErrorMsg = 0, 
						   UINT Butt1 = 0, UINT Butt2 = 0, UINT Butt3 = 0, UINT Butt4 = 0,
						   UINT OK = 1, UINT Cancel = 2)

	int ToolInformSeriousError(UINT ToolID, <as above>)

	int	ModuleInformSeriousError(UINT ModuleID, <as above>)


	int InformMessage(UINT ErrorMsg = 0, 
					  UINT Butt1 = 0, UINT Butt2 = 0, UINT Butt3 = 0, UINT Butt4 = 0,
					  UINT OK = 1, UINT Cancel = 2)

	int ToolInformMessage(UINT ToolID, <as above>)

	int	ModuleInformMessage(UINT ModuleID, <as above>)


	Author:		Rik, then Alex
	Created:	15/6/93
	Inputs:		ErrorMsg - The Error Message to display. 0 means use static Error
				Butt1-4  - The Text for the buttons. Up to 4 buttons can be specified
						   If Butt1 is 0, it means use 'IDS_OK', i.e. "OK".
				OK - which button (1-4) should be the OK (default) button.
				Cancel - button should be the 'Cancel' button (and hence responds to Esc).
	Returns:	The number of the button used to close the dialog
	Purpose:	To display a dialog that gives an error message of some description. Up
				to 3 buttons can be defined for the user to select. It should be used
				when an error occurs that the user could make a decision about eg. Camelot
				failed to save the file, so give them the option to give up, try again or
				try a new filename/path etc.
				If ErrorMsg is zero, the box will use the last error returned by a function 
				with the RETURNERROR macro.
				ToolInformError takes an additional parameter on the front of the Tool ID.
				ModuleInformError takes a module ID. Both of these will try to find the strings
				used for errors and buttons in the owner module first, then the main .exe file.
				These are all in-line function stubs which call InformGeneral.
				The equivalent routines with 'Serious' in their name are the equivalent of those
				without except that they pass ERRORTYPE_SERIOUS and not ERRORTYPE_ERROR
				It's also worth noting tat these are just inline functions that call InformGeneral
				and they live solely in Errors.h
	SeeAlso:	InformWarning; RETURNERROR; InformGeneral

********************************************************************************************/





/********************************************************************************************

>	int InformWarning( UINT ErrorMsg, UINT Butt1=0, UINT Butt2=0, UINT Butt3=0, UINT Butt4=0 )

	Author:		Rik
	Created:	15/6/93
	Inputs:		ErrorMsg - The Error Message to display - 0 means use static Error
				Butt1-3  - The Text for the buttons. Up to 3 buttons can be specified
	Returns:	The number of the button used to close the dialog
	Purpose:	To display a dialog that gives a warning message of some description. Up
				to 3 buttons can be defined for the user to select. If ErrorMsg is zero
				the error string will be taken from the static Error variables, bringing 
				up the last error that was set by a function calling RETURNERROR.
	SeeAlso:	InformError

********************************************************************************************/
#endif

// Define the instances of the static variables belonging to the Error class

TCHAR Error::ErrorString[256];
UINT  Error::ErrorID = 0;
UINT  Error::RalphErrorID = 0;
UINT  Error::ModuleID = 0;
UINT  Error::LastErrorLine;							// line number of last error (or 0)
const char* Error::LastErrorFile = "Unknown.File";	// ptr to filename of last error - don't translate


/////////////////////////////////////////////////////////////////////////////
// Error handling routines

/***********************************************************************************************

>	void Error::SetError(UINT number, TCHAR* errstring, UINT module)
	void Error::SetError(UINT number, UINT module)
	void Error::SetErrorTool(UINT number, UINT tool)
	void Error::SetErrorSerious( const TCHAR* errstring )

	Author:		Jim
	Created:	9/7/93
	Inputs:		number is the error number - usually a resource ID
				errstring pointer to a TCHAR array containing the string for the error
				module is a UINT uniquely identifying the module (0 if kernel)
				tool is a tool ID. The SetErrorSerious call is for serious error handlers
				who must succeed in setting the error regardless.
	Outputs:	-
	Returns:	-
	Purpose:	Called before exiting a routine that failed, this function sets up the static 
				variables which allow the calling routine to see what the error was.
				If the errstring parameter is passed, the module UINT is stored, but not used. This
				form of the function would usually be used for routines which want to build up an 
				error string on the fly. If the errstring is absent, the routine uses the UINT to
				identify the module, and loads a string from the appropriate resources.
	Errors:		-
	SeeAlso:	SmartLoadString

***********************************************************************************************/

void Error::SetError(UINT number, const TCHAR* errstring, UINT module)
{
	static TCHAR LastResort[] = _T("Cannot perform SetError (1)");
	// We now check for a real recursive call. Should only happen if we can't get translate
	// an ID to an error string. Perhaps if we're really low on memory. We must then set the
	// minimum acceptable error string...
	if (InSetError)
	{
		TRACE(_T("SetError really has been called recursively (1)\n"));
		RalphErrorID =ErrorID = IDE_EX_BADOP; // should have it's own, but...
		ErrStatus = ERRORSTAT_TEXT;
		ErrorHasBeenReported=FALSE;
		ModuleID = module;
		lstrcpy(ErrorString, LastResort);
		return;
	}
	InSetError++;

// Andy has stubbed this code out as it caused problems, e.g.
// 'save changes' dialog after a GP fault had been caught made the dialog have the 'save
// changes' buttons, with the GP fault text
#if 0
	if (!ErrorHasBeenReported)
	{
		// ENSURE(FALSE, "Recursive SetError call"); IMHO this is pointless - Alex
		TRACE("SetError called twice: ID = %u: %s\n", number, errstring);
		InSetError--;
		return;
	}
#endif

	RalphErrorID = ErrorID = number;
	if (errstring)
		lstrcpy(ErrorString, errstring);
	else
		ErrorString[0] = 0;
	ModuleID = module;
	ErrStatus = ERRORSTAT_TEXT;
	ErrorHasBeenReported = FALSE;
	TRACE("Setting error: ID = %u: %s\n", ErrorID, ErrorString);
	InSetError--;
}                          

void Error::SetErrorSerious( const TCHAR* errstring )
{
	// this must succeed. In particular it must NOT cause any other errors or ENSUREs
	ErrorID = 0; 
	ModuleID = 0;
	InSetError = 0;
	ErrStatus = ERRORSTAT_TEXT;
	ErrorHasBeenReported = FALSE;
	lstrcpy( ErrorString, errstring );
}

void Error::SetError(UINT number, UINT module)
{
	static TCHAR LastResort[] = _T("Cannot perform SetError (2)");
	// We now check for a real recursive call. Should only happen if we can't get translate
	// an ID to an error string. Perhaps if we're really low on memory. We must then set the
	// minimum acceptable error string...
	if (InSetError)
	{
		TRACE(_T("SetError really has been called recursively (2)\n"));
		ErrorID = IDE_EX_BADOP; // should have it's own, but...
		ErrStatus = ERRORSTAT_TEXT;
		ErrorHasBeenReported=FALSE;
		ModuleID = module;
		lstrcpy(ErrorString, LastResort);
		return;
	}
	InSetError++;
	if (!ErrorHasBeenReported)
	{
		// ENSURE(FALSE, "Recursive SetError call"); IMHO this is pointless - Alex
		TRACE("SetError called twice: ID = %u  Module = %u\n", number, module);
		InSetError--;
		return;
	}
	RalphErrorID = ErrorID = number;
	ModuleID = module;
	ErrorString[0] = 0;
	ErrStatus = ERRORSTAT_ID;
	ErrorHasBeenReported = FALSE;
//	if (!SmartLoadString(module, ErrorID, ErrorString, 256))
//		wsprintf(ErrorString, "Error Number %u from module ID %u", ErrorID, ModuleID);
//	TRACE("Setting error: ID = %d: \"%s\"\n", ErrorID, ErrorString);
	InSetError--;
}                          



void Error::SetErrorTool(UINT number, UINT toolID)
{
//	Error::SetError(number, Tool::GetModuleID(toolID));
}



/********************************************************************************************

>	static void Error::ClearError()

	Author:		JustinF
	Created:	8/12/93
	Inputs:		-
	Outputs:	-
	Returns:	-
	Purpose:	Resets the error system to "no current error".
	Errors:		-
	SeeAlso:	Error::SetError

********************************************************************************************/

void Error::ClearError()
{
	TRACEUSER("Chris","Clear Error now \n");
	RalphErrorID = ErrorID = ModuleID = 0;
	ErrorString[0] = 0;

	// Added 15/2/95 so that the function does as it name suggests and actually does clear
	// the current error completely from the system rather than just clearing out the error
	// id and the error string.
	ErrorHasBeenReported = TRUE;
	ErrStatus = ERRORSTAT_NONE;
}



/********************************************************************************************

>	void Error::MarkError( UINT LineNumber, const char *Filename )

	Author:		Andy
	Created:	4/10/94
	Inputs:		Line number and filename, usually generated with __LINE__ and __FILE__
	Purpose:	Used to 'remember' where an error ocurred. Cannot be done with extra params
				to InternalSetError due to vararg strangeness. The function is declared inline
				on retail builds.
	Scope:		Public Static

********************************************************************************************/

/* Tech Note: The MARKERRORBODY macro lives in errors.h. The same macro is used for the inline
	version and out of line version - if you need to change the function body, edit the macro
*/

#if !INLINE_MARKERROR

void Error::MarkError( UINT LineNumber, const char *Filename )
MARKERROR_BODY( LineNumber, Filename )

#endif

#define	ResetWhere()		LastErrorLine = 0; LastErrorFile = "Unknown.File";


// \n's needed on the end of these

#ifdef _DEBUG
void CDECL Error::TraceUser(const char *User, LPCSTR fmt, ...)
{
	if (!IsUserName(User)) return;

	char buf[256];
	va_list marker;
	va_start( marker, fmt );
	vsprintf( buf, fmt, marker );
	va_end( marker );
	TRACE( buf );
}

#ifdef _UNICODE
void CDECL Error::TraceUser(const char *User, LPCTSTR fmt, ...)
{
	if (!IsUserName(User)) return;

	TCHAR buf[256];
	va_list marker;
	va_start( marker, fmt );
	_vstprintf( buf, fmt, marker );
	va_end( marker );
	TRACE( buf );
}
#endif
#endif


void CDECL Error::ReleaseTrace(LPCTSTR fmt, ...)
{
	TCHAR buf[256];
	va_list marker;
	va_start( marker, fmt );
	_vstprintf( buf, fmt, marker );
	va_end( marker );
	OutputDebugString( buf );
}



/********************************************************************************************

>	void CalcInternalMessage( LPTCHAR result, UINT Line, const TCHAR* Filename )

	Author:		Andy
	Created:	4/10/94
	Inputs:		Line number and filename, usually generated with __LINE__ and __FILE__
	Outputs:	result is updated with a suitable string
	Purpose:	Used to translate something such as 1234 in kernel\blobby.cpp into something
				less readable (in fact 1234@k.bl.by). This is so source & line numbers can be
				reported to an ordinary user without giving too much away.

********************************************************************************************/
#define IDS_INTERNAL_ERROR_MSG  "Internal program error %u.%hs"
static void CalcInternalMessage( LPTCHAR result, UINT Line, const char* Filename )
{
	const char* Slash    = strrchr( Filename, '\\' );
	const char* Dot      = strrchr( Filename, '.'  );
	char CodedFile[20];

	if (Slash && Dot)
	{
		char* p = CodedFile;

		*p++ = Filename[0];					// 1st char is first letter of dirname
		*p++ = TEXT('.');					// seperators
		*p++ = Slash[1];					// 1st 2 chars of filename
		*p++ = Slash[2];
		*p++ = TEXT('.');
		*p++ = Dot[-2];						// last 2 chars of filename
		*p++ = Dot[-1];
		*p = 0;

		Filename = CodedFile; 
	}

	// this message should not be translated
	String_256 jcf(IDS_INTERNAL_ERROR_MSG);
	wsprintf(result, jcf, (UINT) Line, (LPCTSTR) Filename);
}



/********************************************************************************************

>	void CDECL Error::XSetError( const char *fmt, ...)

	Author:		Andy
	Created:	4/10/94
	Inputs:		printf-style arguments.
	Purpose:	Do NOT call directly. Called via the ERROR2xx macros. In debug builds, puts
				up an ensure box. In all builds, sets an error. Do NOT put \n's or \r's on the
				end.
	Scope:		Public Static

********************************************************************************************/
#ifndef RALPH
// ** normal XSetError **
void CDECL Error::XSetError( const char *fmt, ...)
{
#ifdef _DEBUG
	char buf [256];

	va_list marker;
	va_start( marker, fmt );
	vsprintf( buf, fmt, marker );
	va_end( marker );
	// in debug builds we put up an ensure box
	EnsureFailedLine( buf, LastErrorFile, LastErrorLine );		// put up box

#endif

	XSetErrorC();
	return;
}
#else
// ** RALPH XSetError() **
void CDECL Error::XSetError( const char *fmt, ...)
{
	char buf[256];
	TRACEUSER("Chris","oOoOo Ralph Set Error \n");

#ifdef _DEBUG

	va_list marker;
	va_start( marker, fmt );
	_vstprintf( buf, fmt, marker );
	va_end( marker );
	// in debug builds we put up an ensure box
	EnsureFailedLine( buf, LastErrorFile, LastErrorLine );		// put up box

#else
	XSetErrorC();
	InformError();
#endif
	return;
}

#endif

/********************************************************************************************

>	void CDECL Error::XSetErrorC()

	Author:		Alex
	Created:	30/5/95
	Inputs:		None
	Purpose:	Do NOT call directly. Sets up the internal error
	Scope:		Public Static

********************************************************************************************/

void CDECL Error::XSetErrorC()
{
	TCHAR buf[256];
	// the error we set features a coded version of where the problem was
	CalcInternalMessage( buf, LastErrorLine, LastErrorFile );
	SetErrorSerious( buf );
	ResetWhere();
	return;
}

/********************************************************************************************

>	void CDECL Error::XSetError( UINT errID, ...)

	Author:		Andy
	Created:	4/10/94
	Inputs:		An error ID, followed by _MakeMsg-style variable arguments
	Purpose:	Do NOT call directly. Called via the ERROR1xx macros.
	Scope:		Public Static

********************************************************************************************/

void CDECL Error::XSetError( UINT errID, ...)
{


	if ( (errID==FALSE) || (errID==TRUE) )
	{
		// someone probably used the wrong macro parameters e.g. TRUE and FALSE instead of ID
		// This call will set an IDE_INTERNAL_ERROR for us
		ERROR2RAW( "ERROR1 macro used with invalid parameters" );
		return;
	}

	TCHAR buf[256];

	va_list marker;

	va_start( marker, errID );

	String_256 result;

	// load the format string as a resoure (note no module ID yet)
//	if (!SmartLoadString(0, errID, buf, sizeof(buf)))
		wsprintf(buf, _T("Error<%u>"), errID); // keep inline


	// now do _MakeMsg type formatting
	result.CCvsprintf(buf, marker);

#ifndef EXCLUDE_FROM_RALPH
	// Set the help context.
//	SetNextMsgHelpContext(errID);
#endif
	// ralph needs this so that he can map the ID to a HRESULT before passing it
	// back to a harness
	TRACEUSER("Chris","oOoOo Ralph Set Error %d \n",RalphErrorID);

	RalphErrorID =errID;

	// and copy result into ErrorString
	SetErrorSerious( result );

	// trace output because SetErrorSerious doesn't bother
	TRACE("Setting error: ID = %d: \"%s\"\n", errID, ErrorString);

	// then tidy up	
	va_end( marker );

	ResetWhere();
}

/********************************************************************************************

>	void CDECL Error::XComplain( const char *fmt, ...)

	Author:		Andy
	Created:	4/10/94
	Inputs:		printf-style arg list.
	Purpose:	Do NOT call directly. This func is used by the ERROR3xx macros. It does the
				_stprintf-thing, then puts up an ensure-type dialog box. It exists only in
				debug builds. Error:MarkError should be called beforehand.
	Scope:		Public Static

********************************************************************************************/

#ifdef _DEBUG
void CDECL Error::XComplain( const char *fmt, ...)
{
	char buf[256];

	va_list marker;
	va_start( marker, fmt );
	vsprintf_s( buf, fmt, marker );								// convert the args
	va_end( marker );

	EnsureFailedLine( buf, LastErrorFile, LastErrorLine );		// put up box
	ResetWhere();
}

void CDECL Error::XComplain( const wchar_t *fmt, ...)
{
	USES_CONVERSION;

	wchar_t buf[256];

	va_list marker;
	va_start( marker, fmt );
	vswprintf_s( buf, fmt, marker );								// convert the args
	va_end( marker );

	EnsureFailedLine( W2A(buf), LastErrorFile, LastErrorLine );		// put up box
	ResetWhere();
}
#endif

int InformError()
{
	TCHAR* pszError = Error::GetErrorString();
	if(pszError && *pszError)
		Error::XComplain(_T("%s"), pszError);
	else
		Error::XComplain(_T("Error %d"), Error::GetErrorNumber());

	Error::ClearError();
	return 0;
}

#if FALSE
int InformError(UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK,	UINT Cancel)
{
	return InformGeneral(ERRORTYPE_ERROR, 0, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4,
						 OK, Cancel);
}


int ToolInformError(UINT ToolID, UINT ErrorMsg,	UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4,	UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_ERROR, Tool::GetModuleID(ToolID), ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ModuleInformError(UINT ModID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_ERROR, ModID, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int InformError(ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_ERROR, pInfo);
}


int ToolInformError(UINT ToolID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_ERROR, pInfo, Tool::GetModuleID(ToolID));
}


int ModuleInformError(UINT ModID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_ERROR, pInfo, ModID);
}


int InformSeriousError(UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_SERIOUS, 0, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ToolInformSeriousError(UINT ToolID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_SERIOUS, Tool::GetModuleID(ToolID), ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ModuleInformSeriousError(UINT ModID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_SERIOUS, ModID, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int InformSeriousError(ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_SERIOUS, pInfo);
}

int ToolInformSeriousError(UINT ToolID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_SERIOUS, pInfo, Tool::GetModuleID(ToolID));
}

int ModuleInformSeriousError(UINT ModID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_SERIOUS, pInfo, ModID);
}



int InformWarning(UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_WARNING, 0, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}



int ToolInformWarning(UINT ToolID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_WARNING, Tool::GetModuleID(ToolID), ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}



int ModuleInformWarning(UINT ModID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_WARNING, ModID, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}

int InformWarning(ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_WARNING, pInfo);
}


int ToolInformWarning(UINT ToolID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_WARNING, pInfo, Tool::GetModuleID(ToolID));
}


int ModuleInformWarning(UINT ModID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_WARNING, pInfo, ModID);
}



int AskQuestion(UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_QUESTION, 0, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ToolAskQuestion(UINT ToolID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_QUESTION, Tool::GetModuleID(ToolID), ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}



int ModuleAskQuestion(UINT ModID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_QUESTION, ModID, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int AskQuestion(ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_QUESTION, pInfo);
}


int ToolAskQuestion(UINT ToolID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_QUESTION, pInfo, Tool::GetModuleID(ToolID));
}


int ModuleAskQuestion(UINT ModID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_QUESTION, pInfo, ModID);
}



int InformMessage(UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_NORMAL, 0, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ToolInformMessage(UINT ToolID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_NORMAL, Tool::GetModuleID(ToolID), ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int ModuleInformMessage(UINT ModID, UINT ErrorMsg, UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4, UINT OK, UINT Cancel)
{
	return InformGeneral(ERRORTYPE_NORMAL, ModID, ErrorMsg, 
						 Butt1, Butt2, Butt3, Butt4, OK, Cancel);
}


int InformMessage(ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_NORMAL, pInfo);
}


int ToolInformMessage(UINT ToolID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_NORMAL, pInfo, Tool::GetModuleID(ToolID));
}


int ModuleInformMessage(UINT ModID, ErrorInfo *pInfo)
{
	return InformGeneral(ERRORTYPE_NORMAL, pInfo, ModID);
}
#endif


#if 0
// old errors test code

int test1(int which)
{
	if (which==0)
	{
		ERROR1RAW( IDE_INTERNAL_ERROR );
		return 0;
	}
	if (which==1)
		ERROR1( 10, IDE_INTERNAL_ERROR );
	if (which==2)
		ERROR1IF( TRUE, 20, IDE_INTERNAL_ERROR );
	if (which==3)
		ERROR1_MSG( 30, (IDT_EPS_BADSYNTAX, 2, 200) );
	if (which==4)
		ERROR1IF_MSG( TRUE, 40, (IDT_EPS_BADSYNTAX, 1, 100) );
	return 0;
}



int test2(int which)
{
	if (which==0)
	{
		ERROR2RAW( "Simple raw error" );
		return 0;
	}
	if (which==1)
		ERROR2( 10, "Simple error" );
	if (which==2)
		ERROR2IF( TRUE, 20, "Simple error if");
	if (which==3)
		ERROR2_PF( 30, ("String (%s)", "blobby") );
	if (which==4)
		ERROR2IF_PF( TRUE, 40, ("Numbers (%d,%d)" , 42,43) );
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void TestErrorStuff()
{
	test1(0);
	test1(1);
	test1(2);
	test1(3);
	test1(4);

	InformError();

	test2(0);
	test2(1);
	test2(2);
	test2(3);
	test2(4);

	InformError();

	ERROR3("simple");
	ERROR3IF( TRUE, "simple" );
	ERROR3_PF( ("doc is %lx", 0x123456) );
	ERROR3IF_PF( TRUE, ("doc %s is %lx", "blobdoc", 0x123456) );

	TRACEUSER("Andy", "simple trace\n");
	TRACEUSER("Andy", "complex %d %s trace", 42, "blobby\n" );
}

#endif
