/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /Camelot/winoil/errors.h 2     21/04/04 17:24 Gerry $
// inform.h : header file
//


/*
*/


#ifndef INC_ERRORS
#define INC_ERRORS

//#include "errordlg.h"

// The various error types below are used to pass to InformGeneral by the inline functions InformError,
// InformWarning etc. They are also stored inside the class CInformErrorDialogue
#define	ERRORTYPE_NORMAL	0
#define	ERRORTYPE_QUESTION	1
#define	ERRORTYPE_WARNING	2
#define	ERRORTYPE_ERROR		3
#define	ERRORTYPE_SERIOUS	4
#define	ERRORTYPE_ENSURE	5

int InformError();

#if FALSE

/********************************************************************************************

>	class ErrorInfo

	Author:		Tim
	Created:	04/07/94
	Purpose:	Describe the error/warning/question that is to be displayed.
				Fields are: 
				MonoOn
				ErrorMsg - Resource ID of the error message.
				Title	 - resource ID of the dialogue title string (optional)
				Button   - array of up to 4 button string resource IDs.
				OK       - the button which is the 'OK' or default button, i.e. the one which 
				 	 	   should be activated by Enter (this in the range 1-4).
				Cancel   - the button which is the 'Cancel' button, i.e. the one which should
						   be activated by Esc (this in the range 1-4).
				Help     - the index of the button which is the 'Help' button (this in the 
					       range 1-4).
				MonoOff
	SeeAlso:	InformGeneral

********************************************************************************************/

class ErrorInfo
{
public:
	ErrorInfo();

	UINT ErrorMsg;
	UINT Title;
	UINT Button[4];
	UINT OK;
	UINT Cancel;
	UINT Help;
};

// The function used throughout the program should be moved to its own .h file really
int CCAPI InformGeneral(UINT Error, 
						UINT modID, UINT ErrorMsg,
						UINT Butt1, UINT Butt2, UINT Butt3, UINT Butt4,
						UINT OK, UINT Cancel);

int CCAPI InformGeneral(UINT Error, ErrorInfo *pInfo, UINT ModID = 0);


void CCAPI InformLastError(UINT Error = ERRORTYPE_ERROR);


// These functions are no longer inline as this just caused huge dependancy problems
// Errors first:
int InformError(UINT ErrorMsg = 0, 
					   UINT Butt1  = 0, 
					   UINT Butt2  = 0, 
					   UINT Butt3  = 0, 
					   UINT Butt4  = 0,
					   UINT OK     = 1,
					   UINT Cancel = 2);

int ToolInformError(UINT ToolID, 
						   UINT ErrorMsg = 0,
						   UINT Butt1  = 0, 
						   UINT Butt2  = 0, 
						   UINT Butt3  = 0, 
						   UINT Butt4  = 0,
						   UINT OK     = 1,
						   UINT Cancel = 2);


int ModuleInformError(UINT ModID, 
							 UINT ErrorMsg = 0,
							 UINT Butt1  = 0, 
							 UINT Butt2  = 0, 
							 UINT Butt3  = 0, 
							 UINT Butt4  = 0,
							 UINT OK     = 1,
							 UINT Cancel = 2);

int InformError(ErrorInfo *pInfo);
int ToolInformError(UINT ToolID, ErrorInfo *pInfo);
int ModuleInformError(UINT ModID, ErrorInfo *pInfo);

// now serious errors
int InformSeriousError(UINT ErrorMsg = 0, 
							  UINT Butt1  = 0, 
							  UINT Butt2  = 0, 
							  UINT Butt3  = 0, 
							  UINT Butt4  = 0,
						   	  UINT OK     = 1,
						      UINT Cancel = 2);

int ToolInformSeriousError(UINT ToolID, 
								  UINT ErrorMsg = 0,
						   		  UINT Butt1  = 0, 
						   		  UINT Butt2  = 0, 
						   		  UINT Butt3  = 0, 
						   		  UINT Butt4  = 0,
								  UINT OK     = 1,
								  UINT Cancel = 2);

int ModuleInformSeriousError(UINT ModID, 
									UINT ErrorMsg = 0,
							 		UINT Butt1  = 0, 
							 		UINT Butt2  = 0, 
							 		UINT Butt3  = 0, 
							 		UINT Butt4  = 0,
								    UINT OK     = 1,
								    UINT Cancel = 2);

int InformSeriousError(ErrorInfo *pInfo);
int ToolInformSeriousError(UINT ToolID, ErrorInfo *pInfo);
int ModuleInformSeriousError(UINT ModID, ErrorInfo *pInfo);

// now warnings:
int InformWarning(UINT ErrorMsg = 0, 
						 UINT Butt1  = 0, 
						 UINT Butt2  = 0, 
						 UINT Butt3  = 0, 
						 UINT Butt4  = 0,
						 UINT OK     = 1,
						 UINT Cancel = 2);


int ToolInformWarning(UINT ToolID, 
							 UINT ErrorMsg = 0,
							 UINT Butt1  = 0, 
							 UINT Butt2  = 0, 
							 UINT Butt3  = 0, 
							 UINT Butt4  = 0,
							 UINT OK     = 1,
						     UINT Cancel = 2);


int ModuleInformWarning(UINT ModID, 
							   UINT ErrorMsg = 0,
							   UINT Butt1  = 0, 
							   UINT Butt2  = 0, 
							   UINT Butt3  = 0, 
							   UINT Butt4  = 0,
						   	   UINT OK     = 1,
						       UINT Cancel = 2);

int InformWarning(ErrorInfo *pInfo);
int ToolInformWarning(UINT ToolID, ErrorInfo *pInfo);
int ModuleInformWarning(UINT ModID, ErrorInfo *pInfo);

// questions:
int AskQuestion(UINT ErrorMsg = 0, 
					   UINT Butt1  = 0, 
					   UINT Butt2  = 0, 
					   UINT Butt3  = 0, 
					   UINT Butt4  = 0,
					   UINT OK     = 1,
					   UINT Cancel = 2);


int ToolAskQuestion(UINT ToolID, 
						   UINT ErrorMsg = 0,
						   UINT Butt1  = 0, 
						   UINT Butt2  = 0, 
						   UINT Butt3  = 0, 
						   UINT Butt4  = 0,
						   UINT OK     = 1,
						   UINT Cancel = 2);


int ModuleAskQuestion(UINT ModID, 
							 UINT ErrorMsg = 0,
							 UINT Butt1  = 0, 
							 UINT Butt2  = 0, 
							 UINT Butt3  = 0, 
							 UINT Butt4  = 0,
							 UINT OK     = 1,
							 UINT Cancel = 2);

int AskQuestion(ErrorInfo *pInfo);
int ToolAskQuestion(UINT ToolID, ErrorInfo *pInfo);
int ModuleAskQuestion(UINT ModID, ErrorInfo *pInfo);


// now just messages:
int InformMessage(UINT ErrorMsg = 0, 
						 UINT Butt1  = 0, 
						 UINT Butt2  = 0, 
						 UINT Butt3  = 0, 
						 UINT Butt4  = 0,
						 UINT OK     = 1,
						 UINT Cancel = 2);


int ToolInformMessage(UINT ToolID, 
							 UINT ErrorMsg = 0,
							 UINT Butt1  = 0, 
							 UINT Butt2  = 0, 
							 UINT Butt3  = 0, 
							 UINT Butt4  = 0,
							 UINT OK     = 1,
							 UINT Cancel = 2);

int ModuleInformMessage(UINT ModID, 
							   UINT ErrorMsg = 0,
							   UINT Butt1  = 0, 
							   UINT Butt2  = 0, 
							   UINT Butt3  = 0, 
							   UINT Butt4  = 0,
							   UINT OK     = 1,
							   UINT Cancel = 2);

int InformMessage(ErrorInfo *pInfo);
int ToolInformMessage(UINT ToolID, ErrorInfo *pInfo);
int ModuleInformMessage(UINT ModID, ErrorInfo *pInfo);

#endif

/***********************************************************************************************

>	macro ERROR(UINT ErrID, retvalue)

	Author:		Jim
	Created:	9/7/93
	Purpose:	This macro should be used to return from a function when an error condition
				has been detected. The rule in Camelot is that every function which might 
				fail should return a BOOL indicating success or failure. Thus, retvalue will
				usually be FALSE. However, functions which return pointers can indicate error
				conditions by returning NULL, so the retvalue parameter is there to give extra
				functionality. Obviously, TRUE or FALSE (or NULL) tells us very little about why
				a function failed, so we have a class containing static variables which must be
				set each time a function fails. A calling routine can look at the error value to
				see what the error was, or use the string to present the error in an error box.
				The macro has two incarnations - within module code in a separate DLL it passes a
				Module ID, which allows the string finding functions to look in another resource, 
				while within kernel code, it just passes the error ID.
				It looks at the symbol MODULE_ID to see if it is being defined within a module,
				and defines the appropriate behaviour. MODULE_ID should be defined as a unique
				module identifier in all code running in a module. It should not be defined in any
				other code.
				Warning: Windows.h defines ERROR as 0 for region functions. Use of this together 
				with errors.h should be treated carefully. 
	SeeAlso:	TOOLERROR; ERRORIF; Error

***********************************************************************************************/

/*********************************************************************************************

>	macro TOOLERROR(UINT ErrID, retvalue, toolID)

	Author:		Andy
	Created:	20/8/93
	Purpose:	Like ERROR, except specially for Tools to use.
				If you are a Tool you can use the TOOLERROR macro which takes a third parameter,
				that of your tool ID. MODULE_ID does not need to be defined for this to work.				
	SeeAlso:	ERROR; TOOLERRORIF; Error

*********************************************************************************************/

#ifdef ERROR
#undef ERROR 
#endif

#ifndef MODULE_ID

#define ERROR(errID, retparam) \
Error::SetError((UINT) errID, 0); \
return retparam;

#else

#define ERROR(errID, retparam) \
Error::SetError(errID, MODULE_ID); \
return retparam;

#endif

#define	TOOLERROR(errID, retparam, toolid) \
Error::SetErrorTool(errID, toolid); \
return retparam;



/***********************************************************************************************

>	macro ERRORIF(condition, UINT ErrID, retvalue)

	Author:		Jim
	Created:	9/7/93
	Inputs:		condition is a boolean expression, TRUE if error should be returned.
				ErrID is an error number/resource (all the same)
				retvalue is the value that will be returned from the function.
	Purpose:	This is a general purpose macro which is a bit like ENSURE, except that it really
				does return an error to the calling code.
				This macro should be used at the beginning of a member function, to test for
				the known conditions that the function works to. All functions should know what 
				conditions they expect before they start, and they should fail if those conditions 
				are not met. The condition should be a boolean expression which should evaluate
				to TRUE if an error condition has manifested itself, at which time the ERROR macro 
				is executed, making the function return whatever you put in retvalue.
				Usually, retvalue will be FALSE, since most functions that fail are expected to
				return a BOOL. However, some functions can usefully return other error indicators.
				For example, a function which returns a pointer can return NULL to indicate an error.
				Thus, you can use the macro as:
				
					ERRORIF(num != 42, ID_BADPARAM, FALSE);
				or
					ERRORIF(!ptr, ID_BADPOINTER, NULL);
					
	SeeAlso:	ERROR; Error

***********************************************************************************************/

/*********************************************************************************************

>	macro TOOLERRORIF(condition, UINT ErrID, retvalue, toolID)

	Author:		Andy
	Created:	20/8/93
	Purpose:	Like ERRORIF, except specially for Tools to use.
				If you are a Tool you can use the TOOLERROR macro which takes a third parameter,
				that of your tool ID. MODULE_ID does not need to be defined for this to work.				
	SeeAlso:	ERRORIF; TOOLERROR; Error

*********************************************************************************************/


#define ERRORIF(cond,returnID, retvalue)\
	if (cond) 							\
	{									\
		ERROR((UINT)returnID, retvalue);\
	}


#define	TOOLERRORIF(cond,returnID,retvalue,toolid)	\
	if (cond)										\
	{												\
		TOOLERROR((UINT)returnID, retvalue,toolid);	\
	}


// INLINE_MARKERROR non-zero if MarkError function is inline
// this is done in retail builds, except in DLL code
#if defined(_DEBUG) || defined(_AFXDLL)
#define	INLINE_MARKERROR	0
#else
#define	INLINE_MARKERROR	1
#endif
// MARKERROR_BODY is used to expand the actual function as required
#define	MARKERROR_BODY(Line,File)	{ LastErrorLine = Line; LastErrorFile = File; }

/***********************************************************************************************

>	class Error

	Author:		Jim
	Created:	9/7/93
	Purpose:	The Error class consists only of static variables, one for the ID of the error,
				another for the string, and another for the ID of the module that generated the 
				error (0 if the error happened in the kernel). The string is a simple TCHAR array
				to avoid the need for dynamic memory allocation during error handling. These 
				variables must be set whenever an error condition manifests itself. The class 
				provides static functions to do this, which are called by the ERRORIF and 
				NERROR macros
	SeeAlso:	ERRORIF; ERROR

***********************************************************************************************/

class CCAPI Error
{
private:
	static UINT ErrorID;

	// Chris introduced RalphErrorID during ralph error handling
	// ralph needs this so that he can map the ID to a HRESULT before passing it
	// back to a harness - the current ErrorId is cleared in SetSeriousError maybe by design
	// To be safe we'll keep our own copy.

	static UINT RalphErrorID;
	static UINT ModuleID;
	static TCHAR ErrorString[256];
	static UINT	RenderThread;			// This is incremented each time we start something that looks
										// like a render, and decremented as we thread out. So if an error
										// occurs, we know we have to be very careful

	static UINT LastErrorLine;			// line number of last error (or 0)
	static const char *LastErrorFile;	// ptr to filename of last error

public:
	Error()
	{
		ErrorString[0] = 0;
		RalphErrorID =ErrorID = ModuleID = 0;      
	}
	
	// convert ID's to HRESULT for ralph harness's
	static HRESULT ErrIDToHRESULT(UINT ErrID);

//	static HRESULT GetRalphError();

	~Error();
	static void SetError(UINT number, const TCHAR* errstring, UINT module);
	static void SetError(UINT number, UINT module = 0);
	static void SetErrorTool(UINT number, UINT toolid);
	static void SetErrorSerious( const TCHAR* );

	static void ClearError();

	inline static TCHAR* GetErrorString();
	
	inline static UINT GetErrorNumber();
	inline static UINT GetRalphErrorNumber();

	inline static UINT GetErrorModule();

	inline static void RenderThreadIn() { RenderThread++; }
	inline static void RenderThreadOut() { RenderThread--; }
	inline static void RenderThreadReset() { RenderThread = 0; }
	inline static BOOL IsInRenderThread() { return ( RenderThread != 0 ); }

	static void DirectStatus( BOOL = FALSE );

	static void MarkError( UINT LineNumber, const char *Filename )
	#if INLINE_MARKERROR
		MARKERROR_BODY( LineNumber, Filename )
	#else
		;
	#endif

	static void CDECL XSetErrorC();
	static void CDECL XSetError(const char *fmt, ...);
	static void CDECL XSetError(UINT, ...);
	static void CDECL ReleaseTrace(LPCTSTR, ...);
	#ifdef _DEBUG
	static void CDECL XComplain(const char *fmt, ...);
	static void CDECL XComplain(const wchar_t *fmt, ...);
	static void CDECL TraceUser(const char *, LPCSTR, ...);
	#ifdef _UNICODE
	static void CDECL TraceUser(const char *, LPCTSTR, ...);
	#endif
	#else
	static void CDECL XComplain(const char *fmt, ...) { }
	static void CDECL XComplain(const wchar_t *fmt, ...) { }
	static void CDECL TraceUser(const char *, LPCSTR, ...) { }
	#ifdef _UNICODE
	static void CDECL TraceUser(const char *, LPCTSTR, ...) { }
	#endif
	#endif

};

/***********************************************************************************************

>	void Error::RenderThreadIn()
	void Error::RenderThreadOut()
	void Error::RenderThreadReset()

	Author:		Alex
	Created:	25/2/94
	Inputs:		-
	Outputs:	-
	Returns:	-
	Purpose:	Notes the threading in or out of rendering (i.e. whether the program is somewhere
				within a render loop). Each RenderThreadIn should be paired with a RenderThreadOut
				and placed around rendering code. They may be nested. RenderThreadReset notes an
				abnormal termination of the thread (for instance an exception). Whilst in render
				threads, errors are handled slightly differently. Hence the need to know.
	Errors:		-
	SeeAlso:	Error::IsInRenderThread

***********************************************************************************************/

/***********************************************************************************************

>	BOOL Error::IsInRenderThread()
   
	Author:		Alex
	Created:	25/2/94
	Inputs:		-
	Outputs:	-
	Returns:	TRUE if in a render thread, else FALSE
	Purpose:	Tells the caller whether or not Camelot is in a render thread. If so, if an error
				occurs, it is likely to be (a) nasty, and (b) repeated many times, so more forceful
				action is taken to get rid of it.
	Errors:		-
	SeeAlso:	Error::RenderThreadIn, Error::RenderThreadOut, Error::RenderThreadReset

***********************************************************************************************/

/***********************************************************************************************

>	TCHAR* Error::GetErrorString()

	Author:		Jim
	Created:	9/7/93
	Inputs:		-
	Outputs:	-
	Returns:	pointer to TCHAR array containing last error
	Purpose:	Gets the string associated with the last error from the static error variables.
				Usually used by an operation to report why the operation failed.
	Errors:		-
	SeeAlso:	Error::GetErrorNumber;Error::GetErrorModule; Error::SetError

***********************************************************************************************/

TCHAR* Error::GetErrorString()
{
	return ErrorString;
}              

/***********************************************************************************************

>	UINT Error::GetRalphErrorNumber()

	Author:		Chris
	Created:	9/7/96
	Inputs:		-
	Outputs:	-
	Returns:	Error number of last error that occurred
	Purpose:	As GetERrorNumber EXCEPT this returns the ralph copy of ErrorID that is not 
				cleared in SetSeriousError()
	Errors:		-
	SeeAlso:	Error::GetErrorString; Error::GetErrorModule; Error::SetError

***********************************************************************************************/

UINT Error::GetRalphErrorNumber()
{
	// if ErrorId is zero (it may have been cleared in SetSeriousError )
	// return RalphErrorID
	if(	ErrorID == 0)
		return RalphErrorID;
	else
		return ErrorID;
} 	

/***********************************************************************************************

>	UINT Error::GetErrorNumber()

	Author:		Jim
	Created:	9/7/93
	Inputs:		-
	Outputs:	-
	Returns:	Error number of last error that occurred
	Purpose:	If a function returns failure, this function will read the error number.
				This is usually associated with a resource of some kind - in Win16 it will
				be the 16bit resource ID, in RISC OS it will be a 32bit resource ID, etc.
				There is always an associated string (reached by GetErrorString) so routines
				wanting to report the error do not need to do any nasty resource loading.
	Errors:		-
	SeeAlso:	Error::GetErrorString; Error::GetErrorModule; Error::SetError

***********************************************************************************************/

UINT Error::GetErrorNumber()
{
	return ErrorID;
}                     


	
/***********************************************************************************************

>	UINT Error::GetErrorModule()

	Author:		Jim
	Created:	9/7/93
	Inputs:		-
	Outputs:	-
	Returns:	Unique UINT identifying the module that generated the last error
				(zero if the last error was in the kernel).
	Purpose:	Mainly for debugging - the UINT is a unique identifier which will tell a
				programmer which module generated the last error. This will never be visible 
				to a user.
	Errors:		-
	SeeAlso:	Error::GetErrorNumber; Error::GetErrorString; Error::SetError

***********************************************************************************************/

UINT Error::GetErrorModule()
{
	return ModuleID;
}



// all-new error macros

#define	MARKWHERE	Error::MarkError( __LINE__, __FILE__ )
#ifdef _DEBUG
#define	MARKWHEREDB	MARKWHERE
#else
#define	MARKWHEREDB
#endif

#define	ERROR1RAW( errorID )						{ MARKWHEREDB; Error::XSetError( errorID ); }
#define	ERROR1( retvalue, errorID )					{ MARKWHEREDB; Error::XSetError( errorID ); return (retvalue); }
#define	ERROR1IF( condition, retvalue, errorID )	if (condition) ERROR1( retvalue, errorID )
#define	ERROR1_MSG( retvalue, args )				{ MARKWHEREDB; Error::XSetError args ; return (retvalue); }
#define	ERROR1IF_MSG( condition, retvalue, args )	if (condition) ERROR1_MSG( retvalue, args )

#ifdef _DEBUG
#define	ERROR2RAW( literal )						{ MARKWHERE; Error::XSetError( literal ); }
#define	ERROR2( retvalue, literal )					{ MARKWHERE; Error::XSetError( literal ); return (retvalue); }
#define	ERROR2IF( condition, retvalue, literal )	if (condition) ERROR2( retvalue, literal )
#define	ERROR2_PF( retvalue, args )					{ MARKWHERE; Error::XSetError args ; return (retvalue); }
#define	ERROR2IF_PF( condition, retvalue, args )	if (condition) ERROR2_PF( retvalue, args )

#define	ERROR2VOID( literal )						{ MARKWHERE; Error::XSetError( literal ); return; }
#define	ERROR2VOIDIF( condition, literal )			if (condition) ERROR2VOID( literal )

#define	ERROR3(literal)								{ MARKWHERE; Error::XComplain(literal); }
#define	ERROR3IF(condition, literal)				if (condition) ERROR3(literal)
#define	ERROR3_PF(args)								{ MARKWHERE; Error::XComplain args; }
#define	ERROR3_2(fmt, arg)							{ MARKWHERE; Error::XComplain(fmt, arg); }
#define	ERROR3IF_PF(condition, args)				if (condition) ERROR3_PF(args)
#define	TRACEUSER									Error::TraceUser
#define	TRACEALL									TRACE
#define RELTRACE									Error::ReleaseTrace
#else
#define	ERROR2RAW( literal )						{ MARKWHERE; Error::XSetErrorC(); }
#define	ERROR2( retvalue, literal )					{ MARKWHERE; Error::XSetErrorC(); return (retvalue); }
#define	ERROR2IF( condition, retvalue, literal )	if (condition) ERROR2( retvalue, literal )
#define	ERROR2_PF( retvalue, args )					{ MARKWHERE; Error::XSetError args ; return (retvalue); }
#define	ERROR2IF_PF( condition, retvalue, args )	if (condition) ERROR2_PF( retvalue, args )

#define	ERROR2VOID( literal )						{ MARKWHERE; Error::XSetError( literal ); return; }
#define	ERROR2VOIDIF( condition, literal )			if (condition) ERROR2VOID( literal )

#define	ERROR3(literal)
#define	ERROR3IF(condition, literal)
#define	ERROR3_PF(args)
#define	ERROR3IF_PF(condition, args)
#define ERROR3_2(fmt, arg)
#define TRACEUSER									1 ? (void)0 : Error::TraceUser
#define	TRACEALL									TRACE
#define RELTRACE									Error::ReleaseTrace
#endif



// The rest of this file is all the various error resource IDs from various 
// Camelot header files
#define IDE_PATH_ERROR                  0x7404
#define IDE_FILE_READ_ERROR             0x70D0
#define IDE_FILE_WRITE_ERROR            0x70D1
#define IDE_FILE_PERM_DENIED            0x70D2
#define IDE_FILE_BAD_FILE               0x70D3
#define IDE_FILE_EXISTS                 0x70D4
#define IDE_FILE_TOO_MANY_OPEN          0x70D5
#define IDE_FILE_NO_EXIST               0x70D6
#define IDE_FILE_NO_SPACE               0x70D7
#define IDE_FILE_UNKNOWN                0x70D8
#define IDE_ZOPEN_ERROR                 0x6c07
#define IDE_ZFILE_READ_ERROR            0x6c08
#define IDE_ZFILE_WRITE_ERROR           0x6c09
#define IDE_ZFILE_BASEEERROR            0x6c0a
#define IDE_ZFILE_ERRORNO               0x6c0b
#define IDE_ZFILE_STREAMERROR           0x6c0c
#define IDE_ZFILE_DATAERROR             0x6c0d
#define IDE_ZFILE_MEMORYERROR           0x6c0e
#define IDE_ZFILE_BUFFERERROR           0x6c0f
#define IDE_OPEN_ERROR                  0x7407
#define IDT_LEX_STRINGTOOLONG           0x707D
#define IDE_IO_ERROR                    0x741E
#define IDE_NULL_FILE                   0x740D
#define IDE_FILE_SIZE_ZERO              0x7414
#define IDE_MEM_BLOCK_FAILURE           0x7415
#define IDE_MEM_CLOSE_ERROR             0x740A
#define IDE_NOTOPEN_ERROR               0x740B
#define IDE_READ_ONLY                   0x7410
#define IDE_WRITE_ONLY                  0x7411
#define IDE_EOF_ERROR                   0x7408
#define IDE_FILEPOS_ERROR               0x7409
#define IDE_STRING_SIZE_ERROR           0x740C
#define IDS_OUT_OF_MEMORY               103
#define IDE_EX_BADOP                    0x77A5



#endif	//INC_ERRORS
