/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /camelot/intel/FIXED16.CPP 30    28/11/94 12:44 Alex $


#include "stdafx.h"
#include "fixed16.h"


/********************************************************************************************
>	void FIXED16::ToString(StringBase* out) const 

	Author: 	Mario Shamtani
	Created:	2/6/93 - rewritten in C 12/8/93 by Andy - rewritten so it works: JCF 31/10/94
	Inputs:		None.
	Outputs:	Returns a string representing the value of a FIXED16 in the 'out' parameter.
				Optional - sign, up	to five digits of integer, then option period and five
				fractional digits then zero terminator (max 13 character result)
	Returns:	-
	Purpose:	Converts value of FIXED16 into a string representing it. The string will be
				ASCII or Unicode depending on the size of TCHAR
	Errors:		-
********************************************************************************************/

void fixed16::ToString(StringBase* pOut) const
{
	// Allocate some working storage etc.
	TCHAR buf[32];
	TCHAR* pch = buf;
	long n = all;

	// If the number is negative then output a sign and make it positive.
	if (n < 0)
	{
		*pch++ = TEXT('-');
		n = -n;
	}

	// Convert and output the integer part of the number.
	pch += wsprintf(pch, TEXT("%u"), unsigned(n >> F16SHIFT));

	// Mask off the fractional part, converting it if it exists.
	n &= (1uL << F16SHIFT) - 1;
	if (n != 0)
	{
		// Output a decimal point.
		*pch++ = TEXT('.');

		// "Normalise" into an integer by multiplying by 100000/65336.
		n = MulDiv32By32(n, 100000L, 65536L);

		// Convert and output.  We must do this ourselves as we must take
		// account of leading zeros.
		long nPowerTen = 10000L;
		while (nPowerTen > 0)
		{
			// Is the remaining part of the number greater than our power of ten?
			if (n >= nPowerTen)
			{
				// Output a non-zero digit and subtract the closest multiple of the
				// power of ten.
				long nFactor = n / nPowerTen;
				*pch++ = TEXT('0') + TCHAR(nFactor);
				n -= (n * nFactor);
			}
			else
			{
				// Output a leading zero.
				*pch++ = TEXT('0');
			}

			// Go on to the next highest power of ten.
			nPowerTen /= 10;
		}
	}

	// Terminate and write into the output parameter.
	*pch = TEXT('\0');
	*pOut = buf;
}




// Friend Functions - Basic Operators
/********************************************************************************************

>	FIXED16_DBL(d) 

	Author: 	Mario Shamtani
	Created:	10/6/93
	Inputs:		double.
	Outputs:	None
	Returns:	FIXED16.
	Purpose:	
		This macro converts constant doubles into FIXED16s at compile time. It is 
		very effecient because there is no runtime overhead unlike normal conversion.  
		
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16::FIXED16 () 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Default constructor for FIXED16. It does not actually do anything!
	Errors:		None.

********************************************************************************************/

// actual (null) function can be found in class

/********************************************************************************************

>	inline FIXED16::FIXED16 (const FIXED16&) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Copy constructor.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16::FIXED16 (int) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16::FIXED16 (short) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16::FIXED16 (double) 

	Author: 	Mario Shamtani
	Created:	2/6/93
	SeeAlso:	Fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::FromRawLong(LONG d) 

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Scope:		Private - is only used to implement FIXED16_DBL macro. Do not call directly!
	Purpose:	converts a long to a FIXED16.
	Errors:		None.
	See Also:	Fixed16::FromShifted16

This function returns the fixed16 associated with the machine implementation representation of
the number passed in. Andy claims this may not be the same on all platforms, so this is the OIL
dependent way to get the internal representation. Compare and contrast FromShifted16 which does
the same thing on this platform.

This is intended for serialisation (not maths use), and also for the FIXED16_DBL macro.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::FromShifted16(LONG d) 

	Author: 	Alex
	Created:	25/11/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Scope:		Public
	Purpose:	converts a long to a FIXED16.
	Errors:		None.
	See Also:	Fixed16::FromShifted16

This function returns the fixed 16 number which is the LONG shifted down 16 places as a LONG,
i.e. divided by (2^16). Andy claims the internal representation may not be the same on all
platforms, so while FromRawLong() takes the internal representation, this is a platform
independent way of setting the 'meaning' of the fixed16, i.e. what you would get if you
divided the LONG by 2^16. On this platform, it's the same as GetRawLong().

This is intended for maths use (not serialisation).

********************************************************************************************/

/********************************************************************************************

>	inline int FIXED16::MakeInt() const 

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED16 to an integer.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline short FIXED16::MakeShort() const 

	Author: 	Mario Shamtani
	Created:	2/6/93
	SeeAlso:	Fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED16 to an integer.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline LONG FIXED16::MakeLong() const

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED16 to a LONG.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline XLONG FIXED16::MakeXlong() const

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED16 to a LONG.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16& operator= (const FIXED16& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline double FIXED16::MakeDouble() const

	Author: 	Mario Shamtani
	Created:	7/6/93
	SeeAlso:	fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:    double. 
	Purpose:	Makes a double (32-bit signed value) out of a fixed point 16-bit value.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16& operator= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixedasm.h
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline LONG fixed16::GetRawLong() const

	Author: 	Andy
	Created:	24/5/94
	Inputs:		-
	Outputs:	-
	Returns:	The longword that holds the whole value, i.e. the value <<16.
	Purpose:	Very efficient way of getting an entire longword. Use sparingly.
	Errors:		fixed16::GetShifted16()

This function returns the machine implementation representation of the number. Andy claims
this may not be the same on all platforms, so this is the OIL dependent way to get the internal
representation. Compare and contrast GetShifted16 which does the same thing on this platform.

This is intended for serialisation (not maths use).

********************************************************************************************/

/********************************************************************************************

>	inline LONG fixed16::GetShifted16() const

	Author: 	Alex
	Created:	22/11/94
	Inputs:		-
	Outputs:	-
	Returns:	The value <<16.
	Purpose:	Efficient way to get the value shifted 16 places
	Errors:		None.
	See Also:	fixed16::GetRawLong

This function returns the fixed 16 number shifted up 16 places as a LONG, i.e. when multiplied.
by (2^16). Andy claims the internal representation may not be the same on all platforms, so 
while GetRawLong() returns the internal representation, this is a platform independent way
of returning the 'meaning' of the fixed16, i.e. what you would get if you multiplied it by
2^16. On this platform, it's the same as GetRawLong().

This is intended for maths use (not serialisation).

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::trunc() const

	Author: 	Phil
	Created:	10/8/94
	Inputs:		Reference to a FIXED16.
	Outputs:	None.
	Returns:	The integer part of the input FIXED16.
	Purpose:	Take the integer part of FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::round() const

	Author: 	Phil
	Created:	10/8/94
	Inputs:		Reference to a FIXED16.
	Outputs:	None.
	Returns:	The closest integer to the input FIXED16.
	Purpose:	Find the closest integer to a FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::abs() const

	Author: 	Phil
	Created:	10/8/94
	Inputs:		Reference to a FIXED16.
	Outputs:	None.
	Returns:	The positive value of the input FIXED16.
	Purpose:	Find the absolute value (magnitude) of a FIXED16.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	inline FIXED16 FIXED16::sgn() const

	Author: 	Phil
	Created:	10/8/94
	Inputs:		Reference to a FIXED16.
	Outputs:	None.
	Returns:	+1 if the input FIXED 16 was positive or 0, -1 if the input was negative.
	Purpose:	Find the sign of a FIXED16.
				Note the difference between this and BASIC SGN functions: This one returns
				1 when the input value is zero!
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator+ (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Friend:		FIXED16
	Inputs:		Two values to be added together.
	Outputs:	None.
	Returns:	A FIXED16 with result of addition.
	Purpose:	Overloading the addition operator for FIXED16s.
	Errors:		None.

NB - inline

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator- (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Friend:		FIXED16
	Inputs:		Two values to be subtracted from each other.
	Outputs:	None.
	Returns:	A FIXED16 with result of subtraction.
	Purpose:	Overloading the subtract operator for FIXED16s.
	Errors:		None.

NB - inline

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator- (const FIXED16& operand1)

	Author: 	Andy
	Created:	30/5/93
	Friend:		FIXED16
	Inputs:		A value to be negated.
	Outputs:	None.
	Returns:	A FIXED16 with result of negation.
	Purpose:	Overloading the negation operator for FIXED16s. Should be inline.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator* (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Mul().
	Friend:		FIXED16
	Inputs:		Two values to be multiplied togther.
	Outputs:	None.
	Returns:	A FIXED16 with result of multiplication.
	Purpose:	Overloading the multiplication operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator/ (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Div().
	Friend:		FIXED16
	Inputs:		Two values to be divided.
	Outputs:	None.
	Returns:	A FIXED16 with result of division.
	Purpose:	Overloading the division operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator>> (const FIXED16& operand1, UINT operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Friend:		FIXED16
	Inputs:		operand1 - value to be shifted.
				operand2 - shift count.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of ASR.
	Purpose:	Overloading the >> operator to mean shift right for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16 operator<< (const FIXED16& operand1, UINT operand2) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Friend:		FIXED16
	Inputs:		operand1 - value to be shifted.
				operand2 - shift count.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of ASL.
	Purpose:	Overloading the << operator to mean shift left for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator== (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the equality operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator== (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	2/6/93
	Friend:		FIXED16
	Inputs:		FIXED16
				SHORT
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the equality operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator!= (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the inequality operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator!= (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		FIXED16
				short
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the inequality operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator> (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the greater-than operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator> (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		FIXED16
				short
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the greater-than operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator< (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the less-than operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator< (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		FIXED16
				short
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the less-than operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator>= (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the greater-than-equal-to operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator>= (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		FIXED16
				short
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the greater-than-equal-to operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator<= (const FIXED16& operand1, const FIXED16& operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		Two FIXED16s to be compared.
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the less-than-equal-to operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	int operator<= (const FIXED16& operand1, const short operand2) 

	Author: 	Mario Shamtani
	Created:	5/5/93
	Friend:		FIXED16
	Inputs:		FIXED16
				short
	Outputs:	None.
	Returns:	0 - False
				1 - True
	Purpose:	Overloading the less-than-equal-to operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

// Assignment operators

/********************************************************************************************

>	FIXED16& FIXED16::operator= (const int operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator= (const double operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	DoubleToFixed16().
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator+= (const FIXED16& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be incremented
				operand to be added - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the plus-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator+= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be incremented
				operand to be added - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the plus-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator-= (const FIXED16& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be decremented
				operand to be subtracted - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the minus-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator-= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be decremented
				operand to be subtracted - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the minus-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

/********************************************************************************************

>	FIXED16& FIXED16::operator*= (const FIXED16& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Mul().
	Inputs:		lvalue to be multiplied
				multiplier - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the times-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
 
/********************************************************************************************

>	FIXED16& FIXED16::operator*= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Mul().
	Inputs:		lvalue to be multiplied
				multiplier - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the times-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
 
/********************************************************************************************

>	FIXED16& FIXED16::operator/= (const FIXED16& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Div().
	Inputs:		lvalue to be divided
				divisor - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the divide-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
 
/********************************************************************************************

>	FIXED16& FIXED16::operator/= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixed16Div().
	Inputs:		lvalue to be divided
				divisor - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the divide-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
 
/********************************************************************************************

>	FIXED16& FIXED16::operator<<= (const UINT operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be shifted
				shift count - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the left-shift-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
 
/********************************************************************************************

>	FIXED16& FIXED16::operator>>= (const UINT operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		lvalue to be shifted
				shift count - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of assignment.
	Purpose:	Overloading the right-shift-equals operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
  
// Increment FIXED16::operators

/********************************************************************************************

>	FIXED16& FIXED16::operator++ () 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of increment.
	Purpose:	Overloading the prefix plus-plus operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
  
/********************************************************************************************

>	FIXED16 FIXED16::operator++ (int dummy) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		dummy value to distinguish between pre- and post-fix application.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of increment.
	Purpose:	Overloading the postfix plus-plus operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
  
/********************************************************************************************

>	FIXED16& FIXED16::operator-- () 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of decrement.
	Purpose:	Overloading the prefix minus-minus operator for FIXED16s.
	Errors:		None.

********************************************************************************************/
  
/********************************************************************************************

>	FIXED16 FIXED16::operator-- (int dummy) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		dummy value to distinguish between pre- and post-fix application.
	Outputs:	None.
	Returns:	A reference to an FIXED16 with result of decrement.
	Purpose:	Overloading the postfix minus-minus operator for FIXED16s.
	Errors:		None.

********************************************************************************************/

