/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /camelot/intel/FIXED16.H 33    28/11/94 12:44 Alex $

/*
 */ 

/*********************************************************************************************

>	class FIXED16

	Author:		Mario Shamtani
	Created:	26/4/1993
	SeeAlso:	CCMaths.h
	SeeAlso:	Fixedasm.h
	SeeAlso:	Div32By32
	SeeAlso:	Keyword FIXED16
	Purpose:	
		An FIXED16 is a signed 32-bit value with a binary point between bits 15 and 16. 
		Its purpose is to represent 32-bit floating point numbers.
		FIXED16 is a fixed point number contained within a 32-bit longword.
		The high 16-bit half contains the integer part, the low half the fractional. (Intel
		implementations store this as the low half first in memory). The integer half ranges
		from -32768 to 32767, the fractional part from 0 to 65535/65536ths (1/65536th=
		0.000015259, 65535/65536=0.999984741). For efficiency all primitives are written
		in assembly-language, though most are used transparently by operator overloading.
		No-one should include fixed16.h directly - always use ccmath.h. You should also
		never use the lower-case type 'fixed16' - always used 'FIXED16'.    
    Errors:		None.
 	
********************************************************************************************/

#ifndef INC_FIXED16
#define INC_FIXED16

// One as a fixed16 as a longword, and 0.5 as...
const long FIXED16_ONE = 0x10000L;
const long FIXED16_HALF= 0x08000L;

class fixed16
{
private:
	long all;



public:

	#define	F16SHIFT 16									// marks binary point position

	// this converts a short into a fixed16 longword by casting to a long then shifting it
	#define	SHORT_FIXED(x)	( ((long)(x)) << F16SHIFT )

	inline fixed16() {};				
	inline fixed16(const fixed16&);
	inline fixed16(int);
	inline fixed16(short);
	inline fixed16(double);

	fixed16& Sin();
	fixed16& Cos();

	inline fixed16 trunc() const;
	inline fixed16 round() const;
	inline fixed16 abs() const;
	inline fixed16 sgn() const;
    
    void ToString(StringBase* out) const;

	// Friend Functions

	inline friend fixed16 Div32By32(const LONG a, const LONG b);
	inline friend fixed16 CCAPI operator+ (const fixed16&, const fixed16&);
	inline friend fixed16 CCAPI operator- (const fixed16&, const fixed16&);
	inline friend fixed16 CCAPI operator- (const fixed16&);
	inline friend fixed16 CCAPI operator* (const fixed16&, const fixed16&);
	inline friend fixed16 CCAPI operator/ (const fixed16&, const fixed16&);
	inline friend fixed16 CCAPI operator>> (const fixed16&, UINT);
	inline friend fixed16 CCAPI operator<< (const fixed16&, UINT);
	inline friend fixed16 CCAPI operator- (const fixed16&);

	// Relational Operators
    
    inline friend int CCAPI operator== (const fixed16&, const fixed16&);
    inline friend int CCAPI operator== (const fixed16&, const short);
    inline friend int CCAPI operator!= (const fixed16&, const fixed16&);
    inline friend int CCAPI operator!= (const fixed16&, const short);
    inline friend int CCAPI operator> (const fixed16&, const fixed16&);
    inline friend int CCAPI operator> (const fixed16&, const short);
    inline friend bool CCAPI operator< (const fixed16&, const fixed16&);
    inline friend int CCAPI operator< (const fixed16&, const short);
    inline friend int CCAPI operator>= (const fixed16&, const fixed16&);
    inline friend int CCAPI operator>= (const fixed16&, const short);
    inline friend int CCAPI operator<= (const fixed16&, const fixed16&);
    inline friend int CCAPI operator<= (const fixed16&, const short);
    
	// Assignment Operators

	inline fixed16& operator= (const fixed16&);
	inline fixed16& operator= (const int);
	inline fixed16& operator= (const short);
	inline fixed16& operator= (const double);
	inline fixed16& operator+= (const fixed16&);
	inline fixed16& operator+= (const short);
	inline fixed16& operator-= (const fixed16&); 
	inline fixed16& operator-= (const short);
	inline fixed16& operator*= (const fixed16&); 
	inline fixed16& operator*= (const short);
	inline fixed16& operator/= (const fixed16&);
	inline fixed16& operator/= (const short); 
	inline fixed16& operator<<= (const UINT); 
	inline fixed16& operator>>= (const UINT); 

	// Increment\Decrement Operators

	inline fixed16& operator++ ();							// prefix
	inline fixed16 operator++ (int);						// postfix 
	inline fixed16& operator-- ();							// prefix 
	inline fixed16 operator-- (int);						// postfix 

	// Conversion functions

	inline int 	 MakeInt() const;
	inline short MakeShort() const;
	inline double MakeDouble() const;
	inline long  MakeLong() const;
	inline xlong MakeXlong() const;

	inline static fixed16 FromRawLong(LONG d);
	inline static fixed16 FromShifted16(LONG d);
	inline LONG GetRawLong() const;
	inline LONG GetShifted16() const;
};


// this is a temporary kludge for the 32-bit MS compiler
#if WIN32
#define	F16ASSIGN( it )		(it).all
#define	F16ASSIGNTHIS		this->all
#else
#define	F16ASSIGN( it )		it
#define	F16ASSIGNTHIS		*this
#endif


#include "fixedasm.h"

/********************************************************************************************

Note that all the comment blocks live in the cpp files so they get compiled in with the help
compiler

********************************************************************************************/

#define FIXED16_DBL(d) fixed16::FromRawLong((LONG)((d)*65536))

inline fixed16::fixed16(const fixed16& initValue) 
{
	all = initValue.all;
}

inline fixed16::fixed16(int initValue) 
{
	all = SHORT_FIXED( initValue );
}

inline fixed16::fixed16(short initValue) 
{
	all = SHORT_FIXED( initValue );
}

inline fixed16::fixed16(double initValue) 
{
#if WIN32
	this->all = DoubleToFixed16(initValue);
#else
	*this = DoubleToFixed16(initValue);
#endif
}

inline fixed16 fixed16::FromRawLong(LONG d)
{
	fixed16 temp;
	temp.all = d;
	return temp;
}

inline fixed16 fixed16::FromShifted16(LONG d)
{
	fixed16 temp;
	temp.all = d;
	return temp;
}

inline int fixed16::MakeInt() const
{
	return (int) (all >> F16SHIFT);
}

inline short fixed16::MakeShort() const
{
	return (short) (all >> F16SHIFT);
}

inline LONG fixed16::MakeLong() const
{
	return all >> F16SHIFT;
}

inline xlong fixed16::MakeXlong() const
{
    return xlong( (short) (all >> F16SHIFT) );
}


inline fixed16& fixed16::operator= (const fixed16& operand)
{ 
	this->all = operand.all;
	
	return *this; 
}

inline double fixed16::MakeDouble() const
{
	return Fixed16ToDouble(*this);
}

inline fixed16& fixed16::operator= (const short operand)
{ 
	this->all = SHORT_FIXED( operand );
	return *this;
}

inline LONG fixed16::GetRawLong() const
{
	return this->all;
}

inline LONG fixed16::GetShifted16() const
{
	return this->all;
}

inline fixed16 fixed16::trunc() const
{
	return FromRawLong( all & 0xFFFF0000L );
}
	
inline fixed16 fixed16::round() const
{
	return FromRawLong( ( all + FIXED16_HALF ) & 0xFFFF0000L );
}
	
inline fixed16 fixed16::abs() const
{
	return FromRawLong( all<0 ? -all : all );
}
	
inline fixed16 fixed16::sgn() const
{
	return FromRawLong( all<0 ? -FIXED16_ONE : FIXED16_ONE );
}
	
inline fixed16 operator+ (const fixed16& operand1, const fixed16& operand2)
{
	fixed16 result;
	result.all = operand1.all + operand2.all;
	return result;
}

inline fixed16 operator- (const fixed16& operand1, const fixed16& operand2)
{
	fixed16 result;
	result.all = operand1.all - operand2.all;
	return result;
}

inline fixed16 operator- (const fixed16& operand)
{
	fixed16 temp;
	temp.all = -operand.all;
	return temp;
}

inline fixed16 operator* (const fixed16& operand1, const fixed16& operand2)
{
	fixed16 result;
	F16ASSIGN(result) = Fixed16Mul( operand1, operand2 );
	return result;
}

inline fixed16 operator/ (const fixed16& operand1, const fixed16& operand2)
{
	fixed16 result;
	F16ASSIGN(result) = Fixed16Div( operand1, operand2 );
	return result;
}

inline fixed16 operator>> (const fixed16& operand1, UINT operand2)
{
	fixed16 result;
	result.all = operand1.all >> operand2;
	return result;
}

inline fixed16 operator<< (const fixed16& operand1, UINT operand2)
{
	fixed16 result;
	
	result.all = operand1.all << operand2;
	
	return result;
}

inline int operator== (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all == operand2.all;
}

inline int operator== (const fixed16& operand1, const short operand2) 
{
	return operand1.all == SHORT_FIXED( operand2 );
}

inline int operator!= (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all != operand2.all;
}

inline int operator!= (const fixed16& operand1, const short operand2) 
{
	return operand1.all != SHORT_FIXED( operand2 );
}

inline int operator> (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all > operand2.all;
}

inline int operator> (const fixed16& operand1, const short operand2) 
{
	return operand1.all > SHORT_FIXED(operand2);
}

inline bool operator< (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all < operand2.all;
}

inline int operator< (const fixed16& operand1, const short operand2) 
{
	return operand1.all < SHORT_FIXED( operand2 );
}

inline int operator>= (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all >= operand2.all;
}

inline int operator>= (const fixed16& operand1, const short operand2) 
{
	return operand1.all >= SHORT_FIXED( operand2 );
}

inline int operator<= (const fixed16& operand1, const fixed16& operand2) 
{
	return operand1.all <= operand2.all;
}

inline int operator<= (const fixed16& operand1, const short operand2) 
{
	return operand1.all <= SHORT_FIXED( operand2 );
}

inline fixed16& fixed16::operator= (const int operand)
{
	this->all = SHORT_FIXED( operand );
	return *this;
}

inline fixed16& fixed16::operator= (const double operand)
{ 
	F16ASSIGNTHIS = DoubleToFixed16(operand);
	return *this;
}

inline fixed16& fixed16::operator+= (const fixed16& operand)
{ 
	this->all += operand.all;
	return *this;
}

inline fixed16& fixed16::operator+= (const short operand)
{ 
	this->all += SHORT_FIXED( operand );
	return *this;
}

inline fixed16& fixed16::operator-= (const fixed16& operand)
{
	this->all -= operand.all;
	return *this;
}

inline fixed16& fixed16::operator-= (const short operand)
{
	this->all -= SHORT_FIXED( operand );
	return *this;
}

inline fixed16& fixed16::operator*= (const fixed16& operand)
{
	F16ASSIGNTHIS = Fixed16Mul(*this, operand);
	
	return *this;
}

inline fixed16& fixed16::operator*= (const short operand)
{                                  
	fixed16 temp;
	
	temp.all = SHORT_FIXED( operand );
	
	F16ASSIGNTHIS = Fixed16Mul(*this, temp);
	
	return *this;
}

inline fixed16& fixed16::operator/= (const fixed16& operand)
{
	F16ASSIGNTHIS = Fixed16Div(*this, operand);
	
	return *this;
}

inline fixed16& fixed16::operator/= (const short operand)
{
	fixed16 temp;
	
	temp.all = SHORT_FIXED( operand );
	
	F16ASSIGNTHIS = Fixed16Div(*this, temp);
	
	return *this;
}

inline fixed16& fixed16::operator<<= (const UINT operand)
{
	this->all <<= operand;
	return *this;
}
  
inline fixed16& fixed16::operator>>= (const UINT operand)
{
	this->all >>= operand;
	return *this;
} 

inline fixed16& fixed16::operator++ ()							// prefix
{
	this->all += FIXED16_ONE;
	return *this;
}

inline fixed16 fixed16::operator++ (int dummy)   				// postfix 
{                
	fixed16 result = *this;
	
	this->all += FIXED16_ONE;
	
	return result;
}

inline fixed16& fixed16::operator-- ()							// prefix 
{
	this->all -= FIXED16_ONE;
	return *this;
}

inline fixed16 fixed16::operator-- (int dummy)				// postfix 
{
	fixed16 result = *this;
	
	this->all -= FIXED16_ONE;
	
	return result;
}
	
#endif
