/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/

// Stardate 070494.10:25. Captain's Log - supplementary
// ====================================================
//
// 		Pirated fixed16 code to base fixed24 upon
// 		This was from revision 1.27 of fixed16.h
//												  Jason


/*********************************************************************************************

>	class FIXED24

	Author:		Mario Shamtani (Hijacked by Jason)
	Created:	26/4/1993 (07/04/94)
	SeeAlso:	CCMaths.h
	SeeAlso:	Fixedasm.h
	SeeAlso:	Div32By32
	SeeAlso:	Keyword FIXED24
	SeeAlso:	Keyword FIXED16
	Purpose:	
		A FIXED24 is a signed 32-bit value with a binary point between bits 23 and 24
		Its purpose is to represent floating point numbers in 32 bits.
		The implementation is very similar to FIXED16, but note that many methods are
		currently not defined in FIXED24, as it is used only in the colour system, which
		does not need large chunks of the FIXED16 functionality.
		No-one should include fixed24.h directly - always use ccmath.h. You should also
		never use the lower-case type 'fixed24' - always used 'FIXED24'.    

		Operations from FIXED16 that are not available in FIXED24 are currently removed
		by the use of #ifdef FULL_FIXED24 directives. (if there is no help on a function,
		then it is currently unsupported in FIXED24)

    Errors:		None.
 	
********************************************************************************************/

#ifndef INC_FIXED24
#define INC_FIXED24

//class CCAPI xlong;


#ifdef FULL_FIXED24
class CCAPI String;
#endif

class CCAPI fixed24
{
private:
	long all;



public:

	#define	F24SHIFT (24)								// marks binary point position

	// this converts a short into a fixed24 longword by casting to a long then shifting it
	#define	SHORT_FIXED24(x)	( ((long)(x)) << F24SHIFT )

	inline fixed24() {};
	inline fixed24(const fixed24&);
	inline fixed24(int);
	inline fixed24(short);
	inline fixed24(double);

#ifdef FULL_FIXED24
	fixed24& Sin();
	fixed24& Cos();
    
    void ToString(String* out) const;
#endif


	// Friend Functions

	friend fixed24 CCAPI operator+ (const fixed24&, const fixed24&);
	friend fixed24 CCAPI operator- (const fixed24&, const fixed24&);
	friend fixed24 CCAPI operator- (const fixed24&);
	friend fixed24 CCAPI operator* (const fixed24&, const fixed24&);
	friend fixed24 CCAPI operator/ (const fixed24&, const fixed24&);
	friend fixed24 CCAPI operator>> (const fixed24&, UINT);
	friend fixed24 CCAPI operator<< (const fixed24&, UINT);

	// Relational Operators
    
    friend int CCAPI operator== (const fixed24&, const fixed24&);
    friend int CCAPI operator!= (const fixed24&, const fixed24&);
    friend int CCAPI operator> (const fixed24&, const fixed24&);
    friend int CCAPI operator< (const fixed24&, const fixed24&);
    friend int CCAPI operator>= (const fixed24&, const fixed24&);
    friend int CCAPI operator<= (const fixed24&, const fixed24&);

    friend int CCAPI operator== (const fixed24&, const short);
    friend int CCAPI operator!= (const fixed24&, const short);
    friend int CCAPI operator> (const fixed24&, const short);
    friend int CCAPI operator< (const fixed24&, const short);
    friend int CCAPI operator>= (const fixed24&, const short);
    friend int CCAPI operator<= (const fixed24&, const short);
    
	// Assignment Operators

	inline fixed24& operator= (const fixed24&);
	fixed24& operator= (const int);
	inline fixed24& operator= (const short);
	fixed24& operator= (const double);
	fixed24& operator+= (const fixed24&);
	fixed24& operator+= (const short);
	fixed24& operator-= (const fixed24&); 
	fixed24& operator-= (const short);
	fixed24& operator*= (const fixed24&); 
	fixed24& operator*= (const short);
	fixed24& operator/= (const fixed24&);
	fixed24& operator/= (const short); 
	fixed24& operator<<= (const UINT); 
	fixed24& operator>>= (const UINT); 


	// Increment\Decrement Operators

	fixed24& operator++ ();							// prefix
	fixed24 operator++ (int);						// postfix 
	fixed24& operator-- ();							// prefix 
	fixed24 operator-- (int);						// postfix 

	// Conversion functions

	inline int 	 MakeInt() const;
	inline short MakeShort() const;
	inline double MakeDouble() const;
	inline long  MakeLong() const;
	inline xlong MakeXlong() const;

	inline static fixed24 FromRawLong(LONG d);		// DO NOT USE!

	inline LONG GetAsFixed24(void);
	inline void SetAsFixed24(LONG NewValue);
};


typedef fixed24 FIXED24;


#include "fix24asm.h"

/********************************************************************************************

>	FIXED24_DBL(d) 

	Author: 	Mario Shamtani
	Created:	10/6/93
	Inputs:		double.
	Outputs:	None
	Returns:	FIXED24.
	Purpose:	
		This macro converts constant doubles into FIXED24s at compile time. It is 
		very effecient because there is no runtime overhead unlike normal conversion.  
		
	Errors:		None.

********************************************************************************************/

#define FIXED24_DBL(d) fixed24::FromRawLong((LONG)((d)*(1<<24)))

/********************************************************************************************

>	inline FIXED24::FIXED24 () 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Default constructor for FIXED24. It does not actually do anything!
	Errors:		None.

********************************************************************************************/

// actual (null) function can be found in class

/********************************************************************************************

>	inline FIXED24::FIXED24 (const FIXED24&) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Copy constructor.
	Errors:		None.

********************************************************************************************/

inline fixed24::fixed24(const fixed24& initValue) 
{
	all = initValue.all;
}

/********************************************************************************************

>	inline FIXED24::FIXED24 (int) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

inline fixed24::fixed24(int initValue) 
{
	all = SHORT_FIXED24( initValue );
}

/********************************************************************************************

>	inline FIXED24::FIXED24 (short) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

inline fixed24::fixed24(short initValue) 
{
	all = SHORT_FIXED24( initValue );
}

/********************************************************************************************

>	inline FIXED24::FIXED24 (double) 

	Author: 	Mario Shamtani
	Created:	2/6/93
	SeeAlso:	Fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	Constructor.
	Errors:		None.

********************************************************************************************/

inline fixed24::fixed24(double initValue) 
{
	this->all = DoubleToFixed24(initValue);
}

/********************************************************************************************

>	inline FIXED24 FIXED24::FromRawLong(LONG d) 

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		A long
	Outputs:	A Fixed24
	Returns:	None.
	Scope:		Private - for FIXED_DBL macro
				If you wish to access the private data in a FIXED24, then please
				use GetAsFixed24, which is an interface inline function which
				will always present the data to the outside world in 8.24 bit
				format, even if we decide to change the internal format one day.
	Purpose:	converts a long to a FIXED24.
	Errors:		None.

********************************************************************************************/

inline fixed24 fixed24::FromRawLong(LONG d)
{
	fixed24 temp;

	temp.all = d;
	return temp;
}



/********************************************************************************************

>	inline LONG FIXED24::GetAsFixed24(void) 

	Author: 	Jason Williams
	Created:	12/04/94
	Inputs:		None
	Outputs:	None
	Returns:	A fixed point value stored as 8.24 bits in a LONG
	Scope:		Sortof private for use in the DocColour system
	Purpose:	converts a FIXED24 to an 8.24 bit fixed point value stored in
				a LONG. (i.e. pulls the 8.24 data word directly out of the FIXED24,
				though obviously if the storage model in a fixed24 changes in the future
				we can change this access function to still return a 8.24 LONG)
	Errors:		None.

********************************************************************************************/

inline LONG fixed24::GetAsFixed24(void)
{
	return((LONG) all);
}


/********************************************************************************************

>	inline LONG FIXED24::SetAsFixed24(void) 

	Author: 	Jason Williams
	Created:	12/04/94
	Inputs:		None
	Outputs:	None
	Returns:	None
	Scope:		Sortof private for use in the DocColour system
	Purpose:	Sets a FIXED24 to an 8.24 bit fixed point value stored in a LONG.
				(i.e. plonks the 8.24 data word directly into the FIXED24, though
				obviously if the storage model in a fixed24 changes in the future
				we can change this access function to still accept a 8.24 LONG)
	Errors:		None.

********************************************************************************************/

inline void fixed24::SetAsFixed24(LONG NewValue)
{
	all = NewValue;
}



/********************************************************************************************

>	inline int FIXED24::MakeInt() const 

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED24 to an integer.
	Errors:		None.

********************************************************************************************/

inline int fixed24::MakeInt() const
{
	return (int) (all >> F24SHIFT);
}

/********************************************************************************************

>	inline short FIXED24::MakeShort() const 

	Author: 	Mario Shamtani
	Created:	2/6/93
	SeeAlso:	Fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED24 to an integer.
	Errors:		None.

********************************************************************************************/

inline short fixed24::MakeShort() const
{
	return (short) (all >> F24SHIFT);
}

/********************************************************************************************

>	inline LONG FIXED24::MakeLong() const

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED24 to a LONG.
	Errors:		None.

********************************************************************************************/

inline LONG fixed24::MakeLong() const
{
	return all >> F24SHIFT;
}

/********************************************************************************************

>	inline XLONG FIXED24::MakeXlong() const

	Author: 	Mario Shamtani
	Created:	2/6/93
	Inputs:		None
	Outputs:	None
	Returns:	None.
	Purpose:	converts a FIXED24 to a LONG.
	Errors:		None.

********************************************************************************************/

inline xlong fixed24::MakeXlong() const
{
    return xlong( (short) (all >> F24SHIFT) );
}

/********************************************************************************************

>	inline FIXED24& operator= (const FIXED24& operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED24 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED24s.
	Errors:		None.

********************************************************************************************/

inline fixed24& fixed24::operator= (const fixed24& operand)
{ 
	this->all = operand.all;
	
	return *this; 
}
/********************************************************************************************

>	inline double FIXED24::MakeDouble() const

	Author: 	Mario Shamtani
	Created:	7/6/93
	SeeAlso:	fixedasm.h
	Inputs:		None
	Outputs:	None
	Returns:    double. 
	Purpose:	Makes a double (32-bit signed value) out of a fixed point 24-bit value.
	Errors:		None.

********************************************************************************************/

inline double fixed24::MakeDouble() const
{
	return Fixed24ToDouble(*this);
}

/********************************************************************************************

>	inline FIXED24& operator= (const short operand) 

	Author: 	Mario Shamtani
	Created:	27/4/93
	SeeAlso:	Fixedasm.h
	Inputs:		operand to be  assigned - rhs of assignment.
	Outputs:	None.
	Returns:	A reference to an FIXED24 with result of assignment.
	Purpose:	Overloading the assignment operator for FIXED24s.
	Errors:		None.

********************************************************************************************/

inline fixed24& fixed24::operator= (const short operand)
{ 
	this->all = SHORT_FIXED24( operand );
	return *this;
}




#endif
