/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/

// low-level functions for fixed-point arithmetic
// should only be called by fixed16.cpp/fixed32.cpp - everyone else should use ccmaths.h

#ifndef INC_FIXEDASM
#define	INC_FIXEDASM

#include <emmintrin.h>
// requires that fixed16 already be defined


inline LONG Fixed16Mul( fixed16 a, fixed16 b ) { return LONG((__emul((LONG&)a,(LONG&)b)+0x8000)>>16); }
inline LONG InternalDiv32by32( LONG a,LONG b ) { return MulDiv(a,0x10000,b); }

// 1/65536 = 0.0000152587890625
inline double Fixed16ToDouble( fixed16 a ) { return (LONG&)a*0.0000152587890625; }

// Pi/(180<<16) = 2.6631610900792382460383465095346e-7
// LONG Fixed16Sin( fixed16 a ) { return DoubleToFixed16( sin((double)(LONG&)a*2.6631610900792382460383465095346e-7) ); }
// LONG Fixed16Cos( fixed16 a ) { return DoubleToFixed16( cos((double)(LONG&)a*2.6631610900792382460383465095346e-7) ); }

inline LONG MatrixCalc( fixed16 a, LONG b, fixed16 c, LONG d ) { return LONG((__emul((LONG&)a,b)+Int32x32To64((LONG&)c,d)+0x8000)>>16); }
inline LONG LongMulFixed16( LONG a, fixed16 b ) { return LONG((__emul(a,(LONG&)b)+0x8000)>>16); }

#  if defined(_M_IX86) && !defined(_M_X64)
inline LONG Fixed16Div( fixed16 a, fixed16 b ) { static const double s = 65536.0; LONG t;
												__asm fild a
												__asm fmul s
												__asm fidiv b
												__asm fistp t
												return t; }
inline LONG LongDivFixed16( LONG a, fixed16 b ) { static const double s = 65536.0; LONG t;
												__asm fild a
												__asm fmul s
												__asm fidiv b
												__asm fistp t
												return t; }
inline LONG MPtoPixel( LONG a, fixed16 b ) { LONG t;
											__asm fild a
											__asm fidiv b
											__asm fistp t
											return t; }
inline LONG MPtoOS256( LONG a, fixed16 b ) { static const double s = 16777216.0; LONG t;
											__asm fild a
											__asm fmul s
											__asm fidiv b
											__asm fistp t
											return t; }
inline LONG DoubleToFixed16( double a ) { static const double s = 65536.0; LONG t;
										__asm fld a
										__asm fmul s
										__asm fistp t
										return t; }
#elif defined(_M_X64)

inline LONG Fixed16Div( fixed16 a,  fixed16 b ) { return _mm_cvtsd_si32( _mm_set_sd((LONG&)a*65536.0   /(LONG&)b) ); }
inline LONG LongDivFixed16( LONG a, fixed16 b ) { return _mm_cvtsd_si32( _mm_set_sd(       a*65536.0   /(LONG&)b) ); }
inline LONG		 MPtoPixel( LONG a, fixed16 b ) { return _mm_cvtsd_si32( _mm_set_sd(       a		   /(LONG&)b) ); }
inline LONG		 MPtoOS256( LONG a, fixed16 b ) { return _mm_cvtsd_si32( _mm_set_sd(       a*16777216.0/(LONG&)b) ); }
inline LONG DoubleToFixed16( double a )			{ return _mm_cvtsd_si32( _mm_set_sd(	   a*65536.0) ); }
#else
inline LONG Fixed16Div( fixed16 a,  fixed16 b ) { return MulDiv((LONG&)a,0x10000,(LONG&)b); }
inline LONG LongDivFixed16( LONG a, fixed16 b ) { return MulDiv(       a,0x10000,(LONG&)b); }
inline LONG MPtoOS256( LONG a, fixed16 b ) { return floor((double)a*16777216.0/(LONG&)b+0.5); }
inline LONG MPtoPixel( LONG a, fixed16 b ) { return floor((double)a/(LONG&)b+0.5); }
inline LONG DoubleToFixed16( double a )	   { return floor(a*65536.0+0.5) ); }
#endif


#endif

/*
// $Header: /camelot/intel/FIXEDASM.H 13    28/11/94 12:44 Alex $

// low-level functions for fixed-point arithmetic
// should only be called by fixed16.cpp/fixed32.cpp - everyone else should use ccmaths.h

#ifndef INC_FIXEDASM
#define	INC_FIXEDASM

// requires that fixed16 already be defined

#if WIN32
#define	retfixed16 LONG
#else
#define	retfixed16 fixed16
#endif

extern "C"
{

	retfixed16 Fixed16Mul( fixed16, fixed16 );
	retfixed16 Fixed16Div( fixed16, fixed16 );

	retfixed16 InternalDiv32by32(LONG,LONG);

	double Fixed16ToDouble( fixed16 );
	retfixed16 DoubleToFixed16( double );

	retfixed16	Fixed16Sin( fixed16 );
	retfixed16	Fixed16Cos( fixed16 );

	LONG MatrixCalc( fixed16, LONG, fixed16, LONG);
	LONG LongMulFixed16( LONG, fixed16 );
	LONG LongDivFixed16( LONG arg, fixed16 );
	LONG MPtoPixel( LONG arg, fixed16 );
	LONG MPtoOS256( LONG arg, fixed16 );	
};

#if WIN32
#undef retfixed16
#endif

#endif
*/
