/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/

#include "stdafx.h"
#include "fixst256.h"


/**************************************************************************************
>	String_256::String_256(const TCHAR* psz = 0)

	Author:		JustinF
	Created:	12th August 1993
	Inputs:		An optional pointer to a (constant) Unicode character array - if
				omitted then the String becomes empty (NOT the null pointer).
	Purpose:	Constructs a string of the appropriate maximum length.
	Errors:		ASSERTion failure if the passed character array is too long for
				this length of string, or if psz is NULL.
	Scope:		Public
***************************************************************************************/
#ifdef _UNICODE
String_256::String_256(const char* psz)
{
	*(text = fixedbuf) = 0;
	length = FIX_LEN_BUFSIZE;
	if (psz != 0)
	{
		ERROR3IF(strlen(psz) >= length, "String_256::String_256 overflow");
		USES_CONVERSION;
		text = cc_lstrcpyn(text, A2T(psz), FIX_LEN_BUFSIZE-1);
	}
}
#endif

String_256::String_256(const TCHAR* psz)
{
	*(text = fixedbuf) = 0;
	length = FIX_LEN_BUFSIZE;
	if (psz != 0)
	{
		ERROR3IF(lstrlen(psz) >= length, "String_256::String_256 overflow");
		lstrcpy(text, psz);
	}
}



/**************************************************************************************
>	String_256::String_256(const StringBase& other)

	Author:		JustinF
	Created:	12th August 1993
	Inputs:		A constant reference to another string (can be of any length)
	Purpose:	Duplicates the text of the other string.
	Errors:		ASSERTion failure if this string is not long enough to hold a copy
				of the other string's text, or if the copied String has not been
				allocated.
	Scope:		Public
***************************************************************************************/

String_256::String_256(const StringBase& other)
{
	*(text = fixedbuf) = 0;
	length = FIX_LEN_BUFSIZE;
	ERROR3IF((const TCHAR*) other == 0, "StringBase to be copied has not been ALLOCated");
	ERROR3IF(lstrlen((const TCHAR*) other) >= length,
				"Constructed String_256 not large enough to hold StringBase copy");
	lstrcpy(text, (const TCHAR*) other);
}



#if FALSE
/**************************************************************************************
>	String_256::String_256(UINT resID, UINT hinst = 0)

	Author:		JustinF
	Created:	12th August 1993
	Inputs:		A string resource numeric identifier, an optional resource module
				handle (if not supplied, the default resource module is used).
	Purpose:	Loads the string resource into this String.
	Errors:		ENSURE failure if this string is not long enough to hold the
				resource, or if the resource cannot be found in the module. If it cannot
				be found, a very short temp string is built of the form "<%X:%X>" containing
				the ID and the module number, or if that is too long then simply "!".
	Scope:		Public
	SeeAlso:	SmartLoadString
***************************************************************************************/

String_256::String_256(UINT resID, UINT hinst)
{
	*(text = fixedbuf) = 0;
	length = FIX_LEN_BUFSIZE;
	if (!Load(resID, hinst))
	{
		ERROR3("String resource failed to load");
		TCHAR buf[128];
		wsprintf(buf, TEXT("<%X:%X>"), (int) resID, (int) hinst);
		lstrcpy(text, (lstrlen(buf) <= 256) ? buf : TEXT("!"));
	}
}
#endif

String_256::String_256(const String_256 &other)
{
	text = &fixedbuf[0];
	length = FIX_LEN_BUFSIZE;
	if (this != &other)
	{
		ERROR3IF((const TCHAR*) other == 0, "StringBase to be copied has not been ALLOCated");
		memcpy((text = &fixedbuf[0]), &other.fixedbuf[0], FIX_LEN_BUFSIZE*sizeof(TCHAR));
	}
}

String_256 & String_256::operator=(const String_256& other)
{
	text = &fixedbuf[0];
	length = FIX_LEN_BUFSIZE;
	if (this == &other)
		return *this;

	ERROR3IF((const TCHAR*) other == 0, "StringBase to be copied has not been ALLOCated");
	memcpy((text = &fixedbuf[0]), &other.fixedbuf[0], FIX_LEN_BUFSIZE*sizeof(TCHAR));
	return *this;
}

/**************************************************************************************
>	virtual BOOL String_256::Alloc(int nSize)

	Author:		JustinF & Andy
	Created:	12th August 1993
	Purpose:	Fixes Alloc() for fixed-length strings, which CAN be safely changed#
				to use a buffer on the heap.
	Scope:		Public
	SeeAlso:	StringBase::Alloc
***************************************************************************************/

BOOL String_256::Alloc(int nSize)
{
	if (text == fixedbuf) text = 0;
	return StringBase::Alloc(nSize);
}

