/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// Header for the controlling class of the v2 file format

// This class has been implemented completely inline for speed reasons
// It is called from every WriteXXXX function in the CXaraFileRecord class...

#ifndef INC_FTTYPLIS
#define INC_FTTYPLIS

#include "fixmem.h"

typedef enum
{
	FTT_BYTE = 1,
	FTT_ULONG,			// 2
	FTT_LONG,			// 3
	FTT_UINT16,			// 4
	FTT_INT16,			// 5
	FTT_FLOAT,			// 6
	FTT_DOUBLE,			// 7
	FTT_BINHEX,			// 8
	FTT_WCHAR,			// 9
	FTT_ASCII,			// A
	FTT_UNICODE,		// B
	FTT_COORD,			// C
	FTT_INTCOORD,		// D
	FTT_REFERENCE,		// E
	FTT_MAXTYPE			// F
} FTType;

// Now a couple of macros to make life easier in the write functions

#define ENTERWRITEFUNCTION(type)							\
	BOOL bFTTLUnlock = FALSE;								\
	if (m_pTypeList)										\
		bFTTLUnlock = m_pTypeList->AddToListAndLock(type);

#define LEAVEWRITEFUNCTION		\
	if (bFTTLUnlock)			\
		m_pTypeList->Unlock();

#define STORELENGTH(len)				\
	((bFTTLUnlock) ? (m_pTypeList->StoreLength(len)) : (ok))

/********************************************************************************************

>	class FTTypeList : public CCObject

	Author:		Gerry
	Created:	10/07/97
	Purpose:	Object used by CXaraFileRecord for recording type information while writing

********************************************************************************************/

class FTTypeList
{
public:
	FTTypeList()
	{
		m_pArray = NULL;
		m_Used = 0;
		m_bLock = FALSE;
		m_Length = 0;
	};

	virtual ~FTTypeList()
	{
		if (m_pArray)
			CCFree(m_pArray);
	};

	DWORD GetCount(void) { return(m_Used); };

	BOOL StoreLength(DWORD len)
	{
		// Ensure 4 bytes of space (chances are this is the last thing to be written)
		if (!Enlarge(4))
			return(FALSE);

		DWORD* pLen = (DWORD*)(m_pArray + m_Used);
		*pLen = len;
		m_Used += 4;
		return(TRUE);
	};
	
	BOOL AddToListAndLock(FTType Type)
	{
		if (m_bLock)
			return(FALSE);

		// Add the type to the list

		if (m_Used == m_Length)
		{
			if (!Enlarge())
				return(FALSE);
		}

		// This must exist now
		m_pArray[m_Used] = Type;
		m_Used++;

		m_bLock = TRUE;
		return(TRUE);
	};

	void Unlock(void) { m_bLock = FALSE; };

	BYTE* GetBuffer(void) { return(m_pArray); };

protected:
	// Ensures at least Size entries available
	// FALSE if failed
	BOOL Enlarge(DWORD Size = 16)
	{
		if ((m_pArray == NULL) ||
			((m_Length - m_Used) < Size))
		{
			DWORD NewLength = m_Length + Size;
			BYTE* pNew = (BYTE*)CCRealloc(m_pArray, NewLength);

			if (pNew == NULL)
				return(FALSE);

			m_pArray = pNew;
			m_Length = NewLength;
		}
		return(TRUE);
	};

protected:
	DWORD m_Length;
	DWORD m_Used;
	BOOL m_bLock;
	BYTE* m_pArray;
};

#endif	// INC_FTTYPLIS
