/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /Camelot/kernel/listitem.h 18    18/04/97 16:25 Ranbir $

/*
 */ 

/**********************************************************************************************

>	class ListItem : public CCObject

	Author:		Mario Shamtani
	Created:	13/4/1993
	Base Class:	CObject
	Purpose:	
		A list item is an abstract class which is used to derive all objects 
		which are linked together in lists.  It has no member functions because
		all useful operations are defined in the List class.  The reason 
		for doing this is that list operations may need to alter the list 
		header.

	Errors:		None.

**********************************************************************************************/
/*
  Technical notes:
  
  When using lists with derived listitems it is necessary to explicitly cast from a listitem 
  pointer to the 'derived listitem' pointer.  
  
  e.g.  List x;
  		MyListItem* y;
        ....              
        y = x.GetHead();					// Wrong!!! need to cast to MyListItem* because
        									//    GetHead() returns a ListItem*   
        
        y = (MyListItem*) x.GetHead();  	// This will work honest!
                                             
  (This is only necesary when retrieving items from the list because type convertion from a 
   derived class to a base class is automatic.)
   
   
  You also have to do the cast if the function call is nested within another function call.
  
  e.g.	void Display(MyListItem*);
  		....	
        Display(x.GetHead());				// Error!
        ....
        Display((MyListItem*)x.GetHead());	// OK!
        
  
**********************************************************************************************/

#ifndef INC_LISTITEM
#define INC_LISTITEM

class ListItem
{
friend class List;								// The class list is a friend of 
												// ListItem 
friend class ActionList;                        // ActionList is also a friend

private:
	ListItem *Next;								// Pointer to NEXT ListItem in List
	ListItem *Previous;							//  "      "  PREVIOUS ListItem " "

public:
	ListItem();

	virtual ~ListItem() {}						// base classes must have a virtual destructor

	virtual ListItem *GetItem(void);			// For compatability with OrderedList
												// traversal methods

	virtual void ClearPointers();
};

#endif
