/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /camelot/winoil/MEMBLK.CPP 5     5/10/94 14:05 Andy $

// class to handle large memory blocks

/*
*/

#include "stdafx.h"
#include "errors.h"
//#include "andy.h"
//#include "simon.h"				// for IDS_CONTINUE
#include "memblk.h"
#include "ensure.h"
#include "camelot.h"

#ifndef WIN32
#define	MAX_BLOCK	0xFFFCL		// 64k-4
#endif

#if USE_VM_BLOCKS
ULONG MemoryBlock::PageSize;				// page size used in Virtual Mem API
#endif

/*********************************************************************************************

>	class MemoryBlock

	Author:		Andy
	Created:	3/3/94
	Purpose:	Some items start small and get large. The MemoryBlock class encapsulates
				memory management for objects that generally need to be fixed in memory
				but need to grow, and are generally large. Items that use the class
				include the handles table and the relocatable heap.
		
*********************************************************************************************/

/*********************************************************************************************

>	MemoryBlock::MemoryBlock()

	Author:		Andy
	Created:	3/3/94
	Inputs:		-
	Returns:	-
	Purpose:	Constructor. Initialises some fields to zero. Does not do any allocations.
		
*********************************************************************************************/

MemoryBlock::MemoryBlock()
{
#if USE_VM_BLOCKS
	MemBase = NULL;
#else
	MemHandle = 0;
#endif
}

/*********************************************************************************************

>	MemoryBlock::~MemoryBlock()

	Author:		Andy
	Created:	3/3/94
	Inputs:		-
	Returns:	-
	Purpose:	Destructor. Doesn't do anything really, though might do a DeInit one day.
	SeeAlso:	MemoryBlock::DeInit
		
*********************************************************************************************/

MemoryBlock::~MemoryBlock()
{
}

/*********************************************************************************************

>	LPVOID MemoryBlock::Init( LONG InitSize, BOOL bAutoZero, MemoryBlockType Type )

	Author:		Andy
	Created:	3/3/94
	Inputs:		InitSize	initial size of block required
				bAutoZero	TRUE if new memory required to be zeroed
				Type		one of MEMORYBLOCK_xxx constants. Determines maximum size of
							block.
	Returns:	Pointer to memory if worked, NULL if failed to initialise.
	Purpose:	Sets up block for subsequent allocations.
		
*********************************************************************************************/

LPVOID MemoryBlock::Init( LONG InitSize, BOOL bAutoZero, MemoryBlockType Type )
{
#if USE_VM_BLOCKS

	// first time we are called get the page size
	if (PageSize==0L)
	{
		SYSTEM_INFO SysInfo;
		GetSystemInfo( &SysInfo );
		PageSize = SysInfo.dwPageSize;
	}

	// Note that the Virtual API always gives us zeroed memory whether we want it or not


	// calculate a suitable maximum size for the type of block
	LONG LargestSize;

	switch (Type)
	{
		case MEMORYBLOCK_HANDLETABLE:
			LargestSize = 0x01000000;				// 16M of handles is enough
			break;
		case MEMORYBLOCK_RELOCHEAP:
			LargestSize = 0x10000000;				// 256M of reloc heap
			break;
		case MEMORYBLOCK_SCREENBUFFER:
			LargestSize = 0x01000000;				// 1024x768x32bits x4 for good measure
			break;
		default:
			ENSURE( FALSE, "Bad type in MemoryBlock::Init");
			return NULL;
	}

	// firstly reserve a gratuitously large chunk of address space

	MemBase = VirtualAlloc( NULL, LargestSize, MEM_RESERVE, PAGE_NOACCESS );
	if (MemBase==NULL)
	{
		if (IsUserName("Andy"))
			TRACE("VirtualAlloc(reserve) failed because %lx\n", GetLastError() );
		return NULL;
	}

	RoundedSize = GetRoundedSize( InitSize );

	// now physically grab the starting size
	LPVOID NewPtr = VirtualAlloc( MemBase, RoundedSize, MEM_COMMIT, PAGE_READWRITE );
	if (NewPtr==NULL)
	{
		if (IsUserName("Andy"))
			TRACE("VirtualAlloc(commit) failed because %lx\n", GetLastError() );
		VirtualFree(MemBase, 0L, MEM_RELEASE );
		MemBase = NULL;
		return NULL;
	}

	if (IsUserName("Andy"))
		TRACE("VirtualAlloced OK at %lx from %lx\n", NewPtr, MemBase );

	return NewPtr;
#else

	#ifndef WIN32
	if (InitSize > MAX_BLOCK)
		return NULL;
	#endif

	AutoZero = bAutoZero;											// remember flag in struct

	UINT Flags = GMEM_MOVEABLE;
	if (AutoZero)
		Flags |= GMEM_ZEROINIT;

	MemHandle = GlobalAlloc( Flags, InitSize );
TRACE("GlobalAlloc %u %lx returned %x\n", Flags, InitSize, (int)MemHandle);
	if (MemHandle==NULL)
		return NULL;

	LPVOID Ptr = GlobalLock( MemHandle );
	if (Ptr == NULL)
	{
		GlobalFree( MemHandle );
		MemHandle = 0;
		return NULL;
	}

	return Ptr;
#endif
}

/*********************************************************************************************

>	void MemoryBlock::DeInit()

	Author:		Andy
	Created:	3/3/94
	Inputs:		-
	Returns:	-
	Purpose:	Frees up any allocations on this memory block. Should always be called
				during exit for a tidy cleanup.
		
*********************************************************************************************/

void MemoryBlock::DeInit()
{
#if USE_VM_BLOCKS
	if (MemBase)
	{
		BOOL Result = VirtualFree(MemBase, 0L, MEM_RELEASE);
		MemBase = NULL;
		ENSURE( Result, "VirtualFree failed");
	}
#else
	if (MemHandle)
	{
		GlobalUnlock( MemHandle );
		GlobalFree( MemHandle );
		MemHandle = 0;
	}
#endif
}

/*********************************************************************************************

	static BOOL DoGrowGlobalHandle( HGLOBAL *pHandle, ULONG OldSize, ULONG NewSize, UINT MoreFlags, LPVOID *Pointer)

		Author:		Andy
		Created:	24/2/94
		Inputs:		pHandle		pointer to global handle whose block we want bigger, might be
								updated upon return
					OldSize		size it is at the moment
					NewSize		desired size of new block
					MoreFlags	extra flags to pass to GlobalReAlloc
					Pointer		where new pointer is to be stored
		Returns:	TRUE if worked, FALSE if failed
		Purpose:	Low-level private primitve for MemoryBlock::Grow. Actually does the
					work. Function does not exist if using VM_BLOCKs.
		SeeAlso:	MemoryBlock::Grow
		
*********************************************************************************************/

#if !USE_VM_BLOCKS

static BOOL DoGrowGlobalHandle( HGLOBAL *pHandle, ULONG OldSize, ULONG NewSize, UINT MoreFlags, LPVOID *Pointer)
{
	if (GlobalUnlock( *pHandle ))
	{
		// if didn't unlock then we won't be able to realloc, so better stop now
		TRACE("GlobalUnlock failed in GrowGlobalHandle\n");
		return FALSE;
	}

	// get Windows to grab us a new bigger block. Important to use GMEM_MOVEABLE as we want
	// our block to move if possible to satisfy the request
	HGLOBAL hNewHandle = GlobalReAlloc( *pHandle, NewSize, GMEM_MOVEABLE | MoreFlags );

	if (hNewHandle)
	{
		ADDR NewPtr = (ADDR)GlobalLock( hNewHandle );
		if (NewPtr == NULL)
		{
			// careful here as we really do want to get our old data back - at the moment
			// all we have is a handle to an unlocked block of memory. To try and recover the
			// situation we'll try setting it back to its old size
			hNewHandle = GlobalReAlloc( hNewHandle, OldSize, GMEM_MOVEABLE );
			if (hNewHandle)
			{
				NewPtr = (ADDR)GlobalLock( hNewHandle );
				if (NewPtr)
				{
					// OK, got old data back but might be in different place so tidy up
					// then exit gracefully
					*Pointer = NewPtr;
					*pHandle = hNewHandle;
					TRACE("WARNING: Made hasty exit from GrowGlobalHandle\n");
					return FALSE;									// indicate failure
				}
			}
			// this is very catastrophic - we cannot get our old data back, so
			// we will communicate this to the user. The program is likely to
			// crash and burn after this
			TRACE("DANGER: Catastrophic failure in GrowGlobalHandle\n");
			if (InformSeriousError( IDE_BAD_GROWHEAP, IDS_ABORT, IDS_CONTINUE ) == 1)
				AfxAbort();

			return FALSE;									// although system is v. unstable
		}

		if (IsUserName("Andy"))
			TRACE("Handle %x changed to %x (%lx-%lx) (%ld-%ld) %x\n",
					(int)*pHandle, (int)hNewHandle,
					(long)*Pointer, (long)NewPtr,
					(long)OldSize, (long)NewSize,
					(int)MoreFlags
				);

		// we've got our new block, update the callers variables
		*Pointer = NewPtr;
		*pHandle = hNewHandle;
		return TRUE;
	}

	// couldn't reallocate it, the old one will still be there though
	TRACE("GlobalReAlloc failed in GrowGlobalHandle\n");
	if (IsUserName("Andy"))
		TRACE("Failed to grow handle %x from %ld.%ld at %lx\n",
				(int)*pHandle,
				(long)OldSize,(long)NewSize,
				(long)*Pointer
			);

	return FALSE;

}
#endif


/*********************************************************************************************

>	BOOL MemoryBlock::Grow( ULONG OldSize, ULONG NewSize, LPVOID *Pointer )

		Author:		Andy
		Created:	24/2/94
		Inputs:		OldSize		size it is at the moment
					NewSize		desired size of new block
					Pointer		where new pointer is to be stored
		Returns:	TRUE if worked, FALSE if failed
		Purpose:	Like GlobalReAlloc in a way, but a safer version that does all it can
					to ensure existing data doesn't get thrown away. Handles unlocking,
					reallocing and relocking the result. Even if the return value is
					FALSE, the data might still have been moved - see if *Pointer has changed
					if this is important to you.
		SeeAlso:	DoGrowGlobalHeap;MemoryBlock::Shrink
		
*********************************************************************************************/

BOOL MemoryBlock::Grow( ULONG OldSize, ULONG NewSize, LPVOID *Pointer )
{
#if USE_VM_BLOCKS

	NewSize = GetRoundedSize( NewSize );				// round new size up
	if (NewSize == RoundedSize)
		return TRUE;									// if same physical size

	LPVOID NewPtr = VirtualAlloc( MemBase, NewSize, MEM_COMMIT, PAGE_READWRITE );
	if (NewPtr==NULL)
	{
		if (IsUserName("Andy"))
			TRACE("VirtualAlloc growing from %ld to %ld failed\n", OldSize, NewSize);
		return FALSE;
	}
	*Pointer = NewPtr;
	RoundedSize = NewSize;
	return TRUE;

#else
	const UINT Flags = AutoZero ? GMEM_ZEROINIT : 0;

	#ifndef WIN32
	if (NewSize > MAX_BLOCK)
		return FALSE;
	#endif

	return DoGrowGlobalHandle( &MemHandle, OldSize, NewSize, Flags, Pointer );
#endif
}

/*********************************************************************************************

>	BOOL MemoryBlock::Shrink( ULONG OldSize, ULONG NewSize, LPVOID *Pointer )

		Author:		Andy
		Created:	3/3/94
		Inputs:		OldSize		size it is at the moment
					NewSize		desired size of new block, smaller than OldSize
					Pointer		where new pointer is to be stored
		Returns:	TRUE if worked, FALSE if failed
		Purpose:   	Reduces the current allocation, without throwing away data. Normally
					this should always work, as giving memory back shouldn't ever fail,
					but a FALSE return means we couldn't. If a FALSE return is combined
					with a NULL Pointer return, it means an extremely bad thing occurred
					and the original data has been lost.
		SeeAlso:	MemoryBlock::Grow
		
*********************************************************************************************/

BOOL MemoryBlock::Shrink( ULONG OldSize, ULONG NewSize, LPVOID *Pointer )
{
#if USE_VM_BLOCKS

	// we do a shrink by freeing up spare blocks

	OldSize = GetRoundedSize( OldSize );
	NewSize = GetRoundedSize( NewSize );

	if (OldSize == NewSize)
		return TRUE;										// no change required

	if (OldSize < NewSize)
	{
		ENSURE(FALSE, "Cannot Shrink backwards");
		return FALSE;
	}

	// lets free up the unwanted bits now
	BOOL Result = VirtualFree( ((LPBYTE)MemBase)+NewSize, OldSize-NewSize, MEM_DECOMMIT );

	if (Result==FALSE)
	{
		if (IsUserName("Andy"))
			TRACE("VirtualFree Shrink from %ld to %ld failed\n", OldSize, NewSize);
		return FALSE;
	}

	RoundedSize = NewSize;
	return TRUE;

#else
	// the data will be left alone
	ENSURE( FALSE, "Cannot shrink memory blocks");
	return FALSE;

#endif
}
