/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/kernel/rechcomp.cpp 7     2/08/96 10:32 Graham $

// Implementation of the colour record handler class of the v2 file format

#include "stdafx.h"

#include "rechcomp.h"	// CompressionRecordHandler for handling compression for v2 native/web files

#include "cxftags.h"	// TAG_DEFINERGBCOLOUR TAG_DEFINECOMPLEXCOLOUR
#include "cxfrec.h"		// TAG_STARTCOMPRESSION TAG_STOPCOMPRESSION


#define new CAM_DEBUG_NEW


/********************************************************************************************

>	virtual ULONG* CompressionRecordHandler::GetTagList()

 	Author:		Neville Humphrys
	Created:	6/6/96
	Inputs:		-
	Returns:	Ptr to a list of tag values, terminated by CXFRH_TAG_LIST_END
	Purpose:	Provides the record handler system with a list of records handled by this
				handler
	SeeAlso:	-

********************************************************************************************/

ULONG* CompressionRecordHandler::GetTagList()
{
	static ULONG TagList[] = {TAG_STARTCOMPRESSION, TAG_ENDCOMPRESSION, CXFRH_TAG_LIST_END};

	return (ULONG*)&TagList;
}

/********************************************************************************************

>	virtual BOOL CompressionRecordHandler::HandleRecord(CXaraFileRecord* pCXaraFileRecord)

 	Author:		Neville Humphrys
	Created:	6/6/96
	Inputs:		pCXaraFileRecord = ptr to record to handle
	Returns:	TRUE if handled successfuly
				FALSE otherwise
	Purpose:	Handles the given record.
	SeeAlso:	-

********************************************************************************************/

BOOL CompressionRecordHandler::HandleRecord(CXaraFileRecord* pCXaraFileRecord)
{
	ERROR2IF(pCXaraFileRecord == NULL,FALSE,"CompressionRecordHandler::HandleRecord pCXaraFileRecord is NULL");
	
	// This should be unused and we should error if this is called
	ERROR2(FALSE,"CompressionRecordHandler::HandleRecord called! Use Streamed form instead");
	
	BOOL ok = TRUE;

	switch (pCXaraFileRecord->GetTag())
	{
		case TAG_STARTCOMPRESSION:
			// Ask the compression system to be turned on
			TRACEUSER("Neville","Turn compression on\n");
			ok = SetCompression(TRUE);
			break;
		case TAG_ENDCOMPRESSION:
			// Ask the compression system to be turned off
			TRACEUSER("Neville","Turn compression on\n");
			ok = SetCompression(FALSE);
			break;

		default:
			ok = FALSE;
			ERROR3_PF(("CompressionRecordHandler::HandleRecord I don't handle records with the tag (%d)\n",pCXaraFileRecord->GetTag()));
			break;
	}

	return ok;
}

/********************************************************************************************

>	virtual BOOL CompressionRecordHandler::HandleStreamedRecord(CXaraFile * pCXFile, ULONG Tag,ULONG Size,ULONG RecordNumber)

	Author:		Neville Humphrys
	Created:	7/6/96
	Inputs:		pCXFile = access to the CXaraFile class 
				Tag	 = the tag value
				Size = size of record
				RecordNumber = the record number in the file
	Returns:	TRUE if handled ok
				FALSE otherwise
	Purpose:	This is the streamed record handler. It is used when 
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CompressionRecordHandler::HandleStreamedRecord(CXaraFile * pCXFile, ULONG Tag,ULONG Size,ULONG RecordNumber)
{
	ERROR2IF(Tag <= 0,FALSE,"CompressionRecordHandler::HandleStreamedRecord Tag is negative!");
	
	BOOL ok = TRUE;

	switch (Tag)
	{
		case TAG_STARTCOMPRESSION:
			// Ask the compression system to be turned on
			TRACEUSER("Neville","Turn compression on\n");
			ok = SetCompression(TRUE);
			break;
		case TAG_ENDCOMPRESSION:
			// Ask the compression system to be turned off
			TRACEUSER("Neville","Turn compression off\n");
			ok = SetCompression(FALSE);
			break;

		default:
			ok = FALSE;
			ERROR3_PF(("CompressionRecordHandler::HandleStreamedRecord I don't handle records with the tag (%d)\n",Tag));
			break;
	}

	return ok;
}


/********************************************************************************************

>	virtual BOOL CompressionRecordHandler::IsStreamed(ULONG Tag)

	Author:		Neville Humphrys
	Created:	7/6/96
	Inputs:		The tag of the record
	Returns:	TRUE if this is a streamed record
				FALSE otherwise
	Purpose:	Function to find out if the record is streamed or not.
	Errors:		-
	SeeAlso:	-

********************************************************************************************/

BOOL CompressionRecordHandler::IsStreamed(ULONG Tag)
{
	// We don't want these records read in or handled by any outsiders as the ZLib code
	// reads things like the CRC and uncompressed size itself when compression is stopped
	return TRUE;
}

