/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// Defines the lists and list items used to manage tags

#include "list.h"
#include "listitem.h"
#include "cxftags.h"

/***********************************************************************************************

>	class AtomicTagListItem : public ListItem

	Author:		Markn
	Created:	16/8/96
	Purpose:	An item that appears in the AtomicTagList

***********************************************************************************************/

class AtomicTagListItem : public ListItem
{
public:
	AtomicTagListItem()				{ Tag = TAG_UNDEFINED; }
	AtomicTagListItem(UINT ThisTag) { Tag = ThisTag; }

	UINT GetTag() { return Tag; }
private:
	UINT Tag;
};

/***********************************************************************************************

>	class EssentialTagListItem : public ListItem

	Author:		Markn
	Created:	16/8/96
	Purpose:	An item that appears in the EssentialTagList

***********************************************************************************************/

class EssentialTagListItem : public ListItem
{
public:
	EssentialTagListItem()				{ Tag = TAG_UNDEFINED; }
	EssentialTagListItem(UINT ThisTag)	{ Tag = ThisTag; }

	UINT GetTag() { return Tag; }
private:
	UINT Tag;
};

/***********************************************************************************************

>	class TagDescriptionListItem : public ListItem

	Author:		Markn
	Created:	16/8/96
	Purpose:	An item that appears in the TagDescriptionList

***********************************************************************************************/

class TagDescriptionListItem : public ListItem
{
public:
	TagDescriptionListItem()									{ Tag = TAG_UNDEFINED; pStr = NULL; }
	TagDescriptionListItem(UINT ThisTag, String_256* pThisStr)	{ Tag = ThisTag; pStr = pThisStr; }

	~TagDescriptionListItem()	{ if (pStr != NULL) delete pStr; }
	
	UINT GetTag()			{ return Tag; }
	String_256* GetString()	{ return pStr; }

private:
	UINT		Tag;
	String_256* pStr;
};

/***********************************************************************************************

>	class AtomicTagList : public List

	Author:		Markn
	Created:	16/8/96
	Purpose:	Keeps a list of all the atomic tags present in the imported file.

				This only relates to new format files

				An atomic tag is a tag that represents the head of a sub-tree that should be
				taken as a single entity.
				E.g. a TextStory tag is an atomic tag because it, and all it's children, represent
				one object (same applies for mould & blend tags).

				If an importer reads a record that it doesn't understand, and it's been defined as
				an atomic record, then it should ignore the record AND ALL ITS CHILD RECORDS!

***********************************************************************************************/

class AtomicTagList : public List
{
public:
	AtomicTagList() {}

	void AddTail(AtomicTagListItem* pItem)				{ List::AddTail(pItem); };
	AtomicTagListItem* GetHead()						{ return (AtomicTagListItem*)List::GetHead(); }
	AtomicTagListItem* GetNext(AtomicTagListItem* pItem){ return (AtomicTagListItem*)List::GetNext(pItem); }
};


/***********************************************************************************************

>	class EssentialTagList : public List

	Author:		Markn
	Created:	16/8/96
	Purpose:	Keeps a list of all the essential tags present in the imported file.

				This only relates to new format files

				An essential tag is a tag that the importer must understand in order to successfully
				load the file.

				If the importer does not recognise the tag, and it's defined to be essential, the importer
				should abort the import.

				We have no examples of such a tag.  This mechanism is in place in case we wish to prevent
				old importers loading in newer versions of the file format.

***********************************************************************************************/

class EssentialTagList : public List
{
public:
	EssentialTagList() {}

	void AddTail(EssentialTagListItem* pItem)					{ List::AddTail(pItem); };
	EssentialTagListItem* GetHead()								{ return (EssentialTagListItem*)List::GetHead(); }
	EssentialTagListItem* GetNext(EssentialTagListItem* pItem)	{ return (EssentialTagListItem*)List::GetNext(pItem); }
};

/***********************************************************************************************

>	class TagDescriptionList : public List

	Author:		Markn
	Created:	16/8/96
	Purpose:	Keeps a list of all the tag descriptions present in the imported file.

				This only relates to new format files

				This will only come into use in newer versions of the file format.  It allows old importers
				to describe an unknown tag to the user.

***********************************************************************************************/

class TagDescriptionList : public List
{
public:
	TagDescriptionList() {}

	void AddTail(TagDescriptionListItem* pItem)						{ List::AddTail(pItem); };
	TagDescriptionListItem* GetHead()								{ return (TagDescriptionListItem*)List::GetHead(); }
	TagDescriptionListItem* GetNext(TagDescriptionListItem* pItem)	{ return (TagDescriptionListItem*)List::GetNext(pItem); }
};

