/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
// $Header: /Camelot/winoil/unicdman.cpp 2     30/11/00 12:45 Andy $
// 

#include "stdafx.h"
#include "UnicdMan.h"

#include "camelot.h"

// We want better memory tracking
#define new CAM_DEBUG_NEW


/********************************************************************************************
>	static inline BOOL UnicodeManager::IsDBCSLeadByte(BYTE TestChar)

	Author:		Peter Arnold
	Created:	26/7/96
	Inputs:		TestChar - the character to test
	Outputs:	-
	Returns:	TRUE if the character is a DBCS lead character, FALSE if not
	Purpose:	Platform independent way of testing for lead bytes.
	Note:		It's inline, in the header
********************************************************************************************/

/********************************************************************************************
>	static inline UINT UnicodeManager::ComposeMultiBytes(BYTE LeadChar, BYTE TrailChar)

	Author:		Peter Arnold
	Created:	25/7/96
	Inputs:		LeadChar	- Lead multichar byte
				TrailChar	- Trail multichar byte
	Outputs:	-
	Returns:	A composed UINT reperesenting the multibyte character
	Purpose:	One standard (correct!) way for composing the lead and trail bytes
	Note:		It's inline, in the header
********************************************************************************************/

/********************************************************************************************
>	static void UnicodeManager::DecomposeMultiBytes(UINT MBChar, BYTE* LeadChar, BYTE* TrailChar);

	Author:		Peter Arnold
	Created:	25/7/96
	Inputs:		MBChar	- the composed multibyte character
	Outputs:	LeadChar	- Lead multichar byte
				TrailChar	- Trail multichar byte
	Returns:	-
	Purpose:	One standard (correct!) way for decomposing the lead and trail bytes
********************************************************************************************/
void UnicodeManager::DecomposeMultiBytes(UINT MBChar, BYTE* LeadChar, BYTE* TrailChar)
{
	ERROR3IF(LeadChar==NULL || TrailChar==NULL, "Output pointers were NULL");

	if (LeadChar!=NULL || TrailChar!=NULL)
	{
		*LeadChar = (MBChar >> 8) & 0xFF;
		*TrailChar = MBChar & 0xFF;
	}
}



/********************************************************************************************
>	static UINT UnicodeManager::UnicodeToMultiByte(WCHAR UnicodeChar)

	Author:		Peter Arnold
	Created:	19/10/95
	Inputs:		UnicodeChar - a single unicode character
	Ouptuts:	-
	Returns:	A single multi-byte character value that represents UnicodeChar
	Purpose:	Converts a single unicode character into it's multi-byte equivelent.
				BUT what about two composed characters? Oh...
	Errors:		If the conversion fails then character 128 (box), or the system defualt is
				returned.
********************************************************************************************/
UINT UnicodeManager::UnicodeToMultiByte(WCHAR UnicodeChar)
{
	if (UnicodeChar < 128)
		return UnicodeChar;
		
	CHAR Return[20];
	LPSTR lpReturn = Return;
	int sizeReturn = sizeof lpReturn;

	// Get the active code page.
	// This is for Win95/98 systems which do not support unicode.
	UINT nCodePage = GetACP();
	ERROR3IF( (!IsValidCodePage(nCodePage)), "UnicodeManager::UnicodeToMultiByte - not a valid codepage" );

	// COnvert from Unicode
	if ( ::WideCharToMultiByte(nCodePage, WC_COMPOSITECHECK | WC_DEFAULTCHAR, &UnicodeChar, 1,
								lpReturn, sizeReturn, NULL, NULL) )
	{
		if (IsDBCSLeadByte(Return[0]))
			return ((unsigned char)Return[0] << 8) | (unsigned char)Return[1];
		else
			return (unsigned char)Return[0];
	}
	else
	{
		ERROR3("MultiByteToWideChar failed");
		return 128;
	}
}


/********************************************************************************************
>	static WCHAR UnicodeManager::MultiByteToUnicode(UINT MBChar)

	Author:		Peter Arnold
	Created:	19/10/95
	Inputs:		MBChar - a single multi-byte character
	Ouptuts:	-
	Returns:	The unicode value that represents MBChar
	Purpose:	Converts a single multi-byte character into it's unicode equivelent.
				BUT what about two composed characters? Oh...  Unimplemented as no one will
				cope with two chars being returned...
	Errors:		If the conversion fails then character 128 (box) is returned.
********************************************************************************************/
WCHAR UnicodeManager::MultiByteToUnicode(UINT MBChar)
{
	CHAR MBArray[3];
	WCHAR ReturnArray[4];

	// Convert UINT to DBCS
	if (IsDBCSLeadByte((MBChar>>8) & 0xFF))
	{
		MBArray[0] = (MBChar>>8) & 0xFF;
		MBArray[1] = MBChar & 0xFF;
		MBArray[2] = 0;
	}
	else
	{
		MBArray[0] = MBChar & 0xFF;
		MBArray[1] = 0;
	}
	
	// Get the active code page.
	// This is for Win95/98 systems which do not support unicode.
	UINT nCodePage = GetACP();
	ERROR3IF( (!IsValidCodePage(nCodePage)), "UnicodeManager::MultiByteToUnicode - not a valid codepage" );

	// Convert to Unicode
	if ( ::MultiByteToWideChar(nCodePage, MB_PRECOMPOSED, MBArray, -1, ReturnArray, 4) )
	{
		return ReturnArray[0];	// Composed chars then?
	}
	else
	{
		ERROR3("MultiByteToWideChar failed");
		/*DWORD dwError = GetLastError();
		//ERROR3_PF ("Error=%lx", dwError);
		char sError[20];
		sprintf (sError, "Error=%lx", dwError);
		ERROR3 (sError);*/
		return 128;
	}
}



/********************************************************************************************
>	BOOL UnicodeManager::IsUnicodeCompleteOS()

	Author:		Peter Arnold
	Created:	19/10/95
	Returns:	TRUE if running on a Unicode Complete OS
********************************************************************************************/
BOOL UnicodeManager::IsUnicodeCompleteOS()
{
	return IsWin32NT();
}



/********************************************************************************************
>	BOOL UnicodeManager::IsDBCSOS()

	Author:		Peter Arnold
	Created:	27/6/99
	Returns:	TRUE if running on a DBCS enabled OS
********************************************************************************************/
BOOL UnicodeManager::IsDBCSOS()
{
	return ::GetSystemMetrics(SM_DBCSENABLED);
}



/********************************************************************************************
>	static UINT UnicodeManager::GetFontCharSet()

	Author:		Peter Arnold
	Created:	31/7/96
	Inputs:		-
	Ouptuts:	-
	Returns:	UINT representing the font character set of the system
	Purpose:	Reads what value the system is using for the character sets of its fonts.
********************************************************************************************/
UINT UnicodeManager::GetFontCharSet()
{
	if (IsDBCSOS())
		return SHIFTJIS_CHARSET;
	else
		return ANSI_CHARSET;

//	UINT CharSet = 0;
//	
//	// This appears to be the only way of doing this.  It's ugly as we have to create a DC.
//	if (!IsWindows31())
//	{
//		CWnd* pWnd = CWnd::GetDesktopWindow();
//		if (pWnd!=NULL && pWnd->GetDC()!=NULL)
//			CharSet = ::GetTextCharset(pWnd->GetDC()->GetSafeHdc());
//	}
//
//	return CharSet;
}