/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
#ifndef INC_XARLIB
#define	INC_XARLIB

#include "cxftags.h"
#include "cxfrec.h"

// DocFlag bit constants
const ULONG  DOCFLAG_ALL_LAYER_VISIBLE  = 1 << 0;
const ULONG  DOCFLAG_MULTI_LAYER    = 1 << 1;
const ULONG  DOCFLAG_LEGACY_APTS    = 1 << 2;
// 1 << 3 not defined
const ULONG  DOCFLAG_ALL_PAGES_SAME   = 1 << 4;
const ULONG  DOCFLAG_SAME_PAGE_LAYERS  = 1 << 5;
const ULONG  DOCFLAG_PROMOTE_URLS   = 1 << 6;
const ULONG  DOCFLAG_SIM_WEB_LENGTH   = 1 << 7;
const ULONG  DOCFLAG_PHOTOSHOP_FORMAT  = 1 << 8;
const ULONG  DOCFLAG_CLIP_WEB_CONTENT  = 1 << 9;
const ULONG  DOCFLAG_EMPTY_LINES_SIGNIFICANT = 1 << 10;
const ULONG  DOCFLAG_FACING_PAGE    = 1 << 11;
const ULONG  DOCFLAG_EXPORT_HIRES   = 1 << 12;
const ULONG  DOCFLAG_WORD_LINE_SPACING  = 1 << 13;
const ULONG  DOCFLAG_ENABLE_ADJUST_PAGESIZE  = 1 << 14;
const ULONG  DOCFLAG_ENABLE_CHANGE_TEXT_SIZE = 1 << 15;
const ULONG  DOCFLAG_ADJUST_PAGESIZE_TYPE = 3 << 16;
const ULONG  DOCFLAG_ADJUST_PAGESIZE_TYPE_SHR= 16;

const LONG DOCFLAG_FACINGPAGES = DOCFLAG_FACING_PAGE;
const LONG DOCFLAG_EMPTYLINESSIGNIFICANT = DOCFLAG_EMPTY_LINES_SIGNIFICANT;
const LONG DOCFLAG_DONT_CLIPWEBCONTENT = DOCFLAG_CLIP_WEB_CONTENT;			// Inverted for back compatibility
const LONG DOCFLAG_PHOTOSHOPFORMATTING = DOCFLAG_PHOTOSHOP_FORMAT;
const LONG DOCFLAG_SIMULATEWEBLENGTHS = DOCFLAG_SIM_WEB_LENGTH;
const LONG DOCFLAG_PROMOTEURLS = DOCFLAG_PROMOTE_URLS;
const LONG DOCFLAG_SAMELAYERSONALLPAGES = DOCFLAG_SAME_PAGE_LAYERS;
const LONG DOCFLAG_ALLPAGESTHESAME = DOCFLAG_ALL_PAGES_SAME;
const LONG DOCFLAG_LEGACYAPTS = DOCFLAG_LEGACY_APTS;					// Legacy flag so that older programs get sensible behaviour
const LONG DOCFLAG_MULTILAYER = DOCFLAG_MULTI_LAYER;
const LONG DOCFLAG_ALLLAYERSVISIBLE = DOCFLAG_ALL_LAYER_VISIBLE;

typedef __int64 XLONG ;
typedef unsigned __int64 UXLONG ;

#if _MSC_VER < 1600
extern "C" INT64 __stdcall __emul(LONG,LONG) ;
#endif

#pragma intrinsic(__emul)
#define XMul(A,B) __emul(A,B)
#define XSqr(A)   __emul(A,A)
#define SMul(A,B) (LONG)(__emul(A,B)+0x8000>>16)

// ADDR is a general pointer type. Using void* would mean casts of every use so it is a 
// pointer to BYTE, to make address calculation marginally easier.
typedef BYTE* ADDR;

// This is the correct definition of MILLIPOINT
typedef LONG MILLIPOINT;

// Type for Tags (unique IDs)
typedef ULONG 	TAG;

class CXaraFileRecord;
class CXaraFile;
class CCDiskFile;
class CCMemFile;

typedef HRESULT (RecordHandler)(void*, CXaraFileRecord*);

class CXarImport
{
public:
	virtual ~CXarImport() {};
	virtual BOOL PrepareImport(TCHAR* pFileName) = 0;
	virtual BOOL PrepareImport(IStream* pStream) = 0;
	virtual BOOL PrepareImport(BYTE* pBuffer, ULONG Size) = 0;
	virtual BOOL SetHandler(void* pMagic, RecordHandler* pHandlerFunk) = 0;
	virtual BOOL DoImport(BOOL bDeleteRecordAfter = TRUE) = 0;
};

class CXarExport
{
public:
	virtual ~CXarExport() {};
	virtual BOOL StartExport(TCHAR* pFileName) = 0;
	virtual BOOL StartExport(IStream* pStream) = 0;
	virtual BOOL StartBufferedExport() = 0;
	virtual BOOL WriteRecord(CXaraFileRecord* pRecord, ULONG* pRecordNum = NULL) = 0;
	virtual BOOL WriteZeroSizedRecord(ULONG Tag, ULONG* pRecordNum = NULL) = 0;
	virtual BOOL GetExportBuffer(BYTE** ppBuffer, ULONG* pSize) = 0;
	virtual LONG GetRecordCount() = 0;
};

namespace XarLibrary
{
	CXarImport* CreateImporter();
	CXarExport* CreateExporter();
}

#endif
