/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /camelot/intel/XLONGASM.H 14    3/09/93 15:59 Andy $

#ifndef INC_XLONGASM
#define	INC_XLONGASM

// prottypes for assembly-language maths routines. This must be kept in snyc
// with the maths routines themselves, currently:
// xlongasm.asm

// Users should never directly include this file - use ccmaths.h only.


// beware that although references are used to pass some parameters,
// this may not always be the case. For example, if it is more efficient then
// some versions might just pass the parameters directly

// Note that the some reference parameters get modified by these calls
// Although this is contrary to the Style Guide, this is the only way we can alter
// the calling convention without breaking existing code


// These functions are private to the xlong class - no-one else should call them
// hence the use of xlong in lower-case

extern "C"
{

// these are dyadic fns which take two input args and return a third
	void XlongAdd( const xlong& ,const xlong& , xlong& );
	void XlongSub( const xlong& ,const xlong& , xlong& );
	void XMul64(   const xlong& ,const xlong& , xlong& );
	void XDiv64(   const xlong& ,const xlong& , xlong& );
	void XlongAsl64( const xlong& input, int shift, xlong& result);
	void XlongAsr64( const xlong& input, int shift, xlong& result);

// these are dyadic fns who pass a reference which is modified by the calls, hence
// the use of the One postfix on the names
	void XlongAddOne( const xlong& , xlong& );
	void XlongSubOne( const xlong& , xlong& );
	void XMul64One(   const xlong& , xlong& );
	void XDiv64One(   const xlong& , xlong& );
	void XlongNegOne( xlong& );
	void XlongAsl64One( int shift, xlong& result);
	void XlongAsr64One( int shift, xlong& result);

// xlong conversion functions
	void LongToXlong( const LONG, xlong& );
	LONG XlongToLong( const xlong& );
	double XlongToDouble( const xlong& );
	void DoubleToXlong( const double, xlong& );

// xlong comparison functions
	int XlongCmpGT( const xlong& first, const xlong& second);
	int XlongCmpGE( const xlong& first, const xlong& second);
	int XlongCmpLT( const xlong& first, const xlong& second);
	int XlongCmpLE( const xlong& first, const xlong& second);
	int XlongCmpEQ( const xlong& first, const xlong& second);
	int XlongCmpNE( const xlong& first, const xlong& second);

// xlong inc & dec functions
	void XlongIncrement( xlong& );
	void XlongDecrement( xlong& );

// utility - might be useful to others: should it be in the public block below
	void XMul32(LONG, LONG, xlong& );

// string conversions
	void XlongToAscii( const xlong&, LPSTR );
	void XlongToUnicode( const xlong&, LPWORD );


	void XlongByFixed16One( xlong*, fixed16 );

	
};



// these functions can be called by others
// although they are prototyped using xlong, params should always be passed as XLONG
// the lower-case version is to allow this file to be included correctly by ccmath.cpp

extern "C"
{
	LONG Mul32Div32( LONG, LONG, LONG );
#ifndef WIN32
	xlong Mul64Div32( xlong, xlong, LONG);
#endif
	int XMatrixCalc( fixed16, const xlong&, fixed16, const xlong&, xlong& );
};

#endif
