/* ================================XARAHEADERSTART===========================
 
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/

=================================XARAHEADEREND============================*/
// $Header: /Camelot/kernel/zinflate.h 4     29/08/96 11:35 Neville $

// This class contains all the file deflating code

#ifndef INC_ZIPINFLATE
#define INC_ZIPINFLATE

#include "zutil.h"

class ZStream;

/* Huffman code lookup table entry--this entry is four bytes for machines
   that have 16-bit pointers (e.g. PC's in the small or medium model).
   Valid extra bits (exop) are 0..13.  exop == -64 is EOB (end of block),
   exop == 16 means that v is a literal, exop < 0 means that v is a pointer
   to the next table, which codes -exop bits, and lastly exop == -128
   indicates an unused code.  If a code with exop == -128 is looked up,
   this implies an error in the data. */

//#if defined(STDC) || defined(sgi)
//typedef signed char Char;
//#else
typedef char Char; /* just hope that char is signed */
//#endif

// ------------------------------------------------------------------------------------------
// From inftrees.h
// ------------------------------------------------------------------------------------------

/* Huffman code lookup table entry--this entry is four bytes for machines
   that have 16-bit pointers (e.g. PC's in the small or medium model). */

typedef struct inflate_huft_s FAR inflate_huft;

struct inflate_huft_s {
  union
  {
    struct
	{
      Byte Exop;        /* number of extra bits or operation */
      Byte Bits;        /* number of bits in this code or subcode */
    } what;
    Bytef *pad;         /* pad structure to a power of 2 (4 bytes for */
  } word;               /*  16-bit, 8 bytes for 32-bit machines) */
  union
  {
    uInt Base;          /* literal, length base, or distance base */
    inflate_huft *Next; /* pointer to next level of table */
  } more;
};

/********************************************************************************************

********************************************************************************************/

/* And'ing with mask[n] masks the lower n bits */

static uInt inflate_mask[] = {
    0x0000,
    0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
    0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
};

// ------------------------------------------------------------------------------------------
// From inftrees.h
// ------------------------------------------------------------------------------------------

/********************************************************************************************
********************************************************************************************/

enum InflateCodesMode
	{   /* waiting for "i:"=input, "o:"=output, "x:"=nothing */
		START,    /* x: set up for LEN */
		LEN,      /* i: get length/literal/eob next */
		LENEXT,   /* i: getting length extra (have base) */
		DIST,     /* i: get distance next */
		DISTEXT,  /* i: getting distance extra */
		COPY,     /* o: copying bytes in window, waiting for space */
		LIT,      /* o: got literal, waiting for output space */
		WASH,     /* o: got eob, possibly still output waiting */
		END,      /* x: got eob and all data flushed */
		BADCODE   /* x: got error */
	};

/********************************************************************************************

>	class inflate_codes_state

	Author:		Neville Humprhys
	Created:	23/5/95
	Purpose:	An inflate codes state object for use by the inflater for the file compressor
				and decompressor.

********************************************************************************************/

class inflate_codes_state
{
public:
	inflate_codes_state() {};
	~inflate_codes_state() {};

public:
  /* mode */
  InflateCodesMode  mode;		/* current inflate_codes mode */

  /* mode dependent information */
  uInt len;
  union
  {
    struct
    {
      inflate_huft *tree;		/* pointer into tree */
      uInt need;				/* bits needed */
    } code;						/* if LEN or DIST, where in tree */

    uInt lit;					/* if LIT, literal */

    struct
    {
      uInt get;					/* bits to get for extra */
      uInt dist;				/* distance back to copy from */
    } copy;						/* if EXT or COPY, where and how much */

  } sub;						/* submode */

  /* mode independent information */
  Byte lbits;					/* ltree bits decoded per branch */
  Byte dbits;					/* dtree bits decoder per branch */
  inflate_huft *ltree;			/* literal/length/eob tree */
  inflate_huft *dtree;			/* distance tree */

};

// ------------------------------------------------------------------------------------------
// From infutil.h
// ------------------------------------------------------------------------------------------

/********************************************************************************************
********************************************************************************************/

enum InflateBlockMode
  		{
			TYPE,     /* get type bits (3, including end bit) */
			LENS,     /* get lengths for stored */
			STORED,   /* processing stored block */
			TABLE,    /* get table lengths */
			BTREE,    /* get bit lengths tree for a dynamic block */
			DTREE,    /* get length, distance trees for a dynamic block */
			CODES,    /* processing fixed or dynamic block */
			DRY,      /* output remaining window bytes */
			BLOCKDONE,/* finished last block, done */
			BLOCKBAD  /* got a data error--stuck here */
	  	};

/********************************************************************************************

>	class inflate_blocks_state

	Author:		Neville Humprhys
	Created:	23/5/95
	Purpose:	An inflate blocks state object for use by the inflater for the file compressor
				and decompressor.

********************************************************************************************/

class inflate_blocks_state
{
public:
	inflate_blocks_state() {};
	~inflate_blocks_state() {};

public:
	/* inflate blocks semi-private state */
    InflateBlockMode mode;	/* current inflate_block mode */

	/* mode dependent information */
	union {
		uInt left;			/* if STORED, bytes left to copy */
		struct
		{
		  uInt table;				/* table lengths (14 bits) */
		  uInt index;				/* index into blens (or border) */
		  uInt *blens;				/* bit lengths of codes */
		  uInt bb;					/* bit length tree depth */
		  inflate_huft *tb;			/* bit length decoding tree */
		} trees;			/* if DTREE, decoding info for trees */
		struct
		{
		  inflate_huft *tl, *td;    /* trees to free */
		  inflate_codes_state *codes;
		} decode;           /* if CODES, current state */
	} sub;					/* submode */
	uInt last;				/* true if this block is the last block */

  /* mode independent information */
  uInt bitk;            /* bits in bit buffer */
  uLong bitb;           /* bit buffer */
  Bytef *window;        /* sliding window */
  Bytef *end;           /* one byte after sliding window */
  Bytef *read;          /* window read pointer */
  Bytef *write;         /* window write pointer */
  check_func checkfn;   /* check function */
  uLong check;          /* check on output */
};

// ------------------------------------------------------------------------------------------
// From ?.c
// ------------------------------------------------------------------------------------------

/********************************************************************************************
********************************************************************************************/

/* current inflate mode */

enum InflateMode
  		{
			METHOD,   /* waiting for method byte */
			FLAG,     /* waiting for flag byte */
			DICT4,    /* four dictionary check bytes to go */
			DICT3,    /* three dictionary check bytes to go */
			DICT2,    /* two dictionary check bytes to go */
			DICT1,    /* one dictionary check byte to go */
			DICT0,    /* waiting for inflateSetDictionary */
			BLOCKS,   /* decompressing blocks */
			CHECK4,   /* four check bytes to go */
			CHECK3,   /* three check bytes to go */
			CHECK2,   /* two check bytes to go */
			CHECK1,   /* one check byte to go */
			DONE,     /* finished check, done */
			BAD       /* got an error--stay here */
		};

/********************************************************************************************

>	class InflateState

	Author:		Neville Humprhys
	Created:	23/5/95
	Purpose:	An inflate state object for use by the inflater for the file compressor
				and decompressor.

********************************************************************************************/

class InflateState
{
public:
	InflateState() {};
	~InflateState() {};

public:
	InflateMode    mode;	/* current inflate mode */

  /* mode dependent information */
  union {
    uInt method;		/* if FLAGS, method byte */
    struct
    {
      uLong was;		/* computed check value */
      uLong need;		/* stream check value */
    } check;			/* if CHECK, check values to compare */
    uInt marker;		/* if BAD, inflateSync's marker bytes count */
  } sub;				/* submode */

  /* mode independent information */
  int  nowrap;						/* flag for no wrapper */
  uInt wbits;						/* log2(window size)  (8..15, defaults to 15) */
  inflate_blocks_state *blocks;		/* current inflate_blocks state */

protected:

};

/********************************************************************************************

>	class ZipInflate

	Author:		Neville Humprhys
	Created:	23/5/95
	Purpose:	The inflater for the file compressor and decompressor.
				Allows the CCDiskFile to offer compression of files. At present just
				used to compress the native file format.

********************************************************************************************/

class ZipInflate
{
public:
	ZipInflate();
	~ZipInflate();

public:
	int Init(ZStream *strm);

	int inflate(ZStream *strm, int flush);

	int SetDictionary(ZStream *z, const Bytef *dictionary, uInt dictLength);

	int End(ZStream *strm);

	int Init(ZStream *strm, int  windowBits);

	int Sync(ZStream *strm);

	int Reset(ZStream *strm);

protected:
	// from infutil.h
	// copy as much as possible from the sliding window to the output area
	int inflate_flush(inflate_blocks_state *, ZStream *, int);

	// from infblocks.h
	inflate_blocks_state *inflate_blocks_new(ZStream *, check_func, uInt);

	int inflate_blocks(inflate_blocks_state *, ZStream *, int);

	void inflate_blocks_reset(inflate_blocks_state *, ZStream *, uLongf *);

	int inflate_blocks_free(inflate_blocks_state *, ZStream *, uLongf *);

	// New version 0.99
	void inflate_set_dictionary(
								inflate_blocks_state *s,
								const Bytef *d,  /* dictionary */
								uInt  n);       /* dictionary length */

	// from inftrees.h
#ifdef DEBUG
	  uInt inflate_hufts;
#endif

	int inflate_trees_bits(
	    uIntf *,                    /* 19 code lengths */
		uIntf *,                    /* bits tree desired/actual depth */
		inflate_huft * FAR *,       /* bits tree result */
	    ZStream *);					/* for zalloc, zfree functions */

	int inflate_trees_dynamic(
		uInt,                       /* number of literal/length codes */
		uInt,                       /* number of distance codes */
		uIntf *,                    /* that many (total) code lengths */
		uIntf *,                    /* literal desired/actual bit depth */
		uIntf *,                    /* distance desired/actual bit depth */
		inflate_huft * FAR *,       /* literal/length tree result */
		inflate_huft * FAR *,       /* distance tree result */
	    ZStream *);					/* for zalloc, zfree functions */

	int inflate_trees_fixed(
		uIntf *,                    /* literal desired/actual bit depth */
		uIntf *,                    /* distance desired/actual bit depth */
		inflate_huft * FAR *,       /* literal/length tree result */
		inflate_huft * FAR *);		/* distance tree result */

	int inflate_trees_free(
	    inflate_huft *,             /* tables to free */
	    ZStream *);					/* for zfree function */

	// inftrees.c
	int huft_build(
					uIntf *,            /* code lengths in bits */
					uInt,               /* number of codes */
					uInt,               /* number of "simple" codes */
					uIntf *,            /* list of base values for non-simple codes */
					uIntf *,            /* list of extra bits for non-simple codes */
					inflate_huft * FAR*,/* result: starting table */
					uIntf *,            /* maximum lookup bits (returns actual) */
				    ZStream *);        /* for zalloc function */

	//voidpf falloc(
	//				voidpf,         /* opaque pointer (not used) */
	//				uInt,               /* number of items */
	//				uInt);             /* size of item */


	// from inffast.h
	int inflate_fast(uInt, uInt, inflate_huft *, inflate_huft *,
				     inflate_blocks_state *, ZStream *);

	// from infcodes.h
	inflate_codes_state *inflate_codes_new( uInt, uInt,
										    inflate_huft *, inflate_huft *,
										    ZStream *);

	int inflate_codes(inflate_blocks_state *, ZStream *, int);

	void inflate_codes_free(inflate_codes_state *, ZStream *);

};

#endif  // INC_ZIPINFLATE


