#include "stdafx.h"
#include "TTCManager.h"
#include "sfnt.h"

/******************************************************************************
   Function:	TTCMatchFontProperties
   Author:		Paul Gruzdev
   Purpose:     Find a font in collection with the properties specified
   Returns:		int - the index of the font in the TTC that matches the properties
   Parameters:	std::vector<CTTCFontInfo> & info  
   Parameters:	[in] std::wstring strFontName  
   Parameters:	[in] int weight  
   Parameters:	[in] bool italic  
   Notes:       
******************************************************************************/
int CTTCManager::TTCMatchFontProperties(std::vector<CTTCFontInfo>& info, std::wstring strFontName, int weight, bool italic)
{
	for(size_t i = 0; i < info.size(); i++)
	{
		if(info[i].fontName == strFontName && info[i].weight == weight && info[i].italic == italic)
			return (int)i;
	}
	return 0;
}

static unsigned long lswap(BYTE* buf)
{
	unsigned long ch1 = *buf++;
	unsigned long ch2 = *buf++;
	unsigned long ch3 = *buf++;
	unsigned long ch4 = *buf++;
	return( (ch1<<24)|(ch2<<16)|(ch3<<8)|ch4 );
}

UINT memchecksum(PBYTE pbyData, UINT count);
/*static unsigned int CalcTableChecksum(PBYTE data, int data_sz)
{
	//assert((data_sz & 3) == 0); // must be 4 byte aligned

	unsigned int checksum = 0, word;
	for (int i=0; i<data_sz; i+=4)
	{
		word = ((data[i]   & 0xff) << 24) +
			((data[i+1] & 0xff) << 16) +
			((data[i+2] & 0xff) <<  8) +
			(data[i+3] & 0xff);
		checksum += word;
	}

	return checksum;
}*/

/******************************************************************************
   Function:	CTTCFontInfo::Decompose
   Author:		Paul Gruzdev
   Purpose:     Decompose TTC font name into font name, weight and italic flag
   Returns:		void
   Parameters:	std::wstring str
   Notes:       
******************************************************************************/
void CTTCFontInfo::Decompose(std::wstring str)
{
	std::wstring strlw = str;
	std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);

	size_t bpos = strlw.find(L" thin");
	if(bpos != std::wstring::npos)
	{
		weight = 100;
		str.erase(bpos, 5);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" extralight");
	if(bpos != std::wstring::npos)
	{
		weight = 200;
		str.erase(bpos, 11);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" light");
	if(bpos != std::wstring::npos)
	{
		weight = 300;
		str.erase(bpos, 6);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" regular");
	if(bpos != std::wstring::npos)
	{
		weight = 400;
		str.erase(bpos, 8);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" medium");
	if(bpos != std::wstring::npos)
	{
		weight = 500;
		str.erase(bpos, 7);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" semibold");
	if(bpos != std::wstring::npos)
	{
		weight = 600;
		str.erase(bpos, 9);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" extrabold");
	if(bpos != std::wstring::npos)
	{
		weight = 800;
		str.erase(bpos, 10);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L" black");
	if(bpos != std::wstring::npos)
	{
		weight = 900;
		str.erase(bpos, 6);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	bpos = strlw.find(L"bold");
	if(bpos != std::wstring::npos)
	{
		weight = 700;
		str.erase(bpos, 4);
		strlw = str;
		std::transform(strlw.begin(), strlw.end(), strlw.begin(), std::tolower);
	}

	size_t ipos = strlw.find(L"italic");
	if(ipos != std::wstring::npos)
	{
		italic = true;
		str.erase(ipos, 6);
	}

	for(std::wstring::iterator it = str.end()-1; it != str.begin(); --it)
	{
		if(*it == ' ')
			str.erase(it-str.begin(), 1);
		else
			break;
	}

	fontName = str;
}

/******************************************************************************
   Function:	GetTTCFontProps
   Author:		Paul Gruzdev
   Purpose:		
   Returns:		size_t
   Parameters:	CCFile * pFile
   Parameters:	std::vector<CTTCFontInfo> & info
   Notes:       
******************************************************************************/
size_t CTTCManager::GetTTCFontProps(CCFile* pFile, std::vector<CTTCFontInfo>& info)
{
	std::vector<std::wstring> names;
	std::vector<ULONG> offsets;

	if(SFNT::SFNTReader::NamesReadTTF(pFile, names, offsets))
	{
		info.clear();
		for(size_t i = 0; i < names.size(); i++)
		{
			CTTCFontInfo fi;
			fi.fontNameOriginal = names[i];
			fi.Decompose(names[i]);
			fi.offset = offsets[i];
			info.push_back(fi);
		}
		return info.size();
	}

	return 0;
}

/******************************************************************************
   Function:	CSplineFontBase::GetTTCFontProps
   Author:		Paul Gruzdev
   Purpose:     Given a True-Type Collection file, extract each font's name
				and an offset in the file
   Returns:		int
   Parameters:	char * pszFilename  
   Parameters:	std::vector<CTTCFontInfo> & info
   Notes:       
******************************************************************************/
size_t CTTCManager::GetTTCFontProps(TCHAR* pszFilename, std::vector<CTTCFontInfo>& info)
{
	CCDiskFile file;
	if(file.open(PathName(pszFilename), ios::in | ios::binary) == FALSE)
		return 0;

	size_t ret = GetTTCFontProps(&file, info);
	file.close();
	return ret;
}

/******************************************************************************
   Function:	CSplineFontBase::ExtractTTFFromTTC
   Author:		Paul Gruzdev
   Purpose:     Given the main offset, extract the TTF from TTC, i.e. all
				the tables specified in the main offset into a separate TTF file
   Returns:		bool
   Parameters:	char * pszTTC  
   Parameters:	char * pszTTF  
   Parameters:	unsigned long offset  
   Notes:       
******************************************************************************/
bool CTTCManager::ExtractTTFFromTTC(TCHAR* pszTTC, TCHAR* pszTTF, unsigned long offset)
{
	CCDiskFile in, out;
	if(in.open(PathName(pszTTC), ios::in | ios::binary) == FALSE)
		return false;

	if(out.open(PathName(pszTTF), ios::in | ios::out | ios::trunc | ios::binary) == FALSE)
		return false;

	bool bRet = ExtractTTFFromTTC(&in, &out, offset);
	in.close();
	out.close();
	return bRet;
}

/******************************************************************************
   Function:	ExtractTTFFromTTC
   Author:		Paul Gruzdev
   Purpose:		The main extract function
   Returns:		bool
   Parameters:	CCFile * pIn
   Parameters:	CCFile * pOut
   Parameters:	unsigned long offset
   Notes:       
******************************************************************************/
bool CTTCManager::ExtractTTFFromTTC(CCFile* pIn, CCFile* pOut, unsigned long offset)
{
	unsigned long		table_header_offset = offset;
	pIn->seek(offset);
	getlong(pIn);
	unsigned short		table_count = getushort(pIn);
	unsigned long		header_length = 0x0C + table_count * 0x10;
	unsigned long		table_length = 0;

	pIn->seek(table_header_offset);
	PBYTE pby = new BYTE[header_length];
	pIn->read(pby, header_length);
	pOut->write(pby, header_length);

	unsigned long	pad = 0;
	for (unsigned short j = 0; j < table_count; j++) 
	{
		unsigned long offset = lswap(&pby[0x0C + 0x08 + j * 0x10]);
		unsigned long length = lswap(&pby[0x0C + 0x0C + j * 0x10]);
		unsigned long current_offset = pOut->tell();
		pOut->seek(0x0C + 0x08 + j * 0x10);
		putlong(pOut, current_offset);
		pOut->seekIn(0, ios_base::end);

		PBYTE buf = new BYTE[length];
		pIn->seek(offset);
		pIn->read(buf, length);
		pOut->write(buf, length);

		pOut->write((void*)&pad, ((length + 3) & ~3) - length);

		unsigned int checksum = memchecksum(buf, length);
		pOut->seek(0x0C + 0x04 + j * 0x10);
		putlong(pOut, checksum);
		pOut->seekIn(0, ios_base::end);

		delete []buf;
	}

	delete []pby;
	return true;
}
