#pragma once

#include "GlyphSource.h"
#include "charset.h"

extern int g_iEM;
extern float g_fXaraEM;

#define OTF_EM_HEIGHT	1000
#define TTF_EM_HEIGHT	2048
#define XARA_EM_HEIGHT	72000.0f //144000.0f//8388607.0f
// This define makes floating point coordinates which leads to HUGE increase of the result font file, increasing
// coords accuracy just a bit
//#define xar2ttf(x) ((float)(x)*OTF_EM_HEIGHT/72000)
// This define rounds-up the coordinates to the nearest integer. Hugely decrease the output file size, the coords
// are in range -2048..2048
#define xar2ttf(x) ((float)rint((float)(x)*g_iEM/g_fXaraEM))
//#define xar2ttf(x) ((float)(x)*g_iEM/g_fXaraEM)
#define xar2ttf_float(x) ((float)(x)*g_iEM/g_fXaraEM)

#if _MSC_VER < 1600
	#define rint(x) ((x) > 0 ? ((int)((x)+0.5)) : ((int)((x)-0.5)))
#endif

#ifndef CHR
#define CHR(ch1,ch2,ch3,ch4) (((ch1)<<24)|((ch2)<<16)|((ch3)<<8)|(ch4))
#endif

struct OTLookup; // forward declaration
struct LookupSubTable;
struct GInfo;

// TrueType Composite glyph flags
#define _ARGS_ARE_WORDS	1
#define _ARGS_ARE_XY	2
#define _ROUND			4		// round offsets so component is on grid
#define _SCALE			8
// 0x10 is reserved
#define _MORE			0x20
#define _XY_SCALE		0x40
#define _MATRIX			0x80
#define _INSTR			0x100
#define _USE_MY_METRICS	0x200
#define _OVERLAP_COMPOUND	0x400	// Used in Apple GX fonts
// Means the components overlap (which? this one and what other?)
// Described in OpenType specs, not by Apple
// amusingly, Apple supports but MS does not
// MS says they support this after Win 2000
#define _SCALED_OFFSETS		0x800	// Use Apple definition of offset interpretation
#define _UNSCALED_OFFSETS	0x1000	// Use MS definition

int getchar(CCFile* pFile);
long getlong(CCFile* pFile);
int getushort(CCFile* pFile);
void putshort(CCFile* pFile, short int sval);
int get3byte(CCFile* pFile);
float get2dot14(CCFile* ttf);
void putlong(CCFile* pFile, int val);

class OTFDataLoader; // forward declaration

namespace SFNT
{
std::string GetGlyphName(int uCode);

enum ETableOrderType { keReference, keActual };
enum EFileFormat { keUndefined, keTrueType, keOpenType, keTTC, keWOFF, keType1, keCFF };

#define TABLEID_CFF		CHR('C', 'F', 'F', ' ')
#define TABLEID_GDEF	CHR('G', 'D', 'E', 'F')
#define TABLEID_GPOS	CHR('G', 'P', 'O', 'S')
#define TABLEID_GSUB	CHR('G', 'S', 'U', 'B')
#define TABLEID_OS2		CHR('O', 'S', '/', '2')
#define TABLEID_CMAP	CHR('c', 'm', 'a', 'p')
#define TABLEID_HEAD	CHR('h', 'e', 'a', 'd')
#define TABLEID_HHEA	CHR('h', 'h', 'e', 'a')
#define TABLEID_HMTX	CHR('h', 'm', 't', 'x')
#define TABLEID_LTSH	CHR('L', 'T', 'S', 'H')
#define TABLEID_VDMX	CHR('V', 'D', 'M', 'X')
#define TABLEID_HDMX	CHR('h', 'd', 'm', 'x')
#define TABLEID_MAXP	CHR('m', 'a', 'x', 'p')
#define TABLEID_NAME	CHR('n', 'a', 'm', 'e')
#define TABLEID_POST	CHR('p', 'o', 's', 't')
#define TABLEID_CVT		CHR('c', 'v', 't', ' ')
#define TABLEID_GASP	CHR('g', 'a', 's', 'p')
#define TABLEID_GLYF	CHR('g', 'l', 'y', 'f')
#define TABLEID_LOCA	CHR('l', 'o', 'c', 'a')
#define TABLEID_KERN	CHR('k', 'e', 'r', 'n')
#define TABLEID_FPGM	CHR('f', 'p', 'g', 'm')
#define TABLEID_PREP	CHR('p', 'r', 'e', 'p')
#define TABLEID_PCLT	CHR('P', 'C', 'L', 'T')
#define TABLEID_DSIG	CHR('D', 'S', 'I', 'G')
#define TABLEID_FFTM	CHR('F', 'F', 'T', 'M')
#define TABLEID_JSTF	CHR('J', 'S', 'T', 'F')

typedef std::vector<BYTE> ByteVector;
typedef std::list<std::pair<DWORD, CCMemFile*>> TableList;

class TableMap
{
public:
	typedef std::map<DWORD, CCMemFile*>::iterator iterator;

	size_t size() { return m_map.size(); }
	iterator begin() { return m_map.begin(); }
	iterator end() { return m_map.end(); }
	bool empty() { return m_map.empty(); }
	iterator find(DWORD id) { return m_map.find(id); }
	CCMemFile*& operator[](DWORD _Keyval) { return m_map[_Keyval]; }
	iterator erase(iterator it) { return m_map.erase(it); }

	void ClearTables()
	{
		for (TableMap::iterator it = m_map.begin(); it != m_map.end(); ++it)
		{
			if (it->second)
			{
				if (it->second->isOpen())
					it->second->close();
				delete it->second;
			}
		}
	}

	virtual ~TableMap()
	{
		ClearTables();
	}

	void Consume(TableMap& other)
	{
		m_map = other.m_map;
		other.m_map.clear();
	}

	void EraseTable(DWORD dwTableID)
	{
		iterator it = m_map.find(dwTableID);
		if (it != m_map.end())
		{
			if(it->second->isOpen())
				it->second->close();
			delete it->second;
			m_map.erase(it);
		}
	}

	iterator FindFirstNonPassed()
	{
		for (iterator it = m_map.begin(); it != m_map.end(); ++it)
			if(m_ActualPassed.find(it->first) == m_ActualPassed.end())
				return it;
			return m_map.end();
	}

public:
	std::map<DWORD, CCMemFile*> m_map;
	size_t m_iActualIdx;
	iterator m_itReference;
	std::set<DWORD> m_ActualPassed;
};

int _FeatureOrderId(bool isgpos, UINT32 tag);
CCMemFile* GetFirstTable(ETableOrderType type, TableMap& map, EFileFormat format, DWORD& dwTableID);
CCMemFile* GetNextTable(ETableOrderType type, TableMap& map, EFileFormat format, DWORD& dwTableID);

class OTLookupCompare
{
public:
	bool operator() (OTLookup* lhs, OTLookup* rhs) const;
};

class OTLookupFileCompare
{
public:
	bool operator() (const std::pair<OTLookup*, DWORD> lhs, const std::pair<OTLookup*, DWORD> rhs) const;
};

class OTLangCompare
{
public:
	bool operator() (UINT32 _ui1, UINT32 _ui2) const;
};

class CKernData
{
public:
	CKernData(int iGID_, MILLIPOINT mpAmt, LookupSubTable* pSub) : iGID(iGID_), wAmount(mpAmt), pSubTable(pSub) {}

	int iGID;				// TO value
	MILLIPOINT wAmount;			// kern amount
	LookupSubTable* pSubTable;	// Lookup sub table we belong to
	
	bool operator<(const CKernData& rhs) const { return this->iGID < rhs.iGID;  }
	bool operator==(const CKernData& rhs) const { return iGID == rhs.iGID && wAmount == rhs.wAmount; }
};

class CLigatureData
{
public:
	CLigatureData(int iLigGID_, LookupSubTable* pSub) : iLigGID(iLigGID_), pSubTable(pSub) {}
	CLigatureData(const CLigatureData& rhs) : iLigGID(rhs.iLigGID), pSubTable(rhs.pSubTable), to(rhs.to) {}
	void AddTo(int iTo) { to.push_back(iTo); }

	// bool operator==(const CLigatureData& rhs) const { return iLigGID == rhs.iLigGID && pSubTable == rhs.pSubTable; }
	bool operator<(const CLigatureData& rhs) const;
public:
	int iLigGID;			// Glyph index of the joined ligature
	std::vector<int> to;	// TO values of GIDs, forming the ligature
	LookupSubTable* pSubTable;
};

class CSubstituteData
{
public:
	CSubstituteData(int from_, int to_, LookupSubTable* pSub) : from(from_), to(to_), pSubTable(pSub) {}
	bool operator<(const CSubstituteData& rhs) const { return this->to < rhs.to; }
	int from;
	int to;
	LookupSubTable* pSubTable;
};

class CSubstRule
{
public:
	CSubstRule() {}
	CSubstRule(std::vector<std::wstring>& _pre, std::vector<std::wstring>& _post, std::vector<std::wstring>& _keys, OTLookup* ref, UINT16 _priority, std::vector<UINT32>& _langs)
		: pre(_pre), post(_post), keys(_keys), pRef(ref), priority(_priority), langs(_langs), pSubTable(0) {}

	std::vector<std::wstring> pre, post, keys;
	UINT16 priority;
	std::vector<UINT32> langs;
	LookupSubTable* pSubTable;
	OTLookup* pRef;
};

typedef std::multiset<OTLookup*, OTLookupCompare> LookupSet;
typedef std::map<std::pair<OTLookup*, DWORD>, CCFile*, OTLookupFileCompare> OTLookupFileMap;
typedef std::set<UINT32, OTLangCompare> OTLangSet;
typedef std::map<int, std::set<CKernData>> KernPairMap;
typedef std::map<int, std::set<CLigatureData>> LigatureMap;
typedef std::map<WCHAR, int> GIDMap;
typedef std::map<int, std::set<CSubstituteData>> SubstituteMap;
typedef std::map<ULONG, std::set<WCHAR>> GIDToCodeMap;

class CharProps
{
public:
	CharProps(LONG width, WCHAR uni) : lCharWidth(width), wUnicode(uni) { memset(&rcBBox, 0, sizeof(RECT)); }
	LONG lCharWidth;	// character advance width
	WCHAR wUnicode;
	RECT rcBBox;
};

class FontDict2Data 
{
public:
	int iEoDictMark = 0;
	int iFillinDictMark = 0;
	int iPrivateLen = 0;
};

class Spline; // forward declaration

class SFNTWriter : public VAddOpenTypeFeature
{
public:
	// gpos==0 => GSUB, gpos==1 => GPOS, gpos==-1 => both, gpos==-2 => Both & morx & kern
	enum ETableType { keBothAndExtra=-2, keBoth=-1, keGSUB=0, keGPOS=1 };

	SFNTWriter(VGlyphSource* pSource, EFileFormat format, VGlyphSource2* pSource2 = NULL);
	virtual ~SFNTWriter();

	// CFF writing methods
	size_t AddString(std::string str);
	void WriteCFFNames(CCFile* pFile, const char* pszFontName);
	void WriteCFFHeader(CCFile* pFile);
	void WriteSID(CCFile* pFile, std::string str, int oper);
	void WriteTopDict(CCFile* pFile);
	void WriteCIDTopDict(CCFile* pFile);
	void WriteStrings(CCFile* pFile);
	void WriteCharset(CCFile* pFile);
	// CID-specific functions
	static void CreateFDSelect(std::vector<BYTE>& vFDSelect, size_t szGlyphCount);
	static int DumpCFFDict(CCFile* pFile, int iSIDPostScriptName);
	static void DumpCFFDictIndex(CCFile* fdarray, std::vector<FontDict2Data>& vfds, int iSIDPostScriptName);
	static void CreateFDArray(CCMemFile* pFile, std::vector<BYTE>& vFDArray);
	// Main CFF writing function that uses the all above
	LONG WriteCFF(CCFile* pFile);
	LONG WriteCFFCID(CCFile* pFile);
	// CFF creation methods
	void CreateCharset();
	size_t GetCharsetSize() { return m_Charset.size()*sizeof(short int) + 1; }
	size_t GetStringsSize();
	size_t GetCharstringsSize();
	static size_t GetCharstringsSize(std::vector<std::vector<BYTE>>& vCharStrings, size_t& szTotalStringSize);
	void CreateNotDefPath(ByteVector& verb, std::vector<DocCoord>& coord, LONG& lWidth);
	bool CreateCharstrings();
	void CreatePrivate();
	void WriteCharstrings(CCFile* pFile);
	void static WriteCharstrings(CCFile* pFile, std::vector<std::vector<BYTE>>& vCharstrings, size_t szTotalCharstringsLength);
	void WriteGlyph(ByteVector& buf, LONG width, BYTE* verb, DocCoord* pCoord, UINT count, bool round);
	void SetWriteCID(bool b) { m_bWriteCID = b; }
	void SetFSType(UINT16 fsType) { m_fsType = fsType; }
	void moveto2(ByteVector& buf, LONG cx, LONG cy, bool round);
	UINT lineto2(ByteVector& buf, PBYTE verb, DocCoord* pCoord, UINT size, bool round);
	//UINT curveto2(ByteVector& buf, DocCoord dcPrevCP, DocCoord dcNextCP, DocCoord dcTo, bool round);
	UINT curveto2(ByteVector& buf, PBYTE verb, DocCoord* pCoord, UINT size, bool round);

	// General
	DWORD GetLastError() { return m_dwLastError; }
	bool CacheCharMetricsFindDefWidth();
	void SetPANOSE(CCPanose* pPanose) { memcpy(m_Panose, pPanose, 10); };
	void FigureCodePages(ULONG CodePage[2]);

	// sfnt tables writing
	LONG WriteUCS2(CCFile* pFile);
	LONG WriteCmap(CCFile* pFile);
	LONG WriteOS2(CCFile* pFile);
	LONG WriteHead(CCFile* pFile, bool bLocaIs32 = false);
	LONG WriteHhea(CCFile* pFile);
	LONG WriteHmtx(CCFile* pFile);
	LONG WriteName(CCFile* pFile);
	LONG WriteMaxp(CCFile* pFile);
	LONG WritePost(CCFile* pFile);
	// ttf-specific
	LONG WriteGASP(CCFile* pFile);
	LONG WriteCVT(CCFile* pFile);
	LONG WriteLOCA(CCFile* pFile);
	LONG DumpKern(CCFile* pFile);
	// top-level writing functions
	DWORD GenerateTables(TableMap& map);
	void WriteSFNT(CCFile* sfnt); // That's the main function that wraps all the above into one sfnt
	void WriteWOFF(CCFile* woff);
	void WriteWOFF(CCFile* woff, const TCHAR* pszInput);

	// GPOS generating functions
	LookupSubTable* SubTableFindOrMake(UINT32 tag, UINT32 script, int lookup_type);
	LookupSubTable* MakeSubTable(UINT32 tag, UINT32 script, int lookup_type);
	bool SFScriptsInLookups(ETableType type, std::set<UINT32>& scripts);
	bool SFLangsInScript(ETableType type, UINT32 script, OTLangSet& langs);
	bool SFFeaturesInScriptLang(ETableType type, UINT32 script, UINT32 lang, std::set<UINT32>& features);
	bool SFLookupsInScriptLangFeature(ETableType type, UINT32 script, UINT32 lang, UINT32 feature, std::vector<OTLookup*>& lookups);
	void FindFeatures(ETableType type, GInfo* ginfo);
	void GlyphsInSubtable(std::vector<int>& glyphs, LookupSubTable* pSub);
	void CreateSameAsTable(std::vector<int>& glyphs, std::vector<int>& sameAs);
	void DumpGPOSPairPos(CCFile* gpos, LookupSubTable* pSub);
	void DumpGSUBLigData(CCFile* gsub, LookupSubTable* pSub);
	void DumpGSUBSimpleSubs(CCFile* gsub, LookupSubTable* pSub);
	void dumpGSUBmultiplesubs(CCFile* gsub, LookupSubTable* pSub);
	void DumpGContextChainCoverage(CCFile* gsub, LookupSubTable* sub);
	void OTFDumpALookup(CCFile* pFile, OTLookup* otl);
	void WriteLookups(CCFile*pFile, bool bGPos);
	// Top level functions
	bool GatherKerningData();
	bool GatherLigatureData();
	void AddKerningPair(WORD wFirst, WORD wSecond, MILLIPOINT iKernAmount);
	virtual LookupSubTable* AddLigature(std::wstring key, WCHAR wLig, DWORD dwLigType);
	virtual LookupSubTable* AddStylisticSetSubstitute(WCHAR c1, WCHAR c2, DWORD dwSetNumber);
	virtual LookupSubTable* AddSubstitute(WCHAR c1, WCHAR c2, DWORD dwTableSubtype);
	void OrderedGlyphsFromString(std::wstring str, std::vector<int>& glyphs);
	virtual bool AddSubstRule(DWORD dwTableSubtype, std::vector<std::wstring>& pre, std::vector<std::wstring>& post, std::vector<std::wstring>& keys, OTLookup* lookup, UINT16 priority, std::vector<UINT32>& langs);
	void DumpTable(CCFile* pFile, ETableType type);
	// Helpers
	static std::wstring CreatePostScriptFontName(std::wstring str, LONG lWeight, bool bItalic);
	static std::wstring CreatePostScriptFontName2(std::wstring str, LONG lWeight, bool bItalic, bool bBold, bool addRegular);

	// TTF writing functions
	void DumpTTFSpace(CCFile* ttf);
	void DumpTTFGlyph(CCFile* pFile, BYTE* pVerb, DocCoord* pCoord, ULONG ulCount, RECT& rcGlyph);
	void DumpTTFBlankGlyph(CCFile* pFile);
	void DumpTTFMissingGlyph(CCFile* pFile, RECT rc);
	void DumpTTFGlyphs(CCFile* pFile);
	BYTE GetCvtIndex(WORD val);
	ULONG AddTTFPoints(BYTE* verb, DocCoord* pCoord, ULONG count, std::vector<DocCoord>& bp, std::vector<BYTE>& flags, RECT& rcBBox);

public:
	static void WriteTablesSFNT(CCFile* sfnt, EFileFormat format, TableMap& tableMap);
	static void WriteTablesWOFF(CCFile* woff, EFileFormat format, TableMap& tableMap);

public:
	// General members
	std::set<DWORD> m_sErrTable;				// Table(s) at which a write error happened.
	DWORD m_dwLastError;						// Last error, from errors.h
	std::vector<CharProps> m_CharProps;			// character properties array: width, unicode, bbox
	std::vector<FontDict2Data> m_fds;			// CID FontDict data array
	GIDMap m_CodeToGID;							// map Unicode --> glyph index in the font
	EFileFormat m_OutputFormat;				// current format we are writing out. Different tables change because of that
	// CFF members
	std::list<std::string> m_strings;			// CFF Strings, i.e. glyph names not from the standard name set
	std::list<size_t> m_Charset;				// array of SIDs for each character's name
	std::vector<std::vector<BYTE>> m_Charstrings; // this is an array of type-2 encoded charstrings for each glyph starting from GID=1
	std::vector<BYTE> m_Private;				// Private data encoded the way ready to write to CFF
	std::vector<BYTE> m_FDSelect;
	std::vector<BYTE> m_FDArray;
	size_t m_szTotalStringsLength;
	size_t m_szTotalCharstringsLength;
	// TTF writing members
	std::list<DWORD> m_PointCount;				// the number of points in each glyph (by GID)
	std::list<DWORD> m_loca;					// 'loca' contents i.e. offset of each glyph in a 'glyf' table
	std::vector<WORD> m_cvt;					// 'cvt ' contents, i.e. a set of numbers that we can address using  BYTE-sized index
	DWORD m_dwMaxPoints;						// maximum number of points that can appear in a TTF glyph for this font
	DWORD m_dwMaxContours;						// maximum number of contours that can appear in a TTF glyph

	// GPOS tables
	LookupSet m_GPOSLookups;
	LookupSet m_GSUBLookups;
	// This is the ordered map of kerning pairs in an order, suitable to be written into GPOS
	// Map format: Glyph index (ascending) --> ordered set of CKernData: [other Glyph index, kerning amount, subtable]
	// set of kern data is also ordered by ascending order of other (to) GID
	KernPairMap m_KPairs;
	ULONG m_ulPairCount; // total amount of kerning pairs added

	LigatureMap m_Ligatures;
	SubstituteMap m_Substitutes; // map of all single substitutes: i.e. 'ss05' --> set of pairs of glyph indexes
	std::map<LookupSubTable*, CSubstRule> m_Rules;
	std::map<UINT16, LookupSubTable*> m_IDToSubTable;


	// This font's properties
	std::string m_sVersion;
	std::string m_sCopyright;
	std::string m_sFamilyName;
	std::string m_sFullName;
	std::string m_sPostScriptName;
	std::string m_sWeight;
	bool m_bWriteCID = false; // Generate CID-keyed CFF font table
	UINT16 m_fsType = 0;
	bool m_bFixedWidth;
	double m_dItalicAngle;
	double m_dUnderlinePos;
	double m_dUnderlineWidth;
	LONG m_lAscent;
	LONG m_lDescent;
	LONG m_lDefaultWidth;		// Most frequent width in the font for all glyphs
	LONG m_lNominalWidth;		// the most frequent integral width distribution wise in -107..107 frame
	LONG m_lMaxAdvanceWidth;	// maximum width of all glyphs
	bool m_bBold;
	bool m_bItalic;
	WCHAR m_wFirstChar, m_wLastChar; // OS/2 first and last char fields, figured from enumerating the glyphs from GlyphSource
	// OS/2 table
	ULONG m_uUnicodeRange[4];
	LONG m_lAvgCharWidth3; // average char width for OS/2 table version 3 (OpenType)
	LONG m_lAvgCharWidth1; // average char width for OS/2 table version 1 (TrueType) - they are calculated differently!!
	LONG m_lWeight;
	LONG m_lXHeight;		// these are actually calculated from our glyphs passed
	LONG m_lCapHeight;
	BYTE m_Panose[10];
	// Running-ups while adding glyphs
	RECT m_rcFontBounds;	// an updating font's bounding box
	float m_fCurX;			// current coords
	float m_fCurY;
	wchar_t m_wGlyphBeingProcessed; // used for debug purposes only. Remove when the code is safe and clean.

	VGlyphSource* m_pSource;	// The virtual interface that comes as a source of glyph outlines and metrics
	VGlyphSource2* m_pSource2 = NULL;
private:
	// Debug offsets for various CFF tables. When writing them in DEBUG, we compare the calculated offset with the real one.
#ifdef _DEBUG
	int m_iCharsetOff;
	int m_iCharstringsOff;
	int m_iPrivateOff;
#endif
};

class Character // Glyph outlines read from 'glyf' or 'CFF ' tables
{
public:
	WCHAR wUnicode = 0;
	ULONG uPrevGID = 0; // when subsetted this is the original glyph index
	LONG lLeftSideBearing = 0;
	std::vector<BYTE> verb;
	std::vector<DocCoord> coord;
	std::vector<BYTE> instructions;
};

class SFNTReader : public VGlyphSource, public VGlyphSource2
{
public:
	class Head
	{
	public:
		Head() : m_bIsRead(false) {}
		bool Read(CCFile* pFile);
		bool IsRead() { return m_bIsRead; }
	public:
		bool m_bIsRead;
		UINT32 version;
		LONG   fontRevison;
		UINT32 checkSumAdjustment;
		UINT32 magicNumber;
		UINT16 flags;
		UINT16 unitsPerEm;
		LONG   createdTime[2];
		LONG   modifiedTime[2];
		INT16  xMin; //	For all glyph bounding boxes.
		INT16  yMin; //	For all glyph bounding boxes.
		INT16  xMax; //	For all glyph bounding boxes.
		INT16  yMax;
		UINT16 macStyle;
		UINT16 lowestRecPPEM;
		INT16  fontDirectionHint;
		INT16  indexToLocFormat;
		INT16  glyphDataFormat;
	};

	class Hhea
	{
	public:
		Hhea() : m_bIsRead(false) {}
		bool Read(CCFile* pFile);
		bool IsRead() { return m_bIsRead; }
	public:
		bool m_bIsRead;
		UINT16	majorVersion; //	Major version number of the horizontal header table  set to 1.
		UINT16	minorVersion; //	Minor version number of the horizontal header table  set to 0.
		INT16	Ascender; //	Typographic ascent(Distance from baseline of highest ascender).
		INT16	Descender; //	Typographic descent(Distance from baseline of lowest descender).
		INT16	LineGap; //	Typographic line gap.
		UINT16	advanceWidthMax; //	Maximum advance width value in 'hmtx' table.
		INT16	minLeftSideBearing; //	Minimum left sidebearing value in 'hmtx' table.
		INT16	minRightSideBearing; //	Minimum right sidebearing value; calculated as Min(aw - lsb - (xMax - xMin)).
		INT16	xMaxExtent; //	Max(lsb + (xMax - xMin)).
		INT16	caretSlopeRise; //	Used to calculate the slope of the cursor(rise / run); 1 for vertical.
		INT16	caretSlopeRun; //	0 for vertical.
		INT16	caretOffset; //	The amount by which a slanted highlight on a glyph needs to be shifted to produce the best appearance.Set to 0 for non - slanted fonts
		INT16	reserved1;
		INT16	reserved2;
		INT16	reserved3;
		INT16	reserved4;
		INT16	metricDataFormat; //	0 for current format.
		UINT16	numberOfHMetrics; //	Number of hMetric entries in 'hmtx' table
	};

	class OS2
	{
	public:
		OS2() : m_bIsRead(false) { memset(this, 0, sizeof(OS2)); }
		bool Read(CCFile* pFile);
		bool IsRead() { return m_bIsRead; }
	public:
		bool m_bIsRead;
		UINT16	version;
		INT16	xAvgCharWidth;
		UINT16	usWeightClass;
		UINT16	usWidthClass;
		UINT16	fsType;
		INT16	ySubscriptXSize;
		INT16	ySubscriptYSize;
		INT16	ySubscriptXOffset;
		INT16	ySubscriptYOffset;
		INT16	ySuperscriptXSize;
		INT16	ySuperscriptYSize;
		INT16	ySuperscriptXOffset;
		INT16	ySuperscriptYOffset;
		INT16	yStrikeoutSize;
		INT16	yStrikeoutPosition;
		INT16	sFamilyClass;
		UINT8	panose[10];
		UINT32	ulUnicodeRange1;	//	Bits 031
		UINT32	ulUnicodeRange2;	//	Bits 3263
		UINT32	ulUnicodeRange3;	//	Bits 6495
		UINT32	ulUnicodeRange4;	//	Bits 96127
		char	achVendID[4];		// The four character identifier for the vendor of the given type face.
		UINT16	fsSelection;
		UINT16	usFirstCharIndex;
		UINT16	usLastCharIndex;
		INT16	sTypoAscender;
		INT16	sTypoDescender;
		INT16	sTypoLineGap;
		UINT16	usWinAscent;
		UINT16	usWinDescent;
		UINT32	ulCodePageRange1;		//	Bits 031
		UINT32	ulCodePageRange2;	//	Bits 3263
		INT16	sxHeight;
		INT16	sCapHeight;
		UINT16	usDefaultChar;
		UINT16	usBreakChar;
		UINT16	usMaxContext;
		UINT16	usLowerOpticalPointSize;
		UINT16	usUpperOpticalPointSize;
	};

	class Maxp
	{
	public:
		Maxp() : m_bIsRead(false) {}
		bool Read(CCFile* pFile);
		bool IsRead() { return m_bIsRead; }

	public:
		bool m_bIsRead;
		LONG version; // Table version number	0x00010000 for version 1.0.
		UINT16	numGlyphs; //	The number of glyphs in the font.
		UINT16	maxPoints; //	Maximum points in a non-composite glyph.
		UINT16	maxContours; //	Maximum contours in a non-composite glyph.
		UINT16	maxCompositePoints; //	Maximum points in a composite glyph.
		UINT16	maxCompositeContours; //	Maximum contours in a composite glyph.
		UINT16	maxZones; //	1 if instructions do not use the twilight zone (Z0), or 2 if instructions do use Z0; should be set to 2 in most cases.
		UINT16	maxTwilightPoints; //	Maximum points used in Z0.
		UINT16	maxStorage; //	Number of Storage Area locations.
		UINT16	maxFunctionDefs; //	Number of FDEFs, equals to the highest function number + 1.
		UINT16	maxInstructionDefs; //	Number of IDEFs.
		UINT16	maxStackElements; //	Maximum stack depth2.
		UINT16	maxSizeOfInstructions; //	Maximum byte count for glyph instructions.
		UINT16	maxComponentElements; //	Maximum number of components referenced at top level for any composite glyph.
		UINT16	maxComponentDepth; //	Maximum levels of recursion; 1 for simple components.
	};

	struct HmtxRecord
	{
		UINT16 advance = 0;
		INT16 lsb = 0;
	};

public:
	SFNTReader() : m_pszFileName(0) {}
	SFNTReader(const TCHAR* pszFileName) : m_pszFileName(pszFileName) {}
	virtual ~SFNTReader() {}

	static EFileFormat GetFontType(TCHAR* filename);
	static std::wstring TTFGetFontName(CCFile* pFile, LONG offset, LONG off2, bool bPostScript = false);
	static std::wstring GetFontName(CCFile* pFile, bool bPostScript = false, LONG nameOffset = 0);
	static std::wstring GetNameString(CCFile* pFile, int nameIndex, LONG nameOffset = 0);
	static bool NamesReadTTF(CCFile* ttf, std::vector<std::wstring>& names, std::vector<ULONG>& offs);
	BOOL ReadSFNT();
	BOOL ReadSFNT(CCFile* pFile);
	TableMap& map() { return m_tableMap; }
	void ClearTables() { m_tableMap.ClearTables(); }

	bool HasEncoding(CCFile* pFile, int platformID, int encodingID);

	bool ReadOS2() { return m_OS2.Read(map()[TABLEID_OS2]); }
	bool ReadLoca(CCFile* pFile);
	bool ReadCmap(CCFile* pFile);
	bool ReadHmtx(CCFile* pFile);
	bool ReadGlyf(CCFile* pFile, CharSet* pSubset = NULL, std::set<ULONG>* pSubsetGID = NULL);

	bool ReadTTFGlyph(CCFile* pFile, size_t idx, std::map<size_t, size_t>& backGID, DWORD start, DWORD end, size_t gid);
	void ReadTTFCompositGlyph(CCFile* ttf, Character& sc, size_t idx, std::map<size_t, size_t>& backGID, DWORD end);

	bool Subset(CharSet& subset, std::set<ULONG>& subsetGID);
	bool SubsetTTF(CharSet& subset, std::set<ULONG>& subsetGID);
	bool SubsetOTF(CharSet& subset, std::set<ULONG>& subsetGID);
	bool ConvertToOTFAndSubset(const CharSet* pSubset, std::set<ULONG>* pSubsetGID, bool bCID);
	bool RepackOTF(const CharSet& sSubset, bool bCID);
	// Proxy function to subset only one CFF table
	bool SubsetCFF(CCMemFile* pMemFile, CCFile* pFile, CharSet& subset, std::set<ULONG>& subsetGID);
	UINT16 GetGlyphWidth(size_t szGlyphIdx) { return szGlyphIdx < m_hmtx.size() ? m_hmtx[szGlyphIdx].advance : 0; }
	bool TTFCreateGIDCompositeSubset(CharSet& subset, std::set<ULONG>& subsetGID, std::set<ULONG>& gids);
	bool GIDExists(ULONG uGID, CharSet& subset);

	// VGlyphSource interface implementation methods
	virtual bool IsGood() { return (m_chars.size() || m_pLoader) && m_head.IsRead() && m_OS2.IsRead(); }
	// This is written to string index=1 of the 'name' table
	virtual std::wstring GetFontFamilyName() { return GetNameString(map()[TABLEID_NAME], 1); }
	// This is written to string index=4 of the 'name' table
	virtual std::wstring GetFullFontName() { return GetNameString(map()[TABLEID_NAME], 4); }
	// This is written to string index=6 of the 'name' table. Optional: if not specified, fontlib will generate it
	virtual std::wstring GetPostScriptFontName() { return GetNameString(map()[TABLEID_NAME], 6); }
	virtual LONG GetWeight() { return m_OS2.usWeightClass; }
	virtual BOOL IsItalic() { return (m_OS2.fsSelection & 1) ? TRUE : FALSE; }

	virtual bool CalcBBox(DocCoord* pCoord, BYTE* pVerb, ULONG ulCount, RECT& rc);
	virtual GlyphState GetGlyphPath(WCHAR wUni, PBYTE* verb, DocCoord** coord, ULONG* pulNumCoords);
	virtual LONG GetCharWidth(WCHAR wUni);
	virtual LONG GetFontAscent();
	virtual LONG GetFontDescent();

	virtual VGlyphSource::iterator begin() { return m_uni.begin(); }
	virtual VGlyphSource::iterator end() { return m_uni.end(); }
	// Kerning pair support
	virtual bool GetFirstKerningPair(WORD& wFirst, WORD& wSecond, MILLIPOINT& iAmount) { return false; }
	virtual bool GetNextKerningPair(WORD& wFirst, WORD& wSecond, MILLIPOINT& iAmount) { return false; }
	// Open Type ligatures support
	virtual bool GetFirstLigature(std::wstring& key, wchar_t& wLigature, DWORD& dwLigType) { return false; }
	virtual bool GetNextLigature(std::wstring& key, wchar_t& wLigature, DWORD& dwLigType) { return false; }

	// VGlyphSource2 interface methods - these only available when the source was read from the font file
	virtual RECT GetFontBBox();
	virtual LONG GetFontCapHeight();
	virtual LONG GetFontXHeight();
	virtual LONG GetFontNominalWidth();
	virtual LONG GetFontDefaultWidth();
	virtual bool HasLocalSubs();
	virtual bool GetLocalSubs(PBYTE& pSubs, size_t& szSubSize);
	virtual bool GetStdHW(int& iStdHW);
	virtual bool GetStdVW(int& iStdVW);

public:
	EFileFormat m_format;
	const TCHAR* m_pszFileName;
	TableMap m_tableMap;
	// Tables
	Head m_head;
	Hhea m_hhea;
	Maxp m_maxp;
	OS2  m_OS2;
	GIDMap m_CodeToGID;					// 'cmap' contents
	GIDToCodeMap m_GIDToCode;			// 'cmap' back-map
	std::set<WCHAR> m_uni;
	std::vector<DWORD> m_loca;			// 'loca' contents i.e. offset of each glyph in a 'glyf' table
	std::vector<HmtxRecord> m_hmtx;		// 'hmtx' contents
	std::vector<Character> m_chars;		// 'glyf' or 'CFF ' contents
	OTFDataLoader* m_pLoader = NULL;	// we can attach an OTF loader here so the VGlyphSource can take glyphs from there

	// Debug offsets for various CFF tables. When writing them in DEBUG, we compare the calculated offset with the real one.
#ifdef _DEBUG
	int m_iCharsetOff;
	int m_iCharstringsOff;
	int m_iPrivateOff;
#endif
};

} // namespace CFF