// stdafx.h : include file for standard system include files,
// or project specific include files that are used frequently, but
// are changed infrequently
//

#pragma once

#ifndef _WIN32_WINNT            // Specifies that the minimum required platform is Windows Vista.
#define _WIN32_WINNT 0x0600     // Change this to the appropriate value to target other versions of Windows.
#endif

#include <assert.h>

#define VC_EXTRALEAN
#define WIN32_LEAN_AND_MEAN             // Exclude rarely-used stuff from Windows headers

#include <io.h>
#include <time.h>
#include <vector>
#include <array>
#include <list>
#include <map>
#include <unordered_map>
#include <set>
#include <string>
#include <sstream>
#include <queue>
#include <fstream>
#include <cctype>
#include <memory>

#ifdef CAMELOT_PRODUCT
#define CAMELOT_LOCK VERSION_PROTECTION_LOCK
#include "CamOilTypes.h"
#define INC_CAMTYPES

typedef enum BoundsTypeEnum
{
	BOUNDS_SIMPLE = 0,		// Just raw bounds of the geometry
	BOUNDS_ATTRS = 1,		// Bounds including the effects of any attributes
	BOUNDS_HITTEST = 2,		// Bounds for hit testing (should be fast)
	BOUNDS_BLOBS = 4,		// Unused: Intended to take over from GetBlobBoundingRect
	BOUNDS_EFFECTS = 8,		// Unused: Intended to take over from GetEffectsBoundingRect
	BOUNDS_SNAPPING = 16		// Unused: Intended to take over GetEorDragBoundingRect
} BoundsTypeEnum;

typedef UINT BoundsType;

#define TypeDef2(name1,name2)					\
	typedef const name2		     c##name1;		\
	typedef		  name2*	     p##name1;		\
	typedef		  name2&	     r##name1;		\
	typedef    c##name1*	    pc##name1;		\
	typedef    c##name1&	    rc##name1;		\
	typedef	   p##name1 const   cp##name1;		\
	typedef   pc##name1	const  cpc##name1;		\
	typedef	   p##name1*	    pp##name1;		\
	typedef   pc##name1*	   ppc##name1;		\
	typedef	  pp##name1 const  cpp##name1;		\
	typedef	 ppc##name1 const cppc##name1;

#define TypeDef(name) TypeDef2(name,name)

// Create lots of useful basic types

typedef UINT64 QWORD;

TypeDef(BOOL)
TypeDef(BYTE)
TypeDef(DOUBLE)
TypeDef(DWORD)
TypeDef(FLOAT)
TypeDef(INT)
TypeDef(INT8)
TypeDef(INT16)
TypeDef(INT32)
TypeDef(INT64)
TypeDef(LONG)
TypeDef(QWORD)
TypeDef(SBYTE)
TypeDef(SHORT)
TypeDef(UINT)
TypeDef(ULONG)
TypeDef(UINT8)
TypeDef(UINT16)
TypeDef(UINT32)
TypeDef(UINT64)
TypeDef(WORD)

typedef const void		 cVOID;
typedef		  void*	     pVOID;
typedef   cVOID*	    pcVOID;
typedef   pVOID const   cpVOID;
typedef  pcVOID const  cpcVOID;
typedef   pVOID*	    ppVOID;
typedef  pcVOID*	   ppcVOID;
typedef  ppVOID const  cppVOID;
typedef ppcVOID const cppcVOID;

// ADDR is a general pointer type. Using void* would mean casts of every use so it is a 
// pointer to BYTE, to make address calculation marginally easier.
typedef BYTE* ADDR;
class NodeRenderableBounded;
class DocColour;

#ifndef SAFE_DELETE
#define SAFE_DELETE(ptr) if (ptr) { delete ptr; ptr = NULL; }
#endif

// Type for Tags (unique IDs)
typedef ULONG 	TAG;

#include "camtrace.h"
#include "millipoint.h"
#include "ensure.h"
#include "binds.h"
// This must come after MILLIPOINT as it includes strings.h, which uses millipoints.
#include "ccmaths.h"						// as XMILLIPOINT needs XLONG
#include "ccobject.h"
#include "fixstr.h"
#include "varstr.h"
#include "doccoord.h"
#include "ccpanose.h"
#include "clikmods.h"
//#include "fontbase.h"
// this enum is used for both GPOS and GSUB subtables in font
// can be used for glyph substitution type identification
typedef enum GlyphSetEnum
{
	GSET_NONE = 0,
	GSET_ACCESS_ALL_ALTERNATES,
	GSET_ABOVE_BASE_FORMS,
	GSET_ABOVE_BASE_MARK,
	GSET_ABOVE_BASE_SUBSTITUTIONS,
	GSET_VERTICAL_FRACTIONS,
	GSET_AKHAND,
	GSET_ANCIENT_LIGATURES,
	GSET_BELOW_BASE_FORMS,
	GSET_BELOW_BASE_MARK,
	GSET_BELOW_BASE_SUBSTITUTIONS,
	GSET_CAPITALS_TO_PETITE_CAPITALS,
	GSET_CAPITALS_TO_SMALL_CAPITALS,
	GSET_CONTEXTUAL_ALTERNATES,
	GSET_CASE_SENSITIVE_FORMS,
	GSET_GLYPH_COMPOSITION_DECOMPOSITION,
	GSET_CONJUNCT_FORM_AFTER_RO,
	GSET_CONJUNCT_FORMS,
	GSET_CONTEXTUAL_LIGATURES,
	GSET_CENTERED_CJK_PUNCTUATION,
	GSET_CAPITAL_SPACING,
	GSET_CONTEXTUAL_SWASH,
	GSET_CURSIVE_ATTACHMENT,
	GSET_CHARACTER_VARIANTS_00,
	GSET_CHARACTER_VARIANTS_01,
	GSET_CHARACTER_VARIANTS_02,
	GSET_CHARACTER_VARIANTS_03,
	GSET_CHARACTER_VARIANTS_04,
	GSET_CHARACTER_VARIANTS_05,
	GSET_CHARACTER_VARIANTS_06,
	GSET_CHARACTER_VARIANTS_07,
	GSET_CHARACTER_VARIANTS_08,
	GSET_CHARACTER_VARIANTS_09,
	GSET_CHARACTER_VARIANTS_10,
	GSET_CHARACTER_VARIANTS_99,
	GSET_DROP_CAPS,
	GSET_DISTANCE,
	GSET_DISCRETIONARY_LIGATURES,
	GSET_DENOMINATORS,
	GSET_DIPTHONGS,
	GSET_DOTLESS_FORMS,
	GSET_EXPERT_FORMS,
	GSET_FINAL_GLYPH_ON_LINE,
	GSET_TERMINAL_FORMS_2,
	GSET_TERMINAL_FORMS_3,
	GSET_TERMINAL_FORMS,
	GSET_FLATTENED_ACCENTS_OVER_CAPITALS,
	GSET_DIAGONAL_FRACTIONS,
	GSET_FULL_WIDTHS,
	GSET_HALF_FORMS,
	GSET_HALANT_FORMS,
	GSET_ALTERNATIVE_HALF_WIDTHS,
	GSET_HISTORICAL_FORMS,
	GSET_HORIZONTAL_KANA_ALTERNATIVES,
	GSET_HISTORIC_LIGATURES,
	GSET_HANJA_TO_HANGUL,
	GSET_HOJO_KANJI_FORMS,
	GSET_HALF_WIDTHS,
	GSET_INITIAL_FORMS,
	GSET_ISOLATED_FORMS,
	GSET_ITALICS,
	GSET_JUSTIFICATION_ALTERNATIVES,
	GSET_JAPANESE_FORMS,
	GSET_JIS2004_FORMS,
	GSET_JIS78_FORMS,
	GSET_JIS83_FORMS,
	GSET_JIS90_FORMS,
	GSET_HORIZONTAL_KERNING,
	GSET_LEFT_BOUNDS,
	GSET_STANDARD_LIGATURES,
	GSET_LEADING_JAMO_FORMS,
	GSET_LINING_FIGURES,
	GSET_LOCALIZED_FORMS,
	GSET_MARK_POSITIONING,
	GSET_MEDIAL_FORMS_2,
	GSET_MEDIAL_FORMS,
	GSET_MATHEMATICAL_GREEK,
	GSET_MARK_TO_MARK,
	GSET_MARK_POSITIONING_VIA_SUBSTITUTION,
	GSET_ALTERNATE_ANNOTATION_FORMS,
	GSET_NLC_KANJI_FORMS,
	GSET_NUKTA_FORMS,
	GSET_NUMERATORS,
	GSET_OLDSTYLE_FIGURES,
	GSET_OPTICAL_BOUNDS,
	GSET_ORDINALS,
	GSET_ORNAMENTS,
	GSET_PROPORTIONAL_ALTERNATE_METRICS,
	GSET_LOWERCASE_TO_PETITE_CAPITALS,
	GSET_PROPORTIONAL_KANA,
	GSET_PROPORTIONAL_NUMBERS,
	GSET_PRE_BASE_FORMS,
	GSET_PRE_BASE_SUBSTITUTIONS,
	GSET_POST_BASE_FORMS,
	GSET_POST_BASE_SUBSTITUTIONS,
	GSET_PROPORTIONAL_WIDTH,
	GSET_QUARTER_WIDTHS,
	GSET_RANDOMIZE,
	GSET_RAKAR_FORMS,
	GSET_REQUIRED_LIGATURES,
	GSET_REPH_FORM,
	GSET_RIGHT_BOUNDS,
	GSET_RIGHT_TO_LEFT_ALTERNATES,
	GSET_RIGHT_TO_LEFT_MIRRORED_FORMS,
	GSET_RUBY_NOTATIONAL_FORMS,
	GSET_STYLISTIC_ALTERNATIVES,
	GSET_SCIENTIFIC_INFERIORS,
	GSET_LOWERCASE_TO_SMALL_CAPITALS,
	GSET_SIMPLIFIED_FORMS,
	GSET_STYLE_SET_1,
	GSET_STYLE_SET_2,
	GSET_STYLE_SET_3,
	GSET_STYLE_SET_4,
	GSET_STYLE_SET_5,
	GSET_STYLE_SET_6,
	GSET_STYLE_SET_7,
	GSET_STYLE_SET_8,
	GSET_STYLE_SET_9,
	GSET_STYLE_SET_10,
	GSET_STYLE_SET_11,
	GSET_STYLE_SET_12,
	GSET_STYLE_SET_13,
	GSET_STYLE_SET_14,
	GSET_STYLE_SET_15,
	GSET_STYLE_SET_16,
	GSET_STYLE_SET_17,
	GSET_STYLE_SET_18,
	GSET_STYLE_SET_19,
	GSET_STYLE_SET_20,
	GSET_SCRIPT_STYLE,
	GSET_SUBSCRIPT,
	GSET_SUPERSCRIPT,
	GSET_SWASH,
	GSET_TITLING,
	GSET_TRAILING_JAMO_FORMS,
	GSET_TRADITIONAL_NAME_FORMS,
	GSET_TABULAR_NUMBERS,
	GSET_TRADITIONAL_FORMS,
	GSET_THIRD_WIDTHS,
	GSET_UNICASE,
	GSET_ALTERNATE_VERTICAL_METRICS,
	GSET_VATTU_VARIANTS,
	GSET_VERTICAL_ALTERNATES,
	GSET_ALTERNATE_VERTICAL_HALF_METRICS,
	GSET_VOWEL_JAMO_FORMS,
	GSET_VERTICAL_KANA_ALTERNATES,
	GSET_VERTICAL_KERNING,
	GSET_PROPORTIONAL_ALTERNATE_VERTICAL_METRICS,
	GSET_VERTICAL_ROTATION_ALTERNATES,
	GSET_SLASHED_ZERO,
	GSET_REQUIRED_FEATURE
};

// maximum number of characters in a ligature - global limit
// increasing this may allow more complex ligatures to be supported, but formatting becomes
// less efficient, and Latin fonts rarely have ligatures of more than 3 characters anyway
#define LIGATURE_MAX_CHARS 3
// maximum size of stylistic sets
#define STYLISTIC_SETS_MAX_SIZE 20

typedef enum LigatureSubsetFlag
{
	LIG_SUB_NONE = 0x01,
	LIG_SUB_STANDARD = 0x02,
	LIG_SUB_HISTORIC = 0x04,
	LIG_SUB_DISCRETIONARY = 0x08,
	LIG_SUB_CONTEXTUAL = 0x10,
	LIG_SUB_ALL = (LIG_SUB_STANDARD | LIG_SUB_HISTORIC | LIG_SUB_DISCRETIONARY | LIG_SUB_CONTEXTUAL)
};

class LigatureEntry
{
public:

	BYTE KeyLth;
	std::wstring Key;
	WCHAR Ligature;
	LigatureSubsetFlag LigType;
	UINT16 Priority;
	DWORD GlyphID;

	LigatureEntry() : Ligature(0), KeyLth(0), LigType(LIG_SUB_NONE),
					  Priority(0), GlyphID(0) {}
};

// initial assumption that stylistic substitution
// won't be greater than ligature
class StylisticSetEntry
{
public:
	WCHAR Key;
	WCHAR StylisticChar;
	DWORD SubGlyphID;

	StylisticSetEntry() : StylisticChar(0), Key(0), SubGlyphID(0) {}
};

using StylisticSetDataSet = std::array<std::unordered_map<WCHAR, WCHAR>, 20>;
using StylisticSetNameSet = std::array<std::unordered_map<LANGID, std::string>, 20>;
using StylisticSetSizeSet = std::array<ULONG, 20>;

struct OS2Metrics
{
	BOOL bUseOS2Metrics{ FALSE };
	MILLIPOINT OS2Ascent{ 0 };
	MILLIPOINT OS2Descent{ 0 };
};

template <class T>
inline void CharHashCombine(std::size_t & seed, const T & v)
{
	std::hash<T> hasher;
	seed ^= hasher(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
}

namespace std
{
	template<> struct hash<std::vector<WCHAR>>
	{
		inline size_t operator()(const std::vector<WCHAR>& v) const
		{
			size_t seed = 13;

			for (auto& pChar : v)
				::CharHashCombine(seed, pChar);

			return seed;
		}
	};
}

typedef enum GlyphSubstitutionType
{
	GSUB_SINGLE = 0,
	GSUB_MULTIPLE,
	GSUB_LIGATURE
};

// Single / Multiple / Ligature substitution table
// for further use from substitution rules
class SubstitutionTable
{
public:
	UINT16 UID{ 0 };
	GlyphSubstitutionType SubstitutionType;
	// key chars to substitution chars & glyph IDs
	std::unordered_map<std::wstring, std::vector<std::pair<WCHAR, DWORD>>> SubstitutionMap;
};

// basic substitution rule:
// if any string of WCHARs precedes or follows
// then the substitution should be applied
class SubstitutionRule
{
public:
	// prefix vector
	std::vector<std::wstring> Pre;
	// sufix vector
	std::vector<std::wstring> Post;
	// key vector
	std::vector<std::wstring> Keys;
	// unique table ID
	UINT16 SubstitutionTableID{ 0 };
	// priority - if lower then higher priority we have
	UINT16 Priority{ 0 };
	// language set for given rule - if none, it will apply to any language
	std::vector<UINT32> Langs;
};

// max sizes for substitution rules
// will come in hand while constructing pre and post character lists
class RuleMaxSizes
{
public:
	UINT8 Pre{ 0 };
	UINT8 Key{ 0 };
	UINT8 Post{ 0 };
};

using SubstitutionRuleSet = std::unordered_map < GlyphSetEnum, std::pair<std::vector<SubstitutionRule>, RuleMaxSizes> >;
using SubstitutionTableSet = std::unordered_map < UINT16, SubstitutionTable >;

struct SubRuleInput
{
	std::vector<GlyphSetEnum> vecGlyphSet;
	std::vector<GlyphSetEnum> vecAuxGlyphSet;
	std::wstring vecPreChars;
	std::wstring vecKeyChars;
	std::wstring vecPostChars;
	GlyphSetEnum OutFeature;
	UINT32 Lang;
	INT16 Priority;
	SubRuleInput() : Lang(0), Priority(0), OutFeature(GSET_NONE) {};
};

#else
#include <atlbase.h>
#include <Windows.h>
#include <stdint.h>
#include "../XarLib_vc7/StdAfx.h"
#include "../XarLib_vc7/xarlib.h"

#define BUILD_TEXT_LIGATURES

struct MillipointKerningPair
{
	WORD		wFirst;
	WORD		wSecond;
	MILLIPOINT  iKernAmount;
};

#endif

#include "ccfile.h"

#if _MSC_VER < 1800		// MSVC version smaller than 12.0 (C++11) VS2013
#define nullptr NULL
#endif

typedef BYTE PathVerb;

#ifndef ASSERT
#define ASSERT assert
#endif