#pragma once

namespace SFNT
{

#define RE_NearZero	.00001
#define RE_Factor	(1024.0*1024.0*4.0)	// 23 bits => divide by 2^22

#define _Off_Curve	0
#define _On_Curve	1
#define _X_Short	2
#define _Y_Short	4
#define _Repeat		8
#define _X_Same		0x10
#define _Y_Same		0x20

bool floatNear(float a, float b);

class DPoint
{
public:
	DPoint() : x(0), y(0) {}
	DPoint(float x_, float y_) : x(x_), y(y_) {}
	DPoint(double x_, double y_) : x((float)x_), y((float)y_) {}
	DPoint(const DPoint& pt) : x(pt.x), y(pt.y) {}
	DPoint(const DocCoord& dc) : x((float)dc.x), y((float)dc.y) {}

	bool operator==(const DPoint& pt2) { return floatNear(x, pt2.x) && floatNear(y, pt2.y); }
	bool operator!=(const DPoint& pt2) { return !(*this == pt2); }

public:
	float x;
	float y;
};

class DBounds
{
public:
	DBounds() : minx(0), maxx(0), miny(0), maxy(0) {}

	float minx, maxx;
	float miny, maxy;
};

class QPoint
{
public:
	DPoint bp;
	DPoint cp;
	double t;
};

class Spline1D
{
public:
	float a, b, c, d;
};

class Spline
{
public:
	Spline() : from(0., 0.), to(0., 0.), cp1(0., 0.), cp2(0., 0.) { memset(splines, 0, sizeof(Spline1D)*2); }
	Spline(DPoint dpFrom, DPoint cp1_, DPoint cp2_, DPoint dpTo);
	bool IsLinear();
	void FindExtrema(const Spline1D* sp, double *_t1, double *_t2);
	bool MinMaxWithin();
	std::wstring Dump();

public:
	DPoint from;
	DPoint to;
	DPoint cp1, cp2;
	Spline1D splines[2];		// splines[0] is the x spline, splines[1] is y
};

} // namespace SFNT