/* ================================XARAHEADERSTART===========================
               PDF2XAR, a PDF file to XAR file converter
                    Copyright (C) 1993-2014 Xara Group Ltd.
       Copyright on certain contributions may be held in joint with their
              respective authors. 


LICENSE TO USE AND MODIFY SOFTWARE
----------------------------------

This file is part of PDF2XAR.

PDF2XAR is free software; you can redistribute it and/or modify it under the 
terms of the GNU General Public License version 2 or version 3 as published
by the Free Software Foundation.

PDF2XAR and its component source files are distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with PDF2XAR (see the file GPL in the root directory of the
distribution); if not, write to the Free Software Foundation, Inc., 51
Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

SCOPE OF LICENSE
----------------

This license applies to this program (PDF2XAR) and its constituent source
files only, and does not apply to other Xara products which may
in part share the same code base, and are subject to their own licensing
terms.

This license does not apply to the binary libraries (if any) within
the "libs" directory, which are subject to a separate license contained
within that directory in the file "LIBS-LICENSE".


ARRANGEMENTS FOR CONTRIBUTION OF MODIFICATIONS
----------------------------------------------

If you wish to contribute modifications please contact Xara via 
www.xara.com/contact

Please note that Xara will not accept modifications which modify any of
the text between the start and end of this header (marked
XARAHEADERSTART and XARAHEADEREND).

MARKS
-----

Xara, PDF2XAR, Xara Web Designer, Xara Photo & Graphic Designer, Xara Page & 
Layout Designer, Xara Designer Pro and Xara designs are trademarks, design-marks, 
and/or service marks of Xara Group Ltd. All rights in these marks are reserved.


      Xara Group Ltd, Gaddesden Place, Hemel Hempstead, HP2 6EX, UK.
                        http://www.xara.com/
=================================XARAHEADEREND============================*/
#include "stdafx.h"
#include <string.h>
#include <assert.h>
#include <sstream>
#include <fstream>
#include <ostream>
#include <iostream>
#include <algorithm>
#include <utility>
#include <math.h>
#include <set>
#include <cctype>
#include <cfloat>
#include <tuple>
#include "resource.h"
#include "XarOutputDev.h"
#include "common/vcFile.h"
#include "TableUnicodeMask.h"
//#include "../../../XarLib_vc7/ccpanose.h"
#include "../../../XarLib_vc7/cxfdash.h"
#include "../../poppler/poppler/Gfx.h"
#include "Type3GlyphVectorizer.h"

#define UNDERLINE_WIDTH_RATIO		1 / 20
#define UNDERLINE_DISTANCE_RATIO	2 * UNDERLINE_WIDTH_RATIO
#define STRIKETHROUGH_DISTANCE_RATIO	1 / 4

#define MAX_RELATIVE_OFFSET 125
#define MAX_CENTER_ALIGN_RELATIVE_OFFSET 400
#define TRACKING_TOLERANCE 6000 //5000 //3000 //1800 //700 //400 //125
#define	MAX_ABSOLUTE_OFFSET 0 // 1000
#define MAX_LEFT_RELATIVE_OFFSET 100//50//10
#define LINESPACE_DELTA 0.1 // 5e-2
#define LINESPACE_RATIO_DELTA 0.10//0.05 //0.015
#define MIN_DROPCAP_SIZE 2.5

#if (defined(_DEBUG) || defined(_EXPLICIT_RD_LOGGING))
#define RD_LOG(a) ss << a
#define RD_LOG2(a) ss2 << a
#else
#define RD_LOG(a) 
#define RD_LOG2(a)
#endif

#define XXX(a) std::cout << "\nXXX: " << std::flush << a << std::flush

#if (defined(_DEBUG) || defined(_EXPLICIT_RD_LOGGING))
	#define LOG_EOL_WCOUT(a) std::wcout << a << std::flush
#else
	#define LOG_EOL_WCOUT(a)
#endif

const double SMALL_KERNING_MAX_FOR_ALIGN_LEFT = 250.;      // maximum kerning value to be replaced by tracking (if enabled)
//const double SMALL_KERNING_MAX_FOR_ALIGN_FULL = 155.;
const double MAX_OFFSET_CUTOFF_KERNING_REPLACEMENT = 4.; // maximum offset (*fontsize)  before disabling kerning replacement for line

AffineTransform GetTextMatrix(GfxState *state, AffineTransform &matrixInv, double &xScaleNorm, double &yScale, double &xText, double &yText, double &shearFactor)
{
	GfxFont *font = state->getFont();
	double fontSize = state->getFontSize();
	double hScale = state->getHorizScaling();

	bool bType3 = font && fontType3 == font->getType();

	if (bType3)
	{
		const double *fontMatrix = font->getFontMatrix();
		const double *fontBBox = font->getFontBBox();

		double scale = (fontBBox[3] - fontBBox[1]) * fontMatrix[3];
		fontSize *= scale;
		if (scale < 0)
			hScale = -hScale;
	}

	AffineTransform matrix;

	const double *m = state->getCTM();
	AffineTransform matrixCtm(m[0], m[1], m[2], m[3], m[4], m[5]);

	/*if (bType3)
	{
		AffineTransform matrixText(1, 0, 0, -1, 0, 0);

		matrix = matrixText * matrixCtm;
	}
	else*/
	{
		double sx = 1, sy = 1;
		if (fontSize < 0)
			sx = sy = -1;
		if (hScale < 0)
			sx = -sx;
		m = state->getTextMat();	
		AffineTransform matrixText(sx * m[0], sx * m[1], sy * m[2], sy * m[3], state->getCurX(), state->getCurY() + state->getRise()/*m[4], m[5]*/);

		matrix = matrixText * matrixCtm;
	}

	double ys = matrix.yScale();
	double xs = matrix.xScale();

	matrix.scaleNonUniform(1./xs, 1./ys);

	xScaleNorm = xs/ys;
	yScale = ys;

	xText = matrix.e();
	yText = matrix.f();
	matrix.setE(0);
	matrix.setF(0);

	shearFactor = 0;

	if (!bType3)
	{
		double axisDotProduct = matrix.a() * matrix.c() + matrix.b() * matrix.d();
		if (axisDotProduct >= 0.1 && axisDotProduct <= 0.3)
		{
			double axisCrossProduct = matrix.a() * matrix.d() - matrix.b() * matrix.c();
			if (axisCrossProduct < 0)
			{
				matrix.setC(matrix.b());
				matrix.setD(-matrix.a());
			}
			else
			{
				matrix.setC(-matrix.b());
				matrix.setD(matrix.a());
			}

			shearFactor = axisDotProduct;
		}
	}

	matrixInv = matrix.inverse();
	matrixInv.map(xText, yText, xText, yText);

	return matrix;
}

//----- update text state
void XarOutputDev::updateFont(GfxState *state)
{
	std::stringstream ss;
	RD_LOG("ref=" << state->getFont()->getID()->num << " size=" << state->getFontSize());
	RD_ARG(ss.str());
}

void XarOutputDev::updateTextMat(GfxState * state)
{RD;
	TextReconstruct();
}

void XarOutputDev::updateTextPos(GfxState * state)
{RD;
	TextReconstruct();
}

void XarOutputDev::TextLine::GetText(const TextChars &chars, std::wstring &text)
{
	size_t c = text.length() + chars.size();
	if (text.capacity() < c) text.reserve(c);

	for (TextChars::const_iterator it = chars.begin(); it != chars.end(); ++it)
		text.push_back(it->c);
}

bool XarOutputDev::TextLine::Append(TextChars &chars)
{
	static const double DELTA = 1e-4;

#if defined(_DEBUG)
	std::wstring text;
	TextLine::GetText(chars, text);
#endif

	if (!empty())
	{
		MILLIPOINT baseline = _round(back().y);
		MILLIPOINT prevAscent = baseline + back().GetFontAscent();
		MILLIPOINT prevDescent = baseline + back().GetFontDescent();

		baseline = _round(chars.front().y);
		MILLIPOINT nextAscent = baseline + chars.front().GetFontAscent();
		MILLIPOINT nextDescent = baseline + chars.front().GetFontDescent();

		double dPrevFontSize = abs(back().GetFontSize()/*attrs.fontSize*/);
		double dNextFontSize = abs(chars.front().GetFontSize()/*attrs.fontSize*/);

		double dMinFontSize = min(dPrevFontSize, dNextFontSize);
		double dMaxFontSize = max(dPrevFontSize, dNextFontSize);

		double dx = chars.front().x - (back().x + back().GetWidth());

		if (dx < -1.0 * dMaxFontSize/*max(abs(back().GetFontSize()), abs(chars.front().GetFontSize()))*/)// || dx > 6.0 * dMaxFontSize)
			return false;

		//if ((back().y <= nextDescent || back().y >= nextAscent) &&
		//    (chars.front().y <= prevDescent || chars.front().y >= prevAscent))
		//	return false;

		double h1 = min(prevAscent, nextAscent) - max(prevDescent, nextDescent);
		double h2 = min(prevAscent - prevDescent, nextAscent - nextDescent);

		if (fabs(dx) < 0.005 * dMaxFontSize) // YoshinoHreplicable.pdf, pages 49-50
		{
			if (h1 < 0.25 * h2)
				return false;
		}
		else if (dx < dMaxFontSize)
		{
			if (h1 < 0.7 * h2)
				return false;
		}
		else if (dx < 4 * dMaxFontSize)
		{
			if (h1 < 0.6 * h2)
				return false;
		}
		else
		{
			if (back().y != chars.front().y)
				return false;
		}

		if (dMinFontSize < DELTA)
			return false;

		double fontDiffFactor = dMaxFontSize / dMinFontSize;

		//if (back().attrs.pFontDesc != chars.front().attrs.pFontDesc)
		//	fontDiffFactor *= 1.15;

		if (fontDiffFactor >= MIN_DROPCAP_SIZE && fabs(dx) > 1.0 * max(abs(back().GetFontSize()), abs(chars.front().GetFontSize())))
			return false;
	}

	splice(end(), chars);

	if (!chars.bbox.empty())
	{
		if (bbox.empty())
			bbox = chars.bbox;
		else
			Rect::Union(bbox, bbox, chars.bbox);
	}

	return true;
}

void XarOutputDev::Append(TextStory& textStory, TextChars &chars)
{
	if (textStory.lines.empty())
		textStory.lines.push_back(TextLine());

	if (!textStory.lines.back().Append(chars))
	{
		if (chars.bbox.empty()) return;

		// DDES-15137/DDES-1464 fix
		bool isNumberedListHead = true;

		auto itChar = chars.begin();

		while (chars.end() != itChar && (itChar->c >= '0' && itChar->c <= '9' || '.' == itChar->c)) ++ itChar;
		while (chars.end() != itChar)
		{
			if (0 == (UNICODE_MASK_INVISIBLE & itChar->GetUnicodeMask()) || itChar->isAnchorPos)
			{
				isNumberedListHead = false;
				break;
			}

			++itChar;
		}
	
		if (isNumberedListHead)
		{
			TextLines::reverse_iterator itLine = textStory.lines.rbegin();
			while (textStory.lines.rend() != itLine)
			{
				if (chars.bbox.x + chars.bbox.width < itLine->bbox.x)
				{
					Rect nextBBox = itLine->front().GetBBox(false);
					Rect prevBBox = chars.back().GetBBox(false);
					double t = min(prevBBox.y + prevBBox.height, nextBBox.y + nextBBox.height) - max(prevBBox.y, nextBBox.y);

					if (t >= 0.95 * max(prevBBox.height, nextBBox.height))
					{
						itLine->splice(itLine->begin(), chars);

						if (itLine->bbox.empty())
							itLine->bbox = chars.bbox;
						else
							Rect::Union(itLine->bbox, itLine->bbox, chars.bbox);

						if (textStory.nodeStory->Bounds().empty())
							textStory.nodeStory->SetBounds(itLine->bbox);
						else
						{
							Rect bboxNew;
							Rect::Union(bboxNew, textStory.nodeStory->Bounds(), itLine->bbox);
							textStory.nodeStory->SetBounds(bboxNew);
						}

						return;
					}
				}
				++itLine;
			}
		}

		textStory.lines.push_back(TextLine());
		textStory.lines.back().Append(chars);
	}

	if (!textStory.lines.back().bbox.empty())
	{
		if (textStory.nodeStory->Bounds().empty())
			textStory.nodeStory->SetBounds(textStory.lines.back().bbox);
		else
		{
			Rect bboxNew;
			Rect::Union(bboxNew, textStory.nodeStory->Bounds(), textStory.lines.back().bbox);
			textStory.nodeStory->SetBounds(bboxNew);
		}
	}
}

void XarOutputDev::TextReconstruct()
{
	if (!textChars.empty())
	{
		TextReconstruct(textChars);
		textChars.clear();
		textChars.bbox.clear();
	}
}

void XarOutputDev::TextReconstruct(TextChars &strElements)
{
	_ASSERT(!g_settings.bTextAsOutline);

	std::stringstream ss;
	static const double DELTA = 1e-4;

#if defined(_DEBUG)
	std::wstring text;
	TextLine::GetText(strElements, text);
#endif

	// Fix for DDES-12123 (Text reconstruction for Rndbt134.PDF worse than V11)
	ClipperLib::Paths clipRect;
	if (1 == m_clip.GetPolygons().size())
	{
		const ClipperLib::Path &path = m_clip.GetPolygons().back();
		if (4 == path.size() &&
		   ((path[0].X == path[3].X && path[1].X == path[2].X && path[0].Y == path[1].Y && path[2].Y == path[3].Y) ||
		    (path[0].X == path[1].X && path[2].X == path[3].X && path[0].Y == path[3].Y && path[1].Y == path[2].Y)))
		{
			ClipperLib::cInt xMin = min(path[0].X, path[2].X);
			ClipperLib::cInt xMax = max(path[0].X, path[2].X);
			ClipperLib::cInt yMin = min(path[0].Y, path[2].Y);
			ClipperLib::cInt yMax = max(path[0].Y, path[2].Y);

			/*if (xMin <= strElements.bbox.x)
				xMin = -1e7;//INT_MIN;

			if (xMax >= (strElements.bbox.x + strElements.bbox.width))
				xMax = 1e7;//INT_MAX;

			if (yMin <= strElements.bbox.y)
				yMin = -1e7;//INT_MIN;

			if (yMax >= (strElements.bbox.y + strElements.bbox.height))
				yMax = 1e7;//INT_MAX;*/

			Rect rect((int)xMin, (int)yMin, (int)(xMax - xMin), (int)(yMax - yMin));
			rect.Inflate(1300, 1300);
			AddRect(clipRect, rect);
		}
	}
	const ClipperLib::Paths &clipPolygons = !clipRect.empty() ? clipRect : m_clip.GetPolygons();

	ClipperLib::Paths stringsPolygons;
	TextLine::GetBoundingPolygons(strElements, stringsPolygons);
	bool bClip = true, bSetClip = false;
	try
	{
		ClipperLib::Clipper c;
		c.AddPaths(stringsPolygons, ClipperLib::ptSubject, true);
		c.AddPaths(clipPolygons, ClipperLib::ptClip, true);

		ClipperLib::Paths polygons;

		if (c.Execute(ClipperLib::ctDifference, polygons, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(m_clip.GetWindingRule())) && polygons.empty())
			bClip = false;

		if (bClip && c.Execute(ClipperLib::ctIntersection, polygons, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(m_clip.GetWindingRule())) && polygons.empty())
			return;
	}
	catch (ClipperLib::clipperException&) {}

#if defined(_TEXT_COMMENT_LAYER) || defined(EXTRA_LOGGING)
	{
		double xAbs, yAbs;
		textMatrix.map(strElements.front().x, strElements.front().y, xAbs, yAbs);

		m_XTextNew = (LONG)xAbs;
		m_YTextNew = (LONG)yAbs;
	}
#endif

	//=======================================================

	TextStories::reverse_iterator textStoriesItChosen = textStories.rend();
	TextLine *lineChosen = NULL;

	bool bIntersection = false;

	for (TextStories::reverse_iterator textStoriesIt = textStories.rbegin(); !bIntersection && textStoriesIt != textStories.rend(); ++textStoriesIt)
	{
		//ClipperLib::Paths stringsPolygons;
		//stringElements.GetBoundingPolygons(stringsPolygons);

		if (/*!m_clip.Empty() && */!stringsPolygons.empty())
		{
			/*try
			{
				ClipperLib::Clipper c;
				c.AddPaths(stringsPolygons, ClipperLib::ptSubject, true);
				c.AddPaths(clipPolygons, ClipperLib::ptClip, true);

				if (c.Execute(ClipperLib::ctIntersection, stringsPolygons, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(m_clip.GetWindingRule())))
				{
					if (stringsPolygons.empty()) return;
				}
			}
			catch (ClipperLib::clipperException&) {}

			if (!stringsPolygons.empty())*/
			{
				// check graphics intersection
				VXfTreeNode *pNext = textStoriesIt->nodeStory->NextSibling();
				while (pNext && !pNext->IsTextStory())
				{
					if (!pNext->IsRemoved() && pNext->IsBounded() && !((VXfBounded *)pNext)->Bounds().empty() &&
						Intersects(((VXfBounded *)pNext)->Bounds(), stringsPolygons))
					{
						bIntersection = true;
						break;
					}
					pNext = pNext->NextSibling();
				}

				if (bIntersection) break;

	  			// check text intersection
				if (textStoriesIt != (--(textStories.rend())))
				{
					if (Intersects(textStoriesIt->nodeStory->Bounds(), stringsPolygons))
					{
						for (TextLines::const_iterator itLine = textStoriesIt->lines.begin(); itLine != textStoriesIt->lines.end(); ++itLine)
						{
							if (itLine->bbox.empty()) continue;

							if (Intersects(itLine->bbox, stringsPolygons))
							{
								for (TextChars::const_iterator itChar = itLine->begin(); itChar != itLine->end(); ++itChar)
								{
									if (itChar->bbox.empty()) continue;
									if (Intersects(itChar->bbox, stringsPolygons))
									{
										bIntersection = true;
										break;
									}
								}
								if (bIntersection) break;
							}
						}
					}
				}

				// check clipping

				if (!bClip)
				{
					if (textStoriesIt->bNeedClipview)
					{
						if (textStoriesIt->clipPolygons.empty())
							continue;
						try
						{
							ClipperLib::Clipper c;
							c.AddPaths(stringsPolygons, ClipperLib::ptSubject, true);
							c.AddPaths(textStoriesIt->clipPolygons, ClipperLib::ptClip, true);

							ClipperLib::Paths diff;

							if (!c.Execute(ClipperLib::ctDifference, diff, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(textStoriesIt->clipWindingRule)) || !diff.empty())
								continue;
						}
						catch (ClipperLib::clipperException&) {}
					}
				}
				else
				{
					//continue;
					if (textStoriesIt->bNeedClipview)
					{
						try
						{
							ClipperLib::Clipper c;
							c.AddPaths(clipPolygons, ClipperLib::ptSubject, true);
							c.AddPaths(textStoriesIt->clipPolygons, ClipperLib::ptClip, true);

							ClipperLib::Paths xor;
						
							if (!c.Execute(ClipperLib::ctXor, xor, XaraFillRuleToPoly(m_clip.GetWindingRule()), XaraFillRuleToPoly(textStoriesIt->clipWindingRule)) || !xor.empty())
								continue;
						}
						catch (ClipperLib::clipperException&) {}
					}
					else
					{
						ClipperLib::Paths textPolygons;

						size_t c = 0;
						for (TextLines::const_iterator itLine = textStoriesIt->lines.begin(); itLine != textStoriesIt->lines.end(); ++itLine)
							c += itLine->size();
						if (textPolygons.capacity() < c) textPolygons.reserve(c);

						for (TextLines::const_iterator itLine = textStoriesIt->lines.begin(); itLine != textStoriesIt->lines.end(); ++itLine)
							itLine->GetBoundingPolygons(textPolygons);

						try
						{
							ClipperLib::Clipper c;
							c.AddPaths(textPolygons, ClipperLib::ptSubject, true);
							c.AddPaths(clipPolygons, ClipperLib::ptClip, true);

							ClipperLib::Paths polygons;

							if (c.Execute(ClipperLib::ctDifference, polygons, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(m_clip.GetWindingRule())) && polygons.empty())
								bSetClip = true;
						}
						catch (ClipperLib::clipperException&) {}

						if (!bSetClip)
							continue;
					}
				}
			}
		}

		if (m_pTextClipController && (!textStoriesIt->nodeStory || m_pTextClipController != textStoriesIt->nodeStory->GetParent())) continue;

		if (fabs(textMatrix.a() - textStoriesIt->matrix.a()) >= DELTA ||
		    fabs(textMatrix.b() - textStoriesIt->matrix.b()) >= DELTA ||
		    fabs(textMatrix.c() - textStoriesIt->matrix.c()) >= DELTA ||
		    fabs(textMatrix.d() - textStoriesIt->matrix.d()) >= DELTA)
			continue;

		textStoriesItChosen = textStoriesIt;
		break;
	}

	if (textStories.rend() == textStoriesItChosen)
	{
		if (strElements.bbox.empty()) return;
		if (textStories.size() >= 1/*g_settings.iMaxStoryCount*/)
		{
			RenderToOptimizedTags(textStories.front());
			textStories.erase(textStories.begin());
		}

		textStories.push_back(TextStory());
		TextStory& textStory = textStories.back();
		textStoriesItChosen = textStories.rbegin();

		textStory.matrix    = textMatrix;
		textStory.matrixInv = textMatrixInv;

		Matrix nodeMat(textStory.matrix.a(), textStory.matrix.b(), textStory.matrix.c(), textStory.matrix.d(),
		               _round(textStory.matrix.e()), _round(textStory.matrix.f()));

		textStory.nodeStory = new CXfTextStoryComplex(nodeMat, g_settings.bReconstructKerningPairs ? 1 : 0);

		bSetClip = true;

		if (m_pTextClipController) // this text story should act as a clip path, PDF reference, TABLE 5.3 Text rendering modes
		{
			m_pTextClipController->AddChild(textStory.nodeStory);
			textStory.nodeStory->SetIsClippingPath(true);
			RD_LOG(" {new story + clipview}");
		}
		else
		{
			MergeSmallBitmaps();
			m_pInsertTo->AddChild(EncloseInSoftMaskGroup(textStory.nodeStory));
			RD_LOG(" {new story}");
		}
	}

	TextStory& textStory = *textStoriesItChosen;

	if (bSetClip)
	{
		if (textStory.bNeedClipview = bClip)
		{
			textStory.clipPolygons    = clipPolygons;
			textStory.clipWindingRule = m_clip.GetWindingRule();
		}
	}

	Append(textStory, strElements);
}

void XarOutputDev::TextStory::DetectTabStops()
{
	const MILLIPOINT tolerance = 15;

	for (TextLines::iterator itLine = lines.begin(); itLine != lines.end();)
	{
		TextLines::iterator itNextLine = itLine;
		++itNextLine;

		if (lines.end() == itNextLine)
			break;

#if defined(_DEBUG)
		std::wstring text;
		TextLine::GetText(*itLine, text);
		std::wstring textNext;
		TextLine::GetText(*itNextLine, textNext);
#endif

		TextChars::iterator it1 = itLine->begin();
		TextChars::iterator it2 = itNextLine->begin();

		TextChars::iterator itPrev1 = itLine->end();
		TextChars::iterator itPrev2 = itNextLine->end();

		while (itLine->end() != it1 && 0 != (UNICODE_MASK_INVISIBLE & it1->GetUnicodeMask())) ++it1;
		while (itNextLine->end() != it2 && 0 != (UNICODE_MASK_INVISIBLE & it2->GetUnicodeMask())) ++it2;

		while (itLine->end() != it1 && itNextLine->end() != it2)
		{
			MILLIPOINT x1 = _round(it1->x);
			MILLIPOINT x2 = _round(it2->x);

			if (abs(x1 - x2) <= min(it1->GetTrackingWidth(tolerance), it2->GetTrackingWidth(tolerance)))
			{
				if ((itLine->end() != itPrev1 || itNextLine->end() != itPrev2) &&
				    (itLine->end() == itPrev1 || (x1 - _round(itPrev1->x)) > max(itPrev1->GetTrackingWidth(1000), it1->GetTrackingWidth(1000))) &&
				    (itNextLine->end() == itPrev2 || (x2 - _round(itPrev2->x)) > max(itPrev2->GetTrackingWidth(1000), it2->GetTrackingWidth(1000))))
				{
					it1->isTabStop = it2->isTabStop = true;
				}

				itPrev1 = it1++;
				itPrev2 = it2++;

				while (itLine->end() != it1 && 0 != (UNICODE_MASK_INVISIBLE & it1->GetUnicodeMask())) ++it1;
				while (itNextLine->end() != it2 && 0 != (UNICODE_MASK_INVISIBLE & it2->GetUnicodeMask())) ++it2;
			}
			else if (x1 < x2)
			{
				do
				{
					if (0 == (UNICODE_MASK_INVISIBLE & it1->GetUnicodeMask()))
						itPrev1 = it1;
					++it1;
				}
				while (itLine->end() != it1 && _round(it1->x) + min(it1->GetTrackingWidth(tolerance), it2->GetTrackingWidth(tolerance)) < x2);
			}
			else
			{
				do
				{
					if (0 == (UNICODE_MASK_INVISIBLE & it2->GetUnicodeMask()))
						itPrev2 = it2;
					++it2;
				}
				while (itNextLine->end() != it2 && _round(it2->x) + min(it1->GetTrackingWidth(tolerance), it2->GetTrackingWidth(tolerance)) < x1);
			}

			while (itLine->end() != it1 && 0 != (UNICODE_MASK_INVISIBLE & it1->GetUnicodeMask())) ++it1;
			while (itNextLine->end() != it2 && 0 != (UNICODE_MASK_INVISIBLE & it2->GetUnicodeMask())) ++it2;
		}

		itLine = itNextLine;
	}
}

// Fix for one-line numbered list issue (DDES-15776)
bool XarOutputDev::TextStory::RevertOneItemList()
{
	Paragraphs::reverse_iterator itLastNonEmpty = rbegin();
	while (rend() != itLastNonEmpty && itLastNonEmpty->empty())
		++itLastNonEmpty;

	if (rend() != itLastNonEmpty && itLastNonEmpty->front().front().isAutoListChar)
	{
		Paragraphs::reverse_iterator itPrevParagraph = itLastNonEmpty;
		++itPrevParagraph;

		if (rend() == itPrevParagraph || !itPrevParagraph->front().front().isAutoListChar)
		{
			while (!empty() && back().empty())
				pop_back();

			back().front().ClearAutoListItemFlags();
			lines.splice(lines.begin(), back(), back().begin(), back().end());
			pop_back();

			return true;
		}
	}

	return false;
}

void XarOutputDev::TextStory::DetectParagraphs()
{
	Paragraph *pParagraph = NULL;

	TextLines::iterator itDropCapNextLine = lines.end();

	while (!lines.empty())
	{
		TextLines::iterator itLine = lines.begin();

#if defined(_DEBUG)
		std::wstring text;
		TextLine::GetText(*itLine, text);
#endif

		if (itLine->front().isAutoListChar)
		{
			pParagraph = NULL;
		}

		if (itDropCapNextLine == itLine)
			itDropCapNextLine = lines.end();

#ifdef _ENABLE_RELATIVE_LINE_SPACING
		double linespaceRatio = 1.0;
#else
		LONG linespaceDelta = _round(LINESPACE_DELTA * itLine->attrs.fontSize());
		LONG linespacing = 0;
#endif

		if (pParagraph && !pParagraph->empty())
		{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
			linespaceRatio = GetLineSpaceRatio(pParagraph->back(), *itLine);
			if (linespaceRatio <= LINESPACE_RATIO_DELTA || linespaceRatio > 3.0)
#else
			linespacing = _round(GetAbsoluteLineSpacing(pParagraph->back(), *itLine));
			if (linespacing <= 0 || linespacing > 3 * max(pParagraph->back().ascent - pParagraph->back().descent, itLine->ascent - itLine->descent))
#endif
			{
#ifdef _ENABLE_TEXT_FLOW_RECONSTRUCTION
				Paragraph *pPrevInFlow = pParagraph;
				if (itLine->front().attrs != pParagraph->back().back().attrs/* || itLine->front().attrs.pFontDesc != pParagraph->back().back().attrs.pFontDesc*/)
					pPrevInFlow = NULL;

				if (pPrevInFlow && (itLine->leftMargin < min(pParagraph->firstMargin, pParagraph->leftMargin) || linespacing <= 0 && itLine->leftMargin < pParagraph->rightMargin))
					pPrevInFlow = NULL;

				if (pPrevInFlow && (abs(pParagraph->rightMargin - pParagraph->back().endMargin) > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : pParagraph->back().GetMaxTrackingWidth(MAX_RELATIVE_OFFSET))))
					pPrevInFlow = NULL;

				if (pPrevInFlow && !pParagraph->back().CheckWrapping(pParagraph->rightMargin - pParagraph->leftMargin, &(*itLine)))
					pPrevInFlow = NULL;

#else
				Paragraph *pPrevInFlow = NULL;
#endif
				
				if (pPrevInFlow)
				{
					push_back(Paragraph());
					pParagraph = &back();

					pParagraph->pPrevInFlow = pPrevInFlow;
					pPrevInFlow->pNextInFlow = pParagraph;
				}
				else pParagraph = NULL;
			}
#ifdef _ENABLE_RELATIVE_LINE_SPACING
			else if (pParagraph->size() > 1 && fabs(pParagraph->linespaceRatio.MakeDouble() - linespaceRatio) > LINESPACE_RATIO_DELTA)
			{
				if (linespaceRatio < pParagraph->linespaceRatio.MakeDouble())
#else
			else if (pParagraph->size() > 1 && abs(pParagraph->linespacing - linespacing) > linespaceDelta)
			{				
				if (linespacing < pParagraph->linespacing)
#endif
				{
					if (min(pParagraph->back().rightMargin, itLine->rightMargin) - max(pParagraph->back().leftMargin, itLine->leftMargin) > 0)
					{
						TextLines::iterator itPrev = pParagraph->end();
						--itPrev;
						lines.splice(lines.begin(), *pParagraph, itPrev);
						itLine = lines.begin();

#if defined(_DEBUG)
						text.clear();
						TextLine::GetText(*itLine, text);
#endif

						pParagraph->firstMargin = pParagraph->leftMargin = pParagraph->front().leftMargin;
						pParagraph->rightMargin = pParagraph->front().rightMargin;
						bool bHasLeftMargin = false;

						for (TextLines::const_iterator it = ++pParagraph->begin(); it != pParagraph->end(); ++it)
						{
							if (!bHasLeftMargin || it->leftMargin < pParagraph->leftMargin)
							{
								pParagraph->leftMargin = it->leftMargin;
								bHasLeftMargin = true;
							}

							if (it->rightMargin > pParagraph->rightMargin)
								pParagraph->rightMargin = it->rightMargin;					
						}

						if (1 == pParagraph->size())
						{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
							pParagraph->linespaceRatio = 1.0;
#else
							pParagraph->linespacing = pParagraph->front().ascent - pParagraph->front().descent;
#endif
						}
					}
				}
				pParagraph = NULL;
			}
			else
			{
				Paragraph *pPrevInFlow = pParagraph->pPrevInFlow;
				while (pPrevInFlow)
				{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
					if (pPrevInFlow->size() > 1 && fabs(pPrevInFlow->linespaceRatio.MakeDouble() - linespaceRatio) > LINESPACE_RATIO_DELTA)
#else
					if (pPrevInFlow->size() > 1 && abs(pPrevInFlow->linespacing - linespacing) > linespaceDelta)
#endif
					{
						pParagraph = NULL;
						break;
					}

					pPrevInFlow = pPrevInFlow->pPrevInFlow;
				}
			}
		}

		if (pParagraph)
		{
			if (itLine->rightMargin + 4.0 * max(itLine->back().GetFontSize(), pParagraph->back().front().GetFontSize()) < pParagraph->leftMargin)
				pParagraph = NULL;
		}

		// checking if current line first break char would fit into previous line
		if (pParagraph && itLine->front().IsBreak())
			pParagraph = NULL;

		if (pParagraph && (!pParagraph->empty() && !pParagraph->back().back().attrs.pFontDesc || pParagraph->empty() && pParagraph->pPrevInFlow && !pParagraph->pPrevInFlow->back().back().attrs.pFontDesc))
			pParagraph = NULL;

		if (pParagraph && !pParagraph->empty() && !pParagraph->front().front().isAutoListChar)
		{
			for (TextChars::const_iterator itChar = pParagraph->back().begin(); itChar != pParagraph->back().end(); ++itChar)
			{
				if (itChar->isTabStop)
				{
					pParagraph = NULL;
					break;
				}
			}
		}

		if (pParagraph && (pParagraph->empty() || !pParagraph->front().front().isAutoListChar))
		{
			for (TextChars::const_iterator itChar = itLine->begin(); itChar != itLine->end(); ++itChar)
			{
				if (itChar->isTabStop)
				{
					pParagraph = NULL;
					break;
				}
			}
		}

		if (pParagraph && !pParagraph->empty()/* && (pParagraph->back().attrs.fontSize() != itLine->attrs.fontSize())*/)
		{
			double prevFontSize = pParagraph->back().back().attrs.fontSize;
			double nextFontSize = itLine->front().attrs.fontSize;
			if (max(prevFontSize, nextFontSize) / min(prevFontSize, nextFontSize) > 1.02)
				pParagraph = NULL;
		}

		if (pParagraph && !pParagraph->empty())
		{
			// Fix for DDES-10976 (Relendex_Form.pdf)

			TextChars::const_reverse_iterator it = pParagraph->back().rbegin();
			while (pParagraph->back().rend() != it && 0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask()))
				++it;
			size_t count = 0;
			while (pParagraph->back().rend() != it)
			{
				switch (it->c)
				{
				case '.': case '_':
					++count;
					++it;
					break;
				case 0x2026: // ellipsis
					count += 3;
					++it;
					break;
				default:
					it = pParagraph->back().rend();
				}
			}
			if (count > 3)
				pParagraph = NULL;
		}

		/*if (pParagraph && !pParagraph->empty() && IsEndOfStatement(pParagraph->back(), *itLine))
			pParagraph = NULL;*/

		// Detecting alignment for paragraph

		if (pParagraph && !pParagraph->empty())
		{
			if (pParagraph->front().front().scriptScale.MakeDouble() >= MIN_DROPCAP_SIZE && 1 == pParagraph->size())
			{
				MILLIPOINT dropCapBaseLine = pParagraph->front().baseLine + pParagraph->front().front().baseLineShift +
					_round(pParagraph->front().front().scriptOffset.MakeDouble() * pParagraph->front().front().attrs.fontSize);

				MILLIPOINT dropCapAscent = dropCapBaseLine + _round(pParagraph->front().front().scriptScale.MakeDouble() * pParagraph->front().front().GetFontAscent());
				MILLIPOINT dropCapDescent = dropCapBaseLine;// + _round(pParagraph->front().front().scriptScale.MakeDouble() * pParagraph->front().front().GetFontDescent());

				itDropCapNextLine = lines.end();
				TextLines::iterator itPrevLine = pParagraph->begin();
				for (TextLines::iterator it = itLine; it != lines.end(); itPrevLine = it++)
				{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
					if (it != itLine && fabs(GetLineSpaceRatio(*itPrevLine, *it) - linespaceRatio) > LINESPACE_RATIO_DELTA)
						break;
#else
					if (it != itLine && abs(_round(GetAbsoluteLineSpacing(*itPrevLine, *it)) - linespacing) > _round(LINESPACE_DELTA * it->attrs.fontSize()))
						break;
#endif

					MILLIPOINT ascent = it->baseLine + it->ascent;
					MILLIPOINT descent = it->baseLine + it->descent;

					if ((min(dropCapAscent, ascent) - max(dropCapDescent, descent)) < 0.5 * min(dropCapAscent - dropCapDescent, ascent - descent))
					{
						itDropCapNextLine = it;
						break;
					}
					else if (it->leftMargin < pParagraph->front().leftMargin)
						break;
				}

				if (itDropCapNextLine == itLine)
					itDropCapNextLine = lines.end();
			}

			if (lines.end() != itDropCapNextLine)
			{
				MILLIPOINT dx = itLine->leftMargin - itDropCapNextLine->leftMargin;
				itLine->firstKern = _round(1000.0 * dx / itLine->front().GetFontSize());
				itLine->leftMargin -= itLine->front().GetKernWidth(itLine->firstKern);
			}

			Paragraph *pFirstInFlow = pParagraph->GetFirstInFlow();;

			UINT possibleAlign = pFirstInFlow->possibleAlign;
			LONG firstMargin, leftMargin;

			LONG rightMargin = itLine->rightMargin;
			rightMargin = 1 == pParagraph->size() ?
				max(rightMargin, pParagraph->back().rightMargin) :
				max(rightMargin, pParagraph->rightMargin);

			if (pParagraph->pPrevInFlow)
			{
				leftMargin = itLine->leftMargin;
				leftMargin = 1 == pParagraph->size() ?
					min(leftMargin, pParagraph->back().leftMargin) :
					min(leftMargin, pParagraph->leftMargin);

				firstMargin = leftMargin;
			}
			else
			{
				leftMargin = itLine->leftMargin;
				switch (pParagraph->size())
				{
				case 1:
					break;
				case 2:
					leftMargin = min(leftMargin, pParagraph->back().leftMargin);
					break;
				default:
					leftMargin = min(leftMargin, pParagraph->leftMargin);
					break;
				}

				firstMargin = pParagraph->front().leftMargin;
			}

			for (TextLines::iterator it = pParagraph->begin(); it != pParagraph->end();)
			{
				if (abs(rightMargin - it->endMargin) > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : it->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
					possibleAlign &= ~(ALIGN_RIGHT | ALIGN_FULL);

				if (abs((it->leftMargin - min(firstMargin, leftMargin)) - (rightMargin - it->endMargin)) > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : it->GetMaxTrackingWidth(MAX_CENTER_ALIGN_RELATIVE_OFFSET)))
					possibleAlign &= ~ALIGN_CENTER;

				TextLines::iterator itNext = it;
				++itNext;
				TextLine *pNextLine = (pParagraph->end() != itNext) ? &(*itNext) : &(*itLine);

				MILLIPOINT fitWidth;

				if (pParagraph->begin() == it)
				{					
					fitWidth = rightMargin - firstMargin;

					if (pParagraph->pPrevInFlow && abs(leftMargin - it->leftMargin) > it->GetMaxTrackingWidth(MAX_LEFT_RELATIVE_OFFSET))
						possibleAlign &= ~(ALIGN_LEFT | ALIGN_FULL);
				}
				else
				{
					fitWidth = rightMargin - leftMargin;

					if (abs(leftMargin - it->leftMargin) > it->GetMaxTrackingWidth(MAX_LEFT_RELATIVE_OFFSET))
						possibleAlign &= ~(ALIGN_LEFT | ALIGN_FULL);
				}

				if (!it->CheckWrapping(fitWidth, pNextLine, 1) && !it->RemoveKernings(fitWidth, 0, TRACKING_TOLERANCE, pNextLine, true))
				{
					pParagraph = NULL;
					break;
				}

				if (!it->CheckWrapping(rightMargin - min(firstMargin, leftMargin), pNextLine))
					possibleAlign &= ~(ALIGN_CENTER | ALIGN_RIGHT);
				it = itNext;
			}

			if (pParagraph)
			{
				if (abs(rightMargin - itLine->endMargin) > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
					possibleAlign &= ~ALIGN_RIGHT;

				if (abs((itLine->leftMargin - min(firstMargin, leftMargin)) - (rightMargin - itLine->endMargin)) > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_CENTER_ALIGN_RELATIVE_OFFSET)))
					possibleAlign &= ~ALIGN_CENTER;

				if (pParagraph->size() > 1)
				{
					if (abs(leftMargin - itLine->leftMargin) > itLine->GetMaxTrackingWidth(MAX_LEFT_RELATIVE_OFFSET))
						possibleAlign &= ~(ALIGN_LEFT | ALIGN_FULL);

					if (ALIGN_RIGHT == possibleAlign && ALIGN_FULL == pFirstInFlow->GetAlign())
						pParagraph = NULL;
				}
				else
				{
					if (pParagraph->pPrevInFlow && abs(leftMargin - itLine->leftMargin) > itLine->GetMaxTrackingWidth(MAX_LEFT_RELATIVE_OFFSET))
						possibleAlign &= ~(ALIGN_LEFT | ALIGN_FULL);
				}

				if (0 == possibleAlign)
					pParagraph = NULL;

				if (pParagraph)
				{
					pFirstInFlow->possibleAlign = possibleAlign;

					//switch (pFirstInFlow->GetAlign())
					//{
					//case ALIGN_CENTER: case ALIGN_RIGHT:
					//	if (!pParagraph->front().front().isAutoListChar)
					//	{
					//		pParagraph->firstMargin = pParagraph->leftMargin = min(firstMargin, leftMargin);
					//		break;
					//	}
					////case ALIGN_RIGHT:
					////	pParagraph->firstMargin = bNeedFirstIndent ? firstMargin : min(firstMargin, leftMargin);
					////	pParagraph->leftMargin = (1 == pParagraph->size()) ? min(firstMargin, leftMargin) : leftMargin;
					////	break;
					//default:
						pParagraph->firstMargin = firstMargin;
						pParagraph->leftMargin  = leftMargin;
					//	break;
					//}

					pParagraph->rightMargin = rightMargin;
				}
			}
		}

		if (!pParagraph)
		{
			if (!itLine->front().isAutoListChar && RevertOneItemList())
			{
				if (empty())
					pParagraph = nullptr;
				else
					pParagraph = &back();

				itDropCapNextLine = lines.end();

				continue;
			}

			if (empty() || !back().empty())
				push_back(Paragraph());
			else if (back().pPrevInFlow)
			{
				back().pPrevInFlow->pNextInFlow = NULL;
				back().pPrevInFlow = NULL;
			}
			pParagraph = &back();
		}

		if (pParagraph->empty())
		{
			/*if (itLine->firstKern)
			{
				itLine->leftMargin += itLine->front().GetKernWidth(itLine->firstKern);
				itLine->firstKern = 0;
			}*/
			itDropCapNextLine = lines.end();

#ifdef _ENABLE_RELATIVE_LINE_SPACING
			pParagraph->linespaceRatio = 1.0;
#else
			pParagraph->linespacing = itLine->ascent - itLine->descent;
#endif

			pParagraph->firstMargin = pParagraph->leftMargin = itLine->leftMargin;
			pParagraph->rightMargin = itLine->rightMargin;
		}
		else if (1 == pParagraph->size())
		{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
			pParagraph->linespaceRatio = linespaceRatio;
#else
			pParagraph->linespacing = linespacing;
#endif
		}

		if (!pParagraph->empty())
		{
			if (!pParagraph->back().back().IsBreak())
				pParagraph->back().AppendSpaces(--pParagraph->back().end(), pParagraph->back().back());
		}
		else if (pParagraph->pPrevInFlow && !pParagraph->pPrevInFlow->back().back().IsBreak())
			pParagraph->pPrevInFlow->back().AppendSpaces(--pParagraph->pPrevInFlow->back().end(), pParagraph->pPrevInFlow->back().back());
		
		pParagraph->splice(pParagraph->end(), lines, itLine);

		if (lines.empty() && RevertOneItemList())
		{
			if (empty())
				pParagraph = nullptr;
			else
				pParagraph = &back();

			itDropCapNextLine = lines.end();

			//continue;
		}
	}

	Paragraph * pFirstInFlow = NULL;
	double linespacingSum = 0;
	size_t count = 0;
	for (Paragraphs::iterator itParagraph = begin(); itParagraph != end();)
	{
		TextLines::iterator itFirstLine = itParagraph->begin();

		if (itParagraph->end() != itFirstLine && itFirstLine->firstKern)
		{
			itFirstLine->leftMargin += itFirstLine->front().GetKernWidth(itFirstLine->firstKern);
			itFirstLine->firstKern = 0;
		}

		if (!pFirstInFlow)
		{
			pFirstInFlow = &(*itParagraph);
			linespacingSum = 0;
			count = 0;
		}

		if (itParagraph->size() > 1)
		{
			// According to DDES-1510, we need to handle slightly different
			// line spacing values across the paragraph. So we need to correct
			// a uniform line spacing across the paragraph so the final line 
			// of the paragraph is positioned correctly.
			// More of that, according to proposal from Neil in DDES-1806, we
			// have to round linespacing value to nearest 0.1pt

			// balancing and rounding

#ifdef _ENABLE_RELATIVE_LINE_SPACING
			linespacingSum += (itParagraph->size() - 1) * GetLineSpaceRatio(*itParagraph);
			count += itParagraph->size() - 1;
#else
			//itParagraph->linespacing = 100 * _round(0.01 * GetAbsoluteLineSpacing(itParagraph->front(), itParagraph->back(), itParagraph->size() - 2));
			linespacingSum += (itParagraph->size() - 1) * GetAbsoluteLineSpacing(itParagraph->front(), itParagraph->back(), itParagraph->size() - 2);
			count += itParagraph->size() - 1;
#endif
		}

		Paragraphs::iterator itNextParagraph = itParagraph;
		++itNextParagraph;

		if (end() == itNextParagraph || !itNextParagraph->pPrevInFlow)
		{
			if (count)
			{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
				double linespaceRatio = _round(100.0 * linespacingSum / count) * 0.01;
#else
				MILLIPOINT linespacing = 100 * _round(0.01 * linespacingSum / (double)count);
#endif
				do
				{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
					pFirstInFlow->linespaceRatio = linespaceRatio;
#else
					pFirstInFlow->linespacing = linespacing;
#endif
					pFirstInFlow = pFirstInFlow->pNextInFlow;
				}
				while (pFirstInFlow);
			}
			else pFirstInFlow = NULL;
		}

		itParagraph = itNextParagraph;
	}

	for (Paragraphs::iterator itParagraph = begin(); itParagraph != end();)
	{
		Paragraphs::iterator itNextParagraph = itParagraph;
		++itNextParagraph;

		itParagraph->ConvertSpaces(1);

		if (end() != itNextParagraph)
		{
			itParagraph->spaceAfter = _round(GetParagraphSpacing(*itParagraph, *itNextParagraph));

			// As in Microsoft Word, the spacing between two paragraphs is *not* the sum of
			// the "space after" value of the first paragraph and the "space before" value
			// of the second, but the *maximum* of these values.
			// So if space_after is negative, the space_before needs to be set too
			if (itParagraph->spaceAfter < 0)
				itNextParagraph->spaceBefore = itParagraph->spaceAfter;
		}

		itParagraph = itNextParagraph;
	}
}

void XarOutputDev::Paragraph::ConvertSpaces(int nbsp)
{
	for (TextLines::iterator itLine = begin(); itLine != end();)
	{
		TextLines::iterator itNextLine = itLine;
		++itNextLine;

		if (end() == itNextLine)
			break;
			
		MILLIPOINT fitWidth;

		if (begin() == itLine)
			fitWidth = rightMargin - firstMargin;
		else
			fitWidth = rightMargin - leftMargin;

		if (!itLine->CheckWrapping(fitWidth, &(*itNextLine)))
		{
			int c = nbsp;
			for (auto itChar = itNextLine->begin(); itChar != itNextLine->end() && c; ++itChar)
			{
				if (itChar->IsSpace())
				{
					itChar->c = 0xA0; // NO-BREAK SPACE
					itChar->isCacheValid = false;
					itChar->ligType = 0;
					itChar->ligChars.clear();
					--c;
				}
			}
		}

		itLine = itNextLine;
	}
}

void XarOutputDev::DetectEmbeddedGraphics(TextStory& textStory, TextLine &line)
{
	static const double DELTA = 1e-4;

	if (line.size() > 1 && textStory.nodeStory &&
	    fabs(1.0 - textStory.matrix.a()) < DELTA &&
	    fabs(textStory.matrix.b()) < DELTA &&
	    fabs(textStory.matrix.c()) < DELTA &&
	    fabs(1.0 - textStory.matrix.d()) < DELTA)
	{
		VXfTreeNode *pParent = textStory.nodeStory->GetParent();		

		for (TextLine::iterator it = line.begin(); it != line.end();)
		{
			TextLine::iterator itNext = it;
			++itNext;

			MILLIPOINT baseline = _round(it->y);
			MILLIPOINT ascent = it->GetFontAscent();
			MILLIPOINT descent = it->GetFontDescent();
			MILLIPOINT fontSize = it->GetFontSize();

			CXfGroup *pGroup = nullptr;
			Rect groupBBox;

			size_t childrenCount = pParent->ChildrenCount();
			for (size_t i = 0; i < childrenCount;)
			{
				VXfTreeNode *pChild = pParent->GetChild(i);

				if (pChild == textStory.nodeStory)
					break;

				if (!pChild->IsPath() || pChild->IsRemoved())
				{
					++i;
					continue;
				}

				Rect bounds = ((VXfBounded *)pChild)->Bounds();

				if (bounds.y >= baseline + descent - 0.03 * fontSize && bounds.y + bounds.height <= baseline + ascent + 0.03 * fontSize &&
				    bounds.height > (ascent - descent) / 2 && // DDES-14850 fix
				    bounds.x >= (it->x + it->GetWidth() - 0.01 * fontSize) &&
				    (line.end() != itNext && (bounds.right() <= (itNext->x + 0.01 * fontSize)) || (line.end() == itNext) && (bounds.right() <= (it->x + it->GetWidth() + 1.01 * fontSize))))
				{
					pParent->RemoveChild(i, false);

					if (!pGroup)
					{
						pGroup = new CXfGroup();
						pParent->InsertBefore(i++, pGroup);
						groupBBox = bounds;
					}
					else
						Rect::Union(groupBBox, groupBBox, bounds);

					pGroup->AddChild(pChild);
				}
				else
					++i;
			}

			if (pGroup)
			{
				TextLine::iterator itNewChar = line.insert(itNext, *it);
				itNewChar->isAnchorPos = true;
				itNewChar->c = m_anchodedGroupTag;
				itNewChar->x = groupBBox.x;
				itNewChar->width = groupBBox.width;


				itNewChar->bbox = groupBBox;

				if (!itNewChar->bbox.empty())
				{
					if (line.bbox.empty())
						line.bbox = itNewChar->bbox;
					else
						Rect::Union(line.bbox, line.bbox, itNewChar->bbox);
				}

				//itNewChar->y = groupBBox.y;
				itNewChar->verticalOffset = groupBBox.y - baseline;
				itNewChar->ascent = max(ascent, groupBBox.y + groupBBox.height - baseline);
				itNewChar->descent = min(descent, groupBBox.y - baseline);

				pGroup->InsertFirstChild(new CXfEmbeddedTransformedAnchoredGroupInfo(TRUE, 1.0, 0.0, 0.0, 1.0));
				pGroup->InsertFirstChild(new CXfAnchoredGroupInfo(m_anchodedGroupTag++, DocCoord(0, 0)));
			}

			it = itNext;
		}
	}
}

XarOutputDev::Paragraph *XarOutputDev::Paragraph::GetFirstInFlow()
{
	Paragraph *pFirstInFlow = this;
	while (pFirstInFlow->pPrevInFlow)
		pFirstInFlow = pFirstInFlow->pPrevInFlow;
	return pFirstInFlow;
}

UINT XarOutputDev::Paragraph::GetAlign() const
{
	if (pPrevInFlow)
		return pPrevInFlow->GetAlign();

	size_t lineCount = size();
	const Paragraph *pNext = pNextInFlow;
	while (pNext)
	{
		lineCount += pNext->size();
		pNext = pNext->pNextInFlow;
	}

	if (lineCount > 2)
	{
		if (possibleAlign & ALIGN_FULL) return ALIGN_FULL;
		if ((possibleAlign & ALIGN_LEFT) && front().front().isAutoListChar) return ALIGN_LEFT;
		if (possibleAlign & ALIGN_CENTER) return ALIGN_CENTER;
		if (possibleAlign & ALIGN_LEFT) return ALIGN_LEFT;
		if (possibleAlign & ALIGN_RIGHT) return ALIGN_RIGHT;
	}
	else
	{
		if ((possibleAlign & ALIGN_LEFT) && (1 == lineCount || front().front().isAutoListChar || front().front().scriptScale.MakeDouble() >= MIN_DROPCAP_SIZE || front().leftMargin == leftMargin)) return ALIGN_LEFT;
		if (possibleAlign & ALIGN_RIGHT) return ALIGN_RIGHT;
		if (possibleAlign & ALIGN_CENTER) return ALIGN_CENTER;
		if (possibleAlign & ALIGN_LEFT) return ALIGN_LEFT;
		if (possibleAlign & ALIGN_FULL) return ALIGN_FULL;
	}

	return 0;
}

MILLIPOINT XarOutputDev::Paragraph::GetMaxWidth() const
{
	MILLIPOINT maxWidth = INT_MAX;
	for (TextLines::const_iterator itLine = begin(); itLine != end();)
	{
		TextLines::const_iterator itNextLine = itLine;
		++itNextLine;

		MILLIPOINT maxLineWidth = INT_MAX;
		if (end() != itNextLine)
			maxLineWidth = itLine->GetMaxWidth(&(*itNextLine));
		else if (pNextInFlow)
			maxLineWidth = itLine->GetMaxWidth(&pNextInFlow->front());

		if (maxLineWidth < maxWidth)
			maxWidth = maxLineWidth;		

		itLine = itNextLine;
	}
	return maxWidth;
}

double XarOutputDev::TextStory::GetLeftMarginMean() const
{
	return leftMarginSum / lines.size();
}

double XarOutputDev::TextStory::GetLeftMarginDeviation() const
{
	double mean = GetLeftMarginMean();
	size_t n = lines.size();
	//return sqrt(leftMarginSqrSum / n - mean * mean);
	return sqrt((leftMarginSqrSum - n * mean * mean) / (n - 1));
}

void XarOutputDev::updateTextShift(GfxState * state, double shift)
{
	if (g_settings.bTextAsOutline)
		return;

	//textStates.back().textShift = -shift;
	RD_ARG(shift);
}

void XarOutputDev::updateCharSpace(GfxState * state)
{RD_ARG(state->getCharSpace());
}

void XarOutputDev::updateRender(GfxState * state)
{RD_ARG(state->getRender());
	TextReconstruct();
}

void XarOutputDev::updateRise(GfxState * state)
{RD_ARG(state->getRise());
	TextReconstruct();
}

void XarOutputDev::updateWordSpace(GfxState * state)
{RD_ARG(state->getWordSpace());
}

void XarOutputDev::updateHorizScaling(GfxState * state)
{RD_ARG(state->getHorizScaling());
}

void XarOutputDev::saveTextPos(GfxState * state)
{RD;
}

void XarOutputDev::restoreTextPos(GfxState * state)
{RD;
}

//----- text drawing
void XarOutputDev::beginTextObject(GfxState * state)
{RD;
	TextReconstruct();
}

void XarOutputDev::endTextObject(GfxState * state)
{RD;
	TextReconstruct();
	OnEndTextObject();
}

void XarOutputDev::beginStringOp(GfxState * state)
{RD;
}

void XarOutputDev::endStringOp(GfxState * state)
{
	TextReconstruct();
}

bool XarOutputDev::IsTextStroked(GfxState* state)
{
	switch(state->getRender())
	{
	case eTextStroke:
	case eTextFillStroke:
	case eTextStrokeClip:
	case eTextFillStrokeClip:
		return true;
	}
	return false;
}

bool XarOutputDev::IsTextFilled(GfxState* state)
{
	switch(state->getRender())
	{
	case eTextFill:
	case eTextFillStroke:
	case eTextFillClip:
	case eTextFillStrokeClip:
		return true;
	}
	return false;
}

Rect GetBounds(const double *points, size_t count)
{
	if (!count) return Rect();

	double xMin = *points, xMax = *points;
	++points;
	double yMin = *points, yMax = *points;
	++points;
	for (size_t i = 1; i < count; ++i)
	{
		if (*points < xMin) xMin = *points;
		if (*points > xMax) xMax = *points;
		++points;
		if (*points < yMin) yMin = *points;
		if (*points > yMax) yMax = *points; 
		++points;
	}

	return Rect(_round(xMin), _round(yMin), _round(xMax - xMin), _round(yMax - yMin));
}

Rect TransformBBox(const Rect &bbox, const AffineTransform &transform)
{
	double points[8];
	double x0 = bbox.x;
	double x1 = bbox.x + bbox.width;
	double y0 = bbox.y;
	double y1 = bbox.y + bbox.height;
	transform.map(x0, y0, points[0], points[1]);
	transform.map(x1, y1, points[2], points[3]);
	transform.map(x0, y1, points[4], points[5]);
	transform.map(x1, y0, points[6], points[7]);	
	return GetBounds(points, 4);
}

void XarOutputDev::SetAttributes(TextAttrs &attrs, GfxState *state)
{
	attrs.lineJoin   = state->getLineJoin();
	attrs.lineCap    = state->getLineCap();
	attrs.lineWidth  = _round(state->getTransformedLineWidth());
	attrs.miterLimit = _round(state->transformWidth(state->getMiterLimit()));

	if (attrs.isStroked = IsTextStroked(state))
	{
		attrs.colorStroke   = Color::MakeFromState(GetStrokeColorSpace(state), GetStrokeColor(state));
		attrs.opacityStroke = Opacity::MakeFromState(state->getStrokeOpacity(), state->getBlendMode());

		double* pDash = NULL;
		double dStartDash = 0;
		int iDashLen = 0;
		state->getLineDash(&pDash, &iDashLen, &dStartDash);
		if (pDash && iDashLen > 0)
		{
			attrs.dash.reserve(iDashLen + 2);
			attrs.dash.push_back(_round(state->transformWidth(dStartDash)));
			attrs.dash.push_back(attrs.lineWidth);
			for(int i = 0; i < iDashLen; ++i)
				attrs.dash.push_back(_round(state->transformWidth(pDash[i])));
		}
	}

	if (attrs.isFilled = IsTextFilled(state))
	{
		attrs.colorFill   = Color::MakeFromState(GetFillColorSpace(state), GetFillColor(state));
		attrs.opacityFill = Opacity::MakeFromState(state->getFillOpacity(), state->getBlendMode());
	}
}

void XarOutputDev::TextParseString(GfxState *state, const GooString *s, double scaleX, double scaleY, double xText, double yText, double shearFactor)
{
	MappedFontDesc *pFontDesc = m_pFontManager->GetFont(state);
	if (!pFontDesc)
		return;

	double fontSize = fabs(state->getFontSize()) * scaleY;
	double aspectRatio = fabs(state->getHorizScaling()) * scaleX;
	scaleX = aspectRatio * fontSize;

	fontSize *= fabs(pFontDesc->scale);

	MILLIPOINT fontSizeMP = _round(fontSize);
	if (!fontSizeMP) return;

	double type3CharPos = 0;

	const char *p = s->c_str();
	int l = s->getLength(); 

	while (l > 0)
	{
		Unicode uBuf[8];

		CharCode charCode = 0;
		Unicode *u = NULL;
		int uLen = 0;
		double dx = 0, dy = 0, originX = 0, originY = 0;

		int n = state->getFont()->getNextChar(p, l, &charCode, &u, &uLen, &dx, &dy, &originX, &originY);
		AdjustUnicode(u, uLen);

		// DDES-14830 Fix (Extra square character added to the text when importing pdf file)
		if (int len = parseCharName(state->getFont(), charCode, uBuf, sizeof(uBuf) / sizeof(uBuf[0])))
		{
			u = uBuf;
			uLen = len;
		}

		// DDES-14874 (Apply ligature flag to the text if the embedded font contains all glyphs that form the ligature)
		if (1 == uLen)
		{
			if (int len = MapLigature(*u, uBuf))
			{
				u = uBuf;
				uLen = len;
			}
		}

		/* from PDF Reference:
			Word spacing shall be applied to every occurrence of
			the single-byte character code 32 in a string when using
			a simple font or a composite font that defines code 32 as
			a single-byte code. It shall not apply to occurrences of
			the byte value 32 in multiple-byte codes.
		*/
		bool useWordSpacing = (n == 1 && 32 == charCode);

		double spacing = state->getCharSpace();
		if (useWordSpacing) spacing += state->getWordSpace();

		if (uLen || pFontDesc->m_bEmbedded)
		{
			// Ligature support: we leave (pass) ligature as is, not converting it into separate characters as the embedded font
			// [subset] may not contain separate character glyphs. Later we may check whether the font actually contain these,
			// but this check should perform really fast as it is per each character in the document
			// Also we only should pass ligature as is [charcode] if the font is embedded, not present in the system so
			// the actual code does not mean much - we provide the ligature glyph in any way. If it not embedded, or the font is present
			// in the system we better translate ligature into correct Unicode characters.

			// DDES-9788: support for fake ligature-type symbols which start with whitespace
			/*while (uLen > 1 && u[0] == L' ')
			{
				++u;
				--uLen;
			}*/

			wchar_t c = 0;
			if (1 == uLen)
				c = *u;
			else if (uLen > 1)
			{
				c = MapLigature(u, uLen);
				if (0 == c)
					c = m_pFontManager->MapNonStandardLigature(*pFontDesc, u, uLen);

				// If we substituted successfully OR we need to retain the ligature because the font is embedded but not present
				// in the system, retain the ligature
				//if(cUni != charCode || mfd.m_bEmbedded && !mfd.bPresentInTheSystem)
				//{
				//}
			}

			// bEmptyGlyph is always set to true here as we can't pick the glyph outline at this step. This param only 
			// affects SPACE or tab. If it is NOT empty glyph, then it's already in the charToUni map we filled-in on embedding step.
			c = m_pFontManager->MapCharacter(*pFontDesc, c, charCode, true);

			// Add the character to the font sub-set
			// We need to collect *unicode* values, not the charcodes, because poppler lib may translate one char code into a
			// sequence of several unicode values!! For example it may translate ligatures like 'fi' 'ff' etc into a sequence
			// of individual letters, if the corresponding glyph letters exist in the font subset.
			pFontDesc->AddCharacter(c);

			bool bGlyphPresent = false;
			bool bGlyphSubstituted = false;
			CXfFontDef* pCurFontDef = pFontDesc->GetFontDef(charCode, bGlyphSubstituted); // Due to possible glyph collisions - same font ref may have different
			// font defs (another one is special for glyph collisions). So check and create a new string element in such case

			// J851-Rots South May 2014  front.pdf: this font has glyphs that can't be enumerated in a regular way via get_next_char
			// So some of them are left unembedded. We need to embed them on request
			if (pFontDesc->m_bEmbedded)
			{
				CXfFontDef* pDef = !bGlyphSubstituted ? 
					// glyph is NOT ns ligatures, nor absent from the system. 
					(pFontDesc->bPresentInTheSystem ? m_pFontManager->FindOrCreateExtraFontDef(*pFontDesc) : pFontDesc->node)
					//(extraDefs[pFontDesc->node].size() ? extraDefs[pFontDesc->node].front() : pFontDesc->pFontDefOrg) 
					: pCurFontDef;
				if (pDef->GetGlyph(c, pFontDesc->isBold, pFontDesc->isItalic) == NULL)
				{
					wprintf(L"%s: missing glyph %X, re-embedded\n", pFontDesc->fontNameRaw.c_str(), c);
					bool bFontDiffersFromSystem = false;
					bGlyphPresent = m_pFontManager->EmbedGlyph(*pFontDesc, charCode, c, false, true, false, bFontDiffersFromSystem);
					pCurFontDef = pFontDesc->GetFontDef(charCode, bGlyphSubstituted);

					if (bFontDiffersFromSystem)
						// The whole mfd is marked as '-emb'.
						pFontDesc->bPresentInTheSystem = false;
				}
				else
					bGlyphPresent = true;
			}

			bool bSlanted = shearFactor > 0;
			if (bSlanted)
			{
				if (pFontDesc->m_pFont && pFontDesc->m_pSplashFont)
				{
					if (SplashPath* pPath = pFontDesc->m_pSplashFont->getGlyphPath(charCode))
					{
						if (!IsPathEmpty(pPath))
						{
							double dAdvance = pFontDesc->m_pSplashFont->getGlyphAdvance(charCode);
							ULONG ulFlags =
								(pFontDesc->isBold ? FG_BOLD : 0) |
								//(pFontDesc->isItalic ? FG_ITALIC : 0);
								FG_ITALIC;

							CXfFontGlyph* pNewGlyph = new CXfFontGlyph(ulFlags, c, (ULONG)(dAdvance * 72000 + 0.5), (int)(pFontDesc->m_pFont->getAscent() * 72000.), (int)(pFontDesc->m_pFont->getDescent() * 72000.), pFontDesc->emWidth);
							SplashPathToGlyph(pPath, pNewGlyph, 1000.0, &AffineTransform(1, 0, shearFactor, sqrt(1.0 - shearFactor * shearFactor), 0, 0));

							bool bDeteteGlyph = true;
							CXfFontGlyph* pCurGlyph = pCurFontDef->GetGlyph(c, pFontDesc->isBold, true/*pFontDesc->isItalic*/);

							if (!pCurGlyph || !m_pFontManager->CompareGlyphs(pCurGlyph, pNewGlyph))
							{
								pCurFontDef = m_pFontManager->FindOrCreateExtraFontDef(*pFontDesc, pNewGlyph, &bDeteteGlyph);
								bGlyphPresent = true;
							}

							if (bDeteteGlyph)
								delete pNewGlyph;
						}
						delete pPath;						
					}
				}
			}

			// A fix from Pavel: we always has to use char width from the system, if the font is present
			//MILLIPOINT glyphWidth = m_pFontManager->GetGlyphWidth(charCode, c, pFontDesc);

			// Don't add the text character, if the font appears to be embedded but no glyph was found in the font file: trial.pdf
			if (!pFontDesc->m_bEmbedded || bGlyphPresent)
			{
				textChars.push_back(TextChar(c, xText - scaleX * originX, yText - scaleY * originY));
				TextChar &ch         = textChars.back();

				//ch.width             = glyphWidth;
				ch.pFontDef          = pCurFontDef;
				ch.attrs.isBold      = pFontDesc->isBold;
				ch.attrs.isItalic    = pFontDesc->isItalic || bSlanted;

				if (uLen > 1 /*&& uLen <= LIGATURE_MAX_CHARS*/)
				{
					ch.ligType = LIG_SUB_STANDARD;

					/*if (pFontDesc->m_bEmbedded)
					{
						for (int i = 0; i < uLen; ++i)
						{
							if (!ch.pFontDef->GetGlyph(u[i], ch.attrs.isBold, ch.attrs.isItalic))
							{
								ch.ligType = 0;
								break;
							}
						}
					}*/

					if (pFontDesc->bPresentInTheSystem)
					{
						ch.ligType = 0;
						if (CXfFontOTLigatures *pFontOTLigatures = GetLigatures(ch.pFontDef, ch.attrs.isBold, ch.attrs.isItalic, false))
						{
							std::wstring ligChars;
							ligChars.resize(uLen);
							for (int i = 0; i < uLen; ++i)
								ligChars[i] = u[i];

							std::vector<XarLigatureEntry>::iterator itLigature = GetLigatureEntry(pFontOTLigatures, ligChars, false);
							if (pFontOTLigatures->ligs.end() != itLigature)
							{
								ch.c = c = itLigature->Ligature;
								ch.ligType = itLigature->LigType;
							}
						}
					}

					if (ch.ligType)
					{
						ch.ligChars.resize(uLen);
						for (int i = 0; i < uLen; ++i)
							ch.ligChars[i] = u[i];

						CXfFontOTLigatures *pFontOTLigatures = GetLigatures(ch.pFontDef, ch.attrs.isBold, ch.attrs.isItalic, true);
						std::vector<XarLigatureEntry>::iterator itLigature = GetLigatureEntry(pFontOTLigatures, ch.ligChars, true);

						itLigature->Ligature = c;
						itLigature->LigType = (LigatureSubsetFlag)ch.ligType;
					}
				}

				ch.attrs.aspectRatio = aspectRatio;
				ch.attrs.pFontDesc   = pFontDesc;
				ch.attrs.fontSize    = fontSizeMP;
				SetAttributes(ch.attrs, state);
				ch.bbox = TransformBBox(ch.GetBBox(true), textMatrix);

				if (!ch.bbox.empty())
				{
					if (textChars.bbox.empty())
						textChars.bbox = ch.bbox;
					else
						Rect::Union(textChars.bbox, textChars.bbox, ch.bbox);
				}
			}
			else if(fontType3 == state->getFont()->getType())
				DrawType3Char(state, type3CharPos, 0, charCode);
		} // if (uLen || pFontDesc->m_bEmbedded)
		xText += (scaleX * dx + aspectRatio * scaleY * spacing);
		yText += scaleY * dy;
		type3CharPos += dx + spacing;

		if (dy)
			TextReconstruct();

		p += n;
		l -= n;
	}
}

int XarOutputDev::TextChar::GetUnicodeMask() const
{
	return isAnchorPos ? 0 : ::GetUnicodeMask(c);
}

bool XarOutputDev::TextChar::IsSpace() const
{
	return !isAnchorPos && L' ' == c;
}

bool XarOutputDev::TextChar::IsHyphen() const
{
	return !isAnchorPos && L'-' == c;
}

bool XarOutputDev::TextChar::IsBreak() const
{
	if (!isAnchorPos)
	{
		switch (c)
		{
		case L' ': case L'-':
			return true;
		}
	}
	return false;
}

void XarOutputDev::TextChar::ApplyScriptAttributes(VXfTreeNode *pTreeNode) const
{
	if (FIXED16(1) != scriptScale || FIXED16(0) != scriptOffset)
	{
		VXfTreeNode *pScriptOn = NULL;
		if (FIXED16(0.5) == scriptScale)
		{
			if (FIXED16(-0.1) == scriptOffset)
				pScriptOn = new CXfZeroSizedNode(TAG_TEXT_SUBSCRIPT_ON);
			else if (FIXED16(0.33) == scriptOffset)
				pScriptOn = new CXfZeroSizedNode(TAG_TEXT_SUPERSCRIPT_ON);
		}
		pTreeNode->AddChild(pScriptOn ? pScriptOn : new CXfTextScriptOn(scriptScale, scriptOffset), false);
	}
}

double XarOutputDev::TextChar::GetScaleY() const
{
	return (double)attrs.fontSize / 72000;
}

double XarOutputDev::TextChar::GetScaleX(double scaleY) const
{
	return scaleY * attrs.aspectRatio.MakeDouble() * scriptScale.MakeDouble();
}

double XarOutputDev::TextChar::GetScaleX() const
{
	return GetScaleX(GetScaleY());
}

MILLIPOINT XarOutputDev::TextChar::GetFontSize() const
{
	return ((MILLIPOINT)(72000 * GetScaleX() + 0.5));
}

void XarOutputDev::TextChar::GetMetrics()
{
	if (isAnchorPos || isCacheValid) return;

	if (CXfFontGlyph* pGlyph = pFontDef->GetGlyph(c, attrs.isBold, attrs.isItalic))
	{
		width   = pGlyph->Width;
		ascent  = pGlyph->FontAscent;
		descent = pGlyph->FontDescent;

#ifdef _ENABLE_RELATIVE_LINE_SPACING
		if (FG_HAS_EXTERNAL_LEADING & pGlyph->Flags)
			descent -= pGlyph->FontExternalLeading;
#endif
	}
	else
	{
		DWORD glyphID = 0;
		if (CXfFontOTLigatures *pFontOTLigatures = pFontDef->GetFontLigatures(attrs.isBold, attrs.isItalic))
		{
			std::vector<XarLigatureEntry>::iterator itLigature = GetLigatureEntry(pFontOTLigatures, ligChars);
			if (pFontOTLigatures->ligs.end() != itLigature)
				glyphID = itLigature->GlyphID;
		}

		width   = glyphID ? CFontDC(attrs.pFontDesc->fontName.c_str(), attrs.pFontDesc->weight, attrs.pFontDesc->isItalic).GetGlyphWidth(glyphID) : attrs.pFontDesc->GetCharWidthOS(c);

		ascent  = attrs.pFontDesc->ascent;
		descent = attrs.pFontDesc->descent;
	}

	isCacheValid = true;
}


MILLIPOINT XarOutputDev::TextChar::GetWidth(MILLIPOINT cachedWidth) const
{
	return ((MILLIPOINT)(cachedWidth * GetScaleX() + 0.5));
}

MILLIPOINT XarOutputDev::TextChar::GetWidth() const
{
	if (isAnchorPos)
		return width;
	((TextChar *)this)->GetMetrics();
	return GetWidth(width);
}

MILLIPOINT XarOutputDev::TextChar::GetWidth(wchar_t ch) const
{
	//if (CXfFontGlyph* pGlyph = pFontDef->GetGlyph(ch, attrs.isBold, attrs.isItalic))
	if (CXfFontGlyph* pGlyph = pFontDef->GetGlyph(ch, attrs.pFontDesc->isBold, attrs.pFontDesc->isItalic))
		return GetWidth(pGlyph->Width);
	else
		return GetWidth(attrs.pFontDesc->GetCharWidthOS(ch));
}

MILLIPOINT XarOutputDev::TextChar::GetFontAscent() const
{
	if (isAnchorPos)
		return ascent;
	((TextChar *)this)->GetMetrics();
	return ((MILLIPOINT)(ascent * GetScaleY() + 0.5));
}

MILLIPOINT XarOutputDev::TextChar::GetFontDescent() const
{
	if (isAnchorPos)
		return descent;
	((TextChar *)this)->GetMetrics();
	return ((MILLIPOINT)(descent * GetScaleY() + 0.5));
}

MILLIPOINT XarOutputDev::TextChar::GetTrackingWidth(LONG tracking) const
{
	return MulDiv(tracking, GetFontSize(), 1000);
}

MILLIPOINT XarOutputDev::TextChar::GetTrackingWidth() const
{
	return tracking ? GetTrackingWidth(tracking) : 0;
}

MILLIPOINT XarOutputDev::TextChar::GetKernWidth(LONG kern) const
{
	return MulDiv(kern, GetFontSize(), 1000);
}

MILLIPOINT XarOutputDev::TextChar::GetKernWidth() const
{
	return kern ? GetKernWidth(kern) : 0;
}

MILLIPOINT XarOutputDev::TextChar::GetAdvance(const TextChar *pNextChar) const
{
	return GetWidth() + GetTrackingWidth() + GetAutoKernSize(pNextChar);
}

MILLIPOINT XarOutputDev::TextChar::GetAutoKernSize(const TextChar *pNextChar) const
{
	MILLIPOINT autoKern = 0;
	if (g_settings.bReconstructKerningPairs && pNextChar && !isAnchorPos && !pNextChar->isAnchorPos &&
	    pFontDef == pNextChar->pFontDef &&
	    attrs.pFontDesc == pNextChar->attrs.pFontDesc &&
	    attrs.fontSize == pNextChar->attrs.fontSize &&
	    attrs.isBold == pNextChar->attrs.isBold &&
	    attrs.isItalic == pNextChar->attrs.isItalic &&
	    scriptScale == pNextChar->scriptScale &&
	    scriptOffset == pNextChar->scriptOffset)
	{
		if (CXfFontGlyph* pGlyph = pFontDef->GetGlyph(c, attrs.isBold, attrs.isItalic))
		{
			if (CXfFontKerningPairs *pKernPairs = pFontDef->GetFontKerningPairs(attrs.isBold, attrs.isItalic))
			{
				for (auto it = pKernPairs->Pairs.begin(); it != pKernPairs->Pairs.end(); ++it)
				{
					if (c == it->First && pNextChar->c == it->Second)
					{
						autoKern = it->Amount;
						break;
					}
				}
			}
		}
		else
		{
			autoKern = attrs.pFontDesc->GetKernSizeOS(c, pNextChar->c);
		}
	}
	return autoKern ? ((MILLIPOINT)(autoKern * GetScaleX() + 0.5)) : 0;
}

Rect XarOutputDev::TextChar::GetBBox(bool bPrecise) const
{
	Rect bbox;

	if (isAnchorPos)
		bPrecise = false;

	if (bPrecise)
	{
		// Paul: since now we again have early embedding, we have all the glyphs, so do a more precise bbox
		// computation according to each glyph bbox, excluding starting and trailing spaces since they do not count in graphics
		// intersection. This fixes intersection with 'er time, ' of an underline on a page 3 of FCR-3-pager-v4.pdf
		if (CXfFontGlyph* pGlyph = pFontDef->GetGlyph(c, attrs.isBold, attrs.isItalic))
			bbox = pGlyph->Bounds();
		else
		{
			if (!attrs.pFontDesc->GetCharBoundingBoxOS(c, bbox))
				bPrecise = false;
		}

		if (bPrecise && bbox.empty())
		{
			bbox.x = (MILLIPOINT)(x + 0.5);
			bbox.y = (MILLIPOINT)(y + 0.5);	
			return bbox;
		}
	}

	if (!bPrecise)
	{
		((TextChar *)this)->GetMetrics();

		bbox.x = 0;
		bbox.width = width;
		bbox.y = descent;
		bbox.height = ascent - descent;
	}

	double sy = isAnchorPos ? 1.0 : GetScaleY();
	double sx = isAnchorPos ? 1.0 : GetScaleX(sy);

	bbox.x      = (MILLIPOINT)(x + bbox.x * sx + 0.5);
	bbox.y      = (MILLIPOINT)(y + bbox.y * sy + 0.5);
	bbox.width  = (MILLIPOINT)(bbox.width * sx + 0.5);
	bbox.height = (MILLIPOINT)(bbox.height * sy + 0.5);

	return bbox;
}

void XarOutputDev::drawString(GfxState * state, const GooString *s)
{
	_ASSERT(!g_settings.bTextAsOutline);

	if (!s->getLength() || state->getRender() == 3 || m_clip.Empty()) // text rendeing mode 3 == invisible text
		return;

	MergeSmallBitmaps();

	if (state->getRender() >= 4 && !m_pTextClipController)
		m_pTextClipController = new CXfClipviewController();

	if (textChars.empty())
	{
	 	textMatrix = GetTextMatrix(state, textMatrixInv, textXScaleNorm, textYScale, xText, yText, textShearFactor);
	}
	else
	{
		AffineTransform matrix, matrixInv;
		matrix = GetTextMatrix(state, matrixInv, textXScaleNorm, textYScale, xText, yText, textShearFactor);
	}

	TextParseString(state, s, textXScaleNorm, textYScale, xText, yText, textShearFactor);
	//TextReconstruct();
}

void XarOutputDev::TextLine::RegisterAttributes()
{
	for (const_iterator it = begin(); it != end(); ++it)
	{
		// text aspect ratio
		attrs.aspectRatio.Register(it->attrs.aspectRatio);

		//attrs.tracking   .Register(it->tracking);

		//attrs.baseLineShift.Register(it->baseLineShift);

		// font
		attrs.pFontDef   .Register(it->pFontDef);
		attrs.fontSize   .Register(it->attrs.fontSize);
		attrs.isBold     .Register(it->attrs.isBold);
		attrs.isItalic   .Register(it->attrs.isItalic);

		// add fill and stroke attributes
		attrs.isStroked  .Register(it->attrs.isStroked);
		attrs.isFilled   .Register(it->attrs.isFilled);

		attrs.lineJoin   .Register(it->attrs.lineJoin);
		attrs.lineCap    .Register(it->attrs.lineCap);
		attrs.miterLimit .Register(it->attrs.miterLimit);
		attrs.lineWidth  .Register(it->attrs.lineWidth);

		if (it->attrs.isStroked)
		{
			attrs.colorStroke.Register(it->attrs.colorStroke);
			attrs.opacityStroke.Register(it->attrs.opacityStroke);

			if (it->attrs.dash.size())
				attrs.dash.Register(it->attrs.dash);
		}

		if (it->attrs.isFilled)
		{
			attrs.colorFill.Register(it->attrs.colorFill);
			attrs.opacityFill.Register(it->attrs.opacityFill);
		}
	}
}

void XarOutputDev::beginString(GfxState * state, const GooString * s)
{
	RD_ARG(s->c_str());
}

void XarOutputDev::endString(GfxState * state)
{RD;
}

void XarOutputDev::DrawType3Char(GfxState * state, double x, double y, CharCode code)
{
	if (fontType3 == state->getFont()->getType() && 3 != (3 & state->getRender()))
	{
		Gfx8BitFont *pFont = (Gfx8BitFont *)state->getFont();
		Object charProc = pFont->getCharProc(code);

		if (charProc.isStream())
		{					
			m_bType3Char = true;

			m_pGfx->saveState();

			const double *textMatrix = state->getTextMat();
			m_pGfx->getState()->concatCTM(textMatrix[0], textMatrix[1], textMatrix[2], textMatrix[3], state->getCurX(), state->getCurY() + state->getRise()/*textMatrix[4], textMatrix[5]*/);
			m_pGfx->getState()->concatCTM(state->getFontSize() * state->getHorizScaling(), 0, 0, state->getFontSize(), 0, 0);
			m_pGfx->getState()->concatCTM(1, 0, 0, 1, x, y);
			const double *fontMatrix = pFont->getFontMatrix();
			m_pGfx->getState()->concatCTM(fontMatrix[0], fontMatrix[1], fontMatrix[2], fontMatrix[3], fontMatrix[4], fontMatrix[5]);
			double matrix[6] = {1, 0, 0, 1, 0, 0};
			m_pGfx->drawForm(&charProc, pFont->getResources(), matrix, pFont->getFontBBox());
			m_pGfx->restoreState();

			m_bType3Char = false;

			if (m_type3FillColorSpace)
			{
				delete m_type3FillColorSpace;
				m_type3FillColorSpace = NULL;
			}
			if (m_type3StrokeColorSpace)
			{
				delete m_type3StrokeColorSpace;
				m_type3StrokeColorSpace = NULL;
			}
		}
	}
}

void XarOutputDev::drawChar(GfxState * state, double x, double y,
							double dx, double dy,
							double originX, double originY,
							CharCode code, int nBytes, Unicode * u, int uLen)
{
	_ASSERT(g_settings.bTextAsOutline);

	if (m_clip.Empty()) return;

	if(fontType3 == state->getFont()->getType())
	{
		DrawType3Char(state, 0, 0, code);
		return;
	}

	SplashPath *path;
	//double m[4];
	//bool horiz;
	std::stringstream ss;

	// check for invisible text -- this is used by Acrobat Capture
	if (3 == state->getRender())
		return;

	SplashCoord mat[4];

	// get the font matrix
	const double *textMat = state->getTextMat();
	double fontSize = state->getFontSize();
	mat[0] = textMat[0] * fontSize * state->getHorizScaling();
	mat[1] = textMat[1] * fontSize * state->getHorizScaling();
	mat[2] = textMat[2] * fontSize;
	mat[3] = textMat[3] * fontSize;

	SplashFont *font = m_pFontManager->CreateSplashFont(state->getFont(), mat);
	if (!font)
		return;

	x -= originX;
	y -= originY;
	RD_LOG("'0x" << std::hex << code << std::dec << "', " << "x: " << x << ", y: " << y);

	//bool doFill = IsTextFilled(state)/*!(render & 1)*/ && !state->getFillColorSpace()->isNonMarking();
	//bool doStroke = IsTextStroked(state)/*((render & 3) == 1 || (render & 3) == 2)*/ &&
	//	!state->getStrokeColorSpace()->isNonMarking();
	//bool doClip = render & 4;

	path = font->getGlyphPath(code);
	/*
	if (doStroke || doClip) {
		if ((path = font->getGlyphPath(code))) {
			path->offset((SplashCoord)x, (SplashCoord)y);
		
		}
	}
	*/
	if (path)
	{
		RD_LOG(", points: " << path->getLength() << ' ');
		RD_ARG(ss.str());
		path->offset((SplashCoord)x, (SplashCoord)y);
		// Paul: we always, ALWAYS have to create path with at least stroked attribute - this is because of the
		// 'outline' mode in Camelot - path ALWAYS should be at least stroked

		CXfPath* pPath = SplashToXaraPath(path, state, TAG_PATH_FILLED_STROKED);
		if (AddGraphics(pPath, pPath, CXF_WINDING_RULE_NONZERO))
		{
			AppendAttributes(pPath, state, IsTextStroked(state), IsTextFilled(state));
			//pPath->AddChild(new CXfWindingRule(byWindingRule));
		}
		else delete pPath;
	}

	RD_ARG(ss.str());
}

void XarOutputDev::incCharCount(int nChars)
{RD;
}

void XarOutputDev::beginActualText(GfxState * state, const GooString * text )
{
	RD_ARG(text->c_str());
}

void XarOutputDev::endActualText(GfxState * state)
{RD;
}

bool XarOutputDev::beginType3Char(GfxState *state, double x, double y, double dx, double dy, CharCode code, Unicode *u, int uLen)
{ RD;
	m_bType3Char = true;
	return OutputDev::beginType3Char(state, x, y, dx, dy, code, u, uLen);
}

void XarOutputDev::endType3Char(GfxState * state)
{RD;
	if (state) OutputDev::endType3Char(state);

	m_bType3Char = false;

	if (m_type3FillColorSpace)
	{
		delete m_type3FillColorSpace;
		m_type3FillColorSpace = NULL;
	}
	if (m_type3StrokeColorSpace)
	{
		delete m_type3StrokeColorSpace;
		m_type3StrokeColorSpace = NULL;
	}
}

void XarOutputDev::type3D0(GfxState *state, double wx, double wy)
{
}

/*A glyph description that begins with the d1 operator should not execute any operators
  that set the colour (or other colour-related parameters) in the graphics state;
  any use of such operators shall be ignored.
  The glyph description is executed solely to determine the glyph's shape.
  Its colour shall be determined by the graphics state in effect
  each time this glyph is painted by a text-showing operator.*/
void XarOutputDev::type3D1(GfxState *state, double wx, double wy, double llx, double lly, double urx, double ury)
{
	if (m_bType3Char)
	{
		if (!m_type3FillColorSpace && state->getFillColorSpace())
		{
			m_type3FillColorSpace = state->getFillColorSpace()->copy();
			m_type3FillColor = *(state->getFillColor());
		}
		if (!m_type3StrokeColorSpace && state->getStrokeColorSpace())
		{
			m_type3StrokeColorSpace = state->getStrokeColorSpace()->copy();
			m_type3StrokeColor = *(state->getStrokeColor());
		}
	}
}

GfxColorSpace *XarOutputDev::GetFillColorSpace(GfxState *state)
{
	return m_type3FillColorSpace ? m_type3FillColorSpace : state->getFillColorSpace();
}

GfxColorSpace *XarOutputDev::GetStrokeColorSpace(GfxState *state)
{
	return m_type3StrokeColorSpace ? m_type3StrokeColorSpace : state->getStrokeColorSpace();
}

const GfxColor *XarOutputDev::GetFillColor(GfxState *state)
{
	return m_type3FillColorSpace ? &m_type3FillColor : state->getFillColor();
}

const GfxColor *XarOutputDev::GetStrokeColor(GfxState *state)
{
	return m_type3StrokeColorSpace ? &m_type3StrokeColor : state->getStrokeColor();
}

XarOutputDev::Paragraphs::iterator XarOutputDev::TextStory::Finalize(Paragraphs::iterator itBegin, Paragraphs::iterator itEnd)
{
	for (Paragraphs::const_iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
	{
		MILLIPOINT paragraphLeftMargin = min(itParagraph->firstMargin, itParagraph->leftMargin);
		if (itBegin == itParagraph)
		{
			leftMargin  = paragraphLeftMargin;
			rightMargin = itParagraph->rightMargin;
		}
		else
		{
			if (paragraphLeftMargin < leftMargin) leftMargin = paragraphLeftMargin;
			if (itParagraph->rightMargin > rightMargin) rightMargin = itParagraph->rightMargin;
		}
	}

	// Fix for DDES-14451 (Add newlines instead of changing right margin setting to restrict width?)
	for (Paragraphs::iterator itParagraph = itBegin; itParagraph != itEnd;)
	{
		Paragraphs::iterator itNextParagraph = itParagraph;
		++itNextParagraph;

		if (ALIGN_LEFT == itParagraph->GetAlign())
		{
			bool bRightMargin = 0 != (rightMargin - itParagraph->rightMargin);

			if (bRightMargin)
			{
				bRightMargin = false;

				for (TextLines::const_iterator itLine = itParagraph->begin(); itLine != itParagraph->end();)
				{
					TextLines::const_iterator itNextLine = itLine;
					++itNextLine;

					if (itParagraph->end() != itNextLine)
					{
						MILLIPOINT leftMargin = itParagraph->begin() == itLine ? itParagraph->firstMargin : itParagraph->leftMargin;
						MILLIPOINT maxWidth = itLine->GetMaxWidth(&(*itNextLine), 1);

						if (leftMargin + maxWidth < rightMargin)
						{
							bRightMargin = true;
							break;
						}
					}

					itLine = itNextLine;
				}

				if (!bRightMargin)
				{
					itParagraph->rightMargin = rightMargin;
					itParagraph->ConvertSpaces(1);
				}
			}

			if (ALIGN_LEFT == itParagraph->GetAlign() && bRightMargin && !itParagraph->pPrevInFlow && !itParagraph->pNextInFlow)
			{
				MILLIPOINT paragraphWidth = itParagraph->rightMargin - min(itParagraph->firstMargin, itParagraph->leftMargin);
				MILLIPOINT rightIndent = rightMargin - itParagraph->rightMargin;

				if (rightIndent > 0.1 * paragraphWidth)
				{
					MILLIPOINT spaceBefore = itParagraph->spaceBefore;
					bool isFirst = (itBegin == itParagraph);
					while (!itParagraph->empty())
					{
						Paragraphs::iterator itNewParagraph = insert(itParagraph, Paragraph());

						if (isFirst)
						{
							itBegin = itNewParagraph;
							isFirst = false;
						}

						itNewParagraph->splice(itNewParagraph->end(), *itParagraph, itParagraph->begin());

#ifdef _ENABLE_RELATIVE_LINE_SPACING
						itNewParagraph->linespaceRatio = itParagraph->linespaceRatio;
#else
						itNewParagraph->linespacing = itParagraph->linespacing;
#endif

						itNewParagraph->spaceBefore = spaceBefore;
						spaceBefore = 0;

						if (itParagraph->empty())
							itNewParagraph->spaceAfter = itParagraph->spaceAfter;
					}

					erase(itParagraph);
				}
			}
		}

		itParagraph = itNextParagraph;
	}
	// End of fix for DDES-14451 (Add newlines instead of changing right margin setting to restrict width?)

	for (Paragraphs::iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
	{
		if (1 == itParagraph->size() && !itParagraph->pPrevInFlow && !itParagraph->pNextInFlow)
		{
			TextLine &line = itParagraph->front();

			bool bHasTabStops = false;
			for (TextLine::const_iterator itChar = line.begin(); itChar != line.end(); ++itChar)
			{	
				if (itChar->isTabStop)
				{
					bHasTabStops = true;
					break;
				}
			}

#if defined(_DEBUG)
			LONG a = (line.leftMargin - leftMargin);
			LONG b = (rightMargin - line.endMargin);
			LONG diff = abs(a - b);
			LONG t = line.GetMaxTrackingWidth(MAX_RELATIVE_OFFSET);
#endif

			if (line.front().isAutoListChar || bHasTabStops || leftMargin == line.leftMargin)
				itParagraph->possibleAlign = ALIGN_LEFT;
			else if (abs((line.leftMargin - leftMargin) - (rightMargin - line.endMargin)) <= (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : line.GetMaxTrackingWidth(MAX_CENTER_ALIGN_RELATIVE_OFFSET)))
				itParagraph->possibleAlign = ALIGN_CENTER;
			else if (abs(rightMargin - line.endMargin) <= (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : line.GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
				itParagraph->possibleAlign = ALIGN_RIGHT;
			else
				itParagraph->possibleAlign = ALIGN_LEFT;

			if (ALIGN_LEFT == itParagraph->GetAlign())
				itParagraph->firstMargin = line.leftMargin;
			else
				itParagraph->firstMargin = leftMargin;

			itParagraph->leftMargin = leftMargin;
			itParagraph->rightMargin = rightMargin;
		}

/*#ifdef _ENABLE_TAB_STOP_DETECTION
		itParagraph->DetectTabStops();
#endif*/
		for (TextLines::iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
		{
			TextChars::iterator itPrevChar = itLine->end();
			for (TextChars::iterator itChar = itLine->begin(); itChar != itLine->end(); itPrevChar = itChar++)
			{
				if (itChar->isTabStop && itLine->end() != itPrevChar)
				{
					itPrevChar->kern += itPrevChar->tracking;
					itPrevChar->tracking = 0;
				}
			}
		}
	}

	UINT align = 0;
	for (Paragraphs::iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
	{
		if (!itParagraph->pPrevInFlow)
		{
			switch( align = itParagraph->GetAlign())
			{
			case ALIGN_CENTER: case ALIGN_RIGHT:
				if (!itParagraph->front().front().isAutoListChar)
				{
					itParagraph->firstMargin = itParagraph->leftMargin = min(itParagraph->firstMargin, itParagraph->leftMargin);
					itParagraph->possibleAlign &= ~(ALIGN_LEFT | ALIGN_FULL);
					break;
				}
			}
		}
		TextLine *pNextLine = itParagraph->pNextInFlow ? &itParagraph->pNextInFlow->front() : NULL;
		for (TextLines::reverse_iterator it = itParagraph->rbegin(); it != itParagraph->rend();)
		{
			TextLines::reverse_iterator itNext = it;
			++itNext;
			MILLIPOINT fitWidth = itParagraph->rightMargin - (itParagraph->rend() == itNext ? itParagraph->firstMargin : itParagraph->leftMargin);
			MILLIPOINT justifyWidth = ALIGN_FULL == align && NULL != pNextLine ? fitWidth : 0;

			if (it->CheckWrapping(fitWidth, pNextLine))
				it->RemoveKernings(fitWidth, justifyWidth, MAX_RELATIVE_OFFSET, pNextLine);
			else
				it->RemoveKernings(fitWidth, justifyWidth, TRACKING_TOLERANCE, pNextLine);

			pNextLine = &(*it);
			it = itNext;
		}
	}

	Rect bbox;
	for (Paragraphs::iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
	{
		bool bRightMargin = itParagraph->pPrevInFlow || itParagraph->pNextInFlow || ALIGN_LEFT != itParagraph->GetAlign();

		for (TextLines::const_iterator itLine = itParagraph->begin(); itLine != itParagraph->end();)
		{
			TextLines::const_iterator itNextLine = itLine;
			++itNextLine;

			if (itParagraph->end() != itNextLine && !bRightMargin)
			{
				MILLIPOINT maxWidth = itLine->GetMaxWidth(&(*itNextLine));
				if (itParagraph->begin() == itLine)
				{
					if (itParagraph->firstMargin + maxWidth < rightMargin)
						bRightMargin = true;
				}
				else
				{
					if (itParagraph->leftMargin + maxWidth < rightMargin)
						bRightMargin = true;
				}
			}

			if (!itLine->bbox.empty())
			{
				if (bbox.empty())
					bbox = itLine->bbox;
				else
					Rect::Union(bbox, bbox, itLine->bbox);
			}

			itLine = itNextLine;
		}

		if (!bRightMargin)
			itParagraph->rightMargin = rightMargin;
	}
	nodeStory->SetBounds(bbox);

	return itBegin;
}

bool ToRect(const ClipperLib::Paths &paths, Rect &rect);

void XarOutputDev::TextStory::RenderClipview(Paragraphs::iterator itBegin, Paragraphs::iterator itEnd)
{
	if (bNeedClipview)
	{
		VXfTreeNode* pParent = nodeStory->GetParent();
		if (TAG_CLIPVIEWCONTROLLER != pParent->GetTag())
		{
			Rect subjRect, clipRect;
			bool bSubjRect = false, bClipRect = false;
			try
			{
				ClipperLib::Paths subjPolygons;

				size_t count = 0;
				for (Paragraphs::const_iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
				{
					for (TextLines::const_iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
						count += itLine->size();
				}
				if (subjPolygons.capacity() < count) subjPolygons.reserve(count);

				for (Paragraphs::const_iterator itParagraph = itBegin; itParagraph != itEnd; ++itParagraph)
				{
					for (TextLines::const_iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
						itLine->GetBoundingPolygons(subjPolygons);
				}

				bSubjRect = ToRect(subjPolygons, subjRect);
				bClipRect = ToRect(clipPolygons, clipRect);

				ClipperLib::Clipper c;
				c.AddPaths(subjPolygons, ClipperLib::ptSubject, true);
				c.AddPaths(clipPolygons, ClipperLib::ptClip, true);

				ClipperLib::Paths diffPolygons;

				if (c.Execute(ClipperLib::ctDifference, diffPolygons, XaraFillRuleToPoly(CXF_WINDING_RULE_NONZERO), XaraFillRuleToPoly(clipWindingRule)) && diffPolygons.empty())
					return;
			}
			catch (ClipperLib::clipperException&) {}

			size_t childrenCount = pParent->ChildrenCount();
			for (size_t i = 0; i < childrenCount; ++i)
			{
				if (pParent->GetChild(i) == nodeStory)
				{
					pParent->RemoveChild((UINT)i, false);
					CXfPath* pClipPath = new CXfPath(TAG_PATH_FILLED_STROKED);

					// DDES-16109 fix
					if (bSubjRect && bClipRect && (clipRect.width >= 5000000 || clipRect.height >= 5000000))
					{
						clipRect.Intersect(subjRect);
						ClipperLib::Paths polygons;
						AddRect(polygons, clipRect);
						PolygonsToPath(pClipPath, polygons);
					}
					else
						PolygonsToPath(pClipPath, clipPolygons);

					CXfClipviewController* pController = CreateClipviewController(pClipPath, clipWindingRule);
					pController->AddChild(nodeStory);
					pParent->InsertBefore(i, pController);
					break;
				}
			}
		}
	}
}

void XarOutputDev::TextBreakStory()
{
	for (TextStories::iterator textStoriesIt = textStories.begin(); textStoriesIt != textStories.end(); ++textStoriesIt)
		RenderToOptimizedTags(*textStoriesIt);
	textStories.clear();
}

void XarOutputDev::OnEndPage()
{
	OnEndTextObject();
	MergeSmallBitmaps();
	TextBreakStory();
	ClearUnderlineStrikethrough();
	endType3Char(NULL);
}

void XarOutputDev::OnEndTextObject()
{
	if (m_pTextClipController)
	{
		m_pTextClipController->AddChild(new CXfZeroSizedNode(TAG_CLIPVIEW));
		MergeSmallBitmaps();
		m_pInsertTo->AddChild(m_pTextClipController);
		SetParentNode(m_pTextClipController);
		m_clipControllers.push_back(m_pTextClipController);
		m_pTextClipController = NULL;
	}
}

/*inline bool CheckBBoxIntersection(double *bbox1, double *bbox2)
{
	double cx1 = bbox1[0] + .5 * bbox1[2];
	double cy1 = bbox1[1] + .5 * bbox1[3];
	double cx2 = bbox2[0] + .5 * bbox2[2];
	double cy2 = bbox2[1] + .5 * bbox2[3];
	return ( 2.*fabs(cx1-cx2) < (bbox1[2]+bbox2[2]) )
		&& ( 2.*fabs(cy1-cy2) < (bbox1[3]+bbox2[3]) );
}*/

#if defined(_ELEMENT_DRAW) || defined(_KERNING_DRAW)
void _Debug_BoundingPointsBBox(VXfTreeNode *layer, const Rect &bbox, double colorGray)
{
	if (!layer)
		return;

	CXfPath* debugPath = new CXfPath(TAG_PATH_FILLED_STROKED);
	debugPath->Verbs.push_back(PT_MOVETO);
	debugPath->Verbs.push_back(PT_LINETO);
	debugPath->Verbs.push_back(PT_LINETO);
	debugPath->Verbs.push_back(PT_LINETO | PT_CLOSEFIGURE);

	debugPath->Coords.push_back(DocCoord(bbox.x, bbox.y));
	debugPath->Coords.push_back(DocCoord(bbox.x + bbox.width, bbox.y));
	debugPath->Coords.push_back(DocCoord(bbox.x + bbox.width, bbox.y + bbox.height));
	debugPath->Coords.push_back(DocCoord(bbox.x, bbox.y + bbox.height));


	COLORREF crRGB = RGB(BYTE(0xFF*colorGray),BYTE(0xFF*colorGray),BYTE(0xFF*colorGray));
	CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, colorGray,colorGray,colorGray, 0);
	debugPath->InsertFirstChild(new CXfLineColor(color));
	debugPath->InsertFirstChild(color);

	layer->AddChild(debugPath);
}
#endif

void XarOutputDev::_Debug_FlowLine(VXfTreeNode *layer, double xStart, double yStart, double xEnd, double yEnd, bool isNewStory)
{
	if (!layer)
		return;

	CXfPath* debugPath = new CXfPath(TAG_PATH_FILLED_STROKED);
	debugPath->Verbs.push_back(PT_MOVETO);
	debugPath->Verbs.push_back(PT_LINETO | PT_CLOSEFIGURE);
	debugPath->Coords.push_back(DocCoord((LONG)xStart, (LONG)yStart));
	debugPath->Coords.push_back(DocCoord((LONG)xEnd,   (LONG)yEnd));

	COLORREF crRGB = RGB(0x00,0x00,0xFF);
	CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, 0.,0.,1., 0);
	debugPath->InsertFirstChild(new CXfLineColor(color));
	debugPath->InsertFirstChild(color);

	layer->AddChild(debugPath);

	if (m_xBaseOld != DBL_MAX && m_yBaseOld != DBL_MAX)
	{
		debugPath = new CXfPath(TAG_PATH_FILLED_STROKED);
		debugPath->Verbs.push_back(PT_MOVETO);
		debugPath->Verbs.push_back(PT_LINETO | PT_CLOSEFIGURE);
		debugPath->Coords.push_back(DocCoord((LONG)xStart,   (LONG)yStart));
		debugPath->Coords.push_back(DocCoord((LONG)m_xBaseOld, (LONG)m_yBaseOld));

		if (isNewStory)
		{
			COLORREF crRGB = RGB(0x00,0x60,0x60);
			CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, 0.,.375,.375, 0);
			debugPath->InsertFirstChild(new CXfLineColor(color));
			debugPath->InsertFirstChild(color);
		}
		else
		{
			COLORREF crRGB = RGB(0x00,0xFF,0x00);
			CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, 0.,1.,0., 0);
			debugPath->InsertFirstChild(new CXfLineColor(color));
			debugPath->InsertFirstChild(color);
		}

		layer->AddChild(debugPath);
	}

	m_xBaseOld = xEnd;
	m_yBaseOld = yEnd;
}

void XarOutputDev::FinalizeLine(TextStory& textStory, TextLine &line)
{
#ifdef _ENABLE_EMBEDDED_GRAPHICS
	DetectEmbeddedGraphics(textStory, line);
#endif

	while (!line.empty() && line.front().bbox.empty()) line.pop_front();
	while (!line.empty() && line.back().bbox.empty()) line.pop_back();

#ifdef _ENABLE_WHITESPACE_RECONSTRUCTION
	line.ReconstructWhiteSpaces();
#endif

	for (TextLine::iterator it = line.begin(); it != line.end();)
	{	
		TextLine::iterator itNext = it;
		++itNext;

		if (line.end() != itNext && /*!it->IsSpace() && !itNext->IsSpace() && */it->attrs.fontSize)
		{
			double scale = fabs((double)itNext->attrs.fontSize / (double)it->attrs.fontSize);
			if (scale > 0.4 && scale < 0.7)
			{
				itNext->scriptOffset = (itNext->y - it->y) / (double)it->attrs.fontSize;
				if (FIXED16(0) != itNext->scriptOffset)
				{
					itNext->y = it->y;
					itNext->scriptScale = scale;
					itNext->attrs.fontSize = it->attrs.fontSize;
				}
			}
			else if (line.begin() == it)
			{
				scale = 1.0 / scale;
				if (scale > 0.4 && scale < 0.7 || scale >= MIN_DROPCAP_SIZE)
				{
					it->scriptOffset = (it->y - itNext->y) / (double)itNext->attrs.fontSize;
					if (FIXED16(0) != it->scriptOffset)
					{
						it->y = itNext->y;
						it->scriptScale = scale;
						it->attrs.fontSize = itNext->attrs.fontSize;
					}
				}
			}
		}

		it = itNext;
	}

	MILLIPOINT def = _round(line.front().y);
	SeriesCollector<MILLIPOINT> y(def);
	for (TextLine::iterator it = line.begin(); it != line.end(); ++it)
		y.Register(_round(it->y));

	line.baseLine = y(def);

	for (TextLine::iterator it = line.begin(); it != line.end(); ++it)
	{	
		it->baseLineShift = _round(it->y) - line.baseLine;

		MILLIPOINT fontSize = it->GetFontSize();
		if (fontSize > line.maxFontSize)
			line.maxFontSize = fontSize;
	}

	DetectUnderlineStrikethrough(textStory, line);
}

void XarOutputDev::TextLine::UpdateAscentDescent()
{
	ascent = descent = 0;

	if (!empty() && front().isAutoListChar)
	{
		MILLIPOINT charAscent  = 0;
		MILLIPOINT charDescent = 0;

		if (CXfFontGlyph* pGlyph = front().pFontDef->GetGlyph(L'M', front().attrs.isBold, front().attrs.isItalic))
		{
			charAscent  = pGlyph->FontAscent;
			charDescent = pGlyph->FontDescent;

#ifdef _ENABLE_RELATIVE_LINE_SPACING
			if (FG_HAS_EXTERNAL_LEADING & pGlyph->Flags)
				charDescent -= pGlyph->FontExternalLeading;
#endif
		}
		else
		{
			const SystemFontInfo &substitute = front().attrs.pFontDesc->GetSubstitute(front().pFontDef, front().attrs.isBold, front().attrs.isItalic);
			FontMetrics metrics = GetFontMetricsOS(substitute.fontName.c_str(), substitute.weight, substitute.isItalic);
			charAscent = metrics.ascent;
			charDescent = metrics.descent;
		}		

		charAscent = ((MILLIPOINT)(charAscent * front().GetScaleY() + 0.5));
		charDescent = ((MILLIPOINT)(charDescent * front().GetScaleY() + 0.5));

		if (charDescent < descent)
			descent = charDescent;
		if (charAscent > ascent)
			ascent = charAscent;
	}

	for (iterator it = begin(); it != end(); ++it)
	{	
		MILLIPOINT charAscent  = /*it->baseLineShift + */it->GetFontAscent();
		MILLIPOINT charDescent = /*it->baseLineShift + */it->GetFontDescent();

		if (charDescent < descent)
			descent = charDescent;
		if (charAscent > ascent)
			ascent = charAscent;
	}
}

/******************************************************************************
   Function:	GetStrikethroughWidth
   Author:	Denis Onishchenko
   Purpose:     Calculates the line width of strikethrough for this character
   Returns:
   Notes:       
******************************************************************************/

MILLIPOINT XarOutputDev::TextChar::GetStrikethroughWidth() const
{
	return (MILLIPOINT)(attrs.fontSize * scriptScale.MakeDouble() + 0.5) * UNDERLINE_WIDTH_RATIO;
}

/******************************************************************************
   Function:	GetStrikethroughPos
   Author:	Denis Onishchenko
   Purpose:     calculates the vertical position of strikethrough for this character
   Returns:
   Notes:       
******************************************************************************/

MILLIPOINT XarOutputDev::TextChar::GetStrikethroughPos() const
{
	return _round(y) + (MILLIPOINT)(attrs.fontSize * scriptScale.MakeDouble() + 0.5) * STRIKETHROUGH_DISTANCE_RATIO;
}

/******************************************************************************
   Function:	GetUnderlinePos
   Author:	Denis Onishchenko
   Purpose:     calculates the vertical position and line width of underline for range of characters
   Returns:
   Parameters:	itBegin - begin of character's range
   Parameters:	itEnd - begin of character's range
   Notes:       
******************************************************************************/

MILLIPOINT XarOutputDev::TextLine::GetUnderlinePos(const_iterator itBegin, const_iterator itEnd, MILLIPOINT &width) const
{
	if (itBegin == itEnd)
	{
		width = 0;
		return baseLine;
	}

	MILLIPOINT maxFontSize = itBegin->attrs.fontSize;
	MILLIPOINT minBaseLineShift = itBegin->baseLineShift;

	for (const_iterator it = ++itBegin; it != itEnd; ++it)
	{
		if (it->attrs.fontSize > maxFontSize)
			maxFontSize = it->attrs.fontSize;

		if (it->baseLineShift < minBaseLineShift)
			minBaseLineShift = it->baseLineShift;
	}

	width = maxFontSize * UNDERLINE_WIDTH_RATIO;
	return baseLine + minBaseLineShift - maxFontSize * UNDERLINE_DISTANCE_RATIO;
}	

/******************************************************************************
   Function:	CheckUnderlineStrikethrough
   Author:	Denis Onishchenko
   Purpose:     checks whether the specified rectangle can be recognized as an underline or strikethrough
   Returns:
   Parameters:	rect - input rectangle, color, opacity
   Notes:       
******************************************************************************/

bool XarOutputDev::TextLine::CheckUnderlineStrikethrough(const Rect &rect, const Color &color, const Opacity &opacity)
{
	static const double HORIZONTAL_POS_TOLERANCE = 0.1; // relative to glyph width
	static const double VERTICAL_POS_TOLERANCE   = 1.1; // relative to width of underline/strikethrough
	static const double WIDTH_TOLERANCE          = 1.1; // relative to width of underline/strikethrough

	for (iterator itBegin = begin(); itBegin != end(); ++itBegin)
	{
		double width = itBegin->GetWidth();
		double tolerance = HORIZONTAL_POS_TOLERANCE * width;

		if (fabs(rect.x - itBegin->x) <= tolerance && (itBegin->x + width) - (rect.x + rect.width) <= tolerance)
		{
			iterator itEnd = itBegin;
			iterator itLast = itEnd++;
			for (; itEnd != end(); itLast = itEnd++)
			{
				width = itEnd->GetWidth();
				tolerance = HORIZONTAL_POS_TOLERANCE * width;

				if ((rect.x - itEnd->x) > tolerance || (itEnd->x + width) - (rect.x + rect.width) > tolerance)
					break;
			}

			width = itLast->GetWidth();
			tolerance = HORIZONTAL_POS_TOLERANCE * width;

			if (fabs((itLast->x + width) - (rect.x + rect.width)) <= tolerance && (rect.x - itLast->x) <= tolerance)
			{
				for (iterator it = itBegin; it != itEnd; ++it)
				{
					if (!it->attrs.isFilled || it->attrs.isStroked || color != it->attrs.colorFill || opacity != it->attrs.opacityFill)
						return false;
				}

				MILLIPOINT lineWidth = rect.height;
				MILLIPOINT linePos   = rect.y + rect.height / 2;

				MILLIPOINT underlineWidth = 0;
				MILLIPOINT underlinePos   = GetUnderlinePos(itBegin, itEnd, underlineWidth);

				if (abs(underlinePos - linePos) <= VERTICAL_POS_TOLERANCE * underlineWidth &&
				    abs(underlineWidth - lineWidth) <= WIDTH_TOLERANCE * underlineWidth)
				{
					for (iterator it = itBegin; it != itEnd; ++it)
						it->isUnderline = true;
					return true;
				}

				MILLIPOINT strikethroughWidth = itBegin->GetStrikethroughWidth();
				MILLIPOINT strikethroughPos   = itBegin->GetStrikethroughPos();

				if (abs(strikethroughPos - linePos) <= VERTICAL_POS_TOLERANCE * strikethroughWidth &&
				    abs(strikethroughWidth - lineWidth) <= WIDTH_TOLERANCE * strikethroughWidth)
				{
					bool bUniform = true;

					iterator it = itBegin;
					for (++it; it != itEnd; ++it)
					{
						MILLIPOINT width = it->GetStrikethroughWidth();
						MILLIPOINT pos   = it->GetStrikethroughPos();

						if (width != strikethroughWidth || pos != strikethroughPos)
						{
							bUniform = false;
							break;
						}
					}

					if (bUniform)
					{
						for (iterator it = itBegin; it != itEnd; ++it)
							it->isStrikethrough = true;
						return true;
					}
				}
			}
		}
	}

	return false;
}

size_t XarOutputDev::TextLine::GetListHeadCharCount() const
{
	size_t count = 0;

	for (auto itChar = begin(); itChar != end() && itChar->isAutoListChar; ++itChar)
		++count;

	return count;
}

void XarOutputDev::TextLine::ClearAutoListItemFlags()
{
	for (auto itChar = begin(); itChar != end() && itChar->isAutoListChar; ++itChar)
		itChar->isAutoListChar = false;
}

void XarOutputDev::RenderToOptimizedTags(TextStory& textStory)
{
	for (TextLines::iterator itLine = textStory.lines.begin(); itLine != textStory.lines.end(); ++itLine)
	{
		FinalizeLine(textStory, *itLine);
	}

#ifdef _ENABLE_TAB_STOP_DETECTION
	textStory.DetectTabStops();
#endif

#ifdef _ENABLE_BULLETED_AND_NUMBERED_LISTS

	for (TextLines::iterator itLine = textStory.lines.begin(); itLine != textStory.lines.end(); ++itLine)
	{
		itLine->RecognizeListItemHead();
	}

	/*for (TextLines::iterator itLine = textStory.lines.begin(); itLine != textStory.lines.end(); ++itLine)
	{
		if (itLine->front().isAutoListChar)
		{
			TextLine::iterator it = itLine->begin();

			while (it != itLine->end() && it->isAutoListChar) ++it;

			for (; it != itLine->end() && 0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask());)
			{
				TextLine::iterator itSpace = it++;
				itLine->erase(itSpace);
			}			

			if (itLine->end() != it)
				it->isTabStop = true;
		}
	}*/
#endif

	for (TextLines::iterator itLine = textStory.lines.begin(); itLine != textStory.lines.end(); ++itLine)
	{
		itLine->UpdateAscentDescent();

#ifdef _ENABLE_SMALL_KERNING_REPLACEMENT
		itLine->ReplaceKernings();
#endif

		itLine->PositionChars();
		//itLine->RegisterAttributes();
	}

/*#ifdef _ENABLE_TAB_STOP_DETECTION
	textStory.DetectTabStops();
#endif*/

	textStory.DetectParagraphs();

	for (Paragraphs::iterator itParagraph = textStory.begin(); itParagraph != textStory.end(); ++itParagraph)
	{
		TextLine &line = itParagraph->front();

		if (line.front().isAutoListChar)
		{
			TextLine::iterator it = line.begin();

			while (it != line.end() && it->isAutoListChar) ++it;

			for (; it != line.end() && 0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask());)
			{
				TextLine::iterator itSpace = it++;
				line.erase(itSpace);
			}			

			if (line.end() != it)
				it->isTabStop = true;
		}

		for (TextLines::iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
		{
			itLine->UpdateAscentDescent();
			itLine->PositionChars();

			itLine->RegisterAttributes();
		}
	}

#ifdef _ENABLE_BULLETED_AND_NUMBERED_LISTS

	// Fix for DDES-15114 (PDF import creates strange representations for bulleted lists)
	// if each list entry has the same symbol at the front, so it is by definition not a numbered list. 

	for (Paragraphs::iterator itParagraph = textStory.begin(); itParagraph != textStory.end();)
	{
		Paragraphs::iterator itNextParagraph = itParagraph;
		++itNextParagraph;

		if (itParagraph->front().front().isAutoListChar && !itParagraph->front().numberingFormat.empty() && 1 == itParagraph->front().GetListHeadCharCount())
		{
			Paragraphs::iterator itFirstItem = itParagraph;
			Paragraphs::iterator itLastItem  = itParagraph;

			while (itNextParagraph != textStory.end() && itNextParagraph->front().front().isAutoListChar && 1 == itNextParagraph->front().GetListHeadCharCount() &&
			       itParagraph->front().number             == itNextParagraph->front().number &&
			       itParagraph->front().numberAlt          == itNextParagraph->front().numberAlt &&
			       itParagraph->front().numberingFormat    == itNextParagraph->front().numberingFormat &&
			       itParagraph->front().numberingFormatAlt == itNextParagraph->front().numberingFormatAlt)
			{
				itLastItem = itNextParagraph;
				++itNextParagraph;
			}

			if (itFirstItem != itLastItem)
			{
				do
				{
					itParagraph->front().numberingFormat.clear();
					itParagraph->front().numberingFormatAlt.clear();
					++itParagraph;
				}
				while (itParagraph != itNextParagraph);

				continue;
			}
		}


		itParagraph = itNextParagraph;
	}
#endif

	bool bIsClippingPath = textStory.nodeStory->IsClippingPath();
	VXfTreeNode* pParent = NULL;
	size_t nodeIndex = 0;

	MILLIPOINT columnRightMargin, prevColumnRightMargin, storyLeftMargin, storyRightMargin;
	bool bPrevColumnRightMargin = false;

	Paragraphs::iterator itPrevParagraph = textStory.end();
	for (Paragraphs::iterator itParagraph = textStory.begin(); itParagraph != textStory.end(); itPrevParagraph = itParagraph++)
	{
#if defined(_DEBUG)
		std::wstring text;
		TextLine::GetText(itParagraph->front(), text);
#endif

		MILLIPOINT nextLeftMargin = min(itParagraph->firstMargin, itParagraph->leftMargin);
		MILLIPOINT nextRightMargin = itParagraph->rightMargin;

		if (textStory.end() != itPrevParagraph)
		{
			MILLIPOINT prevLeftMargin = min(itPrevParagraph->firstMargin, itPrevParagraph->leftMargin);
			MILLIPOINT prevRightMargin = itPrevParagraph->rightMargin;
			
			if (bPrevColumnRightMargin && prevColumnRightMargin > nextLeftMargin)
			{
				bPrevColumnRightMargin = false;
				itParagraph->isStoryBreak = true;
			}
			else
			{
#ifdef _ENABLE_RELATIVE_LINE_SPACING
				double linespaceRatio = TextStory::GetLineSpaceRatio(itPrevParagraph->back(), itParagraph->front());
				if (linespaceRatio <= -0.015/*LINESPACE_RATIO_DELTA*/)
#else
				LONG linespacing = _round(TextStory::GetAbsoluteLineSpacing(itPrevParagraph->back(), itParagraph->front()));
				if (linespacing <= 0)
#endif
				{
					if (nextLeftMargin >= prevRightMargin)
					{
						bPrevColumnRightMargin = true;
						if (nextLeftMargin >= columnRightMargin)
							prevColumnRightMargin = columnRightMargin;
						else
						{
							prevColumnRightMargin = prevRightMargin;
							for (Paragraphs::reverse_iterator it(itParagraph); it != textStory.rend(); ++it)
							{
								if (it->rightMargin > nextLeftMargin)
								{
									if (it->front().baseLine > itParagraph->front().baseLine) // Denis, DDES-16255
										it.base()->isStoryBreak = true;
									break;
								}

								if (it->rightMargin > prevColumnRightMargin)
									prevColumnRightMargin = it->rightMargin;

								if (it->isColumnBreak)
									break;
							}
						}
					}

					itParagraph->isColumnBreak = itParagraph->isStoryBreak = true;
					columnRightMargin = nextRightMargin;
				}
#ifdef _ENABLE_RELATIVE_LINE_SPACING
				else if (linespaceRatio > 4.0)
#else
				else if (linespacing > 4 * min(itPrevParagraph->linespacing, itParagraph->linespacing))
#endif
					itParagraph->isStoryBreak = true;
				else if (itParagraph->pPrevInFlow)
					itParagraph->isStoryBreak = true;
			}

			MILLIPOINT overlapping = min(storyRightMargin, nextRightMargin) - max(storyLeftMargin, nextLeftMargin);

			// DDES-13468 (Poor text column reconstruction with 'Moresby...' PDF file)
			/*if (!itParagraph->isStoryBreak && (min(prevRightMargin, nextRightMargin) - max(prevLeftMargin, nextLeftMargin)) <= 0)//.5 * min(prevRightMargin - prevLeftMargin, nextRightMargin - nextLeftMargin))
				itParagraph->isStoryBreak = true;*/
			if (!itParagraph->isStoryBreak && overlapping <= 0)//.5 * min(storyRightMargin - storyLeftMargin, nextRightMargin - nextLeftMargin))
				itParagraph->isStoryBreak = true;

			MILLIPOINT prevFontSize = abs(itPrevParagraph->back().back().GetFontSize());
			MILLIPOINT nextFontSize = abs(itParagraph->front().front().GetFontSize());

			MILLIPOINT minFontSize = min(prevFontSize, nextFontSize);
			MILLIPOINT maxFontSize = max(prevFontSize, nextFontSize);

			// DDES-16158 (Text reconstruction failures)
			if (nextFontSize < prevFontSize && (min(prevRightMargin, nextRightMargin) - max(prevLeftMargin, nextLeftMargin)) < 0.6 * (prevRightMargin - prevLeftMargin))
				itParagraph->isStoryBreak = true;

			// DDES-16530: 2 -> 2.25
			if (maxFontSize > 2.25 * minFontSize)
				itParagraph->isStoryBreak = true;

			if (nextRightMargin > columnRightMargin)
				columnRightMargin = nextRightMargin;

			if (itParagraph->isStoryBreak)
			{
				storyRightMargin = nextRightMargin;
				storyLeftMargin = nextLeftMargin;
			}
			else
			{
				if (nextRightMargin > storyRightMargin)
					storyRightMargin = nextRightMargin;

				if (nextLeftMargin < storyLeftMargin)
					storyLeftMargin = nextLeftMargin;
			}
		}
		else
		{
			storyRightMargin = columnRightMargin = nextRightMargin;
			storyLeftMargin = nextLeftMargin;
		}
	}

#ifdef _ENABLE_TEXT_FLOW_RECONSTRUCTION
	MILLIPOINT leftMargin = 0, rightMargin = 0;
	Paragraphs::iterator itFirstParagraph = textStory.begin(), itLastParagraph = textStory.end();
	for (Paragraphs::iterator itParagraph = textStory.begin(); itParagraph != textStory.end(); itLastParagraph = itParagraph++)
	{
		MILLIPOINT paragraphLeftMargin = min(itParagraph->firstMargin, itParagraph->leftMargin);
		if (textStory.begin() == itParagraph || itParagraph->isStoryBreak)
		{
			if (textStory.end() != itLastParagraph)
			{
				if (itFirstParagraph->pPrevInFlow)
				{
					if (itFirstParagraph->leftMargin != leftMargin || itFirstParagraph->rightMargin != rightMargin)
					{
						if (rightMargin - leftMargin > itFirstParagraph->GetMaxWidth())
						{
							itFirstParagraph->pPrevInFlow->pNextInFlow = NULL;
							itFirstParagraph->pPrevInFlow = NULL;
						}
						else
						{
							MILLIPOINT dl = abs(itFirstParagraph->leftMargin - leftMargin);
							MILLIPOINT dr = abs(itFirstParagraph->rightMargin - rightMargin);
							UINT align = itFirstParagraph->GetAlign();

							for (TextLines::const_iterator itLine = itFirstParagraph->begin(); itLine != itFirstParagraph->end();)
							{
								TextLines::const_iterator itNextLine = itLine;
								++itNextLine;

								if (dl && 0 != (align & (ALIGN_FULL | ALIGN_LEFT | ALIGN_CENTER)) && dl > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
								{
									itFirstParagraph->pPrevInFlow->pNextInFlow = NULL;
									itFirstParagraph->pPrevInFlow = NULL;
									break;
								}

								if (dr && (0 != (align & (ALIGN_RIGHT | ALIGN_CENTER)) || (ALIGN_FULL == align && (itFirstParagraph->end() != itNextLine || itFirstParagraph->pNextInFlow))) && dr > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
								{
									itFirstParagraph->pPrevInFlow->pNextInFlow = NULL;
									itFirstParagraph->pPrevInFlow = NULL;
									break;
								}

								itLine = itNextLine;
							}

							/*if (itFirstParagraph->pPrevInFlow)
							{
								itFirstParagraph->leftMargin = leftMargin;
								itFirstParagraph->rightMargin = rightMargin;
							}*/
						}
					}
				}

				if (itLastParagraph->pNextInFlow)
				{
					if (itLastParagraph->leftMargin != leftMargin || itLastParagraph->rightMargin != rightMargin)
					{
						if (rightMargin - leftMargin > itLastParagraph->GetMaxWidth())
						{
							itLastParagraph->pNextInFlow->pPrevInFlow = NULL;
							itLastParagraph->pNextInFlow = NULL;
						}
						else
						{
							MILLIPOINT dl = abs(itLastParagraph->leftMargin - leftMargin);
							MILLIPOINT dr = abs(itLastParagraph->rightMargin - rightMargin);
							UINT align = itLastParagraph->GetAlign();

							for (TextLines::const_iterator itLine = itLastParagraph->begin(); itLine != itLastParagraph->end(); ++itLine)
							{
								if (dl && 0 != (align & (ALIGN_FULL | ALIGN_LEFT | ALIGN_CENTER)) && (itLastParagraph->begin() != itLine || itLastParagraph->pPrevInFlow) && dl > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
								{
									itLastParagraph->pNextInFlow->pPrevInFlow = NULL;
									itLastParagraph->pNextInFlow = NULL;
									break;
								}

								if (dr && 0 != (align & (ALIGN_FULL | ALIGN_RIGHT | ALIGN_CENTER)) && dr > (MAX_ABSOLUTE_OFFSET ? MAX_ABSOLUTE_OFFSET : itLine->GetMaxTrackingWidth(MAX_RELATIVE_OFFSET)))
								{
									itLastParagraph->pNextInFlow->pPrevInFlow = NULL;
									itLastParagraph->pNextInFlow = NULL;
									break;
								}
							}

							if (itLastParagraph->pNextInFlow)
							{
								itLastParagraph->leftMargin = leftMargin;
								itLastParagraph->rightMargin = rightMargin;
							}
						}
					}
				}
			}

			itFirstParagraph = itParagraph;
			leftMargin  = paragraphLeftMargin;
			rightMargin = itParagraph->rightMargin;
		}
		else
		{
			if (paragraphLeftMargin < leftMargin) leftMargin = paragraphLeftMargin;
			if (itParagraph->rightMargin > rightMargin) rightMargin = itParagraph->rightMargin;
		}
	}
#endif // _ENABLE_TEXT_FLOW_RECONSTRUCTION

	Paragraphs::iterator itStoryBegin = textStory.begin();
	Paragraphs::iterator itStoryEnd   = textStory.begin();

	while (textStory.end() != itStoryEnd)
	{
		if (pParent)
		{
			Matrix nodeMat(textStory.matrix.a(), textStory.matrix.b(), textStory.matrix.c(), textStory.matrix.d(),
			               _round(textStory.matrix.e()), _round(textStory.matrix.f()));

			textStory.nodeStory = new CXfTextStoryComplex(nodeMat, g_settings.bReconstructKerningPairs ? 1 : 0);
			pParent->InsertBefore(++nodeIndex, textStory.nodeStory);
			textStory.nodeStory->SetIsClippingPath(bIsClippingPath);

			textStory.attrs = TextAttrsCollector();
		}

		itStoryBegin = itStoryEnd;

		do
		{
			++itStoryEnd;
		}
		while (textStory.end() != itStoryEnd && !itStoryEnd->isStoryBreak);

		bool bLink = textStory.end() != itStoryEnd && NULL != itStoryEnd->pPrevInFlow;

		textStory.nodeStory->AddChild(new CXfTextStoryLinkInfo(0, bLink ? m_textAreaID + 1 : -1, m_textAreaID), false);
		++m_textAreaID;

		itStoryBegin = textStory.Finalize(itStoryBegin, itStoryEnd);

		for (Paragraphs::iterator itParagraph = itStoryBegin; itParagraph != itStoryEnd; ++itParagraph)
		{
			for (TextLines::iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
			{
				//itLine->RegisterAttributes();

				for (TextChars::const_iterator itChar = itLine->begin(); itChar != itLine->end(); ++itChar)
					itLine->attrs.tracking.Register(itChar->tracking);

				#define REGISTER_IF_DEFINED(a) if (itLine->attrs.a.IsDefined()) textStory.attrs.a.Register(itLine->attrs.a());

				REGISTER_IF_DEFINED(aspectRatio);
				REGISTER_IF_DEFINED(tracking);
				//REGISTER_IF_DEFINED(baseLineShift);
				REGISTER_IF_DEFINED(pFontDef);
				REGISTER_IF_DEFINED(fontSize);
				REGISTER_IF_DEFINED(isBold);
				REGISTER_IF_DEFINED(isItalic);
				REGISTER_IF_DEFINED(isStroked);
				REGISTER_IF_DEFINED(isFilled);
				REGISTER_IF_DEFINED(lineJoin);
				REGISTER_IF_DEFINED(lineCap);
				REGISTER_IF_DEFINED(miterLimit);
				REGISTER_IF_DEFINED(lineWidth);
				REGISTER_IF_DEFINED(colorStroke);
				REGISTER_IF_DEFINED(opacityStroke);
				REGISTER_IF_DEFINED(dash);
				REGISTER_IF_DEFINED(colorFill);
				REGISTER_IF_DEFINED(opacityFill);
			}
		}


#ifdef _ENABLE_TEXT_AREA_MODE
		bool bHeightInfo = true;
#else
		bool bHeightInfo = bLink;
#endif

		size_t lineCount = 0;
		for (Paragraphs::const_iterator itParagraph = itStoryBegin; lineCount < 2 && itParagraph != itStoryEnd; ++itParagraph)
			lineCount += itParagraph->size();

		// Setting text story width only if it has multiple lines
		bool bWordWrapInfo = bHeightInfo || lineCount > 1;

		double x0Abs, y0Abs;
		if (bHeightInfo)
#ifdef _ENABLE_RELATIVE_LINE_SPACING
			textStory.matrix.map(textStory.leftMargin, itStoryBegin->front().baseLine + itStoryBegin->front().ascent, x0Abs, y0Abs);
#else
			textStory.matrix.map(textStory.leftMargin, itStoryBegin->front().baseLine + itStoryBegin->linespacing, x0Abs, y0Abs);
#endif
		else
			textStory.matrix.map(textStory.leftMargin, itStoryBegin->front().baseLine, x0Abs, y0Abs);
		Matrix matrixStory = textStory.nodeStory->Transform;
		matrixStory.SetTranslation(_round(x0Abs), _round(y0Abs));
		textStory.nodeStory->Transform = matrixStory;

		AddDefaultAttributeTagsForStory(textStory.attrs, textStory.nodeStory);

		if (bWordWrapInfo)
		{
			textStory.nodeStory->AddChild(new CXfTextStoryWordWrapInfo(textStory.rightMargin - textStory.leftMargin), false);

			if (bHeightInfo)
			{
				const Paragraph *pFirstInFlow = &(*itStoryBegin);
				while (pFirstInFlow->pPrevInFlow)
					pFirstInFlow = pFirstInFlow->pPrevInFlow;

				const TextLine &firstLine = itStoryBegin->front();

#ifdef _ENABLE_RELATIVE_LINE_SPACING
				MILLIPOINT pos = pFirstInFlow->linespaceRatio.MakeDouble() < 1.0 ? (MILLIPOINT)((1.0 - pFirstInFlow->linespaceRatio.MakeDouble()) * firstLine.ascent + 0.5): 0;
#else
				MILLIPOINT pos = (MILLIPOINT)(-pFirstInFlow->linespacing * firstLine.GetAscentDescentRatio() + 0.5);
#endif
				MILLIPOINT height = pos;

				for (Paragraphs::const_iterator itParagraph = itStoryBegin; ;)
				{
					if (!itParagraph->pPrevInFlow)
						pFirstInFlow = &(*itParagraph);
#ifdef _ENABLE_RELATIVE_LINE_SPACING
					for (Paragraph::const_iterator itLine = itParagraph->begin(); itLine != itParagraph->end(); ++itLine)
						pos += (MILLIPOINT)(pFirstInFlow->linespaceRatio.MakeDouble() * (itLine->ascent - itLine->descent) + 0.5);
#else
					pos += (MILLIPOINT)(pFirstInFlow->linespacing * itParagraph->size());
#endif
					if (pos > height)
						height = pos;

					Paragraphs::const_iterator itNextParagraph = itParagraph;
					++itNextParagraph;

					if (itStoryEnd == itNextParagraph) break;

					pos += max(itParagraph->spaceAfter, itNextParagraph->spaceBefore);
					if (pos > height)
						height = pos;

					itParagraph = itNextParagraph;
				}

				textStory.nodeStory->AddChild(new CXfHeightInfo(height), false);
			}
		}

		bool isFirstStoryLine = true;
		std::list<TextLine *> listItemStack;
		CXfTextLine *nodeLine = nullptr;
		CXfTextAnchorPosition *anchorPos = nullptr;

		for (Paragraphs::iterator itParagraph = itStoryBegin; itParagraph != itStoryEnd;)
		{
			Paragraphs::iterator itNextParagraph = itParagraph;
			++itNextParagraph;

			std::list<MILLIPOINT> tabStops;
			int numOfAdditionalEOL = 0;
			MILLIPOINT spaceAfter = 0;

			for (TextLines::iterator itLine = itParagraph->begin(); itLine != itParagraph->end();)
			{
				TextLines::iterator itNextLine = itLine;
				++itNextLine;

				const TextChar *pListTextFirstChar = nullptr;

				nodeLine = new CXfTextLine();
				nodeLine->SetBounds(itLine->bbox);
				AddOptimizedAttributeLineTags(textStory.attrs, itLine->attrs, nodeLine);
				textStory.nodeStory->AddChild(nodeLine, false);

				if (!itParagraph->pPrevInFlow && itParagraph->begin() == itLine)
				{
					std::set<MILLIPOINT> tabStopSet;
					for (TextLines::iterator itL = itParagraph->begin(); itL != itParagraph->end(); ++itL)
					{
						MILLIPOINT xPos = _round(itL->front().x);

						for (auto itChar = itL->begin(); itChar != itL->end();)
						{
							auto itNextChar = itChar;
							++itNextChar;

							if (itChar->isTabStop)
								tabStopSet.insert(xPos);

							xPos += itChar->GetAdvance(itL->end() == itNextChar ? nullptr : &(*itNextChar)) + itChar->GetKernWidth();

							itChar = itNextChar;
						}
					}

					if (!tabStopSet.empty())
					{
						for (auto it = tabStopSet.begin(); it != tabStopSet.end(); ++it)
							tabStops.push_back(*it);
						tabStops.sort();

						CXfTextRuler *nodeRuler = new CXfTextRuler();
						nodeLine->AddChild(nodeRuler, false);

						for (auto it = tabStops.begin(); it != tabStops.end(); ++it)
							nodeRuler->Stops.push_back(CXfTextRuler::CTabStop(0, (*it) - textStory.leftMargin));
					}

					/*CXfTextRuler *nodeRuler = NULL;
					MILLIPOINT xPos = _round(itLine->front().x);

					for (auto itChar = itLine->begin(); itChar != itLine->end();)
					{
						auto itNextChar = itChar;
						++itNextChar;

						if (itChar->isTabStop)
						{
							if (!nodeRuler)
							{
								nodeRuler = new CXfTextRuler();
								nodeLine->AddChild(nodeRuler, false);
							}
							nodeRuler->Stops.push_back(CXfTextRuler::CTabStop(0, xPos - textStory.leftMargin));
						}

						xPos += itChar->GetAdvance(itLine->end() == itNextChar ? nullptr : &(*itNextChar)) + itChar->GetKernWidth();
						itChar = itNextChar;
					}*/

#ifdef _ENABLE_RELATIVE_LINE_SPACING
					nodeLine->AddChild(new CXfLinespaceRatio(itParagraph->linespaceRatio), false);
#else
					nodeLine->AddChild(new CXfLinespaceAbsolute(itParagraph->linespacing), false);
#endif

					if (itParagraph->spaceBefore && itStoryBegin != itParagraph)
						nodeLine->AddChild(new CXfTextSpaceBefore(itParagraph->spaceBefore), false);

					if (itStoryEnd != itNextParagraph || itParagraph->pNextInFlow)
					{
						const Paragraph *pLastInFlow = &(*itParagraph);
						while (pLastInFlow->pNextInFlow)
							pLastInFlow = pLastInFlow->pNextInFlow;

						spaceAfter = pLastInFlow->spaceAfter;

						if (spaceAfter > 0 && !itParagraph->pNextInFlow)
						{

#ifdef _ENABLE_RELATIVE_LINE_SPACING
							const TextChar &ch = itParagraph->back().back();
							MILLIPOINT linespacing = (MILLIPOINT)_round(itParagraph->linespaceRatio.MakeDouble() * (ch.GetFontAscent() - ch.GetFontDescent()));
#else

							MILLIPOINT linespacing = itParagraph->linespacing;
#endif
							if (!itParagraph->front().front().isAutoListChar)
							{
								while (spaceAfter > 0.90 * linespacing)
								{
									spaceAfter -= linespacing;
									++numOfAdditionalEOL;
								}
							}

							if (spaceAfter < 0 && itStoryEnd != itNextParagraph)
								itNextParagraph->spaceBefore = spaceAfter;									
						}

						if (spaceAfter && !numOfAdditionalEOL)
							nodeLine->AddChild(new CXfTextSpaceAfter(spaceAfter), false);
					}

					MILLIPOINT firstIndent = itParagraph->firstMargin - textStory.leftMargin;
					MILLIPOINT leftIndent  = itParagraph->leftMargin - textStory.leftMargin;
					MILLIPOINT rightIndent = textStory.rightMargin - itParagraph->rightMargin;

					if (firstIndent)
						nodeLine->AddChild(new CXfTextFirstIndent(firstIndent), false);
					if (leftIndent)
						nodeLine->AddChild(new CXfTextLeftIndent(leftIndent), false);
					if (rightIndent)
						nodeLine->AddChild(new CXfTextRightIndent(rightIndent), false);

					switch (itParagraph->GetAlign())
					{
					case ALIGN_LEFT:
						nodeLine->AddChild(new CXfZeroSizedNode(TAG_TEXT_JUSTIFICATION_LEFT), false);
						break;
					case ALIGN_FULL:
						nodeLine->AddChild(new CXfZeroSizedNode(TAG_TEXT_JUSTIFICATION_FULL), false);
						break;
					case ALIGN_CENTER:
						nodeLine->AddChild(new CXfZeroSizedNode(TAG_TEXT_JUSTIFICATION_CENTRE), false);
						break;
					case ALIGN_RIGHT:
						nodeLine->AddChild(new CXfZeroSizedNode(TAG_TEXT_JUSTIFICATION_RIGHT), false);
						break;
					}

					if (itLine->front().isAutoListChar)
					{
#if defined(_DEBUG)
						std::wstring text;
						TextLine::GetText(*itLine, text);
#endif

						VXfTextRenderable *pItemStart = NULL;

						bool isNumberedListItem = !itLine->numberingFormat.empty();

						if (isNumberedListItem)
						{
							if (!listItemStack.empty() && listItemStack.back()->numberingFormat.empty()) listItemStack.clear();

							for (std::list<TextLine *>::const_reverse_iterator it = listItemStack.rbegin(); it != listItemStack.rend(); ++it)
							{
								if ((*it)->numberingFormat == itLine->numberingFormat && (*it)->number + 1 == itLine->number ||
								    (*it)->numberingFormat == itLine->numberingFormatAlt && (*it)->number + 1 == itLine->numberAlt)
								{
									listItemStack.erase(it.base(), listItemStack.end());
									break;
								}
							}

							//while (!listItemStack.empty() && !(listItemStack.back()->numberingFormat == itLine->numberingFormat && listItemStack.back()->number + 1 == itLine->number ||
							//	listItemStack.back()->numberingFormat == itLine->numberingFormatAlt && listItemStack.back()->number + 1 == itLine->numberAlt) && (itLine->level < listItemStack.back()->level || itLine->level == listItemStack.back()->level && itLine->leftMargin < listItemStack.back()->leftMargin)) listItemStack.pop_back();

							if (listItemStack.empty() || !(listItemStack.back()->numberingFormat == itLine->numberingFormat && listItemStack.back()->number + 1 == itLine->number ||
								listItemStack.back()->numberingFormat == itLine->numberingFormatAlt && listItemStack.back()->number + 1 == itLine->numberAlt))
							{
								itLine->startNumber = itLine->number;
								listItemStack.push_back(&(*itLine));
							}
							else
							{
								if (listItemStack.back()->numberingFormat == itLine->numberingFormatAlt)
								{
									itLine->numberingFormat = itLine->numberingFormatAlt;
									itLine->number = itLine->numberAlt;
								}

								itLine->startNumber = listItemStack.back()->startNumber;
								listItemStack.back() = &(*itLine);
							}

							pItemStart = new CXfTextNumberedItemStart(listItemStack.back()->numberingFormat, listItemStack.back()->number, listItemStack.back()->startNumber, (UINT16)listItemStack.size() - 1, itLine->front().pFontDef, itLine->front().attrs.isBold ? TRUE : FALSE, itLine->front().attrs.isItalic ? TRUE : FALSE);
						}
						else
						{
							if (!listItemStack.empty() && !listItemStack.back()->numberingFormat.empty()) listItemStack.clear();

							for (std::list<TextLine *>::const_reverse_iterator it = listItemStack.rbegin(); it != listItemStack.rend(); ++it)
							{
								if ((*it)->front().c == itLine->front().c)
								{
									listItemStack.erase(it.base(), listItemStack.end());
									break;
								}
							}

							//while (!listItemStack.empty() && listItemStack.back()->front().c != itLine->front().c && itLine->leftMargin < listItemStack.back()->leftMargin) listItemStack.pop_back();

							if (listItemStack.empty() || listItemStack.back()->front().c != itLine->front().c)
							{
								listItemStack.push_back(&(*itLine));
							}
							else
							{
								listItemStack.back() = &(*itLine);
							}

							pItemStart = new CXfTextBulletedItemStart(itLine->front().c, (UINT16)listItemStack.size() - 1, itLine->front().pFontDef, itLine->front().attrs.isBold ? TRUE : FALSE, itLine->front().attrs.isItalic ? TRUE : FALSE);
						}

						TextChars::iterator itListTextFirstChar = itLine->begin();
						while (itLine->end() != itListTextFirstChar && itListTextFirstChar->isAutoListChar)
							++itListTextFirstChar;

						if (itLine->end() != itListTextFirstChar)
							pListTextFirstChar = &(*itListTextFirstChar);

						AddOptimizedAttributeStringTags(itLine->front(), itLine->attrs, pItemStart, nullptr);

						itLine->front().ApplyScriptAttributes(pItemStart);
						pItemStart->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);
						nodeLine->AddChild(pItemStart, false);
					}
					else listItemStack.clear();

					// DDES-16530 (in PDF import use paragraph borders rather than anchored lines)
					if (itNextParagraph != itStoryEnd)
					{
						MILLIPOINT paragraphLeftMargin = min(itParagraph->firstMargin, itParagraph->leftMargin);
						MILLIPOINT paragraphRightMargin = itParagraph->rightMargin;

						MILLIPOINT nextParagraphLeftMargin = min(itNextParagraph->firstMargin, itNextParagraph->leftMargin);
						MILLIPOINT nextParagraphRightMargin = itNextParagraph->rightMargin;

						if (paragraphLeftMargin == nextParagraphLeftMargin && paragraphRightMargin == nextParagraphRightMargin)
						{
							for (QueueUnderlineCheck::iterator itGraphics = queueUnderline.begin(); itGraphics != queueUnderline.end(); ++itGraphics)
							{
								if (0 != itGraphics->opacity.value || CXF_TRANSPARANCY_TYPE_MIX != itGraphics->opacity.mode)
									continue;

								Rect rect;
								if (!itGraphics->TransformToRect(textStory.matrixInv, rect) || 0 == rect.width)
									continue;

								if (abs(rect.x - paragraphLeftMargin) / (double)rect.width <= 0.01 && abs(rect.x + rect.width - paragraphRightMargin) / (double)rect.width <= 0.01 &&
								   (rect.y + rect.height) < (itParagraph->back().baseLine + itParagraph->back().descent) &&
								    rect.y > (itNextParagraph->front().baseLine + itNextParagraph->front().ascent))
								{
									nodeLine->AppendChild(new CXfTxtParaBorder(TAG_TXT_PARA_BOTTOM_BORDER, CreateComplexColour(itGraphics->color), rect.height, 0, 0));
									RemoveTextMarkupPath(*itGraphics);
									break;
								}
							}
						}
					}
				}

				auto itNextTab = tabStops.begin();

				if (anchorPos)
				{
					nodeLine->AddChild(anchorPos, false);
					anchorPos = nullptr;
				}

				MILLIPOINT xPos = itLine->leftMargin;

				if (!itLine->empty() && itLine->firstKern)
				{
					TextChar &ch = itLine->front();

					CXfKerning *nodeKerning = new CXfKerning(DocCoord(itLine->firstKern, 0));
					nodeKerning->AddChild(new CXfZeroSizedNode(TAG_TEXT_FONT_SIZE_RELATIVE_KERN), false);
					AddOptimizedAttributeStringTags(ch, itLine->attrs, nodeKerning, pListTextFirstChar);

					ch.ApplyScriptAttributes(nodeKerning);

					if (ch.isAutoListChar)
						nodeKerning->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);
						
					double bboxAbsKerning[8];
					double y0 = ch.y + ch.GetFontDescent();
					double y1 = ch.y + ch.GetFontAscent();
					textStory.matrix.map(xPos, y0, bboxAbsKerning[0], bboxAbsKerning[1]);
					textStory.matrix.map(xPos, y1, bboxAbsKerning[4], bboxAbsKerning[5]);
					xPos += ch.GetKernWidth(itLine->firstKern);
					textStory.matrix.map(xPos, y1, bboxAbsKerning[2], bboxAbsKerning[3]);
					textStory.matrix.map(xPos, y0, bboxAbsKerning[6], bboxAbsKerning[7]);

					Rect bboxAbsKerningBounded = GetBounds(bboxAbsKerning, 4);

					nodeKerning->SetBounds(bboxAbsKerningBounded);

					#if defined(_KERNING_DRAW)
					_Debug_BoundingPointsBBox(m_pBBoxLayer, bboxAbsKerningBounded, .8);
					#endif

					nodeLine->AddChild(nodeKerning, false);
					AddToAnnotationContext(nodeKerning);
				}

				//CXfTextRuler *nodeRuler = NULL;
				CXfTextString *nodeString = NULL;
				TextChars::iterator itPrevChar = itLine->end();
				//MILLIPOINT xPos = _round(itLine->front().x);
				for (TextChars::iterator itChar = itLine->begin(); itChar != itLine->end();)
				{
					TextChars::iterator itNextChar = itChar;
					++itNextChar;

					#ifdef _DRAW_PARAGRAPH_ALIGNMENT_DETECTION
					BYTE ccR = 0x80, ccG = 0x80, ccB = 0x80;
					switch (itParagraph->GetAlign())
					{
					case ALIGN_FULL:
						ccR = 0x60; ccG = 0xB0;	ccB = 0x60;
						break;
					case ALIGN_RIGHT:
						ccR = 0xB0; ccG = 0x60; ccB = 0x60;
						break;
					case ALIGN_CENTER:
						ccR = 0x50; ccG = 0xA0; ccB = 0xA0;
						break;
					}

					Color cc;
					cc.colorSpaceMode = csDeviceRGB;
					cc.crRGB = RGB(ccR, ccG, ccB);
					cc.rgb.r = byteToCol(ccR);
					cc.rgb.g = byteToCol(ccG);
					cc.rgb.b = byteToCol(ccB);
					itChar->attrs.colorFill = cc;
					#endif

					CXfKerning       *nodeKerning = NULL;
					CXfZeroSizedNode *nodeTab = NULL;
					//if (itChar != itLine->begin()/* && itPrevChar->kern*/)
					if (itLine->end() != itPrevChar)
					{
						if (itChar->isTabStop && itPrevChar->kern > 0)
						{
							while (itNextTab != tabStops.end() && (*itNextTab) < xPos)
								++itNextTab;

							xPos += itPrevChar->GetKernWidth();

							while (itNextTab != tabStops.end() && (*itNextTab) < xPos)
							{
								nodeTab = new CXfZeroSizedNode(TAG_TEXT_TAB);
								AddOptimizedAttributeStringTags(*itPrevChar, itLine->attrs, nodeTab, pListTextFirstChar);
								itPrevChar->ApplyScriptAttributes(nodeTab);
								if (itPrevChar->isAutoListChar)
									nodeTab->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);

								if (nodeString)
								{
									AddToAnnotationContext(nodeString);
									nodeLine->AddChild(nodeString, false);
									nodeString = NULL;
								}

								nodeLine->AddChild(nodeTab, false);

								++itNextTab;
							}

							/*if (!nodeRuler)
							{
								nodeRuler = new CXfTextRuler();
								nodeLine->AddChild(nodeRuler, false);
							}
							nodeRuler->Stops.push_back(CXfTextRuler::CTabStop(0, xPos - textStory.leftMargin));*/

							nodeTab = new CXfZeroSizedNode(TAG_TEXT_TAB);
							AddOptimizedAttributeStringTags(*itPrevChar, itLine->attrs, nodeTab, pListTextFirstChar);
							itPrevChar->ApplyScriptAttributes(nodeTab);
							if (itPrevChar->isAutoListChar)
								nodeTab->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);

						}
						else if (itPrevChar->kern)
						{						
							nodeKerning = new CXfKerning(DocCoord(itPrevChar->kern, 0));
							nodeKerning->AddChild(new CXfZeroSizedNode(TAG_TEXT_FONT_SIZE_RELATIVE_KERN), false);
							AddOptimizedAttributeStringTags(*itPrevChar, itLine->attrs, nodeKerning, pListTextFirstChar);
							itPrevChar->ApplyScriptAttributes(nodeKerning);

							if (itPrevChar->isAutoListChar)
								nodeKerning->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);
						
							double bboxAbsKerning[8];
							double y0 = itPrevChar->y + itPrevChar->GetFontDescent();
							double y1 = itPrevChar->y + itPrevChar->GetFontAscent();

							textStory.matrix.map(xPos, y0, bboxAbsKerning[0], bboxAbsKerning[1]);
							textStory.matrix.map(xPos, y1, bboxAbsKerning[4], bboxAbsKerning[5]);

							xPos += itPrevChar->GetKernWidth();

							textStory.matrix.map(xPos, y1, bboxAbsKerning[2], bboxAbsKerning[3]);
							textStory.matrix.map(xPos, y0, bboxAbsKerning[6], bboxAbsKerning[7]);

							Rect bboxAbsKerningBounded = GetBounds(bboxAbsKerning, 4);

							nodeKerning->SetBounds(bboxAbsKerningBounded);

							#if defined(_KERNING_DRAW)
							_Debug_BoundingPointsBBox(m_pBBoxLayer, bboxAbsKerningBounded, .8);
							#endif
						}
					}

					bool bSoftHyphen = itChar->IsHyphen() && itLine->end() == itNextChar && (bLink || itParagraph->end() != itNextLine);

					if (nodeString && (itChar->isAnchorPos || bSoftHyphen || nodeKerning || nodeTab || itChar->attrs != itPrevChar->attrs ||
						itChar->pFontDef != itPrevChar->pFontDef ||
						itChar->ligType != itPrevChar->ligType ||
						itChar->tracking != itPrevChar->tracking ||
						itChar->baseLineShift != itPrevChar->baseLineShift ||
						itChar->isUnderline != itPrevChar->isUnderline ||
						itChar->isStrikethrough != itPrevChar->isStrikethrough ||
						/*itChar->isAutoListChar != */itPrevChar->isAutoListChar ||
						itChar->scriptOffset != itPrevChar->scriptOffset ||
						itChar->scriptScale != itPrevChar->scriptScale))
					{
						AddToAnnotationContext(nodeString);
						nodeLine->AddChild(nodeString, false);
						nodeString = NULL;
					}

					if (nodeKerning)
					{
						nodeLine->AddChild(nodeKerning, false);
						AddToAnnotationContext(nodeKerning);
					}

					if (nodeTab)
					{
						nodeLine->AddChild(nodeTab, false);
						//AddToAnnotationContext(nodeTab);
					}

					if (nodeString)
					{
						if (itChar->ligType)
						{
							for (size_t i = 0; i < itChar->ligChars.size(); ++i)
								nodeString->m_str.push_back(itChar->ligChars[i]);
						}
						else
							nodeString->m_str.push_back(itChar->c);

						Rect::Union(nodeString->Bounds(), nodeString->Bounds(), itChar->bbox);
					}
					else if (itChar->isAnchorPos)
					{
						CXfTextAnchorPosition *anchorPos = new CXfTextAnchorPosition(itChar->c);

						CXfTextEmbeddedAnchorPosInfo *embeddedAnchorPos = new CXfTextEmbeddedAnchorPosInfo(itChar->verticalOffset, 0, 0, 0, 0,
							itChar->width/* + itChar->GetTrackingWidth()*/, // DDES-14835/DDES-14840 fix
							itChar->ascent, itChar->descent, FALSE);

						anchorPos->AddChild(embeddedAnchorPos, false);

						AddOptimizedAttributeStringTags(*itChar, itLine->attrs, anchorPos, nullptr);
						itChar->ApplyScriptAttributes(anchorPos);

						nodeLine->AddChild(anchorPos, false);
						AddToAnnotationContext(anchorPos);
					}
					else if (bSoftHyphen)
					{
						CXfSpecialHyphen *softHyphen = new CXfSpecialHyphen(1);

						static_cast<VXfBounded *>(softHyphen)->SetBounds(itChar->bbox);

						AddOptimizedAttributeStringTags(*itChar, itLine->attrs, softHyphen, nullptr);
						itChar->ApplyScriptAttributes(softHyphen);

						nodeLine->AddChild(softHyphen, false);
						AddToAnnotationContext(softHyphen);
					}
					else
					{												
						nodeString = new CXfTextString();

						if (itChar->ligType)
						{
							for (size_t i = 0; i < itChar->ligChars.size(); ++i)
								nodeString->m_str.push_back(itChar->ligChars[i]);
						}
						else
							nodeString->m_str.push_back(itChar->c);

						static_cast<VXfBounded *>(nodeString)->SetBounds(itChar->bbox);

						AddOptimizedAttributeStringTags(*itChar, itLine->attrs, nodeString, pListTextFirstChar);
						itChar->ApplyScriptAttributes(nodeString);

						if (itChar->isAutoListChar)
							nodeString->AddChild(new CXfZeroSizedNode(TAG_TEXT_AUTO_LIST_CHAR), false);

						if (itChar->ligType)
							nodeString->AddChild(new CXfTextOpenTypeLigatures(itChar->ligType), false);
					}

					xPos += itChar->GetAdvance(itLine->end() == itNextChar ? nullptr : &(*itNextChar));

					#if defined(_ELEMENT_DRAW)
						#if defined(_LOGICAL_BOUNDS)
						_Debug_BoundingPointsBBox(m_pBBoxLayer, TransformBBox(itChar->GetBBox(false), textStory.matrix), itChar->IsSpace() ? .3 : 0.);
						#else
						_Debug_BoundingPointsBBox(m_pBBoxLayer, itChar->bbox, itChar->IsSpace() ? .3 : 0.);
						#endif
					#endif
					#if defined(_FLOW_DRAW)
						double xStart, xEnd, yStart, yEnd;
						textStory.matrix.map(itChar->x, itChar->y, xStart, yStart);
						textStory.matrix.map(itChar->x + itChar->GetWidth(), itChar->y, xEnd, yEnd);
						_Debug_FlowLine(m_pBBoxLayer, xStart, yStart, xEnd, yEnd, (isFirstStoryLine && itChar == itLine->begin()) );
					#endif

					itPrevChar = itChar;
					itChar = itNextChar;
				}

				if (nodeString)
				{
					AddToAnnotationContext(nodeString);
					nodeLine->AddChild(nodeString, false);
				}

				if ((!bLink || itStoryEnd != itNextParagraph) && itParagraph->end() == itNextLine)
				{
					VXfTreeNode *nodeEOL = new CXfZeroSizedNode(TAG_TEXT_EOL);
					AddOptimizedAttributeStringTags(itLine->back(), itLine->attrs, nodeEOL, nullptr);
					nodeLine->AddChild(nodeEOL, false);

					if (itParagraph->end() == itNextLine)
					{
						for (int i = 0; i < numOfAdditionalEOL; ++i)
						{
							nodeLine = new CXfTextLine();
							AddOptimizedAttributeLineTags(textStory.attrs, itLine->attrs, nodeLine);

							if (spaceAfter && i == numOfAdditionalEOL - 1)
								nodeLine->AddChild(new CXfTextSpaceAfter(spaceAfter), false);
#ifdef _ENABLE_RELATIVE_LINE_SPACING
							nodeLine->AddChild(new CXfLinespaceRatio(itParagraph->linespaceRatio), false);
#else
							nodeLine->AddChild(new CXfLinespaceAbsolute(itParagraph->linespacing), false);
#endif
							VXfTreeNode *nodeEOL = new CXfZeroSizedNode(TAG_TEXT_EOL);
							AddOptimizedAttributeStringTags(itLine->back(), itLine->attrs, nodeEOL, nullptr);
							nodeLine->AddChild(nodeEOL, false);

							textStory.nodeStory->AddChild(nodeLine, false);
						}
					}					

					#ifdef _EOL_DRAW
					double boundingPoints[4];
					textStory.matrix.map(textStory.nodeStory->Bounds().left(), itLine->baseLine, boundingPoints[0], boundingPoints[1]);
					textStory.matrix.map(textStory.nodeStory->Bounds().right(), itLine->baseLine, boundingPoints[2], boundingPoints[3]);

					CXfPath* debugPath = new CXfPath(TAG_PATH_FILLED_STROKED);
					debugPath->Verbs.push_back(PT_MOVETO);
					debugPath->Verbs.push_back(PT_LINETO | PT_CLOSEFIGURE);
					debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[0], (LONG)boundingPoints[1]));
					debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[2], (LONG)boundingPoints[3]));

					if (m_pBBoxLayer)
						m_pBBoxLayer->AddChild(debugPath);

					COLORREF crRGB = RGB(0xFF,0x00,0xFF);
					CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, 1.,0.,1., 0);
					debugPath->InsertFirstChild(new CXfLineColor(color));
					debugPath->InsertFirstChild(color);
					#endif
				}

				itLine = itNextLine;
			}

#ifdef _ENABLE_EMBEDDED_GRAPHICS

			static const double DELTA = 1e-4;

			if (itNextParagraph != itStoryEnd && nodeLine &&
			    fabs(1.0 - textStory.matrix.a()) < DELTA &&
			    fabs(textStory.matrix.b()) < DELTA &&
			    fabs(textStory.matrix.c()) < DELTA &&
			    fabs(1.0 - textStory.matrix.d()) < DELTA)
			{

				MILLIPOINT paragraphLeftMargin = (1 == itParagraph->size()) ? itParagraph->front().leftMargin : min(itParagraph->firstMargin, itParagraph->leftMargin);
				MILLIPOINT paragraphRightMargin = (1 == itParagraph->size()) ? itParagraph->front().rightMargin : itParagraph->rightMargin;

				MILLIPOINT nextParagraphLeftMargin = (1 == itNextParagraph->size()) ? itNextParagraph->front().leftMargin : min(itNextParagraph->firstMargin, itNextParagraph->leftMargin);
				MILLIPOINT nextParagraphRightMargin = (1 == itNextParagraph->size()) ? itNextParagraph->front().rightMargin : itNextParagraph->rightMargin;

				CXfGroup *pGroup = nullptr;
				Rect groupBBox;

				MILLIPOINT textWidth = textStory.rightMargin - textStory.leftMargin;

				pParent = textStory.nodeStory->GetParent();

				size_t childrenCount = pParent->ChildrenCount();
				for (size_t i = 0; i < childrenCount;)
				{
					VXfTreeNode *pChild = pParent->GetChild(i);

					if (pChild == textStory.nodeStory)
					{
						nodeIndex = i;
						break;
					}

					if (!pChild->IsPath() || pChild->IsRemoved())
					{
						++i;
						continue;
					}

					Rect bounds = ((VXfBounded *)pChild)->Bounds();

					if (bounds.left() > (textStory.leftMargin - 0.02 * textWidth) && bounds.right() < (textStory.rightMargin + 0.02 * textWidth) &&
					    //bounds.width > textWidth / 2 && // DDES-14850 fix
					   (((min(bounds.right(), paragraphRightMargin) - max(bounds.left(), paragraphLeftMargin)) >= .5 * (paragraphRightMargin - paragraphLeftMargin)) ||
					   ((min(bounds.right(), nextParagraphRightMargin) - max(bounds.left(), nextParagraphLeftMargin)) >= .5 * (nextParagraphRightMargin - nextParagraphLeftMargin))) &&
					   (bounds.y + bounds.height) < (itParagraph->back().baseLine + itParagraph->back().descent) &&
					   bounds.y > (itNextParagraph->front().baseLine + itNextParagraph->front().ascent))
					{
						pParent->RemoveChild(i, false);

						if (!pGroup)
						{
							pGroup = new CXfGroup();
							pParent->InsertBefore(i++, pGroup);
							groupBBox = bounds;
						}
						else
							Rect::Union(groupBBox, groupBBox, bounds);

						pGroup->AddChild(pChild);
					}
					else
						++i;
				}

				if (pGroup)
				{
					//pGroup->InsertFirstChild(new CXfEmbeddedTransformedAnchoredGroupInfo(TRUE, 1.0, 0.0, 0.0, 1.0));
					pGroup->InsertFirstChild(new CXfAnchoringMode(AnchorH_Column, AnchorV_ParagraphStart));
					pGroup->InsertFirstChild(new CXfAnchoredGroupInfo(m_anchodedGroupTag, DocCoord(itNextParagraph->front().leftMargin, itNextParagraph->front().baseLine)));

					anchorPos = new CXfTextAnchorPosition(m_anchodedGroupTag);

					m_anchodedGroupTag++;
				}
			}
#endif

			itParagraph = itNextParagraph;
		}

		#if defined(_BBOX_DRAW)
		double boundingPoints[8];
		textStory.matrix.map(textStory.nodeStory->Bounds().left(), textStory.nodeStory->Bounds()._bottom(), boundingPoints[0], boundingPoints[1]);
		textStory.matrix.map(textStory.nodeStory->Bounds().right(), textStory.nodeStory->Bounds()._top(), boundingPoints[2], boundingPoints[3]);
		textStory.matrix.map(textStory.nodeStory->Bounds().left(), textStory.nodeStory->Bounds()._top(), boundingPoints[4], boundingPoints[5]);
		textStory.matrix.map(textStory.nodeStory->Bounds().right(), textStory.nodeStory->Bounds()._bottom(), boundingPoints[6], boundingPoints[7]);

		CXfPath* debugPath = new CXfPath(TAG_PATH_FILLED_STROKED);
		debugPath->Verbs.push_back(PT_MOVETO);
		debugPath->Verbs.push_back(PT_LINETO);
		debugPath->Verbs.push_back(PT_LINETO);
		debugPath->Verbs.push_back(PT_LINETO | PT_CLOSEFIGURE);
		debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[0], (LONG)boundingPoints[1]));
		debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[6], (LONG)boundingPoints[7]));
		debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[2], (LONG)boundingPoints[3]));
		debugPath->Coords.push_back(DocCoord((LONG)boundingPoints[4], (LONG)boundingPoints[5]));

		COLORREF crRGB = RGB(0x00,0x00,0xFF);
		CXfComplexColor *color = new CXfComplexColor(crRGB, CXfComplexColor::cmRGB, 0.,0.,1., 0.);
		debugPath->InsertFirstChild(new CXfLineColor(color));
		debugPath->InsertFirstChild(color);

		m_pBBoxLayer->AddChild(debugPath);
		#endif

		textStory.RenderClipview(itStoryBegin, itStoryEnd);

		if (textStory.end() != itStoryEnd && !pParent)
		{
			pParent = textStory.nodeStory->GetParent();
			size_t childrenCount = pParent->ChildrenCount();
			for (size_t i = 0; i < childrenCount; ++i)
			{
				if (pParent->GetChild(i) == textStory.nodeStory)
				{
					nodeIndex = i;
					break;
				}
			}
		}
	}
}

/******************************************************************************
   Function:	CalcKernReplacementTracking
   Author:	Denis Onishchenko
   Purpose:     Calculates average tracking value as kernings replacement
   Returns:
   Notes:       
******************************************************************************/
LONG XarOutputDev::TextLine::CalcKernReplacementTracking(const_iterator itBegin, const_iterator itEnd, bool isSpaces) const
{
	double kernTotal = 0;
	MILLIPOINT fontSizeTotal = 0;

	for (const_iterator it = itBegin; it != itEnd;)
	{
		const_iterator itNext = it;
		++itNext;

		if (end() != itNext && (isSpaces || itEnd != itNext))
		{
			kernTotal  += itNext->x - (it->x + (it->GetWidth() + it->GetAutoKernSize(itEnd == itNext ? nullptr : &(*itNext))));
			fontSizeTotal += it->GetFontSize();
		}

		it = itNext;
	}

	return fontSizeTotal ? _round(kernTotal * 1000. / fontSizeTotal) : 0;
}

LONG XarOutputDev::TextLine::CalcKernReplacementTracking() const
{
	return CalcKernReplacementTracking(begin(), end(), false);
}

bool XarOutputDev::TextLine::ReplaceKernings(iterator itBegin, iterator itEnd, bool isSpaces)
{
	iterator it;

	LONG tracking = CalcKernReplacementTracking(itBegin, itEnd, isSpaces);

	// checking resulting line consistency compared to original line:
	// if tracked string elements are positioned too far from original,
	// disabling kerning replacement for line

	double x = itBegin->x;

	if (!isSpaces)
	{
		for (it = itBegin; it != itEnd;)
		{
			auto itNext = it;
			++itNext;

			if (!it->IsSpace() && fabs(it->x - x) > 0.125 * it->GetFontSize())
				return false;
			x += it->GetWidth() + it->GetAutoKernSize(itEnd == itNext ? nullptr : &(*itNext)) + it->GetTrackingWidth(tracking);

			it = itNext;
		}
		x = itBegin->x;
	}

	for (it = itBegin; it != itEnd;)
	{
		auto itNext = it;
		++itNext;

		it->x = x;
		it->tracking = tracking;
		x += it->GetAdvance(itEnd == itNext ? nullptr : &(*itNext));

		it = itNext;
	}

	if (/*!isSpaces && */end() != it/* && it->IsSpace()*/) it->x = x;

	return true;
}

void XarOutputDev::TextLine::ReplaceKernings()
{
	iterator itBegin = begin();
	while (itBegin != end() && itBegin->isAutoListChar) ++itBegin;
	if (itBegin == end()) return;
	if (ReplaceKernings(itBegin, end(), false)) return;

	iterator itEnd = itBegin;
	++itEnd;
	bool isSpaces = itBegin->IsSpace();

	while (end() != itEnd)
	{
		if (itEnd->IsSpace() != isSpaces)
		{
			ReplaceKernings(itBegin, itEnd, isSpaces);
			itBegin = itEnd;
			isSpaces = !isSpaces;
		}
		++itEnd;
	}

	ReplaceKernings(itBegin, itEnd, isSpaces);
}

MILLIPOINT XarOutputDev::TextLine::GetMaxTrackingWidth(LONG tracking) const
{
	return MulDiv(tracking, maxFontSize, 1000);
}

void XarOutputDev::TextLine::PositionChars()
{
	MILLIPOINT pos = leftMargin = rightMargin = endMargin = _round(front().x);

	iterator it = begin();
	while (true)
	{
		//if (it->isAutoListChar) it->tracking = 0;

		if (it->isAnchorPos)
		{
			// DDES-14835/DDES-14840 fix
			//it->width += it->GetTrackingWidth();
			it->tracking = 0;
		}

		iterator itNext = it;
		++itNext;

		pos += it->GetWidth();
		if (!it->IsSpace())
		{
			endMargin = pos;
			if (pos > rightMargin) rightMargin = pos;
		}

		if (end() != itNext)
		{
			pos += it->GetTrackingWidth() + it->GetAutoKernSize(&(*itNext));
			it->kern = itNext->IsSpace() ? 0 : _round((itNext->x - pos) * 1000. / it->GetFontSize());

			if (it->kern && pos > rightMargin) rightMargin = pos;

			pos += it->GetKernWidth();

			if (it->kern)
			{
				endMargin = pos;
				if (pos > rightMargin) rightMargin = pos;
			}
		}
		else
		{
			it->kern = 0;
			break;
		}
	
		it = itNext;
	}
}

/*MILLIPOINT XarOutputDev::TextLine::GetWidth() const
{
	return rightMargin - leftMargin;
}*/

MILLIPOINT XarOutputDev::TextLine::GetMaxWidth(const TextLine *pNextLine, int nbsp) const
{
	MILLIPOINT pos  = 0;
	for (const_iterator it = begin(); it != end();)
	{
		const_iterator itNext = it;
		++itNext;

		if (bool bSoftHyphen = it->IsHyphen() && end() == itNext)
			break;

		pos += it->GetAdvance(end() == itNext ? nullptr : &(*itNext)) + it->GetKernWidth();

		it = itNext;
	}

	MILLIPOINT maxWidth = 0;

	for (const_iterator it = pNextLine->begin(); it != pNextLine->end();)
	{
		const_iterator itNext = it;
		++itNext;

		if (it->IsSpace())
		{
			if (nbsp)
				--nbsp;
			else
				break;
		}

		pos += it->GetWidth();
		if (pos > maxWidth) maxWidth = pos;

		if (it->IsHyphen()) break;

		pos += it->GetTrackingWidth() + it->GetAutoKernSize(pNextLine->end() == itNext ? nullptr : &(*itNext));
		if (it->kern)
		{
			if (pos > maxWidth) maxWidth = pos;
			pos += it->GetKernWidth(it->kern);
			if (pos > maxWidth) maxWidth = pos;
		}

		it = itNext;
	}

	return maxWidth ? maxWidth - 1 : 0;
}

bool XarOutputDev::TextLine::CheckWrapping(MILLIPOINT fitWidth, const TextLine *pNextLine, int nbsp) const
{
	if (fitWidth < 0) fitWidth = 0;

	MILLIPOINT pos = !empty() && firstKern ? front().GetKernWidth(firstKern) : 0;

	for (const_iterator it = begin(); it != end();)
	{
		const_iterator itNext = it;
		++itNext;

		bool bSoftHyphen = it->IsHyphen() && end() == itNext && pNextLine;
		MILLIPOINT prevPos = pos;

		pos += it->GetWidth();
		if (!it->IsSpace() && pos > fitWidth) return false;
		pos += it->GetTrackingWidth() + it->GetAutoKernSize(end() == itNext ? nullptr : &(*itNext));
		if (it->kern)
		{
			if (pos > fitWidth) return false;
			pos += it->GetKernWidth(it->kern);
			if (pos > fitWidth) return false;
		}

		if (bSoftHyphen)
			pos = prevPos;

		it = itNext;
	}

	if (!pNextLine) return true;

	if (!back().IsBreak() && back().attrs.pFontDesc)
		pos += back().GetWidth(L' ');

	if (!pNextLine->empty() && pNextLine->firstKern)
		pos += pNextLine->front().GetKernWidth(pNextLine->firstKern);

	for (const_iterator it = pNextLine->begin(); it != pNextLine->end();)
	{
		const_iterator itNext = it;
		++itNext;

		if (it->IsSpace())
		{
			if (nbsp)
				--nbsp;
			else
				break;
		}

		pos += it->GetWidth();
		if (pos > fitWidth) return true;

		if (it->IsHyphen()) break;

		pos += it->GetTrackingWidth() + it->GetAutoKernSize(pNextLine->end() == itNext ? nullptr : &(*itNext));
		if (it->kern)
		{
			if (pos > fitWidth) return true;
			pos += it->GetKernWidth(it->kern);
			if (pos > fitWidth) return true;
		}

		it = itNext;
	}

	return false;
}

bool XarOutputDev::TextLine::RemoveKernings(MILLIPOINT fitWidth, MILLIPOINT justifyWidth, LONG tracking, LONG maxRelativeOffset, const TextLine *pNextLine, bool bCheckOnly)
{
#ifdef _ENABLE_SMALL_KERNING_REPLACEMENT

	if (fitWidth < 0) fitWidth = 0;

	// check wrapping

	MILLIPOINT pos = 0;

	for (const_iterator it = begin(); it != end();)
	{
		const_iterator itNext = it;
		++itNext;

		bool bSoftHyphen = it->IsHyphen() && end() == itNext && pNextLine;
		MILLIPOINT prevPos = pos;

		pos += it->GetWidth();
		if (!it->IsSpace() && pos > fitWidth) return false;

		if (it->isAutoListChar || it->isAnchorPos)
		{
			pos += it->GetTrackingWidth() + it->GetAutoKernSize(end() == itNext ? nullptr : &(*itNext));
			if (it->kern)
			{
				if (pos > fitWidth) return false;
				pos += it->GetKernWidth();
				if (pos > fitWidth) return false;
			}
		}
		else
			pos += it->GetTrackingWidth(tracking) + it->GetAutoKernSize(end() == itNext ? nullptr : &(*itNext));

		if (bSoftHyphen)
			pos = prevPos;

		it = itNext;
	}

	if (pNextLine)
	{
		bool bWrapBack = true;
		for (const_iterator it = pNextLine->begin(); it != pNextLine->end() && !it->IsSpace();)
		{
			const_iterator itNext = it;
			++itNext;

			pos += it->GetWidth();
			if (pos > fitWidth)
			{
				bWrapBack = false;
				break;
			}

			if (it->IsHyphen()) break;

			pos += it->GetTrackingWidth() + it->GetAutoKernSize(pNextLine->end() == itNext ? nullptr : &(*itNext));
			if (it->kern)
			{
				if (pos > fitWidth)
				{
					bWrapBack = false;
					break;
				}
				pos += it->GetKernWidth();
				if (pos > fitWidth)
				{
					bWrapBack = false;
					break;
				}
			}

			it = itNext;
		}
		if (bWrapBack) return false;
	}

	// check position error

	MILLIPOINT extraOnSpaces = 0;
	MILLIPOINT extraOnChars  = 0;

	if (justifyWidth)
	{
		MILLIPOINT sumCharAdvances = 0;
		LONG numChars  = 0;
		LONG numSpaces = 0;

		bool bNonSpaceFound = false;
		const_reverse_iterator itPrev = rend();
		for (const_reverse_iterator it = rbegin(); it != rend(); itPrev = it++)
		{
			if (it->IsSpace())
			{
				if (bNonSpaceFound)
				{
					++numSpaces;
					sumCharAdvances += it->GetWidth() + it->GetAutoKernSize(rend() == itPrev ? nullptr : &(*itPrev));
					if (it->isAutoListChar || it->isAnchorPos)
						sumCharAdvances += it->GetTrackingWidth() + it->GetKernWidth();
					else
						sumCharAdvances += it->GetTrackingWidth(tracking);
				}
			}
			else
			{
				++numChars;
				sumCharAdvances += it->GetWidth() + it->GetAutoKernSize(rend() == itPrev ? nullptr : &(*itPrev));
				if (bNonSpaceFound)
				{
					if (it->isAutoListChar || it->isAnchorPos)
						sumCharAdvances += it->GetTrackingWidth() + it->GetKernWidth();
					else
						sumCharAdvances += it->GetTrackingWidth(tracking);
				}
				else bNonSpaceFound = true;
			}
		}

		// if text does not fill line, expand spaces so it does,
		// but if no spaces, and word wrapping, increase inter-char spacing so word fills line
		// if text overflows line, reduce inter-char spacing so words fit line
		LONG gap = justifyWidth - sumCharAdvances;
		if (gap > 0)
		{
			if (numSpaces > 0)
				extraOnSpaces = gap / numSpaces;
			else if (numChars > 1)
				extraOnChars = gap / (numChars - 1);
		}
		else if (numChars > 1)
			extraOnChars = gap / (numChars - 1);
	}

	pos = leftMargin;
	for (const_iterator it = begin(); it != end();)
	{
		const_iterator itNext = it;
		++itNext;

		if (!it->IsSpace() && fabs(it->x - pos) > it->GetTrackingWidth(maxRelativeOffset))
		{
			double delta = fabs(it->x - pos);
			double limit = it->GetTrackingWidth(maxRelativeOffset);
			return false;
		}
		pos += it->GetWidth() + it->GetAutoKernSize(end() == itNext ? nullptr : &(*itNext));
		if (it->isAutoListChar || it->isAnchorPos)
			pos += it->GetTrackingWidth() + it->GetKernWidth();
		else
			pos += it->GetTrackingWidth(tracking);
		pos += extraOnChars;
		if (it->IsSpace()) pos += extraOnSpaces;

		it = itNext;
	}

	if (!bCheckOnly)
	{
		// remove kernings
		for (iterator it = begin(); it != end(); ++it)
		{
			if (!it->isAutoListChar && !it->isAnchorPos)
			{
				it->tracking = tracking;
				it->kern = 0;
			}
		}
	}

	return true;
#else
	return false;
#endif
}

bool XarOutputDev::TextLine::RemoveKernings(MILLIPOINT fitWidth, MILLIPOINT justifyWidth, LONG maxRelativeOffset, const TextLine *pNextLine, bool bCheckOnly)
{
	if (RemoveKernings(fitWidth, justifyWidth, 0, maxRelativeOffset, pNextLine, bCheckOnly))
		return true;

	LONG tracking = justifyWidth ? 0 : CalcKernReplacementTracking();

	if (tracking && RemoveKernings(fitWidth, justifyWidth, tracking, maxRelativeOffset, pNextLine, bCheckOnly))
		return true;

	for (LONG i = 1; i <= 100; ++i)
	{
		if (RemoveKernings(fitWidth, justifyWidth, tracking + i, maxRelativeOffset, pNextLine, bCheckOnly))
			return true;
		if (RemoveKernings(fitWidth, justifyWidth, tracking - i, maxRelativeOffset, pNextLine, bCheckOnly))
			return true;
	}

	return false;
}

/******************************************************************************
   Function:	GetAscentDescentRatio
   Author:	Denis Onishchenko
   Purpose:     Computes line's ascent/descent ratio used in line spacing calculation
   Returns:
   Notes:       
******************************************************************************/

double XarOutputDev::TextLine::GetAscentDescentRatio() const
{
	return (double)descent / (double)(ascent - descent);
}

/******************************************************************************
   Function:	GetLineSpaceRatio
   Author:	Denis Onishchenko
   Purpose:     Computes line space ratio
   Returns:
   Parameters:	l1 - prev line
   Parameters:	l2 - current line
   Notes:       
******************************************************************************/

double XarOutputDev::TextStory::GetLineSpaceRatio(const TextLine &l1, const TextLine &l2)
{
	if (l2.baseLine + l2.ascent <= l1.baseLine + l1.descent) // >= 100%
	{
		return (double)((l1.baseLine + l1.ascent) - (l2.baseLine + l2.ascent)) / (double)(l1.ascent - l1.descent);
	}
	else // < 100%
	{
		return (double)(l1.baseLine - l2.baseLine) / (double)(l2.ascent - l1.descent);
	}
}

/******************************************************************************
   Function:	GetLineSpaceRatio
   Author:	Denis Onishchenko
   Purpose:     Computes line space ratio
   Returns:
   Parameters:	lines
   Notes:       
******************************************************************************/

double XarOutputDev::TextStory::GetLineSpaceRatio(const TextLines &lines)
{
	if (lines.size() < 2)
		return 1.0; // default ratio

	MILLIPOINT t = 0;
	auto it = lines.begin();
	auto itNext = it;
	++itNext;
	do
	{
		t += (itNext->ascent - it->descent);
		it = itNext++;
	}
	while (lines.end() != itNext);

	MILLIPOINT s = lines.front().baseLine - lines.back().baseLine;

	if (s > t) // > 100 %
	{
		s = t = 0;
		it = lines.begin();
		itNext = it;
		++itNext;
		do
		{
			s += ((it->baseLine + it->ascent) - (itNext->baseLine + itNext->ascent));
			t += (it->ascent - it->descent);
			it = itNext++;
		}
		while (lines.end() != itNext);
	}

	return (double)s / (double)t;
}

/******************************************************************************
   Function:	GetAbsoluteLineSpacing
   Author:	Denis Onishchenko
   Purpose:     Computes paragraph's absolute line spacing
   Returns:
   Parameters:	l1 - first paragraph's line
   Parameters:	l2 - last paragraph's line
   Parameters:	m  - number of intermediate lines between the first and last line
   Notes:       
******************************************************************************/

double XarOutputDev::TextStory::GetAbsoluteLineSpacing(const TextLine &l1, const TextLine &l2, size_t m)
{
	return (l1.baseLine - l2.baseLine) / ((m + 1) + l2.GetAscentDescentRatio() - l1.GetAscentDescentRatio());
}

/******************************************************************************
   Function:	GetParagraphSpacing
   Author:	Denis Onishchenko
   Purpose:     Computes additional spacing between paragraphs
   Returns:
   Parameters:	p1, p2 - successive paragraphs
   Notes:       
******************************************************************************/

double XarOutputDev::TextStory::GetParagraphSpacing(const Paragraph &p1, const Paragraph &p2)
{
	if (p1.empty() || p2.empty()) return 0;
	const TextLine &l1 = p1.back();
	const TextLine &l2 = p2.front();
#ifdef _ENABLE_RELATIVE_LINE_SPACING
	double p1Ratio = p1.linespaceRatio.MakeDouble();
	double p2Ratio = p2.linespaceRatio.MakeDouble();
	double p1Bottom = p1Ratio < 1.0 ? (l1.baseLine + p1Ratio * l1.descent) : (l1.baseLine + l1.ascent - p1Ratio * (l1.ascent - l1.descent));
	double p2Top = p2Ratio < 1.0 ? (l2.baseLine + p2Ratio * l2.ascent) : (l2.baseLine + l2.ascent);
	return p1Bottom - p2Top;
#else
	return (l1.baseLine + p1.linespacing * l1.GetAscentDescentRatio()) - (l2.baseLine + p2.linespacing * (1.0 + l2.GetAscentDescentRatio()));
#endif
}

bool XarOutputDev::TextStory::IsEndOfStatement(const TextLine &cur, const TextLine &next)
{
	for (TextChars::const_reverse_iterator it = cur.rbegin(); it != cur.rend(); ++it)
	{
		if (0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask())) continue;
		return 0 != (UNICODE_MASK_STATEMENT_END & it->GetUnicodeMask());
	}

	return false;

	/*// checking first / last characters of next / current lines
	TextChars::const_reverse_iterator itCharCur = cur.rbegin();
	while (cur.rend() != itCharCur && itCharCur->GetUnicodeMask() & UNICODE_MASK_INVISIBLE)
		++itCharCur;

	TextChars::const_iterator itCharNext = next.begin();
	while (next.end() != itCharNext && itCharNext->GetUnicodeMask() & UNICODE_MASK_INVISIBLE)
		++itCharNext;

	if (cur.rend() == itCharCur || next.end() == itCharNext)
		return false;

	int maskCharCur = itCharCur->GetUnicodeMask(); // last visible char-mask of current line
	int maskCharNext = itCharNext->GetUnicodeMask(); // first visible char-mask of next line

	return ((maskCharCur & UNICODE_MASK_STATEMENT_END) || (maskCharCur & UNICODE_MASK_QUOTES_CLOSE)) &&
	       ((maskCharNext & UNICODE_MASK_LETTER_CAPITAL) || (maskCharCur & UNICODE_MASK_NUMBER) || (maskCharCur & UNICODE_MASK_QUOTES_OPEN));*/
}

void XarOutputDev::AddDefaultAttributeTagsForStory(const XarOutputDev::TextAttrsCollector &attrs, VXfTreeNode *node)
{
	node->AddChild(new CXfTextFontSize(attrs.fontSize()), false);

	if (attrs.pFontDef.IsDefined() && attrs.pFontDef())
		node->AddChild(new CXfFontTypeface(attrs.pFontDef()), false);
	if (attrs.isBold())
		node->AddChild( new CXfZeroSizedNode(TAG_TEXT_BOLD_ON), false );
	if (attrs.isItalic())
		node->AddChild( new CXfZeroSizedNode(TAG_TEXT_ITALIC_ON), false );

	/*if (attrs.baseLineShift.IsDefined() && attrs.baseLineShift(0))
	{
		node->AddChild(new CXfTextBaseline(attrs.baseLineShift(0)), false);
	}*/

	if (attrs.tracking.IsDefined() && attrs.tracking(0))
	{
		node->AddChild(new CXfTextTracking(attrs.tracking(0)), false);
		node->AddChild(new CXfZeroSizedNode(TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING), false);
	}

	if (attrs.aspectRatio.IsDefined() && attrs.aspectRatio(1) != FIXED16(1))
		node->AddChild(new CXfAspectRatio(attrs.aspectRatio(1)), false);

	if (attrs.isFilled())
	{
		CXfComplexColor* pComplex = CreateComplexColour(attrs.colorFill());
		node->AddChild(new CXfFlatFill(pComplex), false);
		//node->AddChild(pComplex, false);
	}

	if (attrs.opacityFill.IsDefined() && attrs.opacityFill().value)
		node->AddChild(new CXfFlatTransparentFill(attrs.opacityFill().value, attrs.opacityFill().mode), false);

	if (attrs.isStroked.IsDefined() && attrs.isStroked())
	{
		if (attrs.lineWidth.IsDefined())
			node->AddChild(new CXfLineWidth(attrs.lineWidth()), false);

		if (attrs.dash.IsDefined() && !attrs.dash().empty())
		{
			CXfDefineDash* pDefine = new CXfDefineDash(TAG_DEFINEDASH, attrs.dash()[0], attrs.dash()[1]);
			for (size_t i = 2; i < attrs.dash().size(); i++)
				pDefine->AppendDashDef(attrs.dash()[i]);
			node->AddChild(pDefine, false);
		}

		if (attrs.colorStroke.IsDefined())
		{
			CXfComplexColor* pComplex = CreateComplexColour(attrs.colorStroke());
			node->AddChild(new CXfLineColor(pComplex), false);
			//node->AddChild(pComplex, false);
		}

		if (attrs.opacityStroke.IsDefined() && attrs.opacityStroke().value)
			node->AddChild(new CXfLineTransparency(attrs.opacityStroke().value, attrs.opacityStroke().mode), false);
	}
	else
	{
		// Write 'none' line width
		node->AddChild(new CXfLineWidth(250), false);
		node->AddChild(new CXfLineColor(CXfLineColor::None), false);
	}

	if (attrs.lineJoin.IsDefined())
		node->AddChild(new CXfJoinStyle(attrs.lineJoin()), false);
	if (attrs.lineCap.IsDefined())
		node->AddChild(new CXfStartCap(TAG_STARTCAP, attrs.lineCap()), false);
	if (attrs.miterLimit.IsDefined())
		node->AddChild(new CXfMitreLimit(attrs.miterLimit()), false);
}

void XarOutputDev::AddOptimizedAttributeLineTags(const XarOutputDev::TextAttrsCollector &attrs, const TextAttrsCollector &lineAttrs, VXfTreeNode *node)
{
	if (lineAttrs.pFontDef() && lineAttrs.pFontDef() != attrs.pFontDef(lineAttrs.pFontDef()))
		node->AddChild(new CXfFontTypeface(lineAttrs.pFontDef()), false);
	if (lineAttrs.isBold() != attrs.isBold())
		node->AddChild(new CXfZeroSizedNode(lineAttrs.isBold() ? TAG_TEXT_BOLD_ON : TAG_TEXT_BOLD_OFF), false);
	if (lineAttrs.isItalic() != attrs.isItalic())
		node->AddChild(new CXfZeroSizedNode(lineAttrs.isItalic() ? TAG_TEXT_ITALIC_ON : TAG_TEXT_ITALIC_OFF), false);
	if (lineAttrs.fontSize() != attrs.fontSize())
		node->AddChild(new CXfTextFontSize(lineAttrs.fontSize()), false);

	/*if (lineAttrs.baseLineShift(0) != attrs.baseLineShift(0))
	{
		node->AddChild(new CXfTextBaseline(lineAttrs.baseLineShift(0)), false);
	}*/

	if (lineAttrs.tracking(0) != attrs.tracking(0))
	{
		node->AddChild(new CXfTextTracking(lineAttrs.tracking(0)), false);
		node->AddChild(new CXfZeroSizedNode(TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING), false);
	}

	if (lineAttrs.aspectRatio(1) != attrs.aspectRatio(1))
		node->AddChild(new CXfAspectRatio(lineAttrs.aspectRatio(1)), false);

	if (lineAttrs.isFilled())
	{
		if (!attrs.isFilled() || lineAttrs.colorFill() != attrs.colorFill())
		{
			CXfComplexColor* pComplex = CreateComplexColour(lineAttrs.colorFill());
			node->AddChild(new CXfFlatFill(pComplex), false);
			//node->AddChild(pComplex, false);
		}

		if (!attrs.isFilled() || lineAttrs.opacityFill() != attrs.opacityFill())
			node->AddChild(new CXfFlatTransparentFill(lineAttrs.opacityFill().value, lineAttrs.opacityFill().mode), false);
	}
	else if (attrs.isFilled())
	{
		node->AddChild(new CXfFlatFill(CXfFlatFill::None), false);
	}

	if (lineAttrs.isStroked())
	{
		if (!attrs.isStroked() || lineAttrs.lineWidth() != attrs.lineWidth())
			node->AddChild(new CXfLineWidth(lineAttrs.lineWidth()), false);

		if (!attrs.isStroked() || lineAttrs.dash() != attrs.dash())
		{
			if (!lineAttrs.dash().empty())
			{
				CXfDefineDash* pDefine = new CXfDefineDash(TAG_DEFINEDASH, lineAttrs.dash()[0], lineAttrs.dash()[1]);
				for (size_t i = 2; i < lineAttrs.dash().size(); i++)
					pDefine->AppendDashDef(lineAttrs.dash()[i]);
				node->AddChild(pDefine, false);
			}
			else
			{
				node->AddChild(new CXfDashStyle(REF_DASH_SOLID), false);
			}
		}

		if (!attrs.isStroked() || lineAttrs.colorStroke() != attrs.colorStroke())
		{
			CXfComplexColor* pComplex = CreateComplexColour(lineAttrs.colorStroke());
			node->AddChild(new CXfLineColor(pComplex), false);
			//node->AddChild(pComplex, false);
		}

		if (!attrs.isStroked() || lineAttrs.opacityStroke() != attrs.opacityStroke())
			node->AddChild(new CXfLineTransparency(lineAttrs.opacityStroke().value, lineAttrs.opacityStroke().mode), false);
	}
	else if (attrs.isStroked())
	{
		node->AddChild(new CXfLineWidth(250), false);
		node->AddChild(new CXfLineColor(CXfLineColor::None), false);
	}

	if (lineAttrs.lineJoin() != attrs.lineJoin(lineAttrs.lineJoin()))
		node->AddChild(new CXfJoinStyle(lineAttrs.lineJoin()), false);
	if (lineAttrs.lineCap() != attrs.lineCap(lineAttrs.lineCap()))
		node->AddChild(new CXfStartCap(TAG_STARTCAP, lineAttrs.lineCap()), false);
	if (lineAttrs.miterLimit() != attrs.miterLimit(lineAttrs.miterLimit()))
		node->AddChild(new CXfMitreLimit(lineAttrs.miterLimit()), false);
}

void XarOutputDev::AddOptimizedAttributeStringTags(const TextChar &str, const TextAttrsCollector &lineAttrs, VXfTreeNode *node, const TextChar *pListTextFirstChar)
{
	const TextAttrs &attrsString = str.attrs;

	if (attrsString.fontSize != lineAttrs.fontSize())
		node->AddChild(new CXfTextFontSize(attrsString.fontSize), false);

	// Denis: DDES-16256 (Bold bullets mixed with non-bold bullets)
	/*if (str.isAutoListChar && pListTextFirstChar)
	{
		if (pListTextFirstChar->pFontDef && str.pFontDef != lineAttrs.pFontDef(pListTextFirstChar->pFontDef))
			node->AddChild(new CXfFontTypeface(pListTextFirstChar->pFontDef), false);
		if (pListTextFirstChar->attrs.isBold != lineAttrs.isBold())
			node->AddChild( new CXfZeroSizedNode(pListTextFirstChar->attrs.isBold ? TAG_TEXT_BOLD_ON : TAG_TEXT_BOLD_OFF), false );
		if (pListTextFirstChar->attrs.isItalic != lineAttrs.isItalic())
			node->AddChild( new CXfZeroSizedNode(pListTextFirstChar->attrs.isItalic ? TAG_TEXT_ITALIC_ON : TAG_TEXT_ITALIC_OFF), false );
	}
	else*/
	{
		if (str.pFontDef && str.pFontDef != lineAttrs.pFontDef(str.pFontDef))
			node->AddChild(new CXfFontTypeface(str.pFontDef), false);
		if (attrsString.isBold != lineAttrs.isBold())
			node->AddChild( new CXfZeroSizedNode(attrsString.isBold ? TAG_TEXT_BOLD_ON : TAG_TEXT_BOLD_OFF), false );
		if (attrsString.isItalic != lineAttrs.isItalic())
			node->AddChild( new CXfZeroSizedNode(attrsString.isItalic ? TAG_TEXT_ITALIC_ON : TAG_TEXT_ITALIC_OFF), false );
	}

	if (str.baseLineShift)// != lineAttrs.baseLineShift(0))
	{
		node->AddChild(new CXfTextBaseline(str.baseLineShift), false);
	}

	if (node->IsTextString() && str.tracking != lineAttrs.tracking(0))
	{
		node->AddChild(new CXfTextTracking(str.tracking), false);
		node->AddChild(new CXfZeroSizedNode(TAG_TEXT_FONT_SIZE_RELATIVE_TRACKING), false);
	}

	if (attrsString.aspectRatio != lineAttrs.aspectRatio(1))
		node->AddChild(new CXfAspectRatio(attrsString.aspectRatio), false);

	if (attrsString.isFilled)
	{
		if (!lineAttrs.isFilled() || attrsString.colorFill != lineAttrs.colorFill())
		{
			CXfComplexColor *pComplex = CreateComplexColour(attrsString.colorFill);
			node->AddChild(new CXfFlatFill(pComplex), false);
			//node->AddChild(pComplex, false);
		}

		if (!lineAttrs.isFilled() || attrsString.opacityFill != lineAttrs.opacityFill())
			node->AddChild(new CXfFlatTransparentFill(attrsString.opacityFill.value, attrsString.opacityFill.mode), false);
	}
	else if (lineAttrs.isFilled())
	{
		node->AddChild(new CXfFlatFill(CXfFlatFill::None), false);
	}

	if (attrsString.isStroked)
	{
		if (!lineAttrs.isStroked() || attrsString.lineWidth != lineAttrs.lineWidth())
			node->AddChild(new CXfLineWidth(attrsString.lineWidth), false);

		if (!lineAttrs.isStroked() || attrsString.dash != lineAttrs.dash())
		{
			if (!attrsString.dash.empty())
			{
				CXfDefineDash* pDefine = new CXfDefineDash(TAG_DEFINEDASH, attrsString.dash[0], attrsString.dash[1]);
				for (size_t i = 2; i < attrsString.dash.size(); i++)
					pDefine->AppendDashDef(attrsString.dash[i]);
				node->AddChild(pDefine, false);
			}
			else
			{
				node->AddChild(new CXfDashStyle(REF_DASH_SOLID), false);
			}
		}

		if (!lineAttrs.isStroked() || attrsString.colorStroke != lineAttrs.colorStroke())
		{
			CXfComplexColor* pComplex = CreateComplexColour(attrsString.colorStroke);
			node->AddChild(new CXfLineColor(pComplex), false);
			//node->AddChild(pComplex, false);
		}

		if (!lineAttrs.isStroked() || attrsString.opacityStroke != lineAttrs.opacityStroke())
			node->AddChild(new CXfLineTransparency(attrsString.opacityStroke.value, attrsString.opacityStroke.mode), false);
	}
	else if (lineAttrs.isStroked())
	{
		node->AddChild(new CXfLineWidth(250), false);
		node->AddChild(new CXfLineColor(CXfLineColor::None), false);
	}

	if (attrsString.lineJoin != lineAttrs.lineJoin(attrsString.lineJoin))
		node->AddChild(new CXfJoinStyle(attrsString.lineJoin), false);
	if (attrsString.lineCap != lineAttrs.lineCap(attrsString.lineCap))
		node->AddChild(new CXfStartCap(TAG_STARTCAP, attrsString.lineCap), false);
	if (attrsString.miterLimit != lineAttrs.miterLimit(attrsString.miterLimit))
		node->AddChild(new CXfMitreLimit(attrsString.miterLimit), false);

	if (str.isUnderline)
		node->AddChild(new CXfZeroSizedNode(TAG_TEXT_UNDERLINE_ON), false);
	if (str.isStrikethrough)
		node->AddChild(new CXfZeroSizedNode(TAG_TEXT_STRIKETHROUGH_ON), false);
}

template<class T> void SeriesCollector<T>::Register(const T &op)
{
	std::map<T,int>::iterator it = usage.find(op);
	if (it == usage.end())
		usage.insert(std::make_pair(op,1));
	else
		++((*it).second);
	needUpdateCommon = true;
}

template<class T> const T &SeriesCollector<T>::operator()(const T &default) const
{
	if (usage.empty())
		return default;
	return (*this)();
}

template<class T> const T &SeriesCollector<T>::operator()() const
{
	if (!needUpdateCommon)
		return common;
	int maxCounter = 0;
	T maxT = T();
	for (std::map<T,int>::const_iterator it=usage.begin(); it != usage.end(); ++it)
	{
		if ((*it).second > maxCounter)
		{
			maxCounter = (*it).second;
			maxT = (*it).first;
		}
	}
	needUpdateCommon = false;
	return (common = maxT);
}

void XarOutputDev::TextLine::GetBoundingPolygons(const TextChars &chars, ClipperLib::Paths &polygons)
{
	size_t c = polygons.size() + chars.size();
	if (polygons.capacity() < c) polygons.reserve(c);
	for (TextChars::const_iterator it = chars.begin(); it != chars.end(); ++it)
		AddRect(polygons, it->bbox);
}

void XarOutputDev::TextLine::GetBoundingPolygons(ClipperLib::Paths &polygons) const
{
	GetBoundingPolygons(*this, polygons);
}

std::wstring SafeLogWString(const std::wstring s)
{
	std::wstring out = s;
	for (int i=0; i<out.length(); ++i)
		if (out[i] >= 0x0080)
			out[i] = L'#';
	return out;
}

void XarOutputDev::RemoveTextMarkupPath(GraphicsLine &line)
{
	// delete path tags
	if (line.nodePath)
	{
		line.nodePath->SetRemoved(true);
		line.nodePath = NULL;
	}
}

void XarOutputDev::DetectUnderlineStrikethrough(TextStory& textStory, TextLine &line)
{
	for (QueueUnderlineCheck::iterator itGraphics = queueUnderline.begin(); itGraphics != queueUnderline.end(); ++itGraphics)
	{
		Rect rect;
		if (itGraphics->TransformToRect(textStory.matrixInv, rect) && line.CheckUnderlineStrikethrough(rect, itGraphics->color, itGraphics->opacity))
		{
			RemoveTextMarkupPath(*itGraphics);
			break;
		}
	}
}

bool XarOutputDev::CreateGraphicsLine(CXfPath* pPath, GfxState* state, bool bStroke, GraphicsLine &out)
{
	switch (pPath->Verbs.size())
	{
	case 5:
		if (pPath->Coords[0] != pPath->Coords[4]) return false;
	case 4:
		if (PT_MOVETO != pPath->Verbs[0] || PT_LINETO != pPath->Verbs[1] || PT_LINETO != pPath->Verbs[2]) return false;
		for (size_t i =  3; i < pPath->Verbs.size(); ++i)
		{
			if (PT_LINETO != (pPath->Verbs[i] & ~PT_CLOSEFIGURE)) return false;
		}
		break;
	default:
		return false;
	}

	if (abs((pPath->Coords[1].x - pPath->Coords[0].x) + (pPath->Coords[3].x - pPath->Coords[0].x) - (pPath->Coords[2].x - pPath->Coords[0].x)) > 0 ||
	    abs((pPath->Coords[1].y - pPath->Coords[0].y) + (pPath->Coords[3].y - pPath->Coords[0].y) - (pPath->Coords[2].y - pPath->Coords[0].y)) > 0)
		return false;

	for (size_t i = 0; i < 4; ++i)
		out.coords[i] = pPath->Coords[i];

	if (bStroke)
	{
		out.color   = Color::MakeFromState(GetStrokeColorSpace(state), GetStrokeColor(state));
		out.opacity = Opacity::MakeFromState(state->getStrokeOpacity(), state->getBlendMode());
	}
	else
	{
		out.color   = Color::MakeFromState(GetFillColorSpace(state), GetFillColor(state));
		out.opacity = Opacity::MakeFromState(state->getFillOpacity(), state->getBlendMode());
	}

	return true;
}

bool XarOutputDev::GraphicsLine::TransformToRect(const AffineTransform &matrix, Rect &rect) const
{
	DocCoord c[4];
	for (size_t i = 0; i < 4; ++i)
	{
		double x, y;
		matrix.map(coords[i].x, coords[i].y, x, y);
		c[i].x = _round(x);
		c[i].y = _round(y);
	}

	LONG xmin = c[0].x, ymin = c[0].y;
	LONG xmax = xmin, ymax = ymin;

	for (size_t i = 1; i < 4; ++i)
	{
		if (c[i].x < xmin)
			xmin = c[i].x;
		else if (c[i].x > xmax)
			xmax = c[i].x;

		if (c[i].y < ymin)
			ymin = c[i].y;
		else if (c[i].y > ymax)
			ymax = c[i].y;
	}

	for (size_t i = 0; i < 4; ++i)
	{
		if ((c[i].x != xmin && c[i].x != xmax) || (c[i].y != ymin && c[i].y != ymax))
			return false;
	}

	rect.x = xmin;
	rect.y = ymin;
	rect.width  = xmax - xmin;
	rect.height = ymax - ymin;

	return true;
}

void XarOutputDev::ClearUnderlineStrikethrough()
{
	queueUnderline.clear();
}

void XarOutputDev::AddToUnderlineStrikethroughCheck(XarOutputDev::GraphicsLine &line)
{
	if (queueUnderline.size() >= 100)
		queueUnderline.erase(queueUnderline.begin());
	queueUnderline.push_back(line);
}

/******************************************************************************
   Function:	RomanToNumber
   Author:	Denis Onishchenko
   Purpose:     Converts Roman numeral string into integer
   Returns:
   Parameters:	Roman numeral string
   Notes:       
******************************************************************************/
ULONG RomanToNumber(const std::wstring &roman)
{
	ULONG totalValue = 0, prevValue = 1000, digit = 0, prevDigit = 0, lastDigit = 0, maxDigit = 1000;
	size_t count = 1;
	bool v = false, l = false, d = false;
	for (std::wstring::const_iterator it = roman.begin(); it != roman.end(); ++it)
	{
		switch (*it)
		{
		case 'I': case 'i':
			digit = 1;
			break;
		case 'V': case 'v':
			if (v) return 0; // V, L and D may only appear once in each Roman numeral
			v = true;
			digit = 5;
			break;
		case 'X': case 'x':
			digit = 10;
			break;
		case 'L': case 'l':
			if (l) return 0; // V, L and D may only appear once in each Roman numeral
			l = true;
			digit = 50;
			break;
		case 'C': case 'c':
			digit = 100;
			break;
		case 'D': case 'd': 
			if (d) return 0; // V, L and D may only appear once in each Roman numeral
			d = true;
			digit = 500;
			break;
		case 'M': case 'm':
			digit = 1000;
			break;
		default: return 0;
		}

		// Other single letter may be repeated up to three times consecutively
		if (digit == lastDigit)
		{
			if (4 == ++count) return 0;
		}
		else
		{
			lastDigit = digit;
			count = 1;
		}

		if (prevDigit)
		{
			if (prevDigit > maxDigit) return 0;

			if (prevDigit < digit) // Subtractive combination
			{
				switch (prevDigit)
				{
				case 1: case 10: case 100: // The subtracted digit must be either I, X or C
					break;
				default:
					return 0;
				}

				if (10 * prevDigit < digit) return 0; // The subtracted digit must be at least one tenth of the value of the larger numeral

				maxDigit = prevDigit - 1;
				digit -= prevDigit;
				if (digit > prevValue) return 0; // The value must never increased from left to right
				totalValue += digit;
				prevValue = digit;
				prevDigit = 0;
			}
			else
			{
				if (prevDigit > prevValue) return 0; // The value must never increased from left to right
				totalValue += prevDigit;
				prevValue = prevDigit;
				prevDigit = digit;
			}
		}
		else
		{
			prevDigit = digit;
		}
	}

	if (prevDigit)
	{
		if (prevDigit > maxDigit) return 0;
		if (prevDigit > prevValue) return 0; // The value must never increased from left to right
		totalValue += prevDigit;
	}

	return totalValue;
}

void XarOutputDev::TextLine::RecognizeListItemHead()
{
	numberingFormat.clear();
	numberingFormatAlt.clear();
	number = numberAlt = startNumber = level = 0;

	const size_t MAX_ALPHA_NUM_LEN = 1;
	const ULONG MAX_LEVEL = 7;

	iterator it = begin();
	if (end() == it) return;

	bool isBullet = false;

	if (0 != (UNICODE_MASK_BULLET & it->GetUnicodeMask()))
		isBullet = true;
	else if ('o' == it->c)
	{
		iterator itNext = it;
		++itNext;
		if (end() != itNext && 0 != (UNICODE_MASK_INVISIBLE & itNext->GetUnicodeMask()))
			isBullet = true;
	}

	if (isBullet)
	{
		(it++)->isAutoListChar = true;

		/*
		for (; it != end() && 0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask());)
		{
			iterator itSpace = it++;
			erase(itSpace);
		}

		if (end() != it)
			it->isTabStop = true;
		*/

		return;
	}

	wchar_t cLeftBracket = 0, cRightBracket = 0;
	if (!it->isAnchorPos)
	{
		switch (it->c)
		{
		case '(':
			cLeftBracket = '(';
			cRightBracket = ')';
			++it;
			break;
		/*case '[':
			cLeftBracket = '[';
			cRightBracket = ']';
			++it;
			break;*/
		}
	}
	if (end() == it) return;

	bool bDecimal = false, bRoman = false, bUpperCase = false;
	std::wstring numeral;
	size_t lastComponentOffset = 0;

	iterator itLast = end();

	if (!it->isAnchorPos)
	{
		if (it->c >= '0' && it->c <= '9')
		{
			bDecimal = true;
			numeral.push_back(it->c);
			itLast = it++;

			while (true)
			{
				for (; it != end() && !it->isAnchorPos && it->c >= '0' && it->c <= '9'; itLast = it++)
					numeral.push_back(it->c);

				if (it != end() && !it->isAnchorPos && '.' == it->c)
				{
					iterator itNext = it;
					++itNext;
					if (itNext != end() && !itNext->isAnchorPos && itNext->c >= '0' && itNext->c <= '9' && level < MAX_LEVEL)
					{
						numeral.push_back(it->c);
						lastComponentOffset = numeral.length();
						++level;
						numeral.push_back(itNext->c);
						itLast = itNext;
						it = ++itNext;
						continue;
					}
				}
				break;
			}
		}
		else if (it->c >= 'a' && it->c <= 'z' || it->c >= 'A' && it->c <= 'Z')
		{
			bUpperCase = 0 == (0x20 & it->c);
			numeral.push_back(it->c);
			itLast = it++;

			for (; it != end() && !it->isAnchorPos && (bUpperCase && it->c >= 'A' && it->c <= 'Z' || !bUpperCase && it->c >= 'a' && it->c <= 'z'); itLast = it++)
				numeral.push_back(it->c);
		}
	}

	if (numeral.length() && it != end() && !it->isAnchorPos && (cLeftBracket && cRightBracket == it->c || !cLeftBracket && ('.' == it->c || ')' == it->c || bDecimal/*lastComponentOffset*/)))
	{
		switch (it->c)
		{
		case ')': case ']': case '.':
			itLast = it;
		};

		iterator itNext = itLast;
		++itNext;
		while (itNext != end() && 0 != (UNICODE_MASK_INVISIBLE & itNext->GetUnicodeMask())) ++itNext;

		if (itNext == end())
			return;

		if (!itNext->isTabStop)
		{
			double tabWidth = itNext->x - (itLast->x + itLast->GetWidth() + itLast->GetAutoKernSize(&(*itNext)));
			if (tabWidth < itLast->GetTrackingWidth(150))
				return;
		}

		if (bDecimal)
		{
			for (std::wstring::const_iterator itDigit = numeral.begin() + lastComponentOffset; itDigit != numeral.end(); ++itDigit)
			{
				number *= 10;
				number += (*itDigit - L'0');
			}
		}
		else
		{
			if (number = RomanToNumber(numeral))
				bRoman = true;

			if (numeral.length() <= MAX_ALPHA_NUM_LEN)
			{
				for (std::wstring::const_iterator itDigit = numeral.begin(); itDigit != numeral.end(); ++itDigit)
				{
					numberAlt *= 26;
					numberAlt += 0x1F & (*itDigit);
				}

				if (!bRoman) number = numberAlt;
			}
		}

		if (!bDecimal && !number) return;

		if (cLeftBracket) numberingFormat.push_back(cLeftBracket);
		if (bDecimal)
		{
			numberingFormat += numeral.substr(0, lastComponentOffset);
			numberingFormat += L"%1";
		}
		else
		{
			if (bRoman && numberAlt) numberingFormatAlt = numberingFormat + (bUpperCase ? L"%A" : L"%a");
			numberingFormat += bRoman ? (bUpperCase ? L"%I" : L"%i") : (bUpperCase ? L"%A" : L"%a");
		}

		for (iterator itChar = begin(); itChar != it; ++itChar)
			itChar->isAutoListChar = true;

		if (!it->isAnchorPos)
		{
			switch (it->c)
			{
			case ')': case ']': case '.':
				numberingFormat.push_back(it->c);
				if (numberingFormatAlt.length()) numberingFormatAlt.push_back(it->c);
				(it++)->isAutoListChar = true;
			}
		}
		numberingFormat += L"%t";
		if (numberingFormatAlt.length()) numberingFormatAlt += L"%t";

		/*
		for (; it != end() && 0 != (UNICODE_MASK_INVISIBLE & it->GetUnicodeMask());)
		{
			iterator itSpace = it++;
			erase(itSpace);
		}			

		if (end() != it)
			it->isTabStop = true;
		*/
	}
}

void XarOutputDev::TextLine::AppendSpaces(const_iterator it, const TextChar &ref, size_t count, bool bUpdateAscentDescent)
{
	if (!ref.attrs.pFontDesc) return;

	while (count)
	{		
		const_iterator itNext = it;
		++itNext;

		double x1Last = it->x + it->GetAdvance(end() == itNext ? nullptr : &(*itNext));

		iterator itNew = insert(++it, ref);

		//if (itNew->pFontDef != itNew->attrs.pFontDesc->node)
			itNew->pFontDef = itNew->attrs.pFontDesc->node;

		itNew->c              = L' ';
		itNew->attrs.isItalic = ref.attrs.pFontDesc->isItalic;
		itNew->attrs.isBold   = ref.attrs.pFontDesc->isBold;
		itNew->isAnchorPos    = false;
		itNew->isCacheValid   = false;
		itNew->tracking = 0;
		itNew->bbox.width = itNew->bbox.height = 0;
		itNew->ligType = 0;

		itNew->x = x1Last;

		/*if (bUpdateAscentDescent)
		{
			MILLIPOINT charAscent  = itNew->GetFontAscent();
			MILLIPOINT charDescent = itNew->GetFontDescent();

			if (charDescent < descent)
				descent = charDescent;
			if (charAscent > ascent)
				ascent = charAscent;
		}*/

		--count;
	}
}

// Whitespace character reconstruction.
// The kerning between two consecutive string elements is replaced
// by space character with kerning remained. The problem here is 
// if left and right elements have different sizes or fonts, or even
// different script attribute. A simple heuristics is used to determine
// which attributes does this whitespace potentially have: from left
// string element or from the right one.
void XarOutputDev::TextLine::ReconstructWhiteSpace(const_iterator itPrev, const_iterator itNext)
{
	if (!itPrev->attrs.pFontDesc || !itNext->attrs.pFontDesc || itPrev->IsSpace() || itNext->IsSpace()) return;

	double dx = itNext->x - (itPrev->x + itPrev->GetWidth());

	MILLIPOINT prevSpaceWidth = itPrev->GetWidth(L' ');
	MILLIPOINT nextSpaceWidth = itNext->GetWidth(L' ');

	const_iterator itRef;
	MILLIPOINT spaceWidth;

	if (prevSpaceWidth >= nextSpaceWidth)
	{
		itRef = itPrev;
		spaceWidth = prevSpaceWidth;
	}
	else
	{
		itRef = itNext;
		spaceWidth = nextSpaceWidth;
	}

	if (dx >= 0.67 * spaceWidth) AppendSpaces(itPrev, *itRef, 1, false);
}

void XarOutputDev::TextLine::ReconstructWhiteSpaces()
{
	const_iterator it = begin();
	if (end() == it) return;
	const_iterator itNext = it;
	++itNext;

	while (end() != itNext)
	{
		ReconstructWhiteSpace(it, itNext);
		it = itNext++;		
	}
}
